/**
 * Single flash on the sprite, removes self after a while
 * @moduleName Blink
 */
bento.define('components/blink', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'bento/packedimage'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween,
    PackedImage
) {
    'use strict';
    var fadeSpeed = 1 / 30;
    // Android: swap with glow component
    var Glow = function (settings) {
        var viewport = Bento.getViewport();
        var entity;
        var sprite;
        var renderer = Bento.getRenderer();
        var component = {
            name: 'glow',
            alpha: 1,
            start: function (data) {
                sprite = entity.getComponent('sprite');
            },
            update: function (data) {
                if (component.alpha > 0) {
                    component.alpha -= fadeSpeed;

                    // remove self
                    if (component.alpha <= 0) {
                        entity.remove(component);
                    }
                }
            },
            draw: function (data) {
                // warning: only for 2d context
                if (renderer.name !== 'canvas2d') {
                    return;
                }
                if (!component.visible || !sprite) {
                    return;
                }

                var context = data.renderer.getContext();
                var oldAlpha = context.globalAlpha;
                context.globalCompositeOperation = 'lighter';
                context.globalAlpha = component.alpha;

                // draw sprite again
                sprite.draw(data);
                sprite.draw(data);

                // revert context
                context.globalCompositeOperation = 'source-over';
                context.globalAlpha = oldAlpha;

            },
            attached: function (data) {
                entity = data.entity;
            },
            visible: true
        };
        return component;
    };
    if (Utils.isNativeAndroid() || Utils.isOldIphone()) {
        return Glow;
    }

    return function (settings) {
        var viewport = Bento.getViewport();
        var entity;
        var sprite;
        var renderer = Bento.getRenderer();
        var canvas;
        var context;
        var packedImage;
        var component = {
            name: 'blink',
            visible: true,
            alpha: 1,
            start: function (data) {
                sprite = entity.getComponent('sprite');
                if (Utils.isDefined(settings.alpha)) {
                    component.alpha = settings.alpha;
                }

                canvas = document.createElement('canvas');
                canvas.width = sprite.frameWidth;
                canvas.height = sprite.frameHeight;

                context = canvas.getContext('2d');
                context.globalCompositeOperation = 'source-over';
                context.globalAlpha = 1;
                context.drawImage(
                    sprite.spriteImage.image,
                    sprite.spriteImage.x + sprite.sourceX,
                    sprite.spriteImage.y + sprite.sourceY,
                    sprite.frameWidth,
                    sprite.frameHeight,
                    0,
                    0,
                    sprite.frameWidth,
                    sprite.frameHeight
                );
                // fill with black
                context.globalAlpha = settings.alpha;
                context.globalCompositeOperation = 'source-atop';
                context.fillStyle = "#fff";
                context.fillRect(0, 0, sprite.frameWidth, sprite.frameHeight);

                packedImage = new PackedImage(canvas);
            },
            destroy: function () {
                if (canvas && canvas.dispose) {
                    canvas.dispose();
                }
            },
            update: function (data) {
                if (component.alpha > 0) {
                    component.alpha -= fadeSpeed;

                    // remove self
                    if (component.alpha <= 0) {
                        entity.remove(component);
                    }
                }
            },
            draw: function (data) {
                if (Utils.isNativeAndroid()) {
                    return;
                }
                if (!component.visible) {
                    return;
                }
                var oldAlpha = data.renderer.getOpacity();
                data.renderer.setOpacity(component.alpha);

                // draw canvas as a sprite
                data.renderer.drawImage(
                    packedImage,
                    0,
                    0,
                    packedImage.width,
                    packedImage.height,
                    sprite.origin.x * -1,
                    sprite.origin.y * -1,
                    sprite.frameWidth,
                    sprite.frameHeight
                );

                // restore alpha
                data.renderer.setOpacity(oldAlpha);
            },
            attached: function (data) {
                entity = data.entity;
            }
        };
        return component;
    };
});