/**
 * Camera behavior
 * @moduleName CameraBehavior
 */
bento.define('components/camerabehavior', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'managers/resolution'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween,
    ResolutionManager
) {
    'use strict';
    /**
     * Describe your settings object parameters
     * @param {Object} settings
     */
    return function (settings) {
        var viewport = Bento.getViewport();
        var camera;
        var movable;

        var portraitOffset = new Vector2(0, 0);
        var landscapeOffset = new Vector2(0, -40);

        var activeOffset = portraitOffset;

        var onResolutionChanged = function (e) {
            if (!component.target) {
                return;
            }

            if (e.orientation === 'portrait') {
                activeOffset = portraitOffset;
            } else {
                activeOffset = landscapeOffset;
            }

            //TODO: find difference between current target and actual camera position for a more accurate camera repositioning?
            camera.forceTo(component.target.position.add(activeOffset));

            //the camera position should update instantly, even if the game is paused
            camera.update();
        };

        var component = {
            name: 'cameraBehavior',
            target: null,
            start: function () {
                camera.speed.y = 0.1;
                camera.speed.x = 0.1;
                EventSystem.on('resolutionChanged', onResolutionChanged);
                EventSystem.on('playerSpawned', this.setTarget);

                if (ResolutionManager.getOrientation() === 'landscape') {
                    activeOffset = landscapeOffset;
                }
            },
            destroy: function () {
                EventSystem.off('resolutionChanged', onResolutionChanged);
                EventSystem.off('playerSpawned', this.setTarget);
            },
            setTarget: function (data) {
                component.target = data.player;
                movable = component.target.getComponent('movable');
            },
            update: function () {
                var forward = new Vector2(0, 0);
                if (!this.target) {
                    this.target = Utils.getLocalPlayer();
                    return;
                }
                if (movable) {
                    forward.x = 4 * movable.velocity.x;
                } else {
                    movable = this.target.getComponent('movable');
                    return;
                }
                camera.centerTo(this.target.position.add(forward).add(activeOffset));
            },
            attached: function (data) {
                camera = data.entity;
            }
        };
        return component;
    };
});