/**
 * Crate Behavior
 * @moduleName CrateBehavior
 */
bento.define('components/characters/crate', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'bento/components/sprite'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween,
    Sprite
) {
    'use strict';
    /**
     * Describe your settings object parameters
     * @param {Object} settings
     */
    return function (settings) {
        var viewport = Bento.getViewport();
        var entity;
        var sprite;
        var movable;
        var punchable;

        var isHurt = false;

        var onHit = function (punch) {
            if (isHurt) {
                return;
            }
            movable.velocity.x = punch.pushBack.x * punch.direction;
            movable.velocityZ = -punch.pushBack.y;
            isHurt = true;
        };

        var component = {
            name: 'crateBehavior',
            start: function (data) {},
            destroy: function (data) {},
            update: function (data) {
                if (movable.grounded) {
                    movable.velocity.x = Utils.approach(movable.velocity.x, 0, 0.15);
                    movable.velocity.y = Utils.approach(movable.velocity.y, 0, 0.15);

                    if (isHurt) {
                        isHurt = false;
                    }
                }
            },
            draw: function (data) {},
            attached: function (data) {
                entity = data.entity;
                sprite = entity.getComponent('sprite');
                movable = entity.getComponent('movable');
                punchable = entity.getComponent('punchable');

                punchable.boundingBox = new Rectangle(-8, -8, 16, 16);
                // punchable.debug = true;
                if (entity.family.indexOf('enemy') < 0) {
                    entity.family.push('enemy');
                }

                punchable.onHit = onHit;

                entity.remove(sprite);
                sprite = new Sprite({
                    imageName: 'crate'
                });
                entity.attach(sprite);

                entity.name = 'crate';
            }
        };
        return component;
    };
});