/**
 * Prize Crate.
 * There should be one at the end of each level. The prize crate holds a prize (surprise) and needs to be punched a set amount of times (regardless of player damage).
 * A prize then pops out, end the level end sequence is triggered
 *
 * TODO:
 * 		Multiplayer:
 * 			Sync breaking and prize
 * 			More hits required if there are more active people
 *
 * @moduleName PrizeCrateBehavior
 */
bento.define('components/characters/prizecrate', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'modules/matchmanager',
    'gui/menus/levelend',
    'managers/item',
    'globals'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween,
    MatchManager,
    LevelEndMenu,
    ItemManager,
    Globals
) {
    'use strict';
    /**
     * Describe your settings object parameters
     * @param {Object} settings
     */

    var Prize = function (itemKey) {
        var viewport = Bento.getViewport();
        var targetPositionComponent = {
            startPos: new Vector2(0, 0),
            targetPosition: new Vector2(0, 0),
            duration: 30,
            progress: 0,
            attached: function () {
                entity.z = Globals.Layers.PAUSE + 10;
                entity.float = true;
                entity.position.x -= viewport.x;
                entity.position.y -= viewport.y;

                this.startPos = entity.position.clone();

                var self = this;

                new Tween({
                    from: 0,
                    to: 1,
                    'in': this.duration,
                    ease: Tween.EASEINOUTCUBIC,
                    updateWhenPaused: Globals.PauseLevels.PAUSE,
                    onUpdate: function (v, t) {
                        self.progress = v;
                    }
                });

                new Tween({
                    from: 1,
                    to: 2,
                    'in': this.duration,
                    ease: Tween.EASEOUTELASTIC,
                    updateWhenPaused: Globals.PauseLevels.PAUSE,
                    onUpdate: function (v, t) {
                        entity.scale.x = v;
                        entity.scale.y = v;
                    }
                });
            },
            update: function () {
                entity.position.x = this.startPos.x + (this.targetPosition.x - this.startPos.x) * this.progress;
                entity.position.y = this.startPos.y + (this.targetPosition.y - this.startPos.y) * this.progress;
            }
        };

        var entity = new Entity({
            z: 0,
            name: 'prize',
            updateWhenPaused: Globals.PauseLevels.PAUSE,
            components: [
                ItemManager.getSprite(itemKey)
            ]
        }).extend({
            goToScreenPos: function (position, duration) {
                if (position) {
                    targetPositionComponent.targetPosition = position;
                }

                if (duration) {
                    targetPositionComponent.duration = duration;
                }
                entity.attach(targetPositionComponent);
            }
        });

        return entity;
    };

    return function (settings) {
        var viewport = Bento.getViewport();
        var entity;
        var punchable;
        var sprite;
        var multiScale;

        var punchesLeft = settings.punchesNeeded || 5;
        var spriteName = settings.spriteName || 'crate';

        var itemKey = settings.itemKey;

        var open = function () {
            var prize = new Prize(itemKey);
            var levelEnd;
            prize.position = entity.position.add(new Vector2(0, -4));
            prize.z = entity.z - 1 / viewport.height;

            EventSystem.fire('disableControls');

            //TODO	Pan camera to prizeCrate regardless of player position
            Bento.objects.getByName('camera', function (entities) {
                if (entities.length === 0) {
                    return;
                }
                entities[0].getComponent('cameraBehavior', function (camera) {
                    camera.target = entity;
                });
            });


            //remove crate
            if (entity.family.indexOf('none') < 0) {
                entity.family.push('none');
            }
            new Tween({
                from: 1,
                to: 0,
                'in': 30,
                ease: Tween.LINEAR,
                onUpdate: function (v, t) {
                    entity.alpha = v;
                },
                onComplete: function () {
                    Bento.objects.remove(entity);
                }
            });

            //show prize
            // Bento.objects.attach(prize);
            // new Tween({
            //     from: 0,
            //     to: 1,
            //     'in': 20,
            //     ease: Tween.EASEOUTBACK,
            //     onUpdate: function (v, t) {
            //         prize.scale.x = v;
            //         prize.scale.y = v;
            //     }
            // });

            // new Tween({
            //     from: -24,
            //     to: 0,
            //     'in': 40,
            //     ease: Tween.EASEOUTBOUNCE,
            //     onUpdate: function (v, t) {
            //         prize.transform.y = v;
            //     }
            // });
            // //Have prize float up
            // new Tween({
            //     from: prize.position.y,
            //     to: prize.position.y - 64,
            //     'in': 40,
            //     delay: 60,
            //     ease: Tween.EASEINOUTSINE,
            //     onUpdate: function (v, t) {
            //         prize.position.y = v;
            //     },
            //     onComplete: function () {
            //         Utils.timeout(30, function () {
            new Tween({
                from: 0,
                to: 1,
                in: 100,
                delay: 0,
                ease: 'linear',
                onComplete: function () {
                    levelEnd = new LevelEndMenu({
                            itemKey: itemKey
                        });
                    Bento.objects.attach(levelEnd);
                }
            });

                        // prize.goToScreenPos(levelEnd.getItemPosition(), 30);

            //         });
            //     }
            // });
        };

        var handleMessage = function (m) {
            if (m.type === 'key') {
                itemKey = m.data.k;
                console.log('prize set to', itemKey);
            }
        };

        var onHit = function (punch) {
            punchable.addHealth(punch.attack); //attacks are futile
            punchesLeft--;

            if (punchesLeft <= 0) {
                open();
            }

            //TODO shake crate a bit
            new Tween({
                from: 0.25,
                to: 0,
                'in': 45,
                ease: Tween.EASEOUTELASTIC,
                onUpdate: function (v, t) {
                    multiScale.hurt.x = 1 - v;
                    multiScale.hurt.y = 1 + v;
                }
            });
        };

        var startSync = function () {
            MatchManager.sendData({
                type: 'key',
                data: {
                    k: itemKey
                }
            }, 'prizeCrate'); //should only be one prizeCrate per level anyway

            EventSystem.off('startRoom', startSync);
        };

        var component = {
            name: 'prizeCrate',
            start: function (data) {
                if (MatchManager.isMaster) {
                    EventSystem.on('startRoom', startSync);
                } else {
                    MatchManager.onMessage('prizeCrate', handleMessage);
                }
            },
            destroy: function (data) {
                MatchManager.offMessage('prizeCrate', handleMessage);
            },
            update: function (data) {},
            draw: function (data) {},
            attached: function (data) {
                entity = data.entity;
                punchable = entity.getComponent('punchable');
                sprite = entity.getComponent('sprite');
                multiScale = entity.getComponent('multiScale');

                multiScale.addScale('hurt', new Vector2(1, 1));

                punchable.boundingBox = new Rectangle(-12, -8, 24, 16); //TODO

                punchable.setMaxHealth(99999); //basically immortal
                punchable.setMaxStamina(99999);

                punchable.onHit(onHit);
                if (entity.family.indexOf('destructible') < 0) {
                    entity.family.push('destructible');
                }

                sprite.setSpriteSheet('prizecrate/' + spriteName);
            }
        };
        return component;
    };
});