/**
 * Punch bot behavior. Just punches periodically
 * @moduleName PunchbotBehavior
 */
bento.define('components/characters/punchbot', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'bento/components/sprite',
    'entities/punch'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween,
    Sprite,
    Punch
) {
    'use strict';
    /**
     * Describe your settings object parameters
     * @param {Object} settings
     */
    return function (settings) {
        var viewport = Bento.getViewport();
        var entity;
        var sprite;
        var movable;
        var punchable;

        var isHurt = false;

        var punch = function () {
            sprite.setAnimation('punch', function () {
                sprite.setAnimation('idle');
                Utils.timeout(60, punch);
            });
            var p = new Punch({
                target: ['player'],
                attack: 1,
                direction: 1,
                freeze: 5,
                pushBack: new Vector2(1.5, 1.5),
                parent: entity,
                boundingBox: new Rectangle(0, -5, 24, 10),
                position: entity.position.add(new Vector2(0, 0)),
                onHit: function () {

                },
                // debug: true
            });
            Utils.timeout(8, function () {
                Bento.objects.attach(p);
            });
        };

        var component = {
            name: 'punchbotBehavior',
            start: function (data) {},
            destroy: function (data) {},
            update: function (data) {},
            draw: function (data) {},
            attached: function (data) {
                entity = data.entity;
                sprite = entity.getComponent('sprite');
                movable = entity.getComponent('movable');
                punchable = entity.getComponent('punchable');

                entity.remove(sprite);
                sprite = new Sprite({
                    imageName: 'carrot',
                    frameCountX: 3,
                    frameCountY: 3,
                    animations: {
                        idle: {
                            frames: [3]
                        },
                        punch: {
                            frames: [4, 5, 6, 7, 8],
                            speed: 0.2
                        }
                    }
                });
                entity.attach(sprite);

                punch();

                movable.boundingBox = new Rectangle(-8, -6, 16, 8);

                entity.name = 'punchbot';
            }
        };
        return component;
    };
});