/**
 * Coin counter that follows the player around
 * @moduleName CoinCounter
 */
bento.define('components/coincounter', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'managers/inventory',
    'gui/counter',
    'bento/components/sprite',
    'managers/save',
    'managers/coincollector',
    'modules/text',
    'components/leftshadow'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween,
    InventoryManager,
    Counter,
    Sprite,
    SaveManager,
    CoinCollector,
    Text,
    LeftShadow
) {
    'use strict';
    /**
     * Coin counter that follows the player around
     * @param {Object} settings Options for setting up this component.
     */
    return function (settings) {
        var viewport = Bento.getViewport();
        var screenMid = new Vector2(viewport.width / 2, viewport.height / 2);

        var entity;
        var countTween;
        var alphaTween;
        var alphaCounter = 0;
        var coinValue = CoinCollector.getCollected();
        var ySpacing = 10;

        /*var counter = new Text({
            position: new Vector2(10, 0),
            text: '',
            font: 'font',
            fontSize: 16,
            fontColor: '#ffffff',
            // gradient: 'top-down',
            // gradientColors: ['#fff21d', '#fff21d', '#ffffff', '#ffffff'],
            align: 'left',
            textBaseline: 'top',
            lineWidth: 1,
            strokeStyle: ['#2e1c6a'],
            innerStroke: false,
            pixelStroke: true // workaround for Cocoon bug
        }).attach(new LeftShadow({}));*/
        var counter = new Counter({
            name: 'damageCounter',
            image: Bento.assets.getImage('gui/counters/coin'),
            frameWidth: 12,
            frameHeight: 14,
            spacing: new Vector2(-4, 0),
            align: 'left',
            position: new Vector2(10, 2)
        });
        var coin = new Entity({
            name: 'coin',
            alpha: 0,
            components: [
                new Sprite({
                    originRelative: new Vector2(0.5, 0),
                    imageName: 'gui/coin'
                }),
                counter
            ]
        });
        var component = {
            name: 'coinCounterComponent',
            offset: new Vector2(0, 0),
            attached: function (data) {
                entity = data.entity;
            },
            start: function (data) {
                this.offset.x = 8;
                this.offset.y = entity.dimension.height + ySpacing;

                Bento.objects.attach(coin);
            },
            destroy: function (data) {
                Bento.objects.remove(coin);
            },
            update: function (data) {
                coin.position.x = entity.position.x - this.offset.x;
                coin.position.y = entity.position.y - this.offset.y;

                coin.z = entity.z;

                if (alphaCounter > 0) {
                    coin.visible = true;
                    alphaCounter -= data.speed;
                    if (alphaCounter < 45) {
                        coin.alpha = alphaCounter / 45;
                    } else {
                        coin.alpha = 1;
                    }
                } else {
                    coin.visible = false;
                }
            },
            addCoin: function (amount) {
                if (alphaTween) {
                    alphaTween.stop();
                    Bento.objects.remove(alphaTween);
                }

                // the collection of the actual coin is done in the Coin entity
                coinValue = CoinCollector.getCollected();
                // coinValue += amount;
                counter.setValue(coinValue);

                counter.alpha = 1;
                coin.alpha = 1;
                alphaCounter = 90;
            }
        };

        return component;
    };
});