/**
 * Coin Dropper drops coins when punchable dies
 * @moduleName CoinDropper
 */
bento.define('components/coindropper', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'entities/coin',
    'modules/randomizer',
    'managers/buff',
    'managers/save',
    'modules/matchmanager'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween,
    Coin,
    Randomizer,
    BuffManager,
    SaveManager,
    MatchManager
) {
    'use strict';
    /**
     * Describe your settings object parameters
     * @param {Object} settings
     */
    return function (settings) {
        var viewport = Bento.getViewport();
        var entity;
        var randomizer;
        var coinsToDrop = settings.coins || 0;
        var ignoredFamilies = settings.ignoredFamilies || [];
        var downOnly = settings.downOnly || false;
        var offset = settings.offset || new Vector2(0, 0);
        var speedMultiplier = settings.speedMultiplier || 1;
        var seed;
        // drop coins when dead
        var onDeath = function () {
            var count = coinsToDrop;
            var id = 0;
            var firstBreak = SaveManager.load('firstBreakable');
            if (count <= 0 && !firstBreak) {
                return;
            } else if (firstBreak) {
                // the first breakable that the player ever breaks always has the maximum amount of coins
                if (!MatchManager.isOnline) {
                    if (entity.name.indexOf('breakableProp') > -1) {
                        count = 7;
                        SaveManager.save('firstBreakable', false);
                    }
                }
            }
            Utils.repeat(count, function () {
                var uniqueName = seed + '-' + id;
                var speed = 0.5 + randomizer.getRandomFloat(0.5);
                var angle = randomizer.getRandomFloat(downOnly ? Math.PI : Math.PI * 2);
                var velocityZ = -2 - randomizer.getRandomFloat(2);
                var coin = new Coin({
                    name: 'coin-' + uniqueName,
                    position: entity.position.add(offset),
                    speed: speed * speedMultiplier,
                    angle: angle,
                    velocityZ: velocityZ,
                    ignoredFamilies: ignoredFamilies
                });
                Bento.objects.attach(coin);

                id += 1;
            });
        };
        var component = {
            name: 'coinDropper',
            start: function (data) {
                // create randomizer seed based on parent's name, which is deterministic
                seed = Utils.checksum(entity.name);
                randomizer = new Randomizer({
                    seed: seed
                });

                // register death on punchable
                entity.getComponent('punchable', function (punchable) {
                    punchable.onDeath(onDeath);
                });

                // not during tower
                if (SaveManager.load('isTower')) {
                    component.cancelCoinDrop();
                }
            },
            destroy: function () {},
            cancelCoinDrop: function () {
                // cancel coin drop
                entity.getComponent('punchable', function (punchable) {
                    punchable.offDeath(onDeath);
                });

            },
            drop: function () {
                // force drop
                onDeath();
            },
            attached: function (data) {
                entity = data.entity;
            }
        };
        return component;
    };
});