/**
 * Cutscene 1: meeting ratboss
 * @moduleName CutScene
 */
bento.define('components/cutscenes/cutscene1', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'entities/cutscene',
    'entities/character',
    'modules/localization'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween,
    Cutscene,
    Character,
    Localization
) {
    'use strict';
    var Scene = function (settings) {
        return {
            start: settings.start,
            update: settings.update
        };
    };
    return function (settings) {
        var viewport = Bento.getViewport();
        var cutscene; // the Cutscene instance
        var tiled = Cutscene.tiled; // reference to objects in tiled
        var nextScene; // call this to go to next scene
        var talk; // call this to make actor talk
        var focusOn; // pan camera to actor
        /*
         * Actors
         */
        var actors = {};
        var setupActors = function () {
            actors.lee = new Character({
                position: tiled['player'].position,
                spriteSheet: 'boy/idle'
            });
            actors.granny = new Character({
                position: tiled['granny'].position,
                spriteSheet: 'pedestrians/granny/idle'
            });
            actors.rat = new Character({
                position: tiled['rat'].position,
                spriteSheet: 'rat/idle',
                direction: -1
            });
            actors.ratboss = new Character({
                position: tiled['ratboss'].position,
                spriteSheet: 'ratboss/idle',
                direction: -1
            });

            // attach them
            Bento.objects.attach(actors.lee);
            Bento.objects.attach(actors.granny);
            Bento.objects.attach(actors.rat);
            Bento.objects.attach(actors.ratboss);

            // initial camera position and force it
            focusOn(actors.lee, true);
        };
        /*
         * Array of scenes
         */
        var scenes = [
            // scene1: granny is shocked and runs to lee
            new Scene({
                start: function () {
                    // pan to granny
                    focusOn(actors.granny);

                    // make a little jump
                    Utils.timeout(30, function () {
                        actors.granny.setSpriteSheet('pedestrians/granny/flee');
                        actors.granny.getComponent('movable', function (movable) {
                            movable.velocityZ = -2;

                            // run towards player
                            movable.onLanded = function () {
                                new Tween({
                                    from: actors.granny.position,
                                    to: actors.lee.position.add(new Vector2(32, 0)),
                                    in: 40,
                                    onUpdate: function (v, t) {
                                        actors.granny.position = v;
                                        actors.granny.setDirection(-1);
                                    },
                                    onComplete: function () {
                                        actors.granny.setSpriteSheet('pedestrians/granny/idle');
                                        nextScene();
                                    }
                                });

                            };
                        });
                    });
                }
            }),
            // scene2: granny and lee talk a bunch of stuff
            new Scene({
                start: function () {
                    talk({
                        text: Localization.getText('tutorial')['dialogue13'],
                        who: actors.granny
                    });
                    talk({
                        text: Localization.getText('tutorial')['dialogue14'],
                        who: actors.granny
                    });
                    talk({
                        text: Localization.getText('tutorial')['dialogue15'],
                        who: actors.granny
                    });
                    talk({
                        text: Localization.getText('tutorial')['dialogue16'],
                        who: actors.granny
                    });
                    talk({
                        text: Localization.getText('tutorial')['dialogue17'],
                        who: actors.lee,
                        onComplete: function () {
                            nextScene();
                        }
                    });
                }
            }),
            // scene3: granny runs off
            new Scene({
                start: function () {
                    // sequenced tweens
                    var walkDown = function () {
                        new Tween({
                            from: actors.granny.position,
                            to: actors.granny.position.add(new Vector2(0, 32)),
                            in: 30,
                            onUpdate: function (v, t) {
                                actors.granny.position = v;
                            },
                            onComplete: grannyTalk
                        });
                    };
                    var grannyTalk = function () {
                        talk({
                            text: Localization.getText('tutorial')['dialogue18'],
                            who: actors.granny,
                            onComplete: runOff
                        });
                    };
                    var runOff = function () {
                        new Tween({
                            from: actors.granny.position.x,
                            to: -32,
                            in: 40,
                            ease: 'linear',
                            onStart: function () {
                                actors.granny.setSpriteSheet('pedestrians/granny/flee');
                            },
                            onUpdate: function (v, t) {
                                actors.granny.position.x = v;
                            },
                            onComplete: nextScene
                        });
                    };

                    walkDown();
                }
            }),
            // scene4: this is my chance to be a hero
            new Scene({
                start: function () {
                    focusOn(actors.lee);
                    talk({
                        text: Localization.getText('tutorial')['dialogue19'],
                        who: actors.lee,
                        onComplete: nextScene
                    });
                }
            }),
            // scene5: ratboss talks
            new Scene({
                start: function () {
                    var eatCheese = function () {
                        actors.ratboss.setSpriteSheet('ratboss/eatcheese', ratbossRun);
                    };
                    var ratbossRun = function () {
                        actors.ratboss.setSpriteSheet('ratboss/walk');
                        actors.ratboss.getComponent('multiScale', function (multiScale) {
                            multiScale.direction.x = 1;
                        });
                        actors.ratboss.getComponent('movable', function (movable) {
                            movable.velocity.x = 3;
                        });

                        Utils.timeout(30, nextScene);
                    };
                    focusOn(actors.ratboss);

                    Utils.timeout(30, function () {
                        talk({
                            text: Localization.getText('tutorial')['dialogue20'],
                            who: actors.ratboss,
                            offset: new Vector2(0, -8)
                        });
                        talk({
                            text: Localization.getText('tutorial')['dialogue21'],
                            who: actors.ratboss
                        });
                        talk({
                            text: Localization.getText('tutorial')['dialogue22'],
                            who: actors.ratboss,
                            onComplete: eatCheese
                        });
                    });
                }
            })
        ];

        var component = {
            name: 'cutscene', // should always be called 'cutscene'
            startCutscene: function (data) {
                Utils.forEach(scenes, function (scene, i, l, breakLoop) {
                    cutscene.addScene(scene);
                });
                nextScene();
            },
            setupActors: function () {
                setupActors();
            },
            attached: function (data) {
                cutscene = data.entity;
                nextScene = cutscene.nextScene;
                talk = cutscene.talk;
                focusOn = cutscene.focusOn;
            }
        };
        return component;
    };
});