/**
 * Cutscene 10: Lee reunites with ratboss at long last
 * @moduleName CutScene
 */
bento.define('components/cutscenes/cutscene10', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'entities/cutscene',
    'entities/character',
    'modules/localization'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween,
    Cutscene,
    Character,
    Localization
) {
    'use strict';
    var Scene = function (settings) {
        return {
            start: settings.start,
            update: settings.update
        };
    };
    return function (settings) {
        var viewport = Bento.getViewport();
        var cutscene; // the Cutscene instance
        var tiled = Cutscene.tiled; // reference to objects in tiled
        var nextScene; // call this to go to next scene
        var talk; // call this to make actor talk
        var focusOn; // pan camera to actor
        var dummy = new Entity({}); // dummy entity for camera forcing
        /*
         * Actors
         */
        var actors = {};
        var setupActors = function () {
            actors.lee = new Character({
                position: tiled['lee'].position,
                spriteSheet: 'boy/idle'
            });
            actors.ratboss = new Character({
                position: tiled['ratboss'].position,
                spriteSheet: 'ratboss/idle',
                direction: -1
            });

            // attach them
            Bento.objects.attach(actors.lee);
            Bento.objects.attach(actors.ratboss);

            // initial camera position and force it
            dummy.position = tiled['scenecam'].position.clone();
            focusOn(dummy, true);
        };
        /*
         * Array of scenes
         */
        var scenes = [
            // scene1: Lee and ratboss bants
            new Scene ({
                start: function () {
                    talk({
                        text: Localization.getText('cutscene10')['lee01'],
                        who: actors.lee,
                        onComplete: function () {
                            actors.ratboss.setSpriteSheet('ratboss/hurt1');
                        }
                    });
                    talk({
                        text: Localization.getText('cutscene10')['ratboss02'],
                        who: actors.ratboss,
                        waitTime: 60,
                        onComplete: function () {
                            actors.ratboss.setSpriteSheet('ratboss/idle');
                        }
                    });
                    talk({
                        text: Localization.getText('cutscene10')['ratboss03'],
                        who: actors.ratboss,
                        onComplete: function () {
                            nextScene();
                        }
                    });
                }
            }),
            new Scene ({
               start: function () {
                    actors.ratboss.setDirection(1);
                    new Tween({
                        from: 0,
                        to: 0,
                        in: 30,
                        onComplete: function () {
                            actors.ratboss.setSpriteSheet('ratboss/walk');
                            new Tween({
                                from: actors.ratboss.position.clone(),
                                to: actors.ratboss.position.add(new Vector2(160, 0)),
                                in: 120,
                                onUpdate: function (v, t) {
                                    actors.ratboss.position = v;
                                },
                                onComplete: function () {
                                    nextScene();
                                }
                            });
                            talk({
                                text: Localization.getText('cutscene10')['ratboss04'],
                                who: actors.ratboss
                            });
                        }
                    });
               } 
            }),
        ];

        var component = {
            name: 'cutscene', // should always be called 'cutscene'
            startCutscene: function (data) {
                Utils.forEach(scenes, function (scene, i, l, breakLoop) {
                    cutscene.addScene(scene);
                });
                nextScene();
            },
            setupActors: function () {
                setupActors();
            },
            attached: function (data) {
                cutscene = data.entity;
                nextScene = cutscene.nextScene;
                talk = cutscene.talk;
                focusOn = cutscene.focusOn;
            }
        };
        return component;
    };
});