/**
 * Cutscene 11: Lee and Tas find Fifty Kicks
 * @moduleName CutScene
 */
bento.define('components/cutscenes/cutscene11', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'entities/cutscene',
    'entities/character',
    'modules/localization'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween,
    Cutscene,
    Character,
    Localization
) {
    'use strict';
    var Scene = function (settings) {
        return {
            start: settings.start,
            update: settings.update
        };
    };
    return function (settings) {
        var viewport = Bento.getViewport();
        var cutscene; // the Cutscene instance
        var tiled = Cutscene.tiled; // reference to objects in tiled
        var nextScene; // call this to go to next scene
        var talk; // call this to make actor talk
        var focusOn; // pan camera to actor
        var dummy = new Entity({}); // dummy entity for camera forcing
        /*
         * Actors
         */
        var actors = {};
        var setupActors = function () {
            actors.lee = new Character({
                position: tiled['lee'].position,
                spriteSheet: 'boy/idle'
            });
            actors.tas = new Character({
                position: tiled['tas'].position,
                spriteSheet: 'boxer/idle'
            });
            actors.fiftykicks = new Character({
                position: tiled['fiftykicks'].position,
                spriteSheet: 'gangstaboss/idle',
                direction: -1
            });
            actors.rat1 = new Character({
                position: tiled['rat1'].position,
                spriteSheet: 'punker/idle',
                direction: -1
            });
            actors.rat2 = new Character({
                position: tiled['rat2'].position,
                spriteSheet: 'rat/idle',
                direction: -1
            });
            // attach them
            Bento.objects.attach(actors.lee);
            Bento.objects.attach(actors.tas);
            Bento.objects.attach(actors.fiftykicks);
            Bento.objects.attach(actors.rat1);
            Bento.objects.attach(actors.rat2);

            // initial camera position and force it
            dummy.position = tiled['campos'].position.clone();
            focusOn(actors.lee, true);
        };
        /*
         * Array of scenes
         */
        var scenes = [
            // scene1: Lee runs up to fifty kicks
            new Scene ({
                start: function () {
                    actors.lee.setSpriteSheet('boy/run');
                    new Tween({
                        from: actors.lee.position.clone(),
                        to: tiled['leerunpos'].position.clone(),
                        in: 60,
                        onUpdate: function (v, t) {
                            actors.lee.position = v;
                        },
                        onComplete: function () {
                            actors.lee.setSpriteSheet('boy/idle');
                            focusOn(dummy, false);
                        }
                    });
                    actors.tas.setSpriteSheet('boxer/run');
                    new Tween({
                        from: actors.tas.position.clone(),
                        to: tiled['tasrunpos'].position.clone(),
                        in: 60,
                        onUpdate: function (v, t) {
                            actors.tas.position = v;
                        },
                        onComplete: function () {
                            actors.tas.setSpriteSheet('boxer/idle');
                            focusOn(dummy, false);
                            nextScene();
                        }
                    });
                }
            }),
            // scene2: Brief altercation
            new Scene ({
               start: function () {
                    talk({
                        text: Localization.getText('cutscene11')['fiftykicks01'],
                        who: actors.fiftykicks,
                        waitTime: 180,
                        offset: new Vector2(0, -16)
                    });
                    talk({
                        text: Localization.getText('cutscene11')['tas02'],
                        who: actors.tas,
                        waitTime: 180,
                        onComplete: nextScene
                    });
               } 
            }),
            // scene3: Goons speak up
            new Scene ({
                start: function () {
                    actors.rat2.setSpriteSheet('rat/walk');
                    new Tween({
                        from: actors.rat2.position.clone(),
                        to: actors.rat2.position.add(new Vector2(-24, 0)),
                        in: 10,
                        onUpdate: function (v, t) {
                            actors.rat2.position = v;
                        },
                        onComplete: function () {
                           actors.rat2.setSpriteSheet('rat/idle'); 
                        }
                    });
                    talk({
                        text: Localization.getText('cutscene11')['rat03'],
                        waitTime: 180,
                        who: actors.rat2,
                        onComplete: function () {
                            actors.rat1.setSpriteSheet('punker/walk');
                            new Tween({
                                from: actors.rat1.position.clone(),
                                to: actors.rat1.position.add(new Vector2(-16, 0)),
                                in: 10,
                                onUpdate: function (v, t) {
                                    actors.rat1.position = v;
                                },
                                onComplete: function () {
                                   actors.rat1.setSpriteSheet('punker/idle'); 
                                }
                            });
                        }
                    });
                    talk({
                        text: Localization.getText('cutscene11')['rat04'],
                        waitTime: 180,
                        who: actors.rat1
                    });
                    talk({
                        text: Localization.getText('cutscene11')['fiftykicks05'],
                        who: actors.fiftykicks,
                        onComplete: function () {
                            actors.fiftykicks.setDirection(1);
                            nextScene();
                        }
                    });
                } 
            }),
            new Scene ({
                start: function () {
                    new Tween({
                        from: 0,
                        to: 1,
                        in: 30,
                        onComplete: function () {
                            talk({
                                text: Localization.getText('cutscene11')['fiftykicks06'],
                                who: actors.fiftykicks,
                                onComplete: function () {
                                    nextScene();
                                }
                            });
                            actors.fiftykicks.setSpriteSheet('gangstaboss/walk');
                            new Tween({
                                from: actors.fiftykicks.position.clone(),
                                to: actors.fiftykicks.position.add(new Vector2(256, 0)),
                                in: 120,
                                onUpdate: function (v, t) {
                                    actors.fiftykicks.position = v;
                                }
                            });
                        }
                    });
                }
            })
        ];

        var component = {
            name: 'cutscene', // should always be called 'cutscene'
            startCutscene: function (data) {
                Utils.forEach(scenes, function (scene, i, l, breakLoop) {
                    cutscene.addScene(scene);
                });
                nextScene();
            },
            setupActors: function () {
                setupActors();
            },
            attached: function (data) {
                cutscene = data.entity;
                nextScene = cutscene.nextScene;
                talk = cutscene.talk;
                focusOn = cutscene.focusOn;
            }
        };
        return component;
    };
});