/**
 * Cutscene 12: Lee and Tas have beaten Fifty Kicks
 * @moduleName CutScene
 */
bento.define('components/cutscenes/cutscene12', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'entities/cutscene',
    'entities/character',
    'modules/localization'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween,
    Cutscene,
    Character,
    Localization
) {
    'use strict';
    var Scene = function (settings) {
        return {
            start: settings.start,
            update: settings.update
        };
    };
    return function (settings) {
        var viewport = Bento.getViewport();
        var cutscene; // the Cutscene instance
        var tiled = Cutscene.tiled; // reference to objects in tiled
        var nextScene; // call this to go to next scene
        var talk; // call this to make actor talk
        var focusOn; // pan camera to actor
        var dummy = new Entity({}); // dummy entity for camera forcing
        /*
         * Actors
         */
        var actors = {};
        var setupActors = function () {
            actors.lee = new Character({
                position: tiled['lee'].position,
                spriteSheet: 'boy/idle'
            });
            actors.tas = new Character({
                position: tiled['tas'].position,
                spriteSheet: 'boxer/idle'
            });
            actors.fiftykicks = new Character({
                position: tiled['fiftykicks'].position,
                spriteSheet: 'gangstaboss/dizzy',
                direction: -1
            });
            // attach them
            Bento.objects.attach(actors.lee);
            Bento.objects.attach(actors.tas);
            Bento.objects.attach(actors.fiftykicks);

            // initial camera position and force it
            dummy.position = tiled['campos'].position.clone();
            focusOn(dummy, true);
        };
        /*
         * Array of scenes
         */
        var scenes = [
            // scene1: Lee runs up to fifty kicks
            new Scene ({
                start: function () {
                    talk({
                        text: Localization.getText('cutscene12')['fiftykicks01'],
                        who: actors.fiftykicks,
                        waitTime: 120,
                        offset: new Vector2(0, -16),
                        onComplete: function () {
                            actors.fiftykicks.setDirection(1);
                        }
                    });
                    talk({
                        text: Localization.getText('cutscene12')['fiftykicks02'],
                        who: actors.fiftykicks,
                        waitTime: 120,
                        offset: new Vector2(0, -16),
                        onComplete: function () {
                            actors.fiftykicks.setSpriteSheet('gangstaboss/walk');
                            new Tween({
                                from: actors.fiftykicks.position.clone(),
                                to: actors.fiftykicks.position.add(new Vector2(256, 0)),
                                in: 180,
                                onUpdate: function (v, t) {
                                    actors.fiftykicks.position = v;
                                }
                            });
                            new Tween({
                                from: 0,
                                to: 0,
                                in: 90,
                                onComplete: function () {
                                    nextScene();
                                }
                            });
                        }
                    });
                }
            }),
            new Scene ({
                start: function () {
                    focusOn(actors.lee, false);
                    actors.lee.setSpriteSheet('boy/walk');
                    new Tween({
                        from: actors.lee.position.clone(),
                        to: actors.lee.position.add(new Vector2(32, 0)),
                        in: 60,
                        onUpdate: function (v, t) {
                            actors.lee.position = v;
                        },
                        onComplete: function () {
                            actors.lee.setSpriteSheet('boy/idle');
                        }
                    });
                    talk({
                        text: Localization.getText('cutscene12')['lee03'],
                        who: actors.lee
                    });
                    talk({
                        text: Localization.getText('cutscene12')['tas04'],
                        who: actors.tas,
                        waitTime: 240
                    });
                    talk({
                        text: Localization.getText('cutscene12')['tas05'],
                        who: actors.tas,
                        onComplete: function () {
                            actors.tas.setSpriteSheet('boxer/tired');
                        }
                    });
                    talk({
                        text: Localization.getText('cutscene12')['tas06'],
                        who: actors.tas,
                        onComplete: nextScene
                    });
                }
            })
        ];

        var component = {
            name: 'cutscene', // should always be called 'cutscene'
            startCutscene: function (data) {
                Utils.forEach(scenes, function (scene, i, l, breakLoop) {
                    cutscene.addScene(scene);
                });
                nextScene();
            },
            setupActors: function () {
                setupActors();
            },
            attached: function (data) {
                cutscene = data.entity;
                nextScene = cutscene.nextScene;
                talk = cutscene.talk;
                focusOn = cutscene.focusOn;
            }
        };
        return component;
    };
});