/**
 * Cutscene 13: Lee reunites with Varka
 * @moduleName CutScene
 */
bento.define('components/cutscenes/cutscene13', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'entities/cutscene',
    'entities/character',
    'modules/localization'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween,
    Cutscene,
    Character,
    Localization
) {
    'use strict';
    var Scene = function (settings) {
        return {
            start: settings.start,
            update: settings.update
        };
    };
    return function (settings) {
        var viewport = Bento.getViewport();
        var cutscene; // the Cutscene instance
        var tiled = Cutscene.tiled; // reference to objects in tiled
        var nextScene; // call this to go to next scene
        var talk; // call this to make actor talk
        var focusOn; // pan camera to actor
        var dummy = new Entity({}); // dummy entity for camera forcing
        /*
         * Actors
         */
        var actors = {};
        var setupActors = function () {
            actors.lee = new Character({
                position: tiled['lee'].position,
                spriteSheet: 'boy/idle'
            });
            actors.pigchef = new Character({
                position: tiled['varka'].position,
                spriteSheet: 'pigchef/idle'
            });
            // attach them
            Bento.objects.attach(actors.lee);
            Bento.objects.attach(actors.pigchef);

            // initial camera position and force it
            dummy.position = tiled['scenecam'].position.clone();
            focusOn(dummy, true);
        };
        /*
         * Array of scenes
         */
        var scenes = [
            // scene1: Lee runs up to varka
            new Scene ({
                start: function () {
                    actors.lee.setSpriteSheet('boy/run');
                    new Tween({
                        from: actors.lee.position.clone(),
                        to:  tiled['leepos'].position.clone(),
                        in: 90,
                        onUpdate: function (v, t) {
                            actors.lee.position = v;
                        },
                        onComplete: function () {
                            actors.lee.setSpriteSheet('boy/idle');
                            actors.pigchef.setDirection(-1);
                            actors.pigchef.getComponent('movable', function (movable) {
                                movable.velocityZ = -1;
                            });
                            nextScene();
                        }
                    });
                }
            }),
            // scene2: Varka oneliner
            new Scene ({
                start: function () {
                    new Tween({
                        from: 0,
                        to: 1,
                        in: 20,
                        onComplete: function () {
                            actors.pigchef.setSpriteSheet('pigchef/laugh');
                            talk({
                                text: Localization.getText('cutscene13')['pigchef01'],
                                who: actors.pigchef,
                                onComplete: function () {
                                    actors.pigchef.setSpriteSheet('pigchef/idle');
                                }
                            });
                            talk({
                                text: Localization.getText('cutscene13')['pigchef02'],
                                waitTime: 240,
                                who: actors.pigchef,
                                onComplete: function () {
                                    nextScene();
                                }
                            });
                        }
                    });
                }
            }),
            // scene3: Lee oneliner
            new Scene ({
                start: function () {
                    actors.lee.setSpriteSheet('boy/walk');
                    new Tween({
                        from: actors.lee.position.clone(),
                        to:  actors.lee.position.add(new Vector2(24, 0)),
                        in: 30,
                        onUpdate: function (v, t) {
                            actors.lee.position = v;
                        },
                        onComplete: function () {
                            actors.lee.setSpriteSheet('boy/idle');
                            actors.pigchef.setSpriteSheet('pigchef/hurt1');
                        }
                    });
                    talk({
                        text: Localization.getText('cutscene13')['lee03'],
                        waitTime: 180,
                        who: actors.lee,
                        onComplete: function () {
                            nextScene();
                            actors.pigchef.setSpriteSheet('pigchef/idle');
                        }
                    });
                }
            }),
            // scene4: Varka bounce away
            new Scene ({
                start: function () {
                    actors.pigchef.setDirection(1);
                    talk({
                        text: Localization.getText('cutscene13')['pigchef04'],
                        waitTime: 90,
                        who: actors.pigchef,
                        onComplete: function () {
                            actors.pigchef.setSpriteSheet('pigchef/jump-windup',function () {
                                focusOn(actors.lee, false);
                                actors.pigchef.getComponent('movable', function (movable) {
                                    actors.pigchef.setSpriteSheet('pigchef/jump-air');
                                    movable.velocity.x = 3;
                                    movable.velocityZ = -6;
                                });
                            });
                            new Tween({
                                from: 0,
                                to: 1,
                                in: 90,
                                onComplete: function () {
                                    nextScene();
                                }
                            });
                        }
                    });
                }
            })
        ];
        var component = {
            name: 'cutscene', // should always be called 'cutscene'
            startCutscene: function (data) {
                Utils.forEach(scenes, function (scene, i, l, breakLoop) {
                    cutscene.addScene(scene);
                });
                nextScene();
            },
            setupActors: function () {
                setupActors();
            },
            attached: function (data) {
                cutscene = data.entity;
                nextScene = cutscene.nextScene;
                talk = cutscene.talk;
                focusOn = cutscene.focusOn;
            }
        };
        return component;
    };
});