/**
 * Cutscene 14: Final showdown with Fiftykicks
 * @moduleName CutScene
 */
bento.define('components/cutscenes/cutscene14', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'entities/cutscene',
    'entities/character',
    'modules/localization'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween,
    Cutscene,
    Character,
    Localization
) {
    'use strict';
    var Scene = function (settings) {
        return {
            start: settings.start,
            update: settings.update
        };
    };
    return function (settings) {
        var viewport = Bento.getViewport();
        var cutscene; // the Cutscene instance
        var tiled = Cutscene.tiled; // reference to objects in tiled
        var nextScene; // call this to go to next scene
        var talk; // call this to make actor talk
        var focusOn; // pan camera to actor
        var dummy = new Entity({}); // dummy entity for camera forcing
        /*
         * Actors
         */
        var actors = {};
        var setupActors = function () {
            actors.lee = new Character({
                position: tiled['lee'].position,
                spriteSheet: 'boy/idle'
            });
            actors.tas = new Character({
                position: tiled['tas'].position,
                spriteSheet: 'boxer/idle'
            });
            actors.posse1 = new Character({
                position: tiled['posse1'].position,
                spriteSheet: 'twerka/idle',
                direction: -1
            });
            actors.posse2 = new Character({
                position: tiled['posse2'].position,
                spriteSheet: 'twerka/idle',
                direction: -1
            });
            actors.fiftykicks = new Character({
                position: tiled['fiftykicks'].position,
                spriteSheet: 'gangstaboss/idle',
                direction: -1
            });
            // attach them
            Bento.objects.attach(actors.lee);
            Bento.objects.attach(actors.tas);
            Bento.objects.attach(actors.posse1);
            Bento.objects.attach(actors.posse2);
            Bento.objects.attach(actors.fiftykicks);

            // initial camera position and force it
            dummy.position = tiled['scenecam'].position.clone();
            focusOn(actors.lee, true);
        };
        /*
         * Array of scenes
         */
        var scenes = [
            // scene1: Lee and runs up to 50k
            new Scene ({
                start: function () {
                    actors.lee.setSpriteSheet('boy/run');
                    new Tween({
                        from: actors.lee.position.clone(),
                        to:  tiled['leerunpos'].position.clone(),
                        in: 90,
                        onUpdate: function (v, t) {
                            actors.lee.position = v;
                        },
                        onComplete: function () {
                            actors.lee.setSpriteSheet('boy/idle');
                            nextScene();
                        }
                    });
                    actors.tas.setSpriteSheet('boxer/run');
                    new Tween({
                        from: actors.tas.position.clone(),
                        to:  tiled['tasrunpos'].position.clone(),
                        in: 90,
                        onUpdate: function (v, t) {
                            actors.tas.position = v;
                        },
                        onComplete: function () {
                            actors.tas.setSpriteSheet('boxer/idle');
                        }
                    });
                }
            }),
            // scene2: Lee and runs up to 50k
            new Scene ({
                start: function () {
                    focusOn(dummy, false);
                    actors.fiftykicks.setSpriteSheet('gangstaboss/punch1',function () {
                        actors.fiftykicks.setSpriteSheet('gangstaboss/idle');
                    });
                    talk({
                        text: Localization.getText('cutscene14')['fiftykicks01'],
                        who: actors.fiftykicks,
                        offset: new Vector2(0, -16),
                        waitTime: 240,
                        onComplete: function () {
                            actors.tas.setSpriteSheet('boxer/walk');
                            new Tween({
                                from: actors.tas.position.clone(),
                                to:  actors.tas.position.add(new Vector2(32, 0)),
                                in: 60,
                                onUpdate: function (v, t) {
                                    actors.tas.position = v;
                                },
                                onComplete: function () {
                                    actors.tas.setSpriteSheet('boxer/idle');
                                }
                            });
                        }
                    });
                    talk({
                        text: Localization.getText('cutscene14')['tas02'],
                        who: actors.tas,
                        waitTime: 240
                    });
                    talk({
                        text: Localization.getText('cutscene14')['lee03'],
                        who: actors.lee,
                        onComplete: function () {
                            new Tween({
                                from: 0,
                                to: 1,
                                in: 30,
                                waitTime: 300,
                                onComplete: function () {
                                    actors.tas.setDirection(-1);
                                    talk({
                                        text: Localization.getText('cutscene14')['tas04'],
                                        who: actors.tas,
                                        onComplete: function () {
                                            nextScene();
                                        }
                                    });
                                }
                            });
                        }
                    });
                }
            }),
            new Scene ({
                start: function () {
                    talk({
                        text: Localization.getText('cutscene14')['fiftykicks05'],
                        who: actors.fiftykicks,
                        offset: new Vector2(0, -16),
                        waitTime: 120,
                        onComplete: function () {
                            actors.tas.setDirection(1);
                        }
                    });
                    talk({
                        text: Localization.getText('cutscene14')['tas05'],
                        who: actors.tas,
                        waitTime: 180,
                        onComplete: function () {
                            nextScene();
                        }
                    });
                }
            })
        ];
        var component = {
            name: 'cutscene', // should always be called 'cutscene'
            startCutscene: function (data) {
                Utils.forEach(scenes, function (scene, i, l, breakLoop) {
                    cutscene.addScene(scene);
                });
                nextScene();
            },
            setupActors: function () {
                setupActors();
            },
            attached: function (data) {
                cutscene = data.entity;
                nextScene = cutscene.nextScene;
                talk = cutscene.talk;
                focusOn = cutscene.focusOn;
            }
        };
        return component;
    };
});