/**
 * Cutscene 15: Fiftykicks is ded
 * @moduleName CutScene
 */
bento.define('components/cutscenes/cutscene15', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'entities/cutscene',
    'entities/character',
    'modules/localization'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween,
    Cutscene,
    Character,
    Localization
) {
    'use strict';
    var Scene = function (settings) {
        return {
            start: settings.start,
            update: settings.update
        };
    };
    return function (settings) {
        var viewport = Bento.getViewport();
        var cutscene; // the Cutscene instance
        var tiled = Cutscene.tiled; // reference to objects in tiled
        var nextScene; // call this to go to next scene
        var talk; // call this to make actor talk
        var focusOn; // pan camera to actor
        var dummy = new Entity({}); // dummy entity for camera forcing
        /*
         * Actors
         */
        var actors = {};
        var setupActors = function () {
            actors.lee = new Character({
                position: tiled['lee'].position,
                spriteSheet: 'boy/idle'
            });
            actors.tas = new Character({
                position: tiled['tas'].position,
                spriteSheet: 'boxer/idle'
            });
            actors.fiftykicks = new Character({
                position: tiled['fiftykicks'].position,
                spriteSheet: 'gangstaboss/dizzy',
                direction: -1
            });
            // attach them
            Bento.objects.attach(actors.lee);
            Bento.objects.attach(actors.tas);
            Bento.objects.attach(actors.fiftykicks);

            // initial camera position and force it
            dummy.position = tiled['scenecam'].position.clone();
            focusOn(dummy, true);
        };
        /*
         * Array of scenes
         */
        var scenes = [
            // scene1: 50k goes down
            new Scene ({
                start: function () {
                    talk({
                        text: Localization.getText('cutscene15')['fiftykicks01'],
                        waitTime: 120,
                        who: actors.fiftykicks,
                        onComplete: function () {
                            nextScene();
                        }
                    });
                }
            }),
            // scene2: 50k goes down
            new Scene ({
                start: function () {
                    new Tween({
                        from: 0,
                        to: 1,
                        in: 60,
                        onComplete: function () {
                            actors.fiftykicks.setSpriteSheet('gangstaboss/flyup');
                            actors.fiftykicks.getComponent('movable', function (movable) {
                                movable.velocityZ = -2;
                                // run towards player
                                movable.onLanded = function () {
                                    movable.onLanded = function () {};
                                    movable.velocityZ = -1;
                                    actors.fiftykicks.setSpriteSheet('gangstaboss/liedown');
                                };
                            });
                            new Tween({
                                from: 0,
                                to: 1,
                                in: 10,
                                onComplete: function () {
                                    actors.fiftykicks.setSpriteSheet('gangstaboss/flydown');
                                }
                            });
                            new Tween({
                                from: 0,
                                to: 1,
                                in: 60,
                                onComplete: function () {
                                    focusOn(actors.tas, false);
                                    nextScene();
                                }
                            });
                        }
                    });
                }
            }),
            new Scene ({
                start: function () {
                    talk({
                        text: Localization.getText('cutscene15')['tas02'],
                        waitTime: 120,
                        who: actors.tas,
                        onComplete: function () {
                            actors.tas.setDirection(-1);
                        }
                    });
                    actors.tas.setSpriteSheet('boxer/walk');
                    new Tween({
                        from: actors.tas.position.clone(),
                        to:  actors.tas.position.add(new Vector2(32, 0)),
                        in: 60,
                        onUpdate: function (v, t) {
                            actors.tas.position = v;
                        },
                        onComplete: function () {
                            actors.tas.setSpriteSheet('boxer/idle');
                        }
                    });
                    talk({
                        text: Localization.getText('cutscene15')['tas03'],
                        waitTime: 120,
                        who: actors.tas
                    });
                    talk({
                        text: Localization.getText('cutscene15')['lee04'],
                        waitTime: 120,
                        who: actors.lee
                    });
                    talk({
                        text: Localization.getText('cutscene15')['tas05'],
                        waitTime: 300,
                        who: actors.tas
                    });
                    talk({
                        text: Localization.getText('cutscene15')['tas06'],
                        waitTime: 300,
                        who: actors.tas
                    });
                    talk({
                        text: Localization.getText('cutscene15')['lee07'],
                        waitTime: 180,
                        who: actors.lee,
                        onComplete: nextScene
                    });
                }
            }),
        ];
        var component = {
            name: 'cutscene', // should always be called 'cutscene'
            startCutscene: function (data) {
                Utils.forEach(scenes, function (scene, i, l, breakLoop) {
                    cutscene.addScene(scene);
                });
                nextScene();
            },
            setupActors: function () {
                setupActors();
            },
            attached: function (data) {
                cutscene = data.entity;
                nextScene = cutscene.nextScene;
                talk = cutscene.talk;
                focusOn = cutscene.focusOn;
            }
        };
        return component;
    };
});