/**
 * Cutscene 16: Found Varka
 * @moduleName CutScene
 */
bento.define('components/cutscenes/cutscene16', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'entities/cutscene',
    'entities/character',
    'modules/localization'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween,
    Cutscene,
    Character,
    Localization
) {
    'use strict';
    var Scene = function (settings) {
        return {
            start: settings.start,
            update: settings.update
        };
    };
    return function (settings) {
        var viewport = Bento.getViewport();
        var cutscene; // the Cutscene instance
        var tiled = Cutscene.tiled; // reference to objects in tiled
        var nextScene; // call this to go to next scene
        var talk; // call this to make actor talk
        var focusOn; // pan camera to actor
        var dummy = new Entity({}); // dummy entity for camera forcing
        /*
         * Actors
         */
        var actors = {};
        var setupActors = function () {
            actors.lee = new Character({
                position: tiled['lee'].position,
                spriteSheet: 'boy/idle'
            });
            actors.rin = new Character({
                position: tiled['rin'].position,
                spriteSheet: 'krillin/idle'
            });
            actors.pigchef = new Character({
                position: tiled['pigchef'].position,
                spriteSheet: 'pigchef/laugh',
                direction: -1
            });
            // attach them
            Bento.objects.attach(actors.lee);
            Bento.objects.attach(actors.rin);
            Bento.objects.attach(actors.pigchef);

            // initial camera position and force it
            dummy.position = tiled['scenecam'].position.clone();
            focusOn(dummy, true);
        };
        /*
         * Array of scenes
         */
        var scenes = [
            // scene1: rin and lee run up to varka
            new Scene ({
                start: function () {
                    actors.lee.setSpriteSheet('boy/run');
                    new Tween({
                        from: actors.lee.position.clone(),
                        to:  tiled['leepos'].position.clone(),
                        in: 90,
                        onUpdate: function (v, t) {
                            actors.lee.position = v;
                        },
                        onComplete: function () {
                            actors.lee.setSpriteSheet('boy/idle');
                        }
                    });
                    actors.rin.setSpriteSheet('krillin/run');
                    new Tween({
                        from: actors.rin.position.clone(),
                        to:  tiled['rinpos'].position.clone(),
                        in: 60,
                        onUpdate: function (v, t) {
                            actors.rin.position = v;
                        },
                        onComplete: function () {
                            actors.rin.setSpriteSheet('krillin/idle');
                            nextScene();
                        }
                    });
                }
            }),
            new Scene ({
                start: function () {
                    actors.rin.setSpriteSheet('krillin/grabbing');
                    talk({
                        text: Localization.getText('cutscene16')['rin01'],
                        waitTime: 120,
                        who: actors.rin
                    });
                    talk({
                        text: Localization.getText('cutscene16')['pigchef02'],
                        waitTime: 120,
                        who: actors.pigchef,
                        onComplete: nextScene
                    });
                }
            }),
            new Scene ({
                start: function () {
                    actors.pigchef.setSpriteSheet('pigchef/jump-windup', function () {
                        actors.pigchef.getComponent('movable', function (movable) {
                            movable.velocityZ = -6;
                            movable.velocity.x = 3;
                        });
                        actors.pigchef.setSpriteSheet('pigchef/jump-air');
                    });
                    new Tween({
                        from: 0,
                        to: 1,
                        in: 60,
                        onComplete: function () {
                            talk({
                                text: Localization.getText('cutscene16')['rin03'],
                                waitTime: 120,
                                who: actors.rin,
                                onComplete: nextScene
                            });
                            actors.rin.setSpriteSheet('krillin/run');
                            new Tween({
                                from: actors.rin.position.clone(),
                                to:  actors.rin.position.add(new Vector2(256, 0)),
                                in: 150,
                                onUpdate: function (v, t) {
                                    actors.rin.position = v;
                                },
                                onComplete: function () {
                                    actors.rin.setSpriteSheet('krillin/idle');
                                    nextScene();
                                }
                            });
                        }
                    });
                }
            })
        ];
        var component = {
            name: 'cutscene', // should always be called 'cutscene'
            startCutscene: function (data) {
                Utils.forEach(scenes, function (scene, i, l, breakLoop) {
                    cutscene.addScene(scene);
                });
                nextScene();
            },
            setupActors: function () {
                setupActors();
            },
            attached: function (data) {
                cutscene = data.entity;
                nextScene = cutscene.nextScene;
                talk = cutscene.talk;
                focusOn = cutscene.focusOn;
            }
        };
        return component;
    };
});