/**
 * Cutscene 16: Found Varka
 * @moduleName CutScene
 */
bento.define('components/cutscenes/cutscene17', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'entities/cutscene',
    'entities/character',
    'modules/localization'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween,
    Cutscene,
    Character,
    Localization
) {
    'use strict';
    var Scene = function (settings) {
        return {
            start: settings.start,
            update: settings.update
        };
    };
    return function (settings) {
        var viewport = Bento.getViewport();
        var cutscene; // the Cutscene instance
        var tiled = Cutscene.tiled; // reference to objects in tiled
        var nextScene; // call this to go to next scene
        var talk; // call this to make actor talk
        var focusOn; // pan camera to actor
        var dummy = new Entity({}); // dummy entity for camera forcing
        /*
         * Actors
         */
        var actors = {};
        var setupActors = function () {
            actors.lee = new Character({
                position: tiled['lee'].position,
                spriteSheet: 'boy/idle'
            });
            actors.paperguy = new Character({
                position: tiled['paperguy'].position,
                spriteSheet: 'pedestrians/paperguy/idle'
            });
            actors.pigchef = new Character({
                position: tiled['pigchef'].position,
                spriteSheet: 'pigchef/idle',
                direction: 1
            });
            // attach them
            Bento.objects.attach(actors.lee);
            Bento.objects.attach(actors.paperguy);
            Bento.objects.attach(actors.pigchef);

            // initial camera position and force it
            dummy.position = tiled['scenecam'].position.clone();
            focusOn(dummy, true);
        };
        /*
         * Array of scenes
         */
        var scenes = [
            new Scene({
                start: function () {
                    talk({
                        text: Localization.getText('cutscene17')['pigchef01'],
                        waitTime: 180,
                        who: actors.pigchef,
                        onComplete: nextScene
                    });
                }
            }),
            new Scene({
                start: function () {
                    actors.paperguy.setDirection(-1);
                    actors.paperguy.setSpriteSheet('pedestrians/paperguy/shocked');
                    actors.paperguy.getComponent('movable',function (movable) {
                        movable.velocityZ =-2;
                        movable.onLanded = function () {
                            movable.velocity.x = 2;
                            actors.paperguy.setSpriteSheet('pedestrians/paperguy/flee');
                            actors.paperguy.setDirection(1);
                        };
                    });
                    actors.pigchef.setSpriteSheet('pigchef/attack1',function () {
                        actors.pigchef.setSpriteSheet('pigchef/run');
                        actors.pigchef.getComponent('movable',function (movable) {
                            movable.velocity.x = 2;
                        });
                    });
                    talk({
                        text: Localization.getText('cutscene17')['fooddude02'],
                        waitTime: 120,
                        who: actors.paperguy,
                        onComplete: nextScene
                    });
                }
            }),
            new Scene({
                start: function () {
                    actors.lee.setSpriteSheet('boy/run');
                    new Tween({
                        from: actors.lee.position.clone(),
                        to:  tiled['leepos'].position.clone(),
                        in: 90,
                        onUpdate: function (v, t) {
                            actors.lee.position = v;
                        },
                        onComplete: function () {
                            actors.lee.setSpriteSheet('boy/idle');
                            nextScene();
                        }
                    });
                }
            }),
        ];
        var component = {
            name: 'cutscene', // should always be called 'cutscene'
            startCutscene: function (data) {
                Utils.forEach(scenes, function (scene, i, l, breakLoop) {
                    cutscene.addScene(scene);
                });
                nextScene();
            },
            setupActors: function () {
                setupActors();
            },
            attached: function (data) {
                cutscene = data.entity;
                nextScene = cutscene.nextScene;
                talk = cutscene.talk;
                focusOn = cutscene.focusOn;
            }
        };
        return component;
    };
});