/**
 * Cutscene 18: Found Varka
 * @moduleName CutScene
 */
bento.define('components/cutscenes/cutscene18', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'entities/cutscene',
    'entities/character',
    'modules/localization'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween,
    Cutscene,
    Character,
    Localization
) {
    'use strict';
    var Scene = function (settings) {
        return {
            start: settings.start,
            update: settings.update
        };
    };
    return function (settings) {
        var viewport = Bento.getViewport();
        var cutscene; // the Cutscene instance
        var tiled = Cutscene.tiled; // reference to objects in tiled
        var nextScene; // call this to go to next scene
        var talk; // call this to make actor talk
        var focusOn; // pan camera to actor
        var dummy = new Entity({}); // dummy entity for camera forcing
        /*
         * Actors
         */
        var actors = {};
        var setupActors = function () {
            actors.lee = new Character({
                position: tiled['lee'].position,
                spriteSheet: 'boy/idle'
            });
            actors.rin = new Character({
                position: tiled['rin'].position,
                spriteSheet: 'krillin/idle'
            });
            actors.pigchef = new Character({
                position: tiled['pigchef'].position,
                spriteSheet: 'pigchef/getup',
                direction: -1
            });
            // attach them
            Bento.objects.attach(actors.lee);
            Bento.objects.attach(actors.rin);
            Bento.objects.attach(actors.pigchef);

            // initial camera position and force it
            dummy.position = tiled['scenecam'].position.clone();
            focusOn(dummy, true);
        };

        var spikesUp = function () {
            var spikes = Bento.objects.getByFamily('obstacles');
            for (var i = spikes.length - 1; i >= 0; i--) {
                spikes[i].getComponent('sprite').setSpriteSheet('obstacles/spear/attack');
            }
        };
        /*
         * Array of scenes
         */
        var scenes = [
            new Scene({
                start: function () {
                    talk({
                        text: Localization.getText('cutscene18')['pigchef01'],
                        waitTime: 120,
                        who: actors.pigchef
                    });
                    talk({
                        text: Localization.getText('cutscene18')['rin02'],
                        waitTime: 120,
                        who: actors.rin,
                        onComplete: nextScene
                    });
                }
            }),
            new Scene({
                start: function () {
                    talk({
                        text: Localization.getText('cutscene18')['pigchef03'],
                        waitTime: 120,
                        who: actors.pigchef
                    });
                    actors.pigchef.setSpriteSheet('pigchef/jump-windup', function () {
                        actors.pigchef.setSpriteSheet('pigchef/jump-air');
                        actors.pigchef.getComponent('movable', function (movable) {
                            movable.velocityZ = -3;
                            movable.velocity.x = 1.5;
                            movable.onLanded = function () {
                                movable.velocity.x = 0;
                                spikesUp();
                                actors.pigchef.setSpriteSheet('pigchef/jump-land', function () {
                                    actors.pigchef.setSpriteSheet('pigchef/laugh');
                                });
                                actors.rin.setSpriteSheet('krillin/run');
                                new Tween({
                                    from: actors.rin.position.clone(),
                                    to:  actors.rin.position.add(new Vector2(48,0)),
                                    in: 30,
                                    onUpdate: function (v, t) {
                                        actors.rin.position = v;
                                    },
                                    onComplete: function () {
                                        actors.rin.setSpriteSheet('krillin/idle');
                                        nextScene();
                                    }
                                });
                            };
                        });
                    });
                },
                update: function () {
                    var spikes = Bento.objects.getByFamily('obstacles');
                    for (var i = spikes.length - 1; i >= 0; i--) {
                        var sprite = spikes[i].getComponent('sprite');
                        if (sprite.getCurrentFrame() > 2) {
                            sprite.setFrame(2);
                            sprite.setCurrentSpeed(0);
                        }
                    }
                }
            }),
            new Scene({
                start: function () {
                    talk({
                        text: Localization.getText('cutscene18')['rin04'],
                        waitTime: 240,
                        who: actors.rin,
                        onComplete: function () {
                            actors.rin.setDirection(-1);
                            new Tween({
                                from: 0,
                                to: 1,
                                in: 90,
                                onComplete: function () {
                                    actors.pigchef.setDirection(1);
                                    actors.pigchef.setSpriteSheet('pigchef/run');
                                    new Tween({
                                        from: actors.pigchef.position.clone(),
                                        to:  actors.pigchef.position.add(new Vector2(256,0)),
                                        in: 120,
                                        onUpdate: function (v, t) {
                                            actors.pigchef.position = v;
                                        },
                                        onComplete: function () {
                                            actors.pigchef.setSpriteSheet('pigchef/idle');

                                        }
                                    }); 
                                }
                            });
                        }
                    });
                    talk({
                        text: Localization.getText('cutscene18')['lee05'],
                        waitTime: 120,
                        who: actors.lee
                    });
                    talk({
                        text: Localization.getText('cutscene18')['rin06'],
                        waitTime: 120,
                        who: actors.rin,
                        onComplete: function () {
                            actors.rin.setDirection(1);
                        }
                    });
                    talk({
                        text: Localization.getText('cutscene18')['rin07'],
                        waitTime: 120,
                        who: actors.rin,
                        onComplete: nextScene
                    });
                }
            })
        ];
        var component = {
            name: 'cutscene', // should always be called 'cutscene'
            startCutscene: function (data) {
                Utils.forEach(scenes, function (scene, i, l, breakLoop) {
                    cutscene.addScene(scene);
                });
                nextScene();
            },
            setupActors: function () {
                setupActors();
            },
            attached: function (data) {
                cutscene = data.entity;
                nextScene = cutscene.nextScene;
                talk = cutscene.talk;
                focusOn = cutscene.focusOn;
            }
        };
        return component;
    };
});