/**
 * Cutscene 2: finding ratboss
 * @moduleName CutScene
 */
bento.define('components/cutscenes/cutscene2', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'entities/cutscene',
    'entities/character',
    'modules/localization'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween,
    Cutscene,
    Character,
    Localization
) {
    'use strict';
    var Scene = function (settings) {
        return {
            start: settings.start,
            update: settings.update
        };
    };
    return function (settings) {
        var viewport = Bento.getViewport();
        var cutscene; // the Cutscene instance
        var tiled = Cutscene.tiled; // reference to objects in tiled
        var nextScene; // call this to go to next scene
        var talk; // call this to make actor talk
        var focusOn; // pan camera to actor
        /*
         * Actors
         */
        var actors = {};
        var setupActors = function () {
            actors.lee = new Character({
                position: tiled['player'].position,
                spriteSheet: 'boy/grabbing'
            });
            actors.ratguy = new Character({
                position: tiled['player'].position.add(new Vector2(20, -4)),
                spriteSheet: 'ratguy/hurt2',
                direction: -1
            });
            actors.rat = new Character({
                position: tiled['rat'].position,
                spriteSheet: 'weasel/liedown'
            });
            actors.bigrat = new Character({
                position: tiled['bigrat'].position,
                direction: -1,
                spriteSheet: 'weasel/liedown'
            });

            // attach them
            Bento.objects.attach(actors.lee);
            Bento.objects.attach(actors.ratguy);
            Bento.objects.attach(actors.rat);
            Bento.objects.attach(actors.bigrat);

            // initial camera position and force it
            focusOn(actors.lee, true);
        };
        /*
         * Array of scenes
         */
        var scenes = [
            // scene1: lee grabbing ratguy
            new Scene({
                start: function () {
                    talk({
                        text: Localization.getText('cutscene2')['lee01'],
                        who: actors.lee
                    });
                    talk({
                        text: Localization.getText('cutscene2')['rat02'],
                        who: actors.ratguy
                    });
                    talk({
                        text: Localization.getText('cutscene2')['rat03'],
                        who: actors.ratguy
                    });
                    talk({
                        text: Localization.getText('cutscene2')['lee04'],
                        who: actors.lee
                    });
                    talk({
                        text: Localization.getText('cutscene2')['rat05'],
                        who: actors.ratguy,
                        onComplete: nextScene
                    });
                }
            })
        ];

        var component = {
            name: 'cutscene', // should always be called 'cutscene'
            startCutscene: function (data) {
                Utils.forEach(scenes, function (scene, i, l, breakLoop) {
                    cutscene.addScene(scene);
                });
                nextScene();
            },
            setupActors: function () {
                setupActors();
            },
            attached: function (data) {
                cutscene = data.entity;
                nextScene = cutscene.nextScene;
                talk = cutscene.talk;
                focusOn = cutscene.focusOn;
            }
        };
        return component;
    };
});