/**
 * Cutscene 3: at the factory
 * @moduleName CutScene
 */
bento.define('components/cutscenes/cutscene3', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'entities/cutscene',
    'entities/character',
    'modules/localization'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween,
    Cutscene,
    Character,
    Localization
) {
    'use strict';
    var Scene = function (settings) {
        return {
            start: settings.start,
            update: settings.update
        };
    };
    return function (settings) {
        var viewport = Bento.getViewport();
        var cutscene; // the Cutscene instance
        var tiled = Cutscene.tiled; // reference to objects in tiled
        var nextScene; // call this to go to next scene
        var talk; // call this to make actor talk
        var focusOn; // pan camera to actor
        /*
         * Actors
         */
        var actors = {};
        var setupActors = function () {
            actors.lee = new Character({
                position: tiled['player'].position,
                spriteSheet: 'boy/idle'
            });
            actors.rat = new Character({
                position: tiled['rat'].position,
                spriteSheet: 'rat/idle',
                direction: -1
            });
            actors.bigrat = new Character({
                position: tiled['bigrat'].position,
                spriteSheet: 'bigrat/idle',
                direction: -1
            });
            actors.ratboss = new Character({
                position: tiled['ratboss'].position,
                spriteSheet: 'ratboss/idle',
                direction: -1
            });

            // attach them
            Bento.objects.attach(actors.lee);
            Bento.objects.attach(actors.rat);
            Bento.objects.attach(actors.bigrat);
            Bento.objects.attach(actors.ratboss);

            // initial camera position and force it
            focusOn(actors.ratboss, true);
        };
        /*
         * Array of scenes
         */
        var scenes = [
            // scene1: ratboss talking
            new Scene({
                start: function () {
                    talk({
                        text: Localization.getText('cutscene3')['rat01'],
                        who: actors.ratboss,
                        offset: new Vector2(0, -8),
                        onComplete: function () {
                            // pan to lee
                            focusOn(actors.lee);
                            Utils.timeout(30, function () {
                                // make a jump
                                actors.lee.getComponent('movable', function (movable) {
                                    actors.lee.setSpriteSheet('boy/hurt2');
                                    movable.velocityZ = -2;
                                    movable.onLanded = nextScene;
                                });
                            });
                        }
                    });
                }
            }),
            // scene2: lee talking
            new Scene({
                start: function () {
                    talk({
                        text: Localization.getText('cutscene3')['lee02'],
                        who: actors.lee,
                        onComplete: function () {
                            focusOn(actors.ratboss);
                            talk({
                                text: Localization.getText('cutscene3')['rat03'],
                                who: actors.ratboss,
                                offset: new Vector2(0, -8)
                            });
                            talk({
                                text: Localization.getText('cutscene3')['rat04'],
                                who: actors.ratboss,
                                offset: new Vector2(0, -8),
                                onComplete: function () {
                                    actors.ratboss.setSpriteSheet('ratboss/walk');
                                    actors.ratboss.getComponent('multiScale', function (multiScale) {
                                        multiScale.direction.x = 1;
                                    });
                                    actors.ratboss.getComponent('movable', function (movable) {
                                        movable.velocity.x = 3;
                                    });

                                    focusOn(actors.lee);
                                    actors.lee.setSpriteSheet('boy/idle');
                                    Utils.timeout(40, nextScene);
                                }
                            });

                        }
                    });

                }
            })
        ];

        var component = {
            name: 'cutscene', // should always be called 'cutscene'
            startCutscene: function (data) {
                Utils.forEach(scenes, function (scene, i, l, breakLoop) {
                    cutscene.addScene(scene);
                });
                nextScene();
            },
            setupActors: function () {
                setupActors();
            },
            attached: function (data) {
                cutscene = data.entity;
                nextScene = cutscene.nextScene;
                talk = cutscene.talk;
                focusOn = cutscene.focusOn;
            }
        };
        return component;
    };
});