/**
 * Cutscene 3 post: defeated ratboss 
 * @moduleName CutScene
 */
bento.define('components/cutscenes/cutscene3post', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'entities/cutscene',
    'entities/character',
    'modules/localization'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween,
    Cutscene,
    Character,
    Localization
) {
    'use strict';
    var Scene = function (settings) {
        return {
            start: settings.start,
            update: settings.update
        };
    };
    return function (settings) {
        var viewport = Bento.getViewport();
        var cutscene; // the Cutscene instance
        var tiled = Cutscene.tiled; // reference to objects in tiled
        var nextScene; // call this to go to next scene
        var talk; // call this to make actor talk
        var focusOn; // pan camera to actor
        /*
         * Actors
         */
        var actors = {};
        var setupActors = function () {
            actors.lee = new Character({
                position: tiled['player'].position,
                spriteSheet: 'boy/victory',
                direction: -1
            });
            actors.ratboss = new Character({
                position: tiled['ratboss'].position,
                spriteSheet: 'ratboss/liedown',
                direction: -1
            });

            // attach them
            Bento.objects.attach(actors.lee);
            Bento.objects.attach(actors.ratboss);

            // initial camera position and force it
            focusOn(actors.lee, true);
        };
        /*
         * Array of scenes
         */
        var scenes = [
            // scene1: lee does victory pose
            new Scene({
                start: function () {
                    talk({
                        text: Localization.getText('cutscene3post')['lee01'],
                        who: actors.lee,
                        onComplete: function () {
                            // pan to ratboss
                            focusOn(actors.ratboss);
                            Utils.timeout(30, function () {
                                // jump up
                                actors.ratboss.getComponent('movable', function (movable) {
                                    actors.ratboss.setSpriteSheet('ratboss/jump');
                                    movable.velocityZ = -2;
                                    movable.onLanded = function () {
                                        actors.ratboss.setSpriteSheet('ratboss/idle');
                                        Utils.timeout(60, nextScene);
                                    };
                                });
                            });
                        }
                    });
                }
            }),
            // scene2: ratboss escapes
            new Scene({
                start: function () {
                    talk({
                        text: Localization.getText('cutscene3post')['rat02'],
                        who: actors.ratboss,
                        offset: new Vector2(0, -8),
                        onComplete: function () {
                            actors.ratboss.getComponent('movable', function (movable) {
                                // pan to lee
                                focusOn(actors.lee);
                                
                                actors.ratboss.setDirection(1);
                                actors.ratboss.setSpriteSheet('ratboss/walk');
                                movable.velocity.x = 4;
                                Utils.timeout(60, nextScene);
                            });
                        }
                    });
                }
            }),
            // scene3: lee talks
            new Scene({
                start: function () {
                    actors.lee.setSpriteSheet('boy/idle');
                    // look left and right
                    Utils.timeout(30, function () {
                        actors.lee.setDirection(1);
                    });
                    Utils.timeout(60, function () {
                        actors.lee.setDirection(-1);
                    });
                    Utils.timeout(90, function () {
                        actors.lee.setDirection(1);
                    });
                    Utils.timeout(120, function () {
                        talk({
                            text: Localization.getText('cutscene3post')['lee03'],
                            who: actors.lee,
                            onComplete: function () {
                                nextScene();
                            }
                        });
                    });
                }
            })
        ];

        var component = {
            name: 'cutscene', // should always be called 'cutscene'
            startCutscene: function (data) {
                Utils.forEach(scenes, function (scene, i, l, breakLoop) {
                    cutscene.addScene(scene);
                });
                nextScene();
            },
            setupActors: function () {
                setupActors();
            },
            attached: function (data) {
                cutscene = data.entity;
                nextScene = cutscene.nextScene;
                talk = cutscene.talk;
                focusOn = cutscene.focusOn;
            }
        };
        return component;
    };
});