/**
 * Cutscene 5: defeated rat boss
 * @moduleName CutScene
 */
bento.define('components/cutscenes/cutscene5', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'entities/cutscene',
    'entities/character',
    'modules/localization'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween,
    Cutscene,
    Character,
    Localization
) {
    'use strict';
    var Scene = function (settings) {
        return {
            start: settings.start,
            update: settings.update
        };
    };
    return function (settings) {
        var viewport = Bento.getViewport();
        var cutscene; // the Cutscene instance
        var tiled = Cutscene.tiled; // reference to objects in tiled
        var nextScene; // call this to go to next scene
        var talk; // call this to make actor talk
        var focusOn; // pan camera to actor
        /*
         * Actors
         */
        var actors = {};
        var setupActors = function () {
            actors.lee = new Character({
                position: tiled['player'].position,
                spriteSheet: 'boy/idle'
            });
            actors.ratboss = new Character({
                position: tiled['ratboss'].position,
                spriteSheet: 'ratboss/liedown',
                direction: -1,
            });
            actors.rat = new Character({
                position: tiled['rat'].position,
                spriteSheet: 'ratgreen/liedown',
                direction: -1,
            });
            actors.rat2 = new Character({
                position: tiled['rat'].position,
                spriteSheet: 'rat/liedown'
            });

            // attach them
            Bento.objects.attach(actors.lee);
            Bento.objects.attach(actors.ratboss);
            Bento.objects.attach(actors.rat);
            Bento.objects.attach(actors.rat2);

            // initial camera position and force it
            focusOn(actors.lee, true);
        };
        /*
         * Array of scenes
         */
        var scenes = [
            // scene1:
            new Scene({
                start: function () {
                    actors.lee.setSpriteSheet('boy/victory');
                    talk({
                        text: Localization.getText('cutscene5')['lee01'],
                        who: actors.lee,
                        onComplete: nextScene
                    });
                }
            }),
            // scene2: ratboss talks
            new Scene({
                start: function () {
                    focusOn(actors.ratboss);
                    actors.lee.setSpriteSheet('boy/idle');
                    talk({
                        text: Localization.getText('cutscene5')['rat02'],
                        who: actors.ratboss
                    });
                    talk({
                        text: Localization.getText('cutscene5')['rat03'],
                        who: actors.ratboss
                    });
                    talk({
                        text: Localization.getText('cutscene5')['rat04'],
                        who: actors.ratboss,
                        onComplete: nextScene
                    });
                }
            }),
            // scene3: lee talks
            new Scene({
                start: function () {
                    actors.lee.setSpriteSheet('boy/idle');
                    talk({
                        text: Localization.getText('cutscene5')['lee05'],
                        who: actors.lee,
                        onComplete: nextScene
                    });
                }
            }),
            // scene4: back to rat boss
            new Scene({
                start: function () {
                    focusOn(actors.ratboss);
                    talk({
                        text: Localization.getText('cutscene5')['rat06'],
                        who: actors.ratboss
                    });
                    talk({
                        text: Localization.getText('cutscene5')['rat07'],
                        who: actors.ratboss,
                        onComplete: nextScene
                    });
                }
            }),
            // scene5: back to rat boss
            new Scene({
                start: function () {
                    focusOn(actors.lee);
                    talk({
                        text: Localization.getText('cutscene5')['lee08'],
                        who: actors.lee,
                        onComplete: nextScene
                    });
                    actors.lee.setSpriteSheet('boy/run');
                    actors.lee.getComponent('movable', function (movable) {
                        movable.velocity.x = 1.5;
                    });
                }
            }),
        ];

        var component = {
            name: 'cutscene', // should always be called 'cutscene'
            startCutscene: function (data) {
                Utils.forEach(scenes, function (scene, i, l, breakLoop) {
                    cutscene.addScene(scene);
                });
                nextScene();
            },
            setupActors: function () {
                setupActors();
            },
            attached: function (data) {
                cutscene = data.entity;
                nextScene = cutscene.nextScene;
                talk = cutscene.talk;
                focusOn = cutscene.focusOn;
            }
        };
        return component;
    };
});