/**
 * Cutscene 6: in the park
 * @moduleName CutScene
 */
bento.define('components/cutscenes/cutscene6', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'entities/cutscene',
    'entities/character',
    'modules/localization',
    'entities/effect'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween,
    Cutscene,
    Character,
    Localization,
    Effect
) {
    'use strict';
    var Scene = function (settings) {
        return {
            start: settings.start,
            update: settings.update
        };
    };
    return function (settings) {
        var viewport = Bento.getViewport();
        var cutscene; // the Cutscene instance
        var tiled = Cutscene.tiled; // reference to objects in tiled
        var nextScene; // call this to go to next scene
        var talk; // call this to make actor talk
        var focusOn; // pan camera to actor
        /*
         * Actors
         */
        var actors = {};
        var setupActors = function () {
            actors.lee = new Character({
                position: tiled['player'].position,
                spriteSheet: 'boy/idle'
            });
            actors.paperguy = new Character({
                position: tiled['ped1'].position,
                spriteSheet: 'pedestrians/paperguy/idle',
                direction: -1
            });
            actors.granny1 = new Character({
                position: tiled['ped2'].position,
                spriteSheet: 'pedestrians/granny/idle'
            });
            actors.granny2 = new Character({
                position: tiled['ped3'].position,
                spriteSheet: 'pedestrians/granny/idle'
            });

            // attach them
            Bento.objects.attach(actors.lee);
            Bento.objects.attach(actors.granny1);
            Bento.objects.attach(actors.granny2);
            Bento.objects.attach(actors.paperguy);

            // initial camera position and force it
            focusOn(actors.lee, true);
        };
        /*
         * Array of scenes
         */
        var scenes = [
            // scene1: people running away
            new Scene({
                start: function () {
                    Utils.timeout(30, function () {
                        actors.granny2.setSpriteSheet('pedestrians/granny/flee');
                        actors.granny2.setDirection(-1);
                        actors.granny2.getComponent('movable', function (movable) {
                            movable.velocity.x = -3;
                        });
                    });
                    Utils.timeout(60, function () {
                        actors.paperguy.setSpriteSheet('pedestrians/paperguy/shocked');
                        actors.paperguy.setDirection(1);
                        actors.paperguy.getComponent('movable', function (movable) {
                            movable.velocityZ = -2;
                            movable.onLanded = function () {
                                movable.velocity.x = -3;
                                actors.paperguy.setDirection(-1);
                                actors.paperguy.setSpriteSheet('pedestrians/paperguy/flee');
                            };
                        });
                    });
                    Utils.timeout(70, function () {
                        actors.granny1.setSpriteSheet('pedestrians/granny/flee');
                        actors.granny1.setDirection(-1);
                        actors.granny1.getComponent('movable', function (movable) {
                            movable.velocity.x = -3;
                        });
                    });
                    Utils.timeout(80, function () {
                        talk({
                            text: Localization.getText('cutscene6')['lee01'],
                            who: actors.lee,
                            onComplete: function () {
                                nextScene();
                            }
                        });
                    });
                }
            }),
            // scene3: pig lands
            new Scene({
                start: function () {
                    actors.pigchef = new Character({
                        position: tiled['pigchef'].position,
                        spriteSheet: 'pigchef/jump-fall',
                        direction: -1
                    });
                    Bento.objects.attach(actors.pigchef);
                    focusOn(actors.pigchef);
                    actors.pigchef.getComponent('movable', function (movable) {
                        movable.height = -160;
                        movable.onLanded = function () {
                            Bento.audio.stopSound('sfx_varkastomp');
                            Bento.audio.playSound('sfx_varkastomp');

                            actors.lee.setSpriteSheet('boy/hurt2');
                            actors.lee.getComponent('movable', function (movableLee) {
                                movableLee.velocityZ = -2;
                            });
                            actors.pigchef.setSpriteSheet('pigchef/jump-land', function () {
                                actors.pigchef.setSpriteSheet('pigchef/laugh');
                            });
                            Utils.timeout(60, nextScene);
                        };
                    });
                }
            }),
            // scene3: pig attacks
            new Scene({
                start: function () {
                    actors.pigchef.setSpriteSheet('pigchef/attack1');
                    actors.pigchef.getComponent('sprite', function (sprite) {
                        sprite.setCurrentSpeed(0.15);
                    });
                    actors.lee.setSpriteSheet('boy/idle');

                    actors.rin = new Character({
                        position: tiled['rin'].position,
                        spriteSheet: 'krillin/idle'
                    });
                    Bento.objects.attach(actors.rin);
                    // actors.rin.getComponent('movable', function (movable) {
                    //     movable.velocity.x = 4;
                    // });
                    new Tween({
                        from: actors.rin.position,
                        to: tiled['pigchef'].position.add(new Vector2(-16, 0)),
                        in: 30,
                        delay: 10,
                        ease: 'linear',
                        onStart: function () {
                            Bento.audio.setVolume(0.25, 'sfx_dashattack2');
                            Bento.audio.stopSound('sfx_dashattack2');
                            Bento.audio.playSound('sfx_dashattack2');

                            Bento.audio.stopSound('krillin/sfx_dash');
                            Bento.audio.playSound('krillin/sfx_dash');

                            actors.rin.setSpriteSheet('krillin/dragonkick');
                        },
                        onUpdate: function (v, t) {
                            actors.rin.position = v;
                        },
                        onComplete: function () {
                            focusOn(actors.rin);
                            actors.pigchef.setSpriteSheet('pigchef/block');
                            actors.rin.setSpriteSheet('krillin/idle');

                            Bento.audio.stopSound('sfx_block');
                            Bento.audio.playSound('sfx_block');

                            var effect = new Effect({
                                position: actors.rin.position.clone(),
                                spriteSheet: 'effects/block'
                            });
                            Bento.objects.attach(effect);

                            actors.pigchef.getComponent('movable', function (movable) {
                                movable.velocityZ = -1;
                                movable.velocity.x = 3;

                                movable.onLanded = function () {
                                    actors.pigchef.setSpriteSheet('pigchef/idle');
                                    movable.velocity.x = 0;

                                    nextScene();
                                };
                            });
                        }
                    });
                }
            }),
            // rin talks
            new Scene({
                start: function () {
                    talk({
                        text: Localization.getText('cutscene6')['rin02'],
                        who: actors.rin
                    });
                    talk({
                        text: Localization.getText('cutscene6')['lee03'],
                        who: actors.lee
                    });
                    talk({
                        text: Localization.getText('cutscene6')['rin04'],
                        who: actors.rin,
                        onComplete: nextScene
                    });
                }
            }),
            // pigchef talks
            new Scene({
                start: function () {
                    focusOn(actors.pigchef);
                    actors.pigchef.setSpriteSheet('pigchef/laugh');
                    talk({
                        text: Localization.getText('cutscene6')['varka05'],
                        who: actors.pigchef,
                        offset: new Vector2(0, -8)
                    });
                    talk({
                        text: Localization.getText('cutscene6')['varka06'],
                        who: actors.pigchef,
                        offset: new Vector2(0, -8),
                        onComplete: function () {
                            focusOn(actors.lee);
                            // jump away
                            actors.pigchef.setDirection(1);
                            actors.pigchef.setSpriteSheet('pigchef/jump-windup', function () {
                                actors.pigchef.setSpriteSheet('pigchef/jump-air');
                                actors.pigchef.getComponent('movable', function (movable) {
                                    movable.velocityZ = -8;
                                    movable.velocity.x = 3;
                                    movable.onLanded = null;
                                    Utils.timeout(40, nextScene);
                                });
                            });
                        }
                    });

                }
            }),
            // end
            new Scene({
                start: function () {
                    actors.lee.setSpriteSheet('boy/run');
                    actors.rin.setSpriteSheet('krillin/run');
                    actors.lee.getComponent('movable', function (movable) {
                        movable.velocity.x = 1.5;
                    });
                    actors.rin.getComponent('movable', function (movable) {
                        movable.velocity.x = 1.5;
                    });
                    talk({
                        text: Localization.getText('cutscene6')['lee07'],
                        who: actors.lee,
                        onComplete: nextScene
                    });
                }
            })
        ];

        var component = {
            name: 'cutscene', // should always be called 'cutscene'
            startCutscene: function (data) {
                Utils.forEach(scenes, function (scene, i, l, breakLoop) {
                    cutscene.addScene(scene);
                });
                nextScene();
            },
            setupActors: function () {
                setupActors();
            },
            attached: function (data) {
                cutscene = data.entity;
                nextScene = cutscene.nextScene;
                talk = cutscene.talk;
                focusOn = cutscene.focusOn;
            }
        };
        return component;
    };
});