/**
 * Cutscene 7: defeated pigchef
 * @moduleName CutScene
 */
bento.define('components/cutscenes/cutscene7', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'entities/cutscene',
    'entities/character',
    'modules/localization',
    'entities/effect'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween,
    Cutscene,
    Character,
    Localization,
    Effect
) {
    'use strict';
    var Scene = function (settings) {
        return {
            start: settings.start,
            update: settings.update
        };
    };
    return function (settings) {
        var viewport = Bento.getViewport();
        var cutscene; // the Cutscene instance
        var tiled = Cutscene.tiled; // reference to objects in tiled
        var nextScene; // call this to go to next scene
        var talk; // call this to make actor talk
        var focusOn; // pan camera to actor
        /*
         * Actors
         */
        var actors = {};
        var setupActors = function () {
            actors.lee = new Character({
                position: tiled['player'].position,
                spriteSheet: 'boy/idle'
            });
            actors.pigchef = new Character({
                position: tiled['pigchef'].position,
                spriteSheet: 'pigchef/liedown',
                direction: -1
            });
            actors.rin = new Character({
                position: tiled['rin'].position,
                spriteSheet: 'krillin/idle'
            });
            // attach them
            Bento.objects.attach(actors.lee);
            Bento.objects.attach(actors.pigchef);
            Bento.objects.attach(actors.rin);

            // initial camera position and force it
            focusOn(actors.rin, true);
        };
        /*
         * Array of scenes
         */
        var scenes = [
            // scene1: talking
            new Scene({
                start: function () {
                    talk({
                        text: Localization.getText('cutscene7')['rin01'],
                        who: actors.rin,
                        onComplete: function () {
                            focusOn(actors.lee);
                        }
                    });
                    talk({
                        text: Localization.getText('cutscene7')['lee02'],
                        who: actors.lee,
                        onComplete: function () {
                            talk({
                                text: Localization.getText('cutscene7')['rin03'],
                                who: actors.rin,
                                onComplete: function () {
                                    nextScene();
                                }
                            });
                            focusOn(actors.rin);
                            // walk up
                            new Tween({
                                from: actors.rin.position,
                                to: actors.rin.position.add(new Vector2(0, -16)),
                                in: 30,
                                onStart: function () {
                                    actors.rin.setSpriteSheet('krillin/walk');
                                },
                                onUpdate: function (v, t) {
                                    actors.rin.position = v;
                                },
                                onComplete: function () {
                                    actors.rin.setSpriteSheet('krillin/idle');
                                }
                            });
                        }
                    });
                }
            }),
            // pig gets up
            new Scene({
                start: function () {
                    focusOn(actors.pigchef);
                    talk({
                        text: Localization.getText('cutscene7')['varka04'],
                        who: actors.pigchef,
                        onComplete: function () {
                            // shaking
                            new Tween({ in: 60,
                                onStart: function () {},
                                onUpdate: function (v, t) {
                                    actors.pigchef.transform.x = Utils.getRandomRangeFloat(-1, 1);
                                    actors.pigchef.transform.y = Utils.getRandomRangeFloat(-1, 1);
                                },
                                onComplete: function () {
                                    actors.pigchef.transform.x = 0;
                                    actors.pigchef.transform.y = 0;
                                    // pig gets up
                                    actors.pigchef.setSpriteSheet('pigchef/laugh');
                                    actors.lee.setSpriteSheet('boy/hurt2');
                                    talk({
                                        text: Localization.getText('cutscene7')['varka05'],
                                        who: actors.pigchef,
                                        offset: new Vector2(0, -8),
                                        onComplete: function () {
                                            nextScene();
                                        }
                                    });
                                }
                            });
                        }
                    });
                }
            }),
            // pig escapes
            new Scene({
                start: function () {
                    // jump away
                    focusOn(actors.lee);
                    actors.pigchef.setDirection(1);
                    actors.pigchef.setSpriteSheet('pigchef/jump-windup', function () {
                        actors.pigchef.setSpriteSheet('pigchef/jump-air');
                        actors.pigchef.getComponent('movable', function (movable) {
                            movable.velocityZ = -8;
                            movable.velocity.x = 3;
                            movable.onLanded = null;
                            Utils.timeout(40, nextScene);
                        });
                    });
                }
            }),
            // talk
            new Scene({
                start: function () {
                    actors.lee.setSpriteSheet('boy/idle');
                    talk({
                        text: Localization.getText('cutscene7')['lee06'],
                        who: actors.lee,
                        onComplete: function () {
                            focusOn(actors.rin);
                            actors.rin.setSpriteSheet('krillin/tired');

                            // end
                            talk({
                                text: Localization.getText('cutscene7')['rin07'],
                                who: actors.rin,
                                onComplete: function () {
                                    nextScene();
                                }
                            });
                        }
                    });
                }
            })
        ];

        var component = {
            name: 'cutscene', // should always be called 'cutscene'
            startCutscene: function (data) {
                Utils.forEach(scenes, function (scene, i, l, breakLoop) {
                    cutscene.addScene(scene);
                });
                nextScene();
            },
            setupActors: function () {
                setupActors();
            },
            attached: function (data) {
                cutscene = data.entity;
                nextScene = cutscene.nextScene;
                talk = cutscene.talk;
                focusOn = cutscene.focusOn;
            }
        };
        return component;
    };
});