/**
 * Cutscene 9: Lee meets Tas and the 3 bosses
 * @moduleName CutScene
 */
bento.define('components/cutscenes/cutscene9', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'entities/cutscene',
    'entities/character',
    'modules/localization'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween,
    Cutscene,
    Character,
    Localization
) {
    'use strict';
    var Scene = function (settings) {
        return {
            start: settings.start,
            update: settings.update
        };
    };
    return function (settings) {
        var viewport = Bento.getViewport();
        var cutscene; // the Cutscene instance
        var tiled = Cutscene.tiled; // reference to objects in tiled
        var nextScene; // call this to go to next scene
        var talk; // call this to make actor talk
        var focusOn; // pan camera to actor
        var dummy = new Entity({}); // dummy entity for camera forcing
        /*
         * Actors
         */
        var actors = {};
        var setupActors = function () {
            actors.lee = new Character({
                position: tiled['lee'].position,
                spriteSheet: 'boy/idle'
            });
            actors.tas = new Character({
                position: tiled['tas'].position,
                spriteSheet: 'boxer/idle'
            });
            actors.ratboss = new Character({
                position: tiled['ratboss'].position,
                spriteSheet: 'ratboss/idle',
                direction: -1
            });
            actors.fiftykicks = new Character({
                position: tiled['fiftykicks'].position,
                spriteSheet: 'gangstaboss/idle',
                direction: -1
            });

            // attach them
            Bento.objects.attach(actors.lee);
            Bento.objects.attach(actors.tas);
            Bento.objects.attach(actors.ratboss);
            Bento.objects.attach(actors.fiftykicks);

            // initial camera position and force it
            focusOn(actors.lee, true);
        };
        /*
         * Array of scenes
         */
        var scenes = [
            // scene1: Lee runs up to the gathering
            new Scene({
                start: function () {
                    dummy.position = tiled['scene1cam'].position.clone();
                    new Tween({
                        from: actors.lee.position.clone(),
                        to: tiled['runPos'].position,
                        in: 60,
                        delay: 0,
                        onStart: function () {
                            actors.lee.setSpriteSheet('boy/run');
                        },
                        onUpdate: function (v,t) {
                            actors.lee.position = v;
                        },
                        onComplete: function () {
                            actors.lee.setSpriteSheet('boy/tired');
                            focusOn(dummy, false);
                            nextScene();
                        }
                    });
                }
            }),
            // scene2: fiftykicks and tas have a bit of banter
            new Scene ({
                start: function () {
                    talk({
                        text: Localization.getText('cutscene9')['fiftykicks01'],
                        who: actors.fiftykicks,
                        offset: new Vector2(0, -32),
                        onComplete: function () {
                            actors.lee.setSpriteSheet('boy/idle');
                        }
                    });
                    talk({
                        text: Localization.getText('cutscene9')['fiftykicks02'],
                        who: actors.fiftykicks,
                        offset: new Vector2(0, -32),
                        onComplete: function () {
                            var punchLoop = function () {
                                actors.tas.setSpriteSheet('boxer/punch2',function() {
                                    actors.tas.setSpriteSheet('boxer/punch1',punchLoop);
                                });
                            };
                            actors.tas.setSpriteSheet('boxer/punch1',punchLoop);
                        }
                    });
                    talk({
                        text: Localization.getText('cutscene9')['tas03'],
                        who: actors.tas
                    });
                    talk({
                        text: Localization.getText('cutscene9')['lee04'],
                        who: actors.lee,
                        onComplete: function () {
                            actors.tas.setSpriteSheet('boxer/tired');
                        }
                    });
                    talk({
                        text: Localization.getText('cutscene9')['fiftykicks05'],
                        who: actors.fiftykicks,
                        offset: new Vector2(0, -32),
                        onComplete: function () {
                            actors.tas.setSpriteSheet('boxer/idle');
                        }
                    });
                    talk({
                        text: Localization.getText('cutscene9')['fiftykicks06'],
                        waitTIme: 180,
                        who: actors.fiftykicks,
                        offset: new Vector2(0, -32),
                    });
                    talk({
                        text: Localization.getText('cutscene9')['fiftykicks07'],
                        waitTIme: 180,
                        who: actors.fiftykicks,
                        offset: new Vector2(0, -32),
                        onComplete: nextScene
                    });
                }
            }),
            // scene3: fiftykicks wanders away, and ratboss follows
            new Scene ({
                start: function () {
                    talk({
                        text: Localization.getText('cutscene9')['fiftykicks08'],
                        who: actors.fiftykicks,
                        offset: new Vector2(0, -16)
                    });
                    new Tween({
                        from: actors.fiftykicks.position.clone(),
                        to: actors.fiftykicks.position.add(new Vector2(160, 0)),
                        in: 70,
                        onStart: function () {
                            actors.fiftykicks.setSpriteSheet('gangstaboss/walk');
                            actors.fiftykicks.setDirection(1);
                        },
                        onUpdate: function (v, t) {
                            actors.fiftykicks.position = v;
                        }
                    });
                    new Tween({
                        from: 0,
                        to: 1,
                        in: 150,
                        onComplete: function () {
                            actors.ratboss.setDirection(1);
                            new Tween({
                                from: 0,
                                to: 1,
                                in: 60,
                                onComplete: function () {
                                    actors.ratboss.setDirection(-1);
                                    new Tween({
                                        from: 0,
                                        to: 1,
                                        in: 30,
                                        onComplete: function () {
                                            actors.ratboss.setDirection(1);
                                            new Tween({
                                                from: 0,
                                                to: 1,
                                                in: 30,
                                                onComplete: function () {
                                                actors.ratboss.setDirection(-1);                                            
                                                    new Tween({
                                                        from: 0,
                                                        to: 1,
                                                        in: 15,
                                                        onComplete: function () {
                                                            actors.ratboss.setDirection(1);
                                                            new Tween({
                                                                from: 0,
                                                                to: 1,
                                                                in: 15,
                                                                onComplete: function () {
                                                                    actors.ratboss.setDirection(-1);
                                                                    new Tween({
                                                                        from: 0,
                                                                        to: 1,
                                                                        in: 60,
                                                                        onComplete: function () {
                                                                            new Tween({
                                                                                from: actors.ratboss.position.clone(),
                                                                                to: actors.ratboss.position.add(new Vector2(128, 0)),
                                                                                in: 120,
                                                                                onStart: function () {
                                                                                    actors.ratboss.setSpriteSheet('ratboss/walk');
                                                                                    actors.ratboss.setDirection(1);
                                                                                },
                                                                                onUpdate: function (v, t) {
                                                                                    actors.ratboss.position = v;
                                                                                },
                                                                                onComplete: function () {
                                                                                    focusOn(actors.lee, false);
                                                                                    nextScene();
                                                                                }
                                                                            });
                                                                        }
                                                                    });
                                                                }
                                                            });
                                                        }
                                                    });
                                                }
                                            });
                                        }
                                    });
                                }
                            });
                        }
                    });
                }
            }),
            // scene4: Tas pursues the bosses
            new Scene ({
                start: function () {
                        talk({
                        text: Localization.getText('cutscene9')['tas09'],
                        who: actors.tas,
                    });
                    actors.tas.setSpriteSheet('boxer/run');
                    new Tween({
                        from: actors.tas.position.clone(),
                        to: actors.tas.position.add(new Vector2(256, 0)),
                        in: 180,
                        onUpdate: function (v, t) {
                            actors.tas.position = v;
                        },
                        onComplete: function () {
                            nextScene();
                        }
                    });
                }
            })
        ];

        var component = {
            name: 'cutscene', // should always be called 'cutscene'
            startCutscene: function (data) {
                Utils.forEach(scenes, function (scene, i, l, breakLoop) {
                    cutscene.addScene(scene);
                });
                nextScene();
            },
            setupActors: function () {
                setupActors();
            },
            attached: function (data) {
                cutscene = data.entity;
                nextScene = cutscene.nextScene;
                talk = cutscene.talk;
                focusOn = cutscene.focusOn;
            }
        };
        return component;
    };
});