/**
 * Cutscene 1: meeting shirley1boss
 * @moduleName CutScene
 */
bento.define('components/cutscenes/cutscenebeach1', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'entities/cutscene',
    'entities/character',
    'modules/localization',
    'entities/effect'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween,
    Cutscene,
    Character,
    Localization,
    Effect
) {
    'use strict';
    var Scene = function (settings) {
        return {
            start: settings.start,
            update: settings.update
        };
    };
    return function (settings) {
        var viewport = Bento.getViewport();
        var cutscene; // the Cutscene instance
        var tiled = Cutscene.tiled; // reference to objects in tiled
        var nextScene; // call this to go to next scene
        var talk; // call this to make actor talk
        var focusOn; // pan camera to actor
        /*
         * Actors
         */
        var actors = {};
        var setupActors = function () {
            actors.rad = new Character({
                position: tiled['player'].position,
                spriteSheet: 'rad/idle'
            });
            actors.pam = new Character({
                position: tiled['pam'].position,
                spriteSheet: 'pam/idle',
                direction: -1
            });
            actors.shirley1 = new Character({
                position: tiled['shirley1'].position,
                spriteSheet: 'shirley/idle',
                direction: -1
            });
            actors.shirley2 = new Character({
                position: tiled['shirley2'].position,
                spriteSheet: 'shirley/idle',
                direction: -1
            });

            // attach them
            Bento.objects.attach(actors.rad);
            Bento.objects.attach(actors.pam);
            Bento.objects.attach(actors.shirley1);
            Bento.objects.attach(actors.shirley2);

            // initial camera position and force it
            focusOn(actors.rad, true);
        };

        var tempRadY;
        /*
         * Array of scenes
         */ 
        var scenes = [

            new Scene({
                start: function(){
                    Utils.timeout(40, function () {
                        nextScene();
                    });
                }
            }),
            //scene 1: pam walks to Rad
            new Scene({
                start: function () {
                    // pan to pam
                    focusOn(actors.pam);
                    Utils.timeout(100, function () {
                        actors.rad.setSpriteSheet('rad/victory');

                    });
                    Utils.timeout(60, function () {
                        actors.pam.setSpriteSheet('pam/walk');
                        new Tween({
                            from: actors.pam.position,
                            to: actors.rad.position.add(new Vector2(40, -5)),
                            in: 60,
                            onUpdate: function (v, t) {
                                actors.pam.position = v;
                            },
                            onComplete: function(){
                                actors.pam.setSpriteSheet('pam/idle');
                                Utils.timeout(30, function(){
                                    //punch
                                    actors.pam.setSpriteSheet('pam/attack');
                                    Utils.timeout(15, function(){

                                        Bento.objects.get('camera', function (camera) {
                                            camera.shake(20, 10);
                                        });

                                        var effect = new Effect({
                                            position: actors.pam.position.add(new Vector2(-20, -12)),
                                            spriteSheet: 'effects/hitfx1'
                                        });
                                        Bento.objects.add(effect);


                                        Bento.audio.stopSound('sfx_punch');
                                        Bento.audio.playSound('sfx_punch');
                                        Bento.audio.stopSound('rad/sfx_death');
                                        Bento.audio.playSound('rad/sfx_death');

                                        actors.rad.setSpriteSheet('rad/hurt1');
                                        Utils.timeout(5, function(){
                                            actors.rad.setSpriteSheet('rad/flydown');
                                            Utils.timeout(5, function(){
                                                actors.rad.setSpriteSheet('rad/liedown');
                                            });
                                        });
                                        Utils.timeout(15, function(){
                                            actors.pam.setSpriteSheet('pam/idle');
                                            Utils.timeout(45, function(){
                                                actors.pam.setSpriteSheet('pam/getup');
                                                Bento.audio.stopSound('val/sfx_throw');
                                                Bento.audio.playSound('val/sfx_throw');     
                                                Bento.audio.stopSound('sfx_error');
                                                Bento.audio.playSound('sfx_error');  
                                            });
                                            Utils.timeout(50, function(){
                                                actors.rad.setSpriteSheet('rad/kidnapped');
                                                actors.rad.setDirection(-1,-1); 
                                                actors.rad.settings.components[6].alpha=0;
                                                actors.pam.setSpriteSheet('pam/idle');
                                                effect = new Effect({
                                                    position: new Vector2(actors.rad.position.x+5,actors.rad.position.y-2),
                                                    spriteSheet: 'effects/block'
                                                });

                                                Bento.objects.attach(effect);
                                                new Tween({
                                                    from: actors.rad.position,
                                                    to: new Vector2(actors.pam.position.x,actors.pam.position.y - 5),
                                                    in: 10,
                                                    ease: 'easeOutCubic',
                                                    onUpdate: function (v, t) {
                                                        actors.rad.position = v;
                                                    },
                                                    onComplete: function(){
                                                        Utils.timeout(30, function(){
                                                            actors.pam.setDirection(1,1);
                                                            actors.rad.setDirection(1,1); 
                                                            tempRadY = actors.rad.position.y;
                                                            Utils.timeout(15, function(){
                                                                actors.pam.setSpriteSheet('pam/walk');
                                                                var raddistance = actors.rad.position.x - actors.pam.position.x;
                                                                new Tween({
                                                                    from: actors.pam.position.x,
                                                                    to: 250,
                                                                    in: 60,
                                                                    onUpdate: function (v, t) {
                                                                        actors.pam.position.x = v;
                                                                        actors.rad.position.x = v + raddistance;
                                                                        actors.rad.position.y = tempRadY -3- Math.sin(v/6)*1.2;
                                                                    },
                                                                    onComplete: function(){
                                                                        nextScene();
                                                                    }
                                                                });
                                                            });
                                                        });
                                                    }
                                                });
                                            });
                                        });
                                    });

                                });
                            }
                        });
                    }
                    );
                }
            }),
            new Scene({
                start: function(){
                    focusOn(null); 
                    actors.shirley1.setDirection(1,1);
                    actors.shirley2.setDirection(1,1); 
                    var shirley1distance = actors.shirley1.position.x - actors.pam.position.x;
                    var shirley2distance = actors.shirley2.position.x - actors.pam.position.x;
                    var raddistance = actors.rad.position.x - actors.pam.position.x;
                    actors.shirley1.setSpriteSheet('shirley/walk');
                    actors.shirley2.setSpriteSheet('shirley/walk');

                    actors.rad.settings.components[6].destroy();
                    new Tween({
                        from: 250,
                        to: 400,
                        in: 130,
                        onUpdate: function (v, t) {
                            actors.pam.position.x = v;
                            actors.shirley1.position.x = v + shirley1distance;
                            actors.shirley2.position.x = v + shirley2distance;
                            actors.rad.position.x = v + raddistance;
                            actors.rad.position.y = tempRadY -3- Math.sin(v/6)*1.2;


                        },
                        onComplete: function(){
                            nextScene();
                        }
                    });                                 
                }
            })
        ];

        var component = {
            name: 'cutscene', // should always be called 'cutscene'
            startCutscene: function (data) {
                Utils.forEach(scenes, function (scene, i, l, breakLoop) {
                    cutscene.addScene(scene);
                });
                nextScene();
            },
            setupActors: function () {
                setupActors();
            },
            attached: function (data) {
                cutscene = data.entity;
                nextScene = cutscene.nextScene;
                talk = cutscene.talk;
                focusOn = cutscene.focusOn;
            }
        };
        return component;
    };
});