/**
 * Cutscene 1: meeting shirley1boss
 * @moduleName CutScene
 */
bento.define('components/cutscenes/cutscenebeach2', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'entities/cutscene',
    'entities/character',
    'modules/localization',
    'entities/effect',
    'components/sort',
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween,
    Cutscene,
    Character,
    Localization,
    Effect,
    Sort
) {
    'use strict';
    var Scene = function (settings) {
        return {
            start: settings.start,
            update: settings.update
        };
    };
    return function (settings) {
        var viewport = Bento.getViewport();
        var cutscene; // the Cutscene instance
        var tiled = Cutscene.tiled; // reference to objects in tiled
        var nextScene; // call this to go to next scene
        var talk; // call this to make actor talk
        var focusOn; // pan camera to actor
        /*
         * Actors
         */
        var actors = {};

        var hideAll = function(){
            for (var actor in actors){
                hideActor(actor);
            }
        };
        var hideActor = function(actor){
            actors[actor].visible = false;
            actors[actor].components[6].shadow.components[0].visible = false;
        };
        var showActor = function(actor){
            actors[actor].visible = true;
            actors[actor].components[6].shadow.components[0].visible = true;
        };

        var setupActors = function () {
            actors.start = new Character({
                position: tiled['start'].position,
                spriteSheet: 'rad/tiedup'
            });
            actors.rad = new Character({
                position: tiled['rad'].position,
                spriteSheet: 'rad/tiedup',
            });
            actors.pam = new Character({
                position: tiled['pam'].position,
                spriteSheet: 'pam/idle',
                direction: -1,
            });
            actors.shirley1 = new Character({
                position: tiled['shirley1'].position,
                spriteSheet: 'shirley/idle',
                direction: -1,
            });
            actors.shirley2 = new Character({
                position: tiled['shirley2'].position,
                spriteSheet: 'shirley/idle',
                direction: -1,
            });
            actors.shirley2 = new Character({
                position: tiled['shirley2'].position,
                spriteSheet: 'shirley/idle',
                direction: -1,
            });

            hideAll();
            showActor("rad");
            // attach them
            Bento.objects.attach(actors.rad);
            Bento.objects.attach(actors.pam);
            Bento.objects.attach(actors.shirley1);
            Bento.objects.attach(actors.shirley2);


            // initial camera position and force it
            focusOn(actors.start, true);
        };
        /*
         * Array of scenes
         */ 
        
        var jumpTo = function(useActor,toX,toY,jumpTime,doneFunction){
            var actor = actors[useActor];
            var fromX = actor.position.x;
            var fromY = actor.position.y;
            var midY = (actor.position.y + toY) / 2 - 100;
            actor.components[6].shadow.components[0].visible = false;
            new Tween({
                from: 0,
                to: 100,
                in: jumpTime,
                onUpdate: function (v, t) {
                    actor.position.x = fromX + ((toX-fromX)/100)*v;
                },
                onComplete: function(){

                    console.log("done");
                }
            });
            new Tween({
                from: fromY,
                to: midY,
                in: jumpTime/2,
                ease: 'easeOutCubic',
                onUpdate: function (v, t) {
                    actor.position.y = v;
                },
                onComplete: function(){
                    new Tween({
                        from: midY,
                        to: toY,
                        in: jumpTime/2,
                        ease: 'easeInCubic',
                        onUpdate: function (v, t) {
                            actor.position.y = v;
                        },
                        onComplete: function(){
                            actor.components[6].shadow.components[0].visible = true;
                            if(typeof doneFunction == "function"){
                                doneFunction.apply();
                            }
                        }
                    });
                }
            });
        };
        var splash = function(x,y){
            var effect = new Effect({
                position: new Vector2(x, y),
                spriteSheet: 'effects/watersplash'
            });
            effect.attach(new Sort({}));
            Bento.objects.attach(effect);
        };
        var dust = function(x,y){
            var effect = new Effect({
                position: new Vector2(x, y),
                spriteSheet: 'effects/body-land'
            });
            effect.attach(new Sort({}));
            Bento.objects.attach(effect);
        };
        var jumpDust = function(x,y){
            var effect = new Effect({
                position: new Vector2(x, y),
                spriteSheet: 'effects/jump-dust'
            });
            effect.attach(new Sort({}));
            Bento.objects.attach(effect);
        };
        var scenes = [


            new Scene({
                start: function () {
                    // pan to pam
                    focusOn(actors.rad);
                    Utils.timeout(110, function () {
                        showActor("pam");
                        showActor("shirley1");
                        showActor("shirley2");
                        actors.pam.setSpriteSheet('pam/jump');
                        actors.shirley1.setSpriteSheet('shirley/jump');
                        actors.shirley2.setSpriteSheet('shirley/jump');
                        focusOn(actors.pam);
                        Bento.audio.stopSound('sfx_special');
                        Bento.audio.playSound('sfx_special');
                        Bento.audio.stopSound('sfx_openscroll');
                        Bento.audio.playSound('sfx_openscroll');
                        jumpTo("pam",actors.pam.position.x+20,actors.pam.position.y+50,70,null);
                        jumpTo("shirley1",actors.shirley2.position.x+20,actors.shirley2.position.y+60,70,null);
                        jumpTo("shirley2",actors.shirley1.position.x+20,actors.shirley1.position.y+40,70,nextScene);

                        splash(actors.pam.position.x,actors.pam.position.y);
                        splash(actors.shirley2.position.x,actors.shirley2.position.y);
                        splash(actors.shirley1.position.x,actors.shirley1.position.y);

                    });
                }
            }),
            new Scene({
                start: function(){
                    dust(actors.pam.position.x,actors.pam.position.y);
                    dust(actors.shirley2.position.x,actors.shirley2.position.y);
                    dust(actors.shirley1.position.x,actors.shirley1.position.y);   
                    Bento.audio.stopSound('sfx_body-land');
                    Bento.audio.playSound('sfx_body-land');
                    actors.pam.setSpriteSheet('pam/idle');
                    actors.shirley1.setSpriteSheet('shirley/idle');
                    actors.shirley2.setSpriteSheet('shirley/idle');
                    Utils.timeout(60, function () {
                        focusOn(null);
                        nextScene();
                    });
                }
            }),
            new Scene({
                start: function(){
                    actors.pam.setDirection(1,1);
                    var fromX = actors.pam.position.x;
                    var fromY = actors.pam.position.y;
                    var toX = actors.pam.position.x-10;
                    var toY = actors.pam.position.y-30;
                    actors.pam.setSpriteSheet('pam/walk');
                    new Tween({
                        from: 0,
                        to: 100,
                        in: 40,
                        onUpdate: function (v, t) {
                            actors.pam.position.x = fromX + ((toX-fromX)/100)*v;
                            actors.pam.position.y = fromY + ((toY-fromY)/100)*v;
                        },
                        onComplete: function(){
                            actors.pam.setSpriteSheet('pam/idle');
                            Utils.timeout(30, function () {
                                actors.pam.setSpriteSheet('pam/getup');
                                Utils.timeout(10, function () {
                                    actors.pam.setSpriteSheet('pam/jump');
                                    Bento.audio.stopSound('val/sfx_throw');
                                    Bento.audio.playSound('val/sfx_throw');     
                                    jumpTo("pam",actors.pam.position.x+120,actors.pam.position.y,70,null);
                                    jumpTo("rad",actors.rad.position.x+120,actors.rad.position.y,70,nextScene);
                                    jumpDust(actors.pam.position.x+10,actors.pam.position.y);
                                    Bento.audio.stopSound('sfx_special');
                                    Bento.audio.playSound('sfx_special');
                                });
                            });
                        }
                    });
                }
            }),
            new Scene({
                start: function(){
                    hideActor("pam");
                    hideActor("rad");
                    splash(actors.pam.position.x,actors.pam.position.y);
                    splash(actors.rad.position.x,actors.rad.position.y);
                    Bento.audio.stopSound('sfx_openscroll');
                    Bento.audio.playSound('sfx_openscroll');
                    Utils.timeout(30, function () {
                        actors.shirley1.setSpriteSheet('shirley/block');
                        actors.shirley2.setSpriteSheet('shirley/block');
                        Utils.timeout(15, function () {
                            actors.shirley1.setSpriteSheet('shirley/idle');
                            actors.shirley2.setSpriteSheet('shirley/idle');
                            Utils.timeout(15, function () {
                                actors.shirley1.setSpriteSheet('shirley/block');
                                actors.shirley2.setSpriteSheet('shirley/block');
                                Utils.timeout(15, function () {
                                    actors.shirley1.setSpriteSheet('shirley/idle');
                                    actors.shirley2.setSpriteSheet('shirley/idle');
                                    Utils.timeout(15, function () {
                                        actors.shirley1.setSpriteSheet('shirley/block');
                                        actors.shirley2.setSpriteSheet('shirley/block');
                                        Utils.timeout(15, function () {
                                            actors.shirley1.setSpriteSheet('shirley/idle');
                                            actors.shirley2.setSpriteSheet('shirley/idle');
                                            Utils.timeout(20, function () {
                                                nextScene();
                                            });
                                        });
                                    });
                                });
                            });
                        });
                    });

                }
            }),
            new Scene({
                start: function(){
                    actors.shirley1.setDirection(1,1);
                    actors.shirley2.setDirection(1,1);
                    actors.shirley1.setSpriteSheet('shirley/jump');
                    actors.shirley2.setSpriteSheet('shirley/jump');
                    jumpTo("shirley1",actors.shirley1.position.x+120,actors.shirley1.position.y,70,null);
                    jumpTo("shirley2",actors.shirley2.position.x+120,actors.shirley2.position.y,70,nextScene);
                    jumpDust(actors.shirley2.position.x,actors.shirley2.position.y);
                    jumpDust(actors.shirley1.position.x,actors.shirley1.position.y);
                    Bento.audio.stopSound('sfx_special');
                    Bento.audio.playSound('sfx_special');
                }
            }),
            new Scene({
                start: function(){
                    splash(actors.shirley2.position.x,actors.shirley2.position.y);
                    splash(actors.shirley1.position.x,actors.shirley1.position.y);
                    Bento.audio.stopSound('sfx_openscroll');
                    Bento.audio.playSound('sfx_openscroll');
                    hideActor("shirley1");
                    hideActor("shirley2");
                    Utils.timeout(30, function () {
                        nextScene();
                    });
                }
            })
        ];

        var component = {
            name: 'cutscene', // should always be called 'cutscene'
            startCutscene: function (data) {
                Utils.forEach(scenes, function (scene, i, l, breakLoop) {
                    cutscene.addScene(scene);
                });
                nextScene();
            },
            setupActors: function () {
                setupActors();
            },
            attached: function (data) {
                cutscene = data.entity;
                nextScene = cutscene.nextScene;
                talk = cutscene.talk;
                focusOn = cutscene.focusOn;
            }
        };
        return component;
    };
});