/**
 * Darken's sprite
 * @moduleName Darken
 */
bento.define('components/darken', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'bento/packedimage'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween,
    PackedImage
) {
    'use strict';
    /**
     * Describe your settings object parameters
     * @param {Object} settings
     */
    return function (settings) {
        var viewport = Bento.getViewport();
        var entity;
        var sprite;
        var renderer = Bento.getRenderer();
        var canvas;
        var context;
        var packedImage;
        var component = {
            name: 'darken',
            visible: true,
            alpha: 1,
            start: function (data) {
                sprite = entity.getComponent('sprite');
                if (Utils.isDefined(settings.alpha)) {
                    component.alpha = settings.alpha;
                }

                canvas = document.createElement('canvas');
                canvas.width = sprite.frameWidth;
                canvas.height = sprite.frameHeight;

                context = canvas.getContext('2d');
                context.globalCompositeOperation = 'source-over';
                context.globalAlpha = 1;
                context.drawImage(
                    sprite.spriteImage.image,
                    sprite.spriteImage.x + sprite.sourceX,
                    sprite.spriteImage.y + sprite.sourceY,
                    sprite.frameWidth,
                    sprite.frameHeight,
                    0,
                    0,
                    sprite.frameWidth,
                    sprite.frameHeight
                );
                // fill with black
                context.globalAlpha = settings.alpha;
                context.globalCompositeOperation = 'source-atop';
                context.fillStyle = "#000000";
                context.fillRect(0, 0, sprite.frameWidth, sprite.frameHeight);

                packedImage = new PackedImage(canvas);
            },
            destroy: function () {
                // dispose?
            },
            update: function (data) {},
            draw: function (data) {
                if (!component.visible) {
                    return;
                }
                var oldAlpha = data.renderer.getOpacity();
                data.renderer.setOpacity(component.alpha * oldAlpha);

                // draw canvas as a sprite
                data.renderer.drawImage(
                    packedImage, 
                    0, 
                    0, 
                    packedImage.width, 
                    packedImage.height, 
                    sprite.origin.x * -1, 
                    sprite.origin.y * -1, 
                    sprite.frameWidth, 
                    sprite.frameHeight
                );

                // restore alpha
                data.renderer.setOpacity(oldAlpha);
            },
            attached: function (data) {
                entity = data.entity;
            }
        };
        return component;
    };
});