/**
 * Component for enemies when they are dead. Will remove them once the fly off the screen
 * @moduleName Dead
 */
bento.define('components/dead', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'entities/effect'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween,
    Effect
) {
    'use strict';
    /**
     * Describe your settings object parameters
     * @param {Object} settings
     */
    return function (settings) {
        var viewport = Bento.getViewport();
        var entity;
        var movable;
        var explode = function () {
            var position = entity.position.add(new Vector2(entity.transform.x, entity.transform.y));

            //calculate y-distance from entity's origin to entity's 'center'
            //this assumes that all characters have their origin at (0,0) and their sprites origin at the the bottom-center
            var yOff = (entity.dimension.height / 2) * Math.sin(entity.rotation + Math.PI / 2);
            position.y -= yOff;

            position.x = Utils.clamp(viewport.x, position.x, viewport.x + viewport.width);
            var poof = new Effect({
                position: position,
                spriteSheet: 'effects/flyout'
            });
            Bento.objects.attach(poof);
            Bento.objects.remove(entity);
            // screenshake
            Bento.objects.get('camera', function (camera) {
                camera.shake(10, 10);
            });
            Bento.audio.stopSound('sfx_death');
            Bento.audio.playSound('sfx_death');
        };
        var component = {
            name: 'dead',
            start: function (data) {
                movable = entity.getComponent('movable');
            },
            destroy: function (data) {},
            update: function (data) {
                // out of bounds
                var boundingBox = entity.getBoundingBox();
                if (!viewport.intersect(boundingBox.grow(8))) {
                    explode();
                }
                if (movable && Math.abs(movable.velocity.x) < 0.5) {
                    // somehow not moving anymore
                    explode();
                }
                //entity.rotation += Math.PI / 50;

            },
            draw: function (data) {},
            attached: function (data) {
                entity = data.entity;
            }
        };
        return component;
    };
});