/**
 * Draw object in viewport only
 * @moduleName DrawInViewport
 */
bento.define('components/drawinviewport', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween
) {
    'use strict';
    /**
     * Describe your settings object parameters
     * @param {Object} settings
     */
    var DrawInViewport = function (settings) {
        var entity;
        var edgeSize;
        var isInViewport = function () {
            var viewport = Bento.getViewport().clone();
            return viewport.grow(edgeSize || 0).intersect(entity.getBoundingBox());
        };
        var component = {
            name: 'drawInViewport',
            start: function (data) {
                if (DrawInViewport.disable) {
                    return;
                }
                if (!Utils.isDefined(settings.edgeSize)) {
                    if (!Utils.isDefined(settings.dimension)) {
                        // use half the dimension size as error margin
                        edgeSize = Math.max(entity.dimension.width / 2, entity.dimension.height);
                    } else {
                        // pass dimension if entity itself has no dimension
                        edgeSize = settings.dimension;
                    }
                } else {
                    edgeSize = settings.edgeSize;
                }
            },
            destroy: function (data) {},
            update: function (data) {
                if (DrawInViewport.disable) {
                    return;
                }
                var shouldHide = isInViewport();
                if (!edgeSize && !Utils.isDefined(settings.dimension)) {
                    // update
                    edgeSize = Math.max(entity.dimension.width / 2, entity.dimension.height);
                }
                if (isInViewport()) {
                    entity.transform.visible = true;
                } else {
                    entity.transform.visible = false;
                }
            },
            draw: function (data) {},
            attached: function (data) {
                entity = data.entity;
            }
        };
        return component;
    };
    DrawInViewport.disable = false;
    return DrawInViewport;
});