/**
 * Component that fills a colored rectangle, acts similar to a sprite
 * @moduleName FillRect
 */
bento.define('components/fillrect', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween
) {
    'use strict';
    /**
     * Describe your settings object parameters
     * @param {Object} settings
     * @param {Array} settings.color - Color array
     * @param {Rectangle} settings.size - Size of the array
     */
    return function (settings) {
        var viewport = Bento.getViewport();
        var entity;
        var rectangle = settings.size || settings.rectangle || new Rectangle(0, 0, 1, 1);
        var color = settings.color || [0, 0, 0, 1];
        var origin = settings.origin || new Vector2(0, 0);
        var component = {
            name: 'fillRect',
            start: function (data) {},
            destroy: function (data) {},
            update: function (data) {},
            draw: function (data) {
                data.renderer.translate(Math.round(-origin.x), Math.round(-origin.y));
                data.renderer.fillRect(color, rectangle.x, rectangle.y, rectangle.width, rectangle.height);
                data.renderer.translate(Math.round(origin.x), Math.round(origin.y));
            },
            attached: function (data) {
                entity = data.entity;
                entity.dimension = rectangle.clone();
                entity.dimension.x = -origin.x;
                entity.dimension.y = -origin.y;
            },
            setColor: function (array) {
                color = array;
            },
            setSize: function (size) {
                rectangle = size;
            },
            setOriginRelative: function (originRelative) {
                origin.x = rectangle.width * originRelative.x;
                origin.y = rectangle.height * originRelative.y;
            }
        };

        if (settings.originRelative) {
            component.setOriginRelative(settings.originRelative);
        }

        return component;
    };
});