/**
 * Component that fills the screen with a certain color
 * Similar to fillRect, but always fills the screen and adapts to resolution changes
 * @moduleName FillScreen
 */
bento.define('components/fillscreen', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween
) {
    'use strict';
    /**
     * Describe your settings object parameters
     * @param {Object} settings
     * @param {Array} settings.color - Color array
     * @param {Rectangle} settings.size - Size of the array
     */
    return function (settings) {
        var viewport = Bento.getViewport();
        var entity;
        var color = settings.color || [0, 0, 0, 1];
        var origin = settings.origin || new Vector2(0, 0);
        var padding = 64;
        var component = {
            name: 'fillScreen',
            start: function (data) {},
            destroy: function (data) {},
            update: function (data) {},
            draw: function (data) {
                // data.renderer.translate(Math.round(-entity.position.x), Math.round(-entity.position.y));
                data.renderer.translate(Math.round(-origin.x), Math.round(-origin.y));
                data.renderer.fillRect(color, -padding / 2, -padding / 2, viewport.width + padding, viewport.height + padding);
                data.renderer.translate(Math.round(origin.x), Math.round(origin.y));
                // data.renderer.translate(Math.round(entity.position.x), Math.round(entity.position.y));
            },
            attached: function (data) {
                entity = data.entity;
                entity.dimension = new Rectangle(0, 0, viewport.width, viewport.height);
            },
            setColor: function (array) {
                color = array;
            }
        };

        if (settings.originRelative) {
            origin.x = viewport.width * settings.originRelative.x;
            origin.y = viewport.height * settings.originRelative.y;
        }

        return component;
    };
});