/**
 * Flames visuals behind a sprite
 * @moduleName Flames
 */
bento.define('components/flames', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'bento/components/sprite'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween,
    Sprite
) {
    'use strict';
    /**
     * Describe your settings object parameters
     * @param {Object} settings
     */
    return function (settings) {
        var viewport = Bento.getViewport();
        var color = settings.color || 'red';
        var behavior = {
            name: 'flameBehavior',
            start: function (data) {
                entity.alpha = 0.5;
                // entity.scale.y = 0.8;

                new Tween({
                    from: 0,
                    to: 1,
                    in: 30,
                    delay: 0,
                    ease: 'elastic',
                    decay: 5,
                    oscillations: 1,
                    onStart: function () {},
                    onUpdate: function (v, t) {
                        entity.scale.x = v;
                        entity.scale.y = v;
                    },
                    onComplete: function () {

                    }
                });
            },
            onParentAttached: function (data) {
                // place on top
                entity.parent.moveComponentTo(entity, 0);
            },
            draw: function (data) {
                if (data.renderer.name === 'canvas2d' && color !== 'red') {
                    var context = data.renderer.getContext();
                    context.globalCompositeOperation = 'lighter';
                }
            },
            postDraw: function (data) {
                if (data.renderer.name === 'canvas2d' && color !== 'red') {
                    var context = data.renderer.getContext();
                    context.globalCompositeOperation = 'source-over';
                }
            }
        };
        var entity = new Entity({
            name: 'flames',
            position: new Vector2(0, 0),
            components: [
                behavior,
                new Sprite({
                    spriteSheet: 'effects/flames-' + color
                })
            ]
        });
        return entity;
    };
});