/**
 * Glow by drawing the sprite again
 * @moduleName Glow
 */
bento.define('components/glow', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween
) {
    'use strict';
    /**
     * Describe your settings object parameters
     * @param {Object} settings
     */
    return function (settings) {
        var viewport = Bento.getViewport();
        var entity;
        var sprite;
        var alpha = 1;
        var timer = 0;
        var average = settings.average || 0.25;
        var amplitude = settings.amplitude || 0.25;
        var glowSpeed = settings.glowSpeed || 0.05;
        var renderer = Bento.getRenderer();
        var component = {
            name: 'glow',
            alpha: 1,
            start: function (data) {
                sprite = entity.getComponent('sprite');
            },
            update: function (data) {
                timer += data.speed;
                alpha = average + amplitude * Math.sin(timer * glowSpeed);
                if (settings.update) {
                    settings.update(data);
                }
            },
            draw: function (data) {
                // warning: only for 2d context
                if (renderer.name !== 'canvas2d') {
                    return;
                }
                if (!component.visible || !sprite) {
                    return;
                }

                var context = data.renderer.getContext();
                var oldAlpha = context.globalAlpha;
                context.globalCompositeOperation = 'lighter';
                context.globalAlpha = alpha * component.alpha;

                // draw sprite again
                sprite.draw(data);

                // revert context
                context.globalCompositeOperation = 'source-over';
                context.globalAlpha = oldAlpha;

            },
            attached: function (data) {
                entity = data.entity;
            },
            visible: true
        };
        return component;
    };
});