/**
 * Glow by drawing a sprite
 * @moduleName GlowSprite
 */
bento.define('components/glowsprite', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'bento/components/sprite'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween,
    Sprite
) {
    'use strict';
    /**
     * Describe your settings object parameters
     * @param {Object} settings
     */
    return function (settings) {
        var viewport = Bento.getViewport();
        var sprite;
        var alpha = settings.alpha || 1;
        var timer = 0;
        var average = settings.average || 0.25;
        var amplitude = settings.amplitude || 0.25;
        var glowSpeed = settings.glowSpeed || 0.05;
        var renderer = Bento.getRenderer();
        var behavior = {
            name: 'behavior',
            update: function (data) {
                timer += data.speed;
                component.alpha = alpha * (average + amplitude * Math.sin(timer * glowSpeed));
                if (settings.update) {
                    settings.update(data);
                }
            }
        };
        var component = new Entity({
            name: 'glow',
            position: settings.position,
            components: [
                new Sprite({
                    imageName: 'gui/glow',
                    originRelative: new Vector2(0.5, 0.5),
                }),
                behavior
            ]
        });
        return component;
    };
});