/**
 * Componenent that adds the invincible effect to a player
 * @moduleName Invincible
 */
bento.define('components/invincible', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'components/sparkles',
    'components/glow'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween,
    Sparkles,
    Glow
) {
    'use strict';
    /**
     * Describe your settings object parameters
     * @param {Object} settings
     */
    return function (settings) {
        var viewport = Bento.getViewport();
        var entity;
        var timer;
        var seconds = settings.seconds || 30;
        var sparkles = new Sparkles({
            spawnRate: 0.3,
            position: new Vector2(0, -16)
        });
        var glow = new Glow({
            glowSpeed: 0.2,
        });

        var component = {
            name: 'invincible',
            start: function (data) {
                timer = Math.round(seconds * 60);
                entity.attach(sparkles);
                entity.attach(glow);
            },
            destroy: function (data) {
                entity.getComponent('punchable', function (punchable) {
                    punchable.isInvincible = false;
                });
                entity.remove(sparkles);
                entity.remove(glow);
            },
            update: function (data) {
                timer--;
                if (timer <= 0) {
                    entity.remove(component);
                } else {
                    entity.getComponent('punchable', function (punchable) {
                        punchable.isInvincible = true;
                    });
                }
            },
            draw: function (data) {},
            attached: function (data) {
                entity = data.entity;
            }
        };
        return component;
    };
});