/**
 * Item Dropper drops a consumable or weapon when punchable dies
 * @moduleName ItemDropper
 */
bento.define('components/itemdropper', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'entities/coin',
    'modules/randomizer',
    'managers/buff',
    'entities/consumable',
    'entities/weapon'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween,
    Coin,
    Randomizer,
    BuffManager,
    Consumable,
    Weapon
) {
    'use strict';
    /**
     * Describe your settings object parameters
     * @param {Object} settings
     */
    return function (settings) {
        var viewport = Bento.getViewport();
        var entity;
        var randomizer;
        var seed;
        var itemToDrop = settings.item;
        var downOnly = settings.downOnly || false;
        var speedMultiplier = settings.speedMultiplier || 1;
        var ignoredFamilies = settings.ignoredFamilies || [];
        // drop item when dead
        var onDeath = function () {
            var speed = 0.5 + randomizer.getRandomFloat(0.5);
            var angle = randomizer.getRandomFloat(downOnly ? Math.PI : Math.PI * 2);
            var velocityZ = -2 - randomizer.getRandomFloat(2);
            var key = itemToDrop.split('/');
            if (key.length === 2) {
                if (key[0] === 'consumables') {
                    var consumable = new Consumable({
                        position: entity.position.clone(),
                        speed: speed * speedMultiplier,
                        angle: angle,
                        velocityZ: velocityZ,
                        item: key[1],
                        ignoredFamilies: ignoredFamilies
                    });
                    Bento.objects.attach(consumable);
                } else if (key[0] === 'weapons') {
                    var weapon = new Weapon({
                        type: key[1]
                    });
                    weapon.position = entity.position.add(new Vector2(0, -10));
                    weapon.getComponent('movable', function (movable) {
                        movable.velocityZ = -3;
                        movable.velocity.x = 0.2;
                        movable.gravity = 0.20;
                        movable.height = -20;
                    });
                    Bento.objects.attach(weapon);
                } else {
                    Utils.log('ERROR: key doesn\'t exist: ' + key[0], itemToDrop);
                }
            } else {
                Utils.log('ERROR: itemDropper cannot drop: ' + itemToDrop);
            }
        };
        var component = {
            name: 'itemDropper',
            start: function (data) {
                // create randomizer seed based on parent's name, which is deterministic
                seed = Utils.checksum(entity.name);
                randomizer = new Randomizer({
                    seed: seed
                });

                // register death on punchable
                entity.getComponent('punchable', function (punchable) {
                    punchable.onDeath(onDeath);
                });
            },
            destroy: function () {},
            cancelItemDrop: function () {
                // cancel consumable drop
                entity.getComponent('punchable', function (punchable) {
                    punchable.offDeath(onDeath);
                });

            },
            drop: function () {
                // force drop
                onDeath();
            },
            attached: function (data) {
                entity = data.entity;
            }
        };
        return component;
    };
});