/**
 * Cached text entity with localized 'miss' text
 * @moduleName MissText
 */
bento.define('components/misstext', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'modules/text',
    'components/leftshadow',
    'modules/localization',
    'globals'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween,
    Text,
    LeftShadow,
    Localization,
    Globals
) {
    'use strict';
    /**
     * Describe your settings object parameters
     * @param {Object} settings
     */
    return function (settings) {
        var viewport = Bento.getViewport();
        var entity;
        var miss;
        var height = 38;
        var startHeight = 56;
        var goHeight = 20;
        var dissapearHeight = 20;
        var component = {
            name: 'missText',
            start: function (data) {
                miss = new Text({
                    text: Localization.getText('missSpecialAttack'),
                    font: 'font',
                    fontSize: 16,
                    fontColor: '#ffda03',
                    align: 'center',
                    textBaseline: 'middle',
                    ySpacing: 0,
                    lineWidth: 1,
                    strokeStyle: '#160d32',
                    innerStroke: false,
                    pixelStroke: true // workaround for Cocoon bug
                }).attach(new LeftShadow({}));
                miss.z = Globals.Layers.DAMAGE_FLOATERS;
            },
            update: function (data) {
                if (miss.isAdded) {
                    miss.position.x = entity.position.x;
                    miss.position.y = entity.position.y - height;
                }
            },
            attached: function (data) {
                entity = data.entity;
            },
            showMiss: function () {
                if (miss && !miss.isAdded) {
                    Bento.objects.attach(miss);
                }
                new Tween({
                    from: 1,
                    to: 0,
                    in: 30,
                    delay: 10,
                    ease: 'easeOutQuad',
                    onStart: function () {},
                    onUpdate: function (v, t) {
                        miss.alpha = v;
                    }
                });
                new Tween({
                    from: startHeight,
                    to: startHeight + dissapearHeight,
                    in: 40,
                    delay: 0,
                    ease: 'easeInCubic',
                    onStart: function () {},
                    onUpdate: function (v, t) {
                        height = v;
                    }
                });
                height = startHeight;
                miss.alpha = 1;
            }
        };
        return component;
    };
});