/**
 * move a background by a vector each update, currently only vertically
 * @moduleName MoveBackground
 */
bento.define('components/movebackground', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween
) {
    'use strict';
    /**
     * Describe your settings object parameters
     * @param {Object} settings
     */
    return function (settings) {
        var viewport = Bento.getViewport();
        var screenDimension = Bento.screens.getCurrentScreen().dimension;
        var entity;
        var component = {
            name: 'moveBackground',
            move: settings.move || new Vector2(0, 0),
            start: function (data) {},
            destroy: function (data) {},
            update: function (data) {
                entity.position.y += component.move.y * data.speed;
                entity.position.y = entity.position.y % screenDimension.height;
            },
            draw: function (data) {
                // draw the sprite a second time above current one
                var sprite = entity.getComponent('sprite');
                if(sprite) {
                    data.renderer.drawImage(
                        sprite.spriteImage,
                        sprite.sourceX,
                        sprite.sourceY,
                        sprite.frameWidth,
                        sprite.frameHeight,
                        0, screenDimension.height * -Utils.sign(entity.position.y),
                        sprite.frameWidth,
                        sprite.frameHeight
                    );
                }
            },
            attached: function (data) {
                entity = data.entity;
                entity.removeByName('drawInViewport');
            }
        };
        return component;
    };
});