/**
 * Applies multiple scales to entity
 * @moduleName MultiScale
 */
bento.define('components/multiscale', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween
) {
    'use strict';
    var warned = false;
    return function (settings) {
        var viewport = Bento.getViewport();
        var entity;
        var scaleNames = []; // keep track of scale names
        var preventZero = settings.preventZero || false; // prevents scale from becoming 0
        var lastKnownSign = new Vector2(1, 1);
        var component = {
            name: 'multiScale',
            update: function (data) {
                entity.scale.x = 1;
                entity.scale.y = 1;
                Utils.forEach(scaleNames, function (name) {
                    var scale = component[name];
                    entity.scale.x *= scale.x;
                    entity.scale.y *= scale.y;
                });

                if (preventZero) {
                    entity.scale.x = entity.scale.x || lastKnownSign.x;
                    entity.scale.y = entity.scale.y || lastKnownSign.y;

                    lastKnownSign.x = Utils.sign(entity.scale.x) || 1;
                    lastKnownSign.y = Utils.sign(entity.scale.y) || 1;
                }
            },
            attached: function (data) {
                entity = data.entity;
                if (settings.scales) {
                    Utils.forEach(settings.scales, function (vector, key) {
                        component.addScale(key, vector);
                    });
                }
            },
            addScale: function (name, vector) {
                if (component[name]) {
                    Utils.log("WARNING: scale " + name + " already exists");
                    return;
                }
                if (!warned && vector && vector.x === 0 && vector.y === 0) {
                    Utils.log("WARNING: passing new Vector(0, 0) results into an invisible entity");
                    warned = true;
                }
                component[name] = vector || new Vector2(1, 1);
                scaleNames.push(name);
            }
        };
        return component;
    };
});