/**
 * Radius flasher for explosions
 * @moduleName RadiusFlasher
 */
bento.define('components/radiusflasher', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween
) {
    'use strict';
    var RadiusFlasher = function (settings) {
        var currentColor = 0;
        var colors = settings.colors || [
            [1, 1, 0.5, 1]
        ];
        var radius = settings.radius || 1;
        var width = settings.width || 1;
        var scale = settings.scale || new Vector2(1, 1);
        var timeStart = settings.timeStart || 0;
        var every = settings.every || 10;
        var duration = settings.duration || 5;
        var alpha = settings.alpha || 1;
        var entity;
        var component = {
            name: 'radiusFlasher',
            timer: 0,
            colors: colors,
            update: function (data) {
                component.timer += data.speed;
            },
            draw: function (data) {
                if (component.timer > timeStart && component.timer % every < duration) {
                    data.renderer.save();
                    data.renderer.scale(scale.x, scale.y);
                    data.renderer.strokeCircle(component.colors[currentColor % component.colors.length], 0, 0, radius, 0, Math.PI * 2, width);
                    data.renderer.restore();
                    currentColor += 1;
                }
            },
            attached: function (data) {
                entity = data.entity;
            }
        };
        return component;
    };
    return RadiusFlasher;
});