/**
 * Shadow
 * @moduleName Shadow
 */
bento.define('components/shadow', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/utils',
    'bento/tween',
    'components/drawinviewport'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    Utils,
    Tween,
    DrawInViewport
) {
    'use strict';
    return function (settings) {
        var sprite = new Sprite({
            originRelative: new Vector2(0.5, 0.5),
            imageName: settings.imageName || 'shadow24'
        });
        var offset = settings.offset || 0;
        var offsetX = settings.offsetX || 0;
        var shadow = new Entity({
            z: 1,
            name: 'shadow',
            position: new Vector2(offsetX, offset),
            components: [sprite, new DrawInViewport({})],
            // alpha: 0.25
        });
        var entity;
        var component = {
            name: 'shadow',
            shadow: shadow,
            start: function () {
                Bento.objects.attach(shadow);
            },
            destroy: function () {
                Bento.objects.remove(shadow);
            },
            update: function () {
                shadow.position = entity.position.add(new Vector2(offsetX, offset));
                // shadow.z = entity.z - 1;
            },
            attached: function (data) {
                entity = data.entity;
            }
        };
        return component;
    };
});