/**
 * Shake character as reaction to getting hit
 * @moduleName Shake
 */
bento.define('components/shake', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween
) {
    'use strict';
    /**
     * Describe your settings object parameters
     * @param {Object} settings
     */
    return function (settings) {
        var viewport = Bento.getViewport();
        var entity;
        var shakeTimer = 0;
        var transform = new Vector2(0, 0);
        var component = {
            name: 'shake',
            start: function () {
                entity.moveComponentTo(component, 0);
            },
            update: function (data) {
                // shake
                if (shakeTimer > 0) {
                    shakeTimer -= data.speed;
                    transform.x = Utils.getRandomRange(-1, 1);
                    transform.y = Utils.getRandomRange(-1, 1);

                    if (shakeTimer <= 0) {
                        shakeTimer = 0;
                        transform.x = 0;
                        transform.y = 0;
                    }
                }
            },
            attached: function (data) {
                entity = data.entity;
            },
            draw: function (data) {
                data.renderer.translate(transform.x, transform.y);
            },
            postDraw: function (data) {
                data.renderer.translate(-transform.x, -transform.y);
            },
            shake: function (time) {
                shakeTimer = time;
            }
        };
        return component;
    };
});