/**
 * Sparkle emitter
 * @moduleName Sparkles
 */
bento.define('components/sparkles', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var offset = settings.offset || new Vector2(0, 0);
        var radius = settings.radius || 16;
        var spawnRate = settings.spawnRate || 0.025;
        var speed = settings.speed || 1;
        var spawnSparkle = function () {
            var frame = Utils.getRandom(3);
            var rotationSpeed = Utils.getRandomRangeFloat(0.01, 0.1);
            var verticalSpeed = Utils.getRandomRangeFloat(0.05, 0.25) * speed;
            var shrinkSpeed = 1 / Utils.getRandomRange(50, 90) * speed;
            var sparkleBehavior = {
                name: 'sparkleBehavior',
                update: function () {
                    // rotate
                    sparkle.rotation += rotationSpeed;
                    // fly up
                    sparkleContainer.position.y -= verticalSpeed;
                    // shrink
                    sparkleContainer.scale.x -= shrinkSpeed;
                    sparkleContainer.scale.y -= shrinkSpeed;
                    // sparkleContainer.alpha -= shrinkSpeed;

                    if (sparkleContainer.scale.x < 0) {
                        sparkleContainer.removeSelf();
                    }
                }
            };
            var sparkle = new Entity({
                name: 'sparkle',
                components: [
                    new Sprite({
                        imageName: 'gui/sparkles',
                        frameCountX: 3,
                        frameCountY: 1,
                        originRelative: new Vector2(0.5, 0.5),
                        animations: {
                            default: {
                                speed: 0,
                                frames: [frame]
                            }
                        }
                    })
                ]
            });
            var glow = new Entity({
                name: 'glow',
                components: [
                    new Sprite({
                        imageName: 'gui/sparkles-glow',
                        originRelative: new Vector2(0.5, 0.5)
                    })
                ]
            });
            var sparkleContainer = new Entity({
                name: 'sparkle',
                position: new Vector2(
                    Utils.getRandomRangeFloat(-radius, radius) + offset.x,
                    Utils.getRandomRangeFloat(-radius, radius) + offset.y
                ),
                components: [
                    glow,
                    sparkle,
                    sparkleBehavior
                ]
            });
            entity.attach(sparkleContainer);
            return sparkleContainer;
        };
        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {},
            destroy: function (data) {},
            update: function (data) {
                if (Utils.getRandomFloat(1) < spawnRate) {
                    spawnSparkle();
                }
            }
        };
        var entity = new Entity({
            z: settings.z,
            name: 'sparkles',
            position: settings.position,
            components: [
                behavior
            ]
        }).extend({
            setSpawnRate: function (value) {
                spawnRate = value;
            }
        });
        return entity;
    };
});