/**
 * Draws an extra edge on tiled map backgrounds. 
 * Note: the image/canvas size must be the same as the maxCanvasSize in the Tiled module 
 * otherwise the canvas is surrounded by transparency
 * Bonus: disables drawing when out of screen
 * @moduleName TiledEdge
 */
bento.define('components/tilededge', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween
) {
    'use strict';
    /**
     * Describe your settings object parameters
     * @param {Object} settings
     */
    return function (settings) {
        var viewport = Bento.getViewport();
        var entity;
        var image;
        var origin; // copy from sprite origin
        var component = {
            name: 'tiledEdge',
            start: function (data) {
                entity.getComponent('sprite', function (sprite) {
                    image = sprite.spriteImage;
                    origin = sprite.origin;
                });
            },
            destroy: function (data) {},
            update: function (data) {
                if (viewport.intersect(entity.getBoundingBox().grow(32))) {
                    entity.visible = true;
                } else {
                    entity.visible = false;
                }
            },
            draw: function (data) {
                var originOffsetX = (-origin.x) | 0;
                var originOffsetY = (-origin.y) | 0;
                // draw the lowest horizontal row
                data.renderer.drawImage(
                    image,
                    0,
                    image.height - 1,
                    image.width,
                    1,
                    originOffsetX,
                    originOffsetY + image.height - 1,
                    image.width,
                    2
                );
                // draw the right vertical column
                data.renderer.drawImage(
                    image,
                    image.width - 1,
                    0,
                    1,
                    image.height,
                    originOffsetX + image.width - 1,
                    originOffsetY,
                    2,
                    image.height
                );
            },
            attached: function (data) {
                entity = data.entity;
            }
        };
        return component;
    };
});