/**
 * A trigger component triggers a callback when the player character touches the parent entity
 * @moduleName Trigger
 */
bento.define('components/trigger', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'modules/matchmanager'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween,
    MatchManager
) {
    'use strict';
    return function (settings) {
        /*{
            onTrigger: function () {}, // called when player collides
        }*/
        var viewport = Bento.getViewport();
        var entity;
        var onTrigger = settings.onTrigger || function () {};
        var allPlayers = Utils.isDefined(settings.allPlayers) ? settings.allPlayers : true;

        var component = {
            name: 'trigger',
            active: true,
            start: function (data) {
                // get
            },
            update: function (data) {
                if (!component.active) {
                    return;
                }
                var triggerCount = 0;
                entity.collidesWith({
                    family: 'player',
                    firstOnly: false,
                    onCollide: function (other) {
                        var isDead = false;
                        other.getComponent('stateMachine', function (stateMachine) {
                            if (stateMachine.stateName === 'dead') {
                                isDead = true;
                            }
                        });
                        if (!isDead) {
                            // the player should not be dead
                            triggerCount++;
                        }
                    }
                });

                // count the dead players
                var deadCount = 0;
                Utils.forEach(Bento.objects.getByFamily('player'), function (player, i, l, breakLoop) {
                    player.getComponent('stateMachine', function (stateMachine) {
                        if (stateMachine.stateName === 'dead') {
                            deadCount += 1;
                        } else if (stateMachine.data.punchable && stateMachine.data.punchable.isDead) {
                            deadCount += 1;
                        }
                    });
                });


                if (
                    allPlayers &&
                    triggerCount !== 0 && // there should be at least 1 alive player
                    triggerCount + deadCount === MatchManager.otherPlayers.length + 1
                ) {
                    onTrigger();
                }
                if (!allPlayers && triggerCount > 0) {
                    onTrigger();
                }
            },
            attached: function (data) {
                entity = data.entity;
            }
        };
        return component;
    };
});