/**
 * first  part of the tutorial
 * @moduleName Tutorial1
 */
bento.define('components/tutorials/tutorial1', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'globals',
    'bento/components/sprite',
    'components/anchor',
    'gui/shadowtext',
    'managers/resolution',
    'gui/speechballoon',
    'modules/localization',
    'gui/fade',
    'managers/save',
    'entities/tutorial/target',
    'entities/hand',
    'entities/triggers/solidplayer',
    'entities/triggers/solid',
    'entities/regulatedspeech',
    'entities/tutorial/hitcount',
    'components/leftshadow',
    'modules/text',
    'modules/analytics'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween,
    Globals,
    Sprite,
    Anchor,
    ShadowText,
    ResolutionManager,
    SpeechBalloon,
    Localization,
    Fade,
    SaveManager,
    Target,
    Hand,
    SolidPlayer,
    Solid,
    RegulatedSpeech,
    HitCount,
    LeftShadow,
    Text,
    Analytics
) {
    'use strict';
    /**
     * Describe your settings object parameters
     * @param {Object} settings
     */
    return function (settings) {
        var viewport = Bento.getViewport();
        var entity;
        var phase = 0;
        var sequence = 0;
        var punchingBag;
        var player;
        var playerSprite;
        var sensei;
        var healthBar;
        var equipped;
        var camera;
        var cameraBehavior;

        // add hitcounter
        var hitCount = new HitCount({
            maxHits: 5,
            position: new Vector2(10, 64)
        });

        var leftSolid = new SolidPlayer({
            boundingBox: new Rectangle(0, 0, 16, 320)
        }).attach({
            name: 'leftSolidBehavior',
            update: function () {
                leftSolid.position.x = camera.position.x - 16;
            }
        });


        var rightSolid = new SolidPlayer({
            boundingBox: new Rectangle(0, 0, 16, 320)
        }).attach({
            name: 'leftSolidBehavior',
            update: function () {
                rightSolid.position.x = camera.position.x + viewport.width;
            }
        });

        var speechBalloon;
        var balloonTween;

        var transitionTween = false;

        var attack;
        var chargeCount;
        var charging;
        var chargeAttack;
        var removeGestures;


        var gestureText = new ShadowText({
            z: Globals.Layers.GUI,
            position: new Vector2(0, 0),
            text: 'gestures',
            font: 'font',
            fontSize: 16,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'bottom',
            ySpacing: 0,
            float: true,
            maxWidth: viewport.width - 20,
        }).attach(
            new Anchor({
                portrait: {
                    screenPosition: new Vector2(0.5, 0.6)
                },
                landscape: {
                    screenPosition: new Vector2(0.5, 0.9)
                }
            })
        );

        var awesomeText = new Text({
                z: Globals.Layers.GUI + 3,
                position: new Vector2(0, 0),
                text: Localization.getText('gradeAwesome'),
                font: 'font',
                fontSize: 16,
                fontColor: '#ff366c',
                align: 'center',
                textBaseline: 'bottom',
                ySpacing: 0,
                lineWidth: 1,
                strokeStyle: '#160d32',
                innerStroke: false,
                pixelStroke: true, // workaround for Cocoon bug
                float: true,
            }).attach(new LeftShadow({}))
            .attach(
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.5, 0.6)
                    },
                    landscape: {
                        screenPosition: new Vector2(0.5, 0.9)
                    }
                })
            );



        // gestures
        var tapSprite = new Sprite({
            spriteSheet: 'gui/hand-gesture-tap'
        });
        var tap = new Entity({
            z: Globals.Layers.GUI,
            name: 'tapGesture',
            family: ['gestures'],
            position: new Vector2(0, 0),
            float: true,
            components: [
                tapSprite,
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.5, 0.75),
                    },
                    landscape: {
                        screenPosition: new Vector2(1, 1),
                        offset: new Vector2(-32, -32)
                    }
                })
            ]
        });
        tapSprite.currentAnimation.backTo = 0;
        var hold = new Entity({
            z: Globals.Layers.GUI,
            name: 'holdGesture',
            family: ['gestures'],
            position: new Vector2(0, 0),
            float: true,
            components: [
                new Sprite({
                    spriteSheet: 'gui/hand-gesture-hold'
                }),
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.5, 0.75),
                    },
                    landscape: {
                        screenPosition: new Vector2(1, 1),
                        offset: new Vector2(-32, -32)
                    }
                })
            ]
        });
        var release = new Entity({
            z: Globals.Layers.GUI,
            name: 'releaseGesture',
            family: ['gestures'],
            position: new Vector2(0, 0),
            float: true,
            components: [
                new Sprite({
                    spriteSheet: 'gui/hand-gesture-release'
                }),
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.5, 0.75),
                    },
                    landscape: {
                        screenPosition: new Vector2(1, 1),
                        offset: new Vector2(-32, -32)
                    }
                })
            ]
        });
        var drag = new Entity({
            z: Globals.Layers.GUI,
            name: 'dragGesture',
            family: ['gestures'],
            position: new Vector2(0, 0),
            float: true,
            components: [
                new Sprite({
                    spriteSheet: 'gui/hand-gesture-drag'
                }),
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.5, 0.75),
                    },
                    landscape: {
                        screenPosition: new Vector2(0, 1),
                        offset: new Vector2(42, -42)
                    }
                })
            ]
        });

        var goRight = new Hand({}).attach({
            name: 'hide',
            update: function () {
                // hide when punchingBag is in view
                if (punchingBag && viewport.hasPosition(punchingBag.position)) {
                    goRight.visible = false;
                } else {
                    goRight.visible = true;
                }
            },
            destroy: function () {
                goRight.visible = true;
            }
        });

        //ground target
        var target = new Target({});

        // center speechBalloon to the screen and let the bottom follow sensei
        var centerBalloon = function () {
            if (speechBalloon) {
                speechBalloon.position.x = viewport.x + viewport.width / 2;
                speechBalloon.setBottomPosition(sensei.position.x - (viewport.x + viewport.width / 2) + 5);
            }
        };

        var affixPlayerToOther = function (other) {
            if (player.position.distance(target.position) > 1) {
                //walk
                playerSprite.setSpriteSheet('boy/walk');
                var difference = Vector2(other.position.x - player.position.x, other.position.y - player.position.y).normalize();
                player.position.x += difference.x;
                player.position.y += difference.y;
            } else {
                playerSprite.setSpriteSheet('boy/idle');
                player.position.x = other.position.x;
                player.position.y = other.position.y;
            }
        };

        var component = {
            name: 'tutorialBehavior',
            start: function (data) {
                punchingBag = Bento.objects.get('punchingBag');
                player = Bento.objects.get('player-0');
                playerSprite = player.getComponent('sprite');
                sensei = Bento.objects.get('sensei');
                healthBar = Bento.objects.get('healthBar');
                equipped = Bento.objects.get('equipped');
                camera = Bento.objects.get('camera');
                cameraBehavior = camera.getComponent('cameraBehavior');

                punchingBag.setPositionX(230);
                // player.position.x = 160;
                player.position.x = 210;
                player.position.y = 151;
                sensei.position.x = 90;
                sensei.position.y = 151;

                Bento.objects.attach(target);
                target.position.x = 90 + 32;
                target.position.y = 152;

                player.getComponent('touchController', function (touchController) {
                    touchController.canAttack(false);
                    touchController.canCharge(false);
                    touchController.canDash(false);
                    touchController.canJump(false);
                });

                player.getComponent('traditionalController', function (traditionalController) {
                    traditionalController.canAttack(false);
                    traditionalController.canCharge(false);
                    traditionalController.canDash(false);
                });


                Bento.objects.attach(leftSolid);
                Bento.objects.attach(rightSolid);
            },
            destroy: function (data) {
                EventSystem.clear('hitPunchingBag');
            },
            update: function (data) {
                if (phase === 0) {
                    //move towards sensei
                    var cameraOffset = new Vector2(0, 0);
                    if (ResolutionManager.getOrientation() === 'landscape') {
                        cameraOffset.y = -40;
                    }
                    camera.forceTo(new Vector2(160, 151).add(cameraOffset));
                    if (!speechBalloon) {
                        // speech = "Come over here Lee!"
                        speechBalloon = new RegulatedSpeech({
                            position: Vector2(160, sensei.position.y - 70),
                            touchControl: true,
                            text: Localization.getText('tutorial')['walk'],
                            bottomPosition: -100,
                            bottomFlip: true,
                            layer: Globals.Layers.GUI + 2,
                            disappear: true,
                            waitTime: 120
                        });

                        Bento.objects.attach(speechBalloon);

                        gestureText.setText(Localization.getText('gestures')['walk']);
                        Bento.objects.attach(gestureText);
                        Bento.objects.attach(drag);
                        new Tween({
                            from: 0,
                            to: 1,
                            in: 30,
                            delay: 0,
                            ease: 'linear',
                            onStart: function () {},
                            onUpdate: function (v, t) {
                                gestureText.alpha = v;
                                drag.alpha = v;
                            }
                        });
                    }
                    if (Math.abs(target.position.x - player.position.x) <= 20 &&
                        Math.abs(target.position.y - player.position.y) <= 10) {
                        if (sequence !== 2) {
                            if (sequence === 0) {
                                //remove existing balloon
                                if (speechBalloon) {
                                    speechBalloon.forceEnd();
                                }
                                //disable controls
                                EventSystem.fire('disableControls');
                                player.processAction({
                                    type: 'idle'
                                });

                                new Tween({
                                    from: 1,
                                    to: 0,
                                    in: 30,
                                    delay: 0,
                                    ease: 'linear',
                                    onStart: function () {},
                                    onUpdate: function (v, t) {
                                        gestureText.alpha = v;
                                        drag.alpha = v;
                                        target.alpha = v;
                                    },
                                    onComplete: function () {
                                        sequence = 2;
                                        Bento.objects.remove(gestureText);
                                        Bento.objects.remove(drag);
                                        Bento.objects.remove(target);

                                        // bug fix: by now the phase should be 1
                                        if (phase !== 1) {
                                            phase = 1;
                                            sequence = 0;
                                            Bento.objects.remove(leftSolid);
                                            Bento.objects.remove(rightSolid);
                                        }
                                    }
                                });
                                sequence = 1;
                            }
                        } else {
                            speechBalloon = null;
                            phase = 1;
                            sequence = 0;
                            Bento.objects.remove(leftSolid);
                            Bento.objects.remove(rightSolid);
                        }
                    }
                } else if (phase === 1) {
                    // story and then go to phase 2
                    // start first sequence
                    if (sequence === 0) {
                        player.getComponent('multiScale', function (multiScale) {
                            multiScale.direction.x = -1;
                        });

                        speechBalloon = new RegulatedSpeech({
                            position: Vector2(viewport.x + viewport.width / 2, sensei.position.y - 70),
                            touchControl: true,
                            text: Localization.getText('tutorial')['dialogue0'],
                            bottomPosition: sensei.position.x - (viewport.x + viewport.width / 2) + 5,
                            bottomFlip: true,
                            layer: Globals.Layers.GUI + 2,
                            onComplete: function () {
                                sequence = 2;
                                speechBalloon = new RegulatedSpeech({
                                    position: Vector2(viewport.x + viewport.width / 2, sensei.position.y - 78),
                                    touchControl: true,
                                    text: Localization.getText('tutorial')['dialogue1'],
                                    bottomPosition: sensei.position.x - (viewport.x + viewport.width / 2) + 5,
                                    bottomFlip: true,
                                    layer: Globals.Layers.GUI + 2,
                                    onComplete: function () {
                                        speechBalloon = null;
                                        phase = 2;
                                        sequence = 0;
                                    }
                                });
                                Bento.objects.attach(speechBalloon);

                                // add target
                                Utils.timeout(50, function () {
                                    new Tween({
                                        from: 0,
                                        to: 1,
                                        in: 30,
                                        ease: 'linear',
                                        onUpdate: function (v, t) {
                                            target.alpha = v;
                                        },
                                    });
                                    target.position.x = punchingBag.position.x - 20;
                                    Bento.objects.attach(target);
                                });
                            }
                        });

                        Bento.objects.attach(speechBalloon);
                        sequence = 1;
                    }
                    if (sequence < 2) {
                        affixPlayerToOther(target);
                    }
                    centerBalloon();
                } else if (phase === 2) {
                    if (sequence === 0) {
                        EventSystem.fire('enableControls');
                        Bento.objects.attach(goRight);
                        sequence = 1;
                    } else if (sequence === 1) {

                        // wait till player reaches punchingbag
                        if (Math.abs(target.position.x - player.position.x) <= 20 &&
                            Math.abs(target.position.y - player.position.y) <= 10) {
                            EventSystem.fire('disableControls');
                            player.processAction({
                                type: 'idle'
                            });

                            if (speechBalloon) {
                                speechBalloon.forceEnd();
                            }

                            sequence = 2;
                        }
                    } else if (sequence === 2) {
                        // face punchingbag

                        EventSystem.fire('disableControls');
                        player.processAction({
                            type: 'idle'
                        });

                        new Tween({
                            from: 1,
                            to: 0,
                            in: 30,
                            delay: 0,
                            ease: 'linear',
                            onUpdate: function (v, t) {
                                affixPlayerToOther(target);
                                player.getComponent('multiScale', function (multiScale) {
                                    multiScale.direction.x = 1;
                                });
                                target.alpha = v;
                                goRight.alpha = v;
                            },
                            onComplete: function () {
                                //enable attacks
                                player.getComponent('touchController', function (touchController) {
                                    touchController.canAttack(true);
                                    touchController.canMove(false);
                                });

                                player.getComponent('traditionalController', function (traditionalController) {
                                    traditionalController.canAttack(true);
                                    traditionalController.canMove(false);
                                });
                                phase = 3;
                                sequence = 0;
                                Bento.objects.remove(target);
                                Bento.objects.remove(goRight);
                                EventSystem.fire('enableControls');
                            }
                        });
                        sequence = 3;
                    }
                } else if (phase === 3) {
                    // attack punchingbag 5x
                    player.position.x = target.position.x;
                    if (!speechBalloon && !attack && sequence === 0) {
                        // speech = "Hit the punching bag a couple of times."
                        speechBalloon = new RegulatedSpeech({
                            position: Vector2(viewport.x + viewport.width / 2, sensei.position.y - 70),
                            touchControl: true,
                            blockControlEnd: true,
                            text: Localization.getText('tutorial')['attack'],
                            bottomPosition: sensei.position.x - (viewport.x + viewport.width / 2) + 5,
                            bottomFlip: true,
                            layer: Globals.Layers.GUI + 2,
                            disappear: true,
                            waitTime: 200,
                            onComplete: function () {
                                speechBalloon = null;
                            }
                        });
                        Bento.objects.attach(speechBalloon);

                        Bento.objects.attach(hitCount);

                        // function that removes speech for hitPunchingBag event
                        var removeGuiOnHit = function () {
                            //remove existing balloon
                            if (speechBalloon) {
                                speechBalloon.forceEnd();
                            }
                            EventSystem.off('hitPunchingBag', removeGuiOnHit);
                        };

                        attack = 0;
                        // function that counts the punches
                        var countPunches = function () {
                            attack++;
                            if (attack === 5) {
                                EventSystem.off('hitPunchingBag', countPunches);
                                new Tween({
                                    from: 1,
                                    to: 0,
                                    in: 30,
                                    delay: 0,
                                    ease: 'linear',
                                    onUpdate: function (v, t) {
                                        tap.alpha = v;
                                        gestureText.alpha = v;
                                    },
                                    onComplete: function () {
                                        Bento.objects.remove(tap);
                                        Bento.objects.remove(gestureText);
                                    }
                                });

                                hitCount.setHitCount({
                                    value: attack,
                                    reset: true
                                });

                                // Add awesome text
                                Bento.objects.attach(awesomeText);
                                new Tween({
                                    from: 0,
                                    to: 1.5,
                                    in: 30,
                                    delay: 0,
                                    ease: 'easeOutElastic',
                                    onUpdate: function (v, t) {
                                        awesomeText.scale.x = v;
                                        awesomeText.scale.y = v;
                                    },
                                    onComplete: function () {
                                        new Tween({
                                            from: 1,
                                            to: 0,
                                            in: 30,
                                            delay: 120,
                                            ease: 'linear',
                                            onUpdate: function (v, t) {
                                                awesomeText.alpha = v;
                                            },
                                            onComplete: function () {
                                                Bento.objects.remove(awesomeText);
                                            }
                                        });
                                    }
                                });

                                EventSystem.fire('disableControls');
                            } else {
                                hitCount.setHitCount({
                                    value: attack
                                });
                            }
                        };

                        EventSystem.on('hitPunchingBag', removeGuiOnHit);
                        EventSystem.on('hitPunchingBag', countPunches);

                        // add tap gesture and gesture text
                        gestureText.setText(Localization.getText('gestures')['attack']);
                        Bento.objects.attach(gestureText);
                        Bento.objects.attach(tap);
                        new Tween({
                            from: 0,
                            to: 1,
                            in: 30,
                            ease: 'linear',
                            onUpdate: function (v, t) {
                                tap.alpha = v;
                                gestureText.alpha = v;
                            }
                        });
                        sequence = 1;
                    } else {
                        if (attack >= 5) {
                            attack = null;
                            phase = 4;
                            sequence = 0;
                        } else if (sequence === 1) {
                            var difference = Math.max(viewport.x + 10, viewport.x + viewport.width / 2 - 80) - sensei.position.x;
                            sensei.getComponent('sprite', function (sprite) {
                                if (Math.abs(difference) > 1) {
                                    sprite.setSpriteSheet('sensei/walk');
                                    sensei.scale.x = Utils.sign(difference);
                                } else {
                                    sprite.setSpriteSheet('sensei/idle');
                                    sensei.scale.x = 1;
                                    sequence = 2;
                                }
                            });
                            sensei.position.x += (Math.max(viewport.x + 10, viewport.x + viewport.width / 2 - 80) - sensei.position.x) * 0.05;
                        } else {
                            sensei.getComponent('sprite', function (sprite) {
                                sprite.setSpriteSheet('sensei/idle');
                            });
                        }
                    }
                    centerBalloon();
                } else if (phase === 4) {
                    player.position.x = target.position.x;
                    //charge attack tutorial
                    if (sequence === 0) {
                        // speech = "Those punches and kicks are tight Lee."
                        speechBalloon = new RegulatedSpeech({
                            position: Vector2(viewport.x + viewport.width / 2, sensei.position.y - 70),
                            touchControl: true,
                            text: Localization.getText('tutorial')['dialogue2'],
                            bottomPosition: sensei.position.x - (viewport.x + viewport.width / 2) + 5,
                            bottomFlip: true,
                            layer: Globals.Layers.GUI + 2,
                            disappear: true,
                            waitTime: 200,
                            onComplete: function () {
                                sequence = 2;
                            }
                        });

                        Bento.objects.attach(speechBalloon);
                        sequence = 1;
                    } else if (sequence === 2) {
                        speechBalloon = new RegulatedSpeech({
                            position: Vector2(viewport.x + viewport.width / 2, sensei.position.y - 70),
                            touchControl: true,
                            text: Localization.getText('tutorial')['dialogue4'],
                            bottomPosition: sensei.position.x - (viewport.x + viewport.width / 2) + 5,
                            bottomFlip: true,
                            layer: Globals.Layers.GUI + 2,
                            disappear: true,
                            waitTime: 200,
                            onComplete: function () {
                                phase = 5;
                                EventSystem.fire('enableControls');

                                player.getComponent('touchController', function (touchController) {
                                    touchController.canMove(true);
                                });

                                goRight.alpha = 1;
                                Bento.objects.attach(goRight);
                                goRight.removeByName('hide');
                            }
                        });

                        Bento.objects.attach(speechBalloon);
                        sequence = 3;
                    }
                } else if (phase === 5) {
                    if (player.position.x >= 320 - 16) {
                        new Fade({
                            pause: Globals.PauseLevels.FADE,
                            duration: 10,
                            fadeIn: false,
                            type: 'opacity',
                            onComplete: function () {
                                // next section
                                SaveManager.save('tutorial', 2);
                                Analytics.event('Tutorial', 'gym_0_finish');
                                Bento.screens.show('screens/tutorial');
                            }
                        });
                    }
                }
            },
            draw: function (data) {},
            attached: function (data) {
                entity = data.entity;
            }
        };
        return component;
    };
});