/**
 * Second part of the tutorial
 * @moduleName Tutorial2
 */
bento.define('components/tutorials/tutorial2', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'globals',
    'bento/components/sprite',
    'components/anchor',
    'gui/shadowtext',
    'managers/resolution',
    'gui/speechballoon',
    'modules/localization',
    'entities/regulatedspeech',
    'entities/tutorial/target',
    'entities/hand',
    'gui/fade',
    'managers/save'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween,
    Globals,
    Sprite,
    Anchor,
    ShadowText,
    ResolutionManager,
    SpeechBalloon,
    Localization,
    RegulatedSpeech,
    Target,
    Hand,
    Fade,
    SaveManager
) {
    'use strict';
    /**
     * Describe your settings object parameters
     * @param {Object} settings
     */
    return function (settings) {
        var viewport = Bento.getViewport();
        var entity;
        var phase = 0;
        var sequence = 0;
        var punchingBag;
        var player;
        var playerSprite;
        var sensei;
        var healthBar;
        var equipped;
        var camera;
        var cameraBehavior;

        var count = 0;

        var gestureText = new ShadowText({
            z: Globals.Layers.GUI,
            position: new Vector2(0, 0),
            text: 'gestures',
            font: 'font',
            fontSize: 16,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'bottom',
            ySpacing: 0,
            float: true,
            maxWidth: viewport.width - 20,
        }).attach(
            new Anchor({
                portrait: {
                    screenPosition: new Vector2(0.5, 0.6)
                },
                landscape: {
                    screenPosition: new Vector2(0.5, 0.9)
                }
            })
        );

        var flick = new Entity({
            z: Globals.Layers.GUI,
            name: 'flickGesture',
            family: ['gestures'],
            position: new Vector2(0, 0),
            float: true,
            components: [
                new Sprite({
                    spriteSheet: 'gui/hand-gesture-flick'
                }),
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.5, 0.75),
                    },
                    landscape: {
                        screenPosition: new Vector2(0, 1),
                        offset: new Vector2(42, -42)
                    }
                })
            ]
        });

        var flickAndTap = new Entity({
            z: Globals.Layers.GUI,
            name: 'flickAndTapGesture',
            family: ['gestures'],
            position: new Vector2(0, 0),
            float: true,
            components: [
                new Sprite({
                    spriteSheet: 'gui/hand-gesture-flickntap'
                }),
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.5, 0.75),
                    },
                    landscape: {
                        screenPosition: new Vector2(0, 1),
                        offset: new Vector2(42, -42)
                    }
                })
            ]
        });

        var goRight = new Hand({});

        //ground target
        var target = new Target({});

        var speechBalloon;

        // center speechBalloon to the screen and let the bottom follow sensei
        var centerBalloon = function () {
            if (speechBalloon) {
                speechBalloon.position.x = viewport.x + viewport.width / 2;
                speechBalloon.setBottomPosition(sensei.position.x - (viewport.x + viewport.width / 2) + 5);
            }
        };

        var affixPlayerToOther = function (other) {
            if (player.position.distance(target.position) > 1) {
                //walk
                playerSprite.setSpriteSheet('boy/walk');
                var difference = Vector2(other.position.x - player.position.x, other.position.y - player.position.y).normalize();
                player.position.x += difference.x;
                player.position.y += difference.y;
            } else {
                playerSprite.setSpriteSheet('boy/idle');
                player.position.x = other.position.x;
                player.position.y = other.position.y;
            }
        };

        var moveSensei = function () {
            var difference = Math.max(viewport.x + 10, viewport.x + viewport.width / 2 - 80) - sensei.position.x;
            sensei.getComponent('sprite', function (sprite) {
                if (Math.abs(difference) > 1) {
                    sprite.setSpriteSheet('sensei/walk');
                    sensei.scale.x = Utils.sign(difference);
                } else {
                    sprite.setSpriteSheet('sensei/idle');
                    sensei.scale.x = 1;
                }
            });
            sensei.position.x += (Math.max(viewport.x + 10, viewport.x + viewport.width / 2 - 80) - sensei.position.x) * 0.05;
        };

        var component = {
            name: 'tutorialBehavior',
            start: function (data) {
                punchingBag = Bento.objects.get('punchingBag');
                player = Bento.objects.get('player-0');
                playerSprite = player.getComponent('sprite');
                sensei = Bento.objects.get('sensei');
                healthBar = Bento.objects.get('healthBar');
                equipped = Bento.objects.get('equipped');
                camera = Bento.objects.get('camera');
                cameraBehavior = camera.getComponent('cameraBehavior');

                punchingBag.setPositionX(354);
                player.position.x = 160;
                player.position.y = 151;
                sensei.position.x = 90;
                sensei.position.y = 151;

                target.position.y = 152;

                player.getComponent('touchController', function (touchController) {
                    touchController.canAttack(false);
                    touchController.canCharge(false);
                });

                player.getComponent('traditionalController', function (traditionalController) {
                    traditionalController.canAttack(false);
                    traditionalController.canCharge(false);
                });
                EventSystem.fire('disableControls');


                camera.forceTo(punchingBag.position);
            },
            destroy: function (data) {},
            update: function (data) {
                if (phase === 0) {
                    // hold position on the punching bag so the player knows it is to the right of the screen
                    var cameraOffset = new Vector2(0, 0);
                    if (ResolutionManager.getOrientation() === 'landscape') {
                        cameraOffset.y = -40;
                    }
                    camera.forceTo(punchingBag.position.add(cameraOffset));
                    count++;
                    if (count === 30) {
                        player.moveInDirection(new Vector2(-1, 0));
                        count = 0;
                        phase = 1;
                    }
                } else if (phase === 1) {
                    // pan to player
                    count++;
                    if (count === 10) {
                        player.moveInDirection(new Vector2(-1, 0));
                        count = 0;
                        phase = 2;
                    }
                } else if (phase === 2) {
                    //let sensei explain about dashing
                    //speech = "Fighting isn't all about punching and kicking."
                    if (!speechBalloon) {
                        speechBalloon = new RegulatedSpeech({
                            position: Vector2(160, sensei.position.y - 70),
                            touchControl: true,
                            text: Localization.getText('tutorial')['dialogue5'],
                            bottomPosition: -100,
                            bottomFlip: true,
                            layer: Globals.Layers.GUI + 2,
                            disappear: true,
                            waitTime: 200,
                            onComplete: function () {
                                EventSystem.fire('enableControls');
                                phase = 3;
                                // speech = "Show me your dashing technique."
                                speechBalloon = new RegulatedSpeech({
                                    position: Vector2(160, sensei.position.y - 70),
                                    touchControl: true,
                                    text: Localization.getText('tutorial')['dashing'],
                                    bottomPosition: sensei.position.x - (viewport.x + viewport.width / 2) + 5,
                                    bottomFlip: true,
                                    layer: Globals.Layers.GUI + 2,
                                    disappear: true,
                                    waitTime: 200,
                                    onComplete: function () {
                                        speechBalloon = null;
                                    }
                                });
                                Bento.objects.attach(speechBalloon);

                                // add flick gesture
                                gestureText.setText(Localization.getText('gestures')['dashing']);
                                Bento.objects.attach(gestureText);
                                Bento.objects.attach(flick);
                                new Tween({
                                    from: 0,
                                    to: 1,
                                    in: 30,
                                    ease: 'linear',
                                    onUpdate: function (v, t) {
                                        flick.alpha = v;
                                        gestureText.alpha = v;
                                    }
                                });
                            }
                        });
                        Bento.objects.attach(speechBalloon);
                    }
                } else if (phase === 3) {
                    if (sequence === 0) {
                        player.getComponent('stateMachine', function (stateMachine) {
                            if (stateMachine.stateName === 'running') {
                                new Tween({
                                    from: 1,
                                    to: 0,
                                    in: 30,
                                    delay: 0,
                                    ease: 'linear',
                                    onUpdate: function (v, t) {
                                        flick.alpha = v;
                                        gestureText.alpha = v;
                                    },
                                    onComplete: function () {
                                        Bento.objects.remove(flick);
                                        Bento.objects.remove(gestureText);
                                        speechBalloon = null;
                                        phase = 4;
                                        sequence = 0;
                                    }
                                });
                                if (speechBalloon) {
                                    speechBalloon.forceEnd();
                                }
                                sequence = 1;
                            }
                        });
                    }
                } else if (phase === 4) {
                    if (sequence === 0) {
                        // speech = "Very good, now return to me."
                        speechBalloon = new RegulatedSpeech({
                            position: Vector2(160, sensei.position.y - 70),
                            touchControl: false,
                            text: Localization.getText('tutorial')['dialogue6'],
                            bottomPosition: sensei.position.x - (viewport.x + viewport.width / 2) + 5,
                            bottomFlip: true,
                            layer: Globals.Layers.GUI + 2,
                            disappear: true,
                            waitTime: 200,
                            onComplete: function () {
                                speechBalloon = null;
                            }
                        });
                        Bento.objects.attach(speechBalloon);
                        target.position.x = sensei.position.x + 32;
                        Bento.objects.attach(target);
                        new Tween({
                            from: 0,
                            to: 1,
                            in: 30,
                            ease: 'linear',
                            onUpdate: function (v, t) {
                                target.alpha = v;
                            }
                        });
                        sequence = 1;
                    } else if (sequence === 1) {
                        if (Math.abs(target.position.x - player.position.x) <= 20 &&
                            Math.abs(target.position.y - player.position.y) <= 10) {
                            player.processAction('idle');
                            player.getComponent('multiScale', function (multiScale) {
                                multiScale.direction.x = -1;
                            });
                            if (!speechBalloon) {
                                phase = 5;
                                EventSystem.fire('disableControls');
                                affixPlayerToOther(target);
                            } else {
                                sequence = 2;
                            }
                            new Tween({
                                from: target.alpha,
                                to: 0,
                                in: Math.ceil(target.alpha * 30),
                                ease: 'linear',
                                onUpdate: function (v, t) {
                                    target.alpha = v;
                                },
                                onComplete: function () {
                                    Bento.objects.remove(target);
                                }
                            });
                            if (speechBalloon) {
                                speechBalloon.forceEnd();
                            }
                        }
                    } else if (sequence === 2) {
                        player.getComponent('multiScale', function (multiScale) {
                            multiScale.direction.x = -1;
                        });
                        EventSystem.fire('disableControls');
                        if (Bento.objects.get('target')) {
                            affixPlayerToOther(target);
                        }
                        if (!speechBalloon) {
                            phase = 5;
                        }
                    }
                    centerBalloon();
                } else if (phase === 5) {
                    // speech = "Let's try doing a dash attack."
                    if (!speechBalloon) {
                        speechBalloon = new RegulatedSpeech({
                            position: Vector2(viewport.x + viewport.width / 2, sensei.position.y - 70),
                            touchControl: true,
                            text: Localization.getText('tutorial')['dialogue7'],
                            bottomPosition: sensei.position.x - (viewport.x + viewport.width / 2) + 5,
                            bottomFlip: true,
                            layer: Globals.Layers.GUI + 2,
                            disappear: true,
                            waitTime: 200,
                            onComplete: function () {

                                player.getComponent('touchController', function (touchController) {
                                    touchController.canAttack(true);
                                });

                                player.getComponent('traditionalController', function (traditionalController) {
                                    traditionalController.canAttack(true);
                                });
                                EventSystem.fire('enableControls');
                                phase = 6;

                                // add flick and tap gesture
                                gestureText.setText(Localization.getText('gestures')['dashAttack']);
                                Bento.objects.attach(gestureText);
                                Bento.objects.attach(flickAndTap);
                                new Tween({
                                    from: 0,
                                    to: 1,
                                    in: 30,
                                    ease: 'linear',
                                    onUpdate: function (v, t) {
                                        flickAndTap.alpha = v;
                                        gestureText.alpha = v;
                                    }
                                });
                                speechBalloon = null;

                            }
                        });
                        Bento.objects.attach(speechBalloon);
                    }
                    affixPlayerToOther(target);
                    centerBalloon();
                } else if (phase === 6) {
                    player.getComponent('stateMachine', function (stateMachine) {
                        if (stateMachine.stateName === 'runAttack') {
                            new Tween({
                                from: 1,
                                to: 0,
                                in: 30,
                                delay: 0,
                                ease: 'linear',
                                onUpdate: function (v, t) {
                                    flickAndTap.alpha = v;
                                    gestureText.alpha = v;
                                },
                                onComplete: function () {
                                    Bento.objects.remove(flickAndTap);
                                    Bento.objects.remove(gestureText);
                                    phase = 8;
                                }
                            });
                            phase = 7;
                        }
                    });
                } else if (phase === 8) {
                    count++;
                    if (count === 30) {
                        player.moveInDirection(new Vector2(-1, 0));
                        EventSystem.fire('disableControls');
                        count = 0;
                        phase = 9;
                    }
                    moveSensei();
                } else if (phase === 9) {
                    // speech = "Good, did you know you run like a girl?"
                    if (!speechBalloon) {
                        speechBalloon = new RegulatedSpeech({
                            position: Vector2(viewport.x + viewport.width / 2, sensei.position.y - 70),
                            touchControl: true,
                            text: Localization.getText('tutorial')['dialogue8'],
                            bottomPosition: sensei.position.x - (viewport.x + viewport.width / 2) + 5,
                            bottomFlip: true,
                            layer: Globals.Layers.GUI + 2,
                            disappear: true,
                            waitTime: 200,
                            onComplete: function () {
                                speechBalloon = null;
                                phase = 10;
                            }
                        });
                        Bento.objects.attach(speechBalloon);
                    }
                    centerBalloon();
                    moveSensei();
                } else if (phase === 10) {
                    // speech = "Move along, I'll be with you shortly."
                    if (!speechBalloon) {
                        sensei.getComponent('sprite', function (sprite) {
                            sprite.setSpriteSheet('sensei/idle');
                        });
                        speechBalloon = new RegulatedSpeech({
                            position: Vector2(viewport.x + viewport.width / 2, sensei.position.y - 70),
                            touchControl: true,
                            text: Localization.getText('tutorial')['dialogue9'],
                            bottomPosition: sensei.position.x - (viewport.x + viewport.width / 2) + 5,
                            bottomFlip: true,
                            layer: Globals.Layers.GUI + 2,
                            disappear: true,
                            waitTime: 200,
                            onComplete: function () {
                                EventSystem.fire('enableControls');
                                goRight.alpha = 1;
                                Bento.objects.attach(goRight);
                            }
                        });
                        Bento.objects.attach(speechBalloon);
                    }
                    centerBalloon();
                    phase = 11;
                } else if (phase === 11) {
                    if (player.position.x >= 464 - 16) {
                        new Fade({
                            pause: Globals.PauseLevels.FADE,
                            duration: 10,
                            fadeIn: false,
                            type: 'opacity',
                            onComplete: function () {
                                // next section
                                SaveManager.save('tutorial', 2);
                                Bento.screens.show('screens/tutorial');
                            }
                        });
                    }
                    centerBalloon();
                }
            },
            draw: function (data) {},
            attached: function (data) {
                entity = data.entity;
            }
        };
        return component;
    };
});