/**
 * third part of the tutorial
 * @moduleName Tutorial3
 */
bento.define('components/tutorials/tutorial3', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'globals',
    'bento/components/sprite',
    'components/anchor',
    'gui/shadowtext',
    'managers/resolution',
    'gui/speechballoon',
    'modules/localization',
    'gui/fade',
    'managers/save',
    'entities/tutorial/target',
    'entities/hand',
    'entities/triggers/solidplayer',
    'entities/triggers/solid',
    'entities/regulatedspeech',
    'entities/tutorial/hitcount',
    'components/leftshadow',
    'modules/text',
    'managers/inventory',
    'components/fillscreen',
    'components/damagecounter',
    'entities/props/lootdrop',
    'modules/analytics',
    'entities/cutscene'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween,
    Globals,
    Sprite,
    Anchor,
    ShadowText,
    ResolutionManager,
    SpeechBalloon,
    Localization,
    Fade,
    SaveManager,
    Target,
    Hand,
    SolidPlayer,
    Solid,
    RegulatedSpeech,
    HitCount,
    LeftShadow,
    Text,
    InventoryManager,
    FillScreen,
    DamageCounter,
    LootDrop,
    Analytics,
    Cutscene
) {
    'use strict';
    /**
     * Describe your settings object parameters
     * @param {Object} settings
     */
    return function (settings) {
        var viewport = Bento.getViewport();
        var entity;
        var phase = 0;
        var punchingBag;
        var player;
        var playerSprite;
        var sensei;
        var healthBar;
        var equipped;
        var camera;
        var cameraBehavior;
        var cutscene = new Cutscene({}); // there's no cutscene, but the talk method is useful here

        var speechBalloon;
        var sequence = 0;

        var gestureText = new ShadowText({
            z: Globals.Layers.GUI + 3,
            position: new Vector2(0, 0),
            text: 'gestures',
            font: 'font',
            fontSize: 16,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'bottom',
            ySpacing: 0,
            float: true,
            maxWidth: viewport.width - 20,
        }).attach(
            new Anchor({
                portrait: {
                    screenPosition: new Vector2(0.5, 0.6)
                },
                landscape: {
                    screenPosition: new Vector2(0.5, 0.9)
                }
            })
        );
        var grabable;
        var attack;
        var awesomeText = new Text({
                z: Globals.Layers.GUI + 4,
                position: new Vector2(0, 0),
                text: Localization.getText('gradeAwesome'),
                font: 'font',
                fontSize: 16,
                fontColor: '#ff366c',
                align: 'center',
                textBaseline: 'bottom',
                ySpacing: 0,
                lineWidth: 1,
                strokeStyle: '#160d32',
                innerStroke: false,
                pixelStroke: true, // workaround for Cocoon bug
                float: true,
            }).attach(new LeftShadow({}))
            .attach(
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.5, 0.6)
                    },
                    landscape: {
                        screenPosition: new Vector2(0.5, 0.9)
                    }
                })
            );
        // add hitcounter
        var hitCount = new HitCount({
            position: new Vector2(10, 64),
            maxHits: 3
        });
        // gestures
        var tap = new Entity({
            z: Globals.Layers.GUI,
            name: 'tapGesture',
            family: ['gestures'],
            position: new Vector2(0, 0),
            float: true,
            components: [
                new Sprite({
                    spriteSheet: 'gui/hand-gesture-tap'
                }),
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.5, 0.75),
                    },
                    landscape: {
                        screenPosition: new Vector2(1, 1),
                        offset: new Vector2(-32, -32)
                    }
                })
            ]
        });
        var hold = new Entity({
            z: Globals.Layers.GUI,
            name: 'holdGesture',
            family: ['gestures'],
            position: new Vector2(0, 0),
            float: true,
            components: [
                new Sprite({
                    spriteSheet: 'gui/hand-gesture-hold'
                }),
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.5, 0.75),
                    },
                    landscape: {
                        screenPosition: new Vector2(1, 1),
                        offset: new Vector2(-32, -32)
                    }
                })
            ]
        });
        var release = new Entity({
            z: Globals.Layers.GUI,
            name: 'releaseGesture',
            family: ['gestures'],
            position: new Vector2(0, 0),
            float: true,
            components: [
                new Sprite({
                    spriteSheet: 'gui/hand-gesture-release'
                }),
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.5, 0.75),
                    },
                    landscape: {
                        screenPosition: new Vector2(1, 1),
                        offset: new Vector2(-32, -32)
                    }
                })
            ]
        });
        var drag = new Entity({
            z: Globals.Layers.GUI,
            name: 'dragGesture',
            family: ['gestures'],
            position: new Vector2(0, 0),
            float: true,
            components: [
                new Sprite({
                    spriteSheet: 'gui/hand-gesture-drag'
                }),
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.5, 0.75),
                    },
                    landscape: {
                        screenPosition: new Vector2(0, 1),
                        offset: new Vector2(42, -42)
                    }
                })
            ]
        });
        var goRight = new Hand({});
        var flick = new Entity({
            z: Globals.Layers.GUI,
            name: 'flickGesture',
            family: ['gestures'],
            position: new Vector2(0, 0),
            float: true,
            components: [
                new Sprite({
                    spriteSheet: 'gui/hand-gesture-flick'
                }),
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.5, 0.75),
                    },
                    landscape: {
                        screenPosition: new Vector2(0, 1),
                        offset: new Vector2(42, -42)
                    }
                })
            ]
        });

        //ground target
        var target = new Target({});

        var affixPlayerToOther = function (other) {
            if (player.position.distance(target.position) > 1) {
                //walk
                playerSprite.setSpriteSheet('boy/walk');
                var difference = Vector2(other.position.x - player.position.x, other.position.y - player.position.y).normalize();
                player.position.x += difference.x;
                player.position.y += difference.y;
            } else {
                playerSprite.setSpriteSheet('boy/idle');
                player.position.x = other.position.x;
                player.position.y = other.position.y;
            }
        };

        // center speechBalloon to the screen and let the bottom follow sensei
        var centerBalloon = function () {
            if (speechBalloon) {
                speechBalloon.position.x = viewport.x + viewport.width / 2;
                speechBalloon.setBottomPosition(sensei.position.x - (viewport.x + viewport.width / 2) + 5);
            }
        };
        var hitPunchingBag = function (data) {
            sequence++;
            // if (speechBalloon) {
            //     speechBalloon.forceEnd();
            // }
            if (sequence === 2) {
                EventSystem.off('hitPunchingBag');
            }
        };

        var count = 0;
        var grabFailBehavior = {
            name: 'grabFail',
            isTalking: 0,
            update: function () {
                var isColliding = false;
                sensei.collidesWith({
                    family: 'player',
                    onCollide: function () {
                        if (grabFailBehavior.isTalking === 0 && (!speechBalloon || !speechBalloon.isAdded)) {
                            grabFailBehavior.isTalking = 1;
                            cutscene.talk({
                                text: Localization.getText('tutorial')['grabFail'],
                                who: sensei,
                                onComplete: function () {
                                    grabFailBehavior.isTalking = 2;
                                }
                            });
                        }
                    }
                });
                if (!isColliding && grabFailBehavior.isTalking === 2) {
                    grabFailBehavior.isTalking = 0;
                }
            }
        };
        var component = {
            name: 'tutorialBehavior',
            start: function (data) {
                Bento.objects.attach(cutscene);

                punchingBag = Bento.objects.get('punchingBag');
                player = Bento.objects.get('player-0');
                playerSprite = player.getComponent('sprite');
                sensei = Bento.objects.get('sensei');
                healthBar = Bento.objects.get('healthBar');
                equipped = Bento.objects.get('equipped');
                camera = Bento.objects.get('camera');
                cameraBehavior = camera.getComponent('cameraBehavior');

                // punchingBag.setPositionX(230);

                player.position.x = 148;
                player.position.y = 152;
                player.getComponent('multiScale', function (multiScale) {
                    multiScale.direction.x = -1;
                });

                sensei.position.x = 90;
                sensei.position.y = 151;

                camera.forceTo(punchingBag.position);

                player.getComponent('touchController', function (touchController) {
                    touchController.canJump(false);
                });
                player.canCharge = false;
                player.canDash = false;
                player.canJump = false;
            },
            destroy: function (data) {
                EventSystem.clear('hitPunchingBag');
            },
            update: function (data) {
                var difference;
                // grab during phase 1
                if (phase <= 1 && grabable && grabable.isGrabbed() !== null) {
                    phase = 2;
                    sequence = 0;
                    if (speechBalloon) {
                        speechBalloon.forceEnd();
                    }
                    target.removeSelf();

                    // align player with punchingbag
                    new Tween({
                        from: player.position.y,
                        to: punchingBag.position.y + 1,
                        in: 5,
                        ease: 'linear',
                        onUpdate: function (v, t) {
                            player.position.y = v;
                        },
                        onComplete: function () {

                        }
                    });
                }

                // grab release during phase 3
                if (phase === 3 && grabable && grabable.isGrabbed() === null) {
                    phase = 4;
                    sequence = 0;
                    if (speechBalloon) {
                        speechBalloon.forceEnd();
                    }
                }

                if (phase === 0) {
                    if (sequence === 0) {
                        // hold position on the punching bag so the player knows it is to the right of the screen
                        var cameraOffset = new Vector2(0, 0);
                        if (ResolutionManager.getOrientation() === 'landscape') {
                            cameraOffset.y = -40;
                        }
                        camera.forceTo(punchingBag.position.add(cameraOffset));
                        count++;
                        if (count === 30) {
                            player.moveInDirection(new Vector2(-1, 0));
                            count = 0;
                            sequence = 1;
                        }
                    } else if (sequence === 1) {
                        speechBalloon = new RegulatedSpeech({
                            position: Vector2(160, sensei.position.y - 70),
                            touchControl: true,
                            text: Localization.getText('tutorial')['grab1'],
                            bottomPosition: -100,
                            bottomFlip: true,
                            layer: Globals.Layers.GUI + 4,
                            disappear: true,
                            waitTime: 200,
                            onComplete: function () {
                                if (phase === 0) {
                                    phase = 1;
                                    sequence = 0;
                                    speechBalloon = null;
                                }
                            }
                        });
                        Bento.objects.attach(speechBalloon);

                        target.position = punchingBag.position.clone();
                        Bento.objects.attach(target);

                        sequence = 2;

                        grabable = punchingBag.addGrabBehavior();
                        sensei.attach(grabFailBehavior);

                        gestureText.setText(Localization.getText('gestures')['grab']);
                        Bento.objects.attach(gestureText);
                        Bento.objects.attach(drag);
                        new Tween({
                            from: 0,
                            to: 1,
                            in: 30,
                            ease: 'linear',
                            onUpdate: function (v, t) {
                                drag.alpha = v;
                                gestureText.alpha = v;
                            }
                        });
                    }
                    centerBalloon();
                } else if (phase === 1) {
                    // wait until player grabs
                } else if (phase === 2) {
                    if (sequence === 0) {
                        sensei.remove(grabFailBehavior);
                        speechBalloon = new RegulatedSpeech({
                            position: Vector2(160, sensei.position.y - 70),
                            touchControl: true,
                            text: Localization.getText('tutorial')['grab2'],
                            bottomPosition: -100,
                            bottomFlip: true,
                            layer: Globals.Layers.GUI + 4,
                            disappear: true,
                            waitTime: 200,
                            onComplete: function () {
                                speechBalloon = null;
                            }
                        });
                        Bento.objects.attach(speechBalloon);
                        centerBalloon();

                        Bento.objects.attach(hitCount);

                        // function that removes speech for hitPunchingBag event
                        var removeGuiOnHit = function () {
                            //remove existing balloon
                            if (speechBalloon) {
                                speechBalloon.forceEnd();
                            }
                            EventSystem.off('hitPunchingBag', removeGuiOnHit);
                        };

                        attack = 0;
                        // function that counts the punches
                        var countPunches = function () {
                            attack++;
                            if (attack === 3) {
                                EventSystem.off('hitPunchingBag', countPunches);
                                new Tween({
                                    from: 1,
                                    to: 0,
                                    in: 30,
                                    delay: 0,
                                    ease: 'linear',
                                    onUpdate: function (v, t) {
                                        tap.alpha = v;
                                        gestureText.alpha = v;
                                    },
                                    onComplete: function () {
                                        Bento.objects.remove(tap);
                                        Bento.objects.remove(gestureText);
                                    }
                                });

                                hitCount.setHitCount({
                                    value: attack,
                                    reset: true
                                });

                                // Add awesome text
                                Bento.objects.attach(awesomeText);
                                new Tween({
                                    from: 0,
                                    to: 1.5,
                                    in: 30,
                                    delay: 0,
                                    ease: 'easeOutElastic',
                                    onUpdate: function (v, t) {
                                        awesomeText.scale.x = v;
                                        awesomeText.scale.y = v;
                                    },
                                    onComplete: function () {
                                        Utils.timeout(20, function () {
                                            phase = 3;
                                            sequence = 0;
                                        });
                                        new Tween({
                                            from: 1,
                                            to: 0,
                                            in: 30,
                                            delay: 60,
                                            ease: 'linear',
                                            onUpdate: function (v, t) {
                                                awesomeText.alpha = v;
                                            },
                                            onComplete: function () {
                                                Bento.objects.remove(awesomeText);
                                            }
                                        });
                                    }
                                });

                            } else {
                                hitCount.setHitCount({
                                    value: attack
                                });
                            }
                        };

                        EventSystem.on('hitPunchingBag', removeGuiOnHit);
                        EventSystem.on('hitPunchingBag', countPunches);
                        // add tap gesture and gesture text
                        gestureText.setText(Localization.getText('gestures')['attack']);
                        Bento.objects.attach(tap);
                        new Tween({
                            from: 0,
                            to: 1,
                            in: 30,
                            ease: 'linear',
                            onUpdate: function (v, t) {
                                tap.alpha = v;
                                gestureText.alpha = v;
                            }
                        });

                        drag.removeSelf();

                        // disable throwing
                        player.getComponent('touchController', function (touchController) {
                            touchController.canMove(false);
                        });
                        player.getComponent('traditionalController', function (traditionalController) {
                            traditionalController.canMove(false);
                        });

                        sequence = 1;
                    }
                    difference = Math.max(viewport.x + 10, viewport.x + viewport.width / 2 - 80) - sensei.position.x;
                    sensei.getComponent('sprite', function (sprite) {
                        if (Math.abs(difference) > 1) {
                            sprite.setSpriteSheet('sensei/walk');
                            sensei.scale.x = Utils.sign(difference);
                        } else {
                            sprite.setSpriteSheet('sensei/idle');
                            sensei.scale.x = 1;
                        }
                    });
                    sensei.position.x += (Math.max(viewport.x + 10, viewport.x + viewport.width / 2 - 80) - sensei.position.x) * 0.05;
                } else if (phase === 3) {
                    if (sequence === 0) {
                        tap.removeSelf();
                        gestureText.removeSelf();
                        sequence = 1;
                    } else if (sequence === 1) {
                        speechBalloon = new RegulatedSpeech({
                            position: Vector2(160, sensei.position.y - 70),
                            touchControl: true,
                            text: Localization.getText('tutorial')['grab3'],
                            bottomPosition: -100,
                            bottomFlip: true,
                            layer: Globals.Layers.GUI + 4,
                            disappear: true,
                            waitTime: 200,
                            onComplete: function () {
                                if (phase === 0) {
                                    phase = 1;
                                    sequence = 0;
                                    speechBalloon = null;
                                }
                            }
                        });
                        centerBalloon();
                        Bento.objects.attach(speechBalloon);
                        sequence = 2;

                        // disable attacking
                        player.getComponent('touchController', function (touchController) {
                            touchController.canAttack(false);
                            touchController.canMove(true);
                        });
                        player.getComponent('traditionalController', function (traditionalController) {
                            traditionalController.canAttack(false);
                            traditionalController.canMove(true);
                        });

                        // add tap gesture and gesture text
                        gestureText.setText(Localization.getText('gestures')['dashing']);
                        Bento.objects.attach(gestureText);
                        flick.alpha = 0;
                        gestureText.alpha = 0;
                        Bento.objects.attach(flick);
                        new Tween({
                            from: 0,
                            to: 1,
                            in: 30,
                            delay: 60,
                            ease: 'linear',
                            onUpdate: function (v, t) {
                                flick.alpha = v;
                                gestureText.alpha = v;
                            }
                        });

                    }

                } else if (phase === 4) {
                    if (sequence === 0) {
                        // restore
                        player.getComponent('touchController', function (touchController) {
                            touchController.canAttack(true);
                            touchController.canMove(true);
                        });
                        player.getComponent('traditionalController', function (traditionalController) {
                            traditionalController.canAttack(true);
                            traditionalController.canMove(true);
                        });

                        flick.removeSelf();
                        gestureText.removeSelf();

                        sequence = 1;
                    }
                    if (sequence === 1) {
                        sequence = 2;
                        Utils.timeout(60, function () {
                            phase = 5;
                            sequence = 0;
                        });
                        EventSystem.fire('disableControls');
                        player.processAction({
                            type: 'idle'
                        });
                    }
                } else if (phase === 5) {
                    if (sequence === 0) {
                        player.getComponent('multiScale', function (multiScale) {
                            multiScale.direction.x = -1;
                        });
                        speechBalloon = new RegulatedSpeech({
                            position: Vector2(viewport.x + viewport.width / 2, sensei.position.y - 70),
                            touchControl: true,
                            text: Localization.getText('tutorial')['dialogue10.1'],
                            bottomPosition: sensei.position.x - (viewport.x + viewport.width / 2) + 5,
                            bottomFlip: true,
                            layer: Globals.Layers.GUI + 2,
                            disappear: true,
                            waitTime: 200,
                            onComplete: function () {
                                sequence = 2;
                            }
                        });
                        sequence = 1;
                        Bento.objects.attach(speechBalloon);
                    } else if (sequence === 2) {
                        speechBalloon = new RegulatedSpeech({
                            position: Vector2(viewport.x + viewport.width / 2, sensei.position.y - 70),
                            touchControl: true,
                            text: Localization.getText('tutorial')['dialogue10.2'],
                            bottomPosition: sensei.position.x - (viewport.x + viewport.width / 2) + 5,
                            bottomFlip: true,
                            layer: Globals.Layers.GUI + 2,
                            disappear: true,
                            waitTime: 200,
                            onComplete: function () {
                                sequence = 4;
                            }
                        });

                        Bento.objects.attach(speechBalloon);
                        sequence = 3;
                    } else if (sequence === 4) {
                        Utils.timeout(10, function () {
                            var itemJson = Bento.assets.getJson('scrolls/super');
                            var lootDrop = new LootDrop({
                                spriteSheet: itemJson.inventory.image,
                                speed: 2,
                                position: new Vector2(sensei.position.x, sensei.position.y),
                                dontShow: true,
                                forceDirection: true
                            });
                            Bento.objects.attach(lootDrop);
                        });

                        speechBalloon = new RegulatedSpeech({
                            position: Vector2(viewport.x + viewport.width / 2, sensei.position.y - 70),
                            touchControl: true,
                            text: Localization.getText('tutorial')['dialogue10.3'],
                            bottomPosition: sensei.position.x - (viewport.x + viewport.width / 2) + 5,
                            bottomFlip: true,
                            layer: Globals.Layers.GUI + 2,
                            disappear: true,
                            waitTime: 200,
                            onComplete: function () {
                                phase = 7;
                                sequence = 0;
                                // drop super attack scroll
                                InventoryManager.addItem('scrolls/super', 1);
                            }
                        });
                        sequence = 5;
                        Bento.objects.attach(speechBalloon);
                    }
                } else if (phase === 6) {

                } else if (phase === 7) {
                    // end part
                    if (sequence === 0) {
                        player.getComponent('multiScale', function (multiScale) {
                            multiScale.direction.x = -1;
                        });
                        // speech = "Congratulations, you're now an official Elite Fighter... intern."
                        speechBalloon = new RegulatedSpeech({
                            position: Vector2(160, sensei.position.y - 70),
                            touchControl: true,
                            text: Localization.getText('tutorial')['dialogue10'],
                            bottomPosition: -100,
                            bottomFlip: true,
                            layer: Globals.Layers.GUI + 2,
                            onComplete: function () {
                                // speech = "dialogue11":"Go and save the city."
                                speechBalloon = new RegulatedSpeech({
                                    position: Vector2(160, sensei.position.y - 70),
                                    touchControl: true,
                                    text: Localization.getText('tutorial')['dialogue11'],
                                    bottomPosition: -100,
                                    bottomFlip: true,
                                    layer: Globals.Layers.GUI + 2,
                                    onComplete: function () {
                                        // speech =  "dialogue12":"Intern?"
                                        speechBalloon = new RegulatedSpeech({
                                            position: Vector2(160, sensei.position.y - 70),
                                            touchControl: true,
                                            text: Localization.getText('tutorial')['dialogue12'],
                                            bottomPosition: 0,
                                            layer: Globals.Layers.GUI + 2,
                                            disappear: true,
                                            waitTime: 120,
                                            onComplete: function () {
                                                speechBalloon = null;
                                                // START GAME
                                                // TODO: add title screen instead of world select
                                                new Fade({
                                                    pause: Globals.PauseLevels.FADE,
                                                    duration: 10,
                                                    fadeIn: false,
                                                    type: 'opacity',
                                                    onComplete: function () {
                                                        // next section
                                                        SaveManager.save('tutorial', 3);
                                                        // set recently unlocked to default because of cheat menu force reset
                                                        SaveManager.save('recentlyUnlocked', [1]);
                                                        Analytics.event('Tutorial', 'gym_grab_finish');

                                                        Bento.screens.show('screens/worldselect');
                                                    }
                                                });
                                            }
                                        });
                                        Bento.objects.attach(speechBalloon);
                                    }
                                });
                                Bento.objects.attach(speechBalloon);
                            }
                        });
                        Bento.objects.attach(speechBalloon);
                        sequence = 1;
                    }
                    centerBalloon();

                    difference = Math.max(viewport.x + 10, viewport.x + viewport.width / 2 - 80) - sensei.position.x;
                    sensei.getComponent('sprite', function (sprite) {
                        if (Math.abs(difference) > 1) {
                            sprite.setSpriteSheet('sensei/walk');
                            sensei.scale.x = Utils.sign(difference);
                        } else {
                            sprite.setSpriteSheet('sensei/idle');
                            sensei.scale.x = 1;
                        }
                    });
                    sensei.position.x += (Math.max(viewport.x + 10, viewport.x + viewport.width / 2 - 80) - sensei.position.x) * 0.05;
                }
            },
            attached: function (data) {
                entity = data.entity;
            }
        };
        return component;
    };
});