/**
 * fifth tutorial, first ingame cutscene
 * @moduleName Tutorial5
 */
bento.define('components/tutorials/tutorial5', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'globals',
    'bento/components/sprite',
    'entities/pedestrians/granny',
    'entities/enemies/ratboss',
    'entities/regulatedspeech',
    'modules/localization',
    'managers/save',
    'modules/matchmanager',
    'gui/fade',
    'modules/analytics'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween,
    Globals,
    Sprite,
    Granny,
    RatBoss,
    RegulatedSpeech,
    Localization,
    SaveManager,
    MatchManager,
    Fade,
    Analytics
) {
    'use strict';
    /**
     * Describe your settings object parameters
     * @param {Object} settings
     */
    return function (settings) {
        var viewport = Bento.getViewport();
        var entity;
        var phase = 0;
        var sequence = 0;
        var granny;
        var ratBoss;
        var rat;
        var player;
        var camera;
        var cameraComponent;
        var cameraTarget = {
            position: new Vector2(0, 0)
        };
        var savedTutorial = SaveManager.load('tutorial');
        var moveDirection;
        var stateMachine;
        var speechBalloon;
        var initialize = false;
        var component = {
            name: 'tutorialBehavior',
            start: function () {
                Analytics.event('Tutorial', 'mission_1_cutscene');
            },
            destroy: function (data) {},
            update: function (data) {
                var section;
                if(!initialize) {
                    // CAUTION: cannot do this in the start function because it will double characters in the next screen
                    if (savedTutorial >= 4) {
                        // TODO: instead of going to next level, sync with other players?
                        if (savedTutorial <= 3) {
                            SaveManager.save('tutorial', 4);
                        }
                        section = SaveManager.load('currentSection');
                        section += 1;
                        SaveManager.save('currentSection', section);
                        Bento.screens.show('screens/main');
                    }
                    initialize = true;
                } else if (entity.started) {
                    if (phase === 0) {
                        granny = new Granny({});
                        granny.processAction({
                            type: 'idle'
                        });
                        ratBoss = new RatBoss({});
                        Bento.objects.attach(granny);
                        Bento.objects.attach(ratBoss);
                        rat = Bento.objects.get('character-51');

                        Bento.objects.getByFamily('player', function (players) {
                            Utils.forEach(players, function (obj, i, l, breakLoop) {
                                if (!obj.isRemote) {
                                    player = obj;
                                    breakLoop();
                                }
                            });
                        });

                        granny.position = new Vector2(200, 242);
                        ratBoss.position = new Vector2(280, 242);
                        rat.position = new Vector2(280, 260);

                        Bento.objects.get('camera', function (cam) {
                            camera = cam;
                            cam.getComponent('cameraBehavior', function (camComponent) {
                                cameraComponent = camComponent;
                                camComponent.target = granny;
                            });
                        });
                        phase = 1;
                        Utils.timeout(20, function () {
                            phase = 2;
                        });
                    } else if (phase === 2) {
                        stateMachine = granny.getComponent('stateMachine');
                        stateMachine.data.idleTime = 40;
                        if (sequence === 0) {
                            stateMachine.data.sprite.setSpriteSheet('pedestrians/granny/' + 'shocked');
                            stateMachine.data.movable.grounded = false;
                            stateMachine.data.movable.velocityZ = -2;
                            sequence = 1;
                        } else if (sequence === 1) {
                            if (stateMachine.data.movable.grounded) {
                                sequence = 2;
                                stateMachine.data.sprite.setSpriteSheet('pedestrians/granny/' + 'flee');
                                granny.getComponent('multiScale', function (multiScale) {
                                    multiScale.direction.x = -1;
                                });
                            }
                        } else if (sequence === 2) {
                            moveDirection = player.position.add(new Vector2(32, 0)).subtractFrom(granny.position);
                            if (moveDirection.magnitude() > 3) {
                                moveDirection.normalize();
                                granny.position.x += moveDirection.x * 3;
                                granny.position.y += moveDirection.y * 3;
                            } else {
                                granny.position.addTo(moveDirection);
                                stateMachine.data.sprite.setSpriteSheet('pedestrians/granny/' + 'idle');
                                sequence = 3;
                            }
                        } else if (sequence === 3) {
                            //granny and Lee dialogue
                            speechBalloon = new RegulatedSpeech({
                                position: Vector2(viewport.x + viewport.width / 2, granny.position.y - 70),
                                touchControl: true,
                                text: Localization.getText('tutorial')['dialogue13'],
                                bottomPosition: granny.position.x - (viewport.x + viewport.width / 2),
                                bottomFlip: false,
                                layer: Globals.Layers.GUI + 1,
                                onComplete: function () {
                                    speechBalloon = new RegulatedSpeech({
                                        position: Vector2(viewport.x + viewport.width / 2, granny.position.y - 70),
                                        touchControl: true,
                                        text: Localization.getText('tutorial')['dialogue14'],
                                        bottomPosition: granny.position.x - (viewport.x + viewport.width / 2),
                                        bottomFlip: false,
                                        layer: Globals.Layers.GUI + 1,
                                        onComplete: function () {
                                            speechBalloon = new RegulatedSpeech({
                                                position: Vector2(viewport.x + viewport.width / 2, granny.position.y - 70),
                                                touchControl: true,
                                                text: Localization.getText('tutorial')['dialogue15'],
                                                bottomPosition: granny.position.x - (viewport.x + viewport.width / 2),
                                                bottomFlip: false,
                                                layer: Globals.Layers.GUI + 1,
                                                onComplete: function () {
                                                    speechBalloon = new RegulatedSpeech({
                                                        position: Vector2(viewport.x + viewport.width / 2, granny.position.y - 70),
                                                        touchControl: true,
                                                        text: Localization.getText('tutorial')['dialogue16'],
                                                        bottomPosition: granny.position.x - (viewport.x + viewport.width / 2),
                                                        bottomFlip: false,
                                                        layer: Globals.Layers.GUI + 1,
                                                        onComplete: function () {
                                                            speechBalloon = new RegulatedSpeech({
                                                                position: Vector2(viewport.x + viewport.width / 2, player.position.y - 70),
                                                                touchControl: true,
                                                                text: Localization.getText('tutorial')['dialogue17'],
                                                                bottomPosition: player.position.x - (viewport.x + viewport.width / 2),
                                                                bottomFlip: true,
                                                                layer: Globals.Layers.GUI + 1,
                                                                onComplete: function () {
                                                                    sequence = 5;
                                                                    stateMachine.data.sprite.setSpriteSheet('pedestrians/granny/' + 'walk');
                                                                }
                                                            });
                                                            Bento.objects.attach(speechBalloon);
                                                        }
                                                    });
                                                    Bento.objects.attach(speechBalloon);
                                                }
                                            });
                                            Bento.objects.attach(speechBalloon);
                                        }
                                    });
                                    Bento.objects.attach(speechBalloon);
                                }
                            });
                            Bento.objects.attach(speechBalloon);
                            sequence = 4;
                        }
                        if (sequence === 5) {
                            moveDirection = player.position.add(new Vector2(28, 20)).subtractFrom(granny.position);
                            if (moveDirection.magnitude() > 0.8) {
                                moveDirection.normalize();
                                granny.position.x += moveDirection.x * 0.8;
                                granny.position.y += moveDirection.y * 0.8;
                            } else {
                                granny.position.addTo(moveDirection);
                                stateMachine.data.sprite.setSpriteSheet('pedestrians/granny/' + 'idle');
                                sequence = 6;
                            }
                        } else if (sequence === 6) {
                            speechBalloon = new RegulatedSpeech({
                                position: Vector2(viewport.x + viewport.width / 2, granny.position.y - 70),
                                touchControl: true,
                                text: Localization.getText('tutorial')['dialogue18'],
                                bottomPosition: granny.position.x - (viewport.x + viewport.width / 2),
                                layer: Globals.Layers.GUI + 1,
                                onComplete: function () {
                                    stateMachine.data.setState('flee');
                                    cameraComponent.target = player;
                                    Utils.timeout(30, function () {
                                        phase = 3;
                                        sequence = 0;
                                    });
                                }
                            });
                            Bento.objects.attach(speechBalloon);
                            sequence = 7;
                        }
                    } else if (phase === 3) {
                        if (sequence === 0) {
                            speechBalloon = new RegulatedSpeech({
                                position: Vector2(viewport.x + viewport.width / 2, player.position.y - 70),
                                touchControl: true,
                                text: Localization.getText('tutorial')['dialogue19'],
                                bottomPosition: player.position.x - (viewport.x + viewport.width / 2),
                                bottomFlip: true,
                                layer: Globals.Layers.GUI + 1,
                                onComplete: function () {
                                    cameraComponent.target = ratBoss;
                                    Utils.timeout(30, function () {
                                        sequence = 2;
                                    });
                                }
                            });
                            Bento.objects.attach(speechBalloon);
                            sequence = 1;
                        } else if (sequence === 2) {
                            speechBalloon = new RegulatedSpeech({
                                position: Vector2(viewport.x + viewport.width / 2, ratBoss.position.y - 90),
                                touchControl: true,
                                text: Localization.getText('tutorial')['dialogue20'],
                                bottomPosition: ratBoss.position.x - (viewport.x + viewport.width / 2),
                                layer: Globals.Layers.GUI + 1,
                                onComplete: function () {
                                    speechBalloon = new RegulatedSpeech({
                                        position: Vector2(viewport.x + viewport.width / 2, ratBoss.position.y - 90),
                                        touchControl: true,
                                        text: Localization.getText('tutorial')['dialogue21'],
                                        bottomPosition: ratBoss.position.x - (viewport.x + viewport.width / 2),
                                        layer: Globals.Layers.GUI + 1,
                                        onComplete: function () {
                                            speechBalloon = new RegulatedSpeech({
                                                position: Vector2(viewport.x + viewport.width / 2, ratBoss.position.y - 90),
                                                touchControl: true,
                                                text: Localization.getText('tutorial')['dialogue22'],
                                                bottomPosition: ratBoss.position.x - (viewport.x + viewport.width / 2),
                                                layer: Globals.Layers.GUI + 1,
                                                onComplete: function () {
                                                    phase = 4;
                                                    sequence = 0;
                                                }
                                            });
                                            Bento.objects.attach(speechBalloon);
                                        }
                                    });
                                    Bento.objects.attach(speechBalloon);
                                }
                            });
                            Bento.objects.attach(speechBalloon);
                        }
                        sequence = 3;
                    } else if (phase === 4) {
                        stateMachine = ratBoss.getComponent('stateMachine');
                        if (sequence === 0) {
                            // eat cheese
                            stateMachine.data.sprite.setSpriteSheet('ratboss/eatcheese', function () {
                                sequence = 2;
                            });
                        } else if (sequence === 2) {
                            // run away;
                            stateMachine.data.multiScale.direction.x = 1;
                            stateMachine.data.sprite.setSpriteSheet('ratboss/run');
                            sequence = 3;
                        } else if (sequence === 3) {
                            if (viewport.clone().grow(80).hasPosition(ratBoss.position)) {
                                ratBoss.position.x += 2;
                            } else {
                                Bento.objects.remove(ratBoss);
                                sequence = 4;
                                Utils.timeout(30, function () {
                                    phase = 5;
                                    cameraComponent.target = player;
                                });
                            }
                        }
                    } else if (phase === 5) {
                        new Fade({
                            pause: Globals.PauseLevels.FADE,
                            duration: 10,
                            fadeIn: false,
                            type: 'opacity',
                            onComplete: function () {
                                // next section
                                // TODO: instead of going to next level, sync with other players?
                                if (savedTutorial <= 3) {
                                    SaveManager.save('tutorial', 4);
                                }
                                section = SaveManager.load('currentSection');
                                section += 1;
                                SaveManager.save('currentSection', section);
                                Bento.screens.show('screens/main');
                            }
                        });
                        phase = 6;
                    }
                }
            },
            draw: function (data) {},
            attached: function (data) {
                entity = data.entity;
            }
        };
        return component;
    };
});