/**
 * sixth part of the tutorial - used in worldselect
 * @moduleName Tutorial6
 */
bento.define('components/tutorials/tutorial6', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'globals',
    'bento/components/sprite',
    'components/anchor'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween,
    Globals,
    Sprite,
    Anchor
) {
    'use strict';
    /**
     * Describe your settings object parameters
     * @param {Object} settings
     */
    return function (settings) {
        var viewport = Bento.getViewport();
        var entity;

        var phase = 0;
        var sequence = 0;

        var pointUpBehavior = {
            name: 'pointerBehavior',
            wave: 0,
            update: function () {
                pointAtItem.rotation = Utils.toRadian(90);
                pointUpBehavior.wave += 0.1;
                pointAtItem.position.y = viewport.height - 60 - Math.sin(pointUpBehavior.wave) * 10;
            }
        };

        var pointAtItem = new Entity({
            z: Globals.Layers.GUI - 1,
            name: 'pointAtItem',
            position: new Vector2(136, 148),
            float: true,
            components: [
                new Sprite({
                    imageName: 'gui/hand-right',
                    originRelative: new Vector2(0.5, 0.5)
                }),
                pointUpBehavior,
                new Anchor ({
                    screenPosition: new Vector2(0.5, 1),
                    offset: new Vector2(60, -30)
                })
            ]
        });

        var component = {
            name: 'tutorialBehavior',
            start: function (data) {
                Bento.objects.attach(pointAtItem);
                entity.updateWhenPaused = Globals.PauseLevels.IAP;
                Bento.objects.get('bottomBar', function (bottomBar) {
                    bottomBar.activateStats(false);
                    bottomBar.activateEquipment(false);
                    bottomBar.activateCharacter(false);
                });
                // hide shopbutton
                Bento.objects.get('shopButton', function (shopButton) {
                    shopButton.getComponent('clickButton', function (clickButton) {
                        clickButton.setActive(false);
                    });
                    shopButton.visible = false;
                });
                // hide giftbutton
                Bento.objects.get('giftButton', function (giftButton) {
                    giftButton.getComponent('clickButton', function (clickButton) {
                        clickButton.setActive(false);
                    });
                    giftButton.visible = false;
                });
                // hide mailbutton
                Bento.objects.get('mailButton', function (mailButton) {
                    mailButton.getComponent('clickButton', function (clickButton) {
                        clickButton.setActive(false);
                    });
                    mailButton.visible = false;
                });
            },
            destroy: function (data) {},
            update: function (data) {

            },
            draw: function (data) {},
            attached: function (data) {
                entity = data.entity;
            }
        };
        return component;
    };
});