/**
 * third part of the tutorial
 * @moduleName Tutorial3
 */
bento.define('components/tutorials/tutorial3', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'globals',
    'bento/components/sprite',
    'components/anchor',
    'gui/shadowtext',
    'managers/resolution',
    'gui/speechballoon',
    'modules/localization',
    'gui/fade',
    'managers/save',
    'entities/tutorial/target',
    'entities/hand',
    'entities/triggers/solidplayer',
    'entities/triggers/solid',
    'entities/regulatedspeech',
    'entities/tutorial/hitcount',
    'components/leftshadow',
    'modules/text',
    'managers/inventory',
    'components/fillscreen',
    'components/damagecounter'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween,
    Globals,
    Sprite,
    Anchor,
    ShadowText,
    ResolutionManager,
    SpeechBalloon,
    Localization,
    Fade,
    SaveManager,
    Target,
    Hand,
    SolidPlayer,
    Solid,
    RegulatedSpeech,
    HitCount,
    LeftShadow,
    Text,
    InventoryManager,
    FillScreen,
    DamageCounter
) {
    'use strict';
    /**
     * Describe your settings object parameters
     * @param {Object} settings
     */
    return function (settings) {
        var viewport = Bento.getViewport();
        var entity;
        var phase = 0;
        var punchingBag;
        var player;
        var playerSprite;
        var sensei;
        var healthBar;
        var equipped;
        var camera;
        var cameraBehavior;

        var speechBalloon;
        var sequence = 0;

        var gestureText = new ShadowText({
            z: Globals.Layers.GUI + 3,
            position: new Vector2(0, 0),
            text: 'gestures',
            font: 'font',
            fontSize: 16,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'bottom',
            ySpacing: 0,
            float: true,
            maxWidth: viewport.width - 20,
        }).attach(
            new Anchor({
                portrait: {
                    screenPosition: new Vector2(0.5, 0.6)
                },
                landscape: {
                    screenPosition: new Vector2(0.5, 0.9)
                }
            })
        );

        var darken = new Entity({
            name: 'darken',
            z: Globals.Layers.GUI + 2,
            float: true,
            components: [
                new Sprite({
                    imageName: 'tutorial/itemoverlay',
                })
            ]
        });

        // gestures
        var tap = new Entity({
            z: Globals.Layers.GUI,
            name: 'tapGesture',
            family: ['gestures'],
            position: new Vector2(0, 0),
            float: true,
            components: [
                new Sprite({
                    spriteSheet: 'gui/hand-gesture-tap'
                }),
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.5, 0.75),
                    },
                    landscape: {
                        screenPosition: new Vector2(1, 1),
                        offset: new Vector2(-32, -32)
                    }
                })
            ]
        });
        var hold = new Entity({
            z: Globals.Layers.GUI,
            name: 'holdGesture',
            family: ['gestures'],
            position: new Vector2(0, 0),
            float: true,
            components: [
                new Sprite({
                    spriteSheet: 'gui/hand-gesture-hold'
                }),
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.5, 0.75),
                    },
                    landscape: {
                        screenPosition: new Vector2(1, 1),
                        offset: new Vector2(-32, -32)
                    }
                })
            ]
        });
        var release = new Entity({
            z: Globals.Layers.GUI,
            name: 'releaseGesture',
            family: ['gestures'],
            position: new Vector2(0, 0),
            float: true,
            components: [
                new Sprite({
                    spriteSheet: 'gui/hand-gesture-release'
                }),
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.5, 0.75),
                    },
                    landscape: {
                        screenPosition: new Vector2(1, 1),
                        offset: new Vector2(-32, -32)
                    }
                })
            ]
        });

        var pointUpBehavior = {
            name: 'pointerBehavior',
            wave: 0,
            update: function () {
                pointAtItem.rotation = Utils.toRadian(-90);
                pointUpBehavior.wave += 0.1;
                pointAtItem.position.y = 85 + Math.sin(pointUpBehavior.wave) * 10;
            }
        };

        var pointAtItem = new Entity({
            z: Globals.Layers.GUI + 3,
            name: 'pointAtItem',
            position: new Vector2(61, 90),
            float: true,
            components: [
                new Sprite({
                    imageName: 'gui/hand-right',
                    originRelative: new Vector2(0.5, 0.5)
                }),
                pointUpBehavior
            ]
        });

        var goRight = new Hand({});
        //ground target
        var target = new Target({});

        var affixPlayerToOther = function (other) {
            if (player.position.distance(target.position) > 1) {
                //walk
                playerSprite.setSpriteSheet('boy/walk');
                var difference = Vector2(other.position.x - player.position.x, other.position.y - player.position.y).normalize();
                player.position.x += difference.x;
                player.position.y += difference.y;
            } else {
                playerSprite.setSpriteSheet('boy/idle');
                player.position.x = other.position.x;
                player.position.y = other.position.y;
            }
        };

        // center speechBalloon to the screen and let the bottom follow sensei
        var centerBalloon = function () {
            if (speechBalloon) {
                speechBalloon.position.x = viewport.x + viewport.width / 2;
                speechBalloon.setBottomPosition(sensei.position.x - (viewport.x + viewport.width / 2) + 5);
            }
        };

        var useItem = function (data) {
            if (speechBalloon) {
                speechBalloon.forceEnd();
            }
            new Tween({
                from: 1,
                to: 0,
                in: 30,
                delay: 0,
                ease: 'linear',
                onUpdate: function (v, t) {
                    darken.alpha = v * 0.51;
                    gestureText.alpha = v;
                    pointAtItem.alpha = v;
                },
                onComplete: function () {
                    Bento.objects.remove(darken);
                    Bento.objects.remove(pointAtItem);

                    gestureText.setText(Localization.getText('gestures')['pickup']);
                    Bento.objects.attach(tap);
                    new Tween({
                        from: 0,
                        to: 1,
                        in: 30,
                        delay: 0,
                        ease: 'linear',
                        onStart: function () {},
                        onUpdate: function (v, t) {
                            gestureText.alpha = v;
                            tap.alpha = v;
                        }
                    });

                    EventSystem.fire('enableControls');
                    sequence = 0;
                    phase = 1;
                    EventSystem.on('hitPunchingBag', hitPunchingBag);
                }
            });
            EventSystem.off('consumableUsed', useItem);
        };

        var hitPunchingBag = function (data) {
            sequence++;
            // if (speechBalloon) {
            //     speechBalloon.forceEnd();
            // }
            if (sequence === 2) {
                EventSystem.off('hitPunchingBag');
            }
        };

        var component = {
            name: 'tutorialBehavior',
            start: function (data) {
                punchingBag = Bento.objects.get('punchingBag');
                player = Bento.objects.get('player-0');
                playerSprite = player.getComponent('sprite');
                sensei = Bento.objects.get('sensei');
                healthBar = Bento.objects.get('healthBar');
                equipped = Bento.objects.get('equipped');
                camera = Bento.objects.get('camera');
                cameraBehavior = camera.getComponent('cameraBehavior');

                punchingBag.setPositionX(230);

                player.position.x = 122;
                player.position.y = 152;
                player.getComponent('multiScale', function (multiScale) {
                    multiScale.direction.x = -1;
                });

                sensei.position.x = 90;
                sensei.position.y = 151;

                Bento.objects.attach(darken);
                Bento.objects.attach(pointAtItem);
                new Tween({
                    from: 0,
                    to: 1,
                    in: 30,
                    delay: 0,
                    ease: 'linear',
                    onUpdate: function (v, t) {
                        darken.alpha = v * 0.51;
                        pointAtItem.alpha = v;
                    }
                });
                EventSystem.on('consumableUsed', useItem);
            },
            destroy: function (data) {},
            update: function (data) {
                if (phase === 0) {
                    // fade to dark
                    // speech = "Here is a weapon. Pick it up."
                    if (sequence === 0) {
                        speechBalloon = new RegulatedSpeech({
                            position: Vector2(160, sensei.position.y - 70),
                            touchControl: true,
                            text: Localization.getText('tutorial')['items'],
                            bottomPosition: -100,
                            bottomFlip: true,
                            layer: Globals.Layers.GUI + 4,
                            disappear: true,
                            waitTime: 200
                        });
                        Bento.objects.attach(speechBalloon);

                        gestureText.setText(Localization.getText('gestures')['items']);
                        Bento.objects.attach(gestureText);
                        new Tween({
                            from: 0,
                            to: 1,
                            in: 30,
                            delay: 0,
                            ease: 'linear',
                            onStart: function () {},
                            onUpdate: function (v, t) {
                                gestureText.alpha = v;
                            }
                        });

                        EventSystem.fire('disableControls');
                        sequence = 1;
                    }
                    centerBalloon();
                } else if (phase === 1) {
                    // make punchingbag vulernable once weapon has been picked up
                    if (sequence === 0) {
                        player.getComponent('weapon', function () {
                            new Tween({
                                from: 1,
                                to: 0,
                                in: 30,
                                delay: 0,
                                ease: 'linear',
                                onUpdate: function (v, t) {
                                    gestureText.alpha = v;
                                },
                                onComplete: function () {
                                    Bento.objects.remove(gestureText);
                                    punchingBag.attach(new DamageCounter({
                                        color1: '#26bfff',
                                        color2: '#ffffff'
                                    }));
                                    punchingBag.getComponent('punchable', function (punchable) {
                                        punchable.setMaxHealth(8);
                                        punchable.resetHealth();
                                    });

                                    player.getComponent('touchController', function (touchController) {
                                        touchController.canCharge(false);
                                    });

                                    player.getComponent('traditionalController', function (traditionalController) {
                                        traditionalController.canCharge(false);
                                    });

                                    phase = 2;
                                    sequence = 0;
                                }
                            });
                            sequence = 1;
                        });
                    }
                } else if (phase === 2) {
                    if (sequence === 0) {
                        // add weapon text and dialogue
                        Bento.objects.attach(target);
                        target.position.x = punchingBag.position.x - 32;
                        target.position.y = punchingBag.position.y;
                        // speech = "Now destroy that punching bag!"
                        speechBalloon = new RegulatedSpeech({
                            position: Vector2(160, sensei.position.y - 70),
                            touchControl: true,
                            blockControlEnd: true,
                            text: Localization.getText('tutorial')['weapon'],
                            bottomPosition: -100,
                            bottomFlip: true,
                            layer: Globals.Layers.GUI + 2,
                            disappear: true,
                            waitTime: 200,
                            onComplete: function () {
                                speechBalloon = null;
                            }
                        });
                        Bento.objects.attach(speechBalloon);

                        gestureText.setText(Localization.getText('gestures')['weapon']);
                        Bento.objects.attach(gestureText);
                        new Tween({
                            from: 0,
                            to: 1,
                            in: 30,
                            delay: 0,
                            ease: 'linear',
                            onStart: function () {},
                            onUpdate: function (v, t) {
                                gestureText.alpha = v;
                            }
                        });
                        sequence = 1;

                        Bento.objects.attach(goRight);
                    } else if (sequence === 1) {

                        // wait till player reaches punchingbag
                        if (Math.abs(target.position.x - player.position.x) <= 20 &&
                            Math.abs(target.position.y - player.position.y) <= 10) {
                            if (speechBalloon) {
                                speechBalloon.forceEnd();
                            }
                            //turn player towars punchingbag
                            player.getComponent('multiScale', function (multiScale) {
                                multiScale.direction.x = 1;
                            });
                            player.processAction({
                                type: 'idle'
                            });

                            //disable controls to make sure player doesn't get stuck
                            EventSystem.fire('disableControls');
                            sequence = 2;

                        }
                    } else if (sequence === 2) {
                        // face punchingbag


                        new Tween({
                            from: 1,
                            to: 0,
                            in: 30,
                            delay: 0,
                            ease: 'linear',
                            onUpdate: function (v, t) {
                                affixPlayerToOther(target);
                                player.getComponent('multiScale', function (multiScale) {
                                    multiScale.direction.x = 1;
                                });
                                target.alpha = v;
                                goRight.alpha = v;
                            },
                            onComplete: function () {
                                //enable controls again
                                EventSystem.fire('enableControls');

                                player.getComponent('touchController', function (touchController) {
                                    touchController.canMove(false);
                                });

                                player.getComponent('traditionalController', function (traditionalController) {
                                    traditionalController.canMove(false);
                                });
                                phase = 3;
                                sequence = 0;
                                Bento.objects.remove(target);
                                Bento.objects.remove(goRight);
                            }
                        });
                        sequence = 3;
                    }
                    centerBalloon();

                } else if (phase === 3) {
                    // waiting for punchingbag to be destroyed

                    centerBalloon();
                    if (goRight.alpha === 1) {
                        if (viewport.clone().grow(-16, false, true)
                            .hasPosition(punchingBag.position)) {
                            new Tween({
                                from: 1,
                                to: 0,
                                in: 30,
                                delay: 0,
                                ease: 'linear',
                                onUpdate: function (v, t) {
                                    goRight.alpha = v;
                                },
                                onComplete: function () {
                                    Bento.objects.remove(goRight);
                                }
                            });
                        }
                    }
                    if (sequence === 2) {
                        // destroyed punchingbag
                        new Tween({
                            from: 1,
                            to: 0,
                            in: 30,
                            delay: 0,
                            ease: 'linear',
                            onUpdate: function (v, t) {
                                gestureText.alpha = v;
                                tap.alpha = v;
                            },
                            onComplete: function () {
                                Bento.objects.remove(gestureText);
                                Bento.objects.remove(tap);
                                phase = 4;
                                sequence = 0;
                            }
                        });
                        sequence = 3;
                    }
                } else if (phase === 4) {
                    //throwing - hold gesture
                    if (sequence === 0) {
                        // speech = "Throw it away."
                        speechBalloon = new RegulatedSpeech({
                            position: Vector2(160, sensei.position.y - 70),
                            touchControl: true,
                            blockControlEnd: true,
                            text: Localization.getText('tutorial')['throw'],
                            bottomPosition: -100,
                            bottomFlip: true,
                            layer: Globals.Layers.GUI + 2,
                            disappear: true,
                            waitTime: 200,
                            onComplete: function () {
                                speechBalloon = null;
                            }
                        });
                        Bento.objects.attach(speechBalloon);

                        gestureText.setText(Localization.getText('gestures')['holdThrow']);
                        Bento.objects.attach(gestureText);
                        Bento.objects.attach(hold);
                        new Tween({
                            from: 0,
                            to: 1,
                            in: 30,
                            delay: 0,
                            ease: 'linear',
                            onStart: function () {},
                            onUpdate: function (v, t) {
                                gestureText.alpha = v;
                                hold.alpha = v;
                            },
                            onComplete: function () {
                                player.getComponent('touchController', function (touchController) {
                                    touchController.canCharge(true);
                                });

                                player.getComponent('traditionalController', function (traditionalController) {
                                    traditionalController.canCharge(true);
                                });

                                phase = 5;
                                sequence = 0;
                            }
                        });
                        centerBalloon();
                        sequence = 1;
                    }
                } else if (phase === 5) {
                    // throwing wait for charge
                    player.getComponent('stateMachine', function (stateMachine) {
                        if (stateMachine.stateName === 'prepareThrow') {
                            new Tween({
                                from: 1,
                                to: 0,
                                in: 30,
                                delay: 0,
                                ease: 'linear',
                                onStart: function () {
                                    phase = 6;
                                    sequence = 0;
                                },
                                onUpdate: function (v, t) {
                                    gestureText.alpha = v;
                                },
                                onComplete: function () {
                                    Bento.objects.remove(gestureText);
                                    if (phase === 6) {
                                        gestureText.setText(Localization.getText('gestures')['releaseThrow']);
                                        Bento.objects.attach(gestureText);
                                        new Tween({
                                            from: 0,
                                            to: 1,
                                            in: 30,
                                            delay: 0,
                                            ease: 'linear',
                                            onStart: function () {},
                                            onUpdate: function (v, t) {
                                                gestureText.alpha = v;
                                            },
                                        });
                                    }
                                }
                            });
                        }
                    });
                } else if (phase === 6) {
                    // throwing - wait for throw
                    if (sequence === 0) {
                        player.getComponent('stateMachine', function (stateMachine) {
                            if (stateMachine.stateName === 'throw') {
                                new Tween({
                                    from: 1,
                                    to: 0,
                                    in: 30,
                                    delay: 0,
                                    ease: 'linear',
                                    onStart: function () {
                                        sequence = 1;
                                    },
                                    onUpdate: function (v, t) {
                                        gestureText.alpha = v;
                                        hold.alpha = v;
                                    },
                                    onComplete: function () {
                                        Bento.objects.remove(gestureText);
                                        Bento.objects.remove(hold);
                                        phase = 7;
                                        sequence = 0;

                                        player.getComponent('touchController', function (touchController) {
                                            touchController.canMove(true);
                                        });

                                        player.getComponent('traditionalController', function (traditionalController) {
                                            traditionalController.canMove(true);
                                        });

                                        EventSystem.fire('disableControls');
                                        player.processAction({
                                            type: 'idle'
                                        });
                                    }
                                });
                            }
                        });
                    }
                } else if (phase === 7) {
                    // end part
                    if (sequence === 0) {
                        player.getComponent('multiScale', function (multiScale) {
                            multiScale.direction.x = -1;
                        });
                        // speech = "Congratulations, you're now an official Elite Fighter... intern."
                        speechBalloon = new RegulatedSpeech({
                            position: Vector2(160, sensei.position.y - 70),
                            touchControl: true,
                            text: Localization.getText('tutorial')['dialogue10'],
                            bottomPosition: -100,
                            bottomFlip: true,
                            layer: Globals.Layers.GUI + 2,
                            onComplete: function () {
                                // speech = "dialogue11":"Go and save the city."
                                speechBalloon = new RegulatedSpeech({
                                    position: Vector2(160, sensei.position.y - 70),
                                    touchControl: true,
                                    text: Localization.getText('tutorial')['dialogue11'],
                                    bottomPosition: -100,
                                    bottomFlip: true,
                                    layer: Globals.Layers.GUI + 2,
                                    onComplete: function () {
                                        // speech =  "dialogue12":"Intern?"
                                        speechBalloon = new RegulatedSpeech({
                                            position: Vector2(160, sensei.position.y - 70),
                                            touchControl: true,
                                            text: Localization.getText('tutorial')['dialogue12'],
                                            bottomPosition: 0,
                                            layer: Globals.Layers.GUI + 2,
                                            disappear: true,
                                            waitTime: 120,
                                            onComplete: function () {
                                                speechBalloon = null;
                                                // START GAME
                                                // TODO: add title screen instead of world select
                                                new Fade({
                                                    pause: Globals.PauseLevels.FADE,
                                                    duration: 10,
                                                    fadeIn: false,
                                                    type: 'opacity',
                                                    onComplete: function () {
                                                        // next section
                                                        SaveManager.save('tutorial', 3);
                                                        // set recently unlocked to default because of cheat menu force reset
                                                        SaveManager.save('recentlyUnlocked', [1]);

                                                        Bento.screens.show('screens/worldselect');
                                                    }
                                                });
                                            }
                                        });
                                        Bento.objects.attach(speechBalloon);
                                    }
                                });
                                Bento.objects.attach(speechBalloon);
                            }
                        });
                        Bento.objects.attach(speechBalloon);
                        sequence = 1;
                    }
                    centerBalloon();

                    var difference = Math.max(viewport.x + 10, viewport.x + viewport.width / 2 - 80) - sensei.position.x;
                    sensei.getComponent('sprite', function (sprite) {
                        if (Math.abs(difference) > 1) {
                            sprite.setSpriteSheet('sensei/walk');
                            sensei.scale.x = Utils.sign(difference);
                        } else {
                            sprite.setSpriteSheet('sensei/idle');
                            sensei.scale.x = 1;
                        }
                    });
                    sensei.position.x += (Math.max(viewport.x + 10, viewport.x + viewport.width / 2 - 80) - sensei.position.x) * 0.05;
                }
            },
            attached: function (data) {
                entity = data.entity;
                InventoryManager.setEquippedConsumable(0, 'weapons/bat');
            }
        };
        return component;
    };
});