/**
 * A Google Analytics "View" tracker
 * Many of our views are entities that overlay the entire screen
 * Attaching this component will fire the event for the view on start 
 * and fires an event for the previous view on destroy
 * @moduleName ViewAnalytics
 */
bento.define('components/viewanalytics', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'modules/analytics',
    'managers/save'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween,
    Analytics,
    SaveManager
) {
    'use strict';
    /**
     * Describe your settings object parameters
     * @param {Object} settings
     */
    return function (settings) {
        var viewport = Bento.getViewport();
        var name = settings.name;
        var ignorePrevious = settings.ignorePrevious || false;
        var ignoreKey = settings.ignoreKey;
        var previousView;
        var component = {
            name: 'view',
            start: function (data) {
                if (!name) {
                    Utils.log('You must set a name for this view');
                    return;
                }
                // view is ignored if ignoreKey's save is falsy 
                // if (ignoreKey && !SaveManager.load(ignoreKey)) {
                //     return;
                // }
                previousView = Analytics.getCurrentView();
                Analytics.trackView(name);
            },
            destroy: function (data) {
                if (!name) {
                    return;
                }
                // if (ignoreKey && !SaveManager.load(ignoreKey)) {
                //     return;
                // }
                if (previousView && !ignorePrevious) {
                    Analytics.trackView(previousView);
                }
            }
        };
        return component;
    };
});