/**
 * a sprite with a shadow that flies away, used with items that break
 * @moduleName Broken
 */
bento.define('entities/broken', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'components/sort',
    'components/movable',
    'components/shadow',
    'modules/matchmanager'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Sort,
    Movable,
    Shadow,
    MatchManager
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            spriteSheet: String
            position: Vector2 // positions the entity
            direction: Number // the direction that the entity flies towards
        }*/
        var viewport = Bento.getViewport();
        var sprite = new Sprite({
            spriteSheet: settings.spriteSheet,
        });
        var sort = new Sort({});
        var rotationSpeed = Utils.toRadian(3);
        var movable = new Movable({
            gravity: 0.15,
            bounceZ: 0.5,
            frictionZ: 0.5,
            onLanded: function () {
                rotationSpeed /= 2;
            },
            boundingBox: new Rectangle(-8, -8, 16, 16)
        });
        var shadow = new Shadow({
            imageName: 'shadow16'
        });
        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {
                movable.velocityZ = -2.5;
                movable.velocity.x = 1 * settings.direction;
                movable.height = -10;
                entity.scale.x = settings.direction;

                new Tween({
                    from: 1,
                    to: 0,
                    in: 60,
                    delay: 50,
                    ease: 'linear',
                    onStart: function () {},
                    onUpdate: function (v, t) {
                        entity.alpha = v;
                        shadow.shadow.alpha = v;
                    },
                    onComplete: function () {
                        Bento.objects.remove(entity);
                    }
                });
            },
            destroy: function (data) {},
            update: function (data) {
                entity.rotation += rotationSpeed * data.speed;
                if (movable.height > -10) {
                    movable.velocityZ = -movable.velocityZ * 0.8;
                }
            },
            draw: function (data) {}
        };
        var entity = new Entity({
            z: 0,
            name: 'broken',
            position: settings.position,
            updateWhenPaused: 0,
            components: [
                sprite,
                behavior,
                movable,
                sort,
                shadow
            ]
        });
        return entity;
    };
});