bento.define('entities/camera', [
    'bento',
    'bento/math/vector2',
    'bento/entity',
    'utils',
    'globals',
    'managers/resolution'
], function (
    Bento,
    Vector2,
    Entity,
    Utils,
    Globals,
    ResolutionManager
) {
    'use strict';
    return function () {
        var target = new Vector2(0, 0);
        var ignoreEdge = false;
        var viewport = Bento.getViewport();
        var shakePos = new Vector2(0, 0);
        var shakeTimer = 0;
        var shakeDuration = 0;
        var shakeStrength = 0;
        var offset;
        var shake = function () {
            if (shakeTimer > 0) {
                shakeTimer -= 1;
                if (~~(shakeTimer / 1) % 2) {
                    shakePos.x = shakeStrength * (Math.random() - 0.5) * shakeTimer / shakeDuration;
                    shakePos.y = shakeStrength * (Math.random() - 0.5) * shakeTimer / shakeDuration;
                }
            } else {
                shakePos.x = 0;
                shakePos.y = 0;
            }
        };
        var edge = null;
        var camera = new Entity({
            z: Globals.Layers.CAMERA,
            name: 'camera',
            family: ['camera'],
        }).extend({
            speed: new Vector2(1, 1),
            moveToTarget: true,
            offset: new Vector2(0, 0),
            centerTo: function (pos) {
                var position = camera.position;
                var currentScreen = Bento.screens.getCurrentScreen();
                var screenDimension;
                if (currentScreen) {
                    screenDimension = currentScreen.dimension;
                } else {
                    return;
                }

                if (edge) {
                    screenDimension = edge;
                    edge = null;
                }

                target.x = pos.x - viewport.width / 2;
                target.y = pos.y - viewport.height / 2;

                // cant place target outside screen dimension?
                if (!ignoreEdge) {
                    offset = ResolutionManager.getOffset();
                    target.x = Math.min(Math.max(target.x, screenDimension.x + offset.x), screenDimension.getX2() - offset.x - viewport.width);
                    // target.x = Math.max(Math.min(target.x, screenDimension.getX2() - viewport.width), screenDimension.x);  
                    target.y = Math.max(Math.min(target.y, screenDimension.getY2() - offset.y - viewport.height), screenDimension.y + offset.y);
                } else {
                    ignoreEdge = false;
                }
            },
            setSpeed: function (v) {
                camera.speed = v;
            },
            forceTo: function (pos, ignoreEdges) {
                var position = camera.position;
                var screenDimension;
                var currentScreen = Bento.screens.getCurrentScreen();
                if (currentScreen) {
                    screenDimension = currentScreen.dimension;
                }
                this.centerTo(pos);
                position.x = target.x;
                position.y = target.y;
            },
            shake: function (duration, strength) {
                shakeTimer = duration;
                shakeDuration = duration;
                shakeStrength = strength;
            },
            ignoreEdge: function () {
                // ignores edge once, sets itself to false afterwards
                ignoreEdge = true;
            },
            setEdge: function (rectangle) {
                var diff;
                // sets new edge once, unsets itself afterwards
                edge = rectangle;

                // smaller than viewport?
                /*if (edge.width < viewport.width) {
                    diff = viewport.width - edge.width;
                    edge.x -= diff / 2;
                    edge.width += diff;
                }
                if (edge.height < viewport.height) {
                    diff = viewport.height - edge.height;
                    edge.y -= diff / 2;
                    edge.height += diff;
                }*/
            }
        }).attach({
            start: function () {
                var screen = Bento.screens.getCurrentScreen();
                var screenDimension = screen.dimension;

                // if screen dimension is exactly the same as the viewport there is no reason to add a camera
                if (
                    screenDimension.x === viewport.x &&
                    screenDimension.y === viewport.y &&
                    screenDimension.width === viewport.width &&
                    screenDimension.height === viewport.height
                ) {
                    Utils.log("ERROR: screen dimension needs to be larger than viewport to support camera movement");
                }
            },
            update: function (data) {
                var position = camera.position;
                var currentScreen = Bento.screens.getCurrentScreen();
                var screenDimension;
                if (currentScreen) {
                    screenDimension = currentScreen.dimension;
                } else {
                    return;
                }
                shake();
                // move towards target
                if (camera.moveToTarget) {
                    position.x += (target.x - position.x) * camera.speed.x;
                    position.y += (target.y - position.y) * camera.speed.y;
                }
                // set to scroll
                viewport.x = position.x + shakePos.x + camera.offset.x;
                viewport.y = position.y + shakePos.y + camera.offset.y;
            }
        });

        return camera;
    };
});