/**
 * An object that locks the camera to an area
 * @moduleName CameraLocker
 */
bento.define('entities/cameralocker', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'components/debug',
    'entities/hand',
    'globals'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Debug,
    Hand,
    Globals
) {
    'use strict';
    var CameraLocker = function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2, // positions the entity
            rectangle: Rectangle, // the rectangle to lock the camera to
        }*/
        var viewport = Bento.getViewport();
        var rectangle = settings.rectangle || settings.size;
        var camera;
        var lockComponent = {
            name: 'lock',
            start: function (data) {
                // check if there are other camera locks
                Bento.objects.getByFamily('cameraLocker', function (lockers) {
                    if (lockers.length > 1) {
                        Utils.log("WARNING: other camera locks are present");
                        // remove current
                        Bento.objects.remove(lockers[0]);
                    }
                    CameraLocker.currentLock = entity;
                });
                if (!rectangle) {
                    Utils.log("WARNING: no rectangle parameter");
                }
            },
            update: function (data) {
                // cache camera object
                if (!camera) {
                    camera = Bento.objects.get('camera');
                    if (!camera) {
                        return;
                    }
                }
                // lock camera to edge
                if (rectangle) {
                    // top edge for
                    camera.setEdge(rectangle);
                }
            },
            destroy: function () {
                // remove any solids that were spawned
                Utils.forEach(this.solids, function (solid) {
                    Bento.objects.remove(solid);
                });
                CameraLocker.currentLock = null;

                // whenever a camera lock is destroyed,
                // show a hand to indicate the user can move on
                // Bento.objects.attach(new Hand({}));
            },
            enclose: function () {
                var top = makeSolid(rectangle.x, rectangle.y - 16, rectangle.width, 16);
                var bottom = makeSolid(rectangle.x, rectangle.y + rectangle.height, rectangle.width, 16);
                var left = makeSolid(rectangle.x - 16, rectangle.y, 16, rectangle.height);
                var right = makeSolid(rectangle.x + rectangle.width, rectangle.y, 16, rectangle.height);

                Bento.objects.attach(top);
                Bento.objects.attach(bottom);
                Bento.objects.attach(left);
                Bento.objects.attach(right);
            },
            solids: []
        };
        var makeSolid = function (x, y, width, height) {
            var entity = new Entity({
                z: 0,
                name: 'solid',
                family: ['solidPlayer'],
                boundingBox: new Rectangle(x, y, width, height),
                components: [
                    // new Debug({})
                ]
            });
            lockComponent.solids.push(entity);
            return entity;
        };

        var entity = new Entity({
            z: 0,
            name: 'cameraLocker',
            family: ['cameraLocker'],
            boundingBox: rectangle,
            updateWhenPaused: Globals.PauseLevels.FREEZE,
            components: [
                lockComponent
            ]
        });

        entity.extend({
            // surrounds the camera lock with solidPlayer
            enclose: lockComponent.enclose
        });

        return entity;
    };
    CameraLocker.currentLock = null;
    return CameraLocker;
});