/**
 * (Playable) character
 */
bento.define('entities/character', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'components/sort',
    'components/controllers/touchcontroller',
    'components/movable',
    'components/multiscale',
    'components/shadow',
    'components/debug',
    'components/punchable',
    'components/statemachine',
    'components/drawinviewport'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Sort,
    TouchController,
    Movable,
    MultiScale,
    Shadow,
    Debug,
    Punchable,
    StateMachine,
    DrawInViewport
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();

        var movable = new Movable(settings.movableSettings || {});
        var sprite = new Sprite({
            spriteSheet: settings.spriteSheet // not required to pass this
        });
        var stateMachine = new StateMachine({});
        var multiScale = new MultiScale({
            preventZero: true,
            scales: {
                direction: new Vector2(settings.direction || 1, 1)
            }
        });
        var shadow = new Shadow({});
        var extraComponents = settings.components || [];
        var extraFamily = settings.family || [];
        var entity = new Entity({
            z: settings.z || 0,
            name: 'character',
            family: ['characters'].concat(extraFamily),
            position: settings.position || new Vector2(0, 0),
            boundingBox: new Rectangle(-12, -4, 24, 8),
            updateWhenPaused: 0,
            components: [
                sprite,
                new DrawInViewport({
                    edgeSize: 96
                }),
                movable,
                new Sort({}),
                multiScale,
                stateMachine,
                shadow, //WARNING: Shadow should be attached as late as possible
                // new Debug({})
            ].concat(extraComponents)
        }).extend({
            rootPath: settings.rootPath,
            moveInDirection: function (dir) {
                stateMachine.moveInDirection(dir);
            },
            processAction: function (action) {
                action = action || {
                    type: 'none'
                };
                stateMachine.processAction(action);
            },
            setBehavior: function (behaviorType) {
                // tells the behavior component what "mode" to go into
                stateMachine.data.behavior = behaviorType;
            },
            setSpriteSheet: function (spriteSheetName, callback) {
                sprite.setSpriteSheet(spriteSheetName, callback);
            },
            setDirection: function (value) {
                multiScale.direction.x = value;
            }
        });

        if (settings.debug) {
            entity.attach(new Debug({}));
        }

        return entity;
    };
});