/**
 * Cutscene, helps with creating cutscenes
 * @moduleName Cutscene
 */
bento.define('entities/cutscene', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'components/anchor',
    'bento/managers/screen',
    'globals',
    'gui/speechballoon',
    'managers/ai',
    'modules/matchmanager',
    'modules/localization',
    'components/eventlistener',
    'entities/camera',
    'components/camerabehavior',
    'entities/regulatedspeech',
    'gui/fade',
    'components/modal',
    'gui/shadowtext'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Anchor,
    Screen,
    Globals,
    SpeechBalloon,
    AiManager,
    MatchManager,
    Localization,
    EventListener,
    Camera,
    CameraBehavior,
    RegulatedSpeech,
    Fade,
    Modal,
    ShadowText
) {
    'use strict';
    var Cutscene = function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();

        var tiled = (settings.tiled || {});
        var objectProperties = tiled.objectProperties || {};
        var object = tiled.object || {};
        var tileProperties = tiled.tileProperties || {};
        var nameProperties = Utils.nameToJson(object.name);

        var cleanup = [];

        var scenes = [];
        var currentScene = {};
        var speechBalloon;
        var speechBalloons = [];

        var pauseButtonLayer;
        var ready = false;
        var started = false;
        var complete = false;
        var camReset;
        var roomStarted = false;
        var componentTutorial = false;
        var cameraBehavior = new CameraBehavior({});
        var camera = new Camera({}).attach(cameraBehavior);
        var getBalloonPosition = function (target, balloon) {
            var balloonPosition = new Vector2(0, 0);
            var balloonDimension = balloon ? balloon.dimension : new Rectangle(0, 0, 160, 64);
            var margin = new Vector2(4 + balloonDimension.width / 2, balloonDimension.height / 2);
            balloonPosition.x = Utils.clamp(viewport.x + margin.x, target.position.x + balloon.offset.x, viewport.x + viewport.width - margin.x);
            balloonPosition.y = Utils.clamp(viewport.y + margin.y, target.position.y + balloon.offset.y - 48 - balloonDimension.height / 2, viewport.y + viewport.height - margin.y);
            return balloonPosition;
        };
        var centerBalloon = function () {
            var bottomPos;
            var balloonWidth = speechBalloon.dimension.width;
            var balloonPosition = getBalloonPosition(speechBalloon.target, speechBalloon);
            if (speechBalloon && speechBalloon.target) {
                speechBalloon.position.x = balloonPosition.x;
                speechBalloon.position.y = balloonPosition.y;

                // set bottom of speechballoon
                bottomPos = speechBalloon.target.position.x - speechBalloon.position.x;
                var bottom = speechBalloon.getBottom();
                speechBalloon.setBottomPosition(bottomPos);
                if (bottomPos < balloonWidth / 4) { // this is 3/4 of the balloon
                    bottom.scale.x = -1;
                } else {
                    bottom.scale.x = 1;
                }

                if (hasSkipped && speechBalloon.isComplete) {
                    speechBalloon.timer = Infinity;
                }
            }
        };
        var hasSkipped = false;
        var skipText = new Text({
            position: new Vector2(0, -2),
            text: Localization.getText('skip'),
            font: 'font',
            fontSize: 14,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            maxWidth: 27,
            maxHeight: 14,
            // drawDebug: true,
        });
        var buttonSkip = new ClickButton({
            z: Globals.Layers.GUI + 3,
            name: 'skip',
            sfx: 'sfx_button',
            imageName: 'gui/buttons/skip',
            frameCountX: 2,
            frameCountY: 1,
            float: false,
            updateWhenPaused: 0,
            onButtonDown: function () {
                skipText.position.y = -1;
            },
            onButtonUp: function () {
                skipText.position.y = -2;
            },
            onClick: function () {
                buttonSkip.setActive(false);
                hasSkipped = true;
                scenes = [];
                entity.nextScene();
                // Bento.setGameSpeed(10);
            }
        }).attach(skipText).attach(new Anchor({
            screenPosition: new Vector2(0, 1),
            offset: new Vector2(20, -16)
        }));
        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {
                entity.visible = false;
                tapToContinue.visible = false;

                // put pause button on top

                Bento.objects.attach(camera);

                entity.getComponent('cutscene', function (cutsceneComponent) {
                    cutsceneComponent.setupActors();
                });
            },
            destroy: function (data) {},
            update: function (data) {
                if (currentScene && currentScene.update) {
                    currentScene.update();
                }
                if (speechBalloon) {
                    centerBalloon();
                }
            },
            draw: function (data) {}
        };

        var startCutscene = function () {
            // make the cutscene visible
            entity.visible = true;

            // move black bars into view;
            new Tween({
                from: 1,
                to: 0,
                in: 40,
                delay: 0,
                ease: 'easeOutQuad',
                onStart: function () {

                },
                onUpdate: function (v, t) {
                    topBar.position.y = -viewport.height * 0.2 * v;
                    bottomBar.position.y = (1 + 0.2 * v) * viewport.height + 1;
                },
                onComplete: function () {
                    ready = true;
                }
            });

            started = true;

            // put pausebutton above black bars
            Bento.objects.get('pauseButton', function (pauseButton) {
                pauseButtonLayer = pauseButton.z;
                pauseButton.z = Globals.Layers.GUI + 3;
            });

            entity.getComponent('cutscene', function (cutsceneComponent) {
                cutsceneComponent.startCutscene();
            });

            Bento.audio.stopAllMusic();
            if (Utils.isNativeAndroid()) {
                // QUICKFIX: for some reason android sometimes skips the music
                new Tween({
                    ignoreGameSpeed: true,
                    in: 3,
                    onComplete: function () {
                        Bento.audio.playMusic('bgm_gymtutorial');
                    }
                });
            } else {
                Bento.audio.playMusic('bgm_gymtutorial');
            }
        };

        var endCutscene = function () {
            // move black bars out of screen
            new Tween({
                from: 0,
                to: 1,
                in: 60,
                delay: 0,
                ease: 'easeInQuad',
                onUpdate: function (v, t) {
                    topBar.position.y = -viewport.height * 0.2 * v;
                    bottomBar.position.y = (1 + 0.2 * v) * viewport.height + 1;
                },
                onComplete: function () {
                    Bento.objects.remove(entity);
                }
            });
        };

        var align = function () {
            topBar.scale.x = (viewport.width * 1.1) / 2;
            topBar.scale.y = viewport.height * 0.2;
            bottomBar.scale = new Vector2((viewport.width * 1.1) / 2, viewport.height * 0.2);
        };

        var topBar = new Entity({
            name: 'topBar',
            position: new Vector2(0, 0),
            components: [
                new Sprite({
                    imageName: 'gui/cutscene/blackbar',
                    originRelative: new Vector2(0.5, 0)
                }),
                new Sprite({
                    // padding
                    imageName: 'gui/cutscene/blackbar',
                    originRelative: new Vector2(0.5, 1)
                })
            ]
        }).attach(new Anchor({
            screenPosition: new Vector2(0.5, 0)
        }));

        var bottomBar = new Entity({
            name: 'bottomBar',
            position: new Vector2(0, 0),
            components: [
                new Sprite({
                    imageName: 'gui/cutscene/blackbar',
                    originRelative: new Vector2(0.5, 1)
                }),
                new Sprite({
                    // padding
                    imageName: 'gui/cutscene/blackbar',
                    originRelative: new Vector2(0.5, 0)
                })
            ]
        }).attach(new Anchor({
            screenPosition: new Vector2(0.5, 1)
        }));

        var tapToContinue = new Entity({
            name: 'tapToContinue',
            position: new Vector2(0, 0),
            components: [
                new Sprite({
                    spriteSheet: 'gui/hand-gesture-tap'
                })
            ]
        }).attach(new Anchor({
            screenPosition: new Vector2(1, 1),
            offset: new Vector2(-35, -49)
        }));
        var anchor = new Anchor({
            onComplete: align
        });
        var eventListener = new EventListener({
            name: 'startCutScene',
            onEvent: function () {
                startCutscene();
            }
        });

        var entity = new Entity({
            z: Globals.Layers.GUI + 1,
            name: 'cutscene',
            position: settings.position,
            updateWhenPaused: 0,
            float: true,
            components: [
                topBar,
                bottomBar,
                tapToContinue,
                behavior,
                eventListener,
                buttonSkip,
                anchor
            ]
        }).extend({
            started: started,
            endCutscene: endCutscene,
            addScene: function (fn) {
                scenes.push(fn);
            },
            nextScene: function () {
                if (scenes.length) {
                    currentScene = scenes.shift();
                    // do the cutscene
                    currentScene.start();
                } else {
                    // end of cutscenes
                    new Fade({
                        onComplete: function () {
                            Bento.setGameSpeed(1);
                            EventSystem.fire('endCutScene');
                        }
                    });
                }
            },
            focusOn: function (obj, force) {
                cameraBehavior.target = obj;
                if (force) {
                    camera.forceTo(obj.position);
                    camera.update();
                }
            },
            talk: function (data) {
                // already talking, save for later
                if (speechBalloon) {
                    speechBalloons.push(data);
                    return;
                }

                var obj = data.at || data.who;
                var text = data.text;
                var waitTime = data.waitTime || 120;
                var onComplete = data.onComplete;
                if (!text) {
                    Utils.log("ERROR: text was not found. (Check text id or refresh assets?)");
                }
                speechBalloon = new RegulatedSpeech({
                    // position: getBalloonPosition(obj),
                    touchControl: true,
                    text: text,
                    bottomPosition: -100,
                    bottomFlip: true,
                    layer: Globals.Layers.GUI + 4,
                    disappear: true,
                    waitTime: waitTime,
                    fitToText: true,
                    onComplete: function () {
                        speechBalloon = null;
                        // next speech balloon
                        if (speechBalloons.length) {
                            entity.talk(speechBalloons.shift());
                        }
                        if (onComplete) {
                            onComplete();
                        }
                    }
                });
                speechBalloon.target = obj;
                speechBalloon.offset = data.offset || new Vector2(0, 0);
                // centerBalloon();
                Bento.objects.attach(speechBalloon);
            }
        });
        return entity;
    };

    // stores tiled objects
    Cutscene.tiled = {};

    return Cutscene;
});