/**
 * Big rat enemy
 * @moduleName BigRat
 */
bento.define('entities/enemies/bigrat', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'entities/character',
    'components/controllers/enemycontroller',
    'components/ai/walkingenemy',
    'modules/matchmanager',
    'components/online',
    'components/healthbar',
    'components/punchable',
    'components/coindropper',
    'entities/enemy',
    'components/stats',
    'entities/punch',
    'entities/effect',
    'modules/tapticengine',
    'managers/save',
    'globals'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Character,
    EnemyController,
    WalkingEnemyBehavior,
    MatchManager,
    Online,
    HealthBar,
    Punchable,
    CoinDropper,
    Enemy,
    Stats,
    Punch,
    Effect,
    TapticEngine,
    SaveManager,
    Globals
) {
    'use strict';
    var BigRat = function (settings) {
        // tiled settings
        var tiled = (settings.tiled || {});
        var objectProperties = tiled.objectProperties || {};
        var object = tiled.object || {};
        var tileProperties = tiled.tileProperties || {};
        var nameProperties = Utils.nameToJson(object.name);
        var level = nameProperties.lv || SaveManager.load('enemyLevel') + (nameProperties.addLv || 0) || 1;
        var stats = new Stats(Globals.getDefaultEnemyStats({
            "level": level,
            "attack": 2,
            "skill": 2,
            "health": 5,
            "coins": 1,
            "stamina": 0.334,
            "minHits": 5
        }));

        var onDeath = function () {
            EventSystem.fire('quest-rats', {
                count: 1
            });
        };

        var tutorial = SaveManager.load('tutorialWorldSelect');
        // main behavior
        var behavior = new WalkingEnemyBehavior({
            rootPath: settings.rootPath || tileProperties.rootPath || 'bigrat/',
            attack: stats.attack,
            difficulty: nameProperties.difficulty || settings.difficulty,
        });
        // overwrite states
        var ratComponent = {
            name: 'ratComponent',
            start: function (data) {
                var stateMachine = data.entity.getComponent('stateMachine');
                stateMachine.addState('punch', BigRat.States.punch);

                entity.getComponent('punchable', function (punchable) {
                    punchable.onDeath(onDeath);
                });
            }
        };

        var entity = new Enemy({
            tiled: settings.tiled,
            position: settings.position,
            stats: stats,
            level: level,
            punchRange: 36,
            boundingBox: new Rectangle(-16, -6, 32, 12),
            health: objectProperties.health,
            maxHealth: stats.health,
            healthbarSpacing: 64,
            components: [
                behavior,
                ratComponent
            ]
        });

        return entity;
    };
    BigRat.States = {
        punch: {
            ignoreStun: true,
            ignoreBlock: true,
            ignoreDirection: true,
            start: function (data, transitionData) {
                data.punchAnim = (data.punchAnim || 0) + 1;
                data.sprite.setSpriteSheet(data.rootPath + 'punch' + (data.punchAnim % 2 + 1), function () {
                    data.setState('idle');
                });

                data.punchState = 1;

                //turn towards player
                var target = Bento.objects.getByName(transitionData.target)[0];
                if (target) {
                    data.multiScale.direction.x = Utils.sign(target.position.x - data.entity.position.x);
                }
                data.punchable.isInvincible = false;
            },
            update: function (data) {
                if (data.punchState === 1 && data.sprite.currentFrame >= 2) {
                    var punchObject = new Punch({
                        target: ['player', 'citizen'],
                        attack: data.attack,
                        stamina: 0.5,
                        skill: data.stats.skill,
                        direction: data.multiScale.direction.x,
                        pushBack: new Vector2(1.5, 1.5),
                        parent: data.entity,
                        freeze: 5,
                        boundingBox: new Rectangle(0, -5, data.entity.punchRange || 32, 10),
                        position: data.entity.position.add(new Vector2((data.multiScale.direction.x < 0 ? -data.entity.punchRange : 0), 0)),
                        onHit: Punch.getDefaultCallback({
                            entity: data.entity,
                            effect: 'effects/hitfx3-small',
                            sfx: 'sfx_punch1',
                            taptic: 'medium'
                        })
                    });
                    Bento.objects.attach(punchObject);

                    data.punchState = 2;
                }
            }
        },
    };

    return BigRat;
});