/**
 * Gangster enemy
 * @moduleName Gangster
 */
bento.define('entities/enemies/gangster', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'entities/character',
    'components/controllers/enemycontroller',
    'components/ai/walkingenemy',
    'modules/matchmanager',
    'components/online',
    'components/healthbar',
    'components/punchable',
    'components/coindropper',
    'components/damagecounter',
    'entities/enemy',
    'components/stats',
    'states/standardenemy',
    'managers/save',
    'globals',
    'entities/projectile'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Character,
    EnemyController,
    WalkingEnemyBehavior,
    MatchManager,
    Online,
    HealthBar,
    Punchable,
    CoinDropper,
    DamageCounter,
    Enemy,
    Stats,
    StandardEnemyStates,
    SaveManager,
    Globals,
    Projectile

) {
    'use strict';
    var Gangster = function (settings) {
        var tiled = (settings.tiled || {});
        var objectProperties = tiled.objectProperties || {};
        var object = tiled.object || {};
        var tileProperties = tiled.tileProperties || {};
        var nameProperties = Utils.nameToJson(object.name);
        var level = nameProperties.lv || settings.lv || SaveManager.load('enemyLevel') + (nameProperties.addLv || 0) || 1;
        var lv = level - 1; // start with 0 for calculation
        var stats = new Stats(Globals.getDefaultEnemyStats({
            "level": level,
            "attack": 0,
            "skill": 1,
            "health": 3,
            "coins": 0,
            "stamina": 0.5,
            "minHits": 5
        }));

        var tutorial = SaveManager.load('tutorialWorldSelect');
        // main behavior
        var walkingEnemy = new WalkingEnemyBehavior({
            rootPath: settings.rootPath || tileProperties.rootPath || 'gangster/',
            attack: stats.attack,
            difficulty: nameProperties.difficulty || settings.difficulty,
        });
        // overwrite states
        var gangsterBehavior = {
            name: 'gangsterBehavior',
            start: function (data) {
                var stateMachine = data.entity.getComponent('stateMachine');
                stateMachine.addStates(states);
            }
        };

        var entity = new Enemy({
            tiled: settings.tiled,
            position: settings.position,
            stats: stats,
            punchRange: 90,
            boundingBox: new Rectangle(-16, -6, 32, 12),
            maxHealth: stats.health,
            health: objectProperties.health,
            healthbarSpacing: 52,
            components: [
                walkingEnemy,
                gangsterBehavior
            ]
        });

        var states = {
            // punch is shoot for the gangster for now
            punch: {
                ignoreStun: true,
                ignoreBlock: true,
                ignoreDirection: true,
                start: function (data, transitionData) {

                    Bento.audio.stopSound('sfx_slingshot-charge');
                    Bento.audio.playSound('sfx_slingshot-charge');
                    //turn towards player and check distance
                    var target = Bento.objects.getByName(transitionData.target)[0];
                    if (target) {
                        data.multiScale.direction.x = Utils.sign(target.position.x - entity.position.x);
                        if (Math.abs(target.position.x - entity.position.x) < 28) {
                            data.sync({
                                jump: true
                            });
                        } else {
                            data.sync({
                                jump: false
                            });
                        }
                    }
                    data.sprite.setSpriteSheet(data.rootPath + 'shoot', function () {
                        data.setState('idle');
                    });

                    if (data.syncData.jump) {
                        data.distance = 40;
                        data.jumpHeight = -1.2;
                    } else {
                        data.distance = 0;
                        data.jumpHeight = 0;
                    }

                    data.punchState = 1;
                    data.punchable.isInvincible = false;
                },
                update: function (data) {
                    if (data.punchState === 1 && data.sprite.currentFrame >= 4) {
                        //shoot

                        Bento.audio.stopSound('sfx_slingshot-shoot');
                        Bento.audio.playSound('sfx_slingshot-shoot');
                        Bento.objects.attach(new Projectile({
                            bullet: 'roundbullet',
                            offset: 28,
                            height: 24,
                            direction: new Vector2(data.multiScale.direction.x, 0),
                            position: data.entity.position,
                            attack: Math.ceil(data.attack),
                            stamina: data.stamina,
                            entity: entity
                        }));
                        data.punchState = 2;
                    }

                    //jump back if player is too close
                    var sign = -data.multiScale.direction.x;
                    data.movable.velocity.x = data.distance * 0.1 * sign;
                    data.distance *= 0.9;
                    data.movable.velocityZ += data.jumpHeight;
                    data.jumpHeight = 0;
                }
            }
        };

        return entity;
    };
    return Gangster;
});