/**
 * Hedgehog enemy similar to rat
 * @moduleName Hedgehog
 */
bento.define('entities/enemies/hedgehog', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'entities/character',
    'components/controllers/enemycontroller',
    'components/ai/walkingenemy',
    'modules/matchmanager',
    'components/online',
    'components/healthbar',
    'components/punchable',
    'components/coindropper',
    'components/damagecounter',
    'entities/enemy',
    'components/stats',
    'states/standardenemy',
    'managers/save',
    'globals',
    'entities/punch'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Character,
    EnemyController,
    WalkingEnemyBehavior,
    MatchManager,
    Online,
    HealthBar,
    Punchable,
    CoinDropper,
    DamageCounter,
    Enemy,
    Stats,
    StandardEnemyStates,
    SaveManager,
    Globals,
    Punch
) {
    'use strict';
    var Hedgehog = function (settings) {
        var tiled = (settings.tiled || {});
        var objectProperties = tiled.objectProperties || {};
        var object = tiled.object || {};
        var tileProperties = tiled.tileProperties || {};
        var nameProperties = Utils.nameToJson(object.name);
        var rootPath = settings.rootPath || tileProperties.rootPath || 'hedgehog/';
        var level = nameProperties.lv || settings.lv || SaveManager.load('enemyLevel') + (nameProperties.addLv || 0) || 1;
        var lv = level - 1; // start with 0 for calculation
        var stats = new Stats(Globals.getDefaultEnemyStats({
            "level": level,
            "attack": 0,
            "skill": -2,
            "health": -7,
            "coins": 0,
            "stamina": 0.2,
            "minHits": 3
        }));

        // main behavior
        var walkingEnemy = new WalkingEnemyBehavior({
            rootPath: rootPath,
            attack: stats.attack,
            difficulty: nameProperties.difficulty || settings.difficulty,
        });

        var entity = new Enemy({
            tiled: settings.tiled,
            position: settings.position,
            stats: stats,
            punchRange: 24,
            boundingBox: new Rectangle(-16, -6, 32, 12),
            maxHealth: stats.health,
            health: objectProperties.health,
            healthbarSpacing: 52,
            components: [
                walkingEnemy,
            ]
        });

        return entity;
    };
    return Hedgehog;
});