/**
 * punkgirl enemy, similar to ratboss but does a slidekick instead of shooting
 * @moduleName PunkGirl
 */
bento.define('entities/enemies/punkgirl', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'entities/character',
    'components/controllers/enemycontroller',
    'components/ai/walkingenemy',
    'modules/matchmanager',
    'components/online',
    'components/healthbar',
    'components/punchable',
    'components/coindropper',
    'components/damagecounter',
    'entities/enemy',
    'components/stats',
    'states/standardenemy',
    'managers/save',
    'entities/punch',
    'globals'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Character,
    EnemyController,
    WalkingEnemyBehavior,
    MatchManager,
    Online,
    HealthBar,
    Punchable,
    CoinDropper,
    DamageCounter,
    Enemy,
    Stats,
    StandardEnemyStates,
    SaveManager,
    Punch,
    Globals
) {
    'use strict';
    return function (settings) {
        var tiled = (settings.tiled || {});
        var objectProperties = tiled.objectProperties || {};
        var object = tiled.object || {};
        var tileProperties = tiled.tileProperties || {};
        var nameProperties = Utils.nameToJson(object.name);
        var level = nameProperties.lv || SaveManager.load('enemyLevel') + (nameProperties.addLv || 0) || 1;
        var lv = level - 1; // start with 0 for calculation
        var stats = new Stats(Globals.getDefaultEnemyStats({
            "level": level,
            "attack": 3,
            "skill": 1,
            "health": 0,
            "coins": 0,
            "stamina": 0.5,
            "minHits": 5
        }));
        var stateMachine;
        // main behavior
        var walkingEnemy = new WalkingEnemyBehavior({
            rootPath: settings.rootPath || tileProperties.rootPath || 'punkgirl/',
            attack: stats.attack,
            difficulty: nameProperties.difficulty || settings.difficulty,
        });

        var behavior = {
            name: 'punkgirlBehavior',
            start: function () {
                stateMachine = entity.getComponent('stateMachine');
                stateMachine.addStates(states);
            }
        };

        var entity = new Enemy({
            tiled: settings.tiled,
            position: settings.position,
            stats: stats,
            punchRange: 60,
            boundingBox: new Rectangle(-16, -6, 32, 12),
            maxHealth: stats.health,
            health: objectProperties.health,
            healthbarSpacing: 52,
            sfxBase: 'female/',
            components: [
                walkingEnemy,
                behavior
            ]
        });

        var states = {
            punch: {
                ignoreStun: true,
                ignoreBlock: true,
                ignoreFaint: true,
                ignoreDirection: true,
                start: function (data, transitionData) {
                    data.punchState = 1;

                    //turn towards player and check distance
                    var target = Bento.objects.getByName(transitionData.target)[0];
                    if (target) {
                        data.multiScale.direction.x = Utils.sign(target.position.x - data.entity.position.x);
                        data.sync({
                            distance: Math.abs(target.position.x - data.entity.position.x)
                        });
                    } else {
                        data.sync({
                            distance: 38
                        });
                    }

                    // depending on distance use slide attack or bat attack
                    var attackString;
                    if (data.syncData.distance > 38) {
                        attackString = 'attack2';
                        data.duration = 18;
                        data.attackFrame = 4;
                        data.slideTime = 1;
                        data.slide = 50;
                        // data.punchable.isInvincible = true;
                    } else {
                        attackString = 'attack1';
                        data.duration = 10;
                        data.attackFrame = 3;
                        data.slide = 0;
                        data.punchable.isInvincible = false;
                    }

                    data.sprite.setSpriteSheet(data.rootPath + attackString, function () {
                        data.setState('idle');
                    });

                },
                update: function (data) {
                    if (data.punchState === 1 && data.sprite.currentFrame >= data.attackFrame) {
                        var punchObject = new Punch({
                            target: ['player', 'citizen'],
                            attack: !data.slide ? data.attack + 2 : data.attack || 1,
                            stamina: data.slide ? 1 : data.stamina || 0.334,
                            skill: data.stats.skill,
                            direction: data.multiScale.direction.x,
                            duration: data.duration,
                            pushBack: data.slide ? new Vector2(0.5, 2) : new Vector2(2.2, 1.4),
                            parent: data.entity,
                            boundingBox: new Rectangle(0, -5, 30, 10),
                            position: data.entity.position.add(new Vector2((data.multiScale.direction.x < 0 ? -30 : 0), 0)),
                            onHit: Punch.getDefaultCallback({
                                entity: data.entity,
                                effect: 'effects/hitfx3-medium',
                                sfx: 'sfx_punch1',
                                taptic: 'medium'
                            }),
                            debug: true
                        });
                        Bento.objects.attach(punchObject);
                        data.punchState = 2;
                    }

                    // move when sliding
                    if (data.slide && data.sprite.currentFrame >= data.attackFrame) {
                        var magnitude = data.slideTime * data.slideTime;
                        data.slideTime -= 1 / 28;
                        magnitude -= data.slideTime * data.slideTime;

                        data.movable.velocity.x = magnitude * data.slide * data.multiScale.direction.x;
                        data.movable.velocity.y = 0;
                    }
                },
                end: function (data) {
                    data.punchable.isInvincible = false;
                }
            },
        };

        return entity;
    };
});