/**
 * Rat Boss enemy
 * @moduleName RatBoss
 */
bento.define('entities/enemies/ratboss', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'entities/character',
    'components/controllers/enemycontroller',
    'modules/matchmanager',
    'components/online',
    'components/healthbar',
    'components/punchable',
    'components/stats',
    'components/ai/walkingenemy',
    'entities/enemy',
    'components/dead',
    'entities/projectile',
    'managers/resolution',
    'managers/ai',
    'managers/save',
    'entities/punch',
    'globals',
    'states/bossenemy',
    'states/standardenemy',
    'entities/enemies/rat',
    'managers/enemyactivator',
    'components/boss',
    'components/flash',
    'components/flames'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Character,
    EnemyController,
    MatchManager,
    Online,
    HealthBar,
    Punchable,
    Stats,
    WalkingEnemy,
    Enemy,
    Dead,
    Projectile,
    ResolutionManager,
    AiManager,
    SaveManager,
    Punch,
    Globals,
    BossEnemyStates,
    StandardEnemyStates,
    Rat,
    EnemyActivator,
    Boss,
    Flash,
    Flames
) {
    'use strict';
    return function (settings) {

        var tiled = (settings.tiled || {});
        var objectProperties = tiled.objectProperties || {};
        var object = tiled.object || {};
        var tileProperties = tiled.tileProperties || {};
        var nameProperties = Utils.nameToJson(object.name);
        var level = nameProperties.lv || SaveManager.load('enemyLevel') + (nameProperties.addLv || 0) || 1;
        var flee = settings.flee || nameProperties.flee || objectProperties.flee || false;
        var lv = level - 1; // start with 0 for calculation
        var stats = new Stats(Globals.getDefaultEnemyStats({
            "level": level,
            "attack": 1,
            "skill": 3,
            "health": 30,
            "stamina": 0.334,
            "minHits": 15
        }));
        var meleeDistance = 40;
        var shootDistance = 80;
        var powerup = false;
        var stateMachine;
        // var target;

        var attackState = 0;

        // ratbass has double attackpower
        stats.attack *= 2;

        // id of player used in sequence
        var sId = 0;
        var sTarget;

        var attackSide = 1;
        var isShooting = false;

        var randomTarget;
        var jumpTime;

        var minionId = 0;

        var viewport = Bento.getViewport();
        var levelDimension = Bento.screens.getCurrentScreen().dimension;

        //if offline this will always stay the local players width
        var targetScreen = new Rectangle(0, 0, viewport.width, viewport.height);

        var flash = new Flash({});

        // main behavior
        var walkingEnemy = new WalkingEnemy({
            rootPath: settings.rootPath || tileProperties.rootPath || 'ratboss/',
            attack: stats.attack,
            difficulty: nameProperties.difficulty || settings.difficulty
        });

        var onFaint = function (data) {
            if (stateMachine.data.enemyController.isLocallyControlled) {
                isShooting = false;
                // ++attackState;
            }
        };

        var standardDeath = walkingEnemy.onDeath;
        var onDeath = function (data) {
            if (flee) {
                // state 'die' triggers boss slowmo, stateName gets set to 'die' in start
                /*stateMachine.setState('flee', {}, true);

                // set velocities to make the fall seem more awesome in slowmo
                stateMachine.data.movable.velocityZ = -2;
                stateMachine.data.movable.velocity.x = Math.max(2, Math.abs(stateMachine.data.movable.velocity.x)) * data.punch.direction;

                // if punch has an attackType fire quest event, fleeing is similar to a knockout but it doesnt fire the standard death function so have to do this manually
                if (data.punch &&
                    !data.punch.emulated &&
                    Utils.isDefined(data.punch.type)
                ) {
                    EventSystem.fire('quest-' + data.punch.type, {
                        count: 1
                    });
                }*/
                
                // fleeing rat deprecated
                standardDeath(data);
                
                EventSystem.fire('quest-rats', {
                    count: 1
                });
            } else {
                if (powerup) {
                    standardDeath(data);
                    EventSystem.fire('quest-rats', {
                        count: 1
                    });
                } else {
                    entity.getComponent('punchable', function (punchable) {
                        punchable.isDead = false;
                    });
                    stateMachine.setState('eatCheese');
                }
            }

            // fire the 'Super Mafia Bros' Achievement
            Utils.postAchievement("CgkI9NqdkoAGEAIQBQ");
        };

        var behavior = {
            name: 'behavior',
            start: function (data) {
                stateMachine = entity.getComponent('stateMachine');
                stateMachine.addStates(states);
                // stateMachine.addStates(BossEnemyStates.standard);

                stateMachine.data.enemyController.ai = bossAI;

                // boss should be aggressive
                stateMachine.data.enemyController.aggression = 0.7;

                entity.getComponent('punchable', function (punchable) {
                    punchable.onFaint(onFaint);
                    punchable.offDeath(walkingEnemy.onDeath);
                    punchable.onDeath(onDeath);

                    // put boss component onDeath last, because it will kill the boss before he can eat cheese
                    entity.getComponent('boss', function (bossComponent) {
                        punchable.offDeath(bossComponent.onDeath);
                        punchable.onDeath(bossComponent.onDeath);
                    });

                    // add new onhit function
                    punchable.onHit(function () {
                        stateMachine.data.movable.velocityZ *= 0.7;
                    });
                });


                // because grabbed states are being added later we need to add our own grabstates later too
                Utils.timeout(2, function () {
                    stateMachine.addStates(grabState);
                });
            },
        };

        var shootSfx = function () {
            Bento.audio.stopSound('sfx_shoot');
            Bento.audio.playSound('sfx_shoot');
        };

        var entity = new Enemy({
            tiled: settings.tiled,
            position: settings.position,
            stats: stats,
            punchRange: 36,
            boundingBox: new Rectangle(-18, -6, 36, 12),
            maxHealth: stats.health,
            health: objectProperties.health,
            coins: objectProperties.coins || 8,
            healthbarSpacing: 72,
            components: [
                walkingEnemy,
                behavior,
            ]
        }).attach(flash);

        var bossAI = function (data, t, chaseBehavior) {
            var players;
            var distance;
            var move;
            if (attackState < 2) {
                if (stateMachine.stateName !== 'shoot') {
                    var dontShoot = 1;
                    if (stateMachine.stateName === 'walk') {
                        dontShoot = Utils.getRandom(120);
                    } else if (stateMachine.stateName === 'idle') {
                        dontShoot = Utils.getRandom(480);
                    }
                    if (!dontShoot) {
                        var viewBB = viewport.clone().grow(5);
                        if (viewBB.hasPosition(entity.position)) {
                            // check each player if in shooting path
                            players = Bento.objects.getByFamily('player');
                            Utils.forEach(players, function (p, i, l, breakLoop) {
                                distance = Math.abs(p.position.x - entity.position.x);
                                if (distance < shootDistance &&
                                    distance > meleeDistance
                                ) {
                                    if (Math.abs(p.position.y - entity.position.y) < 16) {
                                        isShooting = true;
                                        stateMachine.processAction({
                                            type: 'shoot',
                                            data: {
                                                target: p.name
                                            }
                                        });
                                        randomTarget = p.name;
                                    }
                                }
                            });
                        }
                    }

                }
                if (!isShooting) {
                    chaseBehavior(data);
                } else {
                    stateMachine.processAction({
                        type: 'shoot',
                        data: {
                            target: randomTarget
                        }
                    });
                }
            } else {
                if (attackState < 10) {
                    //if state is faint or getup wait till it's over and switch to jumping
                    if (stateMachine.stateName !== 'faint' &&
                        stateMachine.stateName !== 'getUp'
                    ) {
                        attackState = 10;
                    }
                } else {
                    //jump to players and shoot
                    players = Bento.objects.getByFamily('player');
                    if (sId < players.length) {
                        if (players[sId].position.x !== data.entity.position.x) {
                            stateMachine.data.multiScale.direction.x = Utils.sign(players[sId].position.x - data.entity.position.x);
                        }
                        if (stateMachine.stateName !== 'shoot' && stateMachine.stateName !== 'punch') {
                            if (!sTarget) {
                                // level constraints
                                var maxLeft = 8;
                                var maxRight = levelDimension.width - 8;

                                // determine attackside
                                attackSide = Utils.sign(entity.position.x - players[sId].position.x) || -attackSide;

                                // find a position next to a player that is inside their screen
                                sTarget = players[sId].position.clone();
                                sTarget.x += shootDistance * attackSide;

                                // switch sides when outside level, assume there is no blockade because this is a boss
                                if (sTarget.x < maxLeft || sTarget.x > maxRight) {
                                    sTarget.x -= shootDistance * attackSide * 2;
                                }

                                // determine the jump
                                jumpTime = sTarget.distance(entity.position) / 2.2;
                                var jumpStrength = Math.min(jumpTime / 40, 1);
                                jumpStrength *= jumpStrength * jumpStrength;
                                var jumpHeight = 2 + jumpStrength * 40;

                                var getJumpVelocity = function (gravity, heightGoal) {
                                    var velocity = 0;
                                    var currentHeight = 0;
                                    while (currentHeight < heightGoal) {
                                        velocity += gravity;
                                        currentHeight += velocity;
                                    }
                                    return velocity;
                                };

                                stateMachine.data.movable.velocityZ = -getJumpVelocity(0.15, jumpHeight);
                                var jumpPoint = sTarget.subtract(entity.position);
                                jumpPoint.normalize();

                                stateMachine.setState('jump', {
                                    jumpX: jumpPoint.x * 2.2,
                                    jumpY: jumpPoint.y * 2.2,
                                    target: players[sId].name
                                });

                            } else {
                                // after jump is done shoot (or punch when too close), even when the player is not aligned
                                if (stateMachine.data.movable.grounded) {
                                    distance = Math.abs(players[sId].position.x - entity.position.x);
                                    stateMachine.data.advance = false;
                                    if (distance > meleeDistance) {
                                        stateMachine.data.setState('shoot', {
                                            target: players[sId].name
                                        });
                                    } else {
                                        stateMachine.data.setState('punch', {
                                            target: players[sId].name
                                        });
                                    }

                                    sTarget = null;
                                    ++sId;
                                }
                            }
                        }
                    } else {
                        sTarget = null;
                        sId = 0;
                        attackState = 0;
                    }
                }
            }
        };

        var states = {
            idle: {
                canGrab: true,
                start: function (data, transitionData) {
                    data.sprite.setSpriteSheet(data.rootPath + 'idle');
                    data.idleTime = Utils.getRandomRange(walkingEnemy.hesitation[0] / 2, walkingEnemy.hesitation[1]);
                    /*if (data.enemyController.isLocallyControlled) {
                        if (!flee && !powerup && entity.timer > 120) {
                            var players = Bento.objects.getByFamily('player');
                            var enemies = Bento.objects.getByFamily('enemy');
                            data.sync({
                                spawnAmount: Math.max(0, players.length - (enemies.length - 1))
                            });
                        }
                    }
                    if (data.syncData.spawnAmount > 0) {
                        var rat;
                        var amount = data.syncData.spawnAmount;
                        var spawnX = 0;
                        Utils.forEach(Bento.objects.getByFamily, function (p, i, l, breakLoop) {
                            if (p.position.x > spawnX) {
                                spawnX = p.position.x;
                            }
                        });
                        spawnX += 180 + 30;
                        while (amount) {
                            rat = new Rat({});
                            rat.name = 'ratBoss-rat-' + minionId;
                            rat.attach(new Boss({}));
                            rat.position.y = entity.position.y;
                            rat.position.x = levelDimension.width + 30;
                            ++minionId;
                            --amount;
                            Bento.objects.attach(rat);
                        }
                        if (data.syncData.spawnAmount) {
                            EnemyActivator.activateEnemies();
                        }
                    }*/
                },
                update: WalkingEnemy.states.idle.update,
                move: WalkingEnemy.states.idle.move,
                action: function (data, action) {
                    if (data.idleTime > 0) {
                        return;
                    }
                    if (action.type == 'attack') {
                        var attackPower = stats.attack;
                        if (powerup) {
                            attackPower += 1;
                        }
                        stateMachine.data.attack = attackPower;
                        data.setState('punch', {
                            target: action.data.target.name
                        });
                    } else if (action.type === 'shoot') {
                        stateMachine.data.attack = stats.attack;
                        data.setState('shoot', {
                            target: action.data.target.name
                        });
                    } else if (action.type == 'eatCheese') {
                        data.setState('eatCheese', {});
                    }
                }
            },
            flee: {
                ignoreDirection: true,
                start: function (data, transitionData) {
                    data.sprite.setSpriteSheet(data.rootPath + 'flyup');
                    data.movable.ignoreCollisions = true;
                    data.fleeing = 0;
                    // set state name to 'die' to trigger boss slowmo and victory screen
                    stateMachine.stateName = 'die';
                },
                update: function (data) {
                    // fall down first before fleeing
                    if (data.movable.grounded) {
                        // lying on the ground
                        if (data.fleeing < 31) {
                            data.sprite.setSpriteSheet(data.rootPath + 'liedown');
                            data.fleeing++;
                        } else if (data.fleeing < 61) {
                            // getting up
                            data.sprite.setSpriteSheet(data.rootPath + 'getup');
                            data.fleeing++;
                        } else {
                            if (data.fleeing === 61) {
                                // can run away now
                                //entity.attach(new Dead({}));
                                data.sprite.setSpriteSheet(data.rootPath + 'walk');
                                data.fleeing++;
                            }
                            data.movable.velocity = new Vector2(3, 0);
                            data.multiScale.direction.x = 1;
                        }
                    } else if (data.movable.velocityZ >= 0) {
                        // flying down
                        data.sprite.setSpriteSheet(data.rootPath + 'flydown');
                        data.multiScale.direction.x = data.punchDirection;
                    }
                }
            },
            // swappped with intent to punch
            punch: {
                ignoreBlock: true,
                ignoreDirection: true,
                ignoreFaint: true,
                ignoreStun: true,
                start: function (data, transitionData) {
                    data.punchDelay = 30;
                    data.transitionDataCache = transitionData;

                    flash.flash = true;
                    flash.alpha = 0.6;
                },
                update: function (data) {
                    if (data.punchDelay) {
                        data.punchDelay -= 1;
                    } else {
                        data.setState('attack', {
                            target: data.transitionDataCache
                        }, true);
                    }
                },
                end: function () {
                    flash.flash = false;
                }
            },
            attack: {
                ignoreBlock: true,
                ignoreDirection: true,
                ignoreFaint: true,
                ignoreStun: true,
                start: function (data, transitionData) {
                    data.punchAnim = (data.punchAnim || 0) + 1;
                    data.sprite.setSpriteSheet(data.rootPath + 'punch' + (data.punchAnim % 2 + 1), function () {
                        if (data.punchAnim % 2 === 1) {
                            data.advance = false;
                            data.setState('attack', {
                                target: data.currentTarget
                            }, true);

                        } else {
                            data.setState('idle');
                        }
                    });

                    data.punchState = 1;
                    data.currentTarget = transitionData.target;

                    //turn towards player
                    var t = Bento.objects.get(transitionData.target);

                    if (t) {
                        data.multiScale.direction.x = Utils.sign(t.position.x - data.entity.position.x);
                    }
                    data.punchable.isInvincible = false;
                },
                update: function (data) {

                    if (data.punchState === 1 && data.sprite.currentFrame >= 2) {
                        var punchObject = new Punch({
                            target: ['player', 'citizen'],
                            attack: data.attack * 0.5 || 1,
                            stamina: data.stamina || 0.334,
                            skill: data.stats.skill,
                            direction: data.multiScale.direction.x,
                            //pushBack: new Vector2(0.5, 1.5),
                            pushBack: data.punchAnim % 2 === 1 ? new Vector2(0.2, 3) : new Vector2(2, 1.5),
                            parent: data.entity,
                            boundingBox: new Rectangle(0, -5, data.entity.punchRange || 44, 10),
                            position: data.entity.position.add(new Vector2((data.multiScale.direction.x < 0 ? -44 : 0), 0)),
                            onHit: Punch.getDefaultCallback({
                                entity: data.entity,
                                effect: 'effects/hitfx3',
                                sfx: 'sfx_punch1',
                                taptic: 'medium'
                            }),
                            debug: true
                        });
                        Bento.objects.attach(punchObject);

                        data.punchState = 2;
                    }
                },
                end: function (data) {
                    if (data.advance) {
                        ++attackState;
                    } else {
                        data.advance = true;
                    }
                }
            },
            shoot: {
                ignoreDirection: true,
                canGrab: true,
                start: function (data, transitionData) {
                    data.punchable.isInvincible = true;
                    data.sprite.setSpriteSheet(data.rootPath + 'shoot', function () {
                        data.setState('idle');
                    });
                    data.attackState = 1;

                    var t = Bento.objects.get(transitionData.target);
                    if (t) {
                        data.multiScale.direction.x = Utils.sign(t.position.x - entity.position.x);
                    }
                    data.sync({
                        bullet1: Utils.getRandomRangeFloat(-0.1, 0.1),
                        bullet2: Utils.getRandomRangeFloat(-0.1, 0.1),
                        bullet3: Utils.getRandomRangeFloat(-0.1, 0.1)
                    });
                    flash.flash = true;
                    flash.alpha = 0.6;
                },
                update: function (data) {
                    if (data.attackState === 1 && data.sprite.currentFrame >= 3) {
                        flash.flash = false;
                        flash.alpha = 1;
                        Bento.objects.attach(new Projectile({
                            direction: new Vector2(data.multiScale.direction.x, data.syncData.bullet1),
                            position: data.entity.position,
                            attack: Math.ceil(data.attack / 2),
                            stamina: data.stamina,
                            entity: entity
                        }));
                        shootSfx();
                        Utils.timeout(5, function () {
                            Bento.objects.attach(new Projectile({
                                direction: new Vector2(data.multiScale.direction.x, data.syncData.bullet2),
                                position: data.entity.position,
                                attack: Math.ceil(data.attack / 2),
                                stamina: data.stamina,
                                entity: entity
                            }));
                            shootSfx();
                        });
                        Utils.timeout(10, function () {
                            Bento.objects.attach(new Projectile({
                                direction: new Vector2(data.multiScale.direction.x, data.syncData.bullet2),
                                position: data.entity.position,
                                attack: Math.ceil(data.attack / 2),
                                stamina: data.stamina,
                                entity: entity
                            }));
                            shootSfx();
                        });

                        Bento.audio.stopSound('sfx_boss-shoot');
                        Bento.audio.playSound('sfx_boss-shoot');

                        data.attackState = 2;
                    }
                },
                end: function (data) {
                    data.punchable.isInvincible = false;
                    isShooting = false;
                    if (data.advance) {
                        ++attackState;
                    } else {
                        data.advance = true;
                    }
                }
            },
            eatCheese: {
                ignoreSync: true,
                ignoreBlock: true,
                ignoreDirection: true,
                start: function (data) {
                    data.sprite.setSpriteSheet(data.rootPath + 'eatcheese', function () {
                        data.setState('idle');
                        data.punchable.setHealth(stats.health);
                        data.punchable.isDead = false;
                        powerup = true;
                    });
                    data.punchable.isInvincible = true;
                    attackState = 0;
                },
                end: function (data) {
                    attackState = 2;
                    data.punchable.isInvincible = false;
                    data.walkingEnemyBehavior.hurry = 0.8;

                    data.punchable.setHealth(stats.health);
                    data.punchable.isDead = false;
                    powerup = true;

                    flash.flash = true;
                    data.entity.attach(new Flames({
                        color: 'yellow'
                    }));

                    new Tween({
                        from: 0.6,
                        to: 0,
                        in: 60,
                        delay: 0,
                        ease: 'linear',
                        onStart: function () {},
                        onUpdate: function (v, t) {
                            flash.alpha = v;
                        },
                        onComplete: function () {
                            flash.flash = false;
                            flash.alpha = 1;
                        }
                    });
                }
            },
            getUp: {
                ignoreDirection: true,
                start: StandardEnemyStates.getUp.start,
                update: function (data) {
                    data.getUpTimer--;
                    if (data.getUpTimer <= 0) {
                        ++attackState;
                        if (attackState < 2) {
                            data.setState('jumpBack');
                        } else {
                            data.setState('idle');
                        }
                    }
                }
            }
        };

        var grabState = {
            grabbed: BossEnemyStates.grabbed,
            grabbedThrow: BossEnemyStates.grabbedThrow
        };

        return entity;
    };
});