/**
 * Ratguy enemy
 * @moduleName RatGuy
 */
bento.define('entities/enemies/ratguy', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'entities/character',
    'components/controllers/enemycontroller',
    'components/ai/walkingenemy',
    'modules/matchmanager',
    'components/online',
    'components/healthbar',
    'components/punchable',
    'components/coindropper',
    'components/damagecounter',
    'entities/enemy',
    'components/stats',
    'entities/punch',
    'entities/effect',
    'modules/tapticengine',
    'managers/save',
    'globals'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Character,
    EnemyController,
    WalkingEnemyBehavior,
    MatchManager,
    Online,
    HealthBar,
    Punchable,
    CoinDropper,
    DamageCounter,
    Enemy,
    Stats,
    Punch,
    Effect,
    TapticEngine,
    SaveManager,
    Globals
) {
    'use strict';
    return function (settings) {
        var tiled = (settings.tiled || {});
        var objectProperties = tiled.objectProperties || {};
        var object = tiled.object || {};
        var tileProperties = tiled.tileProperties || {};
        var nameProperties = Utils.nameToJson(object.name);
        var level = nameProperties.lv || SaveManager.load('enemyLevel') + (nameProperties.addLv || 0) || 1;
        var lv = level - 1; // start with 0 for calculation
        var stats = new Stats(Globals.getDefaultEnemyStats({
            "level": level,
            "attack": 0,
            "skill": 2,
            "health": 0,
            "coins": 1,
            "stamina": 0.334,
            "minHits": 5
        }));
        var stateMachine;
        var hesitationMin = 10;
        var hesitationMax = 40;
        var comboHesitation = 10;
        var normalRange = 22;
        var legRange = 32;
        // main behavior
        var walkingEnemy = new WalkingEnemyBehavior({
            rootPath: settings.rootPath || tileProperties.rootPath || 'ratguy/',
            attack: stats.attack,
            difficulty: nameProperties.difficulty || settings.difficulty,
        });

        var behavior = {
            name: 'ratGirlBehavior',
            start: function () {
                stateMachine = entity.getComponent('stateMachine');
                stateMachine.addStates(states);
            }
        };

        var entity = new Enemy({
            tiled: settings.tiled,
            position: settings.position,
            stats: stats,
            punchRange: normalRange,
            boundingBox: new Rectangle(-16, -6, 32, 12),
            maxHealth: stats.health,
            health: objectProperties.health,
            healthbarSpacing: 52,
            components: [
                walkingEnemy,
                behavior
            ]
        });

        var states = {
            punch: {
                ignoreBlock: true,
                ignoreDirection: true,
                start: function (data, transitionData) {
                    data.punchAnim = (data.punchAnim || 0);
                    data.sprite.setSpriteSheet(data.rootPath + 'punch' + (data.punchAnim % 3 + 1), function () {
                        data.setState('idle');
                    });
                    if (data.punchAnim % 3 > 0) {
                        data.entity.punchRange = legRange;
                    } else {
                        data.entity.punchRange = normalRange;
                    }
                    data.punchAnim += 1;
                    data.punchState = 1;

                    //turn towards player
                    var target = Bento.objects.getByName(transitionData.target)[0];
                    if (target) {
                        data.multiScale.direction.x = Utils.sign(target.position.x - data.entity.position.x);
                    }
                    data.punchable.isInvincible = false;
                },
                update: function (data) {
                    if (data.punchState === 1 && data.sprite.currentFrame >= 2) {
                        var punchObject = new Punch({
                            target: ['player', 'citizen'],
                            attack: data.attack / 3 || 1,
                            stamina: data.punchAnim % 3 !== 0 ? 0.3 : 0.8,
                            skill: data.stats.skill,
                            direction: data.multiScale.direction.x,
                            pushBack: data.punchAnim % 3 === 1 ? new Vector2(0.5, 1.5) : new Vector2(0, 2.2),
                            parent: data.entity,
                            boundingBox: new Rectangle(0, -5, data.entity.punchRange + 5 || 32, 10),
                            position: data.entity.position.add(new Vector2((data.multiScale.direction.x < 0 ? -data.entity.punchRange - 5 : 0), 0)),
                            onHit: Punch.getDefaultCallback({
                                entity: data.entity,
                                effect: 'effects/hitfx3-small',
                                sfx: 'sfx_punch1',
                                taptic: 'medium'
                            }),
                            debug: true
                        });
                        Bento.objects.attach(punchObject);

                        data.punchState = 2;
                    }
                },
                end: function (data) {
                    if (data.punchAnim % 3 === 1) {
                        data.walkingEnemyBehavior.hesitation[0] = comboHesitation;
                        data.walkingEnemyBehavior.hesitation[1] = comboHesitation;
                        // console.log(data.enemyController);
                        data.enemyController.attackDistance = legRange + 8;
                    } else if (data.punchAnim % 3 === 0) {
                        data.walkingEnemyBehavior.hesitation[0] = hesitationMin;
                        data.walkingEnemyBehavior.hesitation[1] = hesitationMax;
                        data.enemyController.attackDistance = normalRange + 8;
                    }
                }
            }
        };

        return entity;
    };
});