/**
 * Base 'class' for enemies
 *
 * @moduleName Enemy
 */
bento.define('entities/enemy', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'components/healthbar',
    'components/coindropper',
    'components/punchable',
    'components/online',
    'modules/matchmanager',
    'entities/character',
    'components/controllers/enemycontroller',
    'components/damagecounter',
    'components/stats',
    'components/shake',
    'components/grabable',
    'managers/save',
    'globals'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    HealthBar,
    CoinDropper,
    Punchable,
    Online,
    MatchManager,
    Character,
    EnemyController,
    DamageCounter,
    Stats,
    Shake,
    Grabable,
    SaveManager,
    Globals
) {
    'use strict';
    return function (settings) {
        var extraComponents = settings.components || [];

        var tiled = settings.tiled || {};
        var objectProperties = tiled.objectProperties || {};
        var object = tiled.object || {};
        var tileProperties = tiled.tileProperties || {};
        var nameProperties = Utils.nameToJson(object.name);
        
        var stats = settings.stats;

        // add any extra stats from tiled now
        stats.health *= nameProperties.setHp || nameProperties.setHealth || 1;
        stats.health += nameProperties.addHp || nameProperties.addHealth || 0;
        stats.health = Math.round(stats.health);
        stats.attack += nameProperties.addAttack || 0;
        stats.skill += nameProperties.addSkill || 0;
        stats.coins += nameProperties.addCoins || 0;
        stats.minHits += nameProperties.addMinHits || 0;

        var punchable = new Punchable({
            boundingBox: settings.boundingBox || new Rectangle(-16, -6, 32, 12),
            health: settings.health,
            maxHealth: stats.health,
            defense: stats.defense,
            minHits: stats.minHits,
            stamina: parseFloat(stats.stamina),
            syncDeath: true
        });

        var coinDropper = new CoinDropper({
            coins: stats.coins || settings.coins || 0,
        });

        // healthbar
        var healthBar = new HealthBar({
            level: stats.level,
            punchable: punchable,
            spacing: settings.healthbarSpacing || 0
        });

        var onlineSync = new Online({});

        var damageCounter = new DamageCounter({});

        var grabable = new Grabable({});
        var enemyController = new EnemyController({
            attackDistance: settings.punchRange + 8 // +8 to take account width of player
        });

        var entity = new Character({
            spriteSheet: settings.spriteSheet,
            position: settings.position,
            components: [
                stats,
                punchable,
                healthBar,
                onlineSync,
                coinDropper,
                damageCounter,
                new Shake({}),
                enemyController,
                grabable // should come after enemyController
            ].concat(extraComponents)
        });

        // extra properties
        entity.extend({
            punchRange: settings.punchRange,
            sfxBase: settings.sfxBase || 'male/'
        });

        // enemies start looking left
        entity.getComponent('multiScale', function (multiScale) {
            multiScale.direction.x = -1;
        });

        return entity;
    };
});