/**
 * Adaptation of the rain entity from Sky Chasers
 * @moduleName Rain
 */
bento.define('entities/environment/rain', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'globals'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Globals
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();

        var tiled = (settings.tiled || {});
        var object = tiled.object || {};
        var nameProperties = Utils.nameToJson(object.name);

        var edgeSize = 128; // extra edge
        var drops = [];
        var wind = nameProperties.wind || 0;
        var spawnAmount = nameProperties.amount || 30;
        // var sfx = wind ? 'sfx_w2_windrain_loop' : 'sfx_w2_rain_loop';

        var getRandomX = function () {
            return Math.floor(Utils.getRandomRange(viewport.x - edgeSize, viewport.x + viewport.width + edgeSize) / 4) * 4;
        };

        var spawnDrop = function () {
            var pos = new Vector2(
                getRandomX(),
                Math.floor(Utils.getRandomRange(viewport.y - edgeSize, viewport.y + viewport.height + edgeSize) / 4) * 4
            );
            var acc = 0.1 + Utils.getRandom(10) / 10;
            var started = false;
            var speed = new Vector2(0, 8);
            var sprite = new Sprite({
                imageName: 'environment/raindrops',
                frameCountX: 3,
                frameCountY: 1,
                animations: {
                    '0': {
                        speed: 0,
                        frames: [0]
                    },
                    '1': {
                        speed: 0,
                        frames: [1]
                    },
                    '2': {
                        speed: 0,
                        frames: [2]
                    }
                }
            });
            var drop = new Entity({
                z: Globals.Layers.RAIN, // behind tiles?
                name: 'drop',
                position: pos.substract(new Vector2(0, viewport.height)), // spawn above screen
                originRelative: new Vector2(0, 0),
                family: ['rain'],
                components: [sprite]
            });
            var position = drop.position;

            drop.speed = speed;
            sprite.setAnimation(Utils.getRandom(3).toString());

            if (wind) {
                speed.x = wind;
                drop.rotation = -Math.PI / 4 * wind / 8;
            }
            // behavior
            drop.attach({
                name: 'dropBehavior',
                update: function (data) {
                    position.y += speed.y * data.speed;
                    position.x += speed.x * data.speed;

                    // wrap
                    if (position.x < viewport.x - edgeSize) {
                        position.x += viewport.width + edgeSize * 2;
                    }
                    if (position.x > viewport.x + viewport.width + edgeSize) {
                        position.x -= viewport.width + edgeSize * 2;
                    }
                    if (position.y > viewport.y + viewport.height + edgeSize) {
                        position.y -= viewport.height + edgeSize * 2;
                        // rerandomize x
                        position.x = getRandomX();
                        if (drop.end) {
                            // destroy after falling through the screen
                            Bento.objects.remove(drop);
                        }
                    }
                }
            });
            drop.getComponent('translation', function (component) {
                component.subPixel = true;
            });
            Bento.objects.attach(drop);
            drops.push(drop);

            drop.end = false;
            return drop;
        };

        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {
                // make rain
                Utils.repeat(spawnAmount, spawnDrop);

                Bento.audio.setVolume(1, 'sfx_rain_loop');
                Bento.audio.stopSound('sfx_rain_loop');
                Bento.audio.playSound('sfx_rain_loop', true);
            },
            destroy: function (data) {
                Bento.audio.stopSound('sfx_rain_loop');
            },
            update: function (data) {},
            draw: function (data) {}
        };

        var entity = new Entity({
            z: 0,
            name: 'rain',
            position: settings.position || new Vector2(0, 0),
            family: [''],
            components: [
                behavior,
            ]
        });

        // check if rain already exists
        if (Bento.objects.get('rain')) {
            // return dummy entity
            return new Entity({});
        }

        return entity;
    };
});