/**
 * Explosion with multiple sprites
 */
bento.define('entities/explosion', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'globals'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    Utils,
    Tween,
    Globals
) {
    'use strict';
    return function (settings) {
        var radius = settings.radius;
        var explosions = settings.explosions || 3;
        var centerExplosion = Utils.isDefined(settings.center) ? settings.center : true;
        var createExplosion = function (position, delay) {
                var sprite = new Sprite({});
                var entity = new Entity({
                    z: settings.z || Globals.Layers.EFFECTS,
                    name: 'explosion',
                    position: position,
                    components: [
                        sprite
                    ]
                });
                Utils.timeout(delay, function () {
                    Bento.objects.attach(entity);
                    sprite.setSpriteSheet('effects/explosion', function () {
                        Bento.objects.remove(entity);
                    });
                });
                return entity;
            };
        var explode = function () {
                var i = 0,
                    n = explosions;

                if (centerExplosion) {
                    createExplosion(settings.position.add(new Vector2(0, 0)), 0);
                }

                for (i = 0; i < n; ++i) {
                    createExplosion(settings.position.add(new Vector2(
                        radius * Math.cos(2 * Math.PI / n * i),
                        radius * Math.sin(2 * Math.PI / n * i)
                    )), (i + 1) * 3);
                }
            };

        explode();
    };
});