/**
 * Freeze controller
 * @moduleName FreezeController
 */
bento.define('entities/freezecontroller', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'globals'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween,
    Globals
) {
    'use strict';
    /**
     * Describe your settings object parameters
     * @param {Object} settings
     */
    var freezeTimer = 0;
    var freezeDelay = -1;

    var update = function () {
        if (freezeDelay >= 0) {
            freezeDelay--;
            if (freezeDelay <= 0) {
                freezeDelay = -1;
                if (Bento.objects.isPaused() === Globals.PauseLevels.NONE) {
                    Bento.objects.pause(Globals.PauseLevels.FREEZE);
                }
            }
            return;
        }

        if (freezeTimer <= 0) {
            return;
        }
        if (Bento.isPaused > Globals.PauseLevels.FREEZE) {
            return;
        }
        freezeTimer--;
        if (freezeTimer <= 0) {
            if (Bento.objects.isPaused() === Globals.PauseLevels.FREEZE) {
                Bento.objects.pause(Globals.PauseLevels.NONE);
            }
        }
    };

    var freeze = function (duration, delay) {
        freezeTimer = Math.max(freezeTimer, duration);
        delay = delay || 0;
        if (freezeDelay === -1) {
            freezeDelay = delay;
        }
        freezeDelay = Math.min(delay, freezeDelay);

    };

    EventSystem.on('preUpdate', update);


    var freezeController = {
        freeze: freeze
    };

    return freezeController;
});