/**
 * Hand pointing right to go forward
 * @moduleName Hand
 */
bento.define('entities/hand', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'globals',
    'components/anchor'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Globals,
    Anchor
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var sprite = new Sprite({
            originRelative: new Vector2(0.5, 0.5),
            imageName: 'gui/hand-right'
        });
        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {
                var hands = Bento.objects.getByFamily('hands');
                if (hands.length > 1) {
                    // hand already exists, remove self
                    entity.removeSelf();
                    return;
                }
            },
            destroy: function (data) {},
            update: function (data) {
                if (Anchor.isPortrait()) {
                    entity.position.x = viewport.width - 32 - 8 * Math.abs(Math.cos(entity.timer / 20));
                } else {
                    entity.position.x = viewport.width - 64 - 8 * Math.abs(Math.cos(entity.timer / 20));
                }
            },
            draw: function (data) {}
        };
        var entity = new Entity({
            z: Globals.Layers.GUI,
            name: 'hand',
            position: new Vector2(viewport.width - 32, viewport.height / 3),
            float: true,
            family: ['hands'],
            components: [
                sprite,
                behavior
            ]
        });
        return entity;
    };
});