/**
 * Sensei that appears in the dojo, walk into him to talk
 * @moduleName DojoSensei
 */
bento.define('entities/npc/dojosensei', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'entities/character',
    'entities/cutscene',
    'modules/localization',
    'managers/inventory',
    'gui/inventorypopup',
    'globals',
    'gui/menus/scroll'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Character,
    Cutscene,
    Localization,
    InventoryManager,
    InventoryPopup,
    Globals,
    ScrollMenu
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var behavior = {
            name: 'behaviorComponent',
            isTalking: 0,
            hasScrolls: function () {
                var hasAnyScroll = false;
                var scrolls = InventoryManager.getAllItems(['scrolls']);

                hasAnyScroll = (scrolls.length > 0);

                // memoize function
                behavior.hasScrolls = function () {
                    return hasAnyScroll;
                };
                return hasAnyScroll;
            },
            start: function (data) {
                entity.setDirection(-1);
                Bento.objects.attach(cutscene);

            },
            destroy: function (data) {},
            update: function (data) {
                var isColliding = false;
                entity.collidesWith({
                    family: 'player',
                    onCollide: function () {
                        isColliding = true;
                        if (behavior.isTalking === 0) {
                            behavior.isTalking = 1;
                            if (!behavior.hasScrolls()) {
                                cutscene.talk({
                                    text: Localization.getText('dojo')['needScrolls'],
                                    who: entity,
                                    onComplete: function () {
                                        behavior.isTalking = 2;
                                    }
                                });
                            } else {
                                EventSystem.fire('disableControls');
                                cutscene.talk({
                                    text: Localization.getText('dojo')['explainScrolls'],
                                    who: entity,
                                    onComplete: function () {
                                        var inventoryPopup = new InventoryPopup({
                                            z: Globals.Layers.PAUSE,
                                            pauseLevel: Globals.PauseLevels.SCROLL,
                                            filter: 'scrolls',
                                            hideShopKeeper: true,
                                            title: 'scrollsTitle',
                                            rows: 2,
                                            columns: 3,
                                            maxItems: 4,
                                            skipItemInfo: true,
                                            onComplete: function (slot) {
                                                if (!slot) {
                                                    return;
                                                }
                                                inventoryPopup.removeSelf();
                                                Bento.objects.attach(new ScrollMenu({
                                                    z: Globals.Layers.PAUSE + 1,
                                                    scrollType: slot.key
                                                }));
                                            }
                                        });
                                        EventSystem.fire('enableControls');
                                        behavior.isTalking = 2;
                                        Bento.objects.attach(inventoryPopup);
                                    }
                                });
                            }
                        }
                    }
                });

                if (!isColliding && behavior.isTalking === 2) {
                    behavior.isTalking = 0;
                }
            },
            draw: function (data) {}
        };
        var cutscene = new Cutscene({}); // there's no cutscene, but the talk method is useful here
        var entity = new Character({
            spriteSheet: 'sensei/idle',
            components: [
                behavior
            ]
        });
        return entity;
    };
});