/**
 * Barrel obstacle
 * @moduleName Barrel
 */
bento.define('entities/obstacles/barrel', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'components/sort',
    'entities/punch',
    'globals',
    'modules/matchmanager',
    'entities/broken',
    'components/drawinviewport',
    'managers/buff',
    'components/punchsyncer'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Sort,
    Punch,
    Globals,
    MatchManager,
    Broken,
    DrawInViewport,
    BuffManager,
    PunchSyncer
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        // var currentScreen = Bento.screens.getCurrentScreen();
        var offScreen = settings.offScreen || 128;
        var isRemote = settings.isRemote || false;
        var sprite = new Sprite({
            spriteSheet: 'obstacles/barrel/barrel'
        });
        var shadowSprite = new Sprite({
            spriteSheet: 'obstacles/barrel/barrel-shadow'
        });

        var isListening = false;

        var punches = Bento.objects.getByFamily('punches');

        var attack = 0.1;

        var sort = new Sort({});
        var behavior = {
            name: 'behaviorComponent',
            speed: settings.speed || -1.5,
            punch: null,
            start: function (data) {
                behavior.punch = new Punch({
                    target: isRemote ? ['player'] : ['player', 'enemy'], // can hit both enemies and player
                    attack: 2,
                    multiplier: BuffManager.current.enviromentDamage,
                    stamina: 1,
                    direction: Utils.sign(behavior.speed),
                    pushBack: new Vector2(0.1, 3),
                    duration: Infinity,
                    // parent: entity,
                    boundingBox: new Rectangle(-18, -4, 36, 14),
                    position: entity.position.clone(),
                    onHit: Punch.getDefaultCallback({
                        entity: entity,
                        effect: 'effects/hitfx3-medium',
                        sfx: 'sfx_punch1',
                        taptic: 'medium'
                    }),
                    attacker: entity,
                    preHit: function (other) {
                        other.getComponent('punchable', function (punchable) {
                            behavior.punch.attack = punchable.getMaxHealth() * attack;
                        });
                    }
                });
                Bento.objects.attach(behavior.punch);


                Bento.objects.attach(warning);
                warning.visible = false;

                if (behavior.speed > 0) {
                    entity.scale.x = -1;
                }

                if (!isListening) {
                    behavior.listenToIncoming();
                }
            },
            destroy: function (data) {
                behavior.stopListening();
                Bento.objects.remove(behavior.punch);
                Bento.objects.remove(warning);
            },
            update: function (data) {
                entity.position.x += behavior.speed * data.speed;
                behavior.punch.position.x = entity.position.x;

                var bb = behavior.punch.boundingBox.offset(entity.position);
                Utils.forEach(punches, function (punch, i, l, breakLoop) {
                    punch.collidesWith({
                        rectangle: bb,
                        onCollide: function (p) {
                            if (punch !== behavior.punch) {
                                // break the barrel and let remotes know
                                var msg = {
                                    d: punch.direction
                                };
                                behavior.breakBarrel(msg);
                                MatchManager.sendData(msg, entity.name);
                                EventSystem.fire('punchHit', {
                                    punch: punch,
                                    victim: entity,
                                    attacker: punch.attacker
                                });
                                punch.onHit(entity);
                                breakLoop();
                            }
                        }
                    });
                });

                // // collision function
                // var doCollision = function (character) {
                //     if (!behavior.punch.hasAlreadyPunched(character)) {
                //         var otherBB = character.getComponent('movable').boundingBox.offset(character.position);
                //         if (otherBB.intersect(bb)) {
                //             character.getComponent('punchable', function (punchable) {
                //                 behavior.punch.attack = punchable.getMaxHealth() * attack;
                //                 punchable.hit(behavior.punch);
                //             });
                //         }
                //     }
                // };
                // // check collision with local player
                // Utils.forEach(Bento.objects.getByFamily('player'), function (player, i, l, breakLoop) {
                //     if (!player.isRemote) {
                //         doCollision(player);
                //         breakLoop();
                //     }
                // });
                // // check collision with local enemies
                // Utils.forEach(Bento.objects.getByFamily('enemy'), function (enemy, i, l, breakLoop) {
                //     enemy.getComponent('online', function (online) {
                //         if (!online.isRemote()) {
                //             doCollision(enemy);
                //         }
                //     });
                // });

                // show a warning when getting close to the screen
                if (entity.position.x > viewport.x + viewport.width && entity.position.x < viewport.x + viewport.width + offScreen) {
                    warning.visible = Math.floor(entity.timer / 10) % 2;
                    // position the warning so it's always on screen
                    warning.position.y = Utils.clamp(viewport.y + 20, entity.position.y, viewport.y + viewport.height - 20);
                    warning.position.x = viewport.x + viewport.width - 20;
                } else {
                    warning.visible = false;
                }

                // moving out of screen
                if (behavior.speed < 0 && entity.position.x < viewport.x - 64) {
                    entity.removeSelf();
                    EventSystem.fire('destroyEnemyController');
                }
            },
            breakBarrel: function (data) {
                Bento.objects.attach(new Broken({
                    spriteSheet: 'obstacles/barrel/broken',
                    position: entity.position.clone(),
                    direction: data.d
                }));
                entity.removeSelf();
                EventSystem.fire('destroyEnemyController');
            },
            // listen to incoming messages
            listenToIncoming: function () {
                MatchManager.onMessage(entity.name, behavior.onMessage);
                isListening = true;
            },
            stopListening: function () {
                MatchManager.offMessage(entity.name, behavior.onMessage);
                isListening = false;
            },
            onMessage: function (msg) {
                behavior.breakBarrel(msg);
                // always return true if the message is handled
                return true;
            },
        };
        var warning = new Entity({
            z: Globals.Layers.EFFECTS,
            name: 'warning',
            position: new Vector2(0, 0),
            components: [
                new Sprite({
                    imageName: 'city/car-warning',
                    originRelative: new Vector2(0.5, 0.5)
                })
            ]
        });
        var entity = new Entity({
            name: 'barrel-' + settings.id,
            family: ['obstacles', 'enemy', 'ignoreEnemy'],
            position: settings.position,
            boundingBox: new Rectangle(-18, -4, 36, 14),
            components: [
                shadowSprite,
                sprite,
                behavior,
                sort,
                new PunchSyncer({}),
                new DrawInViewport({})
            ]
        });
        return entity;
    };
});