/**
 * Barrel obstacle
 * @moduleName BikeRat
 */
bento.define('entities/obstacles/bikerat', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'components/sort',
    'entities/punch',
    'globals',
    'modules/matchmanager',
    'entities/broken',
    'entities/enemies/rat',
    'components/boss',
    'components/punchable',
    'components/drawinviewport',
    'managers/buff',
    'entities/triggers/blockade',
    'components/movable'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Sort,
    Punch,
    Globals,
    MatchManager,
    Broken,
    Rat,
    Boss,
    Punchable,
    DrawInViewport,
    BuffManager,
    Blockade,
    Movable
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        // var currentScreen = Bento.screens.getCurrentScreen();
        var offScreen = settings.offScreen || 64;
        var currentScreen = Bento.screens.getCurrentScreen();
        var sprite = new Sprite({
            spriteSheet: 'obstacles/bikerat/ride'
        });
        var shadowSprite = new Sprite({
            spriteSheet: 'obstacles/bikerat/ride-shadow'
        });

        var punches = Bento.objects.getByFamily('punches');
        var players = Bento.objects.getByFamily('player');

        var id = settings.id;
        var spawner = settings.spawner;
        var hasDismounted = false;

        var isRemote;
        var isListening;

        var interpolate = new Vector2(0, 0);

        var attack = 0.1;

        var goForPlayer = false;

        var comingFromTheRightSide = function () {
            if (entity.position.x > viewport.x + viewport.width &&
                bikeMovable.velocity.x < 0) {
                return true;
            }
            return false;
        };

        var comingFromTheLeftSide = function () {
            if (entity.position.x < viewport.x &&
                bikeMovable.velocity.x > 0) {
                return true;
            }
            return false;
        };

        var insideLevel = function () {
            var bb = entity.boundingBox.offset(entity.position);
            if (bb.getX2() > currentScreen.dimension.width || bb.x < 0) {
                return false;
            }
            return true;
        };

        var insideBlockade = function () {
            if (Blockade.active) {
                var bb = entity.boundingBox.offset(entity.position);
                if (bb.getX2() > Blockade.active.position.x) {
                    return false;
                }
            }
            return true;
        };

        var manageSound = function () {
            var engine = Bento.objects.getByFamily('engines');
            if (engine && engine.length === 0) {
                Bento.audio.stopSound('sfx_motorcycle-loop');
            }
        };

        var sort = new Sort({});
        var behavior = {
            name: 'behaviorComponent',
            maxSpeed: settings.speed || 2.5,
            speed: 0,
            acceleration: settings.acceleration || 0.2,
            direction: settings.direction || -1,
            punch: null,
            updateInterval: 10,
            timer: 0,
            start: function (data) {
                behavior.punch = new Punch({
                    target: ['player'],
                    attack: 2,
                    multiplier: BuffManager.current.enviromentDamage,
                    stamina: 1.5,
                    direction: Utils.sign(behavior.speed),
                    pushBack: new Vector2(0.1, 3),
                    duration: Infinity,
                    // parent: entity,
                    boundingBox: new Rectangle(-18, -8, 36, 12),
                    position: entity.position.clone(),
                    onHit: Punch.getDefaultCallback({
                        entity: entity,
                        effect: 'effects/hitfx3-medium',
                        sfx: 'sfx_punch3',
                        taptic: 'medium'
                    }),
                    preHit: function (other) {
                        other.getComponent('punchable', function (punchable) {
                            behavior.punch.attack = punchable.getMaxHealth() * attack;
                        });
                    }
                });
                behavior.punch.rememberPunched = false;
                Bento.objects.attach(behavior.punch);

                Bento.objects.attach(warning);
                warning.visible = false;

                if (behavior.speed > 0) {
                    entity.scale.x = -1;
                }

                entity.scale.x = behavior.direction;

                Bento.audio.stopSound('sfx_motorcycle-loop');
                Bento.audio.setVolume(0.25, 'sfx_motorcycle-loop');
                Bento.audio.playSound('sfx_motorcycle-loop', true);
            },
            destroy: function (data) {
                Bento.objects.remove(behavior.punch);
                Bento.objects.remove(warning);
                if (settings.boss) {
                    var index = Boss.currentBosses.indexOf(entity);
                    Boss.currentBosses.splice(index, 1);
                }
                // check if sound should still play
                manageSound();
                behavior.stopListening();
            },
            update: function (data) {
                // check if local player is master
                isRemote = !MatchManager.isMaster;
                var isInside = false;

                if (!isListening) {
                    behavior.listenToIncoming();
                }

                if (sprite.currentSpriteSheet === 'obstacles/bikerat/ride') {
                    if (Math.abs(bikeMovable.velocity.x) < behavior.maxSpeed) {
                        bikeMovable.velocity.x += behavior.acceleration * behavior.direction;
                        if (Math.abs(bikeMovable.velocity.x) < behavior.maxSpeed) {
                            bikeMovable.velocity.x = behavior.maxSpeed * behavior.direction;
                        }
                    }
                    behavior.punch.position.x = entity.position.x;
                    behavior.punch.position.y = entity.position.y;
                } else {
                    bikeMovable.velocity.x *= 0.95;
                    behavior.punch.position.x = entity.position.x;
                    behavior.punch.position.y = entity.position.y;
                }

                var interpolateCoordinate = function (key) {
                    if (interpolate[key] !== 0) {
                        if (Math.abs(interpolate[key]) > 0.04) {
                            entity.position[key] += interpolate[key] * 0.2;
                            interpolate[key] *= 0.8;
                        } else {
                            entity.position[key] += interpolate[key];
                            interpolate[key] = 0;
                        }
                    }
                };
                // interpolate
                interpolateCoordinate('x');
                interpolateCoordinate('y');

                // check if inside playable area
                if (insideLevel() && insideBlockade()) {
                    isInside = true;
                    // can get hit by players
                    var bb = entity.boundingBox.offset(entity.position);
                    Utils.forEach(punches, function (punch, i, l, breakLoop) {
                        punch.collidesWith({
                            rectangle: bb,
                            onCollide: function (p) {
                                var target = punch.target.indexOf('enemy');
                                if (target > -1) {
                                    punch.onHit(entity);
                                    // handle punch locally
                                    behavior.dismount({
                                        direction: punch.direction
                                    });

                                    // send it to remotes
                                    var msg = {
                                        d: punch.direction
                                    };
                                    MatchManager.sendData(msg, entity.name);
                                    Bento.objects.get('camera', function (camera) {
                                        camera.shake(20, 5);
                                    });

                                    breakLoop();
                                }
                            }
                        });
                    });
                }

                // show a warning when getting close to the screen
                if (comingFromTheLeftSide() || comingFromTheRightSide()) {
                    warning.visible = Math.floor(entity.timer / 10) % 2;
                    // position the warning so it's always on screen
                    warning.position.y = Utils.clamp(viewport.y + 20, entity.position.y - 10, viewport.y + viewport.height - 20);
                    if (entity.position.x - (viewport.x + viewport.width / 2) < 0) {
                        warning.position.x = viewport.x + 20;
                    } else {
                        warning.position.x = viewport.x + viewport.width - 20;
                    }
                } else {
                    warning.visible = false;
                }

                if (!isRemote) {

                    var leftSide;
                    var rightSide;
                    // check if there is a player at index 0, sometimes at the beginning of a level it is empty.
                    if (players[0]) {
                        // if past all screens turn around and let remotes know
                        leftSide = Math.max(players[0].position.x - 180 - offScreen, 0);
                        rightSide = Math.min(players[0].position.x + 180 + offScreen, currentScreen.dimension.width);
                        var i = players.length;

                        // set up target y coordinate to go to
                        var closestDistance = currentScreen.dimension.width;
                        var targetY = entity.position.y;

                        while (i > 0) {
                            --i;
                            if (players[i]) {
                                players[i].getComponent('stateMachine', function (stateMachine) {
                                    if (stateMachine.stateName !== 'dead') {
                                        leftSide = Math.max(Math.min(leftSide, players[i].position.x - 180 - offScreen), 0);
                                        rightSide = Math.min(Math.max(rightSide, players[i].position.x + 180 + offScreen), currentScreen.dimension.width);
                                        // find best target y coordinate to go to
                                        if (Utils.sign(players[i].position.x - entity.position.x) === behavior.direction) {
                                            var dist = Math.abs(players[i].position.x - entity.position.x) + Math.abs(players[i].position.y - entity.position.y);
                                            if (dist < closestDistance) {
                                                closestDistance = dist;
                                                targetY = players[i].position.y;
                                            }
                                        }
                                    }
                                });
                            }
                        }
                        var diffY = targetY - entity.position.y;
                        var bikeZone = 40;
                        Utils.forEach(Bento.objects.getByFamily('bikers'), function (biker) {
                            // try to avoid others as much as possible
                            if (biker !== entity) {
                                var bikeDistance = Math.abs(entity.position.y - biker.position.y) * 2 + Math.abs(entity.position.x - biker.position.x);
                                if (bikeDistance < bikeZone) {
                                    diffY += Utils.sign(entity.position.y - biker.position.y || -1) * (bikeZone - Math.abs(entity.position.x - biker.position.x));
                                }
                            }
                        });
                        if (sprite.currentSpriteSheet === 'obstacles/bikerat/ride' && isInside && goForPlayer) {
                            bikeMovable.velocity.y = Math.min(Math.abs(diffY), 0.6) * Utils.sign(diffY);
                        } else {
                            bikeMovable.velocity.y = 0;
                        }
                        behavior.punch.position.y = entity.position.y;
                    } else {
                        leftSide = 0;
                        rightSide = currentScreen.dimension.width;
                    }


                    // send interpolation data when interval has been reached
                    behavior.timer += data.speed;
                    if (behavior.timer > behavior.updateInterval) {
                        // let remotes know current position for interpolation
                        var msg = {
                            x: Math.round(entity.position.x),
                            y: Math.round(entity.position.y),
                            f: behavior.direction
                        };
                        MatchManager.sendData(msg, entity.name);
                        behavior.timer -= behavior.updateInterval;
                    }

                    var turnAround = function () {
                        behavior.turnAround();
                        MatchManager.sendData({}, entity.name);
                    };

                    if (sprite.currentSpriteSheet === 'obstacles/bikerat/ride') {
                        if ((entity.position.x <= leftSide && behavior.direction < 0) ||
                            (entity.position.x >= rightSide && behavior.direction > 0)) {
                            turnAround();
                        } else if (isInside) {
                            // check for collision with solid and tileColliders
                            var solids = Bento.objects.getByFamily('solid');
                            var tileColliders = Bento.objects.get('tileColliders');

                            var predictBb = bikeMovable.boundingBox.offset(entity.position.add(new Vector2(behavior.direction * 40, 0)));
                            predictBb.height -= 0.2;
                            Utils.forEach(solids, function (other, a, l, breakLoop) {
                                if (other.collidesWith({
                                        rectangle: predictBb
                                    })) {
                                    turnAround();
                                    breakLoop();
                                }
                            });
                            if (tileColliders && tileColliders.collidesWith(predictBb, new Vector2(0, 0))) {
                                turnAround();
                            }
                        }
                    }
                }

                // everyone listens
                if (!isListening) {
                    behavior.listenToIncoming();
                    isListening = true;
                }
            },
            interpolate: function (data) {
                interpolate.x = data.x - entity.position.x;
                interpolate.y = data.y - entity.position.y;
            },
            setFacing: function (data) {
                if (data) {
                    behavior.direction = data.f;
                }
                entity.scale.x = behavior.direction;
            },
            turnAround: function () {
                // behavior.slide = behavior.speed * behavior.direction;
                behavior.direction = -behavior.direction;
                entity.scale.x = behavior.direction;
                sprite.setSpriteSheet('obstacles/bikerat/turn', function () {
                    sprite.setSpriteSheet('obstacles/bikerat/ride');
                    shadowSprite.setSpriteSheet('obstacles/bikerat/ride-shadow');
                });
                shadowSprite.setSpriteSheet('obstacles/bikerat/turn-shadow');
                behavior.speed = 0;
                goForPlayer = true;
            },
            dismount: function (data) {
                if (hasDismounted) {
                    return;
                }
                hasDismounted = true;
                //throw bike
                Bento.objects.attach(new Broken({
                    spriteSheet: 'obstacles/bikerat/riderless',
                    position: entity.position.clone(),
                    direction: data.direction,
                }));

                // check if sound should still play
                manageSound();

                // add rat to the game
                var rat = new Rat({
                    position: entity.position.clone(),
                    rootPath: 'ratgreen/',
                    lv: settings.boss ? settings.lv : undefined
                });

                rat.name = 'character-' + id;

                rat.getComponent('enemyController', function (enemyController) {
                    enemyController.active = true;
                });

                rat.getComponent('movable', function (movable) {
                    movable.velocityZ = -3;
                    movable.velocity.x = 1 * data.direction;
                });

                rat.getComponent('punchable', function (punchable) {
                    punchable.isInvincible = true;
                });

                Utils.timeout(1, function () {
                    rat.getComponent('stateMachine', function (stateMachine) {
                        stateMachine.setState('faint', {});
                    });
                });


                if (settings.boss) {
                    rat.attach(new Boss({
                        skipIntro: settings.boss > 1
                    }));
                }

                Bento.objects.attach(rat);

                // remove self and spawner
                entity.removeSelf();
                spawner.removeSelf();
            },
            // listen to incoming messages
            listenToIncoming: function () {
                MatchManager.onMessage(entity.name, behavior.onMessage);
                isListening = true;
            },
            stopListening: function () {
                MatchManager.offMessage(entity.name, behavior.onMessage);
                isListening = false;
            },
            onMessage: function (msg) {
                if (Utils.isDefined(msg.x)) {
                    // go interpolate
                    behavior.interpolate(msg);
                    behavior.setFacing(msg);
                } else if (msg.d) {
                    behavior.dismount({
                        direction: msg.d
                    });
                } else {
                    behavior.turnAround();
                }

                // always return true if the message is handled
                return true;
            },

        };
        var bikeMovable = new Movable({
            boundingBox: new Rectangle(-24, -6, 48, 8)
        });
        var warning = new Entity({
            z: Globals.Layers.EFFECTS,
            name: 'warning',
            position: new Vector2(0, 0),
            components: [
                new Sprite({
                    imageName: 'city/car-warning',
                    originRelative: new Vector2(0.5, 0.5)
                })
            ]
        });
        var entity = new Entity({
            name: 'bikeRat-' + id,
            family: ['obstacles', 'engines', 'bikers'],
            position: settings.position,
            boundingBox: new Rectangle(-24, -9, 48, 14),
            components: [
                shadowSprite,
                sprite,
                behavior,
                bikeMovable,
                sort,
                new DrawInViewport({})
            ]
        });

        if (settings.boss) {
            shadowSprite.name = 'shadow';
            entity.attach(new Punchable({
                boundingBox: new Rectangle(0, 0, 1, 1),
                health: 10,
                maxHealth: 10
            }));
            entity.attach(new Boss({}));
        }
        return entity;
    };
});