/**
 * Spawns barrels at the right side of the screen
 * Syncs online
 * @moduleName BikeSpawner
 */
bento.define('entities/obstacles/bikespawner', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'entities/obstacles/barrel',
    'modules/matchmanager',
    'entities/triggers/blockade',
    'managers/resolution',
    'entities/obstacles/bikerat',
    'managers/save',
    'entities/bossintro'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Barrel,
    MatchManager,
    Blockade,
    ResolutionManager,
    BikeRat,
    SaveManager,
    BossIntro
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var id = settings.tiled.object.id;
        var name = settings.tiled.object.name;
        var properties = Utils.nameToJson(name);
        var start = false;

        var isListening = false;

        var done = false;

        var offScreen = 96;

        var level = properties.lv || SaveManager.load('enemyLevel') + (properties.addLv || 0);

        var behavior = {
            name: 'behaviorComponent',
            timer: 0,
            start: function (data) {
                EventSystem.on('startRoom', behavior.onStartRoom);
                if (properties.isBoss) {
                    var msg = {
                        x: Math.round(entity.position.x)
                    };
                    behavior.spawnBike(msg);
                }
            },
            destroy: function (data) {
                behavior.stopListening();
                EventSystem.off('startRoom', behavior.onStartRoom);
            },
            update: function (data) {

                if (done) {
                    return;
                } else {
                    if (!isListening) {
                        behavior.listenToIncoming();
                    }

                    // check to see if in screen
                    if (viewport.hasPosition(entity.position)) {
                        var msg = {
                            x: Math.round(viewport.x + viewport.width / 2 + 160 + offScreen)
                        };
                        behavior.spawnBike(msg);
                        MatchManager.sendData(msg, entity.name);
                    }
                }
            },
            spawnBike: function (data) {
                done = true;
                var bikeRat = new BikeRat({
                    id: id,
                    position: new Vector2(data.x, entity.position.y),
                    spawner: entity,
                    boss: properties.isBoss ? properties.isBoss : undefined,
                    lv: properties.isBoss ? level : undefined
                });
                Bento.objects.attach(bikeRat);
            },
            // listen to incoming messages
            listenToIncoming: function () {
                MatchManager.onMessage(entity.name, behavior.onMessage);
                isListening = true;
            },
            stopListening: function () {
                MatchManager.offMessage(entity.name, behavior.onMessage);
                isListening = false;
            },
            onMessage: function (msg) {
                behavior.spawnBike(msg);

                // always return true if the message is handled
                return true;
            },
            onStartRoom: function () {
                start = true;
            },
            removeSelf: function () {
                //remove this
                entity.removeSelf();
                EventSystem.fire('destroyEnemyController');
            }
        };

        var entity = new Entity({
            z: 0,
            name: 'bikeSpawner-' + id,
            family: ['obstacles', 'enemy', 'ignoreEnemy'],
            position: settings.position,
            components: [
                behavior
            ]
        });

        return entity;
    };
});


//.getEnemiesLeft()