/**
 * Spawns cars at the side of the screen
 * Syncs online
 * @moduleName CarSpawner
 */
bento.define('entities/obstacles/carspawner', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'entities/obstacles/car',
    'modules/matchmanager'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Car,
    MatchManager
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var id = settings.tiled.object.id;
        var carNumber = 0;
        var name = settings.tiled.object.name;
        var properties = Utils.nameToJson(name);
        var timeUntilNextCar = properties.time || 60 * 10;
        var startDelay = properties.startDelay || 0;
        var start = false;

        var isRemote;
        var isListening = false;
        var direction = -1;
        var offScreen = 256;
        var behavior = {
            name: 'behaviorComponent',
            timer: 0,
            start: function (data) {
                var currentScreen = Bento.screens.getCurrentScreen();
                // check side
                if (entity.position.x < currentScreen.dimension.width / 2) {
                    // turn around cars
                    direction = 1;
                    entity.position.x = Math.min(entity.position.x, -offScreen);
                } else {
                    entity.position.x = Math.max(entity.position.x, currentScreen.dimension.width + offScreen);
                }

                // set spawn timer
                behavior.timer = timeUntilNextCar - startDelay;

                EventSystem.on('startRoom', behavior.onStartRoom);
            },
            destroy: function (data) {
                behavior.stopListening();
                EventSystem.off('startRoom', behavior.onStartRoom);
            },
            update: function (data) {
                // we always check if local player is master
                isRemote = !MatchManager.isMaster;

                if (!start) {
                    // wait for the event "startRoom"
                    return;
                }

                if (!isRemote) {
                    // local: behavior will spawn cars
                    behavior.timer += data.speed;
                    if (behavior.timer > timeUntilNextCar) {
                        // spawn car and let remotes know
                        behavior.spawnCar();
                        behavior.timer = 0;

                        var msg = {};
                        MatchManager.sendData(msg, entity.name);
                    }
                }

                // isRemote changed value
                if (isRemote && !isListening) {
                    behavior.listenToIncoming();
                } else if (!isRemote && isListening) {
                    behavior.stopListening();
                }
            },
            spawnCar: function () {
                // spawn a car
                var car = new Car({
                    position: entity.position.clone(),
                    speed: 3 * direction,
                    offScreen: offScreen,
                    isRemote: isRemote,
                    id: id + '-' + carNumber
                });
                ++carNumber;
                Bento.objects.attach(car);
            },
            // listen to incoming messages
            listenToIncoming: function () {
                MatchManager.onMessage(entity.name, behavior.onMessage);
                isListening = true;
            },
            stopListening: function () {
                MatchManager.offMessage(entity.name, behavior.onMessage);
                isListening = false;
            },
            onMessage: function (msg) {
                behavior.spawnCar();

                // always return true if the message is handled
                return true;
            },
            onStartRoom: function () {
                start = true;
            }
        };
        var entity = new Entity({
            z: 0,
            name: 'carSpawner-' + id,
            family: ['avoider'],
            position: settings.position,
            components: [
                behavior
            ]
        });
        return entity;
    };
});