/**
 * Mine that gets triggered when player is near.
 * Syncs online
 * @moduleName Mine
 */
bento.define('entities/obstacles/mine', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'modules/matchmanager',
    'entities/punch',
    'components/sort',
    'entities/effect',
    'entities/explosion',
    'components/drawinviewport',
    'managers/buff',
    'components/radiusflasher'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    MatchManager,
    Punch,
    Sort,
    Effect,
    Explosion,
    DrawInViewport,
    BuffManager,
    RadiusFlasher
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var id = settings.tiled.object.id;
        var name = settings.tiled.object.name;
        var properties = Utils.nameToJson(name);
        // var timeUntilNextCar = properties.time || 60 * 10;
        var start = false;

        var isRemote;
        var isListening = false;

        var sprite = new Sprite({
            spriteSheet: 'obstacles/mine/mine'
        });

        var triggered = false;
        var out = false;

        var attack = 0.1;
        var localPlayer;
        var beep = 0;
        var behavior = {
            name: 'behaviorComponent',
            timer: 0,
            start: function (data) {
                localPlayer = Utils.getLocalPlayer();
                EventSystem.on('startRoom', behavior.onStartRoom);
            },
            destroy: function (data) {
                EventSystem.off('startRoom', behavior.onStartRoom);
                behavior.stopListening();
            },
            update: function (data) {
                if (!start) {
                    // wait for the event "startRoom"
                    return;
                }

                if (!triggered) {
                    if (!localPlayer || !localPlayer.isAdded) {
                        localPlayer = Utils.getLocalPlayer();
                        return;
                    }
                    //check for distance of local player
                    if (Math.abs(localPlayer.position.x - entity.position.x) <= 36 && Math.abs(localPlayer.position.y - entity.position.y) <= 18) {
                        // trigger mine and let others now
                        behavior.trigger();
                        var msg = {};
                        MatchManager.sendData(msg, entity.name);
                    }
                } else {
                    var currentFrame = sprite.currentAnimation.frames[Math.floor(sprite.currentFrame)];
                    if (beep !== currentFrame) {
                        if (currentFrame === 1) {
                            // play sound
                            Bento.audio.setVolume(0.4, 'sfx_mine');
                            Bento.audio.stopSound('sfx_mine');
                            Bento.audio.playSound('sfx_mine');
                        }
                        beep = currentFrame;
                    }
                }

                if (!isListening) {
                    behavior.listenToIncoming();
                }
            },
            trigger: function () {
                var radius = 40;
                var radiusScale = new Vector2(1, 0.8);
                entity.attach(new RadiusFlasher({
                    colors: [
                        [1, 0, 0, 1],
                        [1, 1, 0, 1]
                    ],
                    radius: radius,
                    width: 2,
                    scale: radiusScale,
                    timeStart: 40,
                    every: 5,
                    duration: 1
                }));
                sprite.setSpriteSheet('obstacles/mine/triggered', function () {
                    Bento.audio.stopSound('sfx_explode');
                    Bento.audio.playSound('sfx_explode');

                    var explosion = new Explosion({
                        position: entity.position.add(new Vector2(0, -16)),
                        radius: 28,
                        explosions: 3,
                        // center: false
                    });
                    Bento.objects.attach(explosion);

                    // all characters inside area get damage and knockback
                    var punchObject = new Punch({
                        target: ['player', 'enemy'],
                        attack: 5,
                        multiplier: BuffManager.current.enviromentDamage,
                        stamina: 1,
                        skill: Infinity,
                        direction: 1,
                        pushBack: new Vector2(0, 0),
                        parent: entity,
                    });

                    var doCollision = function (character) {
                        if (
                            Math.abs(character.position.x - entity.position.x) <= radius * radiusScale.x &&
                            Math.abs(character.position.y - entity.position.y) <= radius * radiusScale.y
                        ) {
                            // hit
                            character.getComponent('punchable', function (punchable) {
                                // check for invincibility and apply forces before punch
                                if (!punchable.isInvincible) {
                                    character.getComponent('movable', function (movable) {
                                        var pushBack = new Vector2(character.position.x - entity.position.x, character.position.y - entity.position.y + 5);
                                        pushBack.normalize();
                                        pushBack.scalarMultiplyWith(2.5);
                                        movable.velocityZ = -3;
                                        movable.velocity = pushBack;
                                    });
                                }
                                punchObject.attack = punchable.getMaxHealth() * attack;
                                punchable.hit(punchObject);
                            });
                        }
                    };

                    // check collision with local player
                    Utils.forEach(Bento.objects.getByFamily('player'), function (player, i, l, breakLoop) {
                        if (!player.isRemote) {
                            doCollision(player);
                            breakLoop();
                        }
                    });
                    // check collision with local enemies
                    Utils.forEach(Bento.objects.getByFamily('enemy'), function (enemy, i, l, breakLoop) {
                        enemy.getComponent('online', function (online) {
                            if (!online.isRemote()) {
                                doCollision(enemy);
                            }
                        });
                    });

                    Bento.objects.remove(entity);
                });
                triggered = true;
            },
            // listen to incoming messages
            listenToIncoming: function () {
                MatchManager.onMessage(entity.name, behavior.onMessage);
                isListening = true;
            },
            stopListening: function () {
                MatchManager.offMessage(entity.name, behavior.onMessage);
                isListening = false;
            },
            onMessage: function (msg) {
                if (!triggered) {
                    behavior.trigger();
                }
                // always return true if the message is handled
                return true;
            },
            onStartRoom: function () {
                start = true;
                EventSystem.off('startRoom', behavior.onStartRoom);
            }
        };
        var entity = new Entity({
            z: 0,
            name: 'mine-' + id,
            family: ['obstacles'],
            position: settings.position,
            components: [
                behavior,
                sprite,
                new DrawInViewport({}),
                // new Sort({})
            ]
        });
        return entity;
    };
});