/**
 * Spikes get triggered when player is standing on them, after a certain amount of time they come out of the ground.
 * Syncs online
 * @moduleName Spike
 */
bento.define('entities/obstacles/popspike', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'entities/obstacles/car',
    'modules/matchmanager',
    'entities/punch',
    'components/sort',
    'components/drawinviewport'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Car,
    MatchManager,
    Punch,
    Sort,
    DrawInViewport
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var id = settings.tiled.object.id;
        var name = settings.tiled.object.name;
        var properties = Utils.nameToJson(name);
        // var timeUntilNextCar = properties.time || 60 * 10;
        var start = false;

        var isRemote;
        var isListening = false;

        var sprite = new Sprite({
            spriteSheet: 'obstacles/popspike/hole'
        });

        var inActive = false;
        var out = false;
        var localPlayer;
        var behavior = {
            name: 'behaviorComponent',
            timer: 0,
            start: function (data) {
                localPlayer = Utils.getLocalPlayer();
                EventSystem.on('startRoom', behavior.onStartRoom);
            },
            destroy: function (data) {
                EventSystem.off('startRoom', behavior.onStartRoom);
                behavior.stopListening();
            },
            update: function (data) {
                if (!start) {
                    // wait for the event "startRoom"
                    return;
                }

                if (!inActive) {
                    if (!localPlayer || !localPlayer.isAdded) {
                        localPlayer = Utils.getLocalPlayer();
                        return;
                    }
                    //check for distance of local player
                    if (Math.abs(localPlayer.position.x - entity.position.x) <= 16 && Math.abs(localPlayer.position.y - entity.position.y) <= 6) {
                        // trigger spike and let others now
                        behavior.triggerSpike();
                        var msg = {};
                        MatchManager.sendData(msg, entity.name);
                    }
                } else {
                    if (!out && sprite.currentFrame >= 1) {
                        var punchObject = new Punch({
                            target: ['player', 'enemy'],
                            attack: 5,
                            stamina: 0.5,
                            skill: Infinity,
                            direction: 1,
                            pushBack: new Vector2(0, 1.5),
                            parent: entity,
                            boundingBox: new Rectangle(-16, -6, 32, 12),
                            position: entity.position.clone(),
                            onHit: Punch.getDefaultCallback({
                                entity: entity,
                                effect: 'effects/hitfx3',
                                sfx: 'sfx_punch1',
                                taptic: 'medium'
                            })
                        });
                        Bento.objects.attach(punchObject);
                        out = true;
                    }
                }
                // }

                if (!isListening) {
                    console.log('isListening');
                    behavior.listenToIncoming();
                }
            },
            triggerSpike: function () {
                sprite.setSpriteSheet('obstacles/popspike/spike', function () {
                    sprite.setSpriteSheet('obstacles/popspike/retract', function () {
                        sprite.setSpriteSheet('obstacles/popspike/hole');
                        inActive = false;
                        out = false;
                    });
                });
                inActive = true;
            },
            // listen to incoming messages
            listenToIncoming: function () {
                MatchManager.onMessage(entity.name, behavior.onMessage);
                isListening = true;
            },
            stopListening: function () {
                MatchManager.offMessage(entity.name, behavior.onMessage);
                isListening = false;
            },
            onMessage: function (msg) {
                if (!inActive) {
                    behavior.makeSpike();
                }
                // always return true if the message is handled
                return true;
            },
            onStartRoom: function () {
                start = true;
                EventSystem.off('startRoom', behavior.onStartRoom);
            }
        };
        var entity = new Entity({
            z: 0,
            name: 'spike-' + id,
            position: settings.position,
            components: [
                behavior,
                sprite,
                new Sort({}),
                new DrawInViewport({})
            ]
        });
        return entity;
    };
});