/**
 * saw that goes from one side to the other horizontally
 * Syncs online
 * @moduleName Saw
 */
bento.define('entities/obstacles/saw', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'modules/matchmanager',
    'components/sort',
    'entities/punch',
    'managers/buff'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    MatchManager,
    Sort,
    Punch,
    BuffManager
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var id = settings.tiled.object.id;
        var name = settings.tiled.object.name;
        var properties = Utils.nameToJson(name);
        // var timeUntilNextSpear = (properties.time || 60 * 2) - 46;
        // var timeOffset = properties.offset || 0;
        var waitTime = 30;

        var start = false;

        var isRemote;
        var isListening = false;
        var attacking = false;

        var railLength = properties.length || 40;
        var direction = properties.direction || 1;
        var goToPositionX = 0;

        var attack = 0.1;

        var sawblade = new Entity({
            name: 'sawblade',
            family: ['obstacles'],
            components: [
                new Sprite({
                    spriteSheet: 'obstacles/saw/sawblade'
                }),
                new Sort({})
            ]
        });
        var railLeft = new Entity({
            name: 'rail-left',
            position: new Vector2(0, 0),
            components: [
                new Sprite({
                    spriteSheet: 'obstacles/saw/rail-left'
                })
            ]
        });
        var railRight = new Entity({
            name: 'rail-right',
            position: new Vector2(0, 0),
            components: [
                new Sprite({
                    spriteSheet: 'obstacles/saw/rail-right'
                })
            ]
        });
        var rail = new Entity({
            name: 'rail-center',
            position: new Vector2(0, 0),
            components: [
                new Sprite({
                    spriteSheet: 'obstacles/saw/rail-center'
                })
            ]
        });

        var behavior = {
            name: 'behaviorComponent',
            timer: 0,
            start: function (data) {
                EventSystem.on('startRoom', behavior.onStartRoom);
                if (direction === -1) {
                    entity.position.x += -railLength + 40;
                    goToPositionX = railLength - 40;
                }
                entity.position.x += 24;
                rail.position.x = -20;
                railLeft.position.x = -20;
                railRight.position.x = -20 + railLength;
                rail.scale.x = railLength;

                sawblade.position.y = entity.position.y;
                sawblade.position.x = entity.position.x + goToPositionX;
                Bento.objects.attach(sawblade);

                // set punchobject
                behavior.punch = new Punch({
                    target: ['player'],
                    attack: 2,
                    multiplier: BuffManager.current.enviromentDamage,
                    stamina: 1,
                    skill: Infinity,
                    direction: 1,
                    duration: Infinity,
                    pushBack: new Vector2(0, 1.5),
                    boundingBox: new Rectangle(-20, -3, 40, 4),
                    position: sawblade.position,
                    onHit: Punch.getDefaultCallback({
                        entity: sawblade,
                        effect: 'effects/hitfx3-medium',
                        sfx: 'sfx_punch1',
                        taptic: 'medium'
                    }),
                    preHit: function (other) {
                        other.getComponent('punchable', function (punchable) {
                            behavior.punch.attack = punchable.getMaxHealth() * attack;
                        });
                    }
                });
                behavior.punch.rememberPunched = false;
                Bento.objects.attach(behavior.punch);
            },
            destroy: function (data) {
                behavior.stopListening();
                EventSystem.off('startRoom', behavior.onStartRoom);
            },
            update: function (data) {
                // we always check if local player is master
                isRemote = !MatchManager.isMaster;

                if (!start) {
                    // wait for the event "startRoom"
                    return;
                }

                var localPositionSaw = sawblade.position.x - entity.position.x;
                if (localPositionSaw !== goToPositionX) {
                    if (Math.abs(goToPositionX - localPositionSaw) > 3) {
                        sawblade.position.x += Utils.sign(goToPositionX - localPositionSaw) * 3;
                    } else {
                        sawblade.position.x = entity.position.x + goToPositionX;
                    }
                }

                if (!isRemote) {
                    if (sawblade.position.x - entity.position.x === goToPositionX) {
                        behavior.timer += data.speed;
                        if (behavior.timer > waitTime) {
                            behavior.changePosition();
                            MatchManager.sendData({}, entity.name);
                            behavior.timer = 0;
                        }
                    }
                }

                // isRemote changed value
                if (isRemote && !isListening) {
                    behavior.listenToIncoming();
                } else if (!isRemote && isListening) {
                    behavior.stopListening();
                }

                // var bb = behavior.punch.boundingBox.offset(entity.position);
                // // check collision with local player
                // Utils.forEach(Bento.objects.getByFamily('player'), function (player, i, l, breakLoop) {
                //     if (!player.isRemote) {
                //         // collision
                //         var otherBB = player.getComponent('movable').boundingBox.offset(player.position);
                //         if (otherBB.intersect(bb)) {
                //             player.getComponent('punchable', function (punchable) {
                //                 behavior.punch.attack = punchable.getMaxHealth() * attack;
                //                 punchable.hit(behavior.punch);
                //             });
                //         }

                //         breakLoop();
                //     }
                // });
            },
            changePosition: function () {
                if (goToPositionX === 0) {
                    goToPositionX = railLength - 40;
                } else {
                    goToPositionX = 0;
                }
            },
            // listen to incoming messages
            listenToIncoming: function () {
                MatchManager.onMessage(entity.name, behavior.onMessage);
                isListening = true;
            },
            stopListening: function () {
                MatchManager.offMessage(entity.name, behavior.onMessage);
                isListening = false;
            },
            onMessage: function (msg) {
                behavior.changePosition();

                // always return true if the message is handled
                return true;
            },
            onStartRoom: function () {
                start = true;
            }
        };
        var entity = new Entity({
            z: 1,
            name: 'spear-' + id,
            family: ['obstacles'],
            position: settings.position,
            components: [
                behavior,
                railLeft,
                rail,
                railRight,
            ]
        });
        return entity;
    };
});