/**
 * Spear comes from the ground and retracts automatically
 * Syncs online
 * @moduleName Spear
 */
bento.define('entities/obstacles/spear', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'modules/matchmanager',
    'components/sort',
    'entities/punch',
    'components/drawinviewport',
    'managers/buff'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    MatchManager,
    Sort,
    Punch,
    DrawInViewport,
    BuffManager
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var id = settings.tiled.object.id;
        var name = settings.tiled.object.name;
        var properties = Utils.nameToJson(name);
        var timeUntilNextSpear = (properties.time || 60 * 2) - 46;
        var timeOffset = properties.offset || 0;
        var start = false;

        var isRemote;
        var isListening = false;
        var attacking = false;
        var attack = 0.1;

        var sprite = new Sprite({
            spriteSheet: 'obstacles/spear/hole'
        });

        var punchObject;
        var punchAttached = false;

        var behavior = {
            name: 'behaviorComponent',
            timer: timeOffset,
            start: function (data) {
                EventSystem.on('startRoom', behavior.onStartRoom);
                entity.position.x += 40;

                // set punchobject
                behavior.punch = new Punch({
                    target: ['player'],
                    attack: 5,
                    multiplier: BuffManager.current.enviromentDamage,
                    stamina: 1,
                    skill: Infinity,
                    direction: 1,
                    pushBack: new Vector2(0, 1.5),
                    parent: entity,
                    boundingBox: new Rectangle(-8, -6, 16, 12),
                    position: entity.position.clone(),
                    onHit: Punch.getDefaultCallback({
                        entity: entity,
                        effect: 'effects/hitfx3-medium',
                        sfx: 'sfx_punch1',
                        taptic: 'medium'
                    }),
                    preHit: function (other) {
                        other.getComponent('punchable', function (punchable) {
                            behavior.punch.attack = punchable.getMaxHealth() * attack;
                        });
                    }
                });
                behavior.punch.rememberPunched = false;
            },
            destroy: function (data) {
                behavior.stopListening();
                EventSystem.off('startRoom', behavior.onStartRoom);
            },
            update: function (data) {
                // we always check if local player is master
                isRemote = !MatchManager.isMaster;

                if (!start) {
                    // wait for the event "startRoom"
                    return;
                }

                if (!isRemote) {
                    if (!attacking) {
                        // local: behavior will set the animation
                        behavior.timer += data.speed;
                        if (behavior.timer > timeUntilNextSpear) {
                            // set the spear animation and let others know
                            behavior.attackSpear();
                            behavior.timer = 0;

                            var msg = {};
                            MatchManager.sendData(msg, entity.name);
                        }
                    }
                }

                // isRemote changed value
                if (isRemote && !isListening) {
                    behavior.listenToIncoming();
                } else if (!isRemote && isListening) {
                    behavior.stopListening();
                }

                // attack local player if spear animation
                if (sprite.currentSpriteSheet === 'obstacles/spear/attack') {
                    // // check collision with local player
                    // Utils.forEach(Bento.objects.getByFamily('player'), function (player, i, l, breakLoop) {
                    //     if (!player.isRemote) {
                    //         // collision
                    //         if (Math.abs(player.position.x - entity.position.x) <= 16 && Math.abs(player.position.y - entity.position.y + 3) <= 10) {
                    //             player.getComponent('punchable', function (punchable) {
                    //                 punchObject.attack = punchable.getMaxHealth() * attack;
                    //                 punchable.hit(punchObject);
                    //             });
                    //         }

                    //         breakLoop();
                    //     }
                    // });
                    if(!punchAttached) {
                        Bento.objects.attach(behavior.punch);
                        punchAttached = true;
                    }
                } else {
                    if (punchAttached) {
                        Bento.objects.remove(behavior.punch);
                        punchAttached = false;
                    }
                }


            },
            attackSpear: function () {
                sprite.setSpriteSheet('obstacles/spear/attack', function () {
                    sprite.setSpriteSheet('obstacles/spear/hole');
                    attacking = false;
                });
                attacking = true;
            },
            // listen to incoming messages
            listenToIncoming: function () {
                MatchManager.onMessage(entity.name, behavior.onMessage);
                isListening = true;
            },
            stopListening: function () {
                MatchManager.offMessage(entity.name, behavior.onMessage);
                isListening = false;
            },
            onMessage: function (msg) {
                behavior.attackSpear();

                // always return true if the message is handled
                return true;
            },
            onStartRoom: function () {
                start = true;
            }
        };
        var entity = new Entity({
            z: 0,
            name: 'spear-' + id,
            family: ['obstacles'],
            position: settings.position,
            components: [
                behavior,
                sprite,
                new Sort({}),
                new DrawInViewport({})
            ]
        });
        return entity;
    };
});