/**
 * Spikes get triggered when player is standing on them, after a certain amount of time they come out of the ground.
 * Syncs online
 * @moduleName Spike
 */
bento.define('entities/obstacles/spike', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'modules/matchmanager',
    'entities/punch',
    'components/sort',
    'components/drawinviewport',
    'managers/buff'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    MatchManager,
    Punch,
    Sort,
    DrawInViewport,
    BuffManager
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var id = settings.tiled.object.id;
        var name = settings.tiled.object.name;
        var properties = Utils.nameToJson(name);
        var start = false;

        var attack = 0.05;

        var behavior = {
            name: 'behaviorComponent',
            timer: 0,
            start: function (data) {
                EventSystem.on('startRoom', behavior.onStartRoom);

                // set punchobject
                behavior.punch = new Punch({
                    target: ['player'],
                    attack: 2,
                    multiplier: BuffManager.current.enviromentDamage,
                    stamina: 1,
                    skill: Infinity,
                    direction: 1,
                    duration: Infinity,
                    pushBack: new Vector2(0, 1.5),
                    // parent: entity,
                    boundingBox: new Rectangle(-16, -12, 33, 12),
                    position: entity.position.clone(),
                    onHit: Punch.getDefaultCallback({
                        entity: entity,
                        effect: 'effects/hitfx3-small',
                        sfx: 'sfx_punch3',
                        taptic: 'medium'
                    }),
                    preHit: function (other) {
                        other.getComponent('punchable', function (punchable) {
                            behavior.punch.attack = punchable.getMaxHealth() * attack;
                        });
                    }
                });
                behavior.punch.rememberPunched = false;
                Bento.objects.attach(behavior.punch);
            },
            destroy: function (data) {
                EventSystem.off('startRoom', behavior.onStartRoom);
                Bento.objects.remove(behavior.punch);
            },
            update: function (data) {
                if (!start) {
                    // wait for the event "startRoom"
                    return;
                }

                // var bb = behavior.punch.boundingBox.offset(entity.position);
                // // check collision with local player
                // Utils.forEach(Bento.objects.getByFamily('player'), function (player, i, l, breakLoop) {
                //     if (!player.isRemote) {
                //         // collision
                //         var otherBB = player.getComponent('movable').boundingBox.offset(player.position);
                //         if (otherBB.intersect(bb)) {
                //             player.getComponent('punchable', function (punchable) {
                //                 behavior.punch.attack = punchable.getMaxHealth() * attack;
                //                 punchable.hit(behavior.punch);
                //             });
                //         }

                //         breakLoop();
                //     }
                // });
            },
            onStartRoom: function () {
                start = true;
                EventSystem.off('startRoom', behavior.onStartRoom);
            }
        };
        var entity = new Entity({
            z: 0,
            name: 'spike-' + id,
            family: ['obstacles'],
            position: settings.position,
            components: [
                behavior,
                new Sprite({
                    spriteSheet: 'obstacles/spike/spike'
                }),
                new DrawInViewport({}),
            ]
        });
        return entity;
    };
});