/**
 * Tnt
 * @moduleName Tnt
 */
bento.define('entities/obstacles/tnt', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'components/sort',
    'entities/punch',
    'globals',
    'modules/matchmanager',
    'entities/broken',
    'components/drawinviewport',
    'managers/buff',
    'components/movable',
    'components/multiscale',
    'components/shadow',
    'entities/effect',
    'components/flash',
    'entities/explosion',
    'components/radiusflasher'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Sort,
    Punch,
    Globals,
    MatchManager,
    Broken,
    DrawInViewport,
    BuffManager,
    Movable,
    MultiScale,
    Shadow,
    Effect,
    Flash,
    Explosion,
    RadiusFlasher
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var direction = settings.direction || 1;
        var sprite = new Sprite({
            spriteSheet: 'obstacles/tnt/tnt'
        });
        var sort = new Sort({});
        var movable = new Movable({
            bounceZ: 0.5,
            frictionZ: 0.5,
            boundingBox: new Rectangle(-16, -8, 32, 16),
            onLanded: function () {
                sprite.setCurrentSpeed(sprite.getCurrentSpeed() / 2);
            }
        });
        var attack = 0.1;
        var multiScale = new MultiScale({
            preventZero: true,
            scales: {
                direction: new Vector2(direction, 1)
            }
        });
        var flash = new Flash({});
        var radius = 40;
        var radiusScale = new Vector2(1, 0.8);
        var behavior = {
            name: 'behaviorComponent',
            detonationTime: 90,
            showRadius: function () {
                entity.attach(new RadiusFlasher({
                    colors: [
                        [1, 0, 0, 1],
                        [1, 1, 0, 1]
                    ],
                    radius: radius,
                    width: 2,
                    scale: radiusScale,
                    timeStart: 0,
                    every: 5,
                    duration: 1
                }));
                // can only be called once
                behavior.showRadius = function () {};
            },
            start: function (data) {
                movable.height = -32;
                movable.velocityZ = settings.velocityZ || -1;
                movable.velocity = settings.velocity || new Vector2(0, 0);
                flash.flash = false;

            },
            destroy: function (data) {},
            update: function (data) {
                if (behavior.detonationTime) {
                    behavior.detonationTime -= data.speed;
                }
                if (behavior.detonationTime < 30) {
                    flash.flash = true;
                    behavior.showRadius();
                }
                if (behavior.detonationTime <= 0 && movable.height < 4) {
                    behavior.explode();
                }
            },
            explode: function () {
                Bento.audio.stopSound('sfx_explode');
                Bento.audio.playSound('sfx_explode');

                var explosion = new Explosion({
                    position: entity.position.add(new Vector2(0, -16)),
                    radius: 28,
                    explosions: 3,
                    // center: false
                });
                Bento.objects.attach(explosion);

                // all characters inside area get damage and knockback
                var punchObject = new Punch({
                    target: ['player', 'enemy'],
                    attack: 5,
                    multiplier: BuffManager.current.enviromentDamage,
                    stamina: 1,
                    skill: Infinity,
                    direction: 1,
                    pushBack: new Vector2(0, 0),
                    parent: entity,
                });

                var doCollision = function (character) {
                    if (
                        Math.abs(character.position.x - entity.position.x) <= radius * radiusScale.x &&
                        Math.abs(character.position.y - entity.position.y) <= radius * radiusScale.y
                    ) {
                        // hit
                        character.getComponent('punchable', function (punchable) {
                            // check for invincibility and apply forces before punch
                            if (!punchable.isInvincible) {
                                character.getComponent('movable', function (component) {
                                    var pushBack = new Vector2(character.position.x - entity.position.x, character.position.y - entity.position.y + 5);
                                    pushBack.normalize();
                                    component.velocityZ = -3;
                                    component.velocity = pushBack;
                                });
                            }
                            punchObject.attack = punchable.getMaxHealth() * attack;
                            punchable.hit(punchObject);
                        });
                    }
                };

                // check collision with local player
                Utils.forEach(Bento.objects.getByFamily('player'), function (player, i, l, breakLoop) {
                    if (!player.isRemote) {
                        doCollision(player);
                        breakLoop();
                    }
                });
                // check collision with local enemies
                Utils.forEach(Bento.objects.getByFamily('enemy'), function (enemy, i, l, breakLoop) {
                    enemy.getComponent('online', function (online) {
                        if (!online.isRemote()) {
                            doCollision(enemy);
                        }
                    });
                });
                entity.removeSelf();
            }
        };
        var entity = new Entity({
            name: 'tnt',
            position: settings.position,
            boundingBox: new Rectangle(-16, -8, 32, 16),
            components: [
                new Shadow({}),
                sprite,
                behavior,
                sort,
                movable,
                multiScale,
                flash,
                new DrawInViewport({})
            ]
        });
        return entity;
    };
});