 /**
  * Pedestrian. An NPC that walks by and can be interacted with.
  * Some pedestrians run away if they witness violence nearby, others are a bit less easily intimidated.
  *
  * @moduleName Pedestrian
  */
 bento.define('entities/pedestrian', [
     'bento',
     'bento/math/vector2',
     'bento/math/rectangle',
     'bento/components/sprite',
     'bento/components/clickable',
     'bento/entity',
     'bento/eventsystem',
     'bento/gui/clickbutton',
     'bento/gui/counter',
     'modules/text',
     'bento/utils',
     'bento/tween',
     'entities/character',
     'components/punchable',
     'components/ai/pedestrian'
 ], function (
     Bento,
     Vector2,
     Rectangle,
     Sprite,
     Clickable,
     Entity,
     EventSystem,
     ClickButton,
     Counter,
     Text,
     Utils,
     Tween,
     Character,
     Punchable,
     PedestrianBehavior
 ) {
     'use strict';
     return function (settings) {

         /**
          * get a property from either the settings, the tiled settings, or fall back to a default value
          */
         var getProperty = function (name, def) {
             var prop = settings[name];

             if (!Utils.isDefined(prop) && settings.tiled && settings.tiled.tileProperties) {
                 prop = settings.tiled.tileProperties[name];
             }

             if (!Utils.isDefined(prop)) {
                 prop = def;
             }

             return prop;
         };

         var rootPath = 'pedestrians/' + getProperty('type', 'test') + '/';
         var canWalk = getProperty('canWalk', true);
         var walkSpeed = getProperty('walkSpeed', 1);
         var intimidationChance = getProperty('intimidationChance', 0.5);

         var behavior = new PedestrianBehavior({
             rootPath: rootPath,
             canWalk: canWalk,
             walkSpeed: settings.walkSpeed,
             fleeSpeed: settings.fleeSpeed,
             maxFlee: settings.maxFlee,
             startFlee: settings.startFlee,
             intimidationChance: intimidationChance
         });

         //TODO punchables are now the same size for all pedestrians...
         var punchable = new Punchable({
             boundingBox: new Rectangle(-8, -4, 16, 8),
             maxHealth: 99999999
         });

         var character = new Character({
             position: settings.position,
             components: [
                 behavior,
                 punchable
             ]
         });

         character.family.push('pedestrian');
         character.family.push('destructible');

         return character;
     };
 });