/**
 * Photosnapper is an entity that keeps a lookout for moments to take a screenshot of.
 * @moduleName PhotoSnapper
 */
bento.define('entities/photosnapper', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'modules/snapshot',
    'globals',
    'components/eventlistener',
    'bento/packedimage'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    SnapShot,
    Globals,
    EventListener,
    PackedImage
) {
    'use strict';
    var PhotoSnapper = function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var hits = 0;
        var entity = new Entity({
            z: Globals.Layers.GUI,
            name: 'photoSnapper',
            components: [
                new EventListener({
                    name: 'comboChange',
                    onEvent: function (data) {
                        // randomize a delay
                        // var delay = Utils.getRandom(10);
                        var delay = 30;
                        // take a snapshot every 20 punches

                        if (hits % 20 === 19 && PhotoSnapper.snapshots.length < 2) {
                            Utils.timeout(delay, function () {
                                takeSnapshot(data);
                            });
                        }
                        hits += 1;
                    }
                })
            ]
        });
        var takeSnapshot = function (data) {
            var snapshot;
            var exclusions = [];
            var wasHidden = false;

            var player = Utils.getPlayerById(data.player);
            if (!player) {
                return;
            }
            if (!player.visible) { // player might be flashing
                player.visible = true;
                wasHidden = true;
            }

            // collect exclusions
            exclusions = Bento.objects.getByFamily('gui');
            exclusions = exclusions.concat(Bento.objects.getByFamily('hitEffects'));

            snapshot = new SnapShot({
                scale: 1,
            });
            // take snapshot
            snapshot.snap(exclusions);
            // window.open(snapshot.getBase64(),'_blank');
            PhotoSnapper.snapshots.push({
                snapshot: snapshot,
                position: player.position.subtract(new Vector2(viewport.x, viewport.y)) // save the player's position in the screenshot
            });

            if (wasHidden) {
                // re-hide player
                player.visible = false;
            }
        };

        return entity;
    };
    // trim snapshot to a square before sharing
    var trimPhoto = function () {
        // grab random snapshot
        var snapshotData = PhotoSnapper.snapshots[Utils.getRandom(PhotoSnapper.snapshots.length)];
        if (!snapshotData) {
            return;
        }
        var snapshot = snapshotData.snapshot;
        var position = snapshotData.position;
        var canvas = snapshot.getCanvas();
        var width = canvas.width;
        var height = canvas.height;

        // get the local square inside the canvas and paste it to a new canvas
        var localSquare = new Rectangle(position.x - 80, position.y - 80 - 32, 160, 160);
        // move the square inside the canvas
        if (localSquare.x < 0) {
            localSquare.x = 0;
        } else if (localSquare.x > canvas.width - 160) {
            localSquare.x = canvas.width - 160;
        }
        if (localSquare.y < 0) {
            localSquare.y = 0;
        } else if (localSquare.y > canvas.height - 160) {
            localSquare.y = canvas.height - 160;
        }

        // draw new canvas, 3x normal size
        var scale = 2;
        var newCanvas = document.createElement('canvas');
        var context;
        newCanvas.width = localSquare.width * scale;
        newCanvas.height = localSquare.height * scale;
        context = newCanvas.getContext('2d');
        if (context.imageSmoothingEnabled) {
            context.imageSmoothingEnabled = false;
        }
        if (context.webkitImageSmoothingEnabled) {
            context.webkitImageSmoothingEnabled = false;
        }
        if (context.mozImageSmoothingEnabled) {
            context.mozImageSmoothingEnabled = false;
        }
        if (context.msImageSmoothingEnabled) {
            context.msImageSmoothingEnabled = false;
        }

        context.drawImage(
            canvas,
            localSquare.x,
            localSquare.y,
            localSquare.width,
            localSquare.height,
            0,
            0,
            newCanvas.width,
            newCanvas.height
        );

        // paste title image on topleft corner
        var titleImage = Bento.assets.getImage('titlescreen/logo-' + Globals.getTitleLanguage());
        context.drawImage(
            titleImage.image,
            titleImage.x,
            titleImage.y,
            titleImage.width,
            titleImage.height,
            16,
            16,
            titleImage.width,
            titleImage.height
        );

        // introduce one transparent pixel for twitter (twitter will turn the image into a jpeg otherwise)
        context.globalAlpha = 0.1;
        context.globalCompositeOperation = 'destination-out';
        context.fillRect(0, 0, 1, 1);

        return newCanvas;
    };
    /**
     * Latest snapshot
     */
    PhotoSnapper.snapshots = [];
    PhotoSnapper.reset = function () {
        // clear memory if possible
        Utils.forEach(PhotoSnapper.snapshots, function (data, i, l, breakLoop) {
            if (data && data.snapshot && data.snapshot.dispose) {
                data.snapshot.dispose();
            }
        });
        PhotoSnapper.snapshots = [];
    };

    /**
     * Get the final packedimage of latest snapshot taken
     */
    PhotoSnapper.getPhoto = function () {
        if (!PhotoSnapper.snapshots.length) {
            return null;
        }
        var canvas = trimPhoto();
        return new PackedImage(canvas);
    };

    return PhotoSnapper;
});