/**
 * Playable character: Ana
 * @moduleName PlayerFitGirl
 */
bento.define('entities/players/fitgirl', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'modules/tapticengine',
    'entities/effect',
    'components/sort',
    'entities/player',
    'managers/stats',
    'components/stats',
    'entities/punch',
    'managers/buff',
    'states/standardplayer',
    'gui/super/superattackbars',
    'gui/super/superattackspeedlines',
    'entities/triggers/blockade',
    'globals'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    TapticEngine,
    Effect,
    Sort,
    Player,
    StatsManager,
    Stats,
    Punch,
    BuffManager,
    StandardPlayerStates,
    SuperAttackBars,
    SuperAttackSpeedlines,
    Blockade,
    Globals
) {
    'use strict';
    var FitGirl = function (settings) {
        // WARNING: it is assumed this component comes AFTER PlayerBehavior,
        // because the playerBehavior adds the standard player states
        var addStates = {
            name: 'extraStates',
            start: function (data) {
                var stateMachine = data.entity.getComponent('stateMachine');
                stateMachine.addState('runAttack', FitGirl.States.cartwheel);
                stateMachine.addState('chargeAttack', FitGirl.States.chargeKick);
                stateMachine.addState('comboEnd', FitGirl.States.hair);
                stateMachine.addState('jumpAttack', FitGirl.States.jumpAttack);
                stateMachine.addState('superAttack', FitGirl.States.superAttack);
            }
        };

        var behavior = {
            name: 'fitGirlBehavior',
            start: function () {
                fitGirl.getComponent('stateMachine', function (stateMachine) {
                    stateMachine.data.punchReset = [3, 2, 2, 2];
                });
            }
        };

        var fitGirl = new Player({
            rootPath: 'fitgirl/',
            healthBar: settings.healthBar,
            position: settings.position,
            family: ['player'],
            online: settings.online,
            name: settings.name,
            id: settings.id,
            stats: StatsManager.getAllStats('fitgirl'),
            health: settings.health,
            components: [
                addStates,
                behavior
            ]
        });

        return fitGirl;
    };

    /**
     * Pluggable states
     */
    FitGirl.States = {
        cartwheel: {
            start: function (data, transitionData) {
                data.sprite.setSpriteSheet(data.rootPath + 'cartwheel', function () {
                    data.setState('idle');
                });
                data.specialStage = transitionData.specialStage;
                data.hasPunched = 0;

                Bento.audio.stopSound('sfx_dashattack');
                Bento.audio.playSound('sfx_dashattack');

                Bento.audio.stopSound('fitgirl/sfx_dash');
                Bento.audio.playSound('fitgirl/sfx_dash');

            },
            update: function (data) {
                var speed = [1, 0];
                data.movable.velocity.x = 2 * data.multiScale.direction.x;

                //punches are created first for auto-target purposes, and added later for animation purposes
                var baseAttack = data.stats.attack;
                var multiplier = [1.5, 2];
                var pushHeight = [1.5, 2.5];
                var freeze = [7, 10];
                var punch;

                if (data.hasPunched < 1 && data.sprite.getCurrentFrame() > 2) {
                    punch = data.createPunch({
                        target: ['enemy', 'destructible'],
                        attack: baseAttack / 3,
                        stamina: 0.334 / 2,
                        skill: data.stats.skill,
                        duration: 2,
                        multiplier: multiplier[data.specialStage - 1] * BuffManager.rollCritical('dashAttack'),
                        freeze: 5,
                        pushBack: new Vector2(0.75 * BuffManager.rollCritical('knockback'), 2),
                        boundingBox: new Rectangle(0, -5, 28, 10),
                        offset: new Vector2(0, 0),
                        ignoreFlip: true,
                        onHit: Punch.getDefaultCallback({
                            entity: data.entity,
                            effect: 'effects/hitfx1-medium',
                            sfx: 'sfx_dashattack-hit',
                            taptic: 'heavy',
                            onComplete: function () {
                                data.movable.velocity.x *= 0.5;
                            }
                        }),
                        type: 'dashAttack'
                    });
                    data.movable.velocity.x = 0.5 * data.multiScale.direction.x;
                    Bento.objects.attach(punch);
                    data.hasPunched = 1;
                } else if (data.hasPunched < 2 && data.sprite.getCurrentFrame() > 4) {
                    punch = data.createPunch({
                        target: ['enemy', 'destructible'],
                        attack: baseAttack / 3,
                        stamina: 0.334 / 2,
                        skill: data.stats.skill,
                        duration: 2,
                        multiplier: multiplier[data.specialStage - 1],
                        freeze: 5,
                        pushBack: new Vector2(1 * BuffManager.rollCritical('knockback'), pushHeight[data.specialStage - 1]),
                        boundingBox: new Rectangle(0, -5, 28, 10),
                        offset: new Vector2(0, 0),
                        ignoreFlip: true,
                        onHit: Punch.getDefaultCallback({
                            entity: data.entity,
                            effect: 'effects/hitfx1-medium',
                            sfx: 'sfx_dashattack-hit',
                            taptic: 'heavy',
                            onComplete: function () {
                                data.movable.velocity.x *= 0.5;
                            }
                        }),
                        type: 'dashAttack'
                    });
                    data.movable.velocity.x = Utils.approach(data.movable.velocity.x, 0, 0.1);
                    data.hasPunched = 2;
                    data.movable.velocity.x = 0.5 * data.multiScale.direction.x;
                    Bento.objects.attach(punch);

                } else if (data.sprite.getCurrentFrame() > 6) {
                    data.movable.velocity.x = 0.5 * data.multiScale.direction.x;
                }
                if (data.entity.timer % 4 === 0) {
                    var puff = new Effect({
                        position: data.entity.position.add(new Vector2(
                            data.multiScale.direction.x * -16,
                            0
                        )),
                        spriteSheet: 'effects/walkcloud'
                    });
                    puff.scale.x = data.multiScale.direction.x;
                    puff.attach(new Sort({}));
                    Bento.objects.attach(puff);
                }
            },
            end: function (data) {
                data.specialStage = 0;
            }
        },
        chargeKick: {
            start: function (data, transitionData) {
                data.isLanding = false;
                // TODO
                data.sprite.setSpriteSheet(data.rootPath + 'chargeup-attack', function () {
                    data.isLanding = true;
                    data.sprite.setSpriteSheet(data.rootPath + 'idle');
                    // data.sprite.setAnimation('shoryukenEnd');
                });
                data.specialStage = transitionData.specialStage;

                var baseAttack = data.stats.attack;
                var multiplier = 2 * BuffManager.rollCritical('chargeAttack');
                var bonusAttack = [0, 2];
                var pushHeight = [3, 4];
                var freeze = [7, 10];

                Bento.audio.stopSound('sfx_special');
                Bento.audio.playSound('sfx_special');

                Bento.audio.stopSound('fitgirl/sfx_special');
                Bento.audio.playSound('fitgirl/sfx_special');

                //punches are created first for auto-target purposes, and added later for animation purposes
                data.punch = data.createPunch({
                    target: ['enemy', 'destructible'],
                    attack: baseAttack,
                    stamina: 1,
                    skill: data.stats.skill,
                    multiplier: multiplier,
                    bonusAttack: bonusAttack[data.specialStage - 1],
                    pushBack: new Vector2(0.5 * BuffManager.rollCritical('knockback'), pushHeight[data.specialStage - 1]),
                    boundingBox: new Rectangle(0, -5, 28, 10),
                    offset: new Vector2(8, 0),
                    freeze: 8,
                    onHit: Punch.getDefaultCallback({
                        entity: data.entity,
                        effect: 'effects/hitfx1',
                        sfx: 'sfx_special-hit',
                        taptic: 'heavy'
                    }),
                    type: 'specialAttack'
                });
            },
            update: function (data) {
                var jump = [-3, -4];
                if (data.punch && data.sprite.getCurrentFrame() > 2) {
                    data.movable.velocity.x = 0.2 * data.multiScale.direction.x;
                    // data.movable.velocityZ = jump[data.specialStage - 1];
                    data.movable.grounded = false;

                    Bento.objects.attach(data.punch);
                    data.punch = null;
                }

                if (data.isLanding && data.movable.grounded) {
                    data.setState('idle');
                }
            },
            action: function (data, action) {
                switch (action.type) {
                case 'tap':
                    if (data.isLanding) {
                        // data.setState('airPunch');
                    }
                    break;
                }
            },
            end: function (data) {
                data.isLanding = false;
                data.specialStage = 0;
            }
        },
        jumpAttack: {
            start: function (data, transitionData) {
                EventSystem.fire('playerJumpAttack');
                data.movable.velocity.x = 0;
                data.movable.velocity.y = 0;
                var direction = transitionData.cacheDirection;
                data.multiScale.direction.x = direction;
                data.sprite.setSpriteSheet(data.rootPath + 'jumpattack-windup', function () {
                    Bento.audio.stopSound('sfx_dashattack');
                    Bento.audio.playSound('sfx_dashattack');
                    Bento.audio.stopSound('fitgirl/sfx_throw');
                    Bento.audio.playSound('fitgirl/sfx_throw');
                    data.sprite.setSpriteSheet(data.rootPath + 'jumpattack-loop');
                    data.movable.velocityZ = -2.5;
                    data.movable.velocity.x = direction * 3;
                    data.attackState = 1;
                    data.punchable.setInvincible(true);
                    var effect = new Effect({
                        z: Globals.Layers.ENEMIES,
                        position: data.entity.position.add(new Vector2(0, -8)),
                        spriteSheet: 'effects/jump-dust'
                    });
                    Bento.objects.attach(effect);
                });
                data.attackState = 0;
                data.punchable.setInvincible(false);
            },
            update: function (data) {
                var invincible = true;
                if (data.sprite.currentSpriteSheet === data.rootPath + 'jumpattack-loop') {
                    if (data.attackState === 1 && data.sprite.getCurrentFrame() >= 0) {
                        // add punch
                        data.punch = data.createPunch({
                            target: ['enemy', 'destructible'],
                            attack: data.stats.attack,
                            stamina: 0.5,
                            skill: data.stats.skill,
                            multiplier: data.stats.multiplier,
                            bonusAttack: 0,
                            freeze: 7,
                            height: 32,
                            duration: 20,
                            pushBack: new Vector2(0.5 * BuffManager.rollCritical('knockback'), 0.8),
                            boundingBox: new Rectangle(0, -5, 28, 10),
                            offset: new Vector2(8, 0),
                            ignoreFlip: true,
                            onHit: Punch.getDefaultCallback({
                                entity: data.entity,
                                effect: 'effects/hitfx1-medium',
                                sfx: 'sfx_punch1',
                                taptic: 'medium'
                            }),
                            type: 'jumpAttack'
                        });
                        Bento.objects.attach(data.punch);
                        data.attackState = 2;
                    }
                    // check if about to land
                    if (data.movable.velocityZ > 0 && data.movable.height > -10) {
                        data.sprite.setSpriteSheet(data.rootPath + 'jumpattack-land', function () {
                            data.setState('idle');
                        });
                    }
                } else if (data.sprite.currentSpriteSheet === data.rootPath + 'jumpattack-land' &&
                    data.sprite.getCurrentFrame() >= 1
                ) {
                    invincible = false;
                }
                data.punchable.setInvincible(invincible);
            }
        },
        hair: {
            start: function (data) {
                var baseAttack = data.stats.attack;
                data.sprite.setSpriteSheet(data.rootPath + 'hair', function () {
                    data.setState('idle');
                });

                data.combo = 0;

                Bento.audio.stopSound('sfx_kick');
                Bento.audio.playSound('sfx_kick');
                Bento.audio.stopSound('fitgirl/sfx_dash');
                Bento.audio.playSound('fitgirl/sfx_dash');

                //punches are created first for auto-target purposes, and added later for animation purposes
                data.punch = data.createPunch({
                    target: ['enemy', 'destructible'],
                    attack: baseAttack,
                    stamina: 1,
                    skill: data.stats.skill,
                    multiplier: BuffManager.rollCritical('comboEnd'),
                    freeze: 5,
                    pushBack: new Vector2(0.75 * BuffManager.rollCritical('knockback'), 2.0),
                    boundingBox: new Rectangle(0, -5, 18, 10),
                    offset: new Vector2(8, 0),
                    onHit: Punch.getDefaultCallback({
                        entity: data.entity,
                        effect: 'effects/hitfx1-medium',
                        sfx: 'sfx_punch3',
                        taptic: 'medium',
                        onComplete: function () {
                            data.movable.velocity.x = 1 * data.multiScale.direction.x;
                        }
                    })
                });
            },
            update: function (data) {
                if (data.punch && data.sprite.getCurrentFrame() > 2) {
                    Bento.objects.attach(data.punch);
                    data.punch = null;
                    data.movable.velocity.x = 1.3 * data.multiScale.direction.x;
                }
            }
        },
        superAttack: {
            start: function (data, transitionData) {
                if (!data.isRemote) {
                    EventSystem.fire('quest-doSuperAttack', {
                        count: 1
                    });
                }
                if (!data.isRemote) {
                    var enemies = [];
                    // check for enemies on screen
                    var screenBB = Bento.getViewport().clone();
                    //shrink screen bounding box by players boundingbox
                    screenBB.grow(-data.movable.boundingBox.width, false, true);
                    Utils.forEach(Bento.objects.getByFamily('enemy'), function (enemy, i, l, breakLoop) {
                        var ignoreEnemy = enemy.family.indexOf('ignoreEnemy');

                        // check if dead
                        enemy.getComponent('punchable', function (punchable) {
                            if (punchable.isDead || punchable.getHealth() <= 0) {
                                ignoreEnemy = 1;
                            }
                        });

                        enemy.getComponent('movable', function (movable) {
                            if (!movable.grounded) {
                                ignoreEnemy = 1;
                            }
                        });

                        if (Math.abs(enemy.position.x - data.entity.position.x) < 90 &&
                            screenBB.hasPosition(enemy.position) &&
                            ignoreEnemy === -1
                        ) {
                            enemies.push(enemy);
                        }
                    });

                    // target enemies in front, with highest health
                    var health = 0;
                    var hp = 0;
                    var target;
                    var sign = data.multiScale.direction.x;
                    var range = 40;
                    var distance = 90;
                    Utils.forEach(enemies, function (enemy, i, l, breakLoop) {
                        if (Utils.sign(enemy.position.x - data.entity.position.x) === sign &&
                            Math.abs(enemy.position.y - data.entity.position.y) < range
                        ) {
                            if (Math.abs(data.entity.position.x - enemy.position.x) < distance) {
                                target = enemy;
                                distance = Math.abs(data.entity.position.x - enemy.position.x);
                            }
                        }
                    });

                    // sync data for remotes
                    data.sync({
                        target: target ? target.name : '',
                        superAttack: target ? true : false
                    });

                    if (target) {
                        Bento.objects.get('healthBar', function (healthBar) {
                            healthBar.depleteComboBar();
                        });
                    }

                    data.entity.getComponent('weapon', function (weapon) {
                        weapon.getComponent('weapon').drop(0);
                    });
                }

                if (data.syncData.superAttack) {
                    data.attackState = 0;
                    if (!data.isRemote) {
                        data.punchable.setInvincible(true);
                        EventSystem.fire('superAttack');
                    }
                    data.sprite.setSpriteSheet(data.rootPath + 'idle');
                    data.movable.settings.ignoreSolids = true;
                } else {
                    data.setState('superAttackDud');
                }
                data.currentTarget = null;
            },
            update: function (data) {
                var target = data.currentTarget;
                if (!target) {
                    // find and cache the target
                    data.currentTarget = Bento.objects.get(data.syncData.target);
                    target = data.currentTarget;
                }
                if (!target) {
                    // target could not be found
                    data.setState('idle');
                    return;
                }

                //set invincible every update
                if (!data.isRemote) {
                    data.punchable.setInvincible(true);
                }

                // align with target
                if (data.attackState === 0) {
                    target.getComponent('stateMachine', function (stateMachine) {
                        stateMachine.setState('targetSuperAttack');
                    });

                    data.sprite.setSpriteSheet(data.rootPath + 'walk');

                    var xSign = Utils.sign(target.position.x - data.entity.position.x);
                    if (xSign === 0) {
                        xSign = 1 + Utils.getRandom(2) * -2;

                        //lower music volume
                        new Tween({
                            from: 1,
                            to: 0.3,
                            in: 20,
                            delay: 0,
                            ease: 'easeInQuad',
                            onUpdate: function (v, t) {
                                Bento.audio.setVolume(v, Globals.currentMusic);
                            },
                        });
                    }

                    if (!data.isRemote) {
                        data.blackBar = new SuperAttackBars({});
                        Bento.objects.attach(data.blackBar);
                    }

                    // clamp position inside playable area
                    var targetPoint = target.position.add(new Vector2(-xSign * 100, 0));
                    var rightSide = Bento.screens.getCurrentScreen().dimension.width - 10;
                    if (Blockade.active) {
                        rightSide = Blockade.active.position.x;
                    }
                    targetPoint.x = Utils.clamp(10, targetPoint.x, rightSide);

                    new Tween({
                        from: data.entity.position,
                        to: targetPoint,
                        in: 30,
                        delay: 0,
                        ease: 'linear',
                        onStart: function () {},
                        onUpdate: function (v, t) {
                            data.entity.position = v;
                            data.multiScale.direction.x = xSign;
                        },
                        onComplete: function () {
                            data.sprite.setSpriteSheet(data.rootPath + 'idle');
                            // Utils.timeout(30, function () {
                            data.attackState = 2;

                            if (!data.isRemote) {
                                // set layers for player and target to be above everything else
                                data.entity.removeByName('sort');
                                data.entity.z = Globals.Layers.SUPERATTACK;
                                target.removeByName('sort');
                                target.z = Globals.Layers.SUPERATTACK - 2;
                                // force camera between player and enemy
                                Bento.objects.get('camera', function (camera) {
                                    camera.getComponent('cameraBehavior', function (camBehavior) {
                                        camBehavior.target = {
                                            position: new Vector2(data.entity.position.x * 0.5 + target.position.x * 0.5, data.entity.position.y * 0.5 + target.position.y * 0.5)
                                        };
                                    });
                                });
                            }

                            var effect = new Effect({
                                position: data.entity.position.add(new Vector2(-xSign * 4, -16)),
                                spriteSheet: 'effects/superattack-chargefx'
                            });

                            if (!data.isRemote) {
                                effect.z = Globals.Layers.SUPERATTACK - 1;
                            }
                            effect.scale.x = data.multiScale.direction.x;
                            Bento.objects.add(effect);

                            // set player sprite and add fx
                            Utils.timeout(2, function () {
                                //play supercharge sound
                                Bento.audio.stopSound('sfx_super-charge');
                                Bento.audio.playSound('sfx_super-charge');
                                data.sprite.setSpriteSheet(data.rootPath + 'superattack-windup');
                            });

                            Utils.timeout(42, function () {
                                data.sprite.setSpriteSheet(data.rootPath + 'superattack-spinning');
                                data.movable.velocityZ = -4;
                                data.sprite.setCurrentSpeed(0.4);

                                data.attackState = 3;
                            });
                            // });
                        }
                    });
                    data.attackState = 1;
                } else if (data.attackState === 3) {
                    // dash towards target
                    data.attackState = 4;
                    Bento.audio.stopSound(data.rootPath + 'sfx_attack');
                    Bento.audio.playSound(data.rootPath + 'sfx_attack');
                    new Tween({
                        from: data.entity.position.x,
                        to: target.position.x,
                        in: 40,
                        delay: 0,
                        ease: 'easeOutQuad',
                        onStart: function () {
                            var effect = new Effect({
                                position: data.entity.position.add(new Vector2(Utils.sign(target.position.x - data.entity.position.x) * 40, 0)),
                                spriteSheet: 'effects/superattack-dashfx'
                            });

                            if (!data.isRemote) {
                                effect.z = Globals.Layers.SUPERATTACK - 1;
                            }
                            effect.scale.x = data.multiScale.direction.x;
                            Bento.objects.add(effect);
                        },
                        onUpdate: function (v, t) {
                            data.entity.position.x = v;
                            data.movable.velocityZ += (t / 40) * (t / 40) * 0.15;
                        },
                        onComplete: function () {
                            data.attackState = 5;
                            // arrived at target, show hit fx
                            new Tween({
                                from: -24,
                                to: -38,
                                in: 32,
                                delay: 8,
                                ease: 'easeOutQuad',
                                onStart: function () {
                                    data.attackState = 6;
                                },
                                onUpdate: function (v, t) {
                                    data.movable.height = v;
                                    data.movable.velocityZ = 0;
                                },
                                onComplete: function () {

                                }
                            });

                            Utils.timeout(3, function () {
                                Bento.audio.stopSound(data.rootPath + 'sfx_dash');
                                Bento.audio.playSound(data.rootPath + 'sfx_dash');
                                Bento.audio.stopSound('sfx_punch1');
                                Bento.audio.playSound('sfx_punch1');
                            });

                            // enemy should look hurt when jumped upon
                            target.getComponent('stateMachine', function (stateMachine) {
                                stateMachine.data.sprite.setSpriteSheet(stateMachine.data.rootPath + 'hurt1');
                                Utils.timeout(20, function () {
                                    stateMachine.data.sprite.setSpriteSheet(stateMachine.data.rootPath + 'idle');
                                });
                            });

                            data.sprite.setSpriteSheet(data.rootPath + 'superattack-jump', function () {
                                // drop down
                                data.attackState = 6;
                                data.sprite.setSpriteSheet(data.rootPath + 'superattack-cannonball');
                                data.movable.height = -74;
                                data.multiScale.direction.x = -data.multiScale.direction.x;

                                new Tween({
                                    from: -74,
                                    to: 0,
                                    in: 10,
                                    delay: 0,
                                    ease: 'easeInCubic',
                                    onUpdate: function (v, t) {
                                        data.movable.velocityZ = 0;
                                        data.movable.height = v;
                                    },
                                    onComplete: function () {
                                        data.sprite.setSpriteSheet(data.rootPath + 'superattack-land', function () {
                                            data.setState('idle');
                                            if (!data.isRemote) {
                                                Bento.objects.get('camera', function (camera) {
                                                    camera.getComponent('cameraBehavior', function (camBehavior) {
                                                        camBehavior.target = data.entity;
                                                    });
                                                });
                                            }
                                        });
                                        data.attackState = 7;
                                    }
                                });
                            });

                            new Tween({
                                from: 1.3,
                                to: 1,
                                in: 10,
                                delay: 0,
                                ease: 'easeInBack',
                                onStart: function () {},
                                onUpdate: function (v, t) {
                                    target.getComponent('multiScale', function (multiScale) {
                                        multiScale.hurt.x = v;
                                        multiScale.hurt.y = 1 / v;
                                    });
                                },
                                onComplete: function () {

                                }
                            });

                            if (!data.isRemote) {
                                data.blackBar.flash();
                            }
                        }
                    });
                } else if (data.attackState === 5) {
                    data.movable.height = -24;
                    data.movable.velocityZ = 0;
                } else if (data.attackState === 7) {

                    if (!data.isRemote) {
                        Bento.objects.attach(new SuperAttackSpeedlines({
                            actors: [
                                data.entity,
                                target
                            ],
                            rootPath: data.rootPath
                        }));
                        data.blackBar.removeSelf();
                        data.blackBar = undefined;

                        Bento.objects.get('camera', function (camera) {
                            camera.shake(30, 15);
                        });
                    }

                    var effect = new Effect({
                        position: target.position.add(new Vector2(0, -12)),
                        spriteSheet: 'effects/hitfx1'
                    });
                    if (!data.isRemote) {
                        effect.z = Globals.Layers.SUPERATTACK - 1;
                    }
                    Bento.objects.add(effect);
                    effect = new Effect({
                        position: target.position.add(new Vector2(0, 0)),
                        spriteSheet: 'effects/superattack-dust-ground'
                    });
                    if (!data.isRemote) {
                        effect.z = Globals.Layers.SUPERATTACK - 1;
                    }
                    Bento.objects.add(effect);

                    Utils.timeout(2, function () {
                        if (!data.isRemote) {
                            target.getComponent('stateMachine', function (stateMachine) {
                                stateMachine.data.punchable.setInvincible(false);
                                stateMachine.data.punchable.hit(data.createPunch({
                                    target: ['enemy', 'destructible'],
                                    attack: data.stats.attack,
                                    stamina: 1,
                                    skill: Infinity,
                                    multiplier: 15,
                                    direction: -data.multiScale.direction.x,
                                    pushBack: new Vector2(3, 2.0),
                                    boundingBox: new Rectangle(0, -5, 16, 10),
                                    offset: new Vector2(8, 0),
                                    onHit: Punch.getDefaultCallback({
                                        entity: data.entity,
                                        effect: 'effects/hitfx1',
                                        sfx: 'sfx_finalpunch',
                                        taptic: 'heavy',
                                        onComplete: function () {
                                            data.movable.velocity.x = 1 * data.multiScale.direction.x;
                                        }
                                    }),
                                    type: 'superAttack'
                                }));
                            });

                            TapticEngine.impact({
                                style: 'heavy'
                            });
                        }
                    });

                    new Tween({
                        from: data.entity.position.x,
                        to: target.position.x,
                        in: 20,
                        delay: 0,
                        ease: 'easeOutQuad',
                        onUpdate: function (v, t) {
                            data.entity.position.x = v;
                        }
                    });

                    Bento.audio.stopSound('fitgirl/sfx_special');
                    Bento.audio.playSound('fitgirl/sfx_special');

                    if (!data.isRemote) {
                        new Tween({
                            from: 0.3,
                            to: 1,
                            in: 20,
                            delay: 0,
                            ease: 'easeInQuad',
                            onUpdate: function (v, t) {
                                Bento.audio.setVolume(v, Globals.currentMusic);
                            },
                        });
                    }

                    data.attackState = 8;
                }
            },
            end: function (data) {
                if (!data.isRemote) {
                    Utils.timeout(80, function () {
                        data.punchable.setInvincible(false);
                    });
                }
                data.movable.settings.ignoreSolids = false;
            }
        }
    };
    return FitGirl;
});