/**
 * Project entity currently used for bullets
 * @moduleName Projectile
 */
bento.define('entities/projectile', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'entities/punch',
    'components/sort',
    'managers/buff'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Punch,
    Sort,
    BuffManager
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var screenDimension = Bento.screens.getCurrentScreen().dimension;
        settings.direction.normalize();
        var bulletSpeed = 3;
        var startOffset = settings.offset || 40;
        var startHeight = settings.height || 24;
        var behavior = {
            name: 'bulletBehavior',
            onParentAttached: function () {
                Bento.objects.attach(punchObject);
            },
            update: function () {
                // set the screen target
                var targetCoordinate = viewport.x - 8;
                if (settings.direction.x > 0) {
                    targetCoordinate += viewport.width + 16;
                }

                // move when target hasnt been reached yet, else remove
                if (Utils.sign(targetCoordinate - bulletObject.position.x) === Utils.sign(settings.direction.x)) {
                    bulletObject.position.x += settings.direction.x * bulletSpeed;
                    bulletObject.position.y += settings.direction.y * bulletSpeed;
                } else {
                    Bento.objects.remove(bulletObject);
                    punchObject.disable();
                }
            }
        };

        var bulletSprite = new Sprite({
            imageName: settings.bullet || 'bossbullet',
        });

        var bulletObject = new Entity({
            name: 'bulletguide',
            position: settings.position.add(new Vector2((settings.direction.x < 0 ? -startOffset : startOffset), 0)),
            components: [
                bulletSprite,
                new Sort({}),
                behavior
            ]
        });
        bulletSprite.origin.x = bulletSprite.frameWidth / 2;
        bulletSprite.origin.y = startHeight + bulletSprite.frameHeight / 2;
        bulletObject.scale.x = settings.direction.x;

        var punchBehavior = {
            name: 'punchBehavior',
            update: function () {
                var bb = punchObject.boundingBox.offset(punchObject.position);
                var players = Bento.objects.getByFamily('player');

                // collision checks to remove the bullet if other players are hit in multiplayer
                Utils.forEach(players, function (player, i, l) {
                    if (player.isRemote) {
                        if (player.collidesWith({
                                rectangle: bb
                            })) {
                            // cannot see other players invincible status so check animation instead to see if bullet passes through or not
                            var sprite = player.getComponent('sprite');
                            var animation = sprite.currentSpriteSheet.split('/')[1];
                            if (animation !== 'liedown' && animation !== 'getup') {
                                Bento.objects.remove(bulletObject);
                                punchObject.disable();
                            }
                        }
                    }
                });
            }
        };

        var punchObject = new Punch({
            target: ['player', 'citizen'],
            duration: Infinity,
            attack: settings.attack || 1,
            multiplier: BuffManager.current.enviromentDamage,
            stamina: settings.stamina || 0.334,
            direction: Utils.sign(settings.direction.x),
            pushBack: new Vector2(0.7, 1.2),
            parent: bulletObject,
            boundingBox: new Rectangle(-10, -8, 20, 16),
            position: bulletObject.position,
            onHit: Punch.getDefaultCallback({
                entity: settings.entity,
                effect: 'effects/hitfx3-small',
                sfx: 'sfx_punch3',
                taptic: 'medium',
                onComplete: function () {
                    Bento.objects.remove(bulletObject);
                    punchObject.disable();
                }
            }),
            debug: true
        }).attach(
            punchBehavior
        );



        return bulletObject;
    };
});