/**
 * Destructible prop
 * @moduleName DestructibleProp
 */
bento.define('entities/props/destructible', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'entities/props/generic',
    'components/destructible',
    'components/punchable',
    'managers/buff'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    GenericProp,
    Destructible,
    Punchable,
    BuffManager
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var id = settings.tiled.object.id;
        var tileProperties = settings.tiled.tileProperties;
        var rootPath = tileProperties.rootPath || '';
        var properties = settings.tiled.object.properties || {};
        var coins = parseInt(properties.coins) || 0;
        var item;
        // extra properties extracted from name
        var objectName = settings.tiled.object.name;
        var nameProperties = objectName.split(',');
        // get coins from name
        Utils.forEach(nameProperties, function (prop) {
            var values = prop.split(':');
            var key = values[0];
            var value = parseFloat(values[1]);
            if (isNaN(value)) {
                // probably a string
                value = values[1];
            }

            // coins
            if (key === 'coins') {
                coins = value;
            } else if (key === 'item') {
                item = 'consumables/' + value;
            } else if (key === 'weapon') {
                item = 'weapons/' + value;
            }
        });

        var boundingBox = (function () {
            var coords;
            if (!tileProperties || !tileProperties.boundingBox) {
                return null;
            }
            coords = tileProperties.boundingBox.split(',');
            return new Rectangle(
                parseInt(coords[0]),
                parseInt(coords[1]),
                parseInt(coords[2]),
                parseInt(coords[3])
            );
        })();

        var destructible = new Destructible({
            health: 3,
            coins: coins,
            item: item,
            spriteSheet: rootPath + 'broken'
        });
        var punchable = new Punchable({
            boundingBox: boundingBox,
            health: Infinity,
            maxHealth: Infinity,
            breakable: true
        });

        var prop = new GenericProp(settings)
            .attach(destructible)
            .attach(punchable);

        return prop;
    };
});