/**
 * Gate in tower challenge
 * @moduleName Gate
 * @snippet Gate.snippet
Gate({
    position: new Vector2(0, 0)
})
 */
bento.define('entities/props/gate', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/tween',
    'entities/bossintro',
    'modules/matchmanager'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    BossIntro,
    MatchManager
) {
    'use strict';
    return function (settings) {
        var viewport = Bento.getViewport();
        var position = settings.position;
        var sprite = new Sprite({
            spriteSheet: 'props/gate/gate'
        });
        var player;
        var hasOpened = false;
        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {
                MatchManager.onMessage(entity.name, behavior.onMessage);
            },
            destroy: function (data) {
                MatchManager.onMessage(entity.name, behavior.onMessage);
            },
            onMessage: function () {
                openGate();
                // handled message
                return true;
            },
            update: function (data) {
                // player nears the gate
                if (!player) {
                    player = Utils.getLocalPlayer();
                    return;
                }
                if (!hasOpened && entity.position.x - player.position.x < 96) {
                    openGate();
                }
            },
            draw: function (data) {}
        };
        var openGate = function () {
            if (hasOpened) {
                return;
            }
            hasOpened = true;
            MatchManager.sendData({}, entity.name);
            // camera shake
            Bento.objects.get('camera', function (camera) {
                camera.shake(16, 15);
            });

            Bento.audio.stopSound('sfx_boulder');
            Bento.audio.playSound('sfx_boulder');

            new Tween({
                from: entity.position.y,
                to: entity.position.y - 360,
                in: 120,
                delay: 30,
                applyOnDelay: 0,
                ease: 'easeInCirc',
                decay: 1,
                oscillations: 1,
                onStart: function () {},
                onUpdate: function (v, t) {
                    entity.position.y = v;
                },
                onComplete: function () {
                    Bento.objects.attach(new BossIntro({}));
                }
            });
        };
        var entity = new Entity({
            name: 'towerGate',
            family: ['solid'],
            components: [
                sprite,
                behavior
            ]
        });
        return entity;
    };
});