/**
 * Generic prop spawned from a tileset
 * @moduleName GenericProp
 */
bento.define('entities/props/generic', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'components/sort',
    'components/drawinviewport'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Sort,
    DrawInViewport
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var tileProperties = settings.tiled.tileProperties;
        var rootPath = tileProperties.rootPath || '';

        // var imageName = settings.tiled.tileSet.name; // assume name of tileset is the imageName
        var fromSheet = tileProperties.fromSheet || '';
        // var imageName = tileProperties.imageName || '';
        var imageName = fromSheet ? settings.tiled.tileSet.name : tileProperties.imageName || '';
        var tileSet = settings.tiled.tileSet;
        var tileWidth = tileSet.tilewidth;
        var tileHeight = tileSet.tileheight;
        var tileIndex = settings.tiled.tileIndex;

        var sheetSettings = {
            originRelative: new Vector2(0.5, 1),
            imageName: imageName,
            frameWidth: tileWidth,
            frameHeight: tileHeight,
            animations: {
                default: {
                    speed: 0,
                    frames: [tileIndex]
                }
            }
        };

        var boundingBox = (function () {
            var coords;
            if (!tileProperties || !tileProperties.boundingBox) {
                return null;
            }
            coords = tileProperties.boundingBox.split(',');
            return new Rectangle(
                parseInt(coords[0]),
                parseInt(coords[1]),
                parseInt(coords[2]),
                parseInt(coords[3])
            );
        })();
        var sprite = new Sprite(fromSheet ? sheetSettings : {
            spriteSheet: rootPath + 'normal'
        });
        var entity = new Entity({
            z: 0,
            name: 'prop',
            family: boundingBox ? ['solid'] : [],
            boundingBox: boundingBox,
            components: [
                sprite,
                new Sort({}),
                new DrawInViewport({})
            ]
        });
        return entity;
    };
});