/**
 * item that drops from boss
 * @moduleName LootDrop
 */
bento.define('entities/props/lootdrop', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'components/sort',
    'components/movable',
    'components/shadow',
    'modules/matchmanager',
    'globals',
    'gui/shadowtext',
    'components/orientationlock'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Sort,
    Movable,
    Shadow,
    MatchManager,
    Globals,
    ShadowText,
    OrientationLock
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var sprite = settings.sprite || new Sprite({
            imageName: settings.imageName,
            spriteSheet: settings.spriteSheet
        });
        var sort = new Sort({});
        var rotationSpeed = Math.PI / 20;
        var movable = new Movable({
            gravity: 0.15,
            bounceZ: 0.5,
            frictionZ: 0.5,
            onLanded: function () {
                rotationSpeed /= 2;
            },
            boundingBox: new Rectangle(-8, -8, 16, 16)
        });
        var shadow = new Shadow({
            imageName: 'shadow16'
        });
        var player;
        var amountText = new ShadowText({
            position: new Vector2(8, 8),
            text: '',
            font: 'font',
            fontSize: 16,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle'
        });
        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {
                sprite.setOriginRelative(new Vector2(0.5, 0.5));
                movable.velocityZ = -4;
                movable.velocity.x = Utils.sign(viewport.x + viewport.width / 2 - entity.position.x) * 0.5;
                player = Utils.getLocalPlayer();
                // move up half the dimension height because of origin difference
                entity.position.y -= entity.dimension.height / 2;

                if (settings.dontShow) {
                    if (!settings.forceDirection) {
                        // change direction
                        movable.velocity = new Vector2(0, 0.75);
                        movable.velocity.rotateDegree(Utils.getRandom(360));

                        // change rotation speed
                        rotationSpeed += Utils.getRandomRangeFloat(-rotationSpeed / 2, rotationSpeed / 2);
                    }
                    return;
                }
                Utils.timeout(180, function () {
                    movable.velocity.x = 0;
                    movable.velocity.y = 0;
                    movable.velocityZ = 0;
                    movable.gravity = 0;
                    new Tween({
                        from: entity.position.x,
                        to: viewport.x + viewport.width / 2,
                        // to: player.position.x,
                        in: 20,
                        ease: 'easeOutQuart',
                        onUpdate: function (v, t) {
                            entity.position.x = v;
                        }
                    });
                    new Tween({
                        from: entity.position.y,
                        to: viewport.y + viewport.height / 2 - 32,
                        // to: player.position.y - 64,
                        in: 30,
                        delay: 20,
                        ease: 'easeOutBack',
                        onStart: function () {
                            shadow.visible = false;
                            entity.z = Globals.Layers.VICTORY;
                            entity.remove(sort);
                        },
                        onUpdate: function (v, t) {
                            entity.position.y = v;
                        }
                    });
                    new Tween({
                        from: 1,
                        to: 2,
                        in: 30,
                        delay: 20,
                        ease: 'easeOutBack',
                        onStart: function () {
                            entity.attach(amountText);
                        },
                        onUpdate: function (v, t) {
                            entity.scale.x = v;
                            entity.scale.y = v;
                        }
                    });
                    new Tween({
                        from: entity.rotation % (2 * Math.PI),
                        to: 2 * Math.PI,
                        in: 50,
                        delay: 0,
                        ease: 'linear',
                        onStart: function () {
                            rotationSpeed = 0;
                        },
                        onUpdate: function (v, t) {
                            entity.rotation = v;
                        }
                    });
                });
            },
            destroy: function (data) {},
            update: function (data) {
                entity.rotation += rotationSpeed * data.speed;
            },
            draw: function (data) {}
        };
        var entity = new Entity({
            z: 0,
            name: 'lootDrop',
            position: settings.position,
            updateWhenPaused: 0,
            components: [
                sprite,
                behavior,
                movable,
                sort,
                shadow,
                // don't rotate during loot drop
                new OrientationLock({})
            ]
        }).extend({
            setAmount: function (value) {
                if (value > 1) {
                    amountText.setText(value.toString());
                }
            }
        });
        return entity;
    };
});