/**
 * Prop particle
 * @moduleName Particle
 */
bento.define('entities/props/particle', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'components/sort',
    'components/movable',
    'components/shadow'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Sort,
    Movable,
    Shadow
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var sprite = new Sprite({
            imageName: settings.imageName,
            spriteSheet: settings.spriteSheet
        });
        var sort = new Sort({});
        var rotationSpeed = Math.PI / 20;
        var movable = new Movable({
            gravity: 0.15,
            bounceZ: 0.5,
            frictionZ: 0.5,
            onLanded: function () {
                rotationSpeed /= 2;
            },
            boundingBox: new Rectangle(-8, -8, 16, 16)
        });
        var shadow = new Shadow({});
        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {
                movable.velocityZ = -3;
                movable.velocity.x = 1 * settings.punch.direction;

                // move up half the dimension height because of origin difference
                entity.position.y -= entity.dimension.height / 2;

                new Tween({
                    delay: 30,
                    from: 1,
                    to: 0,
                    in: 40,
                    ease: 'linear',
                    onUpdate: function (v, t) {
                        entity.alpha = v;
                    },
                    onComplete: function () {
                        Bento.objects.remove(entity);
                    }
                });
            },
            destroy: function (data) {},
            update: function (data) {
                // entity.visible = (entity.timer % 2) < 1;
                entity.rotation += rotationSpeed * data.speed;
            },
            draw: function (data) {}
        };
        var entity = new Entity({
            name: 'propParticle',
            position: settings.position,
            components: [
                sprite,
                behavior,
                movable,
                sort
            ]
        });
        return entity;
    };
});