/**
 * Tower boss sitting on his throne, blocks attacks
 * @moduleName TowerBossProp
 */
bento.define('entities/props/towerboss', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/tween',
    'globals',
    'components/sort',
    'managers/save',
    'components/eventlistener',
    'components/coindropper',
    'components/itemdropper',
    'managers/tower'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Globals,
    Sort,
    SaveManager,
    EventListener,
    CoinDropper,
    ItemDropper,
    Tower
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var sprite = new Sprite({
            spriteSheet: 'props/towerboss/bossman-breathing'
        });
        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {
                entity.position.x -= 22;
                entity.position.y += 2;
            },
            destroy: function (data) {},
            update: function (data) {
                // block the players attack
                if (sprite.currentSpriteSheet === 'props/towerboss/bossman-breathing') {
                    var bb = entity.boundingBox.offset(entity.position);
                    Utils.forEach(Bento.objects.getByFamily('punches'), function (punch, i, l, breakLoop) {
                        var target = punch.target.indexOf('enemy');
                        if (target > -1) {
                            punch.collidesWith({
                                rectangle: bb,
                                onCollide: function (p) {
                                    var source = punch.parentEntity;
                                    if (Math.abs(source.position.x - entity.position.x) > 28) {
                                        var sign = Utils.sign(source.position.x - entity.position.x);
                                        if (sign === -1) {
                                            sprite.setSpriteSheet('props/towerboss/bossman-block-left', function () {
                                                sprite.setSpriteSheet('props/towerboss/bossman-breathing');
                                            });
                                        } else {
                                            sprite.setSpriteSheet('props/towerboss/bossman-block-right', function () {
                                                sprite.setSpriteSheet('props/towerboss/bossman-breathing');
                                            });
                                        }
                                        source.getComponent('movable', function (movable) {
                                            var force = source.position.subtract(entity.position);
                                            var forceMultiplier = 0.4;
                                            if (punch.attacker === punch.parentEntity) {
                                                // when the player is punching multiply the force
                                                forceMultiplier += 2.6;
                                            }
                                            force.normalize();
                                            force.scalarMultiplyWith(forceMultiplier);
                                            movable.velocity.x *= -0.2;
                                            movable.velocity.y *= -0.2;
                                            movable.velocity.addTo(force);
                                            movable.velocityZ += -1.2;
                                        });
                                        Bento.audio.stopSound('sfx_block');
                                        Bento.audio.playSound('sfx_block');
                                    }
                                }
                            });

                        }
                    });
                }
            },
            draw: function (data) {}
        };
        var eventListener = EventListener({
            name: 'slowmoEnd',
            onEvent: function (data) {
                var currentFloor = SaveManager.load('towerFloor');
                var coinDropper = new CoinDropper({
                    coins: 10,
                    downOnly: true,
                    ignoredFamilies: ['towerBoss'],
                    speedMultiplier: 2
                });
                var itemDropper = new ItemDropper({
                    item: Utils.pickRandom('consumables/apple', 'consumables/candy', 'consumables/banana'),
                    downOnly: true,
                    ignoredFamilies: ['towerBoss'],
                    speedMultiplier: 2
                });
                Utils.timeout(25, function () {
                    entity.attach(coinDropper);
                    coinDropper.drop();
                    // is there a gacha at this floor?
                    if (Globals.towerGachaFloors.indexOf(currentFloor) >= 0 && !Tower.shouldShowGacha()) {
                        entity.attach(itemDropper);
                        itemDropper.drop();
                    }                    
                });
                sprite.setSpriteSheet('props/towerboss/bossman-throw', function () {
                    sprite.setSpriteSheet('props/towerboss/bossman-breathing');
                });
            }
        });
        var entity = new Entity({
            name: 'towerBossProp',
            family: ['solid', 'towerBoss'],
            position: settings.position,
            boundingBox: new Rectangle(-32, -32, 64, 32),
            components: [
                new Sort({}),
                sprite,
                behavior,
                eventListener
            ]
        });
        return entity;
    };
});