/**
 * Punchingbag used in tutorial
 * @moduleName Punchingbag
 */
bento.define('entities/punchingbag', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'components/multiscale',
    'components/shadow',
    'components/sort',
    'components/punchable',
    'components/movable',
    'entities/punch',
    'components/damagecounter',
    'entities/tutorial/punchingbagdeath',
    'components/grabable',
    'components/statemachine'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    MultiScale,
    Shadow,
    Sort,
    Punchable,
    Movable,
    Punch,
    DamageCounter,
    PunchingBagDeath,
    Grabable,
    StateMachine
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();

        var boundingBox = new Rectangle(-12, -6, 24, 12);
        var spawnedX = settings.position ? settings.position.x : 0;
        var bagHeight = 149;
        // var bagHeight = 51;
        var punchable = new Punchable({
            boundingBox: boundingBox,
            health: Infinity,
            maxHealth: Infinity
        });

        // var damageCounter = new DamageCounter({
        //     color1: '#26bfff',
        //     color2: '#ffffff'
        // });

        var multiScale = new MultiScale({
            scales: {
                direction: new Vector2(1, 1)
            }
        });

        var shadow = new Shadow({});

        var bagPhysics = {
            name: 'bagPhysics',
            update: function () {
                punchingBag.rotation *= 0.95;
            }
        };
        var punchingBag = new Entity({
            name: 'punchingBag',
            position: new Vector2(0, 100),
            components: [
                new Sprite({
                    spriteSheet: 'punchingbag/punchingbag'
                }),
                multiScale,
                bagPhysics
            ]
        });

        var chainPhysics = {
            name: 'ropePhysics',
            force: 0,
            gravity: 0.4,
            update: function () {
                var players;
                if (grabable.isGrabbed() !== null) {
                    return;
                }
                // get the coordinates of the bottom of the bag
                var bagBottom = new Vector2(Math.sin(chain.rotation) * bagHeight, Math.cos(chain.rotation) * bagHeight);
                // get new angle after applying gravity
                var forceAngle = Math.atan2(bagBottom.x, bagBottom.y + chainPhysics.gravity);
                // get the difference between current rotation and rotationalgravity
                var forceRads = chain.rotation - forceAngle;
                // apply air friction
                chainPhysics.force *= 0.99;
                // add gravity force to current force
                chainPhysics.force -= forceRads;
                // apply force to rotation
                chain.rotation += chainPhysics.force;
                // set entity and chain position
                setBagPosition();
            }
        };

        var setBagPosition = function () {
            // angle difference in pixels
            var swayX = Math.sin(chain.rotation) * bagHeight * 0.9;
            // apply difference to chain and entity(punchingBag)
            chain.position.x = swayX;
            entity.position.x = spawnedX - swayX;
        };

        var chain = new Entity({
            name: 'rope',
            position: new Vector2(0, -156),
            components: [
                new Sprite({
                    spriteSheet: 'punchingbag-chain'
                }),
                chainPhysics,
                punchingBag
            ]
        });

        var onHit = function (data) {
            var punch = data.punch;
            var getHurt = true;
            var pushBackFactor = 1;
            var staminaFactor = 1;

            // movable.velocityZ = -punch.pushBack.y;
            // movable.velocity.x = punch.pushBack.x * punch.direction;

            if (!punchable.isDead) {
                // deplete health and stamina
                punchable.depleteHealth(punch);

                // add multiscale
                new Tween({
                    from: 0.5,
                    to: 0,
                    'in': 30,
                    ease: Tween.ELASTIC,
                    alpha: -5,
                    beta: 2,
                    onUpdate: function (v, t) {
                        multiScale.hurt.x = 1 + v;
                        multiScale.hurt.y = 1 - v;
                    }
                });
                // add rotational force to chain and bag
                var rotationDirection = Utils.toRadian(-data.punch.direction);
                if (Utils.sign(chainPhysics.force) !== Utils.sign(rotationDirection)) {
                    chainPhysics.force = rotationDirection * 0.10 * punch.multiplier;
                } else {
                    chainPhysics.force += rotationDirection * 0.10 * punch.multiplier;
                }
                punchingBag.rotation += rotationDirection * 13 * punch.multiplier * punch.multiplier;

            }
            EventSystem.fire('hitPunchingBag', {
                punch: punch,
                punchable: punchable
            });
        };

        var onDeath = function (data) {
            // TODO: come off the chain
            chain.remove(punchingBag);
            entity.remove(shadow);
            Bento.objects.attach(new PunchingBagDeath({
                position: new Vector2(entity.position.x, entity.position.y),
                direction: data.punch.direction
            }));
            // Bento.objects.remove(entity);
        };

        // grab behavior for tutorial
        var grabable = new Grabable({
            grabDuration: Infinity
        });
        var grabBehavior = {
            name: 'grabBehavior',
            start: function () {
                // for grabbing
                stateMachine.addState('idle', {
                    canGrab: true,
                    start: function (data) {
                        var dummy = new Entity({});
                        data.sprite = punchingBag.getComponent('sprite');
                        data.rootPath = 'punchingbag/';
                        data.movable = new Movable({});
                        data.multiScale = new MultiScale({
                            scales: {
                                direction: new Vector2(1, 1)
                            }
                        });
                        dummy.attach(data.multiScale);
                    }
                });
                stateMachine.setState('idle');
                // overwrite grabbedThrow
                stateMachine.addState('grabbedThrow', {
                    start: function (data, transitionData) {
                        data.setState('idle');

                        // swing
                        grabable.setGrabbed(null);
                        chainPhysics.force = transitionData.velocity.x * -0.01;
                    }
                });
                // overwrite hack
                grabable.update = null;
            }
        };
        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {
                if (!settings.position) {
                    entity.setPositionX(entity.position.x);
                }
                punchable.onHit(onHit);
                punchable.onDeath(onDeath);
                // Punchable.debug = true;
                multiScale.addScale('hurt', new Vector2(1, 1));
            },
            destroy: function (data) {},
            update: function (data) {
                // console.log(entity.getComponent('punchable'));
            },
            draw: function (data) {}
        };
        var stateMachine = new StateMachine({});
        var entity = new Entity({
            z: 0,
            name: 'punchingBag',
            family: ['characters', 'enemy', 'ignoreEnemy'],
            position: settings.position,
            updateWhenPaused: 0,
            boundingBox: boundingBox,
            components: [
                behavior,
                chain,
                punchable,
                // damageCounter,
                shadow,
                new Sort({}),
                stateMachine
            ],
        }).extend({
            setPositionX: function (value) {
                spawnedX = value;
                entity.position.x = value;
                setBagPosition();
            },
            getPositionX: function () {
                return spawnedX;
            },
            addGrabBehavior: function () {
                entity.attach(grabable);
                entity.attach(grabBehavior);

                return grabable;
            }
        });

        return entity;
    };
});