/**
 * Regulates the appearing and disappearing of a speechbubble automatically with or without user input
 * @moduleName RegulatedSpeech
 */
bento.define('entities/regulatedspeech', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'gui/speechballoon',
    'globals'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    SpeechBalloon,
    Globals
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
            touchControl: Boolean // if speechballoon reacts to user input
            blockControlEnd: Boolean // if closing the speechBalloon should not listen to user input
            text: String // text to display on speechballoon
            bottomPosition: Number // Position of the bottom element of the speechballoon
            bottomFlip: Boolean // if the bottom element should be flipped or not
            layer: Number // what layer the speechballoon should be on
            disappear: Boolean // if entity should disappear on automatically
            waitTime: Number // time to wait till removal if disappear is defined, if 0 or Undefined will immediately start removal.
            onUpdate: Function // fires during disappearing update
            onComplete: Function // fires once disappearing is complete
        }*/

        var viewport = Bento.getViewport();

        var waitTime = settings.waitTime ? settings.waitTime : 0;
        var disappear = settings.disappear;

        var removeTween;
        var onUpdate = settings.onUpdate;
        var onComplete = settings.onComplete;

        var complete = false;
        var popInComplete = false;

        var createRemoveTween = function () {
            removeTween = new Tween({
                from: 1,
                to: 0,
                in: 10,
                ease: 'linear',
                onUpdate: function (v, t) {
                    speechBalloon.alpha = v;
                    if (onUpdate) {
                        onUpdate();
                    }
                },
                onComplete: function () {
                    complete = true;
                }
            });
        };

        var behavior = {
            name: 'tutorialBehavior',
            start: function (data) {},
            destroy: function (data) {},
            update: function (data) {
                if (settings.touchControl) {
                    if (speechBalloon.complete && !settings.blockControlEnd) {
                        if (!removeTween) {
                            Bento.audio.stopSound('sfx_button');
                            Bento.audio.playSound('sfx_button');
                            createRemoveTween();
                        }
                    }
                }

                if (disappear) {
                    if (!removeTween) {
                        if (speechBalloon.timer < waitTime) {
                            return;
                        }
                        createRemoveTween();
                    }
                }

                if (complete) {
                    Bento.objects.remove(speechBalloon);
                    if (onComplete) {
                        onComplete();
                    }
                    if (!popInComplete) {
                        Bento.objects.remove(popIn);
                    }
                    speechBalloon.isComplete = true;
                }
            },
            draw: function (data) {}
        };

        var speechBalloon = new SpeechBalloon({
            typewriter: true,
            touchControl: settings.touchControl,
            text: settings.text,
            position: settings.position,
            bottomPosition: settings.bottomPosition,
            bottomFlip: settings.bottomFlip,
            fitToText: settings.fitToText,
            z: settings.layer
        }).extend({
            isComplete: false,
            forceEnd: function () {
                if (!removeTween) {
                    createRemoveTween();
                }
            }
        }).attach(behavior);

        var popIn = new Tween({
            from: 0,
            to: 1,
            in: 20,
            delay: 0,
            ease: 'elastic',
            decay: 5,
            oscillations: 1,
            onUpdate: function (v, t) {
                speechBalloon.scale.x = v;
                speechBalloon.scale.y = v;
            },
            onComplete: function () {
                popInComplete = true;
            }
        });

        return speechBalloon;
    };
});