/**
 * Module description
 * @moduleName Shaker
 */
bento.define('entities/shaker', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
		var tiled = (settings.tiled || {});
        var object = tiled.object || {};
        var nameProperties = Utils.nameToJson(object.name);

        var camera = Bento.objects.get('camera');
        var shakePosition = new Vector2(0, 0);
        var minAmp = nameProperties.minimum || settings.minimum || 1;
        var maxAmp = nameProperties.maximum || settings.maximum || 3;
        var frequency = nameProperties.time || settings.time || 5;
        var amplitude = Utils.getRandomRange(minAmp, maxAmp);
        var goToAmp = amplitude;
        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {
                if (camera) {
                    camera.offset = shakePosition;
                }
            },
            destroy: function (data) {},
            update: function (data) {
            	// change amplitude
            	if(entity.timer % 20 === 0) {
            		goToAmp = Utils.getRandomRange(minAmp, maxAmp);
            	}
            	amplitude *= 0.9;
            	amplitude += goToAmp * 0.1;

            	// update shakeposition
            	shakePosition.y = Math.sin(entity.timer / frequency) * amplitude * entity.power;
            },
            draw: function (data) {}
        };
        var entity = new Entity({
            z: 0,
            name: 'shaker',
            family: 'shaker',
            position: settings.position,
            updateWhenPaused: 0,
            components: [
                behavior
            ]
        }).extend({
            power: 1
        });
        return entity;
    };
});