/**
 * Door in a level that opens and spawns new enemies
 * @moduleName SpawnDoorAnimated
 */
bento.define('entities/spawners/spawnanimdoor', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'entities/spawner'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Spawner
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var tiled = (settings.tiled || {});
        var object = tiled.object || {};
        var spawnProperties = Utils.nameToJson(object.name);
        var imageName = spawnProperties.door || 'subwaydoor';
        var sprite = new Sprite({
            spriteSheet: 'props/doors/' + imageName + '-close'
        });

        var entity;

        settings.movePosition = function () {
            entity.position.x += Math.floor((48 - sprite.frameWidth) / 2);
        };

        entity = new Spawner(settings)
            .attach(sprite)
            .extend({
                preSpawn: function (canSpawn) {
                    if (!canSpawn) {
                        return;
                    }
                    entity.visible = true;
                    sprite.setSpriteSheet('props/doors/' + imageName + '-open');
                },
                postSpawn: function (canSpawn) {
                    if (!canSpawn) {
                        return;
                    }
                    sprite.setSpriteSheet('props/doors/' + imageName + '-close', function () {
                        entity.visible = false;
                    });
                }
            });
        entity.spawnY = 26;
        entity.visible = false;
        return entity;
    };
});