/**
 * Spawn enemies in the beach levels (Rad event)
 * @moduleName Spawnbeach
 */
bento.define('entities/spawners/spawnbeach', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'entities/spawner',
    'components/sort',
    'managers/ai',
    'modules/matchmanager',
    'entities/effect',
    'managers/save'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Spawner,
    Sort,
    AiManager,
    MatchManager,
    Effect,
    SaveManager
) {
    'use strict';
    return function (settings) {
        var viewport = Bento.getViewport();
        var spawner;
        var tiled = settings.tiled || {};
        var objectHeight = tiled.object.height;
        var enemies = [
            'pam', 'pam', 'pam',
            'shirley', 'shirley', 'shirley',
            'dog', 'karp', 'ratguy', 'twerka', 'gangster', 'punkgirl'
        ];
        var currentMission = SaveManager.load('currentMission');
        if (currentMission.indexOf('beachnight') >= 0) {
            enemies = [
                'pam', 'pam', 'pam',
                'shirley', 'shirley', 'shirley',
                'cat', 'bird', 'pig', 'mrpigeon', 'punker', 'ratgirl'
            ];
        }

        // randomize spawning
        var amount = 3;
        var props = {}; // note that all tiled/json properties are ignored and overwritten!
        var i;
        for (i = 0; i < amount; ++i) {
            // save into properties as strings
            if (Utils.getRandom(2)) {
                props['wave' + i] = 'wave-' + i + ':entities/enemies/' + enemies[Utils.getRandom(enemies.length)];
            }
        }
        settings.tiled.object.properties = props;

        spawner = new Spawner(settings);
        spawner.spawnEnemy = function (enemy) {
            var effect = new Effect({
                position: spawner.position.add(new Vector2(0, -objectHeight)),
                spriteSheet: 'effects/watersplash'
            });
            effect.attach(new Sort({}));
            Bento.objects.attach(effect);
            enemy.position.x = spawner.position.x;
            enemy.position.y = spawner.position.y - objectHeight;
            enemy.getComponent('movable', function (movable) {
                //ignore solids
                movable.settings.ignoreSolids = true;
                movable.velocityZ = -4;
            });

            Bento.objects.attach(enemy);

            enemy.getComponent('enemyController', function (enemyController) {
                if (!enemyController.active) {
                    enemyController.active = spawner.activateEnemy;
                }
            });

            // set jump after attaching enemy to stage otherwise state doesn't exist
            var jump = new Vector2(Utils.getRandomRangeFloat(-0.5, 0.5), 0);

            enemy.getComponent('stateMachine', function (stateMachine) {
                stateMachine.setState('jump', {
                    jumpX: jump.x,
                    jumpY: jump.y
                });
            });
            Utils.timeout(54, function () {
                enemy.getComponent('movable', function (movable) {
                    //jump lasts about 27 frames, stop ignoring solids after 25
                    movable.settings.ignoreSolids = false;
                    if (MatchManager.isOnline) {
                        // bug: when playing online, the enemy has the tendency to freeze here
                        AiManager.distributeAiTasks();
                    }
                });
            });

            // ensure the enemy lands X amount of tiles below
            new Tween({
                from: enemy.position.y,
                to: enemy.position.y + objectHeight,
                in: 54,
                ease: 'linear',
                onUpdate: function (v, t) {
                    enemy.position.y = v;
                }
            });
        };
        return spawner;
    };
});