/**
 * Bush in a level that spawns new enemies
 * @moduleName SpawnBush
 */
bento.define('entities/spawners/spawnbush', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'entities/spawner',
    'components/sort',
    'managers/ai',
    'modules/matchmanager'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Spawner,
    Sort,
    AiManager,
    MatchManager
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();

        var sprite = new Sprite({
            spriteSheet: 'props/bush/bush-medium'
        });

        var entity;
        var spawnEnemy;

        settings.movePosition = function () {
            entity.position.x += Math.floor((80 - sprite.frameWidth) / 2);
        };

        entity = new Spawner(settings)
            .attach(sprite)
            .attach(new Sort({}))
            .extend({
                preSpawn: function (canSpawn) {
                    if (!canSpawn) {
                        return;
                    }
                    // squish bush with tween
                    new Tween({
                        from: 1.5,
                        to: 1,
                        in: 30,
                        ease: 'easeOutBack',
                        onUpdate: function (v, t) {
                            entity.scale.x = v;
                            entity.scale.y = 1 / v;
                        }
                    });
                },
            });
        entity.spawnEnemy = function (enemy) {
            // ignore collisions
            enemy.getComponent('movable', function (movable) {
                movable.settings.ignoreSolids = true;
            });
            // position behind bush
            enemy.position.x = entity.position.x;
            enemy.position.y = entity.position.y + 5;
            enemy.getComponent('movable', function (movable) {
                movable.velocityZ = -2;
                movable.height = -12;
            });
            //attach to stage, otherwise states dont exist
            Bento.objects.attach(enemy);
            // set state to jump
            var jump = new Vector2(entity.randomizer.getRandomRangeFloat(-4, 4), entity.randomizer.getRandomRangeFloat(1, 1.5));
            jump.normalize();
            jump.scalarMultiplyWith(entity.randomizer.getRandomRangeFloat(1.2, 1.6));
            enemy.getComponent('stateMachine', function (stateMachine) {
                stateMachine.setState('jump', {
                    jumpX: jump.x,
                    jumpY: jump.y
                });
            });

            Utils.timeout(32, function () {
                // put back collision families and add velocities
                enemy.getComponent('movable', function (movable) {
                    movable.settings.ignoreSolids = false;
                    if (MatchManager.isOnline) {
                        // bug: when playing online, the enemy has the tendency to freeze here
                        AiManager.distributeAiTasks();
                    }
                });

                enemy.getComponent('enemyController', function (enemyController) {
                    if (!enemyController.active) {
                        enemyController.active = entity.activateEnemy;
                    }
                });
            });
        };
        return entity;
    };
});