/**
 * spawns enemies from above
 * @moduleName SpawnRoof
 */
bento.define('entities/spawners/spawnroof', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'entities/spawner'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Spawner
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var tiled = settings.tiled || {};
        var spawnProperties = Utils.nameToJson(tiled.object.name);
        var spawnHeight = spawnProperties.height || 160;

        var entity;

        settings.movePosition = function () {
            entity.position.x += 18;
        };

        entity = new Spawner(settings);
        entity.spawnEnemy = function (enemy) {
            enemy.position.x = entity.position.x;
            enemy.position.y = entity.position.y;
            enemy.getComponent('movable', function (movable) {
                movable.velocityZ = 3;
                movable.height = -spawnHeight;
            });

            Bento.objects.attach(enemy);

            enemy.getComponent('enemyController', function (enemyController) {
                if (!enemyController.active) {
                    enemyController.active = entity.activateEnemy;
                }
            });

            var shadow = enemy.getComponent('shadow');
            new Tween({
                from: 0,
                to: 1,
                in: 20,
                delay: 0,
                ease: 'easeInQuad',
                onUpdate: function (v, t) {
                    enemy.alpha = v;
                    if (shadow) {
                        shadow.shadow.alpha = v;
                    }
                },
            });

            // set jump after attaching enemy to stage otherwise state doesn't exist
            var jump = new Vector2(entity.randomizer.getRandomRangeFloat(-1.2, 1.2), entity.randomizer.getRandomRangeFloat(-1.2, 1.2));
            jump.normalize();
            jump.scalarMultiplyWith(entity.randomizer.getRandomRangeFloat(0.3, 0.4));
            enemy.getComponent('stateMachine', function (stateMachine) {
                stateMachine.setState('jump', {
                    jumpX: jump.x,
                    jumpY: jump.y
                });
            });
        };
        return entity;
    };
});