/**
 * Door in a level that opens and spawns new enemies
 * @moduleName SpawnSubway
 */
bento.define('entities/spawners/spawnsubway', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'entities/spawner',
    'modules/randomizer',
    'globals',
    'modules/matchmanager',
    'managers/ai'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Spawner,
    Randomizer,
    Globals,
    MatchManager,
    AiManager
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();

        var entity;

        settings.movePosition = function () {
            entity.position.x += 24;
        };

        entity = new Spawner(settings);
        entity.spawnEnemy = function (enemy) {
            // remove sorting
            var sort = enemy.getComponent('sort');
            if (sort) {
                enemy.remove(sort);
            }
            // ignore collisions
            enemy.getComponent('movable', function (movable) {
                movable.settings.ignoreSolids = true;
            });
            // invisible shadow
            var shadow = enemy.getComponent('shadow');
            if (shadow) {
                shadow.shadow.visible = false;
            }
            // put on LAYER ENEMYSPAWNBEHIND
            enemy.z = Globals.Layers.ENEMYSPAWNBEHIND;
            // position out of sight
            enemy.position.x = entity.position.x;
            enemy.position.y = entity.position.y;
            //attach to stage, otherwise states dont exist
            Bento.objects.attach(enemy);
            // sprite jump animation
            enemy.getComponent('stateMachine', function (stateMachine) {
                stateMachine.setState('puppet', {
                    animation: 'jump'
                });
            });
            // tween jump above the platform
            var jump = new Vector2(entity.randomizer.getRandomRangeFloat(-4, 4), entity.randomizer.getRandomRangeFloat(0.4, 1));
            jump.normalize();
            jump.scalarMultiplyWith(entity.randomizer.getRandomRangeFloat(1.2, 1.6));

            enemy.getComponent('multiScale', function (multiScale) {
                if (jump.x !== 0) {
                    multiScale.direction.x = Utils.sign(jump.x);
                }
            });

            new Tween({
                from: 0,
                to: -48,
                in: 20,
                delay: 0,
                ease: 'easeOutCubic',
                onStart: function () {},
                onUpdate: function (v, t) {
                    var xLerp = t / 20;
                    xLerp *= xLerp;
                    enemy.position.x += jump.x * (t / 20);
                    enemy.getComponent('movable', function (movable) {
                        movable.height = v;
                    });
                },
                onComplete: function () {
                    // put back sort
                    if (sort) {
                        enemy.attach(sort);
                    }
                    // visible shadow
                    if (shadow) {
                        shadow.shadow.visible = true;
                        new Tween({
                            from: 0,
                            to: 1,
                            in: 20,
                            delay: 0,
                            ease: 'easeInQuad',
                            onUpdate: function (v, t) {
                                shadow.shadow.alpha = v;
                            },
                        });
                    }
                    enemy.z = Globals.Layers.ENEMIES + enemy.position.y / viewport.height;

                    // put back collision families and add velocities
                    enemy.getComponent('movable', function (movable) {
                        movable.velocityZ = 0;
                        movable.height = -46;
                        movable.settings.ignoreSolids = false;
                        if (MatchManager.isOnline) {
                            // bug: when playing online, the enemy has the tendency to freeze here
                            Utils.timeout(26, function () {
                                AiManager.distributeAiTasks();
                            });
                        }
                    });

                    enemy.getComponent('enemyController', function (enemyController) {
                        if (!enemyController.active) {
                            enemyController.active = entity.activateEnemy;
                        }
                    });

                    // set state to jump
                    enemy.getComponent('stateMachine', function (stateMachine) {
                        stateMachine.setState('jump', {
                            jumpX: jump.x,
                            jumpY: jump.y
                        });
                    });
                }
            });
        };
        return entity;
    };
});