/**
 * Very simple hashgrid to be used for ground tiles
 * @moduleName tileColliders
 */
bento.define('entities/tilecolliders', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/utils',
    'bento/tween'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    Utils,
    Tween
) {
    'use strict';
    var floor = Math.floor;
    return function () {
        var grid = {},
            cellSize = 16,
            invCellSize = 1 / cellSize,
            entity = new Entity({
                z: 0,
                name: 'tileColliders',
                family: ['tileColliders']
            }),
            posToKey = function (x, y) {
                return floor(x * invCellSize).toString() + ':' + floor(y * invCellSize);
            };
        // rectangle to triangle collision
        // where triangle is a top-right half of a square
        var rectangleTriangleCollision = function (rectangle, triangleX, triangleY, gridEntry) {
            // only topright and bottomleft triangles
            // relative x and y
            var x;
            var y;
            var bottomTriangle = gridEntry.isTriangle < 0;
            if (bottomTriangle) {
                x = rectangle.getX2() - triangleX;
                y = rectangle.y - triangleY;
            } else {
                x = rectangle.x - triangleX;
                y = rectangle.getY2() - triangleY;                
            }

            // is this in the topright half?
            if (bottomTriangle ? x < y : x > y) {
                return false;
            } else {
                // also pass collision data
                gridEntry.collisionData = new Vector2(x, y);
                return true;
            }
        };

        // public
        entity.extend({
            insert: function (x, y, triangleDir) {
                var key = posToKey(x, y);
                // insert make a dummy entity for collision purposes
                var dummy = new Entity({
                    name: 'gridEntry',
                    position: new Vector2(x, y)
                });
                var dimension = dummy.dimension;
                dimension.width = cellSize;
                dimension.height = cellSize;

                dummy.isTriangle = triangleDir;

                grid[key] = dummy;
            },
            remove: function (x, y) {
                // clear the entity there
                var key = posToKey(x, y);
                // delete or set to null?
                delete grid[key];
            },
            checkCell: function (position) {
                var key = posToKey(position.x, position.y);
                return grid[key];
            },
            // overwrite original collision function
            collidesWith: function (other, offset, callback) {
                var box = other.isRectangle ? other : other.getBoundingBox();

                var minX;
                var maxX;
                var minY;
                var maxY;
                var x;
                var y;
                var key;
                var gridEntry;
                var hasCollided = false;

                box.x += offset.x;
                box.y += offset.y;

                minX = floor(box.x * invCellSize);
                maxX = floor(box.getX2() * invCellSize);
                minY = floor(box.y * invCellSize);
                maxY = floor(box.getY2() * invCellSize);

                // check all grids
                for (x = minX; x <= maxX; ++x) {
                    for (y = minY; y <= maxY; ++y) {
                        key = x.toString() + ':' + y;
                        gridEntry = grid[key];
                        if (gridEntry) {
                            // check for triangle collision
                            if (gridEntry.isTriangle && !rectangleTriangleCollision(box, x * cellSize, y * cellSize, gridEntry)) {
                                continue;
                            }

                            if (callback) {
                                callback(gridEntry);
                            }
                            hasCollided = true;
                        }
                    }
                }
                return hasCollided;
            }
        });

        return entity;
    };
});