/**
 * Singleton which keeps track of the heights an enemey can spawn at given a specific x coordinate
 * @moduleName EnemySpawnHeight
 */
bento.define('entities/triggers/enemyspawnheight', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween
) {
    'use strict';
    /**
     * Describe your settings object parameters
     * @param {Object} settings
     */

    var areas = [];

    var addArea = function (rect) {
        areas.push({
            border: rect.getX2(),
            top: rect.y,
            bottom: rect.getY2()
        });

        areas.sort(function (a, b) {
            return a.border - b.border;
        });
    };

    var getRangeAt = function (x) {
        var ii;
        var range = {
            min: 196, //NOTE: default values. These should only be returned if no areas were specified
            max: 256
        };

        for (ii = 0; ii < areas.length; ii++) {
            range.min = areas[ii].top;
            range.max = areas[ii].bottom;

            if (areas[ii].border > x) {
                break;
            }
        }

        return range;
    };

    var getRandomYAt = function (x) {
        var range = getRangeAt(x);
        return Utils.getRandomRange(range.min, range.max);
    };

    var manager = {
        addArea: addArea,
        reset: function () {
            areas = [];
        },
        getRangeAt: getRangeAt,
        getRandomYAt: getRandomYAt
    };

    return manager;
});