/**
 * Trigger for leaving an area
 * @moduleName Exit
 */
bento.define('entities/triggers/exit', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'components/trigger',
    'globals',
    'components/debug',
    'gui/fade',
    'bento/components/nineslice',
    'modules/matchmanager',
    'managers/save',
    'components/eventlistener'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Trigger,
    Globals,
    Debug,
    Fade,
    NineSlice,
    MatchManager,
    SaveManager,
    EventListener
) {
    'use strict';
    var Exit = function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2,
            // size of trigger
            rectangle: new Rectangle(),
            // Exit.Direction.UP
            direction: '',
        }*/
        var viewport = Bento.getViewport();
        var rectangle = settings.rectangle || new Rectangle(0, 0, 0, 0);
        var direction = settings.direction || Exit.Direction.UP;
        var trigger = new Trigger({
            onTrigger: function () {
                if (!MatchManager.isMaster && !MatchManager.noSync) {
                    return;
                }
                // exit to new screen
                Exit.lastTrigger = settings;
                Bento.objects.remove(entity);
                new Fade({
                    pause: Globals.PauseLevels.FADE,
                    duration: 10,
                    fadeIn: false,
                    type: 'opacity',
                    onComplete: function () {
                        // next section
                        var section = SaveManager.load('currentSection');
                        section += 1;
                        SaveManager.save('currentSection', section);
                        Bento.screens.show('screens/main');
                    }
                });

                if (!MatchManager.noSync) {
                    MatchManager.sendData({
                        type: 'areaChange'
                    }, 'general');
                }
            }
        });
        var entity = new Entity({
            z: 1,
            name: 'exit',
            family: ['triggers'],
            position: new Vector2(rectangle.x, rectangle.y),
            boundingBox: new Rectangle(0, 0, rectangle.width, rectangle.height),
            components: [
                trigger,
                // new Debug({})
            ]
        });

        entity.attach(new EventListener({
            name: 'startRoom',
            onEvent: function () {
                // trim rectangle
                var currentScreen = Bento.screens.getCurrentScreen();

                // trigger is outside the viewport
                if (MatchManager.otherPlayers.length > 0) {
                    if (entity.position.x >= currentScreen.dimension.width) {
                        console.log('WARNING: exit trigger is out of screen');
                        entity.position.x = currentScreen.dimension.width - 16;
                    }
                }

                if (entity.position.x + rectangle.width > currentScreen.dimension.width && entity.position.x < currentScreen.dimension.width) {
                    rectangle.width = currentScreen.dimension.width - entity.position.x;
                }

                // move any blockade
                entity.collidesWith({
                    family: 'blockade',
                    onCollide: function (other) {
                        other.position.x -= 16;
                        console.log('WARNING: Blockade is moved to the left');
                    }
                });

                if (MatchManager.otherPlayers.length > 0) {
                    //visualize trigger area
                    var areaSlice = new NineSlice({
                        imageName: 'triggerslice',
                        width: rectangle.width,
                        height: rectangle.height
                    });

                    var area = new Entity({
                        z: 1,
                        name: 'areaSlice',
                        position: entity.position,
                        components: [
                            areaSlice
                        ]
                    });

                    Bento.objects.attach(area);
                }

            }
        }));

        return entity;
    };

    // refer to this when reloading map
    Exit.lastTrigger = null;

    // direction enum
    Exit.Direction = {
        UP: 'up',
        DOWN: 'down',
        LEFT: 'left',
        RIGHT: 'right',
        // alias used in tiled
        'top': 'up',
        'bottom': 'down',
        'up': 'up',
        'down': 'down',
        'left': 'left',
        'right': 'right'
    };

    return Exit;
});