/**
 * Finish trigger
 * @modulename Finish
 */
bento.define('entities/triggers/finish', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'components/trigger',
    'modules/matchmanager',
    'globals',
    'bento/components/nineslice',
    'components/debug',
    'gui/menus/levelend'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Trigger,
    MatchManager,
    Globals,
    NineSlice,
    Debug,
    LevelEndMenu
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var rectangle = settings.rectangle || new Rectangle(0, 0, 0, 0);
        var trigger = new Trigger({
            onTrigger: function () {
                if (!MatchManager.isMaster) {
                    return;
                }
                EventSystem.fire('disableControls');
                Utils.timeout(30, function () {
                    Bento.objects.attach(new LevelEndMenu({}));
                });
                Bento.objects.remove(entity);
            }
        });
        var entity = new Entity({
            z: 1,
            name: 'exit',
            family: ['triggers'],
            position: new Vector2(rectangle.x, rectangle.y),
            boundingBox: new Rectangle(0, 0, rectangle.width, rectangle.height),
            components: [
                trigger,
                // new Debug({})
            ]
        });

        if (MatchManager.otherPlayers.length > 1) {
            //visualize trigger area
            var areaSlice = new NineSlice({
                imageName: 'triggerslice',
                width: rectangle.width,
                height: rectangle.height
            });

            var area = new Entity({
                z: 1,
                name: 'areaSlice',
                position: entity.position,
                components: [
                    areaSlice
                ]
            });

            Bento.objects.attach(area);
        }
        return entity;
    };
});