/**
 * Show a gesture
 * @moduleName Gesture
 */
bento.define('entities/tutorial/gesture', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'globals',
    'components/anchor',
    'gui/shadowtext',
    'modules/localization',
    'components/eventlistener',
    'gui/senseipopup',
    'gui/highlight'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Globals,
    Anchor,
    ShadowText,
    Localization,
    EventListener,
    SenseiPopup,
    Highlight
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var type = settings.type || 'dash';
        var sprite = new Sprite({});
        var text = new ShadowText({
            position: new Vector2(0, -40),
            text: '',
            font: 'font',
            fontSize: 16,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            maxWidth: 140,
            maxHeight: 32
        });
        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {
                if (type === 'scrolls/dash') {
                    behavior.flickntap();
                } else if (type === 'scrolls/charge') {
                    behavior.hold();
                } else if (type === 'scrolls/grab') {
                    behavior.grab();
                } else if (type === 'scrolls/jump') {
                    behavior.flickup();
                }
            },
            destroy: function (data) {},
            update: function (data) {},
            draw: function (data) {},
            // show flick
            flick: function () {
                sprite.setSpriteSheet('gui/hand-gesture-flick');
                text.setText(Localization.getText('gestures')['dashing']);

                // wait for gesture to show flick and tap
                entity.attach(new EventListener({
                    name: 'playerDash',
                    onEvent: function () {
                        // remove self
                        entity.removeByName('eventListener');

                        behavior.flickntap();
                    }
                }));
            },
            // show flick and tap
            flickntap: function () {
                sprite.setSpriteSheet('gui/hand-gesture-flickntap');
                text.setText(Localization.getText('gestures')['dashAttack']);

                // done
                entity.attach(new EventListener({
                    name: 'playerDashAttack',
                    onEvent: function () {
                        // remove self
                        entity.removeSelf();
                    }
                }));
            },
            // show flick up
            flickup: function () {
                sprite.setSpriteSheet('gui/hand-gesture-flickup');
                text.setText(Localization.getText('gestures')['jumpAttack']);

                // done
                entity.attach(new EventListener({
                    name: 'playerJumpAttack',
                    onEvent: function () {
                        // remove self
                        entity.removeSelf();

                        // also show that it uses combo points
                        Utils.timeout(20, function () {
                            var highlight = new Highlight({
                                z: Globals.Layers.GUI + 1,
                                radius: 20,
                                alpha: 0.8,
                                position: new Vector2(viewport.x + 80, viewport.y + 26)
                            });
                            highlight.float = true;

                            Bento.objects.attach(new SenseiPopup({
                                z: Globals.Layers.GUI + 2,
                                modal: true,
                                onNoMoreText: function () {
                                    this.dismiss();
                                    new Tween({
                                        from: 1,
                                        to: 0,
                                        in: 30,
                                        onUpdate: function (v, t) {
                                            highlight.alpha = v;
                                        },
                                        onComplete: function () {
                                            highlight.removeSelf();
                                        }
                                    });
                                },
                                onPageEnd: function (index) {
                                    if (index === 1) {
                                        // highlight combometer
                                        Bento.objects.attach(highlight);
                                    }
                                },
                                text: Localization.getText('tutorial')['comboPoints']
                            }));
                            Bento.input.resetPointers();
                        });
                    }
                }));
            },
            // hold and then release
            hold: function () {
                sprite.setSpriteSheet('gui/hand-gesture-hold');
                text.setText(Localization.getText('gestures')['holdSpecial']);
                // Utils.timeout(120, behavior.release);

                // wait for gesture to show release
                entity.attach(new EventListener({
                    name: 'playerCharge',
                    onEvent: function () {
                        // remove self
                        entity.removeByName('eventListener');

                        behavior.release();
                    }
                }));
            },
            release: function () {
                sprite.setSpriteSheet('gui/hand-gesture-release');
                text.setText(Localization.getText('gestures')['releaseSpecial']);
                Utils.timeout(60, behavior.hold);

                // done
                entity.attach(new EventListener({
                    name: 'playerChargeAttack',
                    onEvent: function () {
                        // remove self
                        entity.removeSelf();
                    }
                }));
            },
            // grabbing enemies
            grab: function () {
                sprite.setSpriteSheet('gui/hand-gesture-drag');
                text.setText(Localization.getText('gestures')['grab']);
                // Utils.timeout(120, behavior.grabAttack);

                // wait for gesture to show release
                entity.attach(new EventListener({
                    name: 'playerGrab',
                    onEvent: function () {
                        // remove self
                        entity.removeByName('eventListener');

                        behavior.grabAttack();
                    }
                }));
            },
            grabAttack: function () {
                sprite.setSpriteSheet('gui/hand-gesture-tap');
                sprite.currentAnimation.backTo = 0;
                text.setText(Localization.getText('gestures')['attack']);
                // Utils.timeout(180, behavior.grabThrow);

                // done
                entity.attach(new EventListener({
                    name: 'playerGrabAttack',
                    onEvent: function () {
                        // remove self
                        entity.removeByName('eventListener');

                        behavior.grabThrow();
                    }
                }));
            },
            grabThrow: function () {
                sprite.setSpriteSheet('gui/hand-gesture-flick');
                text.setText(Localization.getText('gestures')['grabThrow']);
                // consider the tutorial finished
                Utils.timeout(180, function () {
                    entity.removeSelf();
                });
            }
        };
        var anchor = new Anchor({
            portrait: {
                screenPosition: new Vector2(0.5, 1),
                offset: new Vector2(0, -40)
            },
            landscape: {
                screenPosition: new Vector2(1, 0.5),
                offset: new Vector2(-70, 0)
            }
        });
        var entity = new Entity({
            z: Globals.Layers.GUI,
            name: 'gesture',
            position: settings.position,
            float: true,
            components: [
                sprite,
                text,
                behavior,
                anchor
            ]
        });
        return entity;
    };
});