/**
 * Hitcounter based of combo counter for the tutorial
 * @moduleName HitCount
 */
bento.define('entities/tutorial/hitcount', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'modules/localization',
    'components/leftshadow',
    'globals'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Localization,
    LeftShadow,
    Globals
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();

        var counter = new Text({
            text: '12',
            gradient: 'top-down',
            gradientColors: ['#35c4ff', '#35c4ff', '#FFFFFF', '#FFFFFF'],
            font: 'font',
            fontSize: 24,
            fontColor: '#ffffff',
            align: 'left',
            textBaseline: 'middle',
            ySpacing: 0,
            lineWidth: 1,
            strokeStyle: '#160d32',
            innerStroke: false,
            pixelStroke: true // workaround for Cocoon bug
        }).attach(new LeftShadow({}));

        var combo = new Text({
            text: '/ ' + (settings.maxHits || 5) + ' hits',
            gradient: 'top-down',
            gradientColors: ['#35c4ff', '#35c4ff', '#FFFFFF', '#FFFFFF'],
            font: 'font',
            fontSize: 16,
            fontColor: '#ffffff',
            align: 'left',
            textBaseline: 'middle',
            ySpacing: 0,
            lineWidth: 1,
            strokeStyle: '#160d32',
            innerStroke: false,
            pixelStroke: true // workaround for Cocoon bug
        }).attach(new LeftShadow({}));

        var setHitCount = function (hits) {
            //change counter text to new value
            counter.setText(hits.value);
            counter.getComponent('leftShadow').refresh();

            //add scale tween for the counter
            new Tween({
                from: 2,
                to: 1,
                'in': 10,
                ease: Tween.EASEINBACK,
                onUpdate: function (v, t) {
                    counter.scale.x = v;
                    counter.scale.y = v;
                }
            });

            if (!hits.reset) {
                //if entity not visible yet scroll into the screen from the left
                if (!entity.visible) {
                    new Tween({
                        from: -64,
                        to: settings.position.x,
                        'in': 30,
                        ease: 'elastic',
                        alpha: -6,
                        beta: 1,
                        onUpdate: function (v, t) {
                            entity.position.x = v;
                        }
                    });
                    entity.visible = true;
                }
            } else {
                //fade out tween
                new Tween({
                    from: 1,
                    to: 0,
                    in: 60,
                    ease: 'linear',
                    onUpdate: function (v, t) {
                        entity.alpha = v;
                    },
                    onComplete: function () {
                        entity.visible = false;
                        entity.alpha = 1;
                    }
                });
            }
        };

        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {
                entity.visible = false;
            },
            update: function (data) {
                //set position of the combo text to match the counter text size
                combo.position.x = counter.position.x + (counter.dimension.width - 2) * counter.scale.x;
            },
            draw: function (data) {}
        };
        var entity = new Entity({
            z: Globals.Layers.GUI,
            name: 'comboGui',
            position: settings.position || new Vector2(0, 0),
            float: true,
            components: [
                behavior,
                combo,
                counter
            ]
        }).extend({
            setHitCount: setHitCount
        });
        return entity;
    };
});