/**
 * punchingbag off the chain
 * @moduleName PunchingBagDeath
 */
bento.define('entities/tutorial/punchingbagdeath', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'components/sort',
    'components/movable',
    'components/shadow',
    'modules/matchmanager'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Sort,
    Movable,
    Shadow,
    MatchManager
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var sprite = new Sprite({
            imageName: 'tutorial/punchingbagdeath',
            originRelative: new Vector2(0.5, 0.5)
        });
        var sort = new Sort({});
        var rotationSpeed = Utils.toRadian(3);
        var movable = new Movable({
            gravity: 0.15,
            bounceZ: 0.5,
            frictionZ: 0.5,
            onLanded: function () {
                rotationSpeed /= 2;
            },
            boundingBox: new Rectangle(-8, -8, 16, 16)
        });
        var shadow = new Shadow({
            imageName: 'shadow16'
        });
        var player;
        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {
                movable.velocityZ = -4;
                movable.velocity.x = 2 * settings.direction;
                player = Bento.objects.getByFamily('player')[0];
                // console.log(playerId, player);
                // move up half the dimension height because of origin difference
                entity.position.y -= entity.dimension.height / 2;

                new Tween({
                    from: 1,
                    to: 0,
                    in: 60,
                    delay: 50,
                    ease: 'linear',
                    onStart: function () {},
                    onUpdate: function (v, t) {
                        entity.alpha = v;
                    },
                    onComplete: function () {
                        Bento.objects.remove(entity);
                    }
                });
            },
            destroy: function (data) {},
            update: function (data) {
                entity.rotation += rotationSpeed * data.speed;
            },
            draw: function (data) {}
        };
        var entity = new Entity({
            z: 0,
            name: 'punchingbagDeath',
            position: settings.position,
            updateWhenPaused: 0,
            components: [
                sprite,
                behavior,
                movable,
                sort,
                shadow
            ]
        });
        return entity;
    };
});