/**
 * Weapon
 * @moduleName Weapon
 */
bento.define('entities/weapon', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'components/sort',
    'components/debug',
    'components/weapon',
    'modules/matchmanager',
    'components/punchsyncer'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Sort,
    Debug,
    Weapon,
    MatchManager,
    PunchSyncer
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var tiled = settings.tiled || {};
        var tileProperties = tiled.tileProperties || {};
        var type = settings.type || tileProperties.type;
        var initialDurability = settings.durability;
        var json = Bento.assets.getJson('weapons/' + type);
        var boundingBox = new Rectangle(
            json.boundingBox.x,
            json.boundingBox.y,
            json.boundingBox.width,
            json.boundingBox.height
        );
        var weaponComponent;
        var initWeaponComponent = function () {
            weaponComponent = new Weapon(json);

            // overwrite durability
            if(initialDurability) {
                weaponComponent.setDurability(initialDurability);
            }
            return weaponComponent;
        };
        var sprite = new Sprite({
            spriteSheet: json.passiveSprite
        });
        var sort = new Sort({});
        var entity = new Entity({
            name: 'weapon',
            family: ['weapons'],
            boundingBox: boundingBox,
            components: [
                sprite,
                sort,
                initWeaponComponent(),
                // new Debug({})
            ]
        }).extend({
            throwingForce: json.throw.force,
            damage: json.hit.damage,
            stamina: json.hit.stamina,
            range: json.hit.range,
            onHit: function () {
                weaponComponent.removeDurability(json.hit.durabilityCost);
            },
            drop: function () {
                weaponComponent.drop(0);
            },
            throw: function (direction) {
                weaponComponent.drop(json.throw.force * direction);
            },
            getType: function () {
                return type;
            }
        });

        if (MatchManager.isOnline) {
            entity.attach(new PunchSyncer({}));
        }

        return entity;
    };
});