/**
 * Global values
 * @moduleName Globals
 */
bento.define('globals', [
    'bento',
    'bento/utils'
], function (
    Bento,
    Utils
) {
    'use strict';
    var Globals;
    var missionsJson;

    var statFormula = function (a, b, c, d, level) {
        return Math.round(a + b * level + c * Math.exp(d * level));
    };

    // for debugging purposes
    if (Utils.isDev()) {
        window.setDebug = function (name) {
            bento.require([name], function (Module) {
                Module.debug = !Module.debug;
            });
        };
    }

    Globals = {
        /**
         * Flags
         */
        BETA_TEST: false,
        /**
         * Cache if user is live broadcasting
         */
        isBroadcasting: false,
        isRecording: false,
        /**
         * Cached room code invitiation
         */
        roomCodeInvite: null,
        /**
         * Cached music to change volume
         */
        currentMusic: '',
        /**
         * Cached death sound so that you dont hear multiple at the same time
         */
        deathSound: 'male/sfx_death1',
        /**
         * Amount of available death sounds per sfxBase
         */
        availableDeathSounds: {
            'female/': 5,
            'male/': 7
        },
        /**
         * Combo points
         */

        maxComboPoints: 60,
        /**
         * Combo points for each attack
         */
        attackComboPoints: {
            boy: {
                dash: 2,
                jump: 4,
                charge: 5
            },
            krillin: {
                dash: 2,
                jump: 5,
                charge: 5
            },
            boxer: {
                dash: 5,
                jump: 5,
                charge: 5
            },
            fitgirl: {
                dash: 2,
                jump: 4,
                charge: 5
            },
            jun: {
                dash: 2,
                jump: 4,
                charge: 6
            },
            val: {
                dash: 2,
                jump: 4,
                charge: 15
            },
            pat: {
                dash: 2,
                jump: 5,
                charge: 5
            },
            oni: {
                dash: 3,
                jump: 4,
                charge: 8
            },
            rad: {
                dash: 5,
                jump: 5,
                charge: 8
            },
        },
        /**
         * Conversion of gems to coins
         */
        gemToCoins: [{
            gems: 30,
            coins: 360
        }, {
            gems: 500,
            coins: 6600 // value 10%
        }, {
            gems: 1000,
            coins: 13200 // value 25%
        }],
        /**
         * Table of level-token
         */
        tokenCosts: {
            '1': 2,
            '2': 3,
            '3': 3,
            '4': 3,
            '5': 3,
            '6': 3,
            '7': 3,
            '8': 3,
            '9': 3,
            '10': 3,
            '11': 3,
            '12': 3,
            '13': 3,
            '14': 3,
            '15': 3,
            '16': 3,
            '17': 3,
            '18': 3,
            '19': 3,
            '20': 5,
            '21': 5,
            '22': 5,
            '23': 5,
            '24': 5,
            '25': 5,
            '26': 5,
            '27': 5,
            '28': 5,
            '29': 5,
            '30': 7,
            '31': 7,
            '32': 7,
            '33': 7,
            '34': 7,
            '35': 7,
            '36': 7,
            '37': 7,
            '38': 7,
            '39': 7,
            '40': 7,
            '41': 7,
            '42': 7,
            '43': 7,
            '44': 7,
            '45': 7,
            '46': 7,
            '47': 7,
            '48': 7,
            '49': 7,
            '50': 7
        },
        /**
         * Total amount of backups allowed per character
         */
        totalBackups: 2,
        hoursUntilBackup: 6,
        /**
         * Jump in enemy levels between chosen difficulties
         */
        difficultyJump: 3,
        /**
         * Jump in enemy reaction between chosen difficulties
         */
        difficulty: {
            '0': 0,
            '1': 0.3,
            '2': 0.55
        },
        /**
         * Tower Challenge sections
         */
        towerSections: 15,
        /**
         * Tower Challenge Boss sections
         */
        towerBossSections: 2,
        /**
         * Gacha floors
         */
        towerGachaFloors: [2, 4, 6, 8, 12, 16, 20, 24],
        /**
         * Information regarding multiplayer, contains a seed used by tower challenge
         */
        towerInfo: {
            towerDay: 0,
            seed: 0,
            receivedData: true,
        },
        /**
         * Max amount of tickets
         */
        maxTickets: 5,
        /**
         * Minutes to regenerate ticket
         */
        minutesForTicket: 30,
        /**
         * Conversion of gems to ticket
         */
        gemToTickets: 16,

        /**
         * Conversion of gems to energy
         */
        gemToEnergy: 80,
        /**
         * Conversion of gems to revive
         */
        gemToRevive: function (mission) {
            // for every 5 missions 1 extra, up to 15
            // return Math.min(Math.ceil(mission / 5), 15);
            var costs = [
                [0, 10], // [minimumMission, cost]
                [11, 10],
                [21, 20],
                [31, 40],
                [41, 60],
                [51, 80],
                [61, 100]
            ];
            var cost;
            Utils.forEach(costs, function (data, i, l, breakLoop) {
                if (data[0] < mission) {
                    cost = data[1];
                }
            });
            // note: mission may be a string
            if (mission === 'tower') {
                cost = 80;
            }
            return cost || 10;
        },
        /**
         * Conversion of gems to refresh reward
         */
        gemToRefreshReward: 10,
        /**
         * Coins needed to level up, beyond this array, a formula is needed
         */
        levelUpCosts: [
            100, // lv 1
            150,
            200,
            250,
            300
        ],
        /**
         * Gem amount for daily gems
         */
        dailyGemsOfferAmount: 30,
        /**
         * Types of chests, how many gems they cost and what they will give
         */
        chests: {
            // note: Also see screens/main for the evolution of the classic chest
            classic: {
                cost: 3,
                tokens: 3,
                consumables: 1,
                enemytokens: 1,
                evolve: 1,
                coins: 0
            },
            special: {
                cost: 900,
                tokens: 30,
                consumables: 6,
                enemytokens: 5,
                evolve: 20,
                coins: 5000
            },
            mythic: {
                cost: 2500,
                tokens: 90,
                consumables: 6,
                enemytokens: 10,
                evolve: 60,
                coins: 12000
            }
        },
        // max amount of items to buy in the shop
        maxShopItems: 5,
        // max ads in free gift
        maxAdsFreeGift: 4,
        // max ads for daily shop refresh
        maxAdsDailyShop: 0,
        gemToRefreshShop: 5,
        // max ads for energy
        maxAdsEnergy: 2,
        // max ads for coins (there is also a timer running, see videoadtimer)
        maxAdsCoins: 1,
        // points per reputation rank
        pointsPerRank: 5,
        /**
         * Get coin amount for rewarded ads, based on progression
         * Returns an array for ad stacking
         */
        getCoinsForAd: function (unlockedMissionsCount) {
            var rewards = [
                [0, [50, 75, 100]], // [minimumMissions, coins]
                [21, [75, 125, 150]],
                [31, [125, 150, 175]],
                [51, [150, 250, 400]],
                [71, [200, 300, 500]]
            ];
            var coinArray = [50, 75, 100];
            Utils.forEach(rewards, function (data, i, l, breakLoop) {
                if (data[0] < unlockedMissionsCount) {
                    coinArray = data[1];
                }
            });
            return coinArray;
        },
        /**
         * Get coin amount for free gift
         */
        getFreeGiftCoins: function (unlockedMissionsCount) {
            var rewards = [
                [0, 200], // [minimumMissions, coins]
                [21, 300],
                [31, 400],
                [41, 500]
            ];
            var coins = 200;
            Utils.forEach(rewards, function (data, i, l, breakLoop) {
                if (data[0] < unlockedMissionsCount) {
                    coins = data[1];
                }
            });
            return coins;
        },
        /**
         * Get amount of coins needed to level up per level
         */
        getLevelUpCost: function (level, diff) {
            // diff is an optional parameter, set >1 to check tokencost to upgrade several levels
            diff = diff || 1;

            // get from predetermined list, beyond that use a formula
            var lv = level - 1;
            var cost = 125 + lv * 75;

            if (diff > 1) {
                // add the extra amount for the diff by calling self recursively
                cost += Globals.getLevelUpCost(level + 1, diff - 1);
            }

            return cost;
        },
        /**
         * Get amount of coins for (quick) loot
         */
        getExtraLootCoins: function (mission) {
            if (!missionsJson) {
                missionsJson = Bento.assets.getJson('missions/missions');
            }
            var enemyLevel = missionsJson[mission].enemyLevel;
            var levelUpCost = Globals.getLevelUpCost(enemyLevel);
            var min = 100;
            var coins = Math.round(levelUpCost / 6 / 10) * 10;
            return Math.max(min, coins);
        },
        /**
         * Get amount of tokens needed to level up per level
         */
        getTokenCost: function (level, diff) {
            // diff is an optional parameter, set >1 to check tokencost to upgrade several levels
            diff = diff || 1;

            var lv = level;
            var tokens = Globals.tokenCosts[lv];
            var result;
            // extrapolate from lv30
            if (lv > 30) {
                tokens = Globals.tokenCosts['50'];
            }

            result = tokens || Globals.tokenCosts['50'];

            if (diff > 1) {
                // add the extra amount for the diff by calling self recursively
                result += Globals.getTokenCost(level + 1, diff - 1);
            }

            return result;
        },
        /**
         * Get stats from character per level
         */
        getCharacterStatsFromLevel: function (character, level) {
            var baseStats = Bento.assets.getJson('stats/' + character);
            var lv = level - 1; // start with 0 for calculation
            var stats = {
                health: 0,
                attack: 0,
                skill: 0
            };
            if (!baseStats) {
                Utils.log('can\'t find base stats for character ' + character);
                return stats;
            }

            stats.health = statFormula(20, 15, 25, 0.1, lv);
            stats.attack = statFormula(2, 1.5, 2, 0.1, lv);
            stats.skill = baseStats.skill + lv; // sqrt(3)

            return stats;
        },
        /**
         * Get stats from enemy per level
         * The additions object will be added to each stats
         */
        getDefaultEnemyStats: function (additions) {
            var level = additions.level;
            var lv = level - 1; // start with 0 for calculation
            var stats = {
                "level": level,
                "health": statFormula(10, 15, 20, 0.1, lv),
                "attack": statFormula(0, 1, 2, 0.1, lv),
                "skill": 1 + lv, // Math.sqrt(2)
                "coins": Math.round(1 + Math.sqrt(1 + Math.floor(lv / 5) * 5)),
                "stamina": 0,
                "minHits": 0
            };
            Utils.forEach(additions, function (value, key) {
                stats[key] = stats[key] || 0;
                stats[key] += value;
            });

            // overwrite level, don't add
            stats.level = level;

            return stats;
        },
        // retrieve title language code, defined in preloader.js after Localization is initialized
        getTitleLanguage: function () {
            return 'en';
        },
        /**
         * Layer constants
         */
        WorldSelectLayers: {
            BACKGROUND: 0,
            LEVELS: 100,
            GUI: 400,
            TOPBAR: 400,
            // bottom bar
            SHOP: 420,
            QUESTS: 430,
            IAP: 449,
            BOTTOMBAR: 450,
            //
            DAILYCHEST: 500,
            BUYCOINSPOPUP: 500,
            TUTORIAL: 500,
            INVENTORY: 600,
            LEVELDETAILS: 600,
            GIFTS: 610,
            SETTINGS: 620,
            UPGRADE: 640,
            IAPPOPUP: 650,
            CHESTPOPUP: 660,
            MULTIPLAYER: 660,
            TOWERCHALLENGE: 399,

            EQUIPMENT: 670,
            ADSTACKING: 700,
            CHESTOVERLAY: 750,
            SENSEI: 800,
            CHEAT: 800,
            DIALOGS: 900
        },
        Layers: {
            CAMERA: -10,
            BACKGROUND: -3,
            SUBWAY: -2,
            ENEMYSPAWNBEHIND: -1,
            ENEMIES: 100,
            PLAYER: 200,
            EFFECTS: 300,
            FOREGROUND: 400,
            RAIN: 410,
            DAMAGE_FLOATERS: 450,
            GUI: 500,
            COMBOORB: 505,
            SUPERATTACK: 506,
            SUPERATTACKBAR: 508,
            BOSSINTRO: 510,
            PAUSE: 600,
            SHOP: 650,
            VICTORY: 650,
            FADE: 700,
            DIALOGS: 800,
            NOTIFICATIONS: 900,
            SYSTEM: 1000
        },
        PauseLevels: {
            NONE: 0,
            // WorldSelect Bottombar
            LOADOUT: 1,
            BOTTOMBAR: 9,
            // worldselect
            LEVELDETAILS: 10,
            UPGRADE: 10,
            IAP: 20,
            GIFTS: 20,
            TUTORIAL: 30,
            // game
            FREEZE: 10,
            SCROLL: 20,
            FADE: 20,
            BOSSINTRO: 30,
            STARTSYNC: 40,
            PAUSE: 50,
            ALWAYS: 60,
            //
            SYSTEM: 70
        }
    };
    return Globals;
});