/**
 * Ask user to watch another ad for more coins
 * @moduleName AdStackingCoins
 */
bento.define('gui/adstackingcoins', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'gui/headerbox',
    'modules/localization',
    'components/modal',
    'gui/blurbackground',
    'globals',
    'components/anchor',
    'gui/shadowtext',
    'gui/iconbutton',
    'managers/save',
    'gui/watchrewardedad',
    'managers/currency',
    'gui/coinreward',
    'modules/analytics',
    'components/backbutton'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    HeaderBox,
    Localization,
    Modal,
    BlurBackground,
    Globals,
    Anchor,
    ShadowText,
    IconButton,
    SaveManager,
    WatchRewardedAd,
    CurrencyManager,
    CoinReward,
    Analytics,
    BackButton
) {
    'use strict';
    var AdStackingCoins = function (settings) {
        var viewport = Bento.getViewport();
        var blurBackground = new BlurBackground({
            darken: 0.5,
            blurriness: 15
        });
        var watched = settings.watched || 1;
        var rewardAmount = Globals.getCoinsForAd(SaveManager.load('unlockedLevels').length)[watched];
        /**
         * Box
         */
        var text = new Text({
            position: new Vector2(0, -56),
            text: Localization.getText('anotherAd'),
            font: 'font',
            fontSize: 20,
            fontColor: '#fffd00',
            align: 'center',
            textBaseline: 'top',
            maxWidth: 140,
            maxHeight: 40,
            linebreaks: true,
            components: []
        });
        var coinText = new ShadowText({
            position: new Vector2(8, 0),
            text: '×' + rewardAmount.toString(),
            font: 'font',
            fontSize: 16,
            fontColor: '#ffffff',
            align: 'left',
            textBaseline: 'middle',
        });
        var rewardIcon = new Entity({
            name: 'reward',
            position: new Vector2(0, 8),
            components: [
                coinText,
                new Entity({
                    name: 'coin',
                    position: new Vector2(-16, 16),
                    components: [
                        new Sprite({
                            spriteSheet: 'coin/sack',
                            originRelative: new Vector2(0.5, 0.5)
                        })
                    ]
                })
            ]
        });
        var buttonClose = new IconButton({
            name: 'buttonClose',
            sfx: 'sfx_button',
            imageName: 'gui2/misc/button_small_secondary',
            iconName: 'gui2/misc/button_icon_back',
            position: new Vector2(-52, 64),
            frameCountX: 1,
            frameCountY: 3,
            onClick: function () {
                // tween out
                buttonClose.setActive(false);
                button.setActive(false);
                new Tween({
                    from: 1,
                    to: 0,
                    in: 10,
                    ease: 'linear',
                    onUpdate: function (v, t) {
                        blurBackground.alpha = v;
                        headerBox.scale.x = v;
                        headerBox.scale.y = v;
                    },
                    onComplete: function () {
                        entity.removeSelf();
                        Analytics.event('Ads', 'fraction_ad_stack_' + watched, 0);
                    }
                });
            },
            components: [
                new BackButton({
                    removeSelf: true,
                    onPressed: function (data) {
                        data.entity.doCallback();
                    }
                })
            ]
        });
        var button = new IconButton({
            z: 0,
            name: 'item',
            position: new Vector2(24, 64),
            sfx: 'sfx_button',
            iconName: 'gui2/misc/button_icon_watchadlarge',
            imageName: 'gui2/misc/button_large_primary_excited',
            frameCountX: 6,
            frameCountY: 7,
            padding: 1,
            animations: {
                up: {
                    speed: 0.34,
                    frames: [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35]
                },
                down: {
                    frames: [37]
                },
                inactive: {
                    frames: [36]
                }
            },
            float: false,
            updateWhenPaused: 0,
            onClick: function () {
                // watch ad
                WatchRewardedAd(function (completed) {
                    var coinReward;
                    var coins = rewardAmount;

                    if (completed) {
                        CurrencyManager.addCoins(coins, true);                    
                        // remove self
                        entity.removeSelf();
                        // show reward
                        coinReward = new CoinReward({
                            z: entity.z + 1,
                            coins: coins,
                            onComplete: function () {
                                // ad stacking
                                if (watched < 2) {
                                    Bento.objects.attach(new AdStackingCoins({
                                        watched: watched + 1
                                    }));
                                }
                            }
                        });
                        Bento.objects.attach(coinReward);
                        Analytics.event('Ads', 'watch_for_coins_x' + watched);
                        Analytics.event('Ads', 'fraction_ad_stack_' + watched, 1);
                    }
                });
            }
        });
        var headerBox = new HeaderBox({
            name: 'box',
            position: new Vector2(0, 0),
            size: new Rectangle(0, 0, 180, 160),
            windowImage: 'gui2/misc/splash_background_9slice',
            components: [
                new Anchor({
                    screenPosition: new Vector2(0.5, 0.5),
                    offset: new Vector2(0, 0),
                    onComplete: function () {}
                }),
                text,
                rewardIcon,
                buttonClose,
                button
            ]
        });
        /**
         * Other
         */
        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {
                // tween in
                new Tween({
                    from: 0,
                    to: 1,
                    in: 20,
                    ease: 'linear',
                    onUpdate: function (v, t) {
                        blurBackground.alpha = v;
                    }
                });
                new Tween({
                    from: 0,
                    to: 1,
                    in: 30,
                    delay: 0,
                    ease: 'elastic',
                    decay: 8, // for 'exponential' and 'elastic'
                    oscillations: 1, // for 'elastic', 'sin' and 'cos'
                    onUpdate: function (v, t) {
                        headerBox.scale.x = v;
                        headerBox.scale.y = v;
                    }
                });
            },
            destroy: function (data) {},
            update: function (data) {},
            draw: function (data) {}
        };
        var modal = new Modal({});
        /**
         * Main entity
         */
        var entity = new Entity({
            z: Globals.WorldSelectLayers.ADSTACKING,
            name: 'adStacking',
            float: true,
            components: [
                modal,
                blurBackground,
                headerBox,
                behavior
            ]
        });
        return entity;
    };
    return AdStackingCoins;
});