/**
 * Scrolling tiled background
 * @moduleName BackScroller
 */
bento.define('gui/backscroller', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'components/anchor'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Anchor
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
            imageName: String,
            spacing: Vector2
        }*/
        var viewport = Bento.getViewport();
        var image = Bento.assets.getImage(settings.imageName);
        var padding = settings.padding || 0;
        var behavior = {
            name: 'scrollComponent',
            offset: new Vector2(0, 0),
            fillSize: settings.fillSize || viewport.clone(),
            velocity: settings.velocity || new Vector2(0.5, -0.5),
            spacing: settings.spacing || new Vector2(0, 0),
            countX: 0,
            countY: 0,
            updateCount: function () {
                if (!image) {
                    return;
                }
                // count x and y
                behavior.countX = this.fillSize.width / (image.width + this.spacing.x);
                behavior.countY = this.fillSize.height / (image.height + this.spacing.y);
            },
            start: function (data) {
                // extra padding
                behavior.fillSize = settings.fillSize || viewport.clone();
                behavior.fillSize.grow(padding);
                behavior.updateCount();
            },
            update: function (data) {
                behavior.offset.x += behavior.velocity.x;
                behavior.offset.y += behavior.velocity.y;
            },
            draw: function (data) {
                if (!image) {
                    return;
                }
                if (entity.alpha !== 1 && settings.spacing.x === -1 && this.spacing.x === -1) {
                    // temporarily reset spacing
                    this.spacing.x = 0;
                    this.spacing.y = 0;
                } else if (entity.alpha === 1 && this.spacing.x !== settings.spacing.x) {
                    this.spacing.x = settings.spacing.x;
                    this.spacing.y = settings.spacing.x;
                }
                var i, j;
                var countX = behavior.countX;
                var countY = behavior.countY;
                var effWidth = (image.width + this.spacing.x);
                var effHeight = (image.height + this.spacing.y);
                var renderer = data.renderer;
                // clamp offset
                var offset = new Vector2(0, 0);
                offset.x = Utils.modulo(behavior.offset.x, effWidth);
                offset.y = Utils.modulo(behavior.offset.y, effHeight);

                for (i = -1; i < countX; ++i) {
                    for (j = -1; j < countY; ++j) {
                        renderer.drawImage(
                            image,
                            0,
                            0,
                            image.width,
                            image.height,
                            offset.x + i * effWidth,
                            offset.y + j * effHeight,
                            image.width,
                            image.height
                        );
                    }
                }
            }
        };
        var anchor = new Anchor({
            onComplete: function () {
                behavior.fillSize = viewport.clone();
                behavior.updateCount();
            }
        });
        // container entity
        var entitySettings = Utils.extend({
            z: 0,
            name: 'backScroller',
            position: new Vector2(0, 0),
            originRelative: new Vector2(0, 0),
            updateWhenPaused: 0,
            float: true,
            components: [
                anchor,
                behavior
            ]
        }, settings);
        var entity = new Entity(entitySettings);

        return entity;
    };
});