/**
 * Backup popup
 * @moduleName BackupPopup
 */
bento.define('gui/backuppopup', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'bento/components/nineslice',
    'gui/blurbackground',
    'globals',
    'components/modal',
    'components/anchor',
    'gui/shadowtext',
    'gui/iconbutton',
    'gui/headerbox',
    'modules/localization',
    'managers/character',
    'gui/worldselect/topbar',
    'managers/save',
    'managers/currency',
    'modules/analytics',
    'gui/menus/iap',
    'managers/ai',
    'managers/stats',
    'gui/scrollinglist',
    'bento/gui/togglebutton',
    'components/masksprite',
    'gui/upgrade/stats',
    'gui/energyforgempopup',
    'managers/time',
    'components/orientationlock',
    'gui/buttons/startbutton',
    'gui/fade',
    'managers/coincollector',
    'managers/buff',
    'components/backbutton',
    'gui/watchrewardedad'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    NineSlice,
    BlurBackground,
    Globals,
    Modal,
    Anchor,
    ShadowText,
    IconButton,
    HeaderBox,
    Localization,
    CharacterManager,
    Topbar,
    SaveManager,
    CurrencyManager,
    Analytics,
    IAPMenu,
    AiManager,
    StatsManager,
    ScrollingList,
    ToggleButton,
    MaskSprite,
    Stats,
    EnergyForGemPopup,
    TimeManager,
    OrientationLock,
    StartButton,
    Fade,
    CoinCollector,
    BuffManager,
    BackButton,
    WatchRewardedAd
) {
    'use strict';
    var BackupPopup = function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var currentMission = SaveManager.load('currentMission');
        var energyCost = SaveManager.load('currentCost');
        var previousCharacter = CharacterManager.getSelected();
        /**
         * Buttons
         */
        var buttonClose = new IconButton({
            name: 'buttonClose',
            sfx: 'sfx_button',
            imageName: 'gui2/misc/button_small_secondary',
            iconName: 'gui2/misc/button_icon_home',
            position: new Vector2(-52, 64),
            frameCountX: 1,
            frameCountY: 3,
            onButtonDown: function () {
                scrollingList.cancel();
            },
            onClick: function () {
                // tweenOut(function () {
                //     var analytics = function () {
                //         var currentSection = SaveManager.load('currentSection');
                //         var currentLevel = StatsManager.getLevel(CharacterManager.getSelected());
                //         // analytics before the first victory
                //         if (SaveManager.load('missionStars-' + currentMission) < 0) {
                //             Analytics.event('LevelMissionFail', currentMission, currentLevel);
                //         }
                //     };
                //     analytics();

                //     Analytics.event('Backup', 'cancelled', CharacterManager.getEnergy());
                //     // revert to previous character
                //     CharacterManager.setSelected(previousCharacter);
                //     Bento.screens.show('screens/worldselect');
                // });

                // revert any character choice
                CharacterManager.setSelected(SaveManager.load('originallySelectedCharacter'));

                headerBox.removeSelf();
                stats.removeSelf();
                largePortrait.removeSelf();
                BackupPopup.showGameOver(entity);

            },
            components: [
                new BackButton({
                    removeSelf: true,
                    onPressed: function (data) {
                        data.entity.doCallback();
                    }
                })
            ]
        });
        var costText = new Text({
            position: new Vector2(8, 6),
            text: energyCost.toString(),
            font: 'font',
            fontSize: 12,
            fontColor: '#00baff',
            align: 'left',
            textBaseline: 'middle',
            lineWidth: 1.5,
            strokeStyle: '#4f3673',
            pixelStroke: true
        });
        var costIcon = new Entity({
            name: 'costIcon',
            position: new Vector2(-11 - 24, -2),
            components: [
                new Sprite({
                    imageName: 'gui2/misc/energy',
                    originRelative: new Vector2(0, 0.5)
                }),
                costText
            ]
        });
        var playText = new ShadowText({
            position: new Vector2(14, -12),
            originRelative: new Vector2(0, 0.5),
            text: Localization.getText('play'),
            font: 'font',
            fontSize: 22,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'top',
            strokeStyle: '#000000',
            maxWidth: 48,
            maxHeight: 22
        });
        var buttonInfo = new Entity({
            name: 'buttonInfo',
            components: [
                costIcon,
                playText
            ]
        });
        var buttonContinue = new IconButton({
            z: 0,
            name: 'item',
            sfx: 'sfx_button',
            position: new Vector2(24, 64),
            imageName: 'gui2/misc/button_large_primary',
            icon: buttonInfo,
            frameCountX: 1,
            frameCountY: 3,
            animations: {
                'up': {
                    frames: [0]
                },
                'down': {
                    frames: [1]
                },
                'inactive': {
                    frames: [0]
                },
            },
            float: false,
            updateWhenPaused: 0,
            onButtonDown: function () {
                scrollingList.cancel();
            },
            onClick: function () {
                if (CharacterManager.hasEnergy(energyCost)) {
                    CharacterManager.subtractEnergy(energyCost);
                    nextFighter();
                } else {
                    var dialog = new EnergyForGemPopup({
                        onBoughtEnergy: function (hasBought) {
                            if (hasBought) {
                                // now that player bought energy, subtract it
                                if (CharacterManager.hasEnergy(energyCost)) {
                                    CharacterManager.subtractEnergy(energyCost);
                                    nextFighter();
                                }
                            }
                        },
                        onWatchedAd: function () {
                            nextFighter();
                        }
                    });
                    Bento.objects.attach(dialog);
                }
            }
        });
        var resetTime = SaveManager.load('backupResetTime');
        var resetText = new Text({
            position: new Vector2(0, 32),
            text: 'Reset in 00:00:00',
            font: 'font',
            fontSize: 12,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            ySpacing: 0,
            lineWidth: 0, // set to add an outline
            strokeStyle: '#000000',
            components: []
        });
        /**
         * Characters
         */
        var scrollingList = new ScrollingList({
            position: new Vector2(0, -2),
            direction: 'x', // 'x' or 'y'
            spacing: 2, // additional spacing between items
            snap: true, // snap items (cancels maxOffset and minOffset)
            snapMinSlideSpeed: 1, // minimum slidespeed before starting to snap
            snapSpeed: 0.3, // speed of snapping
            damping: 0.9 // speed decrease,
        });
        var largePortraitSprite = new Sprite({});
        var largePortrait = new Entity({
            name: 'largePortrait',
            position: new Vector2(0, 0),
            components: [
                largePortraitSprite,
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.3, 0.3),
                        offset: new Vector2(0, 0)
                    },
                    landscape: {
                        screenPosition: new Vector2(0.2, 0.5),
                        offset: new Vector2(0, 0)
                    },
                    onComplete: function () {

                    }
                })
            ]
        });
        var stats = new Stats({
            character: CharacterManager.getSelected(),
            displayMode: Stats.DISPLAY_COMBINED,
            showBackground: true,
            components: [
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.75, 0.35),
                        offset: new Vector2(0, 0)
                    },
                    landscape: {
                        screenPosition: new Vector2(0.3, 0.75),
                        offset: new Vector2(4, 0)
                    },
                    onComplete: function () {}
                })
            ]
        });
        /**
         * Background
         */
        var blurBackground = new BlurBackground({
            darken: 0.5,
            blurriness: 15
        });
        var headerBox = new HeaderBox({
            name: 'box',
            size: new Rectangle(0, 0, 180, 140),
            title: Localization.getText('nextFighter'),
            windowImage: 'gui2/misc/splash_background_9slice',
            position: new Vector2(0, 0),
            components: [
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.5, 0.7),
                        offset: new Vector2(0, 0),
                    },
                    landscape: {
                        screenPosition: new Vector2(0.7, 0.5),
                        offset: new Vector2(0, 0),
                    },
                    onComplete: function () {}
                }),
                buttonContinue,
                buttonClose,
                resetText,
                scrollingList
            ]
        });
        /**
         * Other componentes
         */
        var modal = new Modal({});
        /**
         * Create character buttons
         */
        var makeButtons = function () {
            var currentCharacter = CharacterManager.getSelected();
            var unlockedCharacters = CharacterManager.getUnlocked();
            var characterButtons = [];
            var didSelectFirst = false;
            var selected = 0;
            var selectCharacter = function (character) {
                // save selected character
                CharacterManager.setSelected(character);
                // set portrait
                largePortraitSprite.setup({
                    spriteSheet: 'characterchange/' + character
                });
                stats.setCharacter(character);
            };
            var makeButton = function (index, character) {
                var characterPortraitSprite = new Sprite({
                    imageName: 'gui2/characterchange/smallportrait_' + character,
                    originRelative: new Vector2(0.5, 0.5)
                });
                var maskSpriteButton = new MaskSprite({
                    boundingBox: new Rectangle(-83, -83, 166, 166)
                });
                var maskSprite = new MaskSprite({
                    sprite: characterPortraitSprite,
                    boundingBox: new Rectangle(-83, -83, 166, 166)
                });
                var koText = new Text({
                    position: new Vector2(0, 8),
                    text: Localization.getText('ko'),
                    font: 'font',
                    fontSize: 16,
                    fontColor: '#ffffff',
                    align: 'center',
                    textBaseline: 'middle',
                    maxWidth: 32,
                    maxHeight: 16,
                    linebreaks: false
                });
                var backupsLeft = Globals.totalBackups - SaveManager.load('backups-' + character);
                var backupsText = new Text({
                    position: new Vector2(0, -32),
                    text: backupsLeft + '/' + Globals.totalBackups,
                    font: 'font',
                    fontSize: 12,
                    fontColor: '#23243a',
                    align: 'center',
                    textBaseline: 'middle',
                    maxWidth: 32,
                    maxHeight: 16,
                    linebreaks: false,
                    components: [{
                        name: 'setColor',
                        toggled: false,
                        update: function () {
                            if (this.toggled && !button.isToggled()) {
                                backupsText.setText(backupsText.getText(), {
                                    fontColor: '#23243a'
                                });
                                this.toggled = false;
                            } else if (!this.toggled && button.isToggled()) {
                                backupsText.setText(backupsText.getText(), {
                                    fontColor: '#ffffff'
                                });
                                this.toggled = true;
                            }
                        }
                    }]
                });
                var maskText = new MaskSprite({
                    sprite: koText.getComponent('sharpnessScaler').getComponent('sprite'),
                    boundingBox: new Rectangle(-83, -83, 166 * 4, 166 * 4)
                });
                var maskBackupText = new MaskSprite({
                    sprite: backupsText.getComponent('sharpnessScaler').getComponent('sprite'),
                    boundingBox: new Rectangle(-83, -83, 166 * 4, 166 * 4)
                });
                var maskBehavior = {
                    name: 'setMask',
                    update: function () {
                        var scroll = scrollingList.getScroll();
                        var dist;
                        // sets the maskSprite's boundingbox
                        maskSprite.boundingBox.x = -83 - button.position.x - scroll;
                        maskSpriteButton.boundingBox.x = -83 - button.position.x - scroll;
                        // * 4 because of text's sharpness
                        maskText.boundingBox.x = (-83 - button.position.x - scroll) * 4;
                        maskBackupText.boundingBox.x = (-83 - button.position.x - scroll) * 4;

                        // scaling based on middle
                        dist = Math.abs(button.position.x + scroll);
                    }
                };
                var shouldSelect = (backupsLeft > 0 && !didSelectFirst && currentCharacter !== character);
                var button = new ToggleButton({
                    name: 'button-' + character,
                    sfx: 'sfx_button',
                    imageName: 'gui2/characterchange/portrait_frame',
                    frameCountX: 2,
                    frameCountY: 1,
                    toggled: shouldSelect,
                    onToggle: function () {
                        if (this.isToggled()) {
                            // untoggle others
                            Utils.forEach(characterButtons, function (other) {
                                if (other !== button) {
                                    other.toggle(false);
                                }
                                selectCharacter(character);
                                scrollingList.snapTo(index);
                            });
                        } else {
                            // re-toggle self
                            button.toggle(true);
                        }
                        new Tween({
                            from: 0.9,
                            to: 1,
                            in: 30,
                            ease: 'elastic',
                            decay: 5,
                            oscillations: 2,
                            onUpdate: function (v, t) {
                                button.scale.x = v;
                                button.scale.y = v;
                            }
                        });
                    },
                    components: [
                        maskSpriteButton,
                        characterPortraitSprite,
                        backupsText,
                        maskSprite,
                        maskText,
                        maskBackupText,
                        maskBehavior
                    ]
                });

                button.dimension.width = 42;

                scrollingList.addItem(button);
                characterButtons.push(button);

                // disable dead player
                if (currentCharacter === character) {
                    button.setActive(false);
                    button.attach(koText);

                    characterPortraitSprite.setup({
                        imageName: 'gui2/characterchange/smallportrait_locked_' + character,
                        originRelative: new Vector2(0.5, 0.5)
                    });
                }
                // maxed out backups
                if (backupsLeft <= 0) {
                    button.setActive(false);
                    characterPortraitSprite.setup({
                        imageName: 'gui2/characterchange/smallportrait_locked_' + character,
                        originRelative: new Vector2(0.5, 0.5)
                    });
                }
                // selected
                if (shouldSelect) {
                    didSelectFirst = true;
                    selected = index;
                    selectCharacter(character);
                }
            };

            Utils.forEach(unlockedCharacters, function (character, i) {
                makeButton(i, character);
            });

            scrollingList.snapTo(selected);

            if (!didSelectFirst) {
                // there is nobody to select
                buttonContinue.setActive(false);
                buttonContinue.removeSelf();
                stats.removeSelf();
                buttonClose.position.x = 0;

                // show a popup to explain?
            }
        };
        /**
         * Check time (do this before creating buttons)
         */
        var checkTime = function () {
            var now = TimeManager.getTime();
            var hourMs = 1000 * 60 * 60;
            // reschedule in future if needed
            if (resetTime < now) {
                // 8 hours
                resetTime = now + hourMs * Globals.hoursUntilBackup;
                // reset backups
                Utils.forEach(CharacterManager.getUnlockedCharacters(), function (character, i, l, breakLoop) {
                    SaveManager.save('backups-' + character, 0);
                });
            }
            SaveManager.save('backupResetTime', resetTime);

            // set time
            var timeLeft = resetTime - now;
            var timeStr = '0m';

            if (timeLeft > hourMs) {
                // show amount of hours
                timeStr = Math.floor(timeLeft / hourMs) + 'h ' + Math.floor((timeLeft % hourMs) / (1000 * 60)) + 'm';
            } else {
                // only show minutes (don't show 0 minutes)
                timeStr = (Math.floor((timeLeft % hourMs) / (1000 * 60)) || 1) + 'm';
            }
            resetText.setText(Localization.getText('resetNext').replace('{TIME}', timeStr));
        };
        /**
         * Spawn next fighter and close popup
         */
        var nextFighter = function () {
            Utils.forEach(Bento.objects.getByFamily('player'), function (player, i, l, breakLoop) {
                if (!player.isRemote) {
                    // destroy that player and respawn
                    player.removeSelf();
                    // find the healthbar
                    Bento.objects.get('healthBar', function (healthBar) {
                        healthBar.removeSelf();
                    });
                    // reset health of local player
                    StatsManager.resetHealth();
                    // reset combopoints
                    SaveManager.save('comboMeter', 0);
                    SaveManager.save('preComboMeter', 0);
                    SaveManager.add('backups-' + CharacterManager.getSelected(), 1);
                    // respawning happens in screens/main
                    EventSystem.fire('respawnPlayer', {
                        position: player.position.clone(),
                        onComplete: function (newPlayer) {
                            newPlayer.getComponent('punchable', function (punchable) {
                                punchable.setInvincibleTime(60);
                            });
                            // fire event to clear up chached values in other modules
                            EventSystem.fire('playerSpawned', {
                                player: newPlayer
                            });
                        }
                    });
                    Analytics.event('Backup', 'used', 0, currentMission.toString());
                    SaveManager.save('calledBackup', true);
                    SaveManager.save('hasRevived', false);
                    breakLoop();
                }
            });
            buttonContinue.setActive(false);
            buttonClose.setActive(false);

            Bento.audio.stopSound('sfx_revive');
            Bento.audio.playSound('sfx_revive');
            // restore music
            new Tween({
                from: 0.25,
                to: 1,
                in: 30,
                onUpdate: function (v, t) {
                    Bento.audio.setVolume(v, Globals.currentMusic);
                }
            });

            tweenOut(function () {
                entity.removeSelf();
            });
        };
        var tweenOut = function (onComplete) {
            // tween out
            buttonClose.setActive(false);
            buttonContinue.setActive(false);
            new Tween({
                from: 1,
                to: 0,
                in: 30,
                ease: 'linear',
                onUpdate: function (v, t) {
                    blurBackground.alpha = v;
                    largePortrait.alpha = v;
                }
            });
            new Tween({
                from: 1,
                to: 0,
                in: 15,
                delay: 15,
                ease: 'linear',
                onUpdate: function (v, t) {
                    headerBox.alpha = v;
                }
            });
            // scale out
            new Tween({
                from: 1,
                to: 0,
                in: 30,
                delay: 0,
                ease: 'easeInBack',
                onUpdate: function (v, t) {
                    headerBox.scale.x = v;
                    headerBox.scale.y = v;
                    stats.scale.x = v;
                    stats.scale.y = v;
                },
                onComplete: onComplete
            });
        };
        /**
         * Main entity
         */
        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {
                new Tween({
                    from: 0,
                    to: 1,
                    in: 20,
                    ease: 'elastic',
                    decay: 5,
                    oscillations: 1,
                    onStart: function () {},
                    onUpdate: function (v, t) {
                        headerBox.scale.x = v;
                        headerBox.scale.y = v;
                    },
                    onComplete: function () {}
                });

                stats.scale.x = 0;
                stats.scale.y = 0;
                new Tween({
                    from: 0,
                    to: 1,
                    in: 20,
                    delay: 10,
                    ease: 'elastic',
                    decay: 5,
                    oscillations: 1,
                    onUpdate: function (v, t) {
                        stats.scale.x = v;
                        stats.scale.y = v;
                    },
                    onComplete: function () {}
                });

                new Tween({
                    from: largePortrait.position.x - viewport.width,
                    to: largePortrait.position.x,
                    in: 30,
                    ease: 'elastic',
                    decay: 5,
                    oscillations: 1,
                    onUpdate: function (v, t) {
                        largePortrait.position.x = v;
                    },
                    onComplete: function () {

                    }
                });
                checkTime();
                makeButtons();
            },
            destroy: function (data) {},
            update: function (data) {},
            draw: function (data) {}
        };
        var entity = new Entity({
            z: Globals.Layers.PAUSE,
            name: 'continuePopup',
            float: true,
            components: [
                modal,
                blurBackground,
                largePortrait,
                stats,
                headerBox,
                new Topbar({}),
                behavior
            ]
        });

        return entity;
    };

    BackupPopup.canCallBackup = function () {
        // needs to have unlocked at least 1 character
        var unlockedCharacters = CharacterManager.getUnlocked();
        //
        if (unlockedCharacters.length <= 1) {
            return false;
        }
        // already called backup before
        if (SaveManager.load('calledBackup')) {
            return false;
        }
        return true;
    };

    /**
     * Show game over if player is about to go home
     */
    BackupPopup.showGameOver = function (parent, hideContinueButton) {
        var viewport = Bento.getViewport();
        var currentMission = SaveManager.load('currentMission');
        var gameOverText = new ShadowText({
            text: Localization.getText('gameOver'),
            font: 'font',
            fontSize: 32,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            maxWidth: viewport.width,
            maxHeight: 32,
            linebreaks: false,
            drawDebug: false,
            components: [
                new Anchor({
                    screenPosition: new Vector2(0.5, 0.35),
                    offset: new Vector2(0, 0)
                }),
                new OrientationLock({})
            ]
        });
        var showButtons = function () {
            var buttonHome = new IconButton({
                name: 'buttonHome',
                sfx: 'sfx_button',
                imageName: 'gui2/misc/button_small_secondary',
                iconName: 'gui2/misc/button_icon_home',
                position: new Vector2(-52, 0),
                frameCountX: 1,
                frameCountY: 3,
                onClick: function () {
                    buttonHome.setActive(false);
                    buttonRetry.setActive(false);
                    new Fade({
                        duration: 60,
                        onComplete: function () {
                            if (SaveManager.load('isTower')) {
                                // show forced ad
                                WatchRewardedAd.showInterstitial(function () {
                                    Bento.screens.show('screens/worldselect', {
                                        target: 'tower'
                                    });
                                });
                            } else {
                                Bento.screens.show('screens/worldselect');
                            }
                        }
                    });
                }
            });
            var energyCost = SaveManager.load('currentCost');
            var costText = new Text({
                position: new Vector2(8, 6),
                text: energyCost.toString(),
                font: 'font',
                fontSize: 12,
                fontColor: '#00baff',
                align: 'left',
                textBaseline: 'middle',
                lineWidth: 1.5,
                strokeStyle: '#4f3673',
                pixelStroke: true
            });
            var costIcon = new Entity({
                name: 'costIcon',
                position: new Vector2(-11 - 24, -2),
                components: [
                    new Sprite({
                        imageName: 'gui2/misc/energy',
                        originRelative: new Vector2(0, 0.5)
                    }),
                    costText
                ]
            });
            var playText = new ShadowText({
                position: new Vector2(14, 0),
                originRelative: new Vector2(0, 0.5),
                text: Localization.getText('tryAgain'),
                font: 'font',
                fontSize: 22,
                fontColor: '#ffffff',
                align: 'center',
                textBaseline: 'middle',
                strokeStyle: '#000000',
                maxWidth: 56,
                maxHeight: 22
            });
            var buttonInfo2 = new Entity({
                name: 'buttonInfo',
                components: [
                    costIcon,
                    playText
                ]
            });
            var buttonRetry = new IconButton({
                z: 0,
                name: 'item',
                sfx: 'sfx_button',
                position: new Vector2(24, 0),
                imageName: 'gui2/misc/button_large_primary',
                icon: buttonInfo2,
                frameCountX: 1,
                frameCountY: 3,
                animations: {
                    'up': {
                        frames: [0]
                    },
                    'down': {
                        frames: [1]
                    },
                    'inactive': {
                        frames: [0]
                    },
                },
                float: false,
                updateWhenPaused: 0,
                onClick: function () {
                    // // make a startbutton and do a callback
                    var startButton = new StartButton({
                        cost: SaveManager.load('currentCost'),
                        enemyLevel: SaveManager.load('enemyLevel'),
                        mission: SaveManager.load('currentMission'),
                        unlocks: SaveManager.load('toUnlock'),
                        rewards: SaveManager.load('possibleRewards'),
                        chosenDifficulty: SaveManager.load('chosenDifficulty')
                    });

                    // revert any character choice
                    CharacterManager.setSelected(SaveManager.load('originallySelectedCharacter'));
                    // workaround: combobar seems to add combometer points when destroyed,
                    // making the reset impossible
                    Bento.objects.get('healthBar', function (healthBar) {
                        healthBar.removeSelf();
                    });

                    // unmark 1st time death tutorial
                    if (SaveManager.load('tutorial1stTimeDeath') === 1) {
                        SaveManager.save('tutorial1stTimeDeath', 0);
                    } else
                    if (SaveManager.load('tutorial1stTimeDeath') === 3) {
                        SaveManager.save('tutorial1stTimeDeath', 2);
                    }

                    // restore music
                    Bento.audio.stopAllMusic();
                    Bento.audio.setVolume(1, Globals.currentMusic);

                    startButton.getComponent('buttonPlay').doCallback();
                }
            });
            var buttonContainer = new Entity({
                z: Globals.Layers.PAUSE,
                name: 'buttonContainer',
                components: [
                    new Anchor({
                        portrait: {
                            screenPosition: new Vector2(0.5, 1),
                            offset: new Vector2(0, -48)
                        },
                        landscape: {
                            screenPosition: new Vector2(0.5, 1),
                            offset: new Vector2(0, -32)
                        }
                    }),
                    buttonHome,
                    buttonRetry
                ]
            });
            parent.attach(buttonContainer);

            new Tween({
                from: buttonContainer.position.y + 64,
                to: buttonContainer.position.y,
                in: 30,
                ease: 'elastic',
                decay: 5,
                oscillations: 1,
                onUpdate: function (v, t) {
                    buttonContainer.position.y = v;
                }
            });

            if (hideContinueButton) {
                buttonRetry.removeSelf();
                buttonHome.position.x = 0;
            }
        };
        var showCoinsCollected = function () {
            var coinsCollected = Math.ceil(CoinCollector.getCollected()) + BuffManager.current.extraCoins;
            var coinText = new ShadowText({
                position: new Vector2(12, 6),
                text: coinsCollected.toString(),
                font: 'font',
                fontSize: 16,
                fontColor: '#ffffff',
                align: 'left',
                textBaseline: 'bottom'
            });
            var coinIcon = new Entity({
                z: Globals.Layers.VICTORY,
                name: 'coin',
                components: [
                    new Sprite({
                        originRelative: new Vector2(0.5, 0.5),
                        imageName: 'gui/coin'
                    }),
                    coinText,
                    new Anchor({
                        screenPosition: new Vector2(0.5, 0.5)
                    })
                ]
            });
            var topBar;
            var tweenInCoins = function () {
                // move towards center
                parent.attach(coinIcon);
                new Tween({
                    from: coinIcon.position.add(new Vector2(viewport.width, 0)),
                    to: coinIcon.position,
                    in: 30,
                    ease: 'elastic',
                    decay: 5,
                    oscillations: 1,
                    onUpdate: function (v, t) {
                        coinIcon.position = v;
                    },
                    onComplete: function () {
                        tweenCoinsCollected();
                    }
                });
            };
            var tweenCoinsCollected = function () {
                new Tween({
                    from: 0,
                    to: 1,
                    in: Math.min(coinsCollected, 120),
                    delay: 30,
                    ease: 'linear',
                    onStart: function () {},
                    onUpdate: function (v, t) {
                        // tween counter
                        coinText.setText(Math.floor((1 - v) * coinsCollected).toString());
                        topBar.setCoins((CurrencyManager.getCoins() + Math.floor(v * coinsCollected)).toString());

                        if (t % 2 < 1) {
                            Bento.audio.stopSound('sfx_coin');
                            Bento.audio.playSound('sfx_coin');
                            topBar.flashCoin();
                        }
                    },
                    onComplete: function () {
                        // add the actual coin amount
                        CurrencyManager.addCoins(coinsCollected);
                        showButtons();

                        // fade out
                        new Tween({
                            from: 1,
                            to: 0,
                            in: 50,
                            onUpdate: function (v, t) {
                                coinIcon.alpha = v;
                            }
                        });
                    }
                });
            };
            // get topbar
            if (parent) {
                parent.getComponent('topBar', function (component) {
                    topBar = component;
                });
                if (!topBar) {
                    parent.attach(new Topbar({}));
                }
            }

            // show coins collected
            if (coinsCollected > 0) {
                tweenInCoins();

                // save amount of coins (used later, to double coins for ad)
                SaveManager.save('coinsCollected', coinsCollected);
            } else {
                showButtons();
            }
        };
        var analytics = function () {
            var currentSection = SaveManager.load('currentSection');
            var currentLevel = StatsManager.getLevel(CharacterManager.getSelected());
            // analytics before the first victory
            if (SaveManager.load('missionStars-' + currentMission) < 0) {
                Analytics.event('LevelMissionFail', currentMission, currentLevel);
            }
        };
        analytics();
        // drop down game over text
        gameOverText.scale.x = 0;
        gameOverText.scale.y = 0;
        new Tween({
            from: 10,
            to: 1,
            in: 60,
            delay: 30,
            ease: 'quadratic',
            onUpdate: function (v, t) {
                gameOverText.scale.x = v;
                gameOverText.scale.y = v;
            },
            onComplete: function () {
                Bento.audio.playSound('sfx_vsdrop');
                // shake
                new Tween({
                    from: 2,
                    to: 0,
                    in: 10,
                    onUpdate: function (v, t) {
                        gameOverText.transform.x = Utils.getRandomRangeFloat(-v, v);
                        gameOverText.transform.y = Utils.getRandomRangeFloat(-v, v);
                    },
                    onComplete: function () {
                        gameOverText.transform.x = 0;
                        gameOverText.transform.y = 0;

                        showCoinsCollected();
                    }
                });
            }
        });

        parent.attach(gameOverText);

        // fade out music entirely
        new Tween({
            from: 0.25,
            to: 0,
            in: 30,
            onUpdate: function (v, t) {
                Bento.audio.setVolume(v, Globals.currentMusic);
            }
        });

        EventSystem.fire('showGameOver');

        // show score
        if (SaveManager.load('isTower')) {
            var scoreText = new ShadowText({
                z: Globals.Layers.GUI,
                text: Localization.getText('score').replace('{NUMBER}', SaveManager.load('towerScore')),
                font: 'font',
                fontSize: 16,
                fontColor: '#ffffff',
                align: 'center',
                textBaseline: 'middle',
                linebreaks: false,
                drawDebug: false,
                float: true,
                components: [
                    new Anchor({
                        portrait: {
                            screenPosition: new Vector2(0.5, 0.65),
                            offset: new Vector2(0, 0),
                        },
                        landscape: {
                            screenPosition: new Vector2(0.5, 0.5),
                            offset: new Vector2(0, 0),
                        },
                        onComplete: function () {}
                    })
                ]
            });
            parent.attach(scoreText);
            new Tween({
                from: 0,
                to: 1,
                in: 30,
                delay: 90,
                applyOnDelay: true,
                ease: 'linear',
                onUpdate: function (v, t) {
                    scoreText.alpha = v;
                }
            });
            EventSystem.fire('endTower');
        }

    };
    return BackupPopup;
});