/**
 * Red badge with number in it
 * @moduleName Badge
 */
bento.define('gui/badge', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'bento/components/nineslice',
    'gui/shadowtext'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    NineSlice,
    ShadowText
) {
    'use strict';
    return function (settings) {
        var viewport = Bento.getViewport();
        var number = settings.number || 0;
        var scale = settings.scale || 1;
        var exclamation = settings.exclamation || false;
        var text = new ShadowText({
            position: new Vector2(0.5, -2),
            text: number.toString(),
            font: 'font',
            fontSize: 12,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle'
        });
        // var nineSlice = new NineSlice({
        //     imageName: settings.imageName || 'gui/badge',
        //     originRelative: new Vector2(0.5, 0.5),
        //     width: Math.max(text.dimension.width, 18),
        //     height: 18
        // });
        var nineSlice = new Sprite({
            imageName: settings.imageName || 'gui/badge',
            originRelative: new Vector2(0.5, 0.5),
        });
        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {},
            destroy: function (data) {},
            update: function (data) {
                if (settings.pulsate) {
                    entity.scale.x = (1 + 0.1 * Math.sin(entity.timer / 20)) * scale;
                    entity.scale.y = (1 + 0.1 * Math.sin(entity.timer / 20)) * scale;
                }
            },
            draw: function (data) {}
        };
        var entity = new Entity({
            name: 'badge',
            position: settings.position,
            scale: new Vector2(scale, scale),
            components: [
                nineSlice,
                text,
                behavior
            ]
        });

        entity.extend({
            setNumber: function (value) {
                if (exclamation) {
                    // text.setText('!');
                    text.setText('');
                } else {
                    number = value;
                    text.setText(number.toString());
                }
                // nineSlice.setWidth(Math.max(text.dimension.width, 18));

                // don't show at all if 0
                entity.visible = (number > 0 || exclamation);
            },
            getNumber: function () {
                return number;
            },
            setExclamation: function (value) {
                exclamation = value;
                entity.setNumber(0);
            }
        });

        entity.setNumber(number);

        return entity;
    };
});