/**
 * Free gift button
 * @moduleName GiftButton
 */
bento.define('gui/buttons/gift', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'globals',
    'components/anchor',
    'gui/menus/gift',
    'managers/save',
    'managers/time',
    'gui/badge',
    'components/glowsprite',
    'gui/iconbutton',
    'gui/timertab'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Globals,
    Anchor,
    GiftMenu,
    SaveManager,
    TimeManager,
    Badge,
    Glow,
    IconButton,
    TimerTab
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var yPosition = 48;

        var button = new IconButton({
            name: 'clickButton',
            sfx: 'sfx_button',
            imageName: 'gui2/misc/buttonmini_small_secondary',
            iconName: 'gui2/misc/button_icon_gift',
            offset: new Vector2(0, -2),
            frameCountX: 1,
            frameCountY: 3,
            updateWhenPaused: 0,
            onClick: function () {
                EventSystem.fire('dismissSensei');
                //go to gift menu
                var giftMenu = new GiftMenu({});
                // try to claim gift
                giftMenu.claim(function (claimed) {
                    if (!claimed) {
                        // shake
                        new Tween({
                            from: 2,
                            to: 0,
                            in: 40,
                            ease: 'elastic',
                            decay: 2,
                            oscillations: 5,
                            onUpdate: function (v, t) {
                                timerTab.transform.x = v;
                            },
                            onComplete: function () {
                                timerTab.transform.x = 0;
                            }
                        });
                        Bento.audio.stopSound('sfx_error');
                        Bento.audio.playSound('sfx_error');
                    }
                });
                // Bento.objects.attach(giftMenu);
            },
            onButtonDown: function () {
                //push button down
                EventSystem.fire('preventScroll');
            },
            onButtonUp: function () {
                //pop labels back up
            }
        });
        var anchor = new Anchor({
            portrait: {
                screenPosition: new Vector2(0, 0),
                offset: new Vector2(26, 48)
            },
            landscape: {
                screenPosition: new Vector2(0, 0),
                offset: new Vector2(26, 24)
            }
        });
        var behavior = {
            name: 'giftChecker',
            start: function () {
                button.attach(glow);
                button.attach(badge);
                glow.visible = false;
            },
            update: function () {
                // every second check if gift can be claimed
                if (entity.timer % 60 === 0) {
                    behavior.checkGifts();
                }

            },
            checkGifts: function () {
                var gift24 = SaveManager.load('gift24');
                var gift12 = SaveManager.load('gift12');
                var gift8 = SaveManager.load('gift8');
                var time = TimeManager.getTime();
                var ready = 0;

                // if (SaveManager.load('tutorialWorldSelect') === 0) {
                //     // not during tutorials
                //     return;
                // }

                if (time >= gift24.time) {
                    ready += 1;
                }
                // if (time >= gift12.time) {
                //     ready += 1;
                // }
                // if (time >= gift8.time) {
                //     ready += 1;
                // }

                // set badge
                if (ready > 0) {
                    glow.visible = true;
                    badge.setExclamation(ready);
                } else {
                    glow.visible = false;
                    badge.setExclamation(0);
                }
            }
        };
        var badge = new Badge({
            exclamation: true,
            imageName: 'gui2/misc/button_alert_secondary',
            position: new Vector2(16, -14),
        });
        var glow = new Glow({
            average: 0.35,
            amplitude: 0.25
        });
        var timerTab = new TimerTab({
            saveId: 'gift24'
        });
        var entity = new Entity({
            z: Globals.WorldSelectLayers.GUI,
            name: 'giftButton',
            float: true,
            updateWhenPaused: 0,
            components: [
                anchor,
                timerTab,
                button,
                behavior
            ]
        }).extend({
            setActive: button.setActive
        });

        return entity;
    };
});