/**
 * Pause button
 * @moduleName PauseButton
 */
bento.define('gui/buttons/pause', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'globals',
    'gui/menus/pause',
    'components/anchor',
    'components/backbutton'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Globals,
    PauseMenu,
    Anchor,
    BackButton
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();


        var disable = function () {
            entity.setActive(false);
        };

        var enable = function () {
            entity.setActive(true);
        };

        var disableBehavior = {
            start: function (data) {
                Bento.input.resetPointers();
                EventSystem.on('enableControls', enable);
                EventSystem.on('disableControls', disable);
            },
            destroy: function (argument) {
                EventSystem.off('enableControls', enable);
                EventSystem.off('disableControls', disable);
            }
        };

        var entity = new ClickButton({
            z: Globals.Layers.GUI,
            name: 'pauseButton',
            sfx: 'sfx_button',
            imageName: 'gui/main/pause',
            frameCountX: 2,
            frameCountY: 1,
            float: true,
            family: ['gui'],
            onClick: function () {
                // spawn pause menu
                var pauseMenu = new PauseMenu({});
                Bento.objects.attach(pauseMenu);

                EventSystem.fire('releaseTouches');
            },
            components: [
                new BackButton({
                    onPressed: function (data) {
                        data.entity.doCallback();
                    }
                })
            ]
        });

        entity.attach(new Anchor({
            screenPosition: new Vector2(1, 0),
            offset: new Vector2(-14, 4 + 15),
            onComplete: function () {
                if (Anchor.isPortrait() && Anchor.isWideScreen() && Utils.isNativeIos()) {
                    entity.position.x -= 10;
                    entity.position.y -= 10;
                }
            }
        }));

        entity.attach(disableBehavior);

        return entity;
    };
});