/**
 * GUI start button
 * @moduleName StartButton
 */
bento.define('gui/buttons/startbutton', [
    'bento',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/gui/clickbutton',
    'bento/math/vector2',
    'gui/shadowtext',
    'managers/stats',
    'managers/save',
    'managers/coincollector',
    'managers/buff',
    'managers/character',
    'gui/watchrewardedad',
    'gui/dialog',
    'modules/localization',
    'modules/analytics',
    'gui/energyforgempopup',
    'entities/photosnapper',
    'modules/matchmanager',
    'globals',
    'modules/replaykit',
    'bento/utils'
], function (
    Bento,
    Sprite,
    Clickable,
    Entity,
    ClickButton,
    Vector2,
    ShadowText,
    StatsManager,
    SaveManager,
    CoinCollector,
    BuffManager,
    CharacterManager,
    WatchRewardedAd,
    Dialog,
    Localization,
    Analytics,
    EnergyForGemPopup,
    PhotoSnapper,
    MatchManager,
    Globals,
    ReplayKit,
    Utils
) {
    'use strict';
    return function (settings) {
        var missionsJson = Bento.assets.getJson('missions/missions');
        var startGame = function () {
            var chosenDifficulty = settings.chosenDifficulty || 0;
            var currentMission = settings.mission;
            var isTower = settings.isTower || currentMission === 'tower' || false;
            var isEvent = settings.isEvent || (Utils.isString(currentMission) && currentMission.indexOf('beach') === 0) || false;
            var difficulty = settings.difficulty || missionsJson[currentMission].difficulty;
            var currentCost = settings.cost;
            var toUnlock = settings.unlocks;
            var possibleRewards = settings.rewards;
            var isJoining = settings.isJoining || false;
            var wantsReputation = settings.wantsReputation || false;

            var selectedChar = CharacterManager.getSelected();

            // reset health of local player
            StatsManager.resetHealth();

            // reset coins collected
            CoinCollector.reset();

            // save enemy level
            var enemyLevel = missionsJson[currentMission].enemyLevel;
            if (chosenDifficulty) {
                enemyLevel += chosenDifficulty * Globals.difficultyJump;
                // difficulty += (1 - difficulty) / 3 * chosenDifficulty;
                difficulty += Globals.difficulty[chosenDifficulty];
            }
            if (isEvent) {
                enemyLevel = setEnemyLevelBeach();
            }
            SaveManager.save('chosenDifficulty', chosenDifficulty);
            SaveManager.save('enemyLevel', enemyLevel);
            SaveManager.save('originalEnemyLevel', enemyLevel);
            // revert to previously selected character after using backup
            if (SaveManager.load('calledBackup')) {
                CharacterManager.setSelected(SaveManager.load('originallySelectedCharacter'));
            } else {
                SaveManager.save('originallySelectedCharacter', selectedChar);
            }
            SaveManager.save('isJoining', isJoining);
            SaveManager.save('wantsReputation', wantsReputation);
            SaveManager.save('hasRevived', false);
            SaveManager.save('calledBackup', false);
            SaveManager.save('isTower', isTower);
            SaveManager.save('isEvent', isEvent);
            SaveManager.save('towerSection', '');
            SaveManager.save('towerFloor', 0);
            SaveManager.save('towerScore', 0);
            // save difficulty
            SaveManager.save('difficulty', difficulty);
            // save current mission
            SaveManager.save('currentMission', currentMission);
            // save current mission cost
            SaveManager.save('currentCost', currentCost);
            // reset section to 1
            SaveManager.save('currentSection', 1);
            // local player should not be holding any weapons
            SaveManager.save('currentWeapon', {});
            // what will unlocked after completion
            SaveManager.save('toUnlock', toUnlock || null);
            // possible rewards after completion
            SaveManager.save('possibleRewards', possibleRewards);
            // is online?
            SaveManager.save('isMissionOnline', MatchManager.isOnline);
            // prepare mission result
            var worldSelectItems = SaveManager.load('worldSelectItems');
            worldSelectItems.stars = 0;
            worldSelectItems.item = '';
            worldSelectItems.mission = currentMission;
            SaveManager.save('worldSelectItems', worldSelectItems);
            // reset combo meter
            SaveManager.save('comboMeter', BuffManager.current.comboMeterStart * Globals.maxComboPoints);
            SaveManager.save('preComboMeter', 0);

            // analytics
            if (MatchManager.isOnline) {
                Analytics.event('Multiplayer', 'total');
                Analytics.event('Multiplayer', 'mission_' + currentMission);
                Analytics.event('MissionMulti', currentMission + '_started');
            } else {
                var currentLevel = StatsManager.getLevel(selectedChar);
                var missionStars = SaveManager.load('missionStars-' + currentMission);
                Analytics.event('Mission', currentMission + '_started', currentLevel, selectedChar);
                // average level of users at this mission, before first victory
                if (missionStars < 0) {
                    Analytics.event('LevelMissionStart', currentMission, currentLevel, selectedChar);
                } else {
                    // event when user replays mission
                    Analytics.event('LevelMissionStartReplay', currentMission, currentLevel, selectedChar);
                }

                if (chosenDifficulty === 2 && missionStars === 3) {
                    Analytics.event('SlowLoot', currentMission, 1);
                }
            }

            BuffManager.loadBuffs();

            // reset photo, because we don't want to show unrelated photos of different missions
            PhotoSnapper.reset();

            // record with replaykit
            if (Globals.isRecording) {
                ReplayKit.isRecording(function (isRecording) {
                    if (!isRecording) {
                        ReplayKit.record();
                    }
                });
            }

            // used up almost all energy
            if (CharacterManager.getEnergy('boy') < 3) {
                Analytics.event('Energy', 'low');
            }

            // preload mission
            Bento.screens.show('screens/preloadmission');
        };
        var setEnemyLevelBeach = function () {
            // level starts at player level - 4 and increases every floor
            var enemyLevel;
            var eventMissionsCleared = SaveManager.load('eventMissionsCleared');
            var currentBeachMission = eventMissionsCleared % 5;
            var highestLevel = SaveManager.load('level-' + SaveManager.load('selectedCharacter'));
            
            enemyLevel = highestLevel + currentBeachMission - 3;
            enemyLevel = Utils.clamp(1, enemyLevel, Infinity);

            return enemyLevel;
        };
        var button = new ClickButton({
            name: 'buttonPlay',
            sfx: 'sfx_button',
            imageName: 'gui/buttons/play',
            frameCountX: 3,
            frameCountY: 1,
            origin: new Vector2(33, 18),
            updateWhenPaused: 0,
            onClick: function () {
                var dialog;
                var exemptFromEnergy = SaveManager.load('missionsExemptFromEnergy');
                var index = exemptFromEnergy.indexOf(settings.mission);
                //if the player has enough energy go to the stage
                if (index >= 0) {
                    exemptFromEnergy.splice(index, 1);
                    SaveManager.save('missionsExemptFromEnergy', exemptFromEnergy);
                    startGame();
                } else if (CharacterManager.hasEnergy(settings.cost)) {
                    CharacterManager.subtractEnergy(settings.cost);
                    startGame();
                } else {
                    // ask to watch ad? should have been done before but we could potentially ask again
                    dialog = new EnergyForGemPopup({
                        onBoughtEnergy: function (hasBought) {
                            if (hasBought) {
                                startGame();
                            }
                        },
                        onWatchedAd: function () {
                            startGame();
                        }
                    });
                    Bento.objects.attach(dialog);
                }
            },
            onButtonDown: function () {
                //push labels of button down
                icon.position.y = -1;
                costText.position.y = -11;
            },
            onButtonUp: function () {
                //pop labels back up
                icon.position.y = -2;
                costText.position.y = -12;
            }
        });

        //energy icon
        var icon = new Entity({
            name: 'icon',
            position: new Vector2(-22, -2),
            components: [
                new Sprite({
                    imageName: 'gui/startlevel/energyicon',
                    originRelative: new Vector2(0, 0.5)
                })
            ]
        });

        //how much the level will cost in energy
        var costText = new ShadowText({
            position: new Vector2(10, -12),
            originRelative: new Vector2(0, 0.5),
            text: settings.cost.toString(),
            font: 'font',
            fontSize: 16,
            fontColor: '#ffffff',
            align: 'right',
            textBaseline: 'top',
            strokeStyle: '#000000',
        });

        //align icon and text together in the middle of the button
        var totalWidth = 11 + costText.dimension.width;
        icon.position.x = -totalWidth * 0.5;
        costText.position.x = totalWidth * 0.5;


        //entity container for multiple stats frames
        var entity = new Entity({
            z: 0,
            name: 'levelInfo',
            position: new Vector2(0, 0),
            components: [
                button,
                // icon,
                // costText
            ]
        }).extend({
            start: function () {
                button.doCallback();
            }
        });

        return entity;
    };
});