/**
 * The start multiplayer button sits next to the start button in the level details menu
 * It will open the multiplayer menu to connect and then immediately start the game.
 * @moduleName StartMultiplayer
 */
bento.define('gui/buttons/startmultiplayer', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'gui/menus/multiplayer',
    'modules/matchmanager',
    'managers/character'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    MultiplayerMenu,
    MatchManager,
    CharacterManager
) {
    'use strict';
    return function (settings) {
        /*settings = {
            position: Vector2 // positions the entity
            startButton: StartButton // needs a reference to the start button
        }*/
        var viewport = Bento.getViewport();
        var startButton = settings.startButton;
        var mission = settings.mission;
        var entity = new ClickButton({
            name: 'buttonStartMultiplayer',
            sfx: 'sfx_button',
            imageName: 'gui/buttons/startmultiplayer',
            frameCountX: 2,
            frameCountY: 1,
            position: settings.position,
            onClick: function () {
                // already matched? disconnect asap
                if (MatchManager.currentMatch) {
                    MatchManager.disconnect();
                    return;
                }

                //change the local name to the KTNickName when available
                Utils.changeLocalNameToKTNickName();

                // TODO: just like the start button, it needs to check if the player has enough energy
                // however it should not subtract the energy from the player until the connection was successful
                // if (!CharacterManager.hasEnergy(settings.cost)) {
                //     return;
                // }

                // connect
                var multiplayerMenu = new MultiplayerMenu({
                    mission: mission,
                    onComplete: function () {
                        // onComplete marks a successful match-up
                        // we now simulate a press on the start button to begin the level
                        startButton.getComponent('buttonPlay', function (button) {
                            button.doCallback();
                        });
                    }
                });
                Bento.objects.attach(multiplayerMenu);

                // it's possible to skip this menu by simulating a click immediately on the connect button
                // in that case: change the min/max players to 2, 4 in modules/onlineservice/cocoon
                // multiplayerMenu.getComponent('buttonConnect', function (button) {
                //     button.doCallback();
                // });
            }
        });
        return entity;
    };
});