/**
 * GUI character container
 * @moduleName CharacterContainer
 */
bento.define('gui/charactercontainer', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'gui/shadowtext',
    'gui/upgrade/stats',
    'managers/stats'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    ShadowText,
    Stats,
    StatsManager
) {
    'use strict';
    return function (settings) {
        var currentLevel = 0;

        var characterLevelText = new ShadowText({
            text: 'Level 1',
            font: 'font',
            fontSize: 16,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'top',
            position: new Vector2(0, -67)
        });

        var characterSprite = new Sprite({
            spriteSheet: settings.character + '/idle'
        });

        var shadowSprite = new Sprite({
            imageName: settings.imageName || 'shadow24',
            originRelative: new Vector2(0.5, 0.5)
        });

        var updateLevel = function () {
            currentLevel = StatsManager.getLevel(settings.character);
            characterLevelText.setText('Level ' + currentLevel);
        };

        var behavior = {
            name: 'containerBehavior',
            start: function () {
                updateLevel();
                entity.dimension = new Rectangle(-24, -40, 48, 80);
            }
        };

        var entity = new Entity({
            name: 'character',
            position: new Vector2(0, 0),
            components: [
                shadowSprite,
                characterSprite,
                // characterLevelText,
                behavior
            ]
        }).extend({
            updateLevel: updateLevel
        });

        return entity;
    };
});