/**
 * A popup that appears above a players head once a consumable has been used
 * @moduleName ConsumablePopup
 */
bento.define('gui/consumablepopup', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var sprite = settings.sprite;
        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {
                sprite.setOriginRelative(new Vector2(0.5, 0.5));
                entity.position.y = -50;
                new Tween({
                    from: entity.position.y + 10,
                    to: entity.position.y - 10,
                    'in': 20,
                    ease: Tween.EASEOUTCUBIC,
                    onUpdate: function (v, t) {
                        entity.position.y = v;
                    }
                });
                new Tween({
                    from: entity.position.y - 10,
                    to: entity.position.y - 30,
                    'in': 30,
                    delay: 60,
                    ease: Tween.EASEINCUBIC,
                    onUpdate: function (v, t) {
                        entity.position.y = v;
                    },
                    onComplete: function () {
                        Bento.objects.remove(entity);
                        entity.active = false;
                    }
                });


                new Tween({
                    from: 0,
                    to: 1,
                    'in': 10,
                    ease: Tween.LINEAR,
                    onUpdate: function (v, t) {
                        entity.alpha = v;
                    }
                });

                new Tween({
                    from: 1,
                    to: 0,
                    'in': 50,
                    delay: 40,
                    ease: Tween.EASEINQUAD,
                    onUpdate: function (v, t) {
                        entity.alpha = v;
                    }
                });

                new Tween({
                    from: 0,
                    to: 1,
                    'in': 20,
                    ease: Tween.EASEOUTBACK,
                    onUpdate: function (v, t) {
                        entity.scale.x = v;
                        entity.scale.y = v;
                    }
                });
            },
            destroy: function (data) {},
            update: function (data) {
                // console.log('popup');
                // entity.position.x++;
            },
            draw: function (data) {}
        };
        var entity = new Entity({
            z: 0,
            name: 'consumablePopup',
            family: [''],
            position: settings.position,
            updateWhenPaused: 0,
            components: [
                sprite,
                behavior
            ]
        });
        return entity;
    };
});