/**
 * Continue popup, replaces the gameover screen in single player
 * Automatically counts down
 * @moduleName ContinuePopup
 */
bento.define('gui/continuepopup', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'bento/components/nineslice',
    'gui/blurbackground',
    'globals',
    'components/modal',
    'components/anchor',
    'gui/shadowtext',
    'gui/iconbutton',
    'gui/headerbox',
    'modules/localization',
    'managers/character',
    'gui/worldselect/topbar',
    'managers/save',
    'managers/currency',
    'modules/analytics',
    'gui/menus/iap',
    'managers/ai',
    'managers/stats',
    'gui/backuppopup',
    'components/orientationlock',
    'gui/fade',
    'components/viewanalytics',
    'components/backbutton'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    NineSlice,
    BlurBackground,
    Globals,
    Modal,
    Anchor,
    ShadowText,
    IconButton,
    HeaderBox,
    Localization,
    CharacterManager,
    Topbar,
    SaveManager,
    CurrencyManager,
    Analytics,
    IAPMenu,
    AiManager,
    StatsManager,
    BackupPopup,
    OrientationLock,
    Fade,
    ViewAnalytics,
    BackButton
) {
    'use strict';
    var ContinuePopup = function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var currentMission = SaveManager.load('currentMission');
        var calledBackup = SaveManager.load('calledBackup');
        var reviveCost = Globals.gemToRevive(currentMission) * (calledBackup ? 2 : 1);
        var canCallBackup = BackupPopup.canCallBackup();
        var countDownSeconds = 5;
        /**
         * Buttons
         */
        var buttonClose = new IconButton({
            name: 'buttonClose',
            sfx: 'sfx_button',
            imageName: 'gui2/misc/button_small_secondary',
            iconName: canCallBackup ? 'gui2/misc/button_icon_back' : 'gui2/misc/button_icon_home',
            position: new Vector2(-52, 64),
            frameCountX: 1,
            frameCountY: 3,
            onClick: function () {
                // tween out
                buttonClose.setActive(false);
                buttonContinue.setActive(false);
                new Tween({
                    from: 1,
                    to: 0,
                    in: 10,
                    ease: 'linear',
                    onUpdate: function (v, t) {
                        // blurBackground.alpha = v;
                        headerBox.scale.x = v;
                        headerBox.scale.y = v;
                    },
                    onComplete: function () {
                        Analytics.event('Economy', 'spendGemsForRevive', 0, 'no');
                        Analytics.event('Economy', 'spendGemsForRevive_' + (canCallBackup ? 'first' : 'second'), 0, 'no');

                        // can call backup
                        if (canCallBackup) {
                            entity.removeSelf();
                            Bento.objects.attach(new BackupPopup({}));
                        } else {
                            // mark 1st time death
                            if (SaveManager.load('tutorial1stTimeDeath') === 0) {
                                SaveManager.save('tutorial1stTimeDeath', 1);
                            } else
                            if (SaveManager.load('tutorial1stTimeDeath') === 2) {
                                SaveManager.save('tutorial1stTimeDeath', 3);
                            }

                            showGameOver();
                        }
                    }
                });
            },
            components: [
                new BackButton({
                    removeSelf: true,
                    onPressed: function (data) {
                        data.entity.doCallback();
                    }
                })
            ]
        });
        var gemText = new ShadowText({
            z: 0,
            position: new Vector2(0, -1),
            text: reviveCost.toString(),
            font: 'font',
            fontSize: 22,
            fontColor: '#ffffff',
            align: 'left',
            textBaseline: 'middle',
            maxWidth: 48,
            linebreaks: false
        });
        var currencyIcon = new Entity({
            name: 'currencyIcon',
            position: new Vector2(-16, 0),
            components: [
                new Sprite({
                    imageName: 'gui/gem',
                    originRelative: new Vector2(0.5, 0.5)
                })
            ]
        });
        var buttonInfo = new Entity({
            name: 'buttonInfo',
            components: [
                currencyIcon,
                gemText
            ]
        });
        var buttonContinue = new IconButton({
            z: 0,
            name: 'item',
            sfx: 'sfx_button',
            position: new Vector2(24, 64),
            imageName: 'gui2/misc/button_large_primary',
            icon: buttonInfo,
            frameCountX: 1,
            frameCountY: 3,
            animations: {
                'up': {
                    frames: [0]
                },
                'down': {
                    frames: [1]
                },
                'inactive': {
                    frames: [0]
                },
            },
            float: false,
            updateWhenPaused: 0,
            onClick: function () {
                if (CurrencyManager.canSpendGems(reviveCost)) {
                    Analytics.event('Economy', 'spendGemsForRevive', 1, 'yes');
                    Analytics.event('Economy', 'spendGemsForRevive_' + (canCallBackup ? 'first' : 'second'), 1, 'yes');

                    buttonContinue.setActive(false);
                    buttonClose.setActive(false);
                    CurrencyManager.spendGems(reviveCost, 'revive');
                    SaveManager.saveToCloud();
                    EventSystem.fire('revivedPlayer');
                    Utils.forEach(Bento.objects.getByFamily('player'), function (player, i, l, breakLoop) {
                        if (!player.isRemote) {
                            player.getComponent('stateMachine', function (stateMachine) {
                                stateMachine.setState('getUp');
                            });
                            player.getComponent('punchable', function (punchable) {
                                punchable.resetHealth();
                            });
                            AiManager.distributeAiTasks();
                            SaveManager.save('hasRevived', true);
                            breakLoop();
                        }
                    });
                    Bento.audio.stopSound('sfx_revive');
                    Bento.audio.playSound('sfx_revive');

                    // tween out
                    new Tween({
                        from: 1,
                        to: 0,
                        in: 10,
                        ease: 'linear',
                        onUpdate: function (v, t) {
                            blurBackground.alpha = v;
                            headerBox.scale.x = v;
                            headerBox.scale.y = v;
                        },
                        onComplete: function () {
                            entity.removeSelf();
                        }
                    });
                    // restore music
                    new Tween({
                        from: 0.25,
                        to: 1,
                        in: 30,
                        onUpdate: function (v, t) {
                            Bento.audio.setVolume(v, Globals.currentMusic);
                        }
                    });
                } else {
                    // not enough gems???? go to shop
                    var iapMenu = new IAPMenu({
                        z: entity.z + 1,
                        scrollTo: 'gems',
                        onComplete: function () {
                            // stop timer if user has enough gems now
                            if (CurrencyManager.canSpendGems(reviveCost)) {
                                // stop timer
                                behavior.isCounting = false;
                            }
                        }
                    });
                    Bento.objects.attach(iapMenu);
                    // Analytics.event('Economy', 'spendGemsForRevive', 1, 'notEnough');
                }
            }
        });
        /**
         * Tired Character
         */
        var character = CharacterManager.getSelected();
        var characterSprite = new Sprite({
            spriteSheet: character + '/tired'
        });
        var characterEntity = new Entity({
            name: 'character',
            position: new Vector2(0, 40),
            components: [
                characterSprite
            ]
        });
        /**
         * Timer
         */
        var timerText = new Text({
            position: new Vector2(0, -20),
            text: countDownSeconds.toString(),
            font: 'font',
            fontSize: 32,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
        });
        /**
         * Background
         */
        var blurBackground = new BlurBackground({
            darken: 0.3,
            blurriness: 15
        });
        var anchor = new Anchor({
            screenPosition: new Vector2(0.5, 0.5),
            offset: new Vector2(0, 0),
            onComplete: function () {}
        });

        var headerBox = new HeaderBox({
            name: 'box',
            size: new Rectangle(0, 0, 180, 140),
            title: Localization.getText('continue'),
            windowImage: 'gui2/misc/splash_background_9slice',
            position: new Vector2(0, 0),
            components: [
                anchor,
                buttonContinue,
                buttonClose,
                characterEntity,
                timerText
            ]
        });
        /**
         * Other componentes
         */
        var modal = new Modal({});
        /**
         * Main entity
         */
        var behavior = {
            name: 'behaviorComponent',
            isCounting: true,
            start: function (data) {
                // count down
                var count = countDownSeconds;
                var dropDownText = function () {
                    count -= 1;
                    if (count < 0 && entity.isAdded && behavior.isCounting) {
                        // count over
                        buttonClose.doCallback();
                        return;
                    }
                    timerText.setText(count.toString());
                    new Tween({
                        from: 3,
                        to: 1,
                        in: 20,
                        ease: 'quadratic',
                        onUpdate: function (v, t) {
                            timerText.scale.x = v;
                            timerText.scale.y = v;
                        },
                        onComplete: function () {
                            if (entity.isAdded && behavior.isCounting) {
                                Bento.audio.playSound('sfx_card');
                            }
                            // shake
                            new Tween({
                                from: 1,
                                to: 0,
                                in: 10,
                                onUpdate: function (v, t) {
                                    timerText.transform.x = Utils.getRandomRangeFloat(-v, v);
                                    timerText.transform.y = Utils.getRandomRangeFloat(-v, v);
                                },
                                onComplete: function () {
                                    timerText.transform.x = 0;
                                    timerText.transform.y = 0;
                                }
                            });
                        }
                    });
                    // opacity tween
                    new Tween({
                        from: 0,
                        to: 1,
                        in: 20,
                        ease: 'linear',
                        onUpdate: function (v, t) {
                            timerText.alpha = v;
                        }
                    });

                    if (entity.isAdded && behavior.isCounting) {
                        Utils.timeout(60, dropDownText);
                    }
                };
                var analytics = function () {
                    var chosenDifficulty = SaveManager.load('chosenDifficulty');
                    var currentSection = SaveManager.load('currentSection');
                    var currentLevel = StatsManager.getLevel(CharacterManager.getSelected());
                    Analytics.event('Game', 'deathsTotal');
                    Analytics.event('Deaths', 'mission_' + currentMission + '_difficulty_' + chosenDifficulty, currentLevel, currentSection.toString());
                    Analytics.event('Deaths', 'section_' + currentMission + '_' + currentSection + '_difficulty_' + chosenDifficulty, currentLevel);
                };
                analytics();

                // is the player allowed to continue in the first place?
                if (SaveManager.load('hasRevived')) {
                    if (canCallBackup) {
                        entity.removeSelf();
                        Bento.objects.attach(new BackupPopup({}));
                    } else {
                        showGameOver();
                    }
                    return;
                }
                // tween in
                new Tween({
                    from: 0,
                    to: 1,
                    in: 30,
                    ease: 'linear',
                    onUpdate: function (v, t) {
                        blurBackground.alpha = v;
                    }
                });
                // tower or (rad) event
                if (SaveManager.load('isTower') || SaveManager.load('isEvent') ) {
                    showGameOver(true);
                    return;
                }

                new Tween({
                    from: 0,
                    to: 1,
                    in: 20,
                    ease: 'elastic',
                    decay: 5, // for 'exponential' and 'elastic'
                    oscillations: 1, // for 'elastic', 'sin' and 'cos'
                    onStart: function () {},
                    onUpdate: function (v, t) {
                        headerBox.scale.x = v;
                        headerBox.scale.y = v;
                    },
                    onComplete: function () {
                        dropDownText();
                    }
                });

                // turn volume down
                new Tween({
                    from: 1,
                    to: 0.25,
                    in: 30,
                    onUpdate: function (v, t) {
                        Bento.audio.setVolume(v, Globals.currentMusic);
                    }
                });
            },
            destroy: function (data) {},
            update: function (data) {},
            draw: function (data) {}
        };
        var showGameOver = function (hideContinueButton) {
            // opacity tween
            new Tween({
                from: 0,
                to: 1,
                in: 40,
                ease: 'linear',
                onUpdate: function (v, t) {
                    timerText.alpha = v;
                }
            });
            behavior.isCounting = false;
            headerBox.removeSelf();

            BackupPopup.showGameOver(entity, hideContinueButton);
        };
        var entity = new Entity({
            z: Globals.Layers.PAUSE,
            name: 'continuePopup',
            float: true,
            components: [
                modal,
                blurBackground,
                headerBox,
                new Topbar({}),
                behavior,
                new ViewAnalytics({
                    name: 'continuePopup'
                })
            ]
        });
        return entity;
    };

    return ContinuePopup;
});