/**
 * Counter with a coin on the left
 * @moduleName CurrencyCounter
 */
bento.define('gui/currencycounter', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'globals',
    'gui/shadowtext'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Globals,
    ShadowText
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        // the currency sprite
        var sprite = new Sprite({
            imageName: settings.imageName
        });
        // clickable coin
        var coin = new Entity({
            z: settings.z || Globals.Layers.GUI,
            name: 'coin',
            position: new Vector2(0, 8),
            originRelative: new Vector2(0.5, 0.5),
            float: true,
            components: [
                sprite
            ]
        });
        var test = {
            name: 'coinBehavior',
            update: function () {
                // TODO: more efficent if coin position updates when counter updates
                container.addValue(10);
            }
        };
        var counter = new ShadowText({
            name: 'counter',
            position: new Vector2(0, -2),
            font: 'font',
            fontSize: 16,
            fontColor: '#ffffff',
            align: settings.alignment || 'right',
            float: true,
            text: (settings.value).toString()
        }).extend({
            setValue: function (value) {
                counter.setText(value.toString());
            }
        });
        var container = new Entity({
            z: settings.z || Globals.Layers.GUI,
            name: settings.name || 'currencyCounter',
            position: settings.position || new Vector2(0, 0),
            float: settings.float,
            components: [
                coin,
                counter
            ]
        });
        var updateCoin = function () {
            if (settings.alignment === 'center') {
                coin.position.x = counter.position.x - counter.dimension.width / 2 - 8;
            } else {
                coin.position.x = counter.position.x - counter.dimension.width - 8;
            }
        };

        // counter.scale.x = 0.75;
        // counter.scale.y = 0.75;

        container.extend({
            setValue: function (value) {
                counter.setValue(value);
                updateCoin();
            },
            addValue: function (value) {
                counter.addValue(value);
                updateCoin();
            },
            getValue: counter.getValue,
        });

        updateCoin();

        return container;
    };
});