/**
 * Horizontal bar with number in it displaying quest progress
 * @moduleName ProgressBar
 */
bento.define('gui/dailyquest/progressbar', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'globals',
    'gui/shadowtext',
    'managers/character'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Globals,
    ShadowText,
    CharacterManager
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var hideText = settings.hideText;
        var frameImage = settings.frameImage || 'gui/upgrade/tokenbar-frame';
        var barImage = settings.barImage || 'gui/upgrade/tokenbar';
        var lerpFactor = settings.lerp || 0.5;
        var tween = settings.tween || false;
        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {},
            destroy: function (data) {},
            update: function (data) {},
            draw: function (data) {}
        };
        var sprite = new Sprite({
            imageName: frameImage,
            originRelative: new Vector2(0.5, 0.5)
        });
        var barBehavior = {
            name: 'barBehavior',
            image: null,
            glow: 0,
            width: 0,
            realWidth: 0,
            start: function () {
                barBehavior.image = Bento.assets.getImage(barImage);
            },
            update: function () {
                if (tween) {
                    // interpolate towards realWidth
                    barBehavior.realWidth += (barBehavior.width - barBehavior.realWidth) * lerpFactor;
                }
            },
            draw: function (data) {
                var image = barBehavior.image;
                var width = barBehavior.width;

                if (tween) {
                    width = barBehavior.realWidth;
                }

                if (width <= 0) {
                    return;
                } else if (width > 1) {
                    width = 1;
                }

                data.renderer.drawImage(
                    image,
                    0,
                    0,
                    width * image.width,
                    image.height,
                    image.width / -2,
                    image.height / -2,
                    width * image.width,
                    image.height
                );
            }
        };
        var bar = new Entity({
            name: 'bar',
            position: new Vector2(0, 0),
            components: [
                barBehavior
            ]
        });

        var text = new ShadowText({
            position: new Vector2(0, -2),
            text: '0/0',
            font: 'font',
            fontSize: 16,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            maxWidth: 38,
            maxHeight: 16
        });

        var entity = new Entity({
            name: 'progressBar',
            position: settings.position,
            components: [
                sprite,
                behavior,
                bar,
                text
            ]
        });
        entity.extend({
            setProgress: function (data, setRealWidth) {
                if (!hideText) {
                    text.setText(data.count + '/' + data.goal);
                }
                barBehavior.width = data.count / data.goal;
                if (setRealWidth) {
                    barBehavior.realWidth = barBehavior.width;
                }
            }
        });

        if (hideText) {
            text.removeSelf();
        }

        if (settings.quest) {
            entity.setProgress(settings.quest);
        }

        return entity;
    };
});