/**
 * frame used by daily quest menu to display todays quests
 * @moduleName QuestFrame
 */
bento.define('gui/dailyquest/questframe', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'gui/shadowtext',
    'modules/localization',
    'gui/dailyquest/progressbar',
    'components/glow',
    'managers/currency',
    'managers/save',
    'gui/coinreward',
    'globals',
    'gui/itemget',
    'managers/quest',
    'gui/watchrewardedad',
    'modules/analytics'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    ShadowText,
    Localization,
    ProgressBar,
    Glow,
    CurrencyManager,
    SaveManager,
    CoinReward,
    Globals,
    ItemGet,
    QuestManager,
    WatchRewardedAd,
    Analytics
) {
    'use strict';
    return function (settings) {
        // text
        var questText = new ShadowText({
            z: 0,
            position: new Vector2(-75, 0),
            text: Localization.getText('quests')[settings.quest.quest.toString()].replace('{NUMBER}', settings.quest.goal),
            font: 'font',
            fontSize: 16,
            fontColor: '#ffffff',
            align: 'left',
            textBaseline: 'top',
            ySpacing: 0,
            lineWidth: 0,
            maxWidth: 100,
            maxHeight: 44,
            linebreaks: true,
            strokeStyle: '#000000',
            innerStroke: false,
            pixelStroke: false // workaround for Cocoon bug
        });

        questText.position.y = -questText.dimension.height / 2;

        // reward
        var currency = new Entity({
            name: 'currency',
            position: new Vector2(0, 0),
            components: [
                new Sprite({
                    imageName: "gui/" + settings.quest.currency,
                    originRelative: new Vector2(0.5, 0.5)
                })
            ]
        });

        var amount = new Text({
            z: 0,
            position: new Vector2(0, -2),
            text: settings.quest.amount > 1 ? settings.quest.amount.toString() : '',
            font: 'font',
            fontSize: 16,
            fontColor: '#ffdf00',
            align: 'left',
            textBaseline: 'middle',
            lineWidth: 1,
            strokeStyle: '#3e1a00',
            pixelStroke: true,
        });

        var reward = new Entity({
            name: 'reward ',
            position: new Vector2(57, -11),
            components: [
                currency,
                amount,
            ]
        });

        // position reward elements
        var rewardWidth = currency.dimension.width + amount.dimension.width + 2;
        currency.position.x = -rewardWidth / 2 + currency.dimension.width / 2;
        amount.position.x = rewardWidth / 2 - amount.dimension.width;

        // progress
        var progress = new ProgressBar({
            position: new Vector2(55, 14),
            quest: settings.quest
        });

        // button
        var claimText = new ShadowText({
            position: new Vector2(0, -3),
            text: Localization.getText('claim'),
            font: 'font',
            fontSize: 16,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            linebreaks: false,
            maxWidth: 38,
            maxHeight: 16
        });

        var glow = new Glow({
            average: 0.25,
            amplitude: 0.25,
        });

        var claimButton = new ClickButton({
                z: 0,
                name: 'claimButton',
                sfx: 'sfx_button',
                imageName: 'gui/buttons/claim',
                frameCountX: 2,
                frameCountY: 1,
                position: new Vector2(55, 12),
                float: false,
                updateWhenPaused: 0,
                onButtonDown: function () {
                    claimText.position.y = -2;
                },
                onClick: function () {
                    if (settings.quest.currency === 'coin') {
                        // claim currency
                        CurrencyManager.addCoins(parseInt(settings.quest.amount));
                        Bento.objects.attach(new CoinReward({
                            z: Globals.WorldSelectLayers.QUESTS,
                            coins: parseInt(settings.quest.amount)
                        }));
                    } else if (settings.quest.currency === 'gem') {
                        CurrencyManager.addGems(settings.quest.amount);
                        var getGems = new ItemGet({
                            z: Globals.WorldSelectLayers.QUESTS + 1,
                            item: 'currency/gem',
                            amount: parseInt(settings.quest.amount)
                        });
                        getGems.getComponent('buttonOk', function (buttonOk) {
                            buttonOk.visible = false;
                            buttonOk.setActive(false);
                            Utils.timeout(100, function () {
                                buttonOk.doCallback();
                            }, Bento.objects.isPaused() + 2);
                        });
                        Bento.objects.attach(getGems);
                    }
                    if (settings.analytics) {
                        settings.analytics(settings.quest);
                    }
                    setClaimed();
                    // claim quest
                    var activeQuests = QuestManager.getQuests();
                    var index = activeQuests.indexOf(settings.quest);
                    // find index
                    Utils.forEach(activeQuests, function (quest, i, l, breakLoop) {
                        if (quest.quest === settings.quest.quest) {
                            index = i;
                            breakLoop();
                        }
                    });
                    // if(index > -1) {
                    // activeQuests.splice(index, 1);
                    activeQuests[index].claimed = true;
                    QuestManager.saveQuests(activeQuests);
                    // }
                    if (claimButton.onComplete) {
                        claimButton.onComplete();
                    }
                    EventSystem.fire('questClaimed');
                },
                onButtonUp: function () {
                    claimText.position.y = -3;
                }
            }).attach(claimText)
            .attach(glow);

        var checkMark = new Entity({
            name: 'checkMark',
            position: new Vector2(54, 0),
            components: [
                new Sprite({
                    imageName: 'gui/dailyquest/checkmark',
                    originRelative: new Vector2(0.5, 0.5)
                })
            ]
        });

        var frameSprite = new Sprite({
            imageName: 'gui/dailyquest/questframe',
            originRelative: new Vector2(0.5, 0.5),
            frameCountX: 1,
            frameCountY: 2,
            animations: {
                default: {
                    speed: 0,
                    frames: [0]
                },
                claimed: {
                    speed: 0,
                    frames: [1]
                }
            }
        });

        var setClaimed = function () {
            frameSprite.setAnimation('claimed');
            frame.remove(progress);
            frame.remove(reward);
            frame.remove(claimButton);

            // set text color
            questText.setText(Localization.getText('quests')[settings.quest.quest.toString()].replace('{NUMBER}', settings.quest.goal), {
                fontColor: '#86858c'
            });
            if (!checkMark.parent) {
                frame.attach(checkMark);
            }
        };

        var urlSchemes = {
            'twitter': 'twitter://user?screen_name=LuckyKatStudios',
            'facebook': 'fb://profile/466268083523876',
            'weibo': 'weibo://userinfo?uid=5564956016',
            'instagram': 'instagram://user?username=luckykatgames',
        };

        var links = {
            'twitter': 'https://twitter.com/LuckyKatStudios',
            'facebook': 'https://www.facebook.com/luckykatstudios',
            'weibo': 'http://www.weibo.com/LuckyKatStudios',
            'instagram': 'https://www.instagram.com/_u/luckykatgames/'
        };

        var getSpecialButton = function (type) {
            var onClick;
            var imageName;
            if (type === 'facebook' ||
                type === 'instagram' ||
                type === 'twitter' ||
                type === 'weibo'
            ) {
                imageName = 'gui/buttons/' + type;
                // set click function for social media
                onClick = function () {
                    // open social media
                    if (Utils.isCocoonJs() && window.Cocoon) {
                        // add pause and resume events, the pause event just saves the type it should receive.
                        // completing the quest is handled bij QuestManager.receiveQuest

                        var onPause = function () {
                            // player left the game to go to twitter/facebook
                            var socialMedia = SaveManager.load('socialMedia');
                            socialMedia.receive = type;
                            SaveManager.save('socialMedia', socialMedia);
                        };

                        var onResume = function () {
                            QuestManager.receiveQuest();
                            onRemove();
                        };

                        var onRemove = function () {
                            EventSystem.off('cocoonSuspended', onPause);
                            EventSystem.off('onResume', onResume);
                            EventSystem.off('onRemove', onResume);
                        };

                        EventSystem.on('cocoonSuspended', onPause);
                        EventSystem.on('onResume', onResume);
                        EventSystem.on('removeQuestFrame', onRemove);
                        if (Utils.isNativeIos()) {
                            window.Cocoon.App.openURL(urlSchemes[type]);
                        } else if (Utils.isNativeAndroid()) {
                            window.Cocoon.App.openURL(links[type]);
                        }
                    } else {
                        var socialStatus = SaveManager.load('socialMedia');
                        socialStatus.receive = type;
                        SaveManager.save('socialMedia', socialStatus);
                        QuestManager.receiveQuest();
                    }
                };
            } else if (type === 'adQuest') {
                imageName = 'gui/buttons/ad-borderless';
                // set click function for ad quest
                onClick = function () {
                    WatchRewardedAd(function (completed) {
                        if (completed) {
                            EventSystem.fire('quest-adQuest', {
                                count: 1
                            });
                            EventSystem.fire('updateDailyQuest');
                            Analytics.event('Ads', 'watch_5_video_quest');
                        }
                    });
                };
            }

            return new ClickButton({
                name: 'specialButton',
                sfx: 'sfx_button',
                imageName: imageName,
                position: new Vector2(-24, -9),
                frameCountX: 4,
                frameCountY: 1,
                animations: {
                    'up': {
                        speed: 0.03,
                        frames: [0, 1]
                    },
                    'down': {
                        frames: [2]
                    },
                    'inactive': {
                        frames: [3]
                    }
                },
                onClick: onClick
            });
        };

        var specialButton;

        var buttonFrame = function () {
            frameSprite.spriteImage = Bento.assets.getImage('gui/dailyquest/videoframe');
            questText.setText(Localization.getText('quests')[settings.quest.quest.toString()].replace('{NUMBER}', settings.quest.goal), {
                fontColor: settings.quest.claimed ? '#86858c' : '#FFFFFF',
                maxWidth: 100,
                maxHeight: 18,
                align: 'center',
                textBaseline: 'middle',
            });
            questText.position.x = -23;
            questText.position.y = 14;

            specialButton = getSpecialButton(settings.quest.special);
            frame.attach(specialButton);

            if (settings.quest.count >= settings.quest.goal) {
                specialButton.setActive(false);
            }
        };

        var updateFrame = function () {
            // update quest
            var quest = QuestManager.getQuest(settings.quest.quest);
            if (!quest) {
                // line 384: TypeError: undefined is not an object (evaluating 'settings.quest.count') Object: bottomBar
                return;
            }
            settings.quest = quest;

            if (settings.quest.count < settings.quest.goal) {
                // show progress
                progress.setProgress(settings.quest);
            } else {
                if (!settings.quest.claimed) {
                    // claim reward
                    frame.remove(progress);
                    if (!claimButton.parent) {
                        frame.attach(claimButton);
                    }
                    if (specialButton) {
                        specialButton.setActive(false);
                    }
                } else {
                    setClaimed();
                    if (specialButton) {
                        specialButton.setActive(false);
                    }
                }
            }
        };

        var behavior = {
            name: 'questframeBehavior',
            start: function () {
                // type of quest frame
                if (settings.quest.count < settings.quest.goal) {
                    // show progress
                    frame.attach(progress);
                } else {
                    if (!settings.quest.claimed) {
                        // claim reward
                        frame.attach(claimButton);
                    } else {
                        setClaimed();
                    }
                }

                // special frames
                if (Utils.isDefined(settings.quest.special)) {
                    // standard behavior, make a frame with a button with text below it
                    buttonFrame();
                }

                // set dimensions for use in scrolling list
                frame.dimension = new Rectangle(-80, -28, 160, 57);

                frame.updateFrame = updateFrame;
            },
            destroy: function () {
                EventSystem.fire('removeQuestFrame');
            }
        };

        var frame = new Entity({
            name: 'questFrame',
            position: new Vector2(0, 0),
            components: [
                behavior,
                frameSprite,
                questText,
                reward,
            ]
        });

        return frame;
    };
});