/**
 * Popup that asks the player to trade gems for energy
 * @moduleName EnergyForGemPopup
 */
bento.define('gui/energyforgempopup', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'gui/dialog',
    'modules/localization',
    'gui/shadowtext',
    'gui/worldselect/topbarsimple',
    'globals',
    'managers/currency',
    'modules/analytics',
    'gui/menus/iap',
    'managers/character',
    'components/sparkles',
    'gui/sparkleburst',
    'components/blink',
    'managers/save',
    'managers/time',
    'gui/watchrewardedad',
    'gui/iconbutton'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Dialog,
    Localization,
    ShadowText,
    TopBarSimple,
    Globals,
    CurrencyManager,
    Analytics,
    IAPMenu,
    CharacterManager,
    Sparkles,
    SparkleBurst,
    Blink,
    SaveManager,
    TimeManager,
    WatchRewardedAd,
    IconButton
) {
    'use strict';
    var EnergyForGemPopup = function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var character = CharacterManager.getSelected() || settings.character;
        var buttonToClick = settings.buttonToClick; // button to click after buying
        var onComplete = settings.onBoughtEnergy;
        var onWatchedAd = settings.onWatchedAd;
        var forceGem = settings.forceGem;

        // ask for ad before asking for gems
        if (EnergyForGemPopup.canWatchAd() && !forceGem) {
            return EnergyForGemPopup.getAdPopup(onWatchedAd, settings);
        }

        var popup = new Dialog({
            z: settings.z || Globals.Layers.DIALOGS,
            name: 'energyForGems',
            ignoreAnalytics: true,
            blurriness: 4,
            size: new Rectangle(0, 0, 180, 160),
            title: Localization.getText('notEnoughEnergyTitle'),
            canClose: true,
            onComplete: function () {
                if (hasBought && buttonToClick) {
                    // do callback of button
                    buttonToClick.doCallback();
                }
                if (onComplete) {
                    onComplete(hasBought);
                }
            },
            onClose: function () {
                Analytics.event('Economy', 'energyForGems', 0, 'yes');
            }
        });
        var showAnimation = function (onAnimationComplete) {
            var finished = false;
            popup.getComponent('windowBox', function (windowBox) {
                windowBox.getComponent('btnClose', function (buttonClose) {
                    buttonClose.removeSelf();
                });
            });
            buttonBuy.removeSelf();
            text.removeSelf();
            new Tween({
                from: 24,
                to: 0,
                in: 30,
                ease: 'easeInBack',
                onUpdate: function (v, t) {
                    characterEntity.position.x = -v;
                    energyEntity.position.x = v;
                },
                onComplete: function () {
                    Bento.audio.stopSound('sfx_getenergy');
                    Bento.audio.playSound('sfx_getenergy');

                    energyEntity.removeSelf();
                    characterEntity.attach(new Blink({}));
                    characterEntity.attach(new SparkleBurst({}));
                    characterSprite.setSpriteSheet(character + '/victory', function () {
                        if (!finished) {
                            finished = true;
                            popup.close();
                        }
                    });
                }
            });
        };
        var hasBought = false;
        var gemsNeeded = Globals.gemToEnergy;
        var topbar = new TopBarSimple({});
        var cost = new ShadowText({
            z: 0,
            position: new Vector2(0, -1),
            text: gemsNeeded.toString(),
            font: 'font',
            fontSize: 22,
            fontColor: '#ffffff',
            align: 'left',
            textBaseline: 'middle',
            maxWidth: 48,
            linebreaks: false
        });
        var currencyIcon = new Entity({
            name: 'currencyIcon',
            position: new Vector2(-16, 0),
            components: [
                new Sprite({
                    imageName: 'gui/gem',
                    originRelative: new Vector2(0.5, 0.5)
                })
            ]
        });
        var buttonInfo = new Entity({
            name: 'buttonInfo',
            components: [
                currencyIcon,
                cost
            ]
        });
        var buttonBuy = new IconButton({
            name: 'buttonBuy',
            sfx: 'sfx_button',
            position: new Vector2(0, 64),
            imageName: 'gui2/misc/button_large_primary',
            icon: buttonInfo,
            iconAlpha: 0.5,
            frameCountX: 1,
            frameCountY: 3,
            onClick: function () {
                // buy coins with the gems needed
                if (CurrencyManager.canSpendGems(gemsNeeded)) {
                    CurrencyManager.spendGems(gemsNeeded, 'energy');
                    CharacterManager.setEnergy(character, CharacterManager.getEnergyData(character).maximum);
                    SaveManager.saveToCloud();
                    showAnimation(function () {
                        popup.close();
                    });
                    hasBought = true;
                    Bento.audio.stopSound('sfx_buy');
                    Bento.audio.playSound('sfx_buy');
                    Analytics.event('Economy', 'energyForGems', 1, 'yes');
                } else {
                    // not enough gems???? go to shop
                    var iapMenu = new IAPMenu({
                        z: popup.z + 1,
                        scrollTo: 'gems'
                    });
                    Bento.objects.attach(iapMenu);
                    Analytics.event('Economy', 'energyForGems', 1, 'notEnough');
                }
            },
            onButtonDown: function () {
                EventSystem.fire('preventScroll');
            }
        });
        var energy = SaveManager.load('energy-boy');
        var maxEnergy = energy.maximum || 20;
        var text = new ShadowText({
            position: new Vector2(0, 28),
            text: Localization.getText('buyMissingEnergy').replace('{NUMBER}', maxEnergy),
            font: 'font',
            fontSize: 16,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            maxWidth: 160 - 16,
            maxHeight: 16
        });
        var characterSprite = new Sprite({
            spriteSheet: character + '/tired'
        });
        var characterEntity = new Entity({
            name: 'character',
            position: new Vector2(-24, -4 + 16),
            components: [
                characterSprite
            ]
        });
        var energyEntity = new Entity({
            name: 'energy',
            position: new Vector2(24, -24 + 16),
            components: [
                new Sprite({
                    imageName: 'gui/energy-large',
                    originRelative: new Vector2(0.5, 0.5)
                }),
                new Sparkles({})
            ]
        });

        popup.getComponent('windowBox', function (windowBox) {
            windowBox.attach(buttonBuy);
            windowBox.attach(text);
            windowBox.attach(characterEntity);
            windowBox.attach(energyEntity);
        });

        popup.attach(topbar);

        return popup;
    };

    EnergyForGemPopup.canWatchAd = function () {
        var dailyAds = SaveManager.load('adsForEnergy');
        // check time
        var day = dailyAds.day;
        var today = Math.floor(TimeManager.getTime() / 1000 / 60 / 60 / 24);

        // is the day different? reset the number of ads watched
        if (today > day) {
            dailyAds.adsWatched = 0;
            dailyAds.day = today;
            SaveManager.save('adsForEnergy', dailyAds);
        }

        // allow 2 ads per day
        if (dailyAds.adsWatched < Globals.maxAdsEnergy) {
            return true;
        } else {
            return false;
        }
    };

    EnergyForGemPopup.getAdPopup = function (onComplete, settings) {
        var declined = false;
        var dialog = new Dialog({
            name: 'watchAdForEnergy',
            title: Localization.getText('notEnoughEnergyTitle'),
            text: Localization.getText('notEnoughEnergyBody'),
            yesImage: 'gui2/misc/button_small_primary',
            yesIconImage: 'gui2/misc/button_icon_watchad',
            yesFrameCountX: 1,
            yesFrameCountY: 3,
            type: 'yes',
            canClose: true,
            onYes: function () {
                declined = false;
            },
            onClose: function () {
                // wait for dialog to move out
                declined = true;
            },
            onComplete: function () {
                var gemDialog;
                if (declined) {
                    // ask for gem afterwards
                    settings.forceGem = true;

                    gemDialog = new EnergyForGemPopup(settings);
                    Bento.objects.attach(gemDialog);
                } else {
                    Analytics.event('Ads', 'watch_for_energy');
                    WatchRewardedAd(function (completed) {
                        if (!completed) {
                            return;
                        }
                        var dailyAds = SaveManager.load('adsForEnergy');
                        dailyAds.adsWatched += 1;
                        SaveManager.save('adsForEnergy', dailyAds);
                        onComplete();
                    });
                }
            }
        });
        return dialog;
    };

    return EnergyForGemPopup;
});