/**
 * Equipment Slots (consumable or non-consumable equipment)
 * @moduleName EquipmentSlots
 */
bento.define('gui/equipmentslots', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'managers/inventory',
    'gui/itemslot',
    'gui/inventorypopup',
    'globals',
    'gui/iteminfo',
    'gui/centeredlist'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    InventoryManager,
    ItemSlot,
    InventoryPopup,
    Globals,
    ItemInfo,
    CenteredList
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var type = settings.type || 'nonConsumable';
        var hideSlot2 = settings.hideSlot2 || false;
        var iconImage = settings.iconImage || '';
        var isNonConsumables = (type === 'nonConsumable');
        var slotSize = settings.slotSize || 'large';
        var get = isNonConsumables ? InventoryManager.getEquipped : InventoryManager.getEquippedConsumables;
        var set = isNonConsumables ? InventoryManager.setEquipped : InventoryManager.setEquippedConsumable;
        var character = settings.character;
        var stats = settings.stats;
        var onClosed = settings.onClosed;
        var equipmentSlots = new CenteredList({
            spacing: 4,
            slotSize: 'small',
            position: new Vector2(16, 0)
        });
        var equipmentIcon = new Entity({
            name: 'icon',
            position: new Vector2(-48, 0),
            components: [
                new Sprite({
                    imageName: iconImage,
                    originRelative: new Vector2(0.5, 0.5)
                })
            ]
        });
        var equipment = new Entity({
            name: 'equipment',
            position: settings.position,
            components: [
                // new Sprite({
                //     imageName: 'gui/backbar',
                //     originRelative: new Vector2(0.5, 0.5)
                // }),
                equipmentIcon,
                equipmentSlots
            ].concat(settings.components || [])
        });
        var slots = [];
        var makeSlot = function (item, slotContainer) {
            var slot = new ItemSlot({
                clickable: true,
                slotSize: slotSize,
                item: item,
                onSelected: function () {
                    var inventoryPopup = new InventoryPopup({
                        z: Globals.WorldSelectLayers.EQUIPMENT + 1,
                        filter: isNonConsumables ? 'equipment' : ['consumables', 'weapons'],
                        title: isNonConsumables ? 'equipmentTitle' : 'itemsTitle',
                        onComplete: function (selectedSlot) {
                            if (selectedSlot && selectedSlot.key) {
                                setItem(selectedSlot.key);

                                if (onClosed) {
                                    onClosed();
                                }
                            }
                        }
                    });

                    slot.deselect();
                    if (slot.key) {
                        // show item info and return
                        // show item info
                        var itemInfo = new ItemInfo({
                            item: slot.key,
                            onYes: function () {}
                        });
                        Bento.objects.attach(itemInfo);
                        return;
                    }
                    Bento.objects.attach(inventoryPopup);
                },
                onDeselected: function () {}
            });
            var setItem = function (key) {
                var idx = slotContainer.getIndex(slot);
                var current;
                slot.setItem({
                    key: key,
                    amount: 1
                });
                slot.attach(removeButton);

                // check for currently equipped
                current = get(character)[idx];
                // put it back into inventory
                if (current) {
                    InventoryManager.addItem(current.key, 1);
                }

                // remove item from inventory
                InventoryManager.addItem(key, -1);
                // set to equipped
                set(idx, key, character);

                if (stats) {
                    stats.updateStats();
                }
            };
            var removeButton = new ClickButton({
                z: 0,
                name: 'removeButton',
                sfx: 'sfx_button',
                imageName: slotSize === 'new' ? 'gui2/loadout/delete_left_item' : 'gui/buttons/exit-small',
                frameCountX: 1,
                frameCountY: 2,
                position: new Vector2(14, -22),
                float: false,
                updateWhenPaused: 0,
                onClick: function () {
                    var idx = slotContainer.getIndex(slot);
                    var current;
                    if (slot.key) {
                        // return item to inventory
                        slot.setItem(undefined);
                        slot.remove(removeButton);

                        current = get(character)[idx];
                        if (current) {
                            InventoryManager.addItem(current.key, 1);
                        }

                        set(idx, undefined, character);
                        if (stats) {
                            stats.updateStats();
                        }
                    }
                    slot.deselect();
                }
            });
            // add to slots so other functions can reach it
            slots.push({
                slot: slot,
                setItem: setItem
            });
            
            if (slotSize === 'small') {
                removeButton.position.x -= 6;
                removeButton.position.y += 4;
            } else if (slotSize === 'new') {
                if (slot === slots[0].slot) {
                    removeButton.position.x = -24;
                    removeButton.position.y = 0;
                } else {
                    removeButton.position.x = 24;
                    removeButton.position.y = 0;
                    removeButton.scale.x = -1;
                }
            }

            if (item) {
                slot.attach(removeButton);
            }

            slotContainer.addItem(slot);
        };
        var equipped;

        if (type === 'nonConsumable') {
            equipped = InventoryManager.getEquipped(character);
        } else {
            equipped = InventoryManager.getEquippedConsumables();
        }
        makeSlot(equipped[0], equipmentSlots);
        if (!hideSlot2) {
            makeSlot(equipped[1], equipmentSlots);
        }

        // public
        equipment.extend({
            getSlot: function (index) {
                return equipped[index];
            },
            setSlot: function (index, item) {
                var slot = slots[index];
                slot.setItem(item);
            }
        });

        return equipment;
    };
});