/**
 * GUI element allowing player to use the items they equipped in the worldselect screen
 *
 * @moduleName Equipped
 */
bento.define('gui/equipped', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'globals',
    'components/anchor',
    'managers/inventory',
    'managers/item',
    'modules/matchmanager',
    'components/debug',
    'managers/save',
    'gui/sparkleburst',
    'components/eventlistener'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Globals,
    Anchor,
    InventoryManager,
    ItemManager,
    MatchManager,
    Debug,
    SaveManager,
    SparkleBurst,
    EventListener
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var player;
        var isEnabled = true;
        var findControlledPlayer = function () {
            var ii;
            var allPlayers = Bento.objects.getByFamily('player');

            for (ii = 0; ii < allPlayers.length; ii++) {
                if (allPlayers[ii].isRemote === false) {
                    player = allPlayers[ii];
                    break;
                }
            }
        };
        var Slot = function (index) {
            var itemKey = items[index] ? items[index].key : undefined;
            var item;

            var clickable = new Clickable({
                onClick: function (data) {
                    if (!isEnabled) {
                        return;
                    }
                    findControlledPlayer();

                    if (item && player) {
                        if (ItemManager.useItem(itemKey, player, player.position.clone())) {
                            Bento.audio.playSound('sfx_consumable');

                            container.removeItem();

                            MatchManager.sendData({
                                type: 'useItem',
                                data: {
                                    k: itemKey,
                                    n: player.playerId,
                                    x: player.position.x,
                                    y: player.position.y
                                }
                            }, 'general');

                            EventSystem.fire('consumableUsed', {
                                id: index,
                                key: itemKey
                            });
                        }
                    }
                }
            });

            var container = new Entity({
                z: 0,
                name: 'itemSlot',
                position: new Vector2(0, 0),
                boundingBox: new Rectangle(-12, -12, 24, 24),
                components: [
                    clickable
                ]
            }).extend({
                removeItem: function () {
                    if (!item) {
                        return;
                    }
                    container.remove(item);
                    InventoryManager.setEquippedConsumable(index, undefined);
                    item = null;
                }
            });
            var itemSprite;

            if (itemKey) {
                itemSprite = ItemManager.getSprite(itemKey);
                itemSprite.setOriginRelative(new Vector2(0.5, 0.5));
                item = new Entity({
                    z: 0,
                    name: 'item',
                    position: new Vector2(0, 0),
                    components: [
                        itemSprite
                    ]
                });
                // fit item in the smaller slot
                // item.scale.y = 22 / 30;
                // item.scale.x = 22 / 30;
                container.attach(item);
            }

            container.extend({
                getItem: function () {
                    return item;
                }
            });

            return container;
        };
        var items = InventoryManager.getEquippedConsumables();
        var slotA = new Slot(0);
        var slotB = new Slot(1);
        slotA.position.x = 15;
        slotA.position.y = -13;
        slotB.position.x = 15;
        slotB.position.y = 13;

        var disableEvent = new EventListener({
            name: 'disableControls',
            onEvent: function () {
                isEnabled = false;
            }
        });
        var enableEvent = new EventListener({
            name: 'enableControls',
            onEvent: function () {
                isEnabled = true;
            }
        });

        var checkSlots = {
            name: 'checkSlots',
            start: function () {
                findControlledPlayer();

                // no items to begin with
                if (!slotA.getItem() && !slotB.getItem()) {
                    slots.visible = false;
                }
            },
            hide: false,
            update: function () {
                if (!slotA.getItem() && !slotB.getItem() && !checkSlots.hide) {
                    // hide
                    checkSlots.hide = true;
                    new Tween({
                        from: slots.position.x,
                        to: slots.position.x + slots.scale.x * -32,
                        in: 30,
                        ease: 'exponential',
                        decay: 5,
                        onUpdate: function (v, t) {
                            slots.position.x = v;
                        },
                        onComplete: function () {
                            slots.visible = false;
                        }
                    });
                }
            }
        };
        var align = function () {
            var leftHanded = SaveManager.load('leftHanded');
            // flip position
            if (Anchor.isPortrait()) {
                if (leftHanded) {
                    entity.position.x = viewport.width;
                    slots.scale.x = -1;
                    slotA.position.x = -15;
                    slotB.position.x = -15;
                } else {
                    slots.scale.x = 1;
                    slotA.position.x = 15;
                    slotB.position.x = 15;
                }
            } else {
                if (Anchor.isWideScreen() && !Anchor.isPortrait()) {
                    entity.position.x += Anchor.getOffset().x;
                }
                slots.scale.x = -1;
                slotA.position.x = -15;
                slotB.position.x = -15;
            }
        };
        var slots = new Entity({
            name: 'slots',
            components: [
                new Sprite({
                    imageName: 'gui2/hud/items',
                    originRelative: new Vector2(0, 0.5)
                })
            ]
        });
        var anchor = new Anchor({
            portrait: {
                screenPosition: new Vector2(0, 1),
                offset: new Vector2(0, -96)
            },
            landscape: {
                screenPosition: new Vector2(1, 1),
                offset: new Vector2(0, -112)
            },
            onComplete: function () {
                align();
            }
        });
        var entity = new Entity({
            z: Globals.Layers.GUI + 1,
            name: 'equipped',
            updateWhenPaused: 0,
            float: true,
            family: ['gui'],
            components: [
                slots,
                anchor,
                new EventListener({
                    name: 'leftHanded',
                    onEvent: function (data) {
                        anchor.resetPosition();
                    }
                }),
                slotA,
                slotB,
                checkSlots,
                disableEvent,
                enableEvent
            ]
        }).extend({
            slot0: slotA,
            slot1: slotB
        });
        return entity;
    };
});