/**
 * Frog pops up in front to tell you to go to the shop
 * @moduleName FrogPopup
 */
bento.define('gui/frogpopup', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'components/anchor',
    'gui/speechballoon',
    'modules/localization',
    'gui/menus/shop'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Anchor,
    SpeechBalloon,
    Localization,
    ShopMenu
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var speechBalloon = new SpeechBalloon({
            text: Localization.getText('inventoryShop'),
            size: new Rectangle(0, 0, 80, 40),
            bottomPosition: -40,
            bottomFlip: true,
            position: new Vector2(80, -72),
            typewriter: true,
            typeSpeed: 1
        });
        var buttonShop = new ClickButton({
            name: 'shopButtonInventory',
            sfx: 'sfx_button',
            imageName: 'gui/buttons/shop-small',
            frameCountX: 3,
            frameCountY: 1,
            animations: {
                up: {
                    speed: 0.03,
                    frames: [0, 1]
                },
                down: {
                    frames: [2]
                }
            },
            onClick: function () {
                var shopMenu = new ShopMenu({
                    z: settings.z + 1,
                    onComplete: function () {
                        if (settings.onClosedShop) {
                            settings.onClosedShop();
                        }
                    }
                });
                Bento.objects.attach(shopMenu);
            }
        }).attach(new Anchor({
            screenPosition: new Vector2(0, 1),
            offset: new Vector2(27, -24)
        }));
        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {
                var tweenOutDelay = 60;
                buttonShop.removeSelf();
                speechBalloon.scale.x = 0;
                speechBalloon.scale.y = 0;

                // tween in speech balloon
                new Tween({
                    from: 0,
                    to: 1,
                    in: 20,
                    delay: 10,
                    ease: 'elastic',
                    decay: 5,
                    oscillations: 1,
                    onUpdate: function (v, t) {
                        speechBalloon.scale.x = v;
                        speechBalloon.scale.y = v;
                    },
                    onComplete: function () {
                        // tween out
                        new Tween({
                            from: 1,
                            to: 0,
                            in: 30,
                            delay: tweenOutDelay,
                            ease: 'linear',
                            onUpdate: function (v, t) {
                                speechBalloon.alpha = v;
                            },
                            onComplete: function () {
                                speechBalloon.removeSelf();
                            }
                        });

                    }
                });
                // tween in from the right
                new Tween({
                    from: -64,
                    to: 0,
                    in: 30,
                    ease: 'elastic',
                    decay: 5,
                    oscillations: 1,
                    onUpdate: function (v, t) {
                        frog.position.x = v;
                    },
                    onComplete: function () {
                        // tween out
                        new Tween({
                            from: frog.position.y,
                            to: frog.position.y + 64,
                            in: 30,
                            delay: tweenOutDelay,
                            ease: 'easeInBack',
                            onUpdate: function (v, t) {
                                frog.position.y = v;
                            },
                            onComplete: function () {
                                frog.removeSelf();
                            }
                        });

                        // tween in button
                        new Tween({
                            from: buttonShop.position.y + 64,
                            to: buttonShop.position.y,
                            in: 30,
                            delay: tweenOutDelay + 20,
                            ease: 'easeOutBack',
                            onStart: function () {
                                entity.attach(buttonShop);
                            },
                            onUpdate: function (v, t) {
                                buttonShop.position.y = v;
                            },
                            onComplete: function () {

                            }
                        });
                    }
                });
                new Tween({
                    from: -Math.PI / 4,
                    to: 0,
                    in: 30,
                    ease: 'elastic',
                    decay: 5,
                    oscillations: 1,
                    onUpdate: function (v, t) {
                        frog.rotation = v;
                    }
                });

            },
            destroy: function (data) {},
            update: function (data) {},
            draw: function (data) {}
        };
        var anchor = new Anchor({
            screenPosition: new Vector2(0, 1),
            offset: new Vector2(32, 24),
            onComplete: function (parent, isPortrait) {
                if (isPortrait) {
                    frog.scale.x = 2;
                    speechBalloon.position = new Vector2(64, -72);
                } else {
                    frog.scale.x = 1.75;
                    speechBalloon.position = new Vector2(16, -104);
                }

                frog.scale.y = frog.scale.x;
            }
        });
        var sprite = new Sprite({
            imageName: 'gui/shop/frog',
            originRelative: new Vector2(0.5, 1)
        });
        var frog = new Entity({
            name: 'frog',
            components: [
                behavior,
                sprite
            ]
        });
        var frogContainer = new Entity({
            name: 'frogContainer',
            components: [
                frog,
                speechBalloon,
                anchor
            ]
        });
        var entity = new Entity({
            z: settings.z,
            name: 'frogpopup',
            position: settings.position,
            components: [
                buttonShop,
                frogContainer
            ]
        });
        return entity;
    };
});