/**
 * Gacha GUI component. Can show some items and scrolls them. Stops at a defined index after a defined time
 */
bento.define('gui/gacha', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'components/debug'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Debug
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();

        var items = [];
        var endIndex = -1;
        var speed = 0;

        var visibleWidth = settings.visibleWidth || 120;
        var halfWidth = visibleWidth / 2;

        var totalWidth = 0;

        var margin = settings.margin || 10;

        var reposition = function () {
            var ii;

            totalWidth = 0;
            for (ii = 0; ii < items.length; ii++) {
                items[ii].position.x = -halfWidth + totalWidth;
                totalWidth += items[ii].getBoundingBox().width + margin;
            }
        };

        var moveItem = function (item) {
            item.position.x -= speed;

            if (item.position.x < -halfWidth) {
                item.position.x += totalWidth;
                item.visible = false;
            }

            if (item.position.x < halfWidth) {
                item.visible = true;
            }
        };

        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {},
            destroy: function (data) {},
            update: function (data) {
                var ii;
                for (ii = 0; ii < items.length; ii++) {
                    moveItem(items[ii]);
                }
            },
            draw: function (data) {
                // data.renderer.fillCircle([1, 0, 0, 1], 0, 0, 5);
                // data.renderer.fillRect([0, 1, 0, 1], -halfWidth, -2, visibleWidth, 4);
            }
        };
        var entity = new Entity({
            z: 0,
            name: 'gacha',
            position: settings.position || new Vector2(0, 0),
            originRelative: new Vector2(0.5, 0.5),
            updateWhenPaused: 0,
            components: [
                behavior
            ]
        }).extend({
            addItem: function (item) {
                //TODO
                items.push(item);
                entity.attach(item);

                reposition();
            },
            begin: function (spd) {
                speed = spd;
            },
            stop: function (index, duration) {
                duration = duration || 120;
                //TODO
            }
        });
        return entity;
    };
});