/**
 * a dialog that shows more info about the item and the ability to claim it.
 * @moduleName GiftDialog
 */
bento.define('gui/gift/giftdialog', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'gui/dialog',
    'managers/item',
    'gui/shadowtext',
    'modules/localization',
    'globals'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Dialog,
    ItemManager,
    ShadowText,
    Localization,
    Globals
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var itemEntity = settings.itemEntity;
        var dialog = new Dialog({
            ignoreAnalytics: true,
            blurriness: 5,
            title: settings.name,
            text: settings.description,
            onComplete: settings.onComplete
        });

        var createItem = function (data) {
            var forces = new Vector2(Utils.getRandomRangeFloat(-2, 2), -3 - Math.random());
            var gravity = 0.3;
            var dropItem = Entity({
                name: 'dropItem',
                z: Globals.WorldSelectLayers.GIFTS + 1,
                position: new Vector2(data.windowBox.position.x, data.windowBox.position.y + itemEntity.position.y),
                float: true,
                components: [
                    new Sprite({
                        imageName: data.sprite.animationSettings.imageName,
                        spriteSheet: data.sprite.currentSpriteSheet,
                        origin: data.sprite.origin
                    }), {
                        name: 'fallingBehavior',
                        start: function () {
                            dropItem.updateWhenPaused = Globals.PauseLevels.GIFTS;
                            console.log(dropItem.updateWhenPaused, Globals.PauseLevels.GIFTS);
                        },
                        update: function () {
                            dropItem.position.addTo(forces);
                            forces.y += gravity;
                            if (dropItem.position.y > viewport.height + 80) {
                                Bento.objects.remove(dropItem);
                            }
                        }
                    }
                ]
            });
            dialog.attach(dropItem);
        };

        var button = new ClickButton({
            name: 'claimButton',
            sfx: 'sfx_button',
            imageName: 'gui/buttons/green-small',
            frameCountX: 1,
            frameCountY: 3,
            position: Vector2(0, 38),
            onButtonDown: function () {
                claimText.position.y = -2;
            },
            onClick: function () {
                button.setActive(false);
                settings.onClaim();
                dialog.close();
            },
            onButtonUp: function () {
                claimText.position.y = -3;
            }
        });

        var claimText = new ShadowText({
            position: new Vector2(0, -3),
            text: Localization.getText('claim'),
            font: 'font',
            fontSize: 16,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            linebreaks: false,
            maxWidth: 38,
            maxHeight: 16
        });

        button.attach(claimText);

        itemEntity.position.y = -32;

        dialog.getComponent('windowBox', function (windowBox) {
            windowBox.attach(itemEntity);
            windowBox.getComponent('shadowText', function (text) {
                text.setText(text.getText(), {
                    maxHeight: 28
                });
                text.position.y += 8;
            });
            windowBox.attach(button);
        });

        return dialog;
    };
});