/**
 * frame for the gifts
 * @moduleName GiftFrame
 */
bento.define('gui/gift/giftframe', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'managers/resolution',
    'gui/shadowtext',
    'managers/time',
    'managers/save',
    'modules/localization',
    'gui/iteminfo',
    'globals',
    'managers/inventory',
    'managers/currency',
    'components/anchor',
    'managers/item',
    'gui/gift/giftdialog',
    'gui/itemget',
    'gui/levelupnotification',
    'gui/senseipopup',
    'modules/firebase',
    'gui/dialog',
    'modules/notifications',
    'managers/receiveitem',
    'managers/character',
    'gui/coinreward',
    'gui/dailyquest/progressbar',
    'gui/iconbutton'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    ResolutionManager,
    ShadowText,
    TimeManager,
    SaveManager,
    Localization,
    ItemInfo,
    Globals,
    InventoryManager,
    CurrencyManager,
    Anchor,
    ItemManager,
    GiftDialog,
    ItemGet,
    LevelUpNotification,
    SenseiPopup,
    FireBase,
    Dialog,
    Notifications,
    ReceiveItem,
    CharacterManager,
    CoinReward,
    ProgressBar,
    IconButton
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
            id: identifier used to get reward and time from savemanager
        }*/
        var identifier = settings.id;
        var currentReward = '';
        // previous times for reference
        var logMinutes = -1;
        var logSeconds = -1;

        var viewport = Bento.getViewport();
        var portrait = true;
        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {
                setVisual();
            },
            destroy: function (data) {},
            update: function (data) {
                if (data.entity.timer % 30 === 0) {
                    setVisual();
                }
            },
            draw: function (data) {}
        };
        var align = function (entity, isPortrait, component) {
            portrait = isPortrait;
        };
        var timeVisual = function () {
            timeFrame.visible = true;
            claimFrame.visible = false;
            claimButton.setActive(false);
        };
        var claimVisual = function () {
            timeFrame.visible = false;
            claimFrame.visible = true;
            claimButton.setActive(true);
        };
        var setBarImmediately = true;
        var setVisual = function () {
            var giftInfo = SaveManager.load('gift' + identifier);
            var currentTime = giftInfo.time - TimeManager.getTime();
            // setImage(giftInfo.reward);
            if (currentTime > 0) {
                timeVisual();
                claimFrame.visible = false;
                var seconds = currentTime / 1000;
                var minutes = seconds / 60;
                var hours = minutes / 60;

                if (logSeconds !== Math.floor(seconds)) {
                    hourText.setText(addDigit(Math.floor(hours)));
                    minuteText.setText(addDigit(Math.floor(minutes) % 60));
                    secondText.setText(addDigit(Math.floor(seconds) % 60));
                }

                logSeconds = Math.floor(seconds);

                progressBar.setProgress({
                    count: 86400000 - currentTime,
                    goal: 86400000
                }, setBarImmediately);
                setBarImmediately = false;
            } else {
                // claim reward
                claimVisual();
            }
        };
        var setImage = function (key) {
            if (currentReward !== key) {
                frame.getComponent('loot', function (loot) {
                    frame.remove(loot);
                });
                var giftInfo = SaveManager.load('gift' + identifier);
                var data = {
                    amount: giftInfo.amount
                };

                var json = Bento.assets.getJson(key);
                data.spriteSheet = json.inventory.image;
                data.name = ItemManager.getName(key);
                data.itemInfo = ItemManager.getDescription(key);

                frame.attach(addLootEntity(data));

                currentReward = key;
            }
        };
        var addLootEntity = function (data) {
            var spriteSheet = data.spriteSheet;
            var imageName = data.imageName;
            var name = data.name;
            var itemInfo = data.itemInfo;
            var amount = data.amount || 1;
            var sprite = new Sprite({
                spriteSheet: spriteSheet,
                imageName: imageName
            });
            var loot = new Entity({
                name: 'loot',
                position: new Vector2(0, 0),
                components: [
                    sprite
                ]
            });
            // add a number if more than 1 loot is given
            var lootAmountText = new ShadowText({
                position: new Vector2(16, 6),
                text: amount,
                font: 'font',
                fontSize: 16,
                fontColor: '#ffffff',
                align: 'right',
                textBaseline: 'middle',
            });

            // overwrite origin to middle
            sprite.setOriginRelative(new Vector2(0.5, 0.5));

            if (amount > 1) {
                loot.attach(lootAmountText);
            }

            loot.extend({
                getName: function () {
                    return name;
                },
                getDescription: function () {
                    return itemInfo;
                },
                getAmount: function () {
                    return amount;
                },
            });

            return loot;
        };
        // add zeroes to the front a string according to amount
        var addDigit = function (value, amount) {
            amount = amount || 2;
            value = value.toString();
            while (value.length < amount) {
                value = '0' + value;
            }
            return value;
        };
        /**
         * Visuals
         */
        var hourText = new ShadowText({
            z: 0,
            position: new Vector2(-17, 6),
            text: '00',
            font: 'font',
            fontSize: 16,
            fontColor: '#ffffff',
            align: 'right',
            textBaseline: 'bottom',
            ySpacing: 0,
            lineWidth: 0,
        });
        var minuteText = new ShadowText({
            z: 0,
            position: new Vector2(5, 6),
            text: '00',
            font: 'font',
            fontSize: 16,
            fontColor: '#ffffff',
            align: 'right',
            textBaseline: 'bottom',
            ySpacing: 0,
            lineWidth: 0,
        });
        var secondText = new ShadowText({
            z: 0,
            position: new Vector2(28, 6),
            text: '00',
            font: 'font',
            fontSize: 16,
            fontColor: '#ffffff',
            align: 'right',
            textBaseline: 'bottom',
            ySpacing: 0,
            lineWidth: 0,
        });
        var hoursIcon = new Entity({
            name: 'hours',
            position: new Vector2(-17, -2),
            components: [
                new Sprite({
                    imageName: 'gui/gift/hours'
                })
            ]
        });
        var minutesIcon = new Entity({
            name: 'minutes',
            position: new Vector2(5, -2),
            components: [
                new Sprite({
                    imageName: 'gui/gift/minutes'
                })
            ]
        });
        var secondsIcon = new Entity({
            name: 'seconds',
            position: new Vector2(28, -1),
            components: [
                new Sprite({
                    imageName: 'gui/gift/seconds'
                })
            ]
        });
        var progressBar = new ProgressBar({
            goal: 1000 * 60 * 60 * 24,
            hideText: true,
            frameImage: 'gui/gift/bar-frame',
            barImage: 'gui/gift/bar',
            tween: true,
            lerp: 0.1
        });
        var timeFrame = new Entity({
            name: 'timeFrame',
            position: new Vector2(0, 0),
            components: [
                progressBar,
                // new Sprite({
                //     imageName: 'gui/gift/timeframe',
                //     originRelative: new Vector2(0.5, 0.5)
                // }),
                hourText,
                minuteText,
                secondText,
                hoursIcon,
                minutesIcon,
                secondsIcon,
            ]
        });
        var frame = new Entity({
            name: 'frame',
            position: new Vector2(0, 0),
            components: [
                new Sprite({
                    imageName: 'gui/gift/giftframe',
                    originRelative: new Vector2(0.5, 0.5)
                })
            ]
        });
        /**
         * Notification
         */
        var askForNotification = function () {
            var dialog = new Dialog({
                z: Globals.WorldSelectLayers.DIALOGS,
                name: 'askForNotification',
                type: 'yesno',
                title: Localization.getText('notifications')['askTitle'],
                text: Localization.getText('notifications')['ask'],
                canClose: false,
                icon: new Entity({
                    components: [
                        new Sprite({
                            imageName: 'gui/notifications-icon',
                            originRelative: new Vector2(0.5, 0.5)
                        })
                    ]
                }),
                onYes: function () {
                    // ask for real notification
                    Notifications.init(function (initialized) {
                        var allowed = initialized ? true : false;
                        SaveManager.save('notificationsAllowed', allowed);
                        if (allowed) {
                            // first time will be 24 hours from now
                            scheduleNotification(24);
                            // fire event to schedule inactive notification
                            EventSystem.fire('allowedNotifications');
                        } else {
                            // user pressed no at the system dialog...
                        }
                    });
                },
                onNo: function () {
                    // do nothing, better luck next time
                }
            });
            Bento.objects.attach(dialog);

            var asked = SaveManager.load('notificationsAsked');
            asked += 1;
            SaveManager.save('notificationsAsked', asked);
        };
        // schedule notification for the 24h gift
        var scheduleNotification = function (deltaTime) {
            var savedNotifications = SaveManager.load('notifications');
            if (Utils.isDefined(deltaTime)) {
                deltaTime = 24;
            }
            // reset badge
            Notifications.resetBadgeNumber();

            // cancel old notification if needed
            var oldId = savedNotifications['gift24'];
            if (oldId) {
                Notifications.cancel(oldId);
            }

            // schedule notification
            var notifications = Utils.getNotifications({
                Localization: Localization,
                time: deltaTime * 60 * 60 * 1000
            });
            var id = Notifications.send(notifications.dailyGift);

            // save id so it can be cancelled if needed
            savedNotifications['gift24'] = id;
            SaveManager.save('notifications', savedNotifications);
        };
        /**
         * Claim
         */
        var claimText = new ShadowText({
            position: new Vector2(0, -3),
            text: Localization.getText('claim'),
            font: 'font',
            fontSize: 16,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            linebreaks: false,
            maxWidth: 38,
            maxHeight: 16
        });
        var claimButton = new IconButton({
            z: 0,
            name: 'item',
            sfx: 'sfx_button',
            imageName: 'gui2/misc/buttonmini_large_primary',
            frameCountX: 1,
            frameCountY: 3,
            position: new Vector2(-1, -1),
            float: false,
            updateWhenPaused: 0,
            onButtonDown: function () {
                claimText.position.y = -2;
            },
            onClick: function () {
                var receiveGifts = function () {
                    Utils.forEach(gifts, function (giftInfo, i, l, breakLoop) {
                        ReceiveItem({
                            key: giftInfo.reward,
                            amount: giftInfo.amount,
                            onComplete: function (data) {
                                if (SaveManager.load('tutorialFreeGift') === 1) {
                                    SaveManager.save('tutorialFreeGift', 2);
                                }
                            }
                        });
                    });
                    currentReward = '';
                    giftFrame.parent.resetReward(identifier);

                    // schedule next notification
                    // if (SaveManager.load('notificationsAllowed')) {
                    //     scheduleNotification(24);
                    // }

                    // show player what items were gotten
                    showGifts(gifts, function () {
                        onItemsGet();
                    });
                };
                // completion callback after all gifts are received
                var onItemsGet = function () {
                    // tutorial: talk about notificaitons
                    /*var tutorialFreeGift = SaveManager.load('tutorialFreeGift');
                    if (tutorialFreeGift === 2) {
                        Bento.objects.attach(new SenseiPopup({
                            darken: true,
                            modal: true,
                            onNoMoreText: function () {
                                this.dismiss();
                            },
                            onComplete: function () {
                                // ask for notifications
                                if (!SaveManager.load('notificationsAllowed')) {
                                    askForNotification();
                                } else {
                                    // schedule first notification
                                    scheduleNotification(24);
                                }

                                SaveManager.save('tutorialFreeGift', 3);

                                if (settings.onTutorialComplete) {
                                    settings.onTutorialComplete();
                                }
                            },
                            text: Localization.getText('tutorial')['worldSelect7']
                        }));
                    }*/
                    if (settings.onClaim) {
                        settings.onClaim();
                    }
                };
                var gifts = settings.getDailyChestItems();
                receiveGifts();
                setVisual();
            },
            onButtonUp: function () {
                claimText.position.y = -3;
            }
        }).attach(claimText);
        var claimFrame = new Entity({
            name: 'claimFrame',
            position: new Vector2(0, 0),
            components: [
                new Sprite({
                    imageName: 'gui/gift/claimframe',
                    originRelative: new Vector2(0.5, 0.5)
                }),
                claimButton
            ]
        });
        var showGifts = function (items, onComplete) {
            // 3 itemget popups after eachother
            var currentItem = 0;
            var showItemGet = function () {
                var onItemComplete = function () {
                    // show next item
                    if (items[currentItem + 1]) {
                        currentItem += 1;
                        showItemGet();
                    } else {
                        // no more items
                        onComplete();
                    }
                };
                var itemGet = new ItemGet({
                    z: Globals.WorldSelectLayers.GIFTS + 1,
                    item: items[currentItem].reward,
                    amount: items[currentItem].amount,
                    onComplete: onItemComplete
                });

                // swap with coin reward if needed
                if (items[currentItem].reward === 'currency/coin') {
                    itemGet = new CoinReward({
                        z: Globals.WorldSelectLayers.GIFTS + 1,
                        coins: items[currentItem].amount,
                        onComplete: onItemComplete
                    });
                }

                Bento.objects.attach(itemGet);
            };
            showItemGet();
        };

        var giftFrame = new Entity({
            z: 0,
            name: 'giftFrame',
            position: settings.position,
            updateWhenPaused: 0,
            components: [
                frame,
                timeFrame,
                claimFrame,
                behavior,
                new Anchor({
                    onComplete: align
                })
            ]
        }).extend({
            isActive: claimButton.isActive,
            claim: function (onClaimed) {
                var giftInfo = SaveManager.load('gift' + identifier);
                var currentTime = giftInfo.time - TimeManager.getTime();
                if (currentTime > 0) {
                    // time not passed yet
                    onClaimed(false);
                } else {
                    claimButton.doCallback();
                    onClaimed(true);
                }
            }
        });
        return giftFrame;
    };
});