/**
 * Window box with a title header
 * @moduleName HeaderBox
 * @snippet HeaderBox.snippet
HeaderBox({
    name: 'box',
    position: new Vector2(0, 0),
    size: new Rectangle(0, 0, 180, 160),
    title: Localization.getText(''),
    titleColor: '#ffdf00',
    windowImage: 'gui2/misc/splash_background_9slice',
    components: []
})
 */
bento.define('gui/headerbox', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'bento/components/nineslice',
    'components/anchor'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    NineSlice,
    Anchor
) {
    'use strict';
    return function (settings) {
        // size of window
        var size = settings.size || new Rectangle(0, 0, 180, 180);
        var titleColor = settings.titleColor || '#ffdf00';
        var windowImage = settings.windowImage || 'gui2/misc/panel_highlight_background_9slice';
        var useInnerSlice = settings.useInnerSlice || false;
        // header on the window
        var title = new Text({
            position: new Vector2(0, -size.height / 2 + 20),
            text: settings.title,
            font: 'font',
            fontSize: 16,
            fontColor: titleColor,
            align: 'center',
            textBaseline: 'middle',
            maxWidth: settings.maxWidth || size.width,
            maxHeight: 16,
        });
        var header = new Entity({
            name: 'header',
            position: new Vector2(0, -size.height / 2 + 6),
            components: [
            ]
        });
        // window
        var nineSlice = new NineSlice({
            imageName: windowImage,
            originRelative: new Vector2(0.5, 0.5),
            width: size.width,
            height: size.height
        });
        var innerNineSlice = new NineSlice({
            imageName: 'gui2/misc/panel_background_9slice',
            originRelative: new Vector2(0.5, 0.5),
            width: size.width - 20,
            height: size.height - 72
        });
        var windowBox = new Entity({
            name: 'windowBox',
            position: settings.position,
            components: [
                nineSlice
            ].concat(settings.components || [])
        });

        windowBox.extend({
            setTitle: function (text) {
                title.setText(text);
            },
            setWidth: function (width) {
                nineSlice.setWidth(width);
            },
            setHeight: function (height) {
                nineSlice.setHeight(height);
            },
            getHeight: function () {
                return nineSlice._height;
            }
        });

        if (useInnerSlice) {
            windowBox.attach(innerNineSlice);
        }

        // if a title is not defined it will not show the header, but empty strings are allowed
        if (Utils.isDefined(settings.title)) {
            windowBox.attach(header);
            // header should go behind nineslice
            windowBox.moveComponentTo(header, 0);
            windowBox.attach(title);
        }
        return windowBox;
    };
});