/**
 * Dark overlay, showing a single highlight, meant for tutorials
 * @moduleName Highlight
 */
bento.define('gui/highlight', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'bento/packedimage',
    'components/anchor'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    PackedImage,
    Anchor
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var padding = 64;
        var canvas;
        var context;
        var alpha = settings.alpha || 0.5;
        var radius = settings.radius || 32;
        var position = settings.position; // note: must be a worldposition
        var sprite;
        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {
                canvas = document.createElement('canvas');
                canvas.width = Math.ceil(viewport.width + padding);
                canvas.height = Math.ceil(viewport.height + padding * 2);
                context = canvas.getContext('2d');

                sprite = new Sprite({
                    image: new PackedImage(canvas)
                });
                entity.attach(sprite);

                entity.updateHighlight();

                // fade in
                new Tween({
                    from: 0,
                    to: 1,
                    in: 30,
                    ease: 'linear',
                    onUpdate: function (v, t) {
                        entity.alpha = v;
                    }
                });
            }
        };
        var entity = new Entity({
            z: settings.z,
            name: 'highlight',
            float: true,
            updateWhenPaused: settings.updateWhenPaused,
            components: [
                behavior,
                new Anchor({
                    onComplete: function () {
                        canvas.width = Math.ceil(viewport.width + padding);
                        canvas.height = Math.ceil(viewport.height + padding * 2);
                        // there is no guarantee the viewportscroller will be updated before this, so we wait 1 frame
                        Utils.timeout(1, function () {
                            entity.updateHighlight();
                        });
                    }
                })
            ]
        }).extend({
            updateHighlight: function (skipHole) {
                var x, y;
                context.globalCompositeOperation = 'source-over';
                context.clearRect(0, 0, canvas.width, canvas.height);
                context.fillStyle = 'rgba(0, 0, 0, ' + alpha + ')';
                context.fillRect(0, 0, canvas.width, canvas.height);
                // cut out hole
                if (!skipHole) {
                    context.globalCompositeOperation = 'destination-out';
                    context.fillStyle = '#fff';
                    x = position.x - viewport.x;
                    y = position.y - viewport.y;
                    context.beginPath();
                    context.arc(x + padding, y + padding, radius, 0, Math.PI * 2, false);
                    context.fill();
                }
                sprite.setup({
                    image: new PackedImage(canvas)
                });
                entity.transform.x = -padding;
                entity.transform.y = -padding;
            },
            tweenOut: function () {
                new Tween({
                    from: entity.alpha,
                    to: 0,
                    in: 30,
                    ease: 'linear',
                    onUpdate: function (v, t) {
                        entity.alpha = v;
                    },
                    onComplete: function () {
                        entity.removeSelf();
                    }
                });
            }
        });
        return entity;
    };
});