/**
 * Popup that asks the player to buy missing tickets with gems
 * @moduleName BuyTicketsPopup
 */
bento.define('gui/iap/buyticketspopup', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'gui/dialog',
    'modules/localization',
    'gui/shadowtext',
    'gui/worldselect/topbarsimple',
    'globals',
    'managers/currency',
    'gui/coinreward',
    'gui/menus/iap',
    'modules/analytics',
    'gui/iconbutton',
    'gui/itemget'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Dialog,
    Localization,
    ShadowText,
    TopBarSimple,
    Globals,
    CurrencyManager,
    CoinReward,
    IAPMenu,
    Analytics,
    IconButton,
    ItemGet
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var ticketsToBuy = settings.ticketsToBuy || 1;
        var buttonToClick = settings.buttonToClick;
        var onComplete = settings.onComplete;
        var popup = new Dialog({
            z: settings.z,
            name: 'buyTicketsWithGems',
            ignoreAnalytics: true,
            blurriness: 4,
            title: ((Localization.getText('currency') || {})['ticket'] || {})['name'],
            canClose: true,
            onComplete: function () {
                if (hasBought && buttonToClick) {
                    // do callback of button
                    buttonToClick.doCallback();
                }
                if (onComplete) {
                    onComplete(hasBought);
                }
            },
            onClose: function () {
                Analytics.event('Economy', 'buyTicketsWithGems', 0, 'yes');
            }
        });
        var hasBought = false;
        var gemsNeeded = (function () {
            // calculate the best deal for user
            var i;
            var gemToTickets = Globals.gemToTickets;
            var ticketsNeeded = ticketsToBuy;
            var needed = ticketsNeeded * gemToTickets;

            // possible the price is 0, player must pay at least 1 gem
            return needed || 1;
        })();
        var topbar = new TopBarSimple({});
        var cost = new ShadowText({
            position: new Vector2(8, -1),
            text: gemsNeeded.toString(),
            font: 'font',
            fontSize: 22,
            fontColor: '#ffffff',
            align: 'left',
            textBaseline: 'middle',
            maxWidth: 48,
            linebreaks: false
        });
        var gem = new Entity({
            name: 'gem',
            position: new Vector2(-16, 0),
            components: [
                new Sprite({
                    imageName: 'gui/gem',
                    originRelative: new Vector2(0.5, 0.5)
                })
            ]
        });
        var icon = new Entity({
            name: 'icon',
            components: [
                gem,
                cost
            ]
        });
        var buttonBuy = new IconButton({
            name: 'buttonBuy',
            sfx: 'sfx_button',
            position: new Vector2(0, 68),
            imageName: 'gui2/misc/button_large_primary_excited',
            frameCountX: 6,
            frameCountY: 7,
            padding: 1,
            icon: icon,
            iconAlpha: 0.5,
            animations: {
                up: {
                    speed: 0.34,
                    frames: [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35]
                },
                down: {
                    frames: [37]
                },
                inactive: {
                    frames: [38]
                }
            },
            onClick: function () {
                var showTicketReward = function () {
                    var itemGet = new ItemGet({
                        z: Globals.WorldSelectLayers.GIFTS + 1,
                        item: 'currency/ticket',
                        amount: ticketsToBuy,
                        autoDismiss: true,
                        onComplete: function () {
                            popup.close();
                        }
                    });
                    Bento.objects.attach(itemGet);
                    buttonBuy.setActive(false);
                };
                // buy coins with the gems needed
                if (CurrencyManager.canSpendGems(gemsNeeded)) {
                    CurrencyManager.spendGems(gemsNeeded, 'tickets');
                    CurrencyManager.addTickets(ticketsToBuy);
                    Analytics.event('SourceSinks', 'gemsSinkedToTicket', gemsNeeded);
                    hasBought = true;
                    showTicketReward();
                    Bento.audio.stopSound('sfx_buy');
                    Bento.audio.playSound('sfx_buy');
                    Analytics.event('Economy', 'buyTicketsWithGems', 1, 'yes');
                } else {
                    // not enough gems???? go to shop
                    var iapMenu = new IAPMenu({
                        z: settings.z + 1,
                        scrollTo: 'gems'
                    });
                    Bento.objects.attach(iapMenu);
                    Analytics.event('Economy', 'buyTicketsWithGems', 1, 'notEnough');
                }
            },
            onButtonDown: function () {
                EventSystem.fire('preventScroll');
                gem.position.y = 0;
                cost.position.y = -1;
            },
            onButtonUp: function () {
                gem.position.y = -1;
                cost.position.y = -2;
            }
        });
        var ticketSprite = new Sprite({
            spriteSheet: 'currency/ticket'
        });
        var ticketEntity = new Entity({
            name: 'ticketEntity',
            position: new Vector2(0, -16),
            components: [
                ticketSprite
            ]
        });
        var ticketText = new ShadowText({
            position: new Vector2(8, 0),
            text: ticketsToBuy.toString(),
            font: 'font',
            fontSize: 16,
            fontColor: '#ffff00',
            align: 'left',
            textBaseline: 'middle',
            margin: new Vector2(0, 8),
            maxWidth: 648,
            maxHeight: 16,
            linebreaks: false
        });
        var text = new ShadowText({
            position: new Vector2(0, 24),
            text: Localization.getText('buyTickets'),
            font: 'font',
            fontSize: 16,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            maxWidth: 160 - 16,
            maxHeight: 16
        });

        popup.getComponent('windowBox', function (windowBox) {
            windowBox.attach(buttonBuy);
            windowBox.attach(text);
            windowBox.attach(ticketEntity);
            windowBox.attach(ticketText);
        });

        popup.attach(topbar);

        // try to get constructor asap
        if (!IAPMenu) {
            bento.require(['gui/menus/iap'], function (Module) {
                IAPMenu = Module;
            });
        }

        return popup;
    };
});