/**
 * Shows chest opening
 * @moduleName ChestOverlay
 */
bento.define('gui/iap/chestoverlay', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'gui/blurbackground',
    'globals',
    'components/anchor',
    'components/modal',
    'entities/effect',
    'gui/shadowtext',
    'modules/localization',
    'gui/scrollinglist',
    'components/multiscale',
    'gui/iteminfo',
    'managers/save',
    'managers/inventory',
    'managers/character',
    'managers/currency',
    'gui/tokenbar',
    'components/sparkles',
    'gui/confetti',
    'gui/sparkleburst',
    'components/blink',
    'managers/stats',
    'gui/levelupnotification',
    'modules/analytics',
    'gui/tier',
    'managers/item',
    'modules/tenjin',
    'components/trigger',
    'gui/iap/gridslot',
    'gui/iap/claimbutton',
    'components/orientationlock',
    'managers/allies',
    'managers/receiveitem',
    'gui/centeredlist',
    'components/eventlistener',
    'managers/time'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    BlurBackground,
    Globals,
    Anchor,
    Modal,
    Effect,
    ShadowText,
    Localization,
    ScrollingList,
    MultiScale,
    ItemInfo,
    SaveManager,
    InventoryManager,
    CharacterManager,
    CurrencyManager,
    TokenBar,
    Sparkles,
    Confetti,
    SparkleBurst,
    Blink,
    StatsManager,
    LevelUpNotification,
    Analytics,
    TierStars,
    ItemManager,
    Tenjin,
    Trigger,
    GridSlot,
    ClaimButton,
    OrientationLock,
    AlliesManager,
    ReceiveItem,
    CenteredList,
    EventListener,
    TimeManager
) {
    'use strict';
    var lootSpacing = 64;
    // event?
    var isChestEvent = function () {
        var today = Math.floor(TimeManager.getTime() / 1000 / 60 / 60 / 24);
        if (today >= 17501 && today <= 17504) {
            // this part can be patched
            return true;
        }
        return false;
    };
    var checkEvent = function () {
        var today = Math.floor(TimeManager.getTime() / 1000 / 60 / 60 / 24);
        var oniToken = Bento.assets.getJson('playertokens/oni');
        var lockedCharacters = [
            'boy',
            'krillin',
            'boxer',
            'val',
            'jun',
            'pat',
            'fitgirl',
            'oni',
            'rad'
        ];
        var highest = 0;
        // find locked
        Utils.forEach(SaveManager.load('unlockedCharacters'), function (char, i, l, breakLoop) {
            lockedCharacters.splice(lockedCharacters.indexOf(char), 1);
        });
        // find the current highest weight
        Utils.forEach(lockedCharacters, function (char, index) {
            var json = Bento.assets.getJson('playertokens/' + char);
            var weight = json.weight || 1;
            if (weight > highest) {
                highest = weight;
            }
        });

        if (highest === 0) {
            // nothing to do
            return;
        }
        // set oni relative to the highest
        if (today >= 17501 && today <= 17503) {
            oniToken.weight = highest * 2;
        }
    };
    /*
     * Slot for the horizontal list
     */
    var Slot = function () {
        var slot = new Entity({
            name: 'slot',
            components: [
                new Sprite({
                    // imageName: 'gui/iap/frame-empty',
                    origin: new Vector2(25, 25)
                })
            ]
        });
        slot.extend({
            getItem: function () {
                return slot.components[1];
            }
        });
        slot.dimension = new Rectangle(0, 0, lootSpacing, lootSpacing);
        return slot;
    };
    /*
     * Weighted randomization
     * array = [{value: Something, weight: Number }, ...]
     */
    var weightedRandom = function (array) {
        // calculate sum of weights
        var weightTotal = 0;
        Utils.forEach(array, function (item, i, l, breakLoop) {
            weightTotal += item.weight || 0;
        });
        // exception: 0 total weight
        if (weightTotal === 0) {
            // Utils.log('WARNING: weighted randomization with 0 weights');
            // assign equal weights
            Utils.forEach(array, function (item) {
                item.weight = 1;
            });
            weightTotal = array.length;
        }

        // randomize a number
        var random = Utils.getRandomRangeFloat(0, weightTotal);

        // find the winner
        var winner;
        var weightIncr = 0;
        Utils.forEach(array, function (item, i, l, breakLoop) {
            if (item.weight <= 0) {
                return;
            }
            weightIncr += item.weight;
            if (weightIncr > random) {
                winner = item.value;
                breakLoop();
                return;
            }
        });
        return winner;
    };
    // cached item names
    var consumables;
    var nonConsumables;
    var collectNames = function () {
        var assets = Bento.assets.getAssets();
        var jsonAssets = assets.json;

        consumables = [];
        nonConsumables = [];

        Utils.forEach(jsonAssets, function (item, key, l, breakLoop) {
            // weapons count as consumables too
            if (key.indexOf('consumables/') === 0 || key.indexOf('weapons/') === 0) {
                consumables.push({
                    key: key,
                    tier: (item.inventory || {}).tier || 0
                });
            } else if (key.indexOf('equipment/') === 0) {
                nonConsumables.push({
                    key: key,
                    tier: (item.inventory || {}).tier || 0
                });
            }
        });
    };
    // exclude rad as a token from the classic chest
    var exclusions = ['rad'];

    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var blurBackground = new BlurBackground({
            darken: 0.3
        });
        var chestType = settings.chest;
        var chestImage = settings.chestImage;
        // predefined chest contents instead of specific chest
        var chestContents = settings.contents;
        var onComplete = settings.onComplete;
        /*
         * Chest entity
         */
        var chestSprite = new Sprite({
            imageName: chestImage || 'gui/iap/chest2-' + chestType,
            frameCountX: 2,
            originRelative: new Vector2(0.5, 1),
            animations: {
                open: {
                    speed: 0,
                    frames: [1]
                },
                closed: {
                    speed: 0,
                    frames: [0]
                }
            }
        });
        var chest = new Entity({
            name: 'chest',
            components: [
                new Anchor({
                    screenPosition: new Vector2(0.5, 0.75),
                    offset: new Vector2(-16, 0),
                    onComplete: function (parent, isPortrait) {
                        if (!isPortrait) {
                            parent.position.y = viewport.height * 0.9;
                        }
                    }
                }),
                chestSprite,
                new Sparkles({
                    spawnRate: 0.05,
                    position: new Vector2(0, -16)
                })
            ]
        });
        /*
         * Title of item
         */
        var lootTitle = new ShadowText({
            text: '',
            font: 'font',
            fontSize: 24,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'bottom',
            maxWidth: viewport.width - 32,
            maxHeight: 24
        }).attach(new Anchor({
            screenPosition: new Vector2(0.5, 0.2)
        }));
        var tapToOpen = new ShadowText({
            text: Localization.getText('tapToOpen'),
            font: 'font',
            fontSize: 20,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'bottom',
            maxWidth: viewport.width - 32,
            maxHeight: 20
        }).attach(new Anchor({
            screenPosition: new Vector2(0.5, 0.5)
        }));
        /*
         * The entities to drop out of the loot box
         */
        var lootEntities = [];
        /*
         * Horizontal list for loot items to drop out
         */
        var dropped = 0;
        var containerPosition = new Vector2(0.5, 0.4);
        var cancelScroll = function () {
            droppedContainer.cancel();
        };
        var droppedContainer = new ScrollingList({
            direction: 'x',
            snap: true,
            area: new Rectangle(0, 0, 0, 0),
            onSnap: function (slot) {
                if (!slot) {
                    return;
                }
                var item = slot.getItem();
                if (item) {
                    // update title
                    item.setText();
                }
            }
        }).attach(new Anchor({
            screenPosition: containerPosition
        })).attach({
            name: 'scaleContents',
            update: function () {
                droppedContainer.iterate(function (slot, i) {
                    var scroll = -droppedContainer.getScroll();
                    var myPos = i * lootSpacing;
                    var distance = Math.abs(myPos - scroll);
                    slot.scale.x = Utils.clamp(1, 2 - distance / lootSpacing, 2);
                    slot.scale.y = slot.scale.x;
                });
            }
        });

        /*
         * Grid container for loot items, at the end
         */
        var lootGrid = new ScrollingList({
            name: 'lootGrid',
            direction: 'y',
            snap: true,
            components: [
                new Anchor({
                    screenPosition: new Vector2(0.5, 0),
                    offset: new Vector2(0, 64),
                    onComplete: function () {
                        lootGrid.arrange();
                    }
                }),
                new EventListener({
                    name: 'preventScroll',
                    onEvent: function (data) {
                        lootGrid.cancel();
                    }
                })
            ]
        }).extend({
            arrange: function () {
                var index = 0;
                var rowIndex = -1;
                lootGrid.iterate(function (row) {
                    rowIndex += 1;
                    row.iterate(function (gridSlot, i, l, breakLoop) {
                        var currentRow = rowIndex;
                        index += 1;
                        // item intro
                        new Tween({
                            from: gridSlot.position.y - 10,
                            to: gridSlot.position.y,
                            in: 60,
                            delay: index * 5,
                            ease: 'easeOutBounce',
                            onUpdate: function (v, t) {
                                gridSlot.position.y = v;
                            }
                        });
                        gridSlot.alpha = 0;
                        new Tween({
                            from: 0,
                            to: 1,
                            in: 30,
                            delay: index * 5,
                            ease: 'linear',
                            onStart: function () {
                                // scroll to bottom row
                                if (i === 0) {
                                    if (Anchor.isPortrait() && currentRow > 4) {
                                        lootGrid.snapTo(currentRow - 4);
                                    } else if (!Anchor.isPortrait() && currentRow > 1) {
                                        lootGrid.snapTo(currentRow - 1);
                                    }
                                }
                            },
                            onUpdate: function (v, t) {
                                gridSlot.alpha = v;
                            },
                            onComplete: function () {
                                gridSlot.attach(new SparkleBurst({}));
                                Bento.audio.setVolume(0.1, 'sfx_card');
                                Bento.audio.stopSound('sfx_card');
                                Bento.audio.playSound('sfx_card');
                            }
                        });
                    });
                });
                lootGrid.snapTo(0);
            }
        });

        /*
         * Character entity that punches the chest open
         */
        var character = SaveManager.load('selectedCharacter');
        var characterSprite = new Sprite({
            spriteSheet: character + '/idle'
        });
        var characterEntity = new Entity({
            name: 'character',
            scale: new Vector2(-1, 1),
            components: [
                characterSprite,
                new Anchor({
                    screenPosition: new Vector2(0.5, 0.75),
                    offset: new Vector2(16, 0),
                    onComplete: function (parent, isPortrait) {
                        if (!isPortrait) {
                            parent.position.y = viewport.height * 0.9;
                        }
                    }
                })
            ]
        });
        // show only 1 tween at a time
        var isShowingItem = false;
        /*
         * Tap to show next item
         */
        var clickable = new Clickable({
            onClick: function (data) {
                if (isShowingItem) {
                    // only 1 tween at a time
                    return;
                }

                tapToOpen.removeSelf();

                isShowingItem = true;

                // punch
                characterSprite.setSpriteSheet(character + '/punch1', function () {
                    characterSprite.setSpriteSheet(character + '/idle');
                });
                // bounce box
                new Tween({
                    from: 1,
                    to: 0,
                    in: 30,
                    delay: 10,
                    ease: 'elastic',
                    decay: 5,
                    oscillations: 3,
                    onStart: function () {
                        var effect = new Effect({
                            spriteSheet: 'effects/hitfx2-small',
                            position: chest.position.add(new Vector2(8, -16))
                        });
                        entity.attach(effect);

                        Bento.audio.stopSound('sfx_lootappear');
                        Bento.audio.playSound('sfx_lootappear');
                        Bento.audio.stopSound('sfx_punch');
                        Bento.audio.playSound('sfx_punch');

                        // reset title
                        lootTitle.setText('');

                        // spawn an item
                        if (lootEntities.length) {
                            dropNextItem();
                        }

                        // last item dropped?
                        if (lootEntities.length === 0) {
                            // chest flies off
                            chest.attach({
                                name: 'flyOff',
                                velocity: new Vector2(-4, -4),
                                update: function () {
                                    chest.position.x += this.velocity.x;
                                    chest.position.y += this.velocity.y;
                                    this.velocity.y += 0.2;
                                    chest.rotation -= 0.02;
                                }
                            });
                        }
                        chestSprite.setAnimation('open');
                    },
                    onUpdate: function (v, t) {
                        chest.scale.x = 1 - v;
                        chest.scale.y = 1 + v;
                    },
                    onComplete: function () {
                        chestSprite.setAnimation('closed');
                        if (lootEntities.length === 0) {
                            Utils.timeout(30, completedLoot);
                        }
                    }
                });

                // jump box if more then one loot to give, last item makes it fly away so the box doesn't need to jump;
                if (lootEntities.length > 1) {
                    jumpBox();
                }
            }
        });
        /*
         * Drop next loot item out of the chest
         */
        var tokenBar;
        var tierStars;
        var dropNextItem = function () {
            var loot = lootEntities.shift();
            var targetPosition = new Vector2(
                containerPosition.x * viewport.width,
                containerPosition.y * viewport.height
            );
            var moveLoot = function () {
                var time = 20;

                // remove tokenBar if necessary
                if (tokenBar) {
                    tokenBar.removeSelf();
                    tokenBar = null;
                }
                if (tierStars) {
                    tierStars.removeSelf();
                    tierStars = null;
                }

                // grow
                new Tween({
                    from: 2,
                    to: -1,
                    in: time,
                    ease: 'cos',
                    decay: 5,
                    oscillations: 0.75,
                    onUpdate: function (v, t) {
                        loot.scale.x = v;
                        loot.scale.y = v;
                    },
                    onComplete: function () {
                        /*var effectRight = new Effect({
                            position: loot.position.add(new Vector2(40, 32)),
                            spriteSheet: 'effects/runcloud'
                        });
                        var effectLeft = new Effect({
                            position: loot.position.add(new Vector2(-40, 32)),
                            spriteSheet: 'effects/runcloud'
                        });
                        effectRight.scale.x = -1;

                        loot.parent.attach(effectLeft);
                        loot.parent.attach(effectRight);*/
                        // bounce
                        new Tween({
                            from: 0.5,
                            to: 0,
                            in: 20,
                            ease: 'elastic',
                            decay: 5, // for 'exponential' and 'elastic'
                            oscillations: 2, // for 'elastic', 'sin' and 'cos'
                            onStart: function () {},
                            onUpdate: function (v, t) {
                                loot.scale.x = 1 + v;
                                loot.scale.y = 1 - v;
                            },
                            onComplete: function () {

                            }
                        });

                        loot.attach(new SparkleBurst({}));
                        loot.attach(new Blink({}));

                        // create tokenbar next to it
                        if (loot.type) {
                            tokenBar = new TokenBar({
                                position: targetPosition.add(new Vector2(56, 0)),
                                type: loot.type,
                                character: loot.character,
                                subtract: loot.amount,
                                // lerp: 0.1
                            });
                            loot.parent.attach(tokenBar);

                            if (loot.tier) {
                                tierStars = new TierStars({
                                    position: targetPosition.add(new Vector2(-24, -20)),
                                    alignment: 'left',
                                    tier: loot.tier
                                });
                                tierStars.scale.x = 2;
                                tierStars.scale.y = 2;
                                new Tween({
                                    from: 0,
                                    to: 2,
                                    in: 30,
                                    delay: 0,
                                    ease: 'elastic',
                                    decay: 5,
                                    oscillations: 2,
                                    onUpdate: function (v, t) {
                                        if (tierStars) {
                                            tierStars.scale.x = v;
                                            tierStars.scale.y = v;
                                        }
                                    }
                                });
                                loot.parent.attach(tierStars);
                            }
                        }

                        if (loot.onDropped) {
                            loot.onDropped();
                        }

                        Bento.audio.stopSound('sfx_lootdrop');
                        Bento.audio.playSound('sfx_lootdrop');
                    }
                });
                // tween towards container
                new Tween({
                    from: loot.position,
                    to: targetPosition,
                    in: time,
                    delay: 0,
                    ease: 'exponential',
                    decay: 3,
                    onUpdate: function (v, t) {
                        loot.position = v;
                    },
                    onComplete: function () {
                        var droppedLoot = droppedContainer.getItems();
                        // loot dropped!
                        loot.removeSelf();
                        // set title fo loot
                        loot.setText();

                        // set position relative to container
                        loot.position = new Vector2(0, 0);
                        // reset loot scale because the x2 scale is now handled by the slot
                        loot.scale = new Vector2(1, 1);
                        dropped += 1;
                        // get last slot and add item to that
                        droppedLoot[droppedLoot.length - 1].attach(loot);
                        // can show next item
                        if (lootEntities.length !== 0) {
                            isShowingItem = false;
                        }
                    }
                });
            };
            var moveContainerLeft = function () {
                var lastLoot;
                var droppedLoot = droppedContainer.getItems();
                // move container to the left to make space for new loot
                // new Tween({
                //     from: droppedContainer.position.x,
                //     to: droppedContainer.position.x - lootSpacing,
                //     in: 10,
                //     delay: 0,
                //     ease: 'linear',
                //     onUpdate: function (v, t) {
                //         droppedContainer.position.x = v;
                //     }
                // });
                // snap to the latest loot slot
                droppedContainer.snapTo(droppedLoot.length);
            };
            loot.position = chest.position.clone();
            loot.scale = new Vector2(0, 0);
            chest.parent.attach(loot);

            // add a slot to drop loot in
            droppedContainer.addItem(new Slot({}));

            if (dropped) {
                // move container left to make room for next loot
                moveContainerLeft();
            }
            moveLoot();
        };
        /*
         * Initialize the loot item
         */
        var addLootEntity = function (data) {
            var keys = data.item.split('/');
            var category = keys[0];
            var itemName = keys[1];
            var itemJson = Bento.assets.getJson(data.item);
            var spriteSheet = itemJson.inventory.image;
            var imageName = data.imageName;
            var textString = ItemManager.getName(data.item);
            var tier = itemJson.inventory.tier;

            var type;
            var char;

            if (category === 'playertokens' || category === 'enemytokens' || category === 'evolve') {
                type = category;
                char = itemName;
            }

            var amount = data.amount || 1;
            var sprite = new Sprite({
                spriteSheet: spriteSheet,
                imageName: imageName
            });
            var loot = new Entity({
                name: 'loot',
                components: [
                    sprite
                ]
            }).extend({
                amount: amount,
                type: type,
                character: char,
                tier: tier
            });
            // add a number if more than 1 loot is given
            var lootAmountText = new ShadowText({
                position: new Vector2(16, 6),
                text: amount,
                font: 'font',
                fontSize: 16,
                fontColor: '#ffffff',
                align: 'right',
                textBaseline: 'middle',
                maxWidth: 20,
                maxHeight: 16,
                linebreaks: false
            });

            // overwrite origin to middle
            sprite.setOriginRelative(new Vector2(0.5, 0.5));

            if (amount > 1) {
                loot.attach(lootAmountText);
            }

            lootEntities.push(loot);

            loot.extend({
                setText: function () {
                    // sets the title
                    lootTitle.setText(textString);
                },
                getAmount: function () {
                    return amount;
                },
                getInfo: function () {
                    return data.item;
                },
                showItemInfo: function () {
                    var itemInfoPopup = new ItemInfo({
                        z: Globals.WorldSelectLayers.IAP + 2,
                        item: data.item
                    });
                    Bento.objects.attach(itemInfoPopup);
                },
                onDropped: data.onDropped
            });
        };
        /*
         * End of dropping items out of the chest
         */
        var completedLoot = function () {
            // all loot collected
            var claimButton = new ClaimButton({
                onClick: function () {
                    claimButton.setActive(false);
                    var delay = 0;
                    lootGrid.iterate(function (row) {
                        row.iterate(function (gridSlot, i, l, breakLoop) {
                            new Tween({
                                from: gridSlot.position,
                                to: new Vector2(0, viewport.height - gridSlot.position.y),
                                in: 30,
                                delay: delay,
                                ease: 'easeInBack',
                                onUpdate: function (v, t) {
                                    gridSlot.position = v;
                                }
                            });
                            new Tween({
                                from: 1,
                                to: 0.2,
                                in: 30,
                                delay: delay + 15,
                                ease: 'easeInBack',
                                onUpdate: function (v, t) {
                                    gridSlot.scale.x = v;
                                    gridSlot.scale.y = v;
                                }
                            });
                            delay += 2;
                        });
                        new Tween({
                            from: 1,
                            to: 0,
                            in: 10,
                            delay: delay + 40,
                            onUpdate: function (v, t) {
                                entity.alpha = v;
                            },
                            onComplete: function () {
                                entity.removeSelf();
                            }
                        });
                    });
                }
            }).attach(new Anchor({
                screenPosition: new Vector2(0.5, 0.88),
                offset: new Vector2(0, 0)
            }));

            // remove tokenBar if necessary
            if (tokenBar) {
                tokenBar.removeSelf();
                tokenBar = null;
            }
            if (tierStars) {
                tierStars.removeSelf();
                tierStars = null;
            }

            // can't click anymore
            blurBackground.remove(clickable);

            // remove character
            new Tween({
                from: 1,
                to: 0,
                in: 30,
                ease: 'linear',
                onUpdate: function (v, t) {
                    characterEntity.alpha = v;
                },
                onComplete: function () {
                    characterEntity.removeSelf();
                }
            });
            // add claimbutton
            entity.attach(claimButton);
            claimButton.visible = false;
            claimButton.setActive(false);
            new Tween({
                from: claimButton.position.y + viewport.height / 2,
                to: claimButton.position.y,
                in: 30,
                delay: 60,
                ease: 'elastic',
                decay: 5,
                oscillations: 1,
                onStart: function () {
                    claimButton.visible = true;
                },
                onUpdate: function (v, t) {
                    claimButton.position.y = v;
                },
                onComplete: function () {
                    claimButton.setActive(true);
                }
            });

            // allow user to scroll back
            // droppedContainer.setArea(undefined);

            // move items to lootGrid
            new Tween({
                from: 1,
                to: 0,
                in: 30,
                onUpdate: function (v, t) {
                    // fade out slots
                    droppedContainer.iterate(function (slot) {
                        slot.alpha = v;
                    });
                },
                onComplete: function () {
                    // split grid into rows
                    var currentRow = 0;
                    var rows = [];
                    var addRow = function () {
                        var row = new CenteredList({
                            name: 'row',
                            dimension: new Rectangle(0, 0, 44, 44),
                            spacing: 2
                        });
                        rows.push(row);
                        lootGrid.addItem(row);
                        return row;
                    };
                    droppedContainer.iterate(function (slot) {
                        // get row
                        var row = rows[currentRow];
                        // not initialized yet
                        if (!row) {
                            row = addRow();
                        }
                        // row is full
                        if (row.length() >= 4) {
                            currentRow += 1;
                            row = addRow();
                        }
                        var item = slot.getItem();
                        var gridSlot = new GridSlot({
                            item: item,
                            onClick: function () {
                                // cancel showing item if user is scrolling the grid
                                return lootGrid.didMove(1);
                            }
                        });
                        row.addItem(gridSlot);
                    });
                    lootGrid.arrange();
                    Bento.audio.stopSound('sfx_lootfinished');
                    Bento.audio.playSound('sfx_lootfinished');

                    // remove title
                    lootTitle.setText('');
                }
            });

            // show confetti
            entity.attach(new Confetti({}));
        };
        /*
         * Initialization behavior
         */
        var behavior = {
            name: 'behavior',
            addChestContents: function () {
                Utils.forEach(chestContents, function (content, i, l, breakLoop) {
                    // content can call one randomizer
                    if (content.fn) {
                        randomizerFunctions[content.fn](content.amount, content.param1, content.param2);
                    } else {
                        // or a content can be the literal keys and amounts
                        var count = content.amount || content.count;
                        var item = content.key || content.item;
                        addLootEntity({
                            amount: count,
                            item: item
                        });
                        // add the item immediately to player inventory
                        ReceiveItem({
                            key: item,
                            amount: count,
                            // manually manage notification
                            skipNotification: true
                        });
                    }
                });
            },
            predefinedChest: function () {
                // Used for giving pre-defined chests
                var classicChestsOpened = SaveManager.load('classicChestsOpened');
                var tutorialChest = SaveManager.load('tutorialChest');
                var tenjinEvents = function () {
                    // Analytics
                    var chestsOpenedTotal = SaveManager.load('chestsOpenedTotal');
                    var chestsClassicTotal = SaveManager.load('chestsClassicTotal');
                    var chestsSpecialTotal = SaveManager.load('chestsSpecialTotal');
                    var chestsMythicTotal = SaveManager.load('chestsMythicTotal');
                    chestsOpenedTotal += 1;
                    if (chestType === 'classic') {
                        chestsClassicTotal += 1;
                        Tenjin.sendEvent('chestClassic');
                    } else if (chestType === 'special') {
                        chestsSpecialTotal += 1;
                        Tenjin.sendEvent('chestSpecial');
                    } else if (chestType === 'mythic') {
                        chestsMythicTotal += 1;
                        Tenjin.sendEvent('chestMythic');
                    }
                    SaveManager.save('chestsOpenedTotal', chestsOpenedTotal);
                    SaveManager.save('chestsClassicTotal', chestsClassicTotal);
                    SaveManager.save('chestsSpecialTotal', chestsSpecialTotal);
                    SaveManager.save('chestsMythicTotal', chestsMythicTotal);

                    // Tenjin
                    Tenjin.sendEventAndValue('chestsOpenedTotal', chestsOpenedTotal);
                    Tenjin.sendEventAndValue('chestsClassicTotal', chestsClassicTotal);
                    Tenjin.sendEventAndValue('chestsSpecialTotal', chestsSpecialTotal);
                    Tenjin.sendEventAndValue('chestsMythicTotal', chestsMythicTotal);

                    if (chestsOpenedTotal === 3) {
                        Tenjin.sendEvent('openedChests_3');
                    }
                };

                // Google analtyics
                Analytics.event('Chests', chestType);

                // Post achievement for if they haven't got the 'Treasure Hunter' achievement yet
                Utils.postAchievement("CgkI9NqdkoAGEAIQCg");

                // see Globals for chest amounts
                if (chestType === 'classic') {
                    if (tutorialChest > 0) {
                        classicChestsOpened += 1;
                        SaveManager.save('classicChestsOpened', classicChestsOpened);
                        chestTier = 1;
                        if (classicChestsOpened === 1) {
                            giveChest1();
                        } else if (classicChestsOpened === 2) {
                            giveChest2();
                        } else {
                            randomizeUpgradeItems(Globals.chests[chestType].tokens, 3);
                            // 1 enemy token of tier 1-2
                            randomizeEnemyToken(Globals.chests[chestType].enemytokens, 1, 2);
                            finishEnemyTokens();
                        }
                        // evolve items
                        randomizeEvolveItems(Globals.chests[chestType].evolve, true);
                        // consumables
                        randomizeConsumableItems(Globals.chests[chestType].consumables);

                        EventSystem.fire('classicChestsOpened');
                    } else {
                        // tutorial: give an exact amount of items
                        giveTutorialChest();
                        // evolve
                        randomizeEvolveItems(Globals.chests[chestType].evolve, true);
                    }
                } else if (chestType === 'special') {
                    chestTier = 2;
                    randomizeUpgradeItems(Globals.chests[chestType].tokens, 5);
                    // enemy tokens of 1-3
                    // guaranteed 4 token of tier 2-3
                    randomizeEnemyToken(Globals.chests[chestType].enemytokens - 4, 1, 3);
                    randomizeEnemyToken(4, 2, 3);
                    finishEnemyTokens();
                    // evolve items
                    randomizeEvolveItems(Globals.chests[chestType].evolve, false);
                    // consumables
                    randomizeConsumableItems(Globals.chests[chestType].consumables);
                    // coins
                    addLootEntity({
                        amount: Globals.chests[chestType].coins,
                        item: 'currency/coin',
                        onClaim: function () {}
                    });
                    ReceiveItem({
                        key: 'currency/coin',
                        amount: Globals.chests[chestType].coins
                    });
                } else if (chestType === 'mythic') {
                    chestTier = 3;
                    randomizeUpgradeItems(Globals.chests[chestType].tokens, 8);
                    // enemy tokens of 2-3
                    // guaranteed 5 token of tier 3
                    randomizeEnemyToken(Globals.chests[chestType].enemytokens - 5, 2, 3);
                    randomizeEnemyToken(5, 3, 3);
                    finishEnemyTokens();
                    // evolve items
                    randomizeEvolveItems(Globals.chests[chestType].evolve, false);
                    // consumables
                    randomizeConsumableItems(Globals.chests[chestType].consumables);
                    // coins
                    addLootEntity({
                        amount: Globals.chests[chestType].coins,
                        item: 'currency/coin',
                        onClaim: function () {}
                    });
                    ReceiveItem({
                        key: 'currency/coin',
                        amount: Globals.chests[chestType].coins
                    });
                }

                if (tutorialChest === 0) {
                    // opened first chest
                    SaveManager.save('tutorialChest', 1);
                    EventSystem.fire('openedFirstChest');
                }

                tenjinEvents();
            },
            start: function () {
                checkEvent();
                // collect all consumable item names
                if (!consumables) {
                    collectNames();
                }
                EventSystem.on('preventScroll', cancelScroll);

                if (chestContents) {
                    // custom contents
                    behavior.addChestContents();
                } else {
                    // predefined contents
                    behavior.predefinedChest();
                }

                // lower music volume for the jingle
                new Tween({
                    from: 1,
                    to: 0,
                    in: 30,
                    onUpdate: function (v, t) {
                        Bento.audio.setVolume(v, 'bgm_worldmap');
                    },
                    onComplete: function () {
                        Bento.audio.stopAllMusic();
                        new Tween({
                            from: 0,
                            to: 1,
                            in: 30,
                            onStart: function () {
                                Bento.audio.setVolume(0, 'bgm_chest');
                                Bento.audio.playMusic('bgm_chest');
                            },
                            onUpdate: function (v, t) {
                                Bento.audio.setVolume(v, 'bgm_chest');
                            }
                        });
                    }
                });

                // bounce in chest
                Bento.audio.stopSound('sfx_chestdrop');
                Bento.audio.playSound('sfx_chestdrop');
                var groundHeight = chest.position.y - 0.38;
                new Tween({
                    from: chest.position.y - 32,
                    to: chest.position.y,
                    in: 30,
                    delay: 0,
                    ease: 'easeOutBounce',
                    onUpdate: function (v, t) {
                        chest.position.y = v;
                        if (chest.position.y >= groundHeight) {
                            Bento.audio.playSound('sfx_chestdrop');
                            Bento.audio.setVolume(0.8 - t / 40 * 0.8, 'sfx_chestdrop');
                        }
                    },
                    onComplete: function () {
                        Bento.audio.setVolume(1, 'sfx_chestdrop');
                    }
                });
            },
            destroy: function () {
                EventSystem.off('preventScroll', cancelScroll);

                // restore audio
                new Tween({
                    from: 1,
                    to: 0,
                    in: 30,
                    onUpdate: function (v, t) {
                        Bento.audio.setVolume(v, 'bgm_chest');
                    },
                    onComplete: function () {
                        Bento.audio.stopAllMusic();
                        new Tween({
                            from: 0,
                            to: 1,
                            in: 30,
                            onStart: function () {
                                Bento.audio.setVolume(0, 'bgm_worldmap');
                                Bento.audio.playMusic('bgm_worldmap');
                            },
                            onUpdate: function (v, t) {
                                Bento.audio.setVolume(v, 'bgm_worldmap');
                            },
                            onComplete: function () {
                                if (onComplete) {
                                    onComplete();
                                }
                            }
                        });
                    }
                });
            }
        };
        var giveTutorialChest = function () {
            InventoryManager.addTokens('playertokens/boy', 3);
            addLootEntity({
                item: 'playertokens/boy',
                amount: 3,
                onDropped: function () {
                    LevelUpNotification.check('boy', 'playertokens', 3);
                }
            });
            // enemy tokens
            InventoryManager.addTokens('enemytokens/rat', 1);
            addLootEntity({
                item: 'enemytokens/rat',
                amount: 1,
                onDropped: function () {}
            });
            // consumable
            addLootEntity({
                amount: 1,
                item: 'consumables/defensebonus2'
            });
            InventoryManager.addItem('consumables/defensebonus2', 1);
        };
        var giveChest1 = function () {
            InventoryManager.addTokens('playertokens/boy', 2);
            addLootEntity({
                item: 'playertokens/boy',
                amount: 2,
                onDropped: function () {
                    LevelUpNotification.check('boy', 'playertokens', 2);
                }
            });
            //
            InventoryManager.addTokens('playertokens/krillin', 1);
            addLootEntity({
                item: 'playertokens/krillin',
                amount: 1,
                onDropped: function () {
                    LevelUpNotification.check('krillin', 'playertokens', 1);
                }
            });
            InventoryManager.addTokens('enemytokens/rat', 1);
            addLootEntity({
                item: 'enemytokens/rat',
                amount: 1,
                onDropped: function () {}
            });

        };
        var giveChest2 = function () {
            InventoryManager.addTokens('playertokens/boy', 2);
            addLootEntity({
                item: 'playertokens/boy',
                amount: 2,
                onDropped: function () {
                    LevelUpNotification.check('boy', 'playertokens', 2);
                }
            });
            //
            InventoryManager.addTokens('playertokens/krillin', 1);
            addLootEntity({
                item: 'playertokens/krillin',
                amount: 1,
                onDropped: function () {
                    LevelUpNotification.check('krillin', 'playertokens', 1);
                }
            });
            InventoryManager.addTokens('enemytokens/bigrat', 1);
            addLootEntity({
                item: 'enemytokens/bigrat',
                amount: 1,
                onDropped: function () {}
            });
        };
        var giveChest3 = function () {
            InventoryManager.addTokens('playertokens/boy', 2);
            addLootEntity({
                item: 'playertokens/boy',
                amount: 2,
                onDropped: function () {
                    LevelUpNotification.check('boy', 'playertokens', 2);
                }
            });
            //
            InventoryManager.addTokens('playertokens/krillin', 1);
            addLootEntity({
                item: 'playertokens/krillin',
                amount: 1,
                onDropped: function () {
                    LevelUpNotification.check('krillin', 'playertokens', 1);
                }
            });
        };
        var chestTier = 0;
        var pickRandom = function (array) {
            var weightedItems = [];
            var winner;
            Utils.forEach(array, function (item, i, l, breakLoop) {
                var weight = 1;
                if (item.tier) {
                    if (item.tier !== chestTier) {
                        weight = 0;
                    }
                    // if (item.tier > chestTier + 1) {
                    //     // cant get item that has more than 1 tier difference
                    //     weight = 0;
                    // } else if (item.tier !== chestTier) {
                    //     // lower chance of getting higher tier or lower tier item
                    //     weight = 0.25;
                    // }
                }
                weightedItems.push({
                    value: item.key,
                    tier: item.tier,
                    weight: weight
                });
            });
            winner = weightedRandom(weightedItems);
            return winner;
        };

        var randomizeUpgradeItems = function (amount, maximum) {
            //get locked and unlocked characters
            var lockedCharacters = CharacterManager.getAvailableCharacters();
            var myCharacters = CharacterManager.getUnlockedCharacters();
            var i;
            var ii;
            for (i = myCharacters.length - 1; i >= 0; i--) {
                ii = lockedCharacters.indexOf(myCharacters[i]);
                if (ii !== -1) {
                    lockedCharacters.splice(ii, 1);
                }
            }
            // clone lockedCharacters
            lockedCharacters = lockedCharacters.concat([]);
            // remove exclusions
            if (chestType !== 'special' && chestType !== 'mythic') {
                for (i = exclusions.length - 1; i >= 0; i--) {
                    ii = lockedCharacters.indexOf(exclusions[i]);
                    if (ii !== -1) {
                        lockedCharacters.splice(ii, 1);
                    }
                }
            }
            var makeWeightsArray = function (originalArray) {
                var iii = 0;
                var array = [];
                var length = originalArray.length;
                var char;
                var weight;
                for (iii; iii < length; ++iii) {
                    weight = Utils.getRandomRangeFloat(0.5, 1);
                    // character is maxed out: 0 weight
                    char = originalArray[iii];
                    if (CharacterManager.getLevel(char) >= CharacterManager.getMaxTotal(char)) {
                        weight = 0;
                    }
                    array.push(weight);
                }
                return array;
            };
            var divide = Utils.divideAndRandomizeRounding;
            var divideLockedWithWeights = function () {
                // we apply weights for the locked playertokens
                // the token with highest weight counts double
                var weights = [];
                var highest = -Infinity;
                var highestIndex = 0;
                var lockedTokens = {};
                // create weights array and find out which one has the highest weight
                Utils.forEach(lockedCharacters, function (char, index) {
                    var json = Bento.assets.getJson('playertokens/' + char);
                    var weight = json.weight || 1;
                    weights[index] = {
                        value: 'playertokens/' + char,
                        weight: weight
                    };

                    if (weight > highest) {
                        highest = weight;
                        highestIndex = index;
                    }
                });
                // double the highest weight
                weights[highestIndex].weight *= 2;

                // apply weighted random to see who gets the tokens
                Utils.repeat(amountLocked, function () {
                    var winner = weightedRandom(weights);
                    lockedTokens[winner] = lockedTokens[winner] || 0;
                    lockedTokens[winner] += 1;
                });

                // collect results
                Utils.forEach(lockedTokens, function (count, item) {
                    var char = item.split('/')[1];
                    addLootEntity({
                        amount: count,
                        item: item,
                        onClaim: function () {},
                        onDropped: function () {
                            LevelUpNotification.check(char, 'playertokens', count);
                        }
                    });
                    // add the item immediately to player inventory
                    InventoryManager.addItem(item, count);
                });
            };
            var divideTokensPerChar = function (amountsPerChar, charArray) {
                Utils.forEach(amountsPerChar, function (amountPerChar, index) {
                    if (amountPerChar === 0) {
                        return;
                    }
                    var char = charArray[index];
                    InventoryManager.addTokens('playertokens/' + char, amountPerChar);
                    addLootEntity({
                        type: 'playertokens',
                        character: char,
                        item: 'playertokens/' + char,
                        amount: amountPerChar,
                        onDropped: function () {
                            LevelUpNotification.check(char, 'playertokens', amountPerChar);
                        }
                    });
                });
            };
            // divide up 35% of amount to locked characters, if there are any locked characters to begin with
            var lockedChance = 0.35;
            if (isChestEvent()) {
                lockedChance = 0.65;
            }
            var amountsCharacters = divide(amount, [1 - lockedChance, lockedChance * (lockedCharacters.length > 0)]);
            var amountUnlocked = amountsCharacters[0];
            var amountLocked = amountsCharacters[1];

            // divide evenly among characters
            var amountsUnlocked;
            var amountsLocked;
            if (amountUnlocked) {
                amountsUnlocked = divide(amountUnlocked, makeWeightsArray(myCharacters));
                divideTokensPerChar(amountsUnlocked, myCharacters);
            }
            if (amountLocked) {
                // amountsLocked = divide(amountLocked, makeWeightsArray(lockedCharacters.length, false));
                // divideTokensPerChar(amountsLocked, lockedCharacters);
                divideLockedWithWeights();
            }
        };
        var randomizeNonConsumableItems = function (amount) {
            Utils.repeat(amount, function () {
                var item = pickRandom(nonConsumables);

                addLootEntity({
                    amount: 1,
                    item: item
                });

                // add the item immediately to player inventory
                InventoryManager.addItem(item, 1);
            });

        };
        var randomizeConsumableItems = function (amount) {
            var amounts = {};
            // randomize a few times and collect result
            Utils.repeat(amount, function () {
                var item = pickRandom(consumables);

                amounts[item] = amounts[item] || 0;
                amounts[item] += 1;
            });
            // collect results
            Utils.forEach(amounts, function (count, item) {
                addLootEntity({
                    amount: count,
                    item: item,
                    onClaim: function () {}
                });
                // add the item immediately to player inventory
                InventoryManager.addItem(item, count);
            });
        };
        var enemyTokenAmounts = {};
        var randomizeEnemyToken = function (amount, minimumTier, maximumTier) {
            // collect enemy tokens and set weights
            var allTokens = [];
            var available = AlliesManager.getAvailable();

            minimumTier = minimumTier || 1;
            Utils.forEach(available, function (allyName) {
                // get json and check tier
                var json = Bento.assets.getJson('enemytokens/' + allyName);
                var tier = json.inventory.tier;
                // set weight
                var weight = 1;

                if (tier < minimumTier) {
                    return;
                }
                if (tier > maximumTier) {
                    return;
                }

                if (tier === 1) {
                    weight = 0.25;
                } else if (tier === 2) {
                    weight = 0.05;
                } else {
                    // ???
                    weight = 0.01;
                }
                allTokens.push({
                    value: 'enemytokens/' + allyName,
                    weight: weight
                });
            });
            // randomize a couple of times
            Utils.repeat(amount, function () {
                var item = weightedRandom(allTokens);
                enemyTokenAmounts[item] = enemyTokenAmounts[item] || 0;
                enemyTokenAmounts[item] += 1;
            });

            // NOTE: call finishEnemyTokens after using randomizeTokens
        };
        var finishEnemyTokens = function () {
            Utils.forEach(enemyTokenAmounts, function (count, item) {
                addLootEntity({
                    amount: count,
                    item: item
                });

                // add the item immediately to player inventory
                InventoryManager.addItem(item, count);
            });
        };
        var randomizeEvolveItems = function (amount, onlyUnlocked) {
            var allCharacters = CharacterManager.getAvailableCharacters();
            var myCharacters = CharacterManager.getUnlockedCharacters();
            var characters = allCharacters;
            var weights = [];
            var isMaxEvolved = function (char) {
                return CharacterManager.getMaxLevel(char) >= CharacterManager.getMaxTotal(char);
            };
            var amounts = {};

            if (onlyUnlocked) {
                characters = myCharacters;

                // quick check: are all your unlocked characters fully evolved
                var allEvolved = true;
                Utils.forEach(characters, function (char, i, l, breakLoop) {
                    if (!isMaxEvolved(char)) {
                        allEvolved = false;
                    }
                });

                // getting evolve items for your unlocked chars is pointless now, switch to all characters
                if (allEvolved) {
                    characters = allCharacters;
                }
            }

            Utils.forEach(characters, function (char, i, l, breakLoop) {
                // randomize weights, 0 weight on characters that were fully evolved
                var maxEvolved = isMaxEvolved(char);
                var isUnlocked = (myCharacters.indexOf(char) >= 0);
                var weight = 1;

                if (isUnlocked) {
                    weight = 9;
                }

                // no point on giving evolve items now
                if (maxEvolved) {
                    weight = 0;
                }

                weights.push({
                    value: 'evolve/' + char,
                    weight: weight
                });
            });
            // randomize evolve items
            Utils.repeat(amount, function () {
                var item = weightedRandom(weights);
                amounts[item] = amounts[item] || 0;
                amounts[item] += 1;
            });
            // give
            Utils.forEach(amounts, function (count, item) {
                addLootEntity({
                    amount: count,
                    item: item
                });

                // add the item immediately to player inventory
                InventoryManager.addItem(item, count);
            });
        };
        var randomizerFunctions = {
            randomizeUpgradeItems: randomizeUpgradeItems,
            randomizeConsumableItems: randomizeConsumableItems,
            randomizeEnemyToken: randomizeEnemyToken,
            finishEnemyTokens: finishEnemyTokens,
            addGems: function (amount) {
                addLootEntity({
                    amount: amount,
                    item: 'currency/gem'
                });
                ReceiveItem({
                    key: 'currency/gem',
                    amount: amount
                });
            }
        };
        var jumpBox = function () {
            chest.getComponent('anchor', function (anchor) {
                anchor.resetPosition();
            });
            new Tween({
                from: chest.position.y,
                to: chest.position.y - 10,
                in: 10,
                delay: 10,
                ease: 'easeOutCubic',
                onUpdate: function (v, t) {
                    chest.position.y = v;
                },
                onComplete: function () {
                    new Tween({
                        from: chest.position.y,
                        to: chest.position.y + 10,
                        in: 10,
                        delay: 0,
                        ease: 'easeOutCubic',
                        onUpdate: function (v, t) {
                            chest.position.y = v;
                        },
                        onComplete: function () {
                            Bento.audio.stopSound('sfx_chestdrop');
                            Bento.audio.playSound('sfx_chestdrop');
                            Bento.audio.setVolume(0.4, 'sfx_chestdrop');
                        }
                    });
                }
            });
        };
        /*
         * Main Entity
         */
        var entity = new Entity({
            z: settings.z || Globals.WorldSelectLayers.CHESTOVERLAY,
            name: 'chestOverlay',
            position: settings.position,
            float: true,
            components: [
                new Modal({}),
                blurBackground.attach(clickable),
                chest,
                behavior,
                characterEntity,
                droppedContainer,
                lootTitle,
                lootGrid,
                tapToOpen,
                new OrientationLock({})
            ]
        });
        return entity;
    };
});