/**
 * Shows chest details before actually opening it
 * @moduleName ChestPopup
 */
bento.define('gui/iap/chestpopup', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'gui/blurbackground',
    'globals',
    'gui/headerbox',
    'components/anchor',
    'gui/shadowtext',
    'components/modal',
    'managers/currency',
    'components/glow',
    'gui/iap/chestoverlay',
    'gui/worldselect/topbarsimple',
    'modules/localization',
    'components/sparkles',
    'gui/tier',
    'gui/taphand',
    'managers/save',
    'managers/resolution',
    'gui/iconbutton',
    'gui/centeredlist',
    'components/backbutton'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    BlurBackground,
    Globals,
    HeaderBox,
    Anchor,
    ShadowText,
    Modal,
    CurrencyManager,
    Glow,
    ChestOverlay,
    TopBarSimple,
    Localization,
    Sparkles,
    TierStars,
    TapHand,
    SaveManager,
    ResolutionManager,
    IconButton,
    CenteredList,
    BackButton
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var onNotEnoughGems = settings.onNotEnoughGems;
        var chestType = settings.chest;
        var cost = Globals.chests[chestType].cost;
        var tokenAmount = Globals.chests[chestType].tokens + Globals.chests[chestType].enemytokens;
        var consumableAmount = Globals.chests[chestType].consumables;
        var coinAmount = Globals.chests[chestType].coins;
        var evolveAmount = Globals.chests[chestType].evolve;
        var blurBackground = new BlurBackground({
            darken: 0.5,
            blurriness: 8
        });
        var topbar = new TopBarSimple({
            preventClick: true // don't open the IAP menu while we're in the IAP menu!
        });
        var buttonClose = new IconButton({
            name: 'buttonClose',
            sfx: 'sfx_button',
            imageName: 'gui2/misc/button_small_secondary',
            iconName: 'gui2/misc/button_icon_back',
            position: new Vector2(-52, 44),
            frameCountX: 1,
            frameCountY: 3,
            onClick: function () {
                // tween out
                buttonClose.setActive(false);
                button.setActive(false);
                new Tween({
                    from: 1,
                    to: 0,
                    in: 10,
                    ease: 'linear',
                    onUpdate: function (v, t) {
                        blurBackground.alpha = v;
                        headerBox.scale.x = v;
                        headerBox.scale.y = v;
                    },
                    onComplete: function () {
                        entity.removeSelf();
                    }
                });
            },
            components: [
                new BackButton({
                    removeSelf: true,
                    onPressed: function (data) {
                        data.entity.doCallback();
                    }
                })
            ]
        });
        var chestBehavior = {
            name: 'chestBehavior',
            start: function () {
                // drop down
                new Tween({
                    from: chest.position.y - 32,
                    to: chest.position.y,
                    in: 10,
                    ease: 'quadratic',
                    onUpdate: function (v, t) {
                        chest.position.y = v;
                    },
                    onComplete: function () {
                        Bento.audio.stopSound('sfx_chestdrop');
                        Bento.audio.playSound('sfx_chestdrop');
                        //add open sfx with a delay of 3 frames to have a smoother transition between drop and open sounds
                        Utils.timeout(3, function () {
                            Bento.audio.stopSound('sfx_chestopen');
                            Bento.audio.playSound('sfx_chestopen');
                        });
                        chestSprite.setAnimation('open');
                        new Tween({
                            from: 1,
                            to: 0,
                            in: 30,
                            ease: 'elastic',
                            decay: 8,
                            oscillations: 1,
                            onUpdate: function (v, t) {
                                chest.scale.x = 2 + v;
                                chest.scale.y = 2 - v;
                            }
                        });
                    }
                });
            }
        };

        var chestPosition = function () {
            if (ResolutionManager.isPortrait()) {
                return new Vector2(0, -69);
            }
            return new Vector2(-124, 22);
        };

        var moveChest = function () {
            chest.position = chestPosition();
        };

        var chestSprite = new Sprite({
            imageName: 'gui/iap/chest2-' + chestType,
            frameCountX: 2,
            originRelative: new Vector2(0.5, 1),
            animations: {
                open: {
                    speed: 0,
                    frames: [1]
                },
                default: {
                    speed: 0,
                    frames: [0]
                }
            }
        });
        var chest = new Entity({
            name: 'chest',
            position: chestPosition(),
            scale: new Vector2(2, 2),
            components: [
                chestSprite,
                chestBehavior,
                new Sparkles({
                    spawnRate: 0.05,
                    position: new Vector2(0, -16)
                })
            ]
        });
        var gemText;
        // swap with keys text
        if (chestType === 'classic') {
            // amount / cost
            var amountText = new ShadowText({
                position: new Vector2(0, 0),
                text: CurrencyManager.getKeys().toString(),
                font: 'font',
                fontSize: 24,
                fontColor: CurrencyManager.getKeys() < cost ? '#ff0000' : '#ffffff',
                align: 'right',
                textBaseline: 'middle',
                maxHeight: 22
            });
            var totalText = new ShadowText({
                position: new Vector2(0, 0),
                text: '/' + cost.toString(),
                font: 'font',
                fontSize: 22,
                fontColor: '#ffffff',
                align: 'left',
                textBaseline: 'middle',
                maxWidth: 22,
                maxHeight: 22,
            });
            gemText = new Entity({
                name: 'gemText',
                position: new Vector2(9, -1),
                components: [
                    amountText,
                    totalText
                ]
            });
        } else {
            gemText = new ShadowText({
                z: 0,
                position: new Vector2(0, -1),
                text: cost.toString(),
                font: 'font',
                fontSize: 22,
                fontColor: '#ffffff',
                align: 'left',
                textBaseline: 'middle',
                maxWidth: 48,
                linebreaks: false
            });
        }
        var currencyIcon = new Entity({
            name: 'currencyIcon',
            position: new Vector2(
                chestType === 'classic' ? -24 : -16,
                0
            ),
            components: [
                new Sprite({
                    imageName: chestType === 'classic' ? 'gui/key' : 'gui/gem',
                    originRelative: new Vector2(0.5, 0.5)
                })
            ]
        });
        var buttonInfo = new Entity({
            name: 'buttonInfo',
            components: [
                currencyIcon,
                gemText
            ]
        });
        var button = new IconButton({
            z: 0,
            name: 'item',
            sfx: 'sfx_button',
            imageName: 'gui2/misc/button_large_primary',
            icon: buttonInfo,
            frameCountX: 1,
            frameCountY: 3,
            animations: {
                'up': {
                    frames: [0]
                },
                'down': {
                    frames: [1]
                },
                'inactive': {
                    frames: [2]
                },
            },
            position: new Vector2(24, 44),
            float: false,
            updateWhenPaused: 0,
            onClick: function () {
                var shouldClose;
                var canSpend = CurrencyManager.canSpendGems;
                var spend = CurrencyManager.spendGems;

                // use keys for classic chest
                if (chestType === 'classic') {
                    canSpend = CurrencyManager.canSpendKeys;
                    spend = CurrencyManager.spendKeys;
                }

                if (!canSpend(cost)) {
                    // depending on the context of the chestpopup, we should either
                    // open the iap shop, or scroll to the iap gem section (because we dont want to open
                    // the shop inside the shop)
                    // TODO: does this also apply to keys??
                    if (onNotEnoughGems) {
                        shouldClose = onNotEnoughGems();
                    }
                    if (shouldClose) {
                        entity.removeSelf();
                    }
                    return;
                }

                // close popup
                entity.removeSelf();
                Bento.audio.stopSound('sfx_chestdrop');
                Bento.audio.stopSound('sfx_chestopen');
                // spend gems
                if (canSpend(cost)) {
                    spend(cost, 'chest-' + chestType);
                    openChest();
                    SaveManager.saveToCloud();
                    if (chestType === 'classic') {
                        EventSystem.fire('quest-classicChest', {
                            count: 1
                        });
                    }
                }
            }
        });
        var openChest = function () {
            var overlay = new ChestOverlay({
                chest: chestType
            });
            Bento.objects.attach(overlay);
        };
        var addAmount = function (amount, height, sign, fontColor) {
            height = height || 29;
            var text = new ShadowText({
                position: new Vector2(0, height),
                text: (sign || '×') + amount.toString(),
                font: 'font',
                fontSize: 16,
                fontColor: fontColor || '#ffffff',
                align: 'center',
                textBaseline: 'middle'
            });
            return text;
        };

        var getTier = function () {
            switch (chestType) {
            case 'classic':
                return 1;
            case 'special':
                return 2;
            case 'mythic':
                return 3;
            default:
                return 3;
            }
        };

        var contents = new CenteredList({
            position: new Vector2(0, -21),
            spacing: 0
        });

        var token = new Entity({
            name: 'token',
            components: [
                new Sprite({
                    imageName: 'gui/token',
                    originRelative: new Vector2(0.5, 0.5)
                })
            ]
        }).attach(addAmount(tokenAmount, 26));

        var consumables = new Entity({
            name: 'consumables',
            components: [
                new Sprite({
                    imageName: 'gui/buttons/equipment',
                    originRelative: new Vector2(0.5, 0.5)
                }),
                new TierStars({
                    position: new Vector2(0, 19),
                    tier: getTier(),
                })
            ]
        }).attach(addAmount(consumableAmount));
        var evolveItems = new Entity({
            name: 'evolveItems',
            components: [
                new Sprite({
                    imageName: 'gui/hat',
                    originRelative: new Vector2(0.5, 0.5)
                })
            ]
        }).attach(addAmount(evolveAmount, 26));
        var coins = new Entity({
            name: 'consumables',
            components: [
                new Sprite({
                    spriteSheet: 'coin/sack'
                })
            ]
        }).attach(addAmount(coinAmount, 24, '+', '#fffd00'));
        var headerBox = new HeaderBox({
            title: Localization.getText('chest-' + chestType),
            windowImage: 'gui2/misc/splash_background_9slice',
            size: new Rectangle(0, 0, 180, 152),
            titleColor: '#80ff23',
            components: [
                contents,
                buttonClose,
                button,
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.5, 0.5),
                        offset: new Vector2(0, 18)
                    },
                    landscape: {
                        screenPosition: new Vector2(0.5, 0.5),
                        offset: new Vector2(40, 16)
                    },
                    onComplete: moveChest
                })
            ]
        });
        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {
                var tweenIn = function (delay, item) {
                    new Tween({
                        from: 0,
                        to: 1,
                        in: 40,
                        delay: delay,
                        ease: 'elastic',
                        decay: 8,
                        oscillations: 2,
                        onStart: function () {
                            item.visible = true;
                        },
                        onUpdate: function (v, t) {
                            item.scale.x = 1 - (1 - v);
                            item.scale.y = 1 - (1 - v);
                        }
                    });
                };
                var dimension = 40;
                token.dimension.width = dimension;
                consumables.dimension.width = dimension;
                evolveItems.dimension.width = dimension;
                coins.dimension.width = dimension;
                coins.getComponent('sprite', function (sprite) {
                    sprite.setOriginRelative(new Vector2(0.5, 0.5));
                    sprite.setCurrentSpeed(0.2);
                });
                // add to list
                contents.addItem(token);
                contents.addItem(consumables);
                contents.addItem(evolveItems);
                // TODO: coinamount is going to be a number
                /*if (coinAmount > 0) {
                    contents.addItem(coins);
                }*/
                // prepare for tween
                token.visible = false;
                consumables.visible = false;
                evolveItems.visible = false;
                // items appear one by one
                tweenIn(30, token);
                tweenIn(40, consumables);
                tweenIn(50, evolveItems);
                // tweenIn(50, coins);

                // tween in
                new Tween({
                    from: 0,
                    to: 1,
                    in: 20,
                    ease: 'linear',
                    onUpdate: function (v, t) {
                        blurBackground.alpha = v;
                    },
                    onComplete: function () {

                    }
                });
                new Tween({
                    from: 0,
                    to: 1,
                    in: 30,
                    delay: 0,
                    ease: 'elastic',
                    decay: 8, // for 'exponential' and 'elastic'
                    oscillations: 1, // for 'elastic', 'sin' and 'cos'
                    onUpdate: function (v, t) {
                        headerBox.scale.x = v;
                        headerBox.scale.y = v;
                    },
                    onComplete: function () {
                        headerBox.attach(chest);
                    }
                });

                // add coin in topright corner
                if (coinAmount > 0) {
                    coins.position = new Vector2(64, -72);
                    headerBox.attach(coins);
                }

                // taphand during tutorial
                if (SaveManager.load('tutorialChest') === 0) {
                    button.attach(new TapHand({
                        position: new Vector2(8, 8),
                        rotation: Math.PI * 3 / 4
                    }));
                }
            },
            destroy: function (data) {
                if (settings.onClose) {
                    settings.onClose();
                }
            }
        };
        var entity = new Entity({
            z: settings.z || Globals.WorldSelectLayers.CHESTPOPUP,
            name: 'chestPopup',
            position: settings.position,
            float: true,
            components: [
                new Modal({}),
                blurBackground,
                headerBox,
                behavior,
                topbar
            ]
        });

        if (chestType === 'classic') {
            if (CurrencyManager.canSpendKeys(cost)) {
                button.attach(new Glow({
                    average: 0.2,
                    amplitude: 0.1
                }));
            }
        } else {
            if (CurrencyManager.canSpendGems(cost)) {
                button.attach(new Glow({
                    average: 0.2,
                    amplitude: 0.1
                }));
                button.getComponent('sprite', function (sprite) {
                    sprite.setup({
                        imageName: 'gui2/misc/button_large_primary_excited',
                        frameCountX: 6,
                        frameCountY: 7,
                        padding: 1,
                        animations: {
                            up: {
                                speed: 0.34,
                                frames: [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35]
                            },
                            down: {
                                frames: [37]
                            },
                            inactive: {
                                frames: [38]
                            }
                        }
                    });
                    sprite.setAnimation('up');
                });
            }
        }
        return entity;
    };
});