/**
 * Section of the store containing chests
 * @moduleName ChestSection
 */
bento.define('gui/iap/chestsection', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'gui/iap/sectionlabel',
    'components/anchor',
    'gui/iap/gempurchase',
    'managers/currency',
    'gui/shadowtext',
    'modules/localization',
    'managers/save',
    'gui/iap/chestoverlay',
    'gui/badge',
    'components/glow',
    'gui/iap/chestpopup',
    'globals',
    'components/sparkles'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    SectionLabel,
    Anchor,
    GemPurchase,
    CurrencyManager,
    ShadowText,
    Localization,
    SaveManager,
    ChestOverlay,
    Badge,
    Glow,
    ChestPopup,
    Globals,
    Sparkles
) {
    'use strict';
    // button to open chests
    var ChestButton = function (settings) {
        var button = new ClickButton({
            name: settings.name,
            sfx: 'sfx_button',
            imageName: 'gui/iap/openbutton',
            frameCountX: 1,
            frameCountY: 3,
            animations: {
                'up': {
                    frames: [0]
                },
                'down': {
                    frames: [1]
                },
                'inactive': {
                    frames: [2]
                }
            },
            position: settings.position,
            onClick: settings.onClick,
            onButtonDown: function () {
                text.position.y = -2;
                EventSystem.fire('preventScroll');
            },
            onButtonUp: function () {
                text.position.y = -3;
            }
        });
        var badge = new Badge({
            number: settings.amount || 0,
            imageName: 'gui/badge-topright',
            position: new Vector2(button.dimension.width / 2 - 2, -button.dimension.height / 2 + 2)
        });
        var text = new ShadowText({
            position: new Vector2(0, -3),
            text: Localization.getText('openChest'),
            font: 'font',
            fontSize: 16,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            linebreaks: false,
            maxWidth: 32,
            maxHeight: 16
        });
        var glow = new Glow({
            update: function () {
                glow.visible = badge.getNumber() > 0;
            }
        });

        if (settings.amount > 0) {
            button.attach(glow);
        }
        button.attach(text);
        button.attach(badge);
        button.extend({
            setAmount: function (value) {
                badge.setNumber(value);
            }
        });

        return button;
    };

    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var onNotEnoughGems = settings.onNotEnoughGems;
        var classicGems = Globals.chests['classic'].cost;
        var specialGems = Globals.chests['special'].cost;
        var mythicGems = Globals.chests['mythic'].cost;

        var label = new SectionLabel({
            text: Localization.getText('chests'),
        });

        var chestClassic = new GemPurchase({
            title: Localization.getText('chest-classic'),
            item: 'chest-classic',
            cost: classicGems,
            type: 'key',
            position: new Vector2(-58, 23),
            onClick: function () {
                var chestPopup = new ChestPopup({
                    chest: 'classic',
                    onNotEnoughGems: onNotEnoughGems
                });
                Bento.objects.attach(chestPopup);
            }
        }).attach(new Sparkles({
            radius: 12,
            position: new Vector2(0, 44)
        }));
        var chestSpecial = new GemPurchase({
            title: Localization.getText('chest-special'),
            item: 'chest-special',
            cost: specialGems,
            position: new Vector2(0, 23),
            onClick: function () {
                var chestPopup = new ChestPopup({
                    chest: 'special',
                    onNotEnoughGems: onNotEnoughGems
                });
                Bento.objects.attach(chestPopup);
            }
        }).attach(new Sparkles({
            radius: 13,
            position: new Vector2(0, 44)
        }));
        var chestMythic = new GemPurchase({
            title: Localization.getText('chest-mythic'),
            item: 'chest-mythic',
            cost: mythicGems,
            position: new Vector2(58, 23),
            onClick: function () {
                var chestPopup = new ChestPopup({
                    chest: 'mythic',
                    onNotEnoughGems: onNotEnoughGems
                });
                Bento.objects.attach(chestPopup);
            }
        }).attach(new Sparkles({
            radius: 14,
            position: new Vector2(0, 44)
        }));

        // Open chest buttons
        var buttonClassic = new ChestButton({
            name: 'buttonClassic',
            position: new Vector2(0, 72),
            amount: SaveManager.load('chests').classic,
            onClick: function () {
                var chests = SaveManager.load('chests');
                var overlay;
                if (chests.classic > 0) {
                    chests.classic -= 1;
                    SaveManager.save('chests', chests);
                    overlay = new ChestOverlay({
                        chest: 'classic'
                    });
                    Bento.objects.attach(overlay);
                    buttonClassic.setAmount(chests.classic);
                    EventSystem.fire('updateChests');
                }
            }
        });
        var buttonSpecial = new ChestButton({
            name: 'buttonSpecial',
            position: new Vector2(0, 72),
            amount: SaveManager.load('chests').special,
            onClick: function () {
                var chests = SaveManager.load('chests');
                var overlay;
                if (chests.special > 0) {
                    chests.special -= 1;
                    SaveManager.save('chests', chests);
                    overlay = new ChestOverlay({
                        chest: 'special'
                    });
                    Bento.objects.attach(overlay);
                    buttonSpecial.setAmount(chests.special);
                    EventSystem.fire('updateChests');
                }
            }
        });
        var buttonMythic = new ChestButton({
            name: 'buttonMythic',
            position: new Vector2(0, 72),
            amount: SaveManager.load('chests').mythic,
            onClick: function () {
                var chests = SaveManager.load('chests');
                var overlay;
                if (chests.mythic > 0) {
                    chests.mythic -= 1;
                    SaveManager.save('chests', chests);
                    overlay = new ChestOverlay({
                        chest: 'mythic'
                    });
                    Bento.objects.attach(overlay);
                    buttonMythic.setAmount(chests.mythic);
                    EventSystem.fire('updateChests');
                }
            }
        });
        // activate/deactivate buttons to open chest
        var checkChests = function () {
            var chests = SaveManager.load('chests');
            if (chests.classic > 0) {
                if (!buttonClassic.parent) {
                    chestClassic.attach(buttonClassic);
                }
                // hide gem button
                chestClassic.disable();
            } else {
                if (buttonClassic.parent) {
                    buttonClassic.removeSelf();
                }
                // show gem button
                chestClassic.enable();
            }
            if (chests.special > 0) {
                if (!buttonSpecial.parent) {
                    chestSpecial.attach(buttonSpecial);
                }
                chestSpecial.disable();
            } else {
                if (buttonSpecial.parent) {
                    buttonSpecial.removeSelf();
                }
                chestSpecial.enable();
            }
            if (chests.mythic > 0) {
                if (!buttonMythic.parent) {
                    chestMythic.attach(buttonMythic);
                }
                chestMythic.disable();
            } else {
                if (buttonMythic.parent) {
                    buttonMythic.removeSelf();
                }
                chestMythic.enable();
            }
        };

        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {
                // chestMythic.setActive(false);
                entity.dimension = new Rectangle(-93, 0, 186, 123);

                chestClassic.attach(buttonClassic);
                chestSpecial.attach(buttonSpecial);
                chestMythic.attach(buttonMythic);

            },
            destroy: function (data) {},
            update: function (data) {
                var currentGems = CurrencyManager.getGems();

                // activate or deactive the button
                /*var setActive = function (item, amount) {
                    if (currentGems >= amount) {
                        if (!item.isActive()) {
                            item.setActive(true);
                        }
                    } else {
                        item.setActive(false);
                    }
                };

                setActive(chestClassic, classicGems);
                setActive(chestSpecial, specialGems);
                setActive(chestMythic, mythicGems);*/
                checkChests();

            },
            draw: function (data) {}
        };
        var entity = new Entity({
            z: 0,
            name: 'chestSection',
            updateWhenPaused: 0,
            components: [
                label,
                chestClassic,
                chestSpecial,
                chestMythic,
                behavior
            ]
        });
        return entity;
    };
});