/**
 * Claim button
 * @moduleName ClaimButton
 */
bento.define('gui/iap/claimbutton', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'gui/shadowtext',
    'modules/localization',
    'components/backbutton'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    ShadowText,
    Localization,
    BackButton
) {
    'use strict';
    /*
     * Button to close overlay
     */
    var ClaimButton = function (settings) {
        var button = new ClickButton({
            name: settings.name,
            sfx: 'sfx_button',
            imageName: 'gui2/misc/button_large_primary',
            frameCountX: 1,
            frameCountY: 3,
            position: settings.position,
            onClick: settings.onClick,
            animations: {
                'up': {
                    frames: [0]
                },
                'down': {
                    frames: [1]
                },
                'inactive': {
                    frames: [0]
                }
            },
            onButtonDown: function () {
                text.position.y = 0;
                EventSystem.fire('preventScroll');
            },
            onButtonUp: function () {
                text.position.y = -1;
            },
            components: [
                new BackButton({
                    removeSelf: true,
                    onPressed: function (data) {
                        data.entity.doCallback();
                    }
                })
            ]
        });
        var text = new ShadowText({
            position: new Vector2(0, -1),
            text: Localization.getText('claim'),
            font: 'font',
            fontSize: 20,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            linebreaks: false,
            maxWidth: 64,
            maxHeight: 20
        });

        button.attach(text);

        return button;
    };
    return ClaimButton;
});