/**
 * Section of the store containing coins
 * @moduleName CoinSection
 */
bento.define('gui/iap/coinsection', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'gui/iap/sectionlabel',
    'components/anchor',
    'gui/iap/gempurchase',
    'managers/currency',
    'globals',
    'gui/coinreward',
    'managers/save',
    'modules/localization',
    'modules/analytics'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    SectionLabel,
    Anchor,
    GemPurchase,
    CurrencyManager,
    Globals,
    CoinReward,
    SaveManager,
    Localization,
    Analytics
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();

        var label = new SectionLabel({
            text: Localization.getText('currency').coin.name,
        });

        var gemsCoins1 = Globals.gemToCoins[0].gems;
        var gemsCoins2 = Globals.gemToCoins[1].gems;
        var gemsCoins3 = Globals.gemToCoins[2].gems;

        var coins1 = new GemPurchase({
            title: Globals.gemToCoins[0].coins.toString(),
            item: 'coins-1',
            cost: gemsCoins1,
            symbol: 'gui2/iap/symbol_coin',
            position: new Vector2(-58, 23),
            onClick: function () {
                // check if can really spend gems to be safe before getting coins
                if (CurrencyManager.canSpendGems(gemsCoins1)) {
                    CurrencyManager.spendGems(gemsCoins1, 'coins');
                    CurrencyManager.addCoins(Globals.gemToCoins[0].coins);
                    Analytics.event('SourceSinks', 'gemsSinkedToCoin', gemsCoins1);
                    Analytics.event('SourceSinks', 'coinsSourcedFromGem', Globals.gemToCoins[0].coins);
                    SaveManager.saveToCloud();
                    Bento.objects.attach(new CoinReward({
                        z: Globals.WorldSelectLayers.IAP,
                        coins: Globals.gemToCoins[0].coins
                    }));
                }
            }
        });

        var coins2 = new GemPurchase({
            title: Globals.gemToCoins[1].coins.toString(),
            item: 'coins-2',
            cost: gemsCoins2,
            position: new Vector2(0, 23),
            symbol: 'gui2/iap/symbol_coin',
            onClick: function () {
                // check if can really spend gems to be safe before getting coins
                if (CurrencyManager.canSpendGems(gemsCoins2)) {
                    CurrencyManager.spendGems(gemsCoins2, 'coins');
                    CurrencyManager.addCoins(Globals.gemToCoins[1].coins);
                    Analytics.event('SourceSinks', 'gemsSinkedToCoin', gemsCoins2);
                    Analytics.event('SourceSinks', 'coinsSourcedFromGem', Globals.gemToCoins[1].coins);
                    SaveManager.saveToCloud();
                    Bento.objects.attach(new CoinReward({
                        z: Globals.WorldSelectLayers.IAP,
                        coins: Globals.gemToCoins[1].coins
                    }));
                }
            }
        });

        var coins3 = new GemPurchase({
            title: Globals.gemToCoins[2].coins.toString(),
            item: 'coins-3',
            cost: gemsCoins3,
            symbol: 'gui2/iap/symbol_coin',
            position: new Vector2(58, 23),
            onClick: function () {
                // check if can really spend gems to be safe before getting coins
                if (CurrencyManager.canSpendGems(gemsCoins3)) {
                    CurrencyManager.spendGems(gemsCoins3, 'coins');
                    CurrencyManager.addCoins(Globals.gemToCoins[2].coins);
                    Analytics.event('SourceSinks', 'gemsSinkedToCoin', gemsCoins3);
                    Analytics.event('SourceSinks', 'coinsSourcedFromGem', Globals.gemToCoins[2].coins);
                    SaveManager.saveToCloud();
                    Bento.objects.attach(new CoinReward({
                        z: Globals.WorldSelectLayers.IAP,
                        coins: Globals.gemToCoins[2].coins
                    }));
                }
            }
        });

        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {
                // coins3.setActive(false);
                entity.dimension = new Rectangle(-93, 0, 186, 123);
            },
            destroy: function (data) {},
            update: function (data) {
                var currentGems = CurrencyManager.getGems();

                // activate or deactive the button
                var setActive = function (item, amount) {
                    if (currentGems >= amount) {
                        item.setActive(true);
                    } else {
                        item.setActive(false);
                    }
                };

                setActive(coins1, gemsCoins1);
                setActive(coins2, gemsCoins2);
                setActive(coins3, gemsCoins3);
            },
            draw: function (data) {}
        };
        var entity = new Entity({
            z: 0,
            name: 'coinSection',
            updateWhenPaused: 0,
            components: [
                label,
                coins1,
                coins2,
                coins3,
                behavior
            ]
        });
        return entity;
    };
});