/**
 * Section of the store containing gems
 * @moduleName GemSection
 */
bento.define('gui/iap/gemsection', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'gui/iap/sectionlabel',
    'components/anchor',
    'gui/iap/moneypurchase',
    'modules/store',
    'managers/currency',
    'gui/iapoverlay',
    'modules/analytics',
    'modules/localization',
    'globals',
    'gui/itemget',
    'gui/shadowtext',
    'components/sparkles'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    SectionLabel,
    Anchor,
    MoneyPurchase,
    Store,
    CurrencyManager,
    IapOverlay,
    Analytics,
    Localization,
    Globals,
    ItemGet,
    ShadowText,
    Sparkles
) {
    'use strict';
    var valueHeight = 18;

    var TextLabel = function (settings) {
        var text = new ShadowText({
            position: new Vector2(-1, 2.5),
            text: settings.text,
            font: 'font',
            fontSize: 9,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            maxWidth: 32,
            maxHeight: 9,
            linebreaks: false
        });
        var entity = new Entity({
            name: 'valueLabel',
            position: new Vector2(6, 90),
            components: [
                new Sprite({
                    imageName: 'gui2/misc/bestdeal',
                    originRelative: new Vector2(0.5, 0.5)
                }),
                text
            ]
        });
        return entity;
    };

    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var label = new SectionLabel({
            text: Localization.getText('currency').gem.name,
            extraText: Localization.getText('iapRemovesAd')
        });
        // final callback for iap purchase
        var onPurchaseComplete = function (completed, iapId, purchaseInfo) {
            if (!completed) {
                // purchase error
                return;
            }
            CurrencyManager.addGems(Utils.rewardsIAP[iapId], true);
            // consumables must be "consumed" for the google play store
            Store.consume(iapId, function () {
                console.log(iapId + ' consumed.');
            });
            // resolve purchase
            Store.resolvePurchase(purchaseInfo);

            var itemGet = new ItemGet({
                z: Globals.WorldSelectLayers.IAP + 1,
                item: 'currency/gem',
                amount: Utils.rewardsIAP[iapId]
            });
            Bento.objects.attach(itemGet);
        };
        var gems1 = new MoneyPurchase({
            item: 'gems-1',
            cost: '1,99',
            iap: 'beatstreetgempack1',
            position: new Vector2(-58, 23),
            onClick: function (iapId) {
                new IapOverlay.Purchase({
                    z: Globals.Layers.SYSTEM,
                    id: iapId,
                    onPurchaseComplete: onPurchaseComplete
                });
            }
        });
        var gems2 = new MoneyPurchase({
            item: 'gems-2',
            cost: '4,99',
            iap: 'beatstreetgempack2',
            position: new Vector2(0, 23),
            onClick: function (iapId) {
                new IapOverlay.Purchase({
                    z: Globals.Layers.SYSTEM,
                    id: iapId,
                    onPurchaseComplete: onPurchaseComplete
                });
            }
        });
        var gems3 = new MoneyPurchase({
            item: 'gems-3',
            cost: '9,99',
            iap: 'beatstreetgempack3',
            position: new Vector2(58, 23),
            onClick: function (iapId) {
                new IapOverlay.Purchase({
                    z: Globals.Layers.SYSTEM,
                    id: iapId,
                    onPurchaseComplete: onPurchaseComplete
                });
            }
        }).attach(new TextLabel({
            text: Localization.getText('mostPopular')
        }));
        var gems4 = new MoneyPurchase({
            item: 'gems-4',
            cost: '24,99',
            iap: 'beatstreetgempack4',
            position: new Vector2(-58, 114 + valueHeight),
            onClick: function (iapId) {
                new IapOverlay.Purchase({
                    z: Globals.Layers.SYSTEM,
                    id: iapId,
                    onPurchaseComplete: onPurchaseComplete
                });
            }
        });
        var gems5 = new MoneyPurchase({
            item: 'gems-5',
            cost: '49,99',
            iap: 'beatstreetgempack5',
            position: new Vector2(0, 114 + valueHeight),
            onClick: function (iapId) {
                new IapOverlay.Purchase({
                    z: Globals.Layers.SYSTEM,
                    id: iapId,
                    onPurchaseComplete: onPurchaseComplete
                });
            }
        });
        var gems6 = new MoneyPurchase({
            item: 'gems-6',
            cost: '99,99',
            iap: 'beatstreetgempack6',
            position: new Vector2(58, 114 + valueHeight),
            onClick: function (iapId) {
                new IapOverlay.Purchase({
                    z: Globals.Layers.SYSTEM,
                    id: iapId,
                    onPurchaseComplete: onPurchaseComplete
                });
            }
        }).attach(new TextLabel({
            text: Localization.getText('bestDeal')
        }));

        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {
                entity.dimension = new Rectangle(-93, 0, 186, 214 + valueHeight * 2);
            },
            destroy: function (data) {},
            update: function (data) {},
            draw: function (data) {}
        };
        var entity = new Entity({
            z: 0,
            name: 'gemSection',
            updateWhenPaused: 0,
            components: [
                label,
                gems1,
                gems2,
                gems3,
                gems4,
                gems5,
                gems6,
                behavior
            ]
        });
        return entity;
    };
});