/**
 * Gridslot
 * @moduleName GridSlot
 */
bento.define('gui/iap/gridslot', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'managers/item',
    'gui/tier'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    ItemManager,
    TierStars
) {
    'use strict';
    /*
     * Slot for the grid
     */
    var GridSlot = function (settings) {
        var item = settings.item;
        var slot = new ClickButton({
            name: 'gridSlot',
            sfx: 'sfx_button',
            imageName: 'gui2/misc/panel-large',
            frameCountX: 2,
            frameCountY: 1,
            onClick: function (data) {
                var cancel = false;
                if (settings.onClick) {
                    cancel = settings.onClick();
                }
                if (!cancel) {
                    item.showItemInfo();
                }
            },
            onButtonDown: settings.onButtonDown,
            onButtonUp: settings.onButtonUp
        });
        // remove item from previous parent
        if (item.parent) {
            item.removeSelf();
        }
        // add to slot

        slot.attach(item);

        // attach tierstars if necessary
        slot.attach(new TierStars({
            position: new Vector2(-16, -10),
            tier: ItemManager.getTier(item.getInfo()),
            align: 'left'
        }));

        return slot;
    };
    return GridSlot;
});