/**
 * frame containing an item you can purchase with real money
 * @moduleName MoneyPurchase
 */
bento.define('gui/iap/moneypurchase', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'gui/shadowtext',
    'modules/store'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    ShadowText,
    Store
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
            item: String // which image to load
            cost: String // what this purchase will cost
            onClick: Function // action that happens when button is pressed
            big: bool // purchase size
        }*/
        var iap = settings.iap;
        var frame = new Entity({
            z: 0,
            name: 'frame',
            components: [
                new Sprite({
                    imageName: !settings.big ? 'gui2/iap/panel_item' : 'gui/iap/framebig',
                    originRelative: new Vector2(0.5, 0)
                })
            ]
        });
        var title = new Text({
            z: 0,
            position: new Vector2(-4, 15),
            text: settings.title || Utils.rewardsIAP[iap].toString(),
            font: 'font',
            fontSize: 16,
            fontColor: '#271a2d',
            align: 'center',
            textBaseline: 'bottom'
        });
        var gemSymbol = new Entity({
            name: 'gemSymbol',
            position: new Vector2(title.dimension.width / 2, 12),
            components: [
                new Sprite({
                    imageName: 'gui2/iap/symbol_gem',
                    originRelative: new Vector2(0.5, 1),
                })
            ]
        });

        // portrait of item
        var assetName = 'gui2/iap/' + settings.item.toString();
        var hasAnimatedSprite = Utils.hasAsset('spritesheets', assetName);
        var spriteName = hasAnimatedSprite ? assetName : undefined;
        var imageName = hasAnimatedSprite ? undefined : assetName;
        var item = new Entity({
            z: 0,
            name: 'itemPortrait',
            position: new Vector2(0, 16),
            components: [
                new Sprite({
                    imageName: imageName,
                    spriteSheet: spriteName,
                    originRelative: new Vector2(0.5, 0)
                })
            ]
        });
        var moneyText = new ShadowText({
            z: 0,
            position: new Vector2(0, -1),
            text: '$' + settings.cost.toString(),
            font: 'font',
            fontSize: 16,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            maxWidth: 37,
            linebreaks: false
        });
        var button = new ClickButton({
            z: 0,
            name: 'item',
            sfx: 'sfx_button',
            imageName: hasAnimatedSprite ? 'gui2/misc/buttonmini_large_secondary_excited' : 'gui2/misc/buttonmini_large_secondary',
            frameCountX: hasAnimatedSprite ? 7 : 1,
            frameCountY: hasAnimatedSprite ? 6 : 3,
            animations: {
                'up': {
                    speed: 0.3,
                    frames: hasAnimatedSprite ? [
                        0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10,
                        11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
                        21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
                        31, 32, 33, 34, 35
                    ] : [0] 
                },
                'down': {
                    frames: hasAnimatedSprite ? [37] : [1]
                },
                inactive: {
                    frames: hasAnimatedSprite ? [38] : [2]
                }
            },
            padding: hasAnimatedSprite ? 1 : 0,
            position: new Vector2(0, 76),
            float: false,
            updateWhenPaused: 0,
            onClick: function () {
                settings.onClick(iap);
            },
            onButtonDown: function () {
                moneyText.position.y = 0;
                EventSystem.fire('preventScroll');
            },
            onButtonUp: function () {
                moneyText.position.y = -1;
            },
            components: [
                moneyText
            ]
        });
        var behavior = {
            name: 'behavior',
            start: function () {
                var products = Store.getProducts();
                var getPrice = function (id) {
                    var i;
                    if (products) {
                        // search
                        for (i = 0; i < products.length; ++i) {
                            if (products[i].productId === id) {
                                return products[i].localizedPrice;
                            }
                        }
                    }
                    // if products can't be loaded, load price from save state
                    else if (Bento.saveState.load(id + 'Price', null)) {
                        return Bento.saveState.load(id + 'Price', null);
                    }
                    return null;
                };
                // try to get the real price
                var price = getPrice(iap);
                if (price) {
                    // replace the space between $ and the number
                    price = price.replace(' ', '');
                    moneyText.setText(price);
                }
            }
        };

        var entity = new Entity({
            z: 0,
            name: 'moneyPurchase-' + settings.title,
            position: settings.position,
            updateWhenPaused: 0,
            components: [
                behavior,
                frame,
                title,
                gemSymbol,
                item,
                button

            ]
        });
        return entity;
    };
});