/**
 * Window with an IAP offer pack
 * @moduleName OfferPack
 */
bento.define('gui/iap/offerpack', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'components/anchor',
    'modules/localization',
    'gui/shadowtext',
    'managers/save',
    'managers/time',
    'gui/iap/offerpackitems',
    'gui/iapoverlay',
    'managers/currency',
    'gui/levelupnotification',
    'managers/inventory',
    'modules/analytics',
    'globals',
    'managers/receiveitem',
    'modules/store',
    'gui/iconbutton',
    'bento/components/nineslice',
    'gui/scrollinglist',
    'managers/item',
    'gui/centeredlist',
    'components/masksprite',
    'gui/iteminfo'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Anchor,
    Localization,
    ShadowText,
    SaveManager,
    TimeManager,
    OfferPackItems,
    IapOverlay,
    CurrencyManager,
    LevelUpNotification,
    InventoryManager,
    Analytics,
    Globals,
    ReceiveItem,
    Store,
    IconButton,
    NineSlice,
    ScrollingList,
    ItemManager,
    CenteredList,
    MaskSprite,
    ItemInfo
) {
    'use strict';
    // the actual contents of an offerpack
    var offerPacksIAP = {
        starter: {
            'currency/coin': 2000,
            'currency/gem': 50,
            'consumables/combo100': 1,
            'consumables/steak': 1,
            'consumables/invincibility': 1,
            'weapons/bat': 1,
            'weapons/selfiestick': 1,
            'weapons/hammer': 1
        },
        weapon: {
            'currency/coin': 8000,
            'currency/gem': 800,
            'consumables/combo100': 1,
            'consumables/steak': 1,
            'consumables/invincibility': 1,
            'weapons/sword': 1,
            'weapons/selfiestick': 1,
            'weapons/hammer': 1
        },
        powerup: {
            'currency/coin': 4000,
            'currency/gem': 100,
            'currency/key': 9
        },
        gem: {
            'currency/gem': 50
        }
    };
    /*
     * Generate all weapon strings into the weapon offer
     * Count categories to put in the details
     */
    var details = {
        /*
        example: [{
            icon: 'gui/buttons/equipment', // backpack
            amount: 12
        }]
        */
        'starter': [],
        'weapon': [],
        'powerup': [],
        'gem': []
    };
    var hasInitWeapons = false;
    var addAllWeapons = function () {
        // should only be called once
        var assets = Bento.assets.getAssets();
        var jsonAssets = assets.json;
        var count = 0;
        var weapons = [];

        /*Utils.forEach(jsonAssets, function (item, key, l, breakLoop) {
            var keys = key.split('/');
            var category = keys[0];
            var array = category;
            if (category === 'weapons') {
                weapons.push({
                    key: key,
                    tier: item.inventory.tier || 1
                });
            }
        });*/
        hasInitWeapons = true;

        // DEPRECATED
        /*// sort by tier
        weapons.sort(function (a, b) {
            return a.tier - b.tier;
        });
        // add to pack
        Utils.forEach(weapons, function (weapon) {
            offerPacksIAP.weapon[weapon.key] = 2;
            count += 2;
        });*/

        // update details
        setDetails();
    };
    var setDetails = function () {
        // will be called by addAllWeapons
        var icons = {
            consumables: 'gui2/offers/icons/powerup',
            weapons: 'gui2/offers/icons/weapon',
            gem: 'gui2/offers/icons/gem',
            coin: 'gui2/offers/icons/coin',
            key: 'gui2/offers/icons/key'
        };
        var amounts = {};
        Utils.forEach(details, function (detailArray, offer) {
            // loop through offers
            var content = offerPacksIAP[offer];

            // reset amounts
            amounts = {};

            // inspect the contents, count how many of each category
            Utils.forEach(content, function (amount, key) {
                var keys = key.split('/');
                var category = keys[0];
                var item = keys[1];
                var iconName;
                var text;

                if (category === 'currency') {
                    // gem or coin
                    iconName = icons[item];
                    text = item;
                } else {
                    iconName = icons[category];
                    text = category;
                }

                // init amount values
                amounts[iconName] = amounts[iconName] || {
                    amount: 0,
                    text: ''
                };
                amounts[iconName].amount += amount;
                amounts[iconName].text = text;
            });

            // add amounts to details
            Utils.forEach(amounts, function (value, iconName) {
                detailArray.push({
                    icon: iconName,
                    amount: value.amount,
                    text: value.text
                });
            });
        });
    };

    /**
     * Main Module
     */
    var OfferPack = function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var offer = settings.offer;
        var onClaim = settings.onClaim;
        var offerDuration = settings.duration; // in hours

        // top
        var scaleBehavior = {
            name: 'scaleBehavior',
            start: function () {
                // scale to window size
                packImage.scale.x = 160 / packImage.dimension.width;
                packImage.scale.y = packImage.scale.x;
            }
        };
        var packImage = new Entity({
            name: 'packImage',
            components: [
                new Sprite({
                    imageName: 'gui2/offers/images/' + offer,
                    originRelative: new Vector2(0.5, 0.5)
                }),
                scaleBehavior
            ]
        });
        var top = new Entity({
            name: 'top',
            components: [
                packImage,
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0, 0),
                        offset: new Vector2(0, -80),
                    },
                    landscape: {
                        screenPosition: new Vector2(0, 0),
                        offset: new Vector2(80, -16),
                    },
                    onComplete: function () {}
                })
            ]
        });
        // bottom
        var scrollingList = ScrollingList({
            area: new Rectangle(-80, 0, 160, 64), // area that user can grab
            direction: 'x',
            // spacing: 2,
            spacing: 152,
            snap: true,
            snapMinSlideSpeed: 1,
            snapSpeed: 0.3,
            damping: 0.9,
            onSnap: function (item, idx) {
                dotList.iterate(function (dot, i) {
                    if (i === idx) {
                        dot.getComponent('sprite', function (sprite) {
                            sprite.setAnimation('down');
                        });
                    } else {
                        dot.getComponent('sprite', function (sprite) {
                            sprite.setAnimation('up');
                        });
                    }
                });
            },
            components: [
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0, 0),
                        offset: new Vector2(0, 24),
                    },
                    landscape: {
                        screenPosition: new Vector2(0, 0),
                        offset: new Vector2(0, 40),
                    }
                })
            ]
        });
        var dotList = new CenteredList({
            spacing: 3,
            components: [
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0, 0),
                        offset: new Vector2(0, 48),
                    },
                    landscape: {
                        screenPosition: new Vector2(0, 0),
                        offset: new Vector2(0, 64),
                    }
                })
            ]
        });
        var detailsBox = new Entity({
            name: 'detailsBox',
            components: [
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0, 0),
                        offset: new Vector2(0, -24),
                    },
                    landscape: {
                        screenPosition: new Vector2(0, 0),
                        offset: new Vector2(0, 0),
                    },
                    onComplete: function () {}
                })
            ]
        });
        var gemBox = new Entity({
            name: 'gemBox',
            components: [
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0, 0),
                        offset: new Vector2(0, -16),
                    },
                    landscape: {
                        screenPosition: new Vector2(0, 0),
                        offset: new Vector2(0, -16),
                    },
                    onComplete: function () {}
                })
            ]
        });
        var nineSlice = new NineSlice({
            imageName: 'gui2/misc/splash_background_9slice',
            originRelative: new Vector2(0.5, 0.5),
            width: 180,
            height: 180
        });
        var bottom = new Entity({
            name: 'bottomBox',
            components: [
                nineSlice,
                detailsBox,
                gemBox,
                dotList,
                scrollingList,
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0, 0),
                        offset: new Vector2(0, 32),
                    },
                    landscape: {
                        screenPosition: new Vector2(0, 0),
                        offset: new Vector2(-80, 0),
                    },
                    onComplete: function () {}
                })
            ]
        });
        var offerText = new Text({
            position: new Vector2(0, 8),
            text: 'Ends in',
            font: 'font',
            fontSize: 10,
            fontColor: '#ffdf00',
            align: 'center',
            textBaseline: 'middle',
            maxWidth: 80,
            maxHeight: 10
        });
        var priceText = new ShadowText({
            position: new Vector2(0, -6),
            text: '$0.00',
            font: 'font',
            fontSize: 20,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            maxWidth: 64,
            maxHeight: 20,
            linebreaks: false
        });
        // purchase callback
        var onPurchaseComplete = function (completed, iapId, purchaseInfo) {
            if (!completed) {
                // purchase error
                return;
            }
            var offerId = Utils.IapToOffer[iapId];
            // show the items you bought
            var contents = new OfferPackItems({
                z: settings.z + 1,
                title: Localization.getText('offerpacks')[offerId],
                items: offerPacksIAP[offerId],
                claim: true,
                onClaim: onClaim
            });
            Bento.objects.attach(contents);

            var offersBought = SaveManager.load('offersBought');
            offersBought[offerId] = offersBought[offerId] || 0;
            offersBought[offerId] += 1;
            SaveManager.save('offersBought', offersBought);

            // add the items to your inventory
            Utils.forEach(offerPacksIAP[offerId], function (amount, key) {
                ReceiveItem({
                    key: key,
                    amount: amount
                });
            });
            // special case: daily gems
            if (offerId === 'gem') {
                SaveManager.add('dailyGems', 30);
                // once bought, the daily gem amount is saved
                Bento.saveState.save('dailyGemsOfferAmount', Globals.dailyGemsOfferAmount);
            }
            if (purchaseInfo) {
                // consumables must be "consumed" for the google play store
                Store.consume(iapId, function () {
                    console.log(iapId + ' consumed.');
                });
                // resolve purchase
                Store.resolvePurchase(purchaseInfo);
            }

            removeOffer();

            // fire event to close offerpack
            EventSystem.fire('offerpackBought', {
                offer: offer
            });

            // analytics
            Analytics.event('OfferPack', offerId, 0);
        };
        // star
        var starText = new ShadowText({
            position: new Vector2(0, -3),
            text: Localization.getText('discountOff').replace('{NUMBER}', 50),
            font: 'font',
            fontSize: 16,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            ySpacing: -2,
            maxWidth: 25,
            maxHeight: 20,
            linebreaks: true,
            rotation: -Math.PI / 16
        });
        var star = new Entity({
            name: 'star',
            position: new Vector2(51, -20),
            components: [
                new Sprite({
                    imageName: 'gui/offerpacks/star',
                    originRelative: new Vector2(0.5, 0.5)
                }),
                starText
            ]
        });
        // buy button
        var iconInfo = new Entity({
            name: 'buttonInfo',
            components: [
                priceText,
                offerText,
                star
            ]
        });
        var buttonBuy = new IconButton({
            name: 'buttonBuy',
            sfx: 'sfx_button',
            imageName: 'gui2/misc/button_large_primary_excited',
            icon: iconInfo,
            iconAlpha: 0.5,
            frameCountX: 6,
            frameCountY: 7,
            padding: 1,
            animations: {
                up: {
                    speed: 0.34,
                    frames: [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35]
                },
                down: {
                    frames: [37]
                },
                inactive: {
                    frames: [38]
                }
            },
            updateWhenPaused: 0,
            onClick: function () {
                // actually buy the pack
                IapOverlay.Purchase({
                    z: Globals.Layers.SYSTEM,
                    id: Utils.IapToOffer[offer],
                    onPurchaseComplete: onPurchaseComplete
                });
            },
            onButtonDown: function () {
                EventSystem.fire('preventScroll');
            },
            components: [
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0, 0.5),
                        offset: new Vector2(0, -48),
                    },
                    landscape: {
                        screenPosition: new Vector2(0, 0),
                        offset: new Vector2(80, 64),
                    },
                    onComplete: function () {}
                })
            ]
        });
        var buttonClose = new ClickButton({
            name: 'btnClose',
            sfx: 'sfx_button',
            imageName: 'gui2/offers/close',
            frameCountX: 1,
            frameCountY: 2,
            onClick: function () {
                if (settings.onClose) {
                    settings.onClose();
                }
            },
            components: [
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.5, -0.5),
                        offset: new Vector2(-12, 32),
                    },
                    landscape: {
                        screenPosition: new Vector2(0.5, -0.5),
                        offset: new Vector2(-16, 16),
                    },
                    onComplete: function () {}
                })
            ]
        });
        // text
        var title = new ShadowText({
            text: Localization.getText('offerpacks')[offer].toUpperCase(),
            font: 'font',
            fontSize: 18,
            fontColor: '#ffe10f',
            shadowColor: '#4f3875',
            align: 'center',
            textBaseline: 'middle',
            maxWidth: 140,
            maxHeight: 18,
            components: [
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0, -0.5),
                        offset: new Vector2(0, 40),
                    },
                    landscape: {
                        screenPosition: new Vector2(0, -0.5),
                        offset: new Vector2(-80, 24),
                    },
                    onComplete: function () {}
                })
            ]
        });
        var subTitle = new ShadowText({
            text: Localization.getText('limitedOfferDiscount'),
            font: 'font',
            fontSize: 12,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            maxWidth: 160,
            maxHeight: 12,
            linebreaks: false,
            components: [
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0, -0.5),
                        offset: new Vector2(0, 40 + 12),
                    },
                    landscape: {
                        screenPosition: new Vector2(0, 0),
                        offset: new Vector2(80, -40 - 16),
                    },
                    onComplete: function () {}
                })
            ]
        });
        var bottomTitle = new Text({
            text: Localization.getText('iapRemovesAd'),
            font: 'font',
            fontSize: 12,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            maxWidth: 154,
            maxHeight: 12,
            linebreaks: false,
            lineWidth: 1,
            strokeStyle: '#45316a',
            components: [
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0, 0.5),
                        offset: new Vector2(0, -12),
                    },
                    landscape: {
                        screenPosition: new Vector2(0, 0),
                        offset: new Vector2(-80, -40 - 16 + 124),
                    },
                    onComplete: function () {}
                })
            ]
        });
        var description = new Text({
            text: Localization.getText('offerpackdescriptions')[offer],
            font: 'font',
            fontSize: 12,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            maxWidth: 154,
            maxHeight: 12,
            linebreaks: false,
            drawDebug: false,
            components: [
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0, 0),
                        offset: new Vector2(0, -24),
                    },
                    landscape: {
                        screenPosition: new Vector2(0, -0.5),
                        offset: new Vector2(-80, 40),
                    },
                    onComplete: function () {}
                })
            ]
        });
        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {
                // get the price
                var products = Store.getProducts();
                var getPrice = function (id) {
                    var i;
                    if (products) {
                        // search
                        for (i = 0; i < products.length; ++i) {
                            if (products[i].productId === id) {
                                return products[i].localizedPrice;
                            }
                        }
                    }
                    // if products can't be loaded, load price from save state
                    else if (Bento.saveState.load(id + 'Price', null)) {
                        return Bento.saveState.load(id + 'Price', null);
                    }
                    return null;
                };
                // try to get the real price
                var price = getPrice(Utils.IapToOffer[offer]);
                if (price) {
                    // replace the space between $ and the number
                    price = price.replace(' ', '');
                    priceText.setText(price);
                }

                // attach icons and numbers to details box
                var detail = details[offer];
                if (!detail) {
                    Utils.log('WARNING: unknown offer');
                    return;
                }
                Utils.forEach(detail, function (detailObj, i, l, breakLoop) {
                    var icon = detailObj.icon;
                    var amount = detailObj.amount;
                    var textId = detailObj.text;
                    var iconEntity = new Entity({
                        name: 'icon',
                        position: new Vector2(-16, 0),
                        components: [
                            new Sprite({
                                imageName: icon,
                                originRelative: new Vector2(0.5, 0.5)
                            })
                        ]
                    });
                    var amountText = new ShadowText({
                        position: new Vector2(72 / 2, 0),
                        text: Localization.getText('details-' + textId).replace('{NUMBER}', amount),
                        font: 'font',
                        fontSize: 16,
                        fontColor: '#ffe10f',
                        align: 'center',
                        textBaseline: 'middle',
                        maxWidth: 72,
                        maxHeight: 16,
                        lineBreaks: false
                    });
                    var detailEntity = new Entity({
                        name: 'detail',
                        scale: new Vector2(0.75, 0.75),
                        position: new Vector2(-56 + 72 * Math.floor(i / 2), -8 + (i % 2) * 16),
                        components: [
                            iconEntity,
                            amountText
                        ]
                    });

                    if (textId === 'gem' || textId === 'coin') {
                        iconEntity.position.x += 16;
                    }

                    detailsBox.attach(detailEntity);
                });

                // create scrolling list
                spawnItems();

                // look up the current offer
                var offers = SaveManager.load('offers');
                var offerData = offers[offer];
                var now = TimeManager.getTime();

                if (!offerData || offerData.time < now) {
                    // offer doesn't exist yet or is expired
                    createOffer(offers);
                    offerData = offers[offer];
                }

                var timeLeft;
                var minutes;
                var hours;

                // set time text
                timeLeft = Math.max(0, offerData.time - now);
                minutes = Math.floor(timeLeft / 1000 / 60) % 60;
                hours = Math.floor(timeLeft / 1000 / 60 / 60);
                offerText.setText(Localization.getText('offerTime').replace('{HOURS}', hours).replace('{MINUTES}', minutes));

                if (settings.canClose) {
                    entity.attach(buttonClose);
                }

                // daily gem pack has different info
                if (offer === 'gem') {
                    detailsBox.removeSelf();
                    scrollingList.removeSelf();
                    makeGemBox();

                    // replace amount text
                    description.setText(
                        description.getText().replace('50', Globals.dailyGemsOfferAmount)
                    );
                }
            }
        };
        var makeGemBox = function () {
            var gemLeft = new Entity({
                name: 'gemLeft',
                position: new Vector2(-40, 0),
                components: [
                    new Sprite({
                        imageName: 'gui2/offers/icons/gem-large',
                        originRelative: new Vector2(0.5, 0.5)
                    }),
                    new Text({
                        position: new Vector2(8, 4),
                        text: offerPacksIAP.gem['currency/gem'],
                        font: 'font',
                        fontSize: 16,
                        fontColor: '#ffffff',
                        align: 'center',
                        textBaseline: 'middle',
                        lineWidth: 1, // set to add an outline
                        strokeStyle: '#45316a',
                    }),
                    new Text({
                        z: 0,
                        position: new Vector2(0, 24),
                        text: Localization.getText('gemInstant'),
                        font: 'font',
                        fontSize: 16,
                        fontColor: '#ffdc12',
                        align: 'center',
                        textBaseline: 'middle',
                        components: []
                    })
                ]
            });
            var plus = new Text({
                z: 0,
                position: new Vector2(0, 0),
                text: '+',
                font: 'font',
                fontSize: 32,
                fontColor: '#ffdc12',
                align: 'center',
                textBaseline: 'middle'
            });
            var chest = new Entity({
                name: 'chest',
                position: new Vector2(24, 8),
                scale: new Vector2(-1, 1),
                components: [
                    new Sprite({
                        imageName: 'gui2/dailychests/chest-0',
                        originRelative: new Vector2(0.5, 0.5),
                        frameCountX: 2,
                        animations: {
                            default: {
                                frames: [1]
                            }
                        }
                    })
                ]
            });
            var gemRight = new Entity({
                name: 'gemRight',
                position: new Vector2(40, -8),
                components: [
                    chest,
                    new Sprite({
                        imageName: 'gui2/offers/icons/gem-large',
                        originRelative: new Vector2(0.5, 0.5)
                    }),
                    new Text({
                        position: new Vector2(8, 4),
                        text: Globals.dailyGemsOfferAmount.toString(),
                        font: 'font',
                        fontSize: 16,
                        fontColor: '#ffffff',
                        align: 'center',
                        textBaseline: 'middle',
                        lineWidth: 1, // set to add an outline
                        strokeStyle: '#45316a',
                    }),
                    new Text({
                        z: 0,
                        position: new Vector2(0, 40),
                        text: Localization.getText('gemEveryDay'),
                        font: 'font',
                        fontSize: 16,
                        fontColor: '#ffdc12',
                        align: 'center',
                        textBaseline: 'middle',
                        maxWidth: 48,
                        maxHeight: 32,
                        components: []
                    })
                ]
            });
            gemBox.attach(gemLeft);
            gemBox.attach(gemRight);
            gemBox.attach(plus);
        };
        /**
         * Spawns items in scrolling list
         */
        var spawnItems = function () {
            var offerObject = offerPacksIAP[offer];
            var itemCount = Utils.getKeyLength(offerObject);
            var pages = [];
            var currentPage;
            // 4 items per page
            var makePage = function () {
                var index = scrollingList.getItems().length;
                var page = new CenteredList({
                    name: 'page',
                    spacing: 2
                }).extend({
                    isFull: function () {
                        return page.length() === 4;
                    },
                    getIndex: function () {
                        return index;
                    }
                });
                var dotSprite = new Sprite({
                    frameCountX: 2,
                    originRelative: new Vector2(0.5, 0.5),
                    imageName: 'gui2/offers/dots',
                    animations: {
                        'up': {
                            frames: [1]
                        },
                        'down': {
                            frames: [0]
                        }
                    }
                });
                dotList.addItem(new Entity({
                    name: 'dot',
                    components: [
                        dotSprite
                    ]
                }));
                if (index === 0) {
                    dotSprite.setAnimation('down');
                } else {
                    dotSprite.setAnimation('up');
                }

                pages.push(page);
                currentPage = page;
                scrollingList.addItem(page);
                return page;
            };
            Utils.forEach(offerObject, function (amount, key) {
                var sprite = ItemManager.getSprite(key);
                var frameSprite = new Sprite({
                    imageName: 'gui2/loadout/item_box',
                    originRelative: new Vector2(0.5, 0.5),
                    frameCountX: 2
                });
                var amountText = new ShadowText({
                    position: new Vector2(14, 6),
                    text: amount,
                    font: 'font',
                    fontSize: 12,
                    fontColor: '#ffffff',
                    align: 'right',
                    textBaseline: 'middle',
                    lineWidth: 1,
                    strokeStyle: '#4f3875',
                    maxWidth: 20,
                    maxHeight: 12,
                    linebreaks: false
                });
                var itemEntity = new Entity({
                    name: 'item',
                    boundingBox: new Rectangle(-18, -15, 36, 30),
                    components: [
                        frameSprite
                    ]
                });
                var clickable = new Clickable({
                    onHoldEnd: function (data) {
                        // not while scrolling
                        if (scrollingList.didMove(1)) {
                            return;
                        }
                        // item must be in active page
                        if (scrollingList.getSelection() !== page.getIndex()) {
                            return;
                        }

                        var itemInfo = new ItemInfo({
                            item: key
                        });
                        Bento.objects.attach(itemInfo);
                        Bento.audio.stopSound('sfx_button');
                        Bento.audio.playSound('sfx_button');
                    }
                });
                var maskSprite = new MaskSprite({
                    sprite: frameSprite,
                    boundingBox: new Rectangle(-80, -80, 160, 160)
                });
                var maskSpriteItem = new MaskSprite({
                    sprite: sprite,
                    boundingBox: new Rectangle(-80, -80, 160, 160)
                });
                var maskSpriteText = new MaskSprite({
                    sprite: amountText.getComponent('sharpnessScaler').getComponent('sprite'),
                    boundingBox: new Rectangle(-80, -80, 160 * 4, 160)
                });
                var maskBehavior = {
                    name: 'setMask',
                    update: function () {
                        var scroll = scrollingList.getScroll();
                        var dist;
                        // sets the maskSprite's boundingbox
                        maskSprite.boundingBox.x = -80 - itemEntity.position.x - page.position.x - scroll;
                        maskSpriteItem.boundingBox.x = -80 - itemEntity.position.x - page.position.x - scroll;
                        // * 4 because of text's sharpness
                        maskSpriteText.boundingBox.x = (-80 - amountText.position.x - itemEntity.position.x - page.position.x - scroll) * 4;
                    }
                };
                var page;

                sprite.setOriginRelative(new Vector2(0.5, 0.5));
                itemEntity.attach(sprite);
                itemEntity.attach(amountText);
                itemEntity.attach(clickable);
                // masks
                itemEntity.attach(maskSprite);
                itemEntity.attach(maskSpriteItem);
                itemEntity.attach(maskSpriteText);
                itemEntity.attach(maskBehavior);
                itemEntity.dimension.width = 36;

                // attach to page
                if (!currentPage || currentPage.isFull()) {
                    page = makePage();
                } else {
                    page = currentPage;
                }
                currentPage.addItem(itemEntity);
                // scrollingList.addItem(itemEntity);
            });

            // only 1 dot
            if (dotList.length() === 1) {
                dotList.removeSelf();
            }
        };
        /**
         * Create new offer data from current offer into the save file
         */
        var createOffer = function (offers) {
            var offerData = {
                name: offer,
                time: TimeManager.getTime() + 1000 * 60 * 60 * offerDuration // scheduled time when offer is over
            };

            // save into offers
            offers[offer] = offerData;
            SaveManager.save('offers', offers);

            return offerData;
        };
        /**
         * Remove current offer data after buying
         */
        var removeOffer = function () {
            var offers = SaveManager.load('offers');
            var offerData = offers[offer];
            var now = TimeManager.getTime();

            // expire the offer immediately by setting the time
            offerData.time = now;
            SaveManager.save('offers', offers);
        };
        var entity = new Entity({
            z: settings.z,
            name: 'offerPack',
            components: [
                behavior,
                bottom,
                top,
                title,
                subTitle,
                bottomTitle,
                description,
                buttonBuy
            ]
        }).extend({
            disableButtons: function () {
                buttonClose.setActive(false);
                buttonBuy.setActive(false);
            }
        });

        // initialize weapon pack if needed
        if (!hasInitWeapons) {
            addAllWeapons();
        }

        return entity;
    };

    // Static function
    OfferPack.hasAnyOffers = function () {
        var hasAny = false;
        var now = TimeManager.getTime();
        var offers = SaveManager.load('offers');

        // check if any offer is valid
        Utils.forEach(offerPacksIAP, function (content, offer, l, breakLoop) {
            var offerData = offers[offer];
            if (offerData && offerData.time >= now) {
                // offer stands
                hasAny = true;
                breakLoop();
            }
        });
        return hasAny;
    };

    OfferPack.getOfferPacksIap = function () {
        // initialize weapon pack if needed
        if (!hasInitWeapons) {
            addAllWeapons();
        }

        return offerPacksIAP;
    };

    return OfferPack;
});