/**
 * Overlay to show the contents of an offerpack
 * @moduleName OfferPackItems
 */
bento.define('gui/iap/offerpackitems', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'gui/blurbackground',
    'components/modal',
    'gui/scrollinglist',
    'components/anchor',
    'gui/shadowtext',
    'gui/centeredlist',
    'gui/iap/gridslot',
    'managers/item',
    'gui/iteminfo',
    'gui/iap/claimbutton',
    'gui/worldselect/topbarsimple',
    'gui/worldselect/topbar',
    'components/backbutton'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    BlurBackground,
    Modal,
    ScrollingList,
    Anchor,
    ShadowText,
    CenteredList,
    GridSlot,
    ItemManager,
    ItemInfo,
    ClaimButton,
    TopBarSimple,
    Topbar,
    BackButton
) {
    'use strict';
    return function (settings) {
        var viewport = Bento.getViewport();
        var items = settings.items;
        var title = settings.title;
        var claim = settings.claim;
        // components
        var blurBackground = new BlurBackground({
            darken: 0.5
        });
        var makeScrollingList = function () {
            return new ScrollingList({}).attach(new Anchor({
                landscape: {
                    screenPosition: new Vector2(0.5, 0.32)
                },
                portrait: {
                    screenPosition: new Vector2(0.5, 0.2)
                }
            }));
        };
        var scrollingList;
        var buttonClose = new ClickButton({
            name: 'btnClose',
            sfx: 'sfx_button',
            imageName: 'gui2/bottom_tabs/bottom_tabs__back',
            frameCountX: 2,
            frameCountY: 1,
            onClick: function () {
                buttonClose.setActive(false);
                new Tween({
                    from: 1,
                    to: 0,
                    in: 15,
                    ease: 'linear',
                    onUpdate: function (v, t) {
                        entity.alpha = v;
                    },
                    onComplete: function () {
                        entity.removeSelf();
                    }
                });
            },
            components: [
                new BackButton({
                    removeSelf: true,
                    onPressed: function (data) {
                        data.entity.doCallback();
                    }
                }),
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.5, 1),
                        offset: new Vector2(-48 + (Anchor.isWideScreen() ? 4 : 0), -14 + (Anchor.isWideScreen() ? Anchor.getOffset().y : 0))
                    },
                    landscape: {
                        screenPosition: new Vector2(0.25, 1),
                        offset: new Vector2(-44, -12)
                    }
                })
            ]
        });
        var titleText = new ShadowText({
            position: new Vector2(0, 0),
            text: title,
            font: 'font',
            fontSize: 24,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            maxWidth: 120,
            linebreaks: false
        }).attach(new Anchor({
            portrait: {
                screenPosition: new Vector2(0.5, 0),
                offset: new Vector2(0, 48)
            },
            landscape: {
                screenPosition: new Vector2(0.5, 0),
                offset: new Vector2(0, 20)
            }
        }));
        var gridSlots = [];
        var claimButton = new ClaimButton({
            onClick: function () {
                claimButton.setActive(false);
                var delay = 0;
                Utils.forEach(gridSlots, function (gridSlot, i, l, breakLoop) {
                    new Tween({
                        from: gridSlot.position,
                        to: new Vector2(0, viewport.height - gridSlot.position.y),
                        in: 30,
                        delay: delay,
                        ease: 'easeInBack',
                        onUpdate: function (v, t) {
                            gridSlot.position = v;
                        }
                    });
                    new Tween({
                        from: 1,
                        to: 0.2,
                        in: 30,
                        delay: delay + 15,
                        ease: 'easeInBack',
                        onUpdate: function (v, t) {
                            gridSlot.scale.x = v;
                            gridSlot.scale.y = v;
                        }
                    });
                    delay += 2;
                });
                new Tween({
                    from: 1,
                    to: 0,
                    in: 10,
                    delay: delay + 40,
                    onUpdate: function (v, t) {
                        entity.alpha = v;
                    },
                    onComplete: function () {
                        entity.removeSelf();
                        if (settings.onClaim) {
                            settings.onClaim();
                        }
                    }
                });
            }
        }).attach(new Anchor({
            screenPosition: new Vector2(0.5, 1),
            offset: new Vector2(0, -20)
        }));
        var spawnItems = function () {
            var rowItemsX = Anchor.isPortrait() ? 3 : 5;
            var currentRow;
            var i = 0;
            var totalHeight = 0;

            // clear old
            if (scrollingList) {
                scrollingList.removeSelf();
            }
            scrollingList = makeScrollingList();
            entity.attach(scrollingList);
            gridSlots = [];

            Utils.forEach(items, function (amount, item, l, breakLoop) {
                if (i % rowItemsX === 0) {
                    // create new row
                    currentRow = new CenteredList({
                        name: 'row',
                        spacing: 4
                    });
                    currentRow.dimension.height = 48;
                    totalHeight += 48;
                }
                var itemSprite = ItemManager.getSprite(item);
                var itemEntity = new Entity({
                    name: 'item',
                    components: [
                        itemSprite
                    ]
                }).extend({
                    getInfo: function () {
                        return item;
                    },
                    showItemInfo: function () {
                        var itemInfoPopup = new ItemInfo({
                            z: settings.z + 2,
                            item: item
                        });
                        Bento.objects.attach(itemInfoPopup);
                    }
                });
                // add a number if more than 1 loot is given
                var amountText = new ShadowText({
                    position: new Vector2(16, 6),
                    text: amount.toString(),
                    font: 'font',
                    fontSize: 16,
                    fontColor: '#ffffff',
                    align: 'right',
                    textBaseline: 'middle',
                    maxWidth: 16,
                    maxHeight: 16,
                    lineBreaks: false
                });
                var gridSlot = new GridSlot({
                    item: itemEntity,
                    onButtonDown: function () {
                        scrollingList.cancel();
                    }
                });

                if (amount > 1) {
                    itemEntity.attach(amountText);
                }

                itemSprite.setOriginRelative(new Vector2(0.5, 0.5));
                currentRow.addItem(gridSlot);
                gridSlots.push(gridSlot);
                if (!currentRow.parent) {
                    // attach to scrolling list
                    scrollingList.addItem(currentRow);
                }

                ++i;
            });

            // put close and title on top
            entity.getComponent('topBar', function (topBar) {
                entity.moveComponentTo(topBar, entity.components.length);
            });
            entity.moveComponentTo(claimButton, entity.components.length);
            entity.moveComponentTo(buttonClose, entity.components.length);
            entity.moveComponentTo(titleText, entity.components.length);

            if (settings.claim) {
                buttonClose.setActive(false);
                buttonClose.visible = false;
            } else {
                claimButton.setActive(false);
                claimButton.visible = false;
            }

            // scroll to bottom
            var maxItems = 12;
            if (!Anchor.isPortrait()) {
                maxItems = 10;
            }
            if (Utils.getKeyLength(items) <= maxItems) {
                // appear from the bottom
                scrollingList.forceTo(viewport.height);
            } else {
                new Tween({
                    from: viewport.height,
                    to: -totalHeight + (viewport.height - 80),
                    in: 60,
                    ease: 'easeOutExpo',
                    onUpdate: function (v, t) {
                        scrollingList.forceTo(v);
                    }
                });
            }


        };
        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {
                new Tween({
                    from: 0,
                    to: 1,
                    in: 5,
                    ease: 'linear',
                    onUpdate: function (v, t) {
                        entity.alpha = v;
                    }
                });

                if (Utils.getKeyLength(items) <= 12) {
                    // don't scroll with less than 12 items
                    if (scrollingList) {
                        scrollingList.setActive(false);
                    }
                }

                if (claim) {
                    Bento.audio.stopSound('sfx_lootfinished');
                    Bento.audio.playSound('sfx_lootfinished');
                }
            },
            destroy: function (data) {},
            update: function (data) {},
            draw: function (data) {}
        };
        var entity = new Entity({
            z: settings.z,
            name: 'offerPackItems',
            float: true,
            components: [
                new Modal({}),
                behavior,
                blurBackground,
                // titleText,
                // claim ? new Topbar({
                //     preventClick: true
                // }) : new TopBarSimple({
                //     preventClick: true
                // }),
                buttonClose,
                claimButton,
                new Anchor({
                    // refresh items on rotate
                    onComplete: function () {
                        spawnItems();
                    }
                })
            ]
        });
        return entity;
    };
});