/**
 * Spawns an offerpack as a popup 
 * @moduleName OfferPackPopup
 */
bento.define('gui/iap/offerpackpopup', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'gui/iap/offerpack',
    'components/anchor',
    'gui/blurbackground',
    'components/viewanalytics',
    'modules/analytics',
    'gui/worldselect/topbar',
    'components/modal',
    'components/orientationlock',
    'components/eventlistener'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    OfferPack,
    Anchor,
    BlurBackground,
    ViewAnalytics,
    Analytics,
    Topbar,
    Modal,
    OrientationLock,
    EventListener
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var blurBackground = new BlurBackground({
            darken: 0.5,
            blurriness: 8
        });
        var offer = settings.offer;
        var offerPack = new OfferPack(Utils.extend(settings, {
            z: settings.z,
            canClose: true,
            onClose: function () {
                // close
                offerPack.disableButtons();
                moveOut();
                // ignored the offer
                Analytics.event('Offer', 'offer-' + offer, 0);
            },
            onClaim: function () {
                // user bought the IAP, close the offer
                offerPack.disableButtons();
                moveOut();

                Analytics.event('Offer', 'offer-' + offer, 1);
            }
        }));
        var moveOut = function () {
            isTweening = true;
            // fade in background
            new Tween({
                from: 1,
                to: 0,
                in: 15,
                ease: 'linear',
                onUpdate: function (v, t) {
                    blurBackground.alpha = v;
                }
            });

            // tween in box
            new Tween({
                from: offerPack.position.y,
                to: offerPack.position.y + viewport.height,
                in: 30,
                ease: 'easeInBack',
                onUpdate: function (v, t) {
                    offerPack.position.y = v;
                },
                onComplete: function () {
                    entity.removeSelf();
                }
            }).attach(new OrientationLock({}));
        };
        var isTweening;
        var behavior = {
            name: 'behavior',
            start: function () {
                isTweening = true;
                // fade in background
                new Tween({
                    from: 0,
                    to: 1,
                    in: 20,
                    ease: 'linear',
                    onUpdate: function (v, t) {
                        blurBackground.alpha = v;
                    }
                });

                // tween in box
                new Tween({
                    from: -viewport.height / 2,
                    to: viewport.height / 2,
                    in: 30,
                    ease: 'elastic',
                    decay: 8,
                    onUpdate: function (v, t) {
                        offerPack.position.y = v;
                    },
                    onComplete: function () {
                        isTweening = false;
                    }
                }).attach(new OrientationLock({}));
            },
            destroy: function () {
                EventSystem.fire('shownOffer');
            },
            attached: function () {
                offerPack.attach(new Anchor({
                    screenPosition: new Vector2(0.5, 0.5)
                }));
            }
        };
        var eventListener = new EventListener({
            name: 'offerpackBought',
            onEvent: function (data) {
                if (data.offer === offer) {
                    offerPack.disableButtons();
                }
            }
        });
        var entity = new Entity({
            z: settings.z,
            name: 'offerPackPopup',
            float: true,
            components: [
                new Modal({}),
                behavior,
                blurBackground,
                // new Topbar({
                //     preventClick: true
                // }),
                offerPack,
                eventListener,
                new ViewAnalytics({
                    name: 'iapOffer-' + offer,
                    ignoreKey: 'tutorialWorldSelect'
                })
            ]
        });

        return entity;
    };
});