/**
 * a label used in IAP store to mark a section (chest, gems, coins, etc.)
 * @moduleName SectionLabel
 */
bento.define('gui/iap/sectionlabel', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'gui/shadowtext'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    ShadowText
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
            text: String // Text on the tab
        }*/
        var viewport = Bento.getViewport();
        var sprite = new Sprite({
            imageName: 'gui2/misc/title_bar',
            originRelative: new Vector2(0.5, 0)
        });

        var text = new Text({
            z: 0,
            position: new Vector2(0, 18),
            text: settings.text,
            font: 'font',
            fontSize: 16,
            fontColor: '#ffdf00',
            align: 'center',
            textBaseline: 'bottom',
            maxWidth: 120,
            maxHeight: 16,
            linebreaks: false
        });
        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {
                if (settings.extraText) {
                    text.position.y -= 4;
                    var extraText = new ShadowText({
                        position: new Vector2(0, 12),
                        text: settings.extraText,
                        font: 'font',
                        fontSize: 10,
                        fontColor: '#ffffff',
                        align: 'center',
                        textBaseline: 'top',
                        maxWidth: 120,
                        linebreaks: false
                    });
                    entity.attach(extraText);
                }
            },
            destroy: function (data) {},
            update: function (data) {},
            draw: function (data) {}
        };
        var entity = new Entity({
            z: 0,
            name: 'sectionLabel',
            position: settings.position,
            updateWhenPaused: 0,
            components: [
                sprite,
                text,
                behavior
            ]
        });
        return entity;
    };
});