/**
 * Button with an icon on top
 * @moduleName IconButton
 */
bento.define('gui/iconbutton', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var entitySettings = {};
        var entity;
        var iconEntity = settings.icon;
        var iconSprite = settings.iconSprite;
        var iconAlpha = settings.iconAlpha;
        var iconName = settings.iconName || '';
        var offset = settings.offset || new Vector2(0, 0);
        var icon = iconEntity || new Entity({
            name: 'icon',
            components: [
                iconSprite || new Sprite({
                    imageName: iconName,
                    originRelative: new Vector2(0.5, 0.5)
                })
            ]
        });

        icon.position = offset.clone();

        Utils.extend(entitySettings, settings);
        // overwrite callbacks
        entitySettings.onButtonDown = function () {
            icon.position.y = offset.y + 1;
            if (settings.onButtonDown) {
                settings.onButtonDown.apply(entity);
            }
        };
        entitySettings.onButtonUp = function () {
            icon.position.y = offset.y;
            if (settings.onButtonUp) {
                settings.onButtonUp.apply(entity);
            }
        };
        // merge components array
        entitySettings.components = [
            icon
        ].concat(settings.components || []);

        // spawn entity
        entity = new ClickButton(entitySettings);

        // allow to change icon
        entity.extend({
            changeIcon: function (newIcon) {
                icon.getComponent('sprite', function (sprite) {
                    sprite.setup(newIcon);
                });
            }
        });

        if (iconAlpha) {
            // add behavior to set alpha of icon when button is disabled
            entity.attach({
                name: 'alpha',
                update: function () {
                    if (entity.isActive()) {
                        icon.alpha = 1;
                    } else {
                        icon.alpha = iconAlpha;
                    }
                }
            });
        }

        return entity;
    };
});