/**
 * Overlay popup for getting a single item
 * @moduleName ItemGet
 */
bento.define('gui/itemget', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'globals',
    'components/modal',
    'components/fillscreen',
    'gui/shadowtext',
    'components/anchor',
    'gui/blurbackground',
    'gui/worldselect/topbar',
    'modules/localization',
    'gui/confetti',
    'managers/item',
    'gui/coinreward',
    'gui/iconbutton',
    'components/backbutton'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Globals,
    Modal,
    FillScreen,
    ShadowText,
    Anchor,
    BlurBackground,
    Topbar,
    Localization,
    Confetti,
    ItemManager,
    CoinReward,
    IconButton,
    BackButton
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var itemKey = settings.item;
        var amount = settings.amount || 1;
        var autoDismiss = settings.autoDismiss || false;

        // swap with coinreward!
        if (itemKey === 'currency/gem') {
            return new CoinReward(Utils.extend(settings, {
                type: 'gem'
            }));
        } else if (itemKey === 'other/energy') {
            return new CoinReward(Utils.extend(settings, {
                type: 'energy'
            }));
        }

        var onComplete = settings.onComplete;
        var blurBackground = new BlurBackground({
            darken: 0.2,
            blurriness: 10
        });
        var overlay = new FillScreen({
            color: [0, 0, 0, 0.5]
        });
        var text = new ShadowText({
            text: Localization.getText('youGot'),
            font: 'font',
            fontSize: 24,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            maxWidth: viewport.width - 32
        }).attach(new Anchor({
            screenPosition: new Vector2(0.5, 0.5),
            offset: new Vector2(0, -64)
        }));
        var amountText = new ShadowText({
            text: amount > 1 ? amount.toString() : '',
            position: new Vector2(16, 6),
            font: 'font',
            fontSize: 16,
            fontColor: '#ffffff',
            align: 'right',
            textBaseline: 'middle'
        });
        var itemNameText = new ShadowText({
            text: '',
            font: 'font',
            fontSize: 24,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            maxWidth: 160,
            maxHeight: 24,
            linebreak: false
        }).attach(new Anchor({
            screenPosition: new Vector2(0.5, 0.5),
            offset: new Vector2(0, 64)
        }));
        var topbar = new Topbar({
            preventClick: true
        });
        var godrays1 = new Entity({
            name: 'godrays',
            alpha: 0.5,
            components: [
                new Anchor({
                    screenPosition: new Vector2(0.5, 0.5)
                }),
                new Sprite({
                    imageName: 'gui/godrays1',
                    originRelative: new Vector2(0.5, 0.5)
                }), {
                    update: function (data) {
                        data.entity.rotation += 0.01;
                    }
                }
            ]
        });
        var godrays2 = new Entity({
            name: 'godrays',
            alpha: 0.5,
            components: [
                new Anchor({
                    screenPosition: new Vector2(0.5, 0.5)
                }),
                new Sprite({
                    imageName: 'gui/godrays2',
                    originRelative: new Vector2(0.5, 0.5)
                }), {
                    update: function (data) {
                        data.entity.rotation -= 0.005;
                    }
                }
            ]
        });
        var itemEntity = new Entity({
            name: 'item',
            components: [
                new Anchor({
                    screenPosition: new Vector2(0.5, 0.5)
                })
            ]
        });
        var buttonOk = new IconButton({
            name: 'buttonOk',
            sfx: 'sfx_button',
            imageName: 'gui2/misc/button_small_primary',
            iconName: 'gui2/misc/button_icon_okay',
            frameCountX: 1,
            frameCountY: 3,
            float: false,
            animations: {
                'up': {
                    frames: [0]
                },
                'down': {
                    frames: [1]
                },
                'inactive': {
                    frames: [0]
                }
            },
            updateWhenPaused: 0,
            onClick: function () {
                buttonOk.setActive(false);
                // close automatically after 2 seconds
                new Tween({
                    from: 1,
                    to: 0,
                    in: 20,
                    ease: 'linear',
                    onUpdate: function (v, t) {
                        entity.alpha = v;
                    },
                    onComplete: function () {
                        entity.removeSelf();
                        if (onComplete) {
                            onComplete();
                        }
                    }
                });
            },
            components: [
                new BackButton({
                    removeSelf: true,
                    onPressed: function (data) {
                        data.entity.doCallback();
                    }
                }),
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.5, 1),
                        offset: new Vector2(0, -32)
                    },
                    landscape: {
                        screenPosition: new Vector2(1, 1),
                        offset: new Vector2(-32, -24)
                    }
                })
            ]
        });
        var itemSprite;
        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {
                var itemJson = Bento.assets.getJson(itemKey);
                var itemSpriteSheet = itemJson.inventory.image;
                itemSprite = new Sprite({
                    spriteSheet: itemSpriteSheet
                });
                itemSprite.setOriginRelative(new Vector2(0.5, 0.5));
                itemEntity.attach(itemSprite);
                itemEntity.attach(amountText);

                itemNameText.setText(ItemManager.getName(itemKey));

                // tween in
                new Tween({
                    from: 0,
                    to: 3,
                    in: 60,
                    delay: 0,
                    ease: 'elastic',
                    decay: 8,
                    oscillations: 3,
                    onUpdate: function (v, t) {
                        itemEntity.scale.x = v;
                        itemEntity.scale.y = v;
                    }
                });

                // tween in button ok
                buttonOk.position.y += 64;
                new Tween({
                    from: buttonOk.position.y,
                    to: buttonOk.position.y - 64,
                    in: 20,
                    delay: 30,
                    ease: 'elastic',
                    decay: 5,
                    oscillations: 1,
                    onUpdate: function (v, t) {
                        buttonOk.position.y = v;
                    }
                });

                entity.attach(itemEntity);

                // play getitem sound
                Bento.audio.stopSound('sfx_itemget');
                Bento.audio.playSound('sfx_itemget');

                if (settings.autoDismiss) {
                    buttonOk.removeSelf();
                    // tween out automatically
                    Utils.timeout(120, function () {
                        buttonOk.doCallback();
                    });
                }
            },
            destroy: function (data) {},
            update: function (data) {

            },
            draw: function (data) {}
        };
        var entity = new Entity({
            z: settings.z || Globals.Layers.GUI,
            name: 'coinReward',
            position: settings.position,
            float: true,
            components: [
                blurBackground,
                new Modal({}),
                text,
                itemNameText,
                godrays1,
                godrays2,
                topbar,
                new Confetti({}),
                buttonOk,
                behavior
            ]
        });
        return entity;
    };
});