/**
 * Item info is a dialog that shows more info about the selected item
 * @moduleName ItemInfo
 * @snippet ItemInfo.snippet
ItemInfo({
    item: '$1'
})
 */
bento.define('gui/iteminfo', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'gui/dialog',
    'managers/item',
    'components/anchor',
    'gui/tier',
    'modules/localization',
    'managers/allies',
    'gui/tokenbar'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Dialog,
    ItemManager,
    Anchor,
    TierStars,
    Localization,
    AlliesManager,
    TokenBar
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var item = settings.itemName || settings.item;
        var itemName = ItemManager.getName(item);
        var split = item.split('/');
        var itemCategory = split[0];
        var itemId = split[1];
        var itemDescription = ItemManager.getDescription(item);
        var itemSprite = ItemManager.getSprite(item);
        var itemTier = ItemManager.getTier(item);
        var tokenBar;
        var borderSprite = new Sprite({
            imageName: 'gui2/misc/panel_highlight_background_9slice',
            originRelative: new Vector2(0.5, 0.5)
        });
        var itemEntity = new Entity({
            name: 'item',
            position: new Vector2(0, 0),
            components: [
                borderSprite,
                itemSprite
            ]
        });
        var getItemDescription = function () {
            if (itemCategory === 'enemytokens') {
                // exception on enemy tokens which need extra effects
                var effect = AlliesManager.getEffectiveness(itemId, itemDescription);
                return effect.string;
            } else {
                return itemDescription;
            }
        };
        var dialog = new Dialog({
            ignoreAnalytics: true,
            blurriness: 5,
            title: itemName,
            text: getItemDescription(),
            type: 'yes',
            canClose: true,
            onClose: settings.onClose,
            onYes: settings.onYes,
            icon: itemEntity
        });

        var tierStars = new TierStars({
            position: new Vector2(-12, -12),
            tier: itemTier,
        });

        itemSprite.setOriginRelative(new Vector2(0.5, 0.5));

        // move the yes button below the dialog window
        dialog.getComponent('windowBox', function (windowBox) {
            windowBox.getComponent('buttonYes', function (buttonYes) {
                if (!settings.onYes) {
                    buttonYes.removeSelf();
                } else {

                }
            });
            // windowBox.getComponent('shadowText', function (text) {
            //     text.position.y += 40;
            // });
            // windowBox.attach(itemEntity);
            itemEntity.attach(tierStars);
        });

        // add token bar
        if (itemCategory === 'playertokens' || itemCategory === 'enemytokens' || itemCategory === 'evolve') {
            tokenBar = new TokenBar({
                position: new Vector2(40, 0),
                type: itemCategory,
                character: itemId
            });
            itemEntity.attach(tokenBar);
            itemEntity.position.x -= 20;
            borderSprite.visible = false;
        }

        return dialog;
    };
});