/**
 * GUI element that converts leftover health into stars
 * @moduleName StarCounter
 */
bento.define('gui/levelend/starcounter', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/entity',
    'modules/text',
    'bento/components/nineslice',
    'bento/tween',
    'managers/save',
    'components/debug',
    'components/blink',
    'gui/sparkleburst',
    'modules/analytics',
    'utils',
    'bento/eventsystem',
    'modules/firebase'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Entity,
    Text,
    NineSlice,
    Tween,
    SaveManager,
    Debug,
    Blink,
    SparkleBurst,
    Analytics,
    Utils,
    EventSystem,
    FireBase
) {
    'use strict';
    return function (settings) {
        var chosenDifficulty = SaveManager.load('chosenDifficulty');
        var stars = chosenDifficulty + 1;

        // create splash and star sprite
        var splashSprite = new Sprite({
            spriteSheet: 'levelend/stars-splash' + stars
        });
        var starsSprite = new Sprite({
            spriteSheet: 'levelend/stars' + stars
        });
        var splash = new Entity({
            name: 'splash',
            scale: new Vector2(0, 0),
            components: [
                splashSprite
            ]
        });
        var starEntity = new Entity({
            name: 'starEntity',
            scale: new Vector2(0, 0),
            components: [
                starsSprite
            ]
        });
        var starContainer = new Entity({
            name: 'starContainer',
            components: [
                splash,
                starEntity
            ]
        });
        //healtbar overlay
        var healthOverlay = new Entity({
            name: 'healthOverlay',
            components: [
                starContainer
            ]
        });

        //function to start the health to stars count behavior
        var startCount = function (onComplete) {
            var gaveStars = function () {
                //save stars if higher then previously saved
                var mission = SaveManager.load('currentMission');
                var currentStars = SaveManager.load('missionStars-' + mission);
                var first3stars = false;
                var firstVictory = false;
                var addedStars = stars - Math.max(currentStars, 0);
                var totalStars = SaveManager.load('totalStars');
                if (currentStars < 0) {
                    firstVictory = true;
                }
                if (stars > currentStars) {
                    // save changed amount of stars to totalStars
                    totalStars += addedStars;
                    SaveManager.save('totalStars', totalStars);
                    // save current mission stars
                    SaveManager.save('missionStars-' + mission, stars);

                    if (addedStars) {
                        EventSystem.fire('starsUpdated', {});
                    }

                    // save amount of new stars to show in the worldselect
                    var worldSelectItems = SaveManager.load('worldSelectItems');
                    worldSelectItems.stars = addedStars;
                    SaveManager.save('worldSelectItems', worldSelectItems);

                    if (stars === 3) {
                        first3stars = true;
                        SaveManager.save('firstTime3Stars', true);
                    }
                }

                if (onComplete) {
                    onComplete(first3stars, firstVictory);
                }

                Analytics.event('Stars', mission.toString(), stars);
                FireBase.setUserProperty('stars', totalStars);
            };

            // tween in splash
            new Tween({
                from: 0,
                to: 1,
                in: 15,
                ease: 'elastic',
                decay: 6,
                oscillations: 1,
                onUpdate: function (v, t) {
                    splash.scale.x = v;
                    splash.scale.y = v;
                }
            });
            new Tween({
                from: 0,
                to: 1,
                in: 10,
                delay: 5,
                ease: 'linear',
                onUpdate: function (v, t) {
                    starEntity.alpha = v;
                }
            });
            new Tween({
                from: 1.2,
                to: 1,
                in: 15,
                delay: 5,
                ease: 'quadratic',
                onUpdate: function (v, t) {
                    starEntity.scale.x = v;
                    starEntity.scale.y = v;
                },
                onComplete: function () {
                    // shake
                    new Tween({
                        from: 0.5,
                        to: 0,
                        in: 30,
                        onUpdate: function (v, t) {
                            starEntity.transform.x = Utils.getRandomRangeFloat(-v, v);
                            starEntity.transform.y = Utils.getRandomRangeFloat(-v, v);
                        },
                        onComplete: function () {
                            starEntity.transform.x = 0;
                            starEntity.transform.y = 0;
                            gaveStars();
                        }
                    });
                    // starEntity.attach(new Blink({}));
                    Bento.audio.stopSound('sfx_addstar' + stars);
                    Bento.audio.playSound('sfx_addstar' + stars);
                }
            });
        };
        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {}
        };
        var entity = new Entity({
            z: 0,
            name: 'starCounter',
            components: [
                healthOverlay,
                behavior
            ]
        }).extend({
            startCount: startCount
        });
        return entity;
    };
});