/**
 * Menu to choose amount of players to play a level
 * @moduleName PlayerCountMenu
 */
bento.define('gui/levelstart/playercount', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'gui/blurbackground',
    'components/modal',
    'gui/textbutton',
    'components/anchor',
    'globals',
    'gui/shadowtext',
    'modules/localization',
    'gui/headerbox',
    'bento/gui/togglebutton',
    'gui/buttons/startbutton',
    'gui/menus/multiplayer',
    'gui/menus/onlinecreate',
    'gui/menus/onlinejoin',
    'gui/worldselect/topbarsimple',
    'bento/components/nineslice'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    BlurBackground,
    Modal,
    TextButton,
    Anchor,
    Globals,
    ShadowText,
    Localization,
    HeaderBox,
    ToggleButton,
    StartButton,
    MultiplayerMenu,
    OnlineCreate,
    OnlineJoin,
    TopBarSimple,
    NineSlice
) {
    'use strict';

    var WifiIcon = function (settings) {
        settings = settings || {};
        return new Entity({
            name: 'wifiIcon',
            position: settings.position || new Vector2(8, 12),
            components: [
                new Sprite({
                    imageName: 'gui/wifi-icon',
                    originRelative: new Vector2(0.5, 0.5)
                })
            ]
        });
    };

    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var selection = 1;
        var onSinglePlayer = settings.onSinglePlayer;
        var onMultiPlayerCreate = settings.onMultiPlayerCreate;
        var onMultiPlayerJoin = settings.onMultiPlayerJoin;
        var tipText = new Text({
            position: new Vector2(0, 0),
            text: Localization.getText('onlineTip'),
            font: 'font',
            fontSize: 10,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            maxWidth: viewport.width - 16,
            maxHeight: 32
        }).attach(new Anchor({
            screenPosition: new Vector2(0, 0.5),
            offset: new Vector2(0, -32),
            onComplete: function (parent, isPortrait) {
                tipText.setText(Localization.getText('onlineTip'), {
                    maxWidth: viewport.width - 16,
                    maxHeight: isPortrait ? 32 : 16
                });
                if (!isPortrait) {
                    tipText.position.y += 8;
                }

                // center playercount vertically
                entity.position.y = viewport.height / 2;
            }
        })).attach({
            name: 'showTip',
            start: function () {
                tipText.visible = false;
            },
            update: function () {
                if (selection === 2) {
                    tipText.visible = true;
                } else {
                    tipText.visible = false;
                }
            }
        });

        var buttonPlay = new ClickButton({
            name: 'startButton',
            position: new Vector2(0, 32),
            sfx: 'sfx_button',
            imageName: 'gui/buttons/play',
            frameCountX: 3,
            frameCountY: 1,
            onClick: function () {
                onSinglePlayer();
            }
        });
        var textJoin = new ShadowText({
            position: new Vector2(0, -4),
            text: Localization.getText('onlineJoin'),
            font: 'font',
            fontSize: 16,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            linebreaks: false,
            maxHeight: 16,
            maxWidth: 54
        });
        var buttonJoin = new ClickButton({
            name: 'buttonJoin',
            sfx: 'sfx_button',
            imageName: 'gui/buttons/play-empty',
            frameCountX: 2,
            frameCountY: 1,
            position: new Vector2(0, -24),
            float: false,
            updateWhenPaused: 0,
            onClick: function () {
                /*var joinMenu = new OnlineJoin({
                    // mission data will be received by host
                    // is this even the right place to show the join button?
                });
                Bento.objects.attach(joinMenu);*/
                onMultiPlayerJoin();
            },
            onButtonDown: function () {
                textJoin.position.y = -3;
            },
            onButtonUp: function () {
                textJoin.position.y = -4;
            }
        }).attach(textJoin);
        var textCreate = new ShadowText({
            position: new Vector2(0, -4),
            text: Localization.getText('onlineCreate'),
            font: 'font',
            fontSize: 16,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            linebreaks: false,
            maxHeight: 16,
            maxWidth: 54
        });
        var buttonCreate = new ClickButton({
            name: 'buttonCreate',
            sfx: 'sfx_button',
            imageName: 'gui/buttons/play-empty',
            frameCountX: 2,
            frameCountY: 1,
            position: new Vector2(0, 24),
            float: false,
            updateWhenPaused: 0,
            onClick: function () {
                /*var createMenu = new OnlineCreate({
                    cost: settings.cost,
                    mission: settings.mission,
                    unlocks: settings.unlocks,
                    rewards: settings.rewards
                });
                Bento.objects.attach(createMenu);*/
                onMultiPlayerCreate();
            },
            onButtonDown: function () {
                textCreate.position.y = -3;
            },
            onButtonUp: function () {
                textCreate.position.y = -4;
            }
        }).attach(textCreate);

        var buttonSingle = new ClickButton({
            name: 'buttonSingle',
            sfx: 'sfx_button',
            imageName: 'gui/buttons/single',
            frameCountX: 2,
            frameCountY: 1,
            position: new Vector2(0, 12),
            onClick: function () {
                onSinglePlayer();
            }
        }).attach({
            name: 'activator',
            update: function (data) {
                var self = data.entity;
                if (self.visible !== self.parent.visible) {
                    self.visible = self.parent.visible;
                    self.setActive(self.parent.visible);
                }
            }
        });
        var boxLeft = new Entity({
            name: 'box',
            position: new Vector2(-37, 0),
            components: [
                new NineSlice({
                    imageName: 'gui/dialog/block-light',
                    originRelative: new Vector2(0.5, 0.5),
                    width: 72,
                    height: 112
                }),
                new ShadowText({
                    position: new Vector2(0, -18),
                    text: Localization.getText('single'),
                    font: 'font',
                    fontSize: 16,
                    fontColor: '#ffffff',
                    align: 'center',
                    textBaseline: 'middle'
                }),
                buttonSingle
            ]
        });

        var buttonMulti = new ClickButton({
            name: 'buttonMulti',
            sfx: 'sfx_button',
            imageName: 'gui/buttons/multi',
            frameCountX: 3,
            frameCountY: 1,
            position: new Vector2(0, 12),
            animations: {
                'up': {
                    frames: [0]
                },
                'down': {
                    frames: [1]
                },
                'inactive': {
                    frames: [2]
                }
            },
            onClick: function () {
                // tween right
                if (selection !== 2) {
                    selection = 2;
                    if (!headerBox2.parent) {
                        entity.attach(headerBox2);
                    }
                    new Tween({
                        from: 0,
                        to: viewport.width,
                        in: 20,
                        ease: 'exponential',
                        decay: 5,
                        onUpdate: function (v, t) {
                            headerBox.position.x = -v;
                            headerBox2.position.x = viewport.width - v;
                        },
                        onComplete: function () {

                        }
                    });
                }
            }
        }).attach({
            name: 'activator',
            update: function (data) {
                var self = data.entity;
                if (self.visible !== self.parent.visible) {
                    self.visible = self.parent.visible;
                    self.setActive(self.parent.visible);
                }
            }
        });

        var boxRight = new Entity({
            name: 'box',
            position: new Vector2(37, 0),
            components: [
                new NineSlice({
                    imageName: 'gui/dialog/block-light',
                    originRelative: new Vector2(0.5, 0.5),
                    width: 72,
                    height: 112
                }),
                new ShadowText({
                    position: new Vector2(0, -18),
                    text: Localization.getText('onlineMulti'),
                    font: 'font',
                    fontSize: 16,
                    fontColor: '#ffffff',
                    align: 'center',
                    textBaseline: 'middle'
                }),
                new WifiIcon({
                    position: new Vector2(0, -32)
                }),
                buttonMulti
            ]
        });

        var headerBox = new HeaderBox({
                size: new Rectangle(0, 0, 160, 126),
                title: Localization.getText('mode'),
                windowImage: 'gui/dialog/window-dark'
            })
            .attach(boxLeft)
            .attach(boxRight);

        /**
         * Multi
         */
        /*var boxLeft2 = new Entity({
            name: 'box',
            position: new Vector2(-37, 0),
            components: [
                new NineSlice({
                    imageName: 'gui/dialog/block-light',
                    originRelative: new Vector2(0.5, 0.5),
                    width: 72,
                    height: 112
                }),
                buttonCreate
            ]
        });
        var boxRight2 = new Entity({
            name: 'box',
            position: new Vector2(37, 0),
            components: [
                new NineSlice({
                    imageName: 'gui/dialog/block-light',
                    originRelative: new Vector2(0.5, 0.5),
                    width: 72,
                    height: 112
                }),
                buttonJoin
            ]
        });*/

        var headerBox2 = new HeaderBox({
                position: new Vector2(viewport.width, 0),
                size: new Rectangle(0, 0, 180, 148),
                title: Localization.getText('onlineMulti'),
                windowImage: 'gui2/misc/splash_background_9slice',
            })
            .attach(buttonJoin)
            .attach(buttonCreate);

        // entity
        var behavior = {
            name: 'behavior',
            start: function () {
                buttonMulti.doCallback();
            }
        };
        var entity = new Entity({
            z: Globals.WorldSelectLayers.LEVELDETAILS,
            name: 'playerCountMenu',
            position: settings.position,
            components: [
                behavior,
                // headerBox,
                tipText
            ]
        });

        entity.extend({
            disableButtons: function () {
                buttonPlay.setActive(false);
                buttonJoin.setActive(false);
                buttonCreate.setActive(false);
            },
            enableButtons: function () {
                buttonPlay.setActive(true);
                buttonJoin.setActive(true);
                buttonCreate.setActive(true);
            },
            closeOnline: function () {
                if (selection === 2) {
                    selection = 1;
                    new Tween({
                        from: viewport.width,
                        to: 0,
                        in: 20,
                        ease: 'exponential',
                        decay: 5,
                        onUpdate: function (v, t) {
                            headerBox.position.x = -v;
                            headerBox2.position.x = viewport.width - v;
                        },
                        onComplete: function () {

                        }
                    });
                }
            },
            isShowingOnline: function () {
                return selection === 2;
            }
        });

        return entity;
    };
});