/**
 * GUI rewards info
 * @moduleName RewardsPopup
 */
bento.define('gui/levelstart/rewardspopup', [
    'bento',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/gui/clickbutton',
    'bento/math/vector2',
    'modules/text',
    'gui/shadowtext',
    'gui/reward',
    'components/modal',
    'components/anchor',
    'globals',
    'gui/headerbox',
    'bento/math/rectangle',
    'gui/blurbackground',
    'modules/localization',
    'bento/tween',
    'managers/character',
    'managers/save',
    'gui/equipmentslots',
    'bento/components/nineslice',
    'components/shadow',
    'gui/senseipopup',
    'gui/highlight',
    'gui/taphand',
    'utils',
    'gui/dialog',
    'gui/watchrewardedad',
    'modules/analytics',
    'gui/energyforgempopup',
    'bento/eventsystem',
    'bento/gui/togglebutton',
    'components/debug',
    'managers/inventory',
    'managers/reward',
    'gui/centeredlist',
    'gui/iconbutton',
    'components/backbutton'
], function (
    Bento,
    Sprite,
    Clickable,
    Entity,
    ClickButton,
    Vector2,
    Text,
    ShadowText,
    Reward,
    Modal,
    Anchor,
    Globals,
    HeaderBox,
    Rectangle,
    BlurBackground,
    Localization,
    Tween,
    CharacterManager,
    SaveManager,
    EquipmentSlots,
    NineSlice,
    Shadow,
    SenseiPopup,
    Highlight,
    TapHand,
    Utils,
    Dialog,
    WatchRewardedAd,
    Analytics,
    EnergyForGemPopup,
    EventSystem,
    ToggleButton,
    Debug,
    InventoryManager,
    RewardManager,
    CenteredList,
    IconButton,
    BackButton
) {
    'use strict';
    return function (settings) {
        var viewport = Bento.getViewport();

        // online reward
        var wifiIcon = new Entity({
            name: 'wifiIcon',
            position: new Vector2(20, 4),
            components: [
                new Sprite({
                    imageName: 'gui/wifi-icon',
                    originRelative: new Vector2(0.5, 0.5)
                })
            ]
        });
        var checkMark = new Entity({
            name: 'checkMark',
            position: new Vector2(0, 0),
            components: [
                new Sprite({
                    imageName: 'gui/dailyquest/checkmark',
                    originRelative: new Vector2(0.5, 0.5)
                })
            ]
        });
        var possibleRewards = new Text({
            position: new Vector2(0, -23 - 8),
            text: Localization.getText('reward'),
            font: 'font',
            fontSize: 12,
            fontColor: '#b2b2b2',
            align: 'center',
            textBaseline: 'middle',
            maxWidth: 56,
            linebreaks: false
        });
        var reward1 = new Reward({
            index: 0,
            mission: settings.mission,
            position: new Vector2(0, 0),
            item: settings.rewards.reward1,
            amount: settings.rewards.amount1,
            titleText: possibleRewards
        });
        var reward2 = new Reward({
            index: 1,
            mission: settings.mission,
            position: new Vector2(0, 0),
            item: settings.rewards.reward2,
            amount: settings.rewards.amount2,
            titleText: possibleRewards,
        });
        var randomizedReward = RewardManager.getReward(settings.mission);
        var reward3 = new Reward({
            index: 2,
            mission: settings.mission,
            position: new Vector2(44, 0),
            item: randomizedReward.reward,
            amount: randomizedReward.amount,
            titleText: possibleRewards,
        });
        var currentMission = settings.mission;
        var chosenDifficulty = settings.chosenDifficulty;
        var isOnline = settings.isOnline;
        var missionStars = SaveManager.load('missionStars-' + currentMission);
        var currentDifficulty = Utils.clamp(0, missionStars, 2);
        var currentCharacter = CharacterManager.getSelected();
        var unlockedCharacters = SaveManager.load('unlockedCharacters');
        var portrait = false;
        var titleText = new Text({
            text: currentMission === 'tower' ? Localization.getText('towerTitle') : Localization.getText('missionX').replace('{NUMBER}', currentMission),
            font: 'font',
            fontSize: 22,
            fontColor: '#fffd00',
            align: 'center',
            textBaseline: 'middle',
            components: [
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0, 0),
                        offset: new Vector2(0, -124)
                    },
                    landscape: {
                        screenPosition: new Vector2(0, 0),
                        offset: new Vector2(0, -76)
                    }
                })
            ]
        });
        /**
         * Rewards
         */
        var rewardsHeader = new Entity({
            name: 'header',
            position: new Vector2(0, -23),
            components: [
                new Sprite({
                    imageName: 'gui2/misc/borderheader',
                    originRelative: new Vector2(0.5, 1)
                })
            ]
        });
        var rewardsBackground = new NineSlice({
            imageName: 'gui2/misc/border_9slice',
            originRelative: new Vector2(0.5, 0.5),
            width: 148,
            height: 45
        });
        var rewardsContainer = new Entity({
            name: 'rewardsContainer',
            position: new Vector2(0, -62),
            components: [
                rewardsHeader,
                rewardsBackground,
                possibleRewards,
                checkMark
            ]
        });
        var itemsTutorial = {
            tapHand: null,
            highlight: null,
            consumablesSlot0: null,
            consumablesSlot1: null,
            start: function () {
                itemsTutorial.consumablesSlot0 = consumables.getComponent('centeredList').get(0);
                itemsTutorial.consumablesSlot1 = consumables.getComponent('centeredList').get(1);
                itemsTutorial.forceItemSelect();
            },
            cleanUp: function () {
                itemsTutorial.isForcingPlay = false;
                itemsTutorial.isForcingItemSelect = false;
                if (itemsTutorial.tapHand) {
                    itemsTutorial.tapHand.removeSelf();
                    itemsTutorial.tapHand = null;
                }
                if (itemsTutorial.highlight) {
                    itemsTutorial.highlight.removeSelf();
                    itemsTutorial.highlight = null;
                }
            },
            isForcingItemSelect: false,
            isForcingPlay: false,
            forceItemSelect: function () {
                // this is a bit hacky, but we need to wait for tweens to finish
                // and the consumables button to be at their final positions
                Utils.timeout(30, function () {
                    itemsTutorial.isForcingItemSelect = true;
                    itemsTutorial.tapHand = new TapHand({
                        z: Globals.WorldSelectLayers.LEVELDETAILS + 1,
                        position: itemsTutorial.consumablesSlot0.getWorldPosition().add(new Vector2(0, -12))
                    });
                    itemsTutorial.highlight = new Highlight({
                        z: Globals.WorldSelectLayers.LEVELDETAILS + 1,
                        position: itemsTutorial.consumablesSlot0.getWorldPosition()
                    });

                    Bento.objects.attach(itemsTutorial.highlight);
                    Bento.objects.attach(itemsTutorial.tapHand);
                });
                buttonPlay.setActive(false);
                entity.parent.getComponent('buttonClose', function (buttonClose) {
                    buttonClose.setActive(false);
                });
            },
            forcePlayButton: function () {
                itemsTutorial.isForcingPlay = true;
                itemsTutorial.tapHand = new TapHand({
                    z: Globals.WorldSelectLayers.LEVELDETAILS + 1,
                    position: buttonPlay.getWorldPosition().add(new Vector2(-24, -12)),
                    rotation: -Math.PI * 1 / 4
                });
                itemsTutorial.highlight = new Highlight({
                    z: Globals.WorldSelectLayers.LEVELDETAILS + 1,
                    position: buttonPlay.getWorldPosition()
                });

                Bento.objects.attach(itemsTutorial.highlight);
                Bento.objects.attach(itemsTutorial.tapHand);
                buttonPlay.setActive(true);
            },
            update: function () {
                // check if player selected any items
                if (itemsTutorial.consumablesSlot0 && itemsTutorial.isForcingItemSelect) {
                    if (itemsTutorial.consumablesSlot0.getItem()) {
                        itemsTutorial.cleanUp();
                        // itemsTutorial.forcePlayButton();
                        buttonPlay.setActive(true);
                    }
                }
            },
            destroy: function () {
                itemsTutorial.cleanUp();
                buttonPlay.setActive(true);
            }
        };
        var behavior = {
            name: 'infoBehavior',
            start: function () {
                var tutorialItem = SaveManager.load('tutorialItem');
                var hasConsumables = (function () {
                    var items = InventoryManager.getAllItems(['consumables', 'weapons']);
                    return items.length > 0;
                })();
                var tweenIn = function (obj, delay) {
                    obj.scale.x = 0;
                    obj.scale.y = 0;
                    new Tween({
                        from: 0,
                        to: 1,
                        in: 20,
                        delay: delay,
                        ease: 'elastic',
                        decay: 7,
                        oscillations: 1,
                        onUpdate: function (v, t) {
                            obj.scale.x = v;
                            obj.scale.y = v;
                        }
                    });
                };
                var autoSelectItems = function () {
                    var itemsInInventory;
                    var slot1 = consumables.getSlot(0);
                    var slot2 = consumables.getSlot(1);
                    var randomItem;

                    itemsInInventory = InventoryManager.getAllItems(['consumables', 'weapons']);

                    // fill slot 1
                    if (!slot1 && itemsInInventory.length > 0) {
                        randomItem = itemsInInventory[Utils.getRandom(itemsInInventory.length)];
                        consumables.setSlot(0, randomItem.key);
                    }

                    // refresh inventory
                    itemsInInventory = InventoryManager.getAllItems(['consumables', 'weapons']);

                    // fill slot 2
                    if (!slot2 && itemsInInventory.length > 0) {
                        randomItem = itemsInInventory[Utils.getRandom(itemsInInventory.length)];
                        consumables.setSlot(1, randomItem.key);
                    }

                };
                // attach rewards
                // show reward2 if it's not collected yet
                var has2nd = (settings.rewards.reward2 !== 'nothing/nothing');
                var has1st = (settings.rewards.reward1 !== 'nothing/nothing');
                var collectedRewards = SaveManager.load('rewards-' + settings.mission);
                if (has2nd && !collectedRewards[1]) {
                    rewardsContainer.attach(reward2);
                } else if (has1st && !collectedRewards[0]) {
                    rewardsContainer.attach(reward1);
                } else {
                    rewardsContainer.attach(reward3);
                }

                // tween in
                tweenIn(entity, 0);
                /*tweenIn(rewardsContainer, 0);
                tweenIn(charSelect, 10);
                tweenIn(consumablesContainer, 20);*/

                // never played level 1 before: don't allow items
                var unlocked = SaveManager.load('unlockedLevels');
                if (unlocked.length === 1) {
                    consumablesContainer.removeSelf();
                    buttonPlay.attach(new TapHand({
                        position: new Vector2(0, -12)
                    }));
                } else if (hasConsumables && tutorialItem === 0 && unlocked.length >= 2 && !settings.invitation) {
                    buttonPlay.setActive(false);
                    Utils.timeout(15, function () {
                        buttonPlay.setActive(true);
                        // NOTE: this is a bit hacky, waiting 15 frames for some tweens to end
                        Bento.objects.attach(new SenseiPopup({
                            text: Localization.getText('tutorial')['worldSelect1'],
                            modal: true,
                            darken: true,
                            onNoMoreText: function (index) {
                                this.dismiss();

                                // point to items
                                entity.attach(itemsTutorial);

                                autoSelect.setActive(false);
                                autoText.alpha = 0.5;
                            }
                        }));
                    });
                } else if (!hasConsumables && tutorialItem === 0) {
                    consumablesContainer.removeSelf();
                } else {
                    // autoselect items
                    if (SaveManager.load('autoSelect')) {
                        autoSelectItems();
                    }
                }
            }
        };
        var titleBar = new Entity({
            name: 'titleBar',
            position: new Vector2(0, 0),
            components: [
                new Sprite({
                    imageName: 'gui2/misc/title_bar',
                    originRelative: new Vector2(0.5, 0.5)
                })
            ]
        });
        var rewardframe = new HeaderBox({
            windowImage: 'gui2/misc/splash_background_9slice',
            // windowImage: 'gui/dialog/window-dark',
            size: new Rectangle(0, 0, 180, 212),
            originRelative: new Vector2(0.5, 0.5),
        }).attach(rewardsContainer);

        /**
         * Difficulty
         */
        var difficultyHeader = new Entity({
            name: 'header',
            components: [
                new Sprite({
                    imageName: 'gui2/misc/borderheader',
                    originRelative: new Vector2(0.5, 1)
                })
            ]
        });
        var difficultyBackground = new NineSlice({
            imageName: 'gui2/misc/border_9slice',
            originRelative: new Vector2(0.5, 0.5),
            width: 148,
            height: 54
        });
        var difficultyTitle = new Text({
            position: new Vector2(0, -difficultyBackground._height / 2 - 8),
            text: Localization.getText('difficulty'),
            font: 'font',
            fontSize: 12,
            fontColor: '#b2b2b2',
            align: 'center',
            textBaseline: 'middle',
            maxWidth: 56,
            linebreaks: false
        });
        var difficultyButtons = [];
        var makeDifficultyButton = function () {
            var index = difficultyButtons.length;
            var ids = ['easy', 'medium', 'hard'];
            var initialToggle = (isOnline ? index === chosenDifficulty : index === currentDifficulty);
            var hasCleared = (index < missionStars);
            var icon = new Entity({
                name: 'icon',
                alpha: initialToggle ? 1 : 0.2,
                position: new Vector2(0, -6),
                components: [
                    new Sprite({
                        imageName: hasCleared ? 'gui/dailyquest/checkmark-small' : 'gui2/misc/difficulty' + index,
                        originRelative: new Vector2(0.5, 0.5)
                    })
                ]
            });
            var text = new ShadowText({
                position: new Vector2(0, 8),
                text: Localization.getText(ids[index]),
                font: 'font',
                fontSize: 10,
                fontColor: initialToggle ? '#ffffff' : '#9330b0',
                align: 'center',
                textBaseline: 'middle',
                maxWidth: 32,
                linebreaks: false,
                components: [{
                    name: 'setColor',
                    light: initialToggle,
                    update: function () {
                        if (this.light && !button.isToggled()) {
                            this.light = false;
                            text.setText(text.getText(), {
                                fontColor: '#9330b0'
                            });
                            icon.alpha = 0.5;
                        } else if (!this.light && button.isToggled()) {
                            this.light = true;
                            text.setText(text.getText(), {
                                fontColor: '#ffffff'
                            });
                            icon.alpha = 1;
                        }
                    }
                }]
            });
            var button = new ToggleButton({
                name: 'button',
                sfx: 'sfx_button',
                imageName: 'gui2/misc/panel',
                frameCountX: 2,
                frameCountY: 1,
                toggled: initialToggle,
                onToggle: function () {
                    if (this.isToggled()) {
                        // untoggle others
                        Utils.forEach(difficultyButtons, function (other) {
                            if (other !== button) {
                                other.toggle(false);
                            } else {
                                currentDifficulty = index;
                            }
                        });
                    } else {
                        // re-toggle self
                        button.toggle(true);
                    }

                    showRewardClaimed();
                },
                components: [
                    icon,
                    text
                ]
            });
            var lock = new Entity({
                name: 'lock',
                position: new Vector2(0, 0),
                components: [
                    new Sprite({
                        imageName: 'gui/lock',
                        originRelative: new Vector2(0.5, 0.5)
                    }),
                    new Clickable({
                        onClick: function (data) {
                            // shake
                            new Tween({
                                from: 2,
                                to: 0,
                                in: 40,
                                ease: 'elastic',
                                decay: 2,
                                oscillations: 5,
                                onUpdate: function (v, t) {
                                    lock.transform.x = v;
                                },
                                onComplete: function () {
                                    lock.transform.x = 0;
                                }
                            });
                            Bento.audio.stopSound('sfx_error');
                            Bento.audio.playSound('sfx_error');

                            var dialog = new Dialog({
                                ignoreAnalytics: true,
                                type: 'yes',
                                title: Localization.getText(ids[index]),
                                text: Localization.getText('difficultyLocked')
                            });
                            Bento.objects.attach(dialog);
                        }
                    })
                ]
            });
            var showRewardClaimed = function () {
                if (hasCleared && currentDifficulty < 2) {
                    checkMark.visible = true;
                    reward1.visible = false;
                    reward3.visible = false;
                } else {
                    checkMark.visible = false;
                    reward1.visible = true;
                    reward3.visible = true;
                }
            };
            difficultyButtons.push(button);

            // disable if not enough stars
            if (!isOnline && index !== 0 && index > missionStars) {
                button.setActive(false);
                button.attach(lock);
                icon.alpha = 0.25;
            }
            if (isOnline) {
                button.setActive(false);
            }

            showRewardClaimed();

            return button;
        };
        var buttonList = new CenteredList({
            position: new Vector2(0, 0)
        });
        var difficultyContainer = new Entity({
            name: 'difficultyContainer',
            components: [
                difficultyHeader,
                difficultyBackground,
                difficultyTitle,
                buttonList
            ]
        });
        rewardframe.attach(difficultyContainer);
        buttonList.addItem(makeDifficultyButton());
        buttonList.addItem(makeDifficultyButton());
        buttonList.addItem(makeDifficultyButton());

        /**
         * Consumables
         */
        var autoText = new ShadowText({
            position: new Vector2(8, -2),
            text: Localization.getText('autoSelect'),
            font: 'font',
            fontSize: 12,
            fontColor: '#ffffff',
            align: 'left',
            textBaseline: 'middle',
            maxWidth: 32,
            maxHeight: 16,
            linebreaks: false
        });
        var autoSelect = new ToggleButton({
            name: 'autoSelect',
            position: new Vector2(0, 0),
            boundingBox: new Rectangle(-16, -10, 48, 20),
            imageName: 'gui/buttons/checkbox',
            frameCountX: 3,
            frameCountY: 1,
            toggled: SaveManager.load('autoSelect'),
            animations: {
                up: {
                    frames: [0]
                },
                down: {
                    frames: [1]
                },
                inactive: {
                    frames: [2]
                }
            },
            onToggle: function () {
                SaveManager.save('autoSelect', autoSelect.isToggled());
            }
        }).attach(autoText);
        var consumablesHeader = new Entity({
            name: 'header',
            components: [
                new Sprite({
                    imageName: 'gui2/misc/borderheader',
                    originRelative: new Vector2(0.5, 1)
                })
            ]
        });
        var consumablesBackground = new NineSlice({
            imageName: 'gui2/misc/border_9slice',
            originRelative: new Vector2(0.5, 0.5),
            width: 148,
            height: 54
        });
        var consumablesTitle = new Text({
            position: new Vector2(0, -consumablesBackground._height / 2 - 8),
            text: Localization.getText('takeBoosts'),
            font: 'font',
            fontSize: 12,
            fontColor: '#b2b2b2',
            align: 'center',
            textBaseline: 'middle',
            maxWidth: 56,
            linebreaks: false
        });
        var consumables = new EquipmentSlots({
            iconImage: 'gui/startlevel/inventory',
            type: 'consumable',
            slotSize: 'new',
            // hideSlot2: true,
            position: new Vector2(0, 4),
        });
        var consumablesContainer = new Entity({
            name: 'consumablesContainer',
            position: new Vector2(0, 64),
            components: [
                consumablesHeader,
                consumablesBackground,
                consumablesTitle,
                consumables,
                autoSelect
            ]
        });
        rewardframe.attach(consumablesContainer);
        /**
         * Close
         */
        var buttonClose = new IconButton({
            name: 'buttonClose',
            sfx: 'sfx_button',
            imageName: 'gui2/misc/button_smaller',
            iconName: 'gui2/misc/button_icon_back_small',
            frameCountX: 1,
            frameCountY: 3,
            onClick: function () {
                // tween out
                if (settings.onClose) {
                    settings.onClose();
                }
            },
            components: [
                new BackButton({
                    removeSelf: true,
                    onPressed: function (data) {
                        data.entity.doCallback();
                    }
                }),
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0, 0.5),
                        offset: new Vector2(-64, -60 + 16)
                    },
                    landscape: {
                        screenPosition: new Vector2(-0.5, -0.5),
                        offset: new Vector2(32, 12)
                    },
                    onComplete: function (parent, isPortrait) {
                        align(parent, isPortrait);
                    }
                })
            ]
        });
        /**
         * Play button
         */
        var exemptFromEnergy = SaveManager.load('missionsExemptFromEnergy');
        var index = exemptFromEnergy.indexOf(settings.mission);
        var costText = new Text({
            position: new Vector2(8, 6),
            text: index >= 0 ? '0' : settings.cost.toString(),
            font: 'font',
            fontSize: 12,
            fontColor: '#00baff',
            align: 'left',
            textBaseline: 'middle',
            lineWidth: 1.5,
            strokeStyle: '#4f3673',
            pixelStroke: true,
        });
        var costIcon = new Entity({
            name: 'costIcon',
            position: new Vector2(-6 - 24, 0),
            components: [
                new Sprite({
                    imageName: 'gui2/misc/energy',
                    originRelative: new Vector2(0, 0.5)
                }),
                costText
            ]
        });
        var playText = new ShadowText({
            position: new Vector2(8, 0),
            originRelative: new Vector2(0, 0.5),
            text: Localization.getText('play'),
            font: 'font',
            fontSize: 22,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            strokeStyle: '#000000',
            maxWidth: 44,
            maxHeight: 22
        });
        var container = new Entity({
            name: 'icon',
            components: [
                costIcon,
                playText
            ]
        });
        var buttonPlay = new IconButton({
            name: 'startButton',
            sfx: 'sfx_button',
            icon: container,
            iconAlpha: 0.5,
            imageName: 'gui2/misc/button_medium_primary',
            frameCountX: 1,
            frameCountY: 3,
            onClick: function () {
                var dialog;
                if (CharacterManager.hasEnergy(settings.cost) || index >= 0) {
                    // character energy will be depleted later
                    if (settings.onClick) {
                        settings.onClick(currentDifficulty, false);
                    }
                } else {
                    // ask to watch ad
                    dialog = new EnergyForGemPopup({
                        z: Globals.WorldSelectLayers.LEVELDETAILS,
                        onBoughtEnergy: function (hasBought) {
                            if (hasBought) {
                                if (settings.onClick) {
                                    settings.onClick(currentDifficulty, false);
                                }
                            }
                        },
                        onWatchedAd: function () {
                            costText.setText('0');
                            exemptFromEnergy.push(settings.mission);
                            SaveManager.save('missionsExemptFromEnergy', exemptFromEnergy);

                            if (settings.onClick) {
                                settings.onClick(currentDifficulty);
                            }
                        }
                    });
                    Bento.objects.attach(dialog);
                    return;
                }
                if (itemsTutorial.tapHand) {
                    itemsTutorial.tapHand.removeSelf();
                    itemsTutorial.tapHand = null;
                }
                if (itemsTutorial.highlight) {
                    itemsTutorial.highlight.removeSelf();
                    itemsTutorial.highlight = null;
                }
                // entity.parent.getComponent('buttonClose', function (buttonClose) {
                //     buttonClose.setActive(true);
                // });
            },
            onButtonDown: function () {},
            onButtonUp: function () {},
            components: [
                new Anchor({
                    screenPosition: new Vector2(0, 0.5),
                    offset: new Vector2(0, -44),
                    onComplete: function (parent, isPortrait) {
                        align(parent, isPortrait);
                    }
                })
            ]
        });
        var align = function (parent, isPortrait) {
            portrait = isPortrait;
            if (isPortrait) {
                rewardframe.setWidth(160 + 24);
                rewardframe.setHeight(192 + 48);
                rewardframe.position.x = 0;
                rewardframe.position.y = 0;
                titleBar.position.y = -rewardframe.getHeight() / 2 + 8;

                // rewards
                rewardsBackground.setWidth(148);
                rewardsContainer.position.x = 0;
                rewardsContainer.position.y = -72;

                // difficulty
                difficultyContainer.position = new Vector2(0, -8);
                difficultyBackground.setWidth(148);
                difficultyBackground.setHeight(48);
                difficultyTitle.position.y = -difficultyBackground._height / 2 - 8;
                difficultyHeader.position.y = -difficultyBackground._height / 2;

                // consumables
                consumablesContainer.position = new Vector2(0, 56);
                consumablesBackground.setWidth(148);
                consumablesBackground.setHeight(48);
                consumablesTitle.position.y = -consumablesBackground._height / 2 - 8;
                consumablesHeader.position.y = -consumablesBackground._height / 2;
                consumables.position = new Vector2(8, 2);
                consumables.getComponent('icon', function (icon) {
                    icon.position = new Vector2(-56, -6);
                });
                autoSelect.position = new Vector2(-64, 16);
            } else {
                buttonPlay.position.x += 64;
                rewardframe.setWidth(200 + 96);
                rewardframe.setHeight(140 + 8);
                rewardframe.position.x = 0;
                rewardframe.position.y = 0;
                titleBar.position.y = -rewardframe.getHeight() / 2 + 12;

                // rewards
                rewardsBackground.setWidth(148);
                rewardsContainer.position.x = -58;
                rewardsContainer.position.y = -25;

                // difficulty
                difficultyBackground.setWidth(148);
                difficultyBackground.setHeight(48);
                difficultyContainer.position.x = -58;
                difficultyContainer.position.y = 38;
                difficultyTitle.position.y = -difficultyBackground._height / 2 - 8;
                difficultyHeader.position.y = -difficultyBackground._height / 2;

                // consumables
                consumablesContainer.position = new Vector2(72, -13);
                consumablesBackground.setWidth(92);
                consumablesBackground.setHeight(72);
                consumablesTitle.position.y = -consumablesBackground._height / 2 - 8;
                consumablesHeader.position.y = -consumablesBackground._height / 2;
                consumables.position = new Vector2(-16, 16);
                consumables.getComponent('icon', function (icon) {
                    icon.position = new Vector2(-4, -32);
                });
                autoSelect.position = new Vector2(0, -12);

            }
            if (currentMission === 'tower') {
                rewardsContainer.removeSelf();
            }
        };

        var entity = new Entity({
            z: Globals.WorldSelectLayers.LEVELDETAILS,
            name: 'rewardsPopup',
            position: new Vector2(0, 0),
            float: true,
            components: [
                titleBar,
                rewardframe,
                titleText,
                buttonClose,
                buttonPlay,
                behavior
            ].concat(settings.components || [])
        });

        entity.extend({
            getChosenDifficulty: function () {
                return currentDifficulty;
            }
        });

        return entity;
    };
});