/**
 * Checks if character can level up or unlock
 * @moduleName LevelUpNotification
 */
bento.define('gui/levelupnotification', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'managers/currency',
    'managers/stats',
    'globals',
    'gui/notification',
    'modules/localization',
    'managers/character',
    'managers/inventory'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    CurrencyManager,
    StatsManager,
    Globals,
    Notification,
    Localization,
    CharacterManager,
    InventoryManager
) {
    'use strict';
    /**
     * Creates a notification if last token added was enough to level up the character
     */
    var shouldNotify = function (character, lastTokenType, lastTokenAmount) {
        var isUnlocked = CharacterManager.isUnlocked(character);
        var level = StatsManager.getLevel(character);
        var upgradeCost = Globals.getTokenCost(level);
        var tokens = InventoryManager.getTokens(lastTokenType + '/' + character);
        var canLevelUp = false;
        var couldLevelUp = false;
        var characterJson;

        // need to unlock first
        if (!isUnlocked) {
            // swap upgradecost with unlock cost
            upgradeCost = CharacterManager.getUnlockCost(character) || 10;
        }

        // has enough now?
        if (tokens >= upgradeCost) {
            canLevelUp = true;
        }

        // could level up before?
        tokens -= lastTokenAmount;
        if (tokens >= upgradeCost) {
            couldLevelUp = true;
        }

        // true if can level up now but not before
        return canLevelUp && !couldLevelUp;
    };
    var makeIcon = function (character) {
        var icon = new Entity({
            name: 'icon',
            position: new Vector2(0, 0),
            components: [
                new Sprite({
                    imageName: 'gui2/characterchange/hud_' + character,
                    originRelative: new Vector2(0.5, 0.5)
                })
            ]
        });
        return icon;
    };
    var show = function (character, isUnlocked) {
        var text = 'canLevelUp';
        if (!isUnlocked) {
            text = 'canUnlock';
        }
        Notification.add(
            Localization.getText(text).replace('{NAME}', Localization.getText('name-' + character)),
            makeIcon(character)
        );
    };

    return {
        check: function (character, lastTokenType, lastTokenAmount) {
            if (shouldNotify(character, lastTokenType, lastTokenAmount)) {
                // create notification
                show(character, CharacterManager.isUnlocked(character));
            }
        },
        show: show
    };
});