/**
 * Dialog with loading spinner in center
 * @moduleName LoadingDialog
 */
bento.define('gui/loadingdialog', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'gui/dialog',
    'gui/loadingspinner'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Dialog,
    LoadingSpinner
) {
    'use strict';
    return function (settings) {
        var text = settings.text;
        var onClose = settings.onClose;
        var modal = settings.modal;
        var dialog = new Dialog({
            ignoreAnalytics: true,
            text: text,
            modal: modal,
            canClose: onClose ? true : false,
            // force a title string if there is an onClose
            title: onClose ? settings.title || '' : undefined,
            onClose: onClose,
            preventTweens: true
        });
        dialog.getComponent('windowBox', function (windowBox) {
            windowBox.attach(new LoadingSpinner({
                position: new Vector2(0, 24)
            }));
            windowBox.getComponent('shadowText', function (textObj) {
                textObj.position.y -= 24;
            });
        });
        return dialog;
    };
});