/**
 * Circles to indicate loading
 * @moduleName LoadingSpinner
 */
bento.define('gui/loadingspinner', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var radius = settings.radius || 10;
        var fadeSpeed = settings.fadeSpeed || 1 / 60;
        var ballsToSpawn = settings.balls || 8;
        var rotateBalls = settings.rotateBalls || false;
        var entity = new Entity({
            z: settings.z,
            name: 'waiting',
            position: settings.position
        });
        var balls = [];
        var makeBall = function () {
            var sprite = new Sprite({
                imageName: 'gui/chardot',
                originRelative: new Vector2(0.5, 0.5)
            });
            var behavior = {
                name: 'alphaBehavior',
                timer: 0,
                fadeSpeed: fadeSpeed,
                start: function () {
                    // count balls, set timer offset
                    var ballCount = balls.length;
                    var index = balls.indexOf(ball);
                    behavior.timer = index / ballCount;
                },
                update: function () {
                    behavior.timer += behavior.fadeSpeed;
                    behavior.timer %= 1;
                    ball.alpha = 1 - behavior.timer;
                }
            };
            var ball = new Entity({
                name: 'ball',
                position: new Vector2(0, 0),
                components: [
                    sprite,
                    behavior
                ]
            });
            balls.push(ball);
            entity.attach(ball);
            return ball;
        };
        var positionBalls = function () {
            var ballCount = balls.length;
            var radialDistance = Math.PI * 2 / ballCount;
            Utils.forEach(balls, function (ball, i, l, breakLoop) {
                var angle = -radialDistance * i;
                ball.position.x = radius * Math.cos(angle);
                ball.position.y = radius * Math.sin(angle);
                if (rotateBalls) {
                    // rotate balls themselves
                    ball.rotation = angle;
                }
            });
        };

        // spawn balls
        Utils.repeat(makeBall, ballsToSpawn);
        // position balls in circle
        positionBalls();

        return entity;
    };
});