/**
 * Combo GUI component
 * @moduleName ComboGui
 */
bento.define('gui/main/combo', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'globals',
    'managers/combo',
    'components/leftshadow',
    'modules/localization',
    'gui/main/comboorb'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Globals,
    ComboManager,
    LeftShadow,
    Localization,
    ComboOrb
) {
    'use strict';
    var cachedText = {};
    var getText = function (settings) {
        var text = settings.text;
        var color = settings.color;
        var entity = cachedText[text] || (new Text({
            position: new Vector2(16, 6),
            text: Localization.getText(text),
            font: 'font',
            fontSize: 16,
            fontColor: color,
            align: 'left',
            textBaseline: 'top',
            ySpacing: 0,
            lineWidth: 1,
            strokeStyle: '#160d32',
            innerStroke: false,
            pixelStroke: true, // workaround for Cocoon bug
        }).attach(new LeftShadow({})));

        cachedText[text] = entity;
        return entity;
    };
    var comboText;
    var getComboText = function () {
        var entity = comboText || new Text({
            text: Localization.getText('combo'),
            gradient: 'top-down',
            gradientColors: ['#35c4ff', '#35c4ff', '#FFFFFF', '#FFFFFF'],
            font: 'font',
            fontSize: 16,
            fontColor: '#ffffff',
            align: 'left',
            textBaseline: 'middle',
            ySpacing: 0,
            lineWidth: 1,
            strokeStyle: '#160d32',
            innerStroke: false,
            pixelStroke: true // workaround for Cocoon bug
        }).attach(new LeftShadow({}));
        comboText = entity;
        return entity;
    };
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();

        var player = settings.playerId || 0;

        // Grade information
        var nextGrade = 0;
        var gradeAmounts = [2, 4, 6, 8, 12];
        var gradeSettings = [
            getText({
                text: 'gradeNice',
                color: '#ffffff'
            }),
            getText({
                text: 'gradeGood',
                color: '#fff000'
            }),
            getText({
                text: 'gradeGreat',
                color: '#ffa200'
            }),
            getText({
                text: 'gradePerfect',
                color: '#ff760d'
            }),
            getText({
                text: 'gradeAwesome',
                color: '#ff366c'
            })
        ];

        var memNew = 0;
        var comboDone = false;

        // var counter = new Text({
        //     text: '12',
        //     gradient: 'top-down',
        //     gradientColors: ['#35c4ff', '#35c4ff', '#FFFFFF', '#FFFFFF'],
        //     font: 'font',
        //     fontSize: 24,
        //     fontColor: '#ffffff',
        //     align: 'left',
        //     textBaseline: 'middle',
        //     ySpacing: 0,
        //     lineWidth: 1,
        //     strokeStyle: '#160d32',
        //     innerStroke: false,
        //     pixelStroke: true // workaround for Cocoon bug
        // }).attach(new LeftShadow({}));
        var counter = new Counter({
            name: 'text',
            image: Bento.assets.getImage('gui/counters/damage'),
            frameWidth: 12,
            frameHeight: 14,
            position: new Vector2(-4, -10),
            spacing: new Vector2(-4, 0),
            align: 'left',
            animations: {
                '0': {
                    frames: [0]
                },
                '1': {
                    frames: [1]
                },
                '2': {
                    frames: [2]
                },
                '3': {
                    frames: [3]
                },
                '4': {
                    frames: [4]
                },
                '5': {
                    frames: [5]
                },
                '6': {
                    frames: [6]
                },
                '7': {
                    frames: [7]
                },
                '8': {
                    frames: [8]
                },
                '9': {
                    frames: [9]
                },
                '-': {
                    frames: [10]
                },
                'M': {
                    frames: [11]
                },
                'A': {
                    frames: [12]
                },
                'X': {
                    frames: [13]
                }
            }
        });

        var combo = getComboText();

        var grade;

        var createOrbs = function (amount) {
            var orb = new ComboOrb({
                last: amount === 1
            });
            Bento.objects.attach(orb);
            --amount;
            if (amount > 0) {
                Utils.timeout(2, function () {
                    createOrbs(amount);
                });
            }
        };

        var onComboChange = function (e) {
            memNew = e.new;
            if (e.reset && e.delta !== 0) {
                comboDone = true;

                if (nextGrade > 0) {
                    // remove previous
                    if (grade && grade.parent) {
                        grade.removeSelf();
                    }

                    //if grade has been achieved set the text and add a tween
                    // grade.setText(gradeSettings[nextGrade - 1].text, {
                    //     fontColor: gradeSettings[nextGrade - 1].color,
                    //     pixelStroke: true, // workaround for Cocoon bug
                    // });
                    grade = gradeSettings[nextGrade - 1];
                    grade.visible = true;
                    grade.scale.y = 0;
                    new Tween({
                        from: 0,
                        to: 1,
                        in: 20,
                        delay: 0,
                        ease: 'easeOutBack',
                        onUpdate: function (v, t) {
                            grade.scale.y = v;
                        },
                    });

                    new Tween({
                        from: 1.3,
                        to: 1,
                        in: 12,
                        delay: 0,
                        ease: 'easeOutQuad',
                        onStart: function () {},
                        onUpdate: function (v, t) {
                            entity.scale.x = v;
                            entity.scale.y = v;
                        },
                        onComplete: function () {

                        }
                    });

                    createOrbs(Math.min(-e.delta, 20));

                    var comboGodrays = new Entity({
                        z: Globals.Layers.GUI - 1,
                        name: 'comboGodrays',
                        position: new Vector2(48, 78),
                        float: true,
                        components: [
                            new Sprite({
                                imageName: 'gui/main/combogodrays',
                                originRelative: new Vector2(0.5, 0.5)
                            }), {
                                name: 'rotate',
                                update: function () {
                                    comboGodrays.rotation += 0.006;
                                }
                            }
                        ]
                    });
                    Bento.objects.attach(comboGodrays);

                    new Tween({
                        from: 0,
                        to: 0.5,
                        in: 10,
                        delay: 0,
                        ease: 'easeOutQuad',
                        onUpdate: function (v, t) {
                            comboGodrays.alpha = v;
                        },
                        onComplete: function () {
                            new Tween({
                                from: 0.5,
                                to: 0,
                                in: 20,
                                delay: 0,
                                ease: 'easeInQuad',
                                onUpdate: function (v, t) {
                                    comboGodrays.alpha = v;

                                },
                                onComplete: function () {
                                    comboGodrays.removeSelf();
                                }
                            });
                        }
                    });


                    entity.attach(grade);
                }

                //fade out tween
                behavior.fadeTimer = 2;
                new Tween({
                    from: 1,
                    to: 0,
                    in: 60,
                    delay: nextGrade > 0 ? 60 : 0,
                    ease: 'linear',
                    onStart: function () {
                        this.grade = grade;
                    },
                    onUpdate: function (v, t) {},
                    onComplete: function () {
                        if (this.grade) {
                            this.grade.visible = false;
                        }

                        comboDone = false;
                        if (memNew > 0) {
                            onComboChange({
                                new: memNew,
                                reset: false,
                                delta: 0
                            });
                        }
                    }
                });
                //reset grade
                nextGrade = 0;
            } else if (!e.reset && !comboDone) {
                //change counter text to new value
                counter.setValue(e.new);
                //if new combo number equals or greater then new grade threshold add and change grade text
                if (e.new >= gradeAmounts[nextGrade]) {
                    // if within length set nextgrade
                    if (nextGrade < gradeAmounts.length) {
                        nextGrade++;
                    }
                }

                //add scale tween for the counter
                new Tween({
                    from: 2.5,
                    to: 1.5,
                    'in': 10,
                    ease: Tween.EASEINBACK,
                    onUpdate: function (v, t) {
                        counter.scale.x = v;
                        counter.scale.y = v;
                    }
                });

                //if entity not visible yet scroll into the screen from the left
                if (!entity.visible) {
                    new Tween({
                        from: -64,
                        to: settings.position.x,
                        'in': 30,
                        ease: 'elastic',
                        alpha: -6,
                        beta: 1,
                        onUpdate: function (v, t) {
                            entity.position.x = v;
                        }
                    });
                    entity.visible = true;
                }
                behavior.fadeTimer = 2;
            }
        };

        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {
                entity.visible = false;
                EventSystem.on('comboChange', onComboChange);
            },
            destroy: function (data) {
                EventSystem.off('comboChange', onComboChange);
                if (grade) {
                    grade.removeSelf();
                }
                if (combo.parent) {
                    combo.removeSelf();
                }
            },
            update: function (data) {
                //set position of the combo text to match the counter text size
                combo.position.x = counter.position.x + (counter.getDigits() * 8 + 4) * counter.scale.x;

                if (behavior.fadeTimer > 0) {
                    behavior.fadeTimer -= 1 / 60;
                } else {
                    behavior.fadeTimer = 0;
                    entity.visible = false;
                }
                entity.alpha = Utils.clamp(0, behavior.fadeTimer, 1);
            },
            fadeTimer: 0,
            draw: function (data) {}
        };
        if (combo.parent) {
            combo.removeSelf();
        }
        var entity = new Entity({
            z: Globals.Layers.GUI,
            name: 'comboGui',
            position: settings.position.clone() || new Vector2(0, 0),
            float: true,
            family: ['gui'],
            components: [
                behavior,
                combo,
                counter
            ]
        });
        return entity;
    };
});