/**
 * Combo bar
 * @moduleName ComboBar
 */
bento.define('gui/main/combobar', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'globals',
    'bento/components/nineslice',
    'components/anchor',
    'managers/character',
    'modules/localization',
    'managers/save',
    'gui/shadowtext',
    'components/glow',
    'components/eventlistener',
    'managers/buff',
    'gui/taphand'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Globals,
    NineSlice,
    Anchor,
    CharacterManager,
    Localization,
    SaveManager,
    ShadowText,
    Glow,
    EventListener,
    BuffManager,
    TapHand
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var comboPoints = 0;
        var preFillPoints = 0;
        var waitingPoints = 0;
        var maxPoints = Globals.maxComboPoints;
        var comboSprite = new Sprite({
            imageName: 'gui2/hud/super_bar'
        });
        var barFullSprite = new Sprite({
            imageName: 'gui2/hud/super_bar_full',
            origin: new Vector2(5, 9),
            frameCountX: 8,
            frameCountY: 8,
            padding: 1,
            animations: {
                default: {
                    frames: [
                        0, 1, 2, 3, 4, 5, 6, 7, 8, 9,
                        10, 11, 12, 13, 14, 15, 16, 17, 18, 19,
                        20, 21, 22, 23, 24, 25, 26, 27, 28, 29,
                        30, 31, 32, 33, 34, 35, 36, 37, 38, 39,
                        40, 41, 42, 43, 44, 45, 46, 47, 48, 49,
                        50, 51, 52, 53, 54, 55, 56, 57
                    ],
                    speed: 0.56
                }
            }
        });
        // overwrite draw
        comboSprite.width = 0;
        comboSprite.setWidth = function (value) {
            comboSprite.width = value;
        };
        comboSprite.draw = function (data) {
            var entity = data.entity;
            var width = Math.max(comboSprite.width, 0);
            // draw bar full
            if (Math.floor(width) === 80) {
                barFullSprite.visible = true;
            } else {
                barFullSprite.visible = false;
            }

            if (width <= 0) {
                return;
            }
            if (!this.currentAnimation || !this.visible) {
                return;
            }

            this.updateFrame();

            data.renderer.drawImage(
                this.spriteImage,
                this.sourceX,
                this.sourceY,
                width,
                this.frameHeight,
                (-this.origin.x) | 0,
                (-this.origin.y) | 0,
                width,
                this.frameHeight
            );
        };

        var preFillSprite = new Sprite({
            imageName: ''
        });
        // overwrite draw
        preFillSprite.width = 0;
        preFillSprite.setWidth = function (value) {
            preFillSprite.width = value;
        };
        preFillSprite.draw = function (data) {
            var entity = data.entity;
            var width = Math.max(preFillSprite.width, 0);
            if (width <= 0) {
                return;
            }

            if (!this.currentAnimation || !this.visible) {
                return;
            }

            this.updateFrame();

            data.renderer.drawImage(
                this.spriteImage,
                this.sourceX,
                this.sourceY,
                width,
                this.frameHeight,
                (-this.origin.x) | 0,
                (-this.origin.y) | 0,
                width,
                this.frameHeight
            );
        };

        var glow = new Glow({
            average: 1,
            amplitude: 0,
            glowSpeed: 0
        });

        var comboEntity = new Entity({
            name: 'comboEntity',
            position: new Vector2(0, 0),
            components: [
                comboSprite,
                glow,
                barFullSprite
            ]
        });

        /**
         * Button
         */
        var lightningSettings = {
            imageName: 'gui2/hud/lightning-big',
            originRelative: new Vector2(0.5, 0.5),
            frameCountX: 8,
            frameCountY: 8,
            padding: 1,
            animations: {
                default: {
                    frames: [
                        0, 1, 2, 3, 4, 5, 6, 7,
                        8, 9, 10, 11, 12, 13, 14, 15,
                        16, 17, 18, 19, 20, 21, 22, 23,
                        24, 25, 26, 27, 28, 29, 30, 31,
                        32, 33, 34, 35, 36, 37, 38, 39,
                        40, 41, 42, 43, 44, 45, 46, 47,
                        48, 49, 50, 51, 52, 53, 54, 55,
                        56, 57
                    ],
                    "speed": 0.56
                }
            }
        };
        var lightning = new Entity({
            name: 'lightning',
            position: new Vector2(0, 0),
            components: [
                new Sprite(lightningSettings)
            ]
        });

        var disableAttackButton = function () {
            button.setActive(false);
        };
        var enableAttackButton = function () {
            button.setActive(true);
        };

        var buttonAnim = {
            up: {
                frames: [
                    0, 1, 2, 3, 4, 5, 6, 7,
                    8, 9, 10, 11, 12, 13, 14, 15,
                    16, 17, 18, 19, 20, 21, 22, 23,
                    24, 25, 26, 27, 28, 29, 30, 31,
                    32, 33, 34, 35
                ]
            },
            down: {
                frames: [0]
            }
        };
        var button = new ClickButton({
            name: 'button',
            sfx: 'sfx_button',
            imageName: 'gui2/hud/super_button_big',
            frameCountX: 6,
            frameCountY: 6,
            padding: 1,
            animations: buttonAnim,
            onClick: function () {
                Utils.forEach(Bento.objects.getByFamily('player'), function (player, i, l, breakLoop) {
                    // code here
                    if (!player.isRemote) {
                        // player.getComponent('stateMachine', function (stateMachine) {
                        //     stateMachine.setState('superAttack');
                        // });
                        player.getComponent('stateMachine', function (stateMachine) {
                            if (stateMachine.stateName === 'idle') {
                                stateMachine.setState('superAttack');
                            }
                        });
                        breakLoop();
                    }
                });
            },
            components: [
                lightning
            ]
        });
        var splash = new Entity({
            name: 'splash',
            position: new Vector2(0, 0),
            components: [
                new Sprite({
                    imageName: 'gui2/hud/splash-big',
                    originRelative: new Vector2(0.5, 0.5),
                })
            ]
        });
        // note: aligned with consumables, see gui/equipped
        var align = function () {
            var leftHanded = SaveManager.load('leftHanded');
            if (Anchor.isPortrait()) {
                if (leftHanded) {
                    superAttackButton.position.x = viewport.width - 15;
                    splash.scale.x = -1;
                } else {
                    splash.scale.x = 1;
                }

                // large button
                lightning.getComponent('sprite', function (sprite) {
                    lightningSettings.imageName = 'gui2/hud/lightning-big';
                    sprite.setup(lightningSettings);
                });
                splash.getComponent('sprite', function (sprite) {
                    sprite.setup({
                        imageName: 'gui2/hud/splash-big',
                        originRelative: new Vector2(0.5, 0.5),
                    });
                });
                button.getComponent('sprite', function (sprite) {
                    sprite.setup({
                        imageName: 'gui2/hud/super_button_big',
                        frameCountX: 6,
                        frameCountY: 6,
                        padding: 1,
                        animations: buttonAnim,
                        originRelative: new Vector2(0.5, 0.5)
                    });
                });

            } else {
                splash.scale.x = -1;

                // small button
                lightning.getComponent('sprite', function (sprite) {
                    lightningSettings.imageName = 'gui2/hud/lightning-small';
                    sprite.setup(lightningSettings);
                });
                splash.getComponent('sprite', function (sprite) {
                    sprite.setup({
                        imageName: 'gui2/hud/splash-small',
                        originRelative: new Vector2(0.5, 0.5),
                    });
                });
                button.getComponent('sprite', function (sprite) {
                    sprite.setup({
                        imageName: 'gui2/hud/super_button_small',
                        frameCountX: 6,
                        frameCountY: 6,
                        padding: 1,
                        animations: buttonAnim,
                        originRelative: new Vector2(0.5, 0.5)
                    });
                });
                if (Anchor.isWideScreen() && !Anchor.isPortrait()) {
                    lightning.position.x = Anchor.getOffset().x;
                    splash.position.x = Anchor.getOffset().x;
                    button.position.x = Anchor.getOffset().x;
                }

            }
        };
        var anchor = new Anchor({
            portrait: {
                screenPosition: new Vector2(0, 1),
                offset: new Vector2(15, -56 + 14)
            },
            landscape: {
                screenPosition: new Vector2(1, 1),
                offset: new Vector2(-15, -72)
            },
            onComplete: function () {
                align();
            }
        });
        var superAttackButton = new Entity({
            z: Globals.Layers.GUI + 1,
            name: 'superAttackButton',
            float: true,
            components: [
                splash,
                button,
                anchor,
                new EventListener({
                    name: 'leftHanded',
                    onEvent: function (data) {
                        anchor.resetPosition();
                    }
                })
            ]
        });

        var filledCheck = function () {
            if (comboPoints === maxPoints) {
                if (!superAttackButton.isAdded) {
                    //super attack tutorial
                    if (SaveManager.load('tutorialSuperAttack') === 0) {
                        SaveManager.save('tutorialSuperAttack', 2);

                        // make taphand
                        var tapHand = new TapHand({
                            position: Anchor.isPortrait() ? new Vector2(8, -8) : new Vector2(0, 0),
                            rotation: Anchor.isPortrait() ? Math.PI / 4 : -Math.PI / 4
                        }).attach(new EventListener({
                            name: 'superAttack',
                            onEvent: function () {
                                tapHand.removeSelf();
                            }
                        }));
                        // attach the hand
                        superAttackButton.attach(tapHand);
                    }

                    Bento.objects.attach(superAttackButton);
                }
            }
        };

        var onComboChange = function (e) {
            var getMultiplier = function (value) {
                if (value < 4) {
                    return 1;
                } else if (value < 6) {
                    return 1.05;
                } else if (value < 8) {
                    return 1.1;
                } else if (value < 12) {
                    return 1.2;
                }
                return 1.5;
            };
            var multiplier;

            if (e.new > 1) {
                multiplier = getMultiplier(e.new);
                preFillPoints = Math.round((e.new + BuffManager.current.comboMeterExtra) * multiplier);
                combobar.processPreFill();
            } else {
                if (e.reset) {
                    if (e.delta !== -1) {
                        preFillPoints = 0;
                        multiplier = getMultiplier(-e.delta);
                        waitingPoints += Math.round((-e.delta + BuffManager.current.comboMeterExtra) * multiplier);
                    }
                }
            }
            SaveManager.save('preComboMeter', preFillPoints);
        };

        var addWaitingPoint = function (last) {
            if (waitingPoints) {
                if (!last) {
                    combobar.add(1);
                    --waitingPoints;
                } else {
                    var points = waitingPoints;
                    waitingPoints = 0;
                    combobar.add(points);
                }
                combobar.processPreFill();
            }
        };

        var addPreFill = function () {
            if (preFillPoints !== 0) {
                var combo = {
                    reset: true,
                    delta: -preFillPoints
                };
                preFillPoints = 0;
                onComboChange(combo);
            }
        };

        var behavior = {
            name: 'behavior',
            width: 0,
            goto: 1,
            prefill: 0,
            start: function (data) {
                EventSystem.on('comboChange', onComboChange);
                EventSystem.on('addComboPoint', addWaitingPoint);
                EventSystem.on('playerDeath', disableAttackButton);
                EventSystem.on('lastPunch', disableAttackButton);
                EventSystem.on('revivedPlayer', enableAttackButton);
                EventSystem.on('respawnPlayer', enableAttackButton);
                comboPoints = SaveManager.load('comboMeter');
                preFillPoints = SaveManager.load('preComboMeter');
                combobar.processBar();
                combobar.processPreFill();
                glow.alpha = 0;
                filledCheck();
                addPreFill();
            },
            update: function () {
                behavior.width += (behavior.goto - behavior.width) * 0.5;
            },
            destroy: function (data) {
                EventSystem.off('comboChange', onComboChange);
                EventSystem.off('addComboPoint', addWaitingPoint);
                EventSystem.off('playerDeath', disableAttackButton);
                EventSystem.off('lastPunch', disableAttackButton);
                EventSystem.off('revivedPlayer', enableAttackButton);
                EventSystem.off('respawnPlayer', enableAttackButton);
                combobar.add(waitingPoints);
                superAttackButton.removeSelf();
            },
            draw: function (data) {
                comboSprite.setWidth(Math.max(behavior.width * 80, 0.01));
                preFillSprite.setWidth(Math.max(behavior.prefill * 80, 0.01));
            }
        };

        var combobar = new Entity({
            name: 'combobar',
            position: new Vector2(41, 25),
            components: [
                preFillSprite,
                comboEntity,
                behavior
            ]
        });
        combobar.extend({
            setBar: function (percent) {
                behavior.goto = percent;
            },
            processBar: function () {
                combobar.setBar(comboPoints / maxPoints);
            },
            setpreFill: function (percent) {
                behavior.prefill = percent;
            },
            processPreFill: function () {
                combobar.setpreFill(Math.min(comboPoints + preFillPoints + waitingPoints, maxPoints) / maxPoints);
            },
            add: function (amount) {
                comboPoints = Math.min(comboPoints + amount, maxPoints);
                SaveManager.save('comboMeter', comboPoints);
                combobar.processBar();
                combobar.processPreFill();
                combobar.flash();
                filledCheck();
            },
            flash: function () {
                new Tween({
                    from: 1,
                    to: 0,
                    in: 5,
                    delay: 0,
                    ease: 'linear',
                    onUpdate: function (v, t) {
                        glow.alpha = v;
                    }
                });
            },
            canSpend: function (amount) {
                if (comboPoints >= amount) {
                    return true;
                }
                // shake healthbar
                if (combobar.parent && combobar.parent.shake) {
                    combobar.parent.shake();
                    Bento.audio.stopSound('sfx_error');
                    Bento.audio.playSound('sfx_error');
                }
                return false;
            },
            deplete: function (amount) {
                if (combobar.canSpend(amount)) {
                    comboPoints -= amount;
                    SaveManager.save('comboMeter', comboPoints);
                    combobar.processBar();
                    combobar.processPreFill();
                    superAttackButton.removeSelf();
                    combobar.flash();

                    return true;
                }
                return false;
            },
            depleteAll: function () {
                comboPoints = 0;
                combobar.processBar();
                combobar.processPreFill();
                superAttackButton.removeSelf();
                SaveManager.save('comboMeter', 0);
            },
            fillUp: function (amount) {
                comboPoints = Math.min(comboPoints + amount, maxPoints);
                // preFillPoints = 0;
                // waitingPoints = 0;
                combobar.processBar();
                combobar.processPreFill();
                behavior.width = behavior.goto;
                SaveManager.save('comboMeter', comboPoints);
                filledCheck();
            }
        });

        return combobar;
    };
});