/**
 * Orb that appears when you completed a combo
 * @moduleName ComboOrb
 */
bento.define('gui/main/comboorb', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'globals'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Globals
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var sprite = new Sprite({
            imageName: 'gui/main/comboorb',
            originRelative: new Vector2(0.5, 0.5),
        });

        var gravity = -0.3;
        var highest = 0;
        var velocity = new Vector2(0, 0);
        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {
                velocity.x = Utils.getRandomRangeFloat(-1, 2);
                velocity.y = Utils.getRandomRangeFloat(3, 4);
                // new Tween({
                //     from: entity.position,
                //     to: new Vector2(82, 32),
                //     in: 15,
                //     delay: 0,
                //     ease: 'easeInQuad',
                //     onStart: function () {},
                //     onUpdate: function (v, t) {
                //         entity.position = v;
                //         if (t > 5) {
                //             entity.z = Globals.Layers.COMBOORB;
                //         }
                //     },
                //     onComplete: function () {
                // EventSystem.fire('addComboPoint');
                //         entity.removeSelf();
                //     }
                // });
                new Tween({
                    from: 1,
                    to: 1.5,
                    in: 20,
                    delay: 0,
                    ease: 'easeOutQuad',
                    onStart: function () {},
                    onUpdate: function (v, t) {

                        entity.scale.x = v;
                        entity.scale.y = v;
                    },
                    onComplete: function () {

                    }
                });
                // entity.scale.x = 2;
                // entity.scale.y = 2;
            },
            destroy: function (data) {
                if (settings.last && settings.onComplete) {
                    settings.onComplete();
                }
            },
            update: function (data) {

                // entity.removeSelf();
                // var healthBar = settings.healthBar;
                // if (!healthBar) {
                //     healthBar = Bento.objects.get('healthBar');
                // }
                // if (healthBar) {
                //     healthBar.addComboPoint();
                // }

                entity.position.addTo(velocity);

                if (entity.position.y - 16 > highest) {
                    highest = entity.position.y - 16;
                }

                velocity.y += gravity;
                if (velocity.y <= 0) {
                    entity.z = Globals.Layers.COMBOORB;
                    var magnitude = 1 - Math.max(entity.position.y - 16, 0) / highest;
                    magnitude *= magnitude;
                    magnitude *= magnitude;
                    velocity.x *= 1 - magnitude;
                    entity.position.x += (82 - entity.position.x) * magnitude;
                }
                if (entity.position.y <= 32) {
                    EventSystem.fire('addComboPoint', settings.last);
                    entity.removeSelf();
                }
            },
            draw: function (data) {}
        };
        var entity = new Entity({
            z: Globals.Layers.GUI - 2,
            name: 'comboOrb',
            family: ['comboOrb'],
            float: true,
            position: settings.position || new Vector2(48, 78),
            updateWhenPaused: 0,
            components: [
                sprite,
                behavior
            ]
        });
        return entity;
    };
});