/**
 * Allies select menu
 * @moduleName AlliesMenu
 */
bento.define('gui/menus/allies', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'globals',
    'components/modal',
    'gui/backscroller',
    'components/anchor',
    'gui/scrollinglist',
    'gui/worldselect/topbar',
    'managers/save',
    'gui/tokenbar',
    'gui/iconbutton',
    'gui/shadowtext',
    'modules/localization',
    'bento/components/nineslice',
    'managers/allies',
    'bento/gui/togglebutton',
    'components/glow',
    'gui/centeredlist',
    'bento/maskedcontainer',
    'managers/inventory',
    'modules/analytics',
    'gui/tokendialog',
    'modules/tenjin',
    'gui/dialog',
    'managers/currency',
    'gui/iap/buycoinspopup',
    'components/blink',
    'gui/sparkleburst',
    'components/eventlistener',
    'managers/buff',
    'gui/tier',
    'gui/badge',
    'components/masksprite',
    'components/viewanalytics',
    'gui/senseipopup',
    'components/backbutton'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Globals,
    Modal,
    BackScroller,
    Anchor,
    ScrollingList,
    Topbar,
    SaveManager,
    TokenBar,
    IconButton,
    ShadowText,
    Localization,
    NineSlice,
    AlliesManager,
    ToggleButton,
    Glow,
    CenteredList,
    MaskedContainer,
    InventoryManager,
    Analytics,
    TokenDialog,
    Tenjin,
    Dialog,
    CurrencyManager,
    BuyCoinsPopup,
    Blink,
    SparkleBurst,
    EventListener,
    BuffManager,
    TierStars,
    Badge,
    MaskSprite,
    ViewAnalytics,
    SenseiPopup,
    BackButton
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var index = settings.index || 0;
        /**
         * Background
         */
        var backgroundFists = new BackScroller({
            imageName: 'gui2/misc/fist_bg',
            velocity: new Vector2(-0.3, 0.3),
            spacing: new Vector2(-1, -1),
            padding: 64
        }).attach(new Anchor({
            onComplete: function () {
                if (Anchor.isWideScreen()) {
                    backgroundFists.transform.x -= Anchor.getOffset().x;
                    backgroundFists.transform.y -= Anchor.getOffset().y;
                }
            }
        }));
        var backgroundFistsLandscape = new Entity({
            name: 'backgroundFistsLandscape',
            components: [
                new Sprite({
                    imageName: 'gui2/misc/fist_bg2',
                    originRelative: new Vector2(1, 0.5),
                }),
                new Sprite({
                    // left padding
                    imageName: 'gui2/misc/fist_bg2',
                    originRelative: new Vector2(2, 0.5),
                }),
                new Anchor({
                    screenPosition: new Vector2(0.38, 0.5),
                    offset: new Vector2(3, 0),
                    onComplete: function () {
                        if (Anchor.isPortrait()) {
                            backgroundFistsLandscape.visible = false;
                        } else {
                            backgroundFistsLandscape.visible = true;
                        }
                        if (!Anchor.isPortrait() && Anchor.isWideScreen()) {
                            backgroundFistsLandscape.position.x -= 12;
                        }
                    }
                })
            ]
        });
        /**
         * Separator
         */
        var separatorPortrait = 0.52;
        var separatorSprite = new Sprite({
            imageName: 'gui2/misc/separator',
            originRelative: new Vector2(0.5, 0.5),
            frameCountY: 2,
            animations: {
                default: {
                    frames: [0]
                },
                wide: {
                    frames: [1]
                }
            }
        });
        var separator = new Entity({
            name: 'separator',
            components: [
                separatorSprite,
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.5, separatorPortrait),
                        offset: new Vector2(0, -12)
                    },
                    landscape: {
                        screenPosition: new Vector2(0.38, 0.5),
                        offset: new Vector2(4, 0)
                    },
                    onComplete: function () {
                        if (Anchor.isPortrait()) {
                            separator.rotation = 0;
                            separatorSprite.setAnimation('default');
                            separator.scale.x = 1;
                            separator.scale.y = 1;
                        } else {
                            separator.rotation = Math.PI / 2;
                            separatorSprite.setAnimation('wide');
                            separator.scale.y = 0.05;
                            separator.scale.x = 50;
                        }
                        if (!Anchor.isPortrait() && Anchor.isWideScreen()) {
                            separator.position.x -= 8;
                            separator.scale.y = 1;
                            separator.scale.x = 1;
                        }
                    }
                })
            ]
        });
        /**
         * Bottom character select
         */
        var characterButtons = [];
        var scrollingList = new ScrollingList({
            damping: 0.9,
            snapSpeed: 0.1,
            snapMinSlideSpeed: 1,
            spacing: 1,
            snap: true,
            direction: 'x',
            components: [
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.5, 0.70)
                    },
                    landscape: {
                        screenPosition: new Vector2(0.7, 0.6),
                        offset: new Vector2(0, 12)
                    }
                }),
                new EventListener({
                    name: 'preventScroll',
                    onEvent: function () {
                        scrollingList.cancel();
                    }
                })
            ],
            onSnap: function (item, idx) {
                dotList.iterate(function (dot, i) {
                    if (i === idx) {
                        dot.getComponent('sprite', function (sprite) {
                            sprite.setAnimation('down');
                        });
                    } else {
                        dot.getComponent('sprite', function (sprite) {
                            sprite.setAnimation('up');
                        });
                    }
                });
            }
        });
        var dotList = new CenteredList({
            spacing: 3,
            components: [
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.5, 1),
                        offset: new Vector2(0, -34)
                    },
                    landscape: {
                        screenPosition: new Vector2(0.7, 1),
                        offset: new Vector2(0, 24)
                    }
                })
            ]
        });
        /**
         * Wall
         */
        var wall = new Entity({
            name: 'wall',
            components: [
                new Sprite({
                    imageName: 'gui2/iap/tiling_wall',
                    originRelative: new Vector2(0.5, 0),
                }),
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.5, separatorPortrait),
                        offset: new Vector2(0, -12)
                    },
                    landscape: {
                        screenPosition: new Vector2(0.75, 0),
                        offset: new Vector2(4, 0)
                    }
                })
            ]
        });
        /**
         * Tokens needed
         */
        var allies = AlliesManager.getAllies();
        var currentCharacter = allies[index] || 'rat';
        var originalCharacter = currentCharacter;
        var getTokenSprite = function () {
            var tokenJson = Bento.assets.getJson('enemytokens/' + currentCharacter);
            var sprite = new Sprite({
                spriteSheet: tokenJson.inventory.image
            });
            sprite.setOriginRelative(new Vector2(0.5, 0.5));
            return sprite;
        };
        var getTokenTier = function () {
            var tokenJson = Bento.assets.getJson('enemytokens/' + currentCharacter);
            return tokenJson.inventory.tier;
        };
        var getEnemySprite = function () {
            var tokenJson = Bento.assets.getJson('enemytokens/' + currentCharacter);
            var sprite = tokenJson.sprite;
            return sprite;
        };
        var getEnemyName = function () {
            return Localization.getText('enemytokens')[currentCharacter].charName;
        };
        var getEnemyDescription = function () {
            var description = Localization.getText('enemytokens')[currentCharacter].desc;
            // replace number
            var effect = AlliesManager.getEffectiveness(currentCharacter, description);
            return effect.string;
        };
        var tokenSprite = getTokenSprite();
        var tokenTier;
        var tokenIcon = new Entity({
            name: 'tokenIcon',
            position: new Vector2(-20, 0),
            components: [
                tokenSprite
            ]
        });
        var tokenBar = new TokenBar({
            position: new Vector2(14, 0),
            character: currentCharacter,
            type: 'enemytokens'
        });
        var tokensRequired = new Entity({
            name: 'tokensRequired',
            position: new Vector2(48, -36),
            components: [
                tokenBar,
                tokenIcon
            ]
        });
        /**
         * Buttons
         */
        var priceText = new Text({
            position: new Vector2(-1, 8),
            text: '',
            font: 'font',
            fontSize: 12,
            fontColor: '#ffdf00',
            align: 'center',
            textBaseline: 'middle',
            lineWidth: 1,
            strokeStyle: '#3e1a00',
            pixelStroke: true,
            maxWidth: 22,
            maxHeight: 12
        });
        var priceEntity = new Entity({
            name: 'price',
            position: new Vector2(-32, -2),
            components: [
                new Sprite({
                    imageName: 'gui/coin2',
                    originRelative: new Vector2(0.5, 0.5)
                }),
                priceText
            ]
        });
        var buttonUpgrade = new IconButton({
            name: 'buttonUpgrade',
            sfx: 'sfx_button',
            imageName: 'gui2/misc/button_large_primary',
            icon: new Entity({
                name: 'upgradeContainer',
                components: [
                    new ShadowText({
                        position: new Vector2(14, -1),
                        text: Localization.getText('levelUp'),
                        font: 'font',
                        fontSize: 18,
                        fontColor: '#ffffff',
                        align: 'center',
                        textBaseline: 'middle',
                        maxWidth: 58,
                        maxHeight: 18
                    }),
                    priceEntity
                ]
            }),
            frameCountX: 1,
            frameCountY: 3,
            position: new Vector2(0, 0),
            float: false,
            updateWhenPaused: 0,
            onButtonDown: function () {
                EventSystem.fire('preventScroll');
            },
            onClick: function () {
                // does the player have enough coins?
                var char = currentCharacter;
                var currentLevel = AlliesManager.getLevel(char);
                var cost = AlliesManager.getUpgradeCostCoins(char);

                var levelUp = function () {
                    //remove upgrade items
                    var upgradeCost = AlliesManager.getUpgradeCost(currentCharacter);
                    InventoryManager.spendTokens('enemytokens/' + char, upgradeCost);

                    // level up and update stats
                    AlliesManager.addLevel(char);

                    // update
                    selectCharacter(currentCharacter);

                    EventSystem.fire('quest-upgradeAlly', {
                        count: 1
                    });

                    Analytics.event('UpgradeEnemyTotal', char);
                    Analytics.event('UpgradeEnemy', char + 'level_' + (currentLevel + 1));

                    Tenjin.sendEvent('upgradeEnemy_' + char);
                    Tenjin.sendEventAndValue('upgradeEnemy__' + char + '_level', (currentLevel + 1));
                };

                if (!AlliesManager.isUnlocked(char)) {
                    console.log('Character was not unlocked yet');
                    return;
                }

                // TEMP: simple solution to max level, should make this better in the future
                if (currentLevel >= AlliesManager.getMaxLevel(currentCharacter)) {
                    Bento.objects.attach(new Dialog({
                        name: 'maxReached',
                        text: Localization.getText('maxLevel'),
                        type: 'yes'
                    }));
                    return;
                }

                if (hasEnoughTokens(char)) {
                    if (CurrencyManager.canSpend(cost)) {
                        characterEntity.attach(new Blink({}));
                        characterEntity.attach(new SparkleBurst({}));
                        buttonUpgrade.attach(new Blink({}));
                        tokenIcon.attach(new Blink({}));
                        tokenBar.attach(new Blink({}));
                        Bento.audio.stopSound('sfx_consumable');
                        Bento.audio.playSound('sfx_consumable');

                        CurrencyManager.spendCoins(cost, 'upgrade-' + char);
                        levelUp();
                    } else {
                        // ask to buy missing coins
                        var buyCoins = new BuyCoinsPopup({
                            z: Globals.WorldSelectLayers.UPGRADE + 1,
                            coinsToBuy: cost - CurrencyManager.getCoins(),
                            buttonToClick: buttonUpgrade
                        });
                        Bento.objects.attach(buyCoins);
                    }
                } else {
                    Bento.objects.attach(new TokenDialog({
                        character: currentCharacter,
                        tokenType: 'enemytokens'
                    }));
                }
            }
        });
        var buttonUnlock = new IconButton({
            name: 'buttonUnlock',
            sfx: 'sfx_button',
            imageName: 'gui2/misc/button_large_primary',
            icon: new Entity({
                name: 'upgradeContainer',
                components: [
                    new ShadowText({
                        position: new Vector2(14, 0),
                        text: Localization.getText('unlock'),
                        font: 'font',
                        fontSize: 18,
                        fontColor: '#ffffff',
                        align: 'center',
                        textBaseline: 'middle',
                        maxWidth: 58,
                        maxHeight: 18
                    }),
                    new Entity({
                        name: 'unlockIcon',
                        position: new Vector2(-32, 0),
                        components: [
                            new Sprite({
                                imageName: 'gui2/misc/button_icon_lock',
                                originRelative: new Vector2(0.5, 0.5)
                            })
                        ]
                    })
                ]
            }),
            frameCountX: 1,
            frameCountY: 3,
            onButtonDown: function () {
                EventSystem.fire('preventScroll');
            },
            onClick: function () {
                var getCharacterCost = function () {
                    return AlliesManager.getUnlockCost(currentCharacter);
                };
                var getCharacterName = function () {
                    var name = Localization.getText('name-' + currentCharacter);
                    return name;
                };
                var tokenCost = getCharacterCost();
                var tokens = InventoryManager.getTokens('playertokens/' + currentCharacter);

                var unlockedMissions = SaveManager.load('unlockedLevels');
                var showPopup = function () {
                    // TODO: show unlock animation
                    characterEntity.attach(new Blink({}));
                    characterEntity.attach(new SparkleBurst({}));
                    buttonUpgrade.attach(new Blink({}));
                    tokenIcon.attach(new Blink({}));
                    tokenBar.attach(new Blink({}));
                    Bento.audio.stopSound('sfx_consumable');
                    Bento.audio.playSound('sfx_consumable');
                };

                // enough tokens?
                if (hasEnoughTokens(currentCharacter)) {
                    // spend the tokens during unlock?
                    InventoryManager.spendTokens('enemytokens/' + currentCharacter, tokenCost);

                    // unlock the actual character
                    AlliesManager.unlock(currentCharacter);

                    // update
                    selectCharacter(currentCharacter);

                    // remove the lock
                    Utils.forEach(characterButtons, function (button) {
                        if (button.name === 'button-' + currentCharacter) {
                            // show character
                            button.unlock();
                            button.attach(new Blink({}));
                        }
                    });

                    Analytics.event('UnlockFigurine', currentCharacter, unlockedMissions.length);

                    showPopup();
                } else {
                    Bento.objects.attach(new TokenDialog({
                        character: currentCharacter,
                        tokenType: 'enemytokens'
                    }));
                }
            }
        });
        var buttonContainer = new Entity({
            name: 'buttonContainer',
            components: [
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.5, 0.45),
                        offset: new Vector2(0, 0)
                    },
                    landscape: {
                        screenPosition: new Vector2(0.8, 0.2),
                        offset: new Vector2(0, 4)
                    },
                    onComplete: function () {
                        if (Anchor.isPortrait()) {
                            tokensRequired.position = new Vector2(48, -36);
                        } else {
                            tokensRequired.position = new Vector2(-88, 0);
                        }
                    }
                }),
                tokensRequired,
                buttonUpgrade
            ]
        });
        var buttonBack = new ClickButton({
            name: 'buttonBack',
            sfx: 'sfx_button',
            imageName: 'gui2/bottom_tabs/bottom_tabs__ok',
            frameCountX: 2,
            frameCountY: 1,
            components: [
                new BackButton({
                    removeSelf: true,
                    onPressed: function (data) {
                        data.entity.doCallback();
                    }
                }),
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.5, 1),
                        offset: new Vector2(-48 + (Anchor.isWideScreen() ? 4 : 0), -12 + (Anchor.isWideScreen() ? Anchor.getOffset().y : 0))
                    },
                    landscape: {
                        screenPosition: new Vector2(0.25, 1),
                        offset: new Vector2(-44, -12)
                    }
                })
            ],
            onButtonDown: function () {
                EventSystem.fire('preventScroll');
            },
            onClick: function () {
                entity.removeSelf();

                if (settings.onClose) {
                    settings.onClose(currentCharacter !== originalCharacter);
                }
            }
        });
        /**
         * Name
         */
        var nineSlice = new NineSlice({
            imageName: 'gui2/misc/splash_background_9slice',
            width: 112,
            height: 112,
            originRelative: new Vector2(0.5, 0.5)
        });
        var nameOffset = -28;
        var nameText = new Text({
            z: 0,
            position: new Vector2(0, nameOffset),
            text: getEnemyName(),
            font: 'font',
            fontSize: 18,
            fontColor: '#ffdf00',
            shadowColor: '#23243a',
            align: 'center',
            textBaseline: 'bottom',
            maxWidth: 96,
            maxHeight: 18
        });
        var levelText = new Text({
            text: Localization.getText('level'),
            font: 'font',
            fontSize: 10,
            fontColor: '#23243a',
            align: 'right',
            textBaseline: 'top',
            position: new Vector2(5, nameOffset),
            maxWidth: 24,
            maxHeight: 10
        });
        var descriptionText = new Text({
            z: 0,
            position: new Vector2(0, nameOffset + 14 + 26),
            text: getEnemyDescription(),
            font: 'font',
            fontSize: 12,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            maxWidth: 96,
            maxHeight: 52
        });
        var levelNumberText = new Text({
            text: AlliesManager.getLevel(currentCharacter).toString(),
            font: 'font',
            fontSize: 10,
            fontColor: '#ffdf00',
            align: 'left',
            textBaseline: 'top',
            position: new Vector2(9, nameOffset)
        });
        var nameContainer = new Entity({
            name: 'nameContainer',
            components: [
                nineSlice,
                nameText,
                levelText,
                levelNumberText,
                descriptionText,
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.3, 0.25),
                        offset: new Vector2(0, -8)
                    },
                    landscape: {
                        screenPosition: new Vector2(0.2, 0.4),
                        offset: new Vector2(0, 0)
                    }
                })
            ]
        });
        /**
         * Character sprite
         */
        var characterSprite = new Sprite({});
        var characterShadowSprite = new Sprite({});
        var characterFigurineBottom = new Sprite({
            imageName: 'gui2/misc/figurine-bottom',
            originRelative: new Vector2(0.5, 0.5),
        });
        var characterEntity = new Entity({
            name: 'character',
            scale: new Vector2(-1, 1),
            components: [
                characterShadowSprite,
                characterFigurineBottom,
                characterSprite,
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.8, 0.28)
                    },
                    landscape: {
                        screenPosition: new Vector2(0.3, 0.95)
                    }
                })
            ]
        });
        /**
         * Other components
         */
        var modal = new Modal({});
        var behavior = {
            name: 'behaviorComponent',
            start: function () {
                // make pages for scrolling list
                var availableCharacters = AlliesManager.getAvailable();
                var currentPage = 0;
                var scrollToPage = 0;
                var pages = {};
                var makeButton = function (character, idx, page) {
                    var have = InventoryManager.getTokens('enemytokens/' + character);
                    var needed = AlliesManager.getUnlockCost(character);
                    var tokenText = new ShadowText({
                        position: new Vector2(-19, -36),
                        text: have + '/' + needed,
                        font: 'font',
                        fontSize: 16,
                        fontColor: have >= needed ? '#fffd00' : '#9442b3',
                        shadowColor: '#000',
                        align: 'center',
                        textBaseline: 'middle'
                    });
                    // var tokenText = new TokenBar({
                    //     position: new Vector2(-19, -32),
                    //     character: character,
                    //     type: 'enemytokens'
                    // });
                    var lockSprite = new Sprite({
                        imageName: 'gui2/characterchange/portrait_frame_lock',
                        frameCountX: 4,
                        frameCountY: 4,
                        originRelative: new Vector2(0.6, 0.6),
                        animations: {
                            up: {
                                frames: [0]
                            },
                            down: {
                                frames: [0]
                            }
                        }
                    });
                    var lock = new Entity({
                        name: 'lock',
                        position: new Vector2(19, 18),
                        components: [
                            lockSprite, //
                            {
                                name: 'setAnimation',
                                update: function () {
                                    lockSprite.setAnimation(button.getComponent('sprite').getAnimationName());
                                }
                            },
                            tokenText
                        ]
                    });
                    var characterEntityBox = new Entity({
                        name: 'character',
                        position: new Vector2(0, 24),
                        components: [
                            new Sprite({
                                spriteSheet: 'allies/' + character,
                            })
                        ]
                    });
                    var shadowSprite = new Sprite({
                        spriteSheet: 'allies/' + character + '-shadow',
                    });
                    var characterEntityBoxShadow = new Entity({
                        name: 'character',
                        position: new Vector2(0, 24),
                        boundingBox: new Rectangle(-26, -24, 52, 48),
                        components: [
                            shadowSprite,
                            new MaskSprite({
                                boundingBox: new Rectangle(-26, -48, 52, 48)
                            })
                        ]
                    });
                    var buttonBehavior = {
                        update: function () {
                            // is in active page?
                            if (scrollingList.getSelection() === page) {
                                if (!button.isActive()) {
                                    button.setActive(true);
                                }
                                // can upgrade?
                                if (badge.parent) {
                                    badge.visible = hasEnoughTokens(character);
                                }
                            } else {
                                if (button.isActive()) {
                                    button.setActive(false);
                                }
                            }
                        }
                    };
                    var badge = new Badge({
                        pulsate: true,
                        exclamation: true,
                        imageName: 'gui2/misc/badge',
                        position: new Vector2(16, -16),
                    });
                    var button = new ToggleButton({
                        name: 'button-' + character,
                        sfx: 'sfx_button',
                        imageName: 'gui2/characterchange/panel',
                        frameCountX: 2,
                        frameCountY: 1,
                        toggled: character === currentCharacter,
                        position: new Vector2(
                            (idx % 3) * 58 - 58,
                            Math.floor(idx / 3) * 58
                        ),
                        onToggle: function () {
                            if (this.isToggled()) {
                                // untoggle others
                                Utils.forEach(characterButtons, function (other) {
                                    if (other !== button) {
                                        other.toggle(false);
                                    } else {
                                        // select character
                                        selectCharacter(character);

                                        SaveManager.save('allies', allies);
                                    }
                                });
                            } else {
                                // re-toggle self
                                button.toggle(true);
                            }
                            new Tween({
                                from: 0.9,
                                to: 1,
                                in: 30,
                                ease: 'elastic',
                                decay: 5,
                                oscillations: 2,
                                onUpdate: function (v, t) {
                                    button.scale.x = v;
                                    button.scale.y = v;
                                }
                            });
                        },
                        components: [
                            characterEntityBoxShadow,
                            characterEntityBox,
                            buttonBehavior
                        ]
                    });
                    characterButtons.push(button);

                    // unlock
                    button.extend({
                        unlock: function () {
                            button.removeByName('lock');
                            characterEntityBox.visible = true;
                        }
                    });

                    // add lock
                    if (!AlliesManager.isUnlocked(character)) {
                        characterEntityBox.visible = false;
                        button.attach(lock);
                        if (have >= needed) {
                            //lock.attach(new Glow({
                            //    average: 0.5
                            //}));

                            lock.getComponent('sprite', function (sprite) {
                                sprite.setup({
                                    imageName: 'gui2/characterchange/portrait_frame_lock',
                                    frameCountX: 4,
                                    frameCountY: 4,
                                    padding: 1,
                                    originRelative: new Vector2(0.6, 0.6),
                                    animations: {
                                        up: {
                                            speed: 0.24,
                                            frames: [1, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15]
                                        },
                                        down: {
                                            frames: [1]
                                        },
                                        inactive: {
                                            frames: [1]
                                        }
                                    }
                                });
                            });

                        }
                        // when lock is removed, attach the badge
                        lock.attach({
                            name: 'addBadge',
                            destroy: function () {
                                button.attach(badge);
                            }
                        });
                    } else {
                        button.attach(badge);
                    }

                    // cache current page to scroll to later
                    if (character === currentCharacter) {
                        scrollToPage = currentPage;
                    }

                    return button;
                };

                Utils.forEach(availableCharacters, function (character, i) {
                    var button;
                    var idx = i % 6;
                    var makePage = function () {
                        pages[currentPage] = new Entity({
                            position: new Vector2(0, -58 / 2),
                            dimension: new Rectangle(0, 0, 192, 80)
                        });
                        scrollingList.addItem(pages[currentPage]);
                        // add dot
                        var dotSprite = new Sprite({
                            frameCountX: 2,
                            originRelative: new Vector2(0.5, 0.5),
                            imageName: 'gui2/misc/pagination_dots',
                            animations: {
                                'up': {
                                    frames: [0]
                                },
                                'down': {
                                    frames: [1]
                                }
                            }
                        });
                        dotList.addItem(new Entity({
                            name: 'dot',
                            components: [
                                dotSprite
                            ]
                        }));
                        if (i === 0) {
                            dotSprite.setAnimation('down');
                        }
                    };
                    currentPage = Math.floor(i / 6);
                    if (!pages[currentPage]) {
                        makePage();
                    }
                    button = makeButton(character, idx, currentPage);
                    pages[currentPage].attach(button);
                    // reverse drawing order
                    pages[currentPage].moveComponentTo(button, 0);
                });

                selectCharacter(currentCharacter);
                scrollingList.snapTo(scrollToPage);

                characterShadowSprite.name = 'shadowSprite';

                if (SaveManager.load('tutorialAllies') === 0) {
                    Bento.objects.attach(new SenseiPopup({
                        darken: true,
                        modal: true,
                        onNoMoreText: function () {
                            this.dismiss();
                        },
                        onComplete: function () {
                            SaveManager.save('tutorialAllies', 1);
                            EventSystem.fire('tutorialAllies');
                        },
                        text: Localization.getText('tutorial')['enemyTokens']
                    }));
                }
            },
            destroy: function (data) {},
            update: function (data) {},
            draw: function (data) {}
        };
        var selectCharacter = function (character) {
            var previouslySelected;
            currentCharacter = character;
            characterSprite.setSpriteSheet('allies/' + character);
            characterShadowSprite.setSpriteSheet('allies/' + character + '-shadow');
            characterShadowSprite.origin.x = characterSprite.origin.x + 4;
            characterShadowSprite.origin.y = characterSprite.origin.y - 3;
            // characterShadowSprite.origin.y -= 3;
            // 
            if (AlliesManager.isUnlocked(currentCharacter)) {
                characterSprite.visible = true;
                characterFigurineBottom.setup({
                    imageName: 'gui2/misc/figurine-bottom',
                    originRelative: new Vector2(0.5, 0.5),
                });
                // save selected character if it was unlocked
                previouslySelected = allies[index];
                allies[index] = currentCharacter;
                EventSystem.fire('updateAlly' + index, currentCharacter);

                // if this ally was already selected in the other slot, we do a swap
                var otherIndex = (index + 1) % allies.length;
                if (allies[index] === allies[otherIndex]) {
                    allies[otherIndex] = previouslySelected;
                    EventSystem.fire('updateAlly' + otherIndex, previouslySelected);
                }
                SaveManager.save('allies', allies);
                // tween ok button
                new Tween({
                    from: 0.5,
                    to: 1,
                    in: 30,
                    delay: 0,
                    ease: 'elastic',
                    decay: 5,
                    oscillations: 2,
                    onUpdate: function (v, t) {
                        buttonBack.scale.x = v;
                        buttonBack.scale.y = v;
                    },
                    onComplete: function () {

                    }
                });
                // reload buffs
                BuffManager.loadBuffs();
            } else {
                characterSprite.visible = false;
                characterFigurineBottom.setup({
                    imageName: 'gui2/misc/figurine-shadow',
                    originRelative: new Vector2(0.5, 0.5),
                });
            }

            // set buttons, tokens, price
            tokenIcon.remove(tokenSprite);
            tokenSprite = getTokenSprite();
            tokenIcon.attach(tokenSprite);
            tokenIcon.remove(tokenTier);
            tokenTier = new TierStars({
                position: new Vector2(-12, -10),
                alignment: 'left',
                tier: getTokenTier()
            });
            tokenIcon.attach(tokenTier);
            priceText.setText(AlliesManager.getUpgradeCostCoins(character).toString());
            tokenBar.setCharacter(currentCharacter);
            // set text
            nameText.setText(getEnemyName());
            levelNumberText.setText(AlliesManager.getLevel(currentCharacter).toString());
            descriptionText.setText(getEnemyDescription());

            setButton();
        };
        var canUnlock = function (character) {
            // required amount of tokens?
            var cost = AlliesManager.getUnlockCost(character);
            var tokens = AlliesManager.getTokens('enemytokens/' + character);

            if (tokens >= cost) {
                return true;
            } else {
                return false;
            }
        };
        var hasEnoughTokens = function (character) {
            var isUnlocked = AlliesManager.isUnlocked(character);
            var level = AlliesManager.getLevel(character);
            var upgradeCost = AlliesManager.getUpgradeCost(character);
            var tokens = InventoryManager.getTokens('enemytokens/' + character);

            if (!isUnlocked) {
                // swap upgradecost with unlock cost
                upgradeCost = AlliesManager.getUnlockCost(character);
            }

            if (tokens >= upgradeCost) {
                return true;
            } else {
                return false;
            }
        };
        /**
         * Set button to upgrade or unlock
         */
        var setButton = function () {
            if (buttonUpgrade.parent) {
                buttonUpgrade.removeSelf();
            }
            if (buttonUnlock.parent) {
                buttonUnlock.removeSelf();
            }

            if (AlliesManager.isUnlocked(currentCharacter)) {
                buttonContainer.attach(buttonUpgrade);
                // can upgrade?
                if (hasEnoughTokens(currentCharacter)) {

                    // set button to excited
                    setExcited(buttonUpgrade, true);
                } else {
                    // set button to normal
                    setExcited(buttonUpgrade, false);
                }
            } else {
                buttonContainer.attach(buttonUnlock);
                // can unlock?
                if (hasEnoughTokens(currentCharacter)) {
                    setExcited(buttonUnlock, true);
                } else {
                    setExcited(buttonUnlock, false);
                }
            }
        };
        var setExcited = function (button, excited) {
            button.getComponent('sprite', function (sprite) {
                if (excited) {
                    sprite.setup({
                        imageName: 'gui2/misc/button_large_primary_excited',
                        frameCountX: 6,
                        frameCountY: 7,
                        padding: 1,
                        animations: {
                            up: {
                                speed: 0.34,
                                frames: [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35]
                            },
                            down: {
                                frames: [37]
                            },
                            inactive: {
                                frames: [38]
                            }
                        }
                    });
                } else {
                    sprite.setup({
                        imageName: 'gui2/misc/button_large_primary_excited',
                        frameCountX: 6,
                        frameCountY: 7,
                        padding: 1,
                        animations: {
                            up: {
                                speed: 0.34,
                                frames: [36]
                            },
                            down: {
                                frames: [37]
                            },
                            inactive: {
                                frames: [38]
                            }
                        }
                    });
                }
                sprite.setAnimation('up');
            });

        };
        /**
         * Main entity
         */
        var entity = new Entity({
            z: settings.z || Globals.WorldSelectLayers.UPGRADE,
            name: 'characterSelect',
            float: true,
            components: [
                modal,
                backgroundFists,
                wall,
                separator,
                scrollingList,
                backgroundFistsLandscape,
                nameContainer,
                characterEntity,
                buttonContainer,
                buttonBack,
                behavior,
                new Topbar({}),
                dotList,
                new ViewAnalytics({
                    name: 'alliesScreen',
                    ignoreKey: 'tutorialWorldSelect'
                })
            ]
        });

        return entity;
    };
});