/**
 * Character select menu
 * @moduleName CharacterSelectMenu
 */
bento.define('gui/menus/characterselect', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'components/modal',
    'gui/backscroller',
    'components/anchor',
    'managers/character',
    'gui/upgrade/statsbg',
    'gui/upgrade/stats',
    'gui/scrollinglist',
    'gui/centeredlist',
    'bento/gui/togglebutton',
    'gui/iconbutton',
    'modules/localization',
    'globals',
    'gui/shadowtext',
    'bento/components/nineslice',
    'gui/worldselect/topbar',
    'managers/stats',
    'managers/currency',
    'managers/inventory',
    'gui/tokenbar',
    'modules/analytics',
    'modules/tenjin',
    'managers/save',
    'gui/dialog',
    'gui/iap/buycoinspopup',
    'gui/unlockcharacterpopup',
    'gui/menus/iap',
    'components/glow',
    'gui/tokendialog',
    'components/eventlistener',
    'gui/badge',
    'components/masksprite',
    'gui/dailyquest/progressbar',
    'components/blink',
    'gui/sparkleburst',
    'gui/taphand',
    'components/backbutton',
    'modules/firebase'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Modal,
    BackScroller,
    Anchor,
    CharacterManager,
    StatsBg,
    Stats,
    ScrollingList,
    CenteredList,
    ToggleButton,
    IconButton,
    Localization,
    Globals,
    ShadowText,
    NineSlice,
    Topbar,
    StatsManager,
    CurrencyManager,
    InventoryManager,
    TokenBar,
    Analytics,
    Tenjin,
    SaveManager,
    Dialog,
    BuyCoinsPopup,
    UnlockCharacterPopup,
    IAPMenu,
    Glow,
    TokenDialog,
    EventListener,
    Badge,
    MaskSprite,
    ProgressBar,
    Blink,
    SparkleBurst,
    TapHand,
    BackButton,
    FireBase
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var tutorialCharacter = SaveManager.load('tutorialCharacter');
        /**
         * Background
         */
        var backgroundFists = new BackScroller({
            imageName: 'gui2/misc/fist_bg',
            velocity: new Vector2(-0.3, 0.3),
            spacing: new Vector2(-1, -1)
        }).attach(new Anchor({
            onComplete: function () {
                if (Anchor.isWideScreen()) {
                    backgroundFists.transform.x -= Anchor.getOffset().x;
                    backgroundFists.transform.y -= Anchor.getOffset().y;
                }
            }
        }));
        var backgroundFistsLandscape = new Entity({
            name: 'backgroundFistsLandscape',
            components: [
                new Sprite({
                    imageName: 'gui2/misc/fist_bg2',
                    originRelative: new Vector2(1, 0.5),
                }),
                new Sprite({
                    // left padding
                    imageName: 'gui2/misc/fist_bg2',
                    originRelative: new Vector2(2, 0.5),
                }),
                new Anchor({
                    screenPosition: new Vector2(0.38, 0.5),
                    offset: new Vector2(3, 0),
                    onComplete: function () {
                        if (Anchor.isPortrait()) {
                            backgroundFistsLandscape.visible = false;
                        } else {
                            backgroundFistsLandscape.visible = true;
                        }
                        if (!Anchor.isPortrait() && Anchor.isWideScreen()) {
                            backgroundFistsLandscape.position.x -= 12;
                        }
                    }
                })
            ]
        });
        /**
         * Separator
         */
        var separatorPortrait = 0.7;
        var separatorSprite = new Sprite({
            imageName: 'gui2/misc/separator',
            originRelative: new Vector2(0.5, 0.5),
            frameCountY: 2,
            animations: {
                default: {
                    frames: [0]
                },
                wide: {
                    frames: [1]
                }
            }
        });
        var separator = new Entity({
            name: 'separator',
            components: [
                separatorSprite,
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.5, separatorPortrait),
                        offset: new Vector2(0, -12)
                    },
                    landscape: {
                        screenPosition: new Vector2(0.38, 0.5),
                        offset: new Vector2(4, 0)
                    },
                    onComplete: function () {
                        if (Anchor.isPortrait()) {
                            separator.rotation = 0;
                            separatorSprite.setAnimation('default');
                            separator.scale.x = 1;
                            separator.scale.y = 1;
                        } else {
                            separator.rotation = Math.PI / 2;
                            separatorSprite.setAnimation('wide');
                            separator.scale.y = 0.05;
                            separator.scale.x = 50;
                        }
                        if (!Anchor.isPortrait() && Anchor.isWideScreen()) {
                            separator.position.x -= 8;
                            separator.scale.y = 1;
                            separator.scale.x = 1;
                        }
                    }
                })
            ]
        });
        /**
         * Character
         */
        var currentCharacter = CharacterManager.getSelected();
        var largePortraitSprite = new Sprite({
            spriteSheet: 'characterchange/' + currentCharacter,
        });
        var maskSpriteLarge = new MaskSprite({
            sprite: largePortraitSprite,
            // kind of guessed these values
            boundingBox: new Rectangle(-72, -viewport.height, 128, viewport.height * 2)
        });
        var largePortrait = new Entity({
            name: 'largePortrait',
            position: new Vector2(0, 0),
            components: [
                largePortraitSprite,
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.25, separatorPortrait),
                        offset: new Vector2(0, -112)
                    },
                    landscape: {
                        screenPosition: new Vector2(0.2, 0.5),
                        offset: new Vector2(4, 0)
                    },
                    onComplete: function () {
                        if (!Anchor.isPortrait() && Anchor.isWideScreen()) {
                            largePortrait.position.x -= 4;
                        }
                    }
                })
            ]
        });
        var largePortraitBackSprite = new Sprite({
            spriteSheet: 'characterchange/splash_' + currentCharacter,
        });
        var maskSpriteSplash = new MaskSprite({
            sprite: largePortraitBackSprite,
            // kind of guessed these values
            boundingBox: new Rectangle(-80, -viewport.height, 128, viewport.height * 2)
        });
        var largePortraitBack = new Entity({
            name: 'largePortraitBack',
            components: [
                largePortraitBackSprite, //
                {
                    name: 'positioner',
                    update: function () {
                        largePortraitBack.position = largePortrait.position.add(new Vector2(8, 8));

                    }
                }
            ]
        });
        var portraitContainerPortrait = new Entity({
            name: 'portr',
            family: [''],
            position: new Vector2(0, 0),
            components: [

            ]
        });
        /**
         * Stats
         */
        var stats = new Stats({
            character: currentCharacter,
            displayMode: Stats.DISPLAY_POSSIBLE,
            components: [
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.75, 0.22),
                        offset: new Vector2(0, 0)
                    },
                    landscape: {
                        screenPosition: new Vector2(0.52, 0.35),
                        offset: new Vector2(4, 0)
                    },
                    onComplete: function () {
                        if (Anchor.isPortrait()) {
                            entity.moveComponentTo(stats, _statsPortrait.rootIndex);
                        } else {
                            entity.moveComponentTo(stats, _statsLandscape.rootIndex);
                        }
                    }
                })
            ]
        });
        var statsBg = new StatsBg({
            components: [
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.75, 0.22),
                        offset: new Vector2(0, 0)
                    },
                    landscape: {
                        screenPosition: new Vector2(0.52, 0.35),
                        offset: new Vector2(4, 0)
                    },
                    onComplete: function () {
                        if (Anchor.isPortrait()) {
                            entity.moveComponentTo(stats, _statsBgPortrait.rootIndex);
                            entity.moveComponentTo(largePortraitBack, _portraitBgPortraitBack.rootIndex);
                            entity.moveComponentTo(largePortrait, _portraitBgPortrait.rootIndex);
                            largePortrait.remove(maskSpriteLarge);
                            largePortraitBack.remove(maskSpriteSplash);
                        } else {
                            entity.moveComponentTo(stats, _statsBgLandscape.rootIndex);
                            entity.moveComponentTo(largePortraitBack, _portraitBgLandscape.rootIndex);
                            entity.moveComponentTo(largePortrait, _portraitBgLandscape.rootIndex);
                            if (!maskSpriteLarge.parent) {
                                largePortrait.attach(maskSpriteLarge);
                            }
                            if (!maskSpriteSplash.parent) {
                                largePortraitBack.attach(maskSpriteSplash);
                            }
                        }
                    }
                })
            ]
        });

        /**
         * Bottom character select
         */
        var scrollingList = new ScrollingList({
            damping: 0.9,
            snapSpeed: 0.1,
            snapMinSlideSpeed: 1,
            spacing: 1,
            snap: true,
            direction: 'x',
            components: [
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.5, 0.77)
                    },
                    landscape: {
                        screenPosition: new Vector2(0.7, 0.77)
                    }
                }),
                new EventListener({
                    name: 'preventScroll',
                    onEvent: function () {
                        scrollingList.cancel();
                    }
                })
            ],
            onSnap: function (item, index) {
                dotList.iterate(function (dot, i) {
                    if (i === index) {
                        dot.getComponent('sprite', function (sprite) {
                            sprite.setAnimation('down');
                        });
                    } else {
                        dot.getComponent('sprite', function (sprite) {
                            sprite.setAnimation('up');
                        });
                    }
                });
            }
        });
        var dotList = new CenteredList({
            spacing: 3,
            components: [
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.5, 1),
                        offset: new Vector2(0, -34)
                    },
                    landscape: {
                        screenPosition: new Vector2(0.7, 1),
                        offset: new Vector2(0, -8)
                    }
                })
            ]
        });
        /**
         * Wall
         */
        var wall = new Entity({
            name: 'wall',
            components: [
                new Sprite({
                    imageName: 'gui2/iap/tiling_wall',
                    originRelative: new Vector2(0.5, 0),
                }),
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.5, separatorPortrait),
                        offset: new Vector2(0, -12)
                    },
                    landscape: {
                        screenPosition: new Vector2(0.75, 0),
                        offset: new Vector2(4, 0)
                    }
                })
            ]
        });
        /**
         * Tokens needed
         */
        var tokenBack = new Entity({
            name: 'tokenBack',
            position: new Vector2(0, -24),
            components: [
                new NineSlice({
                    imageName: 'gui2/characterchange/tokens_required_9slice',
                    width: 96,
                    height: 40,
                    originRelative: new Vector2(0.5, 0.5)
                })
            ]
        });
        var tokenSprite = new Sprite({
            spriteSheet: 'playertokens/' + currentCharacter
        });
        var tokenIcon = new Entity({
            name: 'tokenIcon',
            position: new Vector2(-20, 0),
            components: [
                tokenSprite
            ]
        });
        var tokenBar = new TokenBar({
            position: new Vector2(14, 0),
            character: currentCharacter,
            type: 'playertokens'
        });
        var tokensRequired = new Entity({
            name: 'tokensRequired',
            position: new Vector2(0, -29),
            components: [
                tokenBar,
                tokenIcon
            ]
        });
        /**
         * Button
         */
        var levelsToUpgrade = 1;
        var priceText = new Text({
            position: new Vector2(-1, 8),
            text: '',
            font: 'font',
            fontSize: 12,
            fontColor: '#ffdf00',
            align: 'center',
            textBaseline: 'middle',
            lineWidth: 1,
            strokeStyle: '#3e1a00',
            pixelStroke: true,
            maxWidth: 22,
            maxHeight: 12
        });
        var priceEntity = new Entity({
            name: 'price',
            position: new Vector2(-32, -2),
            components: [
                new Sprite({
                    imageName: 'gui/coin2',
                    originRelative: new Vector2(0.5, 0.5)
                }),
                priceText
            ]
        });
        var upgradeText = new ShadowText({
            position: new Vector2(14, -1),
            text: Localization.getText('levelUp'),
            font: 'font',
            fontSize: 18,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            // ySpacing: -2, // NOTE: may not be the case for other fonts
            maxWidth: 58,
            maxHeight: 18
        });
        var maxLevelText = new ShadowText({
            position: new Vector2(0, -1),
            text: Localization.getText('maxLevel'),
            font: 'font',
            fontSize: 18,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            maxWidth: 80,
            maxHeight: 32,
            visible: false
        });
        var buttonUp = new IconButton({
            name: 'buttonUp',
            sfx: 'sfx_button',
            imageName: 'gui2/misc/buttontiny',
            iconName: 'gui2/misc/button_icon_up',
            iconAlpha: 0.15,
            frameCountX: 1,
            frameCountY: 3,
            animations: {
                up: {
                    frames: [0]
                },
                down: {
                    frames: [1]
                },
                inactive: {
                    frames: [2]
                }
            },
            components: [
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.5, 0.75),
                        offset: new Vector2(75, -48 - 10)
                    },
                    landscape: {
                        screenPosition: new Vector2(0.8, 0.45),
                        offset: new Vector2(16, -56)
                    }
                }), //
                {
                    name: 'disable',
                    update: function () {
                        var level = StatsManager.getLevel(currentCharacter);
                        var maxLevel = CharacterManager.getMaxLevel(currentCharacter);
                        var atMax = (level + levelsToUpgrade >= maxLevel);
                        var hasEnough = hasEnoughTokens(currentCharacter, levelsToUpgrade + 1);

                        if (!(atMax || !hasEnough) && !buttonUp.isActive()) {
                            buttonUp.setActive(true);
                        } else if ((atMax || !hasEnough) && buttonUp.isActive()) {
                            buttonUp.setActive(false);
                        }
                    }
                }
            ],
            onButtonDown: function () {
                EventSystem.fire('preventScroll');
            },
            onClick: function () {
                var level = StatsManager.getLevel(currentCharacter);
                var maxLevel = CharacterManager.getMaxLevel(currentCharacter);

                // not beyond max level
                if (level + levelsToUpgrade < maxLevel) {
                    levelsToUpgrade += 1;
                    setPrice();

                    // bounce
                    new Tween({
                        from: 0.8,
                        to: 1,
                        in: 40,
                        ease: 'elastic',
                        decay: 5,
                        oscillations: 1,
                        onStart: function () {},
                        onUpdate: function (v, t) {
                            tokenIcon.scale.x = v;
                            tokenIcon.scale.y = v;
                            tokenBar.scale.x = v;
                            tokenBar.scale.y = v;
                            priceEntity.scale.x = v;
                            priceEntity.scale.y = v;
                        }
                    });
                }
            }

        });
        var buttonDown = new IconButton({
            name: 'buttonDown',
            sfx: 'sfx_button',
            imageName: 'gui2/misc/buttontiny',
            iconName: 'gui2/misc/button_icon_down',
            iconAlpha: 0.15,
            frameCountX: 1,
            frameCountY: 3,
            animations: {
                up: {
                    frames: [0]
                },
                down: {
                    frames: [1]
                },
                inactive: {
                    frames: [2]
                }
            },
            components: [
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.5, 0.75),
                        offset: new Vector2(75, -48 + 10)
                    },
                    landscape: {
                        screenPosition: new Vector2(0.8, 0.45),
                        offset: new Vector2(40, -56)
                    }
                }), //
                {
                    name: 'disable',
                    update: function () {
                        if (levelsToUpgrade > 1 && !buttonDown.isActive()) {
                            buttonDown.setActive(true);
                        } else if (levelsToUpgrade <= 1 && buttonDown.isActive()) {
                            buttonDown.setActive(false);
                        }
                    }
                }
            ],
            onButtonDown: function () {
                EventSystem.fire('preventScroll');
            },
            onClick: function () {
                if (levelsToUpgrade > 1) {
                    levelsToUpgrade -= 1;
                    setPrice();

                    // bounce
                    new Tween({
                        from: 0.8,
                        to: 1,
                        in: 40,
                        ease: 'elastic',
                        decay: 5,
                        oscillations: 1,
                        onStart: function () {},
                        onUpdate: function (v, t) {
                            tokenIcon.scale.x = v;
                            tokenIcon.scale.y = v;
                            tokenBar.scale.x = v;
                            tokenBar.scale.y = v;
                            priceEntity.scale.x = v;
                            priceEntity.scale.y = v;
                        }
                    });
                }
            }
        });
        var evolveIcon = new Entity({
            name: 'evolveIcon',
            components: [
                new Sprite({
                    imageName: 'gui2/misc/button_icon_evolve',
                    originRelative: new Vector2(0.5, 0.5),
                }),
                new Text({
                    position: new Vector2(0, 11),
                    text: Localization.getText('evolveTitle').toUpperCase(),
                    font: 'font',
                    fontSize: 10,
                    fontColor: '#ffffff',
                    align: 'center',
                    textBaseline: 'middle',
                    maxWidth: 30,
                    maxHeight: 10,
                    linebreaks: false
                })
            ]
        });
        var buttonEvolve = new IconButton({
            name: 'buttonEvolve',
            sfx: 'sfx_button',
            imageName: 'gui2/misc/button_small_secondary',
            icon: evolveIcon,
            iconAlpha: 0.15,
            frameCountX: 1,
            frameCountY: 3,
            components: [
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.5, 0.75),
                        offset: new Vector2(-52 - 12, -48)
                    },
                    landscape: {
                        screenPosition: new Vector2(0.38, 1),
                        offset: new Vector2(-28, -28)
                    }
                })
            ],
            onButtonDown: function () {
                EventSystem.fire('preventScroll');
            },
            onClick: function () {
                var characterData = CharacterManager.getCharacterData(currentCharacter);
                var evolveItems = InventoryManager.getItem('evolve/' + currentCharacter);
                var canEvolve = evolveItems >= CharacterManager.getEvolveItems(currentCharacter);
                var gemsNeeded = CharacterManager.getEvolveCost(currentCharacter);
                var getDialogText = function () {
                    var id;
                    if (reachedMax) {
                        id = 'evolveMax';
                    } else if (canEvolve) {
                        id = 'evolveComplete';
                    } else {
                        id = 'evolveText';
                    }
                    return Localization.getText(id);
                };
                var reachedMax = CharacterManager.getMaxLevel(currentCharacter) >= CharacterManager.getMaxTotal(currentCharacter);
                var evolveBar = new TokenBar({
                    position: new Vector2(20, 4),
                    type: 'evolve',
                    character: currentCharacter
                });
                var item = new Entity({
                    name: 'icon',
                    position: new Vector2(-20, 4),
                    components: [
                        new Sprite({
                            spriteSheet: 'evolve/' + currentCharacter
                        })
                    ]
                });
                var evolveItem = new Entity({
                    name: 'evolveItem',
                    dimension: new Rectangle(0, 0, 32, 32),
                    components: [
                        item,
                        evolveBar
                    ]
                });
                var evolveNowText = new ShadowText({
                    position: new Vector2(0, -2),
                    text: Localization.getText('evolveComplete'),
                    font: 'font',
                    fontSize: 24,
                    fontColor: '#ffffff',
                    shadowColor: '#0e6049',
                    align: 'center',
                    textBaseline: 'middle',
                    maxWidth: 80
                });
                var dialog = new Dialog({
                    name: 'evolveDialog',
                    z: entity.z + 1,
                    title: Localization.getText('evolveTitle'),
                    text: getDialogText(),
                    icon: reachedMax ? undefined : evolveItem,
                    yesIcon: evolveNowText,
                    canClose: true,
                    size: new Rectangle(0, 0, 180, 160)
                });
                var cost = new ShadowText({
                    position: new Vector2(8, -1),
                    text: gemsNeeded.toString(),
                    font: 'font',
                    fontSize: 22,
                    fontColor: '#ffffff',
                    align: 'left',
                    textBaseline: 'middle',
                    maxWidth: 48,
                    linebreaks: false
                });
                var gem = new Entity({
                    name: 'gem',
                    position: new Vector2(-16, 0),
                    components: [
                        new Sprite({
                            imageName: 'gui/gem',
                            originRelative: new Vector2(0.5, 0.5)
                        })
                    ]
                });
                var icon = new Entity({
                    name: 'icon',
                    components: [
                        gem,
                        cost
                    ]
                });
                var dialogButton = new IconButton({
                    name: 'dialogButton',
                    sfx: 'sfx_button',
                    position: new Vector2(0, 58),
                    imageName: 'gui2/misc/button_large_primary_excited',
                    frameCountX: 6,
                    frameCountY: 7,
                    padding: 1,
                    icon: icon,
                    animations: {
                        up: {
                            speed: 0.34,
                            frames: [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35]
                        },
                        down: {
                            frames: [37]
                        },
                        inactive: {
                            frames: [38]
                        }
                    },
                    onClick: function () {
                        var maxLevel = SaveManager.load('maxLevel-' + currentCharacter);
                        if (CurrencyManager.canSpendGems(gemsNeeded) && canEvolve) {
                            // spend gems
                            CurrencyManager.spendGems(gemsNeeded, 'evolution');
                            // spend evolve items
                            InventoryManager.addItem('evolve/' + currentCharacter, -CharacterManager.getEvolveItems(currentCharacter));
                            // update values
                            evolveItems = InventoryManager.getItem('evolve/' + currentCharacter);
                            canEvolve = (evolveItems >= CharacterManager.getEvolveItems(currentCharacter));
                            evolveBar.setCharacter(currentCharacter);
                            // increase max level
                            maxLevel += 10;
                            SaveManager.save('maxLevel-' + currentCharacter, maxLevel);
                            // refresh
                            selectCharacter(currentCharacter);
                            // close dialog
                            Bento.objects.remove(dialog);

                            // TODO: animation?
                            Bento.audio.stopSound('sfx_consumable');
                            Bento.audio.playSound('sfx_consumable');
                            largePortrait.attach(new Blink({}));
                            stats.attach(new SparkleBurst({}));

                            Analytics.event('Evolution', currentCharacter);
                        } else {
                            // // not enough gems???? go to shop
                            var iapMenu = new IAPMenu({
                                z: settings.z + 1,
                                scrollTo: 'gems'
                            });
                            Bento.objects.attach(iapMenu);
                            Analytics.event('Economy', 'buyCoinsWithGems', 1, 'notEnough');
                        }
                    }
                });
                dialog.attach(new Topbar({}));
                Bento.objects.attach(dialog);

                dialog.getComponent('windowBox', function (windowBox) {
                    windowBox.position.y += 8;
                    // add button
                    if (canEvolve && !reachedMax) {
                        windowBox.attach(dialogButton);
                    }
                });
            }
        });
        var buttonUpgrade = new IconButton({
            name: 'buttonUpgrade',
            sfx: 'sfx_button',
            imageName: 'gui2/misc/button_large_primary',
            icon: new Entity({
                name: 'upgradeContainer',
                components: [
                    upgradeText,
                    priceEntity,
                    maxLevelText
                ]
            }),
            frameCountX: 1,
            frameCountY: 3,
            position: new Vector2(0, 0),
            float: false,
            updateWhenPaused: 0,
            onButtonDown: function () {
                EventSystem.fire('preventScroll');
            },
            onClick: function () {
                // does the player have enough coins?
                var unlocked = CharacterManager.getUnlockedCharacters();
                var char = currentCharacter;
                var currentLevel = StatsManager.getLevel(char);
                var cost = Globals.getLevelUpCost(currentLevel, levelsToUpgrade);
                var reachedMaxTotal = CharacterManager.getMaxLevel(currentCharacter) >= CharacterManager.getMaxTotal(currentCharacter);

                var levelUp = function () {
                    //remove upgrade items
                    var upgradeCost = Globals.getTokenCost(currentLevel, levelsToUpgrade);
                    InventoryManager.spendTokens('playertokens/' + char, upgradeCost);

                    // level up and update stats
                    StatsManager.addLevel(char, levelsToUpgrade);

                    // update
                    selectCharacter(currentCharacter);

                    // quest
                    EventSystem.fire('quest-upgrade', {
                        count: 1
                    });

                    Analytics.event('UpgradeTotal', char);
                    Analytics.event('Upgrade', char + 'level_' + (currentLevel + levelsToUpgrade));

                    Tenjin.sendEvent('upgrade_' + char);
                    Tenjin.sendEventAndValue('upgrade_' + char + '_level', (currentLevel + levelsToUpgrade));

                    FireBase.setUserProperty(currentCharacter + '_level', currentLevel + levelsToUpgrade);
                    FireBase.setUserProperty(currentCharacter + '_maxLevel', CharacterManager.getMaxLevel(currentCharacter));
                };

                if (tutorialCharacter === 0 && currentCharacter === 'boy') {
                    // no cost during tutorial
                    // cost = 0; // DEPRECATED
                    tutorialCharacter = 1;
                    SaveManager.save('tutorialCharacter', tutorialCharacter);
                    EventSystem.fire('tutorialCharacter');
                }

                if (unlocked.indexOf(char) < 0) {
                    console.log('Character was not unlocked yet');
                    return;
                }

                if (currentLevel >= CharacterManager.getMaxLevel(currentCharacter)) {
                    if (reachedMaxTotal) {
                        Bento.objects.attach(new Dialog({
                            name: 'maxReached',
                            text: Localization.getText('maxLevel'),
                            type: 'yes'
                        }));
                    } else {
                        // find evolve token message
                        var progress = new TokenBar({
                            position: new Vector2(20, 4),
                            type: 'evolve',
                            character: currentCharacter
                        });
                        var item = new Entity({
                            name: 'icon',
                            position: new Vector2(-20, 0),
                            components: [
                                new Sprite({
                                    spriteSheet: 'evolve/' + currentCharacter
                                })
                            ]
                        });
                        var evolveItem = new Entity({
                            name: 'evolveItem',
                            dimension: new Rectangle(0, 0, 32, 32),
                            components: [
                                item,
                                progress
                            ]
                        });
                        Bento.objects.attach(new Dialog({
                            name: 'maxReached',
                            title: Localization.getText('evolveTitle'),
                            text: Localization.getText('maxLevelEvolve'),
                            type: 'yes',
                            icon: evolveItem
                        }));
                    }
                    return;
                }

                if (hasEnoughTokens(char)) {
                    if (CurrencyManager.canSpend(cost)) {
                        largePortrait.attach(new Blink({}));
                        buttonUpgrade.attach(new Blink({}));
                        tokenIcon.attach(new SparkleBurst({}));
                        tokenIcon.attach(new Blink({}));
                        tokenBar.attach(new Blink({}));
                        Bento.audio.stopSound('sfx_consumable');
                        Bento.audio.playSound('sfx_consumable');

                        CurrencyManager.spendCoins(cost, 'upgrade-' + char);
                        levelUp();
                    } else {
                        // ask to buy missing coins
                        var buyCoins = new BuyCoinsPopup({
                            z: Globals.WorldSelectLayers.UPGRADE + 1,
                            coinsToBuy: cost - CurrencyManager.getCoins(),
                            buttonToClick: buttonUpgrade
                        });
                        Bento.objects.attach(buyCoins);
                    }
                } else {
                    Bento.objects.attach(new TokenDialog({
                        character: currentCharacter
                    }));
                }
            }
        });
        var unlockText = new ShadowText({
            position: new Vector2(14, -1),
            text: Localization.getText('unlock'),
            font: 'font',
            fontSize: 18,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            maxWidth: 58,
            maxHeight: 28
        });
        var buttonUnlock = new IconButton({
            name: 'buttonUnlock',
            sfx: 'sfx_button',
            imageName: 'gui2/misc/button_large_primary',
            icon: new Entity({
                name: 'upgradeContainer',
                components: [
                    unlockText,
                    new Entity({
                        name: 'unlockIcon',
                        position: new Vector2(-32, 0),
                        components: [
                            new Sprite({
                                imageName: 'gui2/misc/button_icon_lock',
                                originRelative: new Vector2(0.6, 0.6)
                            })
                        ]
                    })
                ]
            }),
            frameCountX: 1,
            frameCountY: 3,
            onButtonDown: function () {
                EventSystem.fire('preventScroll');
            },
            onClick: function () {
                var characterData = CharacterManager.getCharacterData(currentCharacter);
                var missionRequired = characterData.unlocksAt;
                var getCharacterCost = function () {
                    return CharacterManager.getUnlockCost(currentCharacter);
                };
                var getCharacterName = function () {
                    var name = Localization.getText('name-' + currentCharacter);
                    return name;
                };
                var tokenCost = getCharacterCost();
                var tokens = InventoryManager.getTokens('playertokens/' + currentCharacter);

                var unlockedMissions = SaveManager.load('unlockedLevels');
                var showPopup = function () {
                    var popup = new UnlockCharacterPopup({
                        character: currentCharacter,
                        name: getCharacterName()
                    });
                    Bento.objects.attach(popup);

                    FireBase.setUserProperty(currentCharacter + '_level', StatsManager.getLevel(currentCharacter));
                    FireBase.setUserProperty(currentCharacter + '_maxLevel', CharacterManager.getMaxLevel(currentCharacter));

                };
                // cleared required mission?
                if (!hasClearedMission(missionRequired)) {
                    // warn user for not clearing mission
                    Bento.objects.attach(new Dialog({
                        name: 'warnMissionNotClearedForUnlock',
                        title: Localization.getText('needMissionTitle'),
                        text: Localization.getText('needMissionText').replace('{NUMBER}', missionRequired),
                        type: 'yes',
                        canClose: true
                    }));
                }
                // enough tokens?
                else if (!hasEnoughTokens(currentCharacter)) {
                    // warn user for not enough tokens
                    Bento.objects.attach(new TokenDialog({
                        character: currentCharacter
                    }));
                } else {
                    // spend the tokens during unlock?
                    // InventoryManager.spendTokens('playertokens/' + currentCharacter, tokenCost);

                    // unlock the actual character
                    CharacterManager.unlock(currentCharacter);
                    // select it
                    CharacterManager.setSelected(currentCharacter);

                    // update
                    selectCharacter(currentCharacter);

                    // remove the lock
                    Utils.forEach(characterButtons, function (button) {
                        if (button.name === 'button-' + currentCharacter) {
                            // show character
                            button.unlock();
                        }
                    });

                    // unlock imessage stickers
                    Utils.unlockSticker(currentCharacter + '0');
                    Utils.unlockSticker(currentCharacter + '1');
                    Utils.unlockSticker(currentCharacter + '2');

                    Analytics.event('Unlock', currentCharacter, unlockedMissions.length);

                    showPopup();
                }
            }
        });
        var buttonContainer = new Entity({
            name: 'buttonContainer',
            components: [
                tokenBack,
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.5, 0.75),
                        offset: new Vector2(24 - 12, -48)
                    },
                    landscape: {
                        screenPosition: new Vector2(0.8, 0.45),
                        offset: new Vector2(0, 0)
                    }
                }),
                buttonUpgrade,
                tokensRequired
            ]
        });
        var buttonBack = new ClickButton({
            name: 'buttonBack',
            sfx: 'sfx_button',
            imageName: 'gui2/bottom_tabs/bottom_tabs__ok',
            frameCountX: 2,
            frameCountY: 1,
            components: [
                new BackButton({
                    removeSelf: true,
                    onPressed: function (data) {
                        data.entity.doCallback();
                    }
                }),
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.5, 1),
                        offset: new Vector2(-48 + (Anchor.isWideScreen() ? 4 : 0), -12 + (Anchor.isWideScreen() ? Anchor.getOffset().y : 0))
                    },
                    landscape: {
                        screenPosition: new Vector2(0.25, 1),
                        offset: new Vector2(-44, -12)
                    }
                })
            ],
            onButtonDown: function () {
                EventSystem.fire('preventScroll');
            },
            onClick: function () {
                entity.removeSelf();
                EventSystem.fire('characterSelectDismissed');
            }
        });
        /**
         * Other components
         */
        var modal = new Modal({});
        var characterButtons = [];
        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {
                // make pages for scrolling list
                var availableCharacters = CharacterManager.getAvailableCharacters();
                var currentPage = 0;
                var selectedPage = 0;
                var pages = {};
                var makeButton = function (character, idx, page) {
                    var lockSprite = new Sprite({
                        imageName: 'gui2/characterchange/portrait_frame_lock',
                        frameCountX: 4,
                        frameCountY: 4,
                        originRelative: new Vector2(0.5, 0.5),
                        animations: {
                            up: {
                                frames: [0]
                            },
                            down: {
                                frames: [0]
                            }
                        }
                    });
                    var lock = new Entity({
                        name: 'lock',
                        position: new Vector2(14, 18),
                        components: [
                            lockSprite, //
                            {
                                name: 'setAnimation',
                                update: function () {
                                    lockSprite.setAnimation(button.getComponent('sprite').getAnimationName());
                                },
                                start: function () {
                                    // set to gray
                                    characterPortraitSprite.setup({
                                        imageName: 'gui2/characterchange/smallportrait_locked_' + character,
                                        originRelative: new Vector2(0.5, 0.5)
                                    });
                                },
                                destroy: function () {
                                    // revert portrait to normal color
                                    characterPortraitSprite.setup({
                                        imageName: 'gui2/characterchange/smallportrait_' + character,
                                        originRelative: new Vector2(0.5, 0.5)
                                    });

                                }
                            }
                        ]
                    });
                    var badge = new Badge({
                        pulsate: true,
                        exclamation: true,
                        imageName: 'gui2/misc/badge',
                        position: new Vector2(16, -16),
                    });
                    var buttonBehavior = {
                        update: function () {
                            // is in active page?
                            if (scrollingList.getSelection() === page) {
                                if (!button.isActive()) {
                                    button.setActive(true);
                                }
                                // can upgrade?
                                if (badge.parent) {
                                    badge.visible = hasEnoughTokens(character, 1);
                                }
                            } else {
                                if (button.isActive()) {
                                    button.setActive(false);
                                }
                            }
                        }
                    };
                    var canUnlock = false;
                    var characterPortraitSprite = new Sprite({
                        imageName: 'gui2/characterchange/smallportrait_' + character,
                        originRelative: new Vector2(0.5, 0.5)
                    });
                    var button = new ToggleButton({
                        name: 'button-' + character,
                        sfx: 'sfx_button',
                        imageName: 'gui2/characterchange/portrait_frame',
                        frameCountX: 2,
                        frameCountY: 1,
                        toggled: character === currentCharacter,
                        onToggle: function () {
                            if (this.isToggled()) {
                                // untoggle others
                                Utils.forEach(characterButtons, function (other) {
                                    if (other !== button) {
                                        other.toggle(false);
                                    }
                                    // select character
                                    selectCharacter(character);
                                    // save selected character if it was unlocked
                                    if (CharacterManager.isUnlocked(character)) {
                                        CharacterManager.setSelected(character);

                                        // tween ok button
                                        new Tween({
                                            from: 0.5,
                                            to: 1,
                                            in: 30,
                                            delay: 0,
                                            ease: 'elastic',
                                            decay: 5,
                                            oscillations: 2,
                                            onUpdate: function (v, t) {
                                                buttonBack.scale.x = v;
                                                buttonBack.scale.y = v;
                                            },
                                            onComplete: function () {

                                            }
                                        });
                                    }
                                });
                            } else {
                                // re-toggle self
                                button.toggle(true);
                            }
                            new Tween({
                                from: 0.9,
                                to: 1,
                                in: 30,
                                ease: 'elastic',
                                decay: 5,
                                oscillations: 2,
                                onUpdate: function (v, t) {
                                    button.scale.x = v;
                                    button.scale.y = v;
                                }
                            });
                        },
                        components: [
                            characterPortraitSprite,
                            buttonBehavior
                        ]
                    });
                    button.dimension.width = 42;
                    characterButtons.push(button);

                    // unlock
                    button.extend({
                        unlock: function () {
                            button.removeByName('lock');
                        }
                    });

                    // add lock
                    if (!CharacterManager.isUnlocked(character)) {
                        button.attach(lock);
                        if (hasEnoughTokens(character)) {
                            //lock.attach(new Glow({}));
                            lock.getComponent('sprite', function (sprite) {
                                sprite.setup({
                                    imageName: 'gui2/characterchange/portrait_frame_lock',
                                    frameCountX: 4,
                                    frameCountY: 4,
                                    padding: 1,
                                    originRelative: new Vector2(0.6, 0.6),
                                    animations: {
                                        up: {
                                            speed: 0.24,
                                            frames: [1, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15]
                                        },
                                        down: {
                                            frames: [1]
                                        },
                                        inactive: {
                                            frames: [1]
                                        }
                                    }
                                });
                            });
                            canUnlock = true;
                        }
                        // when lock is removed, attach the badge
                        lock.attach({
                            name: 'addBadge',
                            destroy: function () {
                                button.attach(badge);
                            }
                        });
                    } else {
                        button.attach(badge);
                    }

                    // this character was selected
                    if (character === currentCharacter || canUnlock) {
                        selectedPage = page;
                    }

                    return button;
                };
                var hideEvolve = function () {
                    // hide evolve so the player is not overwhelmed first time
                    var unlockedMissions = SaveManager.load('unlockedLevels');
                    if (unlockedMissions.length < 4) {
                        buttonEvolve.setActive(false);
                    }
                };
                hideEvolve();

                Utils.forEach(availableCharacters, function (character, i) {
                    var button;
                    currentPage = Math.floor(i / 3);
                    if (!pages[currentPage]) {
                        pages[currentPage] = new CenteredList({
                            spacing: 2
                        });
                        pages[currentPage].dimension.width = 148;
                        scrollingList.addItem(pages[currentPage]);

                        var dotSprite = new Sprite({
                            frameCountX: 2,
                            originRelative: new Vector2(0.5, 0.5),
                            imageName: 'gui2/misc/pagination_dots',
                            animations: {
                                'up': {
                                    frames: [0]
                                },
                                'down': {
                                    frames: [1]
                                }
                            }
                        });
                        dotList.addItem(new Entity({
                            name: 'dot',
                            components: [
                                dotSprite
                            ]
                        }));
                        if (i === 0) {
                            dotSprite.setAnimation('down');
                        }
                    }
                    button = makeButton(character, i, currentPage);
                    pages[currentPage].addItem(button);
                    // reverse drawing order
                    pages[currentPage].moveComponentTo(button, 0);
                });

                // pages[currentPage].addItem(new Entity({
                //     name: 'coming soon',
                //     position: new Vector2(0, 0),
                //     components: [
                //         new Sprite({
                //             imageName: 'gui2/characterchange/portrait_frame_empty',
                //             originRelative: new Vector2(0.5, 0.5),
                //         }),
                //         new Text({
                //             position: new Vector2(0, -16),
                //             text: Localization.getText('comingsoon'),
                //             font: 'font',
                //             fontSize: 10,
                //             fontColor: '#ffffff',
                //             align: 'center',
                //             textBaseline: 'middle',
                //             maxWidth: 32,
                //             maxHeight: 10,
                //             linebreaks: false
                //         })
                //     ]
                // }));

                // init
                stats.glowPlusText();
                selectCharacter(currentCharacter);

                scrollingList.snapTo(selectedPage);

                // tween in
                // new Tween({
                //     from: 0,
                //     to: 1,
                //     in: 5,
                //     delay: 0,
                //     ease: 'linear',
                //     onUpdate: function (v, t) {
                //         entity.alpha = v;
                //     }
                // });
            },
            destroy: function (data) {},
            update: function (data) {},
            draw: function (data) {}
        };
        var tapHand;
        var addTaphand = function () {
            if (tapHand) {
                tapHand.removeSelf();
            }
            tapHand = new TapHand({
                position: new Vector2(0, 8),
                rotation: Math.PI
            }).attach(new EventListener({
                name: 'giftClosed',
                onEvent: function () {
                    tapHand.removeSelf();
                }
            }));
            buttonUpgrade.attach(tapHand);
        };
        var _statsBgPortrait = {};
        var _statsBgLandscape = {};
        var _portraitBgPortraitBack = {};
        var _portraitBgPortrait = {};
        var _portraitBgLandscape = {};
        var _statsPortrait = {};
        var _statsLandscape = {}; // these two components are just for positioning the stats
        var entity = new Entity({
            z: settings.z || Globals.WorldSelectLayers.UPGRADE,
            name: 'characterSelect',
            float: true,
            components: [
                modal,
                backgroundFists,
                _portraitBgPortraitBack,
                largePortraitBack,
                statsBg,
                _statsBgPortrait,
                largePortrait,
                _portraitBgPortrait,
                stats,
                _statsPortrait,
                wall,
                separator,
                scrollingList,
                dotList,
                backgroundFistsLandscape,
                _portraitBgLandscape,
                _statsBgLandscape,
                _statsLandscape,
                buttonContainer,
                buttonEvolve,
                buttonUp,
                buttonDown,
                buttonBack,
                behavior,
                new Topbar({})
            ]
        });
        /**
         * The character selected in the screen (which may not be the same as the CharacterManager.getSelected())
         */
        var getSelectedCharacter = function () {
            // TODO
            //
            return currentCharacter;
        };
        var selectCharacter = function (character) {
            var tokens = InventoryManager.getTokens('playertokens/' + currentCharacter);
            var level = StatsManager.getLevel(character);
            var upgradeCost = Globals.getTokenCost(level, levelsToUpgrade);
            var previousCharacter = currentCharacter;

            // reset levelsToUpgrade when selecting new characters
            levelsToUpgrade = 1;

            currentCharacter = character;
            // set portrait
            largePortraitSprite.setup({
                spriteSheet: 'characterchange/' + (CharacterManager.isUnlocked(currentCharacter) ? '' : 'locked_') + currentCharacter
            });
            largePortraitBackSprite.setup({
                spriteSheet: 'characterchange/splash_' + currentCharacter
            });
            setPrice();

            // tutorial
            if (tutorialCharacter === 0 && hasEnoughTokens(currentCharacter) && currentCharacter === 'boy') {
                addTaphand();
            } else if (tapHand) {
                tapHand.removeSelf();
                tapHand = null;
            }

            if (previousCharacter !== character && Anchor.isPortrait()) {
                // note: disabled tweens for landscape because of masksprites
                largePortrait.getComponent('anchor', function (anchor) {
                    var position = anchor.getPosition();
                    new Tween({
                        from: position.x - 180,
                        to: position.x,
                        in: 30,
                        ease: 'elastic',
                        decay: 8, // for 'exponential' and 'elastic'
                        oscillations: 1, // for 'elastic', 'sin' and 'cos'
                        onUpdate: function (v, t) {
                            largePortrait.position.x = v;
                        }
                    });
                });
            }
        };
        var setPrice = function () {
            var getCoinCost = function () {
                if (tutorialCharacter === 0 && currentCharacter === 'boy') {
                    // return 0; // DEPRECATED
                }
                return Globals.getLevelUpCost(StatsManager.getLevel(currentCharacter), levelsToUpgrade);
            };
            // set buttons, tokens, price
            tokenSprite.setSpriteSheet('playertokens/' + currentCharacter);
            priceText.setText(getCoinCost().toString());
            tokenBar.setCharacter(currentCharacter);
            tokenBar.setLevelDiff(levelsToUpgrade);
            // set stats
            stats.resetPossible();
            stats.setDisplayMode(Stats.DISPLAY_COMBINED);
            stats.setCharacter(currentCharacter);

            // levels
            if (levelsToUpgrade > 1) {
                upgradeText.setText(Localization.getText('levelUp') + '\n' + levelsToUpgrade + '×', {
                    maxHeight: 32
                });
            } else {
                upgradeText.setText(Localization.getText('levelUp'), {
                    maxHeight: 18
                });
            }
            setButton();
        };
        var canUnlock = function (character) {
            // required amount of tokens?
            var characterJson = Bento.assets.getJson('stats/' + character);
            var cost = characterJson.unlockCost || 10;
            var tokens = InventoryManager.getTokens('playertokens/' + character);

            if (tokens >= cost) {
                return true;
            } else {
                return false;
            }
        };
        var hasEnoughTokens = function (character, diff) {
            diff = diff || levelsToUpgrade;
            var isUnlocked = CharacterManager.isUnlocked(character);
            var level = StatsManager.getLevel(character);
            var upgradeCost = Globals.getTokenCost(level, diff);
            var tokens = InventoryManager.getTokens('playertokens/' + character);

            if (!isUnlocked) {
                // swap upgradecost with unlock cost
                upgradeCost = CharacterManager.getUnlockCost(character) || 10;
            }

            if (tokens >= upgradeCost) {
                return true;
            } else {
                return false;
            }
        };
        var showPossibleStats = function (char) {
            var currentStats = StatsManager.getAllDetailedStats(char);
            var currentLevel = StatsManager.getLevel(char);
            var nextStats = Globals.getCharacterStatsFromLevel(char, currentLevel + levelsToUpgrade);
            stats.resetPossible();
            stats.addPossible('health', nextStats['health'] - currentStats['health'].bare);
            stats.addPossible('attack', nextStats['attack'] - currentStats['attack'].bare);
            stats.addPossible('skill', nextStats['skill'] - currentStats['skill'].bare);
            stats.setDisplayMode(Stats.DISPLAY_POSSIBLE);
        };
        var hasClearedMission = function (missionNumber) {
            // needs at least 0 stars
            var stars = SaveManager.load('missionStars-' + missionNumber);
            return stars >= 0;
        };
        /**
         * Set button to upgrade or unlock
         */
        var setButton = function () {
            var characterData = CharacterManager.getCharacterData(currentCharacter);
            var missionRequired = characterData.unlocksAt;
            var evolveItems = InventoryManager.getItem('evolve/' + currentCharacter);
            var canEvolve = (evolveItems >= CharacterManager.getEvolveItems(currentCharacter));
            var reachedMax = (CharacterManager.getLevel(currentCharacter) === CharacterManager.getMaxLevel(currentCharacter));
            var reachedMaxTotal = CharacterManager.getMaxLevel(currentCharacter) >= CharacterManager.getMaxTotal(currentCharacter);

            if (buttonUpgrade.parent) {
                buttonUpgrade.removeSelf();
            }
            if (buttonUnlock.parent) {
                buttonUnlock.removeSelf();
            }

            if (CharacterManager.isUnlocked(currentCharacter)) {
                buttonContainer.attach(buttonUpgrade);
                buttonContainer.moveComponentTo(buttonUpgrade, 1);
                buttonEvolve.setActive(true);

                // max level reached?
                if (reachedMax) {
                    priceEntity.visible = false;
                    upgradeText.visible = false;
                    maxLevelText.visible = true;
                } else {
                    priceEntity.visible = true;
                    upgradeText.visible = true;
                    maxLevelText.visible = false;
                }
                // can upgrade?
                if (hasEnoughTokens(currentCharacter) && !reachedMax) {
                    // show possible
                    showPossibleStats(currentCharacter);
                    // set button to excited
                    setExcited(buttonUpgrade, true);
                } else {
                    // set button to normal
                    setExcited(buttonUpgrade, false);
                }

                // evolve button
                if ((canEvolve || reachedMax) && !reachedMaxTotal) {
                    setExcited(buttonEvolve, true, 'gui2/misc/button_small_primary_excited');
                } else {
                    setExcited(buttonEvolve, false, 'gui2/misc/button_small_primary_excited');
                }

            } else {
                buttonEvolve.setActive(false);
                buttonContainer.attach(buttonUnlock);
                buttonContainer.moveComponentTo(buttonUnlock, 1);
                // can unlock?
                if (!hasClearedMission(missionRequired)) {
                    unlockText.setText(Localization.getText('unlockNeedMission').replace('{NUMBER}', missionRequired));
                    setExcited(buttonUnlock, false);
                } else if (!hasEnoughTokens(currentCharacter)) {
                    unlockText.setText(Localization.getText('unlock'));
                    setExcited(buttonUnlock, false);
                } else {
                    unlockText.setText(Localization.getText('unlock'));
                    setExcited(buttonUnlock, true);
                }
            }
        };
        var setExcited = function (button, excited, imageName) {
            imageName = imageName || 'gui2/misc/button_large_primary_excited';
            button.getComponent('sprite', function (sprite) {
                if (excited) {
                    sprite.setup({
                        imageName: imageName,
                        frameCountX: 6,
                        frameCountY: 7,
                        padding: 1,
                        animations: {
                            up: {
                                speed: 0.34,
                                frames: [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35]
                            },
                            down: {
                                frames: [37]
                            },
                            inactive: {
                                frames: [38]
                            }
                        }
                    });
                } else {
                    sprite.setup({
                        imageName: imageName,
                        frameCountX: 6,
                        frameCountY: 7,
                        padding: 1,
                        animations: {
                            up: {
                                speed: 0.34,
                                frames: [36]
                            },
                            down: {
                                frames: [37]
                            },
                            inactive: {
                                frames: [38]
                            }
                        }
                    });
                }
                if (button.isActive()) {
                    sprite.setAnimation('up');
                } else {
                    sprite.setAnimation('inactive');
                }
            });

        };
        return entity;
    };
});