/**
 * Cheat menu
 * @moduleName CheatMenu
 */
bento.define('gui/menus/cheat', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/tween',
    'gui/blurbackground',
    'components/modal',
    'globals',
    'gui/textbutton',
    'components/anchor',
    'managers/currency',
    'gui/worldselect/topbarsimple',
    'managers/save',
    'managers/character',
    'gui/scrollinglist',
    'managers/inventory',
    'modules/saveexporter',
    'gui/watchrewardedad',
    'gui/dialog',
    'managers/receiveitem',
    'managers/allies',
    'modules/notifications',
    'modules/localization',
    'gui/buttons/startbutton'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    BlurBackground,
    Modal,
    Globals,
    TextButton,
    Anchor,
    CurrencyManager,
    Topbar,
    SaveManager,
    CharacterManager,
    ScrollingList,
    InventoryManager,
    SaveExporter,
    WatchRewardedAd,
    Dialog,
    ReceiveItem,
    AlliesManager,
    Notifications,
    Localization,
    StartButton
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var modal = new Modal({});
        var background = new BlurBackground({});
        var scrollingList = new ScrollingList({
            direction: 'y',
            spacing: 4,
        }).attach(new Anchor({
            screenPosition: new Vector2(0.5, 0.5),
            offset: new Vector2(0, -64)
        }));
        var buttonReset = new TextButton({
            name: 'buttonReset',
            text: 'Reset',
            onButtonDown: function () {
                scrollingList.cancel();
            },
            onClick: function () {
                Bento.saveState.clear();
                Bento.saveState.save('showDeveloperCheats', true);
                CurrencyManager.reset();
                Bento.screens.show('screens/title');
            }
        });
        var buttonSave = new TextButton({
            name: 'buttonSave',
            text: 'Save',
            onButtonDown: function () {
                scrollingList.cancel();
            },
            onClick: function () {
                Utils.prompt('Enter save file name', function (str) {
                    if (!str) {
                        window.alert("You must enter a file name");
                        return;
                    }
                    SaveExporter.export(str, function (fileName) {
                        window.alert("Saved to " + fileName);
                    });
                });
            }
        });
        var buttonLoad = new TextButton({
            name: 'buttonLoad',
            text: 'Load',
            onButtonDown: function () {
                scrollingList.cancel();
            },
            onClick: function () {
                Bento.screens.show('screens/title');
                if (Utils.isCocoonJs()) {
                    Utils.prompt('Enter save file name', function (str) {
                        if (!str) {
                            window.alert("You must enter a file name");
                            return;
                        }
                        SaveExporter.import(str, function (fileName) {
                            window.alert("Loaded from " + fileName);
                        });
                    });
                } else {
                    SaveExporter.import('', function (fileName) {
                        console.log('Load successful');
                    });
                }
            }
        });
        var buttonAd = new TextButton({
            name: 'buttonAd',
            text: 'Test Rewarded',
            type: 'medium',
            onButtonDown: function () {
                scrollingList.cancel();
            },
            onClick: function () {
                WatchRewardedAd(function (completed) {
                    var dialog = new Dialog({
                        title: completed ? 'success' : 'aborted',
                        text: completed ? 'Ad watched' : 'Ad aborted',
                        type: 'yes'
                    });
                    Bento.objects.attach(dialog);
                });
            }
        });
        var buttonHeyZap = new TextButton({
            name: 'buttonHeyZap',
            text: 'HeyZap',
            type: 'medium',
            onButtonDown: function () {
                scrollingList.cancel();
            },
            onClick: function () {
                window.HeyzapAds.showMediationTestSuite();
            }
        });
        var buttonGems = new TextButton({
            name: 'buttonGems',
            text: '1000 gems',
            type: 'medium',
            onButtonDown: function () {
                scrollingList.cancel();
            },
            onClick: function () {
                CurrencyManager.addGems(1000);
            }
        });
        var buttonTokens = new TextButton({
            name: 'buttonTokens',
            text: '100 tokens',
            type: 'medium',
            onButtonDown: function () {
                scrollingList.cancel();
            },
            onClick: function () {
                var characters = CharacterManager.getAvailableCharacters();
                var allies = AlliesManager.getAvailable();
                Utils.forEach(characters, function (item, i, l, breakLoop) {
                    ReceiveItem({
                        key: 'playertokens/' + item,
                        amount: 100
                    });
                });
                Utils.forEach(allies, function (item, i, l, breakLoop) {
                    ReceiveItem({
                        key: 'enemytokens/' + item,
                        amount: 100
                    });
                });
            }
        });
        var buttonEvolve = new TextButton({
            name: 'buttonEvolve',
            text: '100 hats',
            type: 'medium',
            onButtonDown: function () {
                scrollingList.cancel();
            },
            onClick: function () {
                var characters = CharacterManager.getAvailableCharacters();
                Utils.forEach(characters, function (item, i, l, breakLoop) {
                    ReceiveItem({
                        key: 'evolve/' + item,
                        amount: 100
                    });
                });
            }
        });
        var buttonCoins = new TextButton({
            name: 'buttonCoins',
            text: '10000 coins',
            type: 'medium',
            onButtonDown: function () {
                scrollingList.cancel();
            },
            onClick: function () {
                CurrencyManager.addCoins(10000, true);
            }
        });
        var buttonKeys = new TextButton({
            name: 'buttonKeys',
            text: '30 keys',
            type: 'medium',
            onButtonDown: function () {
                scrollingList.cancel();
            },
            onClick: function () {
                CurrencyManager.addKeys(30);
            }
        });
        var buttonGifts = new TextButton({
            name: 'buttonGifts',
            text: 'reset coin ad timer',
            type: 'medium',
            onButtonDown: function () {
                scrollingList.cancel();
            },
            onClick: function () {
                var dailyAds = SaveManager.load('adsForCoins');
                // check time
                dailyAds.time = 0;
                SaveManager.save('adsForCoins', dailyAds);

                Bento.screens.show('screens/worldselect');

                // temp: set chest available
                // var gift24 = SaveManager.load('gift24');
                // gift24.time = 0;
                // SaveManager.save('gift24', gift24);
            }
        });
        var buttonEnergy = new TextButton({
            name: 'buttonEnergy',
            type: 'medium',
            text: 'Unlimited energy',
            onButtonDown: function () {
                scrollingList.cancel();
            },
            onClick: function () {
                Bento.saveState.save('unlimitedEnergy', true);
                Bento.screens.show('screens/worldselect');
            }
        });
        var buttonUnlockAll = new TextButton({
            name: 'buttonUnlockAll',
            type: 'medium',
            text: 'Unlock all stages',
            onButtonDown: function () {
                scrollingList.cancel();
            },
            onClick: function () {
                var missionsJson = Bento.assets.getJson('missions/missions');
                var unlocked = SaveManager.load('unlockedLevels');
                Utils.forEach(missionsJson, function (value, key) {
                    // try turning into integer
                    var name = parseInt(key);
                    if (!name) {
                        name = key;
                    }

                    if (unlocked.indexOf(name) < 0) {
                        unlocked.push(name);
                    }
                });
                SaveManager.save('unlockedLevels', unlocked);
                // also skip tutorials
                SaveManager.save('tutorialDailyShopItems', 2);
                SaveManager.save('tutorialEquipment', 2);
                SaveManager.save('tutorialFreeGift', 3);
                // SaveManager.save('tutorialChest', 2);
                SaveManager.save('tutorialCharacter', 2);
                SaveManager.save('tutorialWorldSelect', 2);
                SaveManager.save('tutorial', 3);

                Bento.screens.show('screens/worldselect');
            }
        });
        var buttonUnlockAllChars = new TextButton({
            name: 'buttonUnlockAll',
            type: 'medium',
            text: 'Unlock all characters',
            onButtonDown: function () {
                scrollingList.cancel();
            },
            onClick: function () {
                var available = CharacterManager.getAvailableCharacters();
                var unlocked = SaveManager.load('unlockedCharacters');

                Utils.forEach(available, function (character) {
                    if (unlocked.indexOf(character) < 0) {
                        unlocked.push(character);
                    }
                });
                SaveManager.save('unlockedCharacters', unlocked);
                Bento.screens.show('screens/worldselect');

            }
        });
        var buttonItems = new TextButton({
            name: 'buttonItems',
            type: 'medium',
            text: 'Get all items',
            onButtonDown: function () {
                scrollingList.cancel();
            },
            onClick: function () {
                var assets = Bento.assets.getAssets();
                var jsonAssets = assets.json;

                Utils.forEach(jsonAssets, function (item, key, l, breakLoop) {
                    // weapons count as consumables too
                    var keys = key.split('/');
                    var category = keys[0];
                    var array = category;
                    if (
                        category === 'weapons' ||
                        category === 'consumables'
                    ) {
                        InventoryManager.addItem(key, 1);
                    }
                });
            }
        });
        var buttonFreeGift = new TextButton({
            name: 'buttonFreeGift',
            text: 'Get free gift',
            type: 'medium',
            onButtonDown: function () {
                scrollingList.cancel();
            },
            onClick: function () {
                var giftInfo = SaveManager.load('gift24');
                giftInfo.time = 0;
                SaveManager.save('gift24', giftInfo);
                Bento.screens.show('screens/worldselect');
            }
        });
        var buttonScrolls = new TextButton({
            name: 'buttonScrolls',
            type: 'medium',
            text: 'Get all scrolls',
            onButtonDown: function () {
                scrollingList.cancel();
            },
            onClick: function () {
                if (!InventoryManager.getItem('scrolls/dash')) {
                    InventoryManager.addItem('scrolls/dash', 1);
                }
                if (!InventoryManager.getItem('scrolls/charge')) {
                    InventoryManager.addItem('scrolls/charge', 1);
                }
                // if (!InventoryManager.getItem('scrolls/grab')) {
                //     InventoryManager.addItem('scrolls/grab', 1);
                // }
                if (!InventoryManager.getItem('scrolls/jump')) {
                    InventoryManager.addItem('scrolls/jump', 1);
                }
                if (!InventoryManager.getItem('scrolls/super')) {
                    InventoryManager.addItem('scrolls/super', 1);
                }
            }
        });
        var buttonNotification = new TextButton({
            name: 'buttonNotification',
            type: 'medium',
            text: 'Test Notification',
            onButtonDown: function () {
                scrollingList.cancel();
            },
            onClick: function () {

                Utils.prompt('0: dailyGift, 1: energy, 2: dailyQuest. 3: inactive', function (str) {
                    var deltaTime = 1 / 60; //1 minute
                    var rand = parseInt(str);
                    var notificationId = ['gift24', 'energy', 'dailyQuest', 'inactive'][rand];
                    var notificationObj = ['dailyGift', 'energy', 'dailyQuest', 'inactive'][rand];
                    var savedNotifications = SaveManager.load('notifications');

                    if (!notificationObj) {
                        window.alert('Not valid');
                        return;
                    }

                    window.alert('Scheduled ' + notificationObj + ' in 1 minute');

                    // reset badge
                    Notifications.resetBadgeNumber();

                    // cancel old notification if needed
                    var oldId = savedNotifications[notificationId];
                    if (oldId) {
                        Notifications.cancel(oldId);
                    }

                    // schedule notification
                    var notifications = Utils.getNotifications({
                        Localization: Localization,
                        time: deltaTime * 60 * 60 * 1000
                    });
                    var id = Notifications.send(notifications[notificationObj]);

                    // save id so it can be cancelled if needed
                    savedNotifications[notificationId] = id;
                    SaveManager.save('notifications', savedNotifications);

                });
            }
        });

        var buttonCharacterLevel = new TextButton({
            name: 'buttonCharacterLevel',
            type: 'medium',
            text: 'Set character level',
            onButtonDown: function () {
                scrollingList.cancel();
            },
            onClick: function () {
                characterLevelPrompt();
            }
        });

        var buttonDailyQuests = new TextButton({
            name: 'buttonDailyQuests',
            type: 'medium',
            text: 'Reset daily quests',
            onButtonDown: function () {
                scrollingList.cancel();

            },
            onClick: function () {
                SaveManager.save('tutorialQuest', 3);

                if (SaveManager.load('questDay') !== -1) {
                    SaveManager.save('questDay', -1);
                } else {
                    var socialStatus = {
                        facebook: false,
                        instagram: false,
                        twitter: false,
                        weibo: false
                    };
                    SaveManager.save('socialMedia', socialStatus);
                }
                Bento.screens.show('screens/worldselect');
            }
        });
        var buttonMissionViewer = new TextButton({
            name: 'buttonMissionViewer',
            type: 'medium',
            text: 'Mission Viewer',
            onButtonDown: function () {
                scrollingList.cancel();
            },
            onClick: function () {
                Bento.audio.stopAllMusic();
                Bento.screens.show('screens/missionviewer');
            }
        });
        var buttonTowerChallenge = new TextButton({
            name: 'buttonTowerChallenge',
            type: 'medium',
            text: 'Tower Challenge',
            onButtonDown: function () {
                scrollingList.cancel();
            },
            onClick: function () {
                Bento.audio.stopAllMusic();
                // Bento.screens.show('screens/preloadtower');
                var startButton = new StartButton({
                    isTower: true,
                    mission: 'tower',
                    chosenDifficulty: 0,
                    difficulty: 0,
                    cost: 0,
                    unlocks: [],
                    rewards: []
                });
                startButton.start();
            }
        });
        var buttonErrors = new TextButton({
            name: 'buttonErrors',
            type: 'medium',
            text: 'Errors',
            onButtonDown: function () {
                scrollingList.cancel();
            },
            onClick: function () {
                var errorScroll = new ScrollingList({
                    z: 0,
                    position: new Vector2(0, 80),
                    direction: 'y', // 'x' or 'y'
                    spacing: 8, // additional spacing between items
                });
                var buttonCloseHistory = new ClickButton({
                    name: 'btnClose',
                    sfx: 'sfx_button',
                    imageName: 'gui2/bottom_tabs/bottom_tabs__back',
                    frameCountX: 2,
                    frameCountY: 1,
                    components: [
                        new Anchor({
                            portrait: {
                                screenPosition: new Vector2(0.5, 1),
                                offset: new Vector2(-48 + (Anchor.isWideScreen() ? 4 : 0), -14 + (Anchor.isWideScreen() ? Anchor.getOffset().y : 0))
                            },
                            landscape: {
                                screenPosition: new Vector2(0.25, 1),
                                offset: new Vector2(-44, -12)
                            }
                        })
                    ],
                    onButtonDown: function () {
                        errorScroll.cancel();
                    },
                    onClick: function () {
                        Bento.objects.remove(history);
                    }
                });
                var history = new Entity({
                    z: Globals.WorldSelectLayers.CHEAT,
                    name: 'historyMenu',
                    float: true,
                    components: [
                        new Modal({}),
                        new BlurBackground({
                            darken: 0.3,
                            blurriness: 15
                        }),
                        errorScroll,
                        buttonCloseHistory
                    ]
                });
                var errors = Bento.saveState.load('errorHistory', []);

                // add one dummy object to make it scrollable
                errorScroll.addItem(new Entity({
                    name: 'dummy'
                }));

                Utils.forEach(errors, function (error, i, l, breakLoop) {
                    var txt = new Text({
                        text: error.summary +
                            '\nScreen: ' + error.screen +
                            '\nFatal: ' + error.isFatal +
                            '\nComponent: ' + error.componentName +
                            '\nDate: ' + error.date,
                        position: new Vector2(4, 0),
                        font: 'helvetipixel',
                        fontSize: 12,
                        fontColor: '#ffffff',
                        align: 'left',
                        textBaseline: 'bottom',
                        maxWidth: viewport.width - 8,
                        linebreaks: true,
                        components: []
                    });
                    errorScroll.addItem(txt);
                });
                Bento.objects.attach(history);
            }
        });
        var buttonOffer = new TextButton({
            name: 'buttonOffer',
            text: 'Get offer',
            type: 'medium',
            onButtonDown: function () {
                scrollingList.cancel();
            },
            onClick: function () {
                SaveManager.save('offersBought', {});
                var offer = 'starter';
                Utils.prompt('0: dailyGift, 1: energy, 2: dailyQuest. 3: inactive', function (str) {
                    if (str) {
                        offer = str;
                        SaveManager.save('showOffer', offer);
                    }
                });
            }
        });
        var buttonCheatCheck = new TextButton({
            name: 'buttonCheatCheck',
            text: 'Cheater reset',
            type: 'medium',
            onButtonDown: function () {
                scrollingList.cancel();
            },
            onClick: function () {
                var isJailbreak = Bento.saveState.load('jailBreak', false);
                var isFakeIap = Bento.saveState.load('fakeIAP', false);
                window.alert("Jailbreak: " + isJailbreak + '  \n' + "Fake IAP: " + isFakeIap);
                Bento.saveState.save('fakeIAP', false);
                Bento.saveState.save('jailBreak', false);
                Bento.screens.show('screens/title');
            }
        });
        var buttonMission = new TextButton({
            name: 'buttonMission',
            type: 'medium',
            text: 'Mission...',
            onButtonDown: function () {
                scrollingList.cancel();
            },
            onClick: function () {
                Utils.prompt("Enter mission", function (res) {
                    var startButton = new StartButton({
                        mission: res,
                        chosenDifficulty: 0,
                        difficulty: 0,
                        cost: 0,
                        unlocks: [],
                        rewards: []
                    });
                    startButton.start();
                });
            }
        });
        var buttonRad = new TextButton({
            name: 'buttonRad',
            type: 'medium',
            text: 'RadEvent: ' + (Bento.saveState.load('radEventTest', false) ? 'on' : 'off'),
            onButtonDown: function () {
                scrollingList.cancel();
            },
            onClick: function () {
                if (Bento.saveState.load('radEventTest', false)) {
                    Bento.saveState.save('radEventTest', false);
                } else {
                    Bento.saveState.save('radEventTest', true);
                }
                Bento.screens.show('screens/worldselect');
            }
        });
        var characterLevelPrompt = function () {
            // function that runs on succes
            var checkCharacter = function (character) {
                var characters = CharacterManager.getAvailableCharacters();
                if (characters.indexOf(character) > -1) {
                    //succes
                    characterSucces(character);
                } else {
                    characterFail();
                }
            };

            if (Utils.isCocoonJS() && window.Cocoon) {
                window.Cocoon.Dialog.prompt({
                    title: 'Hello',
                    message: 'Which character do you want to edit?'
                }, {
                    success: function (text) {
                        checkCharacter(text);
                    },
                    fail: function () {
                        characterFail();
                    }
                });
            } else {
                var character = window.prompt("Which character do you want to edit?");
                checkCharacter(character);
            }
        };

        var characterFail = function () {
            if (Utils.isCocoonJS() && window.Cocoon) {
                window.Cocoon.Dialog.confirm({
                    title: "Character doesn't exist",
                    message: "Retry?"
                }, function (accepted) {
                    if (accepted) {
                        characterLevelPrompt();
                    }
                });
            } else {
                window.alert("Character doesn't exist");
            }
        };

        var characterSucces = function (character) {
            var level = 0;
            if (Utils.isCocoonJS() && window.Cocoon) {
                window.Cocoon.Dialog.prompt({
                    title: 'Hello',
                    message: 'What level should ' + character + ' be?'
                }, {
                    success: function (text) {
                        level = text;
                        if (level) {
                            SaveManager.save('level-' + character, parseInt(level));
                        }
                    }
                });
            } else {
                level = window.prompt('What level should ' + character + ' be?');
                if (level) {
                    SaveManager.save('level-' + character, parseInt(level));
                }
            }
        };

        var topbar = new Topbar({});

        var buttonClose = new ClickButton({
            z: 0,
            name: 'buttonClose',
            sfx: 'sfx_button',
            position: new Vector2(20, 20),
            imageName: 'gui/buttons/exit',
            frameCountX: 2,
            frameCountY: 1,
            onButtonDown: function () {
                scrollingList.cancel();
            },
            onClick: function () {
                Bento.objects.remove(entity);
            }
        });
        var entity = new Entity({
            z: Globals.WorldSelectLayers.CHEAT,
            name: 'cheatMenu',
            position: settings.position,
            float: true,
            components: [
                background,
                modal,
                scrollingList,
                topbar,
                buttonClose,
            ]
        });
        scrollingList.addItem(buttonReset);
        scrollingList.addItem(buttonSave);
        scrollingList.addItem(buttonLoad);
        scrollingList.addItem(buttonAd);
        scrollingList.addItem(buttonGems);
        scrollingList.addItem(buttonCoins);
        scrollingList.addItem(buttonKeys);
        scrollingList.addItem(buttonTokens);
        scrollingList.addItem(buttonEvolve);
        scrollingList.addItem(buttonGifts);
        scrollingList.addItem(buttonEnergy);
        scrollingList.addItem(buttonUnlockAll);
        scrollingList.addItem(buttonUnlockAllChars);
        scrollingList.addItem(buttonCharacterLevel);
        scrollingList.addItem(buttonItems);
        scrollingList.addItem(buttonFreeGift);
        scrollingList.addItem(buttonErrors);
        scrollingList.addItem(buttonScrolls);
        scrollingList.addItem(buttonDailyQuests);
        scrollingList.addItem(buttonMissionViewer);
        scrollingList.addItem(buttonHeyZap);
        scrollingList.addItem(buttonNotification);
        scrollingList.addItem(buttonOffer);
        scrollingList.addItem(buttonCheatCheck);
        scrollingList.addItem(buttonMission);
        scrollingList.addItem(buttonRad);

        return entity;
    };
});