/**
 * Credits menu
 * @moduleName Credits
 */
bento.define('gui/menus/credits', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'components/anchor',
    'gui/scrollinglist',
    'components/modal',
    'globals',
    'gui/blurbackground',
    'gui/shadowtext'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Anchor,
    ScrollingList,
    Modal,
    Globals,
    BlurBackground,
    ShadowText
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var scrollingList = new ScrollingList({
            spacing: 0
        }).attach(new Anchor({
            landscape: {
                screenPosition: new Vector2(0.5, 0.5),
                offset: new Vector2(0, 0)
            },
            portrait: {
                screenPosition: new Vector2(0.5, 0.5),
                offset: new Vector2(0, -64)
            }
        }));
        var blurBackground = new BlurBackground({
            darken: 0.2,
        });
        var creditsText = new ShadowText({
            position: new Vector2(0, 0),
            text: 'Made by Lucky Kat\n\n-Producer-\nHerdjie Zhou\n\n-Developers-\nHernan Zhou\nRowan Brak\nRobin Zaagsma\nHenri de Rochefort-Nash\n\n-Art-\nRik Nicol\nGregory Parisi\nHernan Zhou\nAnders Gullmarsvik\nPixelHans\nDiane de Wilde\n\n-Sound & Music-\nMaxo\n\n-Voices-\nLucy Patterson',
            font: 'font',
            fontSize: 16,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'top'
        });
        var luckyKat = new Entity({
            name: 'LuckyKat',
            components: [
                new Sprite({
                    imageName: 'luckykat-160',
                    originRelative: new Vector2(0.5, 0.5)
                })
            ]
        });
        var buttonClose = new ClickButton({
            z: 0,
            name: 'buttonClose',
            sfx: 'sfx_button',
            position: new Vector2(20, 20),
            imageName: 'gui/buttons/exit',
            frameCountX: 2,
            frameCountY: 1,
            onButtonDown: function () {
                // scrollingList.cancel();
            },
            onClick: function () {
                Bento.objects.remove(entity);
            }
        });
        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {
                luckyKat.dimension.height = 80;
                // creditsText.dimension.height += 80;
                scrollingList.addItem(luckyKat);
                scrollingList.addItem(creditsText);
                scrollingList.addItem(new Entity({}));

                scrollingList.forceTo(viewport.height);
            },
            destroy: function (data) {},
            update: function (data) {},
            draw: function (data) {}
        };
        var entity = new Entity({
            z: Globals.WorldSelectLayers.SETTINGS + 1,
            name: 'creditsMenu',
            position: settings.position,
            float: true,
            components: [
                new Modal({}),
                blurBackground,
                scrollingList,
                buttonClose,
                behavior
            ]
        });
        return entity;
    };
});