/**
 * Daily chest menu
 * @moduleName DailyChest
 */
bento.define('gui/menus/dailychest', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'bento/components/nineslice',
    'gui/blurbackground',
    'components/anchor',
    'components/modal',
    'globals',
    'gui/scrollinglist',
    'modules/localization',
    'components/masksprite',
    'gui/shadowtext',
    'gui/iconbutton',
    'gui/iap/chestoverlay',
    'components/darken',
    'components/sparkles',
    'components/glow',
    'managers/save',
    'managers/time',
    'modules/notifications',
    'gui/dialog',
    'gui/senseipopup',
    'components/backbutton'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    NineSlice,
    BlurBackground,
    Anchor,
    Modal,
    Globals,
    ScrollingList,
    Localization,
    MaskSprite,
    ShadowText,
    IconButton,
    ChestOverlay,
    Darken,
    Sparkles,
    Glow,
    SaveManager,
    TimeManager,
    Notifications,
    Dialog,
    SenseiPopup,
    BackButton
) {
    'use strict';
    /**
     * Notification
     */
    var askForNotification = function () {
        var hasAllowed = SaveManager.load('notificationsAllowed');
        var dialog = new Dialog({
            z: Globals.WorldSelectLayers.DIALOGS,
            name: 'askForNotification',
            type: 'yesno',
            title: Localization.getText('notifications')['askTitle'],
            text: Localization.getText('notifications')['ask'],
            canClose: false,
            icon: new Entity({
                components: [
                    new Sprite({
                        imageName: 'gui/notifications-icon',
                        originRelative: new Vector2(0.5, 0.5)
                    })
                ]
            }),
            onYes: function () {
                // ask for real notification
                Notifications.init(function (initialized) {
                    var allowed = initialized ? true : false;
                    SaveManager.save('notificationsAllowed', allowed);
                    if (allowed) {
                        // first time will be 24 hours from now
                        scheduleNotification(24);
                        // fire event to schedule inactive notification
                        EventSystem.fire('allowedNotifications');
                    } else {
                        // user pressed no at the system dialog...
                    }
                });
            },
            onNo: function () {
                // do nothing, better luck next time
            }
        });

        if (hasAllowed) {
            // already turned on notifications
            return;
        }

        Bento.objects.attach(dialog);

        var asked = SaveManager.load('notificationsAsked');
        asked += 1;
        SaveManager.save('notificationsAsked', asked);
    };
    // schedule notification for the 24h gift
    var scheduleNotification = function (deltaTime) {
        var savedNotifications = SaveManager.load('notifications');
        if (Utils.isDefined(deltaTime)) {
            deltaTime = 24;
        }
        // reset badge
        Notifications.resetBadgeNumber();

        // cancel old notification if needed
        var oldId = savedNotifications['gift24'];
        if (oldId) {
            Notifications.cancel(oldId);
        }

        // schedule notification
        var notifications = Utils.getNotifications({
            Localization: Localization,
            time: deltaTime * 60 * 60 * 1000
        });
        var id = Notifications.send(notifications.dailyGift);

        // save id so it can be cancelled if needed
        savedNotifications['gift24'] = id;
        SaveManager.save('notifications', savedNotifications);
    };

    var DailyChest = function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var dailyChestData = SaveManager.load('dailyChestData');
        var tutorialDailyChest = SaveManager.load('tutorialDailyChest');
        var blurBackground = new BlurBackground({
            darken: 0.2
        });
        var nineSlice = new NineSlice({
            imageName: 'gui2/misc/splash_background_9slice',
            originRelative: new Vector2(0.5, 0.5),
            width: 180,
            height: 180
        });
        var spacing = 72;
        var scrollingList = new ScrollingList({
            direction: 'x',
            spacing: spacing,
            snap: true
        });
        /**
         * Buttons
         */
        var buttonClose = new ClickButton({
            name: 'btnClose',
            sfx: 'sfx_button',
            imageName: 'gui2/bottom_tabs/bottom_tabs__back',
            frameCountX: 2,
            frameCountY: 1,
            onClick: function () {
                buttonClose.setActive(false);
                buttonClaim.setActive(false);
                entity.removeByName('modal');
                new Tween({
                    from: 1,
                    to: 0,
                    in: 15,
                    ease: 'linear',
                    onUpdate: function (v, t) {
                        entity.alpha = v;
                    },
                    onComplete: function () {
                        entity.removeSelf();
                        if (settings.onComplete) {
                            settings.onComplete();
                        }
                    }
                });
            },
            components: [
                new BackButton({
                    removeSelf: true,
                    onPressed: function (data) {
                        data.entity.doCallback();
                    }
                }),
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.5, 1),
                        offset: new Vector2(-48 + (Anchor.isWideScreen() ? 4 : 0), -14 + (Anchor.isWideScreen() ? Anchor.getOffset().y : 0))
                    },
                    landscape: {
                        screenPosition: new Vector2(0.25, 1),
                        offset: new Vector2(-44, -12)
                    }
                })
            ]
        });
        var buttonClaim = new IconButton({
            name: 'buttonUpgrade',
            sfx: 'sfx_button',
            position: new Vector2(0, 60),
            imageName: 'gui2/misc/button_large_primary_excited',
            frameCountX: 6,
            frameCountY: 7,
            padding: 1,
            animations: {
                up: {
                    speed: 0.34,
                    frames: [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35]
                },
                down: {
                    frames: [37]
                },
                inactive: {
                    frames: [38]
                }
            },
            icon: new ShadowText({
                position: new Vector2(0, -1),
                text: Localization.getText('claim'),
                font: 'font',
                fontSize: 20,
                fontColor: '#ffffff',
                align: 'center',
                textBaseline: 'middle',
                maxWidth: 80,
                maxHeight: 20
            }),
            onClick: function () {
                // open today's chest
                var chestOverlay = new ChestOverlay({
                    z: entity.z + 1,
                    chestImage: 'gui2/dailychests/chest-' + todaysChest,
                    contents: getChestContent(),
                    onComplete: function () {
                        // chest closed
                        // snap to next chest and add tomorrow text
                        if (todaysChest + 1 < scrollingList.getItems().length) {
                            scrollingList.snapTo(todaysChest + 1);
                            box.attach(tomorrowText);

                            // tween
                            new Tween({
                                from: 0,
                                to: 1,
                                in: 30,
                                ease: 'elastic',
                                decay: 5,
                                oscillations: 1,
                                onUpdate: function (v, t) {
                                    tomorrowText.scale.x = v;
                                    tomorrowText.scale.y = v;
                                }
                            });

                            // next chest starts sparkling
                            chests[todaysChest + 1].attach(new Sparkles({
                                offset: new Vector2(0, -16)
                            }));
                            chests[todaysChest + 1].attach(new Glow({}));
                        }

                        // allow scrolling
                        scrollingList.setActive(true);
                        if (tutorialDailyChest === 1 && !SaveManager.load('notificationsAllowed')) {
                            Utils.timeout(30, popupSensei);
                        } else {
                            // schedule next notification
                            if (SaveManager.load('notificationsAllowed')) {
                                scheduleNotification(24);
                            }
                        }
                        // allow close
                        entity.attach(buttonClose);
                        new Tween({
                            from: buttonClose.position.y + 64,
                            to: buttonClose.position.y,
                            in: 20,
                            ease: 'elastic',
                            decay: 8,
                            oscillations: 1,
                            onUpdate: function (v, t) {
                                buttonClose.position.y = v;
                            }
                        });

                        chests[todaysChest].getComponent('sprite', function (sprite) {
                            sprite.setAnimation('default');
                        });
                    }
                });
                Bento.objects.attach(chestOverlay);

                // update chest data
                dailyChestData.day = Math.floor(TimeManager.getLocalTime() / 1000 / 60 / 60 / 24);
                dailyChestData.claimed += 1;
                SaveManager.save('dailyChestData', dailyChestData);

                // remove claim button
                buttonClaim.removeSelf();

                // close current chest
                chests[todaysChest].removeByName('sparkles');
                chests[todaysChest].removeByName('glow');
                // chests[todaysChest].alpha = 0.5;
                chests[todaysChest].parent.getComponent('day', function (dayText) {
                    dayText.setText(Localization.getText('claimed'), {
                        fontColor: '23243a'
                    });
                });
            }
        });
        /**
         * Text after claiming
         */
        var tomorrowText = new ShadowText({
            position: new Vector2(0, 40),
            text: Localization.getText('dailyChestTomorrow'),
            font: 'font',
            fontSize: 16,
            // fontColor: '#b8a8c7',
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            maxWidth: 160,
            maxHeight: 48
        });
        /**
         * Title (only visible in portrait)
         */
        var titleText = new ShadowText({
            text: Localization.getText('dailyChest'),
            font: 'font',
            fontSize: 26,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'bottom',
            maxWidth: viewport.width - 32,
            maxHeight: 26
        }).attach(new Anchor({
            screenPosition: new Vector2(0.5, 0),
            offset: new Vector2(0, 48),
            onComplete: function () {
                if (Anchor.isPortrait()) {
                    titleText.visible = true;
                } else {
                    titleText.visible = false;
                }
            }
        }));
        /**
         * Inner nineslice box
         */
        var box = new Entity({
            name: 'box',
            components: [
                nineSlice,
                scrollingList,
                buttonClaim,
                // tomorrowText,
                new Anchor({
                    screenPosition: new Vector2(0.5, 0.5)
                })
            ]
        });
        var modal = new Modal({});
        var totalChests = 7; // see images in gui2/dailychests/ and getChestContent()
        var todaysChest = dailyChestData.claimed % totalChests;
        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {
                makeChests();
                Bento.audio.stopSound('sfx_chestdrop');
                Bento.audio.playSound('sfx_chestdrop');
                // add open sfx with a delay of 3 frames to have a smoother transition between drop and open sounds
                Utils.timeout(3, function () {
                    Bento.audio.stopSound('sfx_chestopen');
                    Bento.audio.playSound('sfx_chestopen');
                });

                // tween in
                new Tween({
                    from: 0,
                    to: 1,
                    in: 10,
                    ease: 'linear',
                    onUpdate: function (v, t) {
                        blurBackground.alpha = v;
                    }
                });
                new Tween({
                    from: 0,
                    to: 1,
                    in: 30,
                    ease: 'elastic',
                    decay: 8,
                    oscillations: 1,
                    onStart: function () {},
                    onUpdate: function (v, t) {
                        box.scale.x = v;
                        box.scale.y = v;
                    },
                    onComplete: function () {

                    }
                });

            },
            destroy: function (data) {},
            update: function (data) {},
            draw: function (data) {}
        };
        /**
         * Main entity
         */
        var entity = new Entity({
            z: Globals.WorldSelectLayers.DAILYCHEST,
            name: 'dailyChest',
            position: settings.position,
            float: true,
            components: [
                modal,
                blurBackground,
                box,
                titleText,
                behavior
            ]
        });
        /**
         * Make chests in the list 
         */
        var chests = [];
        var makeChests = function () {
            var makeChest = function (index) {
                var sprite = new Sprite({
                    imageName: 'gui2/dailychests/chest-' + index,
                    originRelative: new Vector2(0.5, 1),
                    frameCountX: 2,
                    frameCountY: 1,
                    animations: {
                        default: {
                            speed: 0,
                            frames: [0]
                        },
                        open: {
                            speed: 0,
                            frames: [1]
                        },
                        // TODO
                        empty: {
                            speed: 0,
                            frames: [1]
                        }
                    }
                });
                var maskSprite = new MaskSprite({
                    sprite: sprite,
                    boundingBox: new Rectangle(-83, -83, 166, 166)
                });
                var maskBehavior = {
                    name: 'setMask',
                    start: function () {
                        if (index === todaysChest) {
                            sprite.setAnimation('open');
                        }
                    },
                    update: function () {
                        var scroll = scrollingList.getScroll();
                        var dist;
                        // sets the maskSprite's boundingbox
                        maskSprite.boundingBox.x = -83 - chestContainer.position.x - scroll;
                        // * 4 because of text's sharpness
                        maskText.boundingBox.x = (-83 - chestContainer.position.x - scroll) * 4;

                        // scaling based on middle
                        dist = Math.abs(chestContainer.position.x + scroll);

                        chestContainer.scale.x = Utils.clamp(1, 1 + (spacing - dist) / spacing, 2);
                        chestContainer.scale.y = chestContainer.scale.x;
                    }
                };
                var chest = new Entity({
                    name: 'chest',
                    position: new Vector2(0, 0),
                    components: [
                        sprite,
                        maskSprite,
                        maskBehavior
                    ]
                });
                var day = new Text({
                    name: 'day',
                    position: new Vector2(0, -40),
                    text: Localization.getText('dayX').replace('{NUMBER}', index + 1),
                    font: 'font',
                    fontSize: 10,
                    fontColor: '#fffd00',
                    align: 'center',
                    textBaseline: 'middle'
                });
                var maskText = new MaskSprite({
                    sprite: day.getComponent('sharpnessScaler').getComponent('sprite'),
                    boundingBox: new Rectangle(-83, -83, 166 * 4, 166 * 4)
                });
                var chestContainer = new Entity({
                    name: 'container',
                    position: new Vector2(0, 16),
                    components: [
                        day,
                        maskText,
                        chest
                    ]
                });

                chests.push(chest);

                // today's chest
                if (index === todaysChest) {
                    chest.attach(new Sparkles({
                        offset: new Vector2(0, -16)
                    }));
                    chest.attach(new Glow({}));
                } else if (index < todaysChest) {
                    // past chest
                    // chest.alpha = 0.5;
                    day.setText(Localization.getText('claimed'), {
                        fontColor: '23243a'
                    });
                }

                chest.dimension.width = 64;
                scrollingList.addItem(chestContainer);

                return chestContainer;
            };
            var i = 0;

            Utils.repeat(totalChests, function () {
                makeChest(i);
                i += 1;
            });

            // scroll to today's chest
            scrollingList.snapTo(todaysChest);
            // can't scroll yet
            scrollingList.setActive(false);
        };
        /**
         * Determine chest content
         */
        var getChestContent = function () {
            var claimed = dailyChestData.claimed;
            var type = claimed % totalChests;
            var playerTokens = function (amount) {
                return {
                    fn: 'randomizeUpgradeItems',
                    amount: amount
                };
            };
            var enemyTokens = function (amount) {
                return {
                    fn: 'randomizeEnemyToken',
                    amount: amount,
                    param1: 1,
                    param2: 3
                };
            };
            var finishEnemyTokens = function () {
                return {
                    fn: 'finishEnemyTokens'
                };
            };
            var consumables = function (amount) {
                return {
                    fn: 'randomizeConsumableItems',
                    amount: amount
                };
            };
            var addGems = function (amount) {
                return {
                    fn: 'addGems',
                    amount: amount
                };
            };
            var chestContents = [];

            // Tutorial chest
            if (tutorialDailyChest === 0) {
                tutorialDailyChest = 1;
                SaveManager.save('tutorialDailyChest', tutorialDailyChest);
                return [{
                    key: 'playertokens/boy',
                    amount: 2
                }, {
                    key: 'playertokens/krillin',
                    amount: 1
                }];
            }

            switch (type) {
            case 0:
                chestContents = [playerTokens(3), enemyTokens(1), finishEnemyTokens()];
                break;
            case 1:
                chestContents = [playerTokens(3), enemyTokens(1), finishEnemyTokens(), consumables(1)];
                break;
            case 2:
                chestContents = [playerTokens(4), enemyTokens(2), finishEnemyTokens()];
                break;
            case 3:
                chestContents = [playerTokens(4), enemyTokens(2), finishEnemyTokens(), consumables(1)];
                break;
            case 4:
                chestContents = [playerTokens(4), enemyTokens(3), finishEnemyTokens()];
                break;
            case 5:
                chestContents = [playerTokens(4), enemyTokens(3), finishEnemyTokens(), consumables(1)];
                break;
            case 6:
                chestContents = [playerTokens(5), enemyTokens(3), finishEnemyTokens()];
                break;
            default:
                chestContents = [];
            }

            if (SaveManager.load('dailyGems') > 0) {
                SaveManager.add('dailyGems', -1);
                chestContents.push(addGems(
                    // 50 is the old value before patch
                    // the correct value is saved in offerpack.js
                    Bento.saveState.load('dailyGemsOfferAmount', 50)
                )); 
            }

            return chestContents;
        };
        /**
         * Notification
         */
        var popupSensei = function () {
            // tutorial: talk about notificaitons
            if (tutorialDailyChest === 1) {
                Bento.objects.attach(new SenseiPopup({
                    darken: true,
                    modal: true,
                    onNoMoreText: function () {
                        this.dismiss();
                    },
                    onComplete: function () {
                        tutorialDailyChest = 2;
                        SaveManager.save('tutorialDailyChest', tutorialDailyChest);

                        // ask for notifications
                        if (!SaveManager.load('notificationsAllowed')) {
                            askForNotification();
                        } else {
                            // schedule first notification
                            scheduleNotification(24);
                        }
                    },
                    text: Localization.getText('dailyChestNotification')
                }));
            }
        };
        return entity;
    };

    DailyChest.canClaim = function () {
        var dailyChestData = SaveManager.load('dailyChestData');
        var today = Math.floor(TimeManager.getLocalTime() / 1000 / 60 / 60 / 24);
        
        // day just needs to be different
        return today > dailyChestData.day;
    };

    return DailyChest;
});