/**
 * Daily Quest Menu
 * @moduleName DailyQuestMenu
 */
bento.define('gui/menus/dailyquest', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'globals',
    'components/anchor',
    'components/modal',
    'gui/iap/chestsection',
    'gui/iap/gemsection',
    'gui/iap/coinsection',
    'gui/iap/energysection',
    'gui/scrollinglist',
    'gui/worldselect/topbarsimple',
    'managers/save',
    'gui/taphand',
    'gui/highlight',
    'components/viewanalytics',
    'components/hidebelow',
    'gui/iap/offerpack',
    'managers/time',
    'components/orientationlock',
    'gui/iap/sectionlabel',
    'modules/localization',
    'gui/shadowtext',
    'gui/blurbackground',
    'managers/quest',
    'managers/currency',
    'gui/itemget',
    'modules/tenjin',
    'modules/analytics',
    'gui/dailyquest/questframe'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Globals,
    Anchor,
    Modal,
    ChestSection,
    GemSection,
    CoinSection,
    EnergySection,
    ScrollingList,
    Topbar,
    SaveManager,
    TapHand,
    Highlight,
    ViewAnalytics,
    HideBelow,
    OfferPack,
    TimeManager,
    OrientationLock,
    SectionLabel,
    Localization,
    ShadowText,
    BlurBackground,
    QuestManager,
    CurrencyManager,
    ItemGet,
    Tenjin,
    Analytics,
    QuestFrame
) {
    'use strict';
    return function (settings) {
        var viewport = Bento.getViewport();
        var invitation = settings.invitation;
        var portraitOffset = 54;
        var landScapeOffset = 37;
        var blurBackground = new BlurBackground({
            blurriness: 8,
            darken: 0.1,
        });
        var backgroundPortrait = new Entity({
            name: 'bgPortrait',
            components: [
                new Sprite({
                    imageName: 'gui/bg-brick-portrait',
                    originRelative: new Vector2(0.5, 0)
                }),
                new Anchor({
                    screenPosition: new Vector2(0.5, 0),
                    onComplete: function () {
                        if (Anchor.isPortrait()) {
                            backgroundPortrait.visible = true;
                            backgroundLandscape.visible = false;
                        } else {
                            backgroundPortrait.visible = false;
                            backgroundLandscape.visible = true;
                        }
                    }
                })
            ]
        });
        var backgroundLandscape = new Entity({
            name: 'bgLandscape',
            components: [
                new Sprite({
                    imageName: 'gui/bg-brick-landscape',
                })
            ]
        });
        if (Anchor.isWideScreen()) {
            backgroundLandscape.scale.x = Anchor.getWideScreenSize().width / backgroundLandscape.dimension.width;
            backgroundLandscape.transform.x -= Anchor.getOffset().x;
        }
        var modal = new Modal({});
        var topbar = new Topbar({});
        var finishedTutorial = SaveManager.load('tutorialQuest');

        var align = function (entity, isPortrait, component) {
            scrollinglist.setArea(new Rectangle(0, 0, viewport.width, viewport.height));
            if (isPortrait) {
                // scrollinglist.setActive(false);
                // scrollinglist.forceTo(0);
            } else {
                scrollinglist.setActive(true);
                // scrollinglist.setMaxOffset(-viewport.height + landScapeOffset + 53);
            }
        };

        var buttonClose = new ClickButton({
            name: 'buttonClose',
            position: new Vector2(20, 20),
            sfx: 'sfx_button',
            imageName: 'gui/buttons/exit',
            frameCountX: 2,
            frameCountY: 1,
            onClick: function () {
                new Tween({
                    from: 1,
                    to: 0,
                    in: 6,
                    ease: 'linear',
                    onUpdate: function (v, t) {
                        entity.alpha = v;
                    },
                    onComplete: function () {
                        this.removeByName('modal');
                        Bento.objects.remove(entity);
                        EventSystem.fire('questTutorial');
                        EventSystem.fire('dismissSensei');
                    }
                }).attach(new Modal({}));
            }
        });

        var dailyQuestLabel = new Entity({
            z: 0,
            name: 'dailyQuestLabel',
            position: new Vector2(0, 0),
            components: [
                new Sprite({
                    imageName: 'gui2/misc/title_bar',
                    originRelative: new Vector2(0.5, 0.5)
                }),
                new Text({
                    z: 0,
                    position: new Vector2(0, -16),
                    text: Localization.getText('dailyQuests'),
                    font: 'font',
                    fontSize: 16,
                    fontColor: '#ffdf00',
                    align: 'center',
                    textBaseline: 'middle',
                    ySpacing: 0,
                    maxWidth: 120,
                    maxHeight: 16,
                    linebreaks: false
                })
            ]
        });

        var scrollinglist = new ScrollingList({
            area: new Rectangle(0, 0, viewport.width, viewport.height),
            damping: 0.9,
            snap: false,
            direction: 'y',
            position: new Vector2(viewport.width / 2, portraitOffset),
        }).attach(
            new Anchor({
                portrait: {
                    offset: new Vector2(0, portraitOffset),
                },
                landscape: {
                    offset: new Vector2(0, landScapeOffset),
                },
                screenPosition: new Vector2(0.5, 0),
                onComplete: align
            })
        );

        var getRefreshTimer = function () {
            var text = Localization.getText('refreshQuestsIn');
            var currentTime = TimeManager.getTime();
            var nextDay = Math.ceil(TimeManager.getTime() / 1000 / 60 / 60 / 24) * 1000 * 60 * 60 * 24;
            var timeUntilNextDay = nextDay - currentTime;
            var hours = Math.floor(timeUntilNextDay / 1000 / 60 / 60);
            var minutes = Math.ceil(timeUntilNextDay / 1000 / 60) % 60;
            return text.replace('{HOURS}', hours).replace('{MINUTES}', minutes);
        };

        var timer = new ShadowText({
            z: 0,
            position: new Vector2(0, -1),
            text: getRefreshTimer(),
            font: 'font',
            fontSize: 12,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            ySpacing: 0,
            lineWidth: 0,
            strokeStyle: '#000000',
            innerStroke: false,
            pixelStroke: false // workaround for Cocoon bug
        }).attach({
            name: 'updateTimer',
            update: function () {
                // if(timer.timgetRefreshTimer()
                if (timer.timer % 60 === 0) {
                    timer.setText(getRefreshTimer());
                }
            }
        });

        var bottomBar = new Entity({
            name: 'bottomBar',
            position: new Vector2(0, 0),
            components: [
                new Sprite({
                    imageName: 'gui/worldselect/bottombar',
                    originRelative: new Vector2(0.5, 0.5)
                }),
                timer,
                new Anchor({
                    screenPosition: new Vector2(0.5, 1),
                    offset: new Vector2(0, -10)
                })
            ]
        });

        var questFrames = {};

        var setTutorialGift = function () {
            questFrames['-1'].getComponent('claimButton', function (claimButton) {
                claimButton.attach(new TapHand({
                    rotation: -Math.PI * 3 / 4
                }));
                claimButton.onComplete = function () {
                    buttonClose.attach(new TapHand({
                        rotation: Math.PI * 3 / 4
                    }));
                    finishedTutorial = 3;
                    SaveManager.save('tutorialQuest', finishedTutorial);
                };
            });
        };

        var totalClaimed = SaveManager.load('totalQuestsClaimed');
        var analytics = function (questObject) {
            if (questObject && questObject.quest) {
                if (questObject.quest.toString() !== '-1') {
                    totalClaimed += 1;
                    SaveManager.save('totalQuestsClaimed', totalClaimed);

                    var analyticType = 'normal';
                    if (questObject.quest === 'totalQuest' ||
                        questObject.quest === 'facebook' ||
                        questObject.quest === 'twitter' ||
                        questObject.quest === 'adQuest'
                    ) {
                        analyticType = questObject.quest;
                    }

                    // Tenjin
                    Tenjin.sendEvent('dailyQuests');
                    Tenjin.sendEventAndValue('dailyQuestsTotal', totalClaimed);

                    // Analytics
                    Analytics.event('dailyQuestsType', analyticType, 1);
                    if (questObject.quest.toString()) {
                        Analytics.event('dailyQuestsCompleted', QuestManager.names[questObject.quest].toString(), 1);
                    }
                } else {
                    // tutorial quest analytics
                    Tenjin.sendEvent('dailyQuestTutorialComplete');
                    Analytics.event('dailyQuests', 'tutorial', 1);
                }
            }
        };

        var updateFrames = function () {
            Utils.forEach(questFrames, function (frame, i, l, breakLoop) {
                frame.updateFrame();
            });
        };

        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {
                dailyQuestLabel.dimension.height = 22;
                scrollinglist.addItem(dailyQuestLabel);

                QuestManager.initialize();

                // array with quest objects
                var activeQuests = QuestManager.getQuests();
                var i = 0;
                var key;
                for (i = 0; i < activeQuests.length; i++) {
                    key = activeQuests[i].quest.toString();
                    questFrames[key] = new QuestFrame({
                        quest: activeQuests[i],
                        analytics: analytics
                    });
                }

                // add the frames to the scrolling list
                Utils.forEach(questFrames, function (frame, ii, l, breakLoop) {
                    scrollinglist.addItem(frame);
                });

                new Tween({
                    from: 0,
                    to: 1,
                    in: 15,
                    onUpdate: function (v, t) {
                        blurBackground.alpha = v;
                    }
                });

                // tween in
                new Tween({
                    from: 0,
                    to: 1,
                    in: 30,
                    ease: 'elastic',
                    decay: 8,
                    oscillations: 1,
                    onUpdate: function (v, t) {
                        scrollinglist.scale.x = v;
                        scrollinglist.scale.y = v;
                    }
                });

                if (finishedTutorial === 2) {
                    setTutorialGift();
                }

                EventSystem.on('updateDailyQuest', updateFrames);
            },
            destroy: function (data) {
                EventSystem.off('updateDailyQuest', updateFrames);
            },
            update: function (data) {},
            draw: function (data) {}
        };

        var entity = new Entity({
            z: Globals.WorldSelectLayers.QUESTS,
            name: 'dailyQuests',
            float: true,
            components: [
                modal,
                blurBackground,
                backgroundPortrait,
                backgroundLandscape,
                scrollinglist,
                bottomBar,
                topbar,
                // buttonClose,
                behavior
            ]
        });

        Analytics.trackView('dailyQuest');
        
        return entity;
    };
});