/**
 * Game over menu
 * @moduleName GameOverMenu
 */
bento.define('gui/menus/gameover', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'components/fillscreen',
    'globals',
    'components/anchor',
    'modules/matchmanager',
    'gui/blurbackground',
    'gui/shadowtext',
    'modules/localization',
    'managers/inventory',
    'gui/playercoins',
    'components/eventlistener',
    'managers/currency',
    'managers/save',
    'managers/character',
    'managers/stats',
    'managers/coincollector',
    'managers/buff',
    'gui/buttons/startbutton',
    'managers/ai',
    'modules/analytics',
    'components/viewanalytics',
    'gui/dialog',
    'gui/menus/iap',
    'gui/worldselect/topbarsimple',
    'gui/continuepopup',
    'gui/watchrewardedad'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    FillScreen,
    Globals,
    Anchor,
    MatchManager,
    BlurBackground,
    ShadowText,
    Localization,
    InventoryManager,
    PlayerCoins,
    EventListener,
    CurrencyManager,
    SaveManager,
    CharacterManager,
    StatsManager,
    CoinCollector,
    BuffManager,
    StartButton,
    AiManager,
    Analytics,
    ViewAnalytics,
    Dialog,
    IAPMenu,
    TopBarSimple,
    ContinuePopup,
    WatchRewardedAd
) {
    'use strict';
    return function (settings) {
        // single player: show continuepopup
        if (!MatchManager.isOnline) {
            return new ContinuePopup(settings);
        }

        // how much should it cost to revive??
        var viewport = Bento.getViewport();
        var portrait = true;
        var tweened = false;
        var currentCharacter = SaveManager.load('selectedCharacter');
        var reviveCost = Globals.gemToRevive(SaveManager.load('currentMission'));

        var fillScreen = new FillScreen({
            color: [0, 0, 0, 0.3]
        });
        var darken = new Entity({
            name: 'darken',
            components: [
                fillScreen
            ]
        });
        var fadeOut = function () {
            new Tween({
                from: 0,
                to: 1,
                in: 30,
                ease: 'linear',
                onUpdate: function (v, t) {
                    gameOverText.alpha = 1 - v;
                    btnContinue.alpha = 1 - v;
                    btnQuit.alpha = 1 - v;
                    faintedPlayer.alpha = 1 - v;
                    energyIcon.alpha = 1 - v;
                    toolTip.alpha = 1 - v;
                    fillScreen.setColor([0, 0, 0, 0.3 + v * 0.7]);
                },
                onComplete: function () {
                    Analytics.event('Economy', 'spendGemsForRevive', 0, 'no');
                    // mark 1st time death
                    if (SaveManager.load('tutorial1stTimeDeath') === 0) {
                        SaveManager.save('tutorial1stTimeDeath', 1);
                    } else
                    if (SaveManager.load('tutorial1stTimeDeath') === 2) {
                        SaveManager.save('tutorial1stTimeDeath', 3);
                    }
                    var isTower = SaveManager.load('isTower');
                    if (SaveManager.load('isMissionOnline') || isTower) {
                        WatchRewardedAd.showInterstitial(function () {
                            if (isTower) {
                                EventSystem.fire('endTower');
                            }
                            Bento.screens.show('screens/worldselect', {
                                target: isTower ? 'tower' : null
                            });
                        });
                        return;
                    }

                    Bento.screens.show('screens/worldselect');
                }
            });
            // fade out music entirely
            new Tween({
                from: 0.25,
                to: 0,
                in: 30,
                onUpdate: function (v, t) {
                    Bento.audio.setVolume(v, Globals.currentMusic);
                }
            });
        };
        var isEveryoneDead = function () {
            var everyoneDead = true;

            // TODO: it's not entirely accurate to look at the stateMachine components because there might be
            // a message incoming that a player revived but hasn't arrived yet
            // to truly check if everyone is dead, we must confirm this with everyone
            Utils.forEach(Bento.objects.getByFamily('player'), function (player, i, l, breakLoop) {
                player.getComponent('stateMachine', function (stateMachine) {
                    if (stateMachine.stateName !== 'dead') {
                        everyoneDead = false;
                    }
                });
            });

            return everyoneDead;
        };

        var previousCameraTarget = null;
        var behavior = {
            name: 'behaviorComponent',
            revive: function () {
                Analytics.event('Economy', 'spendGemsForRevive', 1, 'yes');

                Utils.forEach(Bento.objects.getByFamily('player'), function (player, i, l, breakLoop) {
                    if (!player.isRemote) {
                        player.getComponent('stateMachine', function (stateMachine) {
                            stateMachine.setState('getUp');
                        });
                        player.getComponent('punchable', function (punchable) {
                            punchable.resetHealth();
                        });
                        AiManager.distributeAiTasks();
                        breakLoop();
                    }
                });
                Bento.audio.stopSound('sfx_revive');
                Bento.audio.playSound('sfx_revive');
            },
            start: function (data) {
                var currentMission = SaveManager.load('currentMission');
                var currentSection = SaveManager.load('currentSection');
                var currentLevel = StatsManager.getLevel(CharacterManager.getSelected());
                Analytics.event('Game', 'deathsTotal');
                Analytics.event('Deaths', 'mission_' + currentMission, currentLevel, currentSection.toString());
                Analytics.event('Deaths', 'section_' + currentMission + '_' + currentSection, currentLevel);

                // analytics before the first victory
                if (SaveManager.load('missionStars-' + currentMission) < 0) {
                    Analytics.event('LevelMissionFail', currentMission, currentLevel);
                }


                if (!MatchManager.isOnline) {
                    // true game over
                    Bento.objects.pause(Globals.PauseLevels.PAUSE);
                } else {
                    // if there is a pause menu, get rid of it
                    Bento.objects.get('pauseMenu', function (pauseMenu) {
                        pauseMenu.removeSelf();
                    });
                    // deactivate pause button
                    Bento.objects.get('pauseButton', function (pauseButton) {
                        pauseButton.setActive(false);
                    });
                    // deactivate controls
                    EventSystem.fire('disableControls');

                    // focus camera on another player?
                    // store original camera target
                    Bento.objects.get('camera', function (camera) {
                        camera.getComponent('cameraBehavior', function (cameraBehavior) {
                            previousCameraTarget = cameraBehavior.target;

                        });
                    });
                    behavior.findNewCameraTarget();
                }
                // drop the gameover
                new Tween({
                    from: -32,
                    to: viewport.height / 4,
                    in: 60,
                    delay: 0,
                    ease: 'easeOutBounce',
                    onUpdate: function (v, t) {
                        gameOverText.position.y = v;
                    },
                    onComplete: function () {
                        // attach home button
                        entity.attach(btnQuit);
                        btnQuit.scale = new Vector2(0, 0);
                        new Tween({
                            from: 0,
                            to: 1,
                            in: 30,
                            delay: 15,
                            ease: 'elastic',
                            decay: 4,
                            oscillations: 2,
                            onUpdate: function (v, t) {
                                btnQuit.scale.x = v;
                                btnQuit.scale.y = v;
                            }
                        });
                        var button;
                        button = btnContinue;

                        entity.attach(button);
                        new Tween({
                            from: 0,
                            to: 1,
                            in: 30,
                            ease: 'elastic',
                            decay: 4,
                            oscillations: 2,
                            onUpdate: function (v, t) {
                                button.scale.x = v;
                                button.scale.y = v;
                            }
                        });

                        if (SaveManager.load('isTower')) {
                            if (!SaveManager.load('hasRevived')) {
                                SaveManager.save('hasRevived', true);
                            } else {
                                // remove revive button: only revive once
                                button.removeSelf();
                                btnQuit.position.x = viewport.width / 2;
                            }
                        }
                    }
                });
                //background tween
                new Tween({
                    from: 0,
                    to: 1,
                    in: 20,
                    ease: 'linear',
                    onUpdate: function (v, t) {
                        background.alpha = v;
                        darken.alpha = v;
                    }
                });
                //player tween
                faintedPlayer.alpha = 0;
                new Tween({
                    from: 0,
                    to: 1,
                    in: 40,
                    delay: 20,
                    ease: 'linear',
                    onUpdate: function (v, t) {
                        faintedPlayer.alpha = v;
                    }
                });

                // turn volume down
                new Tween({
                    from: 1,
                    to: 0.25,
                    in: 30,
                    onUpdate: function (v, t) {
                        Bento.audio.setVolume(v, Globals.currentMusic);
                    }
                });
            },
            update: function (data) {
                // look for new target every few seconds
                if (previousCameraTarget !== null && data.entity.timer % 120 === 0 && !isFadingOut) {
                    behavior.findNewCameraTarget();
                }
            },
            destroy: function (data) {
                Bento.objects.resume();

                // reactivate if necessary
                Bento.objects.get('pauseButton', function (pauseButton) {
                    pauseButton.setActive(true);
                });
                EventSystem.fire('enableControls');
            },
            findNewCameraTarget: function () {
                Bento.objects.get('camera', function (camera) {
                    camera.getComponent('cameraBehavior', function (cameraBehavior) {
                        // pick a new target
                        Utils.forEach(Bento.objects.getByFamily('player'), function (player, i, l, breakLoop) {
                            player.getComponent('stateMachine', function (stateMachine) {
                                if (stateMachine.stateName !== 'dead') {
                                    cameraBehavior.target = player;
                                }
                            });
                        });
                    });
                });
            }
        };

        var gameOverText = new ShadowText({
            position: new Vector2(0, 0),
            text: Localization.getText('gameOver'),
            font: 'font',
            fontSize: 24,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle'
        });
        gameOverText.attach(new Anchor({
            screenPosition: new Vector2(0.5, 0.25),
            offset: new Vector2(0, 0)
        }));


        // current characters energy and the energy icon
        var characterEnergy = new ShadowText({
            position: new Vector2(-6, 22),
            text: CharacterManager.getEnergy(currentCharacter) + '/10',
            font: 'font',
            fontSize: 16,
            align: 'left',
            textBaseline: 'bottom',
        });
        var characterEnergyIcon = new Entity({
            name: 'icon',
            position: new Vector2(-16, 22 - 6),
            components: [
                new Sprite({
                    imageName: 'gui/startlevel/energyicon',
                    originRelative: new Vector2(0.5, 0.5)
                })
            ]
        });

        // current character lying down
        var faintedPlayer = new Entity({
            z: 0,
            name: 'faintedPlayer',
            position: new Vector2(0, viewport.height * 0.35),
            components: [
                new Sprite({
                    spriteSheet: currentCharacter + '/liedown'
                }),
                characterEnergy,
                characterEnergyIcon
            ]
        }).attach(
            new Anchor({
                screenPosition: new Vector2(0.5, 0.35)
            })
        );

        // needs energy
        var costText = new ShadowText({
            position: new Vector2(-4, -10),
            originRelative: new Vector2(0, 0.5),
            text: Localization.getText('needs').replace('{NUMBER}', ''),
            font: 'font',
            fontSize: 16,
            fontColor: '#ffffff',
            align: 'right',
            textBaseline: 'top',
            strokeStyle: '#000000',
        });
        var energyText = new ShadowText({
            position: new Vector2(10, -10),
            originRelative: new Vector2(0, 0.5),
            text: SaveManager.load('currentCost').toString(),
            font: 'font',
            fontSize: 16,
            fontColor: '#ffffff',
            align: 'left',
            textBaseline: 'top',
            strokeStyle: '#000000',
        });
        var energyIcon = new Entity({
            name: 'icon',
            position: new Vector2(14, -28),
            components: [
                new Sprite({
                    imageName: 'gui/startlevel/energyicon',
                    originRelative: new Vector2(0.5, 0.5)
                }),
                costText,
                energyText
            ]
        });

        var continueText = new ShadowText({
            position: new Vector2(0, -28),
            text: Localization.getText('continue'),
            font: 'font',
            fontSize: 16,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle'
        });
        var btnContinue = new ClickButton({
            z: 0,
            name: 'btnContinue',
            sfx: 'sfx_button',
            imageName: 'gui/buttons/continue',
            frameCountX: 2,
            frameCountY: 1,
            float: false,
            updateWhenPaused: 0,
            onClick: function () {
                if (CurrencyManager.canSpendGems(reviveCost)) {
                    CurrencyManager.spendGems(reviveCost, 'revive');
                    SaveManager.saveToCloud();
                    behavior.revive();
                    EventSystem.fire('revivedPlayer');
                    // remove game over menu
                    fadeOutContinue();
                    btnContinue.setActive(false);
                    btnQuit.setActive(false);
                } else {
                    var iapMenu = new IAPMenu({
                        z: Globals.Layers.SHOP + 1,
                        scrollTo: 'gems'
                    });
                    Bento.objects.attach(iapMenu);
                }
            }
        }).attach(continueText);
        var isFadingOut = false;
        var fadeOutContinue = function () {
            isFadingOut = true;
            // restore camera
            Bento.objects.get('camera', function (camera) {
                camera.getComponent('cameraBehavior', function (cameraBehavior) {
                    // cameraBehavior.target = previousCameraTarget;

                    // just find the local player
                    Utils.forEach(Bento.objects.getByFamily('player'), function (player, i, l, breakLoop) {
                        if (!player.isRemote) {
                            cameraBehavior.target = player;
                            breakLoop();
                        }
                    });
                });
            });
            // restore music
            new Tween({
                from: 0.25,
                to: 1,
                in: 30,
                onUpdate: function (v, t) {
                    Bento.audio.setVolume(v, Globals.currentMusic);
                }
            });

            new Tween({
                from: 1,
                to: 0,
                in: 60,
                onUpdate: function (v, t) {
                    entity.alpha = v;
                },
                onComplete: function () {
                    Bento.objects.remove(entity);
                }
            });
        };
        btnContinue.attach(new ShadowText({
            position: new Vector2(9, -1),
            text: reviveCost.toString(),
            font: 'font',
            fontSize: 16,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            maxWidth: 23,
            maxHeight: 16
        }));
        btnContinue.attach(new Anchor({
            screenPosition: new Vector2(0.5, 1),
            offset: new Vector2(32, -32)
        }));

        var btnQuit = new ClickButton({
            name: 'btnQuit',
            sfx: 'sfx_button',
            imageName: 'gui/buttons/home',
            frameCountX: 1,
            frameCountY: 2,
            onClick: function () {
                fadeOut();
                btnContinue.setActive(false);
                btnQuit.setActive(false);

                var coinsCollected = Math.ceil(CoinCollector.getCollected()) + BuffManager.current.extraCoins;
                CurrencyManager.addCoins(coinsCollected);
            }
        });
        btnQuit.attach(new Anchor({
            screenPosition: new Vector2(0.5, 1),
            offset: new Vector2(-32, -32)
        }));

        var toolTip = new ShadowText({
            z: 0,
            position: new Vector2(0, 0),
            text: Localization.getText('gameoverTip'),
            font: 'font',
            fontSize: 12,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'bottom',
            maxWidth: viewport.width - 48,
            maxHeight: 34
        }).attach(
            new Anchor({
                screenPosition: new Vector2(0.5, 1),
                offset: new Vector2(0, -8)
            })
        );

        var bossWinEventListener = new EventListener({
            name: 'bossWin',
            onEvent: function () {
                // weird situation: local player is dead, but the other players defeated the boss
                // we remove the game over overlay
                Bento.objects.remove(entity);
            }
        });

        var entity = new Entity({
            z: Globals.Layers.PAUSE,
            name: 'gameOverMenu',
            position: new Vector2(0, 0),
            updateWhenPaused: Globals.PauseLevels.PAUSE,
            float: true,
            components: [
                darken,
                behavior,
                // faintedPlayer,
                gameOverText,
                // toolTip,
                bossWinEventListener,
                new TopBarSimple({}),
                new ViewAnalytics({
                    name: 'gameOver'
                })
            ]
        });
        var background = new BlurBackground({});

        // blur background only when player is offline
        if (!MatchManager.isOnline) {
            entity.attach(background);
            entity.moveComponentTo(background, 0);
        }

        entity.extend({
            isEveryoneDead: isEveryoneDead
        });

        return entity;
    };
});