/**
 * Gift Menu
 * @moduleName GiftMenu
 */
bento.define('gui/menus/gift', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'globals',
    'components/anchor',
    'components/modal',
    'gui/worldselect/bargems',
    'gui/worldselect/barcoins',
    'gui/gift/giftframe',
    'managers/save',
    'managers/time',
    'managers/character',
    'gui/shadowtext',
    'modules/localization',
    'ads/heyzap',
    'gui/dialog',
    'gui/watchrewardedad',
    'gui/taphand',
    'components/viewanalytics',
    'components/hidebelow',
    'modules/notifications',
    'components/orientationlock',
    'entities/character',
    'entities/effect',
    'components/sparkles',
    'components/blink',
    'gui/sparkleburst',
    'gui/blurbackground',
    'modules/analytics',
    'gui/worldselect/topbar',
    'gui/iconbutton',
    'managers/currency'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Globals,
    Anchor,
    Modal,
    Bargems,
    Barcoins,
    GiftFrame,
    SaveManager,
    TimeManager,
    CharacterManager,
    ShadowText,
    Localization,
    HeyZap,
    Dialog,
    WatchRewardedAd,
    TapHand,
    ViewAnalytics,
    HideBelow,
    Notifications,
    OrientationLock,
    Character,
    Effect,
    Sparkles,
    Blink,
    SparkleBurst,
    BlurBackground,
    Analytics,
    Topbar,
    IconButton,
    CurrencyManager
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var portrait = true;
        var topOffset = 42;
        // var isOnline = TimeManager.online || Utils.isDev();
        var isOnline = true;
        var speedMultiplier = 1;
        var finishedTutorial = SaveManager.load('tutorialFreeGift');
        var getGemAmount = function () {
            if (finishedTutorial === 0) {
                // give enough gems to buy lowest tier gems
                return Globals.chests.classic.cost;
            }
            return 5;
        };
        var align = function (entity, isPortrait, component) {
            portrait = isPortrait;
            if (isPortrait) {
                backgroundPortrait.visible = true;
                backgroundLandscape.visible = false;
            } else {
                backgroundLandscape.visible = true;
                backgroundPortrait.visible = false;
            }
            //reset scrollinglist
            // scrollinglist.setArea(new Rectangle(0, 0, viewport.width, viewport.height));
            // scrollinglist.setMaxOffset(-viewport.height + topOffset + sectionEnergy.dimension.height);
        };
        var blurBackground = new BlurBackground({
            darken: 0.2,
        });
        var backgroundPortrait = new Entity({
            name: 'bgPortrait',
            components: [
                new Sprite({
                    imageName: 'gui/bg-brick-portrait',
                    originRelative: new Vector2(0.5, 0)
                }),
                new Anchor({
                    screenPosition: new Vector2(0.5, 0)
                })
            ]
        });
        var backgroundLandscape = new Entity({
            name: 'bgLandscape',
            components: [
                new Sprite({
                    imageName: 'gui/bg-brick-landscape',
                })
            ]
        });
        var titleText = new ShadowText({
            text: Localization.getText('dailyChest'),
            font: 'font',
            fontSize: 24,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'bottom',
            maxWidth: viewport.width - 32,
            maxHeight: 24
        }).attach(new Anchor({
            screenPosition: new Vector2(0.5, 0),
            offset: new Vector2(0, 48),
            onComplete: function (parent, isPortrait) {
                titleText.setText(Localization.getText('dailyChest'), {
                    maxWidth: viewport.width - 32,
                    maxHeight: 24
                });

                if (isPortrait) {
                    titleText.position.y += 32;
                }
            }
        }));

        var btnClose = new ClickButton({
            name: 'btnClose',
            sfx: 'sfx_button',
            imageName: 'gui2/bottom_tabs/bottom_tabs__back',
            frameCountX: 2,
            frameCountY: 1,
            onClick: function () {
                moveOut();
            },
            components: [
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.5, 1),
                        offset: new Vector2(-48 + (Anchor.isWideScreen() ? 4 : 0), -14 + (Anchor.isWideScreen() ? Anchor.getOffset().y : 0))
                    },
                    landscape: {
                        screenPosition: new Vector2(0.25, 1),
                        offset: new Vector2(-44, -12)
                    }
                })
            ]
        });

        var topBar = new Topbar({});

        var anchor = new Anchor({
            onComplete: align
        });

        // move the menu into the screen on start
        var moveIn = function () {
            var fromPosition;
            if (portrait) {
                fromPosition = viewport.height;
            } else {
                fromPosition = viewport.width;
            }
            new Tween({
                from: 0,
                to: 1,
                in: 5,
                onUpdate: function (v, t) {
                    gifts.alpha = v;
                }
            }).attach(new OrientationLock({}));
        };

        // move the menu out of the screen
        var moveOut = function () {
            var toPosition;
            btnClose.setActive(false);
            if (portrait) {
                toPosition = viewport.height;
            } else {
                toPosition = viewport.width;
            }
            new Tween({
                from: 1,
                to: 0,
                in: 10,
                onStart: function () {
                    gifts.removeByName('modal');
                },
                onUpdate: function (v, t) {
                    gifts.alpha = v;
                },
                onComplete: function () {
                    blurBackground.removeSelf();
                    Bento.objects.remove(gifts);
                    EventSystem.fire('freeGiftClosed');
                }
            }).attach(new OrientationLock({}));
        };

        var setRewards = function () {
            var gift24 = SaveManager.load('gift24');
            if (gift24.reward === 'none') {
                resetReward('24');
            }
        };

        var pickRandomTokens = function (gift) {
            if (lockedTokens.length === 0 || Math.random() <= 0.8) {
                gift.reward = unlockedTokens[Utils.getRandom(unlockedTokens.length)];
            } else {
                gift.reward = lockedTokens[Utils.getRandom(lockedTokens.length)];
            }
        };

        var resetReward = function (id) {
            var gift;
            var currentTime = TimeManager.getTime();
            id = id.toString();
            if (id === '24') {
                // reset all
                gift = SaveManager.load('gift24');
                gift.reward = 'currency/gem';
                gift.amount = getGemAmount();
                gift.time = currentTime + 3 * 3600000;
                SaveManager.save('gift24', gift);
            }
        };

        var showAdButton = function () {
            if (!adButton) {
                return;
            }
            if (isOnline) {
                var adInfo = SaveManager.load('giftAd');
                if (adInfo.views >= 0) {
                    speedMultiplier = 1 + adInfo.views / 4;
                } else {
                    speedMultiplier = 1;
                }
                if (adInfo.views >= 0 && adInfo.views < Globals.maxAdsFreeGift && !giftFrame.isActive()) {
                    if (!adButton.parent) {
                        gifts.attach(adButton);
                        adButton.setActive(true);
                    }
                } else {
                    if (adButton.isActive()) {
                        adButton.setActive(false);
                        new Tween({
                            from: adButton.position.x,
                            to: viewport.width + 80,
                            in: 30,
                            delay: 0,
                            ease: 'easeInBack',
                            onStart: function () {},
                            onUpdate: function (v, t) {
                                adButton.position.x = v;
                            },
                            onComplete: function () {
                                gifts.remove(adButton);
                            }
                        });
                    }
                }
                // check time, reset when over
                if (adInfo.time !== 'unset') {
                    if (adInfo.time - TimeManager.getTime() <= 0) {
                        adInfo.views = 0;
                        adInfo.time = 'unset';
                        SaveManager.save('giftAd', adInfo);
                    }
                }
            }
        };

        var createAdButton = function (data) {
            var speedUpText = new ShadowText({
                position: new Vector2(0, -30),
                text: '',
                font: 'font',
                fontSize: 16,
                fontColor: '#ffffff',
                align: 'center',
                textBaseline: 'middle'
            });
            var onComplete = function (hasCompleted) {
                if (hasCompleted) {
                    // half all reward times
                    var currentTime = TimeManager.getTime();
                    var halfTime = function (id) {
                        var gift = SaveManager.load('gift' + id);
                        gift.time = (gift.time - currentTime) / 2 + currentTime;
                        SaveManager.save('gift' + id, gift);

                        if (id === '24' && SaveManager.load('notificationsAllowed')) {
                            // reschedule 24 hour gift
                            // schedule notification
                            var notifications = Utils.getNotifications({
                                Localization: Localization,
                                time: Math.abs(gift.time - currentTime) * 60 * 60 * 1000
                            });
                            var savedNotifications = SaveManager.load('notifications');

                            // cancel old notification
                            Notifications.cancel(savedNotifications['gift24']);

                            // schedule new one and save new id
                            var identifier = Notifications.send(notifications.dailyGift);
                            savedNotifications['gift24'] = identifier;
                            SaveManager.save('notifications', savedNotifications);
                        }
                    };
                    halfTime('24');
                    halfTime('12');
                    halfTime('8');
                    var adInfo = SaveManager.load('giftAd');
                    if (adInfo.time === 'unset') {
                        // adInfo.time = currentTime + 60000;
                        adInfo.time = currentTime + 3 * 3600000;
                    }
                    adInfo.views += 1;
                    speedMultiplier = 1 + adInfo.views / 4;
                    SaveManager.save('giftAd', adInfo);

                    Analytics.event('Ads', 'watch_for_speedup_x' + adInfo.views);

                    giftFrame.getComponent('timeFrame', function (timeFrame) {
                        timeFrame.getComponent('progressBar', function (progressBar) {
                            progressBar.attach(new Blink({}));
                        });
                    });
                } else {
                    // nothing
                }
            };
            var adButton = new IconButton({
                    z: 0,
                    name: 'adbutton',
                    sfx: 'sfx_button',
                    imageName: 'gui2/misc/button_small_primary_excited',
                    iconName: 'gui2/misc/button_icon_watchad',
                    frameCountX: 6,
                    frameCountY: 7,
                    padding: 1,
                    animations: {
                        up: {
                            speed: 0.34,
                            frames: [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35]
                        },
                        down: {
                            frames: [37]
                        },
                        inactive: {
                            frames: [38]
                        }
                    },
                    onClick: function () {
                        WatchRewardedAd(onComplete);
                    }
                })
                .attach(speedUpText)
                .attach(new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.5, 0),
                        offset: new Vector2(0, 261)
                    },
                    landscape: {
                        screenPosition: new Vector2(1, 0.5),
                        offset: new Vector2(-48, 59)
                    }
                }));

            return adButton;
        };

        var consumables = [];
        var unlockedTokens = [];
        var lockedTokens = [];

        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {
                if (finishedTutorial > 2) {
                    adButton = createAdButton();
                }

                moveIn();

                // fill consumables array
                var assets = Bento.assets.getAssets();
                var jsonAssets = assets.json;
                Utils.forEach(jsonAssets, function (item, key, l, breakLoop) {
                    // weapons count as consumables too
                    if (key.indexOf('consumables/') === 0 || key.indexOf('weapons/') === 0) {
                        consumables.push({
                            key: key,
                            tier: (item.inventory || {}).tier || 0
                        });
                    }
                });

                // check locked and unlocked characters
                var lockedCharacters = CharacterManager.getAvailableCharacters();
                var myCharacters = CharacterManager.getUnlockedCharacters();
                var i;
                var ii;
                for (i = myCharacters.length - 1; i >= 0; i--) {
                    ii = lockedCharacters.indexOf(myCharacters[i]);
                    if (ii !== -1) {
                        lockedCharacters.splice(ii, 1);
                    }
                }

                // fill array with locked tokens
                i = lockedCharacters.length;
                ii = 0;
                Utils.repeat(i, function () {
                    lockedTokens.push('upgrades/health-' + lockedCharacters[ii]);
                    lockedTokens.push('upgrades/attack-' + lockedCharacters[ii]);
                    lockedTokens.push('upgrades/skill-' + lockedCharacters[ii]);
                    ++ii;
                });

                // fill array with unlocked tokens
                i = myCharacters.length;
                ii = 0;
                Utils.repeat(i, function () {
                    unlockedTokens.push('upgrades/health-' + myCharacters[ii]);
                    unlockedTokens.push('upgrades/attack-' + myCharacters[ii]);
                    unlockedTokens.push('upgrades/skill-' + myCharacters[ii]);
                    ++ii;
                });
                showAdButton();
                setRewards();

                if (finishedTutorial <= 1) {
                    setTutorialGift();
                }
            },
            destroy: function (data) {
                EventSystem.fire('giftClosed');
            },
            update: function (data) {
                showAdButton();
            },
            draw: function (data) {}
        };
        var adButton;
        var setTutorialGift = function () {
            var gift = SaveManager.load('gift24');

            // decrease time for gems to 0
            gift.time = TimeManager.getTime();
            SaveManager.save('gift24', gift);

            finishedTutorial = 1;
            EventSystem.fire('tutorialFreeGift');
            btnClose.setActive(false);
            SaveManager.save('tutorialFreeGift', finishedTutorial);
        };

        /**
         * Character punching the chest
         */
        var characterBehavior = {
            timer: 0,
            start: function () {
                // softer punch volume
                // Bento.audio.stopSound('sfx_punch');
                // Bento.audio.setVolume(0.15, 'sfx_punch');
            },
            destroy: function () {
                // reset punch volume
                // Bento.audio.stopSound('sfx_punch');
                // Bento.audio.setVolume(1, 'sfx_punch');
            },
            update: function () {
                characterBehavior.timer += speedMultiplier;

                // check if done
                if (giftFrame.isActive()) {
                    /*if (speedMultiplier > 1) {
                        characterSprite.setSpriteSheet(character + '/tired');
                    } else {
                        characterSprite.setSpriteSheet(character + '/idle');
                    }
                    chestSprite.setAnimation('default');
                    if (!chestSparkles.parent) {
                        chest.attach(chestSparkles);
                        chest.attach(new Blink({}));
                        chest.attach(new SparkleBurst({}));
                        if (finishedTutorial <= 1) {
                            giftFrame.attach(new TapHand({
                                rotation: Math.PI * 3 / 4
                            }));
                        }
                    }*/
                    if (!chestSparkles.parent) {
                        chest.attach(chestSparkles);
                        chestSprite.setAnimation('closed');

                        // perform final punch
                        characterSprite.setSpriteSheet(character + '/chargeup', function () {
                            if (!giftFrame.isActive()) {
                                return;
                            }
                            new Tween({
                                from: 1.2,
                                to: 1,
                                'in': 20,
                                ease: Tween.EASEINBACK,
                                onUpdate: function (v, t) {
                                    characterEntity.scale.x = v;
                                    characterEntity.scale.y = 1 / v;
                                }
                            });
                            characterSprite.setSpriteSheet(character + '/chargeup-ready');
                            Utils.timeout(10, function () {
                                if (!giftFrame.isActive()) {
                                    return;
                                }
                                characterSprite.setSpriteSheet(character + '/chargeup-attack', function () {
                                    if (speedMultiplier > 1) {
                                        characterSprite.setSpriteSheet(character + '/tired');
                                    } else {
                                        characterSprite.setSpriteSheet(character + '/idle');
                                    }
                                });

                                bounceChest({
                                    decay: 5,
                                    oscillations: 3,
                                    in: 60,
                                    sfx: 'sfx_special-hit',
                                    onStart: function () {
                                        if (!giftFrame.isActive()) {
                                            return;
                                        }
                                        characterBehavior.createLockParticle();

                                        Bento.audio.stopSound('sfx_consumable');
                                        Bento.audio.playSound('sfx_consumable');

                                        chestSprite.setAnimation('default');
                                        chest.attach(new Blink({}));
                                        chest.attach(new SparkleBurst({}));
                                        if (finishedTutorial <= 1) {
                                            giftFrame.attach(new TapHand({
                                                rotation: Math.PI * 3 / 4
                                            }));
                                        }

                                    }
                                });
                            });

                            gifts.attach(new Effect({
                                spriteSheet: 'effects/charge',
                                position: characterEntity.position.add(new Vector2(-4, -16))
                            }));
                            Bento.audio.stopSound('sfx_chargeready');
                            Bento.audio.playSound('sfx_chargeready');
                        });
                    }
                } else {
                    if (chestSprite.getAnimationName() === 'default') {
                        startPunching();
                        chestSprite.setAnimation('closed');
                        if (chestSparkles.parent) {
                            chest.remove(chestSparkles);
                        }
                    }
                }
            },
            createLockParticle: function () {
                var particleBehavior = {
                    name: 'behavior',
                    speedY: -6,
                    update: function () {
                        particle.position.y += particleBehavior.speedY;
                        particleBehavior.speedY += 0.3;
                        particle.position.x += 2;
                        particle.rotation += 0.1;

                        if (particle.timer > 120) {
                            particle.removeSelf();
                        }
                    }
                };
                var particle = new Entity({
                    name: 'lockParticle',
                    position: chest.position.add(new Vector2(0, -8)),
                    components: [
                        particleBehavior,
                        new Sprite({
                            imageName: 'gui/chest-daily-lock',
                            originRelative: new Vector2(0.5, 0.5)
                        })
                    ]
                });
                gifts.attach(particle);
            }
        };
        var character = SaveManager.load('selectedCharacter');
        var characterSprite = new Sprite({
            spriteSheet: character + '/idle'
        });
        var characterEntity = new Entity({
            name: 'character',
            components: [
                characterBehavior,
                characterSprite,
                new Anchor({
                    screenPosition: new Vector2(0.5, 0.5),
                    offset: new Vector2(-16, 0),
                    onComplete: function (parent, isPortrait) {
                        if (!isPortrait) {
                            characterEntity.position.y += 24;
                            chest.position.y += 24;
                            giftFrame.position.y += 24;
                        }
                    }
                })
            ]
        });
        var chestSprite = new Sprite({
            imageName: 'gui/chest-daily',
            frameCountX: 2,
            frameCountY: 1,
            originRelative: new Vector2(0.5, 1),
            animations: {
                closed: {
                    speed: 0,
                    frames: [0]
                },
                default: {
                    speed: 0,
                    frames: [1]
                }
            }
        });
        var chestSparkles = new Sparkles({
            offset: new Vector2(0, -16)
        });
        var chest = new Entity({
            name: 'dailyChest',
            scale: new Vector2(-1, 1),
            components: [
                chestSprite,
                new Anchor({
                    screenPosition: new Vector2(0.5, 0.5),
                    offset: new Vector2(16, 0)
                })
            ]
        });
        var combo = 1;
        var startPunching = function () {
            // DEPRECATED
            return;
            characterSprite.setSpriteSheet(character + '/punch' + combo, function () {
                startPunching();
            });
            Bento.audio.stopSound('sfx_punch');
            Bento.audio.playSound('sfx_punch');

            characterSprite.setCurrentSpeed(characterSprite.getCurrentSpeed() * speedMultiplier);
            combo += 1;
            bounceChest({});
            if (combo > 2) {
                combo = 1;
            }
        };
        var bounceChest = function (data) {
            new Tween({
                from: 0.5,
                to: 0,
                in: data.in || 30,
                delay: 10,
                ease: 'elastic',
                decay: data.decay || 8,
                oscillations: data.oscillations || 1,
                onStart: function () {
                    var effect = new Effect({
                        spriteSheet: 'effects/hitfx2-small',
                        position: chest.position.add(new Vector2(0, -16))
                    });
                    gifts.attach(effect);

                    if (data.sfx) {
                        Bento.audio.playSound(data.sfx);
                    }
                    if (data.onStart) {
                        data.onStart();
                    }
                },
                onUpdate: function (v, t) {
                    chest.scale.x = -(1 - v);
                    chest.scale.y = 1 + v;
                },
                onComplete: function () {

                }
            });
        };
        /**
         * Daily chest randomizaiton
         */
        var getRandomConsumable = function () {
            return consumables[Utils.getRandom(consumables.length)].key;
        };
        var getRandomToken = function (isUnlocked) {
            if (!isUnlocked && lockedTokens.length) {
                return lockedTokens[Utils.getRandom(lockedTokens.length)];
            } else {
                return unlockedTokens[Utils.getRandom(unlockedTokens.length)];
            }
        };
        var getDailyChestItems = function () {
            var chestItems = [];
            var giftRotation = SaveManager.load('giftRotation');
            var rotate = function () {
                giftRotation += 1;
                giftRotation %= 4;
            };

            // decrease gem odds with 50% if user has > 30 gems
            if (CurrencyManager.getGems() >= 30) {
                if (giftRotation === 2 && Utils.getRandom(2) === 0) {
                    rotate();
                }
            }

            // if (CharacterManager.getEnergyRelative() > 1) {
            //     // already over full energy
            //     if (giftRotation === 3) {
            //         rotate();
            //     }
            // }

            if (giftRotation === 0) {
                chestItems.push({
                    reward: 'currency/key',
                    amount: 1
                });
            } else if (giftRotation === 1) {
                chestItems.push({
                    reward: 'currency/coin',
                    amount: Globals.getFreeGiftCoins(SaveManager.load('unlockedLevels').length)
                });
            } else if (giftRotation === 2) {
                chestItems.push({
                    reward: 'currency/gem',
                    amount: 10
                });
            } else if (giftRotation === 3) {
                chestItems.push({
                    reward: getRandomConsumable(),
                    amount: 1
                });
            }

            rotate();
            SaveManager.save('giftRotation', giftRotation);

            return chestItems;
        };

        var giftFrame = new GiftFrame({
            id: 24,
            getDailyChestItems: getDailyChestItems,
            onClaim: function () {
                // reset views to reset punch speed
                var adInfo = SaveManager.load('giftAd');
                if (adInfo.views === Globals.maxAdsFreeGift) {
                    // maxed out: set to -1 to reset
                    adInfo.views = -1;
                } else {
                    // reset to 0 and allow ads to be watched again
                    adInfo.views = 0;
                }
                SaveManager.save('giftAd', adInfo);
                speedMultiplier = 1;
                Bento.audio.stopSound('sfx_chargeready');
                // Bento.audio.setVolume(0.15, 'sfx_punch');
                startPunching();
            },
            onTutorialComplete: function () {
                // remove any taphand if necessary
                giftFrame.getComponent('tapHand', function (tapHand) {
                    // move it to the close button
                    tapHand.removeSelf();
                    btnClose.attach(new TapHand({
                        rotation: 0
                    }));
                });
                btnClose.setActive(true);
            }
        }).attach(new Anchor({
            screenPosition: new Vector2(0.5, 0.5),
            offset: new Vector2(0, 24)
        }));
        giftFrame.removeByName('frame');

        var gifts = new Entity({
            z: Globals.WorldSelectLayers.GIFTS,
            name: 'gifts',
            updateWhenPaused: Globals.PauseLevels.GIFTS,
            position: settings.position,
            float: true,
            components: [
                new Modal({}),
                anchor,
                blurBackground,
                // backgroundPortrait,
                // backgroundLandscape,
                topBar,
                btnClose,
                giftFrame,
                behavior,
                chest,
                characterEntity,
                titleText,
                new ViewAnalytics({
                    name: 'freegift',
                    ignoreKey: 'tutorialWorldSelect'
                })
            ]
        }).extend({
            claim: function (onClaimed) {
                // simulate pressing the claim button
                behavior.start();
                giftFrame.claim(onClaimed);
                EventSystem.fire('giftClaimed');
            },
            resetReward: resetReward
        });

        return gifts;
    };
});