/**
 * In App Purchase Menu
 * @moduleName IAPMenu
 */
bento.define('gui/menus/iap', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'globals',
    'components/anchor',
    'components/modal',
    'gui/iap/chestsection',
    'gui/iap/gemsection',
    'gui/iap/coinsection',
    'gui/iap/energysection',
    'gui/scrollinglist',
    'gui/worldselect/topbarsimple',
    'managers/save',
    'gui/taphand',
    'gui/highlight',
    'components/viewanalytics',
    'components/hidebelow',
    'gui/iap/offerpack',
    'managers/time',
    'components/orientationlock',
    'components/eventlistener',
    'managers/currency',
    'gui/senseipopup',
    'modules/localization',
    'modules/analytics',
    'components/backbutton',
    'gui/dialog'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Globals,
    Anchor,
    Modal,
    ChestSection,
    GemSection,
    CoinSection,
    EnergySection,
    ScrollingList,
    Topbar,
    SaveManager,
    TapHand,
    Highlight,
    ViewAnalytics,
    HideBelow,
    OfferPack,
    TimeManager,
    OrientationLock,
    EventListener,
    CurrencyManager,
    SenseiPopup,
    Localization,
    Analytics,
    BackButton,
    Dialog
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var portrait = true;
        var topOffset = 42;
        var scrollTo = settings.scrollTo;
        var isPopUp = Utils.isDefined(settings.isPopUp) ? settings.isPopUp : true;
        var align = function (entity, isPortrait, component) {
            portrait = isPortrait;
            if (isPortrait) {
                backgroundPortrait.visible = true;
                backgroundLandscape.visible = false;
            } else {
                backgroundLandscape.visible = true;
                backgroundPortrait.visible = false;
            }
            //reset scrollinglist
            scrollinglist.setArea(new Rectangle(0, 0, viewport.width, viewport.height));
            scrollinglist.setMaxOffset(-viewport.height + topOffset /*+ sectionEnergy.dimension.height*/ );
        };

        var backgroundPortrait = new Entity({
            name: 'bgPortrait',
            components: [
                new Sprite({
                    imageName: 'gui/bg-brick-portrait',
                    originRelative: new Vector2(0.5, 0)
                }),
                new Anchor({
                    screenPosition: new Vector2(0.5, 0)
                })
            ]
        });

        var backgroundLandscape = new Entity({
            name: 'bgLandscape',
            components: [
                new Sprite({
                    imageName: 'gui/bg-brick-landscape',
                })
            ]
        });
        if (Anchor.isWideScreen()) {
            backgroundLandscape.scale.x = Anchor.getWideScreenSize().width / backgroundLandscape.dimension.width;
            backgroundLandscape.transform.x -= Anchor.getOffset().x;
        }

        var btnClose = new ClickButton({
            z: 0,
            name: 'btnClose',
            sfx: 'sfx_button',
            imageName: 'gui2/bottom_tabs/bottom_tabs__back',
            frameCountX: 2,
            frameCountY: 1,
            components: [
                new BackButton({
                    removeSelf: true,
                    onPressed: function (data) {
                        data.entity.doCallback();
                    }
                }),
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.5, 1),
                        offset: new Vector2(-48 + (Anchor.isWideScreen() ? 4 : 0), -12 + (Anchor.isWideScreen() ? Anchor.getOffset().y : 0))
                    },
                    landscape: {
                        screenPosition: new Vector2(0.25, 1),
                        offset: new Vector2(-44, -12)
                    }
                })
            ],
            onClick: function () {
                moveOut();
            },
            onButtonDown: function () {
                EventSystem.fire('preventScroll');
            }
        });

        var topBar = new Topbar({
            preventClick: true // don't open the IAP menu while we're in the IAP menu!
        });

        var onNotEnoughGems = function () {
            // scroll to gems
            var scrollToPosition = sectionGems.position.y;
            new Tween({
                from: scrollinglist.getScroll(),
                to: -scrollToPosition,
                in: 20,
                ease: 'easeOutExpo',
                onUpdate: function (v, t) {
                    scrollinglist.forceTo(v);
                }
            });
            // close the popup
            return true;
        };

        // make various store section
        var sectionChests = new ChestSection({
            onNotEnoughGems: onNotEnoughGems
        });

        var sectionGems = new GemSection({});

        var sectionCoins = new CoinSection({});

        // var sectionEnergy = new EnergySection({});

        // the scrollinglist that contains all off the store
        var scrollinglist = new ScrollingList({
            area: new Rectangle(0, 0, viewport.width, viewport.height),
            damping: 0.9,
            snap: false,
            direction: 'y',
            position: new Vector2(viewport.width / 2, topOffset),
            maxOffset: -viewport.height + topOffset /*+ sectionEnergy.dimension.height*/
        }).attach(
            new Anchor({
                screenPosition: new Vector2(0.5, 0),
                offset: new Vector2(0, topOffset)
            })
        );

        var anchor = new Anchor({
            onComplete: align
        });

        // move the menu into the screen on start
        var moveIn = function () {
            var scrollToPosition;

            // scroll to designated area
            if (scrollTo) {
                if (scrollTo === 'coins') {
                    scrollToPosition = sectionCoins.position.y;
                } else if (scrollTo === 'gems') {
                    scrollToPosition = sectionGems.position.y;
                }
                new Tween({
                    from: 0,
                    to: -scrollToPosition,
                    in: 20,
                    ease: 'easeOutExpo',
                    onUpdate: function (v, t) {
                        scrollinglist.forceTo(v);
                    }
                });
            } else {
                new Tween({
                    from: viewport.height,
                    to: 0,
                    in: 20,
                    ease: 'easeOutExpo',
                    onUpdate: function (v, t) {
                        scrollinglist.forceTo(v);
                    }
                });
            }
        };

        // move the menu out of the screen
        var moveOut = function () {
            var toPosition;
            btnClose.setActive(false);
            Bento.objects.remove(store);
            if (settings.onComplete) {
                settings.onComplete();
            }
        };

        // when button is pressed cancel scrolling behavior
        var cancelScroll = function () {
            scrollinglist.cancel();
        };

        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {
                EventSystem.on('preventScroll', cancelScroll);

                // find offers
                /*var offers = SaveManager.load('offers');
                var now = TimeManager.getTime();
                var newestOffer;
                var newestTime = 0;
                Utils.forEach(offers, function (offerData, i, l, breakLoop) {
                    // is valid offerpack?
                    if (!offerData || offerData.time < now) {
                        return;
                    }

                    var offerPack = new OfferPack({
                        z: Globals.WorldSelectLayers.IAP,
                        float: true,
                        offer: offerData.name,
                        duration: 24
                    });
                    var eventListener = new EventListener({
                        name: 'offerpackBought',
                        onEvent: function (eventData) {
                            var items = scrollinglist.getItems();
                            var index = items.indexOf(container);
                            if (eventData.offer === offerData.name && index >= 0) {
                                // remove from iap menu
                                scrollinglist.removeItem(index);
                            }
                        }
                    });
                    var container = new Entity({
                        name: 'offerPack',
                        dimension: new Rectangle(0, 0, 0, 158),
                        components: [
                            offerPack,
                            eventListener
                        ]
                    });
                    offerPack.position.y += 64;


                    // attach
                    // scrollinglist.addItem(container);

                    // or only show the newest?
                    if (offerData.time > newestTime) {
                        newestTime = offerData.time;
                        newestOffer = container;
                    }
                });

                if (newestOffer) {
                    scrollinglist.addItem(newestOffer);
                }*/

                if (!settings.preventChests) {
                    scrollinglist.addItem(sectionChests);
                }
                scrollinglist.addItem(sectionGems);
                scrollinglist.addItem(sectionCoins);
                // scrollinglist.addItem(sectionEnergy);
                // extra spacing on the bottom for the bottom bar
                scrollinglist.addItem(new Entity({
                    dimension: new Rectangle(0, 0, 0, 48)
                }));
                scrollinglist.addItem(new Entity({
                    dimension: new Rectangle(0, 0, 0, 0)
                }));
                moveIn();

            },
            destroy: function (data) {
                EventSystem.off('preventScroll', cancelScroll);
                if (SaveManager.load('tutorialChest') === 1 && !settings.skipTutorial) {
                    EventSystem.fire('chestClosed');
                }
            },
            update: function (data) {},
            draw: function (data) {}
        };
        var tutorialChest = SaveManager.load('tutorialChest');
        var tutorialChecker = {
            name: 'tutorialChecker',
            tapHand: null,
            highlight: null,
            start: function () {
                if (tutorialChest === 0) {
                    tutorialChecker.pointToChest();
                    btnClose.setActive(false);
                }

                EventSystem.on('openedFirstChest', tutorialChecker.pointToExit);
            },
            destroy: function () {
                tutorialChecker.clean();
                EventSystem.off('openedFirstChest', tutorialChecker.pointToExit);
            },
            clean: function () {
                if (tutorialChecker.highlight) {
                    tutorialChecker.highlight.removeSelf();
                    tutorialChecker.highlight = null;
                }
                if (tutorialChecker.tapHand) {
                    tutorialChecker.tapHand.removeSelf();
                    tutorialChecker.tapHand = null;
                }
            },
            pointToChest: function () {
                // point to the chest
                sectionChests.getComponent('gemPurchase-Classic', function (gemPurchase) {
                    gemPurchase.getComponent('item', function (button) {
                        tutorialChecker.tapHand = new TapHand({
                            z: Globals.WorldSelectLayers.IAPPOPUP,
                            // position: button.getWorldPosition(),
                            rotation: Math.PI * 3 / 4
                        });
                        tutorialChecker.highlight = new Highlight({
                            z: Globals.WorldSelectLayers.IAPPOPUP,
                            position: button.getWorldPosition()
                        });
                        // Bento.objects.attach(tutorialChecker.highlight);
                        button.attach(tutorialChecker.tapHand);
                    });
                    gemPurchase.parent.moveComponentTo(gemPurchase, gemPurchase.parent.components.length - 1);
                });
                sectionChests.parent.moveComponentTo(sectionChests, sectionChests.parent.components.length - 1);

                // disable scrolling
                // scrollinglist.setActive(false);
            },
            pointToExit: function () {
                btnClose.setActive(true);
                tutorialChecker.clean();
                btnClose.attach(new TapHand({
                    rotation: Math.PI * 3 / 4
                }));
            }
        };

        var store = new Entity({
            z: settings.z || (isPopUp ? Globals.WorldSelectLayers.IAPPOPUP : Globals.WorldSelectLayers.IAP),
            name: 'chestsAndGems',
            position: settings.position,
            float: true,
            components: [
                new Modal({
                    pauseLevel: Globals.PauseLevels.IAP
                }),
                anchor,
                backgroundPortrait,
                backgroundLandscape,
                scrollinglist,
                isPopUp ? btnClose : {},
                topBar,
                behavior,
                new HideBelow({})
            ]
        });

        var checkEvent = function () {
            var today = Math.floor(TimeManager.getTime() / 1000 / 60 / 60 / 24);
            // set oni relative to the highest
            if (today >= 17501 && today <= 17503) {
                var oniToken = new Entity({
                    name: 'token',
                    position: new Vector2(64, 0),
                    components: [
                        new Sprite({
                            spriteSheet: 'playertokens/oni'
                        }),
                        {
                            name: 'wobble',
                            update: function () {
                                oniToken.rotation = Math.PI / 30 * Math.sin(oniToken.timer / 20);
                            }
                        },
                        new Clickable({
                            onClick: function (data) {
                                var dialog = new Dialog({
                                    name: 'dialog',
                                    title: 'Event',
                                    text: 'This weekend: increased odds of Oni tokens!',
                                    canClose: false,
                                    type: 'yes',
                                    onYes: function () {},
                                    onNo: function () {},
                                    onClose: function () {},
                                    onComplete: function () {}
                                });
                                Bento.objects.attach(dialog);
                            },
                        })
                    ]
                });
                sectionChests.attach(oniToken);
            }
        };

        if (tutorialChest === 0 && CurrencyManager.canSpendKeys(Globals.chests.classic.cost) && !settings.skipTutorial && !settings.scrollTo) {
            store.attach(tutorialChecker);
        }

        if (!isPopUp) {
            Analytics.trackView('chest_and_gems');
        } else {
            // store is a popup
            store.attach(new ViewAnalytics({
                name: 'chest_and_gems',
                ignoreKey: 'tutorialWorldSelect'
            }));
        }

        checkEvent();

        return store;
    };
});