/**
 * Level end menu
 * @moduleName LevelEndMenu
 */
bento.define('gui/menus/levelend', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'components/fillscreen',
    'globals',
    'components/anchor',
    'gui/gacha',
    'managers/item',
    'managers/inventory',
    'gui/playercoins',
    'gui/blurbackground',
    'gui/levelend/starcounter',
    'gui/shadowtext',
    'managers/save',
    'components/fillrect',
    'modules/localization',
    'ads/heyzap',
    'gui/dialog',
    'managers/currency',
    'gui/coinreward',
    'gui/iap/chestpopup',
    'components/modal',
    'gui/watchrewardedad',
    'modules/videoadtimer',
    'components/viewanalytics',
    'modules/analytics',
    'managers/stats',
    'components/eventlistener',
    'modules/firebase',
    'modules/tenjin',
    'components/orientationlock',
    'gui/menus/iap',
    'entities/photosnapper',
    'modules/bezier',
    'managers/time',
    'modules/matchmanager',
    'managers/character',
    'components/glow',
    'gui/iconbutton',
    'gui/worldselect/adforcoins',
    'gui/adstackingcoins',
    'gui/centeredlist',
    'gui/itemslot',
    'gui/iteminfo',
    'modules/replaykit'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    FillScreen,
    Globals,
    Anchor,
    Gacha,
    ItemManager,
    InventoryManager,
    PlayerCoins,
    BlurBackground,
    StarCounter,
    ShadowText,
    SaveManager,
    FillRect,
    Localization,
    HeyZap,
    Dialog,
    CurrencyManager,
    CoinReward,
    ChestPopup,
    Modal,
    WatchRewardedAd,
    VideoAdTimer,
    ViewAnalytics,
    Analytics,
    StatsManager,
    EventListener,
    FireBase,
    Tenjin,
    OrientationLock,
    IAPMenu,
    PhotoSnapper,
    Bezier,
    TimeManager,
    MatchManager,
    CharacterManager,
    Glow,
    IconButton,
    AdForCoins,
    AdStackingCoins,
    CenteredList,
    ItemSlot,
    ItemInfo,
    ReplayKit
) {
    'use strict';
    return function (settings) {
        var rewards = settings.rewards;
        var itemKey = settings.itemKey;
        var itemName = itemKey ? ItemManager.getName(itemKey) : '';
        var itemDesc = itemKey ? ItemManager.getDescription(itemKey) : '';
        var previousStars = SaveManager.load('missionStars-' + SaveManager.load('currentMission')); // before stars are updated

        var viewport = Bento.getViewport();
        var tweened = false;
        var portrait = true;

        // function to place everything properly on the screen
        var align = function (entity, isPortrait, component) {
            if (tweened) {
                if (isPortrait) {
                    victoryText.position.x = viewport.width * 0.5;
                    victoryText.position.y = 25;

                    starCounter.position.x = viewport.width * 0.5;
                    starCounter.position.y = 46 + 32;
                } else {
                    victoryText.position.x = viewport.width * 0.5;
                    victoryText.position.y = viewport.height * 0.5 - 76;

                    starCounter.position.x = viewport.width * 0.5;
                    starCounter.position.y = viewport.height * 0.5 - 55 + 32;
                }
            } else {
                //if not yet tweened in set visibility of everything but playerCoins to false
                homeButton.visible = false;
                victoryText.visible = false;
                starCounter.visible = false;
            }

            playerCoins.position.x = viewport.width - 8;
            playerCoins.position.y = 8;

            portrait = isPortrait;
        };
        var width = Math.max(viewport.width, viewport.height);
        var minWidth = Math.min(viewport.width, viewport.height);
        var buttonSlot1 = new Entity({
            name: 'slot1',
            scale: new Vector2(1, 0),
            components: [
                new FillRect({
                    size: new Rectangle(0, 0, width, 28),
                    originRelative: new Vector2(0.5, 0.5),
                    color: [0, 0, 0, 0.4]
                }),
                new Anchor({
                    screenPosition: new Vector2(0.5, 0.5),
                    onComplete: function (parent, isPortrait) {}
                })
            ]
        });
        var buttonSlot2 = new Entity({
            name: 'slot2',
            scale: new Vector2(1, 0),
            components: [
                new FillRect({
                    size: new Rectangle(0, 0, width, 28),
                    originRelative: new Vector2(0.5, 0.5),
                    color: [0, 0, 0, 0.4]
                }),
                new Anchor({
                    screenPosition: new Vector2(0.5, 0.5),
                    offset: new Vector2(0, 48),
                    onComplete: function (parent, isPortrait) {
                        if (isPortrait) {

                        } else {
                            buttonSlot2.position.y -= 8;
                        }
                    }
                })
            ]
        });
        var anchor = new Anchor({
            onComplete: align
        });
        //blur and dark background
        var blurBackground = new BlurBackground({
            darken: 0.3
        });
        var playerCoins = new PlayerCoins({});
        // 'Victory!' text
        var victoryText = new ShadowText({
            z: 0,
            position: new Vector2(0, 0),
            text: '',
            font: 'font',
            fontSize: 16,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'top',
            strokeStyle: '#000000',
            pixelStroke: false // workaround for Cocoon bug
        });
        //star counter gui element
        var starCounter = new StarCounter({});
        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {

                // send analytics about victory
                var currentMission = SaveManager.load('currentMission');
                var health = StatsManager.health;
                Analytics.event('Victory', 'mission_' + currentMission, health);

                blur();
                if (!isJoining) {
                    // joiners do not get stars
                    appearStarCounter();
                } else {
                    determineSlots();
                    removeHealthBar();
                }

                if (SaveManager.load('isMissionOnline')) {
                    Analytics.event('VictoryMultiplayer', (MatchManager.otherPlayers.length + 1) + 'p_mission_' + currentMission);
                    Analytics.event('MissionMulti', currentMission + '_cleared');
                } else {
                    var selectedChar = CharacterManager.getSelected();
                    var currentLevel = StatsManager.getLevel(selectedChar);
                    Analytics.event('Mission', currentMission + '_cleared', currentLevel, selectedChar);
                    // send analytics about the player's level first complete
                    if (SaveManager.load('missionStars-' + currentMission) < 0) {
                        Analytics.event('LevelMissionClear', currentMission, currentLevel, selectedChar);
                    } else {
                        Analytics.event('LevelMissionClearReplay', currentMission, currentLevel, selectedChar);
                    }
                }

                // orientation analytics
                Analytics.event('Orientation', 'mission_clear', Anchor.isPortrait() ? 1 : 0);

                // view analytics
                Analytics.trackView('victory');
            },
            destroy: function () {
                if (canWatchAd && !didWatchAd) {
                    // ignored the ad
                    Analytics.event('Ads', 'rate_levelEnd', 0);
                    SaveManager.add('ignoredAds', 1);
                }
            }
        };

        var canUpgrade = (function () {
            var character = CharacterManager.getSelected();
            var level = StatsManager.getLevel(character);
            var upgradeCost = Globals.getTokenCost(level);
            var tokens = InventoryManager.getTokens('playertokens/' + character);
            var coins = Globals.getLevelUpCost(level);
            return tokens >= upgradeCost && CurrencyManager.canSpend(coins);
        })();
        var buttonText = new Entity({
            components: [
                new ShadowText({
                    z: 0,
                    position: new Vector2(0, -18),
                    text: Localization.getText(canUpgrade ? 'upgradeTitle' : 'continueTitle'),
                    font: 'font',
                    fontSize: 10,
                    fontColor: '#ffffff',
                    align: 'center',
                    textBaseline: 'bottom'
                })
            ]
        });
        var homeButton = new IconButton({
            name: 'homeButton',
            icon: buttonText,
            sfx: 'sfx_button',
            imageName: canUpgrade ? 'gui/buttons/upgrade' : 'gui/buttons/homebutton',
            frameCountX: 2,
            frameCountY: 1,
            position: new Vector2(0, 0),
            float: false,
            onClick: function () {
                var currentMission = SaveManager.load('currentMission');
                var target;
                var isEvent = SaveManager.load('isEvent');
                if (canUpgrade) {
                    // see screens/worldselect -> gui/worldselect/bottombar
                    target = 'upgrade';
                } else if (isEvent) {
                    target = 'beach';
                }

                // check for the existence of a post cutscene
                if (Utils.hasAsset('json', 'missions/' + currentMission + '/post')) {
                    Bento.screens.show('screens/cutscene', {
                        mapToLoad: 'missions/' + currentMission + '/post',
                        next: 'screens/worldselect',
                        target: target,
                        hidePauseButton: true
                    });
                } else {

                    // player has been ignoring ads for a while
                    var now = TimeManager.getTime();
                    if (
                        SaveManager.load('ignoredAds') >= 2 && // ignored at least 2 ads
                        Bento.saveState.load('testGroup', '') !== 'noForcedAds' && // not part of test group
                        now - SaveManager.load('lastTimeWatchedAd') > 1000 * 60 * 4 && // 3 minutes
                        Bento.saveState.load('iapAmount', 0) === 0 // never bought an iap
                    ) {
                        WatchRewardedAd.showInterstitial(function () {
                            Bento.screens.show('screens/worldselect', {
                                target: target
                            });
                        });
                    } else if (
                        // Multiplayer: (almost) always show ad to users
                        !didWatchAd &&
                        SaveManager.load('isMissionOnline') && // always show interstitial after online match
                        Bento.saveState.load('iapAmount', 0) === 0 // never bought an iap
                    ) {
                        WatchRewardedAd.showInterstitial(function () {
                            Bento.screens.show('screens/worldselect', {
                                target: target
                            });
                        });
                    // } else if (Bento.saveState.load('cheatSuspect', false)) {
                    //     // cheat suspect
                    //     WatchRewardedAd.showInterstitial(function () {
                    //         Bento.screens.show('screens/worldselect', {
                    //             target: target
                    //         });
                    //     });
                    } else {
                        // just go back to home screen
                        Bento.screens.show('screens/worldselect', {
                            target: target
                        });
                    }
                }

                // cordova.plugins.Replay.stopRecording(function () {
                //     console.log('stopRecording success');
                // }, function (err) {
                //     console.log('stopRecording fail ' + err);
                // });
            },
            components: [
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.5, 1),
                        offset: new Vector2(0, -24),
                    },
                    landscape: {
                        screenPosition: new Vector2(0.5, 1),
                        offset: new Vector2(0, -16),
                    },
                    onComplete: function () {
                        if (Anchor.isPortrait()) {
                            homeButton.scale.x = 1;
                            homeButton.scale.y = 1;
                        } else {
                            homeButton.scale.x = 0.8;
                            homeButton.scale.y = 0.8;
                        }
                    }
                })
            ]
        });
        var entity = new Entity({
            z: Globals.Layers.VICTORY,
            name: 'levelEndMenu',
            position: new Vector2(0, 0),
            float: true,
            components: [
                new Modal({}),
                anchor,
                blurBackground,
                victoryText,
                starCounter,
                homeButton,
                behavior,
                buttonSlot1,
                buttonSlot2,
                // dont rotate at all during end level
                new OrientationLock({})
            ]
        }).extend({
            getItemPosition: function () {
                return anchor.position;
            }
        });
        /*
         * "Crossy road" buttons
         */
        var canWatchAd = false;
        var didWatchAd = false;
        var createAdButton = function (data) {
            var firstVictory = data.firstVictory;
            var first3Stars = data.first3Stars;
            var coinRewardAmount = (25 * firstVictory || 0) + (50 * first3Stars || 0);
            var adCoins = new Entity({
                name: 'adCoins',
                position: new Vector2(12, -24),
                components: [
                    new Sprite({
                        imageName: 'gui/coin',
                        originRelative: new Vector2(0.5, 0.5)
                    })
                ]
            });
            var coinAmount = new ShadowText({
                position: new Vector2(12 - 10, -25),
                text: '+' + coinRewardAmount,
                font: 'font',
                fontSize: 16,
                fontColor: '#ffffff',
                align: 'right',
                textBaseline: 'middle',
                maxWidth: 24,
                maxHeight: 16
            });
            var onComplete = function (hasCompleted) {
                if (hasCompleted) {
                    // remove button and slot
                    adButton.parent.removeSelf();
                    // give money
                    CurrencyManager.addCoins(coinRewardAmount);

                    // show coin popup
                    var coinReward = new CoinReward({
                        z: Globals.Layers.VICTORY + 1,
                        coins: coinRewardAmount
                    });
                    Bento.objects.attach(coinReward);
                    didWatchAd = true;
                } else {
                    // nothing
                }
            };
            var adButton = new ClickButton({
                    z: 0,
                    name: 'adbutton',
                    sfx: 'sfx_button',
                    imageName: 'gui/buttons/ad',
                    frameCountX: 4,
                    frameCountY: 1,
                    animations: {
                        up: {
                            speed: 0.03,
                            frames: [0, 1]
                        },
                        down: {
                            frames: [2]
                        }
                    },
                    onClick: function () {
                        if (firstVictory && first3Stars) {
                            Analytics.event('Ads', 'watch_for_coins_both');
                        } else if (firstVictory) {
                            Analytics.event('Ads', 'watch_for_coins_victory');
                        } else if (first3Stars) {
                            Analytics.event('Ads', 'watch_for_coins_3stars');
                        }
                        Analytics.event('Ads', 'rate_levelEnd', 1);
                        WatchRewardedAd(onComplete);
                    }
                })
                .attach(adCoins)
                .attach(coinAmount);

            var text;

            if (firstVictory && first3Stars) {
                text = Localization.getText('firstVictoryAndStars');
            } else if (firstVictory) {
                text = Localization.getText('firstVictory');
            } else if (first3Stars) {
                text = Localization.getText('firstStars');
            }

            return {
                button: adButton,
                text: text
            };
        };
        var createDoubleCoinsButton = function (data) {
            var chosenDifficulty = SaveManager.load('chosenDifficulty');
            var id = (chosenDifficulty === previousStars) ? chosenDifficulty : 0;
            var textType = {
                '1': 'tripleCoins',
                '2': 'quadrupleCoins'
            };
            var multiplier = {
                '1': 2,
                '2': 3
            };
            var coinRewardAmount = Math.max(SaveManager.load('coinsCollected'), 10) * (multiplier[id] || 1);
            var adCoins = new Entity({
                name: 'adCoins',
                position: new Vector2(12, -24),
                components: [
                    new Sprite({
                        imageName: 'gui/coin',
                        originRelative: new Vector2(0.5, 0.5)
                    })
                ]
            });
            var coinAmount = new ShadowText({
                position: new Vector2(12 - 10, -25),
                text: '+' + coinRewardAmount,
                font: 'font',
                fontSize: 16,
                fontColor: '#ffffff',
                align: 'right',
                textBaseline: 'middle',
                maxWidth: 24,
                maxHeight: 16
            });
            var onComplete = function (hasCompleted) {
                if (hasCompleted) {
                    // remove button and slot
                    adButton.parent.removeSelf();
                    // give money
                    CurrencyManager.addCoins(coinRewardAmount);

                    // show coin popup
                    var coinReward = new CoinReward({
                        z: Globals.Layers.VICTORY + 1,
                        coins: coinRewardAmount
                    });
                    Bento.objects.attach(coinReward);
                    didWatchAd = true;
                } else {
                    // nothing
                }
            };
            var adButton = new ClickButton({
                z: 0,
                name: 'adbutton',
                sfx: 'sfx_button',
                imageName: 'gui/buttons/ad',
                frameCountX: 4,
                frameCountY: 1,
                animations: {
                    up: {
                        speed: 0.03,
                        frames: [0, 1]
                    },
                    down: {
                        frames: [2]
                    }
                },
                onClick: function () {
                    Analytics.event('Ads', 'watch_for_double');
                    Analytics.event('Ads', 'rate_levelEnd', 1);
                    WatchRewardedAd(onComplete);
                },
                components: [
                    adCoins,
                    coinAmount
                ]
            });
            var text = Localization.getText(textType[id] || 'doubleCoins');

            return {
                button: adButton,
                text: text
            };
        };
        var createAdForCoinsButton = function () {
            var coinRewardAmount = Globals.getCoinsForAd(SaveManager.load('unlockedLevels').length)[0];
            var adCoins = new Entity({
                name: 'adCoins',
                position: new Vector2(18, -16),
                components: [
                    new Sprite({
                        imageName: 'gui2/misc/button_coinalert_secondary',
                        originRelative: new Vector2(0.5, 0.5)
                    })
                ]
            });
            var onComplete = function (hasCompleted) {
                if (hasCompleted) {
                    // remove button and slot
                    adButton.parent.removeSelf();
                    // give money
                    CurrencyManager.addCoins(coinRewardAmount);

                    // show coin popup
                    var coinReward = new CoinReward({
                        z: Globals.Layers.VICTORY + 1,
                        coins: coinRewardAmount,
                        onComplete: function () {
                            Bento.objects.attach(new AdStackingCoins({}));
                        }
                    });
                    Bento.objects.attach(coinReward);


                    Analytics.event('Ads', 'watch_for_coins');
                } else {
                    // nothing
                }
            };
            var adButton = new ClickButton({
                    z: 0,
                    name: 'adbutton',
                    sfx: 'sfx_button',
                    imageName: 'gui/buttons/ad',
                    frameCountX: 4,
                    frameCountY: 1,
                    animations: {
                        up: {
                            speed: 0.03,
                            frames: [0, 1]
                        },
                        down: {
                            frames: [2]
                        },
                        inactive: {
                            frames: [3]
                        }
                    },
                    onClick: function () {
                        AdForCoins.play({
                            z: Globals.Layers.VICTORY + 1,
                            onComplete: function () {
                                // remove button and slot
                                adButton.parent.removeSelf();
                            }
                        });
                    }
                })
                .attach(adCoins);

            adButton.setActive(false);

            return {
                button: adButton,
                text: '',
                onAttached: function (textObject) {
                    var fontSize = 0;
                    textObject.attach({
                        name: 'updateTimer',
                        start: function () {
                            // check fontSize
                            fontSize = textObject.setText(Localization.getText('XTimeForNext').replace('{TIME}', '00:00'));
                        },
                        update: function () {
                            // check every second
                            if (textObject.timer % 60 === 0) {
                                if (AdForCoins.canPlay() && !adButton.isActive()) {
                                    adButton.setActive(true);
                                    textObject.setText(Localization.getText('getCoins'));
                                } else if (!AdForCoins.canPlay()) {
                                    textObject.setText(Localization.getText('XTimeForNext').replace('{TIME}', AdForCoins.getTimeString()), {
                                        maxWidth: null,
                                        fontSize: fontSize
                                    });
                                }
                            }
                        }
                    });
                }
            };
        };
        var createChestButton = function () {
            var myGems = CurrencyManager.getGems();
            var chestImage;
            var chestButton;
            var chestType;
            var text = Localization.getText('getChest');
            var currentCost;
            var textObject;
            var deactiveButton = function () {
                if (myGems < currentCost) {
                    chestButton.setActive(false);
                    text = Localization.getText('XGemsToGo').replace('{NUMBER}', currentCost - myGems);
                    if (textObject) {
                        textObject.setText(text);
                    }
                }
            };
            // determine what sprite to use for chest
            if (myGems >= Globals.chests['mythic'].cost) {
                chestImage = 'gui/buttons/chest-mythic';
                chestType = 'mythic';
                currentCost = Globals.chests['mythic'].cost;
            } else if (myGems >= Globals.chests['special'].cost) {
                chestImage = 'gui/buttons/chest-special';
                chestType = 'special';
                currentCost = Globals.chests['special'].cost;
            } else {
                chestImage = 'gui/buttons/chest-classic';
                chestType = 'classic';
                currentCost = Globals.chests['classic'].cost;
            }

            chestButton = new ClickButton({
                name: 'chestButton',
                sfx: 'sfx_button',
                imageName: chestImage,
                frameCountX: 4,
                frameCountY: 1,
                animations: {
                    up: {
                        speed: 0.03,
                        frames: [0, 1]
                    },
                    down: {
                        frames: [2]
                    },
                    inactive: {
                        frames: [3]
                    }
                },
                position: new Vector2(0, 0),
                float: false,
                onClick: function () {
                    /*var chestPopup = new ChestPopup({
                        chest: chestType,
                        onNotEnoughGems: function () {
                            // open the IAP shop
                            var iapMenu = new IAPMenu({
                                z: entity.z + 1,
                                scrollTo: 'gems'
                            });
                            Bento.objects.attach(iapMenu);
                            return false;
                        }
                    });
                    Bento.objects.attach(chestPopup);*/

                    // open the IAP shop instead of showing the chest
                    var iapMenu = new IAPMenu({
                        z: entity.z + 1,
                        // TODO: scrolling to chests breaks everything somehow
                        // scrollTo: 'chests'
                    });
                    Bento.objects.attach(iapMenu);
                }
            });

            // check if button should be deactivated
            chestButton.attach(new EventListener({
                name: 'updateGems',
                onEvent: function (gems) {
                    myGems = gems;
                    deactiveButton();
                }
            }));

            deactiveButton();

            return {
                button: chestButton,
                text: text,
                onAttached: function (textObj) {
                    textObject = textObj;
                }
            };
        };
        var addButtonToSlot = function (target, button, str, delay, callback) {
            var buttonWidth = portrait ? 52 : 64;
            var text = new Text({
                text: str,
                font: 'font',
                fontSize: 16,
                fontColor: '#ffffff',
                align: 'center',
                textBaseline: 'middle',
                // move text to the right in landscape
                maxWidth: Anchor.isPortrait() ? viewport.width - buttonWidth - 16 : viewport.width - buttonWidth - 64,
                maxHeight: 16
            });
            // scale y
            new Tween({
                from: 0,
                to: 1,
                in: 30,
                delay: delay,
                ease: Tween.EXPONENTIAL,
                alpha: -5,
                beta: 1,
                onUpdate: function (v, t) {
                    target.scale.y = v;
                }
            });
            // move button from right to left
            new Tween({
                from: viewport.width / 2 + buttonWidth,
                to: viewport.width / 2 - buttonWidth / 2,
                in: 30,
                delay: 40 + delay,
                ease: Tween.ELASTIC,
                alpha: -8,
                beta: 1,
                onStart: function () {
                    target.attach(button);
                },
                onUpdate: function (v, t) {
                    button.position.x = Math.floor(v);
                }
            });
            // move text from right to left
            new Tween({
                from: -buttonWidth / 2 + viewport.width,
                // move text to the right in landscape
                to: Anchor.isPortrait() ? -buttonWidth / 2 : -buttonWidth / 2 + 32,
                in: 30,
                delay: 30 + delay,
                ease: Tween.ELASTIC,
                alpha: -8,
                beta: 1,
                onStart: function () {
                    target.attach(text);
                },
                onUpdate: function (v, t) {
                    text.position.x = Math.floor(v);
                }
            });

            if (callback) {
                callback(text);
            }
        };
        var showRewards = function (callback) {
            var target = buttonSlot1;
            // 'Rewards:'' text
            var rewardText = new ShadowText({
                z: 0,
                position: new Vector2(-72, -22),
                text: Localization.getText('receivedRewards'),
                font: 'font',
                fontSize: 12,
                fontColor: '#ffffff',
                align: 'left',
                textBaseline: 'middle',
                strokeStyle: '#000000',
                pixelStroke: false // workaround for Cocoon bug
            });
            var centeredList = new CenteredList({
                position: new Vector2(0, 0),
                spacing: 3
            });
            // scale y
            new Tween({
                from: 0,
                to: 1,
                in: 30,
                ease: Tween.EXPONENTIAL,
                alpha: -5,
                beta: 1,
                onUpdate: function (v, t) {
                    target.scale.y = v;
                },
                onComplete: function () {
                    target.attach(rewardText);
                    target.attach(centeredList);

                    // centeredList.addItem(rewardText);
                    Utils.forEach(rewards, function (reward, i, l, breakLoop) {
                        var sprite = ItemManager.getSprite(reward.key);
                        var clickTarget = new ClickButton({
                            name: 'itemInfo',
                            imageName: 'gui2/loadout/item_box',
                            frameCountX: 2,
                            frameCountY: 1,
                            animations: {
                                up: {
                                    frames: [0]
                                },
                                down: {
                                    frames: [1]
                                }
                            },
                            onClick: function (data) {
                                var itemInfo = ItemInfo({
                                    item: reward.key
                                });
                                Bento.objects.attach(itemInfo);
                                Bento.audio.stopSound('sfx_button');
                                Bento.audio.playSound('sfx_button');
                            }
                        });
                        var item = new Entity({
                            name: 'item',
                            position: new Vector2(0, 0),
                            components: [
                                clickTarget,
                                sprite
                            ]
                        });

                        item.dimension = new Rectangle(0, 0, 40, 40);
                        sprite.setOriginRelative(new Vector2(0.5, 0.5));
                        centeredList.addItem(item);

                        // add amount
                        if (reward.amount > 1) {
                            item.attach(new ShadowText({
                                position: new Vector2(12, 6),
                                text: reward.amount.toString(),
                                font: 'font',
                                fontSize: 16,
                                fontColor: '#ffffff',
                                align: 'center',
                                textBaseline: 'middle',
                                maxWidth: 17,
                                maxHeight: 16
                            }));
                        }
                    });

                    // tween in items
                    var delay = 0;
                    new Tween({
                        from: rewardText.position.x + viewport.width,
                        to: rewardText.position.x,
                        in: 30,
                        delay: delay,
                        ease: 'elastic',
                        decay: 8,
                        oscillations: 1,
                        onUpdate: function (v, t) {
                            rewardText.position.x = v;
                        }
                    });
                    rewardText.position.x += viewport.width;
                    delay += 10;
                    centeredList.iterate(function (item, i, l, breakLoop) {
                        new Tween({
                            from: item.position.x + viewport.width,
                            to: item.position.x,
                            in: 30,
                            delay: delay,
                            ease: 'elastic',
                            decay: 8,
                            oscillations: 1,
                            onUpdate: function (v, t) {
                                item.position.x = v;
                            },
                            onComplete: function () {

                            }
                        });
                        item.position.x += viewport.width;
                        delay += 10;
                    });
                }
            });

        };
        /*
         * Tweening functions
         */
        var blur = function () {
            //background tween
            new Tween({
                from: 0,
                to: 1,
                in: 10,
                ease: 'linear',
                onUpdate: function (v, t) {
                    blurBackground.alpha = v;
                }
            });
        };
        var appearStarCounter = function () {
            // star counter tween
            new Tween({
                from: 0,
                to: 1,
                in: 16,
                delay: 0,
                ease: 'easeOutCubic',
                onStart: function () {
                    victoryText.visible = true;
                    starCounter.visible = true;
                },
                onUpdate: function (v, t) {

                    victoryText.position.x = portrait ? -61 + (61 + viewport.width * 0.5) * v : viewport.width * 0.5;
                    victoryText.position.y = portrait ? 120 : -64 + (64 + (viewport.height * 0.5 - 76)) * v + 32;

                    starCounter.position.x = victoryText.position.x;
                    starCounter.position.y = portrait ? victoryText.position.y + 21 : viewport.height / 2 + 16;
                },
                onComplete: function () {
                    Utils.timeout(10, function () {
                        starCounter.startCount(function (first3Stars, firstVictory) {
                            // after star count we can determine what slot buttons can be added
                            hasFirst3Stars = first3Stars;
                            hasFirstVictory = firstVictory;
                            determineSlots();

                            Utils.timeout(10, function () {
                                removeHealthBar();

                                // at this point, the slot buttons should be determined
                                // if there are no slot buttons at all, we dont move the stars to the top
                                if (rewards || slot1 || slot2) {
                                    moveStarCounterTop();
                                }
                            });
                        });
                    });
                }
            });
        };
        var moveStarCounterTop = function () {
            //starCounter return to normal position
            new Tween({
                from: 0,
                to: 1,
                in: 20,
                ease: 'easeInOutCubic',
                onUpdate: function (v, t) {
                    if (portrait) {
                        victoryText.position.y = 120 - v * 64;
                        starCounter.scale.x = 1 - 0.25 * v;
                        starCounter.scale.y = 1 - 0.25 * v;
                        starCounter.position.y = victoryText.position.y + 21;
                    } else {
                        starCounter.position.y = viewport.height / 2 + 16 - 58 * v;
                        starCounter.scale.x = 1 - 0.5 * v;
                        starCounter.scale.y = 1 - 0.5 * v;
                    }

                    starCounter.position.x = victoryText.position.x;
                }
            });
        };
        var removeHealthBar = function () {
            Bento.objects.get('healthBar', function (healthBar) {
                new Tween({
                    from: 1,
                    to: 0,
                    in: 30,
                    ease: 'easeInCubic',
                    onUpdate: function (v, t) {
                        healthBar.alpha = v;
                    },
                    onComplete: function () {
                        showPolaroid();
                    }
                });
            });
        };
        var showPolaroid = function () {
            var photoImage = PhotoSnapper.getPhoto();
            var polaroid;
            var buttonShare = new ClickButton({
                name: 'buttonShare',
                sfx: 'sfx_button',
                position: new Vector2(32, viewport.height - 20),
                imageName: 'gui/buttons/share',
                frameCountX: 1,
                frameCountY: 2,
                onClick: function () {
                    var webLink = Utils.getStoreLink('smart');
                    var text = Localization.getText('shareScreenshot');

                    // replace link
                    text = text.replace('{LINK}', '').replace('{link}', '');

                    // currently sharing the app icon as image
                    Utils.share(photoImage.image.toDataURL(), text, webLink, function (activity, completed) {
                        // fire quest if succesful
                        activity = activity || '';
                        if (completed) {
                            if (activity === 'com.apple.mobilenotes.SharingExtension' ||
                                activity === 'com.apple.UIKit.activity.SaveToCameraRoll'
                            ) {
                                return;
                            }
                            EventSystem.fire('quest-shareScreenshot', {
                                count: 1
                            });
                        }
                    });
                }
            });
            var onComplete = function () {
                tweenInHomeButton();
                makeSlotButtons();
            };
            var movePolaroidToCorner = function () {
                var bounceButton = function () {
                    var scale = buttonShare.scale.x;
                    new Tween({
                        from: scale,
                        to: 0,
                        'in': 50,
                        ease: Tween.ELASTIC,
                        decay: 6,
                        oscillations: 3,
                        onUpdate: function (v, t) {
                            buttonShare.scale.x = scale + v;
                            buttonShare.scale.y = scale - v;
                        }
                    });
                };
                var curve = new Bezier(
                    polaroid.position,
                    new Vector2(0, viewport.height / 2),
                    buttonShare.position.add(new Vector2(4, -24))
                );
                // move
                new Tween({
                    from: 0,
                    to: 1,
                    in: 40,
                    ease: 'quadratic',
                    onUpdate: function (v, t) {
                        polaroid.position = Utils.toVector2(curve.get(v));
                    }
                });
                // scale
                new Tween({
                    from: polaroid.scale.x,
                    to: polaroid.scale.x * 0.45,
                    in: 40,
                    ease: 'linear',
                    onUpdate: function (v, t) {
                        polaroid.scale.x = v;
                        polaroid.scale.y = v;
                    }
                });
                // rotate
                new Tween({
                    from: polaroid.rotation,
                    to: -Math.PI / 16,
                    in: 40,
                    ease: 'linear',
                    onUpdate: function (v, t) {
                        polaroid.rotation = v;
                    }
                });
            };
            if (photoImage) {
                polaroid = makePolaroid(photoImage);
            } else {
                onComplete();
                return;
            }
            entity.attach(polaroid);
            // slide in polaroid
            new Tween({
                from: viewport.height + 80,
                to: viewport.height / 2,
                in: 30,
                ease: 'exponential',
                decay: 5,
                onUpdate: function (v, t) {
                    polaroid.position.y = v;
                },
                onComplete: function () {

                }
            });
            new Tween({
                from: 0,
                to: Utils.getRandomRangeFloat(-Math.PI / 20, Math.PI / 20),
                in: 30,
                ease: 'exponential',
                decay: 5,
                onUpdate: function (v, t) {
                    polaroid.rotation = v;
                },
                onComplete: function () {
                    Utils.timeout(30, function () {
                        onComplete();
                        movePolaroidToCorner();

                        // at this point also, close recording
                        if (Globals.isRecording) {
                            ReplayKit.isRecording(function (isRecording) {
                                if (isRecording) {
                                    ReplayKit.stop(function () {
                                        EventSystem.fire('updateRecordingStatus');
                                    });
                                }
                            });
                            // turn off recording
                            Globals.isRecording = false;
                        }
                    });
                }
            });

            // tween in shareButton
            new Tween({
                from: -20,
                to: Anchor.isPortrait() ? 20 : 32,
                in: 30,
                delay: 0,
                ease: 'elastic',
                decay: 5, // for 'exponential' and 'elastic'
                oscillations: 1, // for 'elastic', 'sin' and 'cos'
                onStart: function () {
                    entity.attach(buttonShare);
                    buttonShare.scale.x = 0.75;
                    buttonShare.scale.y = 0.75;
                },
                onUpdate: function (v, t) {
                    buttonShare.position.x = v;
                },
                onComplete: function () {

                }
            });
        };
        var tweenInHomeButton = function () {
            // homebutton tween
            new Tween({
                from: homeButton.position.y + 64,
                to: homeButton.position.y,
                in: 30,
                delay: 90,
                ease: 'elastic',
                decay: 5,
                oscillations: 1,
                onStart: function () {
                    homeButton.visible = true;
                },
                onUpdate: function (v, t) {
                    homeButton.position.y = v;
                },
                onComplete: function () {
                    tweened = true;
                }
            });

        };
        var slot1;
        var slot2;
        var hasFirstVictory = false;
        var hasFirst3Stars = false;
        var determineSlots = function () {
            var coinsCollected = SaveManager.load('coinsCollected');
            var adButton = createDoubleCoinsButton();
            var chestButton = createChestButton();

            // slot1: rewards
            // slot2: possible ad
            // can watch ad?
            if (coinsCollected > 0) {
                slot2 = adButton;
                canWatchAd = true;
            } else if (AdForCoins.canPlay()) {
                // show timed ad for coins
                slot2 = createAdForCoinsButton();
            }
        };
        var makeSlotButtons = function () {
            Utils.timeout(10, function () {
                // if (slot1) {
                //     addButtonToSlot(buttonSlot1, slot1.button, slot1.text, 0, slot1.onAttached);
                // }
                if (rewards && rewards.length) {
                    showRewards();
                }
                if (slot2) {
                    addButtonToSlot(buttonSlot2, slot2.button, slot2.text, 20, slot2.onAttached);
                }
            });
        };
        // polaroid of a random snapshot
        var makePolaroid = function (photoImage) {
            var photoSprite = new Sprite({
                image: photoImage,
                originRelative: new Vector2(0.5, 0.5)
            });
            var fillRect = new FillRect({
                originRelative: new Vector2(0.5, 0.45),
                color: [1, 1, 1, 1],
                size: new Rectangle(
                    0,
                    0,
                    photoSprite.frameWidth * 1.1,
                    photoSprite.frameHeight * 1.3
                )
            });

            var polaroid = new Entity({
                z: entity.z,
                name: 'polaroid',
                position: viewport.getExtents(),
                float: true,
                components: [
                    fillRect,
                    photoSprite
                ]
            });
            // scale down the polaroid to game size
            var targetSize = 80;
            polaroid.scale.x = targetSize / photoSprite.frameWidth;
            polaroid.scale.y = polaroid.scale.x;

            return polaroid;
        };
        // unlock next mission
        var isJoining = SaveManager.load('isJoining');
        var currentMission = parseInt(SaveManager.load('currentMission'));
        var unlockedLevels = SaveManager.load('unlockedLevels');
        var toUnlockArray = SaveManager.load('toUnlock') || []; // may be empty in case of joining an online match
        var recentlyUnlocked = [];
        var hasUnlocked = false; // this boolean is checked for "first victory bonus"
        Utils.forEach(toUnlockArray, function (toUnlock, i, l, breakLoop) {
            if (!isJoining && unlockedLevels.indexOf(toUnlock) < 0) {
                // not unlocked yet
                unlockedLevels.push(toUnlock);
                recentlyUnlocked.push(toUnlock);
                SaveManager.save('unlockedLevels', unlockedLevels);
                SaveManager.save('recentlyUnlocked', recentlyUnlocked);
                hasUnlocked = true;

                if (Utils.isInt(currentMission)) {
                    // afaik "level up" in Firebase means clearing stages and checking which stages are too difficult
                    FireBase.logEvent(FireBase.Events.LEVEL_UP, {
                        level: currentMission
                    });
                    Tenjin.sendEventAndValue("missionClear", currentMission);

                    if (currentMission === 4) {
                        Tenjin.sendEvent("missionsCleared-4");
                    }
                }

                // triggers for consumable packs
                if (currentMission === 1) {
                    SaveManager.save('showStarterPack', true);
                } else if (currentMission === 5) {
                    // SaveManager.save('showOffer', 'gem');
                } else if (currentMission === 9) {
                    // SaveManager.save('showOffer', 'powerup');

                }
            }
        });

        // Post 'Hero in Training' achievement (we do this on every level end as to prevent player's missing the achievement)
        Utils.postAchievement("CgkI9NqdkoAGEAIQBA");

        // Firebase
        FireBase.setUserProperty('mission', unlockedLevels.length - 1);

        // might ask for a review when player clears a level
        // will not repeat on Android because it saves per Beat Street version,
        // on iOS the system will determine whether a rating popup will appear or not
        if (unlockedLevels.length >= 5) {
            SaveManager.save('shouldAskForReview', true);
        }

        // cleared mission (used for offering packs)
        if (SaveManager.load('usedWeapon')) {
            SaveManager.save('usedWeapon', 2);
        }
        if (SaveManager.load('usedConsumable')) {
            SaveManager.save('usedConsumable', 2);
        }

        if (canUpgrade) {
            homeButton.attach(new Glow({}));
        }

        // give reputation point
        if (SaveManager.load('wantsReputation')) {
            SaveManager.save('wantsReputation', false);
            SaveManager.add('reputationPoints', 1);
        }

        return entity;
    };
});