/**
 * Loadout menu
 * @moduleName LoadOutMenu
 */
bento.define('gui/menus/loadout', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'globals',
    'components/modal',
    'components/anchor',
    'modules/localization',
    'gui/shadowtext',
    'managers/save',
    'managers/character',
    'gui/equipmentslots',
    'bento/gui/togglebutton',
    'managers/inventory',
    'gui/menus/settings',
    'components/fillrect',
    'gui/iconbutton',
    'gui/buttons/gift',
    'gui/worldselect/adforcoins',
    'gui/menus/characterselect',
    'components/eventlistener',
    'gui/menus/allies',
    'managers/allies',
    'gui/worldselect/topbar',
    'gui/badge',
    'managers/stats',
    'gui/taphand',
    'components/glow',
    'gui/menus/dailychest',
    'components/orientationlock',
    'modules/analytics',
    'managers/time',
    'gui/dialog',
    'modules/notifications',
    'gui/senseipopup',
    'gui/highlight',
    'gui/scrollinglist',
    'gui/iap/offerpack',
    'gui/iap/offerpackpopup',
    'gui/centeredlist',
    'components/masksprite',
    'entities/crosspromo',
    'entities/interstitial'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Globals,
    Modal,
    Anchor,
    Localization,
    ShadowText,
    SaveManager,
    CharacterManager,
    EquipmentSlots,
    ToggleButton,
    InventoryManager,
    SettingsMenu,
    FillRect,
    IconButton,
    GiftButton,
    AdForCoins,
    CharacterSelectMenu,
    EventListener,
    AlliesMenu,
    AlliesManager,
    Topbar,
    Badge,
    StatsManager,
    TapHand,
    Glow,
    DailyChest,
    OrientationLock,
    Analytics,
    TimeManager,
    Dialog,
    Notifications,
    SenseiPopup,
    Highlight,
    ScrollingList,
    OfferPack,
    OfferPackPopup,
    CenteredList,
    MaskSprite,
    StaticCrossPromo,
    StaticInterstitial

) {
    'use strict';
    // this becomes false throughout the rest of the game
    var firstSession = true;

    Sprite.suppressWarnings = true;
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var bottomBar = settings.bottomBar;
        var tweenIn = settings.tweenIn;
        var background = new Entity({
            name: 'background',
            components: [
                new Sprite({
                    imageName: 'gui2/loadout/background',
                    originRelative: new Vector2(0.5, 0.5)
                }),
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.5, 0.5)
                    },
                    landscape: {
                        screenPosition: new Vector2(0.3, 0.65)
                    },
                    onComplete: function () {
                        if (Anchor.isWideScreen()) {
                            if (!Anchor.isPortrait()) {
                                background.position.x += 12;
                                background.scale.x = Anchor.getWideScreenSize().width / 320;
                                background.transform.x -= Anchor.getOffset().x;
                            } else {
                                background.transform.x = 0;
                                background.scale.x = 1;
                            }
                        }
                    }
                })
            ]
        });
        /**
         * Main character
         */
        var selectedSprite = new Sprite({});
        var selectedShadowSprite = new Sprite({});
        var selectedCharacterShadow = new Entity({
            name: 'selectedCharacterShadow',
            position: new Vector2(0, 8 - 2),
            components: [
                selectedShadowSprite
            ]
        });
        var selectedCharacter = new Entity({
            name: 'selectedCharacter',
            position: new Vector2(0, 8),
            components: [
                selectedSprite, {
                    update: function () {
                        // selectedCharacter.scale.x = 1 + 0.025 * Math.sin(selectedCharacter.timer / 20);
                        // selectedCharacter.scale.y = 1 - 0.025 * Math.sin(selectedCharacter.timer / 20);
                    }
                }
            ]
        });
        var badgeMain = new Badge({
            pulsate: true,
            exclamation: true,
            imageName: 'gui2/misc/badge',
            position: new Vector2(32, 16),
        });
        var baseMain = new ClickButton({
            name: 'baseMain',
            boundingBox: new Rectangle(-35, -85, 70, 100),
            onButtonDown: function () {
                this.scale.x = 0.95;
                this.scale.y = 0.95;
            },
            onButtonUp: function () {
                this.scale.x = 1;
                this.scale.y = 1;
            },
            onClick: function () {
                var characterSelect = new CharacterSelectMenu({});
                Bento.objects.attach(characterSelect);
                Bento.audio.stopSound('sfx_button');
                Bento.audio.playSound('sfx_button');
            },
            components: [
                selectedCharacterShadow,
                new Sprite({
                    imageName: 'gui2/loadout/select_main_character',
                    frameCountX: 5,
                    frameCountY: 1,
                    originRelative: new Vector2(0.5, 0.5),
                    animations: {
                        default: {
                            speed: 0.2,
                            frames: [0, 1, 2, 3, 4]
                        }
                    }
                }),
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.5, 0.5),
                        offset: new Vector2(0, 12)
                    },
                    landscape: {
                        screenPosition: new Vector2(0.3, 0.75),
                        offset: new Vector2(0, 0)
                    }
                }),
                selectedCharacter,
                new EventListener({
                    name: 'updateCharacter',
                    onEvent: function (character) {
                        setSelectedSprite();
                    }
                }),
                badgeMain
            ]
        });
        /**
         * Buddies
         */
        var badgeBuddy1 = new Badge({
            pulsate: true,
            exclamation: true,
            imageName: 'gui2/misc/badge',
            position: new Vector2(16, 8),
        });
        var selectedBuddy1Sprite = new Sprite({});
        var selectedBuddy1ShadowSprite = new Sprite({});
        var selectedBuddy1Shadow = new Entity({
            name: 'selectedBuddy1Shadow',
            position: new Vector2(-8, 0),
            scale: new Vector2(-1, 1),
            components: [
                selectedBuddy1ShadowSprite
            ]
        });
        var selectedBuddy1Bottom = new Sprite({
            imageName: 'gui2/misc/figurine-bottom',
            originRelative: new Vector2(0.5, 0.5),
        });
        var selectedBuddy1 = new Entity({
            name: 'selectedBuddy1',
            // position: new Vector2(-8, 0),
            scale: new Vector2(-1, 1),
            components: [
                selectedBuddy1Bottom,
                selectedBuddy1Sprite
            ]
        });
        var baseBuddy1 = new ClickButton({
            name: 'baseBuddy1',
            boundingBox: new Rectangle(-20, -50, 40, 60),
            onButtonDown: function () {
                this.scale.x = 0.95;
                this.scale.y = 0.95;
            },
            onButtonUp: function () {
                this.scale.x = 1;
                this.scale.y = 1;
            },
            onClick: function () {
                var characterSelect = new AlliesMenu({
                    index: 0,
                    onClose: function (changed) {
                        if (changed) {
                            new Tween({
                                from: -32,
                                to: 0,
                                in: 30,
                                ease: 'easeOutBounce',
                                onUpdate: function (v, t) {
                                    selectedBuddy1.position.y = v;
                                }
                            });
                        }
                    } 
                });
                Bento.objects.attach(characterSelect);
                Bento.audio.stopSound('sfx_button');
                Bento.audio.playSound('sfx_button');
            },
            components: [
                selectedBuddy1Shadow,
                new Sprite({
                    imageName: 'gui2/loadout/select_buddy_character',
                    frameCountX: 5,
                    frameCountY: 1,
                    originRelative: new Vector2(0.5, 0.5),
                    animations: {
                        default: {
                            speed: 0.2,
                            frames: [0, 1, 2, 3, 4]
                        }
                    }
                }),
                selectedBuddy1,
                new EventListener({
                    name: 'updateAlly0',
                    onEvent: function (character) {
                        if (!character) {
                            selectedBuddy1Sprite.visible = false;
                            selectedBuddy1ShadowSprite.visible = false;
                            selectedBuddy1Bottom.visible = false;
                            return;
                        }
                        selectedBuddy1Sprite.visible = true;
                        selectedBuddy1Bottom.visible = true;
                        selectedBuddy1ShadowSprite.visible = true;
                        setSelectedBuddy1Sprite(character);
                    }
                }),
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.5, 0.5),
                        offset: new Vector2(-64, 8)
                    },
                    landscape: {
                        screenPosition: new Vector2(0.3, 0.75),
                        offset: new Vector2(-64, -6)
                    }
                }),
                badgeBuddy1
            ]
        });
        var badgeBuddy2 = new Badge({
            pulsate: true,
            exclamation: true,
            imageName: 'gui2/misc/badge',
            position: new Vector2(16, 8),
        });
        var selectedBuddy2Sprite = new Sprite({});
        var selectedBuddy2ShadowSprite = new Sprite({});
        var selectedBuddy2Shadow = new Entity({
            name: 'selectedBuddy2Shadow',
            position: new Vector2(8, 0),
            components: [
                selectedBuddy2ShadowSprite
            ]
        });
        var selectedBuddy2Bottom = new Sprite({
            imageName: 'gui2/misc/figurine-bottom',
            originRelative: new Vector2(0.5, 0.5),
        });
        var selectedBuddy2 = new Entity({
            name: 'selectedBuddy2',
            // position: new Vector2(8, 0),
            components: [
                selectedBuddy2Bottom,
                selectedBuddy2Sprite
            ]
        });
        var baseBuddy2 = new ClickButton({
            name: 'baseBuddy2',
            boundingBox: new Rectangle(-20, -50, 40, 60),
            onButtonDown: function () {
                this.scale.x = 0.95;
                this.scale.y = 0.95;
            },
            onButtonUp: function () {
                this.scale.x = 1;
                this.scale.y = 1;
            },
            onClick: function () {
                var characterSelect = new AlliesMenu({
                    index: 1,
                    onClose: function (changed) {
                        if (changed) {
                            new Tween({
                                from: -32,
                                to: 0,
                                in: 30,
                                ease: 'easeOutBounce',
                                onUpdate: function (v, t) {
                                    selectedBuddy2.position.y = v;
                                }
                            });
                        }
                    } 
                });
                Bento.objects.attach(characterSelect);
                Bento.audio.stopSound('sfx_button');
                Bento.audio.playSound('sfx_button');
            },
            components: [
                selectedBuddy2Shadow,
                new Sprite({
                    imageName: 'gui2/loadout/select_buddy_character',
                    frameCountX: 5,
                    frameCountY: 1,
                    originRelative: new Vector2(0.5, 0.5),
                    animations: {
                        default: {
                            speed: 0.2,
                            frames: [0, 1, 2, 3, 4]
                        }
                    }
                }),
                selectedBuddy2,
                new EventListener({
                    name: 'updateAlly1',
                    onEvent: function (character) {
                        if (!character) {
                            selectedBuddy2Sprite.visible = false;
                            selectedBuddy2Bottom.visible = false;
                            selectedBuddy2ShadowSprite.visible = false;
                            return;
                        }
                        selectedBuddy2Sprite.visible = true;
                        selectedBuddy2Bottom.visible = true;
                        selectedBuddy2ShadowSprite.visible = true;
                        setSelectedBuddy2Sprite(character);
                    }
                }),
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.5, 0.5),
                        offset: new Vector2(64, 8)
                    },
                    landscape: {
                        screenPosition: new Vector2(0.3, 0.75),
                        offset: new Vector2(64, -6)
                    }
                }),
                badgeBuddy2
            ]
        });
        /**
         * Play button
         */
        var textPlay = new ShadowText({
            position: new Vector2(0, -2),
            text: Localization.getText('play'),
            font: 'font',
            fontSize: 24,
            fontColor: '#ffffff',
            shadowColor: '#0e6049',
            align: 'center',
            textBaseline: 'middle',
            maxWidth: 96
        });
        var buttonPlay = new ClickButton({
            name: 'buttonPlay',
            sfx: 'sfx_button',
            imageName: 'gui2/misc/button_large_primary',
            frameCountX: 1,
            frameCountY: 3,
            float: false,
            updateWhenPaused: 0,
            components: [
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.5, 0.8),
                        offset: new Vector2(0, -8)
                    },
                    landscape: {
                        screenPosition: new Vector2(0.75, 0.6),
                        offset: new Vector2(0, 12)
                    }
                }),
                textPlay
            ],
            onButtonDown: function () {
                textPlay.position.y = -1;
            },
            onButtonUp: function () {
                textPlay.position.y = -2;
            },
            onClick: function () {
                // modal
                Bento.objects.pause(Bento.objects.isPaused() + 1);
                var direction = 'y';
                if (!Anchor.isPortrait()) {
                    direction = 'x';
                }
                // tween out
                new Tween({
                    from: entity.position[direction],
                    to: entity.position[direction] - 180,
                    in: 20,
                    ease: 'easeInBack',
                    onUpdate: function (v, t) {
                        entity.position[direction] = v;
                    }
                }).attach(new OrientationLock({}));

                // temporarily re-attach so it doesn't move
                topBar.removeSelf();
                Bento.objects.attach(topBar);

                EventSystem.fire('loadOutClosed');
                // bug in replaykit
                EventSystem.fire('updateRecordingStatus');

                new Tween({
                    from: 1,
                    to: 0,
                    in: 20,
                    onUpdate: function (v, t) {
                        entity.alpha = v;
                        bottomBar.alpha = v;
                    },
                    onComplete: function () {
                        Bento.objects.pause(Bento.objects.isPaused() - 1);
                        bottomBar.hide();
                        bottomBar.alpha = 1;
                        topBar.removeSelf();
                        if (SaveManager.load('tutorialWorldSelect') > 0) {
                            Analytics.trackView('worldMap');
                        }
                    }
                });
            }
        });
        /**
         * Separator
         */
        var separatorSprite = new Sprite({
            imageName: 'gui2/misc/separator',
            originRelative: new Vector2(0.5, 0.5),
            frameCountY: 2,
            animations: {
                default: {
                    frames: [0]
                },
                wide: {
                    frames: [1]
                }
            }
        });
        var separator = new Entity({
            name: 'separator',
            components: [
                separatorSprite,
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.5, 0.75),
                        offset: new Vector2(0, -12)
                    },
                    landscape: {
                        screenPosition: new Vector2(0.75, 0.5),
                        offset: new Vector2(4, 0)
                    },
                    onComplete: function () {
                        if (Anchor.isPortrait()) {
                            separator.rotation = 0;
                            separatorSprite.setAnimation('default');
                        } else {
                            separator.rotation = Math.PI / 2;
                            separatorSprite.setAnimation('wide');
                        }
                    }
                })
            ]
        });
        /**
         * Consumables
         */
        var consumables = new EquipmentSlots({
            iconImage: '',
            type: 'consumable',
            slotSize: 'new',
            components: [
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.5, 0.65),
                        offset: new Vector2(-16, 0)
                    },
                    landscape: {
                        screenPosition: new Vector2(0.75, 0.3),
                        offset: new Vector2(-16, 0)
                    }
                })
            ]
        });
        var autoText = new Text({
            position: new Vector2(16, -2),
            text: Localization.getText('autoSelect'),
            font: 'font',
            fontSize: 12,
            fontColor: '#717171',
            align: 'left',
            textBaseline: 'middle',
            maxWidth: 64,
            maxHeight: 24,
        });
        var autoSelect = new ToggleButton({
            name: 'autoSelect',
            boundingBox: new Rectangle(-16, -10, 48, 20),
            imageName: 'gui2/loadout/autoselect_items',
            frameCountX: 2,
            frameCountY: 1,
            toggled: SaveManager.load('autoSelect'),
            animations: {
                up: {
                    frames: [0]
                },
                down: {
                    frames: [1]
                }
            },
            onToggle: function () {
                SaveManager.save('autoSelect', autoSelect.isToggled());
            },
            components: [
                autoText,
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.5, 0.5),
                        offset: new Vector2(-26, 72)
                    },
                    landscape: {
                        screenPosition: new Vector2(0.75, 0.3),
                        offset: new Vector2(-26, 32)
                    }
                })
            ]
        });
        var autoSelectItems = function () {
            var itemsInInventory;
            var slot1 = consumables.getSlot(0);
            var slot2 = consumables.getSlot(1);
            var randomItem;

            itemsInInventory = InventoryManager.getAllItems(['consumables', 'weapons']);

            // fill slot 1
            if (!slot1 && itemsInInventory.length > 0) {
                randomItem = itemsInInventory[Utils.getRandom(itemsInInventory.length)];
                consumables.setSlot(0, randomItem.key);
            }

            // refresh inventory
            itemsInInventory = InventoryManager.getAllItems(['consumables', 'weapons']);

            // fill slot 2
            if (!slot2 && itemsInInventory.length > 0) {
                randomItem = itemsInInventory[Utils.getRandom(itemsInInventory.length)];
                consumables.setSlot(1, randomItem.key);
            }

        };
        /**
         * Settings button
         */
        var buttonSettings = new IconButton({
            name: 'buttonSettings',
            sfx: 'sfx_button',
            // imageName: 'gui2/misc/buttonmini_small_secondary',
            // iconName: 'gui/buttons/settings',
            imageName: 'gui/buttons/settings',
            frameCountX: 2,
            frameCountY: 1,
            float: false,
            updateWhenPaused: 0,
            onClick: function () {
                var menu = new SettingsMenu({});
                Bento.objects.attach(menu);
            },
            components: [
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0, 1),
                        offset: new Vector2(19, -44)
                    },
                    landscape: {
                        screenPosition: new Vector2(0, 1),
                        offset: new Vector2(20, -20)
                    }
                })
            ]
        });
        /**
         * Gift button
         */
        var buttonGift = new GiftButton({});
        /**
         * Ad button
         */
        var buttonAd = new AdForCoins({});
        /**
         * Behavior
         */
        var availableCharacters = CharacterManager.getAvailableCharacters();
        var availableAllies = AlliesManager.getAvailable();
        var initialPause;
        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {
                // another check for unresolved IAP
                var unresolvedPurchases = Bento.saveState.load('unresolvedPurchases', []);
                if (unresolvedPurchases.length) {
                    Bento.screens.show('screens/iap');
                    return;
                }

                // disable allies until first unlocked or not reached mission 6 yet
                var unlockedLevels = SaveManager.load('unlockedLevels').length;
                var tutorialAllies = SaveManager.load('tutorialAllies');
                if (tutorialAllies === 0 && unlockedLevels < 7) {
                    baseBuddy1.visible = false;
                    baseBuddy1.setActive(false);
                    baseBuddy2.visible = false;
                    baseBuddy2.setActive(false);
                }

                initialPause = Bento.objects.isPaused();
                // set main character
                setSelectedSprite();
                // set buddies
                var allies = AlliesManager.getAllies();

                if (allies[0]) {
                    selectedBuddy1Bottom.visible = true;
                    setSelectedBuddy1Sprite(allies[0]);
                } else {
                    selectedBuddy1Bottom.visible = false;
                }
                if (allies[1]) {
                    selectedBuddy2Bottom.visible = true;
                    setSelectedBuddy2Sprite(allies[1]);
                } else {
                    selectedBuddy2Bottom.visible = false;
                }

                // show daily chest
                if (!Globals.roomCodeInvite) {
                    if (canClaimDailyChest()) {
                        Utils.timeout(30, function () {
                            if (TimeManager.hasUpdatedTime || firstSession) {
                                // we only check for the daily chest if time manager is properly updated
                                // with exception for the first time of the app lifetime
                                firstSession = false;
                                showDailyChest(function () {
                                    showTutorial();
                                });
                            }
                        });
                    } else {
                        showTutorial();
                    }
                }

                // tween in
                if (tweenIn) {
                    new Tween({
                        from: 0,
                        to: 1,
                        in: 5,
                        onUpdate: function (v, t) {
                            entity.alpha = v;
                        }
                    });
                }
            },
            destroy: function (data) {},
            update: function (data) {
                // check if badges should be shown
                var showMain = false;
                var showBuddy = false;
                Utils.forEach(availableCharacters, function (character, i, l, breakloop) {
                    if (behavior.checkMain(character)) {
                        showMain = true;
                        breakloop();
                    }
                });
                Utils.forEach(availableAllies, function (character, i, l, breakloop) {
                    if (behavior.checkAlly(character)) {
                        showBuddy = true;
                        breakloop();
                    }
                });

                badgeMain.visible = showMain;
                badgeBuddy1.visible = showBuddy;
                badgeBuddy2.visible = showBuddy;

            },
            checkMain: function (character) {
                // we check if player has enough tokens to unlock or upgrade
                // don't care about coins
                var isUnlocked = CharacterManager.isUnlocked(character);
                var characterJson = Bento.assets.getJson('stats/' + character);
                var level = StatsManager.getLevel(character);
                var upgradeCost = Globals.getTokenCost(level);
                var tokens = InventoryManager.getTokens('playertokens/' + character);

                if (!isUnlocked) {
                    // swap upgradecost with unlock cost
                    upgradeCost = CharacterManager.getUnlockCost(character);
                }

                if (tokens >= upgradeCost) {
                    return true;
                } else {
                    return false;
                }
            },
            checkAlly: function (character) {
                var isUnlocked = AlliesManager.isUnlocked(character);
                var level = AlliesManager.getLevel(character);
                var upgradeCost = AlliesManager.getUpgradeCost(character);
                var tokens = InventoryManager.getTokens('enemytokens/' + character);

                if (!isUnlocked) {
                    // swap upgradecost with unlock cost
                    upgradeCost = AlliesManager.getUnlockCost(character);
                }

                if (tokens >= upgradeCost) {
                    return true;
                } else {
                    return false;
                }
            },
            draw: function (data) {}
        };
        var setSelectedSprite = function () {
            // selectedSprite.setup({
            //     imageName: 'gui2/loadout/characters/' + CharacterManager.getSelected(),
            //     originRelative: new Vector2(0.5, 1)
            // });
            selectedShadowSprite.setup({
                imageName: 'gui2/loadout/characters/shadow_' + CharacterManager.getSelected(),
                originRelative: new Vector2(0.5, 0)
            });
            selectedSprite.setup({
                spriteSheet: CharacterManager.getSelected() + '/idle'
            });
            selectedCharacter.scale.x = 2;
            selectedCharacter.scale.y = 2;
        };
        var setSelectedBuddy1Sprite = function (character) {
            // selectedBuddy1Sprite.setup({
            //     imageName: 'gui2/loadout/buddies/' + character,
            //     originRelative: new Vector2(0.5, 1)
            // });
            selectedBuddy1ShadowSprite.setup({
                imageName: 'gui2/loadout/buddies/shadow_' + character,
                originRelative: new Vector2(0.5, 0)
            });
            selectedBuddy1Sprite.setup({
                spriteSheet: 'allies/' + character
            });
            // selectedBuddy1.scale.x = -1.25;
            // selectedBuddy1.scale.y = 1.25;
        };
        var setSelectedBuddy2Sprite = function (character) {
            // selectedBuddy2Sprite.setup({
            //     imageName: 'gui2/loadout/buddies/' + character,
            //     originRelative: new Vector2(0.5, 1)
            // });
            selectedBuddy2ShadowSprite.setup({
                imageName: 'gui2/loadout/buddies/shadow_' + character,
                originRelative: new Vector2(0.5, 0)
            });
            selectedBuddy2Sprite.setup({
                spriteSheet: 'allies/' + character
            });
            // selectedBuddy2.scale.x = 1.25;
            // selectedBuddy2.scale.y = 1.25;
        };
        /**
         * Show daily chest
         */
        var dailyChestMenu = null;
        var showDailyChest = function (onComplete) {
            if (dailyChestMenu) {
                // already showing daily chest
                return;
            }
            if (!canClaimDailyChest() || initialPause !== Bento.objects.isPaused()) {
                // check again to be sure
                return;
            }
            dailyChestMenu = new DailyChest({
                onComplete: function () {
                    // on the 20th chest claimed, we show the elite fighter pack offer
                    var dailyChestData = SaveManager.load('dailyChestData');
                    if (dailyChestData.claimed === 20) {
                        // SaveManager.save('showOffer', 'weapon');
                    }

                    if (onComplete) {
                        onComplete();
                    }
                    dailyChestMenu = null;
                }
            });
            Bento.objects.attach(dailyChestMenu);
        };
        var canClaimDailyChest = function () {
            return DailyChest.canClaim() && unlockedMissionsCount >= 2;
        };
        /**
         * Offer banner
         */
        var dotList = new CenteredList({
            spacing: 1,
            position: new Vector2(0, 26)
        });
        var offersContainer = new Entity({
            name: 'offers',
            components: [
                new Sprite({
                    imageName: 'gui2/offers/banner/bg',
                    originRelative: new Vector2(0.5, 0.5),
                }),
                dotList,
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.5, 0),
                        offset: new Vector2(0, 48),
                    },
                    landscape: {
                        screenPosition: new Vector2(0.75, 0.6),
                        offset: new Vector2(0, -40),
                    },
                    onComplete: function () {}
                })
            ]
        });
        var makeOfferBanners = function () {
            var offers = SaveManager.load('offers');
            var now = TimeManager.getTime();
            var newestOffer;
            var newestTime = 0;
            var scrollingList = new ScrollingList({
                area: new Rectangle(-32, -21, 64, 42), // area that user can grab
                // area: new Rectangle(0, 0, 0, 0), // area that user can grab
                direction: 'x',
                spacing: 4,
                snap: true,
                onSnap: function (item, idx) {
                    dotList.iterate(function (dot, i) {
                        if (i === idx) {
                            dot.getComponent('sprite', function (sprite) {
                                sprite.setAnimation('down');
                            });
                        } else {
                            dot.getComponent('sprite', function (sprite) {
                                sprite.setAnimation('up');
                            });
                        }
                    });
                    // disable buttons by changing their boundingbox
                    scrollingList.iterate(function (button, i) {
                        if (i === idx) {
                            button.boundingBox = new Rectangle(-32, -21, 64, 42);
                        } else {
                            button.boundingBox = new Rectangle(0, 0, 0, 0);
                        }
                    });
                    // reset scroll timer
                    scrollingList.timer = 0;
                },
                snapMinSlideSpeed: 1,
                snapSpeed: 0.3,
                damping: 0.9
            });
            // auto scroll behavior
            scrollingList.attach({
                name: 'scrollBehavior',
                update: function (data) {
                    var currentIndex;
                    var newIndex;
                    var items;
                    var button;
                    if (scrollingList.timer % 240 === 239) {
                        // scroll to next
                        currentIndex = scrollingList.getSelection();
                        newIndex = currentIndex + 1;
                        items = scrollingList.getItems();
                        newIndex %= items.length;
                        if (newIndex !== currentIndex) {
                            // disable button while scrolling
                            button = items[currentIndex];
                            button.boundingBox = new Rectangle(0, 0, 0, 0);
                            scrollingList.snapTo(newIndex);
                        }
                    }
                }
            });

            Utils.forEach(offers, function (offerData, i, l, breakLoop) {
                // is valid offerpack?
                if (!offerData || offerData.time < now) {
                    return;
                }
                var showOffer = function () {
                    var offerMenu = new OfferPackPopup({
                        z: Globals.WorldSelectLayers.IAP + 1,
                        float: true,
                        offer: offerData.name,
                        duration: 48
                    });
                    Bento.objects.attach(offerMenu);
                };
                var offerPack = new Sprite({
                    spriteSheet: 'offers/' + offerData.name
                });
                var title = new ShadowText({
                    position: new Vector2(0, -18),
                    text: Localization.getText('offerpacks')[offerData.name].toUpperCase(),
                    font: 'font',
                    fontSize: 10,
                    fontColor: '#ffe10f',
                    shadowColor: '#4f3875',
                    align: 'center',
                    textBaseline: 'middle',
                    maxWidth: 60,
                    maxHeight: 10
                });
                var extraText = new Text({
                    position: new Vector2(0, 20),
                    text: Localization.getText('limitedOffer'),
                    font: 'font',
                    fontSize: 6,
                    fontColor: '#ffffff',
                    align: 'center',
                    textBaseline: 'middle',
                    lineWidth: 1,
                    strokeStyle: '#4f3875',
                    maxWidth: 60,
                    maxHeight: 6
                });
                var maskSprite = new MaskSprite({
                    sprite: offerPack,
                    boundingBox: new Rectangle(-32, -80, 64, 160)
                });
                var maskSpriteText = new MaskSprite({
                    sprite: title.getComponent('sharpnessScaler').getComponent('sprite'),
                    boundingBox: new Rectangle(-32, -80, 64 * 4, 160)
                });
                var maskSpriteText2 = new MaskSprite({
                    sprite: extraText.getComponent('sharpnessScaler').getComponent('sprite'),
                    boundingBox: new Rectangle(-32, -80, 64 * 4, 160)
                });
                var maskBehavior = {
                    name: 'setMask',
                    start: function () {
                        maskBehavior.update();
                    },
                    update: function () {
                        var scroll = scrollingList.getScroll();
                        var dist;
                        // sets the maskSprite's boundingbox
                        maskSprite.boundingBox.x = -32 - offerPack.parent.position.x - scroll;
                        // * 4 because of text's sharpness
                        maskSpriteText.boundingBox.x = (-32 - title.parent.position.x - scroll) * 4;
                        maskSpriteText2.boundingBox.x = (-32 - extraText.parent.position.x - scroll) * 4;
                    }
                };
                var eventListener = new EventListener({
                    name: 'offerpackBought',
                    onEvent: function (eventData) {
                        var items = scrollingList.getItems();
                        var index = items.indexOf(container);
                        if (eventData.offer === offerData.name && index >= 0) {
                            // remove from list
                            scrollingList.removeItem(index);

                            // list is empty
                            if (scrollingList.parent && scrollingList.getItems().length === 0) {
                                scrollingList.parent.removeSelf();
                            }
                        }
                    }
                });
                var container = new ClickButton({
                    name: 'offerPack',
                    sfx: 'sfx_button',
                    position: new Vector2(0, -2),
                    boundingBox: scrollingList.length() === 0 ? new Rectangle(-32, -21, 64, 42) : new Rectangle(0, 0, 0, 0),
                    onClick: function () {
                        if (scrollingList.didMove(1)) {
                            return;
                        }
                        showOffer();
                    },
                    components: [
                        offerPack,
                        title,
                        extraText,
                        eventListener,
                        maskSprite,
                        maskSpriteText,
                        maskSpriteText2,
                        maskBehavior
                    ]
                });
                var dotSprite = new Sprite({
                    frameCountX: 2,
                    originRelative: new Vector2(0.5, 0.5),
                    imageName: 'gui2/offers/banner/dots',
                    animations: {
                        'up': {
                            frames: [1]
                        },
                        'down': {
                            frames: [0]
                        }
                    }
                });
                dotList.addItem(new Entity({
                    name: 'dot',
                    components: [
                        dotSprite
                    ]
                }));
                if (scrollingList.getItems().length === 0) {
                    dotSprite.setAnimation('down');
                } else {
                    dotSprite.setAnimation('up');
                }
                // attach
                scrollingList.addItem(container);

                // set time text
                var timeLeft;
                var minutes;
                var hours;
                timeLeft = Math.max(0, offerData.time - now);
                minutes = Math.floor(timeLeft / 1000 / 60) % 60;
                hours = Math.floor(timeLeft / 1000 / 60 / 60);
                extraText.setText(Localization.getText('offerTime').replace('{HOURS}', hours).replace('{MINUTES}', minutes));
            });

            // only attach if there are any offers
            if (scrollingList.getItems().length > 0) {
                offersContainer.attach(scrollingList);
                entity.attach(offersContainer);
                new Tween({
                    from: 0,
                    to: 1,
                    in: 15,
                    ease: 'elastic',
                    decay: 5,
                    oscillations: 1,
                    onUpdate: function (v, t) {
                        offersContainer.scale.x = v;
                        offersContainer.scale.y = v;
                    }
                });
            }
        };
        /**
         * Show offerpack at random
         */
        var showOffer = function () {
            var unlocked = SaveManager.load('unlockedLevels').length;
            var bootups = SaveManager.load('bootupsForOffer');
            var offersBought = SaveManager.load('offersBought');
            var showOfferPopup = function (offer) {
                var offerPack = new OfferPackPopup({
                    z: Globals.WorldSelectLayers.CHESTPOPUP,
                    float: true,
                    offer: offer,
                    duration: times[offer] || 24
                });
                Bento.objects.attach(offerPack);
                SaveManager.add('offersShown-' + offer, 1);
            };
            var times = {
                starter: 24 * 2,
                powerup: 24 * 2,
                gem: 24 * 2,
                weapon: 24 * 2
            };
            // uncomment this one to test
            /*Bento.objects.attach(new OfferPackPopup({
                z: Globals.WorldSelectLayers.CHESTPOPUP,
                float: true,
                offer: 'powerup',
                duration: 48
            }));*/

            // don't show anything for beta testers
            if (Globals.BETA_TEST) {
                return;
            }

            // show starter pack after certain missions
            if (SaveManager.load('showStarterPack') && !offersBought['starter']) {
                SaveManager.save('showStarterPack', false);
                showOfferPopup('starter');
                return;
            }
            if (SaveManager.load('showOffer') === 'powerup' && !offersBought['powerup']) {
                SaveManager.save('showOffer', '');
                showOfferPopup('powerup');
                return;
            }
            if (SaveManager.load('showOffer') === 'gem' && !offersBought['gem']) {
                SaveManager.save('showOffer', '');
                showOfferPopup('gem');
                return;
            }
            if (SaveManager.load('showOffer') === 'weapon' && !offersBought['weapon']) {
                SaveManager.save('showOffer', '');
                showOfferPopup('weapon');
                return;
            }
        };
        /**
         * Worldselect ready
         */
        var eventListener = new EventListener({
            name: 'worldSelectReady',
            onEvent: function () {
                var didShow = false;
                // check the pauselevel to see if there are no modals active
                if (Bento.objects.isPaused() === Globals.PauseLevels.LOADOUT) {
                    didShow = showCrossPromo();
                    if (!didShow) {
                        showInterstitial();
                    }
                }
            }
        });
        var characterSelectDismissed = new EventListener({
            name: 'characterSelectDismissed',
            onEvent: function (data) {
                var didShow = false;
                // check the pauselevel to see if there are no modals active
                if (Bento.objects.isPaused() === Globals.PauseLevels.LOADOUT) {
                    didShow = showCrossPromo();
                    if (!didShow) {
                        showInterstitial();
                    }
                }
            }
        });
        /**
         * Check for daily chest after resume
         */
        var dailyChestEventListener = new EventListener({
            name: 'serverTimeUpdated',
            onEvent: function (data) {
                // we also check the pause level to be sure there are no other modal popups currently open
                if (canClaimDailyChest() && initialPause === Bento.objects.isPaused()) {
                    showDailyChest();
                }
            }
        });
        /**
         * Modal
         */
        var modal = new Modal({
            pauseLevel: Globals.PauseLevels.LOADOUT
        });
        var topBar = new Topbar({});
        /**
         * Main entity
         */
        var entity = new Entity({
            z: Globals.WorldSelectLayers.GUI,
            name: 'startScreen',
            float: true,
            components: [
                modal,
                background,
                baseBuddy1,
                baseBuddy2,
                baseMain,
                separator,
                /*consumables,
                autoSelect,*/
                buttonPlay,
                buttonSettings,
                // buttonGift,
                // buttonAd,
                topBar,
                eventListener,
                dailyChestEventListener,
                characterSelectDismissed,
                behavior // must be on bottom
            ]
        }).extend({
            openCharacterSelect: function () {
                var characterSelect = new CharacterSelectMenu({});
                Bento.objects.attach(characterSelect);
            }
        });
        var unlockedMissionsCount = SaveManager.load('unlockedLevels').length;
        var askNotificationEnergy = function () {
            var dialog = new Dialog({
                z: Globals.WorldSelectLayers.DIALOGS,
                name: 'askForNotificationEnergy',
                type: 'yesno',
                title: Localization.getText('notifications')['askTitle'],
                text: Localization.getText('notifications')['askEnergy'],
                canClose: false,
                icon: new Entity({
                    components: [
                        new Sprite({
                            imageName: 'gui/notifications-icon',
                            originRelative: new Vector2(0.5, 0.5)
                        })
                    ]
                }),
                onYes: function () {
                    // ask for real notification
                    Notifications.init(function (initialized) {
                        var allowed = initialized ? true : false;
                        SaveManager.save('notificationsAllowed', allowed);
                        if (allowed) {
                            // fire event to schedule inactive notification
                            EventSystem.fire('allowedNotifications');
                            // fire event to schedule energy notification
                            EventSystem.fire('energySubtracted', SaveManager.load('energy-boy'));
                        } else {
                            // user pressed no at the system dialog...
                        }
                    });
                },
                onNo: function () {
                    // do nothing, better luck next time
                }
            });
            Bento.objects.attach(dialog);

            var asked = SaveManager.load('notificationsAsked');
            asked += 1;
            SaveManager.save('notificationsAsked', asked);

            SaveManager.add('askNotificationEnergy', 1);
        };
        var showCrossPromo = function () {
            // show crosspromo when energy is starting to get low
            var lastTimeSawCrossPromo = SaveManager.load('lastTimeSawCrossPromo');
            var now = TimeManager.getTime();
            var timeDiff = now - lastTimeSawCrossPromo;
            var didShow = false;
            if (
                timeDiff > 1000 * 60 * 10 && // 10 minutes
                CharacterManager.getEnergy('boy') < 2 &&
                Bento.saveState.load('iapAmount', 0) < 1
            ) {
                // static image as crosspromo
                new StaticCrossPromo({
                    z: Globals.WorldSelectLayers.BOTTOMBAR + 1,
                    pauseLevel: Globals.PauseLevels.BOTTOMBAR + 10,
                    sfx: 'sfx_button'
                });
                SaveManager.save('lastTimeSawCrossPromo', TimeManager.getTime());
                didShow = true;
            }
            return didShow;
        };
        var showInterstitial = function (force) {
            var lastTimeSawInterstitial = SaveManager.load('lastTimeSawInterstitial');
            var now = TimeManager.getTime();
            var timeDiff = now - lastTimeSawInterstitial;
            if (
                timeDiff > 1000 * 60 * 20 || lastTimeSawInterstitial === null || force// 20 minutes
            ) {
                // static image as crosspromo
                new StaticInterstitial({
                    z: Globals.WorldSelectLayers.BOTTOMBAR + 1,
                    pauseLevel: Globals.PauseLevels.BOTTOMBAR + 10,
                    sfx: 'sfx_button'
                });
                SaveManager.save('lastTimeSawInterstitial', TimeManager.getTime());
            }
        };
        var showTutorial = function () {
            var tutorialCharacter = SaveManager.load('tutorialCharacter');
            var tutorialAllies = SaveManager.load('tutorialAllies');
            var isShowingTapHandLevelUp = false;
            var didShowTutorial = false;
            var canLevelUp = function () {
                // can boy level up first time?
                var upgradeCost = Globals.getTokenCost(1);
                var tokens = InventoryManager.getTokens('playertokens/boy');
                return tokens >= upgradeCost;
            };
            var addHandToPlayButton = function () {
                // put a taphand on the play button
                var tapHandPlay = new TapHand({
                    position: new Vector2(0, -8),
                    components: [
                        new Anchor({
                            onComplete: function () {
                                if (Anchor.isPortrait()) {
                                    tapHandPlay.position.x = 0;
                                    tapHandPlay.rotation = 0;
                                } else {
                                    tapHandPlay.position.x = -32;
                                    tapHandPlay.rotation = -Math.PI / 4;
                                }
                            }
                        })
                    ]
                });
                buttonPlay.attach(tapHandPlay);

                if (isShowingTapHandLevelUp) {
                    // wait until user upgrades
                    tapHandPlay.visible = false;
                    tapHandPlay.attach(new EventListener({
                        name: 'tutorialCharacter',
                        onEvent: function (data) {
                            tapHandPlay.visible = true;
                        }
                    }));
                }
            };
            var addHandToCharacter = function (removeOnDismiss) {
                var mainTapHand = new TapHand({
                    position: new Vector2(-8, -16),
                    rotation: Math.PI * -3 / 4,
                    components: [
                        // remove self when tutorial is over
                        new EventListener({
                            name: 'tutorialCharacter',
                            onEvent: function () {
                                mainTapHand.removeSelf();
                            }
                        })
                    ]
                });
                var offerListener = new EventListener({
                    name: 'characterSelectDismissed',
                    onEvent: function (data) {
                        var offersBought = SaveManager.load('offersBought');
                        if (removeOnDismiss) {
                            // taphand 2nd time
                            mainTapHand.removeSelf();
                        } else if (!offersBought['starter']) {
                            // show offers now
                            Bento.objects.attach(new SenseiPopup({
                                z: Globals.WorldSelectLayers.BOTTOMBAR + 2,
                                darken: true,
                                modal: true,
                                onNoMoreText: function () {
                                    this.dismiss();
                                },
                                onComplete: function () {
                                    showOffer();
                                    makeOfferBanners();
                                },
                                text: Localization.getText('offerSensei')
                            }));
                            baseMain.remove(offerListener);
                        }
                    }
                });
                baseMain.attach(mainTapHand);
                isShowingTapHandLevelUp = true;

                // when user closes character select, the offer will show
                baseMain.attach(offerListener);
            };

            // user can level up the first time
            if (tutorialCharacter === 0 && canLevelUp()) {
                addHandToCharacter();
                didShowTutorial = true;
            }

            if (unlockedMissionsCount <= 1) {
                // start off in the worldmap instead of loadout
                bottomBar.hide();
                Bento.objects.get('buttonBack', function (buttonBack) {
                    buttonBack.removeSelf();
                });
                didShowTutorial = true;
            } else if (unlockedMissionsCount === 2) {
                addHandToPlayButton();
                didShowTutorial = true;
            } else if (unlockedMissionsCount === 3) {
                // show hand a 2nd time on character
                if (tutorialCharacter > 0 && canLevelUp()) {
                    addHandToCharacter(true);
                }
                didShowTutorial = true;
            }

            // put hand on free gift
            if (SaveManager.load('tutorialFreeGift') < 1 && buttonGift.parent) {
                var giftTapHand = new TapHand({
                    position: new Vector2(0, 8),
                    rotation: Math.PI
                });
                buttonGift.attach(giftTapHand);

                // remove self when tutorial is over
                giftTapHand.attach(new EventListener({
                    name: 'tutorialFreeGift',
                    onEvent: function () {
                        giftTapHand.removeSelf();
                    }
                }));
                didShowTutorial = true;
            }

            if (tutorialCharacter > 0 && tutorialAllies === 0) {
                // check if any ally can be leveled up
                Utils.forEach(availableAllies, function (character, i, l, breakloop) {
                    if (behavior.checkAlly(character)) {
                        // add taphands
                        var buddy1TapHand = new TapHand({
                            position: new Vector2(0, -8),
                            rotation: 0,
                            components: [
                                new EventListener({
                                    name: 'tutorialAllies',
                                    onEvent: function () {
                                        buddy1TapHand.removeSelf();
                                    }
                                })
                            ]
                        });
                        var buddy2TapHand = new TapHand({
                            position: new Vector2(0, -8),
                            rotation: 0,
                            components: [
                                new EventListener({
                                    name: 'tutorialAllies',
                                    onEvent: function () {
                                        buddy2TapHand.removeSelf();
                                    }
                                })
                            ]
                        });
                        baseBuddy1.attach(buddy1TapHand);
                        baseBuddy2.attach(buddy2TapHand);
                        // re-activate
                        baseBuddy1.visible = true;
                        baseBuddy1.setActive(true);
                        baseBuddy2.visible = true;
                        baseBuddy2.setActive(true);
                        didShowTutorial = true;

                        breakloop();
                    }
                });
            }

            // ask user for notifications
            if (
                SaveManager.load('askNotificationEnergy') === 0 &&
                CharacterManager.getEnergy('boy') < 2 &&
                !SaveManager.load('notificationsAllowed')
            ) {
                Utils.timeout(10, function () {
                    Bento.objects.attach(new SenseiPopup({
                        z: Globals.WorldSelectLayers.BOTTOMBAR + 2,
                        darken: false,
                        modal: true,
                        onNoMoreText: function () {
                            this.dismiss();
                            new Tween({
                                from: 1,
                                to: 0,
                                in: 10,
                                ease: 'linear',
                                onUpdate: function (v, t) {
                                    highlight.alpha = v;
                                },
                                onComplete: function () {
                                    highlight.removeSelf();
                                }
                            });
                        },
                        onComplete: function () {
                            askNotificationEnergy();
                        },
                        text: Localization.getText('outOfEnergy')
                    }));
                    var highlight = new Highlight({
                        z: Globals.WorldSelectLayers.BOTTOMBAR + 1,
                        radius: 24,
                        alpha: 0.8,
                        position: new Vector2(24 + viewport.x, 12 + viewport.y)
                    });
                    Bento.objects.attach(highlight);
                });
                didShowTutorial = true;
            }

            // show offers if any (after user leveled up)
            if (tutorialCharacter) {
                showOffer();
                makeOfferBanners();
            }

            return didShowTutorial;
        };

        // attach gift button after mission 2
        if (unlockedMissionsCount > 2) {
            entity.attach(buttonGift);
        }

        // attach ad button after mission 3
        if (unlockedMissionsCount > 2) {
            entity.attach(buttonAd);
        }
        return entity;
    };
});