/**
 * Mission Details menu
 * @moduleName MissionDetailsSingle
 */
bento.define('gui/menus/missiondetailssingle', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'components/anchor',
    'gui/buttons/startbutton',
    'gui/levelstart/rewardspopup',
    'managers/save',
    'managers/stats',
    'gui/dialog',
    'modules/localization',
    'globals',
    'components/modal',
    'gui/blurbackground',
    'gui/worldselect/topbar',
    'gui/iconbutton',
    'gui/menus/missiondetails',
    'gui/menus/online',
    'gui/shadowtext',
    'bento/gui/togglebutton',
    'modules/replaykit',
    'components/backbutton'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Anchor,
    StartButton,
    RewardsPopup,
    SaveManager,
    StatsManager,
    Dialog,
    Localization,
    Globals,
    Modal,
    BlurBackground,
    Topbar,
    IconButton,
    MissionDetails,
    OnlineMenu,
    ShadowText,
    ToggleButton,
    ReplayKit,
    BackButton
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var missionData = settings;
        var blurbackground = new BlurBackground({
            darken: 0.5
        });
        var startGame = function (chosenDifficulty) {
            var startButton = new StartButton({
                enemyLevel: missionData.enemyLevel,
                cost: missionData.cost,
                mission: missionData.mission,
                unlocks: missionData.unlocks,
                rewards: missionData.rewards,
                chosenDifficulty: chosenDifficulty
            });
            startButton.getComponent('buttonPlay').doCallback();
        };
        var textOnline = new ShadowText({
            position: new Vector2(0, 8),
            text: Localization.getText('coop').toUpperCase(),
            font: 'font',
            fontSize: 12,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            maxWidth: 40,
            maxHeight: 12,
            components: []
        });
        var iconWifi = new Entity({
            name: 'iconWifi',
            position: new Vector2(0, -4),
            components: [
                new Sprite({
                    imageName: 'gui/wifi-icon',
                    originRelative: new Vector2(0.5, 0.5)
                })
            ]
        });
        var iconInfo = new Entity({
            name: 'icon',
            components: [
                textOnline,
                iconWifi
            ]
        });
        var buttonOnline = new IconButton({
            name: 'buttonOnline',
            sfx: 'sfx_button',
            icon: iconInfo,
            imageName: 'gui2/misc/button_small_secondary',
            frameCountX: 1,
            frameCountY: 3,
            onClick: function () {
                // var missionDetails = new MissionDetails(settings);
                // Bento.objects.attach(missionDetails);
                settings.chosenDifficulty = rewardsPopup.getChosenDifficulty();

                var onlineMenu = new OnlineMenu(settings);
                Bento.objects.attach(onlineMenu);
            },
            components: [
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0, 0.5),
                        offset: new Vector2(64, -44),
                    },
                    landscape: {
                        screenPosition: new Vector2(0, 0.5),
                        offset: new Vector2(64 + 64, -44),
                    },
                    onComplete: function () {}
                })
            ]
        });
        var rewardsPopup = new RewardsPopup({
            isOnline: false,
            mission: missionData.mission,
            cost: missionData.cost,
            unlocks: missionData.unlocks,
            enemyLevel: missionData.enemyLevel,
            rewards: missionData.rewards,
            onClick: function (chosenDifficulty) {
                var dialog;
                var currentCharacter;
                // single player: start the game
                currentCharacter = SaveManager.load('selectedCharacter');
                // check if the enemy level is 4 higher
                if (StatsManager.getLevel(currentCharacter) + 5 <= missionData.enemyLevel + Globals.difficultyJump * chosenDifficulty) {
                    // check if mission is not too difficult
                    dialog = new Dialog({
                        name: 'tooDifficulty',
                        type: 'yesno',
                        title: Localization.getText('warningTitle'),
                        text: Localization.getText('warnTooDifficult'),
                        canClose: false,
                        onYes: function () {
                            startGame(chosenDifficulty);
                        }
                    });
                    Bento.objects.attach(dialog);
                } else {
                    startGame(chosenDifficulty);
                }
            },
            onClose: function () {
                // TODO: tween
                entity.removeSelf();
            },
            components: [
                buttonOnline,
                new Anchor({
                    screenPosition: new Vector2(0.5, 0.5),
                    offset: new Vector2(0, 8),
                    onComplete: function (parent, isPortrait) {
                        if (!isPortrait) {
                            rewardsPopup.position.y += 8;
                        }
                    }
                })
            ]
        });
        var topbar = new Topbar({});
        var modal = new Modal({
            surpressWarnings: true,
            pauseLevel: Globals.PauseLevels.LEVELDETAILS
        });
        var recIcon = new Entity({
            name: 'recIcon',
            position: new Vector2(8 + 7, 0),
            components: [
                new Sprite({
                    imageName: 'gui/rec',
                    originRelative: new Vector2(0.5, 0.5),
                })
            ]
        });
        var recordText = new ShadowText({
            position: new Vector2(8 + 14, -2),
            text: Localization.getText('record'),
            font: 'font',
            fontSize: 12,
            fontColor: '#ffffff',
            align: 'left',
            textBaseline: 'middle',
            maxWidth: 32,
            maxHeight: 12,
            linebreaks: false
        });
        var buttonRecord = new ToggleButton({
            name: 'buttonRecord',
            boundingBox: new Rectangle(-16, -10, 48, 20),
            imageName: 'gui/buttons/checkbox',
            sfx: 'sfx_button',
            frameCountX: 3,
            frameCountY: 1,
            toggled: Globals.isRecording,
            animations: {
                up: {
                    frames: [0]
                },
                down: {
                    frames: [1]
                }
            },
            onToggle: function () {
                Globals.isRecording = buttonRecord.isToggled();
            },
            components: [
                recIcon,
                recordText,
                new Anchor({
                    portrait: {
                        // screenPosition: new Vector2(0.5, -0.5),
                        // offset: new Vector2(-48, 58),
                        screenPosition: new Vector2(0, 0.5),
                        offset: new Vector2(-16, -66),
                    },
                    landscape: {
                        screenPosition: new Vector2(0.5, -0.5),
                        offset: new Vector2(-64, 16),
                    },
                    onComplete: function () {
                        if (Anchor.isPortrait()) {
                            buttonOnline.position.y += 8;
                            rewardsPopup.getComponent('startButton', function (component) {
                                component.position.y += 8;
                            });
                            rewardsPopup.getComponent('buttonClose', function (component) {
                                component.position.y += 8;
                            });

                            if (Anchor.isWideScreen()) {
                                buttonRecord.position.y -= 16;
                            }
                        }
                    }
                })
            ]
        });
        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {
                var unlocked = SaveManager.load('unlockedLevels');
                // only on iOS
                // it's not possible to record while broadcasting
                if (!Globals.isBroadcasting && Utils.isNativeIos() && unlocked.length > 1) {
                    ReplayKit.isAvailable(function (available) {
                        if (available) {
                            rewardsPopup.attach(buttonRecord);
                        } else {
                            console.log('ReplayKit recorder is not available');
                        }
                    });
                }
                // TEST
                if (Bento.isDev() && unlocked.length > 1) {
                    rewardsPopup.attach(buttonRecord);
                }

                if (false && Bento.saveState.load('jailBreak', false)) {
                    buttonOnline.setActive(false);
                }
            },
            destroy: function (data) {},
            update: function (data) {},
            draw: function (data) {}
        };
        var entity = new Entity({
            z: Globals.WorldSelectLayers.LEVELDETAILS,
            name: 'missionDetails',
            float: true,
            components: [
                modal,
                blurbackground,
                behavior,
                rewardsPopup,
                topbar
            ]
        });
        return entity;
    };
});