/**
 * The multiplayer menu is the where you can select how many players to connect with
 * @moduleName MultiplayerMenu
 */
bento.define('gui/menus/multiplayer', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'globals',
    'components/modal',
    'gui/blurbackground',
    'components/anchor',
    'modules/matchmanager',
    'gui/headerbox',
    'modules/localization',
    'gui/shadowtext',
    'gui/dialog',
    'gui/loadingspinner',
    'gui/loadingdialog'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Globals,
    Modal,
    BlurBackground,
    Anchor,
    MatchManager,
    HeaderBox,
    Localization,
    ShadowText,
    Dialog,
    LoadingSpinner,
    LoadingDialog
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var mission = settings.mission;
        var buttonClose = new ClickButton({
            z: 0,
            name: 'buttonClose',
            sfx: 'sfx_button',
            imageName: 'gui/buttons/exit',
            frameCountX: 2,
            frameCountY: 1,
            onClick: function () {
                Bento.objects.remove(entity);
            }
        }).attach(new Anchor({
            screenPosition: new Vector2(0, 0),
            offset: new Vector2(20, 20)
        }));
        var background = new BlurBackground({});
        var behavior = {
            name: 'behavior',
            start: function () {
                // fade in background
                new Tween({
                    from: 0,
                    to: 1,
                    in: 15,
                    ease: 'linear',
                    onUpdate: function (v, t) {
                        background.alpha = v;
                    }
                });
            }
        };
        // headerbox where you can select how many player to connect with
        var playersCount = settings.players || 2;
        var playersText = new ShadowText({
            position: new Vector2(0, -4),
            text: playersCount,
            font: 'font',
            fontSize: 24,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle'
        });
        var buttonPlus = new ClickButton({
            name: 'buttonPlus',
            sfx: 'sfx_button',
            imageName: 'gui/buttons/plus',
            frameCountX: 2,
            frameCountY: 1,
            position: new Vector2(32, 0),
            onClick: function () {
                playersCount += 1;
                if (playersCount > 4) {
                    playersCount = 4;
                }
                playersText.setText(playersCount);
            }
        });
        var buttonMinus = new ClickButton({
            name: 'buttonMinus',
            sfx: 'sfx_button',
            imageName: 'gui/buttons/minus',
            frameCountX: 2,
            frameCountY: 1,
            position: new Vector2(-32, 0),
            onClick: function () {
                playersCount -= 1;
                if (playersCount < 2) {
                    playersCount = 2;
                }
                playersText.setText(playersCount);
            }
        });
        var playersBox = new HeaderBox({
                title: Localization.getText('players'),
                size: new Rectangle(0, 0, 160, 48)
            })
            .attach(new Anchor({
                screenPosition: new Vector2(0.5, 0.5),
                offset: new Vector2(0, 0)
            }))
            .attach(buttonPlus)
            .attach(buttonMinus)
            .attach(playersText);

        // connect
        var connectText = new ShadowText({
            position: new Vector2(0, -4),
            text: Localization.getText('connect'),
            font: 'font',
            fontSize: 16,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle'
        });
        var connectAnchor = new Anchor({
            screenPosition: new Vector2(0.5, 0.5),
            offset: new Vector2(0, 48)
        });
        var buttonConnect = new ClickButton({
                name: 'buttonConnect',
                sfx: 'sfx_button',
                imageName: 'gui/buttons/empty',
                frameCountX: 1,
                frameCountY: 3,
                position: new Vector2(0, 0),
                float: false,
                updateWhenPaused: 0,
                onClick: function () {
                    var findMatch = function () {
                        // show waiting dialog until logged in
                        var loadingMatch = new LoadingDialog({
                            text: Localization.getText('searchingPlayers'),
                            onClose: function () {
                                // user closes while searching for player: this should only happen if there is an error but
                                // cocoon failed to notifiy that
                                Bento.objects.remove(loadingMatch);
                                Bento.objects.remove(entity);
                                MatchManager.disconnect();
                            }
                        });
                        Bento.objects.attach(loadingMatch);

                        MatchManager.findMatch(playersCount, mission, function (status) {
                            // close loading
                            Bento.objects.remove(loadingMatch);

                            // close the menu!
                            Bento.objects.remove(entity);

                            if (!status.success) {
                                var text = Localization.getText('connectError') + status.error;
                                if (status.error === null) {
                                    // not sure, but this could indicate that user canceled
                                    text = Localization.getText('cancelledByUser');
                                }
                                var dialog = new Dialog({
                                    name: 'error_multiplayer',
                                    title: Localization.getText('errorTitle'),
                                    text: text,
                                    canClose: false,
                                    type: 'yes'
                                });
                                Bento.objects.attach(dialog);
                                return;
                            }
                            // Bento.screens.show('screens/main');

                            if (settings.onComplete) {
                                settings.onComplete();
                            }
                        });
                    };
                    var onLogin = function (status) {
                        Bento.objects.remove(loadingLogin);

                        if (!status.success) {
                            var dialog = new Dialog({
                                name: 'error_connectFail',
                                title: 'Error',
                                text: Localization.getText('connectFail'),
                                canClose: false,
                                type: 'yes'
                            });
                            Bento.objects.attach(dialog);
                        } else {
                            // continue with find match
                            findMatch();
                        }
                    };
                    var loadingLogin = new LoadingDialog({
                        text: Localization.getText('connecting')
                    });
                    // connect first
                    if (!MatchManager.isLoggedIn()) {
                        Bento.objects.attach(loadingLogin);
                        MatchManager.login(onLogin);
                    } else {
                        findMatch();
                    }
                },
                onButtonDown: function () {
                    connectText.position.y = -3;
                },
                onButtonUp: function () {
                    connectText.position.y = -4;
                }
            })
            .attach(connectText)
            .attach(connectAnchor);

        // container entity
        var entity = new Entity({
            z: settings.z || Globals.WorldSelectLayers.MULTIPLAYER,
            name: 'multiplayerMenu',
            float: true,
            components: [
                new Modal({}),
                background,
                buttonClose,
                playersBox,
                buttonConnect,
                behavior
            ]
        });
        return entity;
    };
});