/**
 * Menu that lets the player pick Create or Join
 * @moduleName OnlineMenu
 * @snippet OnlineMenu.snippet
OnlineMenu({
    position: new Vector2(0, 0)
})
 */
bento.define('gui/menus/online', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'gui/shadowtext',
    'gui/headerbox',
    'modules/localization',
    'gui/blurbackground',
    'globals',
    'components/anchor',
    'components/modal',
    'gui/iconbutton',
    'gui/menus/onlinecreate',
    'gui/menus/onlinejoin',
    'gui/levelstart/rewardspopup',
    'managers/save',
    'gui/worldselect/topbar',
    'managers/character',
    'gui/energyforgempopup',
    'components/backbutton',
    'gui/menus/randomjoin',
    'managers/stats'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    ShadowText,
    HeaderBox,
    Localization,
    BlurBackground,
    Globals,
    Anchor,
    Modal,
    IconButton,
    OnlineCreate,
    OnlineJoin,
    RewardsPopup,
    SaveManager,
    Topbar,
    CharacterManager,
    EnergyForGemPopup,
    BackButton,
    RandomJoin,
    StatsManager
) {
    'use strict';
    return function (settings) {
        var viewport = Bento.getViewport();
        var invitation = settings.invitation;
        var onClosed = settings.onClosed || function () {};
        var cost = settings.cost || 0;
        var mission = settings.mission || 1;
        var isJoining = settings.isJoining || false;
        var blurBackground = new BlurBackground({
            darken: 0.3,
            blurriness: 15
        });
        var buttonClose = new ClickButton({
            name: 'buttonClose',
            sfx: 'sfx_button',
            imageName: 'gui2/bottom_tabs/bottom_tabs__back',
            frameCountX: 2,
            frameCountY: 1,
            onClick: function () {
                buttonClose.setActive(false);
                new Tween({
                    from: 1,
                    to: 0,
                    in: 5,
                    ease: 'linear',
                    onUpdate: function (v, t) {
                        entity.alpha = v;
                    },
                    onComplete: function () {
                        entity.removeSelf();
                        onClosed();
                    }
                });
            },
            components: [
                new BackButton({
                    removeSelf: true,
                    onPressed: function (data) {
                        data.entity.doCallback();
                    }
                }),
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.5, 1),
                        offset: new Vector2(-48 + (Anchor.isWideScreen() ? 4 : 0), -14 + (Anchor.isWideScreen() ? Anchor.getOffset().y : 0))
                    },
                    landscape: {
                        screenPosition: new Vector2(0.25, 1),
                        offset: new Vector2(-44, -12)
                    }
                })
            ]
        });
        var textJoin = new ShadowText({
            position: new Vector2(0, 0),
            text: Localization.getText('onlineJoin'),
            font: 'font',
            fontSize: 16,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            linebreaks: false,
            maxHeight: 16,
            maxWidth: 64
        });
        var textCreate = new ShadowText({
            position: new Vector2(12, 0),
            text: Localization.getText('onlineInvite'),
            font: 'font',
            fontSize: 16,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            linebreaks: false,
            maxHeight: 16,
            maxWidth: 48
        });
        var textRandom = new ShadowText({
            position: new Vector2(12, 0),
            text: Localization.getText('onlineRandom'),
            font: 'font',
            fontSize: 16,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            linebreaks: false,
            maxHeight: 16,
            maxWidth: 48
        });
        /**
         * Join button
         */
        var buttonJoin = new IconButton({
            name: 'buttonJoin',
            sfx: 'sfx_button',
            icon: textJoin,
            imageName: 'gui2/misc/button_large_secondary',
            frameCountX: 1,
            frameCountY: 3,
            position: new Vector2(0, -24 + 8),
            onClick: function () {
                var joinMenu = new OnlineJoin({
                    onFoundRoom: function (missionData, onSuccess) {
                        // disable modal
                        joinMenu.removeByName('modal');
                        // hide lobbywindow
                        joinMenu.hide();
                        // joinMenu has a blurbackground with a hidebelow component, remove joinMenu and later re attach it
                        Bento.objects.remove(joinMenu);

                        behavior.hide();
                        // roomCode was inserted and room was found, show mission/reward info before actually joining
                        showRewardsMenu({
                            missionData: missionData,
                            joinRoom: function () {
                                onSuccess();
                                // restore modal
                                joinMenu.attach(new Modal({}));
                                // show lobbywindow
                                joinMenu.show();
                                Bento.objects.attach(joinMenu);
                            },
                            cancel: function () {
                                joinMenu.disconnect();
                                joinMenu.removeSelf();
                                behavior.show();
                            }
                        });
                    },
                    onClose: function () {
                        behavior.show();
                    }
                });
                Bento.objects.attach(joinMenu);

                if (invitation) {
                    joinMenu.showLobbyWindow(invitation, true);
                }
            }
        });
        /**
         * Create button
         */
        var exemptFromEnergy = SaveManager.load('missionsExemptFromEnergy');
        var index = exemptFromEnergy.indexOf(mission);
        var costText = new Text({
            position: new Vector2(8, 6),
            text: index >= 0 ? '0' : cost.toString(),
            font: 'font',
            fontSize: 12,
            fontColor: '#00baff',
            align: 'left',
            textBaseline: 'middle',
            lineWidth: 1.5,
            strokeStyle: '#4f3673',
            pixelStroke: true,
        });
        var costIcon = new Entity({
            name: 'costIcon',
            position: new Vector2(-6 - 24, 0),
            components: [
                new Sprite({
                    imageName: 'gui2/misc/energy',
                    originRelative: new Vector2(0, 0.5)
                }),
                costText
            ]
        });
        var buttonInfo = new Entity({
            name: 'buttonInfo',
            components: [
                costIcon,
                textCreate
            ]
        });
        var buttonCreate = new IconButton({
            name: 'buttonCreate',
            sfx: 'sfx_button',
            icon: buttonInfo,
            imageName: 'gui2/misc/button_large_secondary',
            frameCountX: 1,
            frameCountY: 3,
            position: new Vector2(0, -24),
            onClick: function () {
                // check if enough energy first
                var exempt = exemptFromEnergy.indexOf(mission);
                var dialog;
                if (CharacterManager.hasEnergy(cost) || exempt >= 0) {} else {
                    // ask to watch ad
                    dialog = new EnergyForGemPopup({
                        z: Globals.WorldSelectLayers.LEVELDETAILS,
                        onBoughtEnergy: function (hasBought) {
                            if (hasBought) {
                                // try again 
                                buttonCreate.doCallback();
                            }
                        },
                        onWatchedAd: function () {
                            costText.setText('0');
                            exemptFromEnergy.push(settings.mission);
                            SaveManager.save('missionsExemptFromEnergy', exemptFromEnergy);

                            // try again
                            buttonCreate.doCallback();
                        }
                    });
                    Bento.objects.attach(dialog);
                    return;
                }
                // go to lobby (this will start the game later)
                var createMenu = new OnlineCreate({
                    enemyLevel: settings.enemyLevel,
                    cost: settings.cost,
                    mission: settings.mission,
                    unlocks: settings.unlocks,
                    rewards: settings.rewards,
                    chosenDifficulty: settings.chosenDifficulty
                });
                Bento.objects.attach(createMenu);
            }
        });
        var costText2 = new Text({
            position: new Vector2(8, 6),
            text: isJoining ? 2 : index >= 0 ? '0' : cost.toString(),
            font: 'font',
            fontSize: 12,
            fontColor: '#00baff',
            align: 'left',
            textBaseline: 'middle',
            lineWidth: 1.5,
            strokeStyle: '#4f3673',
            pixelStroke: true,
        });
        var costIcon2 = new Entity({
            name: 'costIcon',
            position: new Vector2(-6 - 24, 0),
            components: [
                new Sprite({
                    imageName: 'gui2/misc/energy',
                    originRelative: new Vector2(0, 0.5)
                }),
                costText2
            ]
        });
        var buttonInfo2 = new Entity({
            name: 'buttonInfo',
            components: [
                costIcon2,
                textRandom
            ]
        });
        var buttonRandom = new IconButton({
            name: 'buttonRandom',
            sfx: 'sfx_button',
            icon: buttonInfo2,
            imageName: 'gui2/misc/button_large_primary',
            frameCountX: 1,
            frameCountY: 3,
            position: new Vector2(0, 24 + 8),
            onClick: function () {
                // JOINING
                if (isJoining) {
                    var randomJoinMenu = new RandomJoin({});
                    Bento.objects.attach(randomJoinMenu);
                    return;
                }

                // CREATING
                // check if enough energy first
                var exempt = exemptFromEnergy.indexOf(mission);
                var dialog;
                if (CharacterManager.hasEnergy(cost) || exempt >= 0) {} else {
                    // ask to watch ad
                    dialog = new EnergyForGemPopup({
                        z: Globals.WorldSelectLayers.LEVELDETAILS,
                        onBoughtEnergy: function (hasBought) {
                            if (hasBought) {
                                // try again 
                                buttonCreate.doCallback();
                            }
                        },
                        onWatchedAd: function () {
                            costText.setText('0');
                            exemptFromEnergy.push(settings.mission);
                            SaveManager.save('missionsExemptFromEnergy', exemptFromEnergy);

                            // try again
                            buttonCreate.doCallback();
                        }
                    });
                    Bento.objects.attach(dialog);
                    return;
                }
                // go to lobby (this will start the game later)
                var roomName;
                if (window.encodeURI) {
                    // escape name to be sure
                    roomName = window.encodeURI(SaveManager.load('localPlayerName')).substring(0, 10);
                } else {
                    roomName = Math.abs(Utils.checksum(SaveManager.load('localPlayerName')));
                }
                var createMenu = new OnlineCreate({
                    roomCode: 'rnd-' + roomName + '-' + Utils.getRandom(1000000),
                    enemyLevel: settings.mission === 'tower' ?
                        StatsManager.getLevel(SaveManager.load('selectedCharacter')) : settings.enemyLevel,
                    cost: settings.cost,
                    mission: settings.mission,
                    unlocks: settings.unlocks,
                    rewards: settings.rewards,
                    chosenDifficulty: settings.chosenDifficulty
                });
                Bento.objects.attach(createMenu);
            }
        });
        /**
         * Rewards popup when joining
         */
        var showRewardsMenu = function (data) {
            var missionData = data.missionData;
            var joinRoom = data.joinRoom;
            var cancel = data.cancel;
            var rewardsPopup = new RewardsPopup({
                isOnline: true,
                mission: missionData.mission,
                cost: missionData.cost,
                unlocks: missionData.unlocks,
                enemyLevel: missionData.enemyLevel,
                rewards: missionData.rewards,
                invitation: invitation,
                chosenDifficulty: missionData.chosenDifficulty,
                onClick: function (chosenDifficulty) {
                    entity.remove(rewardsPopup);
                    joinRoom();
                },
                onClose: function () {
                    // cancel online?
                    entity.remove(rewardsPopup);
                    cancel();
                },
                components: [
                    new Anchor({
                        screenPosition: new Vector2(0.5, 0.5),
                        offset: new Vector2(0, 8),
                        onComplete: function (parent, isPortrait) {
                            if (!isPortrait) {
                                rewardsPopup.position.y += 8;
                            }
                        }
                    })
                ]
            });
            rewardsPopup.z = Globals.WorldSelectLayers.LEVELDETAILS + 1;
            rewardsPopup.float = true;
            entity.attach(rewardsPopup);

        };
        var headerBox = new HeaderBox({
            position: new Vector2(0, 0),
            size: new Rectangle(0, 0, 180, 160),
            title: Localization.getText('onlineMulti'),
            windowImage: 'gui2/misc/splash_background_9slice',
            components: [
                new Anchor({
                    screenPosition: new Vector2(0.5, 0.5),
                    onComplete: function () {

                    }
                }),
                isJoining ? buttonJoin : buttonCreate,
                buttonRandom
            ]
        });
        var behavior = {
            name: 'behavior',
            start: function () {
                new Tween({
                    from: viewport.width / 2 * 3,
                    to: viewport.width / 2,
                    in: 30,
                    ease: 'elastic',
                    decay: 8,
                    onUpdate: function (v, t) {
                        headerBox.position.x = v;
                    },
                    onComplete: function () {

                    }
                });

                if (isJoining) {
                    // remove energy info
                    buttonInfo2.remove(costIcon2);
                    textRandom.position.x = 0;
                }

                if (settings.invitation) {
                    entity.z = Globals.WorldSelectLayers.GUI;
                    buttonCreate.removeSelf();
                    buttonJoin.position.y = 0;
                    buttonJoin.doCallback();
                }
            },
            hide: function () {
                headerBox.removeSelf();
                buttonClose.removeSelf();
            },
            show: function () {
                if (!headerBox.parent) {
                    entity.attach(headerBox);
                }
                if (!buttonClose.parent) {
                    entity.attach(buttonClose);
                }
            }
        };
        var entity = new Entity({
            z: Globals.WorldSelectLayers.LEVELDETAILS,
            name: 'onlineMenu',
            position: new Vector2(0, 0),
            float: true,
            components: [
                new Modal({}),
                behavior,
                blurBackground,
                headerBox,
                buttonClose,
                new Topbar({})
            ]
        });
        return entity;
    };
});