/**
 * Menu for creating a room in multiplayer
 * @moduleName OnlineCreate
 */
bento.define('gui/menus/onlinecreate', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'modules/localization',
    'gui/headerbox',
    'components/modal',
    'gui/blurbackground',
    'globals',
    'components/anchor',
    'gui/loadingspinner',
    'gui/loadingdialog',
    'gui/dialog',
    'modules/matchmanager',
    'gui/shadowtext',
    'managers/save',
    'gui/lobbywindow',
    'gui/buttons/startbutton',
    'components/debug',
    'modules/snapshot',
    'appicon',
    'modules/analytics',
    'gui/iconbutton',
    'components/backbutton',
    'modules/ktplay'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Localization,
    HeaderBox,
    Modal,
    BlurBackground,
    Globals,
    Anchor,
    LoadingSpinner,
    LoadingDialog,
    Dialog,
    MatchManager,
    ShadowText,
    SaveManager,
    LobbyWindow,
    StartButton,
    Debug,
    SnapShot,
    AppIcon,
    Analytics,
    IconButton,
    BackButton,
    KTPlayModule
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var blurBackground = new BlurBackground({
            darken: 0.3
        });
        var currentRoomCode;
        var currentPlayerCount = 1;
        var lobbyWindow = new LobbyWindow({
            create: true,
            roomCode: settings.roomCode,
            cost: settings.cost,
            mission: settings.mission,
            unlocks: settings.unlocks,
            enemyLevel: settings.enemyLevel,
            rewards: settings.rewards,
            chosenDifficulty: settings.chosenDifficulty,

            onMatchStart: function () {
                lobbyWindow.removeSelf();

                // simulate start button
                var startButton = new StartButton({
                    enemyLevel: settings.enemyLevel,
                    cost: settings.cost,
                    mission: settings.mission,
                    unlocks: settings.unlocks,
                    rewards: settings.rewards,
                    chosenDifficulty: settings.chosenDifficulty
                });
                startButton.getComponent('buttonPlay').doCallback();

                Analytics.event('MultiPlayerCount', currentPlayerCount + 'p');
                // per mission
                Analytics.event('MultiPlayerCount', 'mission_' + settings.mission + '_' + currentPlayerCount + 'p');
            },
            onPlayerCountChanged: function (playerCount, maxPlayers) {
                // once playercount >= 1 then host can start the game
                // buttonPlay.setActive(playerCount > 1);
                currentPlayerCount = playerCount;
                // infoText.setText(Localization.getText(buttonPlay.isActive() ? 'ready' : 'needPlayers'));

                if (playerCount === maxPlayers) {
                    // start automatically in 3 seconds
                    behavior.startCountDown();
                } else {
                    // cancel countdown
                    behavior.stopCountDown();
                }
            },
            onRoomCodeChanged: function (roomCode) {
                roomCodeText.setText(roomCode);
                currentRoomCode = roomCode;
            }
        });
        var countDownText = new Text({
            text: '0',
            font: 'font',
            position: new Vector2(0, 48),
            fontSize: 16,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            maxWidth: 160,
            maxHeight: 16,
            visible: false,
            components: []
        });
        var shareText = new ShadowText({
            position: new Vector2(0, 4),
            text: Localization.getText('roomCode'),
            font: 'font',
            fontSize: 16,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            maxWidth: 88,
            maxHeight: 16,
            linebreaks: false
        });
        var roomCodeText = new ShadowText({
            position: new Vector2(0, 20),
            text: '1234',
            font: 'font',
            fontSize: 24,
            fontColor: '#22ff22',
            align: 'center',
            textBaseline: 'middle',
        });
        var buttonShare = new ClickButton({
            name: 'buttonShare',
            sfx: 'sfx_button',
            position: new Vector2(28, 54),
            imageName: 'gui/buttons/share',
            frameCountX: 1,
            frameCountY: 2,
            onClick: function () {
                var webLink = Utils.getStoreLink('web');
                var text = Localization.getText('shareRoomCode');
                var url;

                // add room code to weblink
                webLink = webLink + '#room=' + currentRoomCode;

                // replace link
                text = text.replace('{LINK}', '').replace('{link}', '');

                url = webLink;

                // currently sharing the app icon as image
                Utils.share(AppIcon, text, url);
            }
        });
        var buttonKTShare = new ClickButton({
            name: 'buttonKTShare',
            sfx: 'sfx_button',
            position: new Vector2(-28, 54),
            imageName: 'gui/buttons/KTshare',
            frameCountX: 1,
            frameCountY: 3,
            onClick: function () {
                //Go to Messages
                KTPlayModule.openDeeplink("KTRL.1506681109", function () {
                    console.log("Success, Player is sent to KT's the messages page.");
                });
            }
        });
        var infoText = new ShadowText({
            position: new Vector2(0, -32),
            text: Localization.getText('needPlayers'),
            font: 'font',
            fontSize: 16,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            maxWidth: viewport.width - 8,
            maxHeight: 24,
        });
        var buttonPlay = new IconButton({
            name: 'buttonPlay',
            sfx: 'sfx_button',
            iconName: 'gui2/misc/button_icon_play',
            iconAlpha: 0.5,
            imageName: 'gui2/misc/button_small_primary',
            frameCountX: 1,
            frameCountY: 3,
            animations: {
                'up': {
                    frames: [0]
                },
                'down': {
                    frames: [1]
                },
                'inactive': {
                    frames: [2]
                }
            },
            position: new Vector2(0, 0),
            onClick: function () {
                lobbyWindow.startMatch();
            }
        });
        // .attach(infoText);
        var buttonClose = new ClickButton({
            name: 'buttonClose',
            sfx: 'sfx_button',
            imageName: 'gui2/bottom_tabs/bottom_tabs__back',
            frameCountX: 2,
            frameCountY: 1,
            onClick: function () {
                // disconnect and remove
                MatchManager.disconnect();
                Bento.objects.remove(entity);
                if (settings.onClose) {
                    settings.onClose();
                }
            },
            components: [
                new BackButton({
                    removeSelf: true,
                    onPressed: function (data) {
                        data.entity.doCallback();
                    }
                }),
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.5, 1),
                        offset: new Vector2(-48 + (Anchor.isWideScreen() ? 4 : 0), -14 + (Anchor.isWideScreen() ? Anchor.getOffset().y : 0))
                    },
                    landscape: {
                        screenPosition: new Vector2(0.25, 1),
                        offset: new Vector2(-44, -12)
                    }
                })
            ]
        });
        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {
                // passed roomCode: it's a random match
                if (!settings.roomCode) {
                    lobbyWindow.getComponent('windowBox', function (box) {
                        box.attach(buttonShare);
                        box.attach(buttonKTShare);
                        box.attach(shareText);
                        box.attach(roomCodeText);
                        box.attach(countDownText);
                    });
                    KTPlayModule.getLoginStatus(function (isLoggedIn) {
                        buttonKTShare.setActive(isLoggedIn);
                    });
                    MatchManager.shouldDisconnect = false;
                } else {
                    lobbyWindow.getComponent('windowBox', function (box) {
                        var waitText = new Text({
                            text: Localization.getText('onlineWaitingRandom'),
                            font: 'font',
                            position: new Vector2(0, 48),
                            fontSize: 16,
                            fontColor: '#ffffff',
                            align: 'center',
                            textBaseline: 'middle',
                            maxWidth: 160,
                            maxHeight: 32,
                            components: []
                        });
                        box.attach(waitText);
                    });

                }
            },
            destroy: function (data) {
                MatchManager.shouldDisconnect = true;
            },
            update: function (data) {
                if (behavior.isCounting && behavior.count) {
                    if (behavior.count % 60 === 0) {
                        countDownText.setText(Localization.getText('startingIn').replace('{NUMBER}', Math.ceil(behavior.count / 60)));
                    }
                    behavior.count -= 1;

                    // set text
                    if (behavior.count === 0) {
                        // start
                        buttonPlay.doCallback();
                    }
                }
            },
            draw: function (data) {},
            isCounting: false,
            count: 60 * 3,
            startCountDown: function () {
                behavior.isCounting = true;
                buttonShare.setActive(false);
                buttonShare.visible = false;
                buttonKTShare.setActive(false);
                buttonKTShare.visible = false;
                shareText.visible = false;
                roomCodeText.visible = false;
                countDownText.visible = true;
            },
            stopCountDown: function () {
                behavior.isCounting = false;
                behavior.count = 60 * 3;
                buttonShare.setActive(true);
                buttonShare.visible = true;
                buttonKTShare.setActive(true);
                buttonKTShare.visible = true;
                shareText.visible = true;
                roomCodeText.visible = true;
                countDownText.visible = false;
            }
        };
        var entity = new Entity({
            z: Globals.WorldSelectLayers.LEVELDETAILS,
            name: 'onlineCreate',
            position: settings.position,
            float: true,
            components: [
                new Modal({}),
                blurBackground,
                lobbyWindow,
                buttonClose,
                // buttonPlay,
                behavior
            ]
        });

        buttonPlay.setActive(false);

        return entity;
    };
});