/**
 * Menu for joining a room in multiplayer
 * @moduleName OnlineJoin
 */
bento.define('gui/menus/onlinejoin', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'modules/localization',
    'gui/headerbox',
    'components/modal',
    'gui/blurbackground',
    'globals',
    'components/anchor',
    'gui/loadingspinner',
    'gui/loadingdialog',
    'gui/dialog',
    'modules/matchmanager',
    'gui/shadowtext',
    'managers/save',
    'gui/lobbywindow',
    'gui/buttons/startbutton',
    'managers/character',
    'components/fillrect',
    'gui/iconbutton',
    'components/backbutton'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Localization,
    HeaderBox,
    Modal,
    BlurBackground,
    Globals,
    Anchor,
    LoadingSpinner,
    LoadingDialog,
    Dialog,
    MatchManager,
    ShadowText,
    SaveManager,
    LobbyWindow,
    StartButton,
    CharacterManager,
    FillRect,
    IconButton,
    BackButton
) {
    'use strict';
    var NumberPad = function (settings) {
        var onClick = settings.onClick;
        var i;
        var spacing = 36;
        var output = '';
        var buttonOk = new ClickButton({
            name: 'buttonOk',
            sfx: 'sfx_button',
            imageName: 'gui/buttons/ok',
            frameCountX: 3,
            frameCountY: 1,
            animations: {
                'up': {
                    frames: [0]
                },
                'down': {
                    frames: [1]
                },
                'inactive': {
                    frames: [2]
                }
            },
            position: new Vector2(spacing + 4, spacing * 4),
            onClick: function () {
                if (onClick) {
                    onClick(output);
                }
            }
        });
        var titleText = new ShadowText({
            position: new Vector2(0, -spacing * 3),
            text: Localization.getText('enterRoomCode'),
            font: 'font',
            fontSize: 20,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            maxWidth: 160,
            maxHeight: 20,
            linebreaks: false
        });
        var textBack = new FillRect({
            color: [0, 0, 0, 0.5],
            originRelative: new Vector2(0.5, 0.5),
            size: new Rectangle(0, 0, spacing * 2, 24)
        });
        var outputText = new ShadowText({
            position: new Vector2(0, 0),
            text: '',
            font: 'font',
            fontSize: 20,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle'
        });
        var textContainer = new Entity({
            name: 'container',
            position: new Vector2(0, -spacing * 2),
            components: [
                textBack,
                outputText,
                buttonOk
            ]
        });
        var container = new Entity({
            z: settings.z,
            name: 'numberpad',
            position: settings.position,
            components: [
                titleText,
                textContainer
            ]
        });
        var makeButton = function (index, position, callback) {
            var text = new ShadowText({
                text: index.toString(),
                font: 'font',
                fontSize: 16,
                fontColor: '#ffffff',
                align: 'center',
                textBaseline: 'middle'
            });
            var button = new IconButton({
                name: 'button',
                sfx: 'sfx_button',
                icon: text,
                imageName: 'gui2/misc/button_smaller',
                frameCountX: 1,
                frameCountY: 3,
                position: position,
                float: false,
                updateWhenPaused: 0,
                onClick: callback || function () {
                    if (output.length >= 4) {
                        return;
                    }
                    output += index.toString();
                    outputText.setText(output);

                    if (output.length === 4) {
                        buttonOk.setActive(true);
                        buttonOk.getComponent('sprite', function (sprite) {
                            sprite.setAnimation('up');
                        });
                    }
                }
            });
            text.transform.x = 1;
            container.attach(button);
            return button;
        };

        // 9 to 1
        for (i = 0; i < 9; ++i) {
            makeButton(
                i + 1,
                new Vector2(
                    spacing * (i % 3) - spacing,
                    spacing * Math.floor(i / 3) - spacing
                )
            );
        }

        // 0
        makeButton(0, new Vector2(0, spacing * 2));

        // backspace
        makeButton('<', new Vector2(-spacing, spacing * 2), function () {
            if (output.length) {
                output = output.slice(0, output.length - 1);
            }
            outputText.setText(output);
            buttonOk.setActive(false);
        });

        buttonOk.setActive(false);

        return container;
    };

    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var blurBackground = new BlurBackground({});
        var lobbyWindow;
        var buttonClose = new ClickButton({
            name: 'buttonClose',
            sfx: 'sfx_button',
            imageName: 'gui2/bottom_tabs/bottom_tabs__back',
            frameCountX: 2,
            frameCountY: 1,
            onClick: function () {
                // disconnect and remove
                MatchManager.disconnect();
                Bento.objects.remove(entity);

                if (settings.onClose) {
                    settings.onClose();
                }
            },
            components: [
                new BackButton({
                    removeSelf: true,
                    onPressed: function (data) {
                        data.entity.doCallback();
                    }
                }),
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.5, 1),
                        offset: new Vector2(-48 + (Anchor.isWideScreen() ? 4 : 0), -14 + (Anchor.isWideScreen() ? Anchor.getOffset().y : 0))
                    },
                    landscape: {
                        screenPosition: new Vector2(0.25, 1),
                        offset: new Vector2(-44, -12)
                    }
                })
            ]
        });

        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {
                // var roomCode = window.prompt('Enter code');
            },
            destroy: function (data) {},
            update: function (data) {},
            draw: function (data) {}
        };
        var numberPad = new NumberPad({
            onClick: function (roomCode) {
                showLobbyWindow(roomCode);
            }
        }).attach(new Anchor({
            screenPosition: new Vector2(0.5, 0.5),
            offset: new Vector2(0, 0)
        })).attach({
            name: 'makeVisible',
            update: function () {
                if (lobbyWindow && lobbyWindow.parent) {
                    numberPad.visible = false;
                } else {
                    numberPad.visible = true;
                }
            }
        });
        var countDownBehavior = {
            name: 'countDownBehavior',
            count: 60 * 3,
            update: function () {
                if (countDownBehavior.count > 0) {
                    if (countDownBehavior.count % 60 === 0) {
                        countDownText.setText(Localization.getText('startingIn').replace('{NUMBER}', Math.ceil(countDownBehavior.count / 60)));
                    }
                    countDownBehavior.count -= 1;
                }
            }
        };
        var countDownText = new Text({
            text: '',
            font: 'font',
            position: new Vector2(0, 48),
            fontSize: 16,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            maxWidth: 160,
            maxHeight: 16,
            components: [countDownBehavior]
        });
        var entity = new Entity({
            z: Globals.WorldSelectLayers.LEVELDETAILS,
            name: 'onlineJoin',
            position: settings.position,
            float: true,
            components: [
                new Modal({}),
                blurBackground,
                buttonClose,
                behavior,
                numberPad
            ]
        });
        var hasJoinedRoom = false;
        var createInfo = function (mission, cost, rewards) {
            var anchor = new Anchor({
                screenPosition: new Vector2(0.5, 0.9),
                onComplete: function (parent, isPortrait) {
                    if (!isPortrait) {
                        // fix for landscape
                        infoContainer.position.x = viewport.width - 24;
                        infoContainer.position.y -= 4;
                        infoText.setText(infoText.getText(), {
                            maxWidth: 108,
                            maxHeight: 24
                        });
                        lobbyWindow.position.y = 4;
                        lobbyWindow.position.x = -40;
                        energyIcon.position.x = 8;
                        energyIcon.position.y = -16;
                        windowBox.setWidth(100);
                        windowBox.setHeight(80);
                        windowBox.position.y = -28;
                    } else {
                        lobbyWindow.position.y = 0;
                        lobbyWindow.position.x = 0;
                        infoText.setText(infoText.getText(), {
                            maxWidth: viewport.width - 8,
                            maxHeight: 24
                        });
                        energyIcon.position.x = -22;
                        energyIcon.position.y = 0;
                        windowBox.setWidth(160);
                        windowBox.setHeight(40);
                        windowBox.position.y = 0;
                    }
                }
            });
            var windowBox = new HeaderBox({
                title: Localization.getText('missionX').replace('{NUMBER}', mission),
                size: new Rectangle(0, 0, 160, 40)
            });
            // info about mission
            var infoText = new ShadowText({
                position: new Vector2(0, -32),
                text: '',
                font: 'font',
                fontSize: 16,
                fontColor: '#ffffff',
                align: 'center',
                textBaseline: 'middle',
                maxWidth: viewport.width - 8,
                maxHeight: 24
            });
            // energy icon
            var costText = new ShadowText({
                position: new Vector2(-4, -10),
                originRelative: new Vector2(0.5, 0.5),
                text: Localization.getText('needs').replace('{NUMBER}', ''),
                font: 'font',
                fontSize: 16,
                fontColor: '#ffffff',
                align: 'right',
                textBaseline: 'top',
                strokeStyle: '#000000',
            });
            var energyText = new ShadowText({
                position: new Vector2(12, 0),
                text: cost,
                font: 'font',
                fontSize: 16,
                fontColor: '#ffffff',
                align: 'center',
                textBaseline: 'middle'
            });
            var energyIcon = new Entity({
                name: 'energyIcon',
                position: new Vector2(-22, -2),
                components: [
                    new Sprite({
                        imageName: 'gui/startlevel/energyicon',
                        originRelative: new Vector2(0.5, 0.5)
                    }),
                    energyText,
                    costText
                ]
            });
            var rewardIcon;
            var infoContainer = new Entity({
                name: 'infoContainer',
                components: [
                    anchor,
                    windowBox,
                    // infoText
                ]
            });

            windowBox.attach(energyIcon);

            infoText.setText(Localization.getText('missionX').replace('{NUMBER}', mission));
            lobbyWindow.attach(infoContainer);

            return infoContainer;
        };
        var showLobbyWindow = function (roomCode, wasInvited) {
            var unlockedLevels = SaveManager.load('unlockedLevels');
            var mission;
            var cost;
            var rewards;
            var enemyLevel;
            var unlocks;
            var chosenDifficulty;
            lobbyWindow = new LobbyWindow({
                roomCode: roomCode,
                onRoomList: function (roomList, onSuccess) {
                    /*
                     * onSuccess should be called when user wants to join the room
                     */
                    var hasFoundRoom = false;
                    var dialog;
                    var onFoundRoom = function (room) {
                        mission = room.getCustomProperty('mission');
                        cost = room.getCustomProperty('cost');
                        enemyLevel = room.getCustomProperty('enemyLevel');
                        rewards = room.getCustomProperty('rewards');
                        chosenDifficulty = room.getCustomProperty('chosenDifficulty');
                        var missionData = {
                            enemyLevel: enemyLevel,
                            mission: mission,
                            cost: cost,
                            rewards: rewards,
                            chosenDifficulty: chosenDifficulty
                        };

                        /*
                         * callback to missiondetails now
                         */
                        if (settings.onFoundRoom) {
                            settings.onFoundRoom(missionData, function () {
                                onSuccess();
                                hasJoinedRoom = true;
                                lobbyWindow.getComponent('windowBox', function (box) {
                                    box.attach(countDownText);
                                });
                            });
                        }

                        // createInfo(mission, cost, rewards);

                        // the mission that is about to be played may not be unlocked by the user
                        // if thats the case, the user will not unlock the next levels
                        // exception: not during tutorials when user opens invitation
                        if (unlockedLevels.indexOf(mission) >= 0) {
                            unlocks = room.getCustomProperty('unlocks');
                            if (wasInvited && SaveManager.load('tutorialWorldSelect') === 0) {
                                unlocks = null;
                            }
                        }

                        /*if (!CharacterManager.hasEnergy(cost)) {
                            // TODO: add ad implementation here
                            dialog = new Dialog({
                                title: Localization.getText('errorTitle'),
                                text: "TODO: not enough energy, ask to watch ad.",
                                type: 'yes',
                                canClose: false
                            });
                            Bento.objects.attach(dialog);

                        } else {
                            // finish
                            onSuccess();
                        }*/
                    };

                    // find the room the player is trying to join
                    Utils.forEach(roomList, function (room, i, l, breakLoop) {
                        if (room.name === roomCode) {
                            onFoundRoom(room);
                            hasFoundRoom = true;
                            breakLoop();
                            return;
                        }
                    });

                    // warn if the room does not exist
                    if (!hasFoundRoom) {
                        dialog = new Dialog({
                            name: 'error_roomNotFound',
                            title: Localization.getText('errorTitle'),
                            text: Localization.getText('onlineRoomNotFound'),
                            type: 'yes',
                            canClose: false,
                            onComplete: function () {
                                // close all
                                buttonClose.doCallback();
                            }
                        });
                        Bento.objects.attach(dialog);
                    }
                },
                onRoomJoined: function (room) {},
                onMatchStart: function () {
                    lobbyWindow.removeSelf();

                    // simulate start button
                    var startButton = new StartButton({
                        enemyLevel: enemyLevel,
                        cost: cost,
                        unlocks: unlocks,
                        mission: mission,
                        rewards: rewards,
                        chosenDifficulty: chosenDifficulty,
                        isJoining: true
                    });
                    startButton.getComponent('buttonPlay').doCallback();
                }
            });
            entity.attach(lobbyWindow);
        };

        entity.extend({
            showLobbyWindow: showLobbyWindow,
            removeNumberPad: function () {
                entity.remove(numberPad);
            },
            disconnect: function () {
                MatchManager.disconnect();
            },
            hide: function () {
                entity.visible = false;
                buttonClose.setActive(false);
                // move out of the way
                if (numberPad.parent) {
                    entity.remove(numberPad);
                }
            },
            show: function () {
                entity.visible = true;
                buttonClose.setActive(true);
                if (!numberPad.parent) {
                    entity.attach(numberPad);
                }
            },
            hasJoinedRoom: function () {
                return hasJoinedRoom;
            }
        });

        return entity;
    };
});