/**
 * Pause menu
 * @moduleName PauseMenu
 */
bento.define('gui/menus/pause', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'components/fillrect',
    'globals',
    'gui/buttons/togglemusic',
    'gui/buttons/togglesound',
    'components/debug',
    'entities/triggers/exit',
    'components/anchor',
    'components/fillscreen',
    'modules/matchmanager',
    'managers/inventory',
    'managers/item',
    'gui/shadowtext',
    'bento/gui/togglebutton',
    'managers/resolution',
    'components/orientationlock',
    'gui/centeredlist',
    'gui/dialog',
    'modules/localization',
    'gui/iconbutton',
    'managers/save',
    'components/backbutton',
    'gui/watchrewardedad'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    FillRect,
    Globals,
    ToggleMusic,
    ToggleSound,
    Debug,
    Exit,
    Anchor,
    FillScreen,
    MatchManager,
    InventoryManager,
    ItemManager,
    ShadowText,
    ToggleButton,
    ResolutionManager,
    OrientationLock,
    CenteredList,
    Dialog,
    Localization,
    IconButton,
    SaveManager,
    BackButton,
    WatchRewardedAd
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var overlay = new FillScreen({
            color: [0, 0, 0, 0.6]
        });
        var behavior = {
            name: 'behaviorComponent',
            oldPauseLevel: Bento.objects.isPaused(),
            start: function (data) {
                // cache the old pause level
                if (!MatchManager.isOnline) {
                    this.oldPauseLevel = Bento.objects.isPaused();
                    Bento.objects.pause(Globals.PauseLevels.PAUSE);
                } else {
                    EventSystem.fire('disableControls');
                }
                makeItemInfo();
            },
            destroy: function (data) {
                if (!MatchManager.isOnline) {
                    Bento.objects.pause(this.oldPauseLevel);
                } else {
                    EventSystem.fire('enableControls');
                }
            }
        };
        var buttonPlay = new IconButton({
            name: 'buttonPlay',
            sfx: 'sfx_button',
            imageName: 'gui2/misc/button_large_primary',
            iconName: 'gui2/misc/button_icon_play',
            frameCountX: 1,
            frameCountY: 3,
            onClick: function () {
                Bento.objects.remove(entity);
            },
            components: [
                new BackButton({
                    removeSelf: true,
                    onPressed: function (data) {
                        data.entity.doCallback();
                    }
                })
            ]
        });
        buttonPlay.attach(new Anchor({
            portrait: {
                screenPosition: new Vector2(0.5, 0.5),
                offset: new Vector2(0, 0)
            },
            landscape: {
                screenPosition: new Vector2(0.5, 0.5),
                offset: new Vector2(0, 0)
            }
        }));
        var centeredList = new CenteredList({
            spacing: 4
        }).attach(new Anchor({
            screenPosition: new Vector2(0.5, 1),
            offset: new Vector2(0, -24)
        }));

        var buttonHome = new ClickButton({
            name: 'buttonHome',
            sfx: 'sfx_button',
            imageName: 'gui/buttons/home',
            frameCountX: 1,
            frameCountY: 2,
            onClick: function () {
                // playing online: prevent modal so just go to worldselect immediately
                var isTower = SaveManager.load('isTower');
                var isEvent = SaveManager.load('isEvent');
                var currentMission = SaveManager.load('currentMission');
                var target = null;

                if (isTower) {
                    target = 'tower';
                }
                if (isEvent) {
                    target = 'beach';
                }
                if (currentMission === 'dojo') {
                    target = 'worldmap';
                }

                if (SaveManager.load('isMissionOnline') || isTower) {
                    WatchRewardedAd.showInterstitial(function () {
                        if (isTower) {
                            EventSystem.fire('endTower');
                        }
                        Bento.screens.show('screens/worldselect', {
                            target: target
                        });
                    });
                    return;
                }

                var goHome = false;
                var dialog = new Dialog({
                    name: 'returnHome',
                    type: 'yesno',
                    canClose: true,
                    text: Localization.getText('home'),
                    title: Localization.getText('warningTitle'),
                    onYes: function () {
                        goHome = true;
                    },
                    onComplete: function () {
                        if (goHome) {
                            Bento.screens.show('screens/worldselect', {
                                target: target
                            });
                        }
                    }
                });
                Bento.objects.attach(dialog);
            }
        });
        var toggleSound = new ToggleSound({});
        var toggleMusic = new ToggleMusic({});
        var toggleOrientation = new ToggleButton({
            name: 'toggleOrientation',
            sfx: 'sfx_button',
            imageName: 'gui/buttons/orientation',
            frameCountX: 1,
            frameCountY: 2,
            toggled: Bento.saveState.load('orientationLock', false),
            onToggle: function () {
                ResolutionManager.setUserLock(toggleOrientation.isToggled());
            }
        });
        var toggleLeftHanded = new ToggleButton({
            name: 'toggleLeftHanded',
            sfx: 'sfx_button',
            imageName: 'gui/buttons/lefthanded',
            frameCountX: 1,
            frameCountY: 2,
            toggled: SaveManager.load('leftHanded'),
            onToggle: function () {
                SaveManager.save('leftHanded', toggleLeftHanded.isToggled());
                EventSystem.fire('leftHanded');
            }
        });

        var itemIcons = [];
        var makeItemInfo = function () {
            var top = 20;
            var items = InventoryManager.getEquippedConsumables();
            var noItems = true;
            var addIcon = function (item) {
                var slotSprite = new Sprite({
                    imageName: 'gui/itemslot-small',
                    frameCountX: 2,
                    frameCountY: 1,
                    originRelative: new Vector2(0.5, 0.5)
                });
                var sprite = ItemManager.getSprite(item);
                sprite.setOriginRelative(new Vector2(0.5, 0.5));
                sprite.origin.x += 1;
                var description = new ShadowText({
                    position: new Vector2(18, -2),
                    text: ItemManager.getDescription(item),
                    font: 'font',
                    fontSize: 10,
                    fontColor: '#ffffff',
                    align: 'left',
                    textBaseline: 'middle',
                    maxWidth: 120,
                    maxHeight: 28,
                    // drawDebug: true
                });
                var index = itemIcons.length;
                var icon = new Entity({
                    z: 3,
                    name: 'icon',
                    position: new Vector2(20, top + 32 * index),
                    components: [
                        slotSprite,
                        sprite,
                        description
                    ]
                });

                if (index > 0) {
                    icon.attach(new Anchor({
                        onComplete: function (parent, isPortrait) {
                            if (isPortrait) {
                                icon.position = new Vector2(20, top + 32 * index);
                            } else {
                                icon.position = new Vector2(
                                    20 + (index % 2) * 158,
                                    top + 32 * Math.floor(index / 2)
                                );
                            }
                        }
                    }));
                }

                itemIcons.push(icon);
                entity.attach(icon);
            };

            if (items[0] && items[0].amount > 0) {
                addIcon(items[0].key);
                noItems = false;
            }
            if (items[1] && items[1].amount > 0) {
                addIcon(items[1].key);
                noItems = false;
            }

            if (noItems) {
                makeTip();
            }
        };
        var makeTip = function () {
            var tips = Localization.getText('tips');
            var pickRandomTip = function () {
                var random = Utils.getRandom(tips.length);
                // check if it's a tip that requires a scroll --> reroll if user doesnt have it
                if (random === 0 && !InventoryManager.getItem('scrolls/dash')) {
                    return pickRandomTip();
                } else if (random === 1 && !InventoryManager.getItem('scrolls/charge')) {
                    return pickRandomTip();
                }

                return random;
            };
            var tip = new ShadowText({
                z: 3,
                text: Localization.getText('tip') + tips[pickRandomTip()],
                font: 'font',
                fontSize: 12,
                fontColor: '#ffffff',
                align: 'center',
                textBaseline: 'top',
                maxWidth: viewport.width - 16
            }).attach(new Anchor({
                screenPosition: new Vector2(0.5, 0),
                offset: new Vector2(0, 8),
                onComplete: function (parent, isPortrait) {
                    // reset maxWidth
                    tip.setText(tip.getText(), {
                        maxWidth: viewport.width - 16
                    });
                }
            }));
            entity.attach(tip);
        };
        var entity = new Entity({
            z: Globals.Layers.PAUSE,
            name: 'pauseMenu',
            position: new Vector2(0, 0),
            updateWhenPaused: Globals.PauseLevels.PAUSE,
            float: true,
            components: [
                overlay,
                behavior,
                buttonPlay,
                centeredList,
                new OrientationLock({})
            ]
        });

        centeredList.addItem(buttonHome);
        centeredList.addItem(toggleSound);
        centeredList.addItem(toggleMusic);
        centeredList.addItem(toggleLeftHanded);
        centeredList.addItem(toggleOrientation);

        centeredList.scale.x = 0.8;
        centeredList.scale.y = 0.8;

        return entity;
    };
});