/**
 * Menu for randomly joining a room
 * @moduleName RandomJoin
 * @snippet RandomJoin.snippet
RandomJoin({})
 */
bento.define('gui/menus/randomjoin', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/tween',
    'gui/headerbox',
    'globals',
    'components/anchor',
    'components/modal',
    'gui/blurbackground',
    'gui/worldselect/topbar',
    'components/backbutton',
    'gui/iconbutton',
    'gui/shadowtext',
    'modules/localization',
    'managers/save',
    'bento/gui/togglebutton',
    'bento/components/nineslice',
    'gui/equipmentslots',
    'managers/character',
    'gui/energyforgempopup',
    'gui/lobbywindow',
    'gui/dialog',
    'gui/buttons/startbutton',
    'modules/matchmanager',
    'gui/dailyquest/progressbar'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    HeaderBox,
    Globals,
    Anchor,
    Modal,
    BlurBackground,
    Topbar,
    BackButton,
    IconButton,
    ShadowText,
    Localization,
    SaveManager,
    ToggleButton,
    NineSlice,
    EquipmentSlots,
    CharacterManager,
    EnergyForGemPopup,
    LobbyWindow,
    Dialog,
    StartButton,
    MatchManager,
    ProgressBar
) {
    'use strict';
    return function (settings) {
        var viewport = Bento.getViewport();
        var position = settings.position;
        var blurBackground = new BlurBackground({
            darken: 0.3,
            blurriness: 15
        });
        var topBar = new Topbar({});

        var openLobbyWindow = function () {
            var countDownBehavior = {
                name: 'countDownBehavior',
                count: 60 * 3,
                update: function () {
                    if (countDownBehavior.count > 0) {
                        if (countDownBehavior.count % 60 === 0) {
                            countDownText.setText(Localization.getText('startingIn').replace('{NUMBER}', Math.ceil(countDownBehavior.count / 60)));
                        }
                        countDownBehavior.count -= 1;
                    }
                }
            };
            var countDownText = new Text({
                text: '',
                font: 'font',
                position: new Vector2(0, 48),
                fontSize: 16,
                fontColor: '#ffffff',
                align: 'center',
                textBaseline: 'middle',
                maxWidth: 160,
                maxHeight: 16,
                components: [countDownBehavior]
            });
            var unlockedLevels = SaveManager.load('unlockedLevels');
            var mission;
            var cost;
            var rewards;
            var enemyLevel;
            var unlocks;
            var chosenDifficulty;
            var lobbyWindow = new LobbyWindow({
                z: entity.z + 1,
                onRoomList: function (roomList, onSuccess) {
                    /*
                     * onSuccess should be called when user wants to join the room
                     */
                    var hasFoundRoom = false;
                    var dialog;
                    var onFoundRoom = function (room) {
                        mission = room.getCustomProperty('mission');
                        cost = room.getCustomProperty('cost');
                        enemyLevel = room.getCustomProperty('enemyLevel');
                        rewards = room.getCustomProperty('rewards');
                        chosenDifficulty = room.getCustomProperty('chosenDifficulty');
                        var missionData = {
                            enemyLevel: enemyLevel,
                            mission: mission,
                            cost: cost,
                            rewards: rewards,
                            chosenDifficulty: chosenDifficulty
                        };

                        lobbyWindow.getComponent('windowBox', function (box) {
                            box.attach(countDownText);
                        });

                        // the mission that is about to be played may not be unlocked by the user
                        // if thats the case, the user will not unlock the next levels
                        if (unlockedLevels.indexOf(mission) >= 0) {
                            unlocks = room.getCustomProperty('unlocks');
                        }

                        onSuccess();
                        lobbyWindow.visible = true;
                    };

                    // inspect all room codes
                    // find a room with enemy level that is closest to own level
                    // consideration: perhaps also a higher weight to older rooms
                    var playerLevel = CharacterManager.getLevel();
                    var diff;
                    var smallestDiff = Infinity;
                    var currentRoom;
                    Utils.forEach(roomList, function (room, i, l, breakLoop) {
                        // the room code must start with 'rnd-'
                        if (room.name.indexOf('rnd-') === 0 && room.playerCount === 1) {
                            enemyLevel = room.getCustomProperty('enemyLevel');
                            diff = Math.abs(playerLevel - enemyLevel);
                            if (diff < smallestDiff) {
                                smallestDiff = diff;
                                currentRoom = room;
                                hasFoundRoom = true;
                            }
                        }
                    });

                    // warn if the room does not exist
                    if (!hasFoundRoom) {
                        dialog = new Dialog({
                            name: 'error_roomNotFound',
                            title: Localization.getText('errorTitle'),
                            text: Localization.getText('onlineRandomEmpty'),
                            type: 'yes',
                            canClose: false,
                            onComplete: function () {
                                MatchManager.disconnect();
                                lobbyWindowPopup.removeSelf();
                            }
                        });
                        Bento.objects.attach(dialog);
                    } else {
                        lobbyWindow.setRoomCode(currentRoom.name);
                        onFoundRoom(currentRoom);
                    }
                },
                onRoomJoined: function (room) {
                    lobbyWindow.visible = true;
                },
                onMatchStart: function () {
                    // lobbyWindow.removeSelf();

                    // simulate start button
                    var startButton = new StartButton({
                        enemyLevel: enemyLevel,
                        cost: cost,
                        unlocks: unlocks,
                        mission: mission,
                        rewards: rewards,
                        chosenDifficulty: chosenDifficulty,
                        isJoining: true,
                        wantsReputation: true
                    });
                    startButton.getComponent('buttonPlay').doCallback();
                }
            });
            var lobbyWindowPopup = new Entity({
                z: entity.z + 1,
                name: 'lobbyWindowPopup',
                float: true,
                components: [
                    new Modal({}),
                    new BlurBackground({
                        darken: 0.3,
                        blurriness: 15
                    }),
                    lobbyWindow
                ]
            });

            Bento.objects.attach(lobbyWindowPopup);
        };
        /**
         * Close
         */
        var buttonClose = new IconButton({
            name: 'buttonClose',
            sfx: 'sfx_button',
            imageName: 'gui2/misc/button_smaller',
            iconName: 'gui2/misc/button_icon_back_small',
            frameCountX: 1,
            frameCountY: 3,
            onClick: function () {
                entity.removeSelf();
                if (settings.onComplete) {
                    settings.onComplete();
                }
            },
            components: [
                new BackButton({
                    removeSelf: true,
                    onPressed: function (data) {
                        data.entity.doCallback();
                    }
                }),
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0, 0.5),
                        offset: new Vector2(-64, -60 + 16)
                    },
                    landscape: {
                        screenPosition: new Vector2(-0.5, -0.5),
                        offset: new Vector2(32, 12)
                    }
                })
            ]
        });
        /**
         * Title
         */
        var titleText = new Text({
            text: Localization.getText('onlineJoin'),
            font: 'font',
            fontSize: 22,
            fontColor: '#fffd00',
            align: 'center',
            textBaseline: 'middle',
            components: [
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0, 0),
                        offset: new Vector2(0, -12)
                    },
                    landscape: {
                        screenPosition: new Vector2(0, 0),
                        offset: new Vector2(0, -16)
                    }
                })
            ]
        });
        var titleBar = new Entity({
            name: 'titleBar',
            position: new Vector2(0, 0),
            components: [
                new Sprite({
                    imageName: 'gui2/misc/title_bar',
                    originRelative: new Vector2(0.5, 0.5)
                }),
                titleText,
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0, 0),
                        offset: new Vector2(0, -112)
                    },
                    landscape: {
                        screenPosition: new Vector2(0, 0),
                        offset: new Vector2(0, -62)
                    }
                })
            ]
        });
        /**
         * Consumables (note: copied from rewardspopup)
         */
        var autoText = new ShadowText({
            position: new Vector2(8, -2),
            text: Localization.getText('autoSelect'),
            font: 'font',
            fontSize: 12,
            fontColor: '#ffffff',
            align: 'left',
            textBaseline: 'middle',
            maxWidth: 32,
            maxHeight: 16,
            linebreaks: false
        });
        var autoSelect = new ToggleButton({
            name: 'autoSelect',
            position: new Vector2(0, 0),
            boundingBox: new Rectangle(-16, -10, 48, 20),
            imageName: 'gui/buttons/checkbox',
            frameCountX: 3,
            frameCountY: 1,
            toggled: SaveManager.load('autoSelect'),
            animations: {
                up: {
                    frames: [0]
                },
                down: {
                    frames: [1]
                },
                inactive: {
                    frames: [2]
                }
            },
            onToggle: function () {
                SaveManager.save('autoSelect', autoSelect.isToggled());
            }
        }).attach(autoText);
        var consumablesHeader = new Entity({
            name: 'header',
            components: [
                new Sprite({
                    imageName: 'gui2/misc/borderheader',
                    originRelative: new Vector2(0.5, 1)
                })
            ]
        });
        var consumablesBackground = new NineSlice({
            imageName: 'gui2/misc/border_9slice',
            originRelative: new Vector2(0.5, 0.5),
            width: 148,
            height: 54
        });
        var consumablesTitle = new Text({
            position: new Vector2(0, -consumablesBackground._height / 2 - 8),
            text: Localization.getText('takeBoosts'),
            font: 'font',
            fontSize: 12,
            fontColor: '#b2b2b2',
            align: 'center',
            textBaseline: 'middle',
            maxWidth: 56,
            linebreaks: false
        });
        var consumables = new EquipmentSlots({
            iconImage: 'gui/startlevel/inventory',
            type: 'consumable',
            slotSize: 'new',
            // hideSlot2: true,
            position: new Vector2(0, 4),
        });
        var consumablesContainer = new Entity({
            name: 'consumablesContainer',
            position: new Vector2(0, 64),
            components: [
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0, 0),
                        offset: new Vector2(0, 56),
                    },
                    landscape: {
                        screenPosition: new Vector2(0, 0),
                        offset: new Vector2(76, -13)
                    },
                    onComplete: function () {
                        if (Anchor.isPortrait()) {
                            // consumables
                            consumablesBackground.setWidth(148);
                            consumablesBackground.setHeight(48);
                            consumablesTitle.position.y = -consumablesBackground._height / 2 - 8;
                            consumablesHeader.position.y = -consumablesBackground._height / 2;
                            consumables.position = new Vector2(8, 2);
                            consumables.getComponent('icon', function (icon) {
                                icon.position = new Vector2(-56, -6);
                            });
                            autoSelect.position = new Vector2(-64, 16);
                        } else {
                            // consumables
                            consumablesBackground.setWidth(92);
                            consumablesBackground.setHeight(72);
                            consumablesTitle.position.y = -consumablesBackground._height / 2 - 8;
                            consumablesHeader.position.y = -consumablesBackground._height / 2;
                            consumables.position = new Vector2(-16, 16);
                            consumables.getComponent('icon', function (icon) {
                                icon.position = new Vector2(-4, -32);
                            });
                            autoSelect.position = new Vector2(0, -12);
                        }
                    }
                }),
                consumablesHeader,
                consumablesBackground,
                consumablesTitle,
                consumables,
                autoSelect
            ]
        });
        /**
         * Play button (note: copy from rewardspopup.js)
         */
        var energyCost = 2;
        var costText = new Text({
            position: new Vector2(8, 6),
            text: energyCost.toString(),
            font: 'font',
            fontSize: 12,
            fontColor: '#00baff',
            align: 'left',
            textBaseline: 'middle',
            lineWidth: 1.5,
            strokeStyle: '#4f3673',
            pixelStroke: true,
        });
        var costIcon = new Entity({
            name: 'costIcon',
            position: new Vector2(-6 - 24, 0),
            components: [
                new Sprite({
                    imageName: 'gui2/misc/energy',
                    originRelative: new Vector2(0, 0.5)
                }),
                costText
            ]
        });
        var playText = new ShadowText({
            position: new Vector2(8, 0),
            originRelative: new Vector2(0, 0.5),
            text: Localization.getText('play'),
            font: 'font',
            fontSize: 22,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            strokeStyle: '#000000',
            maxWidth: 44,
            maxHeight: 22
        });
        var container = new Entity({
            name: 'icon',
            components: [
                costIcon,
                playText
            ]
        });
        var buttonPlay = new IconButton({
            name: 'startButton',
            sfx: 'sfx_button',
            icon: container,
            iconAlpha: 0.5,
            imageName: 'gui2/misc/button_medium_primary',
            frameCountX: 1,
            frameCountY: 3,
            onClick: function () {
                var dialog;
                if (CharacterManager.hasEnergy(energyCost)) {
                    openLobbyWindow();
                } else {
                    // ask to watch ad
                    dialog = new EnergyForGemPopup({
                        z: Globals.WorldSelectLayers.LEVELDETAILS,
                        onBoughtEnergy: function (hasBought) {
                            if (hasBought) {
                                openLobbyWindow();
                            }
                        },
                        onWatchedAd: function () {
                            energyCost = 0;
                            costText.setText('0');
                            openLobbyWindow();
                        }
                    });
                    Bento.objects.attach(dialog);
                    return;
                }
            },
            components: [
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0, 0.5),
                        offset: new Vector2(0, -44),
                    },
                    landscape: {
                        screenPosition: new Vector2(0, 0.5),
                        offset: new Vector2(76, -44),
                    },
                    onComplete: function () {

                    }
                })
            ]
        });
        /**
         * Explanation text
         */
        var explanationText = new ShadowText({
            text: Localization.getText('onlineExplanation'),
            font: 'font',
            fontSize: 16,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'bottom',
            maxWidth: 150,
            maxHeight: 32,
            components: []
        });
        var reputationText = new Text({
            position: new Vector2(0, 16),
            text: Localization.getText('onlineReputation'),
            font: 'font',
            fontSize: 12,
            fontColor: '#ffffff',
            align: 'right',
            textBaseline: 'top',
            maxWidth: 75,
            maxHeight: 12,
            components: []
        });
        var ranks = Localization.getText('ranks');
        var rankIndex = Math.floor(SaveManager.load('reputationPoints') / Globals.pointsPerRank);
        var rankText = new Text({
            position: new Vector2(2, 16),
            text: ranks[Utils.clamp(0, rankIndex, ranks.length - 1)],
            font: 'font',
            fontSize: 12,
            fontColor: '#fffd00',
            align: 'left',
            textBaseline: 'top',
            maxWidth: 75,
            maxHeight: 12,
            components: []
        });
        var progress = new ProgressBar({
            position: new Vector2(0, 40),
            goal: Globals.pointsPerRank
        }).attach({
            start: function () {
                progress.setProgress({
                    count: SaveManager.load('reputationPoints') % Globals.pointsPerRank,
                    goal: Globals.pointsPerRank
                });
            }
        });
        var reputation = new Entity({
            name: 'container',
            components: [
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0, 0),
                        offset: new Vector2(0, -48),
                    },
                    landscape: {
                        screenPosition: new Vector2(0, 0),
                        offset: new Vector2(-64, -16),
                    }
                }),
                explanationText,
                reputationText,
                rankText,
                progress
            ]
        });
        /**
         * Container for all buttons
         */
        var headerBox = new HeaderBox({
            windowImage: 'gui2/misc/splash_background_9slice',
            size: new Rectangle(0, 0, 180, 212),
            originRelative: new Vector2(0.5, 0.5),
            components: [
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.5, 0.5),
                        offset: new Vector2(0, 8)
                    },
                    landscape: {
                        screenPosition: new Vector2(0.5, 0.5),
                        offset: new Vector2(0, 16)
                    },
                    onComplete: function () {
                        if (Anchor.isPortrait()) {
                            headerBox.setWidth(160 + 24);
                            headerBox.setHeight(192 + 48);
                        } else {
                            headerBox.setWidth(200 + 112);
                            headerBox.setHeight(140 + 12);
                        }
                    }
                }),
                reputation,
                consumablesContainer,
                buttonClose,
                buttonPlay
            ]
        });
        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {
                // tween in
                new Tween({
                    from: 0,
                    to: 1,
                    in: 20,
                    ease: 'elastic',
                    decay: 7,
                    oscillations: 1,
                    onUpdate: function (v, t) {
                        headerBox.scale.x = v;
                        headerBox.scale.y = v;
                    }
                });
                headerBox.attach(titleBar);
                headerBox.moveComponentTo(titleBar, 0);
            },
            destroy: function (data) {},
            update: function (data) {},
            draw: function (data) {}
        };
        /**
         * Main entity
         */
        var entity = new Entity({
            z: Globals.WorldSelectLayers.LEVELDETAILS,
            name: 'randomJoin',
            float: true,
            components: [
                new Modal({}),
                blurBackground,
                topBar,
                headerBox,
                behavior
            ]
        });
        return entity;
    };
});