/**
 * Popup that shows the content of a collectible scroll
 * @moduleName ScrollMenu
 */
bento.define('gui/menus/scroll', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'bento/components/nineslice',
    'globals',
    'bento/maskedcontainer',
    'gui/blurbackground',
    'components/anchor',
    'modules/localization',
    'gui/scrollinglist',
    'entities/effect',
    'entities/character',
    'components/modal',
    'gui/shadowtext',
    'gui/worldselect/dojo',
    'components/orientationlock',
    'managers/save',
    'entities/tutorial/gesture',
    'gui/iconbutton'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    NineSlice,
    Globals,
    MaskedContainer,
    BlurBackground,
    Anchor,
    Localization,
    ScrollingList,
    Effect,
    Character,
    Modal,
    ShadowText,
    Dojo,
    OrientationLock,
    SaveManager,
    Gesture,
    IconButton
) {
    'use strict';
    /*
     * Individual scroll contents
     */
    var scrolls = {
        'scrolls/dash': function () {
            var title = new Text({
                position: new Vector2(0, 0),
                text: Localization.getText('scrollsText')['dash']['title'],
                font: 'font',
                fontSize: 16,
                fontColor: '#733d51',
                align: 'center',
                textBaseline: 'middle',
                maxWidth: 160 - 8,
                maxHeight: 16,
                linebreaks: false
            });
            var text = new Text({
                position: new Vector2(0, 0),
                text: Localization.getText('scrollsText')['dash']['text'],
                font: 'font',
                fontSize: 16,
                fontColor: '#c29b5d',
                align: 'center',
                textBaseline: 'middle',
                maxWidth: 160 - 8,
                maxHeight: 32
            });
            var sprite = new Sprite({
                spriteSheet: 'boy/idle'
            });
            var dashExample = new Entity({
                name: 'dashExample',
                position: new Vector2(-32, 16),
                components: [
                    sprite, {
                        name: 'dashExampleBehavior',
                        start: function () {
                            var setIdle = function () {
                                sprite.setSpriteSheet('boy/idle');
                                gestureSprite.setSpriteSheet('gui/hand-gesture-flick');
                                Utils.timeout(20, setRun);
                                dashExample.position.x = -40;
                            };
                            var setRun = function () {
                                sprite.setSpriteSheet('boy/rundash');
                                new Tween({
                                    from: dashExample.position.x,
                                    to: dashExample.position.x + 56,
                                    in: 30,
                                    ease: 'linear',
                                    onUpdate: function (v, t) {
                                        dashExample.position.x = v;
                                    },
                                    onComplete: setAttack
                                });
                            };
                            var setAttack = function () {
                                new Tween({
                                    from: dashExample.position.x,
                                    to: dashExample.position.x + 8,
                                    in: 30,
                                    ease: 'exponential',
                                    decay: 2,
                                    onUpdate: function (v, t) {
                                        dashExample.position.x = v;
                                    }
                                });
                                gestureSprite.setSpriteSheet('gui/hand-gesture-tap');
                                sprite.setSpriteSheet('boy/chargeuppunch', function () {
                                    sprite.setSpriteSheet('boy/idle');
                                    Utils.timeout(30, setIdle);

                                });
                            };
                            setIdle();
                        }
                    }
                ]
            });
            var gestureSprite = new Sprite({
                spriteSheet: 'gui/hand-gesture-flickntap'
            });
            var gesture = new Entity({
                name: 'gesture',
                position: new Vector2(0, 32),
                scale: new Vector2(0.75, 0.75),
                components: [
                    gestureSprite
                ]
            });
            var gestureContainer = new Entity({
                name: 'gestureContainer',
                position: new Vector2(0, 0),
                components: [
                    gesture
                ]
            });
            var playerContainer = new Entity({
                name: 'player',
                family: [''],
                position: new Vector2(0, 0),
                components: [
                    dashExample
                ]
            });
            var contents = new ScrollingList({
                name: 'contents',
                direction: 'y',
                spacing: 8,
                position: new Vector2(0, -64)
            });
            contents.addItem(title);
            contents.addItem(text);
            contents.addItem(playerContainer);
            contents.addItem(gestureContainer);
            contents.addItem(new Entity({}));

            // initially trying to make the contents scrollable
            // but something seems wrong with the MaskedContainer
            contents.setActive(false);

            return contents;
        },
        'scrolls/jump': function () {
            var title = new Text({
                position: new Vector2(0, 0),
                text: Localization.getText('scrollsText')['jump']['title'],
                font: 'font',
                fontSize: 16,
                fontColor: '#733d51',
                align: 'center',
                textBaseline: 'middle',
                maxWidth: 160 - 8,
                maxHeight: 16,
                linebreaks: false
            });
            var text = new Text({
                position: new Vector2(0, 0),
                text: Localization.getText('scrollsText')['jump']['text'],
                font: 'font',
                fontSize: 16,
                fontColor: '#c29b5d',
                align: 'center',
                textBaseline: 'middle',
                maxWidth: 160 - 8,
                maxHeight: 32
            });
            var sprite = new Sprite({
                spriteSheet: 'boy/idle'
            });
            var jumpExample = new Entity({
                name: 'jumpExample',
                position: new Vector2(-32, 16),
                components: [
                    sprite, {
                        name: 'jumpExampleBehavior',
                        start: function () {
                            var setIdle = function () {
                                sprite.setSpriteSheet('boy/idle');
                                gestureSprite.setSpriteSheet('gui/hand-gesture-flickup', function () {
                                    gestureSprite.setSpriteSheet('gui/hand-gesture-flickup-done');
                                });
                                Utils.timeout(20, setWindup);
                                jumpExample.position.x = -40;
                            };
                            var setWindup = function () {
                                sprite.setSpriteSheet('boy/jumpattack-windup', function () {
                                    sprite.setSpriteSheet('boy/jumpattack-loop');
                                });
                                new Tween({
                                    from: jumpExample.position.x,
                                    to: jumpExample.position.x + 70,
                                    in: 32,
                                    delay: 6,
                                    ease: 'linear',
                                    onUpdate: function (v, t) {
                                        jumpExample.position.x = v;
                                    },
                                    onComplete: setLand
                                });

                                new Tween({
                                    from: jumpExample.position.y,
                                    to: jumpExample.position.y - 20,
                                    in: 16,
                                    delay: 6,
                                    ease: 'easeOutCubic',
                                    onUpdate: function (v, t) {
                                        jumpExample.position.y = v;
                                    },
                                    onComplete: function () {
                                        new Tween({
                                            from: jumpExample.position.y,
                                            to: jumpExample.position.y + 20,
                                            in: 16,
                                            delay: 0,
                                            ease: 'easeInCubic',
                                            onUpdate: function (v, t) {
                                                jumpExample.position.y = v;
                                            }
                                        });
                                        Utils.timeout(12, function () {
                                            sprite.setSpriteSheet('boy/jumpattack-land', function () {
                                                sprite.setSpriteSheet('boy/idle');
                                                Utils.timeout(30, setIdle);
                                            });
                                        });
                                    }
                                });

                            };
                            var setLand = function () {
                                new Tween({
                                    from: jumpExample.position.x,
                                    to: jumpExample.position.x + 8,
                                    in: 30,
                                    ease: 'exponential',
                                    decay: 2,
                                    onUpdate: function (v, t) {
                                        jumpExample.position.x = v;
                                    }
                                });
                            };
                            setIdle();
                        }
                    }
                ]
            });
            var gestureSprite = new Sprite({
                spriteSheet: 'gui/hand-gesture-flickntap'
            });
            var gesture = new Entity({
                name: 'gesture',
                position: new Vector2(0, 32),
                scale: new Vector2(0.75, 0.75),
                components: [
                    gestureSprite
                ]
            });
            var gestureContainer = new Entity({
                name: 'gestureContainer',
                position: new Vector2(0, 0),
                components: [
                    gesture
                ]
            });
            var playerContainer = new Entity({
                name: 'player',
                family: [''],
                position: new Vector2(0, 0),
                components: [
                    jumpExample
                ]
            });
            var contents = new ScrollingList({
                name: 'contents',
                direction: 'y',
                spacing: 8,
                position: new Vector2(0, -64)
            });
            contents.addItem(title);
            contents.addItem(text);
            contents.addItem(playerContainer);
            contents.addItem(gestureContainer);
            contents.addItem(new Entity({}));

            // initially trying to make the contents scrollable
            // but something seems wrong with the MaskedContainer
            contents.setActive(false);

            return contents;
        },
        'scrolls/charge': function () {
            var title = new Text({
                position: new Vector2(0, 0),
                text: Localization.getText('scrollsText')['charge']['title'],
                font: 'font',
                fontSize: 16,
                fontColor: '#733d51',
                align: 'center',
                textBaseline: 'middle',
                maxWidth: 160 - 8,
                maxHeight: 16,
                linebreaks: false
            });
            var text = new Text({
                position: new Vector2(0, 0),
                text: Localization.getText('scrollsText')['charge']['text'],
                font: 'font',
                fontSize: 16,
                fontColor: '#c29b5d',
                align: 'center',
                textBaseline: 'middle',
                maxWidth: 160 - 8,
                maxHeight: 32
            });
            var sprite = new Sprite({
                spriteSheet: 'boy/idle'
            });
            var example = new Entity({
                name: 'example',
                position: new Vector2(0, 16),
                components: [
                    sprite, {
                        name: 'exampleBehavior',
                        start: function () {
                            var setIdle = function () {
                                sprite.setSpriteSheet('boy/idle');
                                gestureSprite.setSpriteSheet('gui/hand-gesture-hold');
                                Utils.timeout(20, setCharge);
                            };
                            var setCharge = function () {
                                sprite.setSpriteSheet('boy/chargeup', function () {
                                    sprite.setSpriteSheet('boy/chargeup-loop');
                                });
                                Utils.timeout(60, function () {
                                    new Tween({
                                        from: 1.2,
                                        to: 1,
                                        'in': 20,
                                        ease: Tween.EASEINBACK,
                                        onUpdate: function (v, t) {
                                            example.scale.x = v;
                                            example.scale.y = 1 / v;
                                        }
                                    });

                                    Bento.objects.attach(new Effect({
                                        z: Globals.Layers.PAUSE + 1,
                                        spriteSheet: 'effects/charge',
                                        position: example.getWorldPosition().add(new Vector2(-4, -16))
                                    }));
                                    sprite.setSpriteSheet('boy/chargeup-ready');
                                    Utils.timeout(30, setAttack);
                                });

                            };
                            var setAttack = function () {
                                sprite.setSpriteSheet('boy/chargeup-attack', function () {
                                    sprite.setSpriteSheet('boy/idle');
                                    Utils.timeout(30, setIdle);

                                });
                                gestureSprite.setSpriteSheet('gui/hand-gesture-release');
                            };
                            setIdle();
                        }
                    }
                ]
            });
            var gestureSprite = new Sprite({
                spriteSheet: 'gui/hand-gesture-flickntap'
            });
            var gesture = new Entity({
                name: 'gesture',
                position: new Vector2(0, 32),
                scale: new Vector2(0.75, 0.75),
                components: [
                    gestureSprite
                ]
            });
            var gestureContainer = new Entity({
                name: 'gestureContainer',
                position: new Vector2(0, 0),
                components: [
                    gesture
                ]
            });
            var playerContainer = new Entity({
                name: 'player',
                family: [''],
                position: new Vector2(0, 0),
                components: [
                    example
                ]
            });
            var contents = new ScrollingList({
                name: 'contents',
                direction: 'y',
                spacing: 8,
                position: new Vector2(0, -64)
            });
            contents.addItem(title);
            contents.addItem(text);
            contents.addItem(playerContainer);
            contents.addItem(gestureContainer);
            contents.addItem(new Entity({}));

            // initially trying to make the contents scrollable
            // but something seems wrong with the MaskedContainer
            contents.setActive(false);

            return contents;
        },
        'scrolls/super': function () {

            var title = new Text({
                position: new Vector2(0, 0),
                text: Localization.getText('scrollsText')['super']['title'],
                font: 'font',
                fontSize: 16,
                fontColor: '#733d51',
                align: 'center',
                textBaseline: 'middle',
                maxWidth: 160 - 8,
                maxHeight: 16,
                linebreaks: false
            });
            var text = new Text({
                position: new Vector2(0, 0),
                text: Localization.getText('scrollsText')['super']['text'],
                font: 'font',
                fontSize: 16,
                fontColor: '#c29b5d',
                align: 'center',
                textBaseline: 'middle',
                maxWidth: 160 - 8,
                maxHeight: 44
            });
            var sprite = new Sprite({
                spriteSheet: 'boy/idle'
            });
            var example = new Entity({
                name: 'example',
                position: new Vector2(-40, 20),
                components: [
                    sprite, {
                        name: 'exampleBehavior',
                        start: function () {
                            var idle = function () {
                                sprite.setSpriteSheet('boy/idle');
                                enemySprite.setSpriteSheet('rat-example/idle');
                                example.position = new Vector2(-40, 20);
                                enemy.position = new Vector2(8, 20);
                                enemy.alpha = 1;
                                Utils.timeout(30, walkToEnemy);
                            };
                            var walkToEnemy = function () {
                                new Tween({
                                    from: example.position.x,
                                    to: example.position.x + 24,
                                    in: 24,
                                    ease: 'linear',
                                    onStart: function () {
                                        sprite.setSpriteSheet('boy/run');
                                    },
                                    onUpdate: function (v, t) {
                                        example.position.x = v;
                                    },
                                    onComplete: attackEnemy
                                });
                            };
                            var attackEnemy = function () {
                                Utils.timeout(10, function () {
                                    attack();
                                });
                            };
                            var attack = function () {
                                var doPunch = function (value) {
                                    sprite.setSpriteSheet('boy/punch' + (value % 2 + 1), function () {
                                        doPunch(value + 1);
                                    });
                                    new Tween({
                                        from: 0.8,
                                        to: 1,
                                        in: 10,
                                        delay: 10,
                                        ease: 'elastic',
                                        decay: 3,
                                        oscillations: 1,
                                        onStart: function () {
                                            var effect = new Effect({
                                                z: Globals.Layers.PAUSE + 1,
                                                position: enemy.getWorldPosition().add(new Vector2(0, -16)),
                                                spriteSheet: 'effects/hitfx2-small'
                                            });
                                            Bento.objects.attach(effect);
                                        },
                                        onUpdate: function (v, t) {
                                            enemy.scale.x = -v;
                                            enemy.scale.y = 1 / v;
                                        }
                                    });
                                };
                                doPunch(0);
                                Utils.timeout(34, kick);
                            };
                            var kick = function () {
                                sprite.setSpriteSheet('boy/kick', function () {
                                    sprite.setSpriteSheet('boy/idle');
                                });
                                Utils.timeout(10, function () {
                                    var effect = new Effect({
                                        z: Globals.Layers.PAUSE + 1,
                                        position: enemy.getWorldPosition().add(new Vector2(0, -16)),
                                        spriteSheet: 'effects/hitfx2'
                                    });
                                    Bento.objects.attach(effect);
                                    enemySprite.setSpriteSheet('rat-example/flyup');
                                    enemy.position.y += 10;
                                    new Tween({
                                        from: enemy.position,
                                        to: enemy.position.add(new Vector2(32, -32)),
                                        in: 14,
                                        ease: 'linear',
                                        onStart: function () {},
                                        onUpdate: function (v, t) {
                                            enemy.position = v;
                                            enemy.position.y += (t / 30) * (t / 30) * 16;
                                        },
                                        onComplete: function () {

                                        }
                                    });

                                    new Tween({
                                        from: 1,
                                        to: 0,
                                        in: 10,
                                        delay: 4,
                                        onUpdate: function (v, t) {
                                            enemy.alpha = v;
                                        }
                                    });

                                    Utils.timeout(60, idle);

                                });

                            };

                            idle();
                        }
                    }
                ]
            });
            var enemySprite = new Sprite({
                spriteSheet: 'rat-example/idle'
            });
            var enemy = new Entity({
                position: new Vector2(8, 20),
                spriteSheet: 'rat-example/idle',
                scale: new Vector2(-1, 1),
                components: [
                    enemySprite
                ]
            });

            var buttonSprite = new Sprite({
                imageName: 'gui2/hud/super_button_small',
                frameCountX: 6,
                frameCountY: 6,
                padding: 1,
                originRelative: new Vector2(0.5, 0.5),
                animations: {
                    default: {
                        speed: 0.3,
                        frames: [
                            0, 1, 2, 3, 4, 5, 6, 7,
                            8, 9, 10, 11, 12, 13, 14, 15,
                            16, 17, 18, 19, 20, 21, 22, 23,
                            24, 25, 26, 27, 28, 29, 30, 31,
                            32, 33, 34, 35
                        ]
                    }
                }
            });

            var electricSprite = new Sprite({
                imageName: 'gui2/hud/lightning-small',
                frameCountX: 8,
                frameCountY: 8,
                padding: 1,
                originRelative: new Vector2(0.5, 0.5),
                animations: {
                    default: {
                        speed: 0.56,
                        frames: [
                            0, 1, 2, 3, 4, 5, 6, 7,
                            8, 9, 10, 11, 12, 13, 14, 15,
                            16, 17, 18, 19, 20, 21, 22, 23,
                            24, 25, 26, 27, 28, 29, 30, 31,
                            32, 33, 34, 35, 36, 37, 38, 39,
                            40, 41, 42, 43, 44, 45, 46, 47,
                            48, 49, 50, 51, 52, 53, 54, 55,
                            56, 57
                        ]
                    }
                }
            });

            var superButton = new Entity({
                name: 'button',
                position: new Vector2(0, 44),
                scale: new Vector2(0.75, 0.75),
                components: [
                    buttonSprite,
                    electricSprite
                ]
            });
            var playerContainer = new Entity({
                name: 'entities',
                position: new Vector2(0, 0),
                components: [
                    enemy,
                    example,
                    superButton
                ]
            });
            var contents = new ScrollingList({
                name: 'contents',
                direction: 'y',
                spacing: 8,
                position: new Vector2(0, -64)
            });
            contents.addItem(title);
            contents.addItem(text);
            contents.addItem(playerContainer);
            contents.addItem(new Entity({}));

            // initially trying to make the contents scrollable
            // but something seems wrong with the MaskedContainer
            contents.setActive(false);

            return contents;
        },
        'scrolls/grab': function () {
            var title = new Text({
                position: new Vector2(0, 0),
                text: Localization.getText('scrollsText')['grab']['title'],
                font: 'font',
                fontSize: 16,
                fontColor: '#733d51',
                align: 'center',
                textBaseline: 'middle',
                maxWidth: 160 - 8,
                maxHeight: 16,
                linebreaks: false
            });
            var text = new Text({
                position: new Vector2(0, 0),
                text: Localization.getText('scrollsText')['grab']['text'],
                font: 'font',
                fontSize: 16,
                fontColor: '#c29b5d',
                align: 'center',
                textBaseline: 'middle',
                maxWidth: 160 - 8,
                maxHeight: 44
            });
            var sprite = new Sprite({
                spriteSheet: 'boy/idle'
            });
            var example = new Entity({
                name: 'example',
                position: new Vector2(-40, 20),
                components: [
                    sprite, {
                        name: 'exampleBehavior',
                        start: function () {
                            var idle = function () {
                                gestureSprite.setSpriteSheet('gui/hand-gesture-drag');
                                sprite.setSpriteSheet('boy/idle');
                                enemySprite.setSpriteSheet('rat-example/idle');
                                example.position = new Vector2(-40, 20);
                                enemy.position = new Vector2(8, 20);
                                enemy.alpha = 1;
                                Utils.timeout(60, walkToEnemy);
                            };
                            var walkToEnemy = function () {
                                new Tween({
                                    from: example.position.x,
                                    to: example.position.x + 32,
                                    in: 30,
                                    ease: 'linear',
                                    onStart: function () {
                                        sprite.setSpriteSheet('boy/run');
                                    },
                                    onUpdate: function (v, t) {
                                        example.position.x = v;
                                    },
                                    onComplete: grabEnemy
                                });
                            };
                            var grabEnemy = function () {
                                enemySprite.setSpriteSheet('rat-example/grabbed');
                                sprite.setSpriteSheet('boy/grabbing');

                                Utils.timeout(60, function () {
                                    attack();
                                });
                            };
                            var attack = function () {
                                gestureSprite.setSpriteSheet('gui/hand-gesture-tap');
                                gestureSprite.currentAnimation.backTo = 0;
                                sprite.setSpriteSheet('boy/grabattack', function () {
                                    new Tween({
                                        from: 0.8,
                                        to: 1,
                                        in: 10,
                                        ease: 'elastic',
                                        decay: 3,
                                        oscillations: 1,
                                        onStart: function () {
                                            var effect = new Effect({
                                                z: Globals.Layers.PAUSE + 1,
                                                position: enemy.getWorldPosition().add(new Vector2(0, -16)),
                                                spriteSheet: 'effects/hitfx2-small'
                                            });
                                            Bento.objects.attach(effect);
                                        },
                                        onUpdate: function (v, t) {
                                            enemy.scale.x = -v;
                                            enemy.scale.y = 1 / v;
                                        }
                                    });
                                });
                                Utils.timeout(120, throwAway);
                            };
                            var throwAway = function () {
                                gestureSprite.setSpriteSheet('gui/hand-gesture-flick');
                                sprite.setSpriteSheet('boy/grabbing');
                                Utils.timeout(30, function () {
                                    enemySprite.setSpriteSheet('rat-example/flyup');

                                    new Tween({
                                        from: enemy.position,
                                        to: enemy.position.add(new Vector2(32, -32)),
                                        in: 30,
                                        ease: 'linear',
                                        onStart: function () {},
                                        onUpdate: function (v, t) {
                                            enemy.position = v;
                                            enemy.position.y += (t / 30) * (t / 30) * 16;
                                        },
                                        onComplete: function () {

                                        }
                                    });
                                    sprite.setSpriteSheet('boy/grabthrow', function () {
                                        sprite.setSpriteSheet('boy/idle');
                                        new Tween({
                                            from: 1,
                                            to: 0,
                                            in: 10,
                                            onUpdate: function (v, t) {
                                                enemy.alpha = v;
                                            }
                                        });

                                    });
                                    Utils.timeout(60, idle);

                                });

                            };

                            idle();
                        }
                    }
                ]
            });
            var enemySprite = new Sprite({
                spriteSheet: 'rat-example/idle'
            });
            var enemy = new Entity({
                position: new Vector2(8, 20),
                spriteSheet: 'rat-example/idle',
                scale: new Vector2(-1, 1),
                components: [
                    enemySprite
                ]
            });
            var gestureSprite = new Sprite({
                spriteSheet: 'gui/hand-gesture-drag'
            });
            var gesture = new Entity({
                name: 'gesture',
                position: new Vector2(54, 32),
                scale: new Vector2(0.75, 0.75),
                components: [
                    gestureSprite
                ]
            });
            var playerContainer = new Entity({
                name: 'entities',
                position: new Vector2(0, 0),
                components: [
                    enemy,
                    example,
                    gesture
                ]
            });
            var contents = new ScrollingList({
                name: 'contents',
                direction: 'y',
                spacing: 8,
                position: new Vector2(0, -64)
            });
            contents.addItem(title);
            contents.addItem(text);
            contents.addItem(playerContainer);
            contents.addItem(new Entity({}));

            // initially trying to make the contents scrollable
            // but something seems wrong with the MaskedContainer
            contents.setActive(false);

            return contents;
        }
    };

    /**
     * Scroll Menu
     */
    var ScrollMenu = function (settings) {
        var viewport = Bento.getViewport();
        var scrollType = settings.scrollType || 'scrolls/dash';
        var onClosed = settings.onClosed;
        var height = 0;
        var width = 158;
        var buttonClose = new IconButton({
            name: 'buttonClose',
            sfx: 'sfx_button',
            imageName: 'gui2/misc/button_small_primary',
            iconName: 'gui2/misc/button_icon_okay',
            frameCountX: 1,
            frameCountY: 3,
            onClick: function () {
                // tween out
                moveOut();
                buttonClose.setActive(false);
                SaveManager.save('openedScroll-' + scrollType, true);
                // remove any existing gesture
                Bento.objects.get('gesture', function (gesture) {
                    Bento.objects.remove(gesture);
                });
                // once the scroll is seen, we show gestures for the user
                Bento.objects.attach(new Gesture({
                    type: scrollType
                }));
            }
        }).attach(new Anchor({
            portrait: {
                screenPosition: new Vector2(0.5, 1),
                offset: new Vector2(0, -24)
            },
            landscape: {
                screenPosition: new Vector2(1, 1),
                offset: new Vector2(-32, -20)
            }
        }));
        var blurBackground = new BlurBackground({
            darken: 0.4,
        });
        var anchor = new Anchor({
            screenPosition: new Vector2(0.5, 0.5)
        });
        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {
                Bento.objects.get('healthBar', function (healthbar) {
                    healthbar.fillCombobar(Globals.maxComboPoints / 2);
                });
                // move in
                moveIn();
            },
            destroy: function (data) {},
            update: function (data) {
                top.position.y = -height / 2;
                bottom.position.y = height / 2;
                contentsContainer.dimension.width = width;
                contentsContainer.dimension.height = height;
                paperNineSlice.setHeight(height + 2);
            },
            draw: function (data) {}
        };
        var top = new Entity({
            name: 'top',
            position: new Vector2(0, 0),
            components: [
                new Sprite({
                    imageName: 'gui/scroll/scroll',
                    originRelative: new Vector2(0.5, 1)
                })
            ]
        });
        var bottom = new Entity({
            name: 'bottom',
            position: new Vector2(0, 0),
            components: [
                new Sprite({
                    imageName: 'gui/scroll/scroll',
                    originRelative: new Vector2(0.5, 0)
                })
            ]
        });
        var paperNineSlice = new NineSlice({
            imageName: 'gui/scroll/scroll-paper',
            originRelative: new Vector2(0.5, 0.5),
            width: width,
            height: height
        });
        /*var contentsContainer = new Entity({
        // var contentsContainer = new MaskedContainer({
            name: 'contentsContainer',
            position: new Vector2(0, 0),
            components: [
                scrolls[scrollType]()
            ]
        });*/
        var contentsContainer = scrolls[scrollType]();
        var scrollContainer = new Entity({
            name: 'scrollContainer',
            position: viewport.getExtents(),
            components: [
                paperNineSlice,
                contentsContainer,
                top,
                bottom,
                anchor
            ]
        });
        var entity = new Entity({
            z: settings.z || Globals.Layers.PAUSE,
            name: 'scrollMenu',
            position: settings.position,
            updateWhenPaused: 0,
            float: true,
            components: [
                new Modal({
                    pauseLevel: Globals.PauseLevels.SCROLL
                }),
                behavior,
                blurBackground,
                scrollContainer
            ]
        });
        var moveIn = function () {
            contentsContainer.alpha = 0;
            new Tween({
                from: 0,
                to: 1,
                in: 10,
                ease: 'linear',
                onUpdate: function (v, t) {
                    blurBackground.alpha = v;
                }
            });
            new Tween({
                from: viewport.width / 2 * 3,
                to: viewport.width / 2,
                in: 20,
                ease: 'exponential',
                decay: 5,
                onUpdate: function (v, t) {
                    scrollContainer.position.x = v;
                },
                onComplete: function () {
                    // open scroll
                    Bento.audio.stopSound('sfx_openscroll');
                    Bento.audio.playSound('sfx_openscroll');
                    new Tween({
                        from: 0,
                        to: 152,
                        in: 40,
                        delay: 0,
                        ease: 'exponential',
                        decay: 6,
                        onUpdate: function (v, t) {
                            height = v;
                        },
                        onComplete: function () {
                            // unhide contents
                            new Tween({
                                from: 0,
                                to: 1,
                                in: 10,
                                onUpdate: function (v, t) {
                                    contentsContainer.alpha = v;
                                }
                            });

                            // attach close button and try button
                            new Tween({
                                from: viewport.height,
                                to: viewport.height - 24,
                                in: 30,
                                ease: 'elastic',
                                delay: 30,
                                decay: 5,
                                oscillations: 1,
                                onStart: function () {
                                    entity.attach(buttonClose);
                                },
                                onUpdate: function (v, t) {
                                    buttonClose.position.y = v;
                                }
                            }).attach(new OrientationLock({}));
                        }
                    });
                }
            }).attach(new OrientationLock({}));

        };
        var moveOut = function () {
            // remove buttons
            buttonClose.removeSelf();
            // hide contents
            new Tween({
                from: 1,
                to: 0,
                in: 10,
                onUpdate: function (v, t) {
                    contentsContainer.alpha = v;
                },
                onComplete: function () {
                    // close scroll
                    new Tween({
                        from: height,
                        to: 0,
                        in: 30,
                        delay: 0,
                        ease: 'exponential',
                        decay: 6,
                        onUpdate: function (v, t) {
                            height = v;
                        },
                        onComplete: function () {
                            new Tween({
                                from: viewport.width / 2,
                                to: -viewport.width / 2,
                                in: 20,
                                ease: 'exponential',
                                decay: 5,
                                onUpdate: function (v, t) {
                                    scrollContainer.position.x = v;
                                },
                                onComplete: function () {
                                    entity.removeSelf();
                                    if (onClosed) {
                                        onClosed();
                                    }
                                }
                            }).attach(new OrientationLock({}));
                            new Tween({
                                from: 1,
                                to: 0,
                                in: 10,
                                ease: 'linear',
                                onUpdate: function (v, t) {
                                    blurBackground.alpha = v;
                                }
                            });
                        }
                    });
                }
            });
        };

        return entity;
    };
    return ScrollMenu;
});