/**
 * Settings menu
 * @moduleName SettingsMenu
 */
bento.define('gui/menus/settings', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'components/modal',
    'gui/blurbackground',
    'gui/scrollinglist',
    'components/anchor',
    'globals',
    'gui/worldselect/topbarsimple',
    'gui/headerbox',
    'modules/localization',
    'gui/buttons/togglesound',
    'gui/buttons/togglemusic',
    'gui/shadowtext',
    'bento/gui/togglebutton',
    'modules/gamecenter',
    'modules/googleplay',
    'gui/menus/cheat',
    'gui/menus/credits',
    'managers/resolution',
    'managers/save',
    'gui/dialog',
    'modules/notifications',
    'modules/cloudsave',
    'components/backbutton',
    'modules/googleplaylogin',
    'gui/centeredlist',
    'components/masksprite',
    'components/scrollingmask',
    'gui/iap/sectionlabel',
    'modules/tapticengine'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Modal,
    BlurBackground,
    ScrollingList,
    Anchor,
    Globals,
    TopBarSimple,
    HeaderBox,
    Localization,
    ToggleSound,
    ToggleMusic,
    ShadowText,
    ToggleButton,
    GameCenter,
    GooglePlay,
    CheatMenu,
    Credits,
    ResolutionManager,
    SaveManager,
    Dialog,
    Notifications,
    CloudSave,
    BackButton,
    GooglePlayLogin,
    CenteredList,
    MaskSprite,
    ScrollingMask,
    SectionLabel,
    TapticEngine
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var modal = new Modal({});
        var background = new BlurBackground({
            darken: 0.2
        });
        var scrollingList = new ScrollingList({
            position: new Vector2(0, 0),
            direction: 'x', // 'x' or 'y'
            spacing: 132, // additional spacing between items
            snap: true, // snap items (cancels maxOffset and minOffset)
            onSnap: function (item, idx) {
                dotList.iterate(function (dot, i) {
                    if (i === idx) {
                        dot.getComponent('sprite', function (sprite) {
                            sprite.setAnimation('down');
                        });
                        
                    } else {
                        dot.getComponent('sprite', function (sprite) {
                            sprite.setAnimation('up');
                        });
                    }
                });
                // enable/disable buttons
                
            }, // called when snap focus changed
            onLoseFocus: function (index) {}, // called when snap focus changed
            snapMinSlideSpeed: 1, // minimum slidespeed before starting to snap
            snapSpeed: 0.3, // speed of snapping
            damping: 0.9 // speed decrease,
        });
        var headerBox = new HeaderBox({
            // title: Localization.getText('settingsTitle'),
            size: new Rectangle(0, 0, 188, 176),
            windowImage: 'gui2/misc/splash_background_9slice',
            components: [
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.5, 0.5),
                        offset: new Vector2(0, 0)
                    },
                    landscape: {
                        screenPosition: new Vector2(0.5, 0.5),
                        offset: new Vector2(0, 8)
                    }
                }),
                scrollingList
            ]
        });
        var alignButtons = function () {
            // DEPRECATED
        };
        var buttons = [];
        var addButton = function (buttonSettings) {
            var button = buttonSettings.button || new ClickButton(buttonSettings);
            // text
            var text = new ShadowText({
                position: new Vector2(20, 0),
                text: buttonSettings.text,
                font: 'font',
                fontSize: 14,
                fontColor: '#ffffff',
                align: 'left',
                textBaseline: 'middle',
                maxWidth: 64,
                maxHeight: 32,
                // drawDebug: true
            });
            button.attach(text);

            // scrollingList.addItem(button);
            // headerBox.attach(button);
            addToPage(button);
            buttons.push(button);

            button.extend({
                setMaxWidth: function (value) {
                    text.setText(buttonSettings.text, {
                        maxWidth: value
                    });
                }
            });
        };
        var buttonClose = new ClickButton({
            name: 'buttonBack',
            sfx: 'sfx_button',
            imageName: 'gui2/bottom_tabs/bottom_tabs__back',
            frameCountX: 2,
            frameCountY: 1,
            components: [
                new BackButton({
                    removeSelf: true,
                    onPressed: function (data) {
                        data.entity.doCallback();
                    }
                }),
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.5, 1),
                        offset: new Vector2(-48 + (Anchor.isWideScreen() ? 4 : 0), -12 + (Anchor.isWideScreen() ? Anchor.getOffset().y : 0))
                    },
                    landscape: {
                        screenPosition: new Vector2(0.25, 1),
                        offset: new Vector2(-44, -12)
                    }
                })
            ],
            onButtonDown: function () {
                EventSystem.fire('preventScroll');
            },
            onClick: function () {
                entity.removeSelf();
            }
        });
        var currentPage;
        var dotList = new CenteredList({
            spacing: 3,
            components: [
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.5, 1),
                        offset: new Vector2(0, -64),
                    },
                    landscape: {
                        screenPosition: new Vector2(0.5, 1),
                        offset: new Vector2(0, -4),
                    },
                    onComplete: function () {
                        if (Anchor.isPortrait()) {
                            dotList.scale = new Vector2(1, 1);
                        } else {
                            dotList.scale = new Vector2(0.5, 0.5);
                        }
                    }
                })
            ]
        });
        var pages = [];
        var makePage = function () {
            var index = scrollingList.getItems().length;
            var items = 0;
            var page = new Entity({
                name: 'page',
                components: []
            }).extend({
                add: function (item) {
                    page.attach(item);
                    item.position = new Vector2(-48, -38 * 1.5 + items * 38);
                    items += 1;
                    return items;
                },
                isFull: function () {
                    return items >= 4;
                }
            });
            var dotSprite = new Sprite({
                frameCountX: 2,
                originRelative: new Vector2(0.5, 0.5),
                imageName: 'gui2/offers/dots',
                animations: {
                    'up': {
                        frames: [1]
                    },
                    'down': {
                        frames: [0]
                    }
                }
            });
            dotList.addItem(new Entity({
                name: 'dot',
                components: [
                    dotSprite
                ]
            }));
            if (index === 0) {
                dotSprite.setAnimation('down');
            } else {
                dotSprite.setAnimation('up');
            }

            pages.push(page);
            currentPage = page;
            scrollingList.addItem(page);
            return page;
        };
        var addToPage = function (button) {
            if (!currentPage || currentPage.isFull()) {
                // create new page
                currentPage = makePage();
            }
            currentPage.add(button);
        };
        var sectionLabel = new SectionLabel({
            text: Localization.getText('settingsTitle'),
            position: new Vector2(0, -104)
        });
        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {
                var toggleSound = new ToggleSound({});
                var toggleMusic = new ToggleMusic({});
                var toggleOrientation = new ToggleButton({
                    name: 'toggleOrientation',
                    sfx: 'sfx_button',
                    imageName: 'gui/buttons/orientation',
                    frameCountX: 1,
                    frameCountY: 2,
                    toggled: Bento.saveState.load('orientationLock', false),
                    onToggle: function () {
                        ResolutionManager.setUserLock(toggleOrientation.isToggled());
                    },
                    onButtonDown: function () {
                        scrollingList.cancel();
                    }
                });
                var toggleHaptics = new ToggleButton({
                    name: 'toggleHaptic',
                    sfx: 'sfx_button',
                    imageName: 'gui/buttons/haptics',
                    frameCountX: 1,
                    frameCountY: 2,
                    toggled: Bento.saveState.load('disableTapticEngine', false),
                    onToggle: function () {
                        Bento.saveState.save('disableTapticEngine', toggleHaptics.isToggled());
                        TapticEngine.enable(!Bento.saveState.load('disableTapticEngine', false));
                    },
                    onButtonDown: function () {
                        scrollingList.cancel();
                    }
                });
                var buttonCredits = new ClickButton({
                    name: 'credits',
                    sfx: 'sfx_button',
                    imageName: 'gui/buttons/credits',
                    frameCountX: 1,
                    frameCountY: 2,
                    onClick: function () {
                        var credits = new Credits({});
                        Bento.objects.attach(credits);
                    },
                    onButtonDown: function () {
                        scrollingList.cancel();
                    }
                });
                var buttonCheat = new ClickButton({
                    name: 'cheat',
                    sfx: 'sfx_button',
                    imageName: 'gui/buttons/credits',
                    frameCountX: 1,
                    frameCountY: 2,
                    onClick: function () {
                        var cheat = new CheatMenu({});
                        Bento.objects.attach(cheat);
                    },
                    onButtonDown: function () {
                        scrollingList.cancel();
                    }
                });
                var buttonNotifications = new ClickButton({
                    name: 'notifications',
                    sfx: 'sfx_button',
                    imageName: 'gui/buttons/notifications',
                    frameCountX: 1,
                    frameCountY: 2,
                    onClick: function () {
                        // ask for notifications again
                        var dialog = new Dialog({
                            z: entity.z + 1,
                            name: 'askForNotificationInSettings',
                            type: 'yesno',
                            title: Localization.getText('notifications')['askTitle'],
                            text: Localization.getText('notifications')['ask'],
                            canClose: false,
                            onYes: function () {
                                // ask for real notification
                                Notifications.init(function (initialized) {
                                    var allowed = initialized ? true : false;
                                    SaveManager.save('notificationsAllowed', allowed);
                                    if (allowed) {
                                        // fire event to schedule inactive notification
                                        EventSystem.fire('allowedNotifications');
                                    }
                                });
                            }
                        });
                        Bento.objects.attach(dialog);
                    },
                    onButtonDown: function () {
                        scrollingList.cancel();
                    }
                });
                var buttonCloudSave = new ClickButton({
                    name: 'cloudsave',
                    sfx: 'sfx_button',
                    imageName: 'gui/buttons/cloudsave',
                    frameCountX: 1,
                    frameCountY: 2,
                    onClick: function () {
                        var dialog;
                        var service;
                        // login first
                        if (!CloudSave.isLoggedIn()) {
                            service = Localization.getText(Utils.isNativeIos() ? 'iOSService' : 'AndroidService');
                            dialog = new Dialog({
                                z: entity.z + 1,
                                name: 'askForCloudLoad',
                                type: 'yesno',
                                title: Localization.getText('cloud'),
                                text: Localization.getText('cloudLogin').replace('{SERVICE}', service),
                                canClose: false,
                                onYes: function () {
                                    CloudSave.login();
                                }
                            });
                            Bento.objects.attach(dialog);
                            return;
                        }

                        // ask first
                        dialog = new Dialog({
                            z: entity.z + 1,
                            name: 'askForCloudLoad',
                            type: 'yesno',
                            title: Localization.getText('cloud'),
                            text: Localization.getText('loadFromCloud'),
                            canClose: false,
                            onYes: function () {
                                CloudSave.loadFromCloud(function (success, reason) {
                                    var errorDialog;
                                    if (success) {
                                        if (reason === 'cloudSaveNone') {
                                            // there was nothing to load at all
                                            errorDialog = new Dialog({
                                                z: entity.z + 2,
                                                name: 'askForCloudLoad',
                                                type: 'yes',
                                                title: Localization.getText('errorTitle'),
                                                text: Localization.getText('cloudNoSaveFile')
                                            });
                                            Bento.objects.attach(errorDialog);
                                            return;
                                        }
                                        // refresh screen
                                        // go to title screen because this is where managers are initialized
                                        // and compatibility is checked
                                        Bento.screens.show('screens/title');
                                        Bento.objects.resume();
                                    } else {
                                        // show error
                                        errorDialog = new Dialog({
                                            z: entity.z + 2,
                                            name: 'askForCloudLoad',
                                            type: 'yes',
                                            title: Localization.getText('errorTitle'),
                                            text: Localization.getText('cloudLoadError')
                                        });
                                        Bento.objects.attach(errorDialog);
                                    }
                                });
                            }
                        });
                        Bento.objects.attach(dialog);
                    },
                    onButtonDown: function () {
                        scrollingList.cancel();
                    }
                });
                var gameService = 'googleplay';
                var gameServiceButton;

                if (Utils.isNativeIos()) {
                    gameService = 'gamecenter';
                }

                gameServiceButton = new ClickButton({
                    name: 'buttonGameCenter',
                    sfx: 'sfx_button',
                    imageName: 'gui/buttons/' + gameService,
                    frameCountX: 1,
                    frameCountY: 2,
                    onClick: function () {
                        if (Utils.isNativeIos()) {
                            GameCenter.showAchievement(function () {});
                        } else {
                            if (GooglePlay.isLoggedIn()) {
                                GooglePlay.showAchievement(function () {});
                            } else {
                                // log in first
                                GooglePlayLogin(function (hasLoggedIn) {
                                    if (hasLoggedIn) {
                                        // try showing achievements again
                                        GooglePlay.showAchievement(function () {});
                                    }
                                });
                            }
                        }
                    },
                    onButtonDown: function () {
                        scrollingList.cancel();
                    }
                });

                // add buttons
                addButton({
                    button: toggleSound,
                    text: Localization.getText('muteSound')
                });
                addButton({
                    button: toggleMusic,
                    text: Localization.getText('muteMusic')
                });
                addButton({
                    button: toggleOrientation,
                    text: Localization.getText('settingsLockRotation')
                });
                if (Bento.isDev() || Utils.isNativeIos()) {
                    addButton({
                        button: toggleHaptics,
                        text: Localization.getText('haptics')
                    });
                }
                // notifications
                if (SaveManager.load('notificationsAllowed') === false) {
                    addButton({
                        button: buttonNotifications,
                        text: Localization.getText('notifications')['askTitle']
                    });
                }
                addButton({
                    button: buttonCloudSave,
                    text: Localization.getText('cloudButton')
                });

                if (Bento.isDev() || Utils.isNativeIos() || Utils.isNativeAndroid()) {
                    addButton({
                        button: gameServiceButton,
                        text: Localization.getText(Utils.isNativeIos() ? 'iOSService' : 'AndroidService')
                    });
                }
                addButton({
                    button: buttonCredits,
                    text: Localization.getText('credits')
                });

                if (Bento.isDev() || Bento.saveState.load('showDeveloperCheats', false)) {
                    // addButton({
                    //     button: buttonCheat,
                    //     text: 'Cheats'
                    // });
                    buttonCheat.attach(new Anchor({
                        screenPosition: new Vector2(1, 1),
                        offset: new Vector2(-20, -24)
                    }));
                    entity.attach(buttonCheat);
                }
                alignButtons();

                // set max width
                if (buttons.length > 6) {
                    Utils.forEach(buttons, function (button, i, l, breakLoop) {
                        button.setMaxWidth(96);
                    });
                }
                new Tween({
                    from: 0,
                    to: 1,
                    in: 30,
                    ease: 'elastic',
                    decay: 8,
                    onUpdate: function (v, t) {
                        headerBox.scale.x = v;
                        headerBox.scale.y = v;
                    }
                });

                // mask all pages
                Utils.forEach(pages, function (page, i, l, breakLoop) {
                    page.attach(new ScrollingMask({
                        boundingBox: new Rectangle(-86, -86, 172, 172),
                        direction: 'x',
                        scrollingList: scrollingList
                    }));
                });

                headerBox.attach(sectionLabel);
                headerBox.moveComponentTo(sectionLabel, 0);
            },
            destroy: function (data) {},
            update: function (data) {},
            draw: function (data) {}
        };
        var topBar = new TopBarSimple({
            preventChests: settings.preventChests,
            preventClick: Utils.isDefined(settings.preventTopBar) ? settings.preventTopBar : true, // don't open the IAP menu during settings
            skipTutorial: settings.skipTutorial
        });
        var versionText = new ShadowText({
            position: new Vector2(1, viewport.height - 1),
            text: 'v' + window.VERSION + '-' + window.BUILD + '.' + (window.PATCHVERSION || 0) + (Bento.isDev() ? 'DEV' : ''),
            font: 'font',
            align: 'right',
            fontSize: 8,
            fontColor: '#ffffff',
            textBaseline: 'bottom'
        }).attach(new Anchor({
            screenPosition: new Vector2(1, 1),
            offset: new Vector2(-1, -1)
        }));
        var entity = new Entity({
            z: Globals.WorldSelectLayers.SETTINGS,
            name: 'settingsMenu',
            position: settings.position,
            float: true,
            components: [
                background,
                modal,
                versionText,
                headerBox,
                topBar,
                dotList,
                buttonClose,
                behavior
            ]
        });
        return entity;
    };
});