/**
 * Shop overlay.
 * @moduleName ShopMenu
 */
bento.define('gui/menus/shop', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/nineslice',
    'bento/entity',
    'bento/utils',
    'bento/gui/clickbutton',
    'globals',
    'bento/tween',
    'components/anchor',
    'managers/resolution',
    'gui/shop/button',
    'modules/text',
    'managers/item',
    'managers/inventory',
    'bento/eventsystem',
    'gui/shadowtext',
    'gui/playercoins',
    'managers/currency',
    'gui/speechballoon',
    'gui/worldselect/topbarsimple',
    'components/modal',
    'managers/save',
    'managers/time',
    'modules/localization',
    'gui/watchrewardedad',
    'managers/character',
    'gui/taphand',
    'components/viewanalytics',
    'modules/analytics',
    'components/hidebelow',
    'components/orientationlock',
    'gui/menus/iap',
    'gui/iconbutton',
    'components/glow'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    NineSlice,
    Entity,
    Utils,
    ClickButton,
    Globals,
    Tween,
    Anchor,
    ResolutionManager,
    ShopButton,
    Text,
    ItemManager,
    InventoryManager,
    EventSystem,
    ShadowText,
    PlayerCoins,
    CurrencyManager,
    SpeechBalloon,
    TopBarSimple,
    Modal,
    SaveManager,
    TimeManager,
    Localization,
    WatchRewardedAd,
    CharacterManager,
    TapHand,
    ViewAnalytics,
    Analytics,
    HideBelow,
    OrientationLock,
    IAPMenu,
    IconButton,
    Glow
) {
    'use strict';
    var exclusions = ['rad'];
    var possibleItems = {
        currency: [{
            key: 'currency/key',
            item: 'key',
            category: 'currency',
            tier: 0
        }], // possibility of a key
        weapons: [],
        consumables: [],
        upgrades: [],
        equipment: [],
        playertokens: [],
        enemytokens: []
    };
    var initPossibleItems = function () {
        var assets = Bento.assets.getAssets();
        var jsonAssets = assets.json;
        var isMaxEvolved = function (char) {
            return CharacterManager.getLevel(char) >= CharacterManager.getMaxTotal(char);
        };
        var isExcluded = function (charName) {
            return !CharacterManager.isUnlocked(charName) && exclusions.indexOf(charName) >= 0;
        };

        Utils.forEach(jsonAssets, function (item, key, l, breakLoop) {
            // weapons count as consumables too
            var keys = key.split('/');
            var category = keys[0];
            var name = keys[1];
            var array = category;
            if (
                category === 'weapons' ||
                category === 'consumables' ||
                category === 'upgrades' ||
                category === 'weapons' ||
                category === 'equipment' ||
                category === 'enemytokens'
            ) {
                possibleItems[array].push({
                    key: key,
                    item: item,
                    category: category,
                    tier: (item.inventory || {}).tier || 0
                });
            }
            if (category === 'playertokens' && !isExcluded(name)) {
                // additional parameter for playertokens: maxEvolved character
                possibleItems[array].push({
                    key: key,
                    item: item,
                    category: category,
                    tier: (item.inventory || {}).tier || 0,
                    maxEvolved: isMaxEvolved(name)
                });
            }
        });
    };

    return function (settings) {
        var viewport = Bento.getViewport();
        var screenMid = new Vector2(viewport.width / 2, viewport.height / 2);
        var portraitWidth = ResolutionManager.isPortrait() ? viewport.width : viewport.height;
        var portraitHeight = ResolutionManager.isPortrait() ? viewport.height : viewport.width;
        var unlockedLevels = SaveManager.load('unlockedLevels').length;

        var buttons = [];
        var buttonWidth = 48;
        var buttonHeight = 48;

        var coinValue = CurrencyManager.getCoins();
        var coinTween;
        var speechBalloonPosition = new Vector2(0, 0);
        var speechBalloonSize = new Rectangle(0, 0, 100, 36);

        var portrait;
        var align = function (parent, isPortrait) {
            portrait = isPortrait;
            if (isPortrait) {
                timeText.position = new Vector2(-viewport.width / 2 + 8, -20);
                speechBalloonPosition = new Vector2(viewport.width / 2, viewport.height / 2 - 112);
                buttonAd.position.y = viewport.height - 16 - 168;
                buttonGem.position.y = viewport.height - 16 - 168;
            } else {
                timeText.position = new Vector2(-viewport.width / 2 + 8, viewport.height / 2 - 4);
                speechBalloonPosition = new Vector2(viewport.width / 2 - 84, viewport.height / 2 - 48);
                buttonAd.position.y = viewport.height - 16 - 8;
                buttonGem.position.y = viewport.height - 16 - 8;
            }
            if (isPortrait && Anchor.isWideScreen()) {
                speechBalloonPosition.y -= 16;
            }
            if (currentSpeechBalloon) {
                currentSpeechBalloon.position = speechBalloonPosition.clone();
            }
        };
        var anchorComponent = new Anchor({
            screenPosition: new Vector2(0.5, 0.5),
            onComplete: align
        });
        var refreshText = new ShadowText({
            position: new Vector2(0, -22),
            text: Localization.getText('refresh'),
            font: 'font',
            fontSize: 12,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            maxWidth: 48,
            maxHeight: 12,
            linebreaks: false
        });
        var buttonAd = new IconButton({
            z: 0,
            name: 'buttonAd',
            sfx: 'sfx_button',
            imageName: 'gui2/misc/buttonmini_small_secondary',
            iconName: 'gui2/misc/button_icon_watchad',
            position: new Vector2(-32, viewport.height - 16),
            frameCountX: 1,
            frameCountY: 3,
            animations: {
                up: {
                    speed: 0.03,
                    frames: [0]
                },
                down: {
                    frames: [1]
                }
            },
            onClick: function () {
                WatchRewardedAd(function (hasCompleted) {
                    var dailyData;
                    var dailyAds = SaveManager.load('dailyAdRefresh');
                    if (hasCompleted) {
                        // refresh shop buttons
                        Utils.forEach(buttons, function (button, i, l, breakLoop) {
                            button.removeSelf();
                        });
                        buttons = [];
                        dailyData = SaveManager.load('dailyShop');
                        dailyData.time = 0;
                        SaveManager.save('dailyShop', dailyData);
                        createShopButtons();
                        align(null, portrait);
                        talk({
                            text: Localization.getText('shopWatched')
                        });
                        Analytics.event('Ads', 'watch_for_dailyshoprefresh');

                        dailyAds.adsWatched += 1;
                        SaveManager.save('dailyAdRefresh', dailyAds);
                        checkAdButton();
                    }
                });
            },
            components: [
                refreshText,
                new Glow({
                    amplitude: 0.2
                })
            ]
        });
        var labelText = new ShadowText({
            position: new Vector2(7, -1),
            text: Globals.gemToRefreshShop.toString(),
            font: 'font',
            fontSize: 16,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            maxWidth: 14,
            maxHeight: 16
        });
        var refreshText2 = new ShadowText({
            position: new Vector2(0, -22),
            text: Localization.getText('refresh'),
            font: 'font',
            fontSize: 12,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            maxWidth: 48,
            maxHeight: 12,
            linebreaks: false
        });
        var buttonGem = new IconButton({
            name: 'buttonGem',
            sfx: 'sfx_button',
            imageName: 'gui2/misc/buttonmini_small_secondary',
            iconName: 'gui2/misc/button_icon_gem',
            offset: new Vector2(-8, 0),
            position: new Vector2(-32, viewport.height - 16),
            frameCountX: 1,
            frameCountY: 3,
            onButtonDown: function () {
                labelText.position.y = 0;
            },
            onButtonUp: function () {
                labelText.position.y = -1;
            },
            onClick: function () {
                var dailyData;
                if (CurrencyManager.canSpendGems(Globals.gemToRefreshShop)) {
                    CurrencyManager.spendGems(Globals.gemToRefreshShop);
                    // refresh shop buttons
                    Utils.forEach(buttons, function (button, i, l, breakLoop) {
                        button.removeSelf();
                    });
                    buttons = [];
                    dailyData = SaveManager.load('dailyShop');
                    dailyData.time = 0;
                    SaveManager.save('dailyShop', dailyData);
                    createShopButtons();
                    align(null, portrait);
                    talk({
                        text: Localization.getText('shopWatched')
                    });
                    Analytics.event('Economy', 'gems_for_dailyshoprefresh', 1);

                    checkAdButton();
                } else {
                    // open iap shop
                    var iapMenu = new IAPMenu({
                        // z: Globals.Layers.SHOP + 1,
                        scrollTo: 'gems'
                    });
                    Bento.objects.attach(iapMenu);
                }
            },
            components: [
                refreshText2,
                labelText,
                new Glow({
                    amplitude: 0.2
                })
            ]
        });

        var checkAdButton = function () {
            var dailyAds = SaveManager.load('dailyAdRefresh');
            // check time
            var day = dailyAds.day;
            var today = Math.floor(TimeManager.getTime() / 1000 / 60 / 60 / 24);

            // is the day different? reset the number of ads watched
            if (today > day) {
                dailyAds.adsWatched = 0;
                dailyAds.day = today;
                SaveManager.save('dailyAdRefresh', dailyAds);
            }

            // allow 2 ads per day
            if (dailyAds.adsWatched < Globals.maxAdsDailyShop) {
                buttonAd.setActive(true);
                buttonAd.visible = true;
                buttonGem.setActive(false);
                buttonGem.visible = false;
            } else {
                buttonAd.removeSelf();
                buttonGem.setActive(true);
                buttonGem.visible = true;
            }

        };

        var shadow = new Entity({
            name: 'shadow',
            components: [
                new Sprite({
                    imageName: 'shadow24',
                    originRelative: new Vector2(0.5, 0.5)
                })
            ]
        });
        var currentCharacter = CharacterManager.getSelected();
        var player = new Entity({
            name: 'player',
            position: new Vector2(-12, 120),
            components: [
                shadow,
                new Sprite({
                    spriteSheet: currentCharacter + '/idle'
                })
            ]
        });
        var merchantSprite = new Sprite({
            spriteSheet: 'shop/frog_thing'
        });
        var merchant = new Entity({
            name: 'merchant',
            position: new Vector2(42, 100),
            components: [
                merchantSprite
            ]
        });
        // disable animation for now
        merchantSprite.setCurrentSpeed(0);

        var spriteBackground = new Sprite({
            imageName: 'gui2/shop/background',
            origin: new Vector2(130, 40)
        });
        var padding = new Sprite({
            imageName: 'gui2/shop/padding',
            origin: new Vector2(-260, 40)
        });
        var background = new Entity({
            name: 'background',
            components: [
                spriteBackground,
                padding,
                player,
                merchant,
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0, -0.5),
                        offset: new Vector2(0, 0)
                    },
                    landscape: {
                        screenPosition: new Vector2(0, -0.5),
                        offset: new Vector2(0, 0)
                    },
                    onComplete: function (parent, isPortrait) {
                        if (!isPortrait) {
                            background.position.x -= 92;
                        }
                        if (Anchor.isWideScreen()) {
                            background.position.x += 2;
                        }
                    }
                })
            ]
        });



        var drawer = new Entity({
            components: [
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0, 0),
                        offset: new Vector2(0, -35)
                    },
                    landscape: {
                        screenPosition: new Vector2(0, 0),
                        offset: new Vector2(-23, 0)
                    },
                    onComplete: function (parent, isPortrait) {
                        var originRelative = isPortrait ?
                            new Vector2(0.5, 0) :
                            new Vector2(0, 0.5);

                        if (Anchor.isPortrait() && Anchor.isWideScreen()) {
                            drawer.position.y += 20;
                            titleBar.position.y += 20;
                        }
                    }
                })
            ]
        });

        var buttonClose = new ClickButton({
            name: 'buttonClose',
            position: new Vector2(20, 20),
            sfx: 'sfx_button',
            imageName: 'gui/buttons/exit',
            frameCountX: 2,
            frameCountY: 1,
            onClick: function () {
                moveOut(function () {
                    Bento.objects.remove(container);
                    if (settings.onComplete) {
                        settings.onComplete();
                    }
                });
                if (tapHand) {
                    tapHand.removeSelf();
                }
            }
        });
        var tapHand;

        var timeText = new ShadowText({
            text: '',
            font: 'font',
            fontSize: 16,
            fontColor: '#ffffff',
            align: 'left',
            textBaseline: 'bottom',
            maxWidth: 120,
            maxHeight: 16,
            linebreaks: false
        });

        var spacing = {
            portrait: 18,
            landscape: 8
        };

        // DEPRECATED: no more tutorial during shop
        // var gotTutorialItems = SaveManager.load('tutorialDailyShopItems');
        var gotTutorialItems = 1;
        var createShopButton = function (key, index) {
            var opts = {};
            var idx = buttons.length;

            var price = ItemManager.getPrice(key);

            opts.item = {};
            opts.item.key = key;
            opts.item.name = ItemManager.getName(key);
            opts.item.currency = ItemManager.getCurrency(key);
            opts.item.tier = ItemManager.getTier(key);
            opts.item.description = ItemManager.getDescription(key);
            opts.item.stats = ItemManager.getStats(key);
            opts.item.sprite = ItemManager.getSprite(key);
            opts.item.price = price;

            if (!gotTutorialItems) {
                opts.item.price = 0;
                opts.item.tutorial = true;
            }


            opts.offset = {
                portrait: new Vector2(
                    10 - portraitWidth / 2,
                    buttonHeight / 2 + spacing.portrait + 46 * idx + 10
                ),
                landscape: new Vector2(
                    buttonWidth / 2 - 16,
                    (buttonHeight - 3) * (idx - 1) - 5
                )
            };

            opts.spacing = spacing;

            var shopButton = new ShopButton({
                z: settings.z,
                width: buttonWidth,
                height: buttonHeight,
                opts: opts,
                index: index,
                onBought: function () {
                    if (gotTutorialItems) {
                        talk({
                            text: Localization.getText('shopThanks'),
                            removeAfter: 240,
                            onComplete: function () {
                                tellRefresh();
                            }
                        });
                    } else {
                        // check if user got all the items
                        var dailyData = SaveManager.load('dailyShop');
                        if (
                            dailyData.bought[0] === Globals.maxShopItems &&
                            dailyData.bought[1] === Globals.maxShopItems &&
                            dailyData.bought[2] === Globals.maxShopItems
                        ) {
                            gotTutorialItems = 1;
                            SaveManager.save('tutorialDailyShopItems', 1);
                            tapHand = new TapHand({
                                z: Globals.WorldSelectLayers.SHOP,
                                position: buttonClose.getWorldPosition(),
                                rotation: Math.PI * 7 / 8
                            });
                            Bento.objects.attach(tapHand);
                            EventSystem.fire('tutorialDailyShopItems');
                        }
                    }
                }
            });

            buttons.push(shopButton);
            return shopButton;
        };

        /**
         * Initialize daily items
         */
        var createShopButtons = function () {
            var shouldUpdate = false;
            var dailyData = SaveManager.load('dailyShop');
            var dailyItems = [];
            var categories1 = ['playertokens', 'enemytokens'];
            var categories2 = ['playertokens', 'enemytokens'];
            var categories3 = ['playertokens', 'currency', 'currency'];
            var addNewItem = function (items, categories) {
                var category = categories[Utils.getRandom(categories.length)];
                var itemsArray = possibleItems[category];
                var randomArray = pickRandom(itemsArray);
                var randomItem = randomArray.key;

                // special case: only tokens that you need
                if (category === 'upgrades') {
                    var keys = randomItem.split('/');
                    var itemName = keys[1].split('-');
                    var tokenType = itemName[0];
                    var tokenCharacter = itemName[1];

                    /*// check amount
                    var amount = CurrencyManager.getToken(tokenCharacter, tokenType);
                    // check needed
                    var level = SaveManager.load('level-' + tokenCharacter);
                    var needed = Globals.getTokenCost(level);

                    if (amount >= needed) {
                        // try again
                        addNewItem(items, categories);
                        return;
                    }*/
                }

                // check if already exists
                if (items.indexOf(randomItem) < 0) {
                    items.push(randomItem);
                } else {
                    // try again
                    addNewItem(items, categories);
                }
            };
            var i = 0;

            // initialize tutorial items
            if (!gotTutorialItems && dailyData.firstTime === true) {
                dailyData = {
                    firstTime: false,
                    time: 0,
                    items: [
                        'consumables/attackbonus1',
                        'equipment/bandana1',
                        'upgrades/health-boy'
                    ],
                    bought: [Globals.maxShopItems - 1, Globals.maxShopItems - 1, Globals.maxShopItems - 1]
                };
            }

            // compare time
            var serverTime = TimeManager.getTime();

            if (serverTime >= dailyData.time && gotTutorialItems) {
                // schedule 3 hours into the future
                dailyData.time = serverTime + 3 * 60 * 60 * 1000;

                // reset data
                dailyData.items = [];
                dailyData.bought = [0, 0, 0];

                // pick 3 items to add
                addNewItem(dailyData.items, categories1);
                addNewItem(dailyData.items, categories2);
                addNewItem(dailyData.items, categories3);
            }
            SaveManager.save('dailyShop', dailyData);

            Utils.forEach(dailyData.items, function (item) {
                dailyItems.push(item);
            });
            while (dailyItems.length > 0) {
                drawer.attach(createShopButton(dailyItems.shift(), i));
                i += 1;
            }

        };

        var pickRandom = function (array) {
            var weightedItems = [];
            var winner;
            Utils.forEach(array, function (item, i, l, breakLoop) {
                var weight = 1;
                if (item.tier) {
                    if (item.tier === 2) {
                        weight = 0.25;
                    } else if (item.tier > 2) {
                        weight = 0.05;
                    }
                }
                if (Utils.isDefined(item.maxEvolved)) {
                    // give 0 weight to max leveled and evolved characters
                    weight *= (item.maxEvolved ? 0 : 1);
                }
                weightedItems.push({
                    key: item.key,
                    item: item.item,
                    weight: weight
                });
            });
            winner = weightedRandom(weightedItems);
            return winner;
        };

        /*
         * Weighted randomization
         * array = [{value: Something, weight: Number }, ...]
         */
        var weightedRandom = function (array) {
            // calculate sum of weights
            var weightTotal = 0;
            Utils.forEach(array, function (item, i, l, breakLoop) {
                weightTotal += item.weight || 0;
            });
            // exception: 0 total weight
            if (weightTotal === 0) {
                // assign equal weights
                Utils.forEach(array, function (item) {
                    item.weight = 1;
                });
                weightTotal = array.length;
            }

            // randomize a number
            var random = Utils.getRandomRangeFloat(0, weightTotal);
            // console.log(weightTotal, random);
            // find the winner
            var winner;
            var weightIncr = 0;
            Utils.forEach(array, function (item, i, l, breakLoop) {
                if (item.weight <= 0) {
                    return;
                }
                weightIncr += item.weight;
                if (weightIncr > random) {
                    winner = item;
                    breakLoop();
                    return;
                }
            });
            // console.log(winner);
            return winner;
        };

        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {
                // init possible items if needed
                if (possibleItems.consumables.length === 0) {
                    initPossibleItems();
                }

                // get daily shop items
                createShopButtons();

                Bento.audio.stopAllMusic();
                if (Utils.isNativeAndroid()) {
                    // QUICKFIX: for some reason android sometimes skips the music
                    new Tween({
                        ignoreGameSpeed: true,
                        in: 3,
                        onComplete: function () {
                            Bento.audio.playMusic('bgm_shop');
                        }
                    });
                } else {
                    Bento.audio.playMusic('bgm_shop');
                }

                moveIn();
                checkAdButton();

                if (ShopButton.isShopEvent()) {
                    var starText = new ShadowText({
                        position: new Vector2(0, -3),
                        text: Localization.getText('discountOff').replace('{NUMBER}', 20),
                        font: 'font',
                        fontSize: 16,
                        fontColor: '#ffffff',
                        align: 'center',
                        textBaseline: 'middle',
                        ySpacing: -2,
                        maxWidth: 25,
                        maxHeight: 20,
                        linebreaks: true,
                        rotation: -Math.PI / 16
                    });
                    var star = new Entity({
                        name: 'star',
                        position: new Vector2(72, -20),
                        components: [
                            new Sprite({
                                imageName: 'gui/offerpacks/star',
                                originRelative: new Vector2(0.5, 0.5)
                            }),
                            starText, {
                                name: 'wobble',
                                update: function () {
                                    star.rotation = Math.PI / 20 * Math.sin(star.timer / 20);
                                }
                            }
                        ]
                    });
                    titleBar.attach(star);
                }
            },
            destroy: function (data) {
                if (currentSpeechBalloon) {
                    currentSpeechBalloon.removeSelf();
                }

                Bento.audio.stopAllMusic();
                if (Utils.isNativeAndroid()) {
                    // QUICKFIX: for some reason android sometimes skips the music
                    new Tween({
                        ignoreGameSpeed: true,
                        in: 3,
                        onComplete: function () {
                            Bento.audio.setVolume(1, 'bgm_worldmap');
                            Bento.audio.playMusic('bgm_worldmap');
                        }
                    });
                } else {
                    Bento.audio.playMusic('bgm_worldmap');
                }

            },
            update: function (data) {}
        };

        var titleBar = new Entity({
            name: 'titleBar',
            components: [
                new Sprite({
                    imageName: 'gui2/misc/title_bar',
                    originRelative: new Vector2(0.5, 0.5),
                }),
                new Text({
                    position: new Vector2(0, -16),
                    text: Localization.getText('todaysdeals'),
                    font: 'font',
                    fontSize: 16,
                    fontColor: '#ffdf00',
                    align: 'center',
                    textBaseline: 'middle',
                    maxWidth: 96,
                    maxHeight: 16,
                    linebreaks: false
                }),
                new Anchor({
                    screenPosition: new Vector2(0, 0),
                    offset: new Vector2(0, 0),
                    onComplete: function () {
                        if (Anchor.isPortrait()) {
                            titleBar.visible = true;
                        } else {
                            titleBar.visible = false;

                        }
                    }
                })
            ]
        });

        var entity = new Entity({
            name: 'shopOverlay',
            components: [
                background,
                titleBar,
                drawer,
                anchorComponent,
                // timeText,
                behavior,
                new HideBelow({})
            ]
        });
        var currentSpeechBalloon;
        var talk = function (data) {
            var speechBalloon = currentSpeechBalloon;
            if (currentSpeechBalloon) {
                currentSpeechBalloon.removeSelf();
            }
            new Tween({
                from: 0,
                to: 1,
                in: 20,
                delay: 0,
                ease: 'elastic',
                decay: 5,
                oscillations: 1,
                onUpdate: function (v, t) {
                    speechBalloon.scale.x = v;
                    speechBalloon.scale.y = v;
                }
            });
            speechBalloon = new SpeechBalloon({
                z: settings.z || Globals.WorldSelectLayers.SHOP,
                float: true,
                position: speechBalloonPosition,
                text: data.text,
                updateWhenPaused: Bento.objects.isPaused(),
                typewriter: true,
                typeSpeed: 1,
                size: speechBalloonSize
            }).attach({
                name: 'removeAfter',
                startTween: false,
                currentTween: null,
                update: function () {
                    var self = this;
                    if (!this.currentTween && speechBalloon.timer > data.removeAfter) {
                        this.currentTween = new Tween({
                            from: 1,
                            to: 0,
                            in: 30,
                            onUpdate: function (v, t) {
                                speechBalloon.alpha = v;
                            },
                            onComplete: function () {
                                self.currentTween = null;
                                speechBalloon.removeSelf();
                                if (data.onComplete) {
                                    data.onComplete();
                                }
                            }
                        });
                    }
                },
                destroy: function () {
                    // remove the tween if needed (user can close the shop during the tween)
                    if (this.currentTween) {
                        this.currentTween.stop();
                    }
                }
            });
            Bento.objects.attach(speechBalloon);

            currentSpeechBalloon = speechBalloon;
        };
        // talk about time until refresh
        var tellRefresh = function () {
            var timeString;
            var dailyData;
            var timeLeft = 0;
            var hours = 0;
            var minutes = 0;
            var serverTime = TimeManager.getTime();

            // update timer
            dailyData = SaveManager.load('dailyShop');
            timeLeft = dailyData.time - serverTime;
            minutes = Math.floor(timeLeft / 1000 / 60) % 60;
            hours = Math.floor(timeLeft / 1000 / 60 / 60);

            // don't show negative numbers
            hours = Math.max(hours, 0);
            minutes = Math.max(minutes, 0);
            talk({
                text: Localization.getText('shopRefresh').replace('{HOURS}', hours).replace('{MINUTES}', minutes),
                removeAfter: 300,
                onComplete: function () {
                    tellAd();
                }
            });
        };
        // talk about ad for refresh
        var tellAd = function () {
            talk({
                text: Localization.getText((buttonAd.isActive() && buttonAd.parent) ? 'shopWatch' : 'shopGem'),
                removeAfter: 300,
                onComplete: function () {
                    tellRefresh();
                }
            });
        };
        var moveIn = function (onComplete) {
            var tutorialLoop = function () {
                // looping froggo saying welcome and stuff is free
                talk({
                    text: Localization.getText('shopWelcome'),
                    removeAfter: 60,
                    onComplete: function () {
                        talk({
                            text: Localization.getText('tutorial')['worldSelect3'][0],
                            removeAfter: 180,
                            onComplete: function () {
                                tutorialLoop();
                            }
                        });
                    }
                });
            };
            if (onComplete) {
                onComplete();
            }

            if (gotTutorialItems) {
                talk({
                    text: Localization.getText('shopWelcome'),
                    removeAfter: 120,
                    onComplete: function () {
                        tellRefresh();
                    }
                });
            } else {
                tutorialLoop();
            }

            new Tween({
                from: -32,
                to: 24,
                in: 25,
                delay: 25,
                ease: 'easeOutBack',
                onUpdate: function (v, t) {
                    buttonAd.position.x = v;
                    buttonGem.position.x = v;
                }
            });
        };

        var moveOut = function (onComplete) {
            currentSpeechBalloon.removeSelf();
            var duration = 25;
            var portraitMode = ResolutionManager.isPortrait();

            var offscreen = new Vector2(
                portraitMode ? viewport.width / 2 : viewport.width + viewport.width / 2,
                portraitMode ? viewport.height + viewport.height / 2 : viewport.height / 2
            );

            // move out shop
            new Tween({
                from: entity.position.clone(),
                to: offscreen,
                'in': duration,
                ease: Tween.EASEINBACK,
                onUpdate: function (v, t) {
                    entity.position = v;
                },
                onComplete: function () {
                    if (onComplete) {
                        onComplete();
                    }
                    // TEMP: quick code for shop music
                    Bento.audio.stopAllMusic();
                    if (Utils.isNativeAndroid()) {
                        // QUICKFIX: for some reason android sometimes skips the music
                        new Tween({
                            ignoreGameSpeed: true,
                            in: 3,
                            onComplete: function () {
                                Bento.audio.playMusic('bgm_worldmap');
                            }
                        });
                    } else {
                        Bento.audio.playMusic('bgm_worldmap');
                    }
                }
            }).attach(new OrientationLock({}));
            // quick fix for glitchy visuals
            entity.removeByName('hideBelow');


            // move out close button
            new Tween({
                from: buttonClose.position.x,
                to: buttonClose.position.x - 80,
                'in': duration,
                ease: Tween.EASEINBACK,
                onCreate: function () {
                    buttonClose.setActive(false);
                },
                onUpdate: function (v, t) {
                    buttonClose.position.x = v;
                }
            });

            new Tween({
                to: -32,
                from: 24,
                in: duration,
                delay: 5,
                ease: 'easeInBack',
                onUpdate: function (v, t) {
                    buttonAd.position.x = v;
                    buttonGem.position.x = v;
                }
            });

        };

        var container = new Entity({
            z: settings.z || Globals.WorldSelectLayers.SHOP,
            float: true,
            name: 'dailyShop',
            components: [
                new Modal({}),
                entity,
                buttonAd,
                buttonGem,
                // buttonClose,
                new TopBarSimple({})
            ]
        });
        var backgroundPortrait = new Entity({
            name: 'bgPortrait',
            components: [
                new Sprite({
                    imageName: 'gui/bg-brick-portrait',
                    originRelative: new Vector2(0.5, 0)
                }),
                new Anchor({
                    screenPosition: new Vector2(0.5, 0),
                    onComplete: function () {
                        if (Anchor.isPortrait()) {
                            backgroundPortrait.visible = true;
                            backgroundLandscape.visible = false;
                        } else {
                            backgroundPortrait.visible = false;
                            backgroundLandscape.visible = true;
                        }
                    }
                })
            ]
        });
        var backgroundLandscape = new Entity({
            name: 'bgLandscape',
            components: [
                new Sprite({
                    imageName: 'gui/bg-brick-landscape',
                })
            ]
        });
        if (Anchor.isWideScreen()) {
            backgroundLandscape.scale.x = Anchor.getWideScreenSize().width / backgroundLandscape.dimension.width;
            backgroundLandscape.transform.x -= Anchor.getOffset().x;
        }
        // don't show shop until mission 7
        if (unlockedLevels < 7) {
            container = new Entity({
                z: settings.z || Globals.WorldSelectLayers.SHOP,
                float: true,
                name: 'dailyShop',
                components: [
                    new Modal({}),
                    backgroundPortrait,
                    backgroundLandscape,
                    new ShadowText({
                        text: Localization.getText('shopClosed'),
                        font: 'font',
                        fontSize: 16,
                        fontColor: '#ffffff',
                        align: 'center',
                        textBaseline: 'middle',
                        maxWidth: 160,
                        linebreaks: true,
                        drawDebug: false,
                        components: [
                            new Anchor({
                                screenPosition: new Vector2(0.5, 0.5),
                                offset: new Vector2(0, 0)
                            })
                        ]
                    }),
                    new TopBarSimple({})
                ]
            });
        }

        if (!gotTutorialItems) {
            // dont show ad during tutorial
            buttonAd.removeSelf();
            buttonGem.removeSelf();
        }

        Analytics.trackView('dailyShop');

        // try to get constructor asap
        if (!IAPMenu) {
            bento.require(['gui/menus/iap'], function (Module) {
                IAPMenu = Module;
            });
        }

        return container;
    };
});