/**
 * UI for tower challenge start/end screen
 * @moduleName TowerMenu
 */
bento.define('gui/menus/towerchallenge', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/tween',
    'components/anchor',
    'globals',
    'components/modal',
    'components/fillscreen',
    'gui/iconbutton',
    'managers/character',
    'gui/shadowtext',
    'modules/localization',
    'components/backbutton',
    'gui/timertab',
    'managers/save',
    'managers/tower',
    'gui/buttons/startbutton',
    'modules/gamecenter',
    'modules/googleplay',
    'modules/googleplaylogin',
    'managers/time',
    'gui/menus/online',
    'gui/equipmentslots',
    'managers/inventory'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Anchor,
    Globals,
    Modal,
    FillScreen,
    IconButton,
    CharacterManager,
    ShadowText,
    Localization,
    BackButton,
    TimerTab,
    SaveManager,
    Tower,
    StartButton,
    GameCenter,
    GooglePlay,
    GooglePlayLogin,
    TimeManager,
    OnlineMenu,
    EquipmentSlots,
    InventoryManager
) {
    'use strict';
    return function (settings) {
        var viewport = Bento.getViewport();
        var position = settings.position;
        var t24 = (function () {
            // reset values upon opening this screen
            var today = Math.floor(TimeManager.getTime() / 1000 / 60 / 60 / 24);
            var tomorrow = today + 1;
            var tomorrowTime = tomorrow * 1000 * 60 * 60 * 24;
            var tower24 = SaveManager.load('tower24');

            // day changed
            if (tower24.day !== today) {
                tower24.day = today;
                tower24.time = tomorrowTime;
                SaveManager.save('tower24', tower24);

                // reset today's score
                SaveManager.save('towerHighestFloorToday', 0);
                SaveManager.save('towerHighestScoreToday', 0);

                // reset gachas
                SaveManager.save('towerGachas', {});
                SaveManager.save('towerGotKeys', 0);
            }
            // set seed
            Globals.towerInfo.seed = today;

            return tower24;
        })();
        /**
         * Background
         */
        var bgImage = new Sprite({
            imageName: 'gui2/towerchallenge/background',
            originRelative: new Vector2(0.5, 0.5),
        });
        var lastPos = new Entity({
            name: 'lastPos',
            components: [
                new Sprite({
                    imageName: 'gui2/towerchallenge/lastpos',
                    originRelative: new Vector2(0.5, 0.5)
                }),
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.5, 0.5),
                        offset: new Vector2(-32, -32),
                    },
                    landscape: {
                        screenPosition: new Vector2(0.5, 0.5),
                        offset: new Vector2(0, 0),
                    },
                    onComplete: function () {}
                })
            ]
        });
        var lastFloor = 0;
        var size = 200;
        var min = -256;
        var bgEntity = new Entity({
            z: 0,
            name: 'towerBackground',
            position: new Vector2(0, 0),
            components: [
                bgImage,
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.74, 0.5),
                        offset: new Vector2(0, 0),
                    },
                    landscape: {
                        screenPosition: new Vector2(0.74, 0.5),
                        offset: new Vector2(0, -64),
                    },
                    onComplete: function () {
                        bgEntity.scale.x = 550 / bgImage.frameWidth;
                        bgEntity.scale.y = bgEntity.scale.x;

                        // tween to show entire building
                        new Tween({
                            from: bgEntity.position.y - 64,
                            to: bgEntity.position.y,
                            in: 360,
                            delay: 0,
                            applyOnDelay: 0,
                            ease: 'squareroot',
                            decay: 1,
                            oscillations: 1,
                            onStart: function () {},
                            onUpdate: function (v, t) {
                                bgEntity.position.y = v;
                            },
                            onComplete: function () {

                            }
                        });

                        // tween to last pos
                        /*new Tween({
                            from: bgEntity.position.y + min,
                            to: bgEntity.position.y + min + size * lastFloor,
                            in: 120,
                            delay: 0,
                            applyOnDelay: 0,
                            ease: 'squareroot',
                            decay: 1,
                            oscillations: 1,
                            onStart: function () {},
                            onUpdate: function (v, t) {
                                bgEntity.position.y = v;
                            },
                            onComplete: function () {

                            }
                        });*/
                    }
                })
            ]
        });

        /**
         * HI-SCORE & TITLE
         */
        var margin = 10;
        var scorewidth = 14;
        var positionTop = 30;
        var hiscoreTextWidth = 80 - margin - margin / 2;
        var hiscoreAndTitleEntity = new Entity({
            name: 'hiscoreEntity',
            components: [
                new ShadowText({
                    text: Localization.getText('todaysrecord'),
                    font: 'font',
                    fontSize: 10,
                    align: 'right',
                    textBaseline: 'middle',
                    linebreaks: false,
                    fontColor: '#ffffff',
                    maxWidth: 60,
                    maxHeight: 28,
                    components: [
                        new Anchor({
                            offset: new Vector2(hiscoreTextWidth - scorewidth + margin, positionTop),
                            onComplete: function (parent) {
                                if (!Anchor.isPortrait()) {
                                    parent.position.y += margin;
                                }
                            }
                        })
                    ]
                }),

                new ShadowText({
                    text: Localization.getText('alltimehigh'),
                    font: 'font',
                    fontSize: 10,
                    align: 'right',
                    textBaseline: 'middle',
                    linebreaks: false,
                    fontColor: '#ffffff',
                    maxWidth: 60,
                    maxHeight: 28,
                    components: [
                        new Anchor({
                            offset: new Vector2(hiscoreTextWidth - scorewidth + margin, positionTop + 14),
                            onComplete: function (parent) {
                                if (!Anchor.isPortrait()) {
                                    parent.position.y += margin;
                                }
                            }
                        })
                    ]
                }),

                new ShadowText({
                    text: SaveManager.load('towerHighestScoreToday').toString(),
                    font: 'font',
                    fontSize: 14,
                    align: 'left',
                    textBaseline: 'middle',
                    linebreaks: false,
                    fontColor: '#fffd00',
                    maxWidth: scorewidth,
                    maxHeight: 28,
                    components: [
                        new Anchor({
                            offset: new Vector2(hiscoreTextWidth - scorewidth + margin, positionTop),
                            onComplete: function (parent) {
                                if (!Anchor.isPortrait()) {
                                    parent.position.y += margin;
                                }
                            }
                        })
                    ]
                }),

                new ShadowText({
                    text: SaveManager.load('towerHighestScoreEver').toString(),
                    font: 'font',
                    fontSize: 14,
                    align: 'left',
                    textBaseline: 'middle',
                    linebreaks: false,
                    fontColor: '#fffd00',
                    maxWidth: scorewidth,
                    maxHeight: 28,
                    components: [
                        new Anchor({
                            offset: new Vector2(hiscoreTextWidth - scorewidth + margin, positionTop + 14),
                            onComplete: function (parent) {
                                if (!Anchor.isPortrait()) {
                                    parent.position.y += margin;
                                }
                            }
                        })
                    ]
                }),

                new ShadowText({
                    text: Localization.getText('towerTitle'),
                    font: 'font',
                    fontSize: 48,
                    align: 'center',
                    textBaseline: 'middle',
                    linebreaks: false,
                    fontColor: '#ffffff',
                    maxWidth: hiscoreTextWidth,
                    maxHeight: 26,
                    components: [
                        new Anchor({
                            screenPosition: new Vector2(1, 0),
                            offset: new Vector2(-margin - hiscoreTextWidth / 2, positionTop + 4),
                            onComplete: function (parent) {}
                        })
                    ]
                }),

                new ShadowText({
                    text: Localization.getText('towerSubTitle'),
                    font: 'font',
                    fontSize: 48,
                    align: 'center',
                    textBaseline: 'middle',
                    linebreaks: false,
                    fontColor: '#ffffff',
                    maxWidth: hiscoreTextWidth,
                    maxHeight: 26,
                    components: [
                        new Anchor({
                            screenPosition: new Vector2(1, 0),
                            offset: new Vector2(-margin - hiscoreTextWidth / 2, positionTop + 28),
                            onComplete: function (parent) {}
                        })
                    ]
                })
            ]
        });

        /**
         * LEADERBOARDS
         */
        var leaderboardIcon = new Entity({
            name: 'icon',
            components: [
                new Entity({
                    name: 'iconLeaderboards',
                    position: new Vector2(0, -4),
                    components: [
                        new Sprite({
                            imageName: 'gui2/towerchallenge/leaderboard_icon',
                            originRelative: new Vector2(0.5, 0.5)
                        })
                    ]
                }),
                new ShadowText({
                    position: new Vector2(0, 8),
                    text: Localization.getText('leaderboards').toUpperCase(),
                    font: 'font',
                    fontSize: 12,
                    fontColor: '#ffffff',
                    align: 'center',
                    textBaseline: 'middle',
                    maxWidth: 35,
                    maxHeight: 12,
                    components: []
                })
            ]
        });
        var leaderboardButton = new IconButton({
            name: 'leaderboardButton',
            sfx: 'sfx_button',
            icon: leaderboardIcon,
            imageName: 'gui2/misc/button_small_secondary',
            frameCountX: 1,
            frameCountY: 3,
            onClick: function () {
                // open Leaderboards page
                // for now it simply opens GameCenter or GooglePlay
                if (Utils.isNativeIos()) {
                    GameCenter.show(function () {}, 'CgkI9NqdkoAGEAIQAQ');
                } else if (Utils.isNativeAndroid()) {
                    // log in first
                    if (GooglePlay.isLoggedIn()) {
                        GooglePlay.show(function () {});
                    } else {
                        GooglePlayLogin(function (hasLoggedIn) {
                            if (hasLoggedIn) {
                                // try showing achievements again
                                GooglePlay.show(function () {}, 'CgkI9NqdkoAGEAIQAQ');
                            }
                        });
                    }
                }
            },
            components: [
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0, 0),
                        offset: new Vector2(30, positionTop + 50),
                    },
                    landscape: {
                        screenPosition: new Vector2(0, 0.5),
                        offset: new Vector2(64 + 48, -44),
                    },
                    onComplete: function () {}
                })
            ]
        });

        /**
         * STAMP CARD
         */
        var rewardTitle = new Entity({
            name: 'titleBar',
            components: [
                new Sprite({
                    imageName: 'gui2/misc/title_bar',
                    originRelative: new Vector2(0.5, 0.5),
                }),
                new Text({
                    position: new Vector2(0, -16),
                    text: Localization.getText('todaysrewards'),
                    font: 'font',
                    fontSize: 16,
                    fontColor: '#ffdf00',
                    align: 'center',
                    textBaseline: 'middle',
                    maxWidth: 96,
                    maxHeight: 16,
                    linebreaks: false
                }),
                new TimerTab({
                    saveId: 'tower24',
                    position: new Vector2(50, -23)
                })
            ]
        });
        var stampCardBG = new Sprite({
            imageName: 'gui2/towerchallenge/loyaltycard',
            originRelative: new Vector2(0.5, 0),
        });
        var rewardsCollected = Tower.countCollectedGachas();
        var rewardFloors = Globals.towerGachaFloors;
        var stampCellWidth = 42;
        var stampCellHeight = 34;
        var stampCelStartX = 0 - stampCardBG.frameWidth / 2 + 1 + stampCellWidth / 2;
        var stampCelStartY = stampCardBG.frameHeight / 2 - stampCellHeight + 8;
        var rewardFloorsText = [];
        var rewardFloorsStamps = [];
        //DRAW EACH CEL
        for (var i = rewardFloors.length - 1; i >= 0; i--) {
            //console.log(i + ": " + rewardFloors[i]);
            //white if not achieved yet
            var floorTextColor = (rewardsCollected >= i + 1) ? "#000000" : "#FFFFFF";
            rewardFloorsText.push(
                new ShadowText({
                    position: new Vector2(stampCelStartX + stampCellWidth * (i % 4), stampCelStartY + stampCellHeight * Math.floor((i / 4 % 4))),
                    text: Localization.getText('floor'),
                    font: 'font',
                    fontSize: 10,
                    fontColor: floorTextColor,
                    align: 'center',
                    textBaseline: 'middle',
                    maxWidth: stampCellWidth,
                    maxHeight: 16,
                    linebreaks: false
                })
            );
            rewardFloorsText.push(
                new ShadowText({
                    position: new Vector2(stampCelStartX + stampCellWidth * (i % 4), stampCelStartY + 12 + stampCellHeight * Math.floor((i / 4 % 4))),
                    text: rewardFloors[i],
                    font: 'font',
                    fontSize: 23,
                    fontColor: floorTextColor,
                    align: 'center',
                    textBaseline: 'middle',
                    maxWidth: stampCellWidth,
                    maxHeight: 20,
                    linebreaks: false
                })
            );
            if (rewardsCollected >= i + 1) {
                //this reward is collected, add a stamp
                rewardFloorsStamps.push(
                    new Entity({
                        z: 0,
                        name: 'stamp' + i,
                        alpha: (rewardsCollected >= 8) ? 0.7 : 1,
                        position: new Vector2(stampCelStartX + stampCellWidth * (i % 4), stampCelStartY + 8 + stampCellHeight * Math.floor((i / 4 % 4))),
                        components: [
                            new Sprite({
                                imageName: 'gui2/towerchallenge/stamps',
                                originRelative: new Vector2(0.5, 0.5),
                                frameCountX: 4,
                                frameCountY: 2,
                                padding: 0,
                                animations: {
                                    "default": {
                                        frames: [i]
                                    }
                                }
                            })
                        ]
                    })
                );
            }
        }
        //all rewards collected - add COMPLETE text
        if (rewardsCollected >= 8) {
            rewardFloorsStamps.push(
                new Entity({
                    z: 0,
                    position: new Vector2(0, 32),
                    name: 'allComplete',
                    rotation: -0.2,
                    components: [
                        new ShadowText({
                            text: Localization.getText('rewardsComplete'),
                            font: 'font',
                            fontSize: 40,
                            fontColor: "#ff4766",
                            align: 'center',
                            textBaseline: 'middle',
                            maxWidth: stampCardBG.frameWidth,
                            maxHeight: 40,
                            linebreaks: false,
                            shadowColor: '#750014',
                            shadow: true,
                            shadowOffset: new Vector2(0, 3)
                        })
                    ]
                })
            );
        }

        var stampCardContents = new Entity({
            z: 0,
            name: 'stampCardContents',
            position: new Vector2(0, 0),
            components: rewardFloorsText.concat(rewardFloorsStamps)
        });
        var stampCardEntity = new Entity({
            z: 0,
            name: 'stampCard',
            position: new Vector2(0, 0),
            components: [
                rewardTitle,
                stampCardBG,
                stampCardContents,
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.5, 1),
                        offset: new Vector2(0, -140),
                    },
                    landscape: {
                        screenPosition: new Vector2(0.5, 1),
                        offset: new Vector2(-64, -80),
                    },
                    onComplete: function () {}
                })
            ]
        });

        /**
         * PLAY
         */
        var energyCost = 0;
        var costText = new Text({
            position: new Vector2(8, 6),
            text: energyCost.toString(),
            font: 'font',
            fontSize: 12,
            fontColor: '#00baff',
            align: 'left',
            textBaseline: 'middle',
            lineWidth: 1.5,
            strokeStyle: '#4f3673',
            pixelStroke: true,
        });
        var costIcon = new Entity({
            name: 'costIcon',
            position: new Vector2(-6 - 24, 0),
            components: [
                new Sprite({
                    imageName: 'gui2/misc/energy',
                    originRelative: new Vector2(0, 0.5)
                }),
                costText
            ]
        });
        var playText = new ShadowText({
            position: new Vector2(8, 0),
            originRelative: new Vector2(0, 0.5),
            text: Localization.getText('play'),
            font: 'font',
            fontSize: 22,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            strokeStyle: '#000000',
            maxWidth: 44,
            maxHeight: 22
        });
        var container = new Entity({
            name: 'icon',
            components: [
                costIcon,
                playText
            ]
        });
        var buttonPlay = new IconButton({
            name: 'startButton',
            sfx: 'sfx_button',
            icon: container,
            iconAlpha: 0.5,
            imageName: 'gui2/misc/button_medium_primary',
            frameCountX: 1,
            frameCountY: 3,
            onClick: function () {
                //start tower challenge
                var startButton = new StartButton({
                    isTower: true,
                    mission: 'tower',
                    chosenDifficulty: 0,
                    difficulty: 0,
                    cost: 0,
                    unlocks: [],
                    rewards: []
                });
                startButton.start();
            },
            components: [
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.5, 1),
                        offset: new Vector2(0, -44),
                    },
                    landscape: {
                        screenPosition: new Vector2(0.5, 1),
                        offset: new Vector2(68, -44),
                    },
                    onComplete: function () {

                    }
                })
            ]
        });
        /**
         * Close
         */
        var buttonClose = new IconButton({
            name: 'buttonClose',
            sfx: 'sfx_button',
            imageName: 'gui2/misc/button_smaller',
            iconName: 'gui2/misc/button_icon_back_small',
            frameCountX: 1,
            frameCountY: 3,
            onClick: function () {
                entity.removeSelf();
                if (settings.onComplete) {
                    settings.onComplete();
                }
            },
            components: [
                new BackButton({
                    removeSelf: true,
                    onPressed: function (data) {
                        data.entity.doCallback();
                    }
                }),
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.5, 1),
                        offset: new Vector2(-64, -60 + 16)
                    },
                    landscape: {
                        screenPosition: new Vector2(0, 0),
                        offset: new Vector2(20, 20)
                    }
                })
            ]
        });
        /**
         * CO-OP
         */
        var textOnline = new ShadowText({
            position: new Vector2(0, 8),
            text: Localization.getText('coop').toUpperCase(),
            font: 'font',
            fontSize: 12,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            maxWidth: 40,
            maxHeight: 12,
            components: []
        });
        var iconWifi = new Entity({
            name: 'iconWifi',
            position: new Vector2(0, -4),
            components: [
                new Sprite({
                    imageName: 'gui/wifi-icon',
                    originRelative: new Vector2(0.5, 0.5)
                })
            ]
        });
        var iconInfo = new Entity({
            name: 'icon',
            components: [
                textOnline,
                iconWifi
            ]
        });
        var buttonOnline = new IconButton({
            name: 'buttonOnline',
            sfx: 'sfx_button',
            icon: iconInfo,
            imageName: 'gui2/misc/button_small_secondary',
            frameCountX: 1,
            frameCountY: 3,
            onClick: function () {
                // start tower challenge in multiplayer mode
                var onlineMenu = new OnlineMenu({
                    mission: 'tower',
                    cost: 0,
                    unlocks: [],
                    enemyLevel: 1,
                    rewards: {
                        "reward1": "nothing/nothing",
                        "amount1": 0,
                        "chance1": 0,
                        "reward2": "nothing/nothing",
                        "amount2": 0,
                        "chance2": 0,
                        "reward3": "nothing/nothing",
                        "amount3": 0,
                        "chance3": 0,
                        "rewardOnline": "nothing/nothing",
                        "amountOnline": 0
                    }
                });
                Bento.objects.attach(onlineMenu);
            },
            components: [
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.5, 1),
                        offset: new Vector2(64, -44),
                    },
                    landscape: {
                        screenPosition: new Vector2(0.5, 1),
                        offset: new Vector2(68 + 64, -44),
                    },
                    onComplete: function () {}
                })
            ]
        });


        var autoSelectItems = function () {
            var consumables = new EquipmentSlots({
                iconImage: 'gui/startlevel/inventory',
                type: 'consumable',
                slotSize: 'new',
                // hideSlot2: true,
                position: new Vector2(0, 4),
            });
            var itemsInInventory;
            var slot1 = consumables.getSlot(0);
            var slot2 = consumables.getSlot(1);
            var randomItem;

            itemsInInventory = InventoryManager.getAllItems(['consumables', 'weapons']);

            // fill slot 1
            if (!slot1 && itemsInInventory.length > 0) {
                randomItem = itemsInInventory[Utils.getRandom(itemsInInventory.length)];
                consumables.setSlot(0, randomItem.key);
            }

            // refresh inventory
            itemsInInventory = InventoryManager.getAllItems(['consumables', 'weapons']);

            // fill slot 2
            if (!slot2 && itemsInInventory.length > 0) {
                randomItem = itemsInInventory[Utils.getRandom(itemsInInventory.length)];
                consumables.setSlot(1, randomItem.key);
            }

        };
        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {
                //hide bottom tab backbutton on this screen
                Bento.objects.get('buttonBack', function (buttonBack) {
                    buttonBack.alpha = 0;
                });

                if (SaveManager.load('autoSelect')) {
                    autoSelectItems();
                }
            },
            destroy: function (data) {
                Bento.objects.get('buttonBack', function (buttonBack) {
                    buttonBack.alpha = 1;
                });
            },
            update: function (data) {},
            draw: function (data) {}
        };

        var entity = new Entity({
            z: Globals.WorldSelectLayers.TOWERCHALLENGE,
            name: 'Tower Challenge',
            position: position,
            float: true,
            components: [
                new Modal({}),
                new FillScreen({
                    color: [39 / 255, 26 / 255, 45 / 255, 1]
                }),
                bgEntity,
                // lastPos,
                hiscoreAndTitleEntity,
                leaderboardButton,
                stampCardEntity,
                buttonPlay,
                buttonClose,
                buttonOnline,
                behavior
            ]
        });
        return entity;
    };
});