/**
 * reward GUI entity
 * @moduleName Reward
 */
bento.define('gui/reward', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/entity',
    'gui/shadowtext',
    'components/darken',
    'gui/iteminfo',
    'bento/components/clickable',
    'managers/time',
    'managers/save',
    'modules/localization',
    'bento/gui/clickbutton',
    'globals',
    'managers/currency',
    'gui/menus/iap',
    'modules/analytics',
    'components/blink',
    'gui/sparkleburst',
    'gui/iconbutton',
    'managers/reward',
    'managers/character',
    'gui/energyforgempopup',
    'gui/itemget',
    'managers/receiveitem',
    'utils',
    'gui/senseipopup',
    'bento/eventsystem',
    'components/eventlistener',
    'gui/taphand',
    'gui/highlight',
    'gui/coinreward',
    'modules/text'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Entity,
    ShadowText,
    Darken,
    ItemInfo,
    Clickable,
    TimeManager,
    SaveManager,
    Localization,
    ClickButton,
    Globals,
    CurrencyManager,
    IAPMenu,
    Analytics,
    Blink,
    SparkleBurst,
    IconButton,
    RewardManager,
    CharacterManager,
    EnergyForGemPopup,
    ItemGet,
    ReceiveItem,
    Utils,
    SenseiPopup,
    EventSystem,
    EventListener,
    TapHand,
    Highlight,
    CoinReward,
    Text
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
            obtained: Boolean // found in level
        }*/
        if (settings.item === 'nothing/nothing') {
            return new Entity({});
        }
        var itemJson = Bento.assets.getJson(settings.item);
        var itemSprite = new Sprite({
            spriteSheet: itemJson.inventory.image
        });
        var overlay = new Darken({
            alpha: 0.7
        });
        var index = settings.index;
        var mission = settings.mission;
        var item = settings.item;
        var amount = settings.amount;
        var titleText = settings.titleText;
        var hideButton = settings.hideButton || false; // should show refresh button
        var hideText = settings.hideText || false;
        var canCollect = function () {
            var time = TimeManager.getTime();
            var refreshTime = SaveManager.load('rewardCooldown-' + mission);
            return time >= refreshTime;
        };
        var canQuickLoot = function () {
            // quickloot can be done if 3 stars were achieved for the misison
            // note: check for reward cooldown separately
            var missionStars = SaveManager.load('missionStars-' + mission);
            return missionStars === 3;
        };
        // show the darkened sprite and timer + refresh button or not
        var showOverlay = function () {
            var hasCollected = !canCollect();
            var quickLoot = !hasCollected && canQuickLoot();

            overlay.visible = hasCollected;
            checkMark.visible = hasCollected;

            if (timeText) {
                timeText.visible = hasCollected;
            }
            if (buttonRefresh) {
                buttonRefresh.visible = hasCollected || quickLoot;
                buttonRefresh.setActive(hasCollected || quickLoot);
            }

            if (hasCollected && !hideButton) {
                // move left
                entity.position.x = -32 * hasCollected;
                if (buttonRefresh && buttonRefresh.name !== 'refresh') {
                    makeRefreshButton();
                }
                if (titleText) {
                    titleText.setText(Localization.getText('refreshReward'));
                }
            } else if (quickLoot && !hideButton) {
                // move left
                entity.position.x = -32;
                if (buttonRefresh && buttonRefresh.name !== 'quickLoot') {
                    makeQuickLootButton();
                }
                if (titleText) {
                    titleText.setText(Localization.getText('quickLoot'));
                }

            } else {
                entity.position.x = 0;
                if (titleText) {
                    titleText.setText(Localization.getText('reward'));
                }
            }

            if (settings.amount) {
                // show amount
                if (checkMark.visible || parseInt(settings.amount) < 2) {
                    text.visible = false;
                } else {
                    text.visible = true;
                }
            } else {
                text.visible = false;
            }
        };
        // update time and check if rewards needs to be uncollected
        var updateTimer = function () {
            var timeLeft;
            var minutes;
            var hours;
            var time = TimeManager.getTime();
            var refreshTime = SaveManager.load('rewardCooldown-' + mission);

            // set time text
            timeLeft = Math.abs(refreshTime - time);
            minutes = Math.floor(timeLeft / 1000 / 60) % 60;
            hours = Math.floor(timeLeft / 1000 / 60 / 60);

            if (!canCollect()) {
                if (timeText) {
                    timeText.setText(Localization.getText('timeHM').replace('{HOURS}', hours).replace('{MINUTES}', minutes));
                }
            }

            // update overlay
            showOverlay();

        };
        var checkMark = new Entity({
            name: 'checkMark',
            position: new Vector2(12, 8),
            components: [
                new Sprite({
                    imageName: 'gui/levelend/clock',
                    originRelative: new Vector2(0.5, 0.5)
                })
            ]
        });
        var timeText;
        var buttonRefresh;
        var text = settings.amount ? new ShadowText({
            z: 0,
            position: new Vector2(5, 6),
            text: settings.amount.toString(),
            font: 'font',
            fontSize: 16,
            fontColor: '#ffffff',
            align: 'middle',
            textBaseline: 'middle',
            maxWidth: 17,
            maxHeight: 16
        }) : new Entity({});

        var behavior = {
            name: 'rewardBehavior',
            start: function () {
                itemSprite.setOriginRelative(new Vector2(0.5, 0.5));

                // darken if collected and player needs to wait until it becomes "uncollected"
                showOverlay();

                // tutorial for first time
                if (!hideButton && canCollect() && canQuickLoot() && SaveManager.load('tutorialQuickLoot') === 0) {
                    // small delay to account for tween
                    Utils.timeout(30, function () {
                        SaveManager.save('tutorialQuickLoot', 1);
                        EventSystem.fire('tutorialQuickLoot');
                        Bento.objects.attach(new SenseiPopup({
                            darken: false,
                            modal: true,
                            onNoMoreText: function () {
                                this.dismiss();
                            },
                            onComplete: function () {
                                highlight.tweenOut();
                                // add taphand to button
                                if (buttonRefresh) {
                                    buttonRefresh.attach(new TapHand({
                                        position: new Vector2(8, -8),
                                        rotation: Math.PI / 4,
                                        components: [
                                            new EventListener({
                                                name: 'quickLooted',
                                                onEvent: function (data) {
                                                    buttonRefresh.removeByName('tapHand');
                                                }
                                            })
                                        ]
                                    }));
                                }
                            },
                            text: Localization.getText('tutorial')['quickLoot']
                        }));
                        var highlight = new Highlight({
                            z: Globals.WorldSelectLayers.SENSEI - 1,
                            position: buttonRefresh.getWorldPosition(),
                            updateWhenPaused: Bento.objects.isPaused() + 2
                        });
                        Bento.objects.attach(highlight);
                    });
                }
            },
            update: function (data) {
                itemSprite.visible = !overlay.visible;

                // check every minute if reward becomes uncollected
                if (data.entity.timer % (60 * 60) === 0) {
                    updateTimer();
                }
            }
        };

        var clickable = new Clickable({
            onClick: function (data) {
                var itemInfo = new ItemInfo({
                    item: item
                });
                Bento.objects.attach(itemInfo);
                Bento.audio.stopSound('sfx_button');
                Bento.audio.playSound('sfx_button');
            }
        });

        var entity = new Entity({
            z: 0,
            name: 'reward' + (index + 1),
            position: settings.position,
            updateWhenPaused: 0,
            boundingBox: new Rectangle(-16, -16, 32, 32),
            components: [
                behavior,
                itemSprite,
                overlay,
                text,
                checkMark,
                clickable
            ]
        }).extend({
            updateVisual: function () {
                showOverlay();
            }
        });
        /**
         * Spawns the refresh button
         */
        var makeRefreshButton = function () {
            var gemsNeeded = Globals.gemToRefreshReward * (1 + Math.floor(mission / 10));
            var labelText = new ShadowText({
                position: new Vector2(10, -1),
                text: gemsNeeded.toString(),
                font: 'font',
                fontSize: 16,
                fontColor: '#ffffff',
                align: 'center',
                textBaseline: 'middle',
                maxWidth: 24,
                maxHeight: 16
            });
            var buttonIcon = new Entity({
                name: 'icon',
                components: [
                    new Entity({
                        name: 'gem',
                        position: new Vector2(-8, 0),
                        components: [
                            new Sprite({
                                imageName: 'gui/gem',
                                originRelative: new Vector2(0.5, 0.5)
                            })
                        ]
                    }),
                    labelText
                ]
            });
            // remove previous if necessary
            if (buttonRefresh && buttonRefresh.parent) {
                buttonRefresh.removeSelf();
                buttonRefresh = null;
            }
            buttonRefresh = new IconButton({
                name: 'refresh',
                sfx: 'sfx_button',
                icon: buttonIcon,
                imageName: 'gui2/misc/buttonmini_large_secondary',
                frameCountX: 1,
                frameCountY: 3,
                position: new Vector2(48, 0),
                onClick: function () {
                    if (CurrencyManager.canSpendGems(gemsNeeded)) {
                        Bento.audio.stopSound('sfx_buy');
                        Bento.audio.playSound('sfx_buy');
                        CurrencyManager.spendGems(gemsNeeded, 'refreshRewards');
                        SaveManager.save('rewardCooldown-' + mission, 0);
                        updateTimer();
                        EventSystem.fire('updateReward-' + mission);

                        entity.attach(new Blink({}));
                        entity.attach(new SparkleBurst({}));

                        Analytics.event('Economy', 'refreshRewards', 1, mission);
                        SaveManager.saveToCloud();
                    } else {
                        // not enough gems???? go to shop
                        var iapMenu = new IAPMenu({
                            z: settings.z + 1,
                            scrollTo: 'gems'
                        });
                        Bento.objects.attach(iapMenu);
                    }
                }
            });
            entity.attach(buttonRefresh);
            itemSprite.setOriginRelative(new Vector2(0.5, 0.5));
        };
        var makeQuickLootButton = function () {
            var energy = RewardManager.getEnergyCost(mission);
            var labelText = new ShadowText({
                position: new Vector2(10, -1),
                text: energy.toString(),
                font: 'font',
                fontSize: 16,
                fontColor: '#ffffff',
                align: 'center',
                textBaseline: 'middle',
                maxWidth: 24,
                maxHeight: 16
            });
            var buttonIcon = new Entity({
                name: 'icon',
                components: [
                    new Entity({
                        name: 'energy',
                        position: new Vector2(-8, 0),
                        components: [
                            new Sprite({
                                imageName: 'gui2/misc/energy',
                                originRelative: new Vector2(0.5, 0.5)
                            })
                        ]
                    }),
                    labelText
                ]
            });
            var coinText = new Text({
                text: '+' + Globals.getExtraLootCoins(mission),
                font: 'font',
                fontSize: 10,
                fontColor: '#ffffff',
                align: 'center',
                textBaseline: 'middle',
                components: []
            });
            var coinIcon = new Entity({
                name: 'coin',
                position: new Vector2(coinText.dimension.width / 2, 1),
                components: [
                    new Sprite({
                        imageName: 'gui2/iap/symbol_coin',
                        originRelative: new Vector2(0, 0.5)
                    })
                ]
            });
            var coinAmountEntity = new Entity({
                name: 'coinAmountEntity',
                position: new Vector2(-4, 12),
                components: [
                    coinText,
                    coinIcon
                ]
            });
            // remove previous if necessary
            if (buttonRefresh && buttonRefresh.parent) {
                buttonRefresh.removeSelf();
                buttonRefresh = null;
            }
            buttonRefresh = new IconButton({
                name: 'quickLoot',
                sfx: 'sfx_button',
                icon: buttonIcon,
                imageName: 'gui2/misc/buttonmini_large_primary',
                frameCountX: 1,
                frameCountY: 3,
                position: new Vector2(48, 0),
                onClick: function () {
                    var dialog;
                    var itemGet;
                    if (CharacterManager.hasEnergy(energy)) {
                        CharacterManager.subtractEnergy(energy);
                        // get item
                        itemGet = new ItemGet({
                            z: Globals.WorldSelectLayers.LEVELDETAILS + 1,
                            item: item,
                            amount: amount,
                            onComplete: function () {
                                // also give coins
                                var coins = Globals.getExtraLootCoins(mission);
                                ReceiveItem({
                                    key: 'currency/coin',
                                    amount: coins
                                });

                                itemGet = new CoinReward({
                                    z: Globals.WorldSelectLayers.LEVELDETAILS + 1,
                                    coins: coins
                                });
                                Bento.objects.attach(itemGet);
                            }
                        });
                        Bento.objects.attach(itemGet);
                        ReceiveItem({
                            key: item,
                            amount: amount
                        });
                        // schedule ahead 6 hours
                        SaveManager.save('rewardCooldown-' + mission, TimeManager.getTime() + 1000 * 60 * 60 * 6);
                        updateTimer();
                        EventSystem.fire('updateReward-' + mission);
                        EventSystem.fire('quickLooted');
                        coinAmountEntity.removeSelf();
                        Analytics.event('QuickLoot', item, amount);
                        if (CharacterManager.getEnergy('boy') < 3) {
                            Analytics.event('Energy', 'low');
                        }

                    } else {
                        // restore energy? (must use gems)
                        dialog = new EnergyForGemPopup({
                            forceGem: true,
                            onBoughtEnergy: function (hasBought) {
                                if (hasBought) {
                                    // try again
                                    buttonRefresh.doCallback();
                                }
                            },
                            onWatchedAd: function () {
                                // try again
                                buttonRefresh.doCallback();
                            }
                        });
                        Bento.objects.attach(dialog);
                    }
                }
            });
            entity.attach(buttonRefresh);

            // move item up
            itemSprite.origin.y += 4;
            // show coin amount
            entity.attach(coinAmountEntity);
        };
        if (!hideButton) {
            makeRefreshButton();
        }
        if (hideText) {
            entity.remove(text);
        } else {
            timeText = new ShadowText({
                position: new Vector2(0, -8),
                text: '',
                font: 'font',
                fontSize: 10,
                fontColor: '#ffffff',
                align: 'center',
                textBaseline: 'middle',
                maxWidth: 60,
                maxHeight: 20
            });
            entity.attach(timeText);
        }

        return entity;
    };
});