/**
 * Shop button
 * @moduleName ShopButton
 */
bento.define('gui/shop/button', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/entity',
    'bento/utils',
    'bento/gui/clickbutton',
    'globals',
    'bento/tween',
    'components/anchor',
    'components/debug',
    'modules/text',
    'managers/inventory',
    'bento/eventsystem',
    'gui/shadowtext',
    'gui/dialog',
    'gui/iteminfo',
    'modules/localization',
    'managers/currency',
    'managers/save',
    'managers/stats',
    'gui/menus/iap',
    'gui/iap/buycoinspopup',
    'gui/itemget',
    'gui/levelupnotification',
    'gui/tokenbar',
    'gui/taphand',
    'components/eventlistener',
    'modules/analytics',
    'gui/tier',
    'gui/iconbutton',
    'managers/allies',
    'managers/character',
    'managers/receiveitem',
    'managers/time'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Entity,
    Utils,
    ClickButton,
    Globals,
    Tween,
    Anchor,
    Debug,
    Text,
    InventoryManager,
    EventSystem,
    ShadowText,
    Dialog,
    ItemInfo,
    Localization,
    CurrencyManager,
    SaveManager,
    StatsManager,
    IAPMenu,
    BuyCoinsPopup,
    ItemGet,
    LevelUpNotification,
    TokenBar,
    TapHand,
    EventListener,
    Analytics,
    TierStars,
    IconButton,
    AlliesManager,
    CharacterManager,
    ReceiveItem,
    TimeManager
) {
    'use strict';
    var isShowingTapHand = true;
    var isShopEvent = function () {
        var today = Math.floor(TimeManager.getTime() / 1000 / 60 / 60 / 24);
        if (today >= 17508 && today <= 17510) {
            // this part can be patched
            return true;
        }
        return false;
    };
    var ShopButton = function (settings) {
        var viewport = Bento.getViewport();
        var screenMid = new Vector2(viewport.width / 2, viewport.height / 2);

        var width = settings.width || 1;
        var height = settings.height || 1;
        var opts = settings.opts;
        var index = settings.index;
        var pricePos = new Vector2(7, -1);
        var itemPos = new Vector2(-2, -25);
        var cost = opts.item.price;
        var tier = opts.item.tier;
        var maxItems = Globals.maxShopItems;
        var category = opts.item.key.split('/')[0];
        var itemKey = opts.item.key.split('/')[1];
        var tokenType = itemKey.split('-')[0];
        var tokenCharacter = itemKey.split('-')[1];

        var item = new Entity({
            name: 'item',
            position: new Vector2(24, 0),
            // position: itemPos.clone(),
            boundingBox: new Rectangle(-20, -20, 128, 40),
            components: [
                opts.item.sprite
            ]
        });

        var price = new ShadowText({
            text: '',
            position: new Vector2(pricePos.x, pricePos.y),
            font: 'font',
            fontSize: 16,
            align: 'center',
            textBaseline: 'middle',
            maxWidth: 26,
            maxHeight: 16,
            linebreaks: false
        });

        var name = new ShadowText({
            text: opts.item.name,
            position: new Vector2(52, -14),
            font: 'font',
            fontSize: 14,
            align: 'left',
            textBaseline: 'middle',
            fontColor: '#fffd00',
            maxWidth: 104,
            maxHeight: 14
        });

        var tierStars;


        var ownedCount = new Text({
            position: new Vector2(12, 8),
            text: '',
            font: 'font',
            fontSize: 16,
            fontColor: '#271a2d',
            align: 'center',
            textBaseline: 'top',
            maxWidth: 32,
            maxHeight: 16,
            linebreaks: false
        });
        var owned = new Text({
            position: new Vector2(52, -6),
            text: '',
            font: 'font',
            fontSize: 10,
            fontColor: '#271a2d',
            align: 'left',
            textBaseline: 'top',
            maxWidth: 64,
            maxHeight: 10,
            linebreaks: false,
            components: [
                ownedCount
            ]
        });
        var tokenBar;

        var buttonInfo = new IconButton({
            name: 'buttonInfo',
            sfx: 'sfx_button',
            imageName: 'gui2/misc/buttonmini_small_secondary',
            iconName: 'gui2/misc/button_icon_info',
            frameCountX: 1,
            frameCountY: 3,
            // position: new Vector2(100, 8),
            position: new Vector2(24, 0),
            onClick: function () {
                Bento.objects.attach(new ItemInfo({
                    item: opts.item.key
                }));
            }
        });
        buttonInfo.visible = false;
        var categoryIcon = new Entity({
            name: 'categoryIcon',
            position: new Vector2(140, 19 - 18),
            alpha: 0.8,
            components: [

            ]
        });
        var currency = opts.item.currency || 'coin';

        var button = new IconButton({
                name: 'buttonShop',
                sfx: 'sfx_button',
                position: new Vector2(146, 8),
                imageName: 'gui2/misc/buttonmini_large_primary',
                iconName: 'gui2/misc/button_icon_' + currency,
                offset: new Vector2(-14, 0),
                frameCountX: 1,
                frameCountY: 3,
                animations: {
                    up: {
                        frames: [0]
                    },
                    down: {
                        frames: [1]
                    },
                    inactive: {
                        frames: [2]
                    }
                },
                onClick: function () {
                    var addItem = function () {
                        var dailyData = SaveManager.load('dailyShop');
                        dailyData.bought[index] += 1;
                        SaveManager.save('dailyShop', dailyData);
                        ReceiveItem({
                            key: opts.item.key,
                            amount: 1
                        });
                        updateItem();

                        if (settings.onBought) {
                            settings.onBought();
                        }
                        Bento.audio.stopSound('sfx_buy');
                        Bento.audio.playSound('sfx_buy');

                        var itemGet = new ItemGet({
                            z: Globals.WorldSelectLayers.BUYCOINSPOPUP,
                            item: opts.item.key
                        });
                        Bento.objects.attach(itemGet);

                        if (SaveManager.load('tutorialWorldSelect') > 0) {
                            Analytics.event('DailyShop', opts.item.key);
                        }
                    };
                    if (currency === 'coin') {
                        if (CurrencyManager.canSpend(cost)) {
                            CurrencyManager.spendCoins(cost, 'dailyShop-' + opts.item.key);
                            addItem();
                        } else {
                            var buyCoins = new BuyCoinsPopup({
                                z: Globals.WorldSelectLayers.BUYCOINSPOPUP,
                                // z: settings.z + 1 || Globals.WorldSelectLayers.SHOP + 1,
                                coinsToBuy: cost - CurrencyManager.getCoins(),
                                buttonToClick: button
                            });
                            Bento.objects.attach(buyCoins);
                        }
                    } else if (currency === 'gem') {
                        if (CurrencyManager.canSpendGems(cost)) {
                            CurrencyManager.spendGems(cost, 'dailyShop-' + opts.item.key);
                            addItem();
                            SaveManager.saveToCloud();
                        } else {
                            var iapMenu = new IAPMenu({
                                // z: settings.z + 1 || Globals.WorldSelectLayers.SHOP + 1,
                                scrollTo: 'gems'
                            });
                            Bento.objects.attach(iapMenu);
                        }
                    }

                    // remove taphand if any
                    if (tapHand && tapHand.visible) {
                        button.removeByName('tapHand');
                        button.removeByName('eventListener');
                        isShowingTapHand = false;
                        EventSystem.fire('showTapHandShop');
                    }
                },
                onButtonDown: function () {
                    price.position.y = pricePos.y + 1;
                },
                onButtonUp: function () {
                    price.position.y = pricePos.y;
                }
            })
            .attach(price);

        var eventListener = new EventListener({
            name: 'showTapHandShop',
            onEvent: function () {
                if (!isShowingTapHand && tapHand && tapHand.parent && tapHand.parent.isActive()) {
                    isShowingTapHand = true;
                    tapHand.visible = true;
                }
            }
        });
        var tapHand = new TapHand({
            rotation: -Math.PI / 8
        });
        var showTapHand = function () {
            var dailyData = SaveManager.load('dailyShop');
            var bought = dailyData.bought[index];

            button.attach(tapHand);
            if (bought < Globals.maxShopItems) {
                tapHand.visible = isShowingTapHand;
                isShowingTapHand = false;
            } else {
                tapHand.visible = false;
            }
            button.attach(eventListener);
        };

        var frame = new Entity({
            name: 'frame',
            components: [
                new Sprite({
                    originRelative: new Vector2(0, 0.5),
                    imageName: 'gui2/shop/item_panel'
                }),
                item,
                // categoryIcon,
                name,
                owned,
                button
            ]
        }).attach(
            new Anchor({
                portrait: {
                    offset: opts.offset.portrait
                },
                landscape: {
                    offset: opts.offset.landscape
                },
                onComplete: function (parent, isPortrait) {
                    if (isPortrait) {
                        // center align
                        frame.position.x = -frame.dimension.width / 2;
                    }
                }
            })
        );
        var updateItem = function () {
            var dailyData = SaveManager.load('dailyShop');
            var bought = dailyData.bought[index];
            var basePrice = opts.item.price;
            var tokens;
            var level = StatsManager.getLevel(tokenCharacter);
            var upgradeCost = Globals.getTokenCost(level);

            // set owned
            if (category === 'upgrades') {
                // check tokens from currency manager
                tokens = CurrencyManager.getToken(tokenCharacter, tokenType);
                // owned.setText(Localization.getText('ownedX').replace('{NUMBER}', tokens + '/' + upgradeCost));
                tokenBar.setCharacter(tokenCharacter);

                // set price, based on how many you bought
                // increase 75 for every item bought
                if (!opts.item.tutorial) {
                    basePrice = basePrice + 75 * bought;
                }

            } else if (category === 'playertokens' || category === 'enemytokens' ) {
                tokens = InventoryManager.getTokens(opts.item.key);
                tokenBar.setCharacter(itemKey);
                // if it's not unlocked yet, the price is more expensive
                if (category === 'enemytokens' && !AlliesManager.isUnlocked(itemKey)) {
                    basePrice *= 2;
                
                    // based on the tier it might be more expensive
                    if (tier) {
                        basePrice *= (1 + tier) / 2;
                    }
                } else if (category === 'playertokens' && !CharacterManager.isUnlocked(itemKey)) {
                    /*// based on weight it might be more expensive
                    var characterJson = Bento.assets.getJson(opts.item.key);
                    var weight = characterJson.weight;
                    // basePrice *= (1 + 1 / weight) / 2;
                    basePrice *= (1 / weight) * 2;*/
                }
                
                // set price, based on how many you bought
                // increase 75 for every item bought
                if (!opts.item.tutorial) {
                    basePrice = basePrice + 75 * bought;
                }
            } else {
                owned.setText(Localization.getText('ownedX'));
                if (opts.item.key === 'currency/keys') {
                    ownedCount.setText(InventoryManager.getItem(opts.item.key).toString());
                } else {
                    ownedCount.setText(CurrencyManager.getKeys().toString());
                }

                // set price, based on how many you bought
                // increase price 50% for every item bought
                Utils.repeat(bought, function () {
                    basePrice *= 1.5;
                });
            }
            var priceFactor = 1;

            if (isShopEvent()) {
                priceFactor = 0.8;
            }

            cost = Math.floor(basePrice * priceFactor);
            price.setText(cost);

            // sold out?
            if (bought >= maxItems) {
                name.setText(Localization.getText('soldOut'), {
                    fontColor: '#ff0000'
                });
                button.setActive(false);
                price.alpha = 0.5;
            }

        };

        // descriptions are optional
        if (opts.item.description) {
            frame.attach(buttonInfo);
        }

        opts.item.sprite.setOriginRelative(new Vector2(0.5, 0.5));

        // add category sprite
        if (category === 'weapons' || category === 'consumables') {
            categoryIcon.attach(new Sprite({
                originRelative: new Vector2(0.5, 1),
                imageName: 'gui/startlevel/inventory'
            }));
        } else if (category === 'equipment') {
            categoryIcon.attach(new Sprite({
                originRelative: new Vector2(0.5, 1),
                imageName: 'gui/startlevel/shirt'
            }));
        } else if (category === 'upgrades') {
            tokenBar = new TokenBar({
                position: new Vector2(72, 12),
                character: tokenCharacter,
                type: tokenType
            });
            owned.parent.attach(tokenBar);
            owned.removeSelf();
        } else if (category === 'enemytokens') {
            tokenBar = new TokenBar({
                position: new Vector2(72, 6),
                character: itemKey,
                type: 'enemytokens'
            });
            owned.parent.attach(tokenBar);
            owned.removeSelf();
        } else if (category === 'playertokens') {
            tokenBar = new TokenBar({
                position: new Vector2(72, 6),
                character: itemKey,
                type: 'playertokens'
            });
            owned.parent.attach(tokenBar);
            owned.removeSelf();
        }

        // add tier stars
        if (category !== 'upgrades') {
            tierStars = new TierStars({
                position: new Vector2(4, -12),
                tier: tier,
                align: 'left'
            });
            frame.attach(tierStars);
        }

        updateItem();

        // DEPRECATED: no more tutorial in shop
        /*if (SaveManager.load('tutorialDailyShopItems') === 0) {
            showTapHand();
        }*/
        
        // try to get constructor asap
        if (!IAPMenu) {
            bento.require(['gui/menus/iap'], function (Module) {
                IAPMenu = Module;
            });
        }

        return frame;
    };

    ShopButton.isShopEvent = isShopEvent;
    
    return ShopButton;
});