/**
 * the bars that come down during a super attack
 * @moduleName SuperAttackBars
 */
bento.define('gui/super/superattackbars', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'globals',
    'components/orientationlock',
    'components/fillscreen',
    'components/anchor'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Globals,
    OrientationLock,
    FillScreen,
    Anchor
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();

        var topbar = new Entity({
            name: 'superAttackTopBar',
            position: new Vector2(0, 0),
            components: [
                new Sprite({
                    imageName: 'superattack-bar',
                    originRelative: new Vector2(0.5, 0.5),
                }),
                new Sprite({
                    imageName: 'superattack-bar',
                    originRelative: new Vector2(0.5, 1),
                })
            ]
        });
        var bottombar = new Entity({
            name: 'superAttackTopBar',
            position: new Vector2(0, 0),
            components: [
                new Sprite({
                    imageName: 'superattack-bar',
                    originRelative: new Vector2(0.5, 0.5),
                }),
                new Sprite({
                    imageName: 'superattack-bar',
                    originRelative: new Vector2(0.5, 1),
                })
            ]
        });

        var fillScreen = new FillScreen({
            color: [0, 0, 0, 0.5]
        });
        var brightness = new Entity({
            name: 'brightness',
            z: Globals.Layers.SUPERATTACK - 3,
            float: true,
            components: [
                fillScreen
            ]
        });

        // var portrait = new Entity({
        //     name: 'portrait',
        //     position: new Vector2(10, 160),
        //     components: [
        //         new Sprite({
        //             spriteSheet: 'characterchange/splash_boy'
        //         }),
        //         new Sprite({
        //             spriteSheet: 'characterchange/boy'
        //         })
        //     ]
        // });

        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {
                topbar.scale.x = -1;
                topbar.scale.y = -1;
                topbar.position.x = viewport.width / 2;
                bottombar.position.x = viewport.width / 2;
                startBars();
                Bento.objects.attach(brightness);
            },
            destroy: function (data) {
                brightness.removeSelf();
            },
            update: function (data) {},
            draw: function (data) {}
        };

        var startBars = function () {
            topbar.scale.x *= (viewport.width + Anchor.getOffset().x * 2) / 180;
            bottombar.scale.x = (viewport.width + Anchor.getOffset().x * 2) / 180;
            var slide = 78 * viewport.height / 320;
            new Tween({
                from: 0,
                to: 1,
                in: 6,
                delay: 0,
                ease: 'easeOutQuad',
                onStart: function () {},
                onUpdate: function (v, t) {
                    topbar.position.y = -78 + slide * v;
                    bottombar.position.y = viewport.height + 78 - v * slide;
                    brightness.alpha = v;
                },
            });
        };


        var entity = new Entity({
            z: Globals.Layers.SUPERATTACKBAR,
            name: 'superAttack',
            position: settings.position,
            updateWhenPaused: 0,
            float: true,
            components: [
                topbar,
                bottombar,
                // portrait,
                behavior,
                new OrientationLock({})
            ]
        }).extend({
            flash: function () {
                brightness.remove(fillScreen);
                fillScreen = new FillScreen({
                    color: [1, 1, 1, 0.5]
                });
                brightness.attach(fillScreen);
                new Tween({
                    from: 1,
                    to: 0,
                    in: 15,
                    delay: 0,
                    ease: 'easeInQuad',
                    onStart: function () {},
                    onUpdate: function (v, t) {
                        brightness.alpha = v;
                    },
                    onComplete: function () {
                        brightness.removeSelf();
                    }
                });
            }
        });
        return entity;
    };
});