/**
 * Module description
 * @moduleName SuperAttackSpeedlines
 */
bento.define('gui/super/superattackspeedlines', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'globals',
    'components/fillscreen',
    'components/sort',
    'gui/backscroller'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Globals,
    FillScreen,
    Sort,
    BackScroller
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();

        var fillScreen = new FillScreen({
            color: [1, 1, 1, 1]
        });
        var lighten = new Entity({
            name: 'lighten',
            z: Globals.Layers.SUPERATTACK - 3,
            components: [
                fillScreen
            ]
        });

        var bgImage = new Sprite({
            imageName: 'superattack-speedlines-' + settings.rootPath.slice(0, -1)
        });

        var speedLinesBehavior = {
            name: 'speedLinesBehavior',
            start: function () {
                new Tween({
                    from: 1,
                    to: 0,
                    in: 20,
                    delay: 0,
                    ease: 'easeInQuad',
                    onUpdate: function (v, t) {
                        lighten.alpha = v;
                    },
                    onComplete: function () {
                        lighten.removeSelf();
                    }
                });
                new Tween({
                    from: 1,
                    to: 0,
                    in: 30,
                    delay: 40,
                    ease: 'easeOutQuad',
                    onStart: function () {
                        backScroller.getComponent('scrollComponent', function (scrollComponent) {
                            scrollComponent.spacing = new Vector2(0, 0);
                        });
                    },
                    onUpdate: function (v, t) {
                        background.alpha = v;
                    },
                    onComplete: function () {
                        background.removeSelf();
                    }
                });
            },
            draw: function (data) {
                // Utils.fillRect(data.renderer, bgImage.spriteImage, 0, 0, viewport.width, viewport.height, viewport.x + background.timer * 4, viewport.y);
            },
            destroy: function () {
                Utils.forEach(settings.actors, function (actor, i, l, breakLoop) {
                    if (actor) {
                        actor.attach(new Sort({}));
                    }
                });
            }
        };
        var backScroller = new BackScroller({
            imageName: 'superattack-speedlines-' + settings.rootPath.slice(0, -1),
            velocity: new Vector2(4, 0),
            spacing: new Vector2(-1, -1)
        });
        var background = new Entity({
            z: Globals.Layers.SUPERATTACK - 4,
            name: 'background',
            family: ['background'],
            float: true,
            position: new Vector2(0, 0),
            components: [
                backScroller,
                speedLinesBehavior,
                lighten,
            ]
        });
        return background;
    };
});