/**
 * Tutorial hand - shows a tapping gesture
 * @moduleName TapHand
 */
bento.define('gui/taphand', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();

        var tapPosition = new Vector2(0, 0);
        var upPosition = settings.upPosition || new Vector2(0, -30);
        var speed = settings.speed || 1;
        var rotation = settings.rotation || 0;
        var startInUpPosition = settings.startInUpPosition || false;

        var sprite = new Sprite({
            imageName: 'gui/taphand',
            originRelative: new Vector2(0.5, 1)
        });

        var moveUp = function () {
            new Tween({
                from: 0,
                to: upPosition.y,
                'in': Math.floor(24 / speed),
                delay: Math.floor(12 / speed),
                ease: Tween.EASEINOUTQUAD,
                onUpdate: function (v, t) {
                    entity.position.y = tapPosition.y + v;
                },
                onComplete: function () {
                    moveDown();
                }
            });

            new Tween({
                from: 0,
                to: upPosition.x,
                'in': Math.floor(24 / speed),
                delay: Math.floor(12 / speed),
                ease: Tween.EASEINOUTQUAD,
                onUpdate: function (v, t) {
                    entity.position.x = tapPosition.x + v;
                }
            });
        };

        var moveDown = function () {
            new Tween({
                from: upPosition.y,
                to: 0,
                'in': Math.floor(8 / speed),
                delay: Math.floor(18 / speed),
                ease: Tween.EASEINQUAD,
                onUpdate: function (v, t) {
                    entity.position.y = tapPosition.y + v;
                },
                onComplete: function () {
                    moveUp();

                    new Tween({
                        from: -0.2,
                        to: 0,
                        'in': 16,
                        ease: Tween.EASEINOUTBACK,
                        onUpdate: function (v, t) {
                            entity.scale.y = 1 + v;
                            entity.scale.x = 1 - v;
                        },
                        onComplete: function () {

                        }
                    });
                }
            });

            new Tween({
                from: upPosition.x,
                to: 0,
                'in': Math.floor(8 / speed),
                delay: Math.floor(18 / speed),
                ease: Tween.EASEINQUAD,
                onUpdate: function (v, t) {
                    entity.position.x = tapPosition.x + v;
                }
            });
        };

        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {
                if (!startInUpPosition) {
                    moveUp();
                } else {
                    entity.position = upPosition.clone();
                    moveDown();
                }
            }
        };
        var entity = new Entity({
            name: 'tutorialhand',
            updateWhenPaused: 0,
            components: [
                sprite,
                behavior
            ]
        });
        var container = new Entity({
            z: settings.z,
            name: 'tapHand',
            position: settings.position,
            float: settings.float || false,
            rotation: rotation,
            components: [
                entity
            ].concat(settings.components || [])
        }).extend({
            hide: function (onComplete) {
                if (entity.alpha !== 1) {
                    return;
                }

                new Tween({
                    from: 1,
                    to: 0,
                    'in': 20,
                    ease: Tween.LINEAR,
                    onUpdate: function (v, t) {
                        entity.alpha = v;
                    },
                    onComplete: onComplete
                });
            }
        });
        return container;
    };
});