/**
 * Button with text inside
 * @moduleName TextButton
 */
bento.define('gui/textbutton', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'gui/shadowtext'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    ShadowText
) {
    'use strict';
    return function (settings) {
        /*settings = {
            z: Number,
            name: String,
            position: Vector2 // positions the entity
            type: String, // small, medium, large (?),
            onClick: Function //
        }*/
        var viewport = Bento.getViewport();
        var type = settings.type || 'small';
        var maxWidths = {
            small: 62,
            medium: 80
        };
        var imageNames = {
            square: 'gui/buttons/empty-square',
            small: 'gui/buttons/empty',
            medium: 'gui/buttons/empty-medium'
        };
        var text = new ShadowText({
            position: new Vector2(0, -4),
            text: settings.text,
            font: 'font',
            fontSize: 16,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            maxWidth: maxWidths[type],
            maxHeight: 16
        });
        var clickButton = new ClickButton({
            z: settings.z || 0,
            name: settings.name || 'clickButton',
            imageName: imageNames[type],
            frameCountX: 1,
            frameCountY: 3,
            sfx: settings.sfx,
            float: settings.float || false,
            updateWhenPaused: settings.updateWhenPaused || 0,
            position: settings.position || new Vector2(0, 0),
            onClick: settings.onClick,
            onButtonUp: function () {
                text.position.y = -4;
                if (settings.onButtonUp) {
                    settings.onButtonUp();
                }
             },
            onButtonDown: function () {
                text.position.y = -3;
                if (settings.onButtonDown) {
                    settings.onButtonDown();
                }
            }
        });

        clickButton.attach(text);

        return clickButton;
    };
});