/**
 * stars that show what tier something is
 * @moduleName TierStars
 */
bento.define('gui/tier', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var tier = settings.tier;
        var alignment = settings.align || 'center';

        var getTierSprite = function () {
            switch (tier) {
            case 1:
                return 'gui/tier-1';
            case 2:
                return 'gui/tier-2';
            case 3:
                return 'gui/tier-3';
            default:
                return '';
            }
        };

        var getAlignment = function () {
            switch (alignment) {
            case 'left':
                return 0;
            case 'right':
                return 1;
            default:
                return 0.5;
            }
        };

        var sprite = new Sprite({
            imageName: getTierSprite(),
            originRelative: new Vector2(getAlignment(), 0.5)
        });

        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {},
            destroy: function (data) {},
            update: function (data) {},
            draw: function (data) {}
        };
        var entity = new Entity({
            z: 0,
            name: 'tierStars',
            position: settings.position,
            components: [
                sprite,
                behavior
            ]
        });
        return entity;
    };
});