/**
 * Timer Tab
 * @moduleName TimerTab
 * @snippet TimerTab.snippet
TimerTab({
    saveId: '',
    useLocal: false
})
 */
bento.define('gui/timertab', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'managers/save',
    'managers/time'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    SaveManager,
    TimeManager
) {
    'use strict';
    var TimerTab = function (settings) {
        var saveId = settings.saveId || 'gift24';
        var useLocal = settings.useLocal;
        var seconds = new Counter({
            name: 'secondsCounter',
            position: new Vector2(8, -3),
            value: '00',
            spacing: new Vector2(1, 0),
            align: 'left',
            sprite: new Sprite({
                imageName: 'gui/worldselect/timernumbers',
                frameWidth: 4,
                frameHeight: 5
            })
        });
        var minutes = new Counter({
            name: 'minutesCounter',
            position: new Vector2(-4, -3),
            value: '00',
            spacing: new Vector2(1, 0),
            align: 'left',
            sprite: new Sprite({
                imageName: 'gui/worldselect/timernumbers',
                frameWidth: 4,
                frameHeight: 5
            })
        });
        var hours = new Counter({
            name: 'hoursCounter',
            position: new Vector2(-16, -3),
            value: '00',
            spacing: new Vector2(1, 0),
            align: 'left',
            sprite: new Sprite({
                imageName: 'gui/worldselect/timernumbers',
                frameWidth: 4,
                frameHeight: 5
            })
        });
        var timeTab = new Entity({
            name: 'timeTab',
            position: new Vector2(0, 19),
            components: [
                new Sprite({
                    imageName: 'gui/worldselect/timertab24',
                    originRelative: new Vector2(0.5, 0.5)
                }),
                hours,
                minutes,
                seconds
            ]
        });
        var addDigit = function (value) {
            return ('00' + value).slice(-2);
        };
        var behavior = {
            name: 'behavior',
            update: function () {
                // update time every second
                if (entity.timer % 60) {
                    return;
                }
                var giftInfo = SaveManager.load(saveId);
                var currentTime = (giftInfo.time || 0) - (useLocal ? TimeManager.getLocalTime() : TimeManager.getTime());
                // setImage(giftInfo.reward);
                if (currentTime > 0) {
                    var s = currentTime / 1000;
                    var m = s / 60;
                    var h = m / 60;

                    hours.setValue(addDigit(Math.floor(h)));
                    minutes.setValue(addDigit(Math.floor(m) % 60));
                    seconds.setValue(addDigit(Math.floor(s) % 60));
                    entity.visible = true;
                } else {
                    entity.visible = false;
                }
            }
        };
        var entity = new Entity({
            z: settings.z,
            name: 'timerTab',
            position: settings.position,
            components: [
                timeTab,
                behavior
            ]
        });

        return entity;
    };
    return TimerTab;
});