/**
 * Horizontal bar with number in it displaying how many tokens left to upgrade
 * @moduleName TokenBar
 */
bento.define('gui/tokenbar', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'managers/currency',
    'managers/stats',
    'globals',
    'gui/shadowtext',
    'managers/character',
    'managers/inventory',
    'managers/allies'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    CurrencyManager,
    StatsManager,
    Globals,
    ShadowText,
    CharacterManager,
    InventoryManager,
    AlliesManager
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var tokenType = settings.type;
        var character = settings.character;
        var level = StatsManager.getLevel(character);
        var lerpFactor = settings.lerp || 0.5;
        var subtract = settings.subtract || 0;
        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {},
            destroy: function (data) {},
            update: function (data) {},
            draw: function (data) {}
        };
        var sprite = new Sprite({
            imageName: 'gui/upgrade/tokenbar-frame',
            originRelative: new Vector2(0.5, 0.5)
        });
        var barBehavior = {
            name: 'barBehavior',
            image: null,
            imageGlow: null,
            glow: 0,
            width: 0,
            realWidth: 0,
            start: function () {
                barBehavior.image = Bento.assets.getImage('gui/upgrade/tokenbar');
                barBehavior.imageGlow = Bento.assets.getImage('gui/upgrade/tokenbar-glow');
            },
            update: function () {
                if (barBehavior.width >= 1) {
                    barBehavior.glow = 0.25 - 0.25 * Math.sin(bar.timer / 10);
                } else {
                    barBehavior.glow = 0;
                }

                // interpolate towards realWidth
                barBehavior.realWidth += (barBehavior.width - barBehavior.realWidth) * lerpFactor;
            },
            draw: function (data) {
                var image = barBehavior.image;
                var imageGlow = barBehavior.imageGlow;
                var oldAlpha = data.renderer.getOpacity();
                var width = barBehavior.realWidth;

                if (width <= 0) {
                    return;
                } else if (width > 1) {
                    width = 1;
                }

                data.renderer.drawImage(
                    image,
                    0,
                    0,
                    width * image.width,
                    image.height,
                    image.width / -2,
                    image.height / -2,
                    width * image.width,
                    image.height
                );

                if (width >= 1) {
                    data.renderer.setOpacity(barBehavior.glow);
                    data.renderer.drawImage(
                        imageGlow,
                        0,
                        0,
                        imageGlow.width,
                        imageGlow.height,
                        imageGlow.width / -2,
                        imageGlow.height / -2,
                        imageGlow.width,
                        imageGlow.height
                    );
                    data.renderer.setOpacity(oldAlpha);
                }
            }
        };
        var bar = new Entity({
            name: 'bar',
            position: new Vector2(0, 0),
            components: [
                barBehavior
            ]
        });
        var arrowBehavior = {
            name: 'arrowBehavior',
            update: function () {
                arrow.position.y = -8 + 0.5 * Math.sin(arrow.timer / 10);
                arrow.visible = (barBehavior.width >= 1);
            }
        };
        var arrow = new Entity({
            name: 'arrow',
            position: new Vector2(18, -8),
            visible: false,
            components: [
                new Sprite({
                    imageName: 'gui/upgrade/arrow',
                    originRelative: new Vector2(0.5, 0.5)
                }),
                arrowBehavior
            ]
        });
        var text = new ShadowText({
            position: new Vector2(0, -2),
            text: '',
            font: 'font',
            fontSize: 16,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            maxWidth: 32,
            maxHeight: 16
        });
        var doAnimation = function () {
            var oldValue = subtract;
            var bounce = function () {
                new Tween({
                    from: 0.1,
                    to: 0,
                    in: 30,
                    delay: 0,
                    ease: 'elastic',
                    decay: 8,
                    oscillations: 2,
                    onUpdate: function (value) {
                        entity.scale.x = 1 + value;
                        entity.scale.y = 1 - value;
                    }
                });
            };

            entity.setCharacter(character);

            new Tween({
                from: subtract,
                to: 0,
                in: Utils.clamp(5, 5 * subtract, 30),
                ease: 'linear',
                onUpdate: function (v, t) {
                    subtract = Math.ceil(v);
                    if (oldValue !== subtract && t % 5 === 0) {
                        oldValue = subtract;
                        // do a little bounce
                        bounce();
                    }
                    entity.setCharacter(character);
                },
                onComplete: function () {

                }
            });
        };
        var diff = 1;
        var entity = new Entity({
            name: 'tokenbar',
            position: settings.position,
            components: [
                sprite,
                behavior,
                bar,
                arrow,
                text
            ]
        });

        entity.extend({
            setCharacter: function (value) {
                var cost;
                var amount;
                var characterJson;
                character = value;

                if (tokenType === 'enemytokens') {
                    level = AlliesManager.getLevel(character);
                    cost = AlliesManager.getUpgradeCost(character, diff);
                    // locked characters need tokens to unlock first:
                    if (!AlliesManager.isUnlocked(character)) {
                        cost = AlliesManager.getUnlockCost(character);
                    }
                } else if (tokenType === 'evolve') {
                    cost = CharacterManager.getEvolveItems(character);
                } else {
                    // playertokens
                    level = StatsManager.getLevel(character);
                    cost = Globals.getTokenCost(level, diff);

                    // locked characters need tokens to unlock first:
                    if (!CharacterManager.isUnlocked(character)) {
                        cost = CharacterManager.getUnlockCost(character);
                    }
                }

                if (tokenType === 'evolve') {
                    amount = InventoryManager.getItem(tokenType + '/' + character) - subtract;
                } else {
                    amount = InventoryManager.getTokens(tokenType + '/' + character) - subtract;
                }
                text.setText(amount + '/' + cost, {
                    fontColor: amount >= cost ? '#ffff00' : '#ffffff'
                });
                barBehavior.width = amount / cost;
            },
            setLevelDiff: function (value) {
                diff = value;
                // update self
                entity.setCharacter(character);
            }
        });

        if (subtract) {
            // do an animation
            doAnimation();
        } else {
            entity.setCharacter(character);
        }

        return entity;
    };
});