/**
 * Dialog that warns the user for not having enough tokens
 * @moduleName TokenDialog
 */
bento.define('gui/tokendialog', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'gui/tokenbar',
    'gui/shadowtext',
    'modules/localization',
    'gui/dialog',
    'gui/menus/iap',
    'globals'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    TokenBar,
    ShadowText,
    Localization,
    Dialog,
    IAPMenu,
    Globals
) {
    'use strict';
    
    /**
     * This dialog shows how many tokens the player still needs to collect
     */
    var TokenDialog = function (settings) {
        var character = settings.character;
        var tokenType = settings.tokenType || 'playertokens';
        var getTokenSprite = function () {
            var tokenJson = Bento.assets.getJson(tokenType + '/' + character);
            var sprite = new Sprite({
                spriteSheet: tokenJson.inventory.image
            });
            sprite.setOriginRelative(new Vector2(0.5, 0.5));
            return sprite;
        };
        var tokenSprite = getTokenSprite();
        var tokenIcon = new Entity({
            name: 'tokenIcon',
            position: new Vector2(-20, 0),
            components: [
                tokenSprite
            ]
        });
        var tokenBar = new TokenBar({
            position: new Vector2(14, 0),
            character: character,
            type: tokenType
        });
        var icon = new Entity({
            name: 'iconContainer',
            position: new Vector2(0, 0),
            dimension: new Rectangle(0, 0, 64, 32),
            components: [
                tokenBar,
                tokenIcon
            ]
        });
        var iconYesButton = new Entity({
            name: 'iconYesButton',
            position: new Vector2(0, 0),
            components: [
                new Entity({
                    name: 'iconChest',
                    position: new Vector2(-24, 0),
                    components: [
                        new Sprite({
                            imageName: 'gui2/misc/button_icon_chest',
                            originRelative: new Vector2(0.5, 0.5),
                        })
                    ]
                }),
                new ShadowText({
                    position: new Vector2(16, -2),
                    text: Localization.getText('getChests'),
                    font: 'font',
                    fontSize: 16,
                    fontColor: '#ffffff',
                    align: 'center',
                    textBaseline: 'middle',
                    maxWidth: 50,
                    maxHeight: 26
                })
            ]
        });
        var accepted = false;
        var dialog = new Dialog({
            name: 'tokensNeededDialog',
            title: Localization.getText('needTokensTitle'),
            text: Localization.getText('needTokensText'),
            icon: icon,
            yesIcon: iconYesButton,
            type: 'yesno',
            canClose: true,
            onYes: function () {
                accepted = true;
            },
            onComplete: function () {
                if (!accepted) {
                    return;
                }
                var iapMenu = new IAPMenu({
                    z: Globals.WorldSelectLayers.IAPPOPUP,
                    isPopUp: true
                });
                Bento.objects.attach(iapMenu);
            }
        });

        return dialog;
    };
    return TokenDialog;
});