/**
 * Popup that unlocks character
 * @moduleName UnlockCharacterPopup
 */
bento.define('gui/unlockcharacterpopup', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'gui/blurbackground',
    'components/modal',
    'gui/headerbox',
    'globals',
    'components/anchor',
    'gui/shadowtext',
    'modules/localization',
    'entities/effect',
    'gui/confetti',
    'components/orientationlock',
    'gui/iconbutton'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    BlurBackground,
    Modal,
    HeaderBox,
    Globals,
    Anchor,
    ShadowText,
    Localization,
    Effect,
    Confetti,
    OrientationLock,
    IconButton
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        /*
         * Settings
         */
        var character = settings.character;
        var fullName = settings.name;
        /*
         * Components
         */
        var blurBackground = new BlurBackground({});
        var modal = new Modal({});
        var youUnlocked = new ShadowText({
            text: Localization.getText('youUnlocked'),
            font: 'font',
            fontSize: 24,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle'
        }).attach(new Anchor({
            screenPosition: new Vector2(0.5, 0.5),
            offset: new Vector2(0, -64)
        }));
        var characterName = new ShadowText({
            text: fullName,
            font: 'font',
            fontSize: 24,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle'
        }).attach(new Anchor({
            screenPosition: new Vector2(0.5, 0.6),
            offset: new Vector2(0, 0)
        }));
        var lock = new Entity({
            name: 'lock',
            position: new Vector2(0, -16),
            components: [
                new Sprite({
                    imageName: 'gui/lock',
                    originRelative: new Vector2(0.5, 0.5)
                })
            ]
        });
        var characterSprite = new Sprite({
            spriteSheet: character + '/idle'
        });
        var characterEntity = new Entity({
            name: 'character',
            components: [
                characterSprite,
                lock,
                new Anchor({
                    screenPosition: new Vector2(0.5, 0.5)
                })
            ]
        });
        var buttonOk = new IconButton({
            name: 'buttonOk',
            sfx: 'sfx_button',
            imageName: 'gui2/misc/button_small_primary',
            iconName: 'gui2/misc/button_icon_okay',
            frameCountX: 1,
            frameCountY: 3,
            position: new Vector2(0, 0),
            float: false,
            updateWhenPaused: 0,
            onClick: function () {
                buttonOk.setActive(false);
                entity.remove(modal);

                // fade out
                new Tween({
                    from: 1,
                    to: 0,
                    in: 15,
                    ease: 'linear',
                    onStart: function () {},
                    onUpdate: function (v, t) {
                        entity.alpha = v;
                    },
                    onComplete: function () {
                        entity.removeSelf();
                    }
                });
            }
        }).attach(new Anchor({
            screenPosition: new Vector2(0.5, 1),
            offset: new Vector2(0, -32)
        }));
        var tweenBehavior = {
            name: 'tweenBehavior',
            start: function () {
                characterName.visible = false;
                youUnlocked.visible = false;
                buttonOk.visible = false;
                buttonOk.setActive(false);

                // lower music volume for the character unlock jingle
                new Tween({
                    from: 1,
                    to: 0.1,
                    in: 30,
                    onUpdate: function (v, t) {
                        Bento.audio.setVolume(v, 'bgm_worldmap');
                    }
                });

                Utils.timeout(30, function () {
                    characterSprite.setSpriteSheet(character + '/punch1', function () {
                        characterSprite.setSpriteSheet(character + '/idle');
                    });
                });
                Utils.timeout(40, function () {
                    // punch away lock
                    var effect = new Effect({
                        spriteSheet: 'effects/hitfx2'
                    });
                    effect.position.y -= 16;
                    characterEntity.attach(effect);

                    Bento.audio.stopSound('sfx_punch0');
                    Bento.audio.playSound('sfx_punch0');

                    new Tween({
                        from: 1,
                        to: 0,
                        in: 10,
                        onUpdate: function (v, t) {
                            lock.transform.x = Utils.getRandomRange(-v, v);
                            lock.transform.y = Utils.getRandomRange(-v, v);
                        },
                        onComplete: function () {
                            // the lock flies off
                            lock.attach({
                                name: 'movement',
                                speed: -3,
                                update: function () {
                                    lock.position.x += 2;
                                    this.speed += 0.15;
                                    lock.position.y += this.speed;
                                    lock.rotation += 0.01;

                                    if (lock.position.y > viewport.height + 32) {
                                        lock.removeSelf();
                                    }
                                }
                            });
                        }
                    });
                });

                // show victory pose
                Utils.timeout(80, function () {
                    characterSprite.setSpriteSheet(character + '/victory');

                    Bento.audio.stopSound('sfx_unlock');
                    Bento.audio.playSound('sfx_unlock');

                    // add confetti
                    entity.attach(new Confetti({}));
                });

                // restore audio
                new Tween({
                    from: 0.1,
                    to: 1,
                    in: 60,
                    delay: 180,
                    onUpdate: function (v, t) {
                        Bento.audio.setVolume(v, 'bgm_worldmap');
                    }
                });

                // tween in buttons
                new Tween({
                    from: youUnlocked.position.x + viewport.width,
                    to: youUnlocked.position.x,
                    in: 30,
                    delay: 80,
                    ease: 'elastic',
                    decay: 6, // for 'exponential' and 'elastic'
                    oscillations: 1, // for 'elastic', 'sin' and 'cos'
                    onStart: function () {
                        youUnlocked.visible = true;
                    },
                    onUpdate: function (v, t) {
                        youUnlocked.position.x = v;
                    }
                });
                new Tween({
                    from: characterName.position.x - viewport.width,
                    to: characterName.position.x,
                    in: 30,
                    delay: 80,
                    ease: 'elastic',
                    decay: 6, // for 'exponential' and 'elastic'
                    oscillations: 1, // for 'elastic', 'sin' and 'cos'
                    onStart: function () {
                        characterName.visible = true;
                    },
                    onUpdate: function (v, t) {
                        characterName.position.x = v;
                    }
                });
                new Tween({
                    from: buttonOk.position.y + viewport.height / 2,
                    to: buttonOk.position.y,
                    in: 30,
                    delay: 80,
                    ease: 'elastic',
                    decay: 6, // for 'exponential' and 'elastic'
                    oscillations: 1, // for 'elastic', 'sin' and 'cos'
                    onStart: function () {
                        buttonOk.visible = true;
                    },
                    onUpdate: function (v, t) {
                        buttonOk.position.y = v;
                    },
                    onComplete: function () {
                        buttonOk.setActive(true);
                    }
                }).attach(new OrientationLock({}));
            }
        };
        /*
         * Entity
         */
        var entity = new Entity({
            z: Globals.WorldSelectLayers.EQUIPMENT + 1,
            name: 'unlockPopup',
            position: settings.position,
            updateWhenPaused: 0,
            float: true,
            components: [
                blurBackground,
                modal,
                characterEntity,
                characterName,
                youUnlocked,
                buttonOk,
                tweenBehavior
            ]
        });
        return entity;
    };
});