/**
 * gui frame that show how many items you have for upgrades
 * @moduleName UpgradeFrame
 */
bento.define('gui/upgrade/upgradeframe', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'gui/shadowtext',
    'managers/inventory',
    'managers/character',
    'managers/stats',
    'managers/currency',
    'globals',
    'gui/tokenbar'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    ShadowText,
    InventoryManager,
    CharacterManager,
    StatsManager,
    CurrencyManager,
    Globals,
    TokenBar
) {
    'use strict';
    return function (settings) {
        var character = CharacterManager.getSelected();
        var level = StatsManager.getLevel(character);
        // upgrade cost
        var cost = Globals.getTokenCost(level);
        var tokens = CurrencyManager.getTokens(character);

        var viewport = Bento.getViewport();
        var frame = new Entity({
            name: 'frame',
            position: new Vector2(0, 4),
            components: [
                new Sprite({
                    originRelative: new Vector2(0.5, 0),
                    imageName: 'gui/upgrade/upgradeframe'
                })
            ]
        });

        var healthText = new ShadowText({
            z: 0,
            position: new Vector2(-50, 44),
            text: tokens.health + '/' + cost,
            font: 'font',
            fontSize: 16,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            maxWidth: 35,
            maxHeight: 16,
            linebreaks: false
        });

        var attackText = new ShadowText({
            z: 0,
            position: new Vector2(0, 44),
            text: tokens.attack + '/' + cost,
            font: 'font',
            fontSize: 16,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            maxWidth: 35,
            maxHeight: 16,
            linebreaks: false
        });

        var skillText = new ShadowText({
            z: 0,
            position: new Vector2(50, 44),
            text: tokens.skill + '/' + cost,
            font: 'font',
            fontSize: 16,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            maxWidth: 35,
            maxHeight: 16,
            linebreaks: false
        });

        var healthSprite = new Sprite({
            spriteSheet: 'upgrades/health-' + character
        });

        var healthToken = new Entity({
            name: 'skillToken',
            position: new Vector2(-50, 16),
            components: [
                healthSprite
            ]
        });

        var attackSprite = new Sprite({
            spriteSheet: 'upgrades/attack-' + character
        });

        var attackToken = new Entity({
            name: 'skillToken',
            position: new Vector2(0, 16),
            components: [
                attackSprite
            ]
        });

        var skillSprite = new Sprite({
            spriteSheet: 'upgrades/skill-' + character
        });

        var skillToken = new Entity({
            name: 'skillToken',
            position: new Vector2(50, 16),
            components: [
                skillSprite
            ]
        });

        var tokenBarHealth = new TokenBar({
            position: new Vector2(-50, 44),
            character: character,
            type: 'health'
        });
        var tokenBarAttack = new TokenBar({
            position: new Vector2(0, 44),
            character: character,
            type: 'attack'
        });
        var tokenBarSkill = new TokenBar({
            position: new Vector2(50, 44),
            character: character,
            type: 'skill'
        });

        var updateFrame = function (currentCharacter) {
            healthSprite.setSpriteSheet('upgrades/health-' + currentCharacter);
            attackSprite.setSpriteSheet('upgrades/attack-' + currentCharacter);
            skillSprite.setSpriteSheet('upgrades/skill-' + currentCharacter);

            tokenBarHealth.setCharacter(currentCharacter);
            tokenBarAttack.setCharacter(currentCharacter);
            tokenBarSkill.setCharacter(currentCharacter);
        };

        var entity = new Entity({
            z: 0,
            name: 'upgradeFrame',
            position: settings.position,
            updateWhenPaused: 0,
            components: [
                frame,
                healthToken,
                attackToken,
                skillToken,
                tokenBarHealth,
                tokenBarAttack,
                tokenBarSkill,
            ]
        }).extend({
            updateFrame: updateFrame
        });
        return entity;
    };
});