/**
 * A standard function to watch ad with callback on complete and error dialog if not available
 * @moduleName WatchRewardedAd
 */
bento.define('gui/watchrewardedad', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'ads/heyzap',
    'gui/dialog',
    'modules/localization',
    'globals',
    'modules/analytics',
    'modules/tenjin',
    'managers/save',
    'managers/time'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    HeyZap,
    Dialog,
    Localization,
    Globals,
    Analytics,
    Tenjin,
    SaveManager,
    TimeManager
) {
    'use strict';
    var RewardedAd = function (onComplete) {
        var dialog;
        var debug = Bento.isDev() && !Utils.isCocoonJs();
        if (HeyZap.canShowRewarded()) {
            Bento.audio.muteMusic(true);
            HeyZap.showRewarded(function () {}, function (hasCompleted) {
                var adsWatched = SaveManager.load('rewardedAdsWatched');
                adsWatched += 1;
                SaveManager.save('rewardedAdsWatched', adsWatched);
                SaveManager.save('ignoredAds', 0);
                SaveManager.save('lastTimeWatchedAd', TimeManager.getTime());
                // restore mute state
                Bento.audio.muteMusic(Bento.saveState.load('muteMusic', false), true);
                Bento.audio.muteSound(Bento.saveState.load('muteSound', false));

                if (!hasCompleted) {
                    if (onComplete) {
                        onComplete(false);
                    }
                    Analytics.event('Ads', 'completionRate', 0);
                    return;
                }
                // Utils.trackAdViews('rewarded');
                if (onComplete) {
                    onComplete(true);
                    EventSystem.fire('quest-videos', {
                        count: 1
                    });
                }
                Analytics.event('Ads', 'completionRate', 1);

                // quick fix: wait until the ad overlay is gone, or Tenjin will crash
                Utils.timeout(30, function () {
                    Tenjin.sendEvent('rewardedAd');
                    Tenjin.sendEventAndValue('watchedAdsTotal', adsWatched);
                    if (adsWatched === 5) {
                        Tenjin.sendEvent('watchedAdsTotal_' + adsWatched);
                    }
                }, Infinity);
            });
        } else if (debug) {
            dialog = new Dialog({
                ignoreAnalytics: true,
                z: Globals.Layers.SYSTEM,
                text: "You are now watching an ad",
                type: 'yes',
                preventTweens: true,
                onComplete: function () {
                    if (onComplete) {
                        onComplete(true);
                        EventSystem.fire('quest-videos', {
                            count: 1
                        });
                    }
                }
            });
            Bento.objects.attach(dialog);
        } else {
            // show error
            dialog = new Dialog({
                z: Globals.Layers.SYSTEM,
                name: 'error_noAd',
                text: Localization.getText('noAd'),
                type: 'yes',
                onComplete: function () {
                    if (onComplete) {
                        onComplete(false);
                    }
                }
            });
            Bento.objects.attach(dialog);
            Analytics.event('Ads', 'noAdAvailable');
        }
    };

    RewardedAd.showInterstitial = function (onComplete) {
        // check if testing on desktop
        var isDesktopTester = function () {
            // tester on desktop should get continue
            return Bento.isDev() && !Utils.isCocoonJs();
        };

        // IAP purchases don't need to watch any
        if (Bento.saveState.load('iapAmount', 0) > 0) {
            onComplete();
            return;
        }

        // stop any music
        Bento.audio.stopAllMusic();

        if (isDesktopTester() || Bento.saveState.load('showDeveloperCheats', false)) {
            // testing on non cordova environment
            window.alert('You are now watching a forced ad.');
            SaveManager.save('lastTimeWatchedAd', TimeManager.getTime());
            onComplete();
        } else if (HeyZap.canShowInterstitial()) {
            Analytics.event('Ads', 'Began_Forced');

            HeyZap.showInterstitial(function () {}, function () {
                Analytics.event('Ads', 'Finished_Forced');
                // reset counters
                SaveManager.save('ignoredAds', 0);
                SaveManager.save('lastTimeWatchedAd', TimeManager.getTime());

                if (onComplete) {
                    onComplete();
                }
            });
        } else {
            // ad not ready
            if (onComplete) {
                onComplete();
            }
        }
    };

    return RewardedAd;
});