/**
 * Scroll through world using touch
 * @moduleName WorldScroller
 */
bento.define('gui/worldscroller', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'gui/scrollinglist'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    ScrollingList
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            scrollX: Boolean,
            scrollY: Boolean,
            width: Number,
            height: Number
        }*/
        var viewport = Bento.getViewport();
        var width = settings.width - viewport.width;
        var height = settings.height - viewport.height;
        // scrollers are just scroll lists
        var scrollX = new ScrollingList({
            direction: 'x',
            maxOffset: width,
            clamp: settings.clamp,
            onScrollEnd: settings.onScrollEndX,
            damping: 0.95
        });
        var scrollY = new ScrollingList({
            direction: 'y',
            maxOffset: height,
            clamp: settings.clamp,
            onScrollEnd: settings.onScrollEndY,
            damping: 0.95
        });
        // scroll behaviors
        var scrollBehaviorX = {
            name: 'scrollBehaviorX',
            update: function () {
                // viewport.x = -scrollX.getScroll();
            }
        };
        var scrollBehaviorY = {
            name: 'scrollBehaviorY',
            update: function () {
                // viewport.y = -scrollY.getScroll();
            }
        };
        // adds a fake item
        var addDummyItem = function (parent) {
            parent.addItem(new Entity({
                name: 'dummyObj'
            }));
        };
        var cancelScroll = function () {
            scrollX.cancel();
            scrollY.cancel();
        };
        var onOrientationChange = function () {
            width = settings.width - viewport.width;
            height = settings.height - viewport.height;
            // scrollX.setSpacing(width);
            // scrollY.setSpacing(height);
            scrollX.setMaxOffset(width);
            scrollY.setMaxOffset(height);
        };

        // add at least 2 items to allow scrolling
        addDummyItem(scrollX);
        addDummyItem(scrollX);
        addDummyItem(scrollY);
        addDummyItem(scrollY);

        // scroll behaviors
        if (settings.scrollX) {
            scrollX.attach(scrollBehaviorX);
        }
        if (settings.scrollY) {
            scrollY.attach(scrollBehaviorY);
        }

        return {
            name: 'worldScroller',
            scrollX: scrollX,
            scrollY: scrollY,
            // centers camera to position
            centerTo: function (vector2) {
                if (settings.scrollX) {
                    scrollX.forceTo(-vector2.x + viewport.width / 2);
                }
                if (settings.scrollY) {
                    scrollY.forceTo(-vector2.y + viewport.height / 2);
                }
            },
            forceTo: function (vector2) {
                if (settings.scrollX) {
                    scrollX.forceTo(-vector2.x);
                }
                if (settings.scrollY) {
                    scrollY.forceTo(-vector2.y);
                }
            },
            // cancels scroll
            cancel: cancelScroll,
            start: function () {
                // attach scrollers
                Bento.objects.attach(scrollX);
                Bento.objects.attach(scrollY);

                EventSystem.on('preventScroll', cancelScroll);
                EventSystem.on('orientationChanged', onOrientationChange);
            },
            destroy: function () {
                Bento.objects.remove(scrollX);
                Bento.objects.remove(scrollY);

                EventSystem.off('preventScroll', cancelScroll);
                EventSystem.off('orientationChanged', onOrientationChange);
            },
            update: function () {
                // console.log(-scrollX.getScroll());
            },
            getScrollPosition: function () {
                var x = -scrollX.getScroll() || 0;
                var y = -scrollY.getScroll() || 0;
                return new Vector2(x, y);
            },
            setScrollArea: function (rectangle) {
                scrollX.setMinOffset(rectangle.x);
                scrollY.setMinOffset(rectangle.y);

                // change settings.width/height because onOrientationChange reads this
                settings.width = rectangle.getX2();
                settings.height = rectangle.getY2();

                onOrientationChange();
            },
            stopSlidingX: function () {
                scrollX.stopSliding();
            },
            stopSlidingY: function () {
                scrollY.stopSliding();
            },
            setActive: function (value) {
                scrollX.setActive(value);
                scrollY.setActive(value);
            }
        };
    };
});