/**
 * Watch ad for coins in worldselect
 * @moduleName AdForCoins
 */
bento.define('gui/worldselect/adforcoins', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'globals',
    'modules/videoadtimer',
    'gui/watchrewardedad',
    'managers/currency',
    'gui/coinreward',
    'modules/analytics',
    'managers/save',
    'managers/time',
    'gui/iconbutton',
    'gui/badge',
    'components/anchor',
    'gui/adstackingcoins',
    'gui/timertab'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Globals,
    VideoAdTimer,
    WatchRewardedAd,
    CurrencyManager,
    CoinReward,
    Analytics,
    SaveManager,
    TimeManager,
    IconButton,
    Badge,
    Anchor,
    AdStackingCoins,
    TimerTab
) {
    'use strict';
    var AdForCoins = function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var isTweening = false;

        var dailyCanPlay = false;
        var badge = new Badge({
            exclamation: true,
            imageName: 'gui2/misc/button_coinalert_tertiary',
            position: new Vector2(16, -12),
        });
        var behavior = {
            name: 'behavior',
            update: function () {
                if (entity.timer % 60 === 0) {
                    // check time
                    if (AdForCoins.canPlay()) {
                        // enable and show badge
                        if (!entity.isActive()) {
                            entity.setActive(true);
                        }
                        badge.visible = true;
                    } else {
                        // disable and hide badge
                        entity.setActive(false);
                        badge.visible = false;
                    }
                    entity.getComponent('icon', function (icon) {
                        icon.alpha = entity.isActive() ? 1 : 0.5;
                    });
                }
            },
            destroy: function () {}
        };
        var entity = new IconButton({
            z: Globals.WorldSelectLayers.GUI,
            name: 'adForCoins',
            sfx: 'sfx_button',
            imageName: 'gui2/misc/buttonmini_blinking',
            iconName: 'gui2/misc/button_icon_watchad',
            frameCountX: 1,
            frameCountY: 4,
            animations: {
                up: {
                    speed: 0.025,
                    frames: [0, 1]
                },
                down: {
                    frames: [2]
                },
                inactive: {
                    frames: [3]
                }
            },
            float: true,
            onClick: function () {
                if (AdForCoins.canPlay()) {
                    AdForCoins.play({});
                }
            },
            components: [
                behavior,
                badge,
                new TimerTab({
                    saveId: 'adsForCoins'
                }),
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(1, 0),
                        offset: new Vector2(-26, 48),
                    },
                    landscape: {
                        screenPosition: new Vector2(1, 0),
                        offset: new Vector2(-26, 26),
                    }
                })
            ]
        });
        return entity;
    };
    // play video
    AdForCoins.play = function (settings) {
        WatchRewardedAd(function (completed) {
            var coinReward;
            var coins = Globals.getCoinsForAd(SaveManager.load('unlockedLevels').length)[0];
            if (completed) {
                CurrencyManager.addCoins(coins, true);
                coinReward = new CoinReward({
                    z: settings.z,
                    coins: coins,
                    onComplete: function () {
                        Bento.objects.attach(new AdStackingCoins({}));
                    }
                });
                Bento.objects.attach(coinReward);
                Analytics.event('Ads', 'watch_for_coins');

                var dailyAds = SaveManager.load('adsForCoins');
                // check time
                var currentTime = TimeManager.getTime();
                // schedule 3 hours ahead
                dailyAds.time = currentTime + 3 * 60 * 60 * 1000;
                dailyAds.adsWatched += 1;

                SaveManager.save('adsForCoins', dailyAds);
            }

            if (settings.onComplete) {
                settings.onComplete();
            }
        });
    };
    AdForCoins.canPlay = function () {
        var dailyAds = SaveManager.load('adsForCoins');
        // check time
        var time = dailyAds.time || 0;
        var currentTime = TimeManager.getTime();

        if (currentTime >= time) {
            return true;
        } else {
            return false;
        }
    };
    AdForCoins.getTimeString = function () {
        var dailyAds = SaveManager.load('adsForCoins');
        var time = dailyAds.time || 0;
        var currentTime = time - TimeManager.getTime();
        var s = currentTime / 1000;
        var m = s / 60;
        var h = m / 60;
        var addDigit = function (value) {
            return ('00' + value).slice(-2);
        };

        if (currentTime > 0) {
            return addDigit(Math.floor(h)) + ':' + addDigit(Math.floor(m) % 60) + ':' + addDigit(Math.floor(s) % 60);
        } else {
            return '00:00:00';
        }
    };
    return AdForCoins;
});