/**
 * coin gui element in top bar of world select
 * @moduleName Barcoins
 */
bento.define('gui/worldselect/barcoins', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/utils',
    'bento/tween',
    'globals',
    'managers/inventory',
    'gui/shadowtext',
    'managers/currency',
    'components/glow',
    'managers/save'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Utils,
    Tween,
    Globals,
    InventoryManager,
    ShadowText,
    CurrencyManager,
    Glow,
    SaveManager
) {
    'use strict';
    var IAPMenu; // need to get constructor later because there is a circular dependency here

    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var coinValue = CurrencyManager.getCoins();
        var preventClick = settings.preventClick;
        var active = true;

        var text = new ShadowText({
            z: Globals.Layers.GUI,
            position: new Vector2(19, 8),
            text: coinValue.toString(), //9999,//
            font: 'font',
            fontSize: 11,
            fontColor: '#ffffff',
            align: 'left',
            textBaseline: 'middle',
            ySpacing: 0,
            lineWidth: 0,
            maxWidth: 22,
            maxHeight: 11,
            linebreaks: false
        });

        var onCoinsChanged = function () {
            coinValue = CurrencyManager.getCoins();
            text.setText(coinValue.toString());
        };

        var sprite = new Sprite({
            imageName: 'gui2/topbar/topbar__coins',
            originRelative: new Vector2(0, 0),
        });

        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {
                EventSystem.on('updateCoins', onCoinsChanged);
                glow.alpha = 0;
            },
            destroy: function (data) {
                EventSystem.off('updateCoins', onCoinsChanged);
            },
            update: function (data) {},
            draw: function (data) {}
        };
        var clickable = new Clickable({
            onClick: function (data) {
                EventSystem.fire('preventScroll');
            },
            onHoldEnd: function (data) {
                if (!IAPMenu || !active) {
                    // not defined yet
                    return;
                }
                var grandParent;
                if (entity.parent && entity.parent.parent) {
                    grandParent = entity.parent.parent;
                }
                var iapMenu = new IAPMenu({
                    z: grandParent ? grandParent.z + 100 : Globals.WorldSelectLayers.IAP,
                    scrollTo: 'coins',
                    preventChests: settings.preventChests,
                    skipTutorial: settings.skipTutorial
                });
                Bento.objects.attach(iapMenu);
            }
        });
        var glow = new Glow({
            average: 1,
            amplitude: 0,
            glowSpeed: 0
        });
        var entity = new Entity({
            z: Globals.Layers.GUI,
            name: 'barcoins',
            position: settings.position,
            components: [
                sprite,
                text,
                behavior,
                glow
            ]
        });

        entity.extend({
            setCoins: function (value) {
                text.setText(coinValue.toString());
            },
            flash: function () {
                new Tween({
                    from: 1,
                    to: 0,
                    in: 5,
                    delay: 0,
                    ease: 'linear',
                    onUpdate: function (v, t) {
                        glow.alpha = v;
                    }
                });
            },
            setActive: function (value) {
                active = value;
            }
        });

        if (!preventClick) {
            entity.attach(clickable);
        }

        // try to get constructor asap
        if (!IAPMenu) {
            bento.require(['gui/menus/iap'], function (Module) {
                IAPMenu = Module;
            });
        }

        return entity;
    };
});