/**
 * energy gui element in top bar of world select
 * @moduleName Barenergy
 */
bento.define('gui/worldselect/barenergy', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/utils',
    'bento/tween',
    'gui/shadowtext',
    'globals',
    'managers/character',
    'components/glow'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Utils,
    Tween,
    ShadowText,
    Globals,
    CharacterManager,
    Glow
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var character;
        var energyString;
        var previousTime;

        var text = new ShadowText({
            z: Globals.Layers.GUI,
            position: new Vector2(24, 8),
            text: '99/99',
            font: 'font',
            fontSize: 11,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            ySpacing: 0,
            lineWidth: 0,
            maxWidth: 25,
            linebreaks: false,
            strokeStyle: '#000000',
            innerStroke: false,
            pixelStroke: false // workaround for Cocoon bug
        });

        var seconds = new Counter({
            name: 'secondsCounter',
            position: new Vector2(20, 18),
            value: '00',
            spacing: new Vector2(1, 0),
            align: 'left',
            sprite: new Sprite({
                imageName: 'gui/worldselect/timernumbers',
                frameWidth: 4,
                frameHeight: 5
            })
        });

        var minutes = new Counter({
            name: 'minutesCounter',
            position: new Vector2(16, 18),
            value: '00',
            spacing: new Vector2(1, 0),
            align: 'right',
            sprite: new Sprite({
                imageName: 'gui/worldselect/timernumbers',
                frameWidth: 4,
                frameHeight: 5
            })
        });


        var sprite = new Sprite({
            imageName: 'gui2/topbar/topbar__energy',
            originRelative: new Vector2(0, 0),
        });

        var timeTab = new Entity({
            name: 'timeTab',
            position: new Vector2(8, 16),
            components: [
                new Sprite({
                    imageName: 'gui/worldselect/timertab'
                })
            ]
        });

        var setEnergyAndTime = function () {
            character = CharacterManager.getSelected();

            //get the energy and maximum variables from the character and change the string if anything is different
            var energy = CharacterManager.getEnergy(character) + '/' + CharacterManager.getMaximum(character);
            if (energyString !== energy) {
                text.setText(energy);
                energyString = energy;
            }

            // add zeroes to the front a string according to amount
            var addDigit = function (value, amount) {
                amount = amount || 2;
                value = value.toString();
                while (value.length < amount) {
                    value = '0' + value;
                }
                return value;
            };

            //get current regen time of the character
            var timer = CharacterManager.getEnergyTime(character);
            // var tString;

            //if not online display no time, else stringify the time
            if (!timer.online) {
                // tString = '';
                previousTime = 0;
                timeTab.visible = false;
                seconds.visible = false;
                minutes.visible = false;
            } else {
                // tString = Math.floor(timer.next / 60000) + ':' + addDigit(Math.floor(timer.next / 1000 % 60));
                timeTab.visible = true;
                seconds.visible = true;
                minutes.visible = true;
            }

            //set text if string has changed
            if (timer.next !== previousTime) {
                // timeText.setText(tString);
                minutes.setValue(Math.floor(timer.next / 60000));
                seconds.setValue(addDigit(Math.floor(timer.next / 1000 % 60)));
                previousTime = timer.next;
            }
        };

        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {
                setEnergyAndTime();
                glow.alpha = 0;
            },
            destroy: function (data) {},
            update: function (data) {
                setEnergyAndTime();
            },
            draw: function (data) {}
        };

        var glow = new Glow({
            average: 1,
            amplitude: 0,
            glowSpeed: 0
        });
        var entity = new Entity({
            z: Globals.Layers.GUI,
            name: 'barstars',
            updateWhenPaused: Globals.PauseLevels.ALWAYS,
            components: [
                sprite,
                timeTab,
                text,
                minutes,
                seconds,
                behavior,
                glow
            ]
        }).extend({
            flash: function () {
                new Tween({
                    from: 1,
                    to: 0,
                    in: 5,
                    delay: 0,
                    ease: 'linear',
                    onUpdate: function (v, t) {
                        glow.alpha = v;
                    }
                });
            }
        });
        return entity;
    };
});