/**
 * gems gui element in top bar of world select
 * @moduleName Bargems
 */
bento.define('gui/worldselect/bargems', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/utils',
    'bento/tween',
    'globals',
    'gui/shadowtext',
    'components/eventlistener',
    'managers/currency',
    'managers/save',
    'components/glow'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Utils,
    Tween,
    Globals,
    ShadowText,
    EventListener,
    CurrencyManager,
    SaveManager,
    Glow
) {
    'use strict';
    var IAPMenu; // need to get constructor later because there is a circular dependency here

    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var preventClick = settings.preventClick;
        var active = true;

        var text = new ShadowText({
            z: Globals.Layers.GUI,
            position: new Vector2(19, 8),
            text: CurrencyManager.getGems().toString(),
            font: 'font',
            fontSize: 11,
            fontColor: '#ffffff',
            align: 'left',
            textBaseline: 'middle',
            lineWidth: 0,
            maxWidth: 14,
            linebreaks: false
        });
        var clickable = new Clickable({
            onClick: function (data) {
                EventSystem.fire('preventScroll');
            },
            onHoldEnd: function (data) {
                if (!IAPMenu || !active) {
                    // not defined yet
                    return;
                }
                var grandParent;
                if (entity.parent && entity.parent.parent) {
                    grandParent = entity.parent.parent;
                }
                var iapMenu = new IAPMenu({
                    z: grandParent ? grandParent.z + 100 : Globals.WorldSelectLayers.IAP,
                    scrollTo: 'gems',
                    preventChests: settings.preventChests,
                    skipTutorial: settings.skipTutorial
                });
                Bento.objects.attach(iapMenu);
            }
        });

        var sprite = new Sprite({
            imageName: 'gui2/topbar/topbar__gems',
            originRelative: new Vector2(0, 0),
        });

        var eventListener = new EventListener({
            name: 'updateGems',
            onEvent: function (gems) {
                text.setText(gems.toString());
            }
        });
        var glow = new Glow({
            average: 1,
            amplitude: 0,
            glowSpeed: 0
        });
        var entity = new Entity({
            z: Globals.Layers.GUI,
            name: 'bargems',
            position: settings.position, //new Vector2(0, 1),
            components: [
                sprite,
                text,
                eventListener,
                glow
            ]
        }).extend({
            setActive: function (value) {
                active = value;
            },
            flash: function () {
                new Tween({
                    from: 1,
                    to: 0,
                    in: 5,
                    delay: 0,
                    ease: 'linear',
                    onUpdate: function (v, t) {
                        glow.alpha = v;
                    }
                });
            }
        });
        if (!preventClick) {
            entity.attach(clickable);
        }

        // try to get constructor asap
        if (!IAPMenu) {
            bento.require(['gui/menus/iap'], function (Module) {
                IAPMenu = Module;
            });
        }
        glow.alpha = 0;

        return entity;
    };
});