/**
 * stars gui element in top bar of world select
 * @moduleName Barstars
 */
bento.define('gui/worldselect/barstars', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'gui/shadowtext',
    'globals',
    'managers/save',
    'components/glow',
    'components/eventlistener',
    'components/blink',
    'gui/sparkleburst'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    ShadowText,
    Globals,
    SaveManager,
    Glow,
    EventListener,
    Blink,
    SparkleBurst
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var text = new ShadowText({
            z: Globals.Layers.GUI,
            position: new Vector2(17, 8),
            text: SaveManager.load('totalStars').toString(),
            font: 'font',
            fontSize: 11,
            fontColor: '#ffffff',
            align: 'left',
            textBaseline: 'middle',
            ySpacing: 0,
            lineWidth: 0,
            maxWidth: 15,
            maxHeight: 11
        });


        var sprite = new Sprite({
            imageName: 'gui2/topbar/topbar__stars',
            originRelative: new Vector2(0, 0),
        });

        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {
                glow.alpha = 0;
            },
            destroy: function (data) {},
            update: function (data) {},
            draw: function (data) {}
        };

        var glow = new Glow({
            average: 1,
            amplitude: 0,
            glowSpeed: 0
        });
        var entity = new Entity({
            z: Globals.Layers.GUI,
            name: 'barstars',
            position: new Vector2(0, 0),
            updateWhenPaused: Globals.PauseLevels.ALWAYS,
            components: [
                sprite,
                text,
                behavior,
                glow,
                new EventListener({
                    name: 'starsUpdated',
                    onEvent: function (data) {
                        text.setText(SaveManager.load('totalStars').toString());

                        entity.attach(new Blink({}));
                        entity.attach(new SparkleBurst({
                            offset: new Vector2(17, 8)
                        }));
                    }
                })
            ]
        }).extend({
            flash: function () {
                new Tween({
                    from: 1,
                    to: 0,
                    in: 5,
                    delay: 0,
                    ease: 'linear',
                    onUpdate: function (v, t) {
                        glow.alpha = v;
                    }
                });
            }
        });
        return entity;
    };
});