/**
 * Dojo in worldmap
 * @moduleName Dojo
 */
bento.define('gui/worldselect/dojo', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'globals',
    'managers/save',
    'managers/stats',
    'gui/buttons/startbutton'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Globals,
    SaveManager,
    StatsManager,
    StartButton
) {
    'use strict';
    var Dojo = function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var sprite = new Sprite({
            imageName: 'gui/worldselect/dojo',
            originRelative: new Vector2(0.5, 0.5)
        });
        var clicked;
        var unlockedMission = SaveManager.load('unlockedLevels');
        var clickable = new Clickable({
            onClick: function (data) {
                clicked = true;
            },
            onClickUp: function (data) {
                if (clicked && data.diffPosition.magnitude() < 3 && unlockedMission.length > 1) {
                    Dojo.goToDojo();
                    SaveManager.save('comboMeter', 0);
                }
                clicked = false;
            }
        });
        var entity = new Entity({
            z: Globals.WorldSelectLayers.LEVELS,
            name: 'dojoButton',
            position: settings.position,
            updateWhenPaused: 0,
            components: [
                sprite,
                clickable
            ]
        });
        return entity;
    };
    Dojo.goToDojo = function (settings) {
        // reset health of local player
        var startButton = new StartButton({
            mission: 'dojo',
            chosenDifficulty: 0,
            difficulty: 0,
            cost: 0,
            unlocks: [],
            rewards: []
        });
        startButton.start();
    };
    return Dojo;
});