/**
 * Level in worldmap
 * @moduleName Mission
 */
bento.define('gui/worldselect/mission', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'globals',
    'managers/save',
    'components/debug',
    'gui/shadowtext',
    'gui/levelstart/rewardspopup',
    'gui/menus/missiondetails',
    'components/drawinviewport',
    'gui/reward',
    'bento/components/nineslice',
    'components/multiscale',
    'components/sort',
    'managers/reward',
    'gui/menus/missiondetailssingle',
    'components/eventlistener'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Globals,
    SaveManager,
    Debug,
    ShadowText,
    RewardsPopup,
    MissionDetails,
    DrawInViewport,
    Reward,
    NineSlice,
    MultiScale,
    Sort,
    RewardManager,
    MissionDetailsSingle,
    EventListener
) {
    'use strict';
    var mapManager;
    var getMissionsJson = function () {
        var missionJson = Bento.assets.getJson('missions/missions');
        getMissionsJson = function () {
            return missionJson;
        };
        return missionJson;
    };
    var cache = {};
    var hasSpriteSheet = function (name) {
        // somehow it's possible that the worldmap icon is not loaded
        var cached = cache[name];
        if (cached !== true || cached !== false) {
            cached = Utils.hasAsset('spritesheets', name);
            // cache result
            cache[name] = cached;
        }
        return cached;
    };
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var tiled = settings.tiled || {};
        var tiledObject = tiled.object || {};
        var objectName = tiledObject.name || '';
        var nameProperties = Utils.nameToJson(objectName, true);
        var objectProperties = tiled.objectProperties || {};
        var mission = parseInt(nameProperties.lv) || nameProperties.lv;
        var icon = settings.icon || nameProperties.icon;
        /*var rewards = {};
        var enemyLevel = parseInt(nameProperties.enemyLv);
        var cost = parseInt(nameProperties.cost) || 2;
        var next = nameProperties.next || ''; // should be a string*/

        // note: getting mission details from missions.json now
        var missionJson = getMissionsJson();
        var missionData = missionJson[mission];
        if (!missionData) {
            console.log('Warning: mission ' + mission + ' does not exist');
            // return a dummy entity
            return new Entity({});
        }
        var enemyLevel = missionData.enemyLevel;
        var cost = missionData.cost || 2;
        var rewards = missionData.rewards || {};
        var next = missionData.next || nameProperties.next || '';
        var unlocks = [];
        var isLocked = true;
        var isActive = true;

        // what unlocks next: if mission is a number, the next mission is always mission+1
        // it is possible to unlock multiple next levels (for example, next = "3+4", unlocking 3 and 4)
        // mission doesn't have to be a number, it could also be a string (for example, "special1")
        if (Utils.isNumber(mission)) {
            unlocks.push(mission + 1);
        }
        // passing "null" as next will unlock nothing, even for numbered mission
        if (next === 'null') {
            unlocks = [];
        } else {
            // parse next to be unlocked
            if (Utils.isNumber(next)) {
                next = next.toString();
            }
            var nextLevels = next.split('+');
            Utils.forEach(nextLevels, function (nextLevel, i, l, breakLoop) {
                // try parsing it as number
                nextLevel = parseInt(nextLevel) || nextLevel;
                // add nextLevel to unlocks
                if (nextLevel && unlocks.indexOf(nextLevel) < 0) {
                    unlocks.push(nextLevel);
                }
            });
        }

        var text = Utils.isNumber(mission) ? new Counter({
            name: 'damageCounter',
            image: Bento.assets.getImage('gui/worldselect/numbers'),
            frameWidth: 12,
            frameHeight: 14,
            spacing: new Vector2(-4, 0),
            align: 'center',
            value: mission,
            position: new Vector2(0, -8),
        }) : new ShadowText({
            position: new Vector2(-1, -5),
            text: mission.toString(),
            font: 'font',
            fontSize: 16,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            maxWidth: 32,
            maxHeight: 16,
            lineWidth: 2,
            strokeStyle: '#d1b41c',
            shadowColor: '#806547',
            pixelStroke: true // workaround for Cocoon bug

        });
        var touchDownPos = null;
        var levelButton = new ClickButton({
            z: Globals.WorldSelectLayers.LEVELS,
            name: 'levelButton',
            imageName: 'gui/worldselect/mission',
            frameCountX: 1,
            frameCountY: 1,
            animations: {
                'down': {
                    frames: [0]
                },
                'up': {
                    frames: [0]
                }
            },
            onClick: function (data) {
                if (!isLocked || !isActive) {
                    return;
                }
                if (data && touchDownPos) {
                    if (Math.abs(touchDownPos.y - data.position.y) > 2) {
                        touchDownPos = null;
                        return;
                    }
                    touchDownPos = null;
                }
                Bento.audio.stopSound('sfx_button');
                Bento.audio.playSound('sfx_button');

                EventSystem.fire('dismissSensei');
                var missionDetails = new MissionDetailsSingle({
                    mission: mission,
                    cost: cost,
                    unlocks: unlocks,
                    enemyLevel: enemyLevel,
                    rewards: rewards
                });
                Bento.objects.attach(missionDetails);

            },
            onButtonDown: function (data) {
                if (!isLocked || !isActive) {
                    return;
                }
                if (!touchDownPos && data && data.position) {
                    touchDownPos = data.position;
                }
            }
        });

        // reward
        var attachRewardBalloon = function () {
            var randomizedReward = RewardManager.getReward(mission);
            var makeReward = function (index) {
                if (index === 1) {
                    return new Reward({
                        index: 0,
                        mission: mission,
                        position: new Vector2(0, -1),
                        item: rewards.reward1,
                        isOnline: false,
                        hideButton: true,
                        hideText: true
                    });
                } else if (index === 2) {
                    return new Reward({
                        index: 1,
                        mission: mission,
                        position: new Vector2(0, -1),
                        item: rewards.reward2,
                        isOnline: false,
                        hideButton: true,
                        hideText: true
                    });
                } else if (index === 3) {
                    return new Reward({
                        index: 2,
                        mission: mission,
                        position: new Vector2(0, -1),
                        item: randomizedReward.reward,
                        isOnline: false,
                        hideButton: true,
                        hideText: true
                    });
                }
            };
            var multiScale = new MultiScale({
                scales: {
                    unScale: new Vector2(1, 1),
                    popOut: new Vector2(1, 1)
                }
            });
            var isShowing = false;
            var unScale = {
                name: 'unscale',
                update: function () {
                    multiScale.unScale.x = 1 / entity.scale.x;
                    multiScale.unScale.y = multiScale.unScale.x;
                }
            };
            var showWhenClose = {
                name: 'showWhenClose',
                start: function () {
                    // move balloon to the left side if needed
                    if (!mapManager) {
                        mapManager = Bento.objects.get('mapManager');
                    }
                    if (mapManager) {
                        if (entity.position.x > mapManager.mapSize / 2 + 16) {
                            balloon.position.x = -balloon.position.x;
                            bottom.position.x = 8;
                            bottom.scale.x = 1;
                        }
                    }
                },
                update: function () {
                    var middle = viewport.getCenter();
                    if (middle.isCloserThan(entity.position, 64)) {
                        if (!isShowing) {
                            new Tween({
                                from: 0.2,
                                to: 1,
                                in: 20,
                                ease: 'elastic',
                                decay: 5,
                                oscillations: 1,
                                onUpdate: function (v, t) {
                                    multiScale.popOut.x = v;
                                    multiScale.popOut.y = v;
                                    balloon.alpha = 1;
                                }
                            });
                        }
                        isShowing = true;
                    } else {
                        if (isShowing) {
                            new Tween({
                                from: 1,
                                to: 0,
                                in: 20,
                                onUpdate: function (v, t) {
                                    balloon.alpha = v;
                                }
                            });
                        }
                        isShowing = false;
                    }
                }
            };
            var bottom = new Entity({
                name: 'bottom',
                position: new Vector2(-8, 15),
                scale: new Vector2(-1, 1),
                components: [
                    new Sprite({
                        imageName: 'gui/speechbub-map-bottom',
                        originRelative: new Vector2(0.5, 0)
                    })
                ]
            });
            var balloon = new Entity({
                name: 'rewardBalloon',
                position: new Vector2(32, -32),
                alpha: 0,
                components: [
                    new NineSlice({
                        imageName: 'gui/speechbub-map',
                        width: 38,
                        height: 38,
                        originRelative: new Vector2(0.5, 0.5)
                    }),
                    bottom,
                    multiScale,
                    unScale,
                    showWhenClose
                ]
            });

            // show reward2 if it's not collected yet
            var has2nd = (rewards.reward2 !== 'nothing/nothing');
            var has1st = (rewards.reward1 !== 'nothing/nothing');
            var collectedRewards = SaveManager.load('rewards-' + mission);
            var reward;
            if (has2nd && !collectedRewards[1]) {
                reward = makeReward(2);
            } else if (has1st && !collectedRewards[0]) {
                reward = makeReward(1);
            } else {
                reward = makeReward(3);
            }
            // fit into balloon
            if (reward.dimension.height > 32) {
                reward.scale.y = 32 / reward.dimension.height;
                reward.scale.x = reward.scale.y;
            } else if (reward.dimension.width > 32) {
                reward.scale.x = 32 / reward.dimension.width;
                reward.scale.y = reward.scale.x;
            }
            balloon.attach(reward);
            // don't click on it
            reward.removeByName('clickable');

            // update
            reward.attach(new EventListener({
                name: 'updateReward-' + mission,
                onEvent: function (data) {
                    reward.updateVisual();
                }
            }));

            entity.attach(balloon);
        };

        // load previously obtained stars from save
        var obtainedStars = SaveManager.load('missionStars-' + mission);
        // obtainedStars = 3;

        // add stars too an array and a set the image to star or emptystar depending on obtained stars
        var starList = [];
        var addStars = function () {
            var locked = isLocked ? '' : '-locked';
            for (var i = 0; i < 3; i++) {
                starList.push(new Entity({
                    z: 0,
                    name: 'star',
                    components: [
                        new Sprite({
                            imageName: i < obtainedStars ? 'gui/worldselect/star' + locked : 'gui/worldselect/emptystar' + locked,
                            origin: new Vector2(10, 11)
                        })
                    ]
                }));
            }
            // set positions of the stars
            starList[0].position = new Vector2(-18, 14);
            starList[1].position = new Vector2(0, 17);
            starList[2].position = new Vector2(18, 14);
            // attach
            entity.attach(starList[0]);
            entity.attach(starList[1]);
            entity.attach(starList[2]);
        };

        //pulsing behavior
        var pulser = {
            name: 'pulsate',
            update: function (data) {
                entity.scale.x = 1 + 0.05 * Math.sin(entity.timer / 20);
                entity.scale.y = entity.scale.x;
            }
        };

        //normal behavior
        var behavior = {
            name: 'behavior',
            start: function () {
                // correction for origin because entity itself has no sprite
                // entity.position = new Vector2(settings.tiled.position.x + 20.5, settings.tiled.position.y - 13);
                entity.position.x += 20.5;
                entity.position.y -= 13;

                // set rewards object
                /*rewards.reward1 = objectProperties.reward1 || 'health-boy';
                rewards.reward2 = objectProperties.reward2 || 'attack-boy';
                rewards.reward3 = objectProperties.reward3 || 'skill-boy';
                rewards.chance1 = objectProperties.chance1 || 0.125;
                rewards.chance2 = objectProperties.chance2 || 0.125;
                rewards.chance3 = objectProperties.chance3 || 0.125;
                rewards.amount1 = objectProperties.amount1 || 1;
                rewards.amount2 = objectProperties.amount2 || 1;
                rewards.amount3 = objectProperties.amount3 || 1;*/

                if (entity.canPlayLevel()) {
                    attachRewardBalloon();
                }
            }
        };
        // component that adds a boss icon to the button
        var addIcon = {
            name: 'addIcon',
            iconEntity: null,
            hasIcon: function () {
                // obtainedStars < 0 indicates the player hasnt beaten the mission yet
                return icon && obtainedStars < 0;
            },
            start: function () {
                if (addIcon.hasIcon() && hasSpriteSheet('worldselect/' + icon)) {
                    addIcon.iconEntity = new Entity({
                        z: Globals.WorldSelectLayers.LEVELS,
                        name: 'bossIcon',
                        position: (settings.iconPosition || entity.position).add(new Vector2(0, -8)),
                        scale: new Vector2(-1, 1),
                        components: [
                            new Sprite({
                                spriteSheet: 'worldselect/' + icon
                            })
                        ]
                    });
                    Bento.objects.attach(addIcon.iconEntity);
                }
            },
            destroy: function () {
                if (addIcon.hasIcon() && addIcon.iconEntity) {
                    addIcon.iconEntity.removeSelf();
                }
            }
        };

        var entity = new Entity({
            z: Globals.WorldSelectLayers.LEVELS,
            name: 'missionButton',
            family: ['missionButtons'],
            missionSettings: settings,
            components: [
                levelButton,
                behavior,
                addIcon,
                new DrawInViewport({
                    dimension: 48
                }),
                new Sort({
                    baseLayer: Globals.WorldSelectLayers.LEVELS
                })
            ]
        });

        entity.extend({
            mission: mission,
            canPlayLevel: function () {
                var unlocked = SaveManager.load('unlockedLevels');
                if (unlocked.indexOf(mission) > -1) {
                    return true;
                } else {
                    return false;
                }
            },
            setActive: function (value) {
                isActive = value;
            },
            getIcon: function () {
                return icon;
            }
        });

        if (!entity.canPlayLevel()) {
            isLocked = false;
            //entity.alpha = 0.5;
            levelButton.remove(levelButton.getComponent('sprite'));
            levelButton.attach(new Sprite({
                originRelative: new Vector2(0.5, 0.5),
                imageName: 'gui/worldselect/lockedmission'
            }));
        } else {
            if (obtainedStars < 0) {
                // only pulse when level has not been played yet
                entity.attach(pulser);
            }
            entity.attach(text);
        }
        if (obtainedStars >= 0) {
            addStars();
        }

        return entity;
    };
});