/**
 * Bar at the top of the worldselect screen containing energy, stars, gems and coin information.
 * @moduleName Topbar
 */
bento.define('gui/worldselect/topbar', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'globals',
    'components/anchor',
    'gui/worldselect/barcoins',
    'gui/worldselect/bargems',
    'gui/worldselect/barstars',
    'gui/worldselect/barenergy',
    'components/eventlistener'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Globals,
    Anchor,
    Barcoins,
    Bargems,
    Barstars,
    Barenergy,
    EventListener
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var padding;

        var align = function (entity, isPortrait, component) {
            entity.position.x = 0;
            entity.position.y = 0;
            // background.scale.x = viewport.width;
            background.position.x = viewport.width / 2;
            gems.position.x = viewport.width / 2 - 8;
            coins.position.x = viewport.width / 2 + 36;
            stars.position.x = viewport.width / 2 - 42;
            energy.position.x = viewport.width / 2 - 84;
        };

        var background = new Entity({
            name: 'background',
            alpha: Anchor.isWideScreen() ? 0.75 : 1,
            components: [
                new Sprite({
                    imageName: 'gui2/topbar/topbar_backboard',
                    originRelative: new Vector2(0.5, 0)
                })
            ]
        });

        var stars = new Barstars({});
        var gems = new Bargems({
            position: new Vector2(0, 0),
            preventClick: settings.preventClick,
            preventChests: settings.preventChests,
            skipTutorial: settings.skipTutorial
        });
        var coins = new Barcoins({
            preventClick: settings.preventClick,
            preventChests: settings.preventChests,
            skipTutorial: settings.skipTutorial
        });
        var energy = new Barenergy({});

        var anchor = new Anchor({
            offset: new Vector2(0, 8),
            onComplete: align
        });

        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {},
            destroy: function (data) {},
            update: function (data) {},
            draw: function (data) {}
        };
        var shakeStars = new EventListener({
            name: 'shakeStars',
            onEvent: function () {
                new Tween({
                    from: 2,
                    to: 0,
                    in: 40,
                    ease: 'elastic',
                    decay: 2, // for 'exponential' and 'elastic'
                    oscillations: 5, // for 'elastic', 'sin' and 'cos'
                    onUpdate: function (v, t) {
                        stars.transform.x = v;
                    },
                    onComplete: function () {
                        stars.transform.x = 0;
                    }
                });
            }
        });
        var entity = new Entity({
            z: settings.z || Globals.WorldSelectLayers.TOPBAR,
            name: 'topBar',
            float: true,
            updateWhenPaused: Bento.objects.isPaused(),
            components: [
                anchor,
                background,
                coins,
                gems,
                stars,
                energy,
                behavior,
                shakeStars
            ]
        });

        entity.extend({
            setCoins: function (value) {
                coins.setCoins(value);
            },
            flashStars: function () {
                stars.flash();
            },
            flashGems: function () {
                gems.flash();
            },
            flashEnergy: function () {
                energy.flash();
            },
            flashCoin: function () {
                coins.flash();
            },
            setActive: function (value) {
                coins.setActive(value);
                gems.setActive(value);
            }
        });

        if (Anchor.isWideScreen()) {
            padding = new Sprite({
                imageName: 'gui2/topbar/topbar_padding',
                originRelative: new Vector2(0.5, 1)
            });
            background.attach(padding);
        }

        return entity;
    };
});