/**
 * This manager stores data about the buffs received from current equipment and consumables
 * @moduleName BuffManager
 */
bento.define('managers/buff', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'managers/inventory',
    'managers/save',
    'managers/allies'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween,
    InventoryManager,
    SaveManager,
    AlliesManager
) {
    'use strict';

    var applyBuffs = function (key) {
        var effect = AlliesManager.getEffectiveness(key);
        switch (key) {
        case 'rat':
            manager.current.health += effect.value;
            break;
        case 'bigrat':
            manager.current.attack += effect.value;
            break;
        case 'weasel':
            manager.current.powerUpBoost += effect.value;
            break;
        case 'raccoon':
            manager.current.throwStrength += effect.value;
            break;
        case 'pig':
            manager.current.extraCoins += effect.value;
            break;
        case 'ratguy':
            manager.current.skill += effect.value;
            break;
        case 'ratgirl':
            manager.current.weaponDurability -= effect.value;
            break;
        case 'gangster':
            manager.current.enviromentDamage *= effect.value;
            break;
        case 'punker':
            manager.current.comboMeterStart += effect.value;
            break;
        case 'punkgirl':
            manager.current.weaponPower += effect.value;
            break;
        case 'mrpigeon':
            manager.current.dashAttack += effect.value;
            break;
        case 'cat':
            manager.current.comboEnd += effect.value;
            break;
        case 'dog':
            manager.current.chargeAttack += effect.value;
            break;
        case 'twerka':
            manager.current.revive += effect.value;
            break;
        case 'ratboss':
            manager.current.comboMeterExtra += effect.value;
            break;
        case 'pigchef':
            manager.current.invincibleTime += effect.value;
            break;
        case 'gangstaboss':
            manager.current.cancelDamage += effect.value;
            break;
        default:
            Utils.log('buffs invalid equipment key ' + key.toString());
        }
    };

    var loadBuffs = function () {
        // reset the buffs back to default
        // console.log('resetbuffs');
        manager.current = copyBuffs();

        var equipped = AlliesManager.getAllies();

        if (equipped[0]) {
            applyBuffs(equipped[0]);
        }
        if (equipped[1]) {
            applyBuffs(equipped[1]);
        }

        manager.current.isSet = true;
    };

    var copyBuffs = function () {
        return JSON.parse(JSON.stringify(buffs));
    };

    var buffs = {
        // consumable buffs - not permanent
        attackBonus: 1,
        skillBonus: 1,
        chargeDuration: 20,
        doubleCoins: false,

        //non-consumable buffs - permanent
        attack: 0,
        health: 0,
        skill: 0,
        // chance based (range 0-1)
        comboEnd: 0,
        chargeAttack: 0,
        dashAttack: 0,
        knockback: 0,
        //
        punchSpeed: 1,
        weaponPower: 0,
        chargeSpeed: 20,
        weaponDurability: 1,
        dashSpeed: 1,
        walkSpeed: 1,
        revive: 0,
        throwStrength: 0,
        luck: 0,
        knockDown: true,
        enviromentDamage: 1,
        extraCoins: 0,
        crushDestructables: false,
        powerUpBoost: 0,
        invincibleTime: 0,
        cancelDamage: 0,
        comboMeterStart: 0,
        comboMeterExtra: 0,

        //for dev reset
        isSet: false
    };

    var manager = {
        current: copyBuffs(),
        loadBuffs: loadBuffs,
        // roll the dice for a critical hit
        rollCritical: function (name) {
            var effect = 1;
            var roll = Utils.getRandomFloat(1);
            if (roll < manager.current[name]) {
                effect = 2;
            }
            return effect;
        }
    };

    return manager;
});