/**
 * Combo manager. Keeps track of the combo of each player
 * @moduleName ComboManager
 */
bento.define('managers/combo', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween
) {
    'use strict';
    /**
     * Describe your settings object parameters
     * @param {Object} settings
     */
    var ComboManager = function () {
        var combos = [];
        var comboTimers = [];

        var onPunchHit = function (e) {
            if (e.punch.target.indexOf('player') > -1) {
                if (e.victim && e.victim.isPlayer) {
                    resetCombo(e.victim.playerId);
                }
            }

            if (e.punch.target.indexOf('enemy') > -1) {
                if (e.attacker && e.attacker.isPlayer && !e.attacker.isRemote) {
                    increaseCombo(e.attacker.playerId, 1);
                }
            }
        };

        var resetCombo = function (player) {
            var delta = -combos[player];

            EventSystem.fire('quest-10combo', {
                count: combos[player],
            });

            EventSystem.fire('quest-20combo', {
                count: combos[player],
            });

            //check if the current combo exceeds the 'Crazy Combo' and 'Super Combo' Achievement limits
            if (combos[player] >= 25) {
                Utils.postAchievement("CgkI9NqdkoAGEAIQCA");
            }
            if (combos[player] >= 50) {
                Utils.postAchievement("CgkI9NqdkoAGEAIQCQ");
            }

            combos[player] = 0;
            comboTimers[player] = -1;

            EventSystem.fire('comboChange', {
                player: player,
                reset: true,
                new: 0,
                delta: delta
            });
        };

        var increaseCombo = function (player, amount) {
            if (!combos[player]) {
                combos[player] = 0;
                comboTimers[player] = 0;
            }

            combos[player] += amount;
            comboTimers[player] = 60 * 1.2; //TODO

            EventSystem.fire('comboChange', {
                player: player,
                reset: false,
                new: combos[player],
                delta: amount
            });
        };

        EventSystem.clear('punchHit');
        // EventSystem.clear('preUpdate');
        // EventSystem.on('preUpdate', onPreUpdate);

        var manager = {
            name: 'comboManager',
            start: function () {
                EventSystem.on('punchHit', onPunchHit);
            },
            destroy: function () {
                EventSystem.clear('punchHit');
            },
            onPunchHit: onPunchHit,
            update: function () {
                var ii;
                for (ii = 0; ii < comboTimers.length; ii++) {
                    if (comboTimers[ii] > 0) {
                        comboTimers[ii]--;
                        if (comboTimers[ii] <= 0) {
                            resetCombo(ii);
                        }
                    }
                }
            },
            getCombos: function () {
                return [].concat(combos);
            },
            setPlayerCount: function (count) {
                /*
                combos = [];
                comboTimers = [];

                var ii;
                for (ii = 0; ii < count; ii++) {
                    combos[ii] = 0;
                    comboTimers[ii] = -1;
                }
                */
            }
        };
        return manager;
    };

    return ComboManager;
});