/**
 * Currency manager
 * @moduleName CurrencyManager
 */
bento.define('managers/currency', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'managers/save',
    'modules/analytics',
    'modules/googleplay',
    'modules/firebase'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween,
    SaveManager,
    Analytics,
    GooglePlay,
    FireBase
) {
    'use strict';
    /**
     * Add to the amount of coins the player has
     */
    var addCoins = function (amount, notSource) {
        amount = parseInt(amount) || 0;
        var coins = SaveManager.load('coins');
        coins += amount;
        SaveManager.save('coins', coins);
        EventSystem.fire('updateCoins', coins);

        if (amount > 20000) {
            // abnormal coin source
            Bento.saveState.save('abnormalCoinSource', true);
            Bento.saveState.save('cheatSuspect', true);
            if (Analytics) {
                Analytics.event('CheatSuspect', 'abnormalCoinSource');
            }
        }

        if (Bento.saveState.load('cheatSuspect', false)) {
            Analytics.event('SucpectSourceSinks', 'sourceCoins', amount);
            return;
        }

        if (!notSource && amount > 0) {
            Analytics.event('SourceSinks', 'sourceCoins', amount);
            GooglePlay.submitEvent('CgkI9NqdkoAGEAIQAg', amount);
        }
        FireBase.setUserProperty('coins', coins);

        if (amount < 0) {
            Utils.log('WARNING: use spendCoins for negative coins');
        }
    };
    /**
     * Get the ampount of coins the player owns
     */
    var getCoins = function () {
        var coins = SaveManager.load('coins');
        return coins;
    };
    /**
     * Add to the amount of coins the player has
     */
    var spendCoins = function (amount, itemName) {
        amount = parseInt(amount) || 0;
        var coins = SaveManager.load('coins');
        if (amount > coins) {
            Utils.log("WARNING: player is trying to spend more than amount of coins");
        }
        coins -= amount;
        SaveManager.save('coins', coins);
        SaveManager.add('coinSink', amount);
        EventSystem.fire('updateCoins', coins);

        EventSystem.fire('quest-spendCoins', {
            count: amount
        });

        if (amount > 0) {
            Analytics.event('SourceSinks', 'sinkCoins', amount);
            GooglePlay.submitEvent('CgkI9NqdkoAGEAIQAw', amount);
            if (itemName) {
                // Standard Firebase event for tracking in-game spenditure
                FireBase.logEvent(FireBase.Events.SPEND_VIRTUAL_CURRENCY, {
                    item_name: itemName,
                    value: amount,
                    virtual_currency_name: 'coins'
                });
            }
        }
        FireBase.setUserProperty('coins', coins);
        FireBase.setUserProperty('coinSink', SaveManager.load('coinSink'));
    };
    /**
     * Can player spend this?
     */
    var canSpend = function (cost) {
        var coins = SaveManager.load('coins');
        return coins >= cost;
    };

    /**
     * Add to the amount of gems the player has
     */
    var addGems = function (amount, notSource) {
        amount = parseInt(amount) || 0;
        var gems = SaveManager.load('gems');
        gems += amount;
        SaveManager.save('gems', gems);
        EventSystem.fire('updateGems', gems);

        if (!notSource && amount > 0) {
            Analytics.event('SourceSinks', 'source', amount);
            FireBase.logEvent('source', {
                value: amount
            });
        }
        if (amount < 0) {
            Utils.log('WARNING: use spendGems for negative gems');
        }
        FireBase.setUserProperty('gems', gems);
    };
    /**
     * Get the ampount of gems the player owns
     */
    var getGems = function () {
        var gems = SaveManager.load('gems');
        return gems;
    };
    /**
     * Add to the amount of gems the player has
     */
    var spendGems = function (amount, itemName) {
        amount = parseInt(amount) || 0;
        var gems = SaveManager.load('gems');
        if (amount > gems) {
            Utils.log("WARNING: player is trying to spend more than amount of gems");
        }
        gems -= amount;
        SaveManager.save('gems', gems);
        SaveManager.add('gemSink', amount);
        EventSystem.fire('updateGems', gems);

        if (amount > 0) {
            Analytics.event('SourceSinks', 'sink', amount);
            if (itemName) {
                // Standard Firebase event for tracking in-game spenditure
                FireBase.logEvent(FireBase.Events.SPEND_VIRTUAL_CURRENCY, {
                    item_name: itemName,
                    value: amount,
                    virtual_currency_name: 'gems'
                });
            }
        }
        FireBase.setUserProperty('gems', gems);
        FireBase.setUserProperty('gemSink', SaveManager.load('gemSink'));
    };
    /**
     * Can player spend this?
     */
    var canSpendGems = function (cost) {
        var gems = SaveManager.load('gems');
        return gems >= cost;
    };

    /*
     * Get the amount of token items of given type
     */
    var getToken = function (character, type) {
        var token = SaveManager.load('token-' + character);
        return token[type];
    };

    /**
     * Add to the amount of tokens the player has
     */
    var addToken = function (character, type, amount) {
        Utils.log('WARNING: Deprecated addToken');
        var token = SaveManager.load('token-' + character);
        token[type] += amount;
        SaveManager.save('token-' + character, token);
    };

    /*
     * Spend the amount of token items of given type
     * returns boolean
     */
    var spendToken = function (character, type, value) {
        Utils.log('WARNING: Deprecated spendToken');
        var token = SaveManager.load('token-' + character);
        if (token[type] >= value) {
            token[type] -= value;
            SaveManager.save('token-' + character, token);
            return true;
        }
        return false;
    };

    /*
     * Get token object
     */
    var getTokens = function (character) {
        var token = SaveManager.load('token-' + character);
        return token;
    };

    /*
     * Spend token object
     */
    var spendTokens = function (character, health, attack, skill) {
        Utils.log('WARNING: Deprecated spendTokens');
        var token = SaveManager.load('token-' + character);
        if (token.health >= health &&
            token.attack >= attack &&
            token.skill >= skill) {
            token.health -= health;
            token.attack -= attack;
            token.skill -= skill;
            SaveManager.save('token-' + character, token);
            return true;
        }
        return false;
    };

    var manager = {
        addCoins: addCoins,
        getCoins: getCoins,
        spendCoins: spendCoins,
        canSpend: canSpend,

        addGems: addGems,
        getGems: getGems,
        spendGems: spendGems,
        canSpendGems: canSpendGems,

        // DEPRECATED, use InventoryManager for tokens now
        getToken: getToken,
        getTokens: getTokens,
        spendToken: spendToken,
        spendTokens: spendTokens,
        addToken: addToken,

        // keys
        addKeys: function (value) {
            SaveManager.add('keys', value);
        },
        getKeys: function () {
            return SaveManager.load('keys');
        },
        canSpendKeys: function (value) {
            return SaveManager.load('keys') >= value;
        },
        spendKeys: function (amount) {
            var keys = SaveManager.load('keys');
            if (amount > keys) {
                Utils.log("WARNING: player is trying to spend more than amount of keys");
            }
            keys -= amount;
            SaveManager.save('keys', keys);
            EventSystem.fire('updateKeys', keys);
        },

        // tickets
        addTickets: function (value) {
            SaveManager.add('tickets', value);
        },
        getTickets: function () {
            return SaveManager.load('tickets');
        },
        canSpendTickets: function (value) {
            return SaveManager.load('tickets') >= value;
        },
        spendTickets: function (amount) {
            var tickets = SaveManager.load('tickets');
            if (amount > tickets) {
                Utils.log("WARNING: player is trying to spend more than amount of tickets");
            }
            tickets -= amount;
            SaveManager.save('tickets', tickets);
            EventSystem.fire('updateTickets', tickets);
        },

        reset: function () {
            // DEPRECATED
        }
    };

    return manager;
});