/**
 * Item Manager
 *
 * The item manager is responsible for turning an item key of the format "[category]/[name]" into usable metadata
 *
 * @moduleName ItemManager
 */
bento.define('managers/item', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'bento/components/sprite',
    'entities/weapon',
    'modules/localization',
    'components/invincible',
    'entities/punch',
    'managers/buff',
    'gui/consumablepopup',
    'gui/sparkleburst',
    'managers/save',
    'components/flames',
    'globals',
    'gui/main/comboorb'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween,
    Sprite,
    Weapon,
    Localization,
    Invincible,
    Punch,
    BuffManager,
    ConsumablePopup,
    SparkleBurst,
    SaveManager,
    Flames,
    Globals,
    ComboOrb
) {
    'use strict';
    var itemHandlers = {
        // candy - 20% heal
        "consumables/candy": function (key, player, position) {
            player.getComponent('punchable', function (punchable) {
                if (punchable.getRelativeHealth() < 1) {
                    punchable.addHealth(Math.ceil(punchable.getMaxHealth() * 0.20));
                }
            });
            player.attach(new SparkleBurst({
                position: new Vector2(0, -16)
            }));
            EventSystem.fire('quest-consumable', {
                count: 1
            });
            return true;
        },
        // banana - 20% heal
        "consumables/banana": function (key, player, position) {
            player.getComponent('punchable', function (punchable) {
                if (punchable.getRelativeHealth() < 1) {
                    punchable.addHealth(Math.ceil(punchable.getMaxHealth() * 0.20));
                }
            });
            player.attach(new SparkleBurst({
                position: new Vector2(0, -16)
            }));
            EventSystem.fire('quest-consumable', {
                count: 1
            });
            return true;
        },
        // apple - 20% heal
        "consumables/apple": function (key, player, position) {
            player.getComponent('punchable', function (punchable) {
                if (punchable.getRelativeHealth() < 1) {
                    punchable.addHealth(Math.ceil(punchable.getMaxHealth() * 0.20));
                }
            });
            player.attach(new SparkleBurst({
                position: new Vector2(0, -16)
            }));
            EventSystem.fire('quest-consumable', {
                count: 1
            });
            return true;
        },
        // strawberry - 20% heal
        "consumables/strawberry": function (key, player, position) {
            player.getComponent('punchable', function (punchable) {
                if (punchable.getRelativeHealth() < 1) {
                    punchable.addHealth(Math.ceil(punchable.getMaxHealth() * 0.20));
                }
            });
            player.attach(new SparkleBurst({
                position: new Vector2(0, -16)
            }));
            EventSystem.fire('quest-consumable', {
                count: 1
            });
            return true;
        },
        // hamburger - 25% heal
        "consumables/burger": function (key, player, position) {
            player.getComponent('punchable', function (punchable) {
                if (punchable.getRelativeHealth() < 1) {
                    punchable.addHealth(Math.ceil(punchable.getMaxHealth() * 0.25));
                }
            });
            player.attach(new SparkleBurst({
                position: new Vector2(0, -16)
            }));
            EventSystem.fire('quest-consumable', {
                count: 1
            });
            return true;
        },
        // pizza - 25% heal
        "consumables/pizza": function (key, player, position) {
            player.getComponent('punchable', function (punchable) {
                if (punchable.getRelativeHealth() < 1) {
                    punchable.addHealth(Math.ceil(punchable.getMaxHealth() * 0.25));
                }
            });
            player.attach(new SparkleBurst({
                position: new Vector2(0, -16)
            }));
            EventSystem.fire('quest-consumable', {
                count: 1
            });
            return true;
        },
        // fries - 25% heal
        "consumables/fries": function (key, player, position) {
            player.getComponent('punchable', function (punchable) {
                if (punchable.getRelativeHealth() < 1) {
                    punchable.addHealth(Math.ceil(punchable.getMaxHealth() * 0.25));
                }
            });
            player.attach(new SparkleBurst({
                position: new Vector2(0, -16)
            }));
            EventSystem.fire('quest-consumable', {
                count: 1
            });
            return true;
        },
        // sushi - 25% heal
        "consumables/sushi": function (key, player, position) {
            player.getComponent('punchable', function (punchable) {
                if (punchable.getRelativeHealth() < 1) {
                    punchable.addHealth(Math.ceil(punchable.getMaxHealth() * 0.25));
                }
            });
            player.attach(new SparkleBurst({
                position: new Vector2(0, -16)
            }));
            EventSystem.fire('quest-consumable', {
                count: 1
            });
            return true;
        },
        // chicken/roast - 50% heal
        "consumables/chicken": function (key, player, position) {
            player.getComponent('punchable', function (punchable) {
                if (punchable.getRelativeHealth() < 1) {
                    punchable.addHealth(Math.ceil(punchable.getMaxHealth() * 0.5));
                }
            });
            player.attach(new SparkleBurst({
                position: new Vector2(0, -16)
            }));
            EventSystem.fire('quest-consumable', {
                count: 1
            });
            return true;
        },
        // fish - 50% heal
        "consumables/fish": function (key, player, position) {
            player.getComponent('punchable', function (punchable) {
                if (punchable.getRelativeHealth() < 1) {
                    punchable.addHealth(Math.ceil(punchable.getMaxHealth() * 0.5));
                }
            });
            player.attach(new SparkleBurst({
                position: new Vector2(0, -16)
            }));
            EventSystem.fire('quest-consumable', {
                count: 1
            });
            return true;
        },
        // steak - 50% heal
        "consumables/steak": function (key, player, position) {
            player.getComponent('punchable', function (punchable) {
                if (punchable.getRelativeHealth() < 1) {
                    punchable.addHealth(Math.ceil(punchable.getMaxHealth() * 0.5));
                }
            });
            player.attach(new SparkleBurst({
                position: new Vector2(0, -16)
            }));
            EventSystem.fire('quest-consumable', {
                count: 1
            });
            return true;
        },
        // bowl of rice - 50% heal
        "consumables/ricebowl": function (key, player, position) {
            player.getComponent('punchable', function (punchable) {
                if (punchable.getRelativeHealth() < 1) {
                    punchable.addHealth(Math.ceil(punchable.getMaxHealth() * 0.5));
                }
            });
            player.attach(new SparkleBurst({
                position: new Vector2(0, -16)
            }));
            EventSystem.fire('quest-consumable', {
                count: 1
            });
            return true;
        },
        // combo - 25%
        "consumables/combo25": function (key, player, position) {
            // the player who used it must be the local player
            if (!player.isRemote) {
                var i = 0;
                Utils.repeat(5, function () {
                    var j = i++;
                    Utils.timeout(j * 2, function () {
                        var viewport = Bento.getViewport().getCorner(Rectangle.TOPLEFT);
                        Bento.objects.attach(new ComboOrb({
                            position: position.subtract(viewport).add(new Vector2(0, -16)),
                            last: j === 4,
                            onComplete: function () {
                                Bento.objects.get('healthBar', function (healthBar) {
                                    healthBar.fillCombobar(Math.round(0.25 * Globals.maxComboPoints));
                                    healthBar.flashCombo();
                                });
                            }
                        }));
                    });
                });
            }
            player.attach(new SparkleBurst({
                position: new Vector2(0, -16)
            }));
            EventSystem.fire('quest-consumable', {
                count: 1
            });
            return true;
        },
        // combo - 50%
        "consumables/combo50": function (key, player, position) {
            // the player who used it must be the local player
            if (!player.isRemote) {
                player.attach(new SparkleBurst({
                    position: new Vector2(0, -16)
                }));
                var i = 0;
                Utils.repeat(5, function () {
                    var j = i++;
                    Utils.timeout(j * 2, function () {
                        var viewport = Bento.getViewport().getCorner(Rectangle.TOPLEFT);
                        Bento.objects.attach(new ComboOrb({
                            position: position.subtract(viewport).add(new Vector2(0, -16)),
                            last: j === 4,
                            onComplete: function () {
                                Bento.objects.get('healthBar', function (healthBar) {
                                    healthBar.fillCombobar(Math.round(0.5 * Globals.maxComboPoints));
                                    healthBar.flashCombo();
                                });
                            }
                        }));
                    });
                });
            }
            player.attach(new SparkleBurst({
                position: new Vector2(0, -16)
            }));
            EventSystem.fire('quest-consumable', {
                count: 1
            });
            return true;
        },
        // combo - 25%
        "consumables/combo100": function (key, player, position) {
            // the player who used it must be the local player
            if (!player.isRemote) {
                player.attach(new SparkleBurst({
                    position: new Vector2(0, -16)
                }));
                var i = 0;
                Utils.repeat(5, function () {
                    var j = i++;
                    Utils.timeout(j * 2, function () {
                        var viewport = Bento.getViewport().getCorner(Rectangle.TOPLEFT);
                        Bento.objects.attach(new ComboOrb({
                            position: position.subtract(viewport).add(new Vector2(0, -16)),
                            last: j === 4,
                            onComplete: function () {
                                Bento.objects.get('healthBar', function (healthBar) {
                                    healthBar.fillCombobar(Math.round(1 * Globals.maxComboPoints));
                                    healthBar.flashCombo();
                                });
                            }
                        }));
                    });
                });
            }
            player.attach(new SparkleBurst({
                position: new Vector2(0, -16)
            }));
            EventSystem.fire('quest-consumable', {
                count: 1
            });
            return true;
        },
        // flaming fist - super attack (AOE screen)
        "consumables/superattack": function (key, player, position) {
            console.log('super attack!');
            Bento.objects.getByFamily('enemy', function (enemies) {
                var playerStateMachine = player.getComponent('stateMachine');
                Utils.forEach(enemies, function (enemy, i, l, breakLoop) {
                    enemy.getComponent('enemyController', function (enemyController) {
                        // hit all active enemies, only those on screen isn't fair to portrait players.
                        if (enemyController.active) {
                            enemy.getComponent('punchable', function (punchable) {
                                var distanceMultiplier = Math.max((1 - Math.abs(enemy.position.x - player.position.x) / 80), 0.5);
                                punchable.hit(playerStateMachine.data.createPunch({
                                    target: ['enemy', 'destructible'],
                                    attack: playerStateMachine.data.stats.attack,
                                    stamina: 1,
                                    skill: Infinity,
                                    multiplier: 1 + distanceMultiplier,
                                    direction: Utils.sign(enemy.position.x - player.position.x) || 1,
                                    pushBack: new Vector2(3 * distanceMultiplier, distanceMultiplier * 3),
                                    onHit: Punch.getDefaultCallback({
                                        entity: player,
                                        effect: 'effects/hitfx1',
                                        sfx: 'sfx_dashattack-hit',
                                        taptic: 'heavy'
                                    })
                                }));
                            });
                        }
                    });
                });
            });
            EventSystem.fire('quest-consumable', {
                count: 1
            });
            return true;
        },
        // piggybank - coin doubler for 2 minutes
        "consumables/coindoubler": function (key, player, position) {
            console.log('coin doubler');
            // check to see if one already exists and add the seconds to the new one
            /*var existing = Bento.objects.get(key);
            var addTime = 0;
            if (existing) {
                addTime = 120 - existing.getTime();
                Bento.objects.remove(existing);
            }
            effectTween(120 + addTime, player, key, null, function (p, t) {
                p.doubleCoins = true;
            });*/
            return false;
        },
        // sword orb - attack +25% for 2 minutes
        "consumables/attackbonus1": function (key, player, position) {
            console.log('+10% attack');
            effectTween(60 + BuffManager.current.powerUpBoost, player, key, null, function (p, t) {
                p.attackBonus += 0.25;
                p.skillBonus += 2;
                p.getComponent('punchable', function (punchable) {
                    punchable.addDefenseBonus(0.9);
                });
                if (!p.getComponent('flames')) {
                    p.attach(new Flames({
                        color: 'yellow'
                    }));
                }

            });
            player.attach(new SparkleBurst({
                position: new Vector2(0, -16)
            }));
            EventSystem.fire('quest-powerup', {
                count: 1
            });
            EventSystem.fire('quest-consumable', {
                count: 1
            });
            return true;
        },
        // sword orb - attack +50% for 2 minutes
        "consumables/attackbonus2": function (key, player, position) {
            console.log('+15% attack');
            effectTween(60 + BuffManager.current.powerUpBoost, player, key, null, function (p, t) {
                p.attackBonus += 0.5;
                p.skillBonus += 3;
                p.getComponent('punchable', function (punchable) {
                    punchable.addDefenseBonus(0.8);
                });
                if (!p.getComponent('flames')) {
                    p.attach(new Flames({
                        color: 'yellow'
                    }));
                }

            });
            player.attach(new SparkleBurst({
                position: new Vector2(0, -16)
            }));
            EventSystem.fire('quest-powerup', {
                count: 1
            });
            EventSystem.fire('quest-consumable', {
                count: 1
            });
            return true;
        },
        // sword orb - attack +100% for 2 minutes
        "consumables/attackbonus3": function (key, player, position) {
            console.log('+25% attack');
            effectTween(60 + BuffManager.current.powerUpBoost, player, key, null, function (p, t) {
                p.attackBonus += 1;
                p.skillBonus += 4;
                p.getComponent('punchable', function (punchable) {
                    punchable.addDefenseBonus(0.7);
                });
                if (!p.getComponent('flames')) {
                    p.attach(new Flames({
                        color: 'yellow'
                    }));
                }

            });
            player.attach(new SparkleBurst({
                position: new Vector2(0, -16)
            }));
            EventSystem.fire('quest-powerup', {
                count: 1
            });
            EventSystem.fire('quest-consumable', {
                count: 1
            });
            return true;
        },
        // shield orb - defense +25% for 2 minutes
        "consumables/defensebonus1": function (key, player, position) {
            console.log('+25% defense');
            effectTween(60 + BuffManager.current.powerUpBoost, player, key, null, function (p, t) {
                p.getComponent('punchable', function (punchable) {
                    punchable.addDefenseBonus(0.75);
                });
                p.attackBonus += 0.1;
                p.skillBonus += 2;
                if (!p.getComponent('flames')) {
                    p.attach(new Flames({
                        color: 'red'
                    }));
                }
            });
            player.attach(new SparkleBurst({
                position: new Vector2(0, -16)
            }));
            EventSystem.fire('quest-powerup', {
                count: 1
            });
            EventSystem.fire('quest-consumable', {
                count: 1
            });
            return true;
        },
        // shield orb - defense +5% for 2 minutes
        "consumables/defensebonus2": function (key, player, position) {
            effectTween(60 + BuffManager.current.powerUpBoost, player, key, null, function (p, t) {
                p.attackBonus += 0.2;
                p.getComponent('punchable', function (punchable) {
                    punchable.addDefenseBonus(0.5);
                });
                p.skillBonus += 3;
                if (!p.getComponent('flames')) {
                    p.attach(new Flames({
                        color: 'red'
                    }));
                }
            });
            player.attach(new SparkleBurst({
                position: new Vector2(0, -16)
            }));
            EventSystem.fire('quest-powerup', {
                count: 1
            });
            EventSystem.fire('quest-consumable', {
                count: 1
            });
            return true;
        },
        // shield orb - defense +75% for 2 minutes
        "consumables/defensebonus3": function (key, player, position) {
            effectTween(60 + BuffManager.current.powerUpBoost, player, key, null, function (p, t) {
                p.attackBonus += 0.3;
                p.getComponent('punchable', function (punchable) {
                    punchable.addDefenseBonus(0.25);
                });
                p.skillBonus += 4;
                if (!p.getComponent('flames')) {
                    p.attach(new Flames({
                        color: 'red'
                    }));
                }

            });
            player.attach(new SparkleBurst({
                position: new Vector2(0, -16)
            }));
            EventSystem.fire('quest-powerup', {
                count: 1
            });
            EventSystem.fire('quest-consumable', {
                count: 1
            });
            return true;
        },
        // skill orb - skill +1 for 2 minutes
        "consumables/skillbonus1": function (key, player, position) {
            effectTween(60 + BuffManager.current.powerUpBoost, player, key, null, function (p, t) {
                p.attackBonus += 0.1;
                p.skillBonus += 3;
                if (!p.getComponent('flames')) {
                    p.attach(new Flames({
                        color: 'blue'
                    }));
                }
            });
            player.attach(new SparkleBurst({
                position: new Vector2(0, -16)
            }));
            EventSystem.fire('quest-powerup', {
                count: 1
            });
            EventSystem.fire('quest-consumable', {
                count: 1
            });
            return true;
        },
        // skill orb - skill +2 for 2 minutes
        "consumables/skillbonus2": function (key, player, position) {
            effectTween(60 + BuffManager.current.powerUpBoost, player, key, null, function (p, t) {
                p.attackBonus += 0.2;
                p.skillBonus += 6;
                if (!p.getComponent('flames')) {
                    p.attach(new Flames({
                        color: 'blue'
                    }));
                }

            });
            player.attach(new SparkleBurst({
                position: new Vector2(0, -16)
            }));
            EventSystem.fire('quest-powerup', {
                count: 1
            });
            EventSystem.fire('quest-consumable', {
                count: 1
            });
            return true;
        },
        // skill orb - skill +3 for 2 minutes
        "consumables/skillbonus3": function (key, player, position) {
            effectTween(60 + BuffManager.current.powerUpBoost, player, key, null, function (p, t) {
                p.attackBonus += 0.3;
                p.skillBonus += 9;
                if (!p.getComponent('flames')) {
                    p.attach(new Flames({
                        color: 'blue'
                    }));
                }
            });
            player.attach(new SparkleBurst({
                position: new Vector2(0, -16)
            }));
            EventSystem.fire('quest-powerup', {
                count: 1
            });
            EventSystem.fire('quest-consumable', {
                count: 1
            });
            return true;
        },
        // star - invincibility for 15 seconds
        "consumables/invincibility": function (key, player, position) {
            console.log('invincibility');
            var invincibilityTime = 15;
            effectTween(invincibilityTime, player, key, function (p) {
                p.attach(new Invincible({
                    seconds: invincibilityTime
                }));
            }, function (p, t) {
                if (!p.getComponent('invincible')) {
                    var tweenTime = invincibilityTime - t / 60;
                    if (tweenTime > 0) {
                        p.attach(new Invincible({
                            seconds: tweenTime
                        }));
                    }
                }
            });
            EventSystem.fire('quest-consumable', {
                count: 1
            });
            return true;
        },
        // heart - revive returns false so that it cant be clicked, is used automatically upon death
        "consumables/revive": function (key, player, position) {
            return false;
        }
    };

    var effectTween = function (seconds, player, key, start, updatePlayer) {
        var destroy = function () {
            Bento.objects.remove(eTween);
        };
        var playerIdCache = player.playerId;
        var currentTime = 0;
        var eTween = new Tween({
            from: 0,
            to: 1,
            in: seconds * 60,
            delay: 0,
            ease: 'linear',
            onStart: function () {
                //turn on destroyTimedEffects event
                EventSystem.on('destroyTimedEffects', destroy);
                if (start) {
                    start(player);
                }
            },
            onUpdate: function (v, t) {
                // save time so you can get it later when adding same effect
                currentTime = t / 60;

                // check to see if player is in the screen, otherwise keep searching for the player
                if (player) {
                    if (!player.isAdded) {
                        Utils.getPlayerById(playerIdCache, function (p) {
                            player = p;
                        });
                    } else {
                        if (updatePlayer) {
                            updatePlayer(player, t);
                        }
                    }
                } else {
                    Utils.getPlayerById(playerIdCache, function (p) {
                        player = p;
                    });
                }
            },
            onComplete: function () {
                // remove any flames
                if (player && player.isAdded) {
                    player.removeByName('flames');
                }
            }
        }).extend({
            name: key,
            global: true,
            destroy: function () {
                //turn off destroyTimedEffects event
                EventSystem.off('destroyTimedEffects', destroy);
            },
            getTime: function () {
                return currentTime;
            }
        });
    };

    var findControlledPlayer = function () {
        var ii;
        var allPlayers = Bento.objects.getByFamily('player');

        for (ii = 0; ii < allPlayers.length; ii++) {
            if (allPlayers[ii].isRemote === false) {
                return allPlayers[ii];
            }
        }
        return allPlayers[0];
    };

    var dropWeapon = function (key, player, position) {
        var type = key.split('/').slice(1).join('/');
        var weapon = new Weapon({
            type: type
        });
        weapon.position = position.add(new Vector2(20, 0));

        Bento.objects.attach(weapon);

        // local player: try put in players hand
        /*var stateMachine = player.getComponent('stateMachine', function () {});
        if (!player.isRemote && stateMachine && (stateMachine.stateName === 'idle' || stateMachine.stateName === 'walk')) {
            weapon.getComponent('weapon', function (weaponComponent) {
                weaponComponent.pickUp(player, false);
            });
        } else {*/
        weapon.getComponent('movable', function (movable) {
            movable.velocityZ = -3;
            movable.velocity.x = 0.2;
            movable.gravity = 0.20;
            movable.height = -20;
        });
        // }
    };

    /**
     * Use the item with the given key in the game
     */
    var useItem = function (key, player, position) {
        // player should not be dead
        var isDead = false;
        player.getComponent('stateMachine', function (stateMachine) {
            if (stateMachine.stateName === 'dead') {
                isDead = true;
            }
        });

        if (isDead) {
            return;
        }

        var category = key.split('/')[0];
        if (category === 'weapons') {
            // drop the weapon
            dropWeapon(key, player, position);
            SaveManager.save('usedConsumable', 1);
            return true;
        } else {
            if (itemHandlers[key]) {
                var use = itemHandlers[key](key, player, position);
                if (use) {
                    SaveManager.save('usedWeapon', 1);
                    player.attach(new ConsumablePopup({
                        sprite: getItemSprite(key)
                    }));
                }
                return use;
            } else {
                Utils.log("WARNING: item " + key + " is missing a handler");
                return false;
            }
        }
        // itemHandlers[category](key, player, position);
    };

    var cache = {};

    var getItemJson = function (key) {
        var json = cache[key];

        if (!json) {
            json = Bento.assets.getJson(key);
            cache[key] = json;
        }

        return json;
    };

    /**
     * get the image associated with the item with the given key
     */
    var getItemSprite = function (key) {
        var json = getItemJson(key);
        return new Sprite({
            spriteSheet: json.inventory.image
        });
    };

    /**
     * get the name of the item associated with the given key
     */
    var getItemName = function (key) {
        var keys = key.split('/');
        var category = keys[0];
        var name = keys[1];
        var loc = Localization.getText(category);
        var item = loc[name] || {};
        var itemName = item['name'] || '';

        if (!itemName) {
            Utils.log('WARNING: name does not exist for ' + key);
        }
        return itemName;
    };

    /**
     * get the (default) shop prize of the item
     */
    var getItemPrice = function (key) {
        var json = getItemJson(key);
        return json.inventory.price;
    };

    /**
     * get the tier of the item
     */
    var getItemTier = function (key) {
        var json = getItemJson(key);
        return json.inventory.tier;
    };

    /**
     * get the (default) shop currency of the item
     */
    var getItemCurrency = function (key) {
        var json = getItemJson(key);
        return json.inventory.currency || 'coin';
    };

    /**
     * get the description of the item with the given key
     */
    var getItemDescription = function (key) {
        var keys = key.split('/');
        var category = keys[0];
        var name = keys[1];
        return Localization.getText(category)[name]['desc'];
    };

    /**
     * get the stats of the item with the given key
     */
    var getItemStats = function (key) {
        var effects = getItemJson(key).effects;
        var effect = '';
        var str = '';

        for (effect in effects) {
            if (str) {
                str += ', ';
            }
            str += Localization.getText('effects')[effect] + ' +' + effects[effect];
        }

        return str;
    };

    var manager = {
        getSprite: getItemSprite,
        getName: getItemName,
        getDescription: getItemDescription,
        getStats: getItemStats,
        getPrice: getItemPrice,
        getTier: getItemTier,
        getCurrency: getItemCurrency,
        useItem: useItem,
        getItemHandlers: function () {
            return itemHandlers;
        }
    };

    return manager;
});