/**
 * Quest manager keeps tabs on todays quests and refreshes the quests on a new day
 * @moduleName QuestManager
 */
bento.define('managers/quest', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/tween',
    'managers/save',
    'managers/time',
    'gui/notification',
    'modules/localization',
    'modules/notifications',
    'managers/inventory'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    SaveManager,
    TimeManager,
    Notification,
    Localization,
    Notifications,
    InventoryManager
) {
    'use strict';
    // (temporarily) removed quests
    var deprecatedQuests = [
        10, // online multiplayer quest
        14, // knockouts by throwing weapon
        8, // get hit by cars
        4, // use powerups
    ];

    var totalQuests = 0; // will be updated
    var dailyAmount = 3;
    var activeQuests;
    var questInfo;
    var listenersSet = false;
    var listeners = [];
    var test = false;
    var init = function () {
        if (!questInfo) {
            questInfo = Bento.assets.getJson('quests');
            // determine amount of quests
            Utils.forEach(questInfo, function (quest, key, l, breakLoop) {
                var number = parseInt(key);

                if (number || number === 0) {
                    // define standard quest (unless it was already defined)
                    if (!questEvents[number]) {
                        questEvents[number] = function (data) {
                            standardQuest(number, data);
                        };
                    }

                    if (number > totalQuests) {
                        totalQuests = number;
                    }
                }
                // fill quest names
                if (!questNames[key]) {
                    questNames[key] = quest.name;
                }
            });
        }

        // initialize the manager
        checkRefresh();
        // set listeners for active quests
        if (!listenersSet) {
            clearListeners();
            setListeners();
        }
        // check to see if there are quests to complete
        receiveQuest();
    };

    var checkRefresh = function () {
        var day = SaveManager.load('questDay');
        var today = Math.floor(TimeManager.getTime() / 1000 / 60 / 60 / 24);
        var tutorial = SaveManager.load('tutorialQuest');
        var socialQuests = [];
        var allQuests = [];
        var newQuests = [];
        var i;
        var pickStandardQuest = function () {
            var random = Utils.getRandom(allQuests.length);

            // check if it's a quest that requires a scroll --> reroll if user doesnt have it
            if (random === 0 && !InventoryManager.getItem('scrolls/dash')) {
                return pickStandardQuest();
            } else if (random === 1 && !InventoryManager.getItem('scrolls/charge')) {
                return pickStandardQuest();
            }

            if (!test) {
                test = true;
                // TODO: remove this, for testing
                // random = 19;
            }

            return random;
        };

        // set new quests if day is different
        if (tutorial < 1) {
            activeQuests = tutorialQuests();
            save();
            SaveManager.save('tutorialQuest', 1);
            listenersSet = false;
        } else if (day !== today && tutorial > 2) {
            // load status of social media
            var socialStatus = SaveManager.load('socialMedia');
            if (Utils.getRandom(2)) {
                if (Localization.isChinese()) {
                    //weibo
                    // if(!socialStatus.weibo && Utils.canOpenUrl('weibo')) {
                    //     socialQuests.push('weibo');
                    // }
                } else {
                    //facebook/twitter/instagram
                    if (!socialStatus.facebook && Utils.canOpenUrl('facebook')) {
                        socialQuests.push('facebook');
                    }
                    if (!socialStatus.twitter && Utils.canOpenUrl('twitter')) {
                        socialQuests.push('twitter');
                    }
                    // if(!socialStatus.instagram && Utils.canOpenUrl('instagram')) {
                    //     socialQuests.push('instagram');
                    // }
                }
            }

            // populate an array with numbers corresponding to quests, special quests have a string for name and wont be added normally
            for (i = 0; i <= totalQuests; i++) {
                allQuests.push(i);
            }

            // choose 3 quests
            i = 0;
            var addSpecialAt = 0;
            while (i < dailyAmount) {
                var rnd;
                var quest;

                if (socialQuests.length > 0 && i === 0) {
                    rnd = Utils.getRandom(socialQuests.length);
                    quest = socialQuests.splice(rnd, 1)[0];
                    ++addSpecialAt;
                } else if (i <= addSpecialAt) {
                    // quest = 'adQuest';
                    // NOTE: quick hack to replace the ad quest with a normal quest
                    rnd = pickStandardQuest();
                    quest = allQuests.splice(rnd, 1)[0];
                } else {
                    rnd = pickStandardQuest();
                    quest = allQuests.splice(rnd, 1)[0];
                }

                if (deprecatedQuests.indexOf(quest) >= 0) {
                    // reroll if quest is deprecated
                    continue;
                }
                if (!questInfo[quest]) {
                    Utils.log('Warning: quest ' + rnd + ' does not exist');
                    continue;
                }

                newQuests.push({
                    quest: quest,
                    count: 0,
                    goal: parseInt(questInfo[quest].goal),
                    currency: questInfo[quest].currency,
                    amount: parseInt(questInfo[quest].amount),
                    claimed: false,
                    special: specialQuests[quest]
                });
                ++i;
            }

            newQuests.push(totalQuest());

            activeQuests = newQuests;
            SaveManager.save('dailyQuests', newQuests);
            SaveManager.save('questDay', today);
            setListeners();
            listenersSet = false;

            // schedule 24 hours ahead for a notification
            // we do not precisely schedule the notification on refresh because it depends
            // on server time and this could be any time, even in the middle of the night
            if (SaveManager.load('notificationsAllowed')) {
                scheduleNotification(24);
            }
        } else {
            activeQuests = SaveManager.load('dailyQuests');
        }
    };

    /**
     * Schedule notification
     */
    var scheduleNotification = function (deltaTime) {
        // TODO: due to the bug in the notification plugin we should not send this notification
        /*
        var savedNotifications = SaveManager.load('notifications');
        if (Utils.isDefined(deltaTime)) {
            deltaTime = 24;
        }
        // reset badge
        Notifications.resetBadgeNumber();

        // cancel old notification if needed
        var oldId = savedNotifications['dailyQuest'];
        if (oldId) {
            Notifications.cancel(oldId);
        }

        // schedule notification
        var notifications = Utils.getNotifications({
            Localization: Localization,
            time: deltaTime * 60 * 60 * 1000
        });
        var id = Notifications.send(notifications.dailyQuest);

        // save id so it can be cancelled if needed
        savedNotifications['dailyQuest'] = id;
        SaveManager.save('notifications', savedNotifications);
        */
    };

    var tutorialQuests = function () {
        return [{
            mission: 0,
            quest: -1,
            count: 0,
            goal: 4,
            currency: 'coin',
            amount: 100,
            claimed: false
        }];
    };

    var totalQuest = function () {
        return {
            quest: 'totalQuest',
            count: 0,
            goal: dailyAmount,
            currency: 'gem',
            amount: 10,
            claimed: false
        };
    };

    var setListeners = function () {
        Utils.forEach(activeQuests, function (quest, i, l, breakLoop) {
            var questKey = quest.quest.toString();
            EventSystem.on('quest-' + questNames[questKey], questEvents[questKey]);
        });
        listenersSet = true;
    };

    var clearListeners = function () {
        Utils.forEach(questNames, function (quest, i, l, breakLoop) {
            EventSystem.clear('quest-' + quest);
        });
    };

    var turnOffListener = function (quest) {
        quest = quest.toString();
        EventSystem.off('quest-' + questNames[quest], questEvents[quest]);
    };

    var unsetListeners = function () {
        Utils.forEach(activeQuests, function (quest, i, l, breakLoop) {
            turnOffListener(quest.quest);
        });
    };

    // add non-standard quests here, the rest will be filled by "standard" quests
    var questEvents = {
        // "Complete all daily quests"
        'totalQuest': function (data) {
            standardQuest('totalQuest', data);
        },
        // "Liked us on Facebook"
        'facebook': function (data) {
            standardQuest('facebook', data);
        },
        // "Followed us on Instagram"
        'instagram': function (data) {
            standardQuest('instagram', data);
        },
        // "Followed us on Twitter"
        'twitter': function (data) {
            standardQuest('twitter', data);
        },
        // "Followed us on Weibo"
        'weibo': function (data) {
            standardQuest('weibo', data);
        },
        // "Watch 5 videos"
        'adQuest': function (data) {
            standardQuest('adQuest', data);
        },
        // "tutorial quest - play 4 levels"
        '-1': function (data) {
            var quest = getQuest(-1);
            if (quest && quest.count < data.mission && quest.count < 4) {
                var previousCount = quest.count;
                quest.count = data.mission;
                quest.mission = data.mission;
                save();
                if (previousCount < quest.goal &&
                    quest.count >= quest.goal
                ) {
                    SaveManager.save('tutorialQuest', 2);
                    notify(quest);
                }
            } else if (data.mission >= 4 && quest && quest.count !== quest.goal) {
                quest.count = quest.goal;
                SaveManager.save('tutorialQuest', 2);
                notify(quest);
            }
        },
        // "Get a 10 hit combo"
        '11': function (data) {
            var quest = getQuest('11');
            if (!quest) {
                return;
            }
            if (quest.count < quest.goal &&
                quest.count < data.count
            ) {
                quest.count = data.count;
                if (quest.count >= quest.goal) {
                    notify(quest);
                }
            }
            save();
        },
        // "Get a 20 hit combo"
        '12': function (data) {
            var quest = getQuest('12');
            if (!quest) {
                return;
            }
            if (quest.count < quest.goal &&
                quest.count < data.count
            ) {
                quest.count = data.count;
                if (quest.count >= quest.goal) {
                    notify(quest);
                }
            }
            save();
        }
    };

    var standardQuest = function (id, data) {
        var quest = getQuest(id);
        var currentMission = SaveManager.load('currentMission');
        if (!quest || currentMission === 'dojo') {
            return;
        }
        var previousCount = quest.count;
        quest.count += data.count;
        save();
        if (previousCount < quest.goal &&
            quest.count >= quest.goal
        ) {
            notify(quest);
        }
    };

    var specialQuests = {
        facebook: 'facebook',
        instagram: 'instagram',
        twitter: 'twitter',
        weibo: 'weibo',
        adQuest: 'adQuest'
    };

    // will be filled by the names in quest.json
    var questNames = {
        // "tutorial quest - play 4 levels"
        '-1': 'mission',
        'totalQuest': 'totalQuest'
    };

    var notify = function (data) {
        Notification.add(
        Localization.getText('completedQuests')[data.quest].replace('{NUMBER}', data.goal),
            new Entity({
                name: 'icon',
                components: [
                    new Sprite({
                        imageName: 'gui/dailyquest/checkmark-small',
                        originRelative: new Vector2(0.5, 0.5)
                    })
                ]
            })
        );
        Notification.add(
            Localization.getText('questComplete'),
            new Entity({
                name: 'icon',
                components: [
                    new Sprite({
                        imageName: 'gui/' + data.currency,
                        originRelative: new Vector2(0.5, 0.5)
                    })
                ]
            })
        );

        if (data.quest !== 'totalQuest') {
            EventSystem.fire('quest-totalQuest', {
                count: 1
            });
        } else {
            // Fire the 'Dayjob' achievement becasue we completed all our daily quests
            Utils.postAchievement("CgkI9NqdkoAGEAIQDQ");
        }
    };

    var save = function () {
        SaveManager.save('dailyQuests', activeQuests);
    };

    var getQuest = function (key) {
        var quest;
        Utils.forEach(activeQuests, function (q, i, l, breakLoop) {
            if (q.quest.toString() === key.toString()) {
                quest = q;
                // prefer quest that havent been completed yet
                if (q.count < q.goal) {
                    breakLoop();
                }
            }
        });
        return quest;
    };

    var getQuests = function () {
        if (!activeQuests) {
            activeQuests = SaveManager.load('dailyQuests');
        }
        return activeQuests;
    };

    var saveQuests = function (value) {
        activeQuests = value || activeQuests;
        save();
    };

    var receiveQuest = function () {
        var socialStatus = SaveManager.load('socialMedia');
        if (Utils.isDefined(socialStatus.receive)) {
            // quest event
            EventSystem.fire('quest-' + socialStatus.receive, {
                count: 1
            });
            // save
            socialStatus[socialStatus.receive] = true;
            socialStatus.receive = undefined;
            SaveManager.save('socialMedia', socialStatus);
            // update frame(s)
            EventSystem.fire('updateDailyQuest');
        }
    };

    var manager = {
        initialize: init,
        getQuest: getQuest,
        getQuests: getQuests,
        saveQuests: saveQuests,
        names: questNames,
        receiveQuest: receiveQuest,
        /**
         * Did user clear any quest?
         */
        canClaim: function () {
            // initialize if needed
            init();

            var quests = getQuests();
            var canClaim = false;
            Utils.forEach(quests, function (quest, i, l, breakLoop) {
                if (quest.count >= quest.goal && !quest.claimed) {
                    canClaim = true;
                    breakLoop();
                }
            });
            return canClaim;
        }
    };

    return manager;
});