/**
 * Player receives a reward, based on an item string
 * This item string can also include gems, coins and tokens
 * Example usage:
 ReceiveItem({
    key: 'currency/gem',
    amount: 100
 });
 * @moduleName ReceiveItem
 * @snippet ReceiveItem.snippet
ReceiveItem({
    key: ${1:'currency/coin'},
    amount: ${2:1}
});
 */
bento.define('managers/receiveitem', [
    'bento',
    'bento/utils',
    'managers/currency',
    'gui/levelupnotification',
    'managers/inventory',
    'managers/character'
], function (
    Bento,
    Utils,
    CurrencyManager,
    LevelUpNotification,
    InventoryManager,
    CharacterManager
) {
    'use strict';
    var ReceiveItem = function (settings) {
        var key = settings.key || settings.item;
        var amount = settings.amount;
        var onComplete = settings.onComplete;
        var skipNotification = settings.skipNotification;
        var notSource = settings.notSource;

        var rewardString = key;
        var itemJson = Bento.assets.getJson(rewardString);
        var rewardStringTokens = rewardString.split('/');
        var category = rewardStringTokens[0];
        var item = rewardStringTokens[1];
        var itemTokens = item.split('-');
        var tokenType = itemTokens[0];
        var tokenCharacter = itemTokens[1];

        // get reward as item
        if (category === 'currency') {
            if (item === 'gem') {
                CurrencyManager.addGems(amount, notSource);
            } else if (item === 'coin') {
                CurrencyManager.addCoins(amount, notSource);
            } else if (item === 'key') {
                CurrencyManager.addKeys(amount);
            } else if (item === 'ticket') {
                CurrencyManager.addTickets(amount);
            } else {
                Utils.log("WARNING: unrecognized key " + key);
            }
        } else if (category === 'upgrades') {
            CurrencyManager.addToken(tokenCharacter, tokenType, amount);
            LevelUpNotification.check(tokenCharacter, tokenType, amount);
        } else if (category === 'other') {
            if (item === 'energy') {
                CharacterManager.addEnergy(amount);
            }
        } else {
            InventoryManager.addItem(rewardString, amount);

            if (category === 'playertokens' && !skipNotification) {
                LevelUpNotification.check(item, 'playertokens', amount);
            }
        }

        // callback if you need to know what was given
        if (onComplete) {
            onComplete({
                key: key,
                amount: amount,
                category: category,
                item: item,
                tokenType: tokenType,
                tokenCharacter: tokenCharacter
            });
        }
    };

    return ReceiveItem;
});