/**
 * Stats manager. Gets the stats for each character.
 * Stats are calculated based on [base stats + upgrades + equipment]
 *
 * @moduleName StatsManager
 */
bento.define('managers/stats', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'managers/save',
    'managers/inventory',
    'globals',
    'managers/allies',
    'managers/buff'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween,
    SaveManager,
    InventoryManager,
    Globals,
    AlliesManager,
    BuffManager
) {
    'use strict';
    /**
     * Describe your settings object parameters
     * @param {Object} settings
     */

    /**
     * Get the total value of a specific stat of a specific character (base + upgrade + equipment)
     */
    var getStat = function (char, stat) {
        return getDetailedStat(char, stat).total;
    };

    /**
     * Helper function to get the total values of all stats of a character
     */
    var getAllStats = function (char) {
        return {
            name: 'stats',
            attack: getStat(char, 'attack'),
            skill: getStat(char, 'skill'),
            health: getStat(char, 'health'),
            stamina: getStat(char, 'stamina')
        };
    };

    var getDetailedStat = function (char, stat) {
        var characterLevel = SaveManager.load('level-' + char);
        var json = Globals.getCharacterStatsFromLevel(char, characterLevel);

        var base = json[stat] || 1;

        var equipped = 0;
        // var equipment = InventoryManager.getEquipped(char);
        // Utils.forEach(equipment, function (eqp, i, l, breakLoop) {
        //     if (!eqp) {
        //         return; //skip
        //     }

        //     json = Bento.assets.getJson(eqp.key);
        //     if (json && json.stats && json.stats[stat]) {
        //         equipped += json.stats[stat];
        //     }
        // });
        
        // check for buffmanager to see any equipped effects
        if (BuffManager.current[stat]) {
            equipped += BuffManager.current[stat];
        }

        return {
            base: base,
            equipped: equipped,
            bare: base,
            total: base + equipped
        };
    };

    var getAllDetailedStats = function (char) {
        return {
            attack: getDetailedStat(char, 'attack'),
            skill: getDetailedStat(char, 'skill'),
            health: getDetailedStat(char, 'health'),
            stamina: getDetailedStat(char, 'stamina')
        };
    };
    var initialHealth = 20;

    // reset health upon refreshing
    if (Utils.hasAsset('json', 'stats/' + SaveManager.load('selectedCharacter'))) {
        initialHealth = getStat(SaveManager.load('selectedCharacter'), 'health');
    }

    var manager = {
        getStat: getStat,
        getAllStats: getAllStats,

        getDetailedStat: getDetailedStat,
        getAllDetailedStats: getAllDetailedStats,

        getLevel: function (character) {
            return SaveManager.load('level-' + character);
        },
        addLevel: function (character, diff) {
            var level = SaveManager.load('level-' + character);
            level += (diff || 1);
            SaveManager.save('level-' + character, level);
        },
        
        // current health is a global property and reset on start of a stage
        // not to be confused with the stat called health (which is the maxHealth!)
        health: initialHealth,
        resetHealth: function (character) {
            // we do need the current character to look up the max health
            if (!character) {
                character = SaveManager.load('selectedCharacter');
            }
            manager.health = getStat(character, 'health');
        }
    };

    // for reloading purposes
    // manager.resetHealth();
    return manager;
});