/**
 * Generic module that determines if user should become a tester or not
 * Usage: define a new testGroup, this is a simple string that is saved in the user's saveState as "testGroup"
 * On DynamoDB, create a new table row with id [testGroup name], counts and maxCounts
 * 
 * The rest will have to be patched 
 * @moduleName ABTest
 */
bento.define('modules/abtest', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'modules/analytics',
    'managers/save'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween,
    Analytics,
    SaveManager
) {
    'use strict';
    var isDownloading = false;
    var onDoneCallback = function () {};
    var link = 'https://35jwwdy03d.execute-api.us-east-1.amazonaws.com/prod/usercounter';
    var download = function (groupName) {
        var xhr = new window.XMLHttpRequest();
        var body = {
            name: groupName
        };
        xhr.open("POST", link);
        xhr.setRequestHeader("Content-Type", "application/json;charset=UTF-8");
        xhr.timeout = 2000; // time in milliseconds
        isDownloading = true;
        xhr.onreadystatechange = function () {
            var jsonResponse;
            if (xhr.readyState === 4) {
                if (xhr.status >= 200 && xhr.status < 300) {
                    // console.log(xhr.responseText);
                    try {
                        jsonResponse = JSON.parse(xhr.responseText);
                        if (jsonResponse.success) {
                            // tester! save into saveState
                            Bento.saveState.save('testGroup', groupName);
                            EventSystem.fire('ABTest', {
                                name: groupName
                            });
                            // fire off analytics event
                            Analytics.event('UserGroups', groupName, 1, groupName);
                        } else {
                            // group is full: not a tester
                        }
                    } catch (e) {
                        // could not parse, not a tester
                    }
                } else {
                    console.log('fail');
                }
                if (isDownloading) {
                    isDownloading = false;
                    onDoneCallback();
                }
            }
        };
        xhr.ontimeout = function (e) {
            if (isDownloading) {
                isDownloading = false;
                onDoneCallback();
            }
        };
        xhr.send(JSON.stringify(body));
    };

    var ABTest = {
        /**
         * Is player part of a test group?
         * @param  {String}  name - Name of test group
         * @return {Boolean}
         */
        isTester: function (name) {
            // (using Bento.saveState for patcher safety)
            return Bento.saveState.load('testGroup', '') === name;
        },
        /**
         * Is player part of any test group?
         * @return {Boolean}
         */
        isAnyTester: function () {
            // (using Bento.saveState for patcher safety)
            return Bento.saveState.load('testGroup', '') !== '';
        },
        /**
         * Check on server if there is room for a tester
         * @param  {String}  name - Name of test group
         * @return {Boolean}
         */
        shouldBecomeTester: function (name) {
            if (!name) {
                return;
            }
            // already a tester in this group
            if (ABTest.isTester(name)) {
                EventSystem.fire('ABTest', {
                    name: name
                });
                return;
            }
            // already a tester in a different group
            if (ABTest.isAnyTester()) {
                return;
            }
            // check with server
            download(name);
        },
        /**
         * Whether the module is still checking AWS to see if user can become a test
         * @param  {Function}  onDone Callback when download is complete
         * @return {Boolean}
         */
        isDownloading: function (onDone) {
            if (onDone) {
                onDoneCallback = onDone;
            }
            return isDownloading;
        }
    };
    return ABTest;
});