/**
 * Wrapper for all analytics plugins
 * @moduleName Analytics
 */
bento.define('modules/analytics', [
    'modules/googleanalytics',
    'bento/utils',
    'bento'
], function (
    GoogleAnalytics,
    Utils,
    Bento
) {
    'use strict';
    var userId = 'unidentified';
    var sessionId = Date.now().toString();
    var googlePlaySink = '';
    var googlePlaySource = '';
    var logEventsInDev = false;
    var currentView = '';
    var isActive = true;
    var categoryPrepend = '';
    var Analytics = {
        init: function (trackingId) {
            var idfa = Bento.saveState.load('idfa', null);
            var setUserId = function () {
                // use default methods
                if (window.device) {
                    userId = window.device.uuid;
                    setGoogleUserId();
                } else if (navigator.isCocoonJS && window.Cocoon && window.Cocoon.Device) {
                    userId = window.Cocoon.Device.getDeviceId();
                    setGoogleUserId();
                }
            };
            var setGoogleUserId = function () {
                GoogleAnalytics.setUserId(userId);
            };

            // don't send analytics during development
            if (Utils.isDev()) {
                Utils.log('Google Analytics not initialized because dev mode is on');
                return;
            }
            // init Google Analytics
            // get user id
            if (idfa) {
                userId = idfa;
                setGoogleUserId();
            } else {
                // try getting idfa
                if (window.plugins && window.plugins.AppleAdvertising) {
                    window.plugins.AppleAdvertising.getIdentifiers(function (identifiers) {
                        if (identifiers.trackingEnabled) {
                            idfa = identifiers.idfa;
                            Bento.saveState.save('idfa', idfa);

                            userId = idfa;
                            setGoogleUserId();
                        } else {
                            // user turned off tracking, resort to using other ids
                            setUserId();
                            setGoogleUserId();
                        }
                    }, function () {
                        console.log("error loading identifiers");
                        setUserId();
                        setGoogleUserId();
                    });
                } else {
                    setUserId();
                    setGoogleUserId();
                }
            }

            // init Facebook
            /*if (window.facebookConnectPlugin) {
                // https://github.com/Wizcorp/phonegap-facebook-plugin/issues/877#issuecomment-74889551
                window.facebookConnectPlugin.logEvent('fb_mobile_activate_app');
            }*/
        },
        event: function (category, action, value, label) {
            if (!isActive) {
                return;
            }
            // only log in dev mode
            if (Utils.isDev() && logEventsInDev) {
                console.log('Analytics.event:', categoryPrepend + category, action, value, label);
            }
            if (!navigator.onLine || !navigator.isCocoonJS) {
                return;
            }
            // send to googleanalytics plugin
            GoogleAnalytics.trackEvent(categoryPrepend + category, action, label, value);

            // facebook plugin
            /*if (window.facebookConnectPlugin) {
                window.facebookConnectPlugin.logEvent(action, {
                    category: category,
                    action: action,
                    value: value,
                    label: label
                }, value, function () {
                    // console.log('fb event logged:' + action);
                }, function () {});
            }*/
        },
        trackPurchase: function (id, name, sku, category, price, quantity, currencyCode) {
            if (!isActive) {
                return;
            }
            // only log in dev mode
            if (Utils.isDev() && logEventsInDev) {
                console.log('Analytics.trackPurchase:', id, name, sku, category, price, quantity, currencyCode);
            }
            // to view the revenue, we must also send transaction data
            GoogleAnalytics.addTransaction(id, '', price * quantity, 0.3 * price * quantity, 0, currencyCode);

            // send 
            GoogleAnalytics.addTransactionItem(id, name, sku, category, price, quantity, currencyCode);

        },
        trackView: function (name) {
            if (!isActive) {
                return;
            }
            // only log in dev mode
            if (Utils.isDev() && logEventsInDev) {
                console.log('Analytics.trackView:', name);
            }
            GoogleAnalytics.trackView(name);
            currentView = name;
        },
        getCurrentView: function () {
            return currentView;
        },
        setActive: function (value) {
            isActive = value;
        },
        setCategoryPrepend: function (value) {
            // categoryPrepend = value;
        }
    };

    return Analytics;
});