/**
 * Swaps text module
 * @moduleName FontSettings
 */
bento.define('modules/fontsettings', [
    'modules/localization',
    'bento/gui/text'
], function (
    Localization,
    Text
) {
    'use strict';
    var fontSettings = {
        'en': {
            'dialog': {
                font: 'font',
                fontSize: 16
            }
        },
        'ja': {
            'dialog': {
                font: 'japanese',
                fontSize: 14,
                ySpacing: 4,
                pixelStroke: false
            }
        },
        'ko': {
            'dialog': {
                font: 'bmhanna',
                fontSize: 14,
                ySpacing: 4,
                pixelStroke: false
            }
        },
        'zh': {
            'dialog': {
                font: 'chinese',
                fontSize: 14,
                ySpacing: 2,
                pixelStroke: false
            }
        },
        'zt': {
            'dialog': {
                font: 'chintrad',
                fontSize: 14,
                ySpacing: 2,
                pixelStroke: false
            }
        },
        'ru': {
            'dialog': {
                font: 'adbxtra',
                fontSize: 12
            }
        }
    };
    var getFontLang = function () {
        // returns a language code for every language that has a different font setting
        if (Localization.getLanguage() === 'zh' ||
            Localization.getLanguage() === 'zt' ||
            Localization.getLanguage() === 'ko' ||
            Localization.getLanguage() === 'ru' ||
            Localization.getLanguage() === 'ja') {
            return Localization.getLanguage();
        } else {
            return 'en';
        }
    };

    var FontSettings = {
        get: function (name) {
            // name = font, fontLarge, fontStroked etc.
            return fontSettings[getFontLang()][name];
        },
        getEn: function (name) {
            // name = font, fontLarge, fontStroked etc.
            return fontSettings['en'][name];
        }
    };
    Text.FontSettings = FontSettings;
    return FontSettings;

});