/**
 * AppWarp multiplayer service
 * @moduleName AppWarpService
 */
bento.define('modules/onlineservice/appwarp', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'appwarp'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween,
    AppWarp
) {
    'use strict';
    if (window.Cocoon || !window.AppWarp) {
        return {};
    }

    var AppWarpMatch = function (room) {
        this.localPlayer = {
            // numerical id
            playerId: -1,
            // id according to online service (may not be numerical)
            uniqueId: -1,
            // username
            userName: ''
        };
        this.players = room.players;

        this.onDataRecieved = function () {};
        this.onStateChanged = function () {};
        this.onConnectionWithPlayerFailed = function () {};

        this._onReadyCallback = null;

        this._isReady = false;

        var self = this;

        var playerIdMap = {};

        /**
         * Listen to events
         */
        EventSystem.on('onChatReceived', function (e) {
            // cleaned up on disconnect            
            var dataObj;

            try {
                dataObj = {
                    //match: matchObj,
                    list: JSON.parse(e.json.chat),
                    playerId: playerIdMap[e.json.sender]
                };
            } catch (err) {
                Utils.log("ERROR: could not parse data!!");
                return;
            }
            self.onDataRecieved(dataObj);
        });
        EventSystem.on('onPrivateChatReceived', function (sender, chat) {
            // cleaned up on disconnect            
            var dataObj;

            try {
                dataObj = {
                    //match: matchObj,
                    list: JSON.parse(chat),
                    playerId: playerIdMap[sender]
                };
            } catch (err) {
                Utils.log("ERROR: could not parse data!!");
                return;
            }
            self.onDataRecieved(dataObj);
        });
        EventSystem.on('onUserLeftRoom', function (evt) {
            /*{
                id: "2112731727"
                maxUsers: 2
                name: "BeatStreetsRoom-player2500128.1491990100259"
                owner: "player2500128"
                user: "player2515129"
                res: 0
            }*/
            self.onStateChanged({
                match: null,
                playerId: playerIdMap[evt.json.user],
                state: 2
            });
        });

        /**
         * Update players
         */
        Utils.forEach(this.players, function (playerData, i, l, breakLoop) {
            // remap
            playerData.uniqueId = playerData.name;
            playerData.playerId = i;
            playerData.userName = playerData.name;

            // setup playerIdMap
            playerIdMap[playerData.uniqueId] = playerData.playerId;
            // self
            if (playerData.name === playerName) {
                self.localPlayer = playerData;
            }
        });

        //this is the 'owner' of the room, and thus responsible for closing it from new players
        if (this.localPlayer.playerId === 0) {
            warpClient.updateRoomProperties(room.id, {
                hasStarted: true
            }, null);
        }
    };

    AppWarpMatch.prototype.playerHasLowestId = function () {
        return this.localPlayer.playerId === 0;
    };

    AppWarpMatch.prototype.onReady = function (callback) {
        callback();
    };

    AppWarpMatch.prototype.start = function () {

    };
    AppWarpMatch.prototype.disconnect = function () {
        // for appwarp we disconnect entirely
        warpClient.disconnect();
        loggedIn = false;

        // clean up events
        EventSystem.clear('onChatReceived');
        EventSystem.clear('onPrivateChatReceived');
        EventSystem.clear('onUserLeftRoom');
    };
    /**
     *
     *
     *
     */
    var playerName = '';
    var loggedIn = false;
    var warpClient = AppWarp.WarpClient.getInstance();

    var onMatchFound = function (room, error) {
        if (room) {
            var awm = new AppWarpMatch(room);
            if (module.onMatchFound) {
                module.onMatchFound(awm);
            }
        } else {
            if (module.onMatchFound) {
                module.onMatchFound(null, error);
            }
        }
    };

    var module = {
        findMatch: function (minPlayers, maxPlayers, playersToInvite, playerGroup, playerAttributes) {
            //join a room and wait for it to fill
            var joinRoom = function (roomId) {
                var users = [];

                var updateUsers = function () {
                    var ii;

                    if (users.length === 2) {
                        for (ii = 0; ii < users.length; ii++) {
                            users[ii] = {
                                name: users[ii],
                                id: ii
                            };
                        }

                        onMatchFound({
                            id: roomId,
                            players: users
                        });
                        // clean up events
                        EventSystem.clear('onUserJoinedRoom');
                    }
                };

                EventSystem.on('onJoinAndSubscribeRoomDone', function (e) {
                    // clean up
                    EventSystem.clear('onJoinAndSubscribeRoomDone');

                    console.log('joins', e);
                    if (e.res !== AppWarp.ResultCode.Success) {
                        console.log('error', e.res);
                        onMatchFound(null, 'code' + e.res);
                        return;
                    }
                    warpClient.getLiveRoomInfo(roomId);

                });

                //another player joined the room, check if there are enough players
                EventSystem.on('onUserJoinedRoom', function (e) {
                    // cleaned up when match starts

                    if (e.json.user === warpClient.userName) {
                        return;
                    }

                    users.push(e.json.user);
                    updateUsers();
                });

                //joined the room, check if there are enough players
                EventSystem.on('onGetLiveRoomInfoDone', function (e) {
                    var roomUsers = [];
                    // console.log('get room info', evt);
                    if (e.json.usernames) {
                        roomUsers = e.json.usernames.split(';');
                    }
                    if (roomUsers[roomUsers.length - 1] === "") {
                        roomUsers.pop();
                    }
                    users = users.concat(roomUsers);
                    updateUsers();
                    // clean up
                    EventSystem.clear('onGetLiveRoomInfoDone');
                });

                warpClient.joinAndSubscribeRoom(roomId);
            };
            var onGetMatchedRoomsDone = function (e) {
                var rooms = [];
                // console.log('Matched rooms', evt);
                // find rooms that are not filled and not started with playing
                Utils.forEach(e.json, function (room, id, l, breakLoop) {
                    room.id = id;
                    rooms.push(room);
                });

                // no rooms? create one
                if (rooms.length === 0) {
                    // make a room and join it
                    EventSystem.on('onCreateRoomDone', onCreateRoomDone);

                    warpClient.createRoom(
                        'BeatStreetsRoom-' + warpClient.userName + '.' + Date.now(), // trying to set this as unique id...
                        warpClient.userName,
                        2, {
                            hasStarted: false
                        }
                    );
                } else {
                    // try to join the first one
                    joinRoom(rooms[0].id);
                }

                // clean up
                EventSystem.off('onGetMatchedRoomsDone', onGetMatchedRoomsDone);
            };
            var onCreateRoomDone = function (e) {
                console.log('new room', e);
                // join newly created room
                joinRoom(e.json.id);
            };

            //check if there is a room that can be joined
            EventSystem.on('onGetMatchedRoomsDone', onGetMatchedRoomsDone);

            warpClient.getRoomsWithProperties({
                hasStarted: false
            });
        },
        onMatchFound: null,
        onInvitation: null,
        onPlayerDisconnects: null,
        sendData: function (match, data, sendMode) {
            warpClient.sendChat(data);
        },
        sendDataTo: function (playerId, match, data, sendMode) {
            var uniqueId; // need to find this id
            Utils.forEach(match.players, function (playerData, i, l, breakLoop) {
                if (playerData.playerId === playerId) {
                    uniqueId = playerData.uniqueId;
                    breakLoop();
                }
            });
            warpClient.sendPrivateChat(uniqueId, data);
        },
        onDataRecieved: null,
        login: function (onLogin) {
            var onConnectDone = function (e) {
                if (e === AppWarp.ResultCode.Success) {
                    loggedIn = true;
                    onLogin({
                        success: true
                    });
                } else {
                    onLogin({
                        success: false,
                        error: AppWarp.ResultCode
                    });
                }
                // clean up
                EventSystem.off('onConnectDone', onConnectDone);
            };
            var enterName = function () {
                Utils.prompt('Enter name', function (name) {
                    if (!name) {
                        enterName();
                    } else {
                        playerName = name;
                        warpClient.connect(name);
                    }
                });
            };
            EventSystem.on('onConnectDone', onConnectDone);
            // enterName();

            // let's just pick a random name
            var now = Date.now();
            now = now % (1000 * 60 * 60); // cut off beyond hours
            playerName = 'player' + now;
            warpClient.connect(playerName);
        },
        isLoggedIn: function () {
            // TODO: catch when user goes offline?
            return loggedIn;
        }
    };
    return module;
});