/**
 * Photon multiplayer service
 * @moduleName Photon
 */
bento.define('modules/onlineservice/photon', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'photon'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween,
    PhotonClasses
) {
    'use strict';
    if (!window.Photon) {
        return {};
    }
    var LoadBalancingClient = window.Photon.LoadBalancing.LoadBalancingClient;
    var localActor = PhotonClasses.client.myActor();

    EventSystem.on('onStateChange', function (args) {
        var state = args[0];
        var msg = args[1];
        switch (state) {
        case LoadBalancingClient.State.Error:
        console.log(state, msg);
            EventSystem.fire('onStateChange.Error', args);
            break;
        case LoadBalancingClient.State.Uninitialized:
            EventSystem.fire('onStateChange.Uninitialized');
            break;
        case LoadBalancingClient.State.ConnectingToNameServer:
            EventSystem.fire('onStateChange.ConnectingToNameServer');
            break;
        case LoadBalancingClient.State.ConnectedToNameServer:
            EventSystem.fire('onStateChange.ConnectedToNameServer');
            break;
        case LoadBalancingClient.State.ConnectingToMasterserver:
            EventSystem.fire('onStateChange.ConnectingToMasterserver');
            break;
        case LoadBalancingClient.State.ConnectedToMaster:
            EventSystem.fire('onStateChange.ConnectedToMaster');
            break;
        case LoadBalancingClient.State.JoinedLobby:
            EventSystem.fire('onStateChange.JoinedLobby');
            break;
        case LoadBalancingClient.State.ConnectingToGameserver:
            EventSystem.fire('onStateChange.ConnectingToGameserver');
            break;
        case LoadBalancingClient.State.ConnectedToGameserver:
            EventSystem.fire('onStateChange.ConnectedToGameserver');
            break;
        case LoadBalancingClient.State.Joined:
            EventSystem.fire('onStateChange.Joined');
            break;
        case LoadBalancingClient.State.Disconnected:
            EventSystem.fire('onStateChange.Disconnected');
            break;
        default:
            break;
        }

        // TEMP: delete this
        // console.log("==STATE " + LoadBalancingClient.StateToName(state));
    });

    /*EventSystem.on('onRoomList', function (rooms) {
        console.log('onRoomList', rooms);
    });
    EventSystem.on('onRoomListUpdate', function (rooms, roomsUpdated, roomsAdded, roomsRemoved) {
        console.log('onRoomListUpdate', rooms, roomsUpdated, roomsAdded, roomsRemoved);

    });
    EventSystem.on('onMyRoomPropertiesChange', function () {
        var room = client.myRoom();
        console.log('onMyRoomPropertiesChange', room);
    });*/

    var PhotonMatch = function (room) {
        this.localPlayer = {
            // numerical id
            playerId: -1,
            // id according to online service (may not be numerical)
            uniqueId: -1,
            // username
            userName: ''
        };
        this.room = room;
        this.players = [];
        this.otherPlayers = [];
        var actors = room.loadBalancingClient.actors;
        var actorToPlayerData = function (actor) {
            var playerData = {};
            playerData.uniqueId = actor.actorNr;
            // playerId can not be determined until the game starts
            playerData.playerId = null;

            playerData.userName = actor.name;
            playerData.character = actor.getCustomProperty('character') || 'boy';
            playerData.level = parseInt(actor.getCustomProperty('level')) || 0;
            playerData.seed = parseInt(actor.getCustomProperty('seed')) || 0;
            return playerData;
        };

        this.onDataRecieved = function () {};
        this.onStateChanged = function () {};
        this.onConnectionWithPlayerFailed = function () {};

        // photon only: rooms are created before players are completely initialized -> the playerId is undetermined
        this.initPlayerIds = function (map) {
            var self = this;
            if (!map) {
                // calculate the playerIds
                Utils.forEach(self.players, function (playerData, i, l, breakLoop) {
                    playerData.playerId = i;
                    playerIdMap[playerData.uniqueId] = playerData.playerId;
                    // otherPlayers have a reference to the same playerData, should be updated as well
                });
            } else {
                // conform tot he playerIdMap
                Utils.forEach(map, function (playerId, uniqueId, l, breakLoop) {
                    var playerData = playerMap[uniqueId];
                    if (!playerData) {
                        console.log(playerMap, playerId, uniqueId);
                        Utils.log("FATAL: playerdata was not found");
                    }
                    playerData.playerId = playerId;
                    playerIdMap[playerData.uniqueId] = playerData.playerId;
                });
            }
            return self.players;
        };

        this._onReadyCallback = null;

        this._isReady = false;

        var self = this;

        var playerIdMap = {};
        var playerMap = {};

        /**
         * Listen to events
         */
        EventSystem.on('onEvent', function (args) {
            // cleaned up on disconnect
            var code = args[0];
            var content = args[1];
            var actorNr = args[2];

            var isPrivate = (code === 1);
            var dataObj;

            try {
                dataObj = {
                    //match: matchObj,
                    list: JSON.parse(content),
                    playerId: playerIdMap[actorNr],
                    uniqueId: actorNr,
                    playerData: playerMap[actorNr]
                };
            } catch (err) {
                Utils.log("ERROR: could not parse data!!");
                console.log(content);
                return;
            }
            self.onDataRecieved(dataObj);
        });
        EventSystem.on('onActorLeave', function (args) {
            var actor = args[0];
            var cleanUp = args[1];
            var stateObj = {
                match: null,
                playerId: playerIdMap[actor.actorNr], // WARNING: may be null if match has not started yet
                playerData: playerMap[actor.actorNr],
                state: 2,
                actor: actor
            };

            self.onStateChanged(stateObj);

            // find and remove
            Utils.forEach(self.players, function (playerData, i, l, breakLoop) {
                if (actor.actorNr === playerData.uniqueId) {
                    self.players.splice(i, 1);
                    breakLoop();
                }
            });
            // will be removed from otherPlayers array by MatchManager
            /*Utils.forEach(self.otherPlayers, function (playerData, i, l, breakLoop) {
                if (playerIdMap[actor.actorNr] === playerData.playerId) {
                    self.otherPlayers.splice(i, 1);
                    breakLoop();
                }
            });*/
        });
        EventSystem.on('onActorJoin', function (args) {
            var actor = args[0];
            // ignore self
            if (actor.isLocal) {
                return;
            }

            // init playerData
            var playerData = actorToPlayerData(actor);
            self.players.push(playerData);
            self.otherPlayers.push(playerData);

            // setup playerIdMap
            playerIdMap[playerData.uniqueId] = playerData.playerId;
            playerMap[playerData.uniqueId] = playerData;

            self.onStateChanged({
                match: this,
                playerId: playerData.playerId, // WARNING: may be null if match has not started yet
                playerData: playerData,
                state: 1,
                actor: actor
            });
        });

        /**
         * Update players that are already in the room
         */
        Utils.forEach(actors, function (actor, i, l, breakLoop) {
            // remap
            var playerData = actorToPlayerData(actor);
            self.players.push(playerData);

            // setup playerMap
            // playerIdMap[playerData.uniqueId] = playerData.playerId;
            playerMap[playerData.uniqueId] = playerData;

            // self
            if (actor.isLocal) {
                self.localPlayer = playerData;
            } else {
                self.otherPlayers.push(playerData);
            }

            // close room for other players
            /*if (room.masterClientId === actor.actorNr) {
                room.setIsOpen(false);
                room.setIsVisible(false);
            }*/
            // --> room needs to be closed when player chooses to start the game
        });
    };

    PhotonMatch.prototype.playerHasLowestId = function () {
        return this.localPlayer.uniqueId === this.room.masterClientId;
    };

    PhotonMatch.prototype.onReady = function (callback) {
        callback();
    };

    PhotonMatch.prototype.start = function () {

    };
    PhotonMatch.prototype.disconnect = function () {
        module.disconnect();

        // clean up events by calling destroy
        this.destroy();
    };
    PhotonMatch.prototype.destroy = function () {
        // clean up events
        EventSystem.clear('onEvent');
        EventSystem.clear('onActorLeave');
        EventSystem.clear('onActorJoin');
    };

    /**
     *
     *
     *
     */
    var playerName = '';
    var loggedIn = false;
    var client = PhotonClasses.client;
    var isConnecting = false;
    var roomNumber = 0;

    var module = {
        onCharacterChanged: function (character) {
            localActor.setCustomProperty('character', character);
        },
        findMatch: function (minPlayers, maxPlayers, playersToInvite, playerGroup, playerAttributes, otherData) {
            //check if there is a room that can be joined
            var create = otherData.create;
            var roomCode = otherData.roomCode;
            var propsListedInLobby = otherData.propsListedInLobby;
            var roomList = client.availableRooms();

            var onJoinRoom = function (args) {
                var createdByMe = args[0];
                var room = client.myRoom();
                // Old method: wait until room is filled
                /*var onActorJoin = function () {
                    if (room.playerCount === room.maxPlayers) {
                        onRoomFilled();
                    }
                };
                var onRoomFilled = function () {
                    var photonMatch = new PhotonMatch(room);
                    if (module.onMatchFound) {
                        module.onMatchFound(photonMatch);
                    }

                    EventSystem.off('onActorJoin', onActorJoin);
                };

                if (room.playerCount === room.maxPlayers) {
                    onRoomFilled();
                } else {
                    // wait for others
                    EventSystem.on('onActorJoin', onActorJoin);
                }*/

                // room joined successfully
                EventSystem.off('onOperationResponse', onOperationResponse);
                EventSystem.off('onJoinRoom', onJoinRoom);

                // set callback immediately
                var photonMatch = new PhotonMatch(room);
                if (module.onMatchFound) {
                    module.onMatchFound(photonMatch);
                }
            };
            // waiting for a particular error, when roomCode is already in use
            var onOperationResponse = function (args) {
                var errorCode = args[0];
                var errorMsg = args[1];
                var code = args[2];
                var content = args[3];
                var Constants = window.Photon.LoadBalancing.Constants;

                if (errorCode === Constants.ErrorCode.GameIdAlreadyExists && code === 227 /*Constants.EventCode.MATCH*/ ) {
                    if (module.onMatchFound) {
                        module.onMatchFound(null, errorMsg);
                    }
                    // clean up
                    EventSystem.off('onOperationResponse', onOperationResponse);
                    EventSystem.off('onJoinRoom', onJoinRoom);
                } else if (errorCode === Constants.ErrorCode.GameClosed && code === Constants.EventCode.AppStats) {
                    if (module.onMatchFound) {
                        module.onMatchFound(null, errorMsg);
                    }
                    // clean up
                    EventSystem.off('onOperationResponse', onOperationResponse);
                    EventSystem.off('onJoinRoom', onJoinRoom);
                }

                console.log(errorCode, errorMsg, code, content);
            };

            if (client.isJoinedToRoom()) {
                client.leaveRoom();
            }

            // joining room may fail
            EventSystem.on('onOperationResponse', onOperationResponse);

            // no rooms
            /*if (roomList.length === 0) {
                // create a room
                roomNumber += 1;
                client.createRoom(playerName + '.' + roomNumber, {
                    maxPlayers: maxPlayers
                });
            } else {
                // TEMP: enter any room that is visible and not full
                client.joinRandomRoom();
                // how to detect failure?
            }*/

            if (create) {
                delete otherData.create;
                delete otherData.roomCode;
                delete otherData.propsListedInLobby;
                client.createRoom(roomCode, {
                    customGameProperties: otherData,
                    propsListedInLobby: propsListedInLobby,
                    maxPlayers: maxPlayers,
                    emptyRoomLiveTime: 60 * 1000 // room stays alive for 60 seconds after everyone leaves
                });
            } else {
                // find room with roomCode
                client.joinRoom(roomCode, {});
            }

            EventSystem.on('onJoinRoom', onJoinRoom);
        },
        onMatchFound: null,
        onInvitation: null,
        onPlayerDisconnects: null,
        sendData: function (match, data, sendMode) {
            client.raiseEvent(0, data);
        },
        sendDataTo: function (uniqueId, match, data, sendMode) {
            client.raiseEvent(1, data, {
                targetActors: [uniqueId]
            });
        },
        onDataRecieved: null,
        onErrorGeneral: function (args) {
            // alert matchmanager by sending an event
            EventSystem.fire('OnlineService.Error', args[1]);
        },
        login: function (onLogin, extraParams) {
            if (isConnecting) {
                return;
            }

            var onConnectDone = function () {
                // wait for room list
                EventSystem.on('onRoomList', onRoomList);
                var roomList = client.availableRooms();

                // listen for errors (e.g. timeout
                EventSystem.on('onStateChange.Error', module.onErrorGeneral);
            };
            var onError = function () {
                isConnecting = false;
                onLogin({
                    success: false,
                    error: "Error trying to connect"
                });
                // clean up
                EventSystem.off('onStateChange.JoinedLobby', onConnectDone);
                EventSystem.off('onStateChange.Error', onError);
            };
            var onRoomList = function (args) {
                var rooms = args[0];
                // console.log(rooms);
                isConnecting = false;
                loggedIn = true;
                onLogin({
                    success: true,
                    roomList: rooms
                });
                // clean up
                EventSystem.off('onRoomList', onRoomList);
                EventSystem.off('onStateChange.JoinedLobby', onConnectDone);
                EventSystem.off('onStateChange.Error', onError);

            };
            // set local name to guest
            playerName = extraParams.name || 'Guest';
            // custom property: set character
            var character = extraParams.character || 'boy';
            var level = extraParams.level || 0;
            var seed = extraParams.seed || 0;

            localActor.setName(playerName);
            localActor.setCustomProperty('character', character);
            localActor.setCustomProperty('level', level);
            localActor.setCustomProperty('seed', seed);
            EventSystem.on('updateCharacter', module.onCharacterChanged);

            client.connect();
            isConnecting = true;

            // successful connection is when lobby is joined
            EventSystem.on('onStateChange.JoinedLobby', onConnectDone);
            // error
            EventSystem.on('onStateChange.Error', onError);
        },
        isLoggedIn: function () {
            // TODO: catch when user goes offline?
            return loggedIn;
        },
        disconnect: function () {
            EventSystem.off('updateCharacter', module.onCharacterChanged);
            EventSystem.off('onStateChange.Error', module.onErrorGeneral);

            client.disconnect();
            loggedIn = false;
        }
    };
    return module;
});