/**
 * Deterministic randomizer based on seed
 * @moduleName Randomizer
 */
bento.define('modules/randomizer', [
    'bento',
    'bento/utils',
    'modules/mersennetwister'
], function (
    Bento,
    Utils,
    MersenneTwister
) {
    'use strict';
    var Randomizer = function (settings) {
        var seed = Utils.isDefined(settings.seed) ? settings.seed : Date.now();
        var randomizer = new MersenneTwister(seed);

        return {
            seed: seed,
            getSeed: function () {
                return this.seed;
            },
            setSeed: function (value) {
                this.seed = value;
                randomizer = new MersenneTwister(value);
            },
            getRandom: function (n) {
                return Math.floor(randomizer.random() * n);
            },
            getRandomRange: function (min, max) {
                var diff = max - min;
                return min + Math.floor(randomizer.random() * diff);
            },
            getRandomFloat: function (n) {
                return randomizer.random() * n;
            },
            getRandomRangeFloat: function (min, max) {
                var diff = max - min;
                return min + randomizer.random() * diff;
            }
        };
    };

    return Randomizer;
});