/**
 * Wrapper for ReplayKit cordova plugin
 * Only available for iOS
 * @moduleName ReplayKit
 * @snippet ReplayKit.isAvailable
ReplayKit.isAvailable(function (available) {
    $1
})
 * @snippet ReplayKit.record
ReplayKit.record()
 * @snippet ReplayKit.stop
ReplayKit.stop()
 * @snippet ReplayKit.isBroadcasting
ReplayKit.isBroadcasting(function (broadcasting) {
    $1
})
 * @snippet ReplayKit.startBroadcast
ReplayKit.startBroadcast()
 * @snippet ReplayKit.stopBroadcast
ReplayKit.stopBroadcast()
 */
bento.define('modules/replaykit', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween
) {
    'use strict';
    var nothing = function () {};
    var isAvailable = false;
    var exec;
    var hasPlugin = function () {
        return window.cordova && window.cordova.plugins && window.cordova.plugins.Replay;
    };
    // replacement for cordova plugin clobber
    var Replay = {
        isAvailable: function (success, error) {
            exec(function (available) {
                isAvailable = available;
                if (success) {
                    success(available);
                }
            }, nothing, 'CordovaReplay', 'isAvailable');
        },
        startRecording: function (isMicrophoneEnabled, success, error) {
            exec(success, error, "CordovaReplay", "startRecording", [isMicrophoneEnabled]);
        },
        stopRecording: function (success, error) {
            exec(success, error, "CordovaReplay", "stopRecording");
        },
        isRecording: function (success, error) {
            exec(success, error, "CordovaReplay", "isRecording");
        },
        startBroadcast: function (success, error) {
            exec(success, error, "CordovaReplay", "startBroadcast");
        },
        stopBroadcast: function (success, error) {
            exec(success, error, "CordovaReplay", "stopBroadcast");
        },
        isBroadcasting: function (success, error) {
            exec(success, error, "CordovaReplay", "isBroadcasting");
        },
        isBroadcastAvailable: function (success, error) {
            exec(success, error, "CordovaReplay", "isBroadcastAvailable");
        }
    };

    if (!hasPlugin()) {
        return {
            isAvailable: function (callback) {
                callback(false);
                return false;
            },
            record: nothing,
            stop: nothing,
            isRecording: function (callback) {
                callback(false);
            },
            // this requires iOS10
            startBroadcast: nothing,
            stopBroadcast: nothing,
            isBroadcasting: function (callback) {
                callback(false);
            },
            isBroadcastAvailable: function (callback) {
                callback(false);
            }
        };
    } else {
        exec = window.cordova.require('cordova/exec');
    }
    // cache isAvailable asap
    Replay.isAvailable();

    return {
        isAvailable: function (callback) {
            // returns cached value, more reliable to use the callback
            Replay.isAvailable(callback);
            return isAvailable;
        },
        isRecording: function (callback) {
            Replay.isRecording(callback, function () {});
        },
        record: function () {
            Replay.startRecording(true, function () {
                console.log('startRecording');
            }, function () {
                console.log('recording failed');
                EventSystem.fire('Notification', {
                    text: 'Recording failed to start.'
                });
            });
        },
        stop: function (success, failure) {
            Replay.stopRecording(success, failure);
        },
        isBroadcastAvailable: function (callback) {
            Replay.isBroadcastAvailable(callback);
        },
        isBroadcasting: function (callback) {
            Replay.isBroadcasting(callback);
        },
        startBroadcast: function (success, failure) {
            Replay.startBroadcast(success || function () {}, failure || function () {});
        },
        stopBroadcast: function (success, failure) {
            Replay.stopBroadcast(success || function () {}, failure || function () {});
        }
    };
});