/**
 * Export (and import) save file
 * @moduleName SaveExporter
 */
bento.define('modules/saveexporter', [
    'bento/utils',
    'bento',
    'modules/file'
], function (
    Utils,
    Bento,
    File
) {
    'use strict';
    var getCurrentSave = function () {
        var i;
        var len;
        var storageCopy = {};
        var keys = Bento.saveState.load('_keys');

        for (i = 0, len = keys.length; i < len; ++i) {
            /*if (excludedKeys.indexOf(keys[i]) >= 0) {
                // skip excluded keys
                continue;
            }*/
            storageCopy[keys[i]] = Bento.saveState.load(keys[i]);
        }

        return storageCopy;
    };
    var clearAndOverwriteSave = function (cloudSave) {
        var key;

        // clear local save file (?)
        Bento.saveState.clear();

        for (key in cloudSave) {
            Bento.saveState.save(key, cloudSave[key]);
        }
    };
    var writeWeb = function (fileName, data) {
        var jsonString;
        jsonString = JSON.stringify(data, null, 4);
        var blob = new window.Blob([jsonString], {
            type: "application/json"
        });
        var url = window.URL.createObjectURL(blob);
        var anchor = document.createElement('a');
        anchor.download = fileName;
        anchor.href = url;
        anchor.click();
    };
    var readWeb = function (onSuccess) {
        var onFileRead = function (e) {
            try {
                var json = JSON.parse(e.target.result);
                clearAndOverwriteSave(json);
                if (onSuccess) {
                    onSuccess();
                }
            } catch (err) {
                window.alert('Could not parse file.');
            }
        };
        var handleFileSelect = function (e) {
            var files = e.target.files,
                i,
                f,
                fileReader,
                out;
            for (i = 0; files[i]; i++) {
                fileReader = new window.FileReader();
                fileReader.readAsText(files[i]);
                fileReader.onload = (function (file) {
                    return onFileRead;
                })(f);
            }
        };
        // <input type="file" id="files" name="files[]" multiple />
        var input = document.createElement('input');
        input.setAttribute('type', 'file');
        input.addEventListener('change', handleFileSelect, false);
        input.click();
    };

    return {
        export: function (fileName, onSuccess) {
            if (fileName.indexOf('.json') < 0) {
                fileName += '.json';
            }
            if (Utils.isCocoonJs()) {
                File.write(fileName, getCurrentSave(), function () {
                    onSuccess(fileName);
                }, function (err) {
                    window.alert('Saving failed.\n' + err);
                });
            } else {
                writeWeb(fileName, getCurrentSave());
            }
        },
        import: function (fileName, onSuccess) {
            if (fileName.indexOf('.json') < 0) {
                fileName += '.json';
            }
            if (Utils.isCocoonJs()) {
                File.read(fileName, function (data) {
                    // load save file
                    clearAndOverwriteSave(data);
                    onSuccess(fileName);
                }, function (err) {
                    window.alert('Saving failed.\n' + err);
                });
            } else {
                readWeb(onSuccess);
            }
        }
    };
});