/**
 * Module that takes snapshots of the screen by redrawing it on a canvas
 * Warning: does not work in WebGL renderer for now
 * @moduleName SnapShot
 */
bento.define('modules/snapshot', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'bento/packedimage',
    'managers/resolution'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    Utils,
    Tween,
    PackedImage,
    ResolutionManager
) {
    'use strict';
    return function (settings) {
        var module,
            viewport = Bento.getViewport(),
            renderer = Bento.getRenderer(),
            canvas = document.createElement('canvas'),
            context = canvas.getContext('2d'),
            packedImage,
            photoEntity,
            scale = settings.scale || 1,
            offset = ResolutionManager.getOffset() || new Vector2(0, 0),
            width = viewport.width,
            height = viewport.height;

        canvas.width = Math.floor(width * scale);
        canvas.height = Math.floor(height * scale);

        if (ResolutionManager.isWideScreen()) {
            canvas.width = Math.floor(ResolutionManager.getWideScreenSize().width * scale);
            canvas.height = Math.floor(ResolutionManager.getWideScreenSize().height * scale);
        }

        if (context.imageSmoothingEnabled) {
            context.imageSmoothingEnabled = false;
        }
        if (context.webkitImageSmoothingEnabled) {
            context.webkitImageSmoothingEnabled = false;
        }
        if (context.mozImageSmoothingEnabled) {
            context.mozImageSmoothingEnabled = false;
        }

        module = {
            name: 'photo',
            // true when there is a screenshot ready to share
            tookPic: false,
            // true if the latest screenshot is not yet shared
            newPic: false,
            getCanvas: function () {
                return canvas;
            },
            getBase64: function () {
                return canvas.toDataURL();
            },
            /**
             * Take a screenshot.
             * @param  {Array} exclusions - which entities to exclude.
             */
            snap: function (exclusions) {
                var i = 0,
                    len = 0,
                    data = Bento.getGameData();
                if (renderer.name === 'webgl') {
                    return; //temp quick fix webgl
                }

                // swap context
                renderer.setContext(context);
                context.save();

                // context.translate(-offset.x, -offset.y);
                context.scale(scale, scale);

                // hide exclusions
                if (exclusions) {
                    for (i = 0, len = exclusions.length; i < len; ++i) {
                        if (exclusions[i]) {
                            exclusions[i].visible = false;
                        }
                    }
                }

                // draw on context
                Bento.objects.draw();

                // show exclusions again
                if (exclusions) {
                    for (i = 0, len = exclusions.length; i < len; ++i) {
                        if (exclusions[i]) {
                            exclusions[i].visible = true;
                        }
                    }
                }
                // restore
                context.restore();
                renderer.restoreContext();

                this.tookPic = true;
                this.newPic = true;

                // console.log(canvas.toDataURL());
                return canvas.toDataURL();
            },
            /**
             * Share the latest screenshot.
             * @param  {String}   text     The message to be shared along with the screenshot
             * @param  {Function} callback (optional)
             */
            share: function (text, callback) {
                this.newPic = false;
                window.Cocoon.Share.share({
                    message: text,
                    image: this.getBase64()
                }, callback);
            },
            getScale: function () {
                return scale;
            },
            setScale: function (value) {
                scale = value;
                canvas.width = Math.floor(width * scale);
                canvas.height = Math.floor(height * scale);

                if (ResolutionManager.isWideScreen()) {
                    canvas.width = Math.floor(ResolutionManager.getWideScreenSize().width * scale);
                    canvas.height = Math.floor(ResolutionManager.getWideScreenSize().height * scale);
                }
            },
            getData: function () {
                return {
                    renderer: renderer,
                    viewport: viewport
                };
            },
            dispose: function () {
                if (canvas && canvas.dispose) {
                    canvas.dispose();
                }
            }
        };

        return module;
    };
});