/**
 * Wrapper for Tenjin cordova plugin
 * @moduleName Tenjin
 */
bento.define('modules/tenjin', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween
) {
    'use strict';
    var nothing = function () {};
    var module = {
        transaction: nothing,
        androidTransaction: nothing,
        iosTransaction: nothing,
        sendEvent: nothing,
        sendEventAndValue: nothing
    };

    if (window.tenjin) {
        // apply tenjin functions to module object
        Utils.forEach(window.tenjin, function (fn, key) {
            module[key] = fn;
        });

        // some functions are broken, (bad devs!!)
        module.iosTransaction = function (productId, currency, quantity, priceString, transactionId, base64Receipt, success, failure) {
            var exec = window.cordova.require('cordova/exec');
            exec(
                success,
                failure,
                'Tenjin',
                'iosTransaction', [
                    productId,
                    currency,
                    quantity,
                    priceString,
                    transactionId,
                    base64Receipt
                ]
            );
        };
        module.androidTransaction = function (productId, currency, quantity, unitPrice, purchaseDataString, dataSignature, success, failure) {
            var exec = window.cordova.require('cordova/exec');
            exec(
                success,
                failure,
                'Tenjin',
                'androidTransaction', [
                    productId,
                    currency,
                    quantity,
                    unitPrice,
                    purchaseDataString,
                    dataSignature
                ]
            );
        };

        module.sendEvent = function (name) {
            window.tenjin.sendEvent(name, function () {}, function () {});
        };
        module.sendEventAndValue = function (name, value) {
            window.tenjin.sendEventAndValue(name, value, function () {}, function () {});
        };
    } else {
        Utils.log("WARNING: Tenjin plugin does not exist.");
    }

    return module;
});