/**
 * Swaps text module
 * @moduleName Text
 */
bento.define('modules/text', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'bento/gui/text',
    'modules/fontsettings',
    'modules/localization'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween,
    Text,
    FontSettings,
    Localization
) {
    'use strict';
    var hasFont = function () {
        // returns a language code for every language that has a different font setting
        if (Localization.getLanguage() === 'zh' ||
            Localization.getLanguage() === 'zt' ||
            Localization.getLanguage() === 'ko' ||
            Localization.getLanguage() === 'ru' ||
            Localization.getLanguage() === 'ja') {
            return true;
        } else {
            return false;
        }
    };
    var fontSettings = {
        'en': {
            font: 'font',
            fontSize: 16
        },
        'ja': {
            font: 'japanese',
            fontSize: 14,
            pixelStroke: false
        },
        'ko': {
            font: 'bmhanna',
            fontSize: 14,
            pixelStroke: false
        },
        'zh': {
            font: 'chinese',
            fontSize: 14,
            ySpacing: 2,
            pixelStroke: false
        },
        'zt': {
            font: 'chintrad',
            fontSize: 14,
            ySpacing: 2,
            pixelStroke: false
        },
        'ru': {
            font: 'roboto',
            fontSize: 12
        }
    };
    var getFontLang = function () {
        // returns a language code for every language that has a different font setting
        if (Localization.getLanguage() === 'zh' ||
            Localization.getLanguage() === 'zt' ||
            Localization.getLanguage() === 'ko' ||
            Localization.getLanguage() === 'ru' ||
            Localization.getLanguage() === 'ja') {
            return Localization.getLanguage();
        } else {
            return 'en';
        }
    };
    return function (settings) {
        var langFont = fontSettings[getFontLang()];
        var font = langFont.font || 'font';
        var pixelStroke = settings.pixelStroke;
        if (Utils.isDefined(langFont.pixelStroke)) {
            pixelStroke = langFont.pixelStroke;
        }
        // swap font
        settings.font = font;
        // swap pixelStroke
        settings.pixelStroke = pixelStroke;

        var text = new Text(settings);
        var setText = text.setText;
        text.setText = function (str, settingsObj) {
            // swap font
            if (settingsObj) {
                settingsObj.font = font;
            }
            return setText(str, settingsObj);
        };
        return text;
    };
});