/**
 * Video ad timer, counts down in game time
 * @moduleName VideoAdTimer
 */
bento.define('modules/videoadtimer', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    Utils,
    Tween
) {
    'use strict';
    var minutesUntilReset = 10;
    var counterBehavior = {
        start: function () {
            // console.log('video ad timer started');
        },
        destroy: function () {
            console.log('Warning: attempted to remove video ad timer');
            // put self back??
        },
        update: function (data) {
            if (videoAdTimer.videoAdTimer > 0) {
                videoAdTimer.videoAdTimer -= 1;
                if (videoAdTimer.videoAdTimer === 0) {
                    EventSystem.fire('videoAdReady');
                }
            } else {
                ++videoAdTimer.framesSinceReady;
            }
            // autosave every 10 seconds
            if (data.entity.videoAdTimer % 600 === 1) {
                Bento.saveState.save('videoAdTimer', data.entity.videoAdTimer);
            }
            if (videoAdTimer.framesSinceReady % 600 === 1) {
                Bento.saveState.save('framesSinceReady', videoAdTimer.framesSinceReady);
            }
        }
    };
    var videoAdTimer = new Entity({
        z: 0,
        name: 'videoAdTimer',
        global: true,
        updateWhenPaused: Infinity,
        components: [counterBehavior]
    });

    // public functions
    videoAdTimer.extend({
        videoAdTimer: Bento.saveState.load('videoAdTimer', 0),
        // amount of frames that have passed since ad was ready to be watched, saved in increments of 600, starts at 1
        framesSinceReady: Bento.saveState.load('framesSinceReady', 0),
        /**
         * Returns the time in string format mm:ss
         */
        getTimeString: function () {
            var totalSeconds = Math.floor(this.videoAdTimer / 60);
            var seconds = totalSeconds % 60;
            var minutes = Math.floor(totalSeconds / 60);
            var str = '' + ('00' + minutes).slice(-2) + ':' + ('00' + seconds).slice(-2);
            return str;
        },
        force: function () {
            videoAdTimer.videoAdTimer = 0;
            Bento.saveState.save('videoAdTimer', 0);
        },
        reset: function () {
            this.videoAdTimer = minutesUntilReset * 60 * 60;
            Bento.saveState.save('videoAdTimer', minutesUntilReset * 60 * 60);
            this.framesSinceReady = 0;
            Bento.saveState.save('framesSinceReady', 0);
        },
        canPlay: function () {
            if (!videoAdTimer.isAdded) {
                Bento.objects.attach(videoAdTimer);
            }
            return this.videoAdTimer <= 0;
        },
        setMinutesUntilReset: function (value) {
            if (value) {
                minutesUntilReset = value;
            } else {
                // reset to original
                minutesUntilReset = 5;
            }
        },
        getRewardAmount: function () {
            // 80-120
            return 80 + Utils.getRandom(41);
        },
    });

    // cheat
    window.gimmeAdNow = function () {
        videoAdTimer.videoAdTimer = 0;
    };

    return videoAdTimer;
});