/**
 * Screen for cutscenes
 */
bento.define('screens/cutscene', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/screen',
    'bento/tween',
    'bento/tiled',
    'gui/fade',
    'entities/camera',
    'gui/buttons/pause',
    'entities/cutscene',
    'components/eventlistener'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Screen,
    Tween,
    Tiled,
    Fade,
    Camera,
    PauseButton,
    Cutscene,
    EventListener
) {
    'use strict';
    var screen;
    var onShow = function (settings) {
        var viewport = Bento.getViewport();
        if (Utils.isDev()) {
            // use cached
            if (!settings) {
                settings = window.cachedSettings;
            } else {
                window.cachedSettings = settings;
            }
        }
        var mapToLoad = settings.mapToLoad;
        var fade = new Fade({
            wait: true,
            duration: 10,
            fadeIn: true,
            type: 'opacity',
            onComplete: function () {
                EventSystem.fire('startCutScene');
            }
        });
        var instances = [];
        var tiled = new Tiled({
            assetName: mapToLoad,
            spawnBackground: true,
            spawnEntities: true,
            attachEntities: false,
            maxCanvasSize: new Vector2(512, 512),
            onObject: function (object) {
                // spawn triggers
                if (!object.gid) {
                    object.position = Utils.toVector2(object);
                    Cutscene.tiled[object.name] = object;
                }
            },
            onSpawn: function (instance, object, extraData) {
                //automatic attaching is turned off
                instances.push(instance);
            },
            onSpawnComplete: function () {
                // synchronous completion
                screen.dimension = this.dimension.clone();

                Utils.forEach(instances, function (instance, i, l, breakLoop) {
                    Bento.objects.attach(instance);
                });

                fade.startFade();
            }
        });
        // gui
        var pauseButton = new PauseButton({});

        if (!settings.hidePauseButton) {
            Bento.objects.attach(pauseButton);
        }
        // wait for the end cutscene event
        Bento.objects.attach(new EventListener({
            name: 'endCutScene',
            onEvent: function () {
                Bento.screens.show(settings.next || 'screens/main', {
                    skip: true,
                    target: settings.target
                });
            }
        }));
    };
    screen = new Screen({
        onShow: onShow
    });

    return screen;
});