/**
 * For compability only
 * Checks if equipment needs to be exchanged for tokens
 */
bento.define('screens/exchange', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/screen',
    'bento/tween',
    'components/anchor',
    'gui/senseipopup',
    'gui/iap/chestoverlay',
    'managers/character',
    'managers/inventory',
    'managers/currency',
    'managers/save',
    'gui/coinreward',
    'gui/itemget',
    'managers/receiveitem',
    'gui/iap/offerpack'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Screen,
    Tween,
    Anchor,
    SenseiPopup,
    ChestOverlay,
    CharacterManager,
    InventoryManager,
    CurrencyManager,
    SaveManager,
    CoinReward,
    ItemGet,
    ReceiveItem,
    OfferPack
) {
    'use strict';
    var exchange = {
        'equipment/bandana1': {
            key: 'enemytokens/rat',
            amount: 2
        },
        'equipment/bandana2': {
            key: 'enemytokens/rat',
            amount: 3
        },
        'equipment/bandana3': {
            key: 'enemytokens/rat',
            amount: 4
        },
        'equipment/baseballglove': {
            key: 'enemytokens/mrpigeon',
            amount: 5
        },
        'equipment/beltblack': {
            key: 'enemytokens/bigrat',
            amount: 4
        },
        'equipment/beltgreen': {
            key: 'enemytokens/bigrat',
            amount: 3
        },
        'equipment/beltyellow': {
            key: 'enemytokens/bigrat',
            amount: 2
        },
        'equipment/boxingglove': {
            key: 'enemytokens/cat',
            amount: 6
        },
        'equipment/bulletproofvest': {
            key: 'enemytokens/gangster',
            amount: 4
        },
        'equipment/crowbar': {
            key: 'currency/coin',
            amount: 200
        },
        'equipment/nail': {
            key: 'enemytokens/punkgirl',
            amount: 5
        },
        'equipment/necklace': {
            key: 'enemytokens/twerka',
            amount: 6
        },
        'equipment/phone1': {
            key: 'enemytokens/ratguy',
            amount: 4
        },
        'equipment/phone2': {
            key: 'enemytokens/ratguy',
            amount: 5
        },
        'equipment/phone3': {
            key: 'enemytokens/ratguy',
            amount: 6
        },
        'equipment/powerbank': {
            key: 'currency/coin',
            amount: 400
        },
        'equipment/runningshoes': {
            key: 'currency/coin',
            amount: 150
        },
        'equipment/screwdriver': {
            key: 'enemytokens/ratgirl',
            amount: 3
        },
        'equipment/shades': {
            key: 'enemytokens/pigchef',
            amount: 10
        },
        'equipment/sneakers': {
            key: 'currency/coin',
            amount: 200
        },
        'equipment/spikedglove': {
            key: 'enemytokens/dog',
            amount: 5
        },
        'equipment/spikedshoe': {
            key: 'enemytokens/raccoon',
            amount: 3
        },
        'equipment/springedglove': {
            key: 'currency/coin',
            amount: 500
        },
        'equipment/stopwatch': {
            key: 'enemytokens/weasel',
            amount: 3
        },
        'equipment/wristwatch': {
            key: 'currency/coin',
            amount: 800
        },
        // consumables
        'consumables/quickcharge': {
            key: 'currency/coin',
            amount: 100
        },
        // consumables
        'consumables/dumbbell': {
            key: 'currency/coin',
            amount: 1000
        }
    };
    var extraCoins = {
        'equipment/bandana2': {
            key: 'currency/coin',
            amount: 125
        },
        'equipment/bandana3': {
            key: 'currency/coin',
            amount: 200
        },
        'equipment/phone2': {
            key: 'currency/coin',
            amount: 125
        },
        'equipment/phone3': {
            key: 'currency/coin',
            amount: 200
        },
        'equipment/beltblack': {
            key: 'currency/coin',
            amount: 200
        },
        'equipment/beltgreen': {
            key: 'currency/coin',
            amount: 125
        },
    };
    var onShow = function (settings) {
        var lastVersionPlayed = SaveManager.load('lastVersionPlayed');
        var backgroundPortrait = new Entity({
            name: 'bgPortrait',
            components: [
                new Sprite({
                    imageName: 'gui/bg-brick-portrait',
                    originRelative: new Vector2(0.5, 0)
                }),
                new Anchor({
                    screenPosition: new Vector2(0.5, 0),
                    onComplete: function () {
                        if (Anchor.isPortrait()) {
                            backgroundPortrait.visible = true;
                            backgroundLandscape.visible = false;
                        } else {
                            backgroundPortrait.visible = false;
                            backgroundLandscape.visible = true;
                        }
                    }
                })
            ]
        });
        var backgroundLandscape = new Entity({
            name: 'bgLandscape',
            components: [
                new Sprite({
                    imageName: 'gui/bg-brick-landscape',
                })
            ]
        });
        var receivingItems = [];
        /**
         * Loop through equipment and consumables and exchange for new items
         */
        var collectEquipment = function () {
            var collection = {}; // new items to receive
            // unequip any
            Utils.forEach(['boy', 'krillin', 'boxer', 'fitgirl'], function (character, i, l, breakLoop) {
                var equipment = InventoryManager.getEquipped(character);
                // return to inventory
                if (equipment[0]) {
                    InventoryManager.addItem(equipment[0].key, 1);
                    InventoryManager.setEquipped(0, null, character);
                }
                if (equipment[1]) {
                    InventoryManager.addItem(equipment[1].key, 1);
                    InventoryManager.setEquipped(1, null, character);
                }

                // exchange tokens
                var tokens = CurrencyManager.getTokens(character);
                var total = tokens.health + tokens.attack + tokens.skill;

                if (total) {
                    receivingItems.push({
                        key: 'playertokens/' + character,
                        amount: total
                    });
                }
                // remove old tokens
                SaveManager.save('token-' + character, {
                    health: 0,
                    attack: 0,
                    skill: 0
                });
            });

            // exchange consumables
            // check equipped consumabels
            var equippedConsumables = InventoryManager.getEquippedConsumables();
            var consumableSlot0 = equippedConsumables[0];
            var consumableSlot1 = equippedConsumables[1];
            if (consumableSlot0 && exchange[consumableSlot0.key]) {
                // unequip
                InventoryManager.addItem(consumableSlot0.key, 1);
                InventoryManager.setEquippedConsumable(0, null);
            }
            if (consumableSlot1 && exchange[consumableSlot1.key]) {
                // unequip
                InventoryManager.addItem(consumableSlot1.key, 1);
                InventoryManager.setEquippedConsumable(1, null);
            }
            // exchange consumables if needed
            var allConsumables = InventoryManager.getAllItems('consumables');
            Utils.forEach(allConsumables, function (item, i, l, breakLoop) {
                var consumableName = item.key;
                if (!exchange[consumableName]) {
                    // consumable does not need to be traded
                    return;
                }
                var newItem = exchange[consumableName];
                var newKey = newItem.key;
                var newAmount = newItem.amount;

                collection[newKey] = collection[newKey] || 0;
                collection[newKey] += newAmount;

                // clear in save file
                InventoryManager.setItem(item.key, 0);
            });

            // exchange equipment
            var allEquipment = InventoryManager.getAllItems('equipment');
            Utils.forEach(allEquipment, function (item, i, l, breakLoop) {
                var equipmentName = item.key;
                var newItem = exchange[equipmentName];
                var newKey = newItem.key;
                var newAmount = newItem.amount;

                collection[newKey] = collection[newKey] || 0;
                collection[newKey] += newAmount;

                // give some extra coins for higher tier equipment
                var extra = extraCoins[equipmentName];
                if (extra) {
                    collection[extra.key] = collection[extra.key] || 0;
                    collection[extra.key] += extra.amount;
                }

                // clear equipment in save file
                InventoryManager.setItem(item.key, 0);
            });

            // convert to array
            Utils.forEach(collection, function (amount, key, l, breakLoop) {
                receivingItems.push({
                    key: key,
                    amount: amount
                });
            });
        };
        /**
         * Compatibility for energy
         * Those who have max 10 energy are upgraded to 20 max energy
         */
        var setEnergy = function () {
            // set energy from 10 to 20
            var energy = SaveManager.load('energy-boy');
            if (energy.maximum === 10) {
                energy.maximum = 20;
                SaveManager.save('energy-boy', energy);
            }
        };
        /**
         * Check if old player left off at a roadbloack
         */
        var checkRoadBlocks = function () {
            // certain missions can not be unlocked anymore because
            // there was a roadblock
            var unlocked = SaveManager.load('unlockedLevels');
            var checkMission = function (mission) {
                // mission should be unlocked + played
                // if mission + 1 is still locked, the player could get stuck
                var missionStars = SaveManager.load('missionStars-' + mission);
                if (missionStars > -1 && unlocked.indexOf(mission + 1) < 0) {
                    // unlock mission + 1
                    unlocked.push(mission + 1);
                    SaveManager.save('unlockedLevels', unlocked);
                }
            };

            // roadblocks used to be placed after these certain missions
            checkMission(10);
            checkMission(20);
            checkMission(33);
            checkMission(50);
            checkMission(70);
        };
        /** 
         * Check if items need to be swapped
         * And perform compatibility checks
         */
        var showPopup = function () {
            collectEquipment();

            setEnergy();
            checkRoadBlocks();

            if (!receivingItems.length) {
                // nothing to give
                Bento.screens.show('screens/worldselect');
                return;
            }
            Bento.objects.attach(new SenseiPopup({
                modal: true,
                imageName: 'gui/shop/frog',
                onNoMoreText: function () {
                    this.dismiss();

                    // give the items
                    Bento.objects.attach(new ChestOverlay({
                        chest: 'classic',
                        contents: receivingItems,
                        onComplete: function () {
                            Bento.screens.show('screens/worldselect');
                        }
                    }));

                },
                onComplete: function () {},
                text: [
                    // this is for softlaunch players only, so no need to translate
                    "Hey there!",
                    "Beat Street has undergone big changes.",
                    "We now have single upgrade tokens!",
                    "Introducing the 'Figurine System':",
                    "It's similar to the equipment system,",
                    "except now you're able to upgrade figurines!",
                    "You will now receive new tokens.",
                    "Enjoy and keep fighting!"
                ]
            }));
        };
        var giveGems = function (amount, iapAmount) {
            Bento.objects.attach(new SenseiPopup({
                modal: true,
                imageName: 'gui/shop/frog',
                onNoMoreText: function () {
                    this.dismiss();

                    // 
                    var gemAmount = amount;

                    // for those who made purchases in the past: give more gems
                    if (Bento.saveState.load('iapAmount', 0) > 0) {
                        gemAmount = iapAmount;
                    }
                    ReceiveItem({
                        key: 'currency/gem',
                        amount: gemAmount
                    });
                    Bento.objects.attach(new ItemGet({
                        item: 'currency/gem',
                        amount: gemAmount,
                        onComplete: function () {
                            Bento.screens.show('screens/worldselect');
                        }
                    }));

                },
                onComplete: function () {},
                text: [
                    // this is for softlaunch players only, so no need to translate
                    "Hey there!",
                    "Thanks for playing Beat Street.",
                    "Here is a token of our appreciation!",
                    "Spend it well!"
                ]
            }));
        };
        var spawnStarterPack = function () {
            var offerPack = new OfferPack({
                z: -1,
                offer: 'starter',
                duration: 48
            });
            var unlocked = SaveManager.load('unlockedLevels').length;
            if (unlocked > 5) {
                offerPack.visible = false;
                Bento.objects.attach(offerPack);
                Bento.objects.remove(offerPack);
            }
        };
        /**
         * Check for previously unlocked characters to unlock iMessage Stickers
         */
        var checkIMessageStickers = function () {
            var unlocked = SaveManager.load('unlockedCharacters');
            Utils.forEach(unlocked, function (currentCharacter, i, l, breakLoop) {
                // unlock imessage stickers
                Utils.unlockSticker(currentCharacter + '0');
                Utils.unlockSticker(currentCharacter + '1');
                Utils.unlockSticker(currentCharacter + '2');
            });
        };

        Bento.objects.attach(backgroundPortrait);
        Bento.objects.attach(backgroundLandscape);

        checkIMessageStickers();

        if (lastVersionPlayed === '1.1.1' || lastVersionPlayed === '1.1.0') {
            giveGems(25, 160);
            // spawn an offer for everyone who have been playing older version
            spawnStarterPack();
        } else if (lastVersionPlayed === '1.1.2') {
            giveGems(10, 25);
            // spawn an offer for everyone who have been playing older version
            spawnStarterPack();
        } else if (lastVersionPlayed === '1.1.3' || lastVersionPlayed === '1.1.4') {
            giveGems(10, 25);
            // spawn gem offer for everyone who have been playing older version
            SaveManager.save('showOffer', 'gem');
        } else {
            // check for exchange before 1.1.0
            showPopup();
        }

        // store last version played
        SaveManager.save('lastVersionPlayed', window.VERSION);
    };

    return new Screen({
        onShow: onShow
    });
});