/**
 * Gacha screen between tower floors
 * Needs to be refactored if we ever make a gacha popup
 */
bento.define('screens/gacha', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/screen',
    'bento/tween',
    'components/anchor',
    'components/debug',
    'gui/worldselect/topbar',
    'components/fillscreen',
    'modules/bezier',
    'components/orientationlock',
    'gui/itemget',
    'managers/receiveitem',
    'globals',
    'gui/coinreward',
    'components/movable',
    'managers/save',
    'gui/fade',
    'gui/shadowtext',
    'managers/tower',
    'modules/localization',
    'gui/equipmentslots',
    'modules/matchmanager'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Screen,
    Tween,
    Anchor,
    Debug,
    Topbar,
    FillScreen,
    Bezier,
    OrientationLock,
    ItemGet,
    ReceiveItem,
    Globals,
    CoinReward,
    Movable,
    SaveManager,
    Fade,
    ShadowText,
    Tower,
    Localization,
    EquipmentSlots,
    MatchManager
) {
    'use strict';
    var GachaBall = function (settings) {
        var bottom = settings.bottom || 'normal';
        var gachaTop = new Entity({
            name: 'top',
            position: new Vector2(0, 0),
            components: [
                new Sprite({
                    imageName: 'gacha/capsules/top',
                    originRelative: new Vector2(0.5, 0.5)
                })
            ]
        });
        var gachaBottom = new Entity({
            name: 'bottom',
            position: new Vector2(0, 0),
            components: [
                new Sprite({
                    imageName: 'gacha/capsules/' + bottom,
                    originRelative: new Vector2(0.5, 0.5)
                })
            ]
        });
        var entity = new Entity({
            z: settings.z,
            name: 'gachaBall',
            position: settings.position,
            components: [
                gachaBottom,
                gachaTop
            ]
        });
        return entity;
    };
    var Gravity = function (settings) {
        var entity;
        var component = {
            name: 'gravity',
            velocity: settings.velocity,
            gravity: settings.gravity,
            spin: settings.spin,
            update: function () {
                entity.rotation += component.spin;
                entity.position.x += component.velocity.x;
                entity.position.y += component.velocity.y;
                component.velocity.y += component.gravity;
            },
            attached: function (data) {
                entity = data.entity;
            }
        };
        return component;
    };

    var onShow = function (settings) {
        var viewport = Bento.getViewport();
        var nextScreen = settings ? settings.next : null;
        var background = new Entity({
            z: 0,
            name: 'background',
            position: new Vector2(0, 0),
            components: [
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.5, 0.5),
                        offset: new Vector2(0, 0),
                    },
                    landscape: {
                        screenPosition: new Vector2(0.5, 0.5),
                        offset: new Vector2(0, -16),
                    },
                    onComplete: function () {}
                }),
                new Sprite({
                    imageName: 'gacha/background',
                    originRelative: new Vector2(0.5, 0.5),
                }),
                new OrientationLock({}) // do not allow rotations here
            ]
        });

        var reflection = new Entity({
            z: 0,
            name: 'reflection',
            position: new Vector2(0, 0),
            components: [
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.5, 0.5),
                        offset: new Vector2(0, 172),
                    },
                    landscape: {
                        screenPosition: new Vector2(0.5, 0.5),
                        offset: new Vector2(0, 156),
                    }
                }),
                new Sprite({
                    imageName: 'gacha/reflection',
                    originRelative: new Vector2(0.5, 0.5),
                }),
            ],
        });
        var shadow = new Entity({
            z: 0,
            name: 'shadow',
            position: new Vector2(0, 0),
            components: [
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.5, 0.5),
                        offset: new Vector2(0, 95),
                    },
                    landscape: {
                        screenPosition: new Vector2(0.5, 0.5),
                        offset: new Vector2(0, 80),
                    }
                }),
                new Sprite({
                    imageName: 'gacha/shadow',
                    originRelative: new Vector2(0.5, 0.5),
                }),
            ],
        });

        var overlay = new Entity({
            z: 3,
            name: 'overlay',
            components: [
                new FillScreen({
                    color: [0, 0, 0, 1]
                })
            ]
        });
        var reward = Tower.getRandomReward();
        var gachaBehavior = {
            name: 'gachaBehavior',
            hasClicked: false,
            getBall: function () {
                var vsSplash = new Sprite({});
                var vsSplashEntity = new Entity({
                    z: 1,
                    name: 'vsSplash',
                    rotation: Anchor.isPortrait() ? 0 : Math.PI / 2,
                    position: viewport.getExtents(),
                    components: [
                        vsSplash,
                    ]
                });
                gachaBehavior.hasClicked = true;
                // add vs splash (copied from bossintro)
                vsSplash.setSpriteSheet('bossintro/vs-flash', function () {
                    vsSplashEntity.removeSelf();
                });
                if (Anchor.isWideScreen() && Anchor.isPortrait()) {
                    vsSplashEntity.scale.y = (viewport.height + Anchor.getOffset().y * 2) / 320;
                }
                Bento.audio.stopSound('sfx_getenergy');
                Bento.audio.playSound('sfx_getenergy');
                Bento.objects.attach(vsSplashEntity);
                // shake machine
                gachaSprite.setSpriteSheet('gacha/on');
                new Tween({
                    from: 0,
                    to: 0.1,
                    in: 30,
                    ease: 'easeOutElastic',
                    onUpdate: function (v, t) {
                        gachaMachine.scale = Vector2(0.9 + v, 1.1 - v);
                    },
                    onComplete: function () {
                        // show overlay
                        Bento.objects.attach(overlay);

                        // tween in overlay
                        new Tween({
                            from: 0,
                            to: 0.4,
                            in: 30,
                            ease: 'linear',
                            onUpdate: function (v, t) {
                                overlay.alpha = v;
                            },
                            onComplete: function () {
                                // show ball
                                gachaBehavior.showGachaBall();
                            }
                        });
                    }
                });


                // fade out arrow
                new Tween({
                    from: 1,
                    to: 0,
                    in: 20,
                    ease: 'linear',
                    onUpdate: function (v, t) {
                        arrow.alpha = v;
                    }
                });

            },
            showGachaBall: function () {
                var gachaBall = new GachaBall({
                    z: 3,
                    position: gachaMachine.position.add(new Vector2(16, -32)),
                    bottom: reward.category === 'currency' ? 'golden' : 'normal'
                });
                var tweenIn = function () {
                    new Tween({
                        from: 0.5,
                        to: 0.75,
                        in: 10,
                        ease: 'linear',
                        onStart: function () {
                            Bento.audio.stopSound('sfx_lootappear');
                            Bento.audio.playSound('sfx_lootappear');
                        },
                        onUpdate: function (v, t) {
                            gachaBall.scale.x = v;
                            gachaBall.scale.y = v;
                        }
                    });
                    new Tween({
                        from: 0,
                        to: 1,
                        in: 10,
                        ease: 'linear',
                        onUpdate: function (v, t) {
                            gachaBall.alpha = v;
                        },
                        onComplete: tweenTowardsMiddle
                    });
                };
                var tweenTowardsMiddle = function () {
                    var curve = new Bezier(
                        gachaBall.position,
                        new Vector2(
                            viewport.width * 0.25,
                            viewport.height * 0.75
                        ),
                        viewport.getExtents()
                    );
                    var delay = 20;
                    new Tween({
                        from: 0,
                        to: 1,
                        in: 60,
                        delay: delay,
                        applyOnDelay: true,
                        ease: 'easeInOutBack',
                        onUpdate: function (v, t) {
                            var curvePos = Utils.toVector2(curve.get(v));
                            gachaBall.position.x = curvePos.x;
                            gachaBall.position.y = curvePos.y;
                        },
                        onComplete: function () {

                        }
                    });
                    new Tween({
                        from: gachaBall.scale.x,
                        to: 1,
                        in: 60,
                        delay: delay,
                        applyOnDelay: true,
                        ease: 'linear',
                        onUpdate: function (v, t) {
                            gachaBall.scale.x = v;
                            gachaBall.scale.y = v;
                        },
                        onComplete: function () {
                            openBall();
                        }
                    });
                };
                var openBall = function () {
                    Bento.audio.playSound('sfx_gift_shrink');
                    new Tween({
                        from: 0,
                        to: 1,
                        in: 30,
                        delay: 15,
                        ease: 'linear',
                        onUpdate: function (v, t) {
                            // shake
                            gachaBall.transform.x = Utils.getRandomRangeFloat(-1, 1);
                            gachaBall.transform.y = Utils.getRandomRangeFloat(-1, 1);
                        },
                        onComplete: function () {
                            var top = gachaBall.getComponent('top');
                            var bottom = gachaBall.getComponent('bottom');
                            top.removeSelf();
                            bottom.removeSelf();
                            gachaBall.removeSelf();
                            showItemGet(tweenOut);

                            // set position
                            top.position = gachaBall.position.clone();
                            bottom.position = gachaBall.position.clone();
                            // place over overlay
                            top.z = Globals.WorldSelectLayers.GIFTS + 2;
                            bottom.z = Globals.WorldSelectLayers.GIFTS + 2;
                            // unpause
                            top.updateWhenPaused = Bento.objects.isPaused();
                            bottom.updateWhenPaused = Bento.objects.isPaused();
                            top.attach(new Gravity({
                                spin: Utils.getRandomRangeFloat(-0.2, 0.2),
                                velocity: new Vector2(
                                    Utils.getRandomRangeFloat(1, 2),
                                    Utils.getRandomRangeFloat(-5, -3)
                                ),
                                gravity: 0.15
                            }));
                            bottom.attach(new Gravity({
                                spin: Utils.getRandomRangeFloat(-0.2, 0.2),
                                velocity: new Vector2(
                                    Utils.getRandomRangeFloat(-2, 2),
                                    Utils.getRandomRangeFloat(2, 4)
                                ),
                                gravity: 0.15
                            }));
                            Bento.objects.attach(top);
                            Bento.objects.attach(bottom);
                        }
                    });
                };
                var showItemGet = function (onComplete) {
                    var item = reward.item;
                    var amount = reward.amount;
                    var category = reward.category;
                    var itemGet = new ItemGet({
                        z: Globals.WorldSelectLayers.GIFTS + 1,
                        item: item,
                        amount: amount,
                        autoDismiss: true,
                        onComplete: onComplete
                    });

                    // swap with coin reward if needed
                    if (item === 'currency/coin') {
                        itemGet = new CoinReward({
                            z: Globals.WorldSelectLayers.GIFTS + 1,
                            coins: amount,
                            onComplete: onComplete
                        });
                    }

                    Bento.objects.attach(itemGet);

                    // give actual reward
                    ReceiveItem({
                        key: item,
                        amount: amount
                    });

                    // tower: equip item
                    if (SaveManager.load('isTower') && (category === 'consumables' || category === 'weapons')) {
                        equipImmediately(reward.item);
                    }

                    var currentFloor = SaveManager.load('towerFloor');
                    var collectedGachas = SaveManager.load('towerGachas');
                    // save into towerGachas
                    collectedGachas[currentFloor] = true;
                    SaveManager.save('towerGachas', collectedGachas);
                };
                var equipImmediately = function (item) {
                    var consumables = new EquipmentSlots({
                        iconImage: 'gui/startlevel/inventory',
                        type: 'consumable',
                        slotSize: 'new',
                        // hideSlot2: true,
                        position: new Vector2(0, 4),
                    });
                    var itemsInInventory;
                    var slot1 = consumables.getSlot(0);
                    var slot2 = consumables.getSlot(1);
                    var randomItem;

                    // fill slot 1
                    if (!slot1) {
                        consumables.setSlot(0, item);
                    } else if (!slot2) {
                        consumables.setSlot(1, item);
                    }
                };
                var tweenOut = function () {
                    new Tween({
                        from: overlay.alpha,
                        to: 1,
                        in: 30,
                        ease: 'linear',
                        onUpdate: function (v, t) {
                            overlay.alpha = v;
                        },
                        onComplete: function () {
                            // exit
                            if (nextScreen) {
                                Bento.screens.show(nextScreen);
                            }
                        }
                    });
                };

                gachaBall.scale = new Vector2(0.75, 0.75);
                Bento.objects.attach(gachaBall);

                // tween in
                tweenIn();
            }
        };
        var gachaSprite = new Sprite({
            spriteSheet: 'gacha/off',
            animations: {
                "default": {
                    frames: [0]
                },
                "bounceIn": {
                    speed: 0.2,
                    frames: [5, 4, 3, 2, 1, 0]
                },
                "on": {
                    speed: 0.2,
                    frames: [0, 1, 2, 3, 4, 5]
                }
            }
        });
        window.gachaSprite = gachaSprite;
        var arrowText = new ShadowText({
            position: new Vector2(32, -64),
            text: Localization.getText('tapToCollect'),
            font: 'font',
            fontSize: 18,
            rotation: -Math.PI / 10,
            fontColor: '#dc3838',
            shadowColor: '#dc7338',
            shadowOffset: new Vector2(0, -1),
            align: 'center',
            textBaseline: 'middle',
            strokeStyle: '#39323c',
            lineWidth: 2,
            pixelStroke: true,
            maxWidth: 80,
            maxHeight: 80,
            linebreaks: true,
            drawDebug: false,
            components: [
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0, 0),
                        offset: new Vector2(28, -54),
                    },
                    landscape: {
                        screenPosition: new Vector2(0, 0),
                        offset: new Vector2(-64, 0),
                    },
                    onComplete: function () {}
                }), //
                {
                    name: 'rotateBehavior',
                    update: function () {
                        arrowText.rotation = -Math.PI / 10 + Math.sin(arrowText.timer / 15) * Math.PI / 80;
                    }
                }
            ]
        });
        var arrow = new Entity({
            z: 2,
            name: 'arrow',
            components: [
                new Sprite({
                    spriteSheet: 'gacha/arrow'
                }),
                arrowText
            ]
        });
        var gachaMachine = new Entity({
            z: 2,
            name: 'gacha',
            components: [
                // new Debug({
                //     move: true
                // }),
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.5, 0.5),
                        offset: new Vector2(0, 100),
                    },
                    landscape: {
                        screenPosition: new Vector2(0.5, 0.5),
                        offset: new Vector2(0, 86),
                    },
                    onComplete: function () {
                        if (Anchor.isPortrait()) {
                            arrow.position.x = -16;
                            arrow.position.y = -168;
                        } else {
                            arrow.position.x = -40;
                            arrow.position.y = -100;
                        }
                    }
                }),
                gachaBehavior,
                gachaSprite,
                arrow,
                new Clickable({
                    onClick: function (data) {
                        // if online, click automatically to keep the pace in
                        if (!MatchManager.isOnline && !gachaBehavior.hasClicked && didBounce) {
                            gachaBehavior.getBall();
                        }
                    },
                })
            ]
        });
        var topBar = new Topbar({
            preventClick: true
        });
        var fade = new Fade({
            duration: 60,
            fadeIn: true,
            onComplete: function () {
                bounceIn();
            }
        });
        var didBounce = false;
        var bounceIn = function () {
            var reflectionStartPos = reflection.position;
            var time = 50;
            var bounces = 0;
            new Tween({
                from: 1,
                to: 0,
                in: time,
                ease: 'easeOutBounce',
                onStart: function () {
                    arrow.visible = true;
                    gachaMachine.visible = true;
                    reflection.visible = true;
                    shadow.visible = true;
                },
                onUpdate: function (v, t) {
                    shadow.alpha = 1 - v / 2 - 0.4;
                    shadow.scale = Vector2(1 + v, 1 + v);
                    reflection.position = new Vector2(reflectionStartPos.x, reflectionStartPos.y + v * 100);
                    gachaMachine.transform.y = -v * 200;
                    // gachaSprite.setOriginRelative(Vector2(0.5, 1 + v * 0.5));
                    if (t > time / 3 && !bounces) {
                        bounces++;
                        gachaSprite.setSpriteSheet('gacha/bounce');
                        Bento.audio.playSound('sfx_chestdrop');
                    }
                    if (t > time / 1.5 && bounces < 2) {
                        bounces++;
                        Bento.audio.playSound('sfx_chestdrop');
                    }
                },
                onComplete: function () {
                    gachaSprite.setSpriteSheet('gacha/off');
                    didBounce = true;

                    if (MatchManager.isOnline) {
                        // automatically get ball because an online user can stall here
                        Utils.timeout(30, function () {
                            gachaBehavior.getBall();
                        });
                    }
                }
            });
        };

        Bento.objects.attach(background);
        Bento.objects.attach(reflection);
        Bento.objects.attach(shadow);
        Bento.objects.attach(gachaMachine);
        // Bento.objects.attach(topBar);
        // 
        arrow.visible = false;
        gachaMachine.visible = false;
        reflection.visible = false;
        shadow.visible = false;


        Bento.audio.stopAllMusic();
        new Tween({
            ignoreGameSpeed: true,
            in: 3,
            onComplete: function () {
                Bento.audio.setVolume(1, 'bgm_chest');
                Bento.audio.playMusic('bgm_chest');
            }
        });
    };

    return new Screen({
        onShow: onShow
    });
});