/**
 * This screen is only shown when users buys IAP outside the IAP menu or Offerpack menu
 * The reward is given here
 */
bento.define('screens/iap', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/screen',
    'bento/tween',
    'components/anchor',
    'gui/senseipopup',
    'modules/localization',
    'modules/store',
    'managers/currency',
    'gui/iap/offerpackitems',
    'gui/iap/offerpack',
    'managers/receiveitem',
    'gui/worldselect/topbar',
    'gui/itemget',
    'managers/save',
    'modules/analytics'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Screen,
    Tween,
    Anchor,
    SenseiPopup,
    Localization,
    Store,
    CurrencyManager,
    OfferPackItems,
    OfferPack,
    ReceiveItem,
    Topbar,
    ItemGet,
    SaveManager,
    Analytics
) {
    'use strict';
    var onShow = function (settings) {
        var backgroundPortrait = new Entity({
            name: 'bgPortrait',
            components: [
                new Sprite({
                    imageName: 'gui/bg-brick-portrait',
                    originRelative: new Vector2(0.5, 0)
                }),
                new Anchor({
                    screenPosition: new Vector2(0.5, 0),
                    onComplete: function () {
                        if (Anchor.isPortrait()) {
                            backgroundPortrait.visible = true;
                            backgroundLandscape.visible = false;
                        } else {
                            backgroundPortrait.visible = false;
                            backgroundLandscape.visible = true;
                        }
                    }
                })
            ]
        });
        var backgroundLandscape = new Entity({
            name: 'bgLandscape',
            components: [
                new Sprite({
                    imageName: 'gui/bg-brick-landscape',
                })
            ]
        });
        var topBar = new Topbar({});
        var unresolvedPurchases = Bento.saveState.load('unresolvedPurchases', []);
        var checkIap = function () {
            if (unresolvedPurchases.length === 0) {
                done();
                return;
            }
            Bento.objects.attach(new SenseiPopup({
                modal: true,
                imageName: 'gui/shop/frog',
                onNoMoreText: function () {
                    this.dismiss();

                    // give the items
                    resolvePurchases();
                },
                onComplete: function () {},
                text: Localization.getText('iapPurchasedThanks')
            }));
        };
        var resolvePurchases = function () {
            var totalGemsReceived = 0;
            var popups = [];
            // show popups (such as gems or offerpack results)
            var showNextPopup = function () {
                if (popups.length) {
                    Bento.objects.attach(popups.pop());
                } else {
                    // no more popups
                    done();
                }
            };
            // give gems
            var buyGems = function (iapId) {
                CurrencyManager.addGems(Utils.rewardsIAP[iapId], true);
                totalGemsReceived += Utils.rewardsIAP[iapId];
            };
            // give offerpack
            var buyOfferPack = function (iapId) {
                var offerId = Utils.IapToOffer[iapId];
                var offerPacksIAP = OfferPack.getOfferPacksIap();
                // show the items you bought
                var contents = new OfferPackItems({
                    z: 1,
                    title: Localization.getText('offerpacks')[offerId],
                    items: offerPacksIAP[offerId],
                    claim: true,
                    onClaim: showNextPopup
                });
                popups.push(contents);

                var offersBought = SaveManager.load('offersBought');
                offersBought[offerId] = offersBought[offerId] || 0;
                offersBought[offerId] += 1;
                SaveManager.save('offersBought', offersBought);

                // add the items to your inventory
                Utils.forEach(offerPacksIAP[offerId], function (amount, key) {
                    ReceiveItem({
                        key: key,
                        amount: amount
                    });
                });
                // special case: daily gems
                if (offerId === 'gem') {
                    SaveManager.add('dailyGems', 30);
                }

                Analytics.event('OfferPack', offerId, 1);
            };

            Utils.forEach(unresolvedPurchases, function (info, i, l, breakLoop) {
                // determine purchase type
                var iapId = info.productId;
                // gems
                if (Utils.rewardsIAP[iapId]) {
                    buyGems(iapId);
                }
                // offerpack 
                else if (Utils.IapToOffer[iapId]) {
                    buyOfferPack(iapId);
                }
            });
            // resolve all purchases
            Store.resolveAllPurchases();

            if (totalGemsReceived) {
                // only show gems once
                var itemGet = new ItemGet({
                    z: 1,
                    item: 'currency/gem',
                    amount: totalGemsReceived,
                    onComplete: showNextPopup
                });
                popups.push(itemGet);
            }

            // show all popups
            showNextPopup();
        };
        var done = function () {
            Bento.screens.show('screens/worldselect');
        };

        Bento.objects.attach(backgroundPortrait);
        Bento.objects.attach(backgroundLandscape);
        Bento.objects.attach(topBar);

        checkIap();
    };

    return new Screen({
        onShow: onShow
    });
});