/**
 * Main screen
 */
bento.define('screens/main', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/screen',
    'bento/tween',
    'bento/tiled',
    'entities/tilecolliders',
    'entities/character',
    'components/characters/player',
    'components/camerabehavior',
    'entities/camera',
    'components/characters/crate',
    'components/characters/punchbot',
    'entities/cameralocker',
    'globals',
    'entities/triggers/exit',
    'gui/fade',
    'components/controllers/keyboardcontroller',
    'components/controllers/enemycontroller',
    'managers/combo',
    'gui/main/combo',
    'gui/main/healthbar',
    'entities/triggers/solid',
    'gui/buttons/pause',
    'entities/triggers/lock',
    'entities/players/boy',
    'modules/matchmanager',
    'entities/triggers/blockade',
    'entities/bossintro',
    'components/boss',
    'gui/equipped',
    'gui/menus/gameover',
    'entities/triggers/finish',
    'managers/item',
    'entities/triggers/enemyspawnheight',
    'entities/triggers/solidplayer',
    'managers/resolution',
    'managers/save',
    'managers/stats',
    'managers/character',
    'managers/enemyactivator',
    'components/eventlistener',
    'gui/notification',
    'modules/localization',
    'gui/main/consumable',
    'gui/loadingdialog',
    'gui/dialog',
    'entities/weapon',
    'managers/buff',
    'modules/analytics',
    'entities/zoom',
    'components/drawinviewport',
    'gui/taphand',
    'managers/inventory',
    'gui/senseipopup',
    'components/punchable',
    'entities/photosnapper',
    'entities/triggers/exitdojo',
    'entities/tutorial/gesture',
    'gui/menus/scroll',
    'components/anchor',
    'components/fillscreen',
    'components/healthbar',
    'modules/randomizer',
    'components/paralax',
    'managers/tower',
    'gui/shadowtext',
    'gui/main/emotemenu'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Screen,
    Tween,
    Tiled,
    TileColliders,
    Character,
    PlayerBehavior,
    CameraBehavior,
    Camera,
    CrateBehavior,
    PunchbotBehavior,
    CameraLocker,
    Globals,
    Exit,
    Fade,
    KeyboardController,
    EnemyController,
    ComboManager,
    ComboGui,
    HealthBar,
    Solid,
    PauseButton,
    Lock,
    PlayerBoy,
    MatchManager,
    Blockade,
    BossIntro,
    Boss,
    Equipped,
    GameOverMenu,
    Finish,
    ItemManager,
    EnemySpawnHeight,
    SolidPlayer,
    ResolutionManager,
    SaveManager,
    StatsManager,
    CharacterManager,
    EnemyActivator,
    EventListener,
    Notification,
    Localization,
    ConsumableButton,
    LoadingDialog,
    Dialog,
    Weapon,
    BuffManager,
    Analytics,
    Zoom,
    DrawInViewport,
    TapHand,
    InventoryManager,
    SenseiPopup,
    Punchable,
    PhotoSnapper,
    ExitDojo,
    Gesture,
    ScrollMenu,
    Anchor,
    FillScreen,
    HealthBarComponent,
    Randomizer,
    Paralax,
    Tower,
    ShadowText,
    EmoteMenu
) {
    'use strict';
    /**
     * Screen show
     */
    var onShow = function (settings) {
        /* Screen starts here */
        var viewport = Bento.getViewport();

        var mapHasBoss = false;
        var fade = new Fade({
            wait: true,
            duration: 10,
            fadeIn: true,
            type: 'opacity',
            onComplete: function () {
                var didShowScroll = false;
                if (mapHasBoss && !isTower) {
                    // boss intro?
                    Bento.objects.attach(new BossIntro({}));
                }
                // show scroll
                if (currentSection === 1) {
                    didShowScroll = openNewScroll();
                }

                // item tutorial
                var equippedConsumables = InventoryManager.getEquippedConsumables();
                if (!didShowScroll &&
                    SaveManager.load('tutorialItem') === 0 &&
                    (equippedConsumables[0] || equippedConsumables[1])
                ) {
                    SaveManager.save('tutorialItem', 2);

                    Bento.objects.attach(new SenseiPopup({
                        darken: false,
                        modal: false,
                        onNoMoreText: function () {
                            var self = this;
                            Utils.timeout(60, function () {
                                self.dismiss();
                                new Tween({
                                    from: 1,
                                    to: 0,
                                    in: 30,
                                    onUpdate: function (v, t) {
                                        tapHand.alpha = v;
                                    },
                                    onComplete: function () {
                                        tapHand.removeSelf();
                                    }
                                });
                            });
                        },
                        text: Localization.getText('tutorial')['useItem']
                    }));

                    // make taphand
                    var tapHand = new TapHand({
                        rotation: Math.PI / 4 * 3
                    }).attach(new EventListener({
                        name: 'consumableUsed',
                        onEvent: function () {
                            EventSystem.fire('dismissSensei');
                            tapHand.removeSelf();
                        }
                    }));
                    if (!Anchor.isPortrait()) {
                        tapHand.rotation = -Math.PI / 4 * 3;
                    }
                    // attach the hand to the right slot
                    if (equippedConsumables[0]) {
                        equipped.slot0.attach(tapHand);
                    } else if (equippedConsumables[1]) {
                        equipped.slot1.attach(tapHand);
                    }
                }

                // show title
                if (isTower) {
                    showMissionTitle(Localization.getText('towerfloorX').replace('{NUMBER}', SaveManager.load('towerFloor')));
                } else if (currentSection === 1 && !isEvent) {
                    showMissionTitle(Localization.getText('missionX').replace('{NUMBER}', SaveManager.load('currentMission')));
                }
            }
        });
        var camera = new Camera({}).attach(new CameraBehavior({}));
        var tileColliders = new TileColliders({});

        // gui
        var pauseButton = new PauseButton({});

        // cache all triggers spawned
        var triggers = [];
        // the current tiled map to load, based on currentMission and currentSection
        // these are set by StartButton, ExitTrigger or the message 'areaChange'
        var isEvent = SaveManager.load('isEvent');
        var isTower = SaveManager.load('isTower');
        var currentMission = SaveManager.load('currentMission');
        var currentSection = SaveManager.load('currentSection');
        var mapToLoad = 'missions/' + currentMission + '/' + currentSection;
        var debugLevel = Utils.getHashParameter('level') || ''; // debug level has priority over normal
        var currentMap = debugLevel || mapToLoad || 'city/demo';
        var tiled;
        var placeAtSpawnPoint = function (player) {
            // the start object should be placed somewhere...
            var start = Bento.objects.get('start');
            if (start) {
                player.position.x = start.position.x - 16 + 32 * (player.playerId % 2);
                player.position.y = start.position.y - 16 + 32 * Math.floor(player.playerId / 2);

                // place camera on the spawnPoint
                camera.forceTo(start.position);
            } else {
                Utils.log('ERROR: dev forgot to place start object');
            }
        };
        var spawnPlayer = function (position, onComplete) {
            // depending on the scrolls that the player has, we disable some moves
            var disableMoves = function () {
                if (!InventoryManager.getItem('scrolls/dash')) {
                    player.canDash = false;
                }
                if (!InventoryManager.getItem('scrolls/charge')) {
                    player.canCharge = false;
                }
                if (!InventoryManager.getItem('scrolls/jump')) {
                    player.getComponent('touchController', function (controller) {
                        controller.canJump(false);
                    });
                }
            };
            var currentCharacter = CharacterManager.getSelected();
            var Constructor = CharacterManager.getCharacterConstructor(currentCharacter);
            var player = new Constructor({
                position: new Vector2(viewport.width / 2, viewport.height / 2 + 80),
                id: MatchManager.isOnline ? MatchManager.localPlayer.playerId : 0,
                name: 'player',
                online: false,
                health: StatsManager.health
            });
            var healthBar = new HealthBar({
                target: player,
                health: StatsManager.health,
                maxHealth: StatsManager.getStat(currentCharacter, 'health')
            });
            var weaponData = SaveManager.load('currentWeapon');

            // spawn weapon if needed
            if (weaponData.type) {
                spawnWeapon(player, weaponData);
            }

            Bento.objects.attach(player);
            Bento.objects.attach(healthBar);

            // also track health in the stats manager
            player.getComponent('punchable', function (punchable) {
                punchable.onHealthChange(function (healthData) {
                    StatsManager.health = healthData.health;
                });
            });

            if (position) {
                player.position = position;
            } else {
                placeAtSpawnPoint(player);
            }

            disableMoves();

            if (onComplete) {
                onComplete(player, healthBar);
            }

            return player;
        };
        var spawnRemotePlayer = function (playerData) {
            var character = playerData.character;
            var Constructor = CharacterManager.getCharacterConstructor(character);
            var player = new Constructor({
                position: new Vector2(viewport.width / 2, viewport.height / 2 + 80),
                id: playerData.playerId,
                name: playerData.userName,
                online: true
            });
            var weaponData = playerData.weaponData;
            var healthBarSyncer = {
                name: 'healthBarSyncer',
                start: function () {
                    MatchManager.onMessage('updatePlayerHealth-' + playerData.playerId, healthBarSyncer.onMessage);
                },
                destroy: function () {
                    MatchManager.offMessage('updatePlayerHealth-' + playerData.playerId, healthBarSyncer.onMessage);
                },
                onMessage: function (msg) {
                    if (msg.health && msg.maxHealth) {
                        healthBarComponent.setHealth(msg.health / msg.maxHealth);
                    }
                    return true;
                }
            };
            var healthBarComponent = new HealthBarComponent({
                level: playerData.level,
                health: playerData.health,
                maxHealth: playerData.maxHealth,
                height: 58,
                alwaysVisible: true
            });

            player.attach(healthBarComponent);
            player.attach(healthBarSyncer);

            if (weaponData.type) {
                spawnWeapon(player, weaponData);
            }

            Bento.objects.attach(player);
            placeAtSpawnPoint(player);

            // rip
            if (playerData.health <= 0) {
                player.getComponent('stateMachine', function (stateMachine) {
                    stateMachine.setState('dead');
                });
            }

            return player;
        };
        var spawnWeapon = function (player, weaponData) {
            var weapon = new Weapon({
                type: weaponData.type,
                durability: weaponData.durability
            });
            // add to world
            Bento.objects.attach(weapon);
            // pick it up
            weapon.getComponent('weapon', function (weaponComponent) {
                weaponComponent.pickUp(player, true);
            });
        };
        /*
         * Spawn extra enemies for every group of enemies, up to the number of players
         * Note, that this operation is asynchronous
         */
        var extraEnemies = [];
        var blockades = [];
        var groomEnemies = function () {
            // this function "grooms" the enemies array
            // we decide which enemies are spawned double by removing or leaving the values in this array
            var groups = {};
            var playerSize = MatchManager.otherPlayers.length + 1;

            // sort the lists first by x-position
            extraEnemies.sort(function (a, b) {
                return a.object.x - b.object.x;
            });
            blockades.sort(function (a, b) {
                return a.x - b.x;
            });

            // split the enemies in groups
            Utils.forEach(extraEnemies, function (enemyData) {
                var currentGroup = 0;

                // find which group this belongs to by checking all the blockades
                Utils.forEach(blockades, function (blockade, i, l, breakLoop) {
                    if (enemyData.object.x < blockade.x) {
                        currentGroup = blockade.x;
                        breakLoop();
                    }
                });

                // get group by x position
                groups[currentGroup] = groups[currentGroup] || [];
                groups[currentGroup].push(enemyData);
            });


            // now that we have our groups, check their sizes
            Utils.forEach(groups, function (group, key, l, breakLoop) {
                var groupSize = group.length;
                var enemiesToSpawn = playerSize * 1 - groupSize; // 2 for each player?

                // trim the group array by adjusting their lengths
                group.length = Utils.clamp(0, enemiesToSpawn, group.length);
            });

            // re-assemble the groups into the enemies array
            extraEnemies = [];
            Utils.forEach(groups, function (group) {
                extraEnemies = extraEnemies.concat(group);
            });
        };
        var spawnExtraEnemies = function (onComplete) {
            var enemiesToLoad = 0;
            var enemiesLoaded = 0;
            var checkDone = function () {
                if (enemiesToLoad > 0 && enemiesToLoad === enemiesLoaded) {
                    onComplete();
                }
            };

            Utils.forEach(extraEnemies, function (enemyData, i, l, breakLoop) {
                var instance = enemyData.instance;
                var extraData = enemyData.extraData;
                var nameProperties = enemyData.nameProperties;

                if (MatchManager.otherPlayers.length && !nameProperties.boss) {
                    // note: the extra spawning is asynchronous
                    enemiesToLoad += 1;
                    bento.require([extraData.moduleName], function (Module) {
                        var extraSettings = Utils.copyObject(instance.settings);

                        // give copy rootPath
                        instance.getComponent('stateMachine', function (stateMachine) {
                            extraSettings.rootPath = stateMachine.data.rootPath;
                        });

                        var extraEnemy = new Module(extraSettings);
                        // position right of the first enemy
                        extraEnemy.position = instance.position.add(new Vector2(32, 0));
                        extraEnemy.name = 'copy-' + instance.name;
                        Bento.objects.attach(extraEnemy);

                        enemiesLoaded += 1;
                        checkDone();

                        // may also be a boss
                        if (nameProperties.boss) {
                            extraEnemy.attach(new Boss({
                                skipIntro: true
                            }));
                        }
                        // 50% extra health for each player
                        if (MatchManager.otherPlayers.length > 0) {
                            extraEnemy.getComponent('punchable', function (punchable) {
                                punchable.increaseHealth(MatchManager.otherPlayers.length * 50);
                            });
                        }
                    });
                }
            });

            // there was nothing to load: call onComplete immediately
            if (enemiesToLoad === 0) {
                onComplete();
            }
        };
        /*
         * Initialize players, called on Tiled.onSpawnComplete
         */
        var startGameAsync = function () {
            var i;
            var players = [];
            var receivedReady = false;
            // TODO: if a player disconnects during this phase, it ends up in disaster
            var messages = MatchManager.getMessages('playerReady');
            var startMessages = MatchManager.getMessages('startRoom');
            var waitBehavior = {
                name: 'waitBehavior',
                timer: 0,
                start: function () {
                    Bento.pause(Globals.PauseLevels.STARTSYNC);
                },
                destroy: function () {
                    Bento.resume();
                },
                update: function () {
                    var isReady = true;
                    if (!MatchManager.isOnline) {
                        // match may end while waiting
                        //playing on our own
                        fade.startFade();
                        Bento.objects.remove(playerWaiter);
                        Bento.objects.remove(loadingDialog);
                        EventSystem.fire('startRoom', {
                            local: true
                        });
                        return;
                    }
                    // everybody sends ready messages every 30 frames
                    if (waitBehavior.timer % 30 === 0) {
                        sendReady();
                    }
                    waitBehavior.timer += 1;

                    // read ready messages
                    Utils.forEach(messages, function (message) {
                        Utils.forEach(players, function (playerData, index, l, breakLoop) {
                            if (playerData.playerId === message.id && !playerData.ready) {
                                // update playerData from ready message
                                playerData.ready = true;
                                playerData.character = message.char;
                                playerData.level = message.level;
                                playerData.weaponData = message.weapon;
                                playerData.health = message.health;
                                playerData.maxHealth = message.maxHealth;
                            }
                        });
                    });
                    messages.length = 0;

                    Utils.forEach(players, function (playerData) {
                        if (!playerData.ready) {
                            isReady = false;
                        }
                    });
                    if (!isReady) {
                        return;
                    }
                    receivedReady = true;
                    if (!MatchManager.isMaster) {
                        /// check if startroom was sent
                        startMessages = MatchManager.getMessages('startRoom');
                        if (startMessages.length) {
                            startMessages.length = 0;
                            doneWaiting();
                        } else {}
                        return;
                    }

                    doneWaiting();

                    // master: send startRoom
                    if (MatchManager.isMaster) {
                        sendReady();
                        MatchManager.sendData({}, 'startRoom');
                    }

                }
            };
            var sendReady = function () {
                var character = CharacterManager.getSelected();
                MatchManager.sendData({
                    type: 'playerReady',
                    id: MatchManager.localPlayer.playerId,
                    char: character,
                    level: SaveManager.load('level-' + character),
                    weapon: SaveManager.load('currentWeapon'),
                    health: StatsManager.health,
                    maxHealth: StatsManager.getStat(character, 'health')
                }, 'playerReady');
            };
            var doneWaiting = function () {
                if (!receivedReady) {
                    // console.log('not ready for start')
                }

                // spawn remote players
                Utils.forEach(players, function (playerData) {
                    spawnRemotePlayer(playerData);
                });

                //all players are ready
                fade.startFade();
                Bento.objects.remove(playerWaiter);
                Bento.objects.remove(loadingDialog);
                EventSystem.fire('startRoom');
            };
            var playerWaiter = new Entity({
                name: 'playerWaiter',
                position: new Vector2(0, 0),
                updateWhenPaused: Globals.PauseLevels.STARTSYNC,
                components: [waitBehavior]
            });
            var loadingDialog = new LoadingDialog({
                text: Localization.getText('synchronizing'),
                modal: false,
                onClose: function () {
                    var dialog = new Dialog({
                        ignoreAnalytics: true,
                        title: Localization.getText('warningTitle'),
                        text: Localization.getText('confirmDisconnect'),
                        type: 'yesno',
                        canClose: false,
                        onYes: function () {
                            MatchManager.disconnect();
                            // return to worldselect
                            Bento.screens.show('screens/worldselect');
                        },
                        onNo: function () {
                            Bento.objects.attach(loadingDialog);
                        }
                    });
                    Bento.objects.attach(dialog);

                    // playerwaiter should never stop
                    playerWaiter.updateWhenPaused = Bento.objects.isPaused();
                }
            });
            var waitForPlayers = function () {
                // show loading dialog
                fade.alpha = 0;
                Bento.objects.attach(loadingDialog);
                loadingDialog.updateWhenPaused = Globals.PauseLevels.STARTSYNC;
                Bento.objects.attach(playerWaiter);
            };
            var playerData;

            // spawn player
            spawnPlayer();

            // collec remotes
            if (MatchManager.currentMatch) {
                for (i = 0; i < MatchManager.otherPlayers.length; i++) {
                    playerData = {
                        playerId: MatchManager.otherPlayers[i].playerId,
                        userName: MatchManager.otherPlayers[i].userName,
                        weaponData: {},
                        character: null,
                        level: 0,
                        health: 0,
                        maxHealth: 0,
                        ready: false
                    };
                    players.push(playerData);
                }
            }

            // wait for players if needed
            if (players.length === 0) {
                //playing on our own
                fade.startFade();
                EventSystem.fire('startRoom', {
                    local: true
                });
            } else {
                waitForPlayers();
            }

            //Activate the first blockade (if there is one)
            Blockade.activateNext();
        };
        /*
         * Increase enemy level for each extra multiplayer player
         * Note that this has to be done before the enemies are spawned
         */
        var increaseEnemyLevel = function () {
            var i;
            var originalEnemyLevel = SaveManager.load('originalEnemyLevel');
            var difficulty = SaveManager.load('difficulty');

            if (MatchManager.currentMatch) {
                // difficulty not allowed to be 0.2
                if (difficulty === 0.2) {
                    difficulty = 0.5;
                    SaveManager.save('difficulty', difficulty);
                }
                // increase enemy levels
                for (i = 0; i < MatchManager.otherPlayers.length; i++) {
                    originalEnemyLevel += 2;
                    if (i > 1) {
                        // 3 and 4 players: even more!
                        originalEnemyLevel += 1;
                    }
                }
            }
            // save into enemyLevel
            SaveManager.save('enemyLevel', originalEnemyLevel);
        };
        /*
         * Opens a recently collected scroll
         */
        var openNewScroll = function () {
            var scrolls = InventoryManager.getAllItems(['scrolls']);
            var showScroll;
            var scroll;

            // not during online
            if (MatchManager.currentMatch) {
                return false;
            }

            Utils.forEach(scrolls, function (itemData, i, l, breakLoop) {
                var hasSeen = SaveManager.load('openedScroll-' + itemData.key);
                if (!hasSeen) {
                    showScroll = itemData.key;
                    breakLoop();
                }
            });
            if (showScroll) {
                scroll = new ScrollMenu({
                    scrollType: showScroll
                });
                Bento.objects.attach(scroll);
                return true;
            }
            return false;
        };
        /**
         * Show mission title
         */
        var showMissionTitle = function (text) {
            var title = new ShadowText({
                text: text,
                font: 'font',
                fontSize: 24,
                fontColor: '#ffffff',
                align: 'center',
                textBaseline: 'middle',
                maxWidth: 160,
                linebreaks: false,
                drawDebug: false,
                components: []
            });
            var blur = new Entity({
                name: 'backBlur',
                components: [
                    new Sprite({
                        imageName: 'backblur',
                        originRelative: new Vector2(0.5, 0.5),
                    })
                ]
            });
            var backBlur = new Entity({
                z: Globals.Layers.GUI - 1,
                name: 'titleContainer',
                float: true,
                components: [
                    new Anchor({
                        screenPosition: new Vector2(0.5, 0.25),
                        offset: new Vector2(0, 0),
                    }),
                    blur,
                    title
                ]
            });
            // fit if needed
            if (blur.dimension.width - 40 < title.dimension.width) {
                blur.scale.x = title.dimension.width / (blur.dimension.width - 40);
            }

            // tween in
            new Tween({
                from: 0,
                to: 1,
                in: 30,
                ease: 'linear',
                onStart: function () {
                    Bento.objects.attach(backBlur);
                },
                onUpdate: function (v, t) {
                    backBlur.alpha = v;
                },
                onComplete: function () {

                }
            });

            // tween out
            new Tween({
                from: 1,
                to: 0,
                in: 60,
                delay: 120,
                ease: 'linear',
                onUpdate: function (v, t) {
                    backBlur.alpha = v;
                },
                onComplete: function () {
                    backBlur.removeSelf();
                }
            });
        };
        /**
         * Setup tower
         */
        var tower;

        if (isTower) {
            // before doing anything, should we show the gacha?
            if (Tower.shouldShowGacha()) {
                Bento.screens.show('screens/gacha', {
                    next: 'screens/main'
                });
                return;
            }
            // setup tower
            tower = new Tower({
                startGameAsync: startGameAsync,
                onHasBoss: function () {
                    mapHasBoss = true;
                },
                onComplete: function (data) {
                    // overwrite current section and map
                    currentSection = SaveManager.load('currentSection');
                    currentMission = SaveManager.load('currentMission');
                    currentMap = data.currentMap;
                }
            });
            tower.setup();
            Bento.objects.attach(tower);
        }

        // load cutscene?
        settings = settings || {};
        if (!settings.skip && currentSection === 1 && Utils.hasAsset('json', 'missions/' + currentMission + '/pre')) {
            Bento.screens.show('screens/cutscene', {
                mapToLoad: 'missions/' + currentMission + '/pre'
            });
            return;
        }

        //Reset the blockades
        Blockade.reset();
        EnemySpawnHeight.reset();

        // start spawning
        increaseEnemyLevel();
        tiled = new Tiled({
            assetName: currentMap,
            spawnBackground: true,
            spawnEntities: true,
            attachEntities: false,
            maxCanvasSize: new Vector2(512, 512),
            onObject: function (object, tileSet, tileIndex) {
                var spawnExit = function () {
                    // parse direction and nextmap from object name
                    var nameProperties = Utils.nameToJson(object.name);

                    var triggerData = {
                        rectangle: Utils.toRectangle(object),
                        // if name was not set, assume the exit is an exit to the right
                        direction: Exit.Direction[nameProperties.direction || 'right'],
                        object: object
                    };
                    var exit = new Exit(triggerData);
                    triggers.push(triggerData);
                    Bento.objects.attach(exit);
                };
                var spawnExitDojo = function () {
                    var exit = new ExitDojo({
                        rectangle: Utils.toRectangle(object)
                    });
                    Bento.objects.attach(exit);
                };
                var spawnLock = function () {
                    var toSpawnStr = (object.properties || {}).toSpawn || '';
                    // TODO: remove whitespaces?
                    // split into array
                    var toSpawn = [];
                    if (toSpawnStr) {
                        toSpawn = toSpawnStr.split(',');
                    }
                    Bento.objects.attach(new Lock({
                        rectangle: Utils.toRectangle(object),
                        toSpawn: toSpawn
                    }));
                };

                // spawn triggers (if they have a gid, they are linked to a tileSet)
                if (!object.gid) {
                    if (object.type === 'ExitTrigger') {
                        // place an exit trigger
                        spawnExit();
                    } else if (object.type === 'ExitDojo') {
                        // place an exit trigger specifically for the dojo
                        spawnExitDojo();
                    } else if (object.type === 'Solid') {
                        // place a solid
                        Bento.objects.attach(new Solid({
                            boundingBox: new Rectangle(object.x, object.y, object.width, object.height)
                        }));
                    } else if (object.type === 'SolidPlayer') {
                        // place a solid
                        Bento.objects.attach(new SolidPlayer({
                            boundingBox: new Rectangle(object.x, object.y, object.width, object.height)
                        }));
                    } else if (object.type === 'CameraLock') {
                        // spawn a cameralock but don't add it to the game, instead add it to the Lock trigger
                        Lock.cameraLockers.push(new CameraLocker({
                            size: Utils.toRectangle(object)
                        }));
                    } else if (object.type === 'LockTrigger') {
                        // place lock trigger
                        spawnLock();
                    } else if (object.type === 'Blockade') {
                        // place blocakde
                        Bento.objects.add(new Blockade({
                            position: new Vector2(object.x, object.y)
                        }));
                        // cache the positions
                        blockades.push(object);
                    } else if (object.type === 'Spawn') {
                        if (isTower) {
                            tower.onObjectSpawnPoint(object);
                        }
                        // spawnSpawner();
                    } else if (object.type === 'Finish') {
                        // place a solid
                        Bento.objects.attach(new Finish({
                            rectangle: Utils.toRectangle(object)
                        }));
                    } else if (object.type === 'EnemySpawnHeight') {
                        Utils.log('WARNING: EnemySpawnHeight is deprecated');
                        // EnemySpawnHeight.addArea(Utils.toRectangle(object));
                    }
                }
            },
            onTile: function (tileX, tileY, tilesetJson, tileIndex) {
                // spawn collider if tile type is 'solid'
                var tileproperties = tilesetJson.tileproperties;
                var tileProperty;
                if (tileproperties) {
                    tileProperty = tileproperties[tileIndex];
                    if (!tileProperty) {
                        return;
                    }
                    if (tileProperty.type === 'solid') {
                        tileColliders.insert(tileX * 16, tileY * 16);
                    } else if (tileProperty.type === 'triangleD') {
                        tileColliders.insert(tileX * 16, tileY * 16, 1);
                    } else if (tileProperty.type === 'triangleU') {
                        tileColliders.insert(tileX * 16, tileY * 16, -1);
                    }
                }
            },
            onComplete: function () {
                // synchronous completion
                screen.dimension = this.dimension.clone();
                Utils.forEach(this.backgrounds, function (backgroundLayer, i, l, breakLoop) {
                    // any layer that is named "foreground" will be put to the foreground
                    if (backgroundLayer.name.toLowerCase().indexOf('foreground') >= 0) {
                        backgroundLayer.z = Globals.Layers.FOREGROUND;
                    }
                    // any layer that is named "background" will be put to the background
                    if (backgroundLayer.name.toLowerCase().indexOf('background') >= 0) {
                        backgroundLayer.z = Globals.Layers.BACKGROUND;
                    }
                    // paralax
                    if (backgroundLayer.name.toLowerCase().indexOf('paralax') >= 0) {
                        backgroundLayer.z = Globals.Layers.BACKGROUND;
                        backgroundLayer.attach(new Paralax({
                            scrollFactor: new Vector2(0.5, 0)
                        }));
                    }
                    backgroundLayer.attach(new DrawInViewport({
                        edgeSize: 0
                    }));
                });

                // spawn tutorial gesture
                if (settings && settings.tutorial) {
                    Bento.objects.attach(new Gesture({
                        type: settings.tutorial
                    }));
                }
            },
            onLayerMergeCheck: function (layer) {
                if (
                    layer.name.toLowerCase() === 'paralax' ||
                    layer.name.toLowerCase() === 'foreground' ||
                    layer.name.toLowerCase() === 'background'
                ) {
                    return false;
                }
                return true;
            },
            onSpawn: function (instance, object, extraData) {
                if (instance.name === 'character' || instance.name === 'prop') {
                    instance.name += '-' + object.id;
                }

                if (instance.name === 'weapon') {
                    instance.weaponId = object.id;
                }

                var nameProperties = Utils.nameToJson(object.name);

                if (isTower) {
                    tower.onSpawnTower(instance, object, extraData, nameProperties);
                }

                // bosses will get the boss component attach, any enemy could be a boss
                if (nameProperties.boss) {
                    mapHasBoss = true;
                    instance.attach(new Boss({
                        skipIntro: nameProperties.boss > 1
                    }));
                }

                if (nameProperties.isBoss) {
                    mapHasBoss = true;
                }
                // spawn extra enemies for multiplayer
                instance.getComponent('enemyController', function () {
                    // cache everything and wait at a later moment to duplicate enemies
                    extraEnemies.push({
                        instance: instance,
                        object: object,
                        extraData: extraData,
                        nameProperties: nameProperties
                    });
                });

                if (MatchManager.isOnline && MatchManager.otherPlayers.length) {
                    instance.getComponent('punchable', function (punchable) {
                        punchable.increaseHealth(MatchManager.otherPlayers.length * 50);
                    });
                }

                // dojo punching bag needs to be grabable
                if (instance.name === 'punchingBag') {
                    instance.addGrabBehavior();
                    instance.attach(new EventListener({
                        name: 'grabbed',
                        onEvent: function (data) {
                            var player = data.player;
                            var punchingBag = data.entity;
                            // align player with punchingbag
                            new Tween({
                                from: player.position.y,
                                to: punchingBag.position.y + 1,
                                in: 5,
                                ease: 'linear',
                                onUpdate: function (v, t) {
                                    player.position.y = v;
                                },
                                onComplete: function () {

                                }
                            });
                        }
                    }));
                    // if we're in the dojo, remove the consumables while we're at it
                    Bento.objects.get('equipped', function (equipped) {
                        equipped.removeSelf();
                    });
                }

                // automatic attaching is turned off
                Bento.objects.attach(instance);
            },
            onSpawnComplete: function () {
                if (isTower) {
                    tower.onSpawnCompleteTower(mapHasBoss);
                } else if (MatchManager.currentMatch) {
                    // before starting, spawn extra enemies
                    groomEnemies();
                    spawnExtraEnemies(startGameAsync);
                } else {
                    startGameAsync();
                }
            }
        });
        var equipped = new Equipped({});
        var background = new Entity({
            z: -1000,
            name: 'fill',
            float: true,
            components: [
                new FillScreen({})
            ]
        });
        // wall on left edge (only needed on iphoneX)
        var wall = new Entity({
            z: Globals.Layers.GUI - 1,
            name: 'leftWall',
            scale: new Vector2(-1, 1),
            position: new Vector2(0, 0),
            components: [
                new Sprite({
                    imageName: 'gui/upgrade/wall-landscape',
                    originRelative: new Vector2(0, 0.5),
                }), //
                {
                    name: 'floatY',
                    update: function () {
                        wall.position.y = viewport.y + viewport.height / 2;
                    }
                }
            ]
        });
        Bento.objects.attach(background);
        Bento.objects.attach(wall);

        var disconnectListener = new EventListener({
            name: MatchManager.Events.PLAYERDISCONNECTS,
            onEvent: function (data) {
                var playerId = data.playerId;

                // find the player and remove it
                var players = Bento.objects.getByFamily('player');
                Utils.forEach(players, function (player, i, l, breakLoop) {
                    if (player.playerId === playerId) {
                        player.removeSelf();

                        // show notification that player was disconnected
                        Notification.add(Localization.getText('disconnected').replace('{NAME}', data.userName));

                        breakLoop();
                    }
                });

                // was it the last player? if so we should disconnect as well
                if (MatchManager.otherPlayers.length === 0) {
                    MatchManager.disconnect();
                    Notification.add(Localization.getText('multiplayerEnd'));
                } else {
                    // else we try to assign a new master
                    MatchManager.determineMaster();
                }


            }
        });

        if (MatchManager.isOnline) {
            Bento.objects.attach(new EmoteMenu({}));
        }

        Bento.objects.attach(disconnectListener);

        Bento.objects.attach(pauseButton);
        Bento.objects.attach(camera);
        Bento.objects.attach(tileColliders);
        Bento.objects.attach(equipped);

        Bento.objects.attach(new ComboGui({
            position: new Vector2(10, 74)
        }));
        Bento.objects.attach(new PhotoSnapper({}));

        // load music from tiled json
        if (currentSection === 1) {
            if (tiled.mapProperties.music) {
                Globals.currentMusic = tiled.mapProperties.music;
            } else {
                // fallback
                Globals.currentMusic = 'bgm_world1';
            }
        }
        if (!Globals.currentMusic) {
            Globals.currentMusic = 'bgm_world1';
        }
        // exception: dojo
        if (currentMission === 'dojo') {
            Globals.currentMusic = 'bgm_gymtutorial';
        }
        // exception: tower
        if (isTower) {
            Globals.currentMusic = 'bgm_towerchallenge';
        }
        if (!Bento.audio.isPlayingMusic(Globals.currentMusic)) {
            Bento.audio.stopAllMusic();
            // music may be muted by boss intro
            Bento.audio.setVolume(1, Globals.currentMusic);
            if (Utils.isNativeAndroid()) {
                // QUICKFIX: for some reason android sometimes skips the music
                new Tween({
                    ignoreGameSpeed: true,
                    in: 3,
                    onComplete: function () {
                        Bento.audio.playMusic(Globals.currentMusic);
                    }
                });
            } else {
                Bento.audio.playMusic(Globals.currentMusic);
            }
        }

        // enemy activator
        Bento.objects.attach(new EnemyActivator({}));

        // combo manager
        Bento.objects.attach(new ComboManager({}));

        // reset buff manager if buffs are empty
        if (Utils.isDev()) {
            if (!BuffManager.current.isSet) {
                BuffManager.loadBuffs();
            }
            // Bento.objects.attach(new Zoom({
            //     zoom: 0.5
            // }));
        }

        MatchManager.onMessage('general', handleMessage);
        EventSystem.on('playerDeath', onPlayerDeath);
        EventSystem.on('startRoom', onRoomStart);

        // TODO: ask player first?
        // invite was opened: open the worldselect with the invite
        Bento.objects.attach(new EventListener({
            name: 'handleUrl',
            onEvent: function () {
                // cancel any pause
                Bento.objects.resume();
                // close any connection
                MatchManager.disconnect();
                // show worldselect
                Bento.screens.show('screens/worldselect');
            }
        }));

        // respawn player after calling backup
        Bento.objects.attach(new EventListener({
            name: 'respawnPlayer',
            onEvent: function (data) {
                spawnPlayer(data.position, data.onComplete);
            }
        }));

        if (SaveManager.load('tutorialWorldSelect') > 0) {
            Analytics.trackView('main_game');
        } else {
            if (SaveManager.load('missionStars-' + currentMission) < 0) {
                Analytics.trackView('main_game_tutorial_' + currentMission);
            } else {
                Analytics.trackView('main_tutorial_' + currentMission + '_difficulty' + SaveManager.load('chosenDifficulty'));
            }
        }
    };
    /**
     * Screen hide
     */
    var onHide = function (data) {
        var assetGroups = Bento.assets.getAssetGroups();
        // clean up
        Lock.cameraLockers = [];
        EventSystem.off('playerDeath', onPlayerDeath);
        MatchManager.offMessage('general', handleMessage);
        MatchManager.clear();

        // reset possible game speed change
        Bento.setGameSpeed(1);

        // unload assets
        if (data.next !== 'screens/main' && data.next !== 'screens/cutscene' && data.next !== 'screens/gacha') {
            Bento.audio.stopAllMusic();
            Bento.assets.unload('missionJson', true);
            Bento.assets.unload('groupMissionAssets', true);

            // unload emergency if needed
            if (assetGroups['emergency']) {
                Bento.assets.unload('emergency', true);
            }

            if (SaveManager.load('isTower')) {
                Bento.objects.get('towerChallenge', function (tower) {
                    tower.unloadAssets();
                });
            }


            // save to cloud every time user leaves this screen
            SaveManager.saveToCloud();
        }
    };
    /**
     * General online messages
     */
    var onRoomStart = function (e) {
        var viewport = Bento.getViewport();
        //make sure the resolution is send at least once.
        MatchManager.sendData({
            type: 'resolutionChange',
            data: {
                w: viewport.width,
                h: viewport.height
            }
        }, 'general');

        //and use the right ID for our own dimension
        if (MatchManager.localPlayer) {
            ResolutionManager.setPlayerDimension(MatchManager.localPlayer.playerId, viewport.width, viewport.height);
        }


        EventSystem.off('startRoom', onRoomStart);
    };
    var onPlayerDeath = function (e) {
        var player = e.player;
        var gameOverMenu = new GameOverMenu();

        Utils.timeout(60, function () {
            Bento.objects.attach(gameOverMenu);
        });
    };
    var handleMessage = function (m) {
        var onPunchableDeath = function () {
            // finds the entity that should be dead
            var entityName = m.data.n;
            var component = Punchable.deathListeners[entityName];
            if (component) {
                component.forceDeath();
            }
        };
        var handled = true;
        var isTower = SaveManager.load('isTower');

        switch (m.type) {
        case 'areaChange':
            // next section
            var section = SaveManager.load('currentSection');
            if (!isTower) {
                section += 1;
            }
            SaveManager.save('currentSection', section);
            Bento.screens.show('screens/main');
            break;
        case 'spawnEnemy':
            var module = 'entities/enemies/' + m.data.t;
            bento.require([module], function (Enemy) {
                var position = new Vector2(m.data.x, m.data.y);

                var enemy = new Enemy({
                    position: position
                });

                enemy.name = m.data.n;
                // dont start deactivated
                enemy.removeByName('deactivate');

                Bento.objects.attach(enemy);
            });
            break;
        case 'useItem':
            Utils.getPlayerById(m.data.n, function (player) {
                ItemManager.useItem(m.data.k, player, new Vector2(m.data.x, m.data.y));
            });
            break;
        case 'resolutionChange':
            ResolutionManager.setPlayerDimension(m.sender, m.data.w, m.data.h);
            break;
        case 'punchableDeath':
            onPunchableDeath();
            break;
        default:
            handled = false;
            break;
        }

        return handled;
    };


    var screen = new Screen({
        onShow: onShow,
        onHide: onHide
    });

    return screen;
});