/**
 * Loads all mission and spawns them
 */
bento.define('screens/missionviewer', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/screen',
    'bento/tween',
    'components/anchor',
    'gui/backscroller',
    'bento/tiled',
    'globals',
    'gui/worldscroller',
    'components/controllers/enemycontroller',
    'gui/textbutton',
    'entities/zoom',
    'components/drawinviewport',
    'components/fillrect'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Screen,
    Tween,
    Anchor,
    BackScroller,
    Tiled,
    Globals,
    WorldScroller,
    EnemyController,
    TextButton,
    Zoom,
    DrawInViewport,
    FillRect
) {
    'use strict';
    var preloadAssets = function () {
        var assetGroups = Bento.assets.getAssetGroups();
        var city = Utils.assetGroups.city;
        var viewport = Bento.getViewport();
        var background = new BackScroller({
            z: 1,
            name: 'background',
            family: ['background'],
            position: new Vector2(0, 0),
            imageName: 'splash/scrollingbackground-luckyred',
            velocity: new Vector2(0, 0),
            spacing: new Vector2(-1, -1)
        });

        var bar = new Entity({
            z: 1,
            name: 'loaderBar',
            position: new Vector2(-37, 0),
            components: [
                new Sprite({
                    imageName: 'barfill-load',
                    originRelative: new Vector2(0, 0.5)
                })
            ]
        });
        var loaderEntity = new Entity({
            z: 1,
            name: 'loaderOutline',
            components: [
                new Sprite({
                    imageName: 'progressbar-white',
                    originRelative: new Vector2(0.5, 0.5)
                }),
                new Anchor({
                    screenPosition: new Vector2(0.5, 0.5),
                    offset: new Vector2(0, 31),
                    onComplete: function (parent, isPortrait) {

                    }
                }),
                bar
            ]
        });
        Bento.objects.attach(background);
        Bento.objects.attach(loaderEntity);
        bar.scale.x = 0;

        assetGroups['city'] = city;

        Bento.assets.load('city', function () {
            // try again
            console.log('load success');
            Bento.screens.show('screens/missionviewer');
        }, function (assetsLoaded, assetCount, name) {
            bar.scale.x = 0.1 + assetsLoaded / assetCount * 0.9;
        });
    };
    var currentMission = 1;
    var currentSection = 1;
    var showMission = function () {
        var viewport = Bento.getViewport();
        var infoText = new Text({
            z: Globals.Layers.GUI + 1,
            position: new Vector2(viewport.width / 2, viewport.height),
            text: 'Mission: ' + currentMission + '-' + currentSection,
            font: 'font',
            fontSize: 16,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'bottom',
            lineWidth: 1, // set to add an outline
            strokeStyle: '#000000',
            float: true,
            components: []
        });
        var background = new BackScroller({
            z: 1,
            name: 'viewerBackground',
            family: ['background'],
            position: new Vector2(0, 0),
            imageName: 'splash/scrollingbackground-luckyred',
            velocity: new Vector2(0, 0),
            spacing: new Vector2(-1, -1)
        });
        var zoom = new Zoom({});
        var zoomValue = 0.5;
        var makeButtons = function () {
            var buttonZoomIn = new TextButton({
                z: Globals.Layers.GUI + 1,
                name: '',
                text: '+',
                type: 'square',
                position: new Vector2(24, 20),
                float: true,
                onClick: function () {
                    zoomValue += 0.1;
                    zoom.zoom(zoomValue);
                }
            });
            var buttonZoomOut = new TextButton({
                z: Globals.Layers.GUI + 1,
                name: '',
                text: '-',
                type: 'square',
                position: new Vector2(24, 20 + 40),
                float: true,
                onClick: function () {
                    zoomValue -= 0.1;
                    zoom.zoom(zoomValue);
                }
            });
            var buttonForceSpawn = new TextButton({
                z: Globals.Layers.GUI + 1,
                name: '',
                text: 'sp',
                type: 'square',
                position: new Vector2(24, 20 + 80),
                float: true,
                onClick: function () {
                    Utils.forEach(Bento.objects.getByFamily('spawners'), function (spawner, i, l, breakLoop) {
                        spawner.force = true;
                        spawner.forceSpawner();
                    });
                }
            });
            var buttonNext = new TextButton({
                z: Globals.Layers.GUI + 1,
                name: '',
                text: '>',
                type: 'square',
                position: new Vector2(viewport.width - 24, 20),
                float: true,
                onClick: function () {
                    currentSection += 1;
                    // check if asset exists

                    if (Utils.hasAsset('json', 'missions/' + currentMission + '/' + currentSection)) {
                        Bento.screens.show('screens/missionviewer');
                        return;
                    }
                    currentSection = 1;
                    currentMission += 1;
                    // check if asset exists
                    if (Utils.hasAsset('json', 'missions/' + currentMission + '/' + currentSection)) {
                        Bento.screens.show('screens/missionviewer');
                        return;
                    }

                }
            });
            var buttonPrevious = new TextButton({
                z: Globals.Layers.GUI + 1,
                name: '',
                text: '<',
                type: 'square',
                position: new Vector2(viewport.width - 24 - 40, 20),
                float: true,
                onClick: function () {
                    currentSection -= 1;
                    // check if asset exists
                    if (Utils.hasAsset('json', 'missions/' + currentMission + '/' + currentSection)) {
                        Bento.screens.show('screens/missionviewer');
                        return;
                    }
                    currentSection = 5;
                    currentMission -= 1;
                    while (currentSection > 1 && !Utils.hasAsset('json', 'missions/' + currentMission + '/' + currentSection)) {
                        currentSection -= 1;
                    }
                    // check if asset exists
                    if (Utils.hasAsset('json', 'missions/' + currentMission + '/' + currentSection)) {
                        Bento.screens.show('screens/missionviewer');
                        return;
                    }
                    // restore
                    currentSection = 1;
                    currentMission += 1;

                }
            });
            var buttonGoto = new TextButton({
                z: Globals.Layers.GUI + 1,
                name: '',
                text: '...',
                type: 'square',
                position: new Vector2(viewport.width - 24 - 80, 20),
                float: true,
                onClick: function () {
                    var res = window.prompt("Enter mission");
                    if (Utils.hasAsset('json', 'missions/' + res + '/1')) {
                        currentMission = res;
                        currentSection = 1;
                        Bento.screens.show('screens/missionviewer');
                        return;
                    }

                }
            });
            Bento.objects.attach(buttonPrevious);
            Bento.objects.attach(buttonNext);
            Bento.objects.attach(buttonZoomIn);
            Bento.objects.attach(buttonZoomOut);
            Bento.objects.attach(buttonForceSpawn);
            Bento.objects.attach(buttonGoto);
        };
        var screen = Bento.screens.getCurrentScreen();
        var tiled = new Tiled({
            assetName: 'missions/' + currentMission + '/' + currentSection,
            spawnBackground: true,
            spawnEntities: true,
            attachEntities: false,
            maxCanvasSize: new Vector2(512, 512),
            onObject: function (object, tileSet, tileIndex) {
                var spawnExit = function () {

                };
                var spawnExitDojo = function () {

                };
                var spawnLock = function () {

                };
                // spawn triggers (if they have a gid, they are linked to a tileSet)
                if (!object.gid) {
                    if (object.type === 'ExitTrigger') {
                        // place an exit trigger
                        spawnExit();
                    } else if (object.type === 'ExitDojo') {
                        // place an exit trigger specifically for the dojo
                        spawnExitDojo();
                    } else if (object.type === 'Solid') {
                        // place a solid

                    } else if (object.type === 'SolidPlayer') {
                        // place a solid

                    } else if (object.type === 'CameraLock') {
                        // spawn a cameralock but don't add it to the game, instead add it to the Lock trigger

                    } else if (object.type === 'LockTrigger') {
                        // place lock trigger
                        spawnLock();
                    } else if (object.type === 'Blockade') {
                        // place blocakde
                        Bento.objects.attach(new Entity({
                            z: Globals.Layers.FOREGROUND + 1,
                            components: [{
                                name: 'drawLine',
                                draw: function (data) {
                                    var ctx = data.renderer.getContext('2d');
                                    ctx.beginPath();
                                    ctx.moveTo(object.x + 1, object.y);
                                    ctx.lineTo(object.x + 1, object.y + object.height);
                                    ctx.lineWidth = 2;
                                    ctx.strokeStyle = '#00E0E0';
                                    ctx.stroke();
                                }
                            }]
                        }));
                    } else if (object.type === 'Spawn') {
                        Utils.log('WARNING: Spawn is deprecated');
                        // spawnSpawner();
                    } else if (object.type === 'Finish') {
                        // place a solid

                    } else if (object.type === 'EnemySpawnHeight') {

                    }
                }
            },
            onTile: function (tileX, tileY, tilesetJson, tileIndex) {

            },
            onComplete: function () {
                // synchronous completion
                screen.dimension = this.dimension.clone();
                Utils.forEach(this.backgrounds, function (backgroundLayer, i, l, breakLoop) {
                    // any layer that is named "foreground" will be put to the foreground
                    if (backgroundLayer.name.toLowerCase().indexOf('foreground') >= 0) {
                        backgroundLayer.z = Globals.Layers.FOREGROUND;
                    }
                    // any layer that is named "background" will be put to the background
                    if (backgroundLayer.name.toLowerCase().indexOf('background') >= 0) {
                        backgroundLayer.z = Globals.Layers.BACKGROUND;
                    }
                });
            },
            onLayerMergeCheck: function (layer) {
                if (
                    layer.name.toLowerCase() === 'foreground' ||
                    layer.name.toLowerCase() === 'background'
                ) {
                    return false;
                }
                return true;
            },
            onSpawn: function (instance, object, extraData) {
                var txt = '';
                var propertiesTxt = '';
                var text = new Text({
                    z: Globals.Layers.FOREGROUND + 1,
                    position: new Vector2(0, 0),
                    text: '',
                    font: 'helvetipixel',
                    fontSize: 16,
                    fontColor: '#ffffff',
                    align: 'center',
                    textBaseline: 'top',
                    ySpacing: 0,
                    lineWidth: 1, // set to add an outline
                    strokeStyle: '#000000',
                    components: []
                });
                var propertiesText = new Text({
                    z: Globals.Layers.FOREGROUND + 1,
                    position: new Vector2(0, 0),
                    text: '',
                    font: 'helvetipixel',
                    fontSize: 16,
                    fontColor: '#0f0',
                    align: 'center',
                    textBaseline: 'bottom',
                    ySpacing: 0,
                    lineWidth: 1, // set to add an outline
                    strokeStyle: '#000000',
                    components: []
                });
                if (instance.name === 'character' || instance.name === 'prop') {
                    instance.name += '-' + object.id;
                }

                if (instance.name === 'weapon') {
                    instance.weaponId = object.id;
                }

                if (instance.name && instance.name.indexOf('spawner') !== -1) {
                    instance.force = true;
                }

                var nameProperties = Utils.nameToJson(object.name);

                Utils.forEach(nameProperties, function (value, key, l, breakLoop) {
                    txt += key + ': ' + value + '\n';
                });

                // automatic attaching is turned off
                Bento.objects.attach(instance);


                text.setText(txt);
                text.position = instance.position;
                Bento.objects.attach(text);


                // draw all properties (???)
                if (object.properties) {
                    // Utils.forEach(object.properties, function (value, key, l, breakLoop) {
                    //     propertiesTxt += key + ': ';
                    //     // can it be converted to nameproperties?
                    //     var stringProperties = Utils.nameToJson(value);
                    //     if (Utils.getKeyLength(stringProperties) > 0) {
                    //         Utils.forEach(stringProperties, function (val, k) {
                    //             // make shorter for paths
                    //             val = val.toString().replace('entities/enemies/', '');
                    //             propertiesTxt += '_' + k + ': ' + val;
                    //         });
                    //         propertiesTxt += '\n';
                    //     } else {
                    //         propertiesTxt += value + '\n';
                    //     }
                    // });

                    // propertiesText.setText(propertiesTxt);
                    // propertiesText.position = instance.position.add(new Vector2(0, -16));
                    // Bento.objects.attach(propertiesText);
                }

                // draw name if it's not a character
                if (instance.name.indexOf('character') === -1) {
                    propertiesTxt = instance.name;
                    propertiesText.setText(propertiesTxt);
                    propertiesText.position = instance.position.add(new Vector2(0, -32));
                    Bento.objects.attach(propertiesText);
                }
            },
            onSpawnComplete: function () {
                var worldScroller = new WorldScroller({
                    scrollX: true,
                    scrollY: true,
                    width: tiled.dimension.width,
                    height: tiled.dimension.height,
                });
                Bento.objects.attach({
                    update: function () {
                        var scrollPosition = worldScroller.getScrollPosition();
                        viewport.x = scrollPosition.x;
                        viewport.y = scrollPosition.y;
                    }
                });
                Bento.objects.attach(worldScroller);
                worldScroller.centerTo(tiled.dimension.getExtents());

                Utils.timeout(30, function () {
                    EventSystem.fire('activateEnemies', {});
                });
            }
        });


        Bento.objects.attach(background);
        Bento.objects.attach(infoText);
        Bento.objects.attach(zoom);
        zoom.zoom(zoomValue);
        makeButtons();
        background.z = -100;
        background.float = true;
        DrawInViewport.disable = true;
    };
    var onShow = function (settings) {
        // restore city asset group if needed first
        var assetGroups = Bento.assets.getAssetGroups();

        if (!assetGroups['city']) {
            preloadAssets();
        } else {
            showMission();
        }
    };

    return new Screen({
        onShow: onShow
    });
});