/**
 * Screen description
 */
bento.define('screens/preloader', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/components/fill',
    'modules/text',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/screen',
    'bento/tween',
    'managers/coincollector',
    'managers/inventory',
    'managers/combo',
    'modules/localization',
    'managers/time',
    'managers/save',
    'managers/character',
    'managers/currency',
    'ads/heyzap',
    'modules/analytics',
    'globals',
    'components/fillscreen',
    'components/anchor',
    'gui/iapoverlay',
    'modules/ktplay',
    'gui/dialog',
    'managers/receiveitem',
    'entities/crosspromo',
    'modules/abtest',
    'modules/abtestgroup'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Fill,
    Text,
    Entity,
    EventSystem,
    Utils,
    Screen,
    Tween,
    CoinCollector,
    InventoryManager,
    ComboManager,
    Localization,
    TimeManager,
    SaveManager,
    CharacterManager,
    CurrencyManager,
    HeyZap,
    Analytics,
    Globals,
    FillScreen,
    Anchor,
    IapOverlay,
    KTPlayModule,
    Dialog,
    ReceiveItem,
    Crosspromo,
    ABTest,
    ABTestGroup
) {
    'use strict';

    /*
     * Initialize game specific managers
     */
    var gameInit = function () {
        Localization.init();

        Bento.audio.muteSound(Bento.saveState.load('muteSound', false));
        Bento.audio.muteMusic(Bento.saveState.load('muteMusic', false));

        SaveManager.add('bootupsForOffer', 1);

        //Initialize time manager
        TimeManager.initialize();

        // retrieves the available langauge codes for title image
        Globals.getTitleLanguage = function () {
            var code = Localization.getLanguage();

            // only titles in these languages
            if (code === 'zh' ||
                code === 'zt' ||
                code === 'ja' ||
                code === 'ko' ||
                code === 'ru') {
                return code;
            } else {
                return 'en';
            }
        };

        if (Bento.isDev() || Bento.saveState.load('showDeveloperCheats', false)) {
            // cheat: kill all enemies on screen
            window.killEveryone = function (onlyActive) {
                // skip first tutorial
                if (SaveManager.load('tutorial') === 0) {
                    Bento.screens.show('screens/worldselect');
                    SaveManager.save('tutorial', 3);
                    return;
                }

                Bento.objects.getByFamily('enemy', function (enemies) {
                    Utils.forEach(enemies, function (enemy, i, l, breakLoop) {
                        enemy.getComponent('punchable', function (punchable) {
                            if (onlyActive) {
                                enemy.getComponent('enemyController', function (enemyController) {
                                    if (enemyController.active) {
                                        punchable.setHealth(0);
                                        punchable.kill();
                                    }
                                });
                            } else {
                                punchable.kill();
                            }
                        });
                    });
                });
                // remove camera locks
                if (!onlyActive) {
                    Bento.objects.getByFamily('cameraLocker', function (lockers) {
                        if (lockers.length > 0) {
                            Bento.objects.remove(lockers[0]);
                        }
                    });
                    // if possible, transport player to the exit trigger
                    Bento.objects.getByFamily('player', function (players) {
                        Utils.forEach(players, function (player, i, l, breakLoop) {
                            if (!player.isRemote) {
                                Bento.objects.get('exit', function (exit) {
                                    player.position.x = exit.position.x + exit.dimension.width / 2;
                                    player.position.y = exit.position.y + exit.dimension.height / 2;
                                });
                            }
                        });
                    });
                }
            };
            // press w
            EventSystem.on('buttonDown-w', window.killEveryone);
            EventSystem.on('buttonDown-e', function () {
                window.killEveryone(true);
            });
        }

        // Set the reward handler function to a function that adds items for each reward
        KTPlayModule.setRewardHandler(function (data) {
            /*
                EXAMPLE 'data.items': 
                [{
                    "typeId":"testToken",
                    "id":"testToken",
                    "name":"Test Token",
                    "value":100,
                    "icon":"https://dwimg.ktplay.com/group1/M00/2C/67/wKgDOFnR6i6AR5j1AAAd0A8xjlM1910618"
                }]
            */

            // get the rewards payload
            var payloadString = data.rewards;
            payloadString = payloadString.replace("\\", "");

            // convert payload to reward JSON
            var rewards = JSON.parse(payloadString);

            // loop through rewards and apply them to the player
            Utils.forEach(rewards.items, function (reward, i, l, breakLoop) {
                // code here
                switch (reward.id) {
                case 'rewardCoins':
                    console.log('REWARD RECEIVED: coins *', reward.value);
                    ReceiveItem({
                        key: 'currency/coin',
                        amount: parseInt(reward.value),
                        notSource: true
                    });
                    break;
                case 'rewardGems':
                    console.log('REWARD RECEIVED: gems *', reward.value);
                    ReceiveItem({
                        key: 'currency/gem',
                        amount: parseInt(reward.value),
                        notSource: true
                    });
                    break;
                case 'rewardEnergy':
                    console.log('REWARD RECEIVED: Energy *', reward.value);
                    CharacterManager.addEnergy(parseInt(reward.value));
                    break;
                default:
                    // receive any item
                    var rewardsSplit = reward.id.split('/');
                    if (rewardsSplit.length === 2) {
                        // must be in format "category/item"
                        ReceiveItem({
                            key: reward.id,
                            amount: parseInt(reward.value),
                            notSource: true
                        });
                    }
                    break;
                }
            });
        });

        // coin check
        var coins = SaveManager.load('coins');
        var gems = SaveManager.load('gems');
        if (Utils.isString(coins)) {
            coins = parseInt(coins) || 0;
            SaveManager.save('coins', coins);
        }
        if (Utils.isString(gems)) {
            gems = parseInt(gems) || 0;
            SaveManager.save('coins', gems);
        }

        // undefine gameInit
        gameInit = function () {};

        EventSystem.fire('gameInit');

        // send fingerprint
        Analytics.event('FingerPrint', 'fingerPrint', 0, Bento.saveState.load('fingerPrint', 'none'));
    };
    var catchExit = function () {
        EventSystem.on('exitApp', function () {
            var dialog = new Dialog({
                name: 'exitAppStandard',
                text: Localization.getText('exit'),
                type: 'yesno',
                ignoreAnalytics: true,
                onYes: function () {
                    if (navigator.app) {
                        navigator.app.exitApp();
                    }
                }
            });
            Bento.objects.attach(dialog);
        });
    };
    var catchFatalError = function () {
        // a fatal error occurred --> we show the player an error and restart
        var hasCrashed = false;
        EventSystem.on('UncaughtError', function (data) {
            if (hasCrashed) {
                // looping error
                return;
            }
            hasCrashed = true;
            var category = 'Error';
            var currentScreen = Bento.screens.getCurrentScreen();
            var currentScreenName = 'none';
            var currentObject = data.currentObject;
            var currentObjectName = 'none';
            var currentComponent;
            var currentComponentName = 'none';
            var summary = data.summary;
            var errorMsg = data.errorMsg;
            var isFatal = data.isFatal;
            var errorHistory = Bento.saveState.load('errorHistory', []);
            var prepareToRestart = function () {
                var dialog = new Dialog({
                    z: Globals.Layers.SYSTEM,
                    name: 'crash',
                    ignoreAnalytics: true,
                    title: 'Game crashed',
                    text: "Unfortunately, the game has stopped responding. An error report was sent. The game will now attempt to recover.",
                    type: 'yes',
                    onComplete: function () {
                        // try to restart main game
                        var next = 'screens/title';
                        if (currentScreenName === 'screens/main') {
                            next = 'screens/main';
                        }
                        Bento.screens.show(next, undefined, function () {
                            hasCrashed = false;
                        });
                    }
                });
                Bento.objects.attach(dialog);
            };
            if (currentScreen) {
                // try to log the current screen
                currentScreenName = currentScreen.name;
            }
            if (currentObject && currentObject.name) {
                // try to log the object name
                currentObjectName = currentObject.name;
                summary += '\nObject: ' + currentObject.name;
                currentComponent = currentObject.currentComponent;
                if (currentComponent && currentComponent.name) {
                    currentComponentName = currentComponent.name;
                    summary += '\nComponent: ' + currentComponentName;

                    // special case: stateMachine
                    if (currentComponentName === 'stateMachine') {
                        summary += '\nstateName: ' + currentComponent.stateName;
                    }
                }
            }
            if (errorMsg.stack) {
                summary += '\nStack: ' + errorMsg.stack;
            }
            if (isFatal) {
                category = 'ErrorFatal';
            }
            // send analytics
            Analytics.event(category, currentScreenName, 1, summary);

            // save into history
            errorHistory.push({
                isFatal: isFatal,
                summary: summary,
                screen: currentScreenName,
                objectName: currentObjectName,
                componentName: currentComponentName,
                date: (new Date()).toString()
            });
            Bento.saveState.save('errorHistory', errorHistory);

            // try to run the game again
            if (isFatal) {
                Bento.objects.removeAll();
                // try to restart screen
                prepareToRestart();
                // try to revive the main loop
                Bento.objects.run(true);
            }
        });
    };

    /*
     * Preloader start
     */
    var onShow = function () {
        /* Screen starts here */
        var viewport = Bento.getViewport();
        var loaded = false;
        var backgroundSprite = new Sprite({});
        var background = new Entity({
            z: 0,
            name: 'background',
            components: [
                // new FillScreen({
                //     color: [1, 1, 1, 1]
                // })
                backgroundSprite,
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.5, 0.5)
                    },
                    landscape: {
                        screenPosition: new Vector2(0.5, 1.1)
                    },
                    onComplete: function () {
                        if (Anchor.isPortrait()) {
                            backgroundSprite.setup({
                                imageName: 'preloader-splash',
                                originRelative: new Vector2(0.5, 0.5)
                            });
                        } else {
                            backgroundSprite.setup({
                                imageName: 'preloader-splash',
                                originRelative: new Vector2(0.5, 0.95)
                            });
                        }
                    }
                }), //
                {
                    start: function () {
                        var extraFill = Anchor.isWideScreen() ? 30 : 0;
                        // fill to height or width
                        if (Anchor.isPortrait()) {
                            background.scale.y = (viewport.height + extraFill) / background.dimension.height;
                            background.scale.x = background.scale.y;
                        } else {
                            background.scale.x = (viewport.width + extraFill) / background.dimension.width;
                            background.scale.y = background.scale.x;
                        }
                    }
                }
            ]
        });

        var bar = new Entity({
            z: 1,
            name: 'loaderBar',
            position: new Vector2(-37, 0),
            components: [
                new Sprite({
                    imageName: 'barfill',
                    originRelative: new Vector2(0, 0.5)
                })
            ]
        });

        var text = new Text({
            z: 1,
            name: 'loadingText',
            font: 'font',
            fontSize: 12,
            fontColor: '#fffd00',
            text: 'Loading...',
            align: 'center',
            textBaseline: 'bottom',
            position: new Vector2(0, -7)
        });

        var player = new Entity({
            z: 1,
            name: 'player',
            position: new Vector2(0, -30),
            components: [
                new Sprite({
                    imageName: 'loading-run',
                    frameCountX: 3,
                    frameCountY: 3,
                    padding: 1,
                    originRelative: new Vector2(0.5, 1),
                    animations: {
                        default: {
                            speed: 0.2,
                            frames: [
                                0,
                                1,
                                2,
                                3,
                                4,
                                5,
                                6,
                                7
                            ],
                            frameSpeeds: [
                                1,
                                1,
                                1,
                                1,
                                1,
                                1,
                                1,
                                1
                            ]
                        }
                    }
                }),
            ]
        });

        var loaderEntity = new Entity({
            z: 1,
            name: 'loaderOutline',
            components: [
                new Sprite({
                    imageName: 'progressbar',
                    originRelative: new Vector2(0.5, 0.5)
                }),
                new Anchor({
                    portrait: {
                        screenPosition: new Vector2(0.5, 1),
                        offset: new Vector2(0, -8)

                    },
                    landscape: {
                        screenPosition: new Vector2(0.85, 1),
                        offset: new Vector2(0, -8)
                    }
                }),
                bar,
                text
            ]
        });

        var end = function () {
            if (loaded) {
                catchExit();
                catchFatalError();
                EventSystem.fire('assetsLoaded');

                var isDownloading = ABTest.isDownloading(function () {
                    // done downloading: move on to splash
                    Bento.screens.show('screens/splash');
                });
                if (!isDownloading) {
                    Bento.screens.show('screens/splash');
                }
            }
        };
        var loadFonts = function () {
            var fonts = Bento.assets.getAssetGroups().preloader.fonts;
            var font;
            if (!fonts) {
                return;
            }
            for (font in fonts) {
                if (!fonts.hasOwnProperty(font)) {
                    continue;
                }
                new Text({
                    position: new Vector2(1000, 1000),
                    text: '.',
                    font: font
                });
            }
        };

        /*
         * Smart asset loading: don't load missions and enemies until needed
         */
        var manipulateAssetGroups = function () {
            var assetGroups = Bento.assets.getAssetGroups();
            var city = assetGroups.city;
            if (!city) {
                // already removed
                return;
            }
            var missions = {
                json: {
                    "missions/missions": "missions/missions.json",
                    // TODO: only load tutorials if needed
                    "missions/tutorial/1": "missions/tutorial/1.json",
                    "missions/tutorial/2": "missions/tutorial/2.json",
                    "missions/tutorial/3": "missions/tutorial/3.json"
                },
                path: city.path
            };
            // we do need the missions.json and tutorial
            assetGroups.missions = missions;

            delete assetGroups.city;

            // TODO: can we skip the images in general?

            Utils.assetGroups = {
                city: city
            };

            // during christmas, we swap out rat and ratgreen for the xmas variant
            var now = Date.now(); // precision is not important
            var today = Math.floor(now / 1000 / 60 / 60 / 24);
            if (today <= 17526) {
                Utils.forEach(city.spritesheets, function (value, key, l, breakLoop) {
                    if (key.indexOf('-xmas/') >= 0) {
                        key = key.replace('-xmas', '');
                        city.spritesheets[key] = value;
                    }
                });
            }
            // clean up the xmas assets to be sure
            Utils.forEach(city.spritesheets, function (value, key, l, breakLoop) {
                if (key.indexOf('-xmas/') >= 0) {
                    delete city.spritesheets[key];
                }
            });
        };

        manipulateAssetGroups();

        Bento.objects.attach(background);
        Bento.objects.attach(loaderEntity);
        bar.scale.x = 0;

        // init game specific modules
        gameInit();

        // preload fonts
        loadFonts();

        // create test group
        ABTestGroup();

        // hide cordova splashscreen if exists
        if (navigator.splashscreen) {
            navigator.splashscreen.hide();
        }
        if (window.UserSplash) {
            // delay hiding with 200ms to make sure there are no black gaps
            window.setTimeout(function () {
                window.UserSplash.hideSplash();
            }, 200);
        }


        var didFindError = 0;
        Bento.assets.loadAllAssets({
            exceptions: ['preloader', 'jpg-preloader'], // preloader was already loaded
            onLoaded: function (current, total, name, type) {
                // show how many assets still to be loaded
                // text.setText('Loading ' + current + '/' + total);
                bar.scale.x = current / total;
                // console.log('Loaded ' + current + '/' + total + ': ' + name + ' ' + type);

                if (current > total) {
                    Utils.log('WARNING: too many assets loaded!!');
                    didFindError = 1;

                    Utils.timeout(300, function () {
                        // retry loading
                        didFindError = 2;
                        Bento.screens.show('screens/preloader');
                    });
                }
            },
            onReady: function (error) {
                // some final checks
                if (error || didFindError === 1 || !Utils.hasAsset('json', 'roadblock') || !Utils.hasAsset('json', 'dojo')) {
                    Utils.log('WARNING: loading has FAILED');
                    // retry loading
                    if (didFindError !== 2) { // already retrying
                        Bento.screens.show('screens/preloader');
                    }
                    return;
                } else {
                    console.log('All assets loaded');
                }
                loaded = true;
                end();
            }
        });
        Analytics.trackView('preloader');
    };

    return new Screen({
        onShow: onShow,
        onHide: function () {
            // for debugging
            if (Utils.isDev()) {
                window.setLanguage = Localization.setLanguage;
            } else {
                // unload preloader assets
                Bento.assets.unload('jpg-preloader', true);
            }
        }
    });
});