/**
 * Preloader for missions
 * Assumption: all dynamically loaded assets reside in assets/city/!
 * Requirements:
 * In Tiled: if a tileSet property is named spriteSheet, that spriteSheet will be loaded (used for breakables and destructibles)
 * In Bento: A module needs to have a .rootPath property to load rootPath sprites (a rootpath is a folder that contains multiple sprites)
 */
bento.define('screens/preloadmission', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/screen',
    'bento/tween',
    'managers/save',
    'globals',
    'bento/tiledreader',
    'components/fillscreen',
    'modules/localization',
    'components/anchor',
    'gui/shadowtext',
    'managers/inventory',
    'managers/item',
    'gui/fade',
    'gui/backscroller',
    'modules/fontsettings'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Screen,
    Tween,
    SaveManager,
    Globals,
    TiledReader,
    FillScreen,
    Localization,
    Anchor,
    ShadowText,
    InventoryManager,
    ItemManager,
    Fade,
    BackScroller,
    Fontsettings
) {
    'use strict';
    var onShow = function (settings) {
        var viewport = Bento.getViewport();
        var nextScreen = (settings || {}).nextScreen || 'screens/main';
        /*var background = new Entity({
            z: 0,
            name: 'background',
            components: [
                new FillScreen({
                    color: [1, 1, 1, 1]
                })
            ]
        });*/
        var background = new BackScroller({
            z: 1,
            name: 'background',
            family: ['background'],
            position: new Vector2(0, 0),
            imageName: 'splash/scrollingbackground-luckyred',
            velocity: new Vector2(0, 0),
            spacing: new Vector2(-1, -1)
        });

        var bar = new Entity({
            z: 1,
            name: 'loaderBar',
            position: new Vector2(-37, 0),
            components: [
                new Sprite({
                    imageName: 'barfill-load',
                    originRelative: new Vector2(0, 0.5)
                })
            ]
        });

        var text = new ShadowText({
            z: 1,
            name: 'loadingText',
            font: 'font',
            fontSize: 12,
            fontColor: '#ffffff',
            text: Localization.getText('loading'),
            align: 'center',
            textBaseline: 'bottom',
            position: new Vector2(0, -7)
        });

        var player = new Entity({
            z: 1,
            name: 'player',
            position: new Vector2(0, -30),
            components: [
                new Sprite({
                    spriteSheet: SaveManager.load('selectedCharacter') + '/run'
                }),
            ]
        });
        var itemIcons = [];
        var makeItemInfo = function () {
            var items = InventoryManager.getEquippedConsumables();
            var addIcon = function (item) {
                var slotSprite = new Sprite({
                    imageName: 'gui/itemslot-small',
                    frameCountX: 2,
                    frameCountY: 1,
                    originRelative: new Vector2(0.5, 0.5)
                });
                var sprite = ItemManager.getSprite(item);
                sprite.setOriginRelative(new Vector2(0.5, 0.5));
                sprite.origin.x += 1;
                var description = new ShadowText({
                    position: new Vector2(18, -2),
                    text: ItemManager.getDescription(item),
                    font: 'font',
                    fontSize: 10,
                    fontColor: '#ffffff',
                    align: 'left',
                    textBaseline: 'middle',
                    maxWidth: 120,
                    maxHeight: 28,
                    // drawDebug: true
                });
                var index = itemIcons.length;
                var icon = new Entity({
                    z: 3,
                    name: 'icon',
                    position: new Vector2(20, 20 + 32 * index),
                    components: [
                        slotSprite,
                        sprite,
                        description
                    ]
                });

                if (index > 0) {
                    icon.attach(new Anchor({
                        onComplete: function (parent, isPortrait) {
                            if (isPortrait) {
                                icon.position = new Vector2(20, 20 + 32 * index);
                            } else {
                                icon.position = new Vector2(
                                    20 + (index % 2) * 158,
                                    20 + 32 * Math.floor(index / 2)
                                );
                            }
                        }
                    }));
                }

                itemIcons.push(icon);
                Bento.objects.attach(icon);
            };

            if (items[0] && items[0].amount > 0) {
                addIcon(items[0].key);
            }
            if (items[1] && items[1].amount > 0) {
                addIcon(items[1].key);
            }
        };
        // Deprecated for now
        // makeItemInfo();

        var makeTip = function () {
            var tips = Localization.getText('tips');
            var pickRandomTip = function () {
                var random = Utils.getRandom(tips.length);
                // check if it's a tip that requires a scroll --> reroll if user doesnt have it
                if (random === 0 && !InventoryManager.getItem('scrolls/dash')) {
                    return pickRandomTip();
                } else if (random === 1 && !InventoryManager.getItem('scrolls/charge')) {
                    return pickRandomTip();
                }

                return random;
            };
            var tip = new ShadowText({
                z: 3,
                text: Localization.getText('tip') + tips[pickRandomTip()],
                font: 'font',
                fontSize: 12,
                fontColor: '#ffffff',
                align: 'center',
                textBaseline: 'top',
                maxWidth: viewport.width - 16
            }).attach(new Anchor({
                screenPosition: new Vector2(0.5, 0),
                offset: new Vector2(0, 8),
                onComplete: function (parent, isPortrait) {
                    // reset maxWidth
                    tip.setText(tip.getText(), {
                        maxWidth: viewport.width - 16
                    });
                }
            }));
            Bento.objects.attach(tip);
        };
        makeTip();

        var loaderEntity = new Entity({
                z: 1,
                name: 'loaderOutline',
                components: [
                    new Sprite({
                        imageName: 'progressbar-white',
                        originRelative: new Vector2(0.5, 0.5)
                    }),
                    new Anchor({
                        screenPosition: new Vector2(0.5, 0.5),
                        offset: new Vector2(0, 31),
                        onComplete: function (parent, isPortrait) {
                            if (isPortrait) {
                                player.position.y = -32;
                            } else {
                                player.position.y = -32 + 8;
                                loaderEntity.position.y += 12;
                            }
                        }
                    })
                ]
            }).attach(bar)
            .attach(player)
            .attach(text);

        var assetGroups = Bento.assets.getAssetGroups();
        var city = Utils.assetGroups.city;

        // these 2 "asset groups" will be loaded, missionJson and missionAssets
        // groupMissionsJson is used to load the Tiled jsons
        var groupMissionsJson = {
            json: {},
            path: city.path
        };
        // groupMissionAssets is used to load assets used by each Tiled json
        var groupMissionAssets = {
            audio: {},
            images: {},
            spritesheets: {},
            path: city.path
        };

        // step 1.1: find the tiled jsons, construct an assetgroup
        var mission = SaveManager.load('currentMission');
        var findTiled = function () {
            var jsonsToLoad = {};
            // copy path
            groupMissionsJson.path = city.path;
            // find the jsons
            Utils.forEach(city.json, function (jsonPath, jsonName, l, breakLoop) {
                if (jsonName.indexOf('missions/' + mission + '/') === 0) {
                    groupMissionsJson.json[jsonName] = jsonPath;
                }
            });
        };
        findTiled();

        // step 1.2: load the tiled jsons
        // (see end, allow functions to be declared first)

        // step 2: inspect the tiled jsons and find which tiled and modules are used
        var tileSets = {}; // individual images to load
        var imageNames = {}; // individual images to load
        var spriteSheets = {}; // individual spritesheets to load
        var audio = {}; // individual audio to load
        var rootPaths = {}; // spritesheet folder to load
        var inspectTiled = function () {
            var tiledToLoad = 0;
            var tiledLoaded = 0;
            Utils.forEach(groupMissionsJson.json, function (p, jsonName, l, breakLoop) {
                var tiledJson = Bento.assets.getJson(jsonName);
                var assetName;
                var objectsToLoad = 0;
                var objectsLoaded = 0;
                var music;

                // check for music
                if (tiledJson.properties && tiledJson.properties.music) {
                    music = tiledJson.properties.music;
                    audio[music] = city.audio[music];
                    audio[music + '-boss'] = city.audio[music + '-boss'];
                    audio[music.replace('bgm', 'sfx') + '-appear'] = city.audio[music.replace('bgm', 'sfx') + '-appear'];
                    groupMissionAssets.audio = audio;
                }

                tiledToLoad += 1;

                // parse tiled
                var tiledReader = new TiledReader({
                    tiled: tiledJson,
                    onExternalTileset: function (source) {
                        // unfortunately, external tileset paths are relative to the tile json path
                        // making it difficult to load (would need to do path parsing etc...)
                        // instead we try to make an educated guess what the asset name is
                        var json;
                        var jsonPath = source.indexOf('json/');
                        var relativePath = source.indexOf('../');
                        var path = source;
                        var split;
                        if (jsonPath >= 0) {
                            // if the name "json/" is there, we can guess the path is after the json/ part
                            path = source.substring(jsonPath + ('json/').length);
                        } else if (relativePath >= 0) {
                            // no json/ is there and the path has relative indicators
                            path = source;

                            if (assetName) {
                                // path parsing, urgh
                                split = assetName.split('/');
                                split.pop(); // remove filename
                                while (path.indexOf('../') >= 0) {
                                    if (split.length === 0) {
                                        throw "ERROR: Impossible path to external tileset";
                                    }
                                    // move up one folder
                                    split.pop();
                                    path = path.replace('../', '');
                                }
                                // final path, may need an extra slash
                                path = split.join('/') + (split.length ? '/' : '') + path;
                            } else {
                                // more dangerous method: try removing all relative indicators
                                while (path.indexOf('../') >= 0) {
                                    path = path.replace('../', '');
                                }
                            }
                        }
                        path = path.replace('.json', '');

                        json = Bento.assets.getJson(path);

                        return json;
                    },
                    onTile: function (tileX, tileY, tileSet, tileIndex, flipX, flipY, flipD) {
                        // collect tilesets
                        tileSets[tileSet.image] = tileSet;
                    },
                    onObject: function (object, tileSet, tileIndex) {
                        // find out what modules are being used!
                        var tileproperties;
                        var moduleName;
                        var imageName;
                        var spriteSheet;
                        var rootPathProperty;
                        var checkProperties = function (properties) {
                            var i;
                            var prop;
                            if (!properties) {
                                return;
                            }
                            moduleName = properties.module;
                            spriteSheet = properties.spriteSheet;
                            imageName = properties.imageName;
                            rootPathProperty = properties.rootPath;

                            if (rootPathProperty) {
                                rootPaths[rootPathProperty] = rootPathProperty;
                            }
                            // quick fix: up to 10 rootPaths
                            for (i = 2; i < 10; i++) {
                                prop = properties['rootPath' + i];
                                if (prop) {
                                    rootPaths[prop] = prop;
                                }
                            }

                            if (spriteSheet) {
                                spriteSheets[spriteSheet] = spriteSheet;
                            }
                            if (imageName) {
                                imageNames[imageName] = imageName;
                            }
                        };
                        checkProperties(object.properties);

                        if (!tileSet) {
                            return;
                        }
                        tileproperties = tileSet.tileproperties;
                        if (!tileproperties) {
                            return;
                        }

                        checkProperties(tileproperties[tileIndex]);

                    },
                    onComplete: function () {
                        // no objects were loaded at all
                        if (objectsToLoad === 0) {
                            onCollectedAssets();
                        }
                    }
                });
                var onCollectedAssets = function () {
                    // check which tileSets were parsed
                    var images = {};
                    Utils.forEach(tileSets, function (tileSet) {
                        // retrieve the corresponding image asset
                        // there is a very high chance the url contains "images/" since the json files
                        // should be stored in the "json/" folder and images in "images/"
                        var imageUrl = tileSet.image;
                        var imagePath;
                        var imageName;
                        imagePath = imageUrl.substring(imageUrl.indexOf('images/') + ('images/').length);
                        imageName = imagePath.replace('.png', '');

                        // add to be loaded
                        images[imageName] = imagePath;
                    });
                    Utils.forEach(images, function (imagePath, imageName) {
                        if (city.images[imageName]) {
                            groupMissionAssets.images[imageName] = imagePath;
                        } else {
                            Utils.log('ERROR: asset ' + imageName + ' does not exist! In ' + jsonName);
                        }
                    });

                    // check which imagenames were parsed
                    Utils.forEach(imageNames, function (imageName) {
                        if (city.images[imageName]) {
                            groupMissionAssets.images[imageName] = imageName + '.png';
                        } else {
                            Utils.log('ERROR: asset ' + imageName + ' does not exist! In ' + jsonName);
                        }
                    });
                    // check which spritesheets were parsed
                    Utils.forEach(spriteSheets, function (spriteSheet) {
                        if (city.spritesheets[spriteSheet]) {
                            groupMissionAssets.spritesheets[spriteSheet] = spriteSheet;
                        } else {
                            Utils.log('ERROR: asset ' + spriteSheet + ' does not exist! In ' + jsonName);
                        }
                    });

                    // check which rootpaths were parsed
                    Utils.forEach(rootPaths, function (rootPath) {
                        var found = false;
                        // which spritesheets are in the rootPath?
                        // we can tell by inspecting the deleted city assetgroup
                        Utils.forEach(city.spritesheets, function (value, citySpriteSheet) {
                            if (citySpriteSheet.indexOf(rootPath) === 0) {
                                if (city.spritesheets[citySpriteSheet]) {
                                    groupMissionAssets.spritesheets[citySpriteSheet] = value;
                                    found = true;
                                } else {
                                    Utils.log('ERROR: asset ' + citySpriteSheet + ' does not exist! In ' + jsonName);
                                }
                            }
                        });
                        if (!found) {
                            Utils.log('WARNING: could not find assets for rootPath ' + rootPath);
                        }
                    });

                    tiledLoaded += 1;

                    // ready?
                    if (tiledLoaded === tiledToLoad && tiledLoaded === l) {
                        loadGroupMissionAssets();
                    }
                };

                tiledReader.read();
            });

        };
        // step 3: load the spritesheets and tiles
        var loadGroupMissionAssets = function () {
            assetGroups['groupMissionAssets'] = groupMissionAssets;
            Bento.assets.load('groupMissionAssets', function () {
                EventSystem.fire('preloadMission', {
                    mission: mission,
                    groupMissionsJson: groupMissionsJson,
                    groupMissionAssets: groupMissionAssets
                });
                // all assets loaded for the mission
                goToMain();
            }, function (assetsLoaded, assetCount, name) {
                bar.scale.x = 0.1 + assetsLoaded / assetCount * 0.9;
            });
            // console.log('groupMissionAssets', groupMissionAssets);
        };

        // step 4: go to designated screen
        var goToMain = function () {
            Utils.timeout(30, function () {
                new Fade({
                    onComplete: function () {
                        Bento.screens.show(nextScreen, settings);
                    }
                });
            });
        };

        // entry point for step 1.2
        assetGroups['missionJson'] = groupMissionsJson;
        Bento.assets.load('missionJson', function () {
            inspectTiled();
        }, function (assetsLoaded, assetCount, name) {
            bar.scale.x = assetsLoaded / assetCount * 0.1;
        });

        Bento.objects.attach(background);
        Bento.objects.attach(loaderEntity);
        bar.scale.x = 0;
    };

    return new Screen({
        onShow: onShow
    });
});