/**
 * Splash screen intro
 */
bento.define('screens/splash', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/screen',
    'bento/tween',
    'globals',
    'managers/save',
    'components/fillrect',
    'entities/effect',
    'components/multiscale',
    'entities/camera',
    'components/fillscreen',
    'gui/fade',
    'managers/resolution',
    'components/orientationlock',
    'gui/backscroller',
    'components/anchor'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Screen,
    Tween,
    Globals,
    SaveManager,
    FillRect,
    Effect,
    MultiScale,
    Camera,
    FillScreen,
    Fade,
    ResolutionManager,
    OrientationLock,
    BackScroller,
    Anchor
) {
    'use strict';
    // entering the splash screen means all necessary assets have been loaded
    var postPreloaderInit = function () {
        /*
         * Handle custom URL scheme
         */
        window.handleOpenURL = function (url) {
            var roomCode = url.replace('lk05://', '');
            Globals.roomCodeInvite = roomCode;
            EventSystem.fire('handleUrl', roomCode);
        };
        var hashParameter = Utils.getHashParameter('roomCode');

        if (window.cachedUrl) {
            // url was passed before the game was done loading
            window.handleOpenURL(window.cachedUrl);
            window.cachedUrl = undefined;
        } else if (hashParameter) {
            window.handleOpenURL(hashParameter);
        }
    };

    var viewport;

    var onShow = function (settings) {
        postPreloaderInit();

        viewport = Bento.getViewport();
        var screenCenter = new Vector2(viewport.width / 2, viewport.height / 2);

        var xOffset = 0;
        var yOffset = 0;

        var bgImage = new Sprite({
            imageName: 'splash/scrollingbackground-luckyred'
        });

        var splashSkipper = new Entity({
            z: 0,
            name: 'splashSkipper',
            position: new Vector2(0, 0),
            originRelative: new Vector2(0, 0),
            components: [
                new Clickable({
                    pointerUp: end,
                })
            ]
        });

        var character = new Entity({
            z: 3,
            name: 'character',
            family: 'character',
            position: new Vector2(viewport.width / 2, viewport.height / 2),
            components: [
                new Sprite({
                    spriteSheet: 'boy/idle'
                })
            ]
        });

        var fillBackground = new Entity({
            z: 0,
            name: 'background',
            family: ['fill'],
            components: [
                new FillScreen({
                    color: [1, 1, 1, 1]
                }),
                new OrientationLock({})
            ]
        });

        var background = new BackScroller({
            z: 1,
            name: 'background',
            family: ['background'],
            position: new Vector2(0, 0),
            imageName: 'splash/scrollingbackground-luckyred',
            velocity: new Vector2(0, 0),
            spacing: new Vector2(-1, -1),
            padding: 64
        }).attach(new Anchor({
            onComplete: function () {
                if (Anchor.isWideScreen()) {
                    background.transform.x -= Anchor.getOffset().x;
                    background.transform.y -= Anchor.getOffset().y;
                }
            }
        })).attach({
            name: 'offset',
            update: function () {
                background.getComponent('scrollComponent', function (scrollComponent) {
                    scrollComponent.offset.x = -viewport.x;
                    scrollComponent.offset.y = -viewport.y;
                });
            }
        });

        var phone = new Entity({
            z: 2,
            name: 'phone',
            family: ['phone'],
            position: new Vector2(viewport.width / 2 + 32, -20),
            components: [
                new Sprite({
                    imageName: 'splash/phone',
                    originRelative: new Vector2(0.5, 1)
                })
            ]
        });

        var cat = new Entity({
            z: 1,
            name: 'cat',
            family: ['cat'],
            position: new Vector2(viewport.width * 1.5, viewport.height / 2 - 45),
            components: [
                new Sprite({
                    imageName: 'splash/cat',
                    originRelative: new Vector2(0.5, 0.5)
                })
            ]
        });

        var luckyLogo = new Entity({
            z: 3,
            name: 'logo',
            family: ['logo'],
            position: new Vector2(viewport.width * 1.5, viewport.height / 2 - 45),
            components: [
                new Sprite({
                    imageName: 'splash/logo',
                    originRelative: new Vector2(0.5, 0.5)
                })
            ]
        });

        Bento.objects.attach(splashSkipper);
        Bento.objects.attach(fillBackground);
        Bento.objects.attach(background);
        Bento.objects.attach(character);
        Bento.objects.attach(phone);
        Bento.objects.attach(cat);
        Bento.objects.attach(luckyLogo);

        luckyLogo.visible = false;

        dropPhone();
    };

    // drop phone down to boy, pan camera, show background
    var dropPhone = function () {
        var phone = Bento.objects.getByFamily('phone')[0];
        var fill = Bento.objects.getByFamily('fill')[0];
        var background = Bento.objects.getByFamily('background')[0];
        // drop phone
        new Tween({
            from: -20,
            to: viewport.height / 2,
            in: 30,
            ease: 'easeInQuad',
            onStart: function () {},
            onUpdate: function (v, t) {
                phone.position.y = v;
            },
            onComplete: function () {
                // give phone some scale
                new Tween({
                    from: 1.2,
                    to: 1,
                    'in': 10,
                    ease: Tween.EASEOUTBACK,
                    onUpdate: function (v, t) {
                        phone.scale.x = v;
                        phone.scale.y = 1 / v;
                    }
                });
                // boy starts charging
                boyCharge();
            }
        });

        // viewport pans to the right, background fades in
        new Tween({
            from: 0,
            to: 1,
            in: 60,
            delay: 0,
            ease: Tween.EASEOUTQUAD,
            onUpdate: function (v, t) {
                viewport.x = v * 16;
                viewport.y = v * -45;
                fill.position = viewport;
                // background.alpha = v;
            },
        });

    };

    // boy/lee starts to charge his special attack
    var boyCharge = function () {
        var character = Bento.objects.getByFamily('character')[0];
        var sprite = character.getComponent('sprite');

        sprite.setSpriteSheet('boy/chargeup', function () {
            sprite.setSpriteSheet('boy/chargeup-loop');
        });

        Utils.timeout(15, function () {
            Bento.audio.stopSound('sfx_charge');
            Bento.audio.playSound('sfx_charge');
        });

        // done charging
        Utils.timeout(40, function () {
            Bento.audio.stopSound('sfx_charge');
            Bento.audio.stopSound('sfx_chargeready');
            Bento.audio.playSound('sfx_chargeready');

            sprite.setSpriteSheet('boy/chargeup-ready');

            new Tween({
                from: 1.2,
                to: 1,
                'in': 20,
                ease: Tween.EASEINBACK,
                onUpdate: function (v, t) {
                    character.scale.x = v;
                    character.scale.y = 1 / v;
                }
            });

            Bento.objects.attach(new Effect({
                spriteSheet: 'effects/charge',
                position: character.position.add(new Vector2(-4, -16))
            }));
        });

        Utils.timeout(70, function () {
            boyFlipKick(sprite);
        });
    };

    var boyFlipKick = function (sprite) {
        var character = Bento.objects.getByFamily('character')[0];
        var phone = Bento.objects.getByFamily('phone')[0];
        sprite.setSpriteSheet('boy/chargeup-attack', function () {
            sprite.setSpriteSheet('boy/idle');
        });

        Bento.audio.stopSound('boy/sfx_special');
        Bento.audio.playSound('boy/sfx_special');

        // hit the phone
        Utils.timeout(10, function () {
            Bento.audio.playSound('sfx_punch2');
            Bento.objects.attach(new Effect({
                spriteSheet: 'effects/hitfx1',
                position: character.position.add(new Vector2(24, -16))
            }));

            // stretch phone
            new Tween({
                from: 1.2,
                to: 1,
                'in': 40,
                ease: Tween.EASEOUTELASTIC,
                onUpdate: function (v, t) {
                    phone.scale.x = v;
                    phone.scale.y = v;
                }
            });

            flyToLogo();
        });
    };

    // phone flies to logo
    var flyToLogo = function () {
        var phone = Bento.objects.getByFamily('phone')[0];
        var logo = Bento.objects.getByFamily('logo')[0];

        // move phone
        new Tween({
            from: phone.position.x,
            to: logo.position.x,
            in: 20,
            delay: 0,
            ease: 'linear',
            onUpdate: function (v, t) {
                phone.position.x = v;
            },
            onComplete: function () {
                Bento.objects.getByFamily('cat')[0].visible = false;
                logo.visible = true;
                Bento.audio.playSound('sfx_punch0');
                new Tween({
                    from: 0.85,
                    to: 1,
                    'in': 30,
                    ease: Tween.EASEOUTELASTIC,
                    onUpdate: function (v, t) {
                        logo.scale.x = v;
                        logo.scale.y = v;
                    },
                    onComplete: function () {
                        Bento.audio.playSound('sfx_meow');
                        Utils.timeout(50, function () {
                            end();
                        });
                    }
                });
            }
        });

        // move camera
        new Tween({
            from: 16,
            to: viewport.width,
            in: 40,
            delay: 0,
            ease: Tween.EASEOUTBACK,
            onUpdate: function (v, t) {
                viewport.x = v;
            },
        });
    };

    // end of splash screen, starts titlescreen
    var end = function () {
        new Fade({
            type: 'opacity',
            fadeIn: false,
            duration: 45,
            onComplete: function () {
                Bento.screens.show('screens/title');
            }
        });
    };

    return new Screen({
        onShow: onShow,
        onHide: function () {
            // TODO: unload splash screen assets
        }
    });
});