/**
 * Screen description
 */
bento.define('screens/title', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/screen',
    'bento/tween',
    'components/anchor',
    'managers/resolution',
    'modules/localization',
    'managers/save',
    'gui/fade',
    'globals',
    'gui/shadowtext',
    'entities/camera',
    'modules/analytics',
    'modules/firebase',
    'modules/tenjin',
    'gui/dialog',
    'gui/loadingspinner',
    'managers/inventory',
    'managers/character',
    'modules/googleplay',
    'modules/googleplaylogin',
    'managers/quest',
    'managers/buff',
    'modules/ktplay',
    'entities/crosspromo'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Screen,
    Tween,
    Anchor,
    ResolutionManager,
    Localization,
    SaveManager,
    Fade,
    Globals,
    ShadowText,
    Camera,
    Analytics,
    FireBase,
    Tenjin,
    Dialog,
    LoadingSpinner,
    InventoryManager,
    CharacterManager,
    GooglePlay,
    GooglePlayLogin,
    QuestManager,
    BuffManager,
    KTPlayModule,
    StaticCrossPromo
) {
    'use strict';
    /**
     * Compatibility with old versions
     */
    var compatibilityCheck = function () {
        // scrolls: inspect which stages contain scrolls
        // if the player already cleared this stage, give the scroll to the player
        var unlockedMissions = SaveManager.load('unlockedLevels');
        var missionsJson = Bento.assets.getJson('missions/missions');
        Utils.forEach(unlockedMissions, function (mission) {
            var missionData = missionsJson[mission];
            if (!missionData) {
                return;
            }
            // check if this particular mission was cleared before by looking at the stars
            var stars = SaveManager.load('missionStars-' + mission);
            var reward1;
            var amount;
            if (stars > 0) {
                // does this mission have any scroll as reward?
                reward1 = (missionData.rewards || {}).reward1;
                if (reward1.indexOf('scrolls/') === 0) {
                    // does the player not have this scroll yet?
                    amount = InventoryManager.getItem(reward1);

                    if (amount === 0) {
                        // give the player this scroll
                        InventoryManager.addItem(reward1, 1);
                        // mark as seen
                        SaveManager.save('openedScroll-' + reward1, true);
                    }
                }
            }

        });
    };

    var onShow = function (screenData) {
        /* Screen starts here */
        var portrait = true;
        var canProceed = false;
        /**
         * Login to Google Play
         */
        var loginToGooglePlay = function (onComplete) {
            var askToLogin = function () {
                var dialog;
                var service = Localization.getText('AndroidService');
                logo.removeByName('cheat');
                dialog = new Dialog({
                    z: 5,
                    name: 'askForCloudLoad',
                    type: 'yes',
                    title: Localization.getText('AndroidService'),
                    text: Localization.getText('cloudLoginGP').replace('{SERVICE}', service),
                    canClose: true,
                    onYes: function () {
                        Bento.saveState.save('loginAttemptGooglePlay', true);
                        login();
                    },
                    onClose: function () {
                        // don't ask on bootup anymore
                        Bento.saveState.save('promptGooglePlayOnBoot', false);
                        if (onComplete) {
                            onComplete();
                        }
                    }
                });
                Bento.objects.attach(dialog);
            };
            var login = function () {
                GooglePlayLogin(function () {
                    if (onComplete) {
                        onComplete();
                    }
                });
            };
            if (Bento.saveState.load('promptGooglePlayOnBoot', true)) {
                // if the player agreed to login to GP, we just login,
                // else we ask first
                if (Bento.saveState.load('loginAttemptGooglePlay', false)) {
                    login();
                } else {
                    askToLogin();
                }
            }
        };
        /**
         * Show dialog about new version
         */
        var showNewVersion = function (onComplete) {
            var dialog;
            if (Utils.newVersion) {
                dialog = new Dialog({
                    name: 'update',
                    title: Localization.getText('title'),
                    text: Localization.getText('newVersion').replace('%1', 'v' + Utils.newVersion),
                    canClose: true,
                    type: 'yesno',
                    onYes: function () {
                        if (Utils.isCocoonJs() && window.Cocoon) {
                            window.Cocoon.App.openURL(Utils.getStoreLink());
                        }
                    },
                    onNo: function () {},
                    onComplete: onComplete
                });
                Bento.objects.attach(dialog);
            } else if (onComplete) {
                onComplete();
            }
        };

        // camera for screenshake
        var camera = new Camera({});

        var fade = new Fade({
            wait: true,
            duration: 10,
            fadeIn: true,
            type: 'opacity'
        });

        var backgroundPortrait = new Entity({
            z: 0,
            name: 'backgroundPortrait',
            components: [
                new Anchor({
                    screenPosition: new Vector2(0.5, 0.5),
                    // onComplete: align
                }),
                new Sprite({
                    originRelative: new Vector2(0.5, 0.5),
                    imageName: 'titlescreen/background-portrait'
                })
            ]
        });
        var backgroundLandscape = new Entity({
            z: 0,
            name: 'backgroundLandscape',
            components: [
                new Anchor({
                    screenPosition: new Vector2(0.5, 0.5)
                }),
                new Sprite({
                    originRelative: new Vector2(0.5, 0.5),
                    imageName: 'titlescreen/background-landscape'
                })
            ]
        });

        var brokenPortrait = new Entity({
            z: 1,
            name: 'brokenPortrait',
            components: [
                new Anchor({
                    screenPosition: new Vector2(0.5, 0.5),
                    // offset: new Vector2(0, -10)
                    // onComplete: align
                }),
                new Sprite({
                    originRelative: new Vector2(0.5, 0.5),
                    imageName: 'titlescreen/broken-portrait'
                })
            ]
        });
        var brokenLandscape = new Entity({
            z: 1,
            name: 'brokenLandscape',
            components: [
                new Anchor({
                    screenPosition: new Vector2(0.5, 0.5),
                    // offset: new Vector2(0, -10)
                }),
                new Sprite({
                    originRelative: new Vector2(0.5, 0.5),
                    imageName: 'titlescreen/broken-landscape'
                })
            ]
        });

        var started = false;
        var pointerDown = false;
        var touched = 0;
        var loadingSpinner = new LoadingSpinner({
            z: 10
        }).attach(new Anchor({
            screenPosition: new Vector2(0.5, 0.75)
        }));
        var logo = new Entity({
            z: 2,
            name: 'logo',
            family: [''],
            components: [
                new Anchor({
                    screenPosition: new Vector2(0.5, 0.5),
                    offset: new Vector2(-4, -24),
                    onComplete: function (parent, isPortrait) {
                        if (isPortrait) {
                            backgroundLandscape.visible = false;
                            brokenLandscape.visible = false;
                            backgroundPortrait.visible = true;
                            brokenPortrait.visible = true;
                        } else {
                            backgroundLandscape.visible = true;
                            brokenLandscape.visible = true;
                            backgroundPortrait.visible = false;
                            brokenPortrait.visible = false;
                        }
                    }
                }),
                new Sprite({
                    imageName: 'titlescreen/logo-' + Globals.getTitleLanguage(),
                    originRelative: Vector2(0.5, 0.5)
                }),
                new Clickable({
                    ignorePauseDuringPointerUpEvent: false,
                    pointerDown: function (data) {
                        if (Bento.input.getPointers().length > 1) {
                            pointerDown = true;
                            touched = 0;
                        }
                    },
                    pointerUp: function (data) {
                        pointerDown = false;
                        if (touched < 240 && started) {
                            startGame();
                        }
                    }
                }), //
                {
                    name: 'cheat',
                    update: function () {
                        if (pointerDown && started) {
                            if (++touched === 240) {
                                ask();
                            }
                        } else {
                            touched = 0;
                        }
                    }
                }
            ]
        });
        var ask = function () {
            var res;
            if (Utils.isCocoonJS() && window.Cocoon) {
                window.Cocoon.Dialog.prompt({
                    title: 'Hello',
                    message: 'What would you like?'
                }, {
                    success: function (text) {
                        showDeveloperCheats(text);
                    }
                });
            } else {
                res = window.prompt("What would you like?");
                showDeveloperCheats(res);
            }
        };

        var showDeveloperCheats = function (code) {
            if (code === 'imadev') {
                Bento.saveState.save('showDeveloperCheats', true);
                Analytics.setActive(false);
            } else if (code === 'reset') {
                var cheats = Bento.saveState.load('showDeveloperCheats', false);
                Bento.saveState.clear();
                Bento.saveState.save('showDeveloperCheats', cheats);
                Bento.screens.show('screens/title');
            } else {
                Bento.saveState.save('showDeveloperCheats', false);
            }
        };

        var viewport = Bento.getViewport();
        var showCrossPromo = function () {
            var openedTitleScreen = Bento.saveState.load('openedTitleScreen', 0);
            var language = Localization.getLanguage();

            // don't show crosspromo the first time!!
            if (
                openedTitleScreen > 0 &&
                Bento.saveState.load('iapAmount', 0) < 1
            ) {
                // static image as crosspromo
                new StaticCrossPromo({
                    z: 20,
                    pauseLevel: 1,
                    sfx: 'sfx_button'
                });
            }

            Bento.saveState.save('openedTitleScreen', openedTitleScreen + 1);
        };

        var startGame = function () {
            var start = function () {
                // Init character manager
                CharacterManager.init();
                // Init inventory manager
                InventoryManager.load();
                //Initialize quest manager
                QuestManager.initialize();
                // reload buffs for allies
                BuffManager.loadBuffs();

                // log into KTplay if user ID is available
                Utils.KTPlayAutoLogin(function (user) {
                    Utils.changeLocalNameToKTNickName();
                }, function (error) {
                    console.log(error);
                });

                // inspect save file
                compatibilityCheck();

                var tutorialState = SaveManager.load('tutorial');
                if (tutorialState < 3) {
                    SaveManager.save('tutorial', 0);
                    Analytics.event('Tutorial', 'gym_start');
                    // Standard Firebase event
                    FireBase.logEvent(FireBase.Events.TUTORIAL_BEGIN);

                    // tenjin event
                    Tenjin.sendEvent('tutorialStart');

                    // preload tutorial
                    SaveManager.save('currentMission', 'tutorial');
                    Bento.screens.show('screens/preloadmission', {
                        nextScreen: 'screens/tutorial'
                    });

                } else {
                    // go to exchange to see if equipment needs to be replaced
                    Bento.screens.show('screens/exchange');
                }

            };
            var fadeOut = function () {
                loadingSpinner.removeSelf();
                new Fade({
                    pause: Globals.PauseLevels.FADE,
                    duration: 10,
                    fadeIn: false,
                    type: 'opacity',
                    onComplete: start
                });
            };

            startText.removeSelf();
            logo.removeByName('clickable');
            logo.removeByName('cheat');

            if (!loadingSpinner.isAdded) {
                Bento.objects.attach(loadingSpinner);
            }

            fadeOut();
        };

        var startText = new ShadowText({
            z: 2,
            text: Localization.getText('touchToStart'),
            font: 'font',
            fontSize: 16,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'bottom',
            ySpacing: 0,
            lineWidth: 0,
            strokeStyle: '#000000',
            innerStroke: false,
            pixelStroke: false // workaround for Cocoon bug
        }).attach(
            new Anchor({
                screenPosition: new Vector2(0.5, 0.9)
            })
        ).attach({
            name: 'startTextBehavior',
            update: function () {
                startText.position.y += Math.sin(startText.timer / 24) * 0.2;
            }
        });


        var fist = new Entity({
            z: 3,
            name: 'fist',
            position: new Vector2(0, 0),
            components: [
                new Anchor({
                    screenPosition: new Vector2(0.5, 0.5)
                }),
                new Sprite({
                    imageName: 'titlescreen/fist',
                    originRelative: Vector2(0.5, 0.5)
                })
            ]
        });

        var breakTheWall = function () {
            var screenMiddleX = viewport.width / 2;
            var screenMiddleY = viewport.height / 2;
            Bento.audio.playSound('sfx_breakwall');
            camera.shake(10, 10);
            makeDebris({
                position: new Vector2(screenMiddleX - 44, screenMiddleY - 60),
                force: new Vector2(-3, -3),
                scale: 0.8,
                rotation: Utils.toRadian(-8),
                id: 0
            });
            makeDebris({
                position: new Vector2(screenMiddleX - 48, screenMiddleY + 32),
                force: new Vector2(-4, -2),
                rotation: Utils.toRadian(9),
                id: 1
            });
            makeDebris({
                position: new Vector2(screenMiddleX + 40, screenMiddleY - 48),
                force: new Vector2(2, -4.8),
                rotation: Utils.toRadian(2),
                id: 2
            });
            makeDebris({
                position: new Vector2(screenMiddleX + 50, screenMiddleY + 48),
                force: new Vector2(3.7, -2.4),
                scale: 0.7,
                rotation: Utils.toRadian(-5),
                id: 3
            });
            makeDebris({
                position: new Vector2(screenMiddleX + 10, screenMiddleY + 5),
                force: new Vector2(5, -2),
                scale: 1.2,
                rotation: Utils.toRadian(-3),
                id: 4
            });
        };

        var makeDebris = function (settings) {
            var behavior = {
                name: 'debrisBehavior',
                start: function () {
                    if (settings.scale) {
                        debris.scale.x = settings.scale;
                        debris.scale.y = settings.scale;
                    }
                    debris.rotation += settings.rotation;
                },
                update: function () {
                    debris.scale.x += 0.03;
                    debris.scale.y += 0.03;
                    debris.position.addTo(settings.force);
                    debris.rotation += settings.rotation;
                    settings.force.y += 0.8;
                }
            };
            var debris = new Entity({
                z: 4,
                name: 'brick-' + settings.id,
                family: ['bricks'],
                position: settings.position,
                rotation: Utils.toRadian(-45),
                components: [
                    new Sprite({
                        imageName: 'titlescreen/brick',
                        originRelative: new Vector2(0.5, 0.5)
                    }),
                    behavior
                ]
            });
            Bento.objects.attach(debris);
        };

        var startFadeIn = function () {
            started = true;
            startText.visible = true;
            startText.alpha = 0;
            new Tween({
                from: 0,
                to: 1,
                in: 30,
                delay: 0,
                ease: 'linear',
                onUpdate: function (v, t) {
                    startText.alpha = v;
                },
            });
        };

        //play music
        Bento.audio.stopAllMusic();
        Bento.audio.playMusic('bgm_title_intro', false, function () {
            var currentScreen = Bento.screens.getCurrentScreen();
            var screenName;
            if (currentScreen) {
                screenName = currentScreen.name;
            }
            // its possible to jump to a different screen before this music is finished
            if (screenName === 'screens/title' || screenName === 'screens/exchange') {
                Bento.audio.playMusic('bgm_title');
            }
        });

        Bento.objects.attach(camera);
        Bento.objects.attach(backgroundPortrait);
        Bento.objects.attach(backgroundLandscape);
        Bento.objects.attach(brokenPortrait);
        Bento.objects.attach(brokenLandscape);
        Bento.objects.attach(logo);
        Bento.objects.attach(fist);
        Bento.objects.attach(startText);

        brokenPortrait.visible = false;
        brokenLandscape.visible = false;
        logo.visible = false;
        fist.visible = false;
        startText.visible = false;

        //start sequence

        new Tween({
            from: 0.5,
            to: 1,
            in: 40,
            delay: 30,
            ease: 'easeOutElastic',
            onStart: function () {
                brokenPortrait.visible = true;
                brokenLandscape.visible = true;
                fist.visible = true;
                logo.visible = true;
                breakTheWall();
            },
            onUpdate: function (v, t) {
                fist.scale.x = v;
                fist.scale.y = v;
            },
            onComplete: function () {
                new Tween({
                    from: 1,
                    to: 0,
                    in: 20,
                    delay: 0,
                    ease: 'easeInQuad',
                    onStart: function () {},
                    onUpdate: function (v, t) {
                        fist.scale.x = v;
                        fist.scale.y = v;
                        fist.alpha = v;
                    },
                    onComplete: function () {
                        Bento.objects.remove(fist);
                        showCrossPromo();
                        startFadeIn();

                        showNewVersion(function () {
                            // we login during the title screen
                            // do this here now because the Google Play login screen is intrusive
                            // and we are able to present some context here (show a dialog with cloud save reason)
                            if (Utils.isNativeAndroid()) {
                                loginToGooglePlay(function () {});
                            }
                        });
                    }
                });
            }
        });

        fade.startFade();
        Analytics.trackView('title');
    };

    return new Screen({
        onShow: onShow,
        onHide: function () {},
        dimension: new Rectangle(0, 0, 640, 640)
    });
});