/**
 * Tutorial screen
 */
bento.define('screens/tutorial', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/screen',
    'bento/tween',
    'gui/fade',
    'entities/camera',
    'components/camerabehavior',
    'entities/players/boy',
    'managers/stats',
    'gui/main/healthbar',
    'gui/equipped',
    'globals',
    'components/fillrect',
    'entities/tilecolliders',
    'entities/triggers/solid',
    'components/sort',
    'components/shadow',
    'entities/punchingbag',
    'gui/speechballoon',
    'modules/localization',
    'bento/tiled',
    'components/anchor',
    'gui/shadowtext',
    'managers/save',
    'components/tutorials/tutorial2',
    'components/tutorials/tutorial1',
    'components/tutorials/tutorial3',
    'modules/analytics',
    'components/viewanalytics',
    'gui/buttons/settings',
    'modules/cloudsave',
    'gui/dialog'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Screen,
    Tween,
    Fade,
    Camera,
    CameraBehavior,
    PlayerBoy,
    StatsManager,
    HealthBar,
    Equipped,
    Globals,
    FillRect,
    TileColliders,
    Solid,
    Sort,
    Shadow,
    Punchingbag,
    SpeechBalloon,
    Localization,
    Tiled,
    Anchor,
    ShadowText,
    SaveManager,
    Tutorial2,
    Tutorial1,
    Tutorial3,
    Analytics,
    ViewAnalytics,
    SettingsButton,
    CloudSave,
    Dialog
) {
    'use strict';
    var onShow = function (settings) {
        /* Screen starts here */
        var viewport = Bento.getViewport();
        var fade = new Fade({
            wait: true,
            duration: 10,
            fadeIn: true,
            type: 'opacity'
        });
        var camera = new Camera({}).attach(new CameraBehavior({}));
        var settingsButton = new SettingsButton({
            preventTopBar: false,
            preventChests: true,
            skipTutorial: true
        });

        // this background should never be seen
        // is pure red during dev
        var debugBackground = new Entity({
            z: -1,
            name: 'debugBackground',
            position: new Vector2(0, 0),
            float: true,
            components: [
                new FillRect({
                    size: viewport.getSize(),
                    color: [Bento.isDev() ? 1 : 0, 0, 0, 1]
                })
            ]
        });

        var currentTutorial = SaveManager.load('tutorial');

        var tiled = new Tiled({
            assetName: 'missions/tutorial/' + (currentTutorial + 1),
            spawnBackground: true,
            spawnEntities: true,
            onObject: function (object, tileSet, tileIndex) {
                // spawn triggers (if they have a gid, they are linked to a tileSet)
                if (!object.gid) {
                    if (object.type === 'Solid') {
                        // place a solid
                        Bento.objects.attach(new Solid({
                            boundingBox: new Rectangle(object.x, object.y, object.width, object.height)
                        }));
                    }
                }
            },
            onSpawn: function (instance, object, extraData) {
                if (instance.name === 'character' || instance.name === 'prop') {
                    instance.name += '-' + object.id;
                }

                if (instance.name === 'weapon') {
                    instance.weaponId = object.id;
                }
                //automatic attaching is turned off
            },
            onComplete: function () {
                // synchronous completion
                screen.dimension = this.dimension.clone();
            }
        });

        var senseiBehavior = {
            name: 'senseibehavior',
            wave: 0,
            update: function () {
                senseiBehavior.wave += 0.1;
                sensei.scale.y = 1.04 + Math.sin(senseiBehavior.wave) * 0.08;
            }
        };
        var sensei = new Entity({
            z: 300,
            name: 'sensei',
            family: ['npc'],
            position: new Vector2(120, 160),
            components: [
                new Sprite({
                    spriteSheet: 'sensei/idle'
                }),
                new Sort({}),
                // senseiBehavior,
                new Shadow({}),

            ]
        });

        var punchingBag = new Punchingbag({
            position: new Vector2(200, 151)
        });

        var player = new PlayerBoy({
            position: new Vector2(200, 240),
            id: 0,
            name: 'player',
            online: false,
            health: StatsManager.health
        });
        StatsManager.resetHealth();
        var healthBar = new HealthBar({
            target: player,
            health: StatsManager.health,
            maxHealth: StatsManager.getStat('boy', 'health')
        });
        var equipped;

        // component that centers speechBalloon to the center and points the bottom towards sensei
        var stayCentered = {
            name: 'stayCentered',
            active: false,
            update: function () {
                tutorialEntity.getComponent('tutorialBehavior', function (tutorial) {
                    if (tutorial.speechBalloon && stayCentered.active) {
                        tutorial.speechBalloon.position.x = viewport.x + viewport.width / 2;
                        tutorial.speechBalloon.setBottomPosition(sensei.position.x - (viewport.x + viewport.width / 2) + 5);
                    }
                });
            }
        };

        var tutorial1 = new Tutorial1({});
        var tutorial2 = new Tutorial2({});
        var tutorial3 = new Tutorial3({});

        var tutorialEntity = new Entity({
            z: 0,
            name: 'tutorial',
            components: [
                stayCentered
            ]
        });

        Bento.objects.attach(debugBackground);
        Bento.objects.attach(tiled);
        Bento.objects.attach(punchingBag);
        Bento.objects.attach(player);
        Bento.objects.attach(sensei);
        Bento.objects.attach(camera);
        Bento.objects.attach(healthBar);

        settingsButton.removeByName('anchor');
        settingsButton.attach(new Anchor({
            screenPosition: new Vector2(1, 0),
            offset: new Vector2(-20, 20),
        }));
        Bento.objects.attach(settingsButton);

        // add correct tutorial
        if (currentTutorial === 0) {
            tutorialEntity.attach(tutorial1);
            tutorialEntity.attach(new ViewAnalytics({
                name: 'tutorial_gym_3',
                ignorePrevious: true
            }));
        } else if (currentTutorial === 1) {
            tutorialEntity.attach(tutorial2);
            tutorialEntity.attach(new ViewAnalytics({
                name: 'tutorial_gym_1',
                ignorePrevious: true
            }));
        } else if (currentTutorial === 2) {
            tutorialEntity.attach(tutorial3);
            tutorialEntity.attach(new ViewAnalytics({
                name: 'tutorial_gym_2',
                ignorePrevious: true
            }));
        }
        Bento.objects.attach(tutorialEntity);


        // add equipped after tutorialcomponent so that tutorial can set item before it gets loaded
        equipped = new Equipped({});
        Bento.objects.attach(equipped);

        // add music if not playing
        if (Globals.currentMusic !== 'bgm_gymtutorial') {
            Bento.audio.stopAllMusic();
            Globals.currentMusic = 'bgm_gymtutorial';

            if (Utils.isNativeAndroid()) {
                // QUICKFIX: for some reason android sometimes skips the music
                new Tween({
                    ignoreGameSpeed: true,
                    in: 3,
                    onComplete: function () {
                        Bento.audio.playMusic(Globals.currentMusic);
                    }
                });
            } else {
                Bento.audio.playMusic(Globals.currentMusic);
            }
        }

        if (currentTutorial === 0 && CloudSave.isAvailable()) {
            CloudSave.hasAnySave(function (hasSave) {
                if (!hasSave) {
                    return;
                }
                // NOTE: duplicate from settings.js
                var dialog = new Dialog({
                    name: 'askForCloudLoadStart',
                    type: 'yesno',
                    title: Localization.getText('cloud'),
                    text: Localization.getText('loadFromCloud'),
                    canClose: false,
                    onYes: function () {
                        CloudSave.loadFromCloud(function (success, reason) {
                            var errorDialog;
                            if (success) {
                                if (reason === 'cloudSaveNone') {
                                    // there was nothing to load at all
                                    errorDialog = new Dialog({
                                        name: 'askForCloudLoad',
                                        type: 'yes',
                                        title: Localization.getText('errorTitle'),
                                        text: Localization.getText('cloudNoSaveFile')
                                    });
                                    Bento.objects.attach(errorDialog);
                                    return;
                                }
                                // refresh screen
                                // go to title screen because this is where managers are initialized
                                // and compatibility is checked
                                Bento.screens.show('screens/title');
                                Bento.objects.resume();
                            } else {
                                // show error
                                errorDialog = new Dialog({
                                    name: 'askForCloudLoad',
                                    type: 'yes',
                                    title: Localization.getText('errorTitle'),
                                    text: Localization.getText('cloudLoadError')
                                });
                                Bento.objects.attach(errorDialog);
                            }
                        });
                    }
                });
                Bento.objects.attach(dialog);
            });
        }

        fade.startFade();
    };

    var screen = new Screen({
        onShow: onShow,
        onHide: function (data) {
            // unload assets
            if (data.next !== 'screens/tutorial') {
                Bento.assets.unload('missionJson', true);
                Bento.assets.unload('groupMissionAssets', true);
            }
        }
    });

    return screen;
});