/**
 * World select screen
 */
bento.define('screens/worldselect', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'modules/text',
    'bento/utils',
    'bento/screen',
    'bento/tween',
    'gui/worldscroller',
    'bento/tiled',
    'components/fillrect',
    'entities/viewportscroller',
    'components/tilededge',
    'gui/currencycounter',
    'components/anchor',
    'gui/dialog',
    'managers/character',
    'gui/playercoins',
    'gui/worldselect/bottombar',
    'modules/agreement',
    'gui/speechballoon',
    'components/modal',
    'gui/loadingspinner',
    'globals',
    'modules/matchmanager',
    'gui/fade',
    'managers/save',
    'gui/worldselect/topbar',
    'gui/buttons/gift',
    'gui/buttons/settings',
    'gui/menus/onlinejoin',
    'gui/levelstart/rewardspopup',
    'gui/menus/missiondetails',
    'components/eventlistener',
    'gui/worldtutorial',
    'gui/worldselect/adforcoins',
    'gui/taphand',
    'gui/highlight',
    'gui/senseipopup',
    'modules/localization',
    'managers/inventory',
    'modules/analytics',
    'components/viewanalytics',
    'entities/zoom',
    'components/drawinviewport',
    'modules/firebase',
    'modules/tenjin',
    'gui/askrating',
    'gui/iap/offerpack',
    'gui/iap/offerpackpopup',
    'gui/worldselect/mission',
    'modules/bezier',
    'gui/sparkleburst',
    'components/blink',
    'managers/time',
    'gui/coinreward',
    'gui/menus/scroll',
    'gui/badge',
    'gui/iconbutton',
    'gui/menus/online',
    'modules/replaykit',
    'bento/packedimage',
    'gui/menus/towerchallenge',
    'gui/menus/beachevent'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Screen,
    Tween,
    WorldScroller,
    Tiled,
    FillRect,
    ViewportScroller,
    TiledEdge,
    CurrencyCounter,
    Anchor,
    Dialog,
    CharacterManager,
    PlayerCoins,
    BottomBar,
    Agreement,
    SpeechBalloon,
    Modal,
    LoadingSpinner,
    Globals,
    MatchManager,
    Fade,
    SaveManager,
    Topbar,
    GiftButton,
    SettingsButton,
    OnlineJoin,
    RewardsPopup,
    MissionDetails,
    EventListener,
    WorldTutorial,
    AdForCoins,
    TapHand,
    Highlight,
    SenseiPopup,
    Localization,
    InventoryManager,
    Analytics,
    ViewAnalytics,
    Zoom,
    DrawInViewport,
    FireBase,
    Tenjin,
    AskRating,
    OfferPack,
    OfferPackPopup,
    Mission,
    Bezier,
    SparkleBurst,
    Blink,
    TimeManager,
    CoinReward,
    ScrollMenu,
    Badge,
    IconButton,
    OnlineMenu,
    ReplayKit,
    PackedImage,
    TowerMenu,
    BeachEventMenu
) {
    'use strict';
    var worldmapImages = {};
    var onShow = function (settings) {
        var viewport = Bento.getViewport();
        // this background should never be seen
        // is pure red during dev
        var debugBackground = new Entity({
            z: -1,
            name: 'debugBackground',
            position: new Vector2(0, 0),
            float: true,
            components: [
                new FillRect({
                    size: viewport.getSize(),
                    color: [Bento.isDev() ? 1 : 0, 0, 0, 1]
                })
            ]
        });
        var tiled;
        var hasUnlockedAnyLevel = false;
        var fade = new Fade({
            wait: true,
            duration: 30,
            fadeIn: true,
            type: 'opacity',
            onComplete: function () {
                // open invite?
                if (Globals.roomCodeInvite) {
                    openInvitation(Globals.roomCodeInvite);
                    Globals.roomCodeInvite = null;
                } else if (SaveManager.load('shouldAskForReview')) {
                    SaveManager.save('shouldAskForReview', false);

                    // ask for a review
                    if (Utils.isNativeIos()) {
                        AskRating.askIos10();
                    } else if (Utils.isNativeAndroid() || Bento.isDev()) {
                        // use old review funnel
                        Bento.objects.attach(new AskRating({
                            z: Globals.WorldSelectLayers.DIALOGS
                        }));
                    }
                }

                if (!hasUnlockedAnyLevel) {
                    EventSystem.fire('worldSelectReady');
                }

                // addMultiPlayerButton();
            }
        });
        var lastKnownViewportPosition;
        var worldScroller = new WorldScroller({
            scrollX: true,
            scrollY: true,
            width: Infinity,
            height: Infinity,
        });
        // array of loaded maps, the mapManager may unload tiled maps to conserve memory
        var loadedMaps = [];
        var currentlyLoading = {};
        // a (hash)grid where the viewport is allowed to go
        var hashGrid = {};
        // used when first time loading all maps and calling a complete callback
        var initialLoad = false;
        var toLoadCount = 0;
        var hasLoadedCount = 0;
        // async completion callback
        var checkDone = function () {
            if (initialLoad) {
                return;
            }
            if (toLoadCount === hasLoadedCount) {
                initialLoad = true;
                // all maps loaded!
                Bento.objects.attach(viewportController);
                fade.startFade();

                // scroll to most recently unlocked level
                // animation for each unlocked level, for now we only scroll to the first entry
                var recentlyUnlocked = SaveManager.load('recentlyUnlocked');
                if (recentlyUnlocked.length > 0) {
                    hasUnlockedAnyLevel = true;

                    scrollToUnlocked(false, 0);
                    // regardless we must activate buttons because there is a chance the button can not be found
                    // (and the game would freeze as a result)
                    Utils.timeout(150, function () {
                        EventSystem.fire('worldSelectReady');
                    });
                }
            }
        };
        // scroll to recently unlocked mission
        var scrollToUnlocked = function (reInitButton, extraDelay) {
            var scrollToButton = function (button, delay) {
                var pulsate;
                var rewardBalloon;
                // hide mission
                button.setActive(false);
                pulsate = button.getComponent('pulsate');
                button.remove(pulsate);
                button.scale.x = 0;
                button.scale.y = 0;
                // remove reward balloon if any
                rewardBalloon = button.getComponent('rewardBalloon');
                if (rewardBalloon) {
                    button.remove(rewardBalloon);
                }

                new Tween({
                    from: Utils.toVector2(SaveManager.load('worldmapPositionVertical')),
                    to: button.position.subtract(viewport.getExtents()),
                    in: 60,
                    delay: delay + extraDelay,
                    ease: 'easeInOutBack',
                    onUpdate: function (v, t) {
                        // keep centered in the middle
                        v.x = mapManager.mapSize / 2 - viewport.width / 2;

                        // scroll to button
                        worldScroller.forceTo(v);
                        worldScroller.setActive(false);
                    },
                    onComplete: function () {
                        // show an animaiton of unlocking
                        new Tween({
                            from: new Vector2(0, 0),
                            to: new Vector2(1, 1),
                            delay: 20,
                            in: 40,
                            ease: 'elastic',
                            decay: 4,
                            oscillations: 2, // for 'elastic', 'sin' and 'cos'
                            onStart: function () {},
                            onUpdate: function (v, t) {
                                button.scale = v;
                            },
                            onComplete: function () {
                                // restore pulsate
                                if (pulsate) {
                                    button.attach(pulsate);
                                }
                                // restore balloon
                                if (rewardBalloon) {
                                    button.attach(rewardBalloon);
                                    // recheck the position of reward balloon
                                    rewardBalloon.getComponent('showWhenClose', function (showWhenClose) {
                                        showWhenClose.start();
                                    });
                                }

                                Utils.timeout(50, function () {
                                    button.setActive(true);
                                });
                                worldScroller.setActive(true);

                            }
                        });
                    }
                });
            };
            var recentlyUnlocked = SaveManager.load('recentlyUnlocked');
            // find recently unlocked missions
            Bento.objects.getByFamily('missionButtons', function (buttons) {
                Utils.forEach(recentlyUnlocked, function (unlocked, i, l, breakLoop) {
                    // find that button
                    Utils.forEach(buttons, function (button) {
                        if (button && button.mission === unlocked) {
                            // rebuild button
                            var buttonSettings;
                            var buttonPosition;
                            // rebuild the button
                            if (reInitButton) {
                                button.removeSelf();
                                buttonSettings = button.settings.missionSettings;
                                buttonSettings.icon = button.getIcon();
                                buttonPosition = button.position.clone();
                                buttonSettings.iconPosition = buttonPosition;

                                button = new Mission(buttonSettings);
                                Bento.objects.attach(button);
                                button.position = buttonPosition;

                                // make sure we can scroll to that, so refresh map
                                mapManager.loadMaps();
                            }

                            // scroll to it
                            scrollToButton(button, 150 * i);
                        }
                    });
                });
            });
            SaveManager.save('recentlyUnlocked', []);
        };
        var isOpeningInvite = false;
        var openInvitation = function (roomCode) {
            var missionDetails = new OnlineMenu({
                invitation: roomCode,
                onClosed: function () {
                    // restart screen
                    Bento.screens.show('screens/worldselect');
                }
            });
            Bento.objects.attach(missionDetails);
            isOpeningInvite = true;
        };
        // map manager
        var mapManager = {
            name: 'mapManager',
            mapSize: 512,
            path: 'worldselect/',
            currentIndex: new Vector2(0, 0),
            update: function () {
                // did the index change?
                var mapSize = mapManager.mapSize;
                var currentIndex = new Vector2(
                    Math.floor(viewport.x / mapSize),
                    Math.floor(viewport.y / mapSize)
                );
                if (currentIndex.x !== mapManager.currentIndex.x || currentIndex.y !== mapManager.currentIndex.y) {
                    mapManager.loadMaps();
                }
                // mapManager.loadMaps();

                // did corners pass
            },
            /**
             * Load maps around the current position
             */
            loadMaps: function (useLastKnownPos) {
                var i, j;
                // determine and update current indexedPosition
                var mapSize = mapManager.mapSize;
                var currentIndex;
                if (useLastKnownPos) {
                    currentIndex = new Vector2(
                        Math.floor(lastKnownViewportPosition.x / mapSize),
                        Math.floor(lastKnownViewportPosition.y / mapSize)
                    );
                } else {
                    currentIndex = new Vector2(
                        Math.floor(viewport.x / mapSize),
                        Math.floor(viewport.y / mapSize)
                    );
                }
                var index = currentIndex.clone();
                mapManager.currentIndex = currentIndex;

                for (i = -2; i <= 2; ++i) {
                    for (j = -2; j <= 2; ++j) {
                        index.x = currentIndex.x + i;
                        index.y = currentIndex.y + j;
                        // check the 16 surrounding maps: does any map need to be unloaded?
                        if (i === -2 || i === 2 || j === -2 || j === 2) {
                            mapManager.unloadMap(index);
                        }
                        // check the 8 neighboring maps: does this map exist?
                        else if (i === -1 || i === 1 || j === -1 || j === 1) {
                            mapManager.loadMap(index);
                        }
                        // center: just try to load
                        else if (i === 0 && j === 0) {
                            mapManager.loadMap(index);
                        }
                    }
                }
            },
            unloadMap: function (index) {
                var mapName = index.x + '_' + index.y;

                Utils.forEach(loadedMaps, function (loadedMap, i, l, breakLoop) {
                    // find the corresponding map
                    if (loadedMap.name !== mapName) {
                        return;
                    }
                    // remove all entities and backgrounds
                    Utils.forEach(loadedMap.entities, function (entity) {
                        Bento.objects.remove(entity);
                    });
                    Utils.forEach(loadedMap.backgrounds, function (background) {
                        Bento.objects.remove(background);
                    });
                    // remove image
                    /*if (worldmapImages[mapName]) {
                        if (worldmapImages[mapName].dispose) {
                            worldmapImages[mapName].dispose();
                        }
                        delete worldmapImages[mapName];
                    }*/
                    // remove this map
                    loadedMaps.splice(i, 1);
                    breakLoop();
                });
            },
            loadMap: function (index) {
                var mapName = index.x + '_' + index.y;
                var mapNameInverted = index.x + '_' + (-index.y); // inverted y axis so we don't have to add - in the asset names
                var loadedMap = mapManager.hasLoadedMap(index); // was the map already loaded?
                var inspectUnlocked = function (levels) {
                    // checks if this map can be scrolled to
                    // Changed to allow full scrolling
                    var hasUnlocked = true;
                    Utils.forEach(levels, function (level, i, l, breakLoop) {
                        // should be a level
                        if (level.name !== 'missionButton') {
                            return;
                        }
                        if (level.canPlayLevel()) {
                            hasUnlocked = true;
                            breakLoop();
                        }
                    });
                    if (hasUnlocked) {
                        hashGrid[mapName] = true;
                    }
                };

                if (loadedMap) {
                    // already loaded this map
                    inspectUnlocked(loadedMap.entities);
                    return;
                }
                if (!mapManager.mapExists(index)) {
                    // the map needs to exist in the assts
                    return;
                }
                // already trying to load this map
                if (currentlyLoading[mapName]) {
                    return;
                }
                currentlyLoading[mapName] = true;
                var offset = new Vector2(index.x * mapManager.mapSize, index.y * mapManager.mapSize);
                var map = new Tiled({
                    name: mapName,
                    assetName: mapManager.path + mapNameInverted,
                    spawnBackground: false,
                    spawnEntities: true,
                    merge: true,
                    maxCanvasSize: new Vector2(mapManager.mapSize / 2, mapManager.mapSize / 2),
                    offset: offset,
                    onSpawnComplete: function () {
                        // can the player scroll here?
                        // an extra requirement: there must be at least one playable level in here!
                        var levels = this.entities;
                        inspectUnlocked(this.entities);

                        hasLoadedCount += 1;

                        // some maps may have no objects and thus load immediately
                        // let's wait at least 1 frame before checking completion
                        Utils.timeout(2, checkDone);

                        currentlyLoading[mapName] = false;
                    },
                    drawTiles: false,
                    onComplete: function () {
                        var image = worldmapImages[mapName];
                        var self = this;
                        var makeBackground = function () {
                            var background = new Entity({
                                name: 'tiledLayer',
                                family: ['backgrounds'],
                                position: offset.clone(),
                                components: [
                                    new Sprite({
                                        image: new PackedImage(image)
                                    })
                                ]
                            });
                            Bento.objects.attach(background);
                            self.backgrounds.push(background);
                        };

                        this.backgrounds = this.backgrounds || [];

                        if (this.backgrounds.length === 0) {
                            image = worldmapImages[mapName];
                            if (!image) {
                                // get image dynamically (done without assetManager for convenience)
                                worldmapImages[mapName] = new Image();
                                image = worldmapImages[mapName];
                                image.src = './assets/city/images/worldselect/' + mapNameInverted + '.png';
                                image.onload = function () {
                                    makeBackground();
                                };
                            } else {
                                makeBackground();
                            }
                        }

                        Utils.forEach(this.backgrounds, function (backgroundLayer, i, l, breakLoop) {
                            backgroundLayer.attach(new DrawInViewport({
                                edgeSize: 0
                            }));
                        });
                    },
                });
                loadedMaps.push(map);
                toLoadCount += 1;

                // gaps between backgrounds, attach the TiledEdge component to draw an extra edge
                Utils.forEach(map.backgrounds, function (background, i, l, breakLoop) {
                    background.attach(new TiledEdge({}));
                });
            },
            mapExists: function (index) {
                var mapName = index.x + '_' + index.y;
                var mapNameInverted = index.x + '_' + (-index.y); // inverted y axis so we don't have to add - in the asset names
                return Utils.hasAsset('json', mapManager.path + mapNameInverted);
            },
            hasLoadedMap: function (index) {
                var mapName = index.x + '_' + index.y;
                // check if not already loaded this before
                var hasLoaded = false;
                Utils.forEach(loadedMaps, function (loadedMap, i, l, breakLoop) {
                    if (loadedMap.name === mapName) {
                        hasLoaded = loadedMap;
                        breakLoop();
                    }
                });
                return hasLoaded;
            }
        };
        var viewportController = new ViewportScroller({
            worldScroller: worldScroller,
            mapManager: mapManager,
            hashGrid: hashGrid
        });
        /**
         * Ensures only vertical scrolling
         */
        var setVerticalScrolling = function () {
            // disable horizontal scrolling
            worldScroller.scrollX.attach({
                name: 'deactivator',
                update: function () {
                    worldScroller.scrollX.setActive(false);
                }
            });
        };
        var activateButtons = function () {
            // activate buttons after fade has ended
            topBar.setActive(false);
            worldScroller.setActive(false);

            Bento.objects.attach(new EventListener({
                name: 'worldSelectReady',
                onEvent: function () {
                    topBar.setActive(true);
                    worldScroller.setActive(true);
                    var shownTutorial = showTutorial();
                    if (shownTutorial || isOpeningInvite || Globals.roomCodeInvite) {
                        // don't show offers during room code invite
                        return;
                    }
                }
            }));
        };
        var showTutorial = function () {
            var highlight;
            var tapHand;
            var tutorialChecker;
            var exempt = SaveManager.load('missionsExemptFromEnergy');
            var unlocked = SaveManager.load('unlockedLevels');
            var buttonSensei;
            var putHighlightOnMission = function (data) {
                Bento.objects.getByFamily('missionButtons', function (buttons) {
                    Utils.forEach(buttons, function (button, i, l, breakLoop) {
                        if (button.mission == data.mission) {
                            // pause so only button can be pressed
                            button.setActive(true);
                            Bento.objects.pause(Globals.PauseLevels.TUTORIAL);
                            button.updateWhenPaused = Globals.PauseLevels.TUTORIAL;
                            // highlight button
                            highlight = new Highlight({
                                z: Globals.WorldSelectLayers.TUTORIAL,
                                position: button.position.clone()
                            });
                            tapHand = new TapHand({
                                z: Globals.WorldSelectLayers.TUTORIAL,
                                position: button.position.add(new Vector2(8, 8)),
                                rotation: Math.PI * 3 / 4
                            });
                            Bento.objects.attach(highlight);
                            Bento.objects.attach(tapHand);

                            viewportController.forceTo(new Vector2(
                                button.position.x - viewport.width / 2,
                                button.position.y - viewport.height / 2
                            ));
                        }
                    });
                });
            };
            var showSenseiMessage = function (data) {
                var tweenIn = function () {
                    new Tween({
                        from: buttonSensei.position.x - 64,
                        to: buttonSensei.position.x,
                        in: 30,
                        ease: 'elastic',
                        decay: 5,
                        oscillations: 1,
                        onUpdate: function (v, t) {
                            buttonSensei.position.x = v;
                        }
                    });
                };
                if (buttonSensei) {
                    return;
                }
                buttonSensei = new ClickButton({
                    z: Globals.WorldSelectLayers.GUI - 1,
                    name: 'buttonSensei',
                    sfx: 'sfx_button',
                    imageName: 'gui/buttons/sensei',
                    frameCountX: 2,
                    frameCountY: 1,
                    float: true,
                    boundingBox: new Rectangle(-20, -20, 40, 40),
                    components: [
                        new Anchor({
                            screenPosition: new Vector2(0, 0),
                            offset: new Vector2(20, 120),
                            onComplete: function () {
                                // tween in
                                tweenIn();
                            }
                        }),
                        new Badge({
                            scale: 0.75,
                            pulsate: true,
                            exclamation: true,
                            imageName: 'gui2/misc/badge',
                            position: new Vector2(12, -8),
                        })
                    ],
                    onClick: function () {
                        // explain
                        var senseiPopup = new SenseiPopup({
                            darken: true,
                            modal: true,
                            onNoMoreText: function () {
                                this.dismiss();
                                SaveManager.save('tutorialQuickLootInfo', 1);
                            },
                            text: data.text
                        });
                        buttonSensei.removeSelf();
                        Bento.objects.attach(senseiPopup);
                    }
                });
                // tween in on start of worldmap
                buttonSensei.attach(new EventListener({
                    name: 'loadOutClosed',
                    onEvent: function () {
                        // tween in
                        tweenIn();
                    }
                }));
                // an event listener that will remove the button
                if (data.removeEvent) {
                    buttonSensei.attach(new EventListener({
                        name: 'tutorialQuickLoot',
                        onEvent: function () {
                            buttonSensei.removeSelf();
                        }
                    }));
                }
                Bento.objects.attach(buttonSensei);
            };
            if (isOpeningInvite || Globals.roomCodeInvite) {
                // don't show tutorial during room code invite
                return false;
            }

            // only show sensei message if quick loot was not done yet
            if (SaveManager.load('tutorialQuickLoot') === 0 && SaveManager.load('firstTime3Stars') === true && SaveManager.load('tutorialQuickLootInfo') === 0) {
                showSenseiMessage({
                    text: Localization.getText('tutorial')['quickLootWorld']
                });
            }

            // attach a taphand to giftbutton for the first time
            /*if (SaveManager.load('tutorialFreeGift') < 1 && giftButton.isAdded) {
                var giftTapHand = new TapHand({
                    z: Globals.WorldSelectLayers.TUTORIAL,
                    position: giftButton.getWorldPosition(),
                    rotation: -Math.PI * 3 / 4
                });
                var giftHighlight = new Highlight({
                    z: Globals.WorldSelectLayers.TUTORIAL,
                    position: giftButton.getWorldPosition()
                });
                // remove self when tutorial is over
                giftTapHand.attach(new EventListener({
                    name: 'giftClosed',
                    onEvent: function () {
                        giftTapHand.removeSelf();
                        giftHighlight.removeSelf();
                        giftButton.updateWhenPaused = 0;
                        Bento.objects.resume();
                    }
                }));
                // pause so only button can be pressed
                Bento.objects.pause(Globals.PauseLevels.TUTORIAL);
                giftTapHand.updateWhenPaused = Globals.PauseLevels.TUTORIAL;
                giftButton.updateWhenPaused = Globals.PauseLevels.TUTORIAL;
                Bento.objects.attach(giftHighlight);
                Bento.objects.attach(giftTapHand);
            }

            // tutorial quest (non-intrusive, just showing a taphand)
            var tutorialQuest = SaveManager.load('tutorialQuest');
            if (tutorialQuest === 2) {
                // find the questbutton and attach a taphand
                var tapHandQuest = new TapHand({
                    position: new Vector2(4, 4),
                    rotation: Math.PI * 3 / 4
                });
                tapHandQuest.attach(new EventListener({
                    // fired by gui/dailyquest/questframe.js
                    name: 'questClaimed',
                    onEvent: function () {
                        tapHandQuest.removeSelf();
                    }
                }));
                questButton.attach(tapHandQuest);
            }

            // non-intrusive daily shop tutorial: after mission 5, player should go to daily shop
            if (unlocked.length > 5 && SaveManager.load('tutorialDailyShopItems') === 0) {
                var tapHandShop = new TapHand({
                    position: new Vector2(0, -6),
                    rotation: 0
                });
                tapHandShop.attach(new EventListener({
                    // fired by gui/menus/shop
                    name: 'tutorialDailyShopItems',
                    onEvent: function () {
                        tapHandShop.removeSelf();
                    }
                }));
                bottomBar.getComponent('centeredList', function (centeredList) {
                    centeredList.getComponent('shopButton', function (shopButton) {
                        shopButton.attach(tapHandShop);
                    });
                });
            }*/

            if (unlocked.length === 2) {
                // place hand on mission 2 (non-intrusively)
                Bento.objects.getByFamily('missionButtons', function (buttons) {
                    Utils.forEach(buttons, function (button, i, l, breakLoop) {
                        if (button.mission == 2) {
                            var hand = new TapHand({
                                z: Globals.WorldSelectLayers.LEVELS + 1,
                                position: button.position.add(new Vector2(0, 8)),
                                rotation: Math.PI * 7 / 8
                            });
                            Bento.objects.attach(hand);
                        }
                    });
                });

            }

            if (unlocked.length === 1) {
                putHighlightOnMission({
                    mission: 1
                });

                return true;
            } else if (SaveManager.load('tutorialWorldSelect') > 0) {
                // show death tutorial, only appears after the worldselect tutorial
                if (SaveManager.load('tutorial1stTimeDeath') === 1) {
                    Bento.objects.attach(new SenseiPopup({
                        darken: true,
                        modal: true,
                        onNoMoreText: function () {
                            this.dismiss();
                        },
                        text: Localization.getText('tutorial')['1stTimeDeath']
                    }));
                    SaveManager.save('tutorial1stTimeDeath', 2);
                } else if (SaveManager.load('tutorial1stTimeDeath') === 3) {
                    // TEMP: remove this message until multiplayer is restored
                    /*Bento.objects.attach(new SenseiPopup({
                        darken: true,
                        modal: true,
                        onNoMoreText: function () {
                            this.dismiss();
                        },
                        text: Localization.getText('tutorial')['2ndTimeDeath']
                    }));*/
                    SaveManager.save('tutorial1stTimeDeath', 4);
                }
                return false;
            } else {
                // if player ever died during the tutorial we do not mark this as the 1st death
                SaveManager.save('tutorial1stTimeDeath', 0);

                // clear tutorial after mission 4
                if (unlocked.length === 4) {
                    SaveManager.save('tutorialWorldSelect', 1);
                    // world select tutorial complete
                    Analytics.event('Tutorial', 'complete');
                    // Standard Firebase event
                    FireBase.logEvent(FireBase.Events.TUTORIAL_COMPLETE);
                    // tenjin
                    Tenjin.sendEvent('tutorialEnd');
                }
            }

            return false;
        };
        /**
         * Online multiplayer mode
         */
        var addMultiPlayerButton = function () {
            var textOnline = new Text({
                position: new Vector2(0, 6),
                text: Localization.getText('onlineMulti').toUpperCase(),
                font: 'font',
                fontSize: 10,
                fontColor: '#ffffff',
                align: 'center',
                textBaseline: 'middle',
                maxWidth: 32,
                maxHeight: 10,
                components: []
            });
            var iconWifi = new Entity({
                name: 'iconWifi',
                position: new Vector2(0, -2),
                components: [
                    new Sprite({
                        imageName: 'gui/wifi-icon',
                        originRelative: new Vector2(0.5, 0.5)
                    })
                ]
            });
            var iconInfo = new Entity({
                name: 'icon',
                components: [
                    textOnline,
                    iconWifi
                ]
            });
            var sprite;
            var isToggled = false;
            var buttonOnline = new IconButton({
                z: Globals.WorldSelectLayers.GUI - 1,
                name: 'buttonOnline',
                sfx: 'sfx_button',
                icon: iconInfo,
                imageName: 'gui2/misc/buttonmini_small_secondary',
                frameCountX: 1,
                frameCountY: 3,
                float: true,
                updateWhenPaused: 0,
                onClick: function () {
                    isToggled = !isToggled;

                    sprite.setup({
                        imageName: 'gui2/misc/buttonmini_small_secondary',
                        frameCountX: 1,
                        frameCountY: 3,
                        animations: {
                            up: {
                                frames: [isToggled ? 1 : 0]
                            },
                            down: {
                                frames: [1]
                            },
                            inactive: {
                                frames: [2]
                            }
                        }
                    });
                    sprite.setAnimation('up');
                },
                components: [
                    new Anchor({
                        screenPosition: new Vector2(1, 1),
                        offset: new Vector2(-32, -20),
                    })
                ]
            });

            sprite = buttonOnline.getComponent('sprite');

            Bento.objects.attach(buttonOnline);
        };
        /**
         * Chest evolution
         */
        var chestEvolution = function () {
            // the classic chest is able to grow into the Fighter Chest, and Champion chest
            // however we will keep the "classic" id
            // We simply swap out the name and amounts
            var classicChestsOpened = SaveManager.load('classicChestsOpened');
            if (classicChestsOpened < 20) {
                Localization.setText('chest-classic', Localization.getText('chest-classic-original'));
                Globals.chests.classic.tokens = 3;
                Globals.chests.classic.enemytokens = 1;
            }
            if (classicChestsOpened >= 20) {
                Localization.setText('chest-classic', Localization.getText('chest-fighter'));
                Globals.chests.classic.tokens = 4;
                Globals.chests.classic.enemytokens = 2;
            }
            if (classicChestsOpened >= 40) {
                Localization.setText('chest-classic', Localization.getText('chest-champion'));
                Globals.chests.classic.tokens = 5;
                Globals.chests.classic.enemytokens = 3;
            }
        };
        var worldPadding = function () {
            // this bottom of the worldmap is visible on iphoneX
            var bottom = new Entity({
                name: 'bottom',
                position: new Vector2(0, 512),
                components: [
                    new Sprite({
                        imageName: 'worldselect/bottompadding',
                        originRelative: new Vector2(0, 0)
                    })
                ]
            });
            Bento.objects.attach(bottom);
        };
        if (Anchor.isWideScreen()) {
            worldPadding();
        }
        chestEvolution();
        Bento.objects.attach(new EventListener({
            name: 'classicChestsOpened',
            onEvent: function (data) {
                chestEvolution();
            }
        }));
        /**
         * Check for unresolved IAP
         */
        var unresolvedPurchases = Bento.saveState.load('unresolvedPurchases', []);
        if (unresolvedPurchases.length) {
            Bento.screens.show('screens/iap');
            return;
        }
        /**
         * Add random join
         */
        var addRandomJoinButton = function () {
            var iconWifi = new Entity({
                name: 'iconWifi',
                position: new Vector2(0, 0),
                components: [
                    new Sprite({
                        imageName: 'gui/wifi-icon',
                        originRelative: new Vector2(0.5, 0.5)
                    })
                ]
            });
            var textOnline = new Text({
                position: new Vector2(0, 11),
                text: Localization.getText('onlineJoin').toUpperCase(),
                font: 'font',
                fontSize: 10,
                fontColor: '#ffffff',
                align: 'center',
                textBaseline: 'middle',
                linebreaks: false,
                maxWidth: 32,
                maxHeight: 10
            });
            var iconInfo = new Entity({
                name: 'icon',
                components: [
                    textOnline,
                    iconWifi
                ]
            });
            var buttonRandom = new IconButton({
                z: Globals.WorldSelectLayers.GUI - 1,
                name: 'buttonRandom',
                sfx: 'sfx_button',
                icon: iconInfo,
                iconAlpha: 0.5,
                imageName: 'gui2/misc/button_smaller',
                frameCountX: 1,
                frameCountY: 3,
                float: true,
                sort: true,
                onClick: function () {
                    var randomJoinMenu = new OnlineMenu({
                        isJoining: true
                    });
                    Bento.objects.attach(randomJoinMenu);
                },
                onButtonDown: function () {
                    EventSystem.fire('preventScroll');
                },
                components: [
                    new Anchor({
                        screenPosition: new Vector2(1, 0),
                        offset: new Vector2(-20, Utils.isNativeIos() || Bento.isDev() ? 40 * 2 : 40),
                        onComplete: function () {}
                    })
                ]
            });
            Bento.objects.attach(buttonRandom);

            // reached worldselect while trying for reputation: penalize player
            if (SaveManager.load('wantsReputation')) {
                SaveManager.save('wantsReputation', false);
                if (!SaveManager.load('isTower')) {
                    SaveManager.add('reputationPoints', -1);
                }
            }
            // clamp reputation points
            var reputationPoints = SaveManager.load('reputationPoints');
            var ranks = Localization.getText('ranks');
            reputationPoints = Utils.clamp(0, reputationPoints, ranks.length * Globals.pointsPerRank);
            SaveManager.save('reputationPoints', reputationPoints);

            if (false && Bento.saveState.load('jailBreak', false)) {
                buttonRandom.setActive(false);
            }
        };
        addRandomJoinButton();

        /**
         * Add tower button
         */
        var addTowerChallengeButton = function () {
            var iconTower = new Entity({
                name: 'iconTower',
                position: new Vector2(0, 0),
                components: [
                    new Sprite({
                        imageName: 'gui2/towerchallenge/tower_icon',
                        originRelative: new Vector2(0.5, 0.82)
                    })
                ]
            });
            var textTower = new Text({
                position: new Vector2(0, 11),
                text: Localization.getText('towerTitle').toUpperCase(),
                font: 'font',
                fontSize: 12,
                fontColor: '#271a2d',
                align: 'center',
                textBaseline: 'middle',
                linebreaks: false,
                maxWidth: 40,
                maxHeight: 10
            });
            var iconInfo = new Entity({
                name: 'icon',
                components: [
                    textTower,
                    iconTower
                ]
            });
            var buttonTower = new IconButton({
                z: Globals.WorldSelectLayers.GUI - 1,
                name: 'buttonTower',
                sfx: 'sfx_button',
                icon: iconInfo,
                iconAlpha: 0.5,
                imageName: 'gui2/misc/button_small_tertiary',
                frameCountX: 1,
                frameCountY: 3,
                float: true,
                sort: true,
                onClick: function () {
                    var towerMenu = new TowerMenu({});
                    Bento.objects.attach(towerMenu);
                },
                onButtonDown: function () {
                    EventSystem.fire('preventScroll');
                },
                components: [
                    new Anchor({
                        screenPosition: new Vector2(0, 0),
                        offset: new Vector2(26, 60),
                        onComplete: function () {}
                    })
                ]
            });
            Bento.objects.attach(buttonTower);

            if (false && Bento.saveState.load('jailBreak', false)) {
                buttonTower.setActive(false);
            }
        };
        if (SaveManager.load('unlockedLevels').length > 5) {
            addTowerChallengeButton();
        }

        var addBeachEventButton = function () {
            var textEvent = new Text({
                position: new Vector2(-9, 16),
                text: Localization.getText('radEvent').toUpperCase(),
                font: 'font',
                fontSize: 30,
                fontColor: '#ffffff',
                align: 'center',
                textBaseline: 'middle',
                lineWidth: 1.5,
                strokeStyle: '#ad50d3',
                innerStroke: false,
                pixelStroke: true,
                linebreaks: false,
                maxWidth: 75,
                maxHeight: 9999
            });
            var buttonBeachEvent = new ClickButton({
                z: Globals.WorldSelectLayers.GUI - 1,
                name: 'buttonBeachEvent',
                sfx: 'sfx_button',
                imageName: 'gui2/beachevent/radeventbutton',
                frameCountX: 8,
                frameCountY: 7,
                padding: 1,
                toggled: false,
                float: true,
                boundingBox: new Rectangle(-52, 0, 103, 48),
                sort: true, // gives priority over mission buttons
                animations: {
                    'default': {
                        speed: 0.35,
                        frames: [0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,30,30,30,30,30,30,31,32,33,34,35,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,37,38,39,40,41,42,43,44,45,46,47,48,49,50]
                    }
                },
                onButtonDown: function () {
                    EventSystem.fire('preventScroll');
                    buttonBeachEvent.transform.y = 1;
                },
                onButtonUp: function () {
                    buttonBeachEvent.transform.y = 0;
                },
                onClick: function () {
                    var beachMenu = new BeachEventMenu({});
                    Bento.objects.attach(beachMenu);
                },
                components: [
                    textEvent,
                    new Anchor({
                        portrait: {
                            screenPosition: new Vector2(1, 1),
                            offset: new Vector2(-45, -33)
                        },
                        landscape: {
                            screenPosition: new Vector2(1, 1),
                            offset: new Vector2(-45, -33)
                        },
                        onComplete: function () {
                            if (Anchor.isPortrait() && Anchor.isWideScreen()) {
                                buttonBeachEvent.position.y += Anchor.getOffset().y;
                            }
                        }
                    })
                ]
            });
            Bento.objects.attach(buttonBeachEvent);

        };
        if (SaveManager.load('unlockedLevels').length > 5 && BeachEventMenu.isBeachEvent()) {
            addBeachEventButton();
        }


        /**
         * Add live broadcast button
         */
        var addLiveBroadcast = function (force) {
            var unlocked = SaveManager.load('unlockedLevels').length;
            if (unlocked.length <= 1) {
                // not during first tutorial
                return;
            }
            ReplayKit.isBroadcastAvailable(function (available) {
                if (force) {
                    // for development
                    available = true;
                }
                if (!available) {
                    return;
                }
                ReplayKit.isBroadcasting(function (broadcasting) {
                    // attach broadcast button
                    var iconSprite = new Sprite({
                        imageName: broadcasting ? 'gui2/misc/button_icon_stop' : 'gui2/misc/button_icon_record',
                        originRelative: new Vector2(0.5, 0.5)
                    });
                    var liveText = new Text({
                        position: new Vector2(0, 11),
                        text: Localization.getText('live'),
                        font: 'font',
                        fontSize: 10,
                        fontColor: '#ffffff',
                        align: 'center',
                        textBaseline: 'middle',
                        linebreaks: false,
                        maxWidth: 32,
                        maxHeight: 10
                    });
                    var icon = new Entity({
                        name: 'icon',
                        components: [
                            iconSprite,
                            liveText
                        ]
                    });
                    var buttonBroadcast = new IconButton({
                        z: Globals.WorldSelectLayers.GUI - 1,
                        name: 'buttonBroadcast',
                        sfx: 'sfx_button',
                        icon: icon,
                        iconAlpha: 0.5,
                        imageName: 'gui2/misc/button_smaller',
                        frameCountX: 1,
                        frameCountY: 3,
                        float: true,
                        sort: true,
                        onClick: function () {
                            buttonBroadcast.setActive(false);
                            if (!broadcasting) {
                                ReplayKit.startBroadcast(function () {
                                    buttonBroadcast.setActive(true);
                                    setIconSprite();
                                }, function () {
                                    buttonBroadcast.setActive(true);
                                    setIconSprite();
                                });
                            } else {
                                promptStop(function () {
                                    ReplayKit.stopBroadcast(function () {
                                        buttonBroadcast.setActive(true);
                                        setIconSprite();
                                    }, function () {
                                        buttonBroadcast.setActive(true);
                                        setIconSprite();
                                    });
                                });
                            }
                        },
                        onButtonDown: function () {
                            EventSystem.fire('preventScroll');
                        },
                        components: [
                            new Anchor({
                                screenPosition: new Vector2(1, 0),
                                offset: new Vector2(-20, 40),
                                onComplete: function () {}
                            }),
                            new EventListener({
                                name: 'updateRecordingStatus',
                                onEvent: function (data) {
                                    setIconSprite();
                                }
                            })
                        ]
                    });
                    var setIconSprite = function () {
                        ReplayKit.isBroadcasting(function (isBroadCasting) {
                            // update values
                            Globals.isBroadcasting = isBroadCasting;
                            broadcasting = isBroadCasting;
                            iconSprite.setup({
                                imageName: isBroadCasting ? 'gui2/misc/button_icon_stop' : 'gui2/misc/button_icon_record',
                                originRelative: new Vector2(0.5, 0.5)
                            });
                        });
                    };
                    var promptStop = function (onYes, onNo) {
                        var dialog = new Dialog({
                            name: 'stopBroadcast',
                            title: Localization.getText('stopTitle'),
                            text: Localization.getText('stopText'),
                            canClose: false,
                            type: 'yesno',
                            onYes: onYes,
                            onNo: onNo
                        });
                        Bento.objects.attach(dialog);
                    };
                    Bento.objects.attach(buttonBroadcast);
                    Globals.isBroadcasting = broadcasting;
                });
            });
        };
        if (Utils.isNativeIos()) {
            addLiveBroadcast();

            // stop any recording if needed
            if (Globals.isRecording) {
                ReplayKit.isRecording(function (isRecording) {
                    if (isRecording) {
                        ReplayKit.stop(function () {
                            EventSystem.fire('updateRecordingStatus');
                        });
                    }
                });
                // turn off recording
                Globals.isRecording = false;
            }
        } else if (Bento.isDev()) {
            addLiveBroadcast(true);
        }

        // revert to previously selected character after using backup
        if (SaveManager.load('calledBackup')) {
            SaveManager.save('calledBackup', false);
            CharacterManager.setSelected(SaveManager.load('originallySelectedCharacter'));
        }

        // infinite scrolling
        worldScroller.setScrollArea(new Rectangle(Infinity, Infinity, Infinity, Infinity));

        setVerticalScrolling();

        // load worldmap position before loading maps
        lastKnownViewportPosition = viewportController.loadPosition();
        mapManager.loadMaps(true);

        var topBar = new Topbar({});

        // remove any timed effects from consumables
        EventSystem.fire('destroyTimedEffects');

        // event when player removes roadblock
        Bento.objects.attach(new EventListener({
            name: 'roadBlockRemoved',
            onEvent: function () {
                // scroll to recently unlocked
                // this will also re-initialize the button and reload the maps
                scrollToUnlocked(true);
            }
        }));

        Bento.objects.attach(worldScroller);
        Bento.objects.attach(mapManager);
        Bento.objects.attach(topBar);
        if (!Globals.roomCodeInvite) {
            Bento.objects.attach(new BottomBar({
                target: settings ? settings.target : null
            }));
            activateButtons();
        }

        // Bento.objects.attach(currencyCounter);

        if (!Bento.audio.isPlayingMusic('bgm_worldmap')) {
            Bento.audio.stopAllMusic();
            if (Utils.isNativeAndroid()) {
                // QUICKFIX: for some reason android sometimes skips the music
                new Tween({
                    ignoreGameSpeed: true,
                    in: 3,
                    onComplete: function () {
                        Bento.audio.setVolume(1, 'bgm_worldmap');
                        Bento.audio.playMusic('bgm_worldmap');
                    }
                });
            } else {
                Bento.audio.playMusic('bgm_worldmap');
            }
        }

        // we now disconnect during the worldselect screen
        if (MatchManager.isOnline) {
            MatchManager.disconnect();
        }

        // invite was opened: re-open the worldselect with the invite
        Bento.objects.attach(new EventListener({
            name: 'handleUrl',
            onEvent: function () {
                // cancel any pause
                Bento.objects.resume();
                // close any connection
                MatchManager.disconnect();
                // re-open
                Bento.screens.show('screens/worldselect');
            }
        }));

        // cheat for devs: restore energy automatically
        if (Bento.saveState.load('unlimitedEnergy', false)) {
            CharacterManager.restoreAllEnergy();
        }

        // count how many times worldmap is opened
        SaveManager.add('worldMapForOffer', 1);

        if (Bento.isDev()) {
            // Bento.objects.attach(new Zoom({
            //     zoom: 0.5
            // }));

        }

        // warn user if offline
        if (!TimeManager.hasWarned && !TimeManager.online) {
            // warn only once per session
            TimeManager.hasWarned = true;
            // EventSystem.fire('Notification', {
            //     textId: 'warningOffline'
            // });
        }
    };

    return new Screen({
        onShow: onShow,
        onHide: function (data) {
            // save to cloud every time user leaves this screen
            if (data.next === 'screens/main') {
                SaveManager.saveToCloud();
            }
            EventSystem.fire('saveViewportPosition');
            // unload all worldmapImages
            Utils.forEach(worldmapImages, function (image, mapName, l, breakLoop) {
                if (image.dispose) {
                    image.dispose();
                }
            });
            worldmapImages = {};
        }
    });
});