/**
 * Standard boss behaviors, add what you need
 * jump back after getting up
 * block when grabbed
 * @moduleName BossEnemyStates
 */
bento.define('states/bossenemy', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'components/grabable',
    'states/standardenemy'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween,
    Grabable,
    StandardEnemyStates
) {
    'use strict';
    /**
     * Describe your settings object parameters
     * @param {Object} settings
     */
    var BossEnemyStates = {
        grabbed: {
            start: function (data, transitionData) {
                // force punchable to block any attack
                data.walkingEnemyBehavior.forceBlock = true;
                Grabable.states.grabbed.start(data, transitionData);
            },
            update: StandardEnemyStates.grabbed.update,
            end: function (data) {
                data.walkingEnemyBehavior.forceBlock = false;
                if (data.stateMachine.stateName !== 'grabbedThrow') {
                    // amplify velocity so the boss flies outside of the reach of the player after blocking)
                    Utils.timeout(1, function () {
                        data.movable.velocity.x *= 1.5;
                    });
                }
                Grabable.states.grabbed.end(data);
            }
        },
        grabbedThrow: {
            start: function (data, transitionData) {
                var grabber = data.lastGrabbed;
                var player = Utils.getPlayerById(grabber);
                if (data.stateMachine.hasState('attack')) {
                    data.setState('attack', {
                        target: player.name
                    }, true);
                } else {
                    data.setState('punch', {
                        target: player.name
                    }, true);
                }

                data.movable.velocityZ = Math.min(data.movable.velocityZ, -1) - 1;
                // add any extra velocity
                if (transitionData.velocity) {
                    data.movable.velocity.x += transitionData.velocity.x * 0.5;
                    data.movable.velocity.y += transitionData.velocity.y * 0.5;
                    data.movable.velocityZ = transitionData.velocity.z;
                }
            }
        }
    };

    return BossEnemyStates;
});