bento.define('utils', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/utils',
    'bento/tween',
    'bento/eventsystem',
    'modules/gamecenter',
    'modules/googleplay',
    'modules/appavailability',
    'modules/ktplay'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    Utils,
    Tween,
    EventSystem,
    GameCenter,
    GooglePlay,
    AppAvailability,
    KTPlayModule
) {
    'use strict';
    var utils = {

        // global modules
        GameCenter: GameCenter,
        GooglePlay: GooglePlay,
        // new version on the App Store
        newVersion: null,
        /**
         * List of in app purchase ids, these ids must exist in the App Store or Google Play
         */
        IAP: [
            'beatstreetgempack1',
            'beatstreetgempack2',
            'beatstreetgempack3',
            'beatstreetgempack4',
            'beatstreetgempack5',
            'beatstreetgempack6',
            'beatstreetofferpack1',
            'beatstreetofferpack2',
            'beatstreetofferpack3',
            'beatstreetofferpack4'
        ],
        rewardsIAP: {
            'beatstreetgempack1': 160,
            'beatstreetgempack2': 440,
            'beatstreetgempack3': 960,
            'beatstreetgempack4': 2600,
            'beatstreetgempack5': 5600,
            'beatstreetgempack6': 12000
        },
        IapToOffer: { // map generic iap name to the actual and vice versa (note: never rename these)
            'beatstreetofferpack1': 'starter',
            'beatstreetofferpack2': 'weapon',
            'beatstreetofferpack3': 'powerup',
            'beatstreetofferpack4': 'gem',
            'starter': 'beatstreetofferpack1',
            'weapon': 'beatstreetofferpack2',
            'powerup': 'beatstreetofferpack3',
            'gem': 'beatstreetofferpack4'
        },
        timeout: function (time, callback, updateWhenPaused) {
            var tween = new Tween({
                from: 0,
                to: 1,
                'in': time,
                ease: 'linear',
                updateWhenPaused: updateWhenPaused,
                onComplete: callback
            });
            return tween;
        },
        /**
         * Converts any object to Vector2
         */
        toVector2: function (obj) {
            return new Vector2(obj.x, obj.y);
        },
        /**
         * Converts any object to Rectangle
         */
        toRectangle: function (obj) {
            return new Rectangle(obj.x, obj.y, obj.width, obj.height);
        },
        isMobile: function () {
            var check = false;
            (function (a) {
                if (/(android|bb\d+|meego|android|ipad|playbook|silk).+mobile|avantgo|bada\/|blackberry|blazer|compal|elaine|fennec|hiptop|iemobile|ip(hone|od)|iris|kindle|lge |maemo|midp|mmp|mobile.+firefox|netfront|opera m(ob|in)i|palm( os)?|phone|p(ixi|re)\/|plucker|pocket|psp|series(4|6)0|symbian|treo|up\.(browser|link)|vodafone|wap|windows (ce|phone)|xda|xiino/i.test(a) || /1207|6310|6590|3gso|4thp|50[1-6]i|770s|802s|a wa|abac|ac(er|oo|s\-)|ai(ko|rn)|al(av|ca|co)|amoi|an(ex|ny|yw)|aptu|ar(ch|go)|as(te|us)|attw|au(di|\-m|r |s )|avan|be(ck|ll|nq)|bi(lb|rd)|bl(ac|az)|br(e|v)w|bumb|bw\-(n|u)|c55\/|capi|ccwa|cdm\-|cell|chtm|cldc|cmd\-|co(mp|nd)|craw|da(it|ll|ng)|dbte|dc\-s|devi|dica|dmob|do(c|p)o|ds(12|\-d)|el(49|ai)|em(l2|ul)|er(ic|k0)|esl8|ez([4-7]0|os|wa|ze)|fetc|fly(\-|_)|g1 u|g560|gene|gf\-5|g\-mo|go(\.w|od)|gr(ad|un)|haie|hcit|hd\-(m|p|t)|hei\-|hi(pt|ta)|hp( i|ip)|hs\-c|ht(c(\-| |_|a|g|p|s|t)|tp)|hu(aw|tc)|i\-(20|go|ma)|i230|iac( |\-|\/)|ibro|idea|ig01|ikom|im1k|inno|ipaq|iris|ja(t|v)a|jbro|jemu|jigs|kddi|keji|kgt( |\/)|klon|kpt |kwc\-|kyo(c|k)|le(no|xi)|lg( g|\/(k|l|u)|50|54|\-[a-w])|libw|lynx|m1\-w|m3ga|m50\/|ma(te|ui|xo)|mc(01|21|ca)|m\-cr|me(rc|ri)|mi(o8|oa|ts)|mmef|mo(01|02|bi|de|do|t(\-| |o|v)|zz)|mt(50|p1|v )|mwbp|mywa|n10[0-2]|n20[2-3]|n30(0|2)|n50(0|2|5)|n7(0(0|1)|10)|ne((c|m)\-|on|tf|wf|wg|wt)|nok(6|i)|nzph|o2im|op(ti|wv)|oran|owg1|p800|pan(a|d|t)|pdxg|pg(13|\-([1-8]|c))|phil|pire|pl(ay|uc)|pn\-2|po(ck|rt|se)|prox|psio|pt\-g|qa\-a|qc(07|12|21|32|60|\-[2-7]|i\-)|qtek|r380|r600|raks|rim9|ro(ve|zo)|s55\/|sa(ge|ma|mm|ms|ny|va)|sc(01|h\-|oo|p\-)|sdk\/|se(c(\-|0|1)|47|mc|nd|ri)|sgh\-|shar|sie(\-|m)|sk\-0|sl(45|id)|sm(al|ar|b3|it|t5)|so(ft|ny)|sp(01|h\-|v\-|v )|sy(01|mb)|t2(18|50)|t6(00|10|18)|ta(gt|lk)|tcl\-|tdg\-|tel(i|m)|tim\-|t\-mo|to(pl|sh)|ts(70|m\-|m3|m5)|tx\-9|up(\.b|g1|si)|utst|v400|v750|veri|vi(rg|te)|vk(40|5[0-3]|\-v)|vm40|voda|vulc|vx(52|53|60|61|70|80|81|83|85|98)|w3c(\-| )|webc|whit|wi(g |nc|nw)|wmlb|wonu|x700|yas\-|your|zeto|zte\-/i.test(a.substr(0, 4))) {
                    check = true;
                }
            })(navigator.userAgent || navigator.vendor || window.opera);
            return check;
        },
        removeFromArray: function (array, obj) {
            var index = array.indexOf(obj);
            if (index >= 0) {
                array.splice(index, 1);
                return true;
            } else {
                return false;
            }
        },
        /**
         * Copy an array
         */
        copyArray: function (array) {
            var i, l;
            var newArray = [];
            for (i = 0, l = array.length; i < l; ++i) {
                newArray.push(array[i]);
            }
            return newArray;
        },
        /**
         * Shows an input prompt
         */
        prompt: function (message, onComplete) {
            var res;
            if (Utils.isCocoonJS() && window.Cocoon) {
                window.Cocoon.Dialog.prompt({
                    message: message
                }, {
                    success: function (text) {
                        onComplete(text);
                    }
                });
            } else {
                res = window.prompt(message);
                onComplete(res);
            }
        },
        hasAsset: function (type, name) {
            // todo: move this function to Bento.assets
            var assets = Bento.assets.getAssets();

            // fail when type doesnt exist
            if (!assets[type]) {
                console.error('Asset type ' + type + " doesn't exist.");
                return false;
            }

            // check if asset exist
            if (assets[type][name]) {
                return true;
            }
            return false;
        },
        getHashParameter: function (key, fallback, url) {
            var match;
            var result;
            url = url || window.location.hash;

            match = RegExp('[#&]' + key + '=([^&]*)').exec(url);
            result = match && decodeURIComponent(match[1].replace(/\+/g, ' '));

            if (result !== null) {
                return result;
            } else if (Utils.isDefined(fallback)) {
                return fallback;
            } else {
                return null;
            }
        },
        /**
         * Merge any tiled properties into the settings object, so they can be accessed as if they were normal settings
         */
        collapseTiledSettings: function (settings) {
            var tiled = settings.tiled || {};
            var tileProperties = tiled.tileProperties || {};
            var objectProperties = tiled.objectProperties || {};

            var key;

            for (key in tileProperties) {
                if (tileProperties.hasOwnProperty(key)) {
                    settings[key] = tileProperties[key];
                }
            }

            for (key in objectProperties) {
                if (objectProperties.hasOwnProperty(key)) {
                    settings[key] = objectProperties[key];
                }
            }

            return settings;
        },
        //http://javascript.about.com/od/problemsolving/a/modulobug.htm
        modulo: function (b, n) {
            return ((b % n) + n) % n;
        },
        /**
         * Split string into Json object
         * String in the form of "key:value,key2:value2"
         * Tries to parse "value" into Number if possible
         */
        nameToJson: function (name, dontParse) {
            if (!name) {
                return {};
            }
            var nameProperties = name.split(',');
            var output = {};
            Utils.forEach(nameProperties, function (prop) {
                var values = prop.split(':');
                var key = values[0];
                var value;
                // if (values.length < 2) {
                //     Utils.log('Warning: cannot parse nameproperties ' + name);
                //     return;
                // }
                if (dontParse) {
                    value = values[1];
                } else {
                    // try parsing as number
                    value = parseFloat(values[1]);
                }
                if (isNaN(value)) {
                    // probably a string
                    value = values[1];
                }

                output[key] = value;
            });
            return output;
        },
        /**
         * Global share function, uses Cocoon.Share
         * 'text' defaults to "shareText" from Localization and inserts storelink
         * Image base64 is optional
         */
        share: function (base64, text, url, onComplete) {
            var shareObj = {
                message: text
            };
            if (!window.Cocoon) {
                window.alert('Sharing only available in App version');
                return;
            }

            if (base64) {
                shareObj.image = base64;
            }

            if (url) {
                shareObj.url = url;
            }

            window.Cocoon.Share.share(shareObj, function (activity, completed, error) {
                // success
                if (onComplete) {
                    onComplete(activity, completed, error);
                }
            });
        },
        /**
         * Retrieves the store link, detects device automatically
         */
        getStoreLink: function (device) {
            var links = {
                ios: 'https://itunes.apple.com/us/app/beat-street/id1209083304',
                android: 'https://play.google.com/store/apps/details?id=com.luckykat.beatstreet',
                web: 'http://beatstreetgame.com/',
                smart: 'http://smarturl.it/beatstreet'
            };
            var link = links.web;
            if (!device) {
                // try to determine automatically
                if (Utils.isNativeIos()) {
                    link = links['ios'];
                } else if (Utils.isNativeAndroid()) {
                    link = links['android'];
                }
            } else if (links[device]) {
                link = links[device];
            }
            return link;
        },
        /**
         * checks to see if we can log in to ktplay in the current state, and if we can, does it
         */
        KTPlayAutoLogin: function (success, failure) {
            KTPlayModule.getLoginStatus(function (isLoggedIn) {
                if (!isLoggedIn) {
                    var playerID = Bento.saveState.load('gameServiceID', null);
                    if (playerID === null) {
                        failure('playerID is NULL');
                        return;
                    }
                    if ((Utils.isNativeIos() && GameCenter.isLoggedIn()) || (Utils.isNativeAndroid() && GooglePlay.isLoggedIn())) {
                        KTPlayModule.customLogin(playerID, success, failure);
                        if (!Bento.saveState.load('didSendKTPlayId', false)) {
                            Bento.saveState.save('didSendKTPlayId', true);
                            EventSystem.fire('Analytics', {
                                category: 'KTPlayId',
                                action: 'id',
                                label: playerID
                            });
                        }
                        if (!Bento.saveState.load('didSendKTUserId', false) && Bento.saveState.load('sentKTUserID', null)) {
                            Bento.saveState.save('didSendKTUserId', true);
                            EventSystem.fire('Analytics', {
                                category: 'KTPlayId',
                                action: 'userId',
                                label: Bento.saveState.load('sentKTUserID', null)
                            });
                        }
                    } else {
                        failure("Auto-login not available as Game Center or Google Play arent logged in");
                    }
                } else {
                    console.log("Auto-login not available as we're already logged in");
                }
            });
        },
        /**
         * Try to change the local player name to the current player ID
         */
        changeLocalNameToKTNickName: function () {
            KTPlayModule.getLoginStatus(function (isLoggedIn) {
                if (isLoggedIn) {
                    KTPlayModule.getCurrentUser(function (user) {
                        if (Utils.isDefined(user.nickname)) {
                            // if the ktplay nickname isn't set (e.g. First time login), then set it to the Google Play/Game Center nickname
                            if (user.autoGeneratedNickName || user.nickname.search('ID') === 0) {
                                // Set nickname to local player name if one is available OR leave the default KT name (That should never happen but it's a precaution)
                                var newNickname = Bento.saveState.load('localPlayerName', user.nickname);
                                KTPlayModule.updateUserProfile({
                                    nickname: newNickname,
                                    avatarPath: '',
                                    gender: 0
                                }, function () {
                                    console.log('Username changed successfully!');
                                }, function (error) {
                                    console.log(error);
                                });
                                // Set the output Username to whatever we just set
                                user.nickname = newNickname;
                            }

                            // set the local player name to the current ktplay nickname
                            Bento.saveState.save('localPlayerName', user.nickname);
                            Bento.saveState.save('ktPlayUserName', user.nickname);
                        } else {
                            console.log('User name is not defined!');
                        }
                    }, function (error) {
                        console.log(error);
                    });
                }
            });
        },
        /**
         * Divide an amount to multiple entries
         * @param {Number} - amount to divide
         * @param {Array} - weights
         */
        divideAndRandomizeRounding: function (amount, weights) {
            if (weights.length < 2) {
                // there is nothing to divide
                return [amount];
            }

            var calculateOdds = function () {
                var total = 0;
                var odds = [];
                Utils.forEach(weights, function (weight) {
                    total += weight;
                });
                Utils.forEach(weights, function (weight) {
                    if (total !== 0) {
                        odds.push(weight / total);
                    } else {
                        // divide by 0: assume equal weights
                        odds.push(1 / weights.length);
                    }
                });
                return odds;
            };
            var division = calculateOdds(); // division of odds, sum should be 1
            var debt = 0;
            var amounts = [];
            Utils.forEach(division, function (fraction) {
                amounts.push(fraction * amount);
            });

            // randomize rounding up or down
            Utils.forEach(amounts, function (currentAmount, i) {
                var method = Utils.pickRandom(Math.floor, Math.ceil);
                var newAmount = method(currentAmount);
                debt += currentAmount - newAmount;
                amounts[i] = newAmount;
            });

            var tries = 0;

            // divide the debt
            debt = Math.round(debt);
            while (debt !== 0) {
                // loop through array randomly, so that each entry has a fair chance of resolving debt
                var indices = [];
                var i;
                for (i = 0; i < amounts.length; ++i) {
                    indices.push(i);
                }
                Utils.shuffle(indices);
                // loop through
                for (i = 0; i < indices.length; ++i) {
                    var index = indices[i];
                    var currentAmount = amounts[index];

                    // can this amount pay off?
                    if (debt < 0 && currentAmount > 0) {
                        currentAmount -= 1;
                        debt += 1;
                        amounts[index] = currentAmount;
                    } else if (debt > 0) {
                        currentAmount += 1;
                        debt -= 1;
                        amounts[index] = currentAmount;
                    }

                    if (debt === 0) {
                        break;
                    }
                }

                tries += 1;
                if (tries > 1000) {
                    Utils.log('WARNING: could not resolve');
                    break;
                }
            }
            return amounts;
        },
        /**
         * Fisher yates shuffle
         * @param  {Array} arr      The array that needs to be shuffled
         * @param  {Boolean} clone  Whether a clone of the array should be made (true) or if the array should be shuffled in place (false)
         * @return {[type]}         The shuffled array
         */
        shuffle: function (arr, clone) {
            clone = clone || false;

            var ii;
            var idx;
            var temp;

            if (clone) {
                arr = [].concat(arr);
            }

            for (ii = arr.length - 1; ii >= 0; ii--) {
                idx = Math.floor(Math.random() * (ii + 1));
                temp = arr[ii];
                arr[ii] = arr[idx];
                arr[idx] = temp;
            }

            return arr;
        },
        /*************************
         * Game specific functions
         *************************/

        /*
         * Retrieves player by their playerId
         */
        getPlayerById: function (id, callback) {
            var players = Bento.objects.getByFamily('player');
            var current;
            Utils.forEach(players, function (player, i, l, breakLoop) {
                if (player.playerId === id) {
                    current = player;
                    if (callback) {
                        callback(current);
                    }
                    breakLoop();
                }
            });
            return current;
        },
        /**
         * Get all possible notifications. Has to be provided using this function
         * because Localization is not initialized right at the start
         * @param  {Object} prms - Object containing optional parameters
         * @param  {Number} prms.time - Time in milliseconds until next notification. Defaults to 24 hours.
         * @return {Object} Object containing all possible notifications
         */
        getNotifications: function (prms) {
            prms = prms || {};
            var time = prms.time || 24 * 60 * 60 * 1000;
            var Localization = prms.Localization;
            if (!Localization) {
                Utils.log('ERROR: pass Localization as parameter');
                return {};
            }

            return {
                dailyGift: {
                    title: Localization.getText('notifications')['dailyGiftTitle'], // ios title text
                    message: Utils.isNativeIos() ?
                        Localization.getText('notifications')['dailyGiftBody'] : Localization.getText('notifications')['dailyGiftTitle'], // ios body text, but android title text
                    soundEnabled: true,
                    badgeNumber: 1,
                    userData: {},
                    contentBody: Utils.isNativeIos() ? "" : Localization.getText('notifications')['dailyGiftBody'], // android body text
                    contentTitle: "",
                    date: new Date().valueOf() + time, // notify again in X hours
                    icon: 'notification_icon'
                },
                energy: {
                    title: Localization.getText('notifications')['energyTitle'], // ios title text
                    message: Utils.isNativeIos() ?
                        Localization.getText('notifications')['energyBody'] : Localization.getText('notifications')['energyTitle'], // ios body text, but android title text
                    soundEnabled: true,
                    badgeNumber: 1,
                    userData: {},
                    contentBody: Utils.isNativeIos() ? "" : Localization.getText('notifications')['energyBody'], // android body text
                    contentTitle: "",
                    date: new Date().valueOf() + time, // notify again in X hours
                    icon: 'notification_icon'
                },
                dailyQuest: {
                    title: Localization.getText('notifications')['questTitle'], // ios title text
                    message: Utils.isNativeIos() ?
                        Localization.getText('notifications')['questBody'] : Localization.getText('notifications')['questTitle'], // ios body text, but android title text
                    soundEnabled: true,
                    badgeNumber: 1,
                    userData: {},
                    contentBody: Utils.isNativeIos() ? "" : Localization.getText('notifications')['questBody'], // android body text
                    contentTitle: "",
                    date: new Date().valueOf() + time, // notify again in X hours
                    icon: 'notification_icon'
                },
                inactive: {
                    title: Localization.getText('notifications')['inactiveTitle'], // ios title text
                    message: Utils.isNativeIos() ?
                        Localization.getText('notifications')['inactiveBody'] : Localization.getText('notifications')['inactiveTitle'], // ios body text, but android title text
                    soundEnabled: true,
                    badgeNumber: 1,
                    userData: {},
                    contentBody: Utils.isNativeIos() ? "" : Localization.getText('notifications')['inactiveBody'], // android body text
                    contentTitle: "",
                    date: new Date().valueOf() + time, // notify again in X hours
                    icon: 'notification_icon'
                }
            };
        },
        //fill a rectangle with a (repeating) picture. The picture can be offset
        fillRect: function (renderer, image, x, y, width, height, offsetX, offsetY) {
            var drawWidth;
            var drawHeight;

            var startWidth = width;
            var startX = x;
            var startOffsetX = Utils.modulo(offsetX, image.width);

            offsetX = Utils.modulo(offsetX, image.width);
            offsetY = Utils.modulo(offsetY, image.height);
            while (height > 0) {
                drawHeight = Math.min(height, image.height - offsetY);
                while (width > 0) {
                    drawWidth = Math.min(width, image.width - offsetX);
                    renderer.drawImage(image, offsetX, offsetY, drawWidth, drawHeight, x, y, drawWidth, drawHeight);

                    offsetX = 0;
                    x += drawWidth;
                    width -= drawWidth;
                }
                //reset width properties for the next row:
                width = startWidth;
                x = startX;
                offsetX = startOffsetX;

                offsetY = 0;
                y += drawHeight;
                height -= drawHeight;
            }
        },
        /**
         * Checks if url can be opened, always true for android
         * Uses https://github.com/ohh2ahh/AppAvailability
         */
        canOpenUrl: function (name) {
            if (Utils.isNativeIos()) {
                return AppAvailability[name] || false;
            } else {
                return true;
            }
        },
        /**
         * Get local player, returns the first result
         */
        getLocalPlayer: function () {
            var players = Bento.objects.getByFamily('player');
            var localPlayer;
            Utils.forEach(players, function (player, i, l, breakLoop) {
                if (!player.isRemote) {
                    localPlayer = player;
                    breakLoop();
                }
            });
            return localPlayer;
        },
        /**
         * General achievement setting, also saves achievement in Bento.saveState
         * Make sure achievement id is the same in GameCenter as in Google Play!
         * The callback is only called when achieving the first time
         */
        submittedAchievements: {},
        postAchievement: function (id, onComplete) {
            // var Utils = utils;

            // this.submitted is a simple check to prevent submitting the same achievements in a row
            if (!this.submittedAchievements[id]) {
                console.log('Posting achievement:' + id);
                this.submittedAchievements[id] = true;
            } else {
                console.log('Already have achievment:' + id);
                return;
            }

            if (Utils.isNativeAndroid() && Utils.GooglePlay.isLoggedIn() && !Bento.saveState.load('GP-' + id, false)) {
                Utils.GooglePlay.postAchievement(
                    id,
                    function () {
                        Bento.saveState.save('GP-' + id, true);
                        if (onComplete) {
                            onComplete();
                        }
                    }
                );
            } else if (Utils.isNativeIos() && Utils.GameCenter.isLoggedIn() && !Bento.saveState.load('GC-' + id, false)) {
                id = id.replace('-', ''); // GC disallows characters such as -, while GP can generate these -> delete these characters!
                Utils.GameCenter.postAchievement(
                    id,
                    function () {
                        Bento.saveState.save('GC-' + id, true);
                        if (onComplete) {
                            onComplete();
                        }
                    }
                );
            }
        },
        /**
         * Unlocks iMessage sticker
         * Will do nothing if cordova-plugin-nsuserdefaults-for-app-groups plugin isn't installed
         */
        unlockSticker: function (name) {
            if (!Utils.isNativeIos()) {
                // does nothing outside of iOS
                return;
            }
            if (!window.AppGroupsUserDefaults) {
                console.log("UserDefaults doesn't exist");
                return;
            }

            window.AppGroupsUserDefaults.save({
                key: name,
                value: "true",
                suite: "group.luckykat.beatstreet"
            }, function () {
                // console.log('UserDefaults saved');
            }, function () {
                console.log('Error saving UserDefaults');
            });
        },
        isOldIphone: function () {
            // devices such as iphone 4 and 5 are considered "old"
            var deviceString;
            var deviceModel;
            var numbers;
            var generation;
            var version;
            var result = false;
            if (!Utils.isNativeIos()) {
                return false;
            }
            if (Utils.isCocoonJs() && window.device) {
                deviceString = window.device.model;
                // get the 2 numbers behind iPad
                // see https://www.theiphonewiki.com/wiki/Models
                numbers = deviceString.replace('iPad', '').replace('iPhone', '').replace('iPod', '');
                numbers = numbers.split(',');
                deviceModel = deviceString.replace(numbers, '');
                generation = parseInt(numbers[0]);
                version = parseInt(numbers[1]);

                if (deviceModel === 'iPod') {
                    result = true;
                } else if (deviceModel === 'iPad' && generation < 3) {
                    result = true;
                } else if (deviceModel === 'iPhone' && generation < 7) {
                    result = true;
                }

                // memoize result
                Utils.isOldIos = function () {
                    return result;
                };
            }
            return result;
        },
        /**
         * Overwrite the standard Utils.log with a version that also sends the error message to Analytics.
         * @param {String} msg - the error message
         */
        log: function (msg) {
            var stack;
            console.error(msg);

            // get stack
            try {
                window.deliberateError();
            } catch (e) {
                // this is the only way to get a stack trace in Cocoon
                if (e.stack) {
                    stack = e.stack;
                }
            }

            // no need to send events during development
            if (Utils.isDev()) {
                return;
            }
            EventSystem.fire('Analytics', {
                category: 'Utils.log',
                action: msg,
                label: stack
            });
        },
    };
    Utils.extend(Utils, utils);
    return Utils;
});